from pprint import pprint
from types import SimpleNamespace

from django.http import JsonResponse
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated

from c123_kubernetes.tasks import get_core_v1_api, get_unused_node_ip, is_path_exists_on_node, is_port_in_use_on_node
from clusterCore.exceptions import APIException404
from clusterCore.tasks import format_password


@api_view(["POST"])
@permission_classes((IsAuthenticated,))
def GetUnusedNodeIp(request):
    data = request.data
    pprint(data)

    host_data = data["host"]
    ip = host_data["name"]
    username = host_data["username"]
    password = host_data["password"]
    private_key = host_data["private_key"]
    ssh_port = host_data["ssh_port"]

    cluster_name = data["cluster"]["name"]
    namespace = data["cluster"]["ex_params"]["namespace"]

    v1 = get_core_v1_api(ip, username, password, private_key, ssh_port)
    node_ip = get_unused_node_ip(v1, namespace, cluster_name)

    if node_ip is None:
        raise APIException404("Not found an unused node")

    return JsonResponse({"node_ip": node_ip}, status=200)


@api_view(["POST"])
@permission_classes((IsAuthenticated,))
def CheckPathExists(request):
    cp_ip = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)
    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")

    node_ip = request.data.get("node_ip")
    path = request.data.get("path")

    cp = SimpleNamespace(**{"ip": cp_ip, "username": username, "password": password, "private_key": private_key, "ssh_port": ssh_port})
    v1 = get_core_v1_api(cp.ip, cp.username, cp.password, cp.private_key, cp.ssh_port)

    nodes = v1.list_node()
    node_name = None
    for node in nodes.items:
        if node_name:
            break

        for address in node.status.addresses:
            if address.type == "InternalIP" and address.address == node_ip:
                node_name = node.metadata.name
                break

    if not node_name:
        raise APIException404(f"No node found with IP: {node_ip}")

    result = is_path_exists_on_node(cp, node_name, path)
    return JsonResponse({"exists": result}, status=200)


@api_view(['POST'])
@permission_classes((IsAuthenticated,))
def CheckPortExists(request):
    cp_ip = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)
    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")

    node_ip = request.data.get("node_ip")
    node_port = int(request.data.get("port"))

    cp = SimpleNamespace(**{"ip": cp_ip, "username": username, "password": password, "private_key": private_key, "ssh_port": ssh_port})
    v1 = get_core_v1_api(cp.ip, cp.username, cp.password, cp.private_key, cp.ssh_port)

    nodes = v1.list_node()
    node_name = None
    for node in nodes.items:
        if node_name:
            break

        for address in node.status.addresses:
            if address.type == "InternalIP" and address.address == node_ip:
                node_name = node.metadata.name
                break

    if not node_name:
        raise APIException404(f"No node found with IP: {node_ip}")

    result = is_port_in_use_on_node(cp, node_name, node_port)
    return JsonResponse({"exists": result}, status=200)