from rest_framework import serializers
from clusterCore.models import c123_node, c123_inventory, c123_host, ClusterSizing, c123_cluster, Task, IntegratedService
from clusterCore.enums import DBType, ClusterRoles, Status, Severity, Schedule, OperatingSystem, TaskState, DataBaseType, Node_Size_Allowed, IntegratedServiceTypes
from clusterCore.api.serializer import c123_CreateHostSerializer, c123_CreateTempClusterSerializer, object_to_json, RECREATION_CONFLICT, OS_CONFLICT, PRIMARY_GONE, FORBID_APP_HOST
from clusterCore.exceptions import *
from clusterCore.tasks import check_os_version, drop_backup, format_rsa_key, license_after_create, ensure_primary_created, get_fully_qualified_domain_name, format_password
import json
from apscheduler.triggers.interval import IntervalTrigger
from django.utils import timezone
class c123_CreateElasticSearchClusterSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'dir_123Cluster', "database_user", "database_pwd", 'ex_params']
    @license_after_create 
    def create(self, validated_data):
        software_version = validated_data['version'] 
        create_run_linux = True
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
        if not proxmox_host:
            is_error, hostname = get_fully_qualified_domain_name(host_data["name"])
            if is_error:
                raise APIException401(hostname)
        else:
            hostname = host_data['name'] 
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException406(FORBID_APP_HOST.format(hostname))
        
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first()   
        host_exist = c123_host.objects.filter(name = hostname).exists()
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        elasticsearchcluster_exists = c123_node.objects.filter(name = node_name)
        if elasticsearchcluster_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.ELASTIC.value, node_name))
        elasticsearchcluster_to_update = elasticsearchcluster_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        host_data['ex_params'] = {}
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.INVALID.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = validated_data['ex_params'].get('distributor')
            release = validated_data['ex_params'].get('release')
        
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.ELASTIC.value
        cluster_data['name'] = node_name
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.ELASTIC.value
        validated_data['cluster_role'] = ClusterRoles.PRIMARY.value 
        own_cluster = c123_cluster.objects.filter(name = cluster_data["name"]).first()
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        validated_data["cluster"] = own_cluster
        sizing = self.context['request'].data.get('sizing')
        validated_data['sizing'] = sizing 
        validated_data['database_pwd'] = format_password(validated_data['database_pwd'])
        validated_data["ex_params"]["elastic_pwd"] = format_password(validated_data["ex_params"]["elastic_pwd"])
        validated_data['ind_deleted'] = False
        elasticsearch_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.ELASTIC.value)  
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        
        match sizing:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = elasticsearch_sys.small_memory
                cpu_cores = elasticsearch_sys.small_cpu
                disk_gb = elasticsearch_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = elasticsearch_sys.medium_memory
                cpu_cores = elasticsearch_sys.medium_cpu
                disk_gb = elasticsearch_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = elasticsearch_sys.large_memory
                cpu_cores = elasticsearch_sys.large_cpu
                disk_gb = elasticsearch_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = elasticsearch_sys.very_large_memory
                cpu_cores = elasticsearch_sys.very_large_cpu
                disk_gb = elasticsearch_sys.very_large_disk
        
        if elasticsearchcluster_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=elasticsearchcluster_to_update.pk).update(**validated_data)
            elasticsearchcluster = c123_node.objects.get(pk=elasticsearchcluster_to_update.pk) 
        else:    
            elasticsearchcluster = c123_node.objects.create(**validated_data)
        elasticsearchcluster.ex_params["ram_mb"] = round((ram_gb * 1024) * 0.5)
        elasticsearchcluster.ex_params["roles"] = ["data", "master"]
        elasticsearchcluster.ex_params["elasticsearch_master_id"] = elasticsearchcluster.pk
        elasticsearchcluster.save()
        if self.context["request"].data.get("auto_delete"):
            elasticsearchcluster.auto_delete =  elasticsearchcluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            elasticsearchcluster.save()

        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = elasticsearchcluster))
        else:
            c123_inventory.objects.filter(node = elasticsearchcluster).exclude(ex_params__belong_to = elasticsearchcluster.type, ex_params__node_version = elasticsearchcluster.version).delete()
        repository = elasticsearchcluster.ex_params["repository"]
        tmp_data = {'node_data': object_to_json(elasticsearchcluster), 'host_data': object_to_json(elasticsearchcluster.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'software_version': software_version, 'repository': repository, 'ram_gb': ram_gb, 'disk_gb': disk_gb, 'cpu_cores': cpu_cores, 'created_by': current_user.id}
        if proxmox_host:
            elasticsearchcluster.ex_params['service_id'] = service.pk
            elasticsearchcluster.save()
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(task='install_elasticsearch_cluster_task',
                                    displayment='creating elasticsearch primary {}'.format(elasticsearchcluster.name),
                                    target_type=DBType.ELASTIC.value,
                                    target_id=elasticsearchcluster.id,
                                    org_id = elasticsearchcluster.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        
        from clusterCore.scheduler import UpdateElasticsearchClusterRoles
        from django.conf import settings
        settings.scheduler.add_job(UpdateElasticsearchClusterRoles, IntervalTrigger(minutes = 1, timezone=settings.TIME_ZONE), replace_existing=True, id= f'update_elasticsearch_roles_{elasticsearchcluster.cluster.id}', kwargs={"cluster_id": elasticsearchcluster.cluster.id})
        
        return elasticsearchcluster, task

class c123_AddElasticSearchNodeSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'dir_123Cluster', 'ex_params']
    @license_after_create
    def create(self, validated_data):
        
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
        if not proxmox_host:
            is_error, hostname = get_fully_qualified_domain_name(host_data["name"])
            if is_error:
                raise APIException401(hostname)
        else:
            hostname = host_data['name'] 
        
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException406(FORBID_APP_HOST.format(hostname))
        master_id = validated_data['ex_params']['elasticsearch_master_id']
        master_node = c123_node.objects.filter(type = DBType.ELASTIC.value, pk=master_id, ind_deleted = False).first()
        if not master_node:
            raise APIException410(PRIMARY_GONE.format(master_id))
        ensure_primary_created(master_node)
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        node_exists = c123_node.objects.filter(name = node_name)
        if node_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.ELASTIC.value, node_name))
        node_to_update = node_exists.filter(ind_deleted = True).first()
        create_run_linux = True
        host_exist = c123_host.objects.filter(name = hostname).exists()
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first()  
        cluster_nodes = c123_node.objects.filter(type = DBType.ELASTIC.value, cluster = master_node.cluster, cluster_role = ClusterRoles.STANDBY.value).exclude(ind_deleted = True)
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])

        host_master = master_node.host

        (master_distributor, master_release) = check_os_version(host_master.__dict__)
        if OperatingSystem.UNKNOWN.value in master_distributor:
            raise APIException401(master_release['stderr'])
        
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.UNKNOWN.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = master_distributor
            release = master_release
        
        if master_distributor != distributor or master_release != release:
            raise APIException406(OS_CONFLICT.format(master_distributor, master_release, distributor, release))
        host_data['ex_params'] = {}
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.ELASTIC.value
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.ELASTIC.value
        validated_data['cluster'] = master_node.cluster
        validated_data['cluster_role'] = ClusterRoles.STANDBY.value
        validated_data['database_user'] = master_node.database_user
        validated_data['database_pwd'] = master_node.database_pwd
        sizing = self.context['request'].data.get('sizing')
        validated_data['sizing'] = sizing
        validated_data['ind_deleted'] = False 
        if not validated_data['ex_params'].get('roles'):
            validated_data['ex_params']['roles'] = []
         
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False  
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host

        if node_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=node_to_update.pk).update(**validated_data)
            node = c123_node.objects.get(pk=node_to_update.pk) 
        else:    
            node = c123_node.objects.create(**validated_data)     
        
        other_nodes = cluster_nodes.exclude(pk = node.pk)
        has_remote_nodes = c123_node.objects.filter(type = DBType.ELASTIC.value, ind_deleted = False, cluster = master_node.cluster).exclude(host__name = host_master.name).exists()
 
        if has_remote_nodes:
            nodes = [{"node": object_to_json(i),"host": object_to_json(i.host)} for i in other_nodes]
        else:
            nodes = [{"node": object_to_json(i)} for i in other_nodes]
        
        if self.context["request"].data.get("auto_delete"):
            node.auto_delete =  node.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
    
        node.cluster = master_node.cluster
        repository = master_node.ex_params["repository"]
        node.ex_params["repository"] = repository
        node.ex_params["elastic_pwd"] = master_node.ex_params["elastic_pwd"]
        node.ex_params['distributor'] = distributor
        node.ex_params['release'] = release
        elasticsearch_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.ELASTIC.value)
        match sizing:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = elasticsearch_sys.small_memory
                cpu_cores = elasticsearch_sys.small_cpu
                disk_gb = elasticsearch_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = elasticsearch_sys.medium_memory
                cpu_cores = elasticsearch_sys.medium_cpu
                disk_gb = elasticsearch_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = elasticsearch_sys.large_memory
                cpu_cores = elasticsearch_sys.large_cpu
                disk_gb = elasticsearch_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = elasticsearch_sys.very_large_memory
                cpu_cores = elasticsearch_sys.very_large_cpu
                disk_gb = elasticsearch_sys.very_large_disk
        node.ex_params["ram_mb"] = round((ram_gb * 1024) * 0.5)
        node.save()
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = node))
        else:
            c123_inventory.objects.filter(node = node).exclude(ex_params__belong_to = node.type, ex_params__node_version = node.version).delete()
        node_data = object_to_json(node)
        tmp_data = {'master_data': object_to_json(master_node), 'node_data': node_data, 'nodes': nodes, 'host_master': object_to_json(host_master), 'host_node': object_to_json(node.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'repository': repository, 'ram_gb': ram_gb, 'disk_gb': disk_gb, 'cpu_cores': cpu_cores, "has_remote_nodes": has_remote_nodes, 'created_by': current_user.id}
        if proxmox_host:
            node.ex_params['service_id'] = service.pk
            node.save()
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(
                                    task = "add_elasticsearch_node_task",
                                    displayment='adding node {}'.format(node.name),
                                    target_type=DBType.ELASTIC.value,
                                    target_id=node.id,
                                    org_id = node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        return node, task