from django.http import JsonResponse
import re, json, pytz
from django.utils import timezone
from datetime import datetime
from rest_framework import generics, status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from .serializer import c123_CreateElasticSearchClusterSerializer, c123_AddElasticSearchNodeSerializer
from clusterCore.api.serializer import c123_NodeSerializer, c123_DatabaseSerializer, format_size, object_to_json, get_create_node_response, c123_inventory_serializer, c123_HostSerializer
from clusterCore.tasks import allow_or_403, cleanup_logs, checkPathExists, create_elasticsearch_insecure_connection, create_ssh_connection, \
remote_command, drop_elasticsearch_node, cleanup_logs, get_cron_schedule, append_status_cmd, format_password, get_backup_description, export_elasticsearch_index, \
get_avail_change_date
from clusterCore.scheduler import schedule_backup, MISFIRE_GRACE_TIME_BACKUP
from clusterCore.exceptions import *
from types import SimpleNamespace
from clusterCore.models import c123_node, c123_host, c123_database, Task, ClusterSizing, c123_inventory
from clusterCore.enums import DBType, ClusterRoles, RestoreOptions, Severity, Status, Inventory, Schedule, CopyOptions, TaskState, Node_Size_Allowed
from dateutil.parser import parse
class ElasticSearchClusterCreateView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_CreateElasticSearchClusterSerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)
        
    
class ElasticSearchNodeAddView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_AddElasticSearchNodeSerializer
    def create(self, request, *args, **kwargs):
       return get_create_node_response(self, request, *args, **kwargs)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckElasticsearchVersionExists(request):
    """check if elasticsearch version exists"""
    host_name = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)
    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")
    version = request.data.get("software_version")
        
    host = SimpleNamespace( **{
        "name": host_name, 
        "username": username, 
        "password": password,
        "private_key": private_key,
        "ssh_port": ssh_port
    })
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    
    jsn = json.loads(remote_command(client, f"sudo rpm -qa | grep -E '^elasticsearch-{version}\.[0-9]+\.[0-9]'", True))
    if(jsn['stderr'] != ''):
            print(jsn['stderr'])
            if 'not found' in jsn['stderr'].lower():
                  jsn = json.loads(remote_command(client, f"sudo dpkg -l |  grep -E '^ii\s+elasticsearch' | grep '{version}.'", True))             
            else:
                return JsonResponse({'error': jsn['stderr']}, status=404)
    if version in jsn['stdout']:
        return JsonResponse({'exists': True}, status=200)
        
    return JsonResponse({'exists': False}, status=200)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def UpgradeElasticsearchNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    target_version = request.data.get('target_version')
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    
    nodes = c123_node.objects.filter(type = DBType.ELASTIC.value, ind_deleted = False, cluster = node.cluster)
    node_list = [{'node': object_to_json(n), 'host': object_to_json(n.host)} for n in nodes]
    
    nodes.update(status = Status.UPGRADING.value)
    
    tmp_data = {'app_host': object_to_json(app_host), 'cluster_nodes': node_list, 'target_version': target_version, 'created_by': current_user.id}
    task = Task.objects.create(task='upgrade_elasticsearch_node_task',
                                displayment='upgrading {} to elasticsearch {}'.format(node.name, target_version),
                                target_type=DBType.ELASTIC.value,
                                target_id=node.id,
                                org_id=node.org_id,
                                status=TaskState.PENDING.value,
                                schedule=Schedule.NOW.value,
                                parameters=tmp_data,
                                created_by=current_user)
    return append_status_cmd(request, task)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckElasticsearchIndexName(request):
    
    master_id = request.data.get('node_id')
    index_name = request.data.get('name')
    master = c123_node.objects.filter(pk=master_id,  type = DBType.ELASTIC.value,  ind_deleted = False).first()
    if not master:
        raise APIException410(f"master id {master_id} not available")
    if(re.match(r'^[a-z][a-z0-9_-]{0,254}$', index_name)):
        client = create_elasticsearch_insecure_connection(master)
        if isinstance(client, tuple):
            if hasattr(client[1], 'strerror'):
                raise APIException401(client[1].strerror)
            raise APIException401(client[1])
        index_exists = client.indices.exists(index = index_name)
        return JsonResponse({'response': "name available"}, status=200) if not index_exists else JsonResponse({'response': "already exists"}, status = 200)
    else:
        return JsonResponse({'error': "invalid name"}, status=406)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreElasticsearchIndex(request):
    """restore a elasticsearch index from dump
    the call is made from client AJAX POST that contains the dump id"""
    dump_id = request.data.get('dump_id')
    api_call = request.data.get('rest_api')
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        dump = c123_inventory.objects.filter(name=dump_id, type = Inventory.EXPORT.value).first()
    else:
        dump = c123_inventory.objects.filter(pk=dump_id, type = Inventory.EXPORT.value).first()
   
    if not dump:
        raise APIException404("dump id {} not found".format(dump_id))
    
    path = dump.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")
    
    restore_option = request.data.get('restore_option')
    restore_time = request.data.get('restore_time')
    index = dump.database
    # serialize objects for the tasks
    node = index.node
    allow_or_403(node)
    forbid_restore = Task.objects.filter(target_id = node.id, task__in = ['restore_elasticsearch_snapshot_task', 'elasticsearch_snapshot_task'], status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
    if forbid_restore:
        raise APIException409(f"Already runs backup/restore for cluster {node.cluster.name}")
    index.status = Status.RESTORING.value
    index.save()
    index_name = index.name
    created_by = request.user
    if RestoreOptions.THIS.value in restore_option:
        date = dump.created_date.strftime("%Y-%m-%d %H:%M:%S")
    elif RestoreOptions.CUSTOM.value in restore_option and restore_time:
        # example date = '2021-12-30 09:08:33'
        # date = datetime.strptime(restore_time, "%Y-%m-%dT%H:%M:%S.%f%z").astimezone()
        # date = date.strftime("%Y-%m-%d %H:%M:%S")
        date = restore_time
    else:
        date = ''
    dump = object_to_json(dump)
    
    cluster_nodes = c123_node.objects.filter(cluster = node.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
    cluster_data = [{"host": object_to_json(i.host), "node": object_to_json(i)} for i in cluster_nodes]
    
    tmp_data = json.dumps({'cluster_data': cluster_data, 'index_id': index.id, 'index_name': index_name,'date': date,'restore_option': restore_option, 'dump': dump, 'created_by': created_by.id})
    task = Task.objects.create(task='restore_elasticsearch_snapshot_task',
                            displayment=f'restore elasticsearch index {index.name}',
                            target_type=DBType.ELASTIC.value,
                            target_id=node.id,
                            org_id = node.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RemoveElasticsearchSchedule(request):
    node_id = request.data.get("node_id")
    target = c123_node.objects.filter(pk=node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not target:
        raise APIException404(f"node id {node_id} not found")
    
    allow_or_403(target)
    
    cluster_nodes = c123_node.objects.filter(cluster = target.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
    from django.conf import settings
    for cluster_node in cluster_nodes:
        snapshot = c123_inventory.objects.filter(node = cluster_node, type = Inventory.BACKUP.value, is_sched = True).last()
        if snapshot:
            job_id = snapshot.ex_params.get("job_id")
            job = settings.scheduler.get_job(job_id=job_id)
            if job:
                settings.scheduler.remove_job(job_id=job_id)
                snapshot.delete()
                notification = f"{job_id} removed"
                return JsonResponse({"notification": notification}, status=200)
    else:
        return JsonResponse({"notification": "no schedule snapshot found"}, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def SnapshotElasticsearchCluster(request):
    """snapshot for elasticsearch cluster"""
    schedule = request.data.get('schedule')
    node_id = request.data.get('node_id')
    compress = request.data.get("compress")
    snapshot_path = request.data.get("backup_path")
    update_schedule = request.data.get("update_schedule")
    api_call = request.data.get('rest_api')
    from django.conf import settings
    if api_call:
        target = c123_node.objects.filter(name=node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    else:
        target = c123_node.objects.filter(pk=node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()

    if not target:
        raise APIException409(f"node id {node_id} is not available")
    if schedule:
        client_timezone = pytz.timezone(request.data.get("client_timezone"))
        schedule_run_time_str = request.data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = request.data.get("days_of_week")
        days_of_month = request.data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
    if update_schedule and schedule:
        allow_or_403(target)
        cluster_nodes = c123_node.objects.filter(cluster = target.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
        for cluster_node in cluster_nodes:
            snapshot = c123_inventory.objects.filter(node = cluster_node, type = Inventory.BACKUP.value, is_sched = True).last()
            if snapshot:
                snapshot.ex_params['description'] = description
                snapshot.ex_params['schedule_run_time'] = schedule_run_time_str
                snapshot.ex_params['days_of_month'] = days_of_month
                snapshot.ex_params['days_of_week'] = days_of_week
                snapshot.save()
                notification = f"{snapshot.node} snapshot updated to '{description}'"
                job_id = snapshot.ex_params.get("job_id")
                job = settings.scheduler.get_job(job_id=job_id)
                if job:
                    job.reschedule(trigger=trigger)
                else:
                    settings.scheduler.add_job(schedule_backup, trigger=trigger, replace_existing=True, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP,  kwargs={"task_id": snapshot.ex_params["task_id"]}, id=notification)
                return JsonResponse({"notification": notification}, status=200)
        
        else:
            return JsonResponse({"notification": "no schedule snapshot found"}, status = 200)
    serializer = c123_inventory_serializer(data = request.data, partial = True)
    if serializer.is_valid():
        app_host = c123_host.objects.filter(ex_params__myself = True).first()
        if not app_host:
            raise APIException404("could not determine the app host")
        allow_or_403(target)
        size_scheme, created = ClusterSizing.objects.get_or_create(db_type = DBType.ELASTIC.value)
        keep_days = getattr(size_scheme, f'{target.sizing.lower()}_backup_keep_days', None)
    # create the task signature
        if schedule:
            cluster_nodes = c123_node.objects.filter(cluster = target.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
            for cluster_node in cluster_nodes:
                existing_snapshot = c123_inventory.objects.filter(node = cluster_node, type = Inventory.BACKUP.value, is_sched = True)
                if existing_snapshot.exists():
                    raise APIException409("Schedule snapshot already exists.\nCan be only one schedule snapshot for cluster.")
            snapshot = serializer.create(serializer.validated_data)
            snapshot.node = target
            if keep_days:
                snapshot.keep_days = keep_days
            snapshot.type = Inventory.BACKUP.value
            snapshot.created_by = request.user
            snapshot.ex_params = {"size": 0}
            snapshot.ex_params["belong_to"] = DBType.ELASTIC.value
            snapshot.ex_params["node_version"] = target.version
            snapshot.is_sched = schedule
            snapshot.ex_params["backup_type"] = "cluster_snapshot"
            snapshot.ex_params["duration"] = 0
            snapshot.ex_params["compress"] = compress
            snapshot.ex_params["modified_date"] = ''
            snapshot.ex_params['description'] = description
            snapshot.ex_params['schedule_run_time'] = schedule_run_time_str
            snapshot.ex_params['days_of_month'] = days_of_month
            snapshot.ex_params['days_of_week'] = days_of_week
            notification =  "{} snapshot {}".format(snapshot.node, description)
            job_id = f"{notification} ({client_timezone})"
            snapshot.ex_params["job_id"] = job_id
            # create a unique string with the date
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            snapshot.name = "{}_{}".format(target.cluster.name, now)
            snapshot.ex_params["backup_path"] = snapshot_path
            snapshot.save()
            # serialize objects for the tasks
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
            cluster_nodes = c123_node.objects.filter(cluster = target.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
            cluster_data = [{"host": object_to_json(i.host), "node": object_to_json(i)} for i in cluster_nodes]
            tmp_data = json.dumps({'backup_host': backup_host, 'cluster_data': cluster_data, 'backup_data': object_to_json(snapshot), 'created_by': created_by.id, 'compress': compress})
            task = Task.objects.create(task='elasticsearch_snapshot_task',
                                            displayment='snapshot of cluster {}'.format(target.cluster.name), 
                                            target_type=DBType.ELASTIC.value, 
                                            target_id=target.id,
                                            org_id = target.org_id,
                                            schedule=Schedule.SCHEDULED.value, 
                                            status=TaskState.ON_DEMAND.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            snapshot.ex_params["task_id"] = task.pk
            snapshot.save()
            settings.scheduler.add_job(schedule_backup, trigger=trigger, replace_existing=True, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP,  kwargs={"task_id": task.pk}, id=notification)
        else:
            cluster_nodes = c123_node.objects.filter(cluster = target.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
            for cluster_node in cluster_nodes:
                existing_snapshot = Task.objects.filter(target_id = target.id, task = 'elasticsearch_snapshot_task', status__in = [TaskState.PENDING.value, TaskState.RUNNING])
                if existing_snapshot:
                    raise APIException409(f"Already runs snapshot for {target.cluster.name}")
            
            snapshot = serializer.create(serializer.validated_data)
            snapshot.node = target
            if keep_days:
                snapshot.keep_days = keep_days
            snapshot.type = Inventory.BACKUP.value
            snapshot.created_by = request.user
            snapshot.ex_params = {"size": 0}
            snapshot.ex_params["belong_to"] = DBType.ELASTIC.value
            snapshot.ex_params["node_version"] = target.version
            snapshot.is_sched = schedule
            snapshot.ex_params["backup_type"] = "cluster_snapshot"
            snapshot.ex_params["duration"] = 0
            snapshot.ex_params["compress"] = compress
            snapshot.ex_params["modified_date"] = ''
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            snapshot.name = "{}_{}".format(target.cluster.name, now)
            snapshot.ex_params["backup_path"] = snapshot_path
            snapshot.save()
            notification = f"backup of {target.name}"
            # serialize objects for the tasks
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
            cluster_nodes = c123_node.objects.filter(cluster = target.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
            cluster_data = [{"host": object_to_json(i.host), "node": object_to_json(i)} for i in cluster_nodes]
            tmp_data = json.dumps({'backup_host': backup_host, 'cluster_data': cluster_data, 'backup_data': object_to_json(snapshot), 'created_by': created_by.id, 'compress': compress})
            task = Task.objects.create(task='elasticsearch_snapshot_task',
                                            displayment='snapshot of cluster {}'.format(target.cluster.name), 
                                            target_type=DBType.ELASTIC.value, 
                                            target_id=target.id,
                                            org_id = target.org_id,
                                            schedule=Schedule.NOW.value, 
                                            status=TaskState.PENDING.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            snapshot.ex_params["task_id"] = task.pk
            snapshot.save()
    else:
        print("invalid form")
        print(serializer.errors)        
        return JsonResponse({'error': serializer.errors}, status=404)
    return append_status_cmd(request, task, notification)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreElasticsearchCluster(request):
    """restore a elasticsearch cluster from snapshot
    the call is made from client AJAX POST that contains the snapshot id"""
    
    sn_id = request.data.get('sn_id')
    # restore_option = 'this'/'latest'/'custom'
    restore_option = request.data.get('restore_option')
    restore_time = request.data.get('restore_time')
    autostart = request.data.get('autostart')
    api_call = request.data.get('rest_api')
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        snapshot = c123_inventory.objects.filter(name=sn_id, type = Inventory.BACKUP.value).first()
    else:
        snapshot = c123_inventory.objects.filter(pk=sn_id, type = Inventory.BACKUP.value).first()
   
    if not snapshot:
        raise APIException404("snapshot id {} not found".format(sn_id))
    node = snapshot.node
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    path = snapshot.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")
    allow_or_403(node)

      
    existing_restore = Task.objects.filter(target_id = node.id, task__in = ['restore_elasticsearch_snapshot_task', 'elasticsearch_snapshot_task'] , status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
    if existing_restore:
        raise APIException409(f"Already runs restore for cluster {node.cluster.name}")
    
    master = c123_node.objects.filter(cluster=node.cluster, type = DBType.ELASTIC.value, cluster_role=ClusterRoles.PRIMARY.value, ind_deleted=False).first()
    if not master:
        #because it can not exist, or exist as deleted
        raise APIException410(f"primary of node {node.name} is not available")
    cluster_nodes = c123_node.objects.filter(cluster = node.cluster, type = DBType.ELASTIC.value, ind_deleted = False).exclude(cluster_role = ClusterRoles.PRIMARY.value)
    node_in_cluster = cluster_nodes.exists()
    nodes_hosts = []
    nodes_id = []
    if node_in_cluster:
        for node in cluster_nodes:
            nodes_id.append(node.id)
            nodes_hosts.append(object_to_json(node.host))
    master.status = Status.RESTORING.value
    master.save()
    if RestoreOptions.THIS.value in restore_option:
        date = snapshot.created_date.strftime("%Y-%m-%d %H:%M:%S")
    elif RestoreOptions.CUSTOM in restore_option and restore_time:
        # example date = '2021-12-30 09:08:33'
        # date = datetime.strptime(restore_time, "%Y-%m-%dT%H:%M:%S.%f%z").astimezone()
        # date = date.strftime("%Y-%m-%d %H:%M:%S")
        date = restore_time
    else:
        date = ''
            
    # serialize objects for the tasks
    snapshot = object_to_json(snapshot)
    created_by = request.user
    cluster_nodes = c123_node.objects.filter(cluster = master.cluster, type = DBType.ELASTIC.value, ind_deleted = False)
    cluster_data = [{"host": object_to_json(i.host), "node": object_to_json(i)} for i in cluster_nodes]
    if node.cluster_role == ClusterRoles.PRIMARY.value and c123_node.objects.filter(type = DBType.ELASTIC.value, cluster = node.cluster, ind_deleted = False, severity = Severity.OK.value, cluster_role = ClusterRoles.STANDBY.value).exists():
        autostart = True
    tmp_data = json.dumps({'cluster_data': cluster_data, 'dump': snapshot, 'restore_option': restore_option, 'autostart': autostart, 'date': date, 'nodes_hosts': nodes_hosts, 'nodes_id': nodes_id, 'created_by': created_by.id})
    task = Task.objects.create(task='restore_elasticsearch_snapshot_task',
                            displayment='restore elasticsearch cluster {}'.format(node.cluster.name),
                            target_type=DBType.ELASTIC.value,
                            target_id=node.id,
                            org_id = node.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckElasticsearchIndexSize(request):
    
    master_id = request.data.get('node_id')
    index_name = request.data.get('index_name')
    master = c123_node.objects.filter(pk=master_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not master:
        raise APIException410(f"master id {master_id} not available")
    client = create_elasticsearch_insecure_connection(master)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1].args[0])
       
    try:
        stats = client.indices.stats(index=index_name, metric="store", forbid_closed_indices=False)
    except Exception as e:
        if hasattr(e, 'strerror'):
            raise APIException401(e.strerror)
        raise APIException401(e.args[0])
        
    index_size = float(stats["_all"]["total"]["store"]["total_data_set_size_in_bytes"]) / 1024
    client = create_ssh_connection(master.host.name, master.host.username, master.host.password, master.host.private_key, master.host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    command = 'df -P . | tail -1 | awk \'{print $4}\''
    jsn = json.loads(remote_command(client, command, True), strict=False)
    if(not jsn['stderr'] == ''):
        print(jsn['stderr'])
        return JsonResponse({'detail': jsn['stderr']}, status=404)
    
    free_space = float(jsn['stdout'])
    return JsonResponse({"size": index_size, 'display_size': format_size(index_size), "free_space": free_space})
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StartElasticsearchNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id,  type = DBType.ELASTIC.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id,  type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    host_data = c123_HostSerializer(node.host).data
    if node.ind_started:
         node.status = Status.RESTARTING.value
         node.save()
         tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': host_data, 'created_by': current_user.id})
         task = Task.objects.create(task='restart_elasticsearch_node_task',
                                    displayment='restarting node {}'.format(node.name),
                                    target_type=DBType.ELASTIC.value,
                                    target_id=node.id,
                                    org_id = node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    else:
        node.ind_started = True
        node.status = Status.STARTING.value
        node.save()
        tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': host_data, 'created_by': current_user.id})
        task = Task.objects.create(task='start_elasticsearch_node_task',
                                    displayment='starting node {}'.format(node.name),
                                    target_type=DBType.ELASTIC.value,
                                    target_id=node.id,
                                    org_id = node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropElasticSearchIndex(request):       
    ind_id = request.data.get("ind_id")
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    if api_call:
        index = c123_database.objects.filter(name = ind_id, node__name = node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    else:
        index = c123_database.objects.filter(pk = ind_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not index:
        raise APIException410("index id {} not found".format(ind_id))
    node = index.node
    allow_or_403(node)
    index.status = Status.DROPPING.value
    index.save()
    cleanup_logs(index.pk, index.type)
    node_data = object_to_json(node)
    index_data = object_to_json(index)
    host_data = c123_HostSerializer(node.host)
    created_by = request.user
    
    tmp_data = json.dumps({'node_data': node_data, 'db': index_data, 'host_data': host_data.data, 'created_by': created_by.id})               
    task = Task.objects.create(task='delete_elasticsearch_index_task',
                            displayment='drop index {}'.format(index.name),
                            target_type=DBType.ELASTIC.value,
                            target_id=index.id,
                            org_id = node.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
         
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StopElasticsearchNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    node.ind_started = False
    node.status = Status.STOPPING.value
    node.save()
    tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': c123_HostSerializer(node.host).data, 'created_by': current_user.id})
    task = Task.objects.create(task='stop_elasticsearch_node_task',
                                    displayment='stopping node {}'.format(node.name),
                                    target_type=DBType.ELASTIC.value,
                                    target_id=node.id,
                                    org_id = node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)


    
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def ElasticsearchDbDetails(request, id):
    database = c123_database.objects.filter(pk = id).first()
    if not database:
        raise APIException404("node id {} not found".format(id))
    response = {'database': c123_DatabaseSerializer(database).data}

    auto_backups = c123_inventory.objects.filter(database=database, type = Inventory.EXPORT.value, is_sched=True)
    if auto_backups.count() > 0:
        if auto_backups.last().status:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
        else:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}
                
    else:
        response["auto_backup"]  = {
                'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}

    exports = c123_inventory_serializer(c123_inventory.objects.filter(db_id = database.id, type = Inventory.EXPORT.value), many = True)
    response["exports"] = exports.data
    cs = ClusterSizing.objects.filter(db_type = DBType.ELASTIC.value).first()
    size_scheme = {
            Node_Size_Allowed.SMALL.name: {
                'cpu': cs.small_cpu,
                'disk': cs.small_disk,
                'memory': cs.small_memory,
                'backup': cs.small_backup_keep_days,
                'export': cs.small_export_keep_days,
            },
            Node_Size_Allowed.MEDIUM.name: {
                'cpu': cs.medium_cpu,
                'disk': cs.medium_disk,
                'memory': cs.medium_memory,
                'backup': cs.medium_backup_keep_days,
                'export': cs.medium_export_keep_days,
            },
            Node_Size_Allowed.LARGE.name: {
                'cpu': cs.large_cpu,
                'disk': cs.large_disk,
                'memory': cs.large_memory,
                'backup': cs.large_backup_keep_days,
                'export': cs.large_export_keep_days,
            },
            Node_Size_Allowed.VERY_LARGE.name: {
                'cpu': cs.very_large_cpu,
                'disk': cs.very_large_disk,
                'memory': cs.very_large_memory,
                'backup': cs.very_large_backup_keep_days,
                'export': cs.very_large_export_keep_days,
            }
        }
    response["avail_schemes"] = size_scheme
    node = database.node

    client = create_elasticsearch_insecure_connection(node)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            error = client[1].strerror
        else:
            error = str(client[1])
       
        response["ind_error"] = True
        response["detail"] = error
        response['node'] = c123_NodeSerializer(c123_node.objects.filter(pk = id,  type = DBType.ELASTIC.value).first()).data
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        return JsonResponse(response, status = 401)
    else:
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        response["ind_error"] = c123_node.objects.filter(type = DBType.ELASTIC.value, cluster = node.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists() or c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).count() < 2
    try:
        node_data = client.cat.nodes(h="http,id,v,u,du,rc,cpu", format="json", full_id=True, bytes='kb')
        stats = client.indices.stats(index=database.name, metric="store", forbid_closed_indices=False)
    except Exception as e:
        error = e.args[0]
        print(error)
        response["detail"] = error
        return JsonResponse(response, status = 401)
  
    current_node =  [i for i in node_data if node.name in i["http"]]
    if len(current_node) > 0:
        current_node = current_node[0]
        response["db_id"] = current_node["id"]
        node.ex_params["db_id"] =  response["db_id"]
        
        response["version"] = current_node['v']
        node.version = (response["version"].split('.'))[0]
        
        response["uptime"] = current_node['u'] if current_node.get('u') else 'N/A'
        response["display_ram"] = format_size(float(current_node["rc"]) / 1024)
        response["cpu_percent"] = current_node["cpu"]
     
    index_size = float(stats["_all"]["total"]["store"]["total_data_set_size_in_bytes"]) / 1024
    database.size = format_size(index_size)
    database.save()
    response['size'] = database.size

    return JsonResponse(response, status = 200)

@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def ElasticsearchDetails(request, id):
    response = {}
    node = c123_node.objects.filter(pk = id,  type = DBType.ELASTIC.value).first()
    if not node:
        raise APIException404("node id {} not found".format(id))
    response = {'node': c123_NodeSerializer(node).data}
    auto_backups = c123_inventory.objects.filter(node=node, type = Inventory.BACKUP.value, is_sched=True)
    if auto_backups.count() > 0:
        if auto_backups.last().status:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
        else:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}
                
    else:
        response["auto_backup"]  = {
                'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}
    
    cluster_snapshots = c123_inventory_serializer(c123_inventory.objects.filter(node_id = node.id, type = Inventory.BACKUP.value), many = True)
    response["snapshots"] = cluster_snapshots.data
    if node.cluster_role == ClusterRoles.PRIMARY.value:
        response["nodes_count"] = c123_node.objects.filter(type = DBType.ELASTIC.value, cluster = node.cluster, ind_deleted = False).exclude(cluster_role = ClusterRoles.PRIMARY.value).count()
    else:
        master = c123_NodeSerializer(c123_node.objects.filter(type = DBType.ELASTIC.value, cluster = node.cluster, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first())
        response["master"] = master.data
    nodes = c123_node.objects.filter(type = DBType.ELASTIC.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False, cluster = node.cluster).exclude(pk = node.pk)
    response['nodes'] = c123_NodeSerializer(nodes, many = True).data
    cs = ClusterSizing.objects.filter(db_type = DBType.ELASTIC.value).first()
    size_scheme = {
            Node_Size_Allowed.SMALL.name: {
                'cpu': cs.small_cpu,
                'disk': cs.small_disk,
                'memory': cs.small_memory,
                'backup': cs.small_backup_keep_days,
                'export': cs.small_export_keep_days,
            },
            Node_Size_Allowed.MEDIUM.name: {
                'cpu': cs.medium_cpu,
                'disk': cs.medium_disk,
                'memory': cs.medium_memory,
                'backup': cs.medium_backup_keep_days,
                'export': cs.medium_export_keep_days,
            },
            Node_Size_Allowed.LARGE.name: {
                'cpu': cs.large_cpu,
                'disk': cs.large_disk,
                'memory': cs.large_memory,
                'backup': cs.large_backup_keep_days,
                'export': cs.large_export_keep_days,
            },
            Node_Size_Allowed.VERY_LARGE.name: {
                'cpu': cs.very_large_cpu,
                'disk': cs.very_large_disk,
                'memory': cs.very_large_memory,
                'backup': cs.very_large_backup_keep_days,
                'export': cs.very_large_export_keep_days,
            }
        }
    response["avail_schemes"] = size_scheme
    response["databases"] = c123_DatabaseSerializer(c123_database.objects.filter(node = node, ind_deleted = False), many = True).data
    client = create_elasticsearch_insecure_connection(node)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            error = client[1].strerror
        else:
            error = str(client[1])
       
        response["ind_error"] = True
        response["detail"] = error
        response['node'] = c123_NodeSerializer(c123_node.objects.filter(pk = id,  type = DBType.ELASTIC.value).first()).data
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        return JsonResponse(response, status = 401)
    else:
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        response["ind_error"] = c123_node.objects.filter(type = DBType.ELASTIC.value, cluster = node.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists() or c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).count() < 2
    try:
        node_data = client.cat.nodes(h="http,id,v,u,du,rc,cpu", format="json", full_id=True, bytes='kb')
    except Exception as e:
        error = e.args[0]
        print(error)
        response["detail"] = error
        return JsonResponse(response, status = 401)
  
    current_node =  [i for i in node_data if node.name in i["http"]]
    if len(current_node) > 0:
        current_node = current_node[0]
        response["db_id"] = current_node["id"]
        node.ex_params["db_id"] =  response["db_id"]
        
        response["version"] = current_node['v']
        node.version = (response["version"].split('.'))[0]
        
        response["uptime"] = current_node['u'] if current_node.get('u') else 'N/A'
        response["display_ram"] = format_size(float(current_node["rc"]) / 1024)
        response["cpu_percent"] = current_node["cpu"]
        try:
            node_stats = client.nodes.stats(node_id=response["db_id"], metric="indices", index_metric="store")
        except Exception as ex:
            error = ex.strerror if hasattr(ex, 'strerror') else str(ex)
            print(error)
            response["detail"] = error
            return JsonResponse(response, status = 401)
        node.size = round(float(current_node['du']), 2) if current_node.get('du') else 0
        node.save()
        response["display_size"] = format_size(node.size)
        
        roles = node_stats["nodes"][response["db_id"]]["roles"]
        if len(roles) == 0:
            response["roles"] = "coordinating-only"
        else:
            response["roles"] = ",".join(roles)
        node.ex_params["roles"] = roles
    client = create_ssh_connection(node.host.name, node.host.username, node.host.password, node.host.private_key, node.host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            error = client[1].strerror
        else:
            error = str(client[1])
        response["detail"] = error
        return JsonResponse(response, status = 401)
   
    jsn = json.loads(remote_command(client, f"tail -n51 {node.dir_123Cluster}/logs_{node.name}/{node.cluster.name}_server.json | grep '}}$' | grep '^{{'", True), strict = False)
    try:
        history = [json.loads(line, strict = False) for line in jsn["stdout"].split("\n") if line.rstrip()]
    except Exception as e:
        error = e.args[0]
        print(error)
        response["detail"] = error
        return JsonResponse(response, status = 401)
    response["history"] = history
    node.save()
    return JsonResponse(response, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropElasticsearchNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    final_backup = request.data.get('final_backup')
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} is not available".format(node_id))
    task = drop_elasticsearch_node(node, final_backup)
         
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CopyElasticsearchIndex(request):
    index_id = request.data.get("index_id")
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    source_id = request.data.get("source_id")
    current_user = request.user
    index_option = request.data.get("option")
    #copy_option = 'one_time' or 'live'
    copy_option = request.data.get("copy_option")
    copy_name = request.data.get("copy_name")
    keep_settings =  request.data.get("keep_settings")
    index_exists = request.data.get("index_exists")
    if api_call:
        node_target = c123_node.objects.filter(name = node_id, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    else:
        node_target = c123_node.objects.filter(pk = node_id, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not node_target:
        raise APIException410(f"target master id {node_id} not found")
    allow_or_403(node_target)
    if api_call:
        node_source = c123_node.objects.filter(name = source_id, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.PRIMARY.value).first()
    else:
        node_source = c123_node.objects.filter(pk = source_id, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.PRIMARY.value).first()
    if not node_source:
        raise APIException404("source master id {} not found".format(source_id))
    allow_or_403(node_source)
    
    if api_call:
        index_target = c123_database.objects.filter(name = index_id, node = node_source, type = DBType.ELASTIC.value, ind_deleted=False).first()
    else:
        index_target = c123_database.objects.filter(pk = index_id, type = DBType.ELASTIC.value, ind_deleted=False).first()
    if not index_target:
        raise APIException410(f"target index id {index_id} not available")
    
    #source index
    index_source = c123_database.objects.get(pk=index_target.pk)
    if(index_option):
        # here we do for existing index
        match index_option:
            case CopyOptions.OVERRIDE.value:
                copy_name = index_target.name
                index_target.ind_started = False
                index_target.node = node_target
                index_target.database_user = node_target.database_user
                index_target.database_pwd = node_target.database_pwd
                index_target.status = Status.REPLICATING
                index_target.save()
            case CopyOptions.COPY.value:
                    existing_dbs = c123_database.objects.filter(node = node_target, name = copy_name)
                    if not existing_dbs.exists():
                        index_target = c123_database.objects.create(node = node_target, name = copy_name,
                        type = index_source.type, size = index_source.size,
                        database_user = node_target.database_user, database_pwd = node_target.database_pwd,
                        created_by = current_user, last_update_by = current_user,
                        severity = Severity.NOT_OK.value, status = Status.REPLICATING.value
                        )
                    else:
                        undeleted = existing_dbs.filter(ind_deleted = False)
                        if undeleted.exists():
                            index_target = undeleted.first()
                        else:
                            index_target = existing_dbs.first()
                            index_target.ind_deleted = False
                            
                        index_target.type = DBType.ELASTIC.value
                        index_target.status = Status.REPLICATING.value
                        index_target.save()
              
        index_source.ind_started = False
        index_source.status = Status.COPYING.value
        index_source.save()
    else:
        #here we do for new index
        index_source.status = Status.COPYING.value
        index_source.ind_started = False
        index_source.save()
        index_target = c123_database.objects.create(node = node_target, name = index_source.name,
        type = index_source.type, size = index_source.size,
        database_user = node_target.database_user, database_pwd = node_target.database_pwd,
        created_by = current_user, last_update_by = current_user,
        severity = Severity.NOT_OK.value, status = Status.REPLICATING.value
        )
    tmp_data = json.dumps({'db_source': object_to_json(index_source), 'db_target' : object_to_json(index_target), 'node_source': object_to_json(node_source), 'node_target': object_to_json(node_target), 'host_source': object_to_json(node_source.host), 'host_target': object_to_json(node_target.host), 'db_exists': index_exists, 'keep_settings': keep_settings, 'db_option': index_option, 'copy_option': copy_option, 'created_by': current_user.id})
    task = Task.objects.create(task='copy_elasticsearch_index_task',
                                displayment=f'copy index {index_source.name} into {node_target.name}',
                                target_type=DBType.ELASTIC.value,
                                target_id=node_target.id,
                                org_id = node_target.org_id,
                                status=TaskState.PENDING.value,
                                schedule=Schedule.NOW.value,
                                parameters=tmp_data,
                                created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ElasticsearchClusterStatus(request, id):
    master = c123_node.objects.filter(pk = id, ind_deleted = False, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.PRIMARY.value).first()
    if not master:
        raise APIException410(f"master id {id} not available")
    client = create_elasticsearch_insecure_connection(master)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1].args[0])
    cluster_stats = client.cluster.health().items()
    filtered_fields = {
        k: v
        for k, v in cluster_stats
        if k in ("cluster_name", "timed_out", "status", "number_of_nodes", "number_of_data_nodes")
    }
    return JsonResponse([filtered_fields], status = 200, safe = False)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DumpElasticsearchIndex(request):
    """dump for elasticsearch index"""

    data = request.data
    data['created_by'] = request.user.id
    schedule = data.get('schedule')
    api_call = data.get('api_call')
    node_id = data.get('node_id')
    db_id = data.get('index_id')

    if not schedule:
        task = export_elasticsearch_index(data)
        return append_status_cmd(request, task)
    else:
        from django.conf import settings
        client_timezone = pytz.timezone(data.get("client_timezone"))
        schedule_run_time_str = data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = data.get("days_of_week")
        days_of_month = data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        if api_call:
            db = c123_database.objects.filter(type = DBType.ELASTIC.value, name=db_id, node__name = node_id, ind_deleted = False).first()
        else:
            db = c123_database.objects.filter(type = DBType.ELASTIC.value, pk=db_id, ind_deleted = False).first()
        if not db:
            raise APIException410("database id {} not available".format(db_id))
        node = db.node
        job_id =  f"export_{db.name}_{node.name}"
        scheduler = settings.scheduler
        export_job = next((i for i in scheduler.get_jobs() if job_id in i.id), None)
        if export_job:
            export_job.reschedule(trigger=trigger)
        else:
             settings.scheduler.add_job(export_elasticsearch_index, replace_existing=True, trigger=trigger,  kwargs={"data": data}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        return JsonResponse({"notification": f"export index {db.name} of {node.name} {description} started"}, status = 200)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def AddElasticsearchIndex(request):
    current_user = request.user
    index_name = request.data.get('index_name')
    if not index_name or index_name.isspace():
          raise APIException406("index name cannot be blank")
    index_name = ''.join(index_name.split())
    node_id = request.data.get('node_id')
    num_shards = request.data.get('num_shards')
    num_replicas = request.data.get('num_replicas')
    api_call = request.data.get('rest_api')
    if api_call:
        node = c123_node.objects.filter(name = node_id,  type = DBType.ELASTIC.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id,  type = DBType.ELASTIC.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    allow_or_403(node)
    host = node.host
    auto_delete = request.data.get('auto_delete')
    auto_delete_days = request.data.get('auto_delete_days')
    
    existing_dbs = c123_database.objects.filter(name=index_name, node=node)
    if existing_dbs.exists():
        undeleted = existing_dbs.filter(ind_deleted = False)
        if undeleted.exists():
            index = undeleted.first()
        else:
            index = existing_dbs.first()
            index.ind_deleted = False
    
        index.created_by = current_user
        index.last_update_by = current_user
        index.status = Status.CREATING.value
        index.database_user = node.database_user
        index.database_pwd = node.database_pwd
        index.type = DBType.ELASTIC.value
        index.ex_params = {"num_shards": num_shards, "num_replicas": num_replicas}
        index.save()
    else:    
        index = c123_database.objects.create(name=index_name,
            type=DBType.ELASTIC.value,
            created_by=current_user,
            last_update_by=current_user,
            status=Status.CREATING.value,
            severity=Severity.NOT_OK.value,
            database_user=node.database_user,
            database_pwd=node.database_pwd,
            ex_params = {"num_shards": num_shards, "num_replicas": num_replicas},
            node = node)
    
    
    tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': c123_HostSerializer(host).data, 'db': object_to_json(index), 'auto_delete': auto_delete, 'created_by': current_user.id})
    task = Task.objects.create(task='create_elasticsearch_index_task',
                                    displayment='create elasticsearch index {}'.format(index_name),
                                    target_type=DBType.ELASTIC.value,
                                    target_id=index.id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def SwitchElasticsearchNode(request):
    node_id = request.data.get("standby_id")
    api_call = request.data.get('rest_api')
    current_user = request.user
    id_list = []
    name_list = []
    created_by = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    if not node:
        raise APIException404("node id {} not found".format(node_id))
    allow_or_403(node)
    master = c123_node.objects.filter(cluster = node.cluster, type = DBType.ELASTIC.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not master:
        raise APIException410(f"primary of node {node.name} not available")
    
    in_switchover = Task.objects.filter(task = 'switchover_elasticsearch_node_task', target_id = master.pk, status__in = [TaskState.PENDING.value, TaskState.RUNNING.value]).first()
    if in_switchover:
        raise APIException406(f"master {master.name} are already in switchover (task id {in_switchover.pk})")
    other_nodes = c123_node.objects.filter(cluster_role = ClusterRoles.STANDBY.value, type = DBType.ELASTIC.value, cluster = master.cluster, ind_deleted = False).exclude(pk = node.pk)
    id_list.append(master.pk)
    name_list.append(master.name)
    if other_nodes.count() > 0:
        for other in other_nodes:
            id_list.append(other.pk)
            name_list.append(other.name)
    
    node.status = Status.SWITCHOVER.value
    master.status = Status.DEMOTING.value.format(node.name)
    node.save()
    master.save()
    host_data = c123_HostSerializer(node.host).data
    node_data = object_to_json(node)
    master_data = object_to_json(master)
    tmp_data = json.dumps({'master_data': master_data, 'node_data': node_data, 'host_data': host_data, 'created_by': created_by.id, 'id_list': id_list, 'name_list': name_list})
    task = Task.objects.create(task='switchover_elasticsearch_node_task',
                                    displayment='switching node {} to master'.format(node.name, master.name),
                                    target_type=DBType.ELASTIC.value,
                                    target_id=master.id,
                                    org_id = master.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)

    
