from rest_framework import serializers
from clusterCore.models import c123_node, c123_inventory, c123_host, c123_cluster, Task, IntegratedService, ClusterSizing
from clusterCore.enums import DBType, ClusterRoles, Severity, Status, Inventory, Schedule, OperatingSystem, TaskState, DataBaseType, IntegratedServiceTypes, Node_Size_Allowed
from clusterCore.api.serializer import c123_CreateHostSerializer, c123_CreateTempClusterSerializer, object_to_json, RECREATION_CONFLICT, OS_CONFLICT, PRIMARY_GONE, FORBID_APP_HOST,OS_UNSUPPORTED
from clusterCore.exceptions import *
from clusterCore.tasks import check_os_version, drop_backup, format_rsa_key, license_after_create, ensure_primary_created, get_fully_qualified_domain_name, format_password
from apscheduler.triggers.interval import IntervalTrigger
import json
from django.utils import timezone
class c123_AddMariaDBStandbySerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'dir_123Cluster', 'ex_params']
    
    @license_after_create 
    def create(self, validated_data):
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
        if not proxmox_host:
            is_error, hostname = get_fully_qualified_domain_name(host_data["name"])
            if is_error:
                raise APIException401(hostname)
        else:
            hostname = host_data['name'] 
        
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException406(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first()  
        primary_id = validated_data['ex_params']['mariadb_master_id']
        primary_cluster = c123_node.objects.filter(type = DBType.MARIADB.value, pk=primary_id, ind_deleted = False).first()
        if not primary_cluster:
            raise APIException410(PRIMARY_GONE.format(primary_id))
        ensure_primary_created(primary_cluster)
        host_primary = primary_cluster.host
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
       
        create_run_linux = True
        host_exist = c123_host.objects.filter(name = hostname).exists()
        # if validated_data["host"]["name"] == primary_cluster.host.name:
        #     raise APIException409("This host already exist at cluster")
        node_name = validated_data["name"]
        mariadbstandby_exists = c123_node.objects.filter(name = node_name)
        if mariadbstandby_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MARIADB.value, node_name))
        mariadbstandby_to_update = mariadbstandby_exists.filter(ind_deleted = True).first()
        
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        (distributor_primary, release_primary) = check_os_version(host_primary.__dict__)
        if OperatingSystem.UNKNOWN.value in distributor_primary:
            raise APIException401(release_primary['stderr'])
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.UNKNOWN.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = distributor_primary
            release = release_primary
        
        if distributor_primary != distributor or release_primary != release:
            raise APIException406(OS_CONFLICT.format(distributor_primary, release_primary,distributor,release))
        host_data['ex_params'] = {}
        host_primary = primary_cluster.host
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.MARIADB.value
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.MARIADB.value
        validated_data ['version'] = primary_cluster.version
        validated_data['cluster_role'] = ClusterRoles.STANDBY.value
        validated_data["cluster"] = primary_cluster.cluster
        validated_data['database_user'] = primary_cluster.database_user
        validated_data['database_pwd'] = primary_cluster.database_pwd
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        mariadb_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.MARIADB.value)  
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = mariadb_sys.small_memory
                cpu_cores = mariadb_sys.small_cpu
                disk_gb = mariadb_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = mariadb_sys.medium_memory
                cpu_cores = mariadb_sys.medium_cpu
                disk_gb = mariadb_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = mariadb_sys.large_memory
                cpu_cores = mariadb_sys.large_cpu
                disk_gb = mariadb_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = mariadb_sys.very_large_memory
                cpu_cores = mariadb_sys.very_large_cpu
                disk_gb = mariadb_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        validated_data['cluster'] = primary_cluster.cluster
        validated_data['ex_params']['rep_pwd'] = format_password(validated_data['ex_params']['rep_pwd'])
        validated_data['ind_deleted'] = False
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        if mariadbstandby_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mariadbstandby_to_update.pk).update(**validated_data)
            mariadbstandby = c123_node.objects.get(pk=mariadbstandby_to_update.pk) 
        else:
            mariadbstandby = c123_node.objects.create(**validated_data)
        
        if self.context["request"].data.get("auto_delete"):
            mariadbstandby.auto_delete =  mariadbstandby.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
        
        mariadbstandby.ex_params['mariadb_root_password'] = primary_cluster.ex_params['mariadb_root_password']
        mariadbstandby.ex_params['ssl'] = primary_cluster.ex_params['ssl']
        repository = primary_cluster.ex_params["repository"]
        mariadbstandby.ex_params["repository"] = repository
        mariadbstandby.ex_params["distributor"] = distributor
        mariadbstandby.ex_params["release"] = release
        mariadbstandby.save()

        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mariadbstandby))
        else:
            c123_inventory.objects.filter(node = mariadbstandby).exclude(type = Inventory.EXPORT.value, ex_params__belong_to__in = [DBType.MYSQL.value, DBType.MARIADB.value]).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mariadbstandby.type, ex_params__node_version = mariadbstandby.version).delete()
        tmp_data = {'primary_data': object_to_json(primary_cluster), 'standby_data': object_to_json(mariadbstandby), 'host_primary': object_to_json(host_primary), 'host_standby': object_to_json(mariadbstandby.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'repository': repository, 'disk_gb': disk_gb, 'ram_gb': ram_gb, 'cpu_cores': cpu_cores, 'created_by': current_user.id}
        if proxmox_host:
            tmp_data['service_data'] = object_to_json(service)
            mariadbstandby.ex_params['service_id'] = service.pk
            mariadbstandby.save()
        task = Task.objects.create(task='add_mariadb_standby_task',
                                    displayment='adding mariadb standby {}'.format(mariadbstandby.name),
                                    target_type=DBType.MARIADB.value,
                                    target_id=mariadbstandby.id,
                                    org_id = mariadbstandby.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        
        return mariadbstandby, task
class c123_CreateMariaDBClusterSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'dir_123Cluster', "database_user", "database_pwd", 'ex_params']
    @license_after_create 
    def create(self, validated_data):
        
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
        if not proxmox_host:
            is_error, hostname = get_fully_qualified_domain_name(host_data["name"])
            if is_error:
                raise APIException401(hostname)
        else:
            hostname = host_data['name']
        software_version = validated_data['version']
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException406(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first()  
        create_run_linux = True
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        host_exist = c123_host.objects.filter(name = hostname).exists()
        mariadbcluster_exists = c123_node.objects.filter(name = node_name)
        if mariadbcluster_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MARIADB.value, node_name))
        mariadbcluster_to_update = mariadbcluster_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        host_data['ex_params'] = {}
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.UNKNOWN.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = validated_data['ex_params'].get('distributor')
            release = validated_data['ex_params'].get('release')
        
        if (OperatingSystem.CENTOS.value in distributor and int(release) > 8 and float(software_version) < 10.5 and software_version not in '10.11') or (OperatingSystem.UBUNTU.value in distributor  and int(release) > 20 and float(software_version) < 10.6 and software_version != '10.11') :
            raise APIException406(OS_UNSUPPORTED.format(DataBaseType.MARIADB.name, software_version, distributor, release))
        
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.MARIADB.value
        cluster_data['name'] = node_name
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value
        validated_data['type'] = DBType.MARIADB.value
        validated_data['cluster_role'] = ClusterRoles.PRIMARY.value
        validated_data['database_pwd'] = format_password(validated_data['database_pwd'])
        validated_data['ex_params']['mariadb_root_password'] = format_password(validated_data['ex_params']['mariadb_root_password'])
        own_cluster = c123_cluster.objects.filter(name = cluster_data["name"]).first()
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        validated_data["cluster"] = own_cluster
        validated_data['sizing'] =  self.context['request'].data.get('sizing')

        mariadb_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.MARIADB.value)  
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = mariadb_sys.small_memory
                cpu_cores = mariadb_sys.small_cpu
                disk_gb = mariadb_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = mariadb_sys.medium_memory
                cpu_cores = mariadb_sys.medium_cpu
                disk_gb = mariadb_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = mariadb_sys.large_memory
                cpu_cores = mariadb_sys.large_cpu
                disk_gb = mariadb_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = mariadb_sys.very_large_memory
                cpu_cores = mariadb_sys.very_large_cpu
                disk_gb = mariadb_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        validated_data['ind_deleted'] = False
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
          
        if mariadbcluster_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mariadbcluster_to_update.pk).update(**validated_data)
            mariadbcluster = c123_node.objects.get(pk=mariadbcluster_to_update.pk) 
        else:    
            mariadbcluster = c123_node.objects.create(**validated_data)
        mariadbcluster.ex_params['mariadb_master_id'] = mariadbcluster.pk 
        mariadbcluster.save()
        repository = mariadbcluster.ex_params.get("repository")
        if self.context["request"].data.get("auto_delete"):
            mariadbcluster.auto_delete =  mariadbcluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mariadbcluster.save()

        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mariadbcluster))
        else:
            c123_inventory.objects.filter(node = mariadbcluster).exclude(type = Inventory.EXPORT.value, ex_params__belong_to__in = [DBType.MYSQL.value, DBType.MARIADB.value]).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mariadbcluster.type, ex_params__node_version = mariadbcluster.version).delete()
        tmp_data = {'node_data': object_to_json(mariadbcluster), 'host_data': object_to_json(mariadbcluster.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'software_version': software_version, 'repository': repository, 'disk_gb': disk_gb, 'ram_gb': ram_gb, 'cpu_cores': cpu_cores, 'created_by': current_user.id}
        if proxmox_host:
            mariadbcluster.ex_params['service_id'] = service.pk
            mariadbcluster.save()
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(task='install_mariadb_standalone_task',
                                    displayment='creating mariadb node {}'.format(mariadbcluster.name),
                                    target_type=DBType.MARIADB.value,
                                    target_id=mariadbcluster.id,
                                    org_id = mariadbcluster.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        
        from clusterCore.scheduler import UpdateMySQLForkClusterRoles
        from django.conf import settings
        settings.scheduler.add_job(UpdateMySQLForkClusterRoles, IntervalTrigger(minutes= 1, timezone=settings.TIME_ZONE), replace_existing=True, next_run_time=timezone.now() + timezone.timedelta(minutes = 5), id= f'update_mysql_fork_roles_{mariadbcluster.cluster.id}',  kwargs={"cluster_id": mariadbcluster.cluster.id})
        
        return mariadbcluster, task