from rest_framework import serializers
from clusterCore.models import c123_node, c123_inventory, c123_host, c123_cluster, Task, IntegratedService, ClusterSizing
from clusterCore.enums import DBType, ClusterRoles, Severity, Status, Inventory, Schedule, OperatingSystem, TaskState, DataBaseType, IntegratedServiceTypes, Node_Size_Allowed
from clusterCore.api.serializer import c123_CreateHostSerializer, c123_CreateTempClusterSerializer, object_to_json, RECREATION_CONFLICT, OS_CONFLICT, PRIMARY_GONE, FORBID_APP_HOST, OS_UNSUPPORTED
from clusterCore.exceptions import *
from clusterCore.tasks import check_os_version, format_rsa_key, create_mongo_connection, drop_backup, license_after_create, ensure_primary_created, get_fully_qualified_domain_name, format_password
from apscheduler.triggers.interval import IntervalTrigger
import json
from django.utils import timezone
class c123_AddMongoNodeSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'sizing', 'database_user', 'database_pwd', 'dir_123Cluster', 'cluster_role', 'ex_params']
    @license_after_create 
    def create(self, validated_data):

        
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
        
   
        hostname = host_data['name'] 

        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException409(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first() 
        primary_id = validated_data['ex_params']['mongo_master_id']
        mongoprimary = c123_node.objects.filter(type = DBType.MONGO.value, pk=primary_id, ind_deleted = False).first()
        if not mongoprimary:
            raise APIException410(PRIMARY_GONE.format(primary_id))
        ensure_primary_created(mongoprimary)
        host_primary = mongoprimary.host  
        rs_name_exists = False
        app_rs = validated_data["ex_params"].get("replicaset_name").lower()
        if app_rs:
             client = create_mongo_connection(mongoprimary)
             if isinstance(client, tuple):
                 if hasattr(client[1], 'strerror'):
                     raise APIException401(client[1].strerror)
                 raise APIException401(str(client[1]))
 
             try:
                 rs_name = str(client.admin.command({ "hello" : 1 }).get('setName'))
             except Exception as e:
                 error = e.details.get("errmsg")
                 raise APIException403(error)
             
             client.close()
             
             if rs_name.lower() == app_rs:
                 rs_name_exists = True
        # check if there is already standby
        node_in_cluster = c123_node.objects.filter(type = DBType.MONGO.value, cluster=mongoprimary.cluster, cluster_role__in = [ClusterRoles.STANDBY.value, ClusterRoles.ARBITER.value], ind_deleted = False).exists()
       
        create_run_linux = True
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        mongonode_exists = c123_node.objects.filter(name = node_name)
        if mongonode_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MONGO.value, node_name))
        mongonode_to_update = mongonode_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_exist = c123_host.objects.filter(name = hostname).exists()
        host_data['name'] = hostname
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        (distributor_primary, release_primary) = check_os_version(host_primary.__dict__)
        if OperatingSystem.UNKNOWN.value in distributor_primary:
            raise APIException401(release_primary['stderr'])
        
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.UNKNOWN.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = distributor_primary
            release = release_primary
        
        if distributor_primary != distributor or release_primary != release:
            raise APIException406(OS_CONFLICT.format(distributor_primary, release_primary,distributor,release))
        host_data['ex_params'] = {}
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['database_user'] = mongoprimary.database_user
        validated_data['database_pwd'] =  mongoprimary.database_pwd
        validated_data['type'] = DBType.MONGO.value
        validated_data['cluster'] = mongoprimary.cluster
        validated_data['ind_deleted'] = False
        mongo_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.MONGO.value)  
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = mongo_sys.small_memory
                cpu_cores = mongo_sys.small_cpu
                disk_gb = mongo_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = mongo_sys.medium_memory
                cpu_cores = mongo_sys.medium_cpu
                disk_gb = mongo_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = mongo_sys.large_memory
                cpu_cores = mongo_sys.large_cpu
                disk_gb = mongo_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = mongo_sys.very_large_memory
                cpu_cores = mongo_sys.very_large_cpu
                disk_gb = mongo_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
           
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        if mongonode_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mongonode_to_update.pk).update(**validated_data)
            mongonode = c123_node.objects.get(pk=mongonode_to_update.pk)         
        else:    
            mongonode = c123_node.objects.create(**validated_data)
        mongoprimary.ex_params["replicaset_name"] = validated_data["ex_params"].get("replicaset_name")
        mongoprimary.save()
        if self.context["request"].data.get("auto_delete"):
            mongonode.auto_delete =  mongonode.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mongonode.save()

        mongonode.cluster = mongoprimary.cluster
        repository = mongoprimary.ex_params["repository"]
        mongonode.ex_params["repository"] = repository
        mongonode.ex_params["distributor"] = distributor
        mongonode.ex_params["release"] = release
        if mongonode.cluster_role == ClusterRoles.ARBITER.value:
            mongonode.ex_params["voting"] = True
        mongonode.save()
        task_name = 'add_node_to_rs_task'
        nodes_count = c123_node.objects.filter(cluster = mongoprimary.cluster, type = DBType.MONGO.value, ind_deleted = False, ex_params__voting=True).count()
        if nodes_count > 7:
            if mongonode.cluster_role == ClusterRoles.ARBITER.value:
                 raise APIException409('Arbiter must be voting, but maximum of 7 voting members reached.Cannot create.') 
            mongonode.ex_params['voting'] = False
            mongonode.save()
        mongonode.ex_params['ssl'] = mongoprimary.ex_params['ssl']
        mongonode.save()
        tmp_data = {'primary_data': object_to_json(mongoprimary), 'node_data': object_to_json(mongonode), 'nodes_count': nodes_count, 'host_primary': object_to_json(host_primary), 'host_standby': object_to_json(mongonode.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'repository': repository, 'node_in_cluster': node_in_cluster, 'rs_name_exists': rs_name_exists, 'disk_gb': disk_gb, 'ram_gb': ram_gb, 'cpu_cores': cpu_cores, 'created_by': current_user.id}
        if proxmox_host:
            mongonode.ex_params['service_id'] = service.pk
            mongonode.save()
            tmp_data['service_data'] = object_to_json(service)
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mongonode))
        else: 
            c123_inventory.objects.filter(node = mongonode).exclude(type = Inventory.EXPORT.value, ex_params__belong_to = mongonode.type).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mongonode.type, ex_params__node_version = mongonode.version).delete()
        task = Task.objects.create(task=task_name,
                                        displayment=f'adding mongodb {"arbiter" if mongonode.cluster_role == ClusterRoles.ARBITER.value else "node"} {mongonode.name}',
                                        target_type=DBType.MONGO.value,
                                        target_id=mongonode.id,
                                        org_id = mongonode.org_id,
                                        status=TaskState.PENDING.value,
                                        schedule=Schedule.NOW.value,
                                        parameters=json.dumps(tmp_data),
                                        created_by=current_user)   
        return mongonode, task
    
class c123_AddMongoShardedClusterSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', "database_user", "database_pwd", 'dir_123Cluster', 'ex_params']
    @license_after_create 
    def create(self, validated_data):
        is_error, hostname = get_fully_qualified_domain_name(validated_data["host"]["name"])
        if is_error:
            raise APIException401(hostname)
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException409(FORBID_APP_HOST.format(hostname))
        software_version = validated_data['version']
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first() 
        host_data = validated_data.pop('host')
        host_data['name'] = hostname
        rs_name = validated_data['ex_params'].get('replicaset_name') 
        create_run_linux = True
        host_exist = c123_host.objects.filter(name = hostname).exists()
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        mongocluster_exists = c123_node.objects.filter(name = node_name)
        if mongocluster_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MONGO.value, node_name))
        mongocluster_to_update = mongocluster_exists.filter(ind_deleted = True).first()
        
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.MONGO.value
        own_cluster = c123_cluster.objects.filter(name = cluster_data["name"]).first()
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        
        own_cluster.ex_params = {}
        own_cluster.ex_params["database_user"] = validated_data['database_user']
        own_cluster.ex_params["database_pwd"] = format_password(validated_data['database_pwd'])
        own_cluster.ex_params["version"] = validated_data["version"]
        own_cluster.ex_params["status"] = Status.CREATING.value
        own_cluster.ex_params["severity"] = Severity.NOT_OK.value
        own_cluster.ex_params["standalone"] = False
        own_cluster.ex_params["what"] = DataBaseType.MONGO.value
        own_cluster.ex_params["repository"] = validated_data["ex_params"]["repository"]
        own_cluster.ex_params["ssl"] = validated_data["ex_params"]["ssl"]
        own_cluster.type = DBType.MONGO.value
        own_cluster.created_by = current_user
        own_cluster.last_update_by = current_user
        own_cluster.save()
        
        (distributor, release) = check_os_version(host_data)
        if OperatingSystem.INVALID.value in distributor:
            raise APIException401(release['stderr'])
        if (OperatingSystem.UBUNTU.value in distributor and ((int(release) < 20 and float(software_version) >= 7.0) or (int(release) > 20 and float(validated_data['version']) < 6.0 ))) or (OperatingSystem.CENTOS.value in distributor and int(release) > 8 and float(validated_data['version']) < 6.0) :
            raise APIException406(OS_UNSUPPORTED.format(DataBaseType.MONGO.name, software_version, distributor, release))

        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.MONGO.value
        validated_data['cluster_role'] = ClusterRoles.BALANCER.value
        validated_data['sizing'] =  'SMALL'
        validated_data['cluster'] = own_cluster
        validated_data['ind_deleted'] = False
        validated_data['database_pwd'] = format(validated_data['database_pwd'])
        
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        if mongocluster_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mongocluster_to_update.pk).update(**validated_data)
            mongocluster = c123_node.objects.get(pk=mongocluster_to_update.pk) 
        else:    
            mongocluster = c123_node.objects.create(**validated_data) 
        mongocluster.ex_params['mongo_master_id'] = mongocluster.pk
        mongocluster.ex_params['voting'] = True
        mongocluster.save()
        if self.context["request"].data.get("auto_delete"):
            mongocluster.auto_delete =  mongocluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mongocluster.save()

        repository = mongocluster.ex_params["repository"]
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mongocluster))
        else:
            c123_inventory.objects.filter(node = mongocluster).exclude(type = Inventory.EXPORT.value, ex_params__belong_to = mongocluster.type).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mongocluster.type, ex_params__node_version = mongocluster.version).delete()
        tmp_data = json.dumps({'node_data': object_to_json(mongocluster), 'host_data': object_to_json(mongocluster.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'software_version': software_version, 'repository': repository, 'replicaset_name': rs_name, 'created_by': current_user.id})
        task = Task.objects.create(task='install_mongos_node_task',
                                    displayment='creating mongo sharded cluster {}'.format(own_cluster.name),
                                    target_type=DBType.MONGO.value,
                                    target_id=mongocluster.id,
                                    org_id = mongocluster.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
        return mongocluster, task
class c123_CreateMongoClusterSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', "database_user", "database_pwd", 'dir_123Cluster', 'ex_params']
    @license_after_create 
    def create(self, validated_data):
        
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
       
        hostname = host_data['name']
        
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException409(FORBID_APP_HOST.format(hostname))
        software_version = validated_data['version']
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first() 
        host_data['name'] = hostname
        rs_name = validated_data['ex_params'].get('replicaset_name') 
        create_run_linux = True
        host_exist = c123_host.objects.filter(name = hostname).exists()
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        mongocluster_exists = c123_node.objects.filter(name = node_name)
        if mongocluster_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MONGO.value, node_name))
        mongocluster_to_update = mongocluster_exists.filter(ind_deleted = True).first()
        
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.MONGO.value
        
        own_cluster = c123_cluster.objects.filter(name = cluster_data["name"]).first()
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.INVALID.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = validated_data['ex_params'].get('distributor')
            release = validated_data['ex_params'].get('release')
        
        if (OperatingSystem.UBUNTU.value in distributor and ((int(release) < 20 and float(software_version) >= 7.0) or (int(release) > 20 and float(validated_data['version']) < 6.0 ))) or (OperatingSystem.CENTOS.value in distributor and int(release) > 8 and float(validated_data['version']) < 6.0) :
            raise APIException406(OS_UNSUPPORTED.format(DataBaseType.MONGO.name, software_version, distributor, release))   
        
       

        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.MONGO.value
        validated_data['cluster_role'] = ClusterRoles.PRIMARY.value
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        mongo_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.MONGO.value)  
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = mongo_sys.small_memory
                cpu_cores = mongo_sys.small_cpu
                disk_gb = mongo_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = mongo_sys.medium_memory
                cpu_cores = mongo_sys.medium_cpu
                disk_gb = mongo_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = mongo_sys.large_memory
                cpu_cores = mongo_sys.large_cpu
                disk_gb = mongo_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = mongo_sys.very_large_memory
                cpu_cores = mongo_sys.very_large_cpu
                disk_gb = mongo_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        validated_data['cluster'] = own_cluster
        validated_data['ind_deleted'] = False
        validated_data['database_pwd'] = format(validated_data['database_pwd'])
        
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        if mongocluster_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mongocluster_to_update.pk).update(**validated_data)
            mongocluster = c123_node.objects.get(pk=mongocluster_to_update.pk) 
        else:    
            mongocluster = c123_node.objects.create(**validated_data) 
        mongocluster.ex_params['mongo_master_id'] = mongocluster.pk
        mongocluster.ex_params['voting'] = True
        mongocluster.save()
        if self.context["request"].data.get("auto_delete"):
            mongocluster.auto_delete =  mongocluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mongocluster.save()
            
        repository = mongocluster.ex_params["repository"]
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mongocluster))
        else:
            c123_inventory.objects.filter(node = mongocluster).exclude(type = Inventory.EXPORT.value, ex_params__belong_to = mongocluster.type).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mongocluster.type, ex_params__node_version = mongocluster.version).delete()
        if own_cluster.ex_params:
            own_cluster.ex_params['primary_count'] = 1
        else:
            own_cluster.ex_params = {'primary_count': 1}
        own_cluster.save()
        tmp_data = {'node_data': object_to_json(mongocluster), 'host_data': object_to_json(mongocluster.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'software_version': software_version, 'repository': repository, 'replicaset_name': rs_name, 'disk_gb': disk_gb, 'ram_gb': ram_gb, 'cpu_cores': cpu_cores, 'created_by': current_user.id}
        if proxmox_host:
            mongocluster.ex_params['service_id'] = service.pk
            mongocluster.save()
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(task='install_mongo_node_task',
                                    displayment='creating mongo cluster {}'.format(mongocluster.name),
                                    target_type=DBType.MONGO.value,
                                    target_id=mongocluster.id,
                                    org_id = mongocluster.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        from clusterCore.scheduler import UpdateMongoClusterRoles
        from django.conf import settings
        settings.scheduler.add_job(UpdateMongoClusterRoles, IntervalTrigger(minutes= 1, timezone=settings.TIME_ZONE), replace_existing=True, kwargs={"cluster_id": mongocluster.cluster.id}, next_run_time=timezone.now() + timezone.timedelta(minutes = 5), id=f"update_mongo_roles_{mongocluster.cluster.id}")
        return mongocluster, task
    

class c123_CreateMongoShardedClusterPrimary(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', "database_user", "database_pwd", 'dir_123Cluster', "cluster_role", 'ex_params']
    @license_after_create 
    def create(self, validated_data):
        is_error, hostname = get_fully_qualified_domain_name(validated_data["host"]["name"])
        if is_error:
            raise APIException401(hostname)
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException409(FORBID_APP_HOST.format(hostname))
        software_version = validated_data['version']
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first() 
        host_data = validated_data.pop('host')
        host_data['name'] = hostname
        rs_name = validated_data['ex_params'].get('replicaset_name') 
        create_run_linux = True
        host_exist = c123_host.objects.filter(name = hostname).exists()
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        mongocluster_exists = c123_node.objects.filter(name = node_name)
        if mongocluster_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MONGO.value, node_name))
        mongocluster_to_update = mongocluster_exists.filter(ind_deleted = True).first()
        
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        (distributor, release) = check_os_version(host_data)
        if OperatingSystem.INVALID.value in distributor:
            raise APIException401(release['stderr'])
        if (OperatingSystem.UBUNTU.value in distributor and ((int(release) < 20 and float(software_version) >= 7.0) or (int(release) > 20 and float(validated_data['version']) < 6.0 ))) or (OperatingSystem.CENTOS.value in distributor and int(release) > 8 and float(validated_data['version']) < 6.0) :
            raise APIException406(OS_UNSUPPORTED.format(DataBaseType.MONGO.name, software_version, distributor, release))

        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.MONGO.value
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        validated_data['ind_deleted'] = False
        validated_data['database_pwd'] = format(validated_data['database_pwd'])
        cluster_data = validated_data.pop('cluster')
        cluster = c123_cluster.objects.filter(name = cluster_data['name'], type = cluster_data['type']).first()
        validated_data['cluster'] = cluster
        
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host

        mongos_node = None
        mongos_host = None

        if validated_data["cluster_role"] == ClusterRoles.PRIMARY.value:
            mongos_node = c123_node.objects.filter(
                type=DBType.MONGO.value,
                cluster=cluster,
                cluster_role=ClusterRoles.BALANCER.value,
                ind_deleted=False
            ).first()

            if not mongos_node:
                raise APIException403("No mongos node available in cluster")
            if Task.objects.filter(target_id = mongos_node.id, task = 'install_mongos_node_task', status__in=[TaskState.PENDING.value, TaskState.RUNNING.value]).exists():
                raise APIException403("Sharded cluster initialization are currently in progress.Could not add new shards.")

            mongos_host = mongos_node.host

        if mongocluster_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mongocluster_to_update.pk).update(**validated_data)
            mongocluster = c123_node.objects.get(pk=mongocluster_to_update.pk) 
        else:    
            mongocluster = c123_node.objects.create(**validated_data) 
        mongocluster.ex_params['mongo_master_id'] = mongocluster.pk
        mongocluster.ex_params['voting'] = mongocluster.cluster_role == ClusterRoles.PRIMARY.value
        mongocluster.save()
        if self.context["request"].data.get("auto_delete"):
            mongocluster.auto_delete = mongocluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mongocluster.save()

        repository = mongocluster.ex_params["repository"]
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mongocluster))
        else:
            c123_inventory.objects.filter(node = mongocluster).exclude(type = Inventory.EXPORT.value, ex_params__belong_to = mongocluster.type).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mongocluster.type, ex_params__node_version = mongocluster.version).delete()

        if validated_data["cluster_role"] == ClusterRoles.BALANCER.value:
            mongocluster.ex_params['mongos_port'] = validated_data['port']
            mongocluster.save()

        tmp_data = json.dumps({'node_data': object_to_json(mongocluster), 'host_data': object_to_json(mongocluster.host), 'app_host': object_to_json(app_host), "mongos_host_data": object_to_json(mongos_host) if mongocluster.cluster_role == ClusterRoles.PRIMARY.value else None,"mongos_node_data": object_to_json(mongos_node) if mongocluster.cluster_role == ClusterRoles.PRIMARY.value else None, 'create_run_linux': create_run_linux, 'software_version': software_version, 'repository': repository, 'replicaset_name': rs_name, 'created_by': current_user.id})
        
        if mongocluster.cluster_role == ClusterRoles.PRIMARY.value:
            task = Task.objects.create(task='install_mongo_shard_task',
                                        displayment='creating mongo shard node {}'.format(mongocluster.name),
                                        target_type=DBType.MONGO.value,
                                        target_id=mongocluster.id,
                                        org_id = mongocluster.org_id,
                                        status=TaskState.PENDING.value,
                                        schedule=Schedule.NOW.value,
                                        parameters=tmp_data,
                                        created_by=current_user)
        elif mongocluster.cluster_role == ClusterRoles.BALANCER.value:
            task = Task.objects.create(task='install_mongos_node_task',
                                        displayment='creating mongos router node {}'.format(mongocluster.name),
                                        target_type=DBType.MONGO.value,
                                        target_id=mongocluster.id,
                                        org_id = mongocluster.org_id,
                                        status=TaskState.PENDING.value,
                                        schedule=Schedule.NOW.value,
                                        parameters=tmp_data,
                                        created_by=current_user)

        from clusterCore.scheduler import UpdateMongoClusterRoles
        from django.conf import settings
        settings.scheduler.add_job(UpdateMongoClusterRoles, IntervalTrigger(minutes=1, timezone=settings.TIME_ZONE), replace_existing=True, kwargs={"cluster_id": mongocluster.cluster.id}, next_run_time=timezone.now() + timezone.timedelta(minutes = 5), id=f"update_mongo_roles_{mongocluster.cluster.id}")
        return mongocluster, task
    
class c123_AddMongoShardedClusterNodeSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'sizing', 'database_user', 'database_pwd', 'dir_123Cluster', 'cluster_role', 'ex_params']
    @license_after_create 
    def create(self, validated_data):
        
        is_error, hostname = get_fully_qualified_domain_name(validated_data["host"]["name"])
        if is_error:
            raise APIException401(hostname)
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException409(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first() 
        primary_id = validated_data['ex_params']['mongo_master_id']
        mongoprimary = c123_node.objects.filter(type = DBType.MONGO.value, pk=primary_id, ind_deleted = False).first()
        if not mongoprimary:
            raise APIException410(PRIMARY_GONE.format(primary_id))
        ensure_primary_created(mongoprimary)
        host_primary = mongoprimary.host  
        rs_name_exists = False
        app_rs = validated_data["ex_params"].get("replicaset_name").lower()
        if app_rs:
             mongos_node = c123_node.objects.filter(type=DBType.MONGO.value, cluster=mongoprimary.cluster, cluster_role=ClusterRoles.BALANCER.value, ind_deleted=False).first()
             if not mongos_node:
                raise APIException403("No mongos node available in cluster")
             client = create_mongo_connection(mongos_node)
             if isinstance(client, tuple):
                 if hasattr(client[1], 'strerror'):
                     raise APIException401(client[1].strerror)
                 raise APIException401(str(client[1]))
 
             try:
                 with client:
                     shards_map = client.admin.command("getShardMap")
                     rs_name = next((v for v in shards_map['map'].values() if v.split('/')[0].strip() == app_rs.strip()), None) 
                     if rs_name:
                        rs_name_exists = True

             except Exception as e:
                 error = e.details.get("errmsg")
                 raise APIException403(error)
            
                 
        # check if there is already standby
        node_in_cluster = c123_node.objects.filter(type = DBType.MONGO.value, ex_params__mongo_master_id = primary_id, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).exists()
       
        create_run_linux = True
        host_data = validated_data.pop('host')
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        mongonode_exists = c123_node.objects.filter(name = node_name)
        if mongonode_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MONGO.value, node_name))
        mongonode_to_update = mongonode_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_exist = c123_host.objects.filter(name = hostname).exists()
        host_data['name'] = hostname
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        (distributor, release) = check_os_version(host_data)
        if OperatingSystem.UNKNOWN.value in distributor:
            raise APIException401(release['stderr'])
        (distributor_primary, release_primary) = check_os_version(host_primary.__dict__)
        if OperatingSystem.UNKNOWN.value in distributor:
            raise APIException401(release_primary['stderr'])
        if distributor_primary != distributor or release_primary != release:
            raise APIException406(OS_CONFLICT.format(distributor_primary, release_primary,distributor,release))
        host_data['ex_params'] = {}
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['database_user'] = mongoprimary.database_user
        validated_data['database_pwd'] =  mongoprimary.database_pwd
        validated_data['type'] = DBType.MONGO.value
        validated_data['cluster'] = mongoprimary.cluster
        validated_data['ind_deleted'] = False
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
           
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        if mongonode_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mongonode_to_update.pk).update(**validated_data)
            mongonode = c123_node.objects.get(pk=mongonode_to_update.pk)         
        else:    
            mongonode = c123_node.objects.create(**validated_data)
        if self.context["request"].data.get("auto_delete"):
            mongonode.auto_delete = mongonode.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mongonode.save()

        mongonode.cluster = mongoprimary.cluster
        repository = mongoprimary.ex_params["repository"]
        mongonode.ex_params["repository"] = repository
        mongonode.save()
        task_name = 'add_node_to_shard_task'
        nodes_count = c123_node.objects.filter(cluster = mongoprimary.cluster, type = DBType.MONGO.value, ind_deleted = False, ex_params__voting=True).count()
        if nodes_count > 7:
            mongonode.ex_params['voting'] = False
            mongonode.save()
        mongonode.ex_params['ssl'] = mongoprimary.ex_params['ssl']
        mongonode.save()
        tmp_data = json.dumps({'primary_data': object_to_json(mongoprimary), 'node_data': object_to_json(mongonode), 'nodes_count': nodes_count, 'host_primary': object_to_json(host_primary), 'host_standby': object_to_json(mongonode.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'repository': repository, 'node_in_cluster': node_in_cluster, 'rs_name_exists': rs_name_exists, 'created_by': current_user.id})
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mongonode))
        else: 
            c123_inventory.objects.filter(node = mongonode).exclude(type = Inventory.EXPORT.value, ex_params__belong_to = mongonode.type).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mongonode.type, ex_params__node_version = mongonode.version).delete()
        task = Task.objects.create(task=task_name,
                                        displayment=f'adding mongodb node {mongonode.name}',
                                        target_type=DBType.MONGO.value,
                                        target_id=mongonode.id,
                                        org_id = mongonode.org_id,
                                        status=TaskState.PENDING.value,
                                        schedule=Schedule.NOW.value,
                                        parameters=tmp_data,
                                        created_by=current_user)   
        return mongonode, task
