import re, json, pytz
from datetime import datetime
from rest_framework import generics, status
from rest_framework.permissions import IsAuthenticated
from django.http import JsonResponse
from django.utils import timezone
from rest_framework.decorators import api_view, permission_classes
from .serializer import c123_CreateMsSQLMasterSerializer
from clusterCore.models import c123_node, c123_host, c123_database, Task, c123_inventory, ClusterSizing
from clusterCore.enums import DBType, ClusterRoles, RestoreOptions, Status, Schedule, Inventory, Severity, TaskState, Node_Size_Allowed
from clusterCore.exceptions import *
from clusterCore.api.serializer import object_to_json, c123_DatabaseSerializer, c123_inventory_serializer, c123_NodeSerializer, format_size, get_create_node_response
from clusterCore.tasks import allow_or_403, create_ssh_connection, remote_command, checkPathExists, check_space, cleanup_logs, \
connect_to_sqlserver_instance, drop_mssql_node, get_cron_schedule, append_status_cmd, get_backup_description, export_mssql_node, \
export_mssql_database, get_avail_change_date
from clusterCore.scheduler import schedule_backup, MISFIRE_GRACE_TIME_BACKUP
from dateutil.parser import parse

class MsSQLClusterCreateView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_CreateMsSQLMasterSerializer
    def create(self, request, *args, **kwargs):
         return get_create_node_response(self, request, *args, **kwargs)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RemoveMsSQLSchedule(request):
    node_id = request.data.get("node_id")
    primary = c123_node.objects.filter(pk=node_id, type = DBType.MSSQL.value, cluster_role = ClusterRoles.PRIMARY.value).first()
    if not primary:
        raise APIException404("primary id {} not found".format(node_id))
    backup = c123_inventory.objects.filter(node = primary, type = Inventory.BACKUP.value, is_sched = True).last()
    from django.conf import settings
    if backup:
        job_id = backup.ex_params.get("job_id")
        job = settings.scheduler.get_job(job_id=job_id)
        if job:
            settings.scheduler.remove_job(job_id=job_id)
            notification = f"{job_id} removed"
            return JsonResponse({"notification": notification}, status=200)
        
        else:
            standbys = c123_node.objects.filter(type = DBType.MSSQL.value, cluster_role = ClusterRoles.STANDBY.value, cluster = primary.cluster)
            if standbys.exists():
                for standby in standbys:
                    backup = c123_inventory.objects.filter(type = Inventory.BACKUP.value, node = standby, is_sched = True).last()
                    if backup:
                        job_id = backup.ex_params.get("job_id")
                        job = settings.scheduler.get_job(job_id=job_id)
                        if job:
                            settings.scheduler.remove_job(job_id=job_id)
                            notification = f"{job_id} removed"
                            return JsonResponse({"notification": notification}, status=200)
                
                return JsonResponse({"notification": "no schedule backup found"}, status = 200)            
            else:
               return JsonResponse({"notification": "no schedule backup found"}, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def BackupMsSQLNode(request):
    """backup for mssql node"""
    schedule = request.data.get('schedule')
    node_id = request.data.get('node_id')
    compress = request.data.get("compress")
    encryption = request.data.get('encryption')
    backup_path = request.data.get('backup_path')
    update_schedule = request.data.get("update_schedule")
    api_call = request.data.get('rest_api')
    shell_safe_transform = str.maketrans({":": "_", ".": "_"})
    from django.conf import settings
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
         mssql_node = c123_node.objects.filter(name=node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    else:
         mssql_node = c123_node.objects.filter(pk=node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    if not mssql_node:
        raise APIException410("node id {} not available".format(node_id))
    allow_or_403(mssql_node)
    primary = c123_node.objects.filter(type = DBType.MSSQL.value, cluster = mssql_node.cluster, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
         raise APIException410(f"primary of {mssql_node.name} not found")
    
    backup_host = object_to_json(app_host)
    if schedule:
        client_timezone = pytz.timezone(request.data.get("client_timezone"))
        schedule_run_time_str = request.data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = request.data.get("days_of_week")
        days_of_month = request.data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
    if update_schedule and schedule: 
        backup = c123_inventory.objects.filter(node = primary, type = Inventory.BACKUP.value, is_sched = True).last()
        if backup:
            backup.ex_params['description'] = description
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            backup.save()
            job_id = backup.ex_params.get("job_id")
            notification = f"{backup.node} backup updated to '{description}'"
            job = settings.scheduler.get_job(job_id=job_id)
            if job:
               job.reschedule(trigger=trigger)
            else:
                settings.scheduler.add_job(schedule_backup, trigger=trigger, kwargs={"task_id":  backup.ex_params["task_id"]}, replace_existing=True, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP, id=job_id)
            return JsonResponse({"notification": notification}, status=200)
        
        else:
            standbys = c123_node.objects.filter(type = DBType.MSSQL.value, cluster_role = ClusterRoles.STANDBY.value, cluster = primary.cluster)
            if standbys.exists():
                for standby in standbys:
                    backup = c123_inventory.objects.filter(node = standby.pk, type = Inventory.BACKUP.value, is_sched = True).last()
                    if backup:
                        backup.ex_params['description'] = description
                        backup.ex_params['schedule_run_time'] = schedule_run_time_str
                        backup.ex_params['days_of_month'] = days_of_month
                        backup.ex_params['days_of_week'] = days_of_week
                        backup.save()
                        notification = f"{backup.node} backup updated to '{description}'"
                        job_id = backup.ex_params.get("job_id")
                        job = settings.scheduler.get_job(job_id=job_id)
                        if job:
                           job.reschedule(trigger=trigger)
                        else:
                            settings.scheduler.add_job(schedule_backup, trigger=trigger, kwargs={"task_id": backup.ex_params["task_id"]}, replace_existing=True, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP, id=job_id)
                        return JsonResponse({"notification": notification}, status=200)
                           
                
                return JsonResponse({"notification": "no schedule backup found"}, status = 200)            
            else:
               return JsonResponse({"notification": "no schedule backup found"}, status = 200)
    serializer = c123_inventory_serializer(data = request.data, partial = True)
    if serializer.is_valid():
    # create the task signature
        check_space(node = mssql_node, path=backup_path)
        size_scheme, created = ClusterSizing.objects.get_or_create(db_type = DBType.MSSQL.value)
        keep_days = getattr(size_scheme, f'{mssql_node.sizing.lower()}_backup_keep_days', None)
        if schedule:
            existing_backup = c123_inventory.objects.filter(node = mssql_node, type = Inventory.BACKUP.value, is_sched = True)
            if existing_backup.exists():
                raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for cluster.")
            
            if mssql_node.cluster_role == ClusterRoles.PRIMARY.value:
                standbys = c123_node.objects.filter(type = DBType.MSSQL.value, cluster_role = ClusterRoles.STANDBY.value, cluster = mssql_node.cluster)
            else:
                standbys = c123_node.objects.filter(type = DBType.MSSQL.value, cluster_role = ClusterRoles.STANDBY.value, cluster = mssql_node.cluster).exclude(pk = mssql_node.pk)
            
            if standbys.exists():
                for standby in standbys:
                    existing_backup = c123_inventory.objects.filter(node = standby, type = Inventory.BACKUP.value, is_sched = True)
                    if existing_backup.exists():
                        raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for cluster.")
            backup = serializer.create(serializer.validated_data)
            backup.node = mssql_node
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0}
            backup.ex_params["belong_to"] = DBType.MSSQL.value
            backup.ex_params["node_version"] = mssql_node.version
            backup.is_sched = schedule
            backup.ex_params["backup_type"] = "dump"
            backup.ex_params["duration"] = 0
            backup.ex_params["compress"] = compress
            backup.ex_params["encryption"] = encryption
            backup.ex_params["modified_date"] = ''
            backup.ex_params['description'] = description
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            # create a unique string with the date
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(mssql_node.name, now).translate(shell_safe_transform)
            backup.ex_params["backup_path"] = f'{backup_path}/{mssql_node.host.name}_{mssql_node.port}/{now}'.replace(".", "_")
            notification =  "{} backup {}".format(backup.node, description)
            job_id = f"{notification} ({client_timezone})"
            backup.ex_params["job_id"] = job_id
            backup.save()
            
            # serialize objects for the tasks
            node_data = object_to_json(mssql_node)
            host_data = object_to_json(mssql_node.host)
            created_by = request.user
            tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_data': object_to_json(backup), 'backup_host': backup_host, 'created_by': created_by.id, 'backup_type': 'physical', 'compress': compress})
            task = Task.objects.create(task='backup_sqlserver_instance_task',
                                            displayment='backup {}'.format(mssql_node.name), 
                                            target_type=DBType.MSSQL.value, 
                                            target_id=mssql_node.id, 
                                            org_id = mssql_node.org_id,
                                            schedule=Schedule.SCHEDULED.value, 
                                            status=TaskState.ON_DEMAND.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
            settings.scheduler.add_job(schedule_backup, trigger, kwargs={"task_id": task.pk}, replace_existing=True, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP, id=job_id)
        else:
            existing_backup = Task.objects.filter(target_id = mssql_node.id, task = 'backup_sqlserver_instance_task', status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
            if existing_backup:
                raise APIException409(f"Already runs backup for {mssql_node.name}")
            backup = serializer.create(serializer.validated_data)
            backup.node = mssql_node
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0}
            backup.ex_params["belong_to"] = DBType.MSSQL.value
            backup.ex_params["node_version"] = mssql_node.version
            backup.is_sched = schedule
            backup.ex_params["backup_type"] = "dump"
            backup.ex_params["duration"] = 0
            backup.ex_params["compress"] = compress
            backup.ex_params["encryption"] = encryption
            backup.ex_params["modified_date"] = ''
            # create a unique string with the date
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(mssql_node.name, now).translate(shell_safe_transform)
            backup.ex_params["backup_path"] = f'{backup_path}/{mssql_node.host.name}_{mssql_node.port}/{now}'.replace(".", "_")
            backup.save()
            notification = f"backup of {mssql_node.name}"
            # serialize objects for the tasks
            node_data = object_to_json(mssql_node)
            host_data = object_to_json(mssql_node.host)
            created_by = request.user
            tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_data': object_to_json(backup), 'backup_host': backup_host,  'created_by': created_by.id, 'backup_type': 'physical', 'compress': compress})
            task = Task.objects.create(task='backup_sqlserver_instance_task',
                                            displayment='backup {}'.format(mssql_node.name), 
                                            target_type=DBType.MSSQL.value, 
                                            target_id=mssql_node.id,
                                            org_id = mssql_node.org_id, 
                                            schedule=Schedule.NOW.value, 
                                            status=TaskState.PENDING.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
    else:
        print("invalid form")
        print(serializer.errors)        
        return JsonResponse({'error': serializer.errors}, status=404)
    return append_status_cmd(request, task, notification)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreMsSQLNode(request):
    """restore a mssql node from backup
    the call is made from client AJAX POST that contains the backup id"""
    
    bu_id = request.data.get('bu_id')
    # restore_option = 'this'/'latest'/'custom'
    restore_option = request.data.get('restore_option')
    restore_time = request.data.get('restore_time')
    autostart = request.data.get('autostart')
    api_call = request.data.get('rest_api')
    if api_call:
        backup = c123_inventory.objects.filter(name=bu_id, type = Inventory.BACKUP.value).first()
    else:
        backup = c123_inventory.objects.filter(pk=bu_id, type = Inventory.BACKUP.value).first()
    if not backup:
        raise APIException404("backup id {} not found".format(bu_id))
    node = backup.node
    primary = c123_node.objects.filter(type = DBType.MSSQL.value, cluster = node.cluster, cluster_role=ClusterRoles.PRIMARY.value, ind_deleted=False).first()
    if not primary:
        #because it can not exist, or exist as deleted
        raise APIException410(f"primary of {node.name} is not available")
        
    allow_or_403(primary)
    
    forbid_restore = Task.objects.filter(target_id = primary.id, task__in = ['restore_sqlserver_instance_task', 'backup_sqlserver_instance_task'], status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
    if forbid_restore:
        raise APIException409(f"Already runs backup/restore for primary {primary.name}")
    
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")

    backup_host = object_to_json(app_host)
    primary.status = Status.RESTORING.value
    primary.save()
    if RestoreOptions.THIS.value in restore_option:
        date = backup.created_date.strftime("%Y-%m-%d %H:%M:%S")
    elif RestoreOptions.CUSTOM.value in restore_option and restore_time:
        # example date = '2021-12-30 09:08:33'
        # date = datetime.strptime(restore_time, "%Y-%m-%dT%H:%M:%S.%f%z").astimezone()
        # date = date.strftime("%Y-%m-%d %H:%M:%S")
        date = restore_time
    else:
        date = ''
    standbys_id = []
    standby_in_cluster = c123_node.objects.filter(type = DBType.MSSQL.value, cluster=primary.cluster, cluster_role=ClusterRoles.STANDBY.value, ind_deleted = False).exists()
    standbys = []
    if standby_in_cluster:
        cluster_standbys = c123_node.objects.filter(type = DBType.MSSQL.value, cluster = primary.cluster, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False)
        for standby in cluster_standbys:
            standbys_id.append(standby.id)
            standbys.append({"host": object_to_json(standby.host), "node": object_to_json(standby)})
    
    # serialize objects for the tasks
    backup = object_to_json(backup)
    node_data = object_to_json(primary)
    host_data = object_to_json(primary.host)
    created_by = request.user
    if node.cluster_role == ClusterRoles.PRIMARY.value and c123_node.objects.filter(type = DBType.MSSQL.value, cluster = node.cluster, ind_deleted = False, Severity = Severity.OK.value, cluster_role = ClusterRoles.STANDBY.value).exists():
        autostart = True
    
    tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_data': backup, 'backup_host': backup_host, 'autostart': autostart, 'restore_option': restore_option, 'date': date, 'standbys_id': standbys_id, 'has_standby': standby_in_cluster, 'standbys': standbys, 'created_by': created_by.id})
    task = Task.objects.create(task='restore_sqlserver_instance_task',
                            displayment='restore mssql node {}'.format(node.name),
                            target_type=DBType.MSSQL.value,
                            target_id=primary.id,
                            org_id = primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CopyMsSQLDatabase(request):
    node_id = request.data.get("node_id")
    source_id = request.data.get("source_id")
    current_user = request.user
    db_option = request.data.get("option")
    #copy_option = 'one_time' or 'live'
    copy_option = request.data.get("copy_option")
    copy_name = request.data.get("copy_name")
    db_exists = request.data.get("db_exists")
    db_id = request.data.get("db_id")
    api_call = request.data.get('rest_api')
    if api_call:
        node_target = c123_node.objects.filter(type = DBType.MSSQL.value, name = node_id, ind_deleted = False).first()
    else:
        node_target = c123_node.objects.filter(type = DBType.MSSQL.value, pk = node_id, ind_deleted = False).first()
    if not node_target:
        raise APIException404("target node id {} not found".format(node_id))
    if api_call:
        node_source = c123_node.objects.filter(type = DBType.MSSQL.value, name = source_id,  ind_deleted = False).first()
    else:
        node_source = c123_node.objects.filter(type = DBType.MSSQL.value, pk = source_id,  ind_deleted = False).first()
    if not node_source:
        raise APIException404("source node id {} not found".format(source_id))
    
    primary_target = c123_node.objects.filter(cluster = node_target.cluster, type = DBType.MSSQL.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary_target:
        raise APIException404("no found primary for target node {}".format(node_target.name))
    allow_or_403(primary_target)
    primary_source = c123_node.objects.filter(cluster = node_source.cluster, type = DBType.MSSQL.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary_source:
        raise APIException404("no found primary for source node {}".format(node_source.name))
    allow_or_403(primary_source)
    #copy data
    if api_call:
        db_source = c123_database.objects.filter(type = DBType.MSSQL.value, name = db_id, node = node_source, ind_deleted = False).first()
    else:
        db_source = c123_database.objects.filter(type = DBType.MSSQL.value, pk = db_id, ind_deleted = False).first()
    if not db_source:
        raise APIException410("source database id {} not available".format(db_id))
    
    db_target = c123_database.objects.filter(pk = db_source.pk).first()
    if(db_option):
        # here we do for existing database
        match db_option:
            case "override":
                copy_name = db_target.name
                db_target.ind_started = False
                db_target.node = primary_target
                db_target.database_user = primary_target.database_user
                db_target.database_pwd = primary_target.database_pwd
                db_target.status = Status.REPLICATING.value
                db_target.save()
            case "copy":
                existing_dbs = c123_database.objects.filter(node = primary_target, name = copy_name)
                if not existing_dbs.exists():
                    db_target = c123_database.objects.create(node = primary_target, name = copy_name,
                        type = db_source.type, size = db_source.size,
                        database_user = primary_target.database_user, database_pwd = primary_target.database_pwd,
                        created_by = current_user, last_update_by = current_user,
                        severity = Severity.NOT_OK.value, status = Status.REPLICATING.value
                    )   
                else:
                    undeleted = existing_dbs.filter(ind_deleted = False)
                    if undeleted.exists():
                        db_target = undeleted.first()
                    else:
                        db_target = existing_dbs.first()
                        db_target.ind_deleted = False
                        
                    db_target.type = DBType.MSSQL.value
                    db_target.status = Status.REPLICATING.value
                    db_target.save()
                
                db_source.ind_started = False
                db_source.status = Status.COPYING.value
                db_source.save()
    else:
        #here we do for new database
        db_source.status = Status.COPYING.value
        db_source.ind_started = False
        db_source.save()
        db_target = c123_database.objects.create(node = primary_target, name = db_source.name,
        type = db_source.type, size = db_source.size,
        database_user = primary_target.database_user, database_pwd = primary_target.database_pwd,
        created_by = current_user, last_update_by = current_user,
        severity = Severity.NOT_OK.value, status = Status.REPLICATING.value
        )
    match copy_option:
        case 'mirror':
            source_port = request.data.get('source_port')
            target_port = request.data.get('target_port')
            is_async = request.data.get('async')
            tmp_data = json.dumps({'db_source': object_to_json(db_source), 'db_target' : object_to_json(db_target), 'node_source': object_to_json(primary_source), 'node_target': object_to_json(node_target), 'host_source': object_to_json(primary_source.host), 'host_target': object_to_json(primary_target.host), 'db_exists': db_exists, 'is_async': is_async, 'source_endpoint_port': source_port,  'target_endpoint_port': target_port, 'created_by': current_user.id})
            task_name = 'mirroring_sqlserver_database_task'
        case _:
            tmp_data = json.dumps({'db_source': object_to_json(db_source), 'db_target' : object_to_json(db_target), 'node_source': object_to_json(primary_source), 'node_target': object_to_json(node_target), 'host_source': object_to_json(primary_source.host), 'host_target': object_to_json(primary_target.host), 'db_option': db_option, 'db_exists': db_exists, 'copy_option': copy_option, 'copy_name': copy_name, 'created_by': current_user.id})
            task_name = 'copy_sqlserver_database_task'
           
    task = Task.objects.create(task=task_name,
                                displayment=f'copy database {db_source.name} into {primary_target.name}',
                                target_type=DBType.MSSQL.value,
                                target_id=primary_target.pk,
                                org_id = primary_target.org_id,
                                status=TaskState.PENDING.value,
                                schedule=Schedule.NOW.value,
                                parameters=tmp_data,
                                created_by=current_user)
    
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckMsSQLDatabaseSize(request):
    node_id = request.data.get('node_id')
    db_name = request.data.get("db_name")
    node = c123_node.objects.filter(pk = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    if not node:
        raise APIException410(f"node id {node_id} not available")
    
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")

    if db_name:
        check_size = f"SELECT SUM(mf.size * 8) AS SizeKB FROM sys.master_files mf JOIN sys.databases d ON mf.database_id = d.database_id WHERE DB_NAME(mf.database_id) = N'{db_name}' GROUP BY mf.database_id"
    else:
        check_size = "SELECT SUM(mf.size * 8) AS SizeKB FROM sys.master_files mf JOIN sys.databases d ON mf.database_id = d.database_id WHERE mf.database_id > 4 GROUP BY mf.database_id"
    
    client = connect_to_sqlserver_instance(node)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    
    try:
        cursor = client.cursor()
        cursor.execute(check_size)
        db_size = float(cursor.fetchone()[0])

        ssh_client = create_ssh_connection(app_host.name, app_host.username, app_host.password, app_host.private_key, app_host.ssh_port)
        if isinstance(ssh_client, tuple):
            if hasattr(ssh_client[1], 'strerror'):
                raise APIException401(ssh_client[1].strerror)
            raise APIException401(ssh_client[1])

        jsn = json.loads(remote_command(ssh_client,'df -P . | tail -1 | awk \'{print $4}\'', True), strict=False)
        if(not jsn['stderr'] == ''):
            print('check_mssql_database_size:', jsn['stderr'])
            return JsonResponse({'detail': jsn['stderr']}, status=404)
    
        free_space = float(jsn['stdout'])

        return JsonResponse({"size": db_size, "display_size": format_size(db_size), 'free_space': free_space}, status = 200)
    except Exception as e:
        if hasattr(e, 'strerror'):
            raise APIException401(e.strerror)
        raise APIException401(e)
    
    
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ExportMsSQLNode(request):

    """export for mssql node"""

    data = request.data
    data['created_by'] = request.user.id
    schedule = data.get('schedule')
    api_call = data.get('api_call')
    node_id = data.get('node_id')

    if not schedule:
        task = export_mssql_node(data)
        return append_status_cmd(request, task)
    else:
        from django.conf import settings
        client_timezone = pytz.timezone(data.get("client_timezone"))
        schedule_run_time_str = data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = data.get("days_of_week")
        days_of_month = data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        if api_call:
            node = c123_node.objects.filter(type = DBType.MSSQL.value, name=node_id, ind_deleted = False).first()
        else:
            node = c123_node.objects.filter(type = DBType.MSSQL.value, pk=node_id, ind_deleted = False).first()
        if not node:
            raise APIException410("node id {} not available".format(node_id))
        job_id =  f"export_{node.name}"
        scheduler = settings.scheduler
        export_job = next((i for i in scheduler.get_jobs() if job_id in i.id), None)
        if export_job:
            export_job.reschedule(trigger=trigger)
        else:
             settings.scheduler.add_job(export_mssql_node, replace_existing=True, trigger=trigger,  kwargs={"data": data}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        return JsonResponse({"notification": f"export {node.name} {description} started"}, status = 200)
   
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ImportMsSQLNode(request):
    """restore a mssql node from export
    the call is made from client AJAX POST that contains the backup id"""
    export_id = request.data.get('export_id')
    api_call = request.data.get('rest_api')
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        export = c123_inventory.objects.filter(name=export_id, type = Inventory.EXPORT.value).first()
    else:
        export = c123_inventory.objects.filter(pk=export_id, type = Inventory.EXPORT.value).first()
    
    if not export:
        raise APIException404(f"export id {export_id} not found")
    
    node = export.node
    primary = c123_node.objects.filter(cluster = node.cluster, type = DBType.MSSQL.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("primary not found")
    allow_or_403(primary)
    primary.status = Status.RESTORING.value
    primary.save()
    
    path = export.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")
        
    backup_host = object_to_json(app_host)
    standby_queryset = c123_node.objects.filter(cluster = node.cluster, type = DBType.MSSQL.value,  cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False)
    standbys = []
    for standby in standby_queryset:
        standbys.append({"node": object_to_json(standby), "host": object_to_json(standby.host)})
    # serialize objects for the tasks
    export = object_to_json(export)
    node_data = object_to_json(primary)
    host_data = object_to_json(primary.host)
    created_by = request.user
    tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_host': backup_host, 'standbys': standbys, 'backup_data': export, 'backup_type': 'dumpall', 'created_by': created_by.id})
    task = Task.objects.create(task='import_sqlserver_instance_task',
                            displayment='restore mssql node {}'.format(primary.name),
                            target_type=DBType.MSSQL.value,
                            target_id=primary.id,
                            org_id = primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckMsSQLDatabaseName(request):
    primary_id = request.data.get('node_id')
    db_name = request.data.get('name')
    primary = c123_node.objects.filter(pk=primary_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    if not primary:
        raise APIException410(f"primary id {primary_id} not available")
    if(re.match(r"^[a-zA-Z][a-zA-Z0-9_]{0,127}$", db_name)):
        # try to connect to the source
        client = connect_to_sqlserver_instance(primary)
        if isinstance(client, tuple):
            if hasattr(client[1], 'strerror'):
                raise APIException401(client[1].strerror)
            raise APIException401(client[1])
        try:
            cursor = client.cursor()
            cursor.execute("SELECT COUNT(*) FROM sys.databases WHERE name = ?", db_name)
            exists = cursor.fetchone()[0] > 0
        except Exception as e:
            if hasattr(e, 'strerror'):
                raise APIException401(e.strerror)
            raise APIException401(e)
        
        if exists:
              return JsonResponse({'error': "already exists"}, status=406)
        return JsonResponse({'response': "name available"}, status=200)
    else:
        return JsonResponse({'error': "invalid name"}, status=406)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropMsSQLDatabase(request):       
    db_id = request.data.get("db_id")
    node_id = request.data.get("node_id")
    api_call = request.data.get('rest_api')
    if api_call:
        db = c123_database.objects.filter(name = db_id, node__name = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    else:
        db = c123_database.objects.filter(pk = db_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    if not db:
        raise APIException410("database id {} not available".format(db_id))
    node = c123_node.objects.filter(pk = db.node.pk, type = DBType.MSSQL.value, ind_deletted = False).first()
    if not node:
        raise APIException404("node id {} not found".format(db.node.id))
    primary = c123_node.objects.filter(cluster = node.cluster, type = DBType.MSSQL.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("no found primary for node {}".format(node.name))
    allow_or_403(primary)
    db.status = Status.DROPPING.value
    db.save()
    cleanup_logs(db.pk, db.type)
    host_data = object_to_json(primary.host)
    node_data = object_to_json(primary)
    db_data = object_to_json(db)
    created_by = request.user
    
    tmp_data = json.dumps({'node_data': node_data, 'db_data': db_data, 'host_data': host_data, 'created_by': created_by.id})               
    task = Task.objects.create(task='drop_sqlserver_database_task',
                            displayment='drop database {}'.format(db.name),
                            target_type=DBType.MSSQL.value,
                            target_id=db.pk,
                            org_id = primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
         
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StopMsSQLNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    node.ind_started = False
    node.status = Status.STOPPING.value
    node.save()
    tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'created_by': current_user.id})
    task = Task.objects.create(task='stop_sqlserver_instance_task',
                                    displayment='stopping cluster {}'.format(node.name),
                                    target_type=DBType.MYSQL,
                                    target_id=node.id,
                                    org_id = node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StartMsSQLNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()      
    if not node:
        raise APIException404("node id {} not found".format(node_id))
    if node.ind_started:
         node.status = Status.RESTARTING.value
         node.save()
         tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'created_by': current_user.id})
         task = Task.objects.create(task='restart_sqlserver_instance_task',
                                    displayment='restarting cluster {}'.format(node.name),
                                    target_type=DBType.MSSQL.value,
                                    target_id=node.id,
                                    org_id = node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    else:
        node.ind_started = True
        node.status = Status.STARTING.value
        node.save()
        tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'created_by': current_user.id})
        task = Task.objects.create(task='start_sqlserver_instance_task',
                                    displayment='starting cluster {}'.format(node.name),
                                    target_type=DBType.MSSQL.value,
                                    target_id=node.id,
                                    org_id = node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
        return append_status_cmd(request, task)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def MsSQLDetails(request, id):
    node = c123_node.objects.filter(pk = id, type = DBType.MSSQL.value).first()
    if not node:
        raise APIException404("node id {} not found".format(id))
    response = {'node': c123_NodeSerializer(node).data}
    auto_backups = c123_inventory.objects.filter(node=node, type = Inventory.BACKUP.value, is_sched=True)
    if auto_backups.count() > 0:
        if auto_backups.last().status:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
        else:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}     
    else:
        response["auto_backup"]  = {
                'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}
    if node.cluster_role == ClusterRoles.PRIMARY.value:
        primary_backups = c123_inventory_serializer(c123_inventory.objects.filter(node_id = node.id, type = Inventory.BACKUP.value), many = True)
        response["backups"] = primary_backups.data
        primary_exports = c123_inventory_serializer(c123_inventory.objects.filter(node_id = node.id, type = Inventory.EXPORT.value), many = True)
        response["exports"] = primary_exports.data
        standbys = c123_NodeSerializer(c123_node.objects.filter(type = DBType.MSSQL.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False, cluster = node.cluster), many = True)
        response["nodes"] = standbys.data
        for i, standby in enumerate(response["nodes"]):
            backups = c123_inventory_serializer(c123_inventory.objects.filter(node_id = standby["id"], type = Inventory.BACKUP.value), many = True)
            response["nodes"][i]["backups"] = backups.data
            exports = c123_inventory_serializer(c123_inventory.objects.filter(node_id = standby["id"], type = Inventory.EXPORT.value), many = True)
            response["nodes"][i]["exports"] = exports.data
    else:
        standbys = c123_NodeSerializer(c123_node.objects.filter(type = DBType.MSSQL.value, cluster = node.cluster, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).exclude(pk = node.pk), many = True)
        response["nodes"] = standbys.data         
    cs = ClusterSizing.objects.filter(db_type = DBType.MSSQL.value).first()
    size_scheme = {
            Node_Size_Allowed.SMALL.name: {
                'cpu': cs.small_cpu,
                'disk': cs.small_disk,
                'memory': cs.small_memory,
                'backup': cs.small_backup_keep_days,
                'export': cs.small_export_keep_days,
            },
            Node_Size_Allowed.MEDIUM.name: {
                'cpu': cs.medium_cpu,
                'disk': cs.medium_disk,
                'memory': cs.medium_memory,
                'backup': cs.medium_backup_keep_days,
                'export': cs.medium_export_keep_days,
            },
            Node_Size_Allowed.LARGE.name: {
                'cpu': cs.large_cpu,
                'disk': cs.large_disk,
                'memory': cs.large_memory,
                'backup': cs.large_backup_keep_days,
                'export': cs.large_export_keep_days,
            },
            Node_Size_Allowed.VERY_LARGE.name: {
                'cpu': cs.very_large_cpu,
                'disk': cs.very_large_disk,
                'memory': cs.very_large_memory,
                'backup': cs.very_large_backup_keep_days,
                'export': cs.very_large_export_keep_days,
            }
        }
    response["avail_schemes"] = size_scheme
    response["databases"] = c123_DatabaseSerializer(c123_database.objects.filter(node = node, ind_deleted = False), many = True).data
    conn = connect_to_sqlserver_instance(node)
    if isinstance(conn, tuple):
        response['ind_error'] = True
        response["detail"] = conn[1].strerror if hasattr(conn[1], 'strerror') else str(conn[1])
        response['node'] = c123_NodeSerializer(c123_node.objects.filter(pk = id,  type = DBType.MSSQL.value).first()).data
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        return JsonResponse(response, status = 401)
    else:
        response["ind_error"] = c123_node.objects.filter(type = DBType.MSSQL.value, cluster = node.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists()  or c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).count() < 2
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        
    try:
        cursor = conn.cursor()
    except Exception as error:
        print("MsSQLDetails: ", str(error))
        response["detail"] = error
        node.severity = Severity.NOT_OK.value
        node.save()
        response['ind_error'] = True
        return JsonResponse(response, status=401)
    
    node.severity = Severity.OK.value
    node.ind_started = True
    node.save()
    
    try:
        cursor.execute("SELECT @@VERSION")
        version = cursor.fetchone()[0]
    except Exception as e:
        error = e.strerror if hasattr(e, 'strerror') else str(e)
        print("MsSQLDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    match = re.search(r"(\d{2,})\.\d+", version)
    if match:
        response['version'] = match.group(1)
        node.version = response['version']
        node.save()
    
    try:
        cursor.execute("SELECT DB_NAME()")
        response["db_id"] = cursor.fetchone()[0]
    except Exception as e:
        error = e.strerror if hasattr(e, 'strerror') else str(e)
        print("MsSQLDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    try:
        cursor.execute("SELECT CAST(value_in_use AS INTEGER) FROM sys.configurations WHERE name = 'default trace enabled'")
        response['logging'] = 'YES' if  bool(cursor.fetchone()[0]) is True else 'NO'
    except Exception as e:
        error = e.strerror if hasattr(e, 'strerror') else str(e)
        print("MsSQLDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    try:
        cursor.execute("SELECT ROUND(SUM(size * 8), 2) AS [Size_KB] FROM sys.databases AS db JOIN sys.master_files AS mf ON db.database_id = mf.database_id")
        response['size'] = float(cursor.fetchone()[0])
        node.size = response['size']
        node.save()
    except Exception as e:
        error = e.strerror if hasattr(e, 'strerror') else str(e)
        print("MsSQLDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    try:
        cursor.execute("EXEC xp_readerrorlog 0, 1,NULL, NULL, NULL, NULL,'DESC'")
        raw_logs = cursor.fetchmany(50)
        logs = []
        for row in raw_logs:
            log = {
                'date': row.LogDate,
                'process': row.ProcessInfo,
                'msg': row.Text
            }
            logs.append(log)
            response['history'] = logs
         
            query = '''
            SELECT 
                sql_process_cpu_time_ms = (cpu_ticks / (cpu_ticks_per_second / 1000)),
                total_cpu_usage_ms = (cpu_ticks_per_second / 1000)
            FROM sys.dm_os_sys_info
            '''

            cursor.execute(query)
            row = cursor.fetchone()
            
            sql_cpu_time_ms = row[0]
            total_cpu_time_ms = row[1]
            response['cpu_percent'] = round(100 * (sql_cpu_time_ms / total_cpu_time_ms))
            query = '''
        SELECT
            physical_memory_in_use_kb
        FROM sys.dm_os_process_memory
        '''
            cursor.execute(query)
            row = cursor.fetchone()
            
            response['display_ram'] = format_size(row[0])

            cursor.close()
    except Exception as e:
        error = e.strerror if hasattr(e, 'strerror') else str(e)
        print("MsSQLDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
      
    return JsonResponse(response, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def AddMsSQLDatabase(request):
    current_user = request.user
    db_name = request.data.get('db_name')
    if not db_name or db_name.isspace():
          raise APIException406("database name cannot be blank")
    db_name = ''.join(db_name.split())
    node_id = request.data.get('node_id')
    api_call = request.data.get('rest_api')
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    primary = c123_node.objects.filter(cluster = node.cluster, type = DBType.MSSQL.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("no found primary for node {}".format(node.name))
    allow_or_403(primary)
    auto_delete = request.data.get('auto_delete')
    auto_delete_days = request.data.get('auto_delete_days')
    recovery_model = request.data.get("recovery_model")
    
    existing_dbs = c123_database.objects.filter(name=db_name, node=primary)
    if existing_dbs.exists():
        undeleted = existing_dbs.filter(ind_deleted = False)
        if undeleted.exists():
            db = undeleted.first()
        else:
            db = existing_dbs.first()
            db.ind_deleted = False
        
        db.created_by = current_user
        db.last_update_by = current_user
        db.status = Status.CREATING.value
        db.database_user = primary.database_user
        db.database_pwd = primary.database_pwd
        db.type = DBType.MSSQL.value
        db.save()
    else:    
        db = c123_database.objects.create(name=db_name,
            type=DBType.MSSQL.value,
            created_by=current_user,
            last_update_by=current_user,
            status=Status.CREATING.value,
            severity=Severity.NOT_OK.value,
            database_user=primary.database_user,
            database_pwd=primary.database_pwd,
            node = primary)
    tmp_data = json.dumps({'node_data': object_to_json(primary), 'host_data': object_to_json(primary.host), 'db': object_to_json(db), 'auto_delete': auto_delete, 'recovery_model': recovery_model, 'created_by': current_user.id})
    task = Task.objects.create(task='create_sqlserver_database_task',
                                    displayment='creating mssql db {}'.format(db_name),
                                    target_type=DBType.MSSQL.value,
                                    target_id=db.pk,
                                    org_id = primary.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropMsSQLNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get('rest_api')
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.MSSQL.value, ind_deleted = False).first()

    if not node:
        raise APIException410(f"node id {node_id} not available")
    
    task = drop_mssql_node(node)
         
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ExportMsSQLDatabase(request):
    """export for database"""
    
    data = request.data
    data['created_by'] = request.user.id
    schedule = data.get('schedule')
    api_call = data.get('api_call')
    node_id = data.get('node_id')
    db_id = data.get('db_id')

    if not schedule:
        task = export_mssql_database(data)
        return append_status_cmd(request, task)
    else:
        from django.conf import settings
        client_timezone = pytz.timezone(data.get("client_timezone"))
        schedule_run_time_str = data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = data.get("days_of_week")
        days_of_month = data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        if api_call:
            db = c123_database.objects.filter(type = DBType.MSSQL.value, name=db_id, node__name = node_id, ind_deleted = False).first()
        else:
            db = c123_database.objects.filter(type = DBType.MSSQL.value, pk=db_id, ind_deleted = False).first()
        if not db:
            raise APIException410("database id {} not available".format(db_id))
        node = db.node
        job_id =  f"export_{db.name}_{node.name}"
        scheduler = settings.scheduler
        export_job = next((i for i in scheduler.get_jobs() if job_id in i.id), None)
        if export_job:
            export_job.reschedule(trigger=trigger)
        else:
             settings.scheduler.add_job(export_mssql_database, replace_existing=True, trigger=trigger,  kwargs={"data": data}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        return JsonResponse({"notification": f"export db {db.name} of {node.name} {description} started"}, status = 200)
    
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreMsSQLDatabase(request):
    """restore a mssql db from export
    the call is made from client AJAX POST that contains the backup id"""
    export_id = request.data.get('export_id')
    api_call = request.data.get('rest_api')
    autostart = True if request.data.get('autostart') == 'true' else False
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        export = c123_inventory.objects.filter(name=export_id, type = Inventory.EXPORT.value).first()
    else:
        export = c123_inventory.objects.filter(pk=export_id, type = Inventory.EXPORT.value).first()
        
    if not export:
        raise APIException404("export id {} not found".format(export_id))
    db = export.database
    node = db.node
    # serialize objects for the tasks
    primary = c123_node.objects.filter(cluster = node.cluster, type = DBType.MSSQL.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("primary not found")
    allow_or_403(primary)
    path = export.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")
    backup_host = object_to_json(app_host)
    db.status = Status.RESTORING.value
    db.save() 
    host_data = object_to_json(db.node.host)
    db_data = object_to_json(db)
    export = object_to_json(export)
    node_data = object_to_json(primary)
    created_by = request.user
    tmp_data = json.dumps({'db_data': db_data, 'node_data': node_data, 'host_data': host_data, 'backup_host': backup_host, 'backup_data': export, 'backup_type': 'dump', 'autostart': autostart, 'created_by': created_by.id})
    task = Task.objects.create(task=' import_sqlserver_database_task',
                            displayment='restore mssql database {}'.format(db.name),
                            target_type=DBType.MSSQL.value,
                            target_id=db.id,
                            org_id = primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)