from rest_framework import serializers
from clusterCore.models import c123_node, c123_inventory, c123_host, c123_cluster, Task, IntegratedService, ClusterSizing
from clusterCore.enums import DBType,ClusterRoles, OperatingSystem, Severity, Status, Schedule, Inventory, TaskState, DataBaseType, IntegratedServiceTypes, Node_Size_Allowed
from clusterCore.api.serializer import c123_CreateHostSerializer, c123_CreateTempClusterSerializer, object_to_json, RECREATION_CONFLICT, OS_CONFLICT, OS_UNSUPPORTED, FORBID_APP_HOST, PRIMARY_GONE
from clusterCore.exceptions import *
from clusterCore.tasks import check_os_version, drop_backup, format_rsa_key, license_after_create, ensure_primary_created, get_fully_qualified_domain_name, format_password
from apscheduler.triggers.interval import IntervalTrigger
import json
from django.utils import timezone
class c123_AddMySQLStandbySerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'dir_123Cluster', 'ex_params']
    
    @license_after_create 
    def create(self, validated_data):
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
        if not proxmox_host:
            is_error, hostname = get_fully_qualified_domain_name(host_data["name"])
            if is_error:
                raise APIException401(hostname)
        else:
            hostname = host_data['name'] 
        
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException406(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first() 
        primary_id = validated_data['ex_params']['mysql_master_id']
        primary_cluster = c123_node.objects.filter(type = DBType.MYSQL.value, pk=primary_id, ind_deleted = False).first()
        if not primary_cluster:
            raise APIException410(PRIMARY_GONE.format(primary_id))
        ensure_primary_created(primary_cluster)
        host_primary = primary_cluster.host
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        # check if there is already standby
        create_run_linux = True
        host_exist = c123_host.objects.filter(name = hostname).exists()
        mysqlstandby_exists = c123_node.objects.filter(name = node_name)
        if mysqlstandby_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MYSQL.value, node_name))
        mysqlstandby_to_update = mysqlstandby_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        (distributor_primary, release_primary) = check_os_version(host_primary.__dict__)
        if OperatingSystem.UNKNOWN.value in distributor_primary:
            raise APIException401(release_primary['stderr'])
        
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.INVALID.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = distributor_primary
            release = release_primary
        if distributor_primary != distributor or release_primary != release:
            raise APIException406(OS_CONFLICT.format(distributor_primary, release_primary,distributor,release))
        
        host_data['ex_params'] = {}
        host_primary = primary_cluster.host
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.MYSQL.value
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.MYSQL.value
        validated_data['cluster'] = primary_cluster.cluster
        validated_data ['version'] = primary_cluster.version
        validated_data['cluster_role'] = ClusterRoles.STANDBY.value
        validated_data['database_user'] = primary_cluster.database_user
        validated_data['database_pwd'] = primary_cluster.database_pwd
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        mysql_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.MYSQL.value)  
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = mysql_sys.small_memory
                cpu_cores = mysql_sys.small_cpu
                disk_gb = mysql_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = mysql_sys.medium_memory
                cpu_cores = mysql_sys.medium_cpu
                disk_gb = mysql_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = mysql_sys.large_memory
                cpu_cores = mysql_sys.large_cpu
                disk_gb = mysql_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = mysql_sys.very_large_memory
                cpu_cores = mysql_sys.very_large_cpu
                disk_gb = mysql_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        validated_data['cluster'] = primary_cluster.cluster
        validated_data['ind_deleted'] = False
        validated_data['ex_params']['rep_pwd'] = format_password(validated_data['ex_params']['rep_pwd'])
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
           
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        if mysqlstandby_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mysqlstandby_to_update.pk).update(**validated_data)
            mysqlstandby = c123_node.objects.get(pk=mysqlstandby_to_update.pk)     
        else:    
            mysqlstandby = c123_node.objects.create(**validated_data)
        
        if self.context["request"].data.get("auto_delete"):
            mysqlstandby.auto_delete =  mysqlstandby.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))

        mysqlstandby.ex_params['mysql_root_password'] = primary_cluster.ex_params['mysql_root_password']
        mysqlstandby.ex_params['ssl'] = primary_cluster.ex_params['ssl']
        mysqlstandby.ex_params['distributor'] = distributor
        mysqlstandby.ex_params['release'] = release
        repository = primary_cluster.ex_params["repository"]
        mysqlstandby.ex_params["repository"] = repository
        mysqlstandby.save()
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mysqlstandby))
        else:
            c123_inventory.objects.filter(node = mysqlstandby).exclude(type = Inventory.EXPORT.value, ex_params__belong_to__in = [DBType.MYSQL.value, DBType.MARIADB.value]).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mysqlstandby.type, ex_params__node_version = mysqlstandby.version).delete()
        tmp_data = {'primary_data': object_to_json(primary_cluster), 'standby_data': object_to_json(mysqlstandby), 'host_primary': object_to_json(host_primary), 'host_standby': object_to_json(mysqlstandby.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'repository': repository, 'ram_gb': ram_gb, 'disk_gb': disk_gb, 'cpu_cores': cpu_cores, 'created_by': current_user.id}
        if proxmox_host:
            mysqlstandby.ex_params['service_id'] = service.pk
            mysqlstandby.save()
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(task='add_mysql_standby_task',
                                    displayment='adding mysql standby {}'.format(mysqlstandby.name),
                                    target_type=DBType.MYSQL.value,
                                    target_id=mysqlstandby.id,
                                    org_id = mysqlstandby.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)            
        return mysqlstandby, task
class c123_CreateMySQLClusterSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'dir_123Cluster', "database_user", "database_pwd", 'ex_params']
    
    @license_after_create 
    def create(self, validated_data):

        software_version = validated_data['version'] 
        create_run_linux = True
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
        if not proxmox_host:
            is_error, hostname = get_fully_qualified_domain_name(host_data["name"])
            if is_error:
                raise APIException401(hostname)
        else:
            hostname = host_data['name']
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException406(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first() 
        host_exist = c123_host.objects.filter(name = hostname).exists()
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        mysqlcluster_exists = c123_node.objects.filter(name = node_name)
        if mysqlcluster_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.MYSQL.value, node_name))
        mysqlcluster_to_update = mysqlcluster_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.INVALID.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = validated_data['ex_params'].get('distributor')
            release = validated_data['ex_params'].get('release')
        
        if (OperatingSystem.CENTOS.value in distributor and ((int(release) > 8 and float(software_version) < 8.0) or (int(release) < 8 and float(software_version) > 8.0))) or (OperatingSystem.UBUNTU.value in distributor and ((int(release) > 18 and float(software_version) < 8.0) or (int(release) < 22 and float(software_version) > 8.0))):
            raise APIException406(OS_UNSUPPORTED.format(DataBaseType.MYSQL.name, software_version, distributor, release))
        
        host_data['ex_params'] = {}
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.MYSQL.value
        cluster_data['name'] = node_name
        own_cluster = c123_cluster.objects.filter(name = cluster_data["name"]).first()
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.MYSQL.value
        validated_data['cluster_role'] = ClusterRoles.PRIMARY.value
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        mysql_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.MYSQL.value)  
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = mysql_sys.small_memory
                cpu_cores = mysql_sys.small_cpu
                disk_gb = mysql_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = mysql_sys.medium_memory
                cpu_cores = mysql_sys.medium_cpu
                disk_gb = mysql_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = mysql_sys.large_memory
                cpu_cores = mysql_sys.large_cpu
                disk_gb = mysql_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = mysql_sys.very_large_memory
                cpu_cores = mysql_sys.very_large_cpu
                disk_gb = mysql_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        validated_data["cluster"] = own_cluster
        validated_data['database_pwd'] = format_password(validated_data['database_pwd'])
        validated_data['ex_params']['mysql_root_password'] = format_password(validated_data['ex_params']['mysql_root_password'])
        validated_data["ind_deleted"] = False
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
           c123_host.objects.filter(name=hostname).update(**host_data)
           host = c123_host.objects.get(name=hostname)
           create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
           
        
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        
        if mysqlcluster_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=mysqlcluster_to_update.pk).update(**validated_data)
            mysqlcluster = c123_node.objects.get(pk=mysqlcluster_to_update.pk) 
        else:    
            mysqlcluster = c123_node.objects.create(**validated_data)
           
        if self.context["request"].data.get("auto_delete"):
            mysqlcluster.auto_delete =  mysqlcluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mysqlcluster.save()
        
        if self.context["request"].data.get("auto_delete"):
            mysqlcluster.auto_delete =  mysqlcluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            mysqlcluster.save()

        mysqlcluster.ex_params["mysql_master_id"] = mysqlcluster.pk
        mysqlcluster.save()
        repository = mysqlcluster.ex_params.get("repository")
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = mysqlcluster))
        else:
            c123_inventory.objects.filter(node = mysqlcluster).exclude(type = Inventory.EXPORT.value, ex_params__belong_to__in = [DBType.MYSQL.value, DBType.MARIADB.value]).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = mysqlcluster.type, ex_params__node_version = mysqlcluster.version).delete()
        tmp_data = {'node_data': object_to_json(mysqlcluster), 'host_data': object_to_json(mysqlcluster.host), 'app_host': object_to_json(app_host), 'create_run_linux': create_run_linux, 'software_version': software_version, 'repository': repository, 'ram_gb': ram_gb, 'cpu_cores': cpu_cores, 'disk_gb': disk_gb, 'created_by': current_user.id}
        if proxmox_host:
            mysqlcluster.ex_params['service_id'] = service.pk
            mysqlcluster.save()
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(task='install_mysql_standalone_task',
                                    displayment='creating mysql node {}'.format(mysqlcluster.name),
                                    target_type=DBType.MYSQL.value,
                                    target_id=mysqlcluster.id,
                                    org_id = mysqlcluster.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        from django.conf import settings
        from clusterCore.scheduler import UpdateMySQLForkClusterRoles
        settings.scheduler.add_job(UpdateMySQLForkClusterRoles, IntervalTrigger(minutes= 1, timezone=settings.TIME_ZONE), replace_existing=True, next_run_time=timezone.now() + timezone.timedelta(minutes = 5), id= f'update_mysql_fork_roles_{mysqlcluster.cluster.id}',  kwargs={"cluster_id": mysqlcluster.cluster.id})
        return mysqlcluster, task