from rest_framework.decorators import api_view, permission_classes
from types import SimpleNamespace
from clusterCore.api.serializer import  c123_HostSerializer, c123_inventory_serializer, object_to_json, c123_NodeSerializer, format_size, get_create_node_response
from clusterCore.api.oracle.serializer import c123_OracleStandaloneSerializer, c123_RegisterCDBSerializer, c123_CreatePDBSerializer, c123_InstallCDBSerializer
from clusterCore.models import c123_node, c123_host, Task, OracleClusterSizing, c123_inventory, c123_Oracle_Standalone
from rest_framework.permissions import IsAuthenticated
from django.utils import timezone
from datetime import datetime
from rest_framework import generics, status
from django.http import JsonResponse
import re, json, pytz, oracledb, os, traceback
from clusterCore.tasks import revoke_license, allow_or_403, check_space, create_ssh_connection, cleanup_logs,connect_to_oracle_database,append_status_cmd, \
connect_to_oracle_database_using_params, remote_command, drop_pdb, get_cron_schedule, checkPathExists, format_password, get_backup_description, \
export_cdb_pdb, get_avail_change_date
from clusterCore.exceptions import *
from clusterCore.api.oracle.forms import FileUploadForm
from clusterCore.scheduler import schedule_backup, MISFIRE_GRACE_TIME_BACKUP
from clusterCore.enums import DBType, ClusterRoles, Schedule, Status, Severity, Inventory, RestoreOptions, TaskState, Node_Size_Allowed, PythonOS
from dateutil.parser import parse

####################################################################
# PDB
####################################################################
class CDBDetailsView(generics.RetrieveAPIView):
    serializer_class = c123_NodeSerializer
    permission_classes =[IsAuthenticated]
    queryset = c123_node.objects.all()
    def retrieve(self, request, *args, **kwargs):
        
        response = {}
        cdb_id = kwargs.get('pk')
        cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
        if not cdb:
            raise APIException410(f"CDB ID {cdb_id} is not available")
        
        # automatic backups
        auto_backups = c123_inventory.objects.filter(node=cdb, type = Inventory.BACKUP.value, is_sched=True)
        if auto_backups.count() > 0:
            if auto_backups.last().status:
                response["auto_backup"] = {
                        'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
            else:
                response["auto_backup"] = {
                        'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}
                    
        else:
            response["auto_backup"]  = {
                    'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}
        exports = c123_inventory_serializer(c123_inventory.objects.filter(node = cdb, type = Inventory.EXPORT.value), many = True)
        backups = c123_inventory_serializer(c123_inventory.objects.filter(node = cdb, type = Inventory.BACKUP.value), many = True)
        flashbacks = c123_inventory_serializer(c123_inventory.objects.filter(node = cdb, type = Inventory.FLASHBACK.value), many = True)
        response["exports"]  = exports.data
        response["backups"]  = backups.data
        response["flashbacks"]  = flashbacks.data
        try:
            conn = connect_to_oracle_database(cdb=cdb)
        except oracledb.DatabaseError as e:
            error, = e.args
            print("CDBDetailsView: ", str(error))
        
        if type(conn) is dict:
            cdb.ind_started = False
            cdb.severity = Severity.NOT_OK.value
            cdb.status = Status.DOWN.value
            cdb.save()
            response["detail"] = conn["detail"]
            response['ind_error'] = True
            serializer = self.get_serializer(cdb)
            response["node"] = serializer.data
            response['node']['ex_params']['avail_change'] =  get_avail_change_date(cdb)
            return JsonResponse(response, status = 401)
        else:
            cdb.ind_started = True
            cdb.severity = Severity.OK.value
            cdb.status = Status.OK.value
            cdb.save()
            response['node']['ex_params']['avail_change'] =  get_avail_change_date(cdb)
        
        try:
            cursor = conn.cursor()
        except Exception as error:
            response['ind_error'] = True
            response['detail'] = str(error)
            print("CDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 403)
       
        try:
            cursor.execute("SELECT PDB_NAME, PDB_ID, OP_TIMESTAMP, OPERATION, CLONED_FROM_PDB_NAME, DB_UNIQUE_NAME FROM cdb_pdb_history ORDER BY OP_TIMESTAMP")
            response["history"] = cursor.fetchall()
        except oracledb.DatabaseError as e:
            error, = e.args
            response["history"] = []
           
        
        try:
            cursor.execute("SELECT DBID, NAME, CURRENT_SCN, FORCE_LOGGING, open_mode, log_mode FROM v$database")
            response["info"] = cursor.fetchone()
            if response["info"]:
                cdb.ex_params["log_mode"] = response["info"][:-1]
                cdb.save()   
        except oracledb.DatabaseError as e:
            error, = e.args
            response['ind_error'] = True
            response['detail'] = str(error)
            print("CDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 403)
        
       
        try:
            
            cursor.execute("SELECT SUM(bytes)/1024 FROM dba_data_files")
            cdb.size = round(float(cursor.fetchone()[0]), 2)
            cdb.save()
            response['display_size'] = format_size(cdb.size)
            cursor.execute("""SELECT
  ROUND(VALUE/100, 2) CPU_Usage_Percentage
FROM
  V$SYSSTAT
WHERE
  NAME = 'CPU used by this session (percentage)';""")
            response['cpu_percent'] = cursor.fetchone()[0]
            cursor.execute("select sum(value)/1024 from v$parameter where name in ('sga_target','pga_aggregate_target');")
            response['display_ram'] = format_size(cursor.fetchone()[0])        
            
            cursor.close()
            conn.close()
        except oracledb.DatabaseError as e:
            error, = e.args
            response['ind_error'] = True
            response['detail'] = str(error)
            print("CDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 403)
        
        serializer = self.get_serializer(cdb)
        response["node"] = serializer.data
        response["ind_error"] = c123_node.objects.filter(cluster = cdb.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists() or c123_node.objects.filter(cluster = cdb.cluster, ind_deleted = False).count() < 2
        return JsonResponse(response, status = 200)
class PDBDetailsView(generics.RetrieveAPIView):
    serializer_class = c123_NodeSerializer
    permission_classes =[IsAuthenticated]
    queryset = c123_node.objects.all()
    def retrieve(self, request, *args, **kwargs):
        pdb_id = kwargs.get('id')
        cdb_id = kwargs.get('pk')
        pdb =  c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
        if not pdb:
            raise APIException410(f"PDB ID {pdb_id} is not available")
        cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
        if not cdb:
            raise APIException410(f"CDB ID {cdb_id} could not be found")
        curr_open_pdbs = c123_node.objects.filter( type = DBType.ORACLE_PDB.value,  cluster=cdb.cluster, ind_deleted = False, ind_started=True).count()
        start_time = timezone.now() + timezone.timedelta(hours=-1)
        # get heartbeat connection speeds
        #heart_beat = HeartBeat.objects.filter(node_id=pdb.id, node_type='PDB_Instance', creation_date__gte=start_time)
        #for hb in heart_beat:
        #    speeds.append(hb.connection_speed.microseconds/1000000)
        # connection_speed = speeds
        # get node performance
        #node_perf = NodePerformance.objects.filter(node_id=cdb.pk, node_type=cdb.__class__.__name__, creation_date__gte=start_time)
        #if node_perf:
        #    node_perf = node_perf
        #    cpucount = node_perf.first().cpucount
        #    total_mem = node_perf.last().swaptotal
        #    for np in node_perf:
        #        cpus.append(np.cpuusage)
        #        memswaps.append(np.swapusage)
        #    cpusage = cpus
        #    memswaps = memswaps
        #else:
        #    node_perf = None
        #    cpucount = 0
        #    total_mem = 0
        # forms
        # create a copy to cdb form
        copy_cdbs = {}
        copy_cdbs['copy_name'] = pdb.name
        cdbs_to_copy_queryset = c123_NodeSerializer(c123_node.objects.filter(type = DBType.ORACLE_CDB.value, version=cdb.version, ind_deleted = False).exclude(pk=cdb.pk), many = True)
        
        copy_cdbs["queryset"] = cdbs_to_copy_queryset.data
        response = {}
        response["copy_to_cdb_form"] = copy_cdbs
        # handle when max open pdb limit has been reached
        if cdb.ex_params:
            max_open_pdbs = cdb.ex_params.get('max_open_pdbs')
            response["can_be_opened"] = not max_open_pdbs  or curr_open_pdbs < max_open_pdbs
        backups_queryset = c123_inventory_serializer(c123_inventory.objects.filter(node = pdb, type = Inventory.BACKUP.value), many = True)
        exports_queryset = c123_inventory_serializer(c123_inventory.objects.filter(node = pdb, type = Inventory.EXPORT.value), many = True)
        flashbacks_queryset = c123_inventory_serializer(c123_inventory.objects.filter(node = pdb, type = Inventory.FLASHBACK.value), many = True)
        
        # backups
        response["backups"] = backups_queryset.data
        response["exports"] = exports_queryset.data
        response["flashbacks"] = flashbacks_queryset.data
        response["oracle_standbys"] = []
        
        ocs = OracleClusterSizing.objects.first()
        pdb_size_scheme = {
            Node_Size_Allowed.SMALL.name: {
                'cpu': ocs.small_cpu,
                'disk': ocs.small_disk,
                'memory': ocs.small_memory,
                'shared_temp': ocs.small_shared_temp,
                'backup': ocs.small_backup_keep_days,
                'restore': ocs.small_restore_points_keep_days,
                'export': ocs.small_export_keep_days,
            },
            Node_Size_Allowed.MEDIUM.name: {
                'cpu': ocs.medium_cpu,
                'disk': ocs.medium_disk,
                'memory': ocs.medium_memory,
                'shared_temp': ocs.medium_shared_temp,
                'backup': ocs.medium_backup_keep_days,
                'restore': ocs.medium_restore_points_keep_days,
                'export': ocs.medium_export_keep_days,
            },
            Node_Size_Allowed.LARGE.name: {
                'cpu': ocs.large_cpu,
                'disk': ocs.large_disk,
                'memory': ocs.large_memory,
                'shared_temp': ocs.large_shared_temp,
                'backup': ocs.large_backup_keep_days,
                'restore': ocs.large_restore_points_keep_days,
                'export': ocs.large_export_keep_days,
            },
            Node_Size_Allowed.VERY_LARGE.name: {
                'cpu': ocs.very_large_cpu,
                'disk': ocs.very_large_disk,
                'memory': ocs.very_large_memory,
                'shared_temp': ocs.very_large_shared_temp,
                'backup': ocs.very_large_backup_keep_days,
                'restore': ocs.very_large_restore_points_keep_days,
                'export': ocs.very_large_export_keep_days,
            }
        }
        response["avail_schemes"] =pdb_size_scheme
        # response["database_sizes"] = DataBaseSize.__members__
        cdb_json = c123_NodeSerializer(cdb)
        pdb_json = c123_NodeSerializer(pdb)
        response["cdb"] = cdb_json.data
        response["pdb"] = pdb_json.data
        response["is_superuser"] = self.request.user.is_superuser
        # HA rating
        response["ha_rating"] = 0
        if pdb.ind_started:
            response["ind_started"] = {'status': "up", 'class': "green-star"}
            response["ha_rating"] = response["ha_rating"] + 1.0
        else:
            response["ind_started"] = {'status': "down", 'class': "red-star"}
        # automatic backups
        auto_backups = c123_inventory.objects.filter(node=pdb, type = Inventory.BACKUP.value, is_sched=True)
        if auto_backups.count() > 0:
            if auto_backups.last().status:
                response["auto_backup"] = {
                        'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
            else:
                response["auto_backup"] = {
                        'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}
                    
        else:
            response["auto_backup"]  = {
                    'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}
        # standby configured
        #standbys_configured = PluggableStandby.objects.filter(pdb=pdb)
        #if len(standbys_configured) > 0:
        #    lag_time = StandbyLagTime.objects.filter(
        #        standby=standbys_configured.first()).order_by('-lag')
        #    if len(lag_time):
        #        lag = lag_time.first().lag
        #       if lag is not None and lag < timedelta(minutes=1):
        #            response["standby_conf"] = {
        #                'status': "Standby is configured", 'class': "green-star"}
        #             response["ha_rating"] = response["ha_rating"] + 1.0
        #        else:
        #             response["standby_conf"] = {
        #                'status': "Standby is configured", 'class': "red-green-star"}
        #             response["ha_rating"] = response["ha_rating"] + 0.5
        #    else:
        #        response["standby_conf"] = {
        #           'status': "Standby is configured", 'class': "red-star"}
        #else:
        #     response["standby_conf"] = {
        #        'status': "Standby isn't configured", 'class': "red-star"}
       
        #for sby in OracleStandby.objects.filter(cdb=cdb):
        #    response["oracle_standbys"].append({
        #        'name': sby.alias,
        #        'pk': sby.pk
        #    })
        
        # make connection
        conn = connect_to_oracle_database(cdb=cdb)
        if type(conn) is dict:
            cdb.severity = Severity.NOT_OK.value
            cdb.status = Status.DOWN.value
            cdb.ind_started = False
            cdb.save()
            response['detail'] = conn['detail']
            response['ind_error'] = True
            cdb_json = c123_NodeSerializer(cdb)
            response["cdb"] = cdb_json.data
            response['cdb']['ex_params']['avail_change'] =  get_avail_change_date(cdb)
            return JsonResponse(response, status = 401)
        else:
            cdb.severity = Severity.OK.value
            cdb.status = Status.OK.value
            cdb.ind_started = True
            cdb.save()
        
        try:    
            cursor = conn.cursor()
        except Exception as error:
            response['detail'] = str(error)
            response['ind_error'] = True
            print("PDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 403)
        # set container = PDB
        try:
            cursor.execute(f"ALTER SESSION SET CONTAINER={pdb.name}")
        except oracledb.DatabaseError as e:
            error, = e.args
            print("PDBDetailsView: ", str(error))
            pdb.ind_started = False
            pdb.severity = Severity.NOT_OK.value
            if 'ORA-65011' in str(error):
                pdb.ind_deleted = True
                pdb.status = Status.DETACHED.value
            else:
                pdb.status = Status.DOWN.value
            pdb.save()
            response['ind_error'] = True
            response['message'] = str(error)
            response['verbose'] = True
            pdb_json = c123_NodeSerializer(pdb)
            response["pdb"] = pdb_json.data
            response['pdb']['ex_params']['avail_change'] =  get_avail_change_date(pdb)
            return JsonResponse(response, status=404)
            
        pdb.ind_started = True
        pdb.ind_deleted = False
        pdb.severity = Severity.OK.value
        pdb.status = Status.OK.value
        pdb.save()
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(pdb)
        # get open mode
        try:
            cursor.execute("SELECT open_mode FROM v$database")
        except oracledb.DatabaseError as e:
            error, = e.args
            pdb.ind_started = False
            pdb.save()
        else:
            response["open_mode"] = cursor.fetchone()[0]
            if 'MOUNTED' in response["open_mode"]:
                pdb.ind_started = False
                pdb.save()
        # history
        try:
            cursor.execute(f"SELECT PDB_NAME, PDB_ID, OP_TIMESTAMP, OPERATION, CLONED_FROM_PDB_NAME, DB_UNIQUE_NAME from cdb_pdb_history where pdb_name='{pdb.name}' order by OP_TIMESTAMP")
        except oracledb.DatabaseError as e:
            error, = e.args
        else:
            response["history"] = cursor.fetchall()  
        # Get PDB current size
        try:
            cursor.execute(
                f"SELECT p.PDB_NAME \"Pluggable database name\", \
                decode(sum(d.BYTES), NULL, 'Not Available since the PDB is down', (sum(d.BYTES))) \"Size(MB)\" \
                FROM DBA_PDBS p, CDB_DATA_FILES d \
                WHERE p.PDB_ID=d.CON_ID(+) AND p.PDB_NAME='{pdb.name}' \
                GROUP BY p.PDB_ID, p.PDB_NAME \
                ORDER BY sum(d.BYTES) desc")
        except oracledb.DatabaseError as e:
            error,=  e.args
            print("PDBDetailsView: ", error)
            if "ORA-01219" in  str(error): 
                response["detail"] = str(error)
                response['ind_error'] = True
                pdb.ind_started = False
                pdb.severity = Severity.NOT_OK.value
                pdb.status = Status.DOWN.value
                pdb.save()
                return JsonResponse(response, status = 404)
            else:
                response["detail"] = str(error)
                response['ind_error'] = True
                return JsonResponse(response, status = 404)
        else:
            fetched_pdb_size = cursor.fetchone()
            if fetched_pdb_size is not None:
                try:
                   pdb_size = round(float(fetched_pdb_size[1])/1024, 2)
                except:
                    pdb_size = 0
            else:
                pdb_size = 0
            
            pdb.size = pdb_size
            pdb.save() 
        # limits
        try:
            cursor.execute(f"ALTER SESSION SET CONTAINER={pdb.name}")
        except oracledb.DatabaseError as e:
            error, = e.args
            response['detail'] = str(error)
            response['ind_error'] = True
            print("PDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 404)
        # max storage
        try:
            cursor.execute("SELECT PROPERTY_VALUE FROM database_properties WHERE property_name = 'MAX_PDB_STORAGE'")
        except oracledb.DatabaseError as e:
            error, = e.args
            response['detail'] = str(error)
            print("PDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 404)
        else:
            try:
                response["MAX_PDB_STORAGE"] = cursor.fetchone()[0]
                if response["MAX_PDB_STORAGE"] == 'UNLIMITED':
                    response["MAX_PDB_STORAGE"] = 10001
            except:
                response["MAX_PDB_STORAGE"] = 10001
        # Temporary tablespace size
        try:
            cursor.execute(f"ALTER SESSION SET CONTAINER={pdb.name}")
            cursor.execute("SELECT PROPERTY_VALUE FROM database_properties \
                                WHERE property_name = 'MAX_SHARED_TEMP_SIZE'")
        except oracledb.DatabaseError as e:
            error, = e.args
            response['detail'] = str(error)
            response['ind_error'] = True
            print("PDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 404)
        else:
            try:
                response["SHARED_TEMP_SIZE"] = cursor.fetchone()[0]
                if response["SHARED_TEMP_SIZE"] == 'UNLIMITED':
                    response["SHARED_TEMP_SIZE"] = 10001
            except:
                response["SHARED_TEMP_SIZE"] = 10001
        # info
        try:
            cursor.execute(f"SELECT PDB_ID, PDB_NAME, DBID, STATUS, CREATION_SCN, LOGGING, FORCE_LOGGING FROM dba_pdbs WHERE PDB_NAME='{pdb.name}'")
        except oracledb.DatabaseError as e:
            error, = e.args
            response['detail'] = str(error)
            print("PDBDetailsView: ", response['detail'])
            response['ind_error'] = True
            return JsonResponse(response, status = 404)
        else:
            response["info"] = cursor.fetchone()
        try:
            cursor.execute(f"ALTER SESSION SET CONTAINER={pdb.name}")
        except oracledb.DatabaseError as e:
            error, = e.args
            print("PDBDetailsView: ", str(error))
            if 'ORA-65011' in str(error):
                response['ind_error'] = True
                response['message'] = str(error)
                response['verbose'] = True
                return JsonResponse(response, status=404)
            
        # cpu count
        try:
            cursor.execute("SELECT value FROM v$parameter WHERE NAME='cpu_count'")
        except oracledb.DatabaseError as e:
            error, = e.args
            response['detail'] = str(error)
            response['ind_error'] = True
            print("PDBDetailsView: ", response['detail'])
            return JsonResponse(response, status = 404)
        else:
            response["CPU_COUNT"] = cursor.fetchone()[0]
            pdb.cpu_count = int(response["CPU_COUNT"])
            pdb.save()
        # get connected sessions
        cursor.execute(f"ALTER SESSION SET CONTAINER={pdb.name}")
        try:
            cursor.execute("SELECT count(*) FROM v$session")
        except oracledb.DatabaseError as e:
            error, = e.args
            response['detail'] = str(error)
            print("PDBDetailsView: ", response['detail'])
            response['ind_error'] = True
            return JsonResponse(response, status = 404)
        else:
            response["conn_sessions"] = cursor.fetchone()[0]
        cursor.close()
        conn.close()
        response["ind_error"] = c123_node.objects.filter(cluster = pdb.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists()  or c123_node.objects.filter(cluster = pdb.cluster, ind_deleted = False).count() < 2
        return JsonResponse(response, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
# pdbs from other containers
def Copy2CDB(request):
    #need to check
    """
    copy a pdb to a different cdb view
    the call is made from client AJAX POST
    CopytoCDBForm which includes
    the source PDB name and ID, the target CDB ID
    the target PDB name and after copy task startup options:
    source_pdb_restart = whether to do nothing or restart RO or RW the source pdb
    target_pdb_restart = whether to do nothing or restart RO or RW the target pdb
    once the task is complete 
    """
    standbys = request.data.get('standbys')
    also_start = request.data.get('also_start')
    autostart = request.data.get('autostart')
    source_restart = request.data.get('source_restart')
    target_restart = request.data.get('target_restart')
    # pdb_name = request.data.get('pdb_name')
    copy_name = request.data.get('copy').upper()
    cdb_id = request.data.get('cdb_id', None)
    pdb_id = request.data.get('pdb_id', None)
    file_dest = request.data.get('file_dest')
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name=location[1], type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    allow_or_403(cdb)
    if api_call:
        location = pdb_id.split('@')
        pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    else:
        pdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_PDB.value,  ind_deleted = False).first()
    if not pdb:
        raise APIException410(f"PDB ID {pdb_id} is not available")
    allow_or_403(pdb)
    pdb_name = pdb.name
    
    # compare versions of pdb with cdb
    cdb_of_pdb = c123_node.objects.filter(cluster=pdb.cluster, type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb_of_pdb:
        raise APIException410(f"CDB of PDB {pdb_name} is not available")
    if cdb.version[3] == '.':
        cdb_version = float(cdb.version[0:3])
    else:     
        cdb_version = float(cdb.version[0:4])
    if cdb_of_pdb.version[3] == '.':
        cdb_of_pdb_version = float(cdb_of_pdb.version[0:3])
    else:     
        cdb_of_pdb_version = float(cdb_of_pdb.version[0:4])
    if not cdb_of_pdb_version > 12:
        too_small_err_msg = f"The source database version is {cdb_of_pdb_version}. Source database version must be 12 and above. Can not continue"
        return JsonResponse({'status': 'false', 'message': too_small_err_msg}, status=200)
    if not cdb_version > 12:
        too_small_err_msg = f"The target database version is {cdb_version}. Target database version must be 12 and above. Can not continue"
        return JsonResponse({'status': 'false', 'message': too_small_err_msg}, status=200)     
    
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args
        raise APIException500(str(error))
        
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:     
        cursor = conn.cursor()
    except Exception as error:
        raise APIException500(str(error))
    try:
        cursor.execute(f"SELECT count(*) FROM dba_pdbs WHERE UPPER(pdb_name)=UPPER('{copy_name}')")
    except oracledb.DatabaseError as e:
        error, = e.args
        raise APIException500(str(error)) 
    if (not 0 in cursor.fetchone()):
        already_exist = "A PDB with the name {} already exists".format(copy_name)
        return JsonResponse({'status': 'false', 'message': already_exist}, status=200)    
    pdb.status = Status.COPYING.value
    pdb.ind_started = False
    pdb.save()
   
    # pdb_id = request.data.get('pdb_id')
    # pdb_name = request.data.get('pdb_name')
    
    new_name = copy_name
    target = cdb
    # pdb = c123_node.objects.get(pk=pdb_id)
    file_dest_source = pdb.ex_params['file_dest']
    try:
        conn = connect_to_oracle_database(cdb=target)
    except Exception as e:
        error, = e.args
        raise APIException500(str(error))
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:     
        cursor = conn.cursor()
    except Exception as error:
        raise APIException500(str(error))
    
    # 3. create database link
    # prepare the query
    pdb_name_to_copy = f"(DESCRIPTION = \
        (ADDRESS_LIST = \
        (ADDRESS = (PROTOCOL = TCP)(HOST = {cdb_of_pdb.host.name})(PORT = {cdb_of_pdb.port})) \
        ) \
        (CONNECT_DATA = \
        (SERVICE_NAME = {pdb_name}) \
        ))"
    # 4. drop any previous links if existed
    try:
        cursor.execute("DROP DATABASE LINK clone_link")
    except oracledb.DatabaseError as e:
        error, = e.args
        if str(error).find('ORA-02024') != -1:
            pass
        else:
            raise APIException500(str(error))
    # 5. create the link
    create_link = f"CREATE DATABASE LINK clone_link CONNECT TO {pdb.database_user} \
                    IDENTIFIED BY \"{pdb.database_pwd}\" USING '{pdb_name_to_copy}'"
    
    try:
        cursor.execute(create_link)
    except oracledb.DatabaseError as e:
        error, = e.args
        raise APIException500(str(error))
    # 6. Test the link in the target cdb.
    try:
        cursor.execute("SELECT count(*) FROM user_tables@clone_link")
    except oracledb.DatabaseError as e:
        error, = e.args
        raise APIException500(str(error))
    cursor.close()
    conn.close()
    
    if re.match(r'^\/(?!\/)[a-zA-Z0-9._-]+(\/[a-zA-Z0-9._-]+)*$', file_dest):
        try:
            client = create_ssh_connection(cdb.host.name, cdb.host.username, cdb.host.password, cdb.host.private_key, cdb.host.ssh_port)
            if isinstance(client, tuple):
                if hasattr(client[1], 'strerror'):
                    raise APIException401(client[1].strerror)
                raise APIException401(client[1])
            re.sub("//", "/", file_dest)
            file_path = f"{file_dest}/{new_name}"
            jsn = json.loads(remote_command(client, f"mkdir -p {file_path}", True), strict=False)
            jsn = json.loads(remote_command(client, f"chown oracle:oinstall {file_path}", True), strict=False) 
        except Exception as e:
            error, = e.args
            raise APIException500(str(error))     
    else:
        return JsonResponse({'status': 'false', 'message': "invalid file destination path"}, status=200)
   
    pdb.ex_params['file_dest'] = file_dest_source    
    pdb.status = Status.COPYING.value
    pdb.save()
            
    source_data = object_to_json(cdb_of_pdb)
    target_data = object_to_json(target)
    pdb_id = pdb.id
    host_source_data = c123_HostSerializer(pdb.host).data
    host_target_data = c123_HostSerializer(cdb.host).data
    created_by = request.user
    # create pdb copy node on target cdb
    pdb.pk = None
    pdb._state.adding = True
    pdb.name = copy_name
    pdb.ex_params['cdb_id'] = cdb.id
    pdb.ex_params['file_dest'] = file_dest
    pdb.status = Status.CREATING.value
    pdb.severity = Severity.NOT_OK.value
    pdb.dir_123Cluster = cdb_of_pdb.dir_123Cluster
    pdb.save() 
    tmp_data = json.dumps({'source_data': source_data, 'target_data': target_data, 'pdb_source_id': pdb_id, 'pdb_target_id': pdb.id,
                        'host_source_data': host_source_data, 'host_target_data': host_target_data,
                        'pdb_name': pdb_name, 'pdb_user': pdb.database_user, 'pdb_pwd': pdb.database_pwd,
                        'copy_name': copy_name, 'file_dest': file_dest,
                        'target_restart': target_restart, 'source_restart': source_restart, 'created_by': created_by.id})               
   
    task = Task.objects.create(task='copy_to_another_cdb_task',
                                displayment=f'copy {pdb.name} to {cdb.name}',
                                target_type=DBType.ORACLE_CDB.value,
                                target_id=pdb_id,
                                org_id = pdb.org_id,
                                schedule=Schedule.NOW.value,
                                status=TaskState.PENDING.value,
                                parameters=tmp_data,
                                created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Check_Credentials_PDB(request):
    username = request.data.get("username")
    password = format_password(request.data.get("password"))
    pdb_id = request.data.get("pdb_id")
    pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    if not pdb:
        raise APIException410(f"PDB ID {pdb_id} is not available")
    
    cdb_id = pdb.ex_params.get('cdb_id')
    cdb = c123_node.objects.get(pk=cdb_id, ind_deleted = False)
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    
    try:
        dsn_tns = oracledb.makedsn(cdb.host.name, pdb.port, pdb.name)
        oracledb.connect(user=username, password=password, dsn=dsn_tns)
    except oracledb.DatabaseError as e:
        error, = e.args
        print('Check_Credentials_PDB:', str(error))
        if "ORA-01033" in str(error):
             return JsonResponse({"result": "credentials valid"}, status = 200)
        return JsonResponse({"result": "credentials invalid"}, status = 406)
    else:
        return JsonResponse({"result": "credentials valid"}, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Update_PDB_Credendials(request):
     pdb_id = request.data.get("pdb_id")
     username = request.data.get("username")
     password = format_password(request.data.get("password"))
     api_call = request.data.get('rest_api')
     if api_call:
        location = pdb_id.split('@')
        pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
     else:
        pdb = c123_node.objects.filter(pk = pdb_id, type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
     if not pdb:
        raise APIException410(f"PDB ID {pdb_id} is not available")
     pdb.database_user = username
     pdb.database_pwd = password
     pdb.save()
     return JsonResponse({}, status = 200)
 
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Backup_PDB(request):
    schedule = request.data.get('schedule')
    backup_path = request.data.get('backup_path')
    node_id = request.data.get('node_id')
    update_schedule = request.data.get('update_schedule')
    compress = request.data.get("compress")
    incremental_level = request.data.get('incremental_level')
    api_call = request.data.get('rest_api')
    from django.conf import settings
    if api_call:
        location = node_id.split('@')
        pdb = c123_node.objects.filter(name=location[0],host__name=location[1], cluster_role = ClusterRoles.PDB.value, ind_deleted = False).first()
    else:
        pdb = c123_node.objects.filter(pk=node_id, cluster_role = ClusterRoles.PDB.value, ind_deleted = False).first()
    if not pdb:
        raise APIException410(f"PDB ID {node_id} is not available")
    allow_or_403(pdb)
    if schedule:
        client_timezone = pytz.timezone(request.data.get("client_timezone"))
        schedule_run_time_str = request.data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = request.data.get("days_of_week")
        days_of_month = request.data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        backup = c123_inventory.objects.filter(node = pdb, type = Inventory.BACKUP.value, is_sched = True).last()
        if backup:
            backup.ex_params['description'] = description
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            backup.save()
            job_id = backup.ex_params.get("job_id")
            job = settings.scheduler.get_job(job_id=job_id)
            if job:
                job.reschedule(trigger=trigger)
            else:
                settings.scheduler.add_job(schedule_backup, trigger=trigger, misfire_grace_time=60, replace_existing=True, kwargs={"task_id": backup.ex_params["task_id"]}, id=job_id)
            notification = f"{backup.node} backup updated to '{description}'"
            return JsonResponse({"notification": notification}, status=200)
        else:
            return JsonResponse({"notification": f"No schedule backup for {pdb.name}"}, status = 200)
    serializer = c123_inventory_serializer(data = request.data, partial = True)
    if serializer.is_valid():
        check_space(node = pdb, path=backup_path, source_path=pdb.dir_123Cluster)
        size_scheme = OracleClusterSizing.objects.first()
        keep_days = getattr(size_scheme, f'{pdb.sizing.lower()}_backup_keep_days', None)
        app_host = c123_host.objects.filter(ex_params__myself = True).first()
        if not app_host:
            raise APIException404("could not determine the app host")
        
        cdb = c123_node.objects.filter(cluster = pdb.cluster, type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
        if not cdb:
            raise APIException410(f"CDB of PDB {pdb.name} is not available")
        if schedule:
            existing_backup = c123_inventory.objects.filter(type = Inventory.BACKUP.value, is_sched = True, node = pdb)
            if existing_backup.exists():
                raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for pdb.")
            
            backup = serializer.create(serializer.validated_data)
            backup.node = pdb
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0, "duration": 0, "exit_code": 1, "compress": compress}
            backup.ex_params["belong_to"] = DBType.ORACLE_PDB.value
            backup.ex_params["incremental_level"] = incremental_level
            backup.ex_params["node_version"] = pdb.version
            backup.is_sched = schedule
            # create a unique string with the date
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(pdb.name, now)
            backup.ex_params["modified_date"] = ''
            backup.ex_params['description'] = description
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            notification =  "{} backup {}".format(backup.node, description)
            job_id = f"{notification} ({client_timezone})"
            backup.ex_params["job_id"] = job_id

            backup.ex_params["backup_path"] = "{}/{}_{}".format(backup_path, pdb.name,now)
            backup.ex_params["start_date"] = ''
            backup.save()
            # serialize objects for the tasks
            pdb_data = object_to_json(pdb)
            cdb_data = object_to_json(cdb)
            host_data = c123_HostSerializer(cdb.host).data
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
            tmp_data = json.dumps({'cdb_data': cdb_data, "pdb_data": pdb_data, 'host_data': host_data, 'backup_data': object_to_json(backup), 'backup_host': backup_host, 'pdb_name': pdb.name, 'pdb_id': pdb.id, 'created_by': created_by.id,  "compress": compress})
            task = Task.objects.create(task='backup_pdb_task',
                                        displayment=f'backup PDB {pdb.name}', 
                                        target_type=DBType.ORACLE_PDB.value, 
                                        target_id=pdb.id,
                                        org_id = pdb.org_id, 
                                        schedule=Schedule.SCHEDULED.value, 
                                        status=TaskState.ON_DEMAND.value,
                                        parameters=tmp_data,
                                        created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
            settings.scheduler.add_job(schedule_backup, trigger=trigger,  kwargs={"task_id": task.pk}, replace_existing=True, id=job_id, misfire_grace_time=60)
        else:
            existing_backup = Task.objects.filter(target_id = pdb.id, task = 'backup_pdb_task', status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
            if existing_backup:
                raise APIException409(f"Already runs backup for {pdb.name}")
            backup = serializer.create(serializer.validated_data)
            backup.node = pdb
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0, "duration": 0, "exit_code": 1,  "compress": compress}
            backup.ex_params["belong_to"] = DBType.ORACLE_PDB.value
            backup.ex_params["node_version"] = pdb.version
            backup.ex_params["incremental_level"] = incremental_level
            backup.is_sched = schedule
            backup.ex_params["modified_date"] = ''
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(pdb.name, now)
            backup.ex_params["backup_path"] = "{}/{}_{}".format(backup_path, pdb.name, now)
            backup.ex_params["start_date"] = ''
            backup.save()
            notification = f"backup of PDB {pdb.name}"
            # serialize objects for the tasks
            pdb_data = object_to_json(pdb)
            cdb_data = object_to_json(cdb)
            host_data = c123_HostSerializer(cdb.host).data
            created_by = request.user
            backup_host = c123_HostSerializer(app_host).data
            tmp_data = json.dumps({'cdb_data': cdb_data, "pdb_data": pdb_data, 'host_data': host_data, 'backup_host': backup_host,  'backup_data': object_to_json(backup), 'pdb_name': pdb.name, 'pdb_id': pdb.id, 'created_by': created_by.id,  "compress": compress})
            task = Task.objects.create(task='backup_pdb_task',
                                        displayment=f'backup PDB {pdb.name}', 
                                        target_type=DBType.ORACLE_PDB.value, 
                                        target_id=pdb.id,
                                        org_id = pdb.org_id, 
                                        schedule=Schedule.NOW.value, 
                                        status=TaskState.PENDING.value,
                                        parameters=tmp_data,
                                        created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
    else:
        print("invalid form")
        print(serializer.errors)        
        return JsonResponse({'error': serializer.errors}, status=404)
    return append_status_cmd(request, task, notification)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Backup_CDB(request):
    schedule = request.data.get('schedule')
    node_id = request.data.get('node_id')
    backup_path = request.data.get('backup_path')
    compress = request.data.get("compress")
    incremental_level = request.data.get("incremental_level")
    # options = 'all' or 'only'
    options = request.data.get('options')
    update_schedule = request.data.get('update_schedule')
    api_call = request.data.get('rest_api')
    from django.conf import settings
    if api_call:
        location = node_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1], type = DBType.ORACLE_CDB.value,cluster_role = ClusterRoles.CDB.value, ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=node_id, type = DBType.ORACLE_CDB.value,cluster_role = ClusterRoles.CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {node_id} is not available")
    allow_or_403(cdb)
    if schedule:
        client_timezone = pytz.timezone(request.data.get("client_timezone"))
        schedule_run_time_str = request.data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = request.data.get("days_of_week")
        days_of_month = request.data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
    if update_schedule and schedule:
        backup = c123_inventory.objects.filter(node = cdb, type = Inventory.BACKUP.value, is_sched = True).last()
        if backup:
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            backup.ex_params['description'] = description
            backup.save()
            job_id = backup.ex_params.get("job_id")
            job = settings.scheduler.get_job(job_id=job_id)
            if job:
                job.reschedule(trigger=trigger)
            else:
                settings.scheduler.add_job(schedule_backup, trigger=trigger, misfire_grace_time=60, replace_existing=True, kwargs={"task_id": backup.ex_params["task_id"]}, id=job_id)
           
            notification = f"{backup.node} updated to '{description}'"
            return JsonResponse({"notification": notification}, status=200)
        else:
            return JsonResponse({"notification": f"No schedule backup for {cdb.name}"}, status = 200)
    serializer = c123_inventory_serializer(data = request.data, partial = True)
    if serializer.is_valid():
        check_space(node = cdb, path=backup_path, source_path=cdb.dir_123Cluster)
        size_scheme = OracleClusterSizing.objects.first()
        keep_days = getattr(size_scheme, f'{cdb.sizing.lower()}_backup_keep_days', None)
        app_host = c123_host.objects.filter(ex_params__myself = True).first()
        if not app_host:
            raise APIException404("could not determine the app host")
        if schedule:
            existing_backup = c123_inventory.objects.filter(type = Inventory.BACKUP.value, is_sched = True, node = cdb)
            if existing_backup.exists():
                raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for CDB.")
            
            backup = serializer.create(serializer.validated_data)
            backup.node = cdb
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0, "duration": 0, "exit_code": 1}
            backup.ex_params["belong_to"] = DBType.ORACLE_CDB.value
            backup.ex_params["node_version"] = cdb.version
            backup.ex_params["compress"] = compress
            backup.ex_params["incremental_level"] = incremental_level
            backup.is_sched = schedule
            backup.ex_params["modified_date"] = ''
            backup.ex_params['description'] = description
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(cdb.name, now)
            backup.ex_params["backup_path"] =  "{}/{}_{}".format(backup_path, cdb.name, now)
            notification =  "{} backup {}".format(backup.node, description)
            job_id = f"{notification} ({client_timezone})"
            backup.ex_params["job_id"] = job_id
            backup.save()
            # serialize objects for the tasks
            cdb_data = object_to_json(cdb)
            host_data = c123_HostSerializer(cdb.host).data
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
            tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data, 'backup_host': backup_host, 'backup_data': object_to_json(backup), 'options': options, 'created_by': created_by.id})
            task = Task.objects.create(task='backup_cdb_task',
                                        displayment=f'backup CDB {cdb.name}', 
                                        target_type=DBType.ORACLE_CDB.value, 
                                        target_id=cdb.id,
                                        org_id = cdb.org_id, 
                                        schedule=Schedule.SCHEDULED.value, 
                                        status=TaskState.ON_DEMAND.value,
                                        parameters=tmp_data,
                                        created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
            settings.scheduler.add_job(schedule_backup, trigger=trigger,  kwargs={"task_id": task.pk}, id=job_id, replace_existing=True, misfire_grace_time=60)
        else:
            existing_backup = Task.objects.filter(target_id = cdb.id, task = 'backup_cdb_task', status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
            if existing_backup:
                raise APIException409(f"Already runs backup for {cdb.name}")
            backup = serializer.create(serializer.validated_data)
            backup.node = cdb
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0, "duration": 0, "exit_code": 1}
            backup.ex_params["belong_to"] = DBType.ORACLE_CDB.value
            backup.is_sched = schedule
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(cdb.name, now)
            backup.ex_params["backup_path"] =  "{}/{}_{}".format(backup_path, cdb.name, now)
            backup.ex_params["compress"] = compress
            backup.ex_params["incremental_level"] = incremental_level
            backup.save()
            notification = f"backup of CDB {cdb.name}"
            # serialize objects for the tasks
            cdb_data = object_to_json(cdb)
            host_data = c123_HostSerializer(cdb.host).data
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
            tmp_data = json.dumps({'cdb_data': cdb_data, "pdb_data": cdb_data, 'host_data': host_data, 'backup_host': backup_host, 'backup_data': object_to_json(backup), 'cdb_name': cdb.name, 'cdb_id': cdb.id, 'options': options, 'created_by': created_by.id})
            task = Task.objects.create(task='backup_cdb_task',
                                        displayment=f'backup CDB {cdb.name}', 
                                        target_type=DBType.ORACLE_CDB.value, 
                                        target_id=cdb.id,
                                        org_id = cdb.org_id,
                                        schedule=Schedule.NOW.value, 
                                        status=TaskState.PENDING.value,
                                        parameters=tmp_data,
                                        created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
    else:
        print("invalid form")
        print(serializer.errors)        
        return JsonResponse({'error': serializer.errors}, status=404)
    return append_status_cmd(request, task, notification)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RemoveOracleSchedule(request):
    node_id = request.data.get("node_id")
    node = c123_node.objects.filter(pk=node_id,  type = DBType.ORACLE_PDB.value,  ind_deleted = False).first()
    if not node:
        raise APIException410(f"PDB ID {node_id} is not available")
    allow_or_403(node)
    from django.conf import settings
    backup = c123_inventory.objects.filter(node = node, is_sched = True).last()
    if backup:
        job_id = backup.ex_params.get("job_id")
        job = settings.scheduler.get_job(job_id=job_id)
        if job:
            settings.scheduler.remove_job(job_id=job_id)
            notification =  notification = f"{job_id} removed"
            return JsonResponse({"notification": notification}, status=200)          
        else:
            return JsonResponse({"notification": f"no schedule backup for {node.name}"}, status = 200) 
####################################################################
# PDB CRUD
####################################################################
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Open_PDB(request):
    """Open an Oracle Pluggable database"""
    pdb_id = request.data.get('pdb_id')
    pdb_name = request.data.get('pdb_name')
    cdb_id = request.data.get('cdb_id')
    option = request.data.get('option')
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    
    if api_call:
        location = pdb_id.split('@')
        pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    else:
        pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    if not pdb:
        raise APIException410(f"PDB ID {pdb_id} is not available")
    
    allow_or_403(pdb)
    cdb_data = object_to_json(cdb)
    
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args         
        raise APIException401(str(error))
    
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:     
        cursor = conn.cursor()
    except Exception as error:
        raise APIException401(str(error)) 
       
    # Check the database current open mode:
    
    try:
        cursor.execute(f"SELECT OPEN_MODE FROM v$PDBS WHERE NAME='{pdb_name}'")
    except Exception as e:
        error, = e.args        
        raise Exception(str(error))
    else:
        open_mode = cursor.fetchone()[0]    
    open_query = f"ALTER PLUGGABLE DATABASE {pdb_name} OPEN "
    # the database is already open in the requested mode
    
    if open_mode == 'READ WRITE':
        if option == 'RW':
            already_open_msg = f"pluggable database {pdb_name} already open, please close it first"
            pdb.ind_started = True 
            pdb.save()           
            return JsonResponse({'msg':already_open_msg}, status=200)
        elif option == 'RO':
            open_query += "READ ONLY"
    elif open_mode == 'READ ONLY':
        if option == 'RO':
            already_open_msg = f"pluggable database {pdb_name} already open, please close it first"
            pdb.ind_started = True
            pdb.save()            
            return JsonResponse({'msg':already_open_msg}, status=200)
    elif open_mode == 'MOUNTED':
        if option == 'RO':
            open_query += "READ ONLY"
    cursor.close()
    conn.close()        
    host_data = c123_HostSerializer(pdb.host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'pdb_id': pdb.pk, 'host_data':host_data, 'open_query': open_query, 'created_by': created_by.id})
    task = Task.objects.create(task='open_pdb_task',
                            displayment=f'open {pdb.name}',
                            target_type=DBType.ORACLE_PDB.value,
                            target_id=pdb.id,
                            org_id = pdb.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    pdb.severity = Severity.NOT_OK.value
    pdb.status = Status.OPENING.value
    # pdb.ind_started = True
    pdb.save()
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Rename_PDB(request):
    """ Rename PDB view function
    the call is made from client AJAX POST call that contains the parameters:
        cdb_id: the local model id of the container
        pdb_id: the local id of the pluggable to be renamed
        new_name: a string with the new name
    the view returns a JSONResponse with the id of renaming task"""
    cdb_id = request.data.get("cdb_id", None)
    pdb_id = request.data.get("pdb_id", None)
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    
    if api_call:
        location = pdb_id.split('@')
        pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    else:
        pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    if not pdb:
        raise APIException410(f"PDB ID {pdb_id} is not available")
    allow_or_403(pdb)
    new_name = request.data.get("rename")
    pdb.status = "renaming to {}".format(new_name)
    pdb.save()  
    cdb_data = object_to_json(cdb)
    pdb_data = object_to_json(pdb)
    host_data = c123_HostSerializer(pdb.host).data
    created_by = request.user
    
    tmp_data = json.dumps({'cdb_data': cdb_data, 'pdb_data': pdb_data, 'host_data': host_data, 'new_name': new_name, 'created_by': created_by.id})
    task = Task.objects.create(task='rename_pdb_task',
                            displayment=f'rename {pdb.name}',
                            target_type=DBType.ORACLE_PDB.value,
                            target_id=pdb.id,
                            org_id=pdb.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Close_PDB(request):
    cdb_id = request.data.get("cdb_id")
    pdb_id = request.data.get("pdb_id")
    pdb_name = request.data.get("pdb_name")
    option = request.data.get("option")
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    
    if api_call:
        location = pdb_id.split('@')
        pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    else:
        pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    if not pdb:
        raise APIException410(f"PDB ID {pdb_id} is not available")
    allow_or_403(pdb)
    cdb_data = object_to_json(cdb)
     
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args          
        raise APIException401(str(error))
    
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:     
        cursor = conn.cursor()
    except Exception as error:
        raise APIException401(str(error))
    
    # check open mode
    
    try:
        cursor.execute(f"SELECT open_mode FROM v$pdbs WHERE name ='{pdb_name}'")
    except Exception as e:
        error, = e.args        
        raise APIException410(str(error))
    else:
        open_state = cursor.fetchone()[0]
    
    # PDB already closed
    
    if 'mounted' in open_state.lower():
        already_close_msg = f"pluggable database {pdb_name} already closed"
        pdb.ind_started = False
        pdb.save()        
        return JsonResponse({'msg':already_close_msg}, status=200) 
    
    # close the PDB
    
    close_query = f"ALTER PLUGGABLE DATABASE {pdb_name} CLOSE "
    if option == 'CI':
        close_query += "IMMEDIATE"
    elif option == 'CA':
        close_query += "ABORT"
    cursor.close()
    conn.close()
    host_data = c123_HostSerializer(pdb.host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'pdb_id': pdb.id, 'close_query': close_query, 'host_data': host_data, 'created_by':created_by.id})
    task = Task.objects.create(task='close_pdb_task',
                            displayment=f'close {pdb.name}',
                            target_type=DBType.ORACLE_PDB.value,
                            target_id=pdb.id,
                            org_id=pdb.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    
    pdb.severity = Severity.NOT_OK.value
    pdb.status = Status.CLOSING.value
    # pdb.ind_started = False
    pdb.save()
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Copy_PDB(request):
    """copy a pdb view
    the call is made from client AJAX POST call that contains the parameters:
        cdb_id: the local model id of the container
        pdb_id: the local id of the pluggable to be copied
        copy_name: a string with the name of the copy
    the view returns a JSONResponse with the id of the cdb"""
    standbys = request.data.get('standbys')
    also_start = request.data.get('also_start')
    autostart = request.data.get('autostart')
    source_restart = request.data.get('source_restart')
    target_restart = request.data.get('target_restart')
    # pdb_name = request.data.get('pdb_name')
    copy_name = request.data.get('copy').upper()
    file_dest = request.data.get('file_dest')
    cdb_id = request.data.get('cdb_id', None)
    pdb_id = request.data.get('pdb_id', None)
    
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    
    if api_call:
        location = pdb_id.split('@')
        pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    else:
        pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
    if not pdb:
        raise APIException410(f"PDB ID {pdb_id} is not available")
    allow_or_403(pdb)
    pdb_source = c123_node.objects.get(pk=pdb.pk)
    pdb.status = Status.COPYING.value
    pdb.severity = Severity.NOT_OK.value
    pdb.ind_started = False
    pdb.save()
    # create pdb copy node
    pdb.pk = None
    pdb._state.adding = True
    pdb.name = copy_name
    pdb.ex_params['file_dest'] = file_dest
    pdb.dir_123Cluster = cdb.dir_123Cluster
    pdb.save() 
    
    
    cdb_data = object_to_json(cdb)
    pdb_data = object_to_json(pdb_source)
    
    host_data = c123_HostSerializer(pdb_source.host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'pdb_data': pdb_data, 'pdb_target_id': pdb.id,
                'host_data': host_data,
                'copy_name': copy_name, 'autostart': autostart, 'file_dest': file_dest,
                'target_restart': target_restart, 'source_restart': source_restart, 'created_by': created_by.id})
    
    task = Task.objects.create(task='copy_pdb_task',
                            displayment=f'copy {pdb_source.name}',
                            target_type=DBType.ORACLE_PDB.value,
                            target_id=pdb_source.id,
                            org_id=pdb_source.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Drop_PDB(request):

        cdb_id = request.data.get('cdb_id')
        pdb_id = request.data.get('pdb_id')
        options = request.data.get('deleteOptions')
        api_call = request.data.get('rest_api')
        if api_call:
            location = cdb_id.split('@')
            cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
        else:
            cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
        if not cdb:
            raise APIException410(f"CDB ID {cdb_id} is not available")
        
        if api_call:
            location = pdb_id.split('@')
            pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
        else:
            pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
        if not pdb:
            raise APIException410(f"PDB ID {pdb_id} is not available")
        task = drop_pdb(cdb, pdb, options)
        if isinstance(task, str) and "already deleted" in task:
            return JsonResponse({"notification": task}, status = 200)
        return append_status_cmd(request, task)
      
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreBackupPDB(request):
    """restore a pdb from backup
    the call is made from client AJAX POST that contains the
    cdb id, pdb id and backup id"""
    bu_id = request.data.get('bu_id')
    restore_option = request.data.get('restore_option')
    restore_time = request.data.get('restore_time')
    autostart = request.data.get('autostart')
    api_call = request.data.get('rest_api') 
    if api_call:
        backup = c123_inventory.objects.filter(name=bu_id, type = Inventory.BACKUP.value).first()
    else:
        backup = c123_inventory.objects.filter(pk=bu_id, type = Inventory.BACKUP.value).first()
    if not backup:
        raise APIException404(f"backup ID {bu_id} could not be found")
    pdb = backup.node
    allow_or_403(pdb)
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    
    forbid_restore = Task.objects.filter(target_id = pdb.pk, task__in = ['restore_pdb_from_backup_task', 'restore_pdb_from_flashback_task', 'backup_pdb_task'], status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
    if forbid_restore:
        raise APIException409(f"Already runs backup/restore for pk {pdb.name}")
    cdb = c123_node.objects.filter(cluster = pdb.cluster,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB of PDB {pdb.name} is not available")
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")

    if RestoreOptions.THIS.value in restore_option:
        date = backup.created_date.strftime("%Y-%m-%d %H:%M:%S")
    elif RestoreOptions.CUSTOM.value in restore_option and restore_time:
        # example date = '2021-12-30 09:08:33'
        # date = datetime.strptime(restore_time, "%Y-%m-%dT%H:%M:%S.%f%z").astimezone()
        # date = date.strftime("%Y-%m-%d %H:%M:%S")
        date = restore_time
    else:
        date = ''
    
    pdb.status = Status.RESTORING.value
    pdb.save()
    # serialize objects for the tasks
    cdb_data = object_to_json(cdb)
    backup = object_to_json(backup)
    backup_host = c123_HostSerializer(app_host).data
    host_data = c123_HostSerializer(pdb.host).data
    backup_host = c123_HostSerializer(app_host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data, 'backup_host': backup_host, 'backup_data': backup, 'pdb_name': pdb.name, 'pdb_id': pdb.id, 'backup_host': backup_host, 'restore_option': restore_option, 'date': date,  'autostart': autostart,  'created_by': created_by.id})
    task = Task.objects.create(task='restore_pdb_from_backup_task',
                            displayment=f'restore PDB {pdb.name} from backup',
                            target_type=DBType.ORACLE_PDB.value,
                            target_id=pdb.id,
                            org_id=pdb.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)     
      
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))      
def DeleteCDB(request):
    """delete a cdb"""    
    # handle api request 
    if 'cdb_id' in request.data:
        cdb_id = request.data.get('cdb_id')
    if 'drop-pdb-cdb_id' in request.data:
        cdb_id = request.data.get('drop-pdb-cdb_id')
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    cdb.status = Status.DROPPING.value
    cdb.severity = Severity.NOT_OK.value
    children = c123_node.objects.filter(cluster = cdb.cluster)
    for child in children:
        cleanup_logs(child.pk, child.type)
            
    cleanup_logs(cdb.pk, cdb.type)
        
    host_count = c123_node.objects.filter(host = cdb.host, type = DBType.ORACLE_CDB.value).count()
    if host_count < 2:
        Task.objects.filter(task = "run_linux_performance_counters", target_id=cdb.host.id).delete()
    # serialize the objects
    cdb_data = object_to_json(cdb)
    host_data = c123_HostSerializer(cdb.host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data, 'created_by': created_by.id})
    task = Task.objects.create(task='unregister_cdb_task',
                                    displayment=f'unregister cdb {cdb.name}',
                                    target_type=DBType.ORACLE_CDB.value,
                                    target_id=cdb.id,
                                    org_id=cdb.org_id,
                                    schedule=Schedule.NOW.value,
                                    status=TaskState.PENDING.value,
                                    parameters=tmp_data,
                                    created_by=created_by)
    revoke_license(cdb)
    return append_status_cmd(request, task)
####################################################################
# PDB Flashbacks
####################################################################
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Flashback_CDB_PDB(request):
    """flashback a cdb or a pdb"""
    cdb_id = request.data.get('cdb_id', None)
    pdb_id = request.data.get('pdb_id', None)
    obj_type = request.data.get('obj_type', None)
    created_by = request.user
    ex_params = {}
    ex_params["exit_code"] = 1
    ex_params["duration"] = 0
    size_scheme = OracleClusterSizing.objects.first()
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
  
    if ClusterRoles.PDB.value in obj_type:
        if api_call:
            location = pdb_id.split('@')
            pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
        else:
            pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
        if not pdb:
            raise APIException410(f"PDB ID {pdb_id} is not available")
        allow_or_403(pdb)
        ex_params["belong_to"] = DBType.ORACLE_PDB.value
        ex_params["node_version"] = pdb.version
        keep_days = getattr(size_scheme, f'{pdb.sizing.lower()}_backup_keep_days', None)
    else:
        allow_or_403(cdb)
        ex_params["belong_to"] = DBType.ORACLE_CDB.value
        ex_params["node_version"] = cdb.version
        keep_days = getattr(size_scheme, f'{cdb.sizing.lower()}_restore_point_keep_days', None)
    form = c123_inventory_serializer(data=request.data, partial=True)
    if form.is_valid():
            flashback = form.create(form.validated_data)
            flashback.name = flashback.name.upper()
            flashback.type = Inventory.FLASHBACK.value
            if ClusterRoles.PDB.value in obj_type:
                flashback.node = pdb
                pdb.status = Status.BACKUP.value
                pdb.save()
                obj_id = pdb.pk
            else:
                flashback.node = cdb
                cdb.status = Status.BACKUP.value
                cdb.save()
                obj_id = cdb.pk
                obj_name = cdb.name
            flashback.created_by = request.user
            ex_params["guaranteed"] = request.data.get("guaranteed")
            flashback.ex_params = ex_params
            if keep_days:
                flashback.keep_days = keep_days
            flashback.save()
            cdb_data = object_to_json(cdb)
            flashback = object_to_json(flashback)
            
            if ClusterRoles.PDB.value in obj_type:
                host_data = c123_HostSerializer(pdb.host).data
                tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data,  'flashback': flashback, 'obj_name': pdb.name, 'obj_type': obj_type, 'obj_id': obj_id, 'created_by': created_by.id})
                task = Task.objects.create(task='flashback_cdb_pdb_task',
                                            displayment=f'flashback for {pdb.name}',
                                            target_type=DBType.ORACLE_PDB.value,
                                            target_id=pdb.id,
                                            org_id=pdb.org_id,
                                            schedule=Schedule.NOW.value,
                                            status=TaskState.PENDING.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            elif ClusterRoles.CDB.value in obj_type:
                host_data = c123_HostSerializer(cdb.host).data
                tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data, 'flashback': flashback, 'obj_name': cdb.name, 'obj_type': obj_type, 'obj_id': obj_id, 'created_by': created_by.id})
                task = Task.objects.create(task='flashback_cdb_pdb_task',
                                            displayment=f'flashback for {cdb.name}',
                                            target_type=DBType.ORACLE_CDB.value,
                                            target_id=cdb.id,
                                            org_id=cdb.org_id,
                                            schedule=Schedule.NOW.value,
                                            status=TaskState.PENDING.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
    else:
        print(form.errors)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropFlashback(request):
    """drop a PDB Flashback restore point"""
    flashback_id = request.data.get("flashback_id")
    api_call = request.data.get('rest_api')
    if api_call:
        flashback = c123_inventory.objects.filter(name=flashback_id, type = Inventory.FLASHBACK.value).first()
    else:
        flashback = c123_inventory.objects.filter(pk=flashback_id, type = Inventory.FLASHBACK.value).first()
    if not flashback:
        raise APIException404(f"Flashback ID {flashback_id} could not be found")
    
    pdb = flashback.node
    cdb_id = pdb.ex_params.get('cdb_id')
    cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
   
    cdb_data = object_to_json(cdb)
    flashback_data = object_to_json(flashback)
    host_data = c123_HostSerializer(pdb.host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data, 'flashback_data': flashback_data, 'pdb_name': pdb.name, 'pdb_id': pdb.id, 'created_by': created_by.id})
    task = Task.objects.create(task='drop_restore_point_task',
                                displayment=f'drop flashback {flashback.name}',
                                target_type=DBType.ORACLE_PDB.value,
                                target_id=pdb.id,
                                org_id=pdb.org_id,
                                schedule=Schedule.NOW.value,
                                status=TaskState.PENDING.value,
                                parameters=tmp_data,
                                created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropStandalone(request):
    """drop a standalone except Demo"""
    standalone_id = request.data.get("standalone_id")
    
    if standalone_id == 1:
        raise APIException406("Demo standalone can't be removed")
    standalone = c123_Oracle_Standalone.objects.filter(pk=standalone_id).first()
    if not standalone:
        raise APIException404(f"Standalone ID {standalone_id} could not be found")
    
    standalone.delete()
    return JsonResponse({}, status=200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))   
def RestoreFromFlashback(request):
    """restore a PDB from a Flashback restore point"""
    flashback_id = request.data.get('flashback_id')
    api_call = request.data.get('rest_api')
    if api_call:
        flashback = c123_inventory.objects.filter(name=flashback_id, type = Inventory.FLASHBACK.value).first()
    else:
        flashback = c123_inventory.objects.filter(pk=flashback_id, type = Inventory.FLASHBACK.value).first()
    if not flashback:
        raise APIException404(f"Flashback ID {flashback_id} could not be found")
    pdb = flashback.node
    allow_or_403(pdb)
    forbid_restore = Task.objects.filter(target_id = pdb.id, task__in = ['restore_pdb_from_backup_task', 'restore_pdb_from_flashback_task', 'backup_pdb_task'], status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
    if forbid_restore:
        raise APIException409(f"Already runs backup/restore for pdb {pdb.name}")
    cdb = c123_node.objects.filter(cluster=pdb.cluster,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB of PDB {pdb.name} is not available")
    pdb.status = Status.RESTORING.value
    pdb.save()
    # serialize objects for the tasks
    cdb_data = object_to_json(cdb)
    flashback_data = object_to_json(flashback)
    host_data = c123_HostSerializer(pdb.host).data
    created_by = request.user   
    tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data, 'flashback_data': flashback_data, 'pdb_name': pdb.name, 'pdb_id': pdb.id, 'created_by': created_by.id})
    task = Task.objects.create(task='restore_pdb_from_flashback_task',
                                displayment=f'restore PDB {pdb.name} from flashback',
                                target_type=DBType.ORACLE_PDB.value,
                                target_id=pdb.id,
                                org_id=pdb.org_id,
                                schedule=Schedule.NOW.value,
                                status=TaskState.PENDING.value,
                                parameters=tmp_data,
                                created_by=created_by)
    return append_status_cmd(request, task)
    
####################################################################
# PDB Import
####################################################################
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ImportPDB(request):
    """view function for importing an exported PDB"""
    # the export object id
    export_id = request.data.get('export_id')
    api_call = request.data.get('rest_api')
    if api_call:
        export = c123_inventory.objects.filter(name=export_id, type = Inventory.EXPORT.value).first()
    else:
        export = c123_inventory.objects.filter(pk=export_id, type = Inventory.EXPORT.value).first()
    pdb = export.node
    allow_or_403(pdb)
    cdb = c123_node.objects.filter(cluster = pdb.cluster,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    
    
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
        
    pdb.status = Status.RESTORING.value
    pdb.save()
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args
        raise APIException401(str(error))
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:     
        cursor = conn.cursor()
    except Exception as error:
        raise APIException401(str(error))
    try:
        cursor.execute(f"SELECT directory_name, directory_path FROM dba_directories WHERE directory_path='{export.ex_params.get('remote_path')}'")
    except Exception as e:
        error, = e.args
        print(str(error))
    dir_exists = cursor.fetchone()
    # 4.a if the directory doesn't exist create it
    
    if dir_exists is None:
        try:
            cursor.execute(f"CREATE DIRECTORY {export.ex_params.get('db_path_name')} AS '{export.ex_params.get('remote_path')}'")
        except Exception as e:
            error, = e.args     
            print(str(error))   
    cursor.close()
    conn.close()
    cdb_data = object_to_json(cdb)
    pdb_data = object_to_json(pdb)
    export = object_to_json(export)
    host_data = c123_HostSerializer(cdb.host).data
    backup_host = c123_HostSerializer(app_host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'pdb_data': pdb_data, 'host_data': host_data, 'backup_host': backup_host, 'export': export, 'created_by': created_by.id})
    task = Task.objects.create(task='import_pdb_task',
                            displayment=f'import PDB {pdb.name}',
                            target_type=DBType.ORACLE_PDB.value,
                            target_id=pdb.id,
                            org_id=pdb.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ImportCDB(request):
    """view function for importing an exported CDB"""
    export_id = request.data.get('export_id')
    api_call = request.data.get('rest_api')
    if api_call:
        export = c123_inventory.objects.filter(name=export_id, type = Inventory.EXPORT.value).first()
    else:
        export = c123_inventory.objects.filter(pk=export_id, type = Inventory.EXPORT.value).first()
    cdb = export.node
    
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")

    cdb.status = Status.RESTORING.value
    cdb.save()
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args
        raise APIException401(str(error))
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:     
        cursor = conn.cursor()
    except Exception as error:
        raise APIException401(str(error))
    try:
        cursor.execute(f"SELECT directory_name, directory_path FROM dba_directories WHERE directory_path='{export.ex_params.get('remote_path')}'")
    except Exception as e:
        error, = e.args
        print(str(error))
    dir_exists = cursor.fetchone()
    # 4.a if the directory doesn't exist create it
    
    if dir_exists is None:
        try:
            cursor.execute(f"CREATE DIRECTORY {export.ex_params.get('db_path_name')} AS '{export.ex_params.get('remote_path')}'")
        except Exception as e:
            error, = e.args     
            print(str(error))   
    cursor.close()
    conn.close()
    cdb_data = object_to_json(cdb)
    export = object_to_json(export)
    host_data = c123_HostSerializer(cdb.host).data
    backup_host = c123_HostSerializer(app_host).data
    created_by = request.user
    tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data, 'backup_host': backup_host, 'export': export, 'created_by': created_by.id})
    task = Task.objects.create(task='import_cdb_task',
                            displayment=f'import CDB {cdb.name}',
                            target_type=DBType.ORACLE_CDB.value,
                            target_id=cdb.id,
                            org_id=cdb.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def checkUserExists(request):
    """check if a user exists for an Oracle import operation"""
    obj_type = request.GET.get('obj_type')
    if obj_type == ClusterRoles.PDB.value:
        pdb_id = request.GET.get('node_id')
        pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
        if not pdb:
            raise APIException410(f"PDB ID {pdb_id} is not available")
        cdb = c123_node.objects.filter( type = DBType.ORACLE_CDB.value,  pk=pdb.ex_params.get("cdb_id"), ind_deleted = False).first()
        if not cdb:
            raise APIException410(f"CDB ID {cdb_id} could not be found")
        pdbName = pdb.name
    else:
        cdb_id = request.GET.get('node_id')
        cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
        if not cdb:
            raise APIException410(f"CDB ID {cdb_id} is not available")
        pdbName = cdb.name
    export_id = request.GET.get('export_id')
    export = c123_inventory.objects.filter(pk=export_id, type = Inventory.EXPORT.value).first()
    if not export:
        raise APIException404(f"Export ID {export_id} could not be found")
    conn = connect_to_oracle_database(cdb=cdb)
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:     
        cursor = conn.cursor()
    except Exception as error:
        raise APIException401(str(error))
    if obj_type == ClusterRoles.PDB.value:
        try:
            # cursor.execute("alter session set container={}".format(pdbName))
            cursor.execute(f"ALTER PLUGGABLE DATABASE {pdbName} OPEN")
        except oracledb.DatabaseError as e:
            error, = e.args
            if "ORA-65019" in str(error):
                print(f"already open {error}")
            else:
                raise APIException404(str(error))
        try:
            cursor.execute(f"ALTER SESSION SET CONTAINER={pdbName}")
        except oracledb.DatabaseError as e:
            error, = e.args
            raise APIException404(str(error))        
    if export.ex_params.get('schema') != 'all_schemas':
        try:
            cursor.execute(f"SELECT username FROM dba_users WHERE username='{export.ex_params.get('schema')}'")
        except oracledb.DatabaseError as e:
            error, = e.args
            print(error)
        user = cursor.fetchone()
    
        if user is None:
            return JsonResponse({'userName': 'None', 'pdbName': pdbName}, status=200)
        else:
            return JsonResponse({'error': str(user[0])}, status=409)
    else:
        return JsonResponse({'userName': 'All users', 'pdbName': pdbName}, status=200)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def checkDumpFile(request):
    """check if a dump file exists for an Oracle import operation"""
    obj_type = request.GET.get('obj_type')
    if obj_type == ClusterRoles.PDB.value:
        pdb_id = request.GET.get('node_id')
        pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
        if not pdb:
            raise APIException410(f"PDB ID {pdb_id} is not available")
        cdb_id = pdb.ex_params.get("cdb_id")
    else:
        cdb_id = request.GET.get('node_id')
    export_id = request.GET.get('export_id')
    cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    export = c123_inventory.objects.filter(pk=export_id, type = Inventory.EXPORT.value).first()
    if not export:
        raise APIException404("Export ID {} could not be found".format(export_id))
    
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    
    if not export.ex_params.get('dump_file')[0].startswith('/') and not export.ex_params.get('remote_path').endswith('/'):
        dump_file = f"{export.ex_params.get('remote_path')}/{export.ex_params.get('dump_file')[0]}"
    else:
        dump_file = f"{export.ex_params.get('remote_path')}{export.ex_params.get('dump_file')[0]}"
    
    exists = checkPathExists(app_host, dump_file)
    if not exists:
        return JsonResponse({'error': f'Dump file {dump_file} does not exist.Cannot continue', 'dump_file': dump_file}, status=404)   
    else:
        return JsonResponse({}, status=200)
####################################################################
# PDB Export
####################################################################
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ExportCDB_PDB(request):
    """view function for initiating CDB/PDB export task"""

    data = request.data
    data['created_by'] = request.user.id
    schedule = data.get('schedule')
    api_call = data.get('api_call')
    cdb_id = data.get('cdb_id')
    pdb_id = data.get('pdb_id')
    obj_type = request.data.get('obj_type')
    

    if not schedule:
        task = export_cdb_pdb(data)
        return append_status_cmd(request, task)
    else:
        if api_call:
            location = cdb_id.split('@')
            cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
        else:
            cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
        if not cdb:
            raise APIException410(f"CDB ID {cdb_id} is not available")
        from django.conf import settings
        client_timezone = pytz.timezone(data.get("client_timezone"))
        schedule_run_time_str = data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = data.get("days_of_week")
        days_of_month = data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        
        
        if ClusterRoles.PDB.value in obj_type:
            if api_call:
                location = pdb_id.split('@')
                pdb = c123_node.objects.filter(name=location[0], host__name=location[1], port=location[2], type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
            else:
                pdb = c123_node.objects.filter(pk=pdb_id,  type = DBType.ORACLE_PDB.value, ind_deleted = False).first()
            if not pdb:
                raise APIException410(f"PDB ID {pdb_id} is not available")       
        
        job_id =  f"export_{f'{cdb.name}' if ClusterRoles.CDB.value in obj_type else f'{cdb.name}_{pdb.name}'}"
        scheduler = settings.scheduler
        export_job = next((i for i in scheduler.get_jobs() if job_id in i.id), None)
        if export_job:
            export_job.reschedule(trigger=trigger)
        else:
             settings.scheduler.add_job(export_cdb_pdb, replace_existing=True, trigger=trigger,  kwargs={"data": data}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        return JsonResponse({"notification": f"export of {f'CDB {cdb.name}' if ClusterRoles.CDB.value in obj_type  else f'PDB {pdb.name} of CDB {cdb.name}' } {description} started"}, status = 200)
    
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def getDBFileDestination(request, obj_id):
    """get cdb id and return his file location"""
    cdb = c123_node.objects.filter(pk=obj_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {obj_id} is not available")
    
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args
        print('getDBFileDestination function:', error)
        return JsonResponse({'status': 'false', 'message': error[:99]}, status=500)
    if type(conn) is dict:
        return JsonResponse(conn, status = 401)
        
    try:
        cur = conn.cursor()
    except Exception as error:
        raise APIException500(str(error))
    
    cur.execute("SELECT value FROM v$parameter WHERE name='db_create_file_dest'")
    file_dest = cur.fetchone()[0]
    if not file_dest:
        cur.execute("SELECT file_name FROM dba_data_files WHERE tablespace_name = 'SYSTEM'")
        file_dest = cur.fetchone()[0]
        file_dest = file_dest.rsplit('/',1)[0]
        message = '''The parameter db_create_file_dest at the container database is empty.    
        Using the container's actual file location.'''
        cur.close()
        conn.close()
        return JsonResponse({'status': 'true', 'message': message, 'file_dest': file_dest}, status=200)
    cur.close()
    conn.close()    
    return JsonResponse({'status': 'true', 'message': '', 'file_dest': file_dest}, status=200)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def CheckPathName(request):
    """async called from ajax pre-duplicate operation
    returns YES the path_name used for copying the standalone
    is available otherwise returns NO"""
    cdb_id = request.GET.get('cdb_id')
    path_name = request.GET.get('path_name')
    cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    # host = c123_host.objects.get(pk=cdb.host.pk)
    if re.match(r'^\/(?!\/)[a-zA-Z0-9._-]+(\/[a-zA-Z0-9._-]+)*$', path_name):
        client = create_ssh_connection(cdb.host.name, cdb.host.username, cdb.host.password, cdb.host.private_key, cdb.host.ssh_port)
        if isinstance(client, tuple):
            if hasattr(client[1], 'strerror'):
                raise APIException401(client[1].strerror)
            raise APIException401(client[1])
        jsn = json.loads(remote_command(client, f"test -e {path_name}  && echo yes || echo no", True), strict=False)
        if 'yes' in jsn['stdout']:    
            return JsonResponse({'error': "already exists"}, status=404) 
        elif 'no' in jsn['stdout']:        
            return JsonResponse({'response': "path available"}, status=200)
        else:
            return JsonResponse({'error': "invalid path"}, status=404)
    else:
        return JsonResponse({'error': "invalid path"}, status=404)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def fetchSchemas(request):
    """returns a list of Users/Schemas of a PDB"""
    obj_type = request.data.get("obj_type")
    cdb_id = request.data.get("cdb_id")
    cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value, ind_deleted = False ).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    if obj_type == ClusterRoles.PDB.value:
        pdb_name = request.data.get("pdb_name")
        pdb = c123_node.objects.filter(name=pdb_name,  type = DBType.ORACLE_PDB.value, cluster = cdb.cluster, ind_deleted = False).first()
        if not pdb:
            raise APIException410(f"PDB {pdb_name} of CDB {cdb.name} is not available")
    
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args
        print('fetchSchemas function:', error)
        raise APIException500(str(error))
    
    if type(conn) is dict:
        return JsonResponse(conn, status = 404)
    
    try:
        cursor = conn.cursor()
    except Exception as error:
        raise APIException500(str(error))
    if obj_type == ClusterRoles.PDB.value and pdb_name is not None:
        try:
            cursor.execute(f"ALTER PLUGGABLE DATABASE {pdb_name} OPEN")
        except oracledb.DatabaseError as e:
            error, = e.args
            if "ORA-65019" in str(error):
                pass
            else:
                raise APIException404(str(error))
        try:
            cursor.execute(f"ALTER SESSION SET CONTAINER={pdb_name}")
        except oracledb.DatabaseError as e:
            error, = e.args
            raise APIException404(str(error))
    
    # query = f"SELECT username FROM dba_users WHERE username NOT IN (\
    #         'ORACLE_OCM','OJVMSYS','SYSKM','XS$NULL','GSMCATUSER','MDDATA','SYSBACKUP','DIP','SYSDG',\
    #         'APEX_PUBLIC_USER','SPATIAL_CSW_ADMIN_USR','SPATIAL_WFS_ADMIN_USR','GSMUSER','AUDSYS','FLOWS_FILES',\
    #         'DVF','MDSYS','ORDSYS','DBSNMP','WMSYS','APEX_040200','APPQOSSYS','GSMADMIN_INTERNAL','ORDDATA','CTXSYS',\
    #         'ANONYMOUS','XDB','ORDPLUGINS','DVSYS','SI_INFORMTN_SCHEMA','OLAPSYS','LBACSYS','OUTLN','SYSTEM','SYS', '{pdb.database_user if obj_type == ClusterRoles.PDB.value else cdb.database_user}')"
    if obj_type == ClusterRoles.PDB.value:
        if not pdb.database_user:
            raise APIException406(f"Update credentials in {pdb.name} details page")
        query = f"SELECT username FROM dba_users WHERE ORACLE_MAINTAINED='N' AND username!='{(pdb.database_user).upper()}'"
    else: 
        query = "SELECT username FROM dba_users WHERE ORACLE_MAINTAINED='N'"
    try:
        cursor.execute(query)
    except oracledb.DatabaseError as e:
        error, = e.args
        print(str(error))
        raise APIException404(str(error))
    
    schemas = cursor.fetchall()
    schemas_length = len(schemas)
    cursor.close()
    conn.close()
    
    if schemas_length == 0:
        raise APIException404("Schemas could not be found")
    return JsonResponse({'schemas': schemas}, status=200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def fetchDisks(request):
    """returns a list of available disks"""
    cdb_id = request.data.get("cdb_id")

    schema = request.data.get("schema")[0]
    cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    try:
        conn = connect_to_oracle_database(cdb=cdb)
    except Exception as e:
        error, = e.args
        print('fetchDisks function:', error)
        return JsonResponse({'status': 'false', 'message': error, 'verbose': True}, status=500)
    
    if type (conn) is dict:
        return JsonResponse(conn, status = 404)
    try:
        cursor = conn.cursor()
    except Exception as error:
        return JsonResponse({'status': 'false', 'message': error, 'verbose': True}, status=500)
    # fetch available disks
    # if obj_type == ClusterRoles.PDB.value and pdb_name is not None:
    #     try:
    #         cursor.execute(f"ALTER SESSION SET CONTAINER={pdb_name}")
    #     except oracledb.DatabaseError as e:
    #         error, = e.args
    #         raise APIException404(str(error))
    
    try:
        # cursor.execute(
        #     "SELECT DIRECTORY_NAME,DIRECTORY_PATH FROM dba_directories WHERE directory_name NOT IN ( \
        #         'ORACLE_HOME', \
        #         'ORACLE_BASE', \
        #         'OPATCH_LOG_DIR', \
        #         'OPATCH_SCRIPT_DIR', \
        #         'OPATCH_INST_DIR', \
        #         'XSDDIR', \
        #         'ORACLE_OCM_CONFIG_DIR', \
        #         'XMLDIR', \
        #         'ORACLE_OCM_CONFIG_DIR2') ")
        cursor.execute("SELECT DIRECTORY_NAME,DIRECTORY_PATH FROM dba_directories WHERE directory_name = 'DIR_123CLUSTER'")
    except oracledb.DatabaseError as e:
        error, = e.args
        raise APIException404(str(error))
    
    disks = cursor.fetchall()
    folders = set()
    for lis in disks:
        folders.add(lis)
    
    # get the disk size
    query = "SELECT sum(bytes)/1024 size_kb FROM dba_segments"
    if schema != 'all_schemas':
        query = f"{query} WHERE OWNER='{schema}'"
    try:
        cursor.execute(query)            
    except oracledb.DatabaseError as e:
        error, = e.args
        raise APIException404(str(error))
    
    size = cursor.fetchone()[0]
    if size == None:
        size = 0 
                  
    cursor.close()
    conn.close()
    return JsonResponse({'disks': disks, 'size': size, 'display_size': format_size(size)}, status=200)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def GetOracleHome(request):
    user = request.GET.get('user')   
    pwd = format_password(request.GET.get('pwd'))
    host = request.GET.get('host')
    port = int(request.GET.get('port'))
    sname = request.GET.get('sname')
        
    try:
        conn = connect_to_oracle_database_using_params(host=host, port=port, sname=sname, user=user, pwd=pwd)
    except Exception as e:
        print("error in get_oracle_home get_connection")
        error, = e.args
        print(error)
        return JsonResponse({'status': 'false', 'message': error,  'verbose': True}, status=500)
    else:
        
        if type (conn) is dict:
            return JsonResponse(conn, status = 404)
        try:
            cursor = conn.cursor()
        except Exception as e:
            error, = e.args
            print("error in get_oracle_home cursor")
            print(error)
            return JsonResponse({'status': 'false', 'message': error, 'verbose': True}, status=500)
    # get the Oracle Home directory
    try:
        cursor.execute("SELECT SYS_CONTEXT ('USERENV','ORACLE_HOME') FROM DUAL")
    except Exception as e:
        error, = e.args
        print(error)
        return JsonResponse({'status': 'false', 'message': error}, status=500)
    
    oracle_home = cursor.fetchone()[0]
    # get number of pdbs in cdb
    try:
        cursor.execute("SELECT PDB_ID,PDB_NAME,STATUS FROM DBA_PDBS")
    except Exception as e:
        error, = e.args
        print(error)
        return JsonResponse({'status': 'false', 'message': error[:99]}, status=500)
    
    pdbs_count = len(cursor.fetchall()) - 1
    data = {"oracle_home": oracle_home, "pdbs_count": pdbs_count}
    cursor.close()
    conn.close()
    return JsonResponse({'data': data}, status=200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def UploadTNSfile(request):
    form = FileUploadForm(data=request.POST, files=request.FILES)
    if form.is_valid():
        # read the file
        from io import TextIOWrapper
        _file = TextIOWrapper(form.files['file_source'])
        sList = _file.read()
        _file.close()
        # ChatGPT solution
        sList = re.sub('#.*\n', '', sList)
        tns_entries = re.findall(r'\n(\w+)\s*=\s*\(DESCRIPTION.*?HOST\s*=\s*([\w\.]+).*?PORT\s*=\s*(\d+).*?SID\s*=\s*([\w\d]+)\)', sList, re.IGNORECASE | re.DOTALL)
        for entry in tns_entries:
            name, host, port, sid = entry
            standalone = c123_Oracle_Standalone.objects.filter(name = name)
            if standalone.exists():
                 #update if provided existing name 
                 standalone.update(host_name = host, port = port, service_name = sid)
            else:
              #create new standalone
              c123_Oracle_Standalone.objects.create(created_by = request.user, host_name = host, name = name, port = port, service_name = sid)
        
        # lines = sList.split('\n')
        # addresses = ([i for i in lines if 'host' in i.lower()])
        # connect = ([i for i in lines if 'sid' in i.lower()])
        # sList = re.sub('#.*\n', '', sList)             # strip all comments
        # # remove "(" to end of line
        # sList = re.sub('\(.*\n', '', sList)
        # sList = re.sub('\)', '', sList)                # clean up ")"
        # sList = re.sub(' ', '', sList)                 # remove spaces
        # # remove carriage returns
        # sList = re.sub('\n', '', sList)
        # sList = re.sub('=$', '', sList)                # remove last "="
        # sList = sList.split("=")
        # sids = []
        # for entry in connect:
        #     sids.append("".join(entry.split()).replace(')', '').replace('(', '').split("="))
        # i = 0  
        # for address in addresses:
        #    field = "".join(address.split()).replace(')', '').split('(')     
        #    host = ([i for i in field if 'host' in i.lower()])[0].split("=")[1]
        #    port = ([i for i in field if 'port' in i.lower()])[0].split("=")[1]
            #  standalone = c123_Oracle_Standalone.objects.filter(host_name = host, port = port, service_name = sids[i][1])
            #  if standalone.exists():
            #      #rename if new standalone name provided, host/port/sid untouched 
            #      standalone.update(name = sList[i])
            #  else:
            #  #create new standalone
            #  c123_Oracle_Standalone.objects.create(created_by = request.user, host_name = host, name = sList[i], port = port, service_name = sids[i][1])
 
            #  i = i + 1
    else:
        print('invalid form')
    return JsonResponse({"errors": form.errors, "notification": "tns file upload finished"}, status=200, safe=False)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def Duplicate_Standalone(request):
    """create a duplicate from standalone database view"""
    cdb_id = request.data.get('cdb_id')
    api_call = request.data.get('rest_api')
    if api_call:
        location = cdb_id.split('@')
        cdb = c123_node.objects.filter(name=location[0], host__name = location[1],  type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    else:
        cdb = c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value, ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    allow_or_403(cdb)
    # host = c123_host.objects.get(pk=cdb.host.pk)
    
    standalone_db_user = request.data.get('user_name')
    standalone_db_pwd = format_password(request.data.get('password'))
    hostname = request.data.get('machine')
    port = request.data.get('port')
    service_name = request.data.get('service_name')
    file_dest = request.data.get('file_dest')
    target_PDB = request.data.get('target_PDB_name').upper()
    # target_db_user = request.data.get('target_user_name')
    # target_db_pwd = request.data.get('target_password')
    start_after_copy = request.data.get('start_after_copy')
    link_name = "lnk_src_123cluster"
    link_query = f"CREATE DATABASE LINK {link_name} CONNECT TO {standalone_db_user} IDENTIFIED BY " + standalone_db_pwd + " USING\n\
        '(DESCRIPTION =\n\
    (ADDRESS_LIST =\n\
    (ADDRESS = (PROTOCOL = TCP)(HOST = " + hostname + ")(PORT = " + str(port) + ")))\n\
    (CONNECT_DATA =\n\
    (SERVICE_NAME=" + service_name + ")))'"
    drop_link_query = f"DROP DATABASE LINK {link_name}"
    # wrong, even if this fails - we still able to use the link
    # fetch_from_link_query = f"SELECT instance_name FROM v$instance@{link_name}"
    link_exists_query =  f"SELECT object_name FROM dba_objects WHERE object_type = 'DATABASE LINK' AND object_name = '{link_name.upper()}'"
    # close_link_query = f"ALTER SESSION CLOSE DATABASE LINK {link_name}"
    
    # Step 1. check if the PDB name is available
    # done when the form is filled by the client   
    # Step 2 - check connection
    # connect as sysdba
    dsn_tns = oracledb.makedsn(hostname, port, service_name)
    
    # try to connect to the source
    try:
        conn = oracledb.connect(user=standalone_db_user, password=standalone_db_pwd, dsn=dsn_tns, mode=oracledb.SYSDBA)
    except Exception as e:
        error, = e.args
        raise APIException401(str(error))
    # Step 3 - Check the source version is 12 and above
    # create a cx cursor for the source
    try:
        cursor = conn.cursor()
    except Exception as error:
        raise APIException403(str(error))
    
    try:
        cursor.execute("SELECT version FROM V$INSTANCE")
        banner = cursor.fetchone()[0]
    except Exception as e:
        error, = e.args
        raise APIException404(str(error))
    
    try:
        standalone_version = float('.'.join(banner.split('.')[:2]))
        container_version = float('.'.join(cdb.version.split('.')[:2]))
    except Exception as e:
        error, = e.args
        raise APIException400(str(error))

    
    
    # standalone_version = float(banner[banner.index("Release")+8:banner.index("Release")+12])
    if not standalone_version > 12:
        too_small_err_msg = f"The source database version is {standalone_version}. Source database version must be 12 and above.\nCan not continue" 
        return JsonResponse({'status': 'false', 'message': too_small_err_msg, 'verbose': True}, status=406)
    
    # Step 4 - Check the source and target versions are compatible
    # container version as float

    # raise if the version is not compatible
    if not standalone_version == container_version:
        incomp_err_msg = f"The source database version {standalone_version} is not compatible with the CDB\'s version {container_version}.\nCan not continue" 
        return JsonResponse({'status': 'false', 'message': incomp_err_msg, 'verbose': True}, status=406)    
    
    # Step 5 - Check the source database is a non CDB
    try:
        cursor.execute("SELECT CDB FROM v$database")
        res = cursor.fetchone()
    except Exception as e:
        error, = e.args
        raise APIException404(str(error))    
    
    if res and 'YES' in res[0]:                
        err ='''The source database is a container database. 
Cannot duplicate an entire container database into a pluggable database.
You can however register the source database as a container and duplicate specific pluggable database from it'''
        return JsonResponse({'status': 'false', 'message': err, 'verbose': True}, status=406) 
    # Step 6 - Check the source database is open in read only mode

    try:
        cursor.execute(f"GRANT CREATE SESSION, CREATE PLUGGABLE DATABASE TO {standalone_db_user}")
    except Exception as e:
        error, = e.args
        print(str(error))
        
    
    try:
        cursor.execute("SELECT open_mode FROM v$database")
        res = cursor.fetchone()
    except Exception as e:
        error, = e.args
        raise APIException404(str(error))

    try:
        cursor.close()
        conn.close()
    except Exception:
        pass  
    
    if not res or 'READ ONLY' not in res[0]:
        read_only_err = '''The source database must be opened for read only.
During the duplicate the source is not available for transactions.
Use this to set the source database in read mode:
SHUTDOWN IMMEDIATE;
STARTUP MOUNT;
ALTER DATABASE OPEN READ ONLY;'''
        return JsonResponse({'status': 'false', 'message': read_only_err, 'verbose' : True}, status=406)  
    
    
    try:
        # connect to target container
        conn2 = connect_to_oracle_database(cdb=cdb)
        cursor = conn2.cursor()
    except Exception as error:
        raise APIException401(str(error))
    
    if type(conn2) is dict:
        return JsonResponse(conn2, status = 404)
    # Step 7 - Ensure the target PDB name isn't taken
    try:
        cursor.execute(f"SELECT count(*) FROM dba_pdbs WHERE UPPER(pdb_name)=UPPER('{target_PDB}')")
        res = cursor.fetchone()
    except Exception as e:
        error, = e.args
        raise APIException404(str(error)) 
    
    if res and int(res[0]) > 0:
        already_exist = f"A PDB with the name {target_PDB} already exists"
        return JsonResponse({'status': 'false', 'message': already_exist, 'verbose': True}, status=409)
    
    # Step 8 - Create a database link to the source
    # drop the database link <link_name>
    
    try:
        cursor.execute(drop_link_query)
    except Exception as e:
        error, = e.args
        error = str(error)
        if not any(err in error for err in  ["ORA-04054","ORA-02024"]):
            raise APIException500(error)
        
    # execute database link <link_name>
    
    try:
        cursor.execute(link_query)
    except oracledb.DatabaseError as e:
        error, = e.args
        raise APIException409(str(error))
    
    # ensure the link creation succeeded
    try:
        cursor.execute(link_exists_query)
        #if there are no link - will be exception, right what we need
        link_name = cursor.fetchone()[0]
    except Exception as e:
        error, = e.args
        link_err = f"Querying the source database using database link {link_name} pointing to the source database failed with\n{str(error)}"
        return JsonResponse({'status': 'false', 'message': link_err, 'verbose': True}, status=404)
        
    try:
        cursor.close()
        conn2.close()
    except Exception:
        pass
    
    # create directory for standalone
    
    if re.match(r'^\/(?!\/)[a-zA-Z0-9._-]+(\/[a-zA-Z0-9._-]+)*$', file_dest):
        try:
            client = create_ssh_connection(cdb.host.name, cdb.host.username, cdb.host.password, cdb.host.private_key, cdb.host.ssh_port)
            if isinstance(client, tuple):
                if hasattr(client[1], 'strerror'):
                    raise APIException401(client[1].strerror)
                raise APIException401(client[1])
            re.sub("//", "/", file_dest)
            file_path = file_dest + '/' + target_PDB
            jsn = json.loads(remote_command(client, f"sudo mkdir -p '{file_path}'", True), strict=False)
            if jsn['stderr'] != '':
                raise APIException409(jsn['stderr'])  
            jsn = json.loads(remote_command(client, f"sudo chown -R oracle:oinstall '{file_dest}'", True), strict=False)
            if jsn['stderr'] != '':
                raise APIException409(jsn['stderr'])     
        except Exception as e:
            error, = e.args
            raise APIException409(str(error))   
    else:
        return JsonResponse({'status': 'false', 'message': "invalid file destination path"}, status=406)
        
    cdb_data = object_to_json(cdb)
    host_data = c123_HostSerializer(cdb.host).data
    created_by = request.user
    data_dir = os.path.join(file_dest,target_PDB)
    if os.name == PythonOS.WINDOWS.value:
        data_dir = data_dir.replace(os.sep, os.altsep)
    
    duplicated_sa = c123_node.objects.create(created_by = created_by, name = target_PDB, type = DBType.ORACLE_PDB.value, last_update_by = created_by,
                port = port, cluster = cdb.cluster, cluster_role = ClusterRoles.PDB.value, status = "duplicated from standalone", host = cdb.host, severity = Severity.NOT_OK.value,
                database_user = standalone_db_user, database_pwd = standalone_db_pwd, sizing = Node_Size_Allowed.MEDIUM.name, dir_123Cluster = data_dir,
                ex_params = 
                {
                    "cdb_id": cdb.id,
                    "file_dest": data_dir,
                    "also_start_standby": False,
                    "replicate_to_standby": False,
                })
    tmp_data = json.dumps({'cdb_data': cdb_data, 'host_data': host_data,  'target_PDB': target_PDB, 'file_dest': file_dest, 'target_db_user': standalone_db_user, 'target_db_pwd': standalone_db_pwd, 'start_after_copy': start_after_copy, 'duplicated_sa_id': duplicated_sa.id, "link_name": link_name, 'created_by':created_by.id})
    task = Task.objects.create(task='duplicate_oracle_standalone_task',
                                displayment=f'duplicate {duplicated_sa.name}',
                                target_type='ORACLE_STANDALONE',
                                target_id=cdb.id,
                                org_id = cdb.org_id,
                                schedule=Schedule.NOW.value,
                                status=TaskState.PENDING.value,
                                parameters=tmp_data,
                                created_by=created_by)  
    return append_status_cmd(request, task)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def CanOpenPDB(request):
    """utility to find if maximum number of PDB has been reached"""
    cdb_id = request.GET.get('cdb_id')
    cdb =  c123_node.objects.filter(pk=cdb_id, type = DBType.ORACLE_CDB.value,  ind_deleted = False).first()
    if not cdb:
        raise APIException410(f"CDB ID {cdb_id} is not available")
    max_num_of_open = cdb.ex_params.get('max_open_pdbs')
    curr_open_pdbs = c123_node.objects.filter(cluster=cdb.cluster,  type = DBType.ORACLE_PDB.value,  is_started=True).count()
    return JsonResponse({'result': not max_num_of_open or curr_open_pdbs >= max_num_of_open}, status=200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckPDBName(request):
    """async called from ajax pre-duplicate operation
    returns YES the pdb_name used for copying the standalone
    is available otherwise returns NO"""
    cdb_id = request.data.get('cdb_id')
    pdb_name = request.data.get('pdb_name')
    cdb = c123_node.objects.filter(pk=cdb_id,  type = DBType.ORACLE_CDB.value, ind_deleted = False ).first()
    if not cdb:
        return JsonResponse({"error": f"cdb id {cdb_id} not available"}, status = 410)
    if re.match(r'^[a-zA-Z][a-zA-Z0-9_$#]{0,127}$', pdb_name):
        # try to connect to the source
        try:
            conn = connect_to_oracle_database(cdb=cdb)
        except Exception as e:
            error, = e.args
            print("CheckPDBName: ", str(error))
            return JsonResponse({'error': error}, status=401)
        
        
        if type (conn) is dict:
            return JsonResponse(conn, status = 404)
        try:
            cursor = conn.cursor()
        except Exception as error:
            print("CheckPDBName: ", str(error))
            return JsonResponse({'error': error}, status=401)
        try:
            cursor.execute(f"SELECT COUNT(*) FROM dba_pdbs WHERE UPPER(pdb_name)=UPPER('{pdb_name}')")
        except oracledb.DatabaseError as e:
            error = e.args[0]
            print("CheckPDBName: ", str(error))
            return JsonResponse({'error': str(error)}, status=410)
        if (not 0 in cursor.fetchone()):
            return JsonResponse({'error': "already exists"}, status=409) 
        else:
            cursor.close()
            conn.close()
            return JsonResponse({'response': "name available"}, status=200)
    else:
        return JsonResponse({'error': "invalid name"}, status=406)
class OracleStandalonesList(generics.ListAPIView):
    permission_classes =[IsAuthenticated]
    serializer_class = c123_OracleStandaloneSerializer
    def get_queryset(self):
        user = self.request.user
        if user.is_superuser or user.groups.filter(name='Administrators').exists():
            return c123_Oracle_Standalone.objects.filter(org_id = user.profile.org_id)

        return c123_Oracle_Standalone.objects.filter(created_by = user, org_id = user.profile.org_id)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))  
def GetMaxPDBs(request):
    """helper view function to return max number of pdbs for a cdb"""
    user = request.GET.get('user')
    pwd = format_password(request.GET.get('pwd'))
    host = request.GET.get('host')
    port = int(request.GET.get('port'))
    sname = request.GET.get('sname')
    try:
        conn = connect_to_oracle_database_using_params(host=host, port=port, sname=sname, user=user, pwd=pwd)
    except Exception as e:
        error, = e.args
        print("GetMaxPDBs: ", str(error))
        return JsonResponse({'error': error}, status=401)
    else:   
        if type (conn) is dict:
            return JsonResponse(conn, status = 404)
        try:
            cursor = conn.cursor()
        except Exception as error:
            print("GetMaxPDBs: ", str(error))
            return JsonResponse({'error': str(error)}, status=500)
    try:
        cursor.execute("SELECT value FROM v$parameter WHERE NAME='MAX_PDBS'")
    except Exception as e:
        error, = e.args
        print("GetMaxPDBs: ", str(error))
        return JsonResponse({'error': error}, status=500)
    else:
        result = cursor.fetchone()
        if result is not None:
            result = result[0]
        else:
            result = 100
            print(f"result is None {result}")
    finally:
        cursor.close()
        conn.close()
    return JsonResponse({'data': result}, status=200)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckOracleVersionExists(request):
    """check if a oracle software version already installed"""
    host_name = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)

    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")
    version = request.data.get("software_version")
        
    host = SimpleNamespace( **{
        "name": host_name, 
        "username": username, 
        "password": password, 
        "private_key": private_key,
        "ssh_port": ssh_port
    })
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    jsn = json.loads(remote_command(client, f"sudo -iu oracle lsnrctl status | grep -oP 'Version\s+\K\d+' | head -1", True))
    if(jsn['stderr'] != ''):
        print("check_oracle_version_exists:", jsn['stderr'])
    if jsn['stdout'] != '':
         return JsonResponse({'exists': str(version) in jsn['stdout']}, status=200)

        
    return JsonResponse({'exists': False}, status=200)


@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def GetCDBSettings(request, id):
    node = c123_node.objects.filter(type = DBType.ORACLE_CDB.value, pk = id,  ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(id))
    
    conn = connect_to_oracle_database(cdb=node)
    if isinstance(conn, tuple):
        if hasattr(conn[1], 'strerror'):
            raise APIException401(conn[1].strerror)
        raise APIException401(conn[1])
    
    try:
        with conn:
            with conn.cursor() as cur:
                cur.execute("SELECT NAME, VALUE, ISDEFAULT, ISSYS_MODIFIABLE, 'default' AS COLOR FROM V$PARAMETER")
                columns = [col[0] for col in cur.description]
                cur.rowfactory = lambda *args: dict(zip(columns, args))
                settings = cur.fetchall()
                return JsonResponse(settings, status = 200, safe = False)

    except Exception as e:
        raise APIException401(e.strerror if hasattr(e, 'strerror') else str(e))
     

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def EditCDBSettings(request):
   
    try:
        node_id = request.data.get('node_id')
        parameters = request.data.get('parameters')
        created_by = request.user
        node = c123_node.objects.filter(type = DBType.ORACLE_CDB.value, pk = node_id,  ind_deleted = False).first()
        if not node:
            raise APIException410("node id {} not available".format(node_id))

        tmp_data = json.dumps({'cdb_data': object_to_json(node), 'host_data': object_to_json(node.host), 'parameters': parameters, 'created_by': created_by.id})
        task = Task.objects.create(task='cdb_parameters_apply_task',
                                displayment=f'applying parameters to cluster of {node.name}',
                                target_type=DBType.ORACLE_CDB.value,
                                target_id=node.pk,
                                org_id=node.org_id,
                                schedule=Schedule.NOW.value,
                                status=TaskState.PENDING.value,
                                parameters=tmp_data,
                                created_by=created_by)
        return append_status_cmd(request, task)

    except Exception as e:
        print(traceback.format_exc())
        raise APIException401(e.strerror if hasattr(e, 'strerror') else str(e))

class NodesFlashbacksList(generics.ListAPIView):
    permission_classes =[IsAuthenticated]
    serializer_class = c123_inventory_serializer
    def get_queryset(self):
        user = self.request.user
        if user.is_superuser or user.groups.filter(name='Administrators').exists():
            c123_inventory.objects.filter(type = Inventory.FLASHBACK.value)
        return c123_inventory.objects.filter(created_by = user, type = Inventory.FLASHBACK.value)
class PDBCreateView(generics.CreateAPIView):
    permission_classes =[IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_CreatePDBSerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)
class CDBCreateView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_RegisterCDBSerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)

class CDBInstallView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_InstallCDBSerializer
    def create(self, request, *args, **kwargs):
       return get_create_node_response(self, request, *args, **kwargs)