from django.http import JsonResponse
from rest_framework import generics, status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from clusterCore.tasks import allow_or_403, check_space, checkPathExists, create_ssh_connection, cleanup_logs, get_postgres_connection, \
find_real_pg_primary, remote_command, drop_pg_node, find_postgres_bin, get_cron_schedule, append_status_cmd, format_password, append_status_cmd, \
get_backup_description, export_pg_node, export_pg_database, get_avail_change_date
from clusterCore.models import c123_host, c123_node, c123_database, c123_inventory, ClusterSizing, Task
from clusterCore.enums import DBType, ClusterRoles, RestoreOptions, CopyOptions, Status, Schedule, Severity, TaskState, Inventory, Node_Size_Allowed
from .serializer import c123_CreatePGClusterSerializer, c123_CreateHAProxySerializer, c123_AddPGStandbySerializer
from clusterCore.api.serializer import c123_DatabaseSerializer, c123_NodeSerializer, c123_inventory_serializer, c123_HostSerializer, object_to_json, format_size, get_create_node_response
from clusterCore.exceptions import *
from clusterCore.scheduler import schedule_backup, MISFIRE_GRACE_TIME_BACKUP
import json, re, pytz, traceback
from types import SimpleNamespace
from datetime import datetime
from django.utils import timezone
from psycopg2.extras import RealDictCursor
from dateutil.parser import parse
class PgClusterCreateView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_CreatePGClusterSerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)
class PgStandbyAddView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_AddPGStandbySerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)
class HAProxyAddView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_CreateHAProxySerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckPgDatabaseName(request):
    
    primary_id = request.data.get('node_id')
    db_name = request.data.get('name')
    primary = c123_node.objects.filter(pk=primary_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not primary:
        raise APIException410(f"primary id {primary_id} not available")
    if(re.match(r'^[a-zA-Z0-9_]{1,63}$', db_name)):
        # try to connect to the source
        conn = get_postgres_connection(primary)
        if isinstance(conn, tuple):
            if hasattr(conn[1], 'strerror'):
                raise APIException401(conn[1].strerror)
            raise APIException401(conn[1])
        
        try:
            with conn:
                with conn.cursor() as cur:
                    cur.execute("SELECT datname FROM pg_catalog.pg_database WHERE datname = %s", (db_name.strip(),))
                    res = cur.fetchone()
                    if res:
                        return JsonResponse({'error': "already exists"}, status=409)
                    return JsonResponse({'response': "name available"}, status=200)
        except Exception as e:
            raise APIException401(e.strerror if hasattr(e, 'strerror') else str(e))
        
    else:
        return JsonResponse({'error': "invalid name"}, status=406)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckVersionPackageExists(request):
    """check if a postgres version package already exists"""
    host_name = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)
    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")
    version = request.data.get("software_version")
        
    host = SimpleNamespace( **{
        "name": host_name, 
        "username": username, 
        "password": password,
        "private_key": private_key,
        "ssh_port": ssh_port
    })
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    jsn = json.loads(remote_command(client, f"sudo rpm -qa | grep 'postgresql{version}'", True))
    if(jsn['stderr'] != ''):
        print(jsn['stderr'])
        if 'not found' in jsn['stderr']:
            jsn = json.loads(remote_command(client, f"sudo dpkg -l |  grep -E '^ii\s+postgresql-{version}'", True))
            
            if(jsn['stderr'] != ''):                
                return JsonResponse({'error': jsn['stderr']}, status=404)
            elif(jsn['stdout'] != ''):
                return JsonResponse({'exists': True}, status=200)
    if(jsn['stdout'] != ''):
        return JsonResponse({'exists': True}, status=200)
        
    return JsonResponse({'exists': False}, status=200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckHAProxyExists(request):
    """check if a haproxy package already installed"""
    host_name = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)
    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")
        
    host = SimpleNamespace( **{
        "name": host_name, 
        "username": username, 
        "password": password, 
        "private_key": private_key,
        "ssh_port": ssh_port
    })
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    jsn = json.loads(remote_command(client, f"sudo sh -c  'command -v haproxy'", True))
    if(jsn['stderr'] != ''):
         raise APIException500(jsn['stderr'])
    
    return JsonResponse({'exists': jsn['stdout'] != ''}, status=200)
@api_view(["POST"])
@permission_classes((IsAuthenticated, ))
def CheckIfPgPrimary(request):
    pk = request.data.get("node_id")
    node = c123_node.objects.filter(pk = pk, type = DBType.POSTGRES.value, ind_deleted = False).first()
    #if no such node in app - terminate
    if not node:
        raise APIException404(f"node id {pk} not found")
    app_primary = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = node.cluster, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not app_primary:
        raise APIException404(f"node id {app_primary.pk} not found")
    primary = find_real_pg_primary(app_primary, silent = False)
    if primary.pk == node.pk:
        return JsonResponse({"is_primary": True}, status = 200)
    
    return JsonResponse({"is_primary": False}, status = 200)    
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckPgUserExists(request):
    """check if a user postgres already exists"""
    host_name = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)
    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")
        
    host = SimpleNamespace( **{
        "name": host_name, 
        "username": username, 
        "password": password, 
        "private_key": private_key,
        "ssh_port": ssh_port
    })
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    jsn = json.loads(remote_command(client, "sudo cat '/etc/shadow' | grep postgres | awk -F: '{ print $2 }' | xargs", True))
    if(jsn['stderr'] != ''):
        raise APIException500(str(jsn['stderr']))
        
    password = jsn['stdout'].split('\n')[0]
    if password and password != "!!" and password != "*":        
        return JsonResponse({'exists': True}, status=200)
    
    return JsonResponse({'exists': False}, status = 204)
    
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckPgDatabaseSize(request):
    
    node_id = request.data.get('node_id')
    db_name = request.data.get("db_name")
    node = c123_node.objects.filter(pk = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not node:
        raise APIException410(f"node id {node_id} not available")
   
    conn = get_postgres_connection(node)
    if isinstance(conn, tuple):
            raise APIException401(conn[1].strerror if hasattr(conn[1], 'strerror') else str(conn[1]))
    try:
        with conn:
            with conn.cursor() as cur:
                if db_name:
                    cur.execute("SELECT pg_database_size(%s)/1024", (db_name.strip(),))
                else:
                    cur.execute("SELECT SUM(pg_catalog.pg_database_size(d.datname))/1024 FROM pg_catalog.pg_database d")
                res = cur.fetchone()
                db_size = res[0] if res and res[0] else 0.0
               
    except Exception as e:
        raise APIException401(e.strerror if hasattr(e, 'strerror') else str(e))

    client = create_ssh_connection(node.host.name, node.host.username, node.host.password, node.host.private_key, node.host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    
    
    command = 'sudo df -P . | tail -1 | awk \'{print $4}\''
    jsn = json.loads(remote_command(client, command, True), strict=False)
    if(not jsn['stderr'] == ''):
        print(jsn['stderr'])
        return JsonResponse({'detail': jsn['stderr']}, status=404)
    
    free_space = float(jsn['stdout'])
    
    return JsonResponse({"size": db_size, "display_size": format_size(db_size), 'free_space': free_space}, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def AddpgDB(request):
    current_user = request.user
    db_name = request.data.get('db_name')
    if not db_name or db_name.isspace():
          raise APIException406("database name cannot be blank")
    db_name = ''.join(db_name.split())
    if db_name.lower() == 'repmgr':
        raise APIException403("name repmgr is restricted")
    node_id = request.data.get('node_id')
    api_call = request.data.get('rest_api')
    if api_call:
         node = c123_node.objects.filter(type = DBType.POSTGRES.value, name = node_id, ind_deleted = False).first()
    else:
         node = c123_node.objects.filter(type = DBType.POSTGRES.value, pk = node_id, ind_deleted = False).first()
   
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    primary = c123_node.objects.filter(cluster = node.cluster, cluster_role = ClusterRoles.PRIMARY.value, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("no found primary for node {}".format(node.name))
    allow_or_403(primary)
    auto_delete = request.data.get('auto_delete')
    auto_delete_days = request.data.get('auto_delete_days')
    
    existing_dbs = c123_database.objects.filter(name=db_name, node=primary)
    if existing_dbs.exists():
        undeleted = existing_dbs.filter(ind_deleted = False)
        if undeleted.exists():
            db = undeleted.first()
        else:
            db = existing_dbs.first()
            db.ind_deleted = False

        db.created_by = current_user
        db.last_update_by = current_user
        db.status = Status.CREATING.value
        db.database_user = primary.database_user
        db.database_pwd = primary.database_pwd
        db.ind_started = True
        db.type = DBType.POSTGRES.value
        db.save()
    else:    
        db = c123_database.objects.create(name=db_name,
            type=DBType.POSTGRES.value,
            created_by=current_user,
            last_update_by=current_user,
            status=Status.CREATING.value,
            severity=Severity.NOT_OK.value,
            database_user=primary.database_user,
            database_pwd=primary.database_pwd,
            ind_started = True,
            node = primary)
    
    db.ex_params = {}
    db.save()
    
    tmp_data = json.dumps({'node_data': object_to_json(primary),
                           'host_data': object_to_json(primary.host),
                           'db': object_to_json(db),
                           'auto_delete': auto_delete,
                           'created_by': current_user.id})
    task = Task.objects.create(task='create_pg_database_task',
                                    displayment='creating pg db {}'.format(db_name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=primary.id,
                                    org_id=primary.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StopPgNode(request):
    node_id = request.data.get("node_id")
    current_user = request.user
    api_call = request.data.get('rest_api')
    if api_call:
        node = c123_node.objects.filter(type = DBType.POSTGRES.value, name = node_id, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(type = DBType.POSTGRES.value, pk = node_id, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    node.ind_started = False
    node.status = Status.STOPPING.value
    node.save()
    tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'created_by': current_user.id})
    task = Task.objects.create(task='stop_pg_node_task',
                                    displayment='stopping node {}'.format(node.name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StopHAProxyNode(request):
    node_id = request.data.get("node_id")
    current_user = request.user
    api_call = request.data.get('rest_api')
    if api_call:
        node = c123_node.objects.filter(type = DBType.HAPROXY.value, name = node_id, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(type = DBType.HAPROXY.value, pk = node_id, ind_deleted = False).first()
    
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    
    haproxies = c123_node.objects.filter(type = DBType.HAPROXY.value,  host = node.host, ind_deleted = False)
    haproxies.update(status = Status.STOPPING.value, ind_started = False)
    node_ids = [i.pk for i in haproxies]
    tmp_data = json.dumps({'node_ids': node_ids, 'host_data': object_to_json(node.host), 'created_by': current_user.id})
    task = Task.objects.create(task='stop_haproxy_task',
                                    displayment='stopping haproxy {}'.format(node.name),
                                    target_type=DBType.HAPROXY.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def TogglePgCluster(request):
    """pause/unpause repmgrd service"""
    primary_id = request.data.get("node_id")
    # toggle_pause = true or false
    toggle_pause = request.data.get("toggle_pause")
    current_user = request.user
    primary = c123_node.objects.filter(type = DBType.POSTGRES.value, pk = primary_id, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException410("primary id {} not available".format(primary_id))
    standbys =  c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = primary.cluster, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False)
    standbys_id = []
    if standbys.count() > 0:
        for standby in standbys:
            if not standby.ex_params.get('repmgr_paused'):
                standby.ex_params['repmgr_paused'] = False
                standby.save()
            standbys_id.append(standby.pk)
    
    if not primary.ex_params.get('repmgr_paused'):
        primary.ex_params['repmgr_paused'] = False
        primary.save()
    tmp_data = json.dumps({'primary_data': object_to_json(primary), 'host_data': object_to_json(primary.host), 'standbys_id': standbys_id, 'toggle_pause': toggle_pause, 'created_by': current_user.id})
    
    if toggle_pause:
        displayment='pausing cluster {}'.format(primary.name)
    else:
        displayment='unpausing cluster {}'.format(primary.name)
    task = Task.objects.create(task='pause_repmgr_task',
                                displayment=displayment,
                                target_type=DBType.POSTGRES.value,
                                target_id=primary.id,
                                org_id=primary.org_id,
                                status=TaskState.PENDING.value,
                                schedule=Schedule.NOW.value,
                                parameters=tmp_data,
                                created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RemovePgSchedule(request):
    node_id = request.data.get("node_id")
    primary = c123_node.objects.filter(type = DBType.POSTGRES.value, pk=node_id, cluster_role = ClusterRoles.PRIMARY.value).first()
    if not primary:
        raise APIException404("primary id {} not found".format(node_id))
    allow_or_403(primary)
    from django.conf import settings
    backup = c123_inventory.objects.filter(node = primary, type = Inventory.BACKUP.value, is_sched = True).last()
    if backup:
        job_id = backup.ex_params.get("job_id")
        job = settings.scheduler.get_job(job_id=job_id)
        if job:
            settings.scheduler.remove_job(job_id=job_id)
            notification =  notification = f"{job_id} removed"
            return JsonResponse({"notification": notification}, status=200)
        
        else:
            standbys = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, cluster = primary.cluster)
            if standbys.exists():
                for standby in standbys:
                    backup = c123_inventory.objects.filter(type = Inventory.BACKUP.value, node = standby, is_sched = True).last()
                    if backup:
                        job_id = backup.ex_params.get("job_id")
                        job = settings.scheduler.get_job(job_id=job_id)
                        if job:
                            settings.scheduler.remove_job(job_id=job_id)
                            notification = f"{job_id} removed"
                            return JsonResponse({"notification": notification}, status=200)
                
                return JsonResponse({"notification": "no schedule backup found"}, status = 200)            
            else:
               return JsonResponse({"notification": "no schedule backup found"}, status = 200)


@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def SwitchPgNode(request):
    standby_id = request.data.get("standby_id")
    current_user = request.user
    api_call = request.data.get('rest_api')
    if api_call:
        standby = c123_node.objects.filter(name = standby_id, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    else:
        standby = c123_node.objects.filter(pk = standby_id, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    if not standby:
        raise APIException404("standby id {} not found".format(standby_id))
    allow_or_403(standby)
    #primary
    primary = c123_node.objects.filter(cluster = standby.cluster, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.PRIMARY.value , ind_deleted = False).first()
    if not primary:
        raise APIException404(f"primary of standby {standby.name} not found")
    primary = find_real_pg_primary(primary, silent = False)
    in_switchover = Task.objects.filter(task = 'switchover_repmgr_task', target_id = primary.pk, status__in = [TaskState.PENDING.value, TaskState.RUNNING.value]).first()
    if in_switchover:
        raise APIException406(f"primary {primary.name} are already in switchover (task id {in_switchover.pk})")
    other_standbys = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, cluster = standby.cluster, ind_deleted = False).exclude(pk = standby.pk)
    siblings = []
    id_list = []
    for i in other_standbys:
        id_list.append(i.pk)
        siblings.append({'host': object_to_json(i.host), 'node': object_to_json(i)})
    id_list.append(primary.pk)
    
    standby.status = Status.SWITCHOVER.value
    primary.status = Status.SWITCHOVER.value.format(standby.name)
    primary.save()
    standby.save()
    tmp_data = json.dumps({'standby_data': object_to_json(standby), 'standby_host': object_to_json(standby.host), 'primary_data': object_to_json(primary), 'primary_hostname': primary.host.name, 'siblings_data': siblings, 'id_list': id_list, 'created_by': current_user.id})
    task = Task.objects.create(task='switchover_repmgr_task',
                                    displayment='switching standby {} to primary'.format(standby.name, primary.name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=primary.id,
                                    org_id=primary.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ClonePgStandby(request):
    standby_id = request.data.get("standby_id")
    current_user = request.user
    api_call = request.data.get('rest_api')
    if api_call:
        standby = c123_node.objects.filter(name = standby_id, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    else:
        standby = c123_node.objects.filter(pk = standby_id, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    if not standby:
        raise APIException404("standby id {} not found".format(standby_id))
    allow_or_403(standby)

    in_clone = Task.objects.filter(task = 'clone_standby_repmgr_task', target_id = standby.pk, status__in = [TaskState.PENDING.value, TaskState.RUNNING.value]).first()
    if in_clone:
        raise APIException406(f"standby {standby.name} are already cloning (task id {in_clone.pk})")

    primary = c123_node.objects.filter(cluster = standby.cluster, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.PRIMARY.value , ind_deleted = False).first()
    if not primary:
        raise APIException404(f"primary of standby {standby.name} not found")
    primary = find_real_pg_primary(primary, silent = False)    
    standby.status = Status.SWITCHOVER.value
    standby.save()
    tmp_data = json.dumps({'standby_data': object_to_json(standby), 'host_standby': object_to_json(standby.host), 'primary_data': object_to_json(primary), 'host_primary': object_to_json(primary.host), 'created_by': current_user.id})
    task = Task.objects.create(task='clone_standby_repmgr_task',
                                    displayment='cloning standby {}'.format(standby.name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=standby.pk,
                                    org_id=standby.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CopyPgDatabase(request):
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    
    db_id = request.data.get("db_id")
    node_id = request.data.get("node_id")
    source_id = request.data.get("source_id")
    current_user = request.user
    db_option = request.data.get("option")
    #copy_option = 'one_time' or 'live'
    copy_option = request.data.get("copy_option")
    copy_name = request.data.get("copy_name")
    db_exists = request.data.get("db_exists")
    api_call = request.data.get("rest_api")
    if api_call:
        node_target = c123_node.objects.filter(name = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    else:
        node_target = c123_node.objects.filter(pk = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    
    if not node_target:
        raise APIException410("target node id {} not available".format(node_id))
    
    if api_call:
        node_source = c123_node.objects.filter(name = source_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    else:
        node_source = c123_node.objects.filter(pk = source_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    
    if not node_source:
        raise APIException410("source node id {} not available".format(source_id))
    
    primary_target = c123_node.objects.filter(cluster = node_target.cluster,  type = DBType.POSTGRES.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary_target:
        raise APIException404("no found primary for target node {}".format(node_target.name))
    allow_or_403(primary_target)

    primary_source = c123_node.objects.filter(cluster = node_source.cluster, cluster_role = ClusterRoles.PRIMARY.value, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not primary_source:
        raise APIException404("no found primary for source node {}".format(node_source.name))
    allow_or_403(primary_source)
    
    if api_call:
        db_target = c123_database.objects.filter(name = db_id, node = node_source, type = DBType.POSTGRES.value, ind_deleted = False).first()
    else:
        db_target = c123_database.objects.filter(pk = db_id, type = DBType.POSTGRES.value, ind_deleted = False).first()

    if not db_target:
        raise APIException410("target database id {} not available".format(db_id))
    
    db_source = c123_database.objects.get(pk=db_target.pk)
    if(db_option):
        # here we do for existing database
        match db_option:
            case CopyOptions.OVERRIDE.value:
                copy_name = db_target.name
                db_target.ind_started = False
                db_target.node = primary_target
                db_target.database_user = primary_target.database_user
                db_target.database_pwd = primary_target.database_pwd
                db_target.status = Status.REPLICATING.value
                db_target.save()
            case CopyOptions.COPY.value:
                existing_dbs = c123_database.objects.filter(node = primary_target, name = copy_name)
                if not existing_dbs.exists():
                        db_target = c123_database.objects.create(node = primary_target, name = copy_name,
                        type = db_source.type, size = db_source.size,
                        database_user = primary_target.database_user, database_pwd = primary_target.database_pwd,
                        created_by = current_user, last_update_by = current_user,
                        severity = Severity.NOT_OK.value, status = Status.REPLICATING.value
                        )
                else:
                    undeleted = existing_dbs.filter(ind_deleted = False)
                    if undeleted.exists():
                        db_target = undeleted.first()
                    else:
                        db_target = existing_dbs.first()
                        db_target.ind_deleted = False
                        
                    db_target.type = DBType.POSTGRES.value
                    db_target.status = Status.REPLICATING.value
                    db_target.save()
                
                db_source.ind_started = False
                db_source.status = Status.REPLICATING.value
                db_source.save()
    else:
        #here we do for new database
        db_source.status = Status.COPYING.value
        db_source.ind_started = False
        db_source.save()
        db_target = c123_database.objects.create(node = primary_target, name = db_source.name,
        type = db_source.type, size = db_source.size,
        database_user = primary_target.database_user, database_pwd = primary_target.database_pwd,
        created_by = current_user, last_update_by = current_user,
        severity = Severity.NOT_OK.value, status = Status.CREATING.value
        )
     
        
    tmp_data = json.dumps({'db_source': object_to_json(db_source), 'db_target' : object_to_json(db_target), 'node_source': object_to_json(primary_source), 'node_target': object_to_json(primary_target), 'host_source': object_to_json(primary_source.host), 'host_target': object_to_json(primary_target.host), 'backup_host': object_to_json(app_host), 'db_option': db_option, 'db_exists': db_exists, 'copy_option': copy_option, 'copy_name': copy_name, 'created_by': current_user.id})
    task = Task.objects.create(task='copy_pg_database_task',
                                displayment=f'{"live replica " if copy_option == "live" else ""}copy database {db_source.name} into {primary_target.name}',
                                target_type=DBType.POSTGRES.value,
                                target_id=primary_target.pk,
                                org_id=primary_target.org_id,
                                status=TaskState.PENDING.value,
                                schedule=Schedule.NOW.value,
                                parameters=tmp_data,
                                created_by=current_user)
    return append_status_cmd(request, task)
    
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StartPgNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    
    if node.ind_started:
         node.status = Status.RESTARTING.value
         node.save()
         tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'created_by': current_user.id})
         task = Task.objects.create(task='restart_pg_node_task',
                                    displayment='restarting node {}'.format(node.name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    else:
        node.ind_started = True
        node.status = Status.STARTING.value
        node.save()
        tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'created_by': current_user.id})
        task = Task.objects.create(task='start_pg_node_task',
                                    displayment='starting node {}'.format(node.name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def UpgradePgNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    target_version = request.data.get('target_version')
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    
    standbys_set = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False, cluster = node.cluster)
    if int(target_version) == 17 and standbys_set.exists():
        raise APIException406("This node has standbys. Only standalone nodes could be upgraded to postgresql 17")
    standbys_list = [{'node': object_to_json(s), 'host': object_to_json(s.host)} for s in standbys_set]
    
    c123_node.objects.filter(cluster = node.cluster, type = node.type, ind_deleted = False).update(status = Status.UPGRADING.value)
    check_space(node = node, path=node.dir_123Cluster, source_path=node.dir_123Cluster)
    
    tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'app_host': object_to_json(app_host), 'target_version': target_version, 'standbys': standbys_list, 'created_by': current_user.id})
    task = Task.objects.create(task='pg_upgrade_node_task',
                                displayment='upgrading {} to postgresql {}'.format(node.name, target_version),
                                target_type=DBType.POSTGRES.value,
                                target_id=node.id,
                                org_id=node.org_id,
                                status=TaskState.PENDING.value,
                                schedule=Schedule.NOW.value,
                                parameters=tmp_data,
                                created_by=current_user)
    return append_status_cmd(request, task)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StartHAProxyNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.HAPROXY.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.HAPROXY.value, ind_deleted = False).first()
    
    if not node:
        raise APIException410("node id {} not available".format(node_id))
     
    haproxies = c123_node.objects.filter(type = DBType.HAPROXY.value,  host = node.host, ind_deleted = False)
    node_ids = [i.pk for i in haproxies]
    haproxies.filter(ind_started = True).update(status = Status.RESTARTING.value)
    haproxies.filter(ind_started = False).update(status = Status.STARTING.value)
    
    if node.ind_started:
         tmp_data = json.dumps({'node_ids': node_ids, 'host_data': object_to_json(node.host), 'created_by': current_user.id})
         task = Task.objects.create(task='restart_haproxy_task',
                                    displayment='restarting haproxy {}'.format(node.name),
                                    target_type=DBType.HAPROXY.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    else:
        node.ind_started = True
        node.status = Status.STARTING.value
        node.save()
        tmp_data = json.dumps({'node_ids': node_ids, 'host_data': object_to_json(node.host), 'created_by': current_user.id})
        task = Task.objects.create(task='start_haproxy_task',
                                    displayment='starting haproxy {}'.format(node.name),
                                    target_type=DBType.HAPROXY.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)



@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def BackupPgNode(request):
    """backup (basebackup) for pg primary"""
    schedule = request.data.get('schedule')
    node_id = request.data.get('node_id')
    compress = request.data.get("compress")
    backup_path = request.data.get('backup_path')
    #incremental wal backup
    incremental = request.data.get('incremental')
    #incremental physical backup
    incremental_physical = request.data.get("incremental_physical")
    update_schedule = request.data.get("update_schedule")
    api_call = request.data.get('rest_api')
    from django.conf import settings
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        pg_node = c123_node.objects.filter(type = DBType.POSTGRES.value, name=node_id, ind_deleted = False).first()
    else:
        pg_node = c123_node.objects.filter(type = DBType.POSTGRES.value, pk=node_id, ind_deleted = False).first()

    if not pg_node:
        raise APIException410("node id {} not available".format(node_id))
    
    if incremental_physical and float(pg_node.version) < 17.0:
         raise APIException406(f"{pg_node.name} version ({pg_node.version}) must be >= 17 to support physical incremental backups")
    
    # they are different, if both present - prefer the recommended incremental physical backups
    if incremental and incremental_physical:
        incremental = False
    
    if schedule:
        client_timezone = pytz.timezone(request.data.get("client_timezone"))
        schedule_run_time_str = request.data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = request.data.get("days_of_week")
        days_of_month = request.data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        
    if update_schedule and schedule:
        allow_or_403(pg_node)
        backup = c123_inventory.objects.filter(node = pg_node, type = Inventory.BACKUP.value, is_sched = True).last()
        if backup:
            backup.ex_params['description'] = description
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            backup.save()
            job_id = backup.ex_params.get("job_id")
            job = settings.scheduler.get_job(job_id=job_id)
            if job:
                job.reschedule(trigger=trigger)
            else:
                settings.scheduler.add_job(schedule_backup, trigger=trigger, replace_existing=True, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP, kwargs={"task_id": backup.ex_params["task_id"]}, id=job_id)
            notification = f"{backup.node} backup updated to '{description}'"
            return JsonResponse({"notification": notification}, status=200)
        
        else:
            standbys = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, cluster = pg_node.cluster)
            if standbys.exists():
                for standby in standbys:
                    backup = c123_inventory.objects.filter(node = standby.pk, type = Inventory.BACKUP.value, is_sched = True).last()
                    if backup:
                        backup.ex_params['schedule_run_time'] = schedule_run_time_str
                        backup.ex_params['days_of_month'] = days_of_month
                        backup.ex_params['days_of_week'] = days_of_week
                        backup.ex_params['description'] = description
                        backup.save()
                        job_id = backup.ex_params.get("job_id")
                        job = settings.scheduler.get_job(job_id=job_id)
                        if job:
                            job.reschedule(trigger=trigger)
                        else:
                            settings.scheduler.add_job(schedule_backup, replace_existing=True, trigger=trigger, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP, kwargs={"task_id": backup.ex_params["task_id"]}, id=job_id)
                        
                        notification = f"{backup.node} backup updated to '{description}'"
                        return JsonResponse({"notification": notification}, status=200)
                
                return JsonResponse({"notification": "no schedule backup found"}, status = 200)            
            else:
               return JsonResponse({"notification": "no schedule backup found"}, status = 200)
            
    serializer = c123_inventory_serializer(data = request.data, partial = True)
    if serializer.is_valid():
        allow_or_403(pg_node)
        check_space(node = pg_node, path=backup_path, source_path=pg_node.dir_123Cluster)
        size_scheme, created = ClusterSizing.objects.get_or_create(db_type = DBType.POSTGRES.value)
        keep_days = getattr(size_scheme, f'{pg_node.sizing.lower()}_backup_keep_days', None)
        log_archive_path = f'/123cluster/archive_logs/{pg_node.type}/{pg_node.cluster.name}'
    # create the task signature
        if schedule:
            existing_backup = c123_inventory.objects.filter(node = pg_node, type = Inventory.BACKUP.value, is_sched = True)
            if existing_backup.exists():
                raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for cluster.")
            
            if pg_node.cluster_role == ClusterRoles.PRIMARY.value:
                standbys = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, cluster = pg_node.cluster)
            else:
                standbys = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, cluster = pg_node.cluster).exclude(pk = pg_node.pk)
            
            if standbys.exists():
                for standby in standbys:
                    existing_backup = c123_inventory.objects.filter(node = standby, type = Inventory.BACKUP.value, is_sched = True)
                    if existing_backup.exists():
                        raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for cluster.")
            backup = serializer.create(serializer.validated_data)
            backup.node = pg_node
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0}
            backup.ex_params["belong_to"] = DBType.POSTGRES.value
            backup.ex_params["node_version"] = pg_node.version
            backup.is_sched = schedule
            backup.ex_params["backup_type"] = "basebackup"
            backup.ex_params["duration"] = 0
            backup.ex_params["compress"] = compress
            backup.ex_params["modified_date"] = ''
            backup.ex_params['description'] = description
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            backup.ex_params['incremental'] = incremental
            backup.ex_params['incremental_physical'] = incremental_physical
            notification =  "{} backup {}".format(backup.node, description)
            job_id = f"{notification} ({client_timezone})"
            backup.ex_params["job_id"] = job_id
            backup.ex_params["backup_path"] = f'{backup_path}/basebackups/schedule/{pg_node.version}/{pg_node.name}'.replace(":", "_")
            backup.ex_params['log_archive_path'] = log_archive_path
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(pg_node.name, now)
            backup.save()
            # serialize objects for the tasks
            node_data = object_to_json(pg_node)
            host_data = c123_HostSerializer(pg_node.host).data
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
        
            tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_data': object_to_json(backup), "backup_host": backup_host, 'created_by': created_by.id, 'backup_type': 'basebackup', 'compress': compress})
            task = Task.objects.create(task='pg_basebackup_task',
                                            displayment='backup {}'.format(pg_node.name), 
                                            target_type=DBType.POSTGRES.value, 
                                            target_id=pg_node.id,
                                            org_id=pg_node.org_id, 
                                            schedule=Schedule.SCHEDULED.value, 
                                            status=TaskState.ON_DEMAND.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
            settings.scheduler.add_job(schedule_backup, replace_existing=True, trigger=trigger,  kwargs={"task_id": task.pk}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        else:
            existing_backup = Task.objects.filter(target_id = pg_node.id, task = 'pg_basebackup_task', status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
            if existing_backup:
                raise APIException409(f"Already runs backup for {pg_node.name}")
            backup = serializer.create(serializer.validated_data)
            backup.node = pg_node
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0}
            backup.ex_params["belong_to"] = DBType.POSTGRES.value
            backup.ex_params["node_version"] = pg_node.version
            backup.is_sched = schedule
            backup.ex_params["backup_type"] = "basebackup"
            backup.ex_params["duration"] = 0
            backup.ex_params["compress"] = compress
            backup.ex_params["modified_date"] = ''
            backup.ex_params['incremental'] = incremental
            now = timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(pg_node.name, now)
            backup.ex_params["backup_path"] = f'{backup_path}/basebackups/{pg_node.host.name}_{pg_node.port}/{now}'
            backup.ex_params['log_archive_path'] = log_archive_path
            backup.save()
            notification = f"backup of {pg_node.name}"
            # serialize objects for the tasks
            node_data = object_to_json(pg_node)
            host_data = c123_HostSerializer(pg_node.host).data
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
            tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_data': object_to_json(backup), "backup_host": backup_host, 'created_by': created_by.id, 'backup_type': 'basebackup', 'compress': compress})
            task = Task.objects.create(task='pg_basebackup_task',
                                            displayment='backup {}'.format(pg_node.name), 
                                            target_type=DBType.POSTGRES.value, 
                                            target_id=pg_node.id,
                                            org_id=pg_node.org_id, 
                                            schedule=Schedule.NOW.value, 
                                            status=TaskState.PENDING.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
    else:
        print("invalid form")
        print(serializer.errors)        
        return JsonResponse({'error': serializer.errors}, status=404)
    return append_status_cmd(request, task, notification)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ExportPgNode(request):
    """export (dumpall) for pg cluster"""
    data = request.data
    data['created_by'] = request.user.id
    schedule = data.get('schedule')
    api_call = data.get('api_call')
    node_id = data.get('node_id')

    if not schedule:
        task = export_pg_node(data)
        return append_status_cmd(request, task)
    else:
        from django.conf import settings
        client_timezone = pytz.timezone(data.get("client_timezone"))
        schedule_run_time_str = data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = data.get("days_of_week")
        days_of_month = data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        if api_call:
            node = c123_node.objects.filter(type = DBType.POSTGRES.value, name=node_id, ind_deleted = False).first()
        else:
            node = c123_node.objects.filter(type = DBType.POSTGRES.value, pk=node_id, ind_deleted = False).first()
        if not node:
            raise APIException410("node id {} not available".format(node_id))
        job_id =  f"export_{node.name}"
        scheduler = settings.scheduler
        export_job = next((i for i in scheduler.get_jobs() if job_id in i.id), None)
        if export_job:
            export_job.reschedule(trigger=trigger)
        else:
             settings.scheduler.add_job(export_pg_node, replace_existing=True, trigger=trigger,  kwargs={"data": data}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        return JsonResponse({"notification": f"export {node.name} {description} started"}, status = 200)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ExportPgDatabase(request):
    """export for database"""

    data = request.data
    data['created_by'] = request.user.id
    schedule = data.get('schedule')
    api_call = data.get('api_call')
    node_id = data.get('node_id')
    db_id = data.get('db_id')

    if not schedule:
        task = export_pg_database(data)
        return append_status_cmd(request, task)
    else:
        from django.conf import settings
        client_timezone = pytz.timezone(data.get("client_timezone"))
        schedule_run_time_str = data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = data.get("days_of_week")
        days_of_month = data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
        if api_call:
            db = c123_database.objects.filter(type = DBType.POSTGRES.value, name=db_id, node__name = node_id, ind_deleted = False).first()
        else:
            db = c123_database.objects.filter(type = DBType.POSTGRES.value, pk=db_id, ind_deleted = False).first()
        if not db:
            raise APIException410("database id {} not available".format(db_id))
        node = db.node
        job_id =  f"export_{db.name}_{node.name}"
        scheduler = settings.scheduler
        export_job = next((i for i in scheduler.get_jobs() if job_id in i.id), None)
        if export_job:
            export_job.reschedule(trigger=trigger)
        else:
             settings.scheduler.add_job(export_pg_database, replace_existing=True, trigger=trigger,  kwargs={"data": data}, id=job_id, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        return JsonResponse({"notification": f"export db {db.name} of {node.name} {description} started"}, status = 200)
    
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropPgPublication(request):
    """drop publication of postgresql database"""
    db_id = request.data.get('db_id')
    db_sub = c123_database.objects.filter(pk = db_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not db_sub:
        raise APIException410("subscriber database id {} not available".format(db_id))
    
    subscription_key = f'sub_{db_sub.name}'
    publication_key = f'pub_{db_sub.name}'
    
    if not db_sub.ex_params:
        raise APIException404("no publisher info found for database {}".format(db_sub.name))
    
    sub_data = db_sub.ex_params.get(subscription_key)
    if not sub_data:
        raise APIException404("no publisher info found for database {}".format(db_sub.name))
    
    sub_name = sub_data.get('name')
    pub_id = sub_data.get('publisher_db_id')
    db_pub = c123_database.objects.filter(pk = pub_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not db_pub:
        raise APIException410("no publisher available by publisher id {} of database {}".format(pub_id, db_sub.name))
    
    pub_name = db_pub.ex_params.get(publication_key)
        
    node_sub = db_sub.node
    node_pub = db_pub.node

    conn_sub = get_postgres_connection(node_sub, db_sub.name)
    if isinstance(conn_sub, tuple):
        if hasattr(conn_sub[1], 'strerror'):
            raise APIException401(conn_sub[1].strerror)
        raise APIException401(conn_sub[1])
    
    conn_pub = get_postgres_connection(node_pub, db_pub.name)
    if isinstance(conn_pub, tuple):
        if hasattr(conn_pub[1], 'strerror'):
            raise APIException401(conn_pub[1].strerror)
        raise APIException401(conn_pub[1])
    
    try:
        conn_pub.autocommit = True
        conn_sub.autocommit = True
        cur_pub =  conn_pub.cursor()
        cur_sub = conn_sub.cursor()
        cur_sub.execute('ALTER SUBSCRIPTION {} DISABLE'.format(sub_name))
        cur_sub.execute('ALTER SUBSCRIPTION {} SET (slot_name = NONE)'.format(sub_name))
        cur_sub.execute('DROP SUBSCRIPTION IF EXISTS {}'.format(sub_name))
        cur_pub.execute('DROP PUBLICATION IF EXISTS {}'.format(pub_name))
        db_sub.ex_params.pop(subscription_key, None)
        db_pub.ex_params.pop(publication_key, None)
        db_pub.save()
        db_sub.save()
    except Exception as e:
        raise APIException403(str(e))

    return JsonResponse({}, status=200)


@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def TogglePgSubscription(request):
    """pause / unpause logical replication on database"""
    
    db_id = request.data.get('db_id')
    db_sub = c123_database.objects.filter(pk = db_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    if not db_sub:
        raise APIException410("subscriber database id {} not available".format(db_id))
    
    subscription_key = f'sub_{db_sub.name}'
    
    if not db_sub.ex_params:
        raise APIException404("no publisher info found for database {}".format(db_sub.name))
    
    sub_data = db_sub.ex_params.get(subscription_key)
    if not sub_data:
        raise APIException404("no publisher info found for database {}".format(db_sub.name))

    node_sub = db_sub.node

    sub_name = sub_data.get('name')

    conn_sub = get_postgres_connection(node_sub, db_sub.name)
    if isinstance(conn_sub, tuple):
        if hasattr(conn_sub[1], 'strerror'):
            raise APIException401(conn_sub[1].strerror)
        raise APIException401(conn_sub[1])

    try:
        conn_sub.autocommit = True
        cur_sub = conn_sub.cursor()
        if db_sub.ind_started:
            cur_sub.execute('ALTER SUBSCRIPTION {} DISABLE'.format(sub_name))
            cur_sub.execute('ALTER SUBSCRIPTION {} SET (slot_name = NONE)'.format(sub_name))

        else:
            cur_sub.execute("ALTER SUBSCRIPTION {} SET (slot_name = '{}')".format(sub_name, sub_name))
            cur_sub.execute('ALTER SUBSCRIPTION {} ENABLE'.format(sub_name))
        db_sub.ind_started = not db_sub.ind_started
        db_sub.save()
    except Exception as e:
        raise APIException403(str(e))

    return JsonResponse({}, status=200)

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreBackupPgDB(request):
    """restore a postgresql db from dump backup
    the call is made from client AJAX POST that contains the backup id"""
    bu_id = request.data.get('bu_id')
    include_data = request.data.get('include_data')
    include_publications = request.data.get('include_publications')
    include_subscriptions = request.data.get('include_subscriptions')
    autostart = True if request.data.get('autostart') == 'true' else False
    api_call = request.data.get('rest_api')
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        backup = c123_inventory.objects.filter(name=bu_id, type = Inventory.EXPORT.value).first()
    else:
        backup = c123_inventory.objects.filter(pk=bu_id, type = Inventory.EXPORT.value).first()

    if not backup:
        raise APIException404("export id {} not found".format(bu_id))
    
    path = backup.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")

    db = backup.database
    node = db.node
    primary = c123_node.objects.filter(cluster = node.cluster, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("primary not found")
    allow_or_403(primary)
    db.status = Status.RESTORING.value
    db.save()
    backup_host = c123_HostSerializer(app_host).data
    db_data = object_to_json(db)
    backup = object_to_json(backup)
    node_data = object_to_json(primary)
    created_by = request.user
    host_primary = c123_HostSerializer(primary.host).data

    tmp_data = json.dumps({'db_data': db_data, 'node_data': node_data, 'backup_host': backup_host, 'host_primary': host_primary,  'backup': backup,  'autostart': autostart, 'include_data': include_data, 'include_publications': include_publications, 'include_subscriptions': include_subscriptions, 'created_by': created_by.id})
    task = Task.objects.create(task='pg_restore_task',
                            displayment='restore pg database {}'.format(db.name),
                            target_type=DBType.POSTGRES.value,
                            target_id=db.id,
                            org_id=primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropPgNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    final_backup = request.data.get('final_backup')
    if api_call:
        node = c123_node.objects.filter(name = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id, type = DBType.POSTGRES.value, ind_deleted = False).first()    
   
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    
    task = drop_pg_node(node, final_backup)
         
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropHAProxyNode(request):
    haproxy_ids = request.data.get('haproxies')
    api_call = request.data.get('rest_api')
    if not haproxy_ids or len(haproxy_ids) < 1:
        raise APIException400('no haproxy ids submitted')
    if api_call:
        haproxies = c123_node.objects.filter(type = DBType.HAPROXY.value, ind_deleted = False, name__in = haproxy_ids)
        haproxy_ids = [i.pk for i in haproxies]
    else:
        haproxies = c123_node.objects.filter(type = DBType.HAPROXY.value, ind_deleted = False, id__in = haproxy_ids)   
    
    if not haproxies.exists():
         raise APIException404('no haproxy found for given ids')
    
    haproxy_data =  [{'port': i.port, 'host': object_to_json(i.host), 'has_other_on_host': c123_node.objects.filter(host = i.host, type = DBType.HAPROXY.value, ind_deleted = False).exclude(name__in=haproxy_ids).exists() if api_call else  c123_node.objects.filter(host = i.host, type = DBType.HAPROXY.value, ind_deleted = False).exclude(id__in=haproxy_ids).exists()} for i in haproxies]
    first = haproxies.first()
    cluster = first.cluster
    if api_call:
        has_other_haproxies = c123_node.objects.filter(type = DBType.HAPROXY.value, cluster = cluster, ind_deleted = False).exclude(name__in = haproxy_ids).exists()
    else:
        has_other_haproxies = c123_node.objects.filter(type = DBType.HAPROXY.value, cluster = cluster, ind_deleted = False).exclude(id__in = haproxy_ids).exists()

    created_by = request.user
    tmp_data = json.dumps({'haproxy_data': haproxy_data, 'haproxy_ids': haproxy_ids, 'cluster_id': cluster.pk, 'has_other_haproxies': has_other_haproxies,  'created_by': created_by.id})               
    task = Task.objects.create(task='drop_haproxy_task',
                            displayment='drop haproxies for cluster {}'.format(cluster.name),
                            target_type=DBType.HAPROXY.value,
                            target_id=first.pk,
                            org_id=first.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)  
         
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropPgDatabase(request):
    db_id = request.data.get("db_id")
    node_id = request.data.get("node_id")
    api_call = request.data.get('rest_api')
    if api_call:
        db = c123_database.objects.filter(type = DBType.POSTGRES.value, name = db_id, node__name = node_id, ind_deleted = False).first()
    else: 
        db = c123_database.objects.filter(type = DBType.POSTGRES.value, pk = db_id, ind_deleted = False).first()
    if not db:
        raise APIException410("database id {} not found".format(db_id))
    node = db.node
    primary = c123_node.objects.filter(cluster = node.cluster, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("no found primary for node {}".format(node.name))
    db.status = Status.DROPPING.value
    db.save()
    cleanup_logs(db.pk, db.type)
    node_data = object_to_json(primary)
    db_data = object_to_json(db)
    host_data = c123_HostSerializer(primary.host).data
    created_by = request.user
    
    tmp_data = json.dumps({'node_data': node_data, 'db_data': db_data, 'host_data': host_data, 'created_by': created_by.id})               
    task = Task.objects.create(task='drop_pg_database_task',
                            displayment='drop database {}'.format(db.name),
                            target_type=DBType.POSTGRES.value,
                            target_id=db.pk,
                            org_id=primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
         
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreBackupPgNode(request):
    """restore a postgresql node from backup
    the call is made from client AJAX POST that contains the backup id"""
    
    bu_id = request.data.get('bu_id')
    # restore_option = 'this'/'latest'/'custom'
    restore_option = request.data.get('restore_option')
    restore_time = request.data.get('restore_time')
    autostart = request.data.get('autostart')
    api_call = request.data.get('rest_api')
    from django.conf import settings  
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        backup = c123_inventory.objects.filter(name=bu_id, type = Inventory.BACKUP.value).first()
    else:
        backup = c123_inventory.objects.filter(pk=bu_id, type = Inventory.BACKUP.value).first()

    if not backup:
        raise APIException404("backup id {} not found".format(bu_id))
    
    path = backup.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")
    
    node = backup.node
    allow_or_403(node)
    
    forbid_restore = Task.objects.filter(target_id = node.id, task__in = ['pg_node_restore_task', 'pg_basebackup_task'], status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
    if forbid_restore:
        raise APIException409(f"Already runs backup/restore for {node.name}")
    
    primary = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = node.cluster, cluster_role=ClusterRoles.PRIMARY.value, ind_deleted=False).first()
    if not primary:
        #because it can not exist, or exist as deleted
        raise APIException410(f"primary of standby {node.name} is not available")
    
    primary = find_real_pg_primary(primary)
    primary.status = Status.RESTORING.value
    primary.save()
    if RestoreOptions.THIS.value in restore_option:
        #as app time zone (if by a chance representation timezone will differ from internal)
        date = backup.created_date.astimezone(pytz.timezone(settings.TIME_ZONE)).strftime("%Y-%m-%d %H:%M:%S")
    elif RestoreOptions.CUSTOM.value in restore_option and restore_time:
        # example date = '2021-12-30 09:08:33'
        # date = datetime.strptime(restore_time, "%Y-%m-%dT%H:%M:%S.%f%z").astimezone()
        # date = date.strftime("%Y-%m-%d %H:%M:%S")
        date = restore_time
    else:
        date = ''
    
    standbys_id = []
       
    # serialize objects for the tasks
    backup_host = c123_HostSerializer(app_host).data
    backup = object_to_json(backup)
    node_data = object_to_json(primary)
    host_data = c123_HostSerializer(primary.host).data    
    
    created_by = request.user
    standbys_list = []
    standbys_set = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False, cluster = node.cluster)
    standbys_exists = standbys_set.exists()
    for standby in standbys_set:
        standbys_list.append({'node': object_to_json(standby), 'host': object_to_json(standby.host)})
        standbys_id.append(standby.id)
    if node.cluster_role == ClusterRoles.PRIMARY.value and c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = node.cluster, ind_deleted = False, severity = Severity.OK.value, cluster_role = ClusterRoles.STANDBY.value).exists():
        autostart = True
    
    tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_host': backup_host, 'backup': backup, 'autostart': autostart, 'standbys_exists': standbys_exists, 'standbys':standbys_list, 'restore_option': restore_option, 'date': date, 'app_db_timezone': settings.TIME_ZONE, 'standbys_id': standbys_id, 'created_by': created_by.id})
    task = Task.objects.create(task='pg_node_restore_task',
                            displayment='restore pg node {}'.format(primary.name),
                            target_type=DBType.POSTGRES.value,
                            target_id=primary.id,
                            org_id=primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def ImportPgNode(request):
    """restore (import) a postgresql cluster from dump_all backup
    the call is made from client AJAX POST that contains the export id"""
    export_id = request.data.get('export_id')
    api_call = request.data.get('rest_api')
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        export = c123_inventory.objects.filter(name=export_id, type = Inventory.EXPORT.value).first()
    else:
        export = c123_inventory.objects.filter(pk=export_id, type = Inventory.EXPORT.value).first()

    if not export:
        raise APIException404(f"export id {export_id} not found")
    
    path = export.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")
    node = export.node
    primary = c123_node.objects.filter(cluster = node.cluster, type = DBType.POSTGRES.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException404("primary not found")
    primary.status = Status.RESTORING.value
    primary.save()
    # serialize objects for the tasks
    backup_host = c123_HostSerializer(app_host).data
    export = object_to_json(export)
    node_data = object_to_json(primary)
    host_data = c123_HostSerializer(primary.host).data
    created_by = request.user
    tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_host': backup_host, 'export': export, 'created_by': created_by.id})
    task = Task.objects.create(task='pg_dumpall_restore_task',
                            displayment=f'restore pg cluster {primary.name}',
                            target_type=DBType.POSTGRES.value,
                            target_id=primary.id,
                            org_id=primary.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def PostgresDetails(request, id):
    node = c123_node.objects.filter(type = DBType.POSTGRES.value, pk = id).first()
    if not node:
        raise APIException404("node id {} not found".format(id))
    response = {'node': c123_NodeSerializer(node).data}
    auto_backups = c123_inventory.objects.filter(node=node, type = Inventory.BACKUP.value, is_sched=True)
    if auto_backups.count() > 0:
        if auto_backups.last().status:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
        else:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}
                
    else:
        response["auto_backup"]  = {
                'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}
    if node.cluster_role == ClusterRoles.PRIMARY.value:
        primary_backups = c123_inventory_serializer(c123_inventory.objects.filter(node_id = node.id, type = Inventory.BACKUP.value), many = True)
        response["backups"] = primary_backups.data
        primary_exports = c123_inventory_serializer(c123_inventory.objects.filter(node_id = node.id, type = Inventory.EXPORT.value), many = True)
        response["exports"] = primary_exports.data
        standbys = c123_NodeSerializer(c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = node.cluster, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False), many = True)
        response["nodes"] = standbys.data
        for i, standby in enumerate(response["nodes"]):
            backups = c123_inventory_serializer(c123_inventory.objects.filter(node_id = standby["id"], type = Inventory.BACKUP.value), many = True)
            response["nodes"][i]["backups"] = backups.data
            exports = c123_inventory_serializer(c123_inventory.objects.filter(node_id = standby["id"], type = Inventory.EXPORT.value), many = True)
            response["nodes"][i]["exports"] = exports.data
    else:
        standbys = c123_NodeSerializer(c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = node.cluster,  cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).exclude(pk = node.pk), many = True)
        response["nodes"] = standbys.data
    
    response["databases"] = c123_DatabaseSerializer(c123_database.objects.filter(node = node, ind_deleted = False), many = True).data
    cs = ClusterSizing.objects.filter(db_type = DBType.POSTGRES.value).first()
    size_scheme = {
            Node_Size_Allowed.SMALL.name: {
                'cpu': cs.small_cpu,
                'disk': cs.small_disk,
                'memory': cs.small_memory,
                'backup': cs.small_backup_keep_days,
                'export': cs.small_export_keep_days,
            },
            Node_Size_Allowed.MEDIUM.name: {
                'cpu': cs.medium_cpu,
                'disk': cs.medium_disk,
                'memory': cs.medium_memory,
                'backup': cs.medium_backup_keep_days,
                'export': cs.medium_export_keep_days,
            },
            Node_Size_Allowed.LARGE.name: {
                'cpu': cs.large_cpu,
                'disk': cs.large_disk,
                'memory': cs.large_memory,
                'backup': cs.large_backup_keep_days,
                'export': cs.large_export_keep_days,
            },
            Node_Size_Allowed.VERY_LARGE.name: {
                'cpu': cs.very_large_cpu,
                'disk': cs.very_large_disk,
                'memory': cs.very_large_memory,
                'backup': cs.very_large_backup_keep_days,
                'export': cs.very_large_export_keep_days,
            }
        }
    response["avail_schemes"] = size_scheme
    
    conn = get_postgres_connection(node)
    if isinstance(conn, tuple):
        error = str(conn[1])
        response['ind_error'] = True
        response["detail"] = error
        response['node'] = c123_NodeSerializer(c123_node.objects.filter(pk = id,  type = DBType.POSTGRES.value).first()).data
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        return JsonResponse(response, status = 401)
    else:
        response["ind_error"] = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = node.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists() or c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).count() < 2
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)

    try:
        cursor = conn.cursor()
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        node.severity = Severity.NOT_OK.value
        node.save()
        response['ind_error'] = True
        return JsonResponse(response, status=401)
    
    node.severity = Severity.OK.value
    node.ind_started = True
    node.save()
    try:
        cursor.execute("SHOW SERVER_VERSION")
        response["version"] = float(cursor.fetchone()[0].strip().split()[0])
        node.version = int(response["version"])
        node.save()
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    try:
        cursor.execute("SHOW log_statement")
        response["logging"] = cursor.fetchone()[0]
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    try:
        cursor.execute("SHOW default_transaction_read_only")
        response["read_only"] = cursor.fetchone()[0]
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    try:
        cursor.execute("SELECT oid FROM pg_database WHERE datname = (SELECT current_database())")
        response["db_id"] = cursor.fetchone()[0]
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    host = node.host
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            response["detail"] = client[1].strerror
        else:
            response["detail"] = str(client[1])
        
        return JsonResponse(response, status=401)

    try: 
        
        command = f"sudo du -bs  '{node.dir_123Cluster}' | awk '{{print $1}}'"
        jsn = json.loads(remote_command(client, command, True), strict=False)
        if(jsn['stderr'] != ''):
            print(jsn['stderr'])
            response["detail"] = jsn['stderr']
            return JsonResponse(response, status=404)
        
        node.size = round(float(jsn['stdout']) / 1024, 2)
        node.save()
        response["display_size"] = format_size(node.size)
        command = "sudo ps -C postgres -o rss= | awk '{sum+=$1} END {print sum}'"
        
        jsn = json.loads(remote_command(client, command, True), strict=False)
        if(jsn['stderr'] != ''):
            print(jsn['stderr'])
            response["detail"] = jsn['stderr']
            return JsonResponse(response, status=404)
        response["display_ram"] = format_size(jsn['stdout'])
        
        command = r"""total_usage=$(top -b -n 1 | grep 'postgres' | awk '{sum += $9} END {print sum}')
    core_count=$(nproc)
    awk -v usage="$total_usage" -v cores="$core_count" 'BEGIN {printf "%i\n", usage / cores}'"""
        jsn = json.loads(remote_command(client, command, True), strict=False)
        if(jsn['stderr'] != ''):
            print(jsn['stderr'])
            response["detail"] = jsn['stderr']
            return JsonResponse(response, status=404)
        response['cpu_percent'] = jsn['stdout']
    
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)

    return JsonResponse(response, status = 200)

@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def PgDbDetails(request, id):
    database = c123_database.objects.filter(pk = id).first()
    if not database:
        raise APIException404("node id {} not found".format(id))
    response = {'database': c123_DatabaseSerializer(database).data}

    auto_backups = c123_inventory.objects.filter(database=database, type = Inventory.EXPORT.value, is_sched=True)
    if auto_backups.count() > 0:
        if auto_backups.last().status:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
        else:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}
                
    else:
        response["auto_backup"]  = {
                'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}

    exports = c123_inventory_serializer(c123_inventory.objects.filter(db_id = database.id, type = Inventory.EXPORT.value), many = True)
    response["exports"] = exports.data
    cs = ClusterSizing.objects.filter(db_type = DBType.POSTGRES.value).first()
    size_scheme = {
            Node_Size_Allowed.SMALL.name: {
                'cpu': cs.small_cpu,
                'disk': cs.small_disk,
                'memory': cs.small_memory,
                'backup': cs.small_backup_keep_days,
                'export': cs.small_export_keep_days,
            },
            Node_Size_Allowed.MEDIUM.name: {
                'cpu': cs.medium_cpu,
                'disk': cs.medium_disk,
                'memory': cs.medium_memory,
                'backup': cs.medium_backup_keep_days,
                'export': cs.medium_export_keep_days,
            },
            Node_Size_Allowed.LARGE.name: {
                'cpu': cs.large_cpu,
                'disk': cs.large_disk,
                'memory': cs.large_memory,
                'backup': cs.large_backup_keep_days,
                'export': cs.large_export_keep_days,
            },
            Node_Size_Allowed.VERY_LARGE.name: {
                'cpu': cs.very_large_cpu,
                'disk': cs.very_large_disk,
                'memory': cs.very_large_memory,
                'backup': cs.very_large_backup_keep_days,
                'export': cs.very_large_export_keep_days,
            }
        }
    response["avail_schemes"] = size_scheme
    node = database.node
    conn = get_postgres_connection(node)
    if isinstance(conn, tuple):
        error = str(conn[1])
        response['ind_error'] = True
        response["detail"] = error
        response['node'] = c123_NodeSerializer(c123_node.objects.filter(pk = id,  type = DBType.POSTGRES.value).first()).data
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        return JsonResponse(response, status = 401)
    else:
        response["ind_error"] = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = node.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists() or c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).count() < 2
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)

    try:
        cursor = conn.cursor()
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        node.severity = Severity.NOT_OK.value
        node.save()
        response['ind_error'] = True
        return JsonResponse(response, status=401)
    
    node.severity = Severity.OK.value
    node.ind_started = True
    node.save()
    try:
        cursor.execute("SHOW SERVER_VERSION")
        response["version"] = float(cursor.fetchone()[0].strip().split()[0])
        node.version = int(response["version"])
        node.save()
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    try:
        cursor.execute("SHOW log_statement")
        response["logging"] = cursor.fetchone()[0]
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    try:
        cursor.execute("SHOW default_transaction_read_only")
        response["read_only"] = cursor.fetchone()[0]
        cursor.execute("SELECT pg_database_size(%s)", (database.name,))
        res = cursor.fetchone()
        database.size = res if res and res[0] else 0.0
        database.save()
        response['display_size'] = format_size(database.size)


    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)
    
    response["db_id"] = database.name

    
    host = node.host
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            response["detail"] = client[1].strerror
        else:
            response["detail"] = str(client[1])
        
        return JsonResponse(response, status=401)

    try: 

        response["display_size"] = format_size(database.size)
        command = "sudo ps -C postgres -o rss= | awk '{sum+=$1} END {print sum}'"
        
        jsn = json.loads(remote_command(client, command, True), strict=False)
        if(jsn['stderr'] != ''):
            print(jsn['stderr'])
            response["detail"] = jsn['stderr']
            return JsonResponse(response, status=404)
        response["display_ram"] = format_size(jsn['stdout'])
        
        command = r"""total_usage=$(top -b -n 1 | grep 'postgres' | awk '{sum += $9} END {print sum}')
    core_count=$(nproc)
    awk -v usage="$total_usage" -v cores="$core_count" 'BEGIN {printf "%i\n", usage / cores}'"""
        jsn = json.loads(remote_command(client, command, True), strict=False)
        if(jsn['stderr'] != ''):
            print(jsn['stderr'])
            response["detail"] = jsn['stderr']
            return JsonResponse(response, status=404)
        response['cpu_percent'] = jsn['stdout']
    
    except Exception as e:
        error = str(e)
        print("PostgresDetails: ", error)
        response["detail"] = error
        return JsonResponse(response, status=403)

    return JsonResponse(response, status = 200)

@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def HAProxyDetails(request, id):
    response = {}
    node = c123_node.objects.filter(type = DBType.HAPROXY.value, pk = id).first()
    if not node:
        raise APIException404("node id {} not found".format(id))
    response['node'] = c123_NodeSerializer(node).data
    primary = c123_node.objects.filter(cluster = node.cluster, cluster_role = ClusterRoles.PRIMARY.value).first()
    
    response['primary_id'] = primary.pk
    
    client = create_ssh_connection(node.host.name, node.host.username, node.host.password, node.host.private_key, node.host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            error = client[1].strerror
        else:
            error = str(client[1])
        response["detail"] = error
        return JsonResponse(response, status = 401)
    
    command = f'sudo journalctl -u haproxy -n 50 -o json | tac'
    jsn = json.loads(remote_command(client, command, True), strict=False)
    if(jsn['stderr'] != ''):
        error = jsn["stderr"]
        print(error)
        response["detail"] = error
        return JsonResponse(response, status=404)
   
    last_logs = [json.loads(line, strict = False) for line in jsn['stdout'].split('\n') if line != '']
    response['history'] = last_logs
    response["ind_error"] = c123_node.objects.filter(cluster = node.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists() or c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).count() < 2
    command = "systemctl is-active haproxy"
    jsn = json.loads(remote_command(client, command, True), strict=False)
    if(jsn['stderr'] != ''):
        error = jsn["stderr"]
        print(error)
        response["detail"] = error
        response["ind_error"] = True
        return JsonResponse(response, status=404)
    
    #haproxy stopped
    if 'inactive' in jsn['stdout']:
        node.status = Status.OK.value
        node.severity = Severity.NOT_OK.value
        node.ind_started = False
        node.save()
    #haproxy does not installed
    elif 'unknown' in jsn['stdout']:
        node.status = Status.OK.value
        node.severity = Severity.NOT_OK.value
        node.ind_started = False
        node.ind_deleted = True
    command = "sudo haproxy -c -f '/etc/haproxy/haproxy.cfg'"
    jsn = json.loads(remote_command(client, command, True), strict=False)
    if(jsn['stderr'] != ''):
        error = jsn["stderr"]
        print(error)
        response["detail"] = error
        response["ind_error"] = True
        return JsonResponse(response, status=404)
    
    if jsn['stdout'] == '' or 'is valid' not in jsn['stdout'].lower():
        response['ind_error'] = True
        response['ind_valid'] = False
    else:
        response['ind_valid'] = True
    return JsonResponse(response, status = 200)

@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def GetPgSettings(request, id):
    node = c123_node.objects.filter(type = DBType.POSTGRES.value, pk = id,  ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(id))
    
    conn = get_postgres_connection(node)
    if isinstance(conn, tuple):
        if hasattr(conn[1], 'strerror'):
            raise APIException401(conn[1].strerror)
        raise APIException401(conn[1])
    
    try:
        with conn:
            with conn.cursor(cursor_factory=RealDictCursor) as cur:
                cur.execute("SELECT name, setting, source, context, 'default' as color FROM pg_settings")
                settings = cur.fetchall()
                return JsonResponse(settings, status = 200, safe = False)

    except Exception as e:
        raise APIException401(e.strerror if hasattr(e, 'strerror') else str(e))
     

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def EditPgSettings(request):
   
    try:
        node_id = request.data.get('node_id')
        parameters = request.data.get('parameters')
        created_by = request.user
        node = c123_node.objects.filter(type = DBType.POSTGRES.value, pk = node_id,  ind_deleted = False).first()
        if not node:
            raise APIException410("node id {} not available".format(node_id))
        cluster_nodes = c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).exclude(type = DBType.HAPROXY.value)
        cluster_data = [{"host": object_to_json(n.host), "node": object_to_json(n)} for n in cluster_nodes]

        tmp_data = json.dumps({'cluster_data': cluster_data, 'parameters': parameters, 'created_by': created_by.id})
        task = Task.objects.create(task='pg_parameters_apply_task',
                                displayment=f'applying parameters to cluster of {node.name}',
                                target_type=DBType.POSTGRES.value,
                                target_id=node.pk,
                                org_id=node.org_id,
                                schedule=Schedule.NOW.value,
                                status=TaskState.PENDING.value,
                                parameters=tmp_data,
                                created_by=created_by)
        return append_status_cmd(request, task)

    except Exception as e:
        print(traceback.format_exc())
        raise APIException401(e.strerror if hasattr(e, 'strerror') else str(e))
   

@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def PgPrimaryStatus(request, id):
    primary = c123_node.objects.filter(type = DBType.POSTGRES.value, pk = id, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not primary:
        raise APIException410("primary id {} not available".format(id))
    
    client = create_ssh_connection(primary.host.name, primary.host.username, primary.host.password, primary.host.private_key, primary.host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    primary = find_real_pg_primary(primary)
    postgres_bin = find_postgres_bin(client, primary.version)
    repmgr_pwd = primary.ex_params.get('repmgr_pwd')
    client = create_ssh_connection(primary.host.name, primary.host.username, primary.host.password, primary.host.private_key, primary.host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    jsn = json.loads(remote_command(client, f"sudo -iu postgres PGPASSWORD='{repmgr_pwd}' {postgres_bin}/repmgr -f '/etc/repmgr/{primary.version}/repmgr.conf' service status", True))
    if(jsn['stderr'] != '' and not "warning" in str(jsn['stderr']).lower()):
        raise APIException500(str(jsn['stderr']))
    
    #status table
    table = [i for i in str(jsn['stdout']).split('\n') if '|' in i]
    #discard headers
    del table[0]
    
    i = 0
    status = []
    for row in table:
        cells = row.split('|')
        status.append({
        "id": cells[0].strip(),
        "name": cells[1].strip(),
        "role": cells[2].strip(),
        "status": cells[3].strip(),
        "upstream": cells[4].strip(),
        "repmgrd": cells[5].strip(),
        "pid": cells[6].strip(),
        "paused": cells[7].strip(),
        "upstream_last_seen": cells[8].strip()
        })
        i = i + 1
     
    return JsonResponse(status, status = 200, safe = False)
