from django.http import JsonResponse
import json, re, pytz
from django.utils import timezone as django_timezone
from datetime import datetime
from rest_framework import generics, status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from clusterCore.api.serializer import c123_NodeSerializer, c123_ClusterSerializer, object_to_json, c123_inventory_serializer, c123_HostSerializer, format_size, get_create_node_response
from clusterCore.tasks import allow_or_403, check_space, checkPathExists, check_redis_role, find_redis_bin, connect_to_redis, drop_redis_node, \
create_ssh_connection, remote_command, get_cron_schedule, append_status_cmd, format_password, get_backup_description, get_avail_change_date
from clusterCore.scheduler import schedule_backup, MISFIRE_GRACE_TIME_BACKUP
from clusterCore.exceptions import *
from clusterCore.models import c123_node, c123_host, c123_cluster, Task, ClusterSizing, c123_inventory
from clusterCore.enums import DBType, ClusterRoles, RestoreOptions, Severity, TaskState, Status, Schedule, Inventory, Node_Size_Allowed
from .serializer import *
from types import SimpleNamespace
from dateutil.parser import parse
class RedisNodeAddView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_AddRedisNodeSerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)
class CreateRedisStandaloneView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_CreateRedisStandaloneSerializer
    def create(self, request, *args, **kwargs):
        return get_create_node_response(self, request, *args, **kwargs)
class RedisClusterAddView(generics.CreateAPIView):
    permission_classes = [IsAuthenticated]
    queryset = c123_node.objects.all()
    serializer_class = c123_AddRedisClusterSerializer
    def create(self, request, *args, **kwargs):
        super().create(request, *args, **kwargs)
        return JsonResponse({"detail": "Object created successfully."}, status=status.HTTP_201_CREATED)
class RedisClusterList(generics.ListAPIView):
    permission_classes =[IsAuthenticated]
    serializer_class = c123_ClusterSerializer
    def get_queryset(self):
        user = self.request.user
        if user.is_superuser or user.groups.filter(name='Administrators').exists():
            return c123_cluster.objects.filter(type = DBType.REDIS.value, ex_params__database_pwd__isnull = False)
        return c123_cluster.objects.filter(type = DBType.REDIS.value, created_by = user, ex_params__database_pwd__isnull = False)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckRedisVersionExists(request):
    """check if a redis version package already exists"""
    host_name = request.data.get("host")
    username = request.data.get("os_username")
    password = request.data.get("os_password")
    if password:
        password = format_password(password)
    private_key = request.data.get("private_key")
    ssh_port = request.data.get("ssh_port")
    version = request.data.get("software_version")
        
    host = SimpleNamespace( **{
        "name": host_name, 
        "username": username,
        "password": password,
        "private_key": private_key,
        "ssh_port": ssh_port
    })
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        if hasattr(client[1], 'strerror'):
            raise APIException401(client[1].strerror)
        raise APIException401(client[1])
    redis_bin = find_redis_bin(client)
    jsn = json.loads(remote_command(client, f"sudo {redis_bin}/redis-server --version | awk '{{print $3}}' | cut -d '=' -f2", True))
    if jsn['stderr'] != '':
        jsn = json.loads(remote_command(client, f"sudo rpm -qa | grep 'redis-{version}'", True))
        if(jsn['stderr'] != ''):
            print(jsn['stderr'])
            if 'not found' in jsn['stderr']:
                jsn = json.loads(remote_command(client, f"sudo apt-cache policy redis | grep 'Installed:'  | xargs | cut -d'-' -f1 | cut -d':' -f3", True))
                if(jsn['stderr'] != ''):
                    print(jsn['stderr'])
                    return JsonResponse({'error': jsn['stderr']}, status=200)
                        
                elif jsn["stdout"].strip() == '':
                    return JsonResponse({'exists': False}, status=200)
                elif str(version) in jsn['stdout']:
                    return JsonResponse({'exists': True}, status=200)
        elif jsn['stdout'].strip() != '':
            return JsonResponse({'exists': True}, status=200)
        
    elif jsn['stdout'] != '' and str(version) in jsn['stdout']:
        return JsonResponse({'exists': True}, status=200)
        
        
    return JsonResponse({'exists': False}, status=200)
@api_view(['GET'])
@permission_classes((IsAuthenticated, ))
def RedisDetails(request, id):
    node = c123_node.objects.filter(pk = id, type = DBType.REDIS.value).first()
    if not node:
        raise APIException404("node id {} not found".format(id))
    response = {'node': c123_NodeSerializer(node).data}
    auto_backups = c123_inventory.objects.filter(node=node, type = Inventory.BACKUP.value, is_sched=True)
    if auto_backups.count() > 0:
        if auto_backups.last().status:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "green-star", "detail": "Automatic backup configured"}
        else:
            response["auto_backup"] = {
                    'status': "Automatic backup", 'class': "red-green-star", "detail": "Automatic backup is configured, but the last run reported errors"}
                
    else:
        response["auto_backup"]  = {
                'status': "No automatic backup", 'class': "red-star", "detail" : "Automatic backup is not configured"}
        
    backups = c123_inventory_serializer(c123_inventory.objects.filter(node_id = node.id, type = Inventory.BACKUP.value), many = True)
    response["backups"] = backups.data
    cs = ClusterSizing.objects.filter(db_type = DBType.REDIS.value).first()
    size_scheme = {
            Node_Size_Allowed.SMALL.name: {
                'cpu': cs.small_cpu,
                'disk': cs.small_disk,
                'memory': cs.small_memory,
                'backup': cs.small_backup_keep_days,
                'export': cs.small_export_keep_days,
            },
            Node_Size_Allowed.MEDIUM.name: {
                'cpu': cs.medium_cpu,
                'disk': cs.medium_disk,
                'memory': cs.medium_memory,
                'backup': cs.medium_backup_keep_days,
                'export': cs.medium_export_keep_days,
            },
            Node_Size_Allowed.LARGE.name: {
                'cpu': cs.large_cpu,
                'disk': cs.large_disk,
                'memory': cs.large_memory,
                'backup': cs.large_backup_keep_days,
                'export': cs.large_export_keep_days,
            },
            Node_Size_Allowed.VERY_LARGE.name: {
                'cpu': cs.very_large_cpu,
                'disk': cs.very_large_disk,
                'memory': cs.very_large_memory,
                'backup': cs.very_large_backup_keep_days,
                'export': cs.very_large_export_keep_days,
            }
        }
    response["avail_schemes"] = size_scheme
    if node.cluster_role == ClusterRoles.PRIMARY.value:
        slaves = c123_NodeSerializer(c123_node.objects.filter(type = DBType.REDIS.value, cluster = node.cluster, cluster_role = ClusterRoles.STANDBY.value, severity = Severity.OK.value, ind_started = True, ind_deleted = False), many = True)
        response["slaves"] = slaves.data
    if node.cluster_role == ClusterRoles.SENTINEL.value:
        masters = c123_NodeSerializer(c123_node.objects.filter(type = DBType.REDIS.value, cluster = node.cluster, cluster_role = ClusterRoles.PRIMARY.value, severity = Severity.OK.value, ind_started = True, ind_deleted = False), many = True)
        response["masters"] = masters.data
        response["sentinels_count"] = c123_node.objects.filter(type = DBType.REDIS.value, cluster = node.cluster, cluster_role = ClusterRoles.SENTINEL.value, ind_deleted = False).count()
    host = node.host
        
    client = connect_to_redis(node)
    if isinstance(client, tuple):
        error = client[1].strerror if hasattr(client, 'strerror') else str(client[1])
        response["ind_error"] = True
        response["detail"] = error
        response['node'] = c123_NodeSerializer(c123_node.objects.filter(pk = id,  type = DBType.REDIS.value).first()).data
        response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
        return JsonResponse(response, status=401)
    else:
         node.ind_started = True
         node.severity = Severity.OK.value if Status.PENDING.value not in node.status else Severity.WARNING.value 
         node.status = Status.OK.value
         node.save()
         response["ind_error"] = c123_node.objects.filter( type = DBType.REDIS.value, cluster = node.cluster, ind_deleted = False, severity__in=[Severity.WARNING.value, Severity.NOT_OK.value]).exists()  or c123_node.objects.filter(cluster = node.cluster, ind_deleted = False).count() < 3
         response['node']['ex_params']['avail_change'] =  get_avail_change_date(node)
    try:
        info = client.info()
    except Exception as ex:
        error = ex.strerror if hasattr(ex, 'strerror') else str(ex)
        response["detail"] = error
        return JsonResponse(response, status=500)
    
    response['uptime'] = f"{info['uptime_in_seconds']} seconds"
    version = info["redis_version"].split(".")
    response["version"] = ".".join(version[:2])
    node.version = response["version"]
    node.save()
    response["is_master"] = info["redis_mode"]
    if 'cluster' in info["redis_mode"]:
            response["is_master"] = info["role"]
            
            try:
                response["db_id"] = client.cluster("myid").decode("ascii")
            except Exception as ex:
                error = ex.strerror if hasattr(ex, 'strerror') else str(ex)
                if "cluster support disabled" in error.lower():
                    response['db_id'] = "no id"
                else:
                    response["detail"] = error
                    return JsonResponse(response, status=500)
            
            response["is_master"] = info["role"]   
    
    elif 'sentinel' in info["redis_mode"]:   
        try:
            response["db_id"] = client.execute_command("sentinel myid").decode("ascii")
        except Exception as ex:
            error = ex.strerror if hasattr(ex, 'strerror') else str(ex)
            response["detail"] = error
            return JsonResponse(response, status=500)
    else:
        response['db_id'] = "no id"
    node.ex_params["db_id"] = response["db_id"]
    if response["is_master"] == "slave":
         node.ex_params['is_master'] = False
         node.cluster_role = ClusterRoles.STANDBY.value
    node.save()
    host = node.host
    client = create_ssh_connection(host.name, host.username, host.password, host.private_key, host.ssh_port)
    if isinstance(client, tuple):
        error = client[1].strerror if hasattr(client[1], "strerror") else str(client[1])
        response["detail"] = error
        return JsonResponse(response, status=401)
    
    command = f"sudo du -bs  '{node.dir_123Cluster}/data' | awk '{{print $1}}'"
    jsn = json.loads(remote_command(client, command, True), strict=False)
    if(jsn['stderr'] != ''):
        print(jsn['stderr'])
        response["detail"] = jsn['stderr']
        return JsonResponse(response, status=404)
    node.size = round(float(jsn['stdout']) / 1024, 2)
    node.save()
    response["display_size"] = format_size(node.size)
    response['display_ram'] = format_size(float(info['used_memory_rss']) / 1024.0)
    response['cpu_percent'] = round(float(info['used_cpu_sys']) + float(info['used_cpu_user']) + float(info['used_cpu_sys_children']) + float(info['used_cpu_user_children']))
    

    command = f"sudo tail '{node.dir_123Cluster}/log/redis.log' -n 51"
    jsn = json.loads(remote_command(client, command, True), strict=False)
    if(jsn['stderr'] != ''):
        error = jsn['stderr']
        print(error)
        response["detail"] = error
        return JsonResponse(response, status=404)
    log_lines = jsn["stdout"].split('\n')
    logs_list = []
    level = ''
    for line in log_lines:
        log_dict = {}
        if '*' in line:
            log_dict['level'] = "notice"
            level = '*'
        elif '#' in line:
             log_dict['level'] = "warning"
             level = '#'
        else:
            #exit on verbose (more than 1 line) logs
            break
        log_parts = line.split(level)
        log_dict['message'] = log_parts[1]
        if ":S" in log_parts[0]:
            log_dict['role'] = "slave"
            role = ":S"
        elif ":M" in log_parts[0]:
            log_dict['role'] = "master"
            role = ":M"
        else:
            log_dict['role'] = "sentinel" if ":X" in log_parts[0] else "child"
            role = ":C" if ":C" in log_parts[0] else ":X"
        
        monitor_data = log_parts[0].split(role)
        log_dict['pid'] = monitor_data[0]
        log_dict['when'] = monitor_data[1] if len(monitor_data) > 1 else ''
        logs_list.append(log_dict)
    response['history'] = logs_list
    return JsonResponse(response, status = 200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RemoveRedisCluster(request):
    cluster_id = request.data.get("cluster_id")
    api_call = request.data.get("rest_api")
    if api_call:
        cluster = c123_cluster.objects.filter(name=cluster_id, type = DBType.REDIS.value).first()
    else:
        cluster = c123_cluster.objects.filter(pk=cluster_id, type = DBType.REDIS.value).first()
    if not cluster:
        raise APIException404(f"cluster ID {cluster_id} could not be found")
    if cluster.ex_params.get("standalone") is not False:
        raise APIException409(f"standalone redis cluster {cluster.name} could not be unregistered")
    cluster_nodes = c123_node.objects.filter(type = DBType.REDIS.value, cluster = cluster)
    has_nodes_in_use = cluster_nodes.filter(ind_deleted = False).exists()
    if has_nodes_in_use:
        raise APIException409(f"non-standalone cluster {cluster.name} has nodes in use.Could not be unregistered")
    for node in cluster_nodes:
        node.cluster = None
        node.save()
    
    cluster_name = cluster.name
    cluster.delete()
    
    return JsonResponse({"notification": f"cluster {cluster_name} has been unregistered"}, status=200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckSentinelMasterName(request):
    node_id = request.data.get('node_id')
    name = request.data.get('name')
    sentinel = c123_node.objects.filter(pk=node_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.SENTINEL.value, ind_deleted = False).first()
    if not sentinel:
        raise APIException410(f"sentinel id {node_id} not available")
    if re.match(r'^[a-z][a-z0-9_-]{0,254}$', name):
        client = connect_to_redis(sentinel)
        if isinstance(client, tuple):
            if hasattr(client[1], 'strerror'):
                raise APIException401(client[1].strerror)
            raise APIException401(client[1])
        try:
            client.execute_command("sentinel master", name)
        except Exception as e:
            error = str(e)
            if 'no such master' in error.lower():
                return JsonResponse({'response': "name available"}, status=200)
            else:
                raise APIException500(error)
            
        return JsonResponse({'response': "already exists"}, status=200)
    
    else:
        return JsonResponse({'error': "invalid name"}, status=406)
    
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def SentinelMonitorMaster(request):
    node_id = request.data.get('node_id')
    sentinel_id = request.data.get('sentinel_id')
    alias = request.data.get('alias')
    parallel_syncs = request.data.get('parallel_syncs')
    quorum = request.data.get('quorum')
    down_ms = request.data.get('down_ms')
    failover_timeout = request.data.get('failover_ms')
    update = request.data.get('update')
    current_user = request.user
    api_call = request.data.get('rest_api')
    if api_call:
        master = c123_node.objects.filter(pk=node_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.PRIMARY.value,ind_deleted = False).first()
    else:
        master = c123_node.objects.filter(pk=node_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.PRIMARY.value,ind_deleted = False).first()
    if not master:
        raise APIException410(f"master id {node_id} not available")
    allow_or_403(master)
    if api_call:
        sentinel = c123_node.objects.filter(name=sentinel_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.SENTINEL.value,  ind_deleted = False).first()
    else:
        sentinel = c123_node.objects.filter(pk=sentinel_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.SENTINEL.value,  ind_deleted = False).first()
    if not sentinel:
        raise APIException410(f"sentinel id {sentinel_id} not available")
    
    monitored_nodes = sentinel.ex_params.get('monitored_nodes')
    if update and monitored_nodes:
        alias = next((k for k, v in monitored_nodes.items() if master.pk in v), alias)
    
    affected_ids = [i.pk for i in c123_node.objects.filter(ind_deleted = False, type = DBType.REDIS.value, cluster_role = ClusterRoles.STANDBY.value, ex_params__slave_of = master.pk)]
    affected_ids.append(master.pk)
    
    node_data = {'id':node_id, 'host': master.host.name, 'port': master.port, 'database_user': master.database_user, 'database_pwd': master.database_pwd, 'alias': alias}
    sentinel_data = {'id': sentinel_id, 'host': sentinel.host.name, 'port': sentinel.port, 'database_user': sentinel.database_user, 'database_pwd': sentinel.database_pwd}
    tmp_data = json.dumps({'node_data': node_data, 'sentinel_data': sentinel_data, 'affected_ids': affected_ids, 'parallel_syncs': parallel_syncs, 'quorum': quorum, 'down_ms': down_ms, 'failover_timeout': failover_timeout, 'created_by': current_user.id})
    task = Task.objects.create(task='sentinel_monitor_master_task',
                                    displayment='adding {} to monitoring'.format(master.name),
                                    target_type=DBType.REDIS.value,
                                    target_id=master.id,
                                    org_id=master.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def SentinelRemoveMonitor(request):
    node_id = request.data.get('node_id')
    sentinel_id = request.data.get('sentinel_id')
    current_user = request.user
    api_call = request.data.get('api_call')
    if api_call:
        master = c123_node.objects.filter(name=node_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.PRIMARY.value,  ind_deleted = False).first()
    else:
        master = c123_node.objects.filter(pk=node_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.PRIMARY.value,  ind_deleted = False).first()

    if not master:
        raise APIException410(f"master id {node_id} not available")
    allow_or_403(master)
    if api_call:
        sentinel = c123_node.objects.filter(name=sentinel_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.SENTINEL.value,  ind_deleted = False).first()
    else:
        sentinel = c123_node.objects.filter(pk=sentinel_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.SENTINEL.value,  ind_deleted = False).first()
    if not sentinel:
        raise APIException410(f"sentinel id {sentinel_id} not available")
    
    monitored_nodes = sentinel.ex_params.get('monitored_nodes')
    if monitored_nodes:
        alias = next((k for k, v in monitored_nodes.items() if master.pk in v), None)
        if not alias:
            raise APIException404(f"could not find {master.name} in monitoring of {sentinel.name}")
    
    node_data = {'id':node_id, 'host': master.host.name, 'port': master.port, 'database_user': master.database_user, 'database_pwd': master.database_pwd, 'alias': alias}
    sentinel_data = {'id': sentinel_id, 'host': sentinel.host.name, 'port': sentinel.port, 'database_user': sentinel.database_user, 'database_pwd': sentinel.database_pwd}
    tmp_data = json.dumps({'node_data': node_data, 'sentinel_data': sentinel_data, 'created_by': current_user.id})
    task = Task.objects.create(task='sentinel_remove_monitor_task',
                                    displayment='removing {} from monitoring'.format(master.name),
                                    target_type=DBType.REDIS.value,
                                    target_id=master.id,
                                    org_id=master.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def SentinelFailover(request):
    master_id = request.data.get('node_id')
    api_call = request.data.get('rest_api')
    current_user = request.user
    if api_call:
        master = c123_node.objects.filter(pk=master_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    else:
        master = c123_node.objects.filter(pk=master_id, type = DBType.REDIS.value, cluster_role = ClusterRoles.PRIMARY.value, ind_deleted = False).first()
    if not master:
        raise APIException410(f"master id {master_id} not available")
    
    sentinel = next((i for i in c123_node.objects.filter(type = DBType.REDIS.value, cluster_role = ClusterRoles.SENTINEL.value,  ind_deleted = False) if master.ex_params.get('monitored_by') and str(i.pk) in master.ex_params['monitored_by']), None)
    if not sentinel:
        raise APIException410(f"no sentinels available for {master.name}")
    
    monitored_nodes = sentinel.ex_params.get('monitored_nodes')
    if monitored_nodes:
        alias = next((k for k, v in monitored_nodes.items() if master.pk in v), None)
        if not alias:
            raise APIException404(f"could not find {master.name} in monitoring of {sentinel.name}")
    
    master.status = Status.STEPDOWN.value
    master.save()   
    node_data = {'id':master_id, 'host': master.host.name, 'port': master.port, 'database_user': master.database_user, 'database_pwd': master.database_pwd, 'alias': alias}
    sentinel_data = {'id': sentinel.pk, 'host': sentinel.host.name, 'port': sentinel.port, 'database_user': sentinel.database_user, 'database_pwd': sentinel.database_pwd}
    other_nodes = [{'id': i.pk, 'host': i.host.name, 'port': i.port} for i in c123_node.objects.filter(ind_deleted = False, type = DBType.REDIS.value, cluster_role = ClusterRoles.STANDBY.value, ex_params__slave_of=master.pk)]
    tmp_data = json.dumps({'node_data': node_data, 'sentinel_data': sentinel_data, 'other_nodes': other_nodes, 'created_by': current_user.id})
    task = Task.objects.create(task='sentinel_failover_task',
                                    displayment='stepping down {}'.format(master.name),
                                    target_type=DBType.REDIS.value,
                                    target_id=master.id,
                                    org_id=master.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def CheckRedisClusterHealth(request):
    node_id = request.data.get('node_id')
    node = c123_node.objects.filter(pk = node_id, type = DBType.REDIS.value, ind_deleted = False).first()
    if not node:
        raise APIException410(f"node id {node_id} is not available")
    cluster = node.cluster
    redis_client = connect_to_redis(node)
    if isinstance(redis_client, tuple):
        error = redis_client[1].strerror if hasattr(redis_client, 'strerror') else str(redis_client[1])
        raise APIException401(error)
    try:
        cluster_state = redis_client.cluster('info').get('cluster_state')
        if not cluster_state:
             node.status = Status.PENDING.value
             node.severity = Severity.WARNING.value
             node.save()
             raise APIException404(f"node {node.name} has not join any cluster")
        if 'fail' in cluster_state:
            cluster.ex_params["status"] = Status.DOWN.value
            cluster.ex_params["severity"] = Severity.NOT_OK.value
            cluster.save()
            raise APIException400(f"Cluster of node {node.name} now in failed state.Make sure you have at least three master nodes, and check the logs of cluster nodes for further hints.")
    except Exception as e:
         error = e.strerror if hasattr(e, 'strerror') else str(e)
         raise APIException500(error)
    
    cluster.ex_params["status"] = Status.OK.value
    cluster.ex_params["severity"] = Severity.OK.value
    cluster.save()
    node.status = Status.OK.value
    node.severity = Severity.OK.value
    node.save()
    return JsonResponse({}, status=200)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StopRedisNode(request):
    node_id = request.data.get("node_id")
    api_call = request.data.get("rest_api")
    current_user = request.user
    if api_call:
        node = c123_node.objects.filter(name = node_id,  type = DBType.REDIS.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id,  type = DBType.REDIS.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    node.ind_started = False
    node.status = Status.STOPPING.value
    node.save()
    tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': c123_HostSerializer(node.host).data, 'created_by': current_user.id})
    task = Task.objects.create(task='stop_redis_node_task',
                                    displayment='stopping node {}'.format(node.name),
                                    target_type=DBType.REDIS.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def DropRedisNode(request):
    node_id = request.data.get("node_id")
    data_only = request.data.get("data_only")
    api_call = request.data.get("rest_api")
    final_backup = request.data.get('final_backup')
    if api_call:
        node = c123_node.objects.filter(name = node_id,type = DBType.REDIS.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id,type = DBType.REDIS.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    
    task = drop_redis_node(node, data_only, final_backup)
    
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def StartRedisNode(request):
    node_id = request.data.get("node_id")
    current_user = request.user
    api_call = request.data.get("rest_api")
    if api_call:
        node = c123_node.objects.filter(name = node_id,  type = DBType.REDIS.value, ind_deleted = False).first()
    else:
        node = c123_node.objects.filter(pk = node_id,  type = DBType.REDIS.value, ind_deleted = False).first()
    if not node:
        raise APIException410("node id {} not available".format(node_id))
    host_data = c123_HostSerializer(node.host).data
    if node.ind_started:
         node.status = Status.RESTARTING.value
         node.save()
         tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': host_data, 'created_by': current_user.id})
         task = Task.objects.create(task='restart_redis_node_task',
                                    displayment='restarting node {}'.format(node.name),
                                    target_type=DBType.REDIS.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    else:
        node.ind_started = True
        node.status = Status.STARTING.value
        node.save()
        tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': host_data, 'created_by': current_user.id})
        task = Task.objects.create(task='start_redis_node_task',
                                    displayment='starting node {}'.format(node.name),
                                    target_type=DBType.REDIS.value,
                                    target_id=node.id,
                                    org_id=node.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def BackupRedisNode(request):
    """backup for redis node"""
    schedule = request.data.get('schedule')
    node_id = request.data.get('node_id')
    compress = request.data.get("compress")
    backup_type = request.data.get("backup_type")
    backup_path = request.data.get('backup_path')
    update_schedule = request.data.get("update_schedule")
    api_call = request.data.get('rest_api')
    from django.conf import settings
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    
    if api_call:
        redis_node = c123_node.objects.filter(name=node_id, type = DBType.REDIS.value, ind_deleted = False).first()
    else:
        redis_node = c123_node.objects.filter(pk=node_id, type = DBType.REDIS.value, ind_deleted = False).first()
    if not redis_node:
        raise APIException410("node id {} not available".format(node_id))
    
    if schedule:
        client_timezone = pytz.timezone(request.data.get("client_timezone"))
        schedule_run_time_str = request.data.get('schedule_run_time')
        schedule_run_time = parse(schedule_run_time_str).astimezone(client_timezone)
        client_hours = schedule_run_time.hour
        client_minutes = schedule_run_time.minute
        client_seconds = schedule_run_time.second
        days_of_week = request.data.get("days_of_week")
        days_of_month = request.data.get("days_of_month")
        if days_of_month:
            int_days_of_month = [parse(d).astimezone(client_timezone).day for d in days_of_month]
        else:
            int_days_of_month = None
        description = get_backup_description(client_hours, client_minutes, client_seconds, days_of_week, int_days_of_month)
        localized_datetime = datetime.now(client_timezone).replace(hour=client_hours, minute=client_minutes, second=client_seconds).astimezone(pytz.timezone(settings.TIME_ZONE))
        hours = localized_datetime.hour
        minutes = localized_datetime.minute
        seconds = localized_datetime.second
        trigger = get_cron_schedule(hours, minutes, seconds, days_of_week, int_days_of_month)
    if update_schedule and schedule:
        allow_or_403(redis_node)
        backup = c123_inventory.objects.filter(node = redis_node, type = Inventory.BACKUP.value, is_sched = True).last()
        if backup:
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            backup.ex_params['description'] = description
            backup.save()
            job_id = backup.ex_params.get("job_id")
            job = settings.scheduler.get_job(job_id=job_id)
            if job:
                job.reschedule(trigger=trigger)
            else:
                settings.scheduler.add_job(schedule_backup, trigger=trigger, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP, replace_existing=True, kwargs={"task_id": backup.ex_params["task_id"]}, id=job_id)
            notification = f"{backup.node} backup updated to '{description}'"
            return JsonResponse({"notification": notification}, status=200)
        
        else:
            slaves = c123_node.objects.filter(type = DBType.REDIS.value, cluster_role = ClusterRoles.STANDBY.value, ex_params__slave_of = redis_node.pk)
            if slaves.exists():
                for slave in slaves:
                    backup = c123_inventory.objects.filter(node = slave.pk, type = Inventory.BACKUP.value, is_sched = True).last()
                    if backup:
                        backup.ex_params['schedule_run_time'] = schedule_run_time_str
                        backup.ex_params['days_of_month'] = days_of_month
                        backup.ex_params['days_of_week'] = days_of_week
                        backup.ex_params['description'] = description
                        backup.save()
                        job_id = backup.ex_params.get("job_id")
                        job = settings.scheduler.get_job(job_id=job_id)
                        if job:
                            job.reschedule(trigger=trigger)
                        else:
                            settings.scheduler.add_job(schedule_backup, trigger=trigger, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP, replace_existing=True, kwargs={"task_id": backup.ex_params["task_id"]}, id=job_id)
                        notification = f"{backup.node} backup to '{description}'"
                        return JsonResponse({"notification": notification}, status=200)
                
                return JsonResponse({"notification": "no schedule backup found"}, status = 200)            
            else:
               return JsonResponse({"notification": "no schedule backup found"}, status = 200)
    serializer = c123_inventory_serializer(data = request.data, partial = True)
    if serializer.is_valid():
        if redis_node.cluster_role == ClusterRoles.SENTINEL.value:
           raise APIException403("backup is not allowed for sentinel node")
        allow_or_403(redis_node)
        check_space(node = redis_node, path=backup_path, source_path=redis_node.dir_123Cluster)
    # create the task signature
        size_scheme, created = ClusterSizing.objects.get_or_create(db_type = DBType.REDIS.value)
        keep_days = getattr(size_scheme, f'{redis_node.sizing.lower()}_backup_keep_days', None)
        if schedule:
            existing_backup = c123_inventory.objects.filter(node = redis_node, type = Inventory.BACKUP.value, is_sched = True)
            if existing_backup.exists():
                raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for cluster.")
            
            if redis_node.cluster_role == ClusterRoles.PRIMARY.value:
                slaves = c123_node.objects.filter(type = DBType.REDIS.value, cluster_role = ClusterRoles.STANDBY.value, ex_params__slave_of = redis_node.id)
            else:
                slaves = c123_node.objects.filter(type = DBType.REDIS.value, cluster_role = ClusterRoles.STANDBY.value, ex_params__slave_of = redis_node.ex_params.get("slave_of")).exclude(pk = redis_node.pk)
            
            if slaves.exists():
                for slave in slaves:
                    existing_backup = c123_inventory.objects.filter(node = slave, type = Inventory.BACKUP.value, is_sched = True)
                    if existing_backup.exists():
                        raise APIException409("Schedule backup already exists.\nCan be only one schedule backup for cluster.")
            backup = serializer.create(serializer.validated_data)
            backup.node = redis_node
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0}
            backup.ex_params["belong_to"] = DBType.REDIS.value
            backup.ex_params["node_version"] = redis_node.version
            backup.is_sched = schedule
            backup.ex_params["backup_type"] = backup_type
            backup.ex_params["duration"] = 0
            backup.ex_params["compress"] = compress
            backup.ex_params["modified_date"] = ''
            backup.ex_params['description'] = description
            backup.ex_params['schedule_run_time'] = schedule_run_time_str
            backup.ex_params['days_of_month'] = days_of_month
            backup.ex_params['days_of_week'] = days_of_week
            backup.ex_params["start_date"] = ''
            notification =  "{} backup {}".format(backup.node, description)
            job_id = f"{notification} ({client_timezone})"
            backup.ex_params["job_id"] = job_id
            backup.ex_params["backup_path"] = f'{backup_path}/schedule/{redis_node.host.name}_{redis_node.port}'
            now = django_timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(redis_node.name, now)
            backup.save()
            # serialize objects for the tasks
            node_data = object_to_json(redis_node)
            host_data = c123_HostSerializer(redis_node.host).data
            created_by = request.user
            backup_host = c123_HostSerializer(app_host).data
            tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_host': backup_host, 'backup_data': object_to_json(backup), 'created_by': created_by.id, 'backup_type': backup_type, 'compress': compress})
            task = Task.objects.create(task='backup_redis_node_task',
                                            displayment='backup {}'.format(redis_node.name), 
                                            target_type=DBType.REDIS.value, 
                                            target_id=redis_node.id,
                                            org_id=redis_node.org_id, 
                                            schedule=Schedule.SCHEDULED.value, 
                                            status=TaskState.ON_DEMAND.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
            settings.scheduler.add_job(schedule_backup, trigger=trigger,  kwargs={"task_id": task.pk}, id=job_id, replace_existing=True, misfire_grace_time=MISFIRE_GRACE_TIME_BACKUP)
        else:
            existing_backup = Task.objects.filter(target_id = redis_node.id, task = 'backup_redis_node_task', status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
            if existing_backup:
                raise APIException409(f"Already runs backup for {redis_node.name}")
            backup = serializer.create(serializer.validated_data)
            backup.node = redis_node
            if keep_days:
                backup.keep_days = keep_days
            backup.type = Inventory.BACKUP.value
            backup.created_by = request.user
            backup.ex_params = {"size": 0}
            backup.ex_params["belong_to"] = DBType.REDIS.value
            backup.ex_params["node_version"] = redis_node.version
            backup.is_sched = schedule
            backup.ex_params["backup_type"] = backup_type
            backup.ex_params["duration"] = 0
            backup.ex_params["compress"] = compress
            backup.ex_params["modified_date"] = ''
            now = django_timezone.now().strftime('%d%m%Y%H%M%S')
            backup.name = "{}_{}".format(redis_node.name, now)
            backup.ex_params["backup_path"] = f'{backup_path}/{redis_node.host.name}_{redis_node.port}/{now}'
            backup.save()
            notification = f"backup of {redis_node.name}"
            # serialize objects for the tasks
            node_data = object_to_json(redis_node)
            host_data = c123_HostSerializer(redis_node.host).data
            backup_host = c123_HostSerializer(app_host).data
            created_by = request.user
            tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_host': backup_host, 'backup_data': object_to_json(backup), 'created_by': created_by.id, 'backup_type': backup_type, 'compress': compress})
            task = Task.objects.create(task='backup_redis_node_task',
                                            displayment='backup {}'.format(redis_node.name), 
                                            target_type=DBType.REDIS.value, 
                                            target_id=redis_node.id,
                                            org_id=redis_node.org_id,
                                            schedule=Schedule.NOW.value, 
                                            status=TaskState.PENDING.value,
                                            parameters=tmp_data,
                                            created_by=created_by)
            
            backup.ex_params['task_id'] = task.pk
            backup.save()
    else:
        print("invalid form")
        print(serializer.errors)        
        return JsonResponse({'error': serializer.errors}, status=404)
    return append_status_cmd(request, task, notification)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RestoreBackupRedisNode(request):
    """restore a redis node from backup
    the call is made from client AJAX POST that contains the backup id"""
    
    bu_id = request.data.get('bu_id')
    # restore_option = 'this'/'latest'/'custom'
    restore_option = request.data.get('restore_option')
    restore_time = request.data.get('restore_time')
    autostart = request.data.get('autostart')
    api_call = request.data.get('rest_api')
    app_host = c123_host.objects.filter(ex_params__myself = True).first()
    if not app_host:
        raise APIException404("could not determine the app host")
    if api_call:
        backup = c123_inventory.objects.filter(name=bu_id, type = Inventory.BACKUP.value).first()
    else:
        backup = c123_inventory.objects.filter(pk=bu_id, type = Inventory.BACKUP.value).first()

    if not backup:
        raise APIException404("backup id {} not found".format(bu_id))
        
    path = backup.ex_params.get('backup_path')
    path_exists = checkPathExists(app_host, path)
    if not path_exists:
        raise APIException404(f"{path}: no such file or directory")
    node = backup.node
    if node.cluster_role == ClusterRoles.SENTINEL.value:
        raise APIException403("backup is not allowed for sentinel node")
    
    forbid_restore = Task.objects.filter(target_id = node.pk, task__in = ['restore_redis_node_task', 'backup_redis_node_task'], status__in = [TaskState.PENDING.value, TaskState.RUNNING.value])
    if forbid_restore:
        raise APIException409(f"Already runs backup/restore for {node.name}")
    
    allow_or_403(node)

    node.status = Status.RESTORING.value
    node.save()
    if RestoreOptions.THIS.value in restore_option:
        date = backup.created_date.strftime("%Y-%m-%d %H:%M:%S")
    elif RestoreOptions.CUSTOM.value in restore_option and restore_time:
        # example date = '2021-12-30 09:08:33'
        # date = datetime.strptime(restore_time, "%Y-%m-%dT%H:%M:%S.%f%z").astimezone()
        # date = date.strftime("%Y-%m-%d %H:%M:%S")
        date = restore_time
    else:
        date = ''

    # serialize objects for the tasks
    backup = object_to_json(backup)
    node_data = object_to_json(node)
    host_data = c123_HostSerializer(node.host).data
    backup_host = c123_HostSerializer(app_host).data
    created_by = request.user
    if node.cluster_role == ClusterRoles.PRIMARY.value and c123_node.objects.filter(type = DBType.REDIS.value, cluster = node.cluster, ex_params__slave_of=node.pk, ind_deleted = False, status = Severity.OK.value, cluster_role = ClusterRoles.STANDBY.value).exists():
         autostart = True
            
    tmp_data = json.dumps({'node_data': node_data, 'host_data': host_data, 'backup_host': backup_host, 'backup': backup, 'autostart': autostart, 'restore_option': restore_option, 'date': date, 'created_by': created_by.id})
    task = Task.objects.create(task='restore_redis_node_task',
                            displayment='restore redis node {}'.format(node.name),
                            target_type=DBType.REDIS.value,
                            target_id=node.id,
                            org_id=node.org_id,
                            schedule=Schedule.NOW.value,
                            status=TaskState.PENDING.value,
                            parameters=tmp_data,
                            created_by=created_by)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def SwitchRedisNode(request):
    slave_id = request.data.get("slave_id")
    current_user = request.user
    id_list = []
    created_by = request.user
    api_call = request.data.get("rest_api")
    if api_call:
        slave = c123_node.objects.filter(type = DBType.REDIS.value, name = slave_id, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    else:
        slave = c123_node.objects.filter(type = DBType.REDIS.value, pk = slave_id, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).first()
    if not slave:
        raise APIException410("replica id {} is not available".format(slave_id))
    allow_or_403(slave)
    role = check_redis_role(slave)
    if 'slave' not in role:
         raise APIException406(f"node {slave.name} is not a replica")
    
    in_switchover = Task.objects.filter(task = 'switchover_mariadb_node_task', target_id = slave.pk, status__in = [TaskState.PENDING.value, TaskState.RUNNING.value]).first()
    if in_switchover:
        raise APIException406(f"replica {slave.name} are already in switchover (task id {in_switchover.pk})")
    
    master_id = slave.ex_params.get("slave_of")
    
    #master
    master = c123_node.objects.filter(type = DBType.REDIS.value, pk = master_id, cluster_role = ClusterRoles.PRIMARY.value , ind_deleted = False).first()
    if not master:
        raise APIException404("master id {} not found".format(master_id))
    other_slaves = c123_node.objects.filter(type = DBType.REDIS.value, cluster_role = ClusterRoles.STANDBY.value, ex_params__slave_of = master.pk, ind_deleted = False).exclude(pk = slave.pk)
    id_list = [i.pk for i in other_slaves]
    id_list.append(master.pk)
    
    host_data = c123_HostSerializer(slave.host).data
    node_data = object_to_json(slave)
    master_data = object_to_json(master)
    tmp_data = json.dumps({'master_data': master_data, 'node_data': node_data, 'host_data': host_data, 'created_by': created_by.id, 'id_list': id_list})
    task = Task.objects.create(task='switchover_redis_node_task',
                                    displayment='switching replica {} to master'.format(slave.name, master.name),
                                    target_type=DBType.REDIS.value,
                                    target_id=slave.id,
                                    org_id=slave.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=tmp_data,
                                    created_by=current_user)
    return append_status_cmd(request, task)
@api_view(['POST'])
@permission_classes((IsAuthenticated, ))
def RemoveRedisSchedule(request):
    node_id = request.data.get("node_id")
    master = c123_node.objects.filter(pk=node_id, type = DBType.REDIS.value, ind_deleted = False, cluster_role = ClusterRoles.PRIMARY.value).first()
    if not master:
        raise APIException404("master id {} not found".format(node_id))
    allow_or_403(master)
    from django.conf import settings
    backup = c123_inventory.objects.filter(node = master, type = Inventory.BACKUP.value, is_sched = True).last()
    if backup:
        job_id = backup.ex_params.get("job_id")
        job = settings.scheduler.get_job(job_id=job_id)
        if job:
            settings.scheduler.remove_job(job_id=job_id)
            notification =  notification = f"{job_id} removed"
            return JsonResponse({"notification": notification}, status=200)
        
        else:
            slaves = c123_node.objects.filter(type = DBType.REDIS.value, cluster_role = ClusterRoles.STANDBY.value, ex_params__slave_of = master.pk)
            if slaves.exists():
                for slave in slaves:
                    backup = c123_inventory.objects.filter(node = slave, type = Inventory.BACKUP.value, is_sched = True).last()
                    if backup:
                        job_id = backup.ex_params.get("job_id")
                        job = settings.scheduler.get_job(job_id=job_id)
                        if job:
                            settings.scheduler.remove_job(job_id=job_id)
                            notification = f"{job_id} removed"
                            return JsonResponse({"notification": notification}, status=200)
                
                return JsonResponse({"notification": "no schedule backup found"}, status = 200)            
            else:
               return JsonResponse({"notification": "no schedule backup found"}, status = 200)
