from django.urls import path
from .views import *
from clusterCore.api.oracle.views import *
from clusterCore.api.postgres.views import *
from clusterCore.api.mongo.views import *
from clusterCore.api.mariadb.views import *
from clusterCore.api.mysql.views import *
from clusterCore.api.elasticsearch.views import *
from clusterCore.api.redis.views import *
from clusterCore.api.mssql.views import *
from clusterCore.api.monitoring.views import *
from clusterCore.api.licensing.views import *
from rest_framework_simplejwt.views import (
    TokenRefreshView,
)
urlpatterns = [

    #urls must have unique name paremeter, to not confuse resolver
    #token urls
    path('token/', SelfTokenObtainPairView.as_view(), name='token_obtain_pair'),
    path('token/refresh/', TokenRefreshView.as_view(), name='token_refresh'),
    path('', getRoutes),
    #user api
    path('current_user/', CurrentUserView.as_view(), name='get_current_user'),
    path('create_demo_user/', CreateDemoUser.as_view(), name='create_demo_user'),
    path('signup/', signup, name = "signup"),
    path('validate_email/', ValidateEmail, name = "validate_email"),
    path('activate/<uidb64>/<token>/', activate, name='activate'),
    path('user_update/<int:pk>/', user_update, name="user_update"),
    path('user_delete/<int:pk>/', DeleteUser, name="user_delete"),
    path("is_quota_exceed/", isQuotaExceed, name = "is_quota_exceed"),
    #node urls
    path('node_list/', NodeList.as_view(), name='node_list'),
    path("download_node_log/", DownloadNodeLogs.as_view(), name='download_node_log'),
    path("get_node_log_files/", list_node_logs, name='get_node_log_files'),
    path('host_available_list/', HostAvailableList.as_view(), name='host_list'),
    path('node_available_list/', NodeAvailableList.as_view(), name='node_available_list'),
    path('cluster_available_tree/', ClusterAvailableTree.as_view(), name='cluster_available_tree'),
    path('is_node_host_reachable/', IsNodeHostReachable, name = 'is_node_host_reachable'),
    path('mark_node_deleted/', MarkNodeDeleted, name = 'mark_node_deleted'),
    path('get_node/<int:pk>/', NodeDetailsView.as_view(), name = "node_details"),
    path('node_search/', NodeSearchView.as_view(), name='node_list_by_search'),
    path('node_list/<int:time_range>/', NodeListByTimeRange.as_view(), name='node_list_by_time_range'),
    path('create_node/', NodeCreateView.as_view(), name='create_node'),
    #path('node_list/db_type/<str:db_type>/', NodeListByDBType.as_view(), name='node_list_by_db_type'),
    #path('node_list/severity/<str:severity>/', NodeListBySeverity.as_view(), name='node_list_by_severity'),
    path('get_node_databases/<int:id>/', GetNodeDatabases, name = "node_databases_list"),
    path('database_users/<int:id>/', GetDatabaseUsers, name = "node_database_users_list"),
    path('edit_database_user/', EditDatabaseUser, name = "edit_database_user"),
    path('drop_database_user/', DropDatabaseUser, name = "drop_database_user"),
    path("drop_backup/", DropBackup, name= "drop_backup"),
    # integrated services
    path("get_integrated_services/", IntegratedServicesList.as_view(), name = "get_integrated_services"),
    path("add_integrated_service/", AddIntegratedService.as_view(), name = "add_integrated_service"),
    path("drop_integrated_service/", DropIntegratedService, name =  "drop_integrated_service"),
    path("edit_integrated_service/", EditIntegratedService, name =  "edit_integrated_service"),
    #licensing
    path("add_license/", AddLicense, name= "add_license"),
    path("remove_license/", RemoveLicense, name= "remove_license"),
    path("get_licensed_clusters/", GetLicensedClusters, name= "get_licensed_clusters"),
    path("move_license/", MoveLicense, name= "move_license"),
    #monitoring
    path('host_monitor_cpu_list/', HostMonitorCpuList.as_view(), name='host_monitor_cpu_list'),
    path('host_monitor_mem_list/', HostMonitorMemList.as_view(), name='host_monitor_mem_list'),
    path("get_mem_max_last/<int:pk>/", MemTotalLast.as_view(), name = "get_mem_max_last"),
    path('hosts_cpu_usages/', HostsCpuUsages, name='hosts_cpu_usages'),
    path("check_cpu_ram_current/", CheckCpuRamCurrent, name = 'check_cpu_ram_current'),
    path('alerts_list/', alerts_list, name = "alerts_list"),
    path('agents_list/', agents_list, name = "agents_list"),
    path('alerts_list/<int:id>/', node_alerts_list, name = "node_alerts_list"),
    path('get_alert_history/', GetAlertHistory, name = 'get_alert_history'),
    path('snooze_monitor/', SnoozeMonitor, name = 'snooze_monitor'),
    path('get_monitors/', GetMonitors, name = 'get_monitors'),
    path('get_monitor_chart_columns/', GetMonitorChartColumns, name = 'get_monitor_chart_columns'),
    path('get_monitor_chart/', GetMonitorChart, name = 'get_monitor_chart'),
    path('get_status_charts/', GetStatusCharts, name = 'get_status_charts'),
    path('get_node_monitor_names/', GetNodeMonitorNames, name = 'get_node_monitor_names'),
    path('get_node_monitors_table/<str:type>/', GetNodeMonitorsTable, name = 'get_node_monitors_table'),
    path('add_monitor/', AddMonitor, name = 'add_monitor'),
    path('add_agent/', AddAgent, name = 'add_agent'),
    path('get_agents_chart/', GetAgentsChart, name = 'get_agents_chart'),
    path('get_agents_summary/', GetAgentsSummary, name = 'get_agents_summary'),
    path('get_last_node_monitor_alert/', GetLastNodeMonitorAlert,  name = 'get_last_node_monitor_alert'),
    path('get_node_alerts_severity_counts/', GetNodeAlertsSeverityCounts,  name = 'get_node_alerts_severity_counts'),
    path('edit_agent/', EditAgent, name = 'edit_agent'),
    path('update_monitor/', UpdateMonitor, name = 'update_monitor'),
    path('delete_monitor/', DeleteMonitor, name = 'delete_monitor'),
    path('get_message/<int:id>/', GetMessage, name = 'get_message'),
    path('delete_agent/<int:id>/', DeleteAgent, name = 'delete_agent'),
    #dashboard monitoring
    path('get_dashboard_monitors/', GetDashboardMonitors, name = 'get_dashboard_monitors'),
    path('reorder_dashboard_monitors/', ReorderDashboardMonitors, name = 'reorder_dashboard_monitors'),
    path('add_dashboard_monitor/', AddDashboardMonitor, name = 'add_dashboard_monitor'),
    path('delete_dashboard_monitor/', DeleteDashboardMonitor, name = 'delete_dashboard_monitor'),
    #task Log
    path('task_log_list/<str:time_period>/', TaskLogList.as_view(), name='task_log_list'),
    path('task_log_list/<int:node_id>/<str:time_period>/', TaskLogList.as_view(), name='task_log_list_by_node'),
    path('task_log_text_list/', TaskLogTextList.as_view(), name='task_log_text_list'),
    path('task_search/', TaskSearchView.as_view(), name='task_list_by_search'),
    path('task_step_log_list/', TaskStepLogList.as_view(), name='task_step_log_list'),
    path('task_step_log_detail/<uuid:task_log_id>/', TaskStepLogDetail.as_view(), name='task_step_log_detail'),
    path('task_log_list_last/', TaskLogLastList.as_view(), name='task_log_list_last'),
    path("user_task_logs_by_type/", UserTaskLogsByType.as_view(), name = "user_task_logs_by_type"),
    #task list
    path("task_running_last_list/", TaskRunningLastList.as_view(), name = "task_running_last_list"),
    path("is_running_completed/", IsRunningCompleted, name = "is_running_completed"),
    #config api
    path('cluster_config/', ConfigView.as_view(), name = "cluster_config"),
    path('cluster_update/', cluster_update, name = 'cluster_update'),
    path("update_system_config/", UpdateSystemConfig, name= "update_system_config"),
    #system settings api
    path("update_system_settings/", UpdateSystemSettings, name= "update_system_settings"),
    path('system_settings/', GetSystemSettings, name = "system_settings"),
    #general layout models list
    path("nodes_backups_list/", NodesBackupsList.as_view(), name = "nodes_backups"),
    path("nodes_exports_list/", NodesExportsList.as_view(),  name = "nodes_exports"),
    path("backups_list/<str:type>/", BackupsListByType.as_view(), name = "backups_by_type"),
    path("exports_list/<str:type>/", ExportsListByType.as_view(),  name = "exports_by_type"),
    path("database_list/", DatabaseList.as_view(), name = "database_list"),
    path("haproxy_list/", HAProxyList.as_view(), name = "haproxy_list"),
    path("databases_exports_list/", DatabasesExportsList.as_view(), name = "databases_exports_list"),
    path("databases_backups_list/", DatabasesBackupsList.as_view(),  name = "databases_backups_list"),
    path("redis_cluster_list/", RedisClusterList.as_view(), name = "redis_cluster_list"),
    #general validation
    path("check_disk_exists/", checkDiskExists, name = "check_disk_exists"),
    path("check_disk_space/", checkDiskSpace, name = "check_disk_space"),
    path("check_inventory_exists/", checkInventoryExists, name = "check_inventory_exists"),
    path("check_export_file/", checkExportFile, name = "check_export_file"),
    path("check_port_exists/", CheckPortExists, name = "check_port_exists"),
    path("check_free_ip/", checkFreeIp, name = "check_free_ip"),
    path("check_path_exists/", CheckPathExists, name = "check_path_exists"),
    path("check_connection/", CheckConnection, name = "check_connection"),
    path("check_proxmox_connection/", CheckProxmoxConnection, name = "check_proxmox_connection"),
    path("check_aws_connection/", CheckAWSConnection, name = "check_aws_connection"),
    path("check_k8s_connection/", CheckK8sControlPlaneStatus, name = "check_k8s_control_plane_status"),
    path("get_subnets/", GetSubnets, name = "get_subnets"),
    path("check_task_status/", CheckTaskStatus, name = "check_task_status"),
    path("verify_backup/", VerifyBackup, name = "verify_backup"),
    #node details api
    path("status/<int:pk>/", NodeDetailsView.as_view(), name = "node_details"),
    path("status/haproxy/<int:id>/", HAProxyDetails, name=  "haproxy_details"),
    path("status/postg/<int:id>/", PostgresDetails, name=  "postgres_details"),
    path("status/postg_db/<int:id>/", PgDbDetails, name=  "postgres_db_details"),
    path("status/mongodb_db/<int:id>/", PgDbDetails, name=  "mongo_db_details"),
    path("status/elastic_db/<int:id>/", ElasticsearchDbDetails, name=  "elastic_db_details"),
    path("status/mariadb_db/<int:id>/", PgDbDetails, name=  "mariadb_db_details"),
    path("status/mysql_db/<int:id>/", PgDbDetails, name=  "mysql_db_details"),
    path("status/mssql_db/<int:id>/", PgDbDetails, name=  "mssql_db_details"),
    path("status/elastic/<int:id>/", ElasticsearchDetails, name=  "elasticsearch_details"),
    path("status/mariadb/<int:id>/", MariaDBDetails, name=  "mariadb_details"),
    path("status/mongo/<int:id>/", MongoDetails, name=  "mongo_details"),
    path("status/mysql/<int:id>/", MySQLDetails, name=  "mysql_details"),
    path("status/mssql/<int:id>/", MsSQLDetails, name=  "mssql_details"),
    path("status/redis/<int:id>/", RedisDetails, name=  "redis_details"),
    path("status/oracle/cdb/<int:pk>/",CDBDetailsView.as_view(), name=  "cdb_details"),
    path("status/oracle/cdb/<int:pk>/pdb/<int:id>/",PDBDetailsView.as_view(), name=  "pdb_details"),
    #cluster status api
    path("pg_primary_status/<int:id>/", PgPrimaryStatus, name = "pg_primary_status"),
    path("mongo_cluster_status/<int:id>/", MongoClusterStatus, name = "mongo_cluster_status"),
    path("mariadb_cluster_status/<int:id>/", MariaDBClusterStatus, name = "mariadb_cluster_status"),
    path("mysql_cluster_status/<int:id>/", MySQLClusterStatus, name = "mysql_cluster_status"),
    path("elasticsearch_cluster_status/<int:id>/", ElasticsearchClusterStatus, name = "elasticsearch_cluster_status"),
    path("check_redis_cluster_health/", CheckRedisClusterHealth, name = "check_redis_cluster_health"),
    #haproxy
    path('add_haproxy/', HAProxyAddView.as_view(), name = "add_haproxy"),
    path('drop_haproxy_node/', DropHAProxyNode, name = "drop_haproxy_node"),
    path('stop_haproxy_node/', StopHAProxyNode, name = "stop_haproxy_node"),
    path('start_haproxy_node/', StartHAProxyNode, name = "start_haproxy_node"),
    path('check_haproxy_exists/', CheckHAProxyExists, name = "check_haproxy_exists"),
    #oracle
    path('create_pdb/', PDBCreateView.as_view(), name='create_pdb'),
    path('install_cdb/', CDBInstallView.as_view(), name='install_cdb'),
    path('drop_pdb/', Drop_PDB, name="drop_pdb"),
    path('delete_cdb/', DeleteCDB, name="delete_cdb"),
    path("backup_pdb/", Backup_PDB, name= "backup_pdb"),
    path("copy_pdb/", Copy_PDB, name="copy_pdb"),
    path("close_pdb/", Close_PDB, name="close_pdb"),
    path("restore_backup_pdb/", RestoreBackupPDB, name="restore_backup_pdb"),
    path("drop_flashback/", DropFlashback, name="drop_flashback"),
    path("restore_from_flashback/", RestoreFromFlashback, name="restore_from_flashback"),
    path("rename_pdb/", Rename_PDB, name="rename_pdb"),
    path("open_pdb/", Open_PDB, name= "open_pdb"),
    path("change_cluster_size/", ChangeClusterSize, name= "change_cluster_size"),
    path("copy_to_cdb/",  Copy2CDB, name= "copy_to_cdb"),
    path("import_pdb/", ImportPDB, name = "import_pdb"),
    path("export_cdb_pdb/", ExportCDB_PDB, name = "export_cdb_pdb"),
    path("flashback_cdb_pdb/", Flashback_CDB_PDB, name="flashback_cdb_pdb"),
    path("duplicate_oracle_standalone/", Duplicate_Standalone, name="duplicate_standalone"),
    path("check_pdb_name/", CheckPDBName, name=  "check_pdb_name"),
    path("get_db_file_dest/<int:obj_id>/", getDBFileDestination, name="get_db_dest"),
    path("get_oracle_home/", GetOracleHome, name = "get_oracle_home"),
    path('create_cdb/', CDBCreateView.as_view(), name ='create_cdb'),
    path('create_pdb/', PDBCreateView.as_view(), name ='create_pdb'),
    path("nodes_flashbacks_list/", NodesFlashbacksList.as_view(), name = "nodes_flashbacks"),
    path('fetch_schemas/', fetchSchemas, name = 'fetch_schemas'),
    path('fetch_disks/', fetchDisks, name = 'fetch_disks' ),
    path('oracle_standalones_list/', OracleStandalonesList.as_view(), name = 'oracle_standalones_list'),
    path("check_oracle_db_path/", CheckPathName, name = "check_oracle_db_path"),
    path("check_dump_file/", checkDumpFile, name = "check_dump_file"),
    path("check_user_exists/", checkUserExists, name = "check_user_exists"),
    path("get_max_pdbs/", GetMaxPDBs, name = "get_max_pdbs"),
    path("tns_file_upload/", UploadTNSfile, name = "upload_tns_file"),
    path("drop_standalone/", DropStandalone, name= "drop_standalone"),
    path("check_credentials_pdb/", Check_Credentials_PDB, name= "check_credentials_pdb"),
    path("update_pdb_credentials/", Update_PDB_Credendials, name = "update_pdb_credentials"),
    path("remove_oracle_schedule/", RemoveOracleSchedule, name = "remove_oracle_schedule"),
    path("backup_cdb/", Backup_CDB, name = "backup_cdb"),
    path("import_cdb/", ImportCDB, name = "import_cdb"),
    path("check_oracle_version_exists/", CheckOracleVersionExists, name = "check_oracle_version_exists"),
    path("get_cdb_settings/<int:id>/", GetCDBSettings, name = "get_cdb_settings"),
    path("edit_cdb_settings/", EditCDBSettings, name = "edit_cdb_settings"),
    #mariadb
    path("check_mariadb_version_exists/", CheckMariaDBVersionExists, name = "check_mariadb_version_exists"),
    path("create_mariadb_standby/", MariaDBStandbyAddView.as_view(), name = "add_mariadb_standby"),
    path("add_mariadb_database/", AddMariaDBDatabase, name = "add_maria_db_database"),
    path("check_mariadb_database_name/", CheckMariaDBDatabaseName, name = "check_mariadb_database_name"),
    path("check_mariadb_database_size/", CheckMariaDBDatabaseSize, name = "check_mariadb_database_size"),
    path("restore_mariadb_node/", RestoreMariaDBNode, name= "restore_mariadb_node"),
    path("export_mariadb_database/", ExportMariaDBDatabase, name= "export_mariadb_database"),
    path("restore_mariadb_database/", RestoreMariaDBDatabase, name= "restore_mariadb_database"),
    path("stop_mariadb_node/", StopMariaDBNode, name= "stop_mariadb_node"),
    path("start_mariadb_node/", StartMariaDBNode, name= "start_mariadb_node"),
    path("drop_mariadb_node/", DropMariaDBNode, name= "drop_mariadb_node"),
    path("copy_mariadb_database/", CopyMariaDBDatabase, name = "copy_mariadb_database"),
    path("drop_mariadb_database/", DropMariaDBDatabase, name = "drop_mariadb_database"),
    path("remove_mariadb_schedule/", RemoveMariaDBSchedule, name = "remove_mariadb_schedule"),
    path("create_mariadb_master/", MariaDBClusterCreateView.as_view(), name="create_mariadb_master"),
    path("switch_mariadb_node/", SwitchoverMariaDBNode, name = "switch_mariadb_node"),
    path("check_mariadb_user_exists/", CheckMariaDBUserExists, name = "check_mariadb_user_exists"),
    path("backup_mariadb_node/", BackupMariaDBNode, name= "backup_mariadb_node"),
    path("export_mariadb_node/", ExportMariaDBNode, name= "export_mariadb_node"),
    path("import_mariadb_node/", ImportMariaDBNode, name = "import_mariadb_node"),
    path("upgrade_mariadb_node/", UpgradeMariaDBNode, name = "upgrade_mariadb_node"),
    #mysql
    path("check_mysql_version_exists/", CheckMySQLVersionExists, name = "check_mysql_version_exists"),
    path("create_mysql_standby/", MySQLStandbyAddView.as_view(), name = "add_mysql_standby"),
    path("add_mysql_database/", AddMySQLDatabase, name = "add_mysql_database"),
    path("check_mysql_database_name/", CheckMySQLDatabaseName, name = "check_mysql_database_name"),
    path("check_mysql_database_size/", CheckMySQLDatabaseSize, name = "check_mysql_database_size"),
    path("restore_mysql_node/", RestoreMySQLNode, name= "restore_mysql_node"),
    path("export_mysql_database/", ExportMySQLDatabase, name= "export_mysql_database"),
    path("export_mysql_node/", ExportMySQLNode, name= "export_mysql_node"),
    path("restore_mysql_database/", RestoreMySQLDatabase, name= "restore_mysql_database"),
    path("stop_mysql_node/", StopMySQLNode, name= "stop_mysql_node"),
    path("start_mysql_node/", StartMySQLNode, name= "start_mysql_node"),
    path("drop_mysql_node/", DropMySQLNode, name= "drop_mysql_node"),
    path("copy_mysql_database/", CopyMySQLDatabase, name = "copy_mysql_database"),
    path("drop_mysql_database/", DropMySQLDatabase, name = "drop_mysql_database"),
    path("remove_mysql_schedule/", RemoveMySQLSchedule, name = "remove_mysql_schedule"),
    path("switch_mysql_node/", SwitchoverMySQLNode, name = "switch_mysql_node"),
    path("create_mysql_master/", MySQLClusterCreateView.as_view(), name="create_mysql_master"),
    path("check_mysql_user_exists/", CheckMySQLUserExists, name = "check_mysql_user_exists"),
    path("backup_mysql_node/", BackupMySQLNode, name= "backup_mysql_node"),
    path("import_mysql_node/", ImportMySQLNode, name = "import_mysql_node"),
    path("get_mysql_settings/<int:id>/", GetMySQLSettings, name = "get_mysql_settings"),
    path("edit_mysql_settings/", EditMySQLSettings, name = "edit_mysql_settings"),
    path("upgrade_mysql_node/", UpgradeMySQLNode, name = "upgrade_mysql_node"),
    #mongo
    path("dump_mongo_database/", DumpMongoDatabase, name= "dump_mongo_database"),
    path("restore_mongo_database/", RestoreMongoDatabase, name= "restore_mongo_database"),
    path("stop_mongo_node/", StopMongoNode, name= "stop_mongo_node"),
    path("start_mongo_node/", StartMongoNode, name= "start_mongo_node"),
    path("copy_mongo_database/", CopyMongoDatabase, name = "copy_mongo_database"),
    path("drop_mongo_database/", DropMongoDatabase, name = "drop_mongo_database"),
    path("remove_mongo_schedule/", RemoveMongoSchedule, name = "remove_mongo_schedule"),
    path("create_mongo_master/", MongoClusterCreateView.as_view(), name="create_mongo_master"),
    path("create_mongo_node/", AddMongoNodeView.as_view(), name ="create_mongo_node"),
    path("dump_mongo_node/", DumpMongoNode, name = "dump_mongo_node"),
    path("check_mongo_database_size/", CheckMongoDatabaseSize, name = "check_mongo_database_size"),
    path("check_mongo_version_exists/", CheckMongoVersionExists, name = "check_mongo_version_exists"),
    path("backup_mongo_node/", BackupMongoNode, name = "backup_mongo_node"),
    path("restore_backup_mongo_node/", RestoreBackupMongoNode, name = "restore_backup_mongo_node"),
    path("restore_dump_mongo_node/", RestoreDumpMongoNode, name = "restore_dump_mongo_node"),
    path("drop_mongo_node/", DropMongoNode, name = "drop_mongo_node"),
    path("step_down_mongo_node/", StepDownMongoNode, name = "step_down_mongo_node"),
    path("check_mongo_database_name/", CheckMongoDatabaseName, name = "check_mongo_database_name"),
    path("add_mongo_database/", AddMongoDB, name = "add_mongo_database"),
    path("add_mongo_sharded_cluster/", MongoShardedClusterAddView.as_view(), name = "add_mongo_sharded_cluster"),
    path("add_mongo_sharded_cluster_primary/", AddMongoShardedClusterPrimaryView.as_view(), name = "add_mongo_sharded_cluster_primary"),
    path("add_mongo_sharded_cluster_node/", AddMongoShardedClusterNodeView.as_view(), name = "add_mongo_sharded_cluster_node"),
    path("drop_mongo_sharded_cluster/", DropMongoShardedCluster, name = 'drop_mongo_sharded_cluster'),
    path("upgrade_mongo_node/", UpgradeMongoNode, name= "upgrade_mongo_node"),

    #elasticsearch
    path("start_elasticsearch_node/", StartElasticsearchNode, name= "start_elasticsearch_node"),
    path("stop_elasticsearch_node/", StopElasticsearchNode, name= "stop_elasticsearch_node"),
    path("remove_elasticsearch_schedule/", RemoveElasticsearchSchedule, name = "remove_elasticsearch_schedule"),
    path("create_elasticsearch_master/", ElasticSearchClusterCreateView.as_view(), name="create_elasticsearch_master"),
    path("check_elasticsearch_index_name/", CheckElasticsearchIndexName, name = "check_elasticsearch_index_name"),
    path("check_elasticsearch_index_size/", CheckElasticsearchIndexSize, name = "check_elasticsearch_index_size"),
    path("add_elasticsearch_index/", AddElasticsearchIndex, name = "add_elasticsearch_index"),
    path("dump_elasticsearch_index/", DumpElasticsearchIndex, name = "dump_elasticsearch_index"),
    path("create_elasticsearch_node/", ElasticSearchNodeAddView.as_view(), name = "create_elasticsearch_node"),
    path("restore_elasticsearch_index/", RestoreElasticsearchIndex, name = "restore_elasticsearch_index"),
    path("copy_elasticsearch_index/", CopyElasticsearchIndex, name = "copy_elasticsearch_index"),
    path("snapshot_elasticsearch_cluster/", SnapshotElasticsearchCluster, name = "snapshot_elasticsearch_cluster"),
    path("restore_elasticsearch_cluster/", RestoreElasticsearchCluster, name = "restore_elasticsearch_cluster"),
    path("drop_elasticsearch_node/", DropElasticsearchNode, name = "drop_elasticsearch_node"),
    path("drop_elasticsearch_index/", DropElasticSearchIndex, name = "drop_elasticsearch_index"),
    path("check_elasticsearch_version_exists/", CheckElasticsearchVersionExists, name = "check_elasticsearch_version_exists"),
    path("switch_elasticsearch_node/", SwitchElasticsearchNode, name = "switch_elasticsearch_node"),
    path("upgrade_elasticsearch_node/", UpgradeElasticsearchNode, name = "upgrade_elasticsearch_node"),
    #postgres
    path("check_pg_version_exists/", CheckVersionPackageExists, name = "check_pg_version_exists"),
    path("create_pg_primary/", PgClusterCreateView.as_view(), name = "create_pg_primary"),
    path("create_pg_standby/", PgStandbyAddView.as_view(), name = "add_pg_standby"),
    path("check_pg_database_name/", CheckPgDatabaseName, name = "check_pg_database_name"),
    path("check_pg_database_size/", CheckPgDatabaseSize, name = "check_pg_database_size"),
    path("add_pg_database/", AddpgDB, name = "add_pg_database"),
    path("backup_pg_node/", BackupPgNode, name= "backup_pg_node"),
    path("upgrade_pg_node/", UpgradePgNode, name= "upgrade_pg_node"),
    path("export_pg_database/", ExportPgDatabase, name= "export_pg_database"),
    path("export_pg_node/", ExportPgNode, name = "export_pg_node"),
    path("restore_pg_database/", RestoreBackupPgDB, name= "restore_pg_database"),
    path("restore_pg_node/", RestoreBackupPgNode, name= "restore_pg_node"),
    path("stop_pg_node/", StopPgNode, name= "stop_pg_node"),
    path("start_pg_node/", StartPgNode, name= "start_pg_node"),
    path("check_pg_user_exists/", CheckPgUserExists, name = "check_pg_user_exists"),
    path("drop_pg_node/", DropPgNode, name= "drop_pg_node"),
    path("copy_pg_database/", CopyPgDatabase, name = "copy_pg_database"),
    path("drop_pg_database/", DropPgDatabase, name = "drop_pg_database"),
    path("switch_pg_node/", SwitchPgNode, name = "switch_pg_node"),
    path("clone_pg_standby/", ClonePgStandby, name = "cline_pg_standby"),
    path("toggle_pg_cluster/", TogglePgCluster, name = "toggle_pg _cluster"),
    path("remove_pg_schedule/", RemovePgSchedule, name = "remove_pg_schedule"),
    path("check_if_pg_primary/", CheckIfPgPrimary, name = "check_if_pg_primary"),
    path("import_pg_node/", ImportPgNode, name = "import_pg_node"),
    path("get_pg_settings/<int:id>/", GetPgSettings, name = "get_pg_settings"),
    path("edit_pg_settings/", EditPgSettings, name = "edit_pg_settings"),
    path("drop_pg_publication/", DropPgPublication, name = "drop_pg_publication"),
    path("toggle_pg_subscription/", TogglePgSubscription, name = "toggle_pg_subscription"),
    #redis
    path("check_sentinel_master_name/", CheckSentinelMasterName, name= "check_sentinel_master_name"),
    path("sentinel_monitor_master/", SentinelMonitorMaster, name= "sentinel_monitor_master"),
    path("sentinel_remove_monitor/", SentinelRemoveMonitor, name= "sentinel_remove_monitor"),
    path("sentinel_failover/", SentinelFailover, name= "sentinel_failover"),
    path("restore_redis_node/", RestoreBackupRedisNode, name= "restore_redis_node"),
    path("stop_redis_node/", StopRedisNode, name= "stop_redis_node"),
    path("drop_redis_node/", DropRedisNode, name= "drop_redis_node"),
    path("backup_redis_node/", BackupRedisNode, name= "backup_redis_node"),
    path("switch_redis_node/", SwitchRedisNode, name= "switchover_redis_node"),
    path("remove_redis_schedule/", RemoveRedisSchedule, name= "remove_redis_schedule"),
    path("create_redis_standalone/", CreateRedisStandaloneView.as_view(), name = "create_redis_standalone"),
    path("check_redis_version_exists/", CheckRedisVersionExists, name = "check_redis_version_exists"),
    path("create_redis_node/", RedisNodeAddView.as_view(), name = "create_redis_node"),
    path("add_redis_cluster/", RedisClusterAddView.as_view(), name = "add_redis_cluster"),
    path("start_redis_node/", StartRedisNode, name= "start_redis_node"),
    path("remove_redis_cluster/", RemoveRedisCluster, name = 'remove_redis_cluster'),
    #mssql
    path("create_mssql_master/", MsSQLClusterCreateView.as_view(), name = "create_mssql_master"),
    path("check_mssql_database_size/", CheckMsSQLDatabaseSize, name = "check_mssql_database_size"),
    path("check_mssql_database_name/", CheckMsSQLDatabaseName, name = "check_mssql_database_name"),
    path("add_mssql_database/", AddMsSQLDatabase, name = "add_mssql_database"),
    path("drop_mssql_node/", DropMsSQLNode, name= "drop_mssql_node"),
    path("drop_mssql_database/", DropMsSQLDatabase, name= "drop_mssql_database"),
    path("export_mssql_database/", ExportMsSQLDatabase, name= "export_mssql_database"),
    path("import_mssql_node/", ImportMsSQLNode, name= "import_mssql_node"),
    path("export_mssql_node/", ExportMsSQLNode, name= "export_mssql_node"),
    path("remove_mssql_schedule/", RemoveMsSQLSchedule, name= "remove_mssql_schedule"),
    path("restore_mssql_database/", RestoreMsSQLDatabase, name= "restore_mssql_database"),
    path("restore_mssql_node/", RestoreMsSQLNode, name= "restore_mssql_node"),
    path("copy_mssql_database/", CopyMsSQLDatabase, name = "copy_mssql_database"),
    path("backup_mssql_node/", BackupMsSQLNode, name= "backup_mssql_node"),
    path("stop_mssql_node/", StopMsSQLNode, name= "stop_mssql_node"),
    path("start_mssql_node/", StartMsSQLNode, name= "start_mssql_node"),
]
