from enum import Enum


class Timeout(Enum):
    BACKEND_SSH_TCP = 15
    BACKEND_SSH_BANNER = 15
    BACKEND_SSH_AUTH = 15
    BACKEND_MONGO = 15000
    BACKEND_MYSQL = 15
    BACKEND_MARIADB = 15
    BACKEND_POSTG = 15
    BACKEND_ELASTIC = 15
    BACKEND_REDIS = 15
    BACKEND_MSSQL = 15
    BACKEND_ORACLE = 15.0
    SERVICE_SSH_TCP = 30
    SERVICE_SSH_BANNER = 30
    SERVICE_SSH_AUTH = 30
    SERVICE_MONGO = 30000
    SERVICE_MARIADB = 30
    SERVICE_MYSQL = 30
    SERVICE_POSTG = 30
    SERVICE_ELASTIC = 30
    SERVICE_REDIS = 30
    SERVICE_MSSQL = 30
    SERVICE_ORACLE = 30.0

### model enums (meaningful are the key, make sure affected models can hold length of key before changing) 
class DataBaseType(Enum):
    MARIADB = 'MariaDB'
    MONGO = 'MongoDB'
    MYSQL = 'MySQL'
    MSSQL = 'SQL Server'
    ORACLE_CDB = 'Oracle CDB'
    ORACLE_PDB = 'Oracle PDB'
    POSTGRES = 'PostgreSQL'
    POSTGRES_K8S = 'PostgreSQL K8S'
    MONGO_K8S = 'MongoDB K8S'
    ELASTIC = 'Elasticsearch'
    REDIS = 'Redis'
    HAPROXY = 'HAProxy'


class IntegratedServiceTypes(Enum):
    PROXMOX = 'Proxmox server'
    AWS = 'AWS account'
    K8S = 'K8s node'

class User_Or_All_Choice(Enum):
    ALL = 'All'
    ONU = 'Only User'

class Node_Size_Allowed(Enum):
    SMALL = 'Small'
    MEDIUM = 'Medium'
    LARGE = 'Large'
    VERY_LARGE = 'Very Large'
    ANY = 'Any Size'
###

### general enums (meaningful are the value)

#for querying types (names - whatever, values - should inherit from DataBaseType names)
#TODO Maybe should be merged with DataBaseType for backend?     
class DBType(Enum):
    MARIADB = DataBaseType.MARIADB.name 
    MONGO = DataBaseType.MONGO.name 
    POSTGRES = DataBaseType.POSTGRES.name 
    MYSQL = DataBaseType.MYSQL.name
    MSSQL = DataBaseType.MSSQL.name
    ELASTIC = DataBaseType.ELASTIC.name
    REDIS = DataBaseType.REDIS.name
    ORACLE_PDB = DataBaseType.ORACLE_PDB.name
    ORACLE_CDB = DataBaseType.ORACLE_CDB.name
    HAPROXY = DataBaseType.HAPROXY.name
    POSTGRES_K8S = DataBaseType.POSTGRES_K8S.name
    MONGO_K8S =  DataBaseType.MONGO_K8S.name

class ClusterRoles(Enum): 
     PRIMARY = "PRIMARY"
     STANDBY = "STANDBY"
     ARBITER = "ARBITER"
     BALANCER = "BALANCER"
     PDB = "PDB"
     CDB = "CDB"
     SENTINEL =  "SENTINEL"

class RestoreOptions(Enum):
    THIS = "this"
    LATEST = "latest" 
    CUSTOM = "custom"


class CopyOptions(Enum):
    OVERRIDE = "override"
    COPY = "copy" 
    MIRROR = "mirror"


class Status(Enum):
    OK = ""
    DROPPING = "being dropped" 
    CREATING = "being created"
    UPGRADING = "being upgraded"
    REGISTERING = "being registered" 
    BACKUP = "being backed up"
    STEPDOWN = "stepping down"
    RESTORING = "being restored"
    PROMOTING = "being promoted to primary"
    DEMOTING = "demoting to standby of {}"
    SPLIT_BRAIN = "primary(split brain)"
    COPYING = "being copied" #source copy status
    REPLICATING = "being replicated" #target copy status
    OPENING = "being opened"
    CLOSING = "being closed"
    STARTING = "starting"
    STOPPING = "stopping"
    STOPPED = "stopped"
    RESTARTING = "restarting"
    PENDING = "pending"
    DETACHED = "removed from container"
    OFFLINE = "no connection to host"
    DOWN = "down"
    FAILED = "failed"
    DROPPED = "dropped"
    UNREGISTERED = "unregistered"
    CLOSED = "closed"
    NO_REPMGR = "no repmgr"
    RENAMING = "renaming to {}"
    SWITCHOVER = "in switchover"
    UP = "up"
    WARNING = "has warnings"


class TaskState(Enum):
    RUNNING = "running"
    PENDING = "ready"
    ON_DEMAND = "done"
    FAILED = "broken"

#name - color
class Severity(Enum):
    NOT_OK = "red"
    WARNING = "yellow"
    OK = "green"

class Inventory(Enum): # backup inventory types
    BACKUP = 'BACKUP'
    EXPORT = 'EXPORT'
    FLASHBACK = 'FLASHBACK'

#distributive types and supertypes - for distributive-specific logic
class OperatingSystem(Enum):
    LINUX = "LINUX"
    WINDOWS = "WINDOWS"
    CENTOS = "centos"
    UBUNTU = "ubuntu"
    DEBIAN = 'debian'
    REDHAT = 'rhel'
    FEDORA = 'fedora'
    ROCKY = 'rocky'
    INVALID = 'error'
    UNKNOWN = 'unknown'

# list of specific os versions could be installed on containers
class ContainerOS(Enum):
    CENTOS_8 = 'centos 8'
    UBUNTU_22 = 'ubuntu 22'

#for some of old generic backend views
class ObjectType(Enum):
    DATABASE = 'DATABASE'
    NODE = 'NODE'

class Schedule(Enum):
    NOW = 'NOW'
    SCHEDULED = 'SCHED'
    MINUTE = '1 minute'
    HOUR = '1 hour'
    DAY = '1 day'
    SECONDS_15 = '15 seconds'
    MINUTES_5 = '5 minutes'
    MINUTES_10 = '10 minutes'
    MINUTES_15 = '15 minutes'
    HOURS_6 = '6 hours'


class LicenseType(Enum): 
    ORACLE = [DBType.ORACLE_CDB.value, DBType.ORACLE_PDB.value]
    MYSQL = [DBType.MYSQL.value]
    MARIADB = [DBType.MARIADB.value]
    MONGO = [DBType.MONGO.value, DBType.MONGO_K8S.value]
    MSSQL = [DBType.MSSQL.value]
    ELASTIC = [DBType.ELASTIC.value]
    POSTGRES = [DBType.POSTGRES.value, DBType.POSTGRES_K8S.value]
    REDIS = [DBType.REDIS.value]

class MonitorCommand(Enum):
    SSH = 'ssh'
    SQL = 'sql'
    PYTHON = 'python'
    PYTHON3 = 'python3'


class PythonOS(Enum):
    WINDOWS = 'nt'
    LINUX = 'posix'
    JAVA = 'java'


class DefaultNamespace(Enum):
    POSTGRES_K8S = '123cluster-postgres'

###