import { useSelector, useDispatch } from "react-redux";
import { BrowserRouter, Navigate, Route, Routes } from "react-router-dom";
import Status from "./pages/status";
import Layout from "./pages/layout";
import Login from "./pages/login";
import Log from "./pages/log";
import Config from "./pages/config";
import Page404 from "./pages/page404";
import CDBStatusInfo from "./pages/oracle/cdbStatusInfo";
import PDBStatusInfo from "./pages/oracle/pdbStatusInfo";
import PrivateRoute from "./routing/privateRoute";
import PublicRoute from "./routing/publicRoute";
import { useEffect, useState } from "react";
import useInterval from "./util/useInterval";
import { ToastsActions } from "./store/toasts/reducer";
import { useRef } from "react";
import TaskLogDetailsDialog from "./dialogs/taskLogDetailsDialog";
import ScrollToTop from "./routing/scrollToTop";
import PostgresDetails from "./pages/postgres/postgresDetails";
import { ToastContainer } from "react-toastify-modernize";
import notify from "./util/notify";
import MongoDetails from "./pages/mongo/mongoDetails";
import MariaDBDetails from "./pages/mariadb/mariadbDetails";
import ElasticsearchDetails from "./pages/elasticsearch/elasticsearchDetails";
import RedisDetails from "./pages/redis/redisDetails";
import MySQLDetails from "./pages/mysql/mysqlDetails";
import MsSQLDetails from "./pages/mssql/mssqlDetails";
import Dashboard from "./pages/dashboard";
import HAProxyDetails from "./pages/balancers/haproxy/haproxyDetails";
import MongoKubernetesDetails from "./pages/mongo/mongoKubernetesDetails";
import PostgresKubernetesDetails from "./pages/postgres/postgresKubenernetesDetails";




const App = () => {
    const refreshToken = useSelector((state) => state.app.refreshToken);

    const { toasts } = useSelector((state) => state.toasts);
    const currentToast = useRef(null);
    const [showTaskLogDetails, setShowTaskLogDetails] = useState(false);
    const dispatch = useDispatch();

    useInterval(() => {
        dispatch(ToastsActions.getToasts());
    }, 60000);

    useEffect(() => {
        if (toasts?.length > 0) {
            toasts.forEach((toast) => toast.what && notify.detailed((
                <div  onClick={() => {
                    currentToast.current = toast;
                    setShowTaskLogDetails(true);
                }}>
                    {`${toast.what} ${toast.status === "green" ? "ok" : "not ok"}`}
                </div>), toast.status, toast.what));
        }

    }, [toasts]);

    return (
        <div>
            <BrowserRouter>
                <ScrollToTop />
                <Routes>
                    {/* PUBLIC ROUTES */}
                    <Route path="/login" element={<PublicRoute user={refreshToken}><Login /></PublicRoute>} />

                    {/* PRIVATE ROUTES */}
                    <Route path="/dashboard" element={<PrivateRoute user={refreshToken}><Dashboard/></PrivateRoute>} />
                    <Route path="/status" element={<PrivateRoute user={refreshToken}><Status /></PrivateRoute>} />
                    <Route path="/layout" element={<PrivateRoute user={refreshToken}><Layout /></PrivateRoute>} />
                    <Route path="/log/" element={<PrivateRoute user={refreshToken}><Log /></PrivateRoute>} />
                    <Route path="/config" element={<PrivateRoute user={refreshToken}><Config /></PrivateRoute>} />
                    <Route path="/status/oracle/cdb/:pk" element={<PrivateRoute user={refreshToken}>< CDBStatusInfo /></PrivateRoute>}></Route>
                    <Route path="/status/oracle/cdb/:pk/pdb/:id" element={<PrivateRoute user={refreshToken}>< PDBStatusInfo /></PrivateRoute>}></Route>
                    <Route path="/status/postg/:id" element={<PrivateRoute user={refreshToken}>< PostgresDetails /></PrivateRoute>}></Route>
                    <Route path="/status/postg_k8s/:id" element={<PrivateRoute user={refreshToken}>< PostgresKubernetesDetails /></PrivateRoute>}></Route>
                    <Route path="/status/mongo/:id" element={<PrivateRoute user={refreshToken}>< MongoDetails /></PrivateRoute>}></Route>
                    <Route path="/status/mongo_k8s/:id" element={<PrivateRoute user={refreshToken}>< MongoKubernetesDetails /></PrivateRoute>}></Route>
                    <Route path="/status/mariadb/:id" element={<PrivateRoute user={refreshToken}>< MariaDBDetails /></PrivateRoute>}></Route>
                    <Route path="/status/elastic/:id" element={<PrivateRoute user={refreshToken}><ElasticsearchDetails /></PrivateRoute>}></Route>
                    <Route path="/status/redis/:id" element={<PrivateRoute user={refreshToken}><RedisDetails /></PrivateRoute>}></Route>
                    <Route path="/status/mysql/:id" element={<PrivateRoute user={refreshToken}><MySQLDetails /></PrivateRoute>}></Route>
                    <Route path="/status/mssql/:id" element={<PrivateRoute user={refreshToken}><MsSQLDetails /></PrivateRoute>}></Route>
                    <Route path="/status/haproxy/:id" element={<PrivateRoute user={refreshToken}><HAProxyDetails /></PrivateRoute>}></Route>
                    <Route path="/" element={<Navigate to="/login" />} />
                    <Route path="/*" element={<Page404 />} />
                </Routes>
            </BrowserRouter>
            <ToastContainer closeOnClick = {false} limit={5} />
            <TaskLogDetailsDialog visible={showTaskLogDetails} onHide={(e) => setShowTaskLogDetails(false)} log={currentToast.current} />
        </div>
    );
}


export default App;