import { clusterSizeOptions } from "../../config";
import { Dropdown } from "primereact/dropdown";
import { useState, useRef } from "react";
import { LayoutActions } from "../../store/layout/reducer";
import { useDispatch } from "react-redux";
import { ConfirmPopup } from "primereact/confirmpopup";

export default function ClusterSizing({ node }) {

    const [newSize, setNewSize] = useState(node?.sizing ? node.sizing : clusterSizeOptions[0].value);
    const [showWarningPopup, setShowWarningPopup] = useState(false);
    const dispatch = useDispatch();
    const oldSize = useRef(newSize);

    return (<div>
        <h3>Cluster Size</h3>
        <Dropdown id = 'cluster-size-options' placeholder="Cluster Size" value={newSize} options={clusterSizeOptions} onChange={(e) => {
            setNewSize(e.value);
            setShowWarningPopup(true);
        }} />
        <ConfirmPopup
            icon="pi pi-exclamation-triangle"
            target={document.getElementById('cluster-size-options')}
            visible={showWarningPopup}
            onHide={() => setNewSize(oldSize.current)}
            accept={() =>  {
                dispatch(LayoutActions.changeClusterSize({ node_id: node?.id, new_size: newSize }));
                oldSize.current = newSize
            }}
            reject={() => setNewSize(oldSize.current)}
            message={`Are you sure you want to change node cluster sizing?`} />
    </div>)

}