import { useState, useEffect } from "react";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { InputText } from "primereact/inputtext";
import { Checkbox } from "primereact/checkbox";
import refreshTooltip from "../../refreshTooltip";
import { dbTypes, timePeriodTypes} from "../../../config";
import { MonitoringActions } from "../../../store/monitoring/reducer";
import { useSelector, useDispatch } from "react-redux";
import useInterval from "../../../util/useInterval";

const DashboardFilter = (props) => {

    const {db_type, time_period} = useSelector((state) => state.app);
    const [search, setSearch] = useState("");
    const [dbType, setDbType] = useState(db_type);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [autoRefresh, setAutoRefresh] = useState(false);
    const dispatch = useDispatch();


    useInterval(() => {
        dispatch(MonitoringActions.getNodes());
    }, autoRefresh ? 30000 : null);

    useEffect(() => {
        const storedSearch = localStorage.getItem("FilterSearch");
        const storedDbType = localStorage.getItem("FilterDbType");
        const storedTimePeriodType = localStorage.getItem("FilterTimePeriodType");
        const storedAutoRefresh = localStorage.getItem("FilterAutoRefresh");

        if (storedSearch) setSearch(storedSearch);
        if (storedDbType) setDbType(storedDbType);
        if (storedTimePeriodType) setTimePeriodType(storedTimePeriodType);
        if (storedAutoRefresh) setAutoRefresh(storedAutoRefresh === "true");
    }, []);

    useEffect(() => {
        localStorage.setItem("FilterSearch", search);
        localStorage.setItem("FilterDbType", dbType);
        localStorage.setItem("FilterTimePeriodType", timePeriodType);
        localStorage.setItem("FilterAutoRefresh", autoRefresh);
    }, [search, dbType, timePeriodType, autoRefresh]);

    const onMonitorChange = (e) => {
        setDbType(e.target.value);
        if (props.onMonitorChange)
            props.onMonitorChange(e.target.value, search);
    }

    const onTimePeriodChange = (e) => {
        setTimePeriodType(e.target.value);
        if (props.onTimePeriodChange)
            props.onTimePeriodChange(e.target.value, dbType, search);
    }

    const onSearchChange = (e) => {
        setSearch(e.target.value);
        if (props.onSearchChange)
            props.onSearchChange(e.target.value.toLowerCase(), dbType);
    }

    const onRefreshClick = () => {
        window.location.reload();
    }

    return (
        <div className="filter-component-wrapper">
            <div className="filter-component">
                <div className="filter-left-container flex flex-grow-1">
                    <div className="filter-title">Filter by:</div>
                    <Dropdown
                        className={"filter-field p-inputtext-sm"}
                        value={dbType}
                        options={dbTypes}
                        onChange={onMonitorChange}
                        placeholder="Monitor Types" />

                    <span className="p-input-icon-right mr-2">
                        <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => {
                            setSearch("");
                            if (props.onSearchChange)
                                props.onSearchChange("", dbType);

                        }} ></i>
                        <InputText className={"filter-field p-inputtext-sm ml-2"} tooltip="Search by node name, location, cluster role, created date, status, monitoring alert" tooltipOptions={{ position: 'bottom' }} placeholder="Search" value={search} onChange={onSearchChange} />
                    </span>

                </div>

                <div className="flex flex-row filter-right-container ml-auto">

                    <Dropdown
                        className={"filter-field time-period p-inputtext-sm"}
                        value={timePeriodType}
                        options={timePeriodTypes}
                        onChange={onTimePeriodChange}
                        placeholder="Time Period" />

                    <div className="align-self-center ml-2 mr-2">


                        <Checkbox inputId="autoupdate_cb" className="ml-1" checked={autoRefresh} value={autoRefresh} onChange={() => {
                            setAutoRefresh(!autoRefresh);
                        }} />
                        <label htmlFor="autoupdate_cb" className="p-checkbox-label"> Auto refresh</label>

                    </div>


                    <Button className={"filter-button refresh-button p-button-sm"} onClick={onRefreshClick} icon="pi pi-refresh" iconPos="right" />
                </div>


            </div>
            {refreshTooltip}
        </div>
    );
}


export default DashboardFilter;
