import { Fragment, useState, useEffect } from "react";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { InputText } from "primereact/inputtext";
import refreshTooltip from "../../refreshTooltip";
import { dbTypes, types } from "../../../config";
import { Checkbox } from "primereact/checkbox";
import useInterval from "../../../util/useInterval";
import { LayoutActions } from "../../../store/layout/reducer";
import { useDispatch, useSelector } from "react-redux";
import { ButtonGroup } from "primereact/buttongroup";

const LayoutFilter = (props) => {

    const {db_type, _search} = useSelector((state) => state.app);
    const [search, setSearch] = useState(_search);
    const [dbType, setDbType] = useState(db_type);
    const [severity, setSeverity] = useState(null);
    const [autoRefresh, setAutoRefresh] = useState(false);
    const dispatch = useDispatch();

    useInterval(() => {
        dispatch(LayoutActions.getNodes());
    }, autoRefresh ? 30000 : null);

    useEffect(() => {
        const storedSearch = localStorage.getItem("FilterSearch");
        const storedDbType = localStorage.getItem("FilterDbType");
        const storedSeverity = localStorage.getItem("FilterSeverity");
        const storedAutoRefresh = localStorage.getItem("FilterAutoRefresh");

        if (storedSearch) setSearch(storedSearch);
        if (storedDbType) setDbType(storedDbType);
        if (storedSeverity) setSeverity(storedSeverity);
        if (storedAutoRefresh) setAutoRefresh(storedAutoRefresh === "true");
    }, []);

    useEffect(() => {
        localStorage.setItem("FilterSearch", search);
        localStorage.setItem("FilterDbType", dbType);
        localStorage.setItem("FilterSeverity", severity);
        localStorage.setItem("FilterAutoRefresh", autoRefresh);
    }, [search, dbType, severity, autoRefresh]);

    const onSeverityClick = (severity) => () => {
        setSeverity(severity);
        if (props.onSeverityChange)
            props.onSeverityChange(severity, search, dbType);
    }

    const onDbChange = (e) => {
        setDbType(e.target.value);
        if (props.onDbChange)
            props.onDbChange(e.target.value, search, severity);
    }

    const onSearchChange = (e) => {
        setSearch(e.target.value);
        if (props.onSearchChange)
            props.onSearchChange(e.target.value.toLowerCase(), dbType, severity);
    }

    const onRefreshClick = () => {
        window.location.reload();
    }

    return (
        <Fragment>
            <div className="filter-component-wrapper">
                <div className="filter-component">
                    <div className="filter-left-container flex-grow-0" >


                        <div className="flex flex-column flex-grow-0">
                            <div className="flex flex-grow-0">

                                <div className="filter-title">Filter by:</div>
                                <Dropdown
                                    className="p-inputtext-sm filter-field"
                                    value={dbType}
                                    options={dbTypes.filter((type) => [types.ORACLE_PDB, types.HAPROXY].indexOf(type.value) === -1)}
                                    onChange={onDbChange}
                                    placeholder="DB Types" />
                                <span className="p-input-icon-right ml-2">
                                    <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                        setSearch("");
                                        if (props.onSearchChange)
                                            props.onSearchChange("", dbType, severity);
                                    }} ></i>

                                    <InputText className="p-inputtext-sm filter-field" tooltip="Search by node name, type, severity" tooltipOptions={{ position: 'bottom' }} placeholder="Search" value={search} onChange={onSearchChange} />
                                </span>

                            </div>

                        </div>
                    </div>

                    <div >
                        <ButtonGroup >
                            <Button className={props.showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={onSeverityClick("green")} label={`${props.severityGreenTotal}`} />
                            <Button className={props.showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={onSeverityClick("yellow")} label={`${props.severityYellowTotal}`} />
                            <Button className={props.showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={onSeverityClick("red")} label={`${props.severityRedTotal}`} />
                        </ButtonGroup>
                    </div>

                    <div className="filter-right-container flex flex-row align-items-center ml-auto">
                        <Checkbox inputId="autoupdate_cb" className="ml-auto" checked={autoRefresh} value={autoRefresh} onChange={() => {
                            setAutoRefresh(!autoRefresh);
                        }} />
                        <label htmlFor="autoupdate_cb" className="p-checkbox-label ml-2 mr-2"> Auto refresh</label>
                        <Button className="filter-button refresh-button p-button-sm" onClick={onRefreshClick} icon="pi pi-refresh" iconPos="right" />

                    </div>
                </div>
            </div >
            {refreshTooltip}
        </Fragment>


    );
}


export default LayoutFilter;
