import { useState, useEffect } from "react";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { InputText } from "primereact/inputtext";
import { Checkbox } from "primereact/checkbox";
import refreshTooltip from "../../refreshTooltip";
import { dbTypes, timePeriodTypes} from "../../../config";
import { StatusActions } from "../../../store/status/reducer";
import { useSelector, useDispatch } from "react-redux";
import useInterval from "../../../util/useInterval";
import { ButtonGroup } from "primereact/buttongroup";




const StatusFilter = (props) => {

    const {db_type, time_period} = useSelector((state) => state.app);
    const [search, setSearch] = useState("");
    const [dbType, setDbType] = useState(db_type);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [severity, setSeverity] = useState(null);
    const [autoRefresh, setAutoRefresh] = useState(false);
    const dispatch = useDispatch();

    useInterval(() => {
        dispatch(StatusActions.getNodes());
    }, autoRefresh ? 30000 : null);

    useEffect(() => {
        const storedSearch = localStorage.getItem("FilterSearch");
        const storedDbType = localStorage.getItem("FilterDbType");
        const storedTimePeriodType = localStorage.getItem("FilterTimePeriodType");
        const storedSeverity = localStorage.getItem("FilterSeverity");
        const storedAutoRefresh = localStorage.getItem("FilterAutoRefresh");

        if (storedSearch) setSearch(storedSearch);
        if (storedDbType) setDbType(storedDbType);
        if (storedTimePeriodType) setTimePeriodType(storedTimePeriodType);
        if (storedSeverity) setSeverity(storedSeverity);
        if (storedAutoRefresh) setAutoRefresh(storedAutoRefresh === "true");
    }, []);

    useEffect(() => {
        localStorage.setItem("FilterSearch", search);
        localStorage.setItem("FilterDbType", dbType);
        localStorage.setItem("FilterTimePeriodType", timePeriodType);
        localStorage.setItem("FilterSeverity", severity);
        localStorage.setItem("FilterAutoRefresh", autoRefresh);
    }, [search, dbType, timePeriodType, severity, autoRefresh]);

    const onSeverityClick = (severity) => () => {
        setSeverity(severity);
        if (props.onSeverityChange)
            props.onSeverityChange(severity, search, dbType);


    }


    const onDbChange = (e) => {
        setDbType(e.target.value);
        if (props.onDbChange)
            props.onDbChange(e.target.value, search, severity);


    }

    const onTimePeriodChange = (e) => {
        setTimePeriodType(e.target.value);
        if (props.onTimePeriodChange)
            props.onTimePeriodChange(e.target.value, dbType, search, severity);


    }

    const onSearchChange = (e) => {
        setSearch(e.target.value);
        if (props.onSearchChange)
            props.onSearchChange(e.target.value.toLowerCase(), dbType, severity);


    }

    const onRefreshClick = () => {
        window.location.reload();
    }


    return (
        <div className="filter-component-wrapper">
            <div className="filter-component">
                <div className="filter-left-container flex flex-grow-0">
                    <div className="filter-title">Filter by:</div>
                    <Dropdown
                        className={"filter-field p-inputtext-sm"}
                        value={dbType}
                        options={dbTypes}
                        onChange={onDbChange}
                        placeholder="DB Types" />

                    <span className="p-input-icon-right mr-2">
                        <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => {
                            setSearch("");
                            if (props.onSearchChange)
                                props.onSearchChange("", dbType, severity);

                        }} ></i>
                        <InputText className={"filter-field p-inputtext-sm ml-2"} tooltip="Search by node name, location, severity, cluster role, created date, status, monitoring alert" tooltipOptions={{ position: 'bottom' }} placeholder="Search" value={search} onChange={onSearchChange} />
                    </span>

                </div>

                    <ButtonGroup>
                        <Button className={props.showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={onSeverityClick("green")}  label={`${props.severityGreenTotal}`} />
                        <Button className={props.showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={onSeverityClick("yellow")}  label={`${props.severityYellowTotal}`} />
                        <Button className={props.showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={onSeverityClick("red")}  label={`${props.severityRedTotal}`} />
                    </ButtonGroup>



                <div className="flex flex-row filter-right-container ml-auto">

                    <Dropdown
                        className={"filter-field time-period p-inputtext-sm"}
                        value={timePeriodType}
                        options={timePeriodTypes}
                        onChange={onTimePeriodChange}
                        placeholder="Time Period" />

                    <div className="align-self-center ml-2 mr-2">


                        <Checkbox inputId="autoupdate_cb" className="ml-1" checked={autoRefresh} value={autoRefresh} onChange={() => {
                            setAutoRefresh(!autoRefresh);
                        }} />
                        <label htmlFor="autoupdate_cb" className="p-checkbox-label"> Auto refresh</label>

                    </div>


                    <Button className={"filter-button refresh-button p-button-sm"} onClick={onRefreshClick} icon="pi pi-refresh" iconPos="right" />
                </div>


            </div >
            {refreshTooltip}
        </div>
    );
}


export default StatusFilter;
