import { useDispatch, useSelector } from 'react-redux';
import { SignupActions } from '../../../store/signup/reducer';
import { UserUpdateActions } from '../../../store/userUpdate/reducer';
import { RadioButton } from 'primereact/radiobutton';
import { Dropdown } from "primereact/dropdown";
import { Slider } from 'primereact/slider';





const PrivilegesForm = (props) => {

  const sizeOptions =
    [
      { label: "Small", value: "SML" },
      { label: "Medium", value: "MED" },
      { label: "Large", value: "LRG" },
      { label: "Very Large", value: "VLA" },
      { label: "Any Size", value: "ANY" },


    ];

  const userOptions =
    [
      { label: "Anyone nodes", value: "ALL" },
      { label: "User`s nodes", value: "ONU" },
    ];

  const { data } = useSelector((state) => props.update ? state.userUpdate : state.signUp);
  const dispatch = useDispatch();

  const onInput = (names, value) => {
    // Ensure names is always an array, even if a single string is passed
    const fields = Array.isArray(names) ? names : [names];
  
    // Create the updated priv_dict
    const updatedPrivDict = fields.reduce((acc, name) => {
      acc[name] = value;
      return acc;
    }, { ...data.priv_dict });
  
    // Dispatch the appropriate action based on props.update
    if (props.update) {
      dispatch(UserUpdateActions.setData({
        ...data,
        priv_dict: updatedPrivDict,
      }));
    } else {
      dispatch(SignupActions.setData({
        ...data,
        priv_dict: updatedPrivDict,
      }));
    }
  };

  return (
    <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
      <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
        <div className="field grid">
          <div className="col">
            <div className="field-radiobutton">
              <RadioButton inputId="superUserOption1" name="superUserOptions" value="True" checked={data.priv_dict.is_superuser === "True"} onChange={(e) => onInput("is_superuser", e.value)} />
              <label htmlFor="superUserOption1">A super user</label>
            </div>
            <div className="field-radiobutton">
              <RadioButton inputId="superUserOption2" name="superUserOptions" value="False" checked={data.priv_dict.is_superuser === "False"} onChange={(e) => onInput("is_superuser", e.value)} />
              <label htmlFor="superUserOption2">A regular user with the following privileges</label>
            </div>
          </div>
        </div>
        <div className="p-datatable p-component">
          <div className="p-datatable-wrapper">

            <table className="p-datatable-table">
              <thead className="p-datatable-thead">
                <tr>
                  <th></th>
                  <th ><div className="p-column-header-content">
                    <span className="p-column-title">
                      Quota
                    </span>
                  </div></th>
                  <th ><div className="p-column-header-content">
                    <span className="p-column-title">

                      View

                    </span>
                  </div></th>

                  <th >
                    <div className="p-column-header-content">
                      <span className="p-column-title">
                        Delete
                      </span>

                    </div>
                  </th>
                </tr>
              </thead>
              <tbody>
                <tr>
                  <td>
                    <div className="col">
                      Oracle PDBs

                    </div>
                  </td>
                  <td >
                    <div className="col flex flex-row">
                      <div className="col">
                        <div className="title">{data.priv_dict.oracle_pdb_quota} PDBs</div>
                        <Slider
                          value={data.priv_dict.oracle_pdb_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("oracle_pdb_quota", e.value)}

                        />

                      </div>

                      <div className='col'>
                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.oracle_pdb_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("oracle_pdb_create", e.value)}
                        />

                      </div>
                    </div>
                  </td>
                  <td >
                    <div className="col">
                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.oracle_pdb_view}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["oracle_pdb_view","oracle_pdb_delete"], e.value)
                        }}
                      />


                    </div>
                  </td>
                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.oracle_pdb_delete}
                        options={userOptions}
                        onChange={(e) => {

                          onInput(["oracle_pdb_delete", "oracle_pdb_view"], e.value);

                        }

                        }
                      />

                    </div>
                  </td>
                </tr>
                <tr className="p-row-odd">
                  <td>
                    <div className="col">
                      Mongo DB
                    </div>
                  </td>
                  <td >
                    <div className="col flex flex-row">

                      <div className="col">
                        <div className="title">{data.priv_dict.mongo_quota} Nodes</div>
                        <Slider
                          value={data.priv_dict.mongo_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("mongo_quota", e.value)}

                        />
                      </div>

                      <div className="col">
                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.mongo_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("mongo_create", e.value)}
                        />


                      </div>

                    </div>



                  </td>

                  <td >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mongo_view}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["mongo_view", "mongo_delete"], e.value);
                        }}
                      />

                    </div>
                  </td>
                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mongo_delete}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["mongo_delete", "mongo_view"], e.value);
                        }}
                      />

                    </div>
                  </td>
                </tr>
                <tr>
                  <td>
                    <div className="col">
                      MySQL
                    </div>
                  </td>
                  <td >
                    <div className="col flex flex-row">

                      <div className="col">
                        <div className="title">{data.priv_dict.mysql_quota} Nodes</div>
                        <Slider
                          value={data.priv_dict.mysql_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("mysql_quota", e.value)}

                        />
                      </div>
                      <div className="col">

                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.mysql_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("mysql_create", e.value)}
                        />

                      </div>
                    </div>
                  </td>
                  <td >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mysql_view}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["mysql_view", "mysql_delete"], e.value);
                        }}
                      />

                    </div>
                  </td>

                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mysql_delete}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["mysql_view", "mysql_delete"], e.value);
                        }
                        }
                      />

                    </div>
                  </td>
                </tr>
                <tr className="p-row-odd">
                  <td>
                    <div className="col">
                      Postgres
                    </div>
                  </td>
                  <td >
                    <div className="col flex flex-row">

                      <div className="col">
                        <div className="title">{data.priv_dict.postgres_quota} Nodes</div>
                        <Slider
                          value={data.priv_dict.postgres_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("postgres_quota", e.value)}
                        />
                      </div>

                      <div className="col">

                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.postgres_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("postgres_create", e.value)}
                        />


                      </div>

                    </div>
                  </td>
                  <td >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.postgres_view}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["postgres_view", "postgres_delete"], e.value);
                        }}
                      />

                    </div>
                  </td>
                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.postgres_delete}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["postgres_delete", "postgres_view"], e.value);

                        }
                        }
                      />

                    </div>
                  </td>
                  <td>
                  </td>
                </tr>
                <tr>
                  <td>
                    <div className="col">
                      MariaDB
                    </div>
                  </td>
                  <td >

                    <div className="col flex flex-row">

                      <div className="col">
                        <div className="title">{data.priv_dict.mariadb_quota} Nodes</div>
                        <Slider
                          value={data.priv_dict.mariadb_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("mariadb_quota", e.value)}
                        />
                      </div>
                      <div className="col">

                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.mariadb_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("mariadb_create", e.value)}
                        />


                      </div>

                    </div>
                  </td>
                  <td >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mariadb_view}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["mariadb_view", "mariadb_delete"], e.value);

                        }
                        }
                      />

                    </div>
                  </td>

                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mariadb_delete}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["mariadb_delete", "mariadb_view"], e.value);
                        }}
                      />

                    </div>
                  </td>
                  <td>
                  </td>
                </tr>
                <tr className="p-row-odd">
                  <td>
                    <div className="col">
                      Elasticsearch
                    </div>
                  </td>
                  <td >

                    <div className="col flex flex-row">

                      <div className="col">
                        <div className="title">{data.priv_dict.elasticsearch_quota} Nodes</div>
                        <Slider
                          value={data.priv_dict.elasticsearch_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("elasticsearch_quota", e.value)}
                        />
                      </div>

                      <div className="col">

                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.elasticsearch_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("elasticsearch_create", e.value)}
                        />


                      </div>

                    </div>

                  </td>
                  <td >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.elasticsearch_view}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["elasticsearch_view", "elasticsearch_delete"], e.value);
                        }}
                      />

                    </div>
                  </td>
                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.elasticsearch_delete}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["elasticsearch_delete", "elasticsearch_view"], e.value);
                        }}
                      />

                    </div>
                  </td>
                  <td>
                  </td>
                </tr>
                <tr>
                  <td>
                    <div className="col">
                      MsSQL
                    </div>
                  </td>
                  <td >
                    <div className="col flex flex-row">

                      <div className="col">
                        <div className="title">{data.priv_dict.mssql_quota} Nodes</div>
                        <Slider
                          value={data.priv_dict.mssql_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("mssql_quota", e.value)}
                        />
                      </div>

                      <div className="col">

                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.mssql_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("mssql_create", e.value)}
                        />
                      </div>

                    </div>

                  </td>
                  <td >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mssql_view}
                        options={userOptions}
                        onChange={(e) => {

                          onInput(["mssql_view", "mssql_delete"], e.value);


                        }}
                      />

                    </div>
                  </td>
                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.mssql_delete}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["mssql_delete", "mssql_view"], e.value);
                        }}
                      />

                    </div>
                  </td>
                  <td>
                  </td>
                </tr>
                <tr>
                  <td>
                    <div className="col">
                      Redis
                    </div>
                  </td>
                  <td >
                    <div className="col flex flex-row">

                      <div className="col">
                        <div className="title">{data.priv_dict.redis_quota} Nodes</div>
                        <Slider
                          value={data.priv_dict.redis_quota}
                          min={0}
                          max={100}
                          step={1}
                          onChange={(e) => onInput("redis_quota", e.value)}
                        />
                      </div>

                      <div className="col">

                        <Dropdown className="p-inputtext-sm" value={data.priv_dict.redis_create}
                          options={sizeOptions}
                          onChange={(e) => onInput("redis_create", e.value)}
                        />
                      </div>

                    </div>

                  </td>
                  <td >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.redis_view}
                        options={userOptions}
                        onChange={(e) => {

                          onInput(["redis_view", "redis_delete"], e.value);


                        }}
                      />

                    </div>
                  </td>
                  <td  >
                    <div className="col">

                      <Dropdown className="p-inputtext-sm" value={data.priv_dict.redis_delete}
                        options={userOptions}
                        onChange={(e) => {
                          onInput(["redis_delete","redis_view"], e.value);
                        }}
                      />

                    </div>
                  </td>
                  <td>
                  </td>
                </tr>
              </tbody>
            </table>

          </div>

        </div>
      </div>
    </div>);

};

export default PrivilegesForm;