import { useState, useRef, useEffect, useCallback } from "react";
import { TabView, TabPanel } from 'primereact/tabview';
import { nanoid } from 'nanoid';
import * as d3 from "d3";
import trashImage from "../../../assets/images/trash.png";
import dbImage from "../../../assets/images/db.png";
import plusIcon from "../../../assets/images/plus.png";
import minusIcon from "../../../assets/images/minus.png";
import node_white from "../../../assets/images/node_white.png";
import node_unavail from '../../../assets/images/node_unavail.png'
import node_un_white from '../../../assets/images/node_un_white.png';
import db_white from "../../../assets/images/db_white.png";
import node_avail from "../../../assets/images/node.png";
import bar from "../../../assets/images/bar.png";
import db_unavail from "../../../assets/images/db_unavail.png";
import db_un_white from "../../../assets/images/db_un_white.png";
import { useNavigate } from 'react-router-dom';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import moment from "moment";
import WarningDialog from "../../../dialogs/warningDialog";
import { useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import { useSelector } from "react-redux";
import DropIndexDialog from "../../../dialogs/elasticsearch/dropIndexDialog";
import DumpDetailsDialog from "../../../dialogs/mongo/dumpDetailsDialog";
import BackupDetailsDialog from "../../../dialogs/postgres/backupDetailsDialog";
import { Button } from "primereact/button";
import DeleteInventoryDialog from "../../../dialogs/deleteInventoryDialog";
import getColorBySeverity from "../../../util/colorBySeverity";
import { ScrollPanel } from "primereact/scrollpanel";
import { Panel } from "primereact/panel";
import ElasticsearchClusterSnapshotDialog from "../../../dialogs/elasticsearch/elasticsearchClusterSnapshotDialog";
import RestoreElasticsearchClusterLayout from "../../../dialogs/elasticsearch/restoreElasticsearchClusterLayout";
import AddIndexDialog from "../../../dialogs/elasticsearch/addIndexDialog";
import RestoreIndexLayoutDialog from "../../../dialogs/elasticsearch/restoreIndexLayoutDialog";
import DumpIndexDialog from "../../../dialogs/elasticsearch/dumpIndexDialog";
import CopyIndexDialog from "../../../dialogs/elasticsearch/copyIndexDialog";
import ElasticSearchStatusDialog from "../../../dialogs/elasticsearch/elasticsearchStatusDialog";
import DropElasticsearchNodeDialog from "../../../dialogs/elasticsearch/dropElasticsearchNodeDialog";
import AddElasticSearchNode from "../../../dialogs/elasticsearch/addElasticSearchNode";
import SwitchElasticsearchNodeDialog from "../../../dialogs/elasticsearch/switchElasticsearchNodeDialog";
import InfoDialog from "../../../dialogs/infoDialog";
import cluster from "../../../assets/images/cluster.png";
import config, { clusterRoles, clusterDropModes, elasticsearchVersionOptions, types } from "../../../config";
import { TabMenu } from "primereact/tabmenu";
import { knobs } from "../../../util/templates";
import { Tag } from "primereact/tag";
import { ConfirmPopup } from "primereact/confirmpopup";
import { Paginator } from "primereact/paginator";
import UpgradeElasticSearchNodeDialog from "../../../dialogs/elasticsearch/upgradeElasticsearchClusterDialog";
import ChooseServiceDialog from "../../../dialogs/chooseServiceDialog";
import { OverlayPanel } from "primereact/overlaypanel";
import ElasticDropClusterDialog from "../../../dialogs/elasticsearch/dropClusterDialog";
import DropModeDialog from "../../../dialogs/dropModeDialog";


const ElasticsearchLayoutItem = (props) => {
    const navigate = useNavigate();
    const [showSwitchover, setShowSwitchover] = useState(false);
    const [showNodes, setShowNodes] = useState(true);
    const [infoIcon, setInfoIcon] = useState('pi pi-info-circle');
    const d3Container = useRef(null);
    const [activeIndex, setActiveIndex] = useState(0);
    const [nodeSelected, setNodeSelected] = useState(false);
    const [primarySelected, setPrimarySelected] = useState(false);
    const [attachedElasticNodes, setAttachedElasticNodes] = useState(null);
    const [foreignElasticNodes, setForeignElasticNodes] = useState(null);
    const currentNodeCoords = useRef({ x: 0, y: 0 });
    const tabRef = useRef(0);
    const inventory = useRef(null);
    const [showDeleteInventory, setShowDeleteInventory] = useState(false);
    const [warningMessage, setWarningMessage] = useState('');
    const [showWarning, setShowWarning] = useState(false);
    const leftContainer = useRef(null);
    const rightContainer = useRef(null);
    const tabsPos = useRef(null);
    const trashPos = useRef(null);
    const [elasticsearchIndices, setElasticsearchIndices] = useState(null);
    const [showDropIndex, setShowDropIndex] = useState(false);
    const [showSnapshot, setShowSnapshot] = useState(false);
    const [showAddNode, setShowAddNode] = useState(false);
    const [showAddIndex, setShowAddIndex] = useState(false);
    const [showDumpIndex, setShowDumpIndex] = useState(false);
    const [showRestoreIndex, setShowRestoreIndex] = useState(false);
    const [showDumpDetails, setShowDumpDetails] = useState(false);
    const [showSnapshotDetails, setShowSnapshotDetails] = useState(false);
    const [showDropNode, setShowDropNode] = useState(false);
    const [showRestoreSnapshotCluster, setShowRestoreSnapshotCluster] = useState(false);
    const [snapshotRow, setSnapshotRow] = useState(null);
    const [standbyRow, setStandbyRow] = useState(null);
    const [indexRow, setIndexRow] = useState(null);
    const [showCopyIndex, setShowCopyIndex] = useState(false);
    const [showStatus, setShowStatus] = useState(false);
    const [IndexNew, setIndexNew] = useState(null);
    const [isPhysical, setIsPhysical] = useState(true);
    const updateSchedule = useRef(null);
    const [expandSnapshot, setExpandSnapshot] = useState(null);
    const [databaseSource, setDatabaseSource] = useState("");
    const [exportRow, setExportRow] = useState(null);;
    const [showInfoDialog, setShowInfoDialog] = useState(false);
    const [infoMessage, setInfoMessage] = useState(false);
    const layoutItemId = useRef(null);
    const layoutCurrHeight = config.layoutMinHeight;
    const layoutMid = config.layoutMinHeight / 2;
    const [layoutSwitchIndex, setLayoutSwitchIndex] = useState(0);
    const [removeScheduleConfirm, setRemoveScheduleConfirm] = useState(false);
    const [pageIndex, setPageIndex] = useState(0);
    const [showUpgrade, setShowUpgrade] = useState(false);
    const [showChooseServiceDialog, setShowChooseServiceDialog] = useState(false);

    const rowsNumber = 6;
    const [first, setFirst] = useState(0);
    const [rows, setRows] = useState(rowsNumber);
    const [nodesPage, setNodesPage] = useState(props.node?.nodes?.length > rowsNumber ? props.node?.nodes?.slice(props.node?.nodes?.length - (rowsNumber + 1), props.node.nodes?.length - 1) : props.node?.nodes);
    const [databasesPage, setDatabasesPage] = useState(props.node?.databases?.length > 6 ? props.node?.databases?.slice(props.node?.databases?.length - (rowsNumber + 1), props.node.databases?.length - 1) : props.node?.databases);
    const node_x = 450;
    const clusterOptions = useRef(null);
    const [showDropCluster, setShowDropCluster] = useState(false);
    const [showDropMode, setShowDropMode] = useState(false);

    const thisNodes = useRef(null);
    const otherNodes = useRef(null);
    const thisDbs = useRef(null);
    const otherDbs = useRef(null);
    const thisBackups = useRef(null);
    const otherBackups = useRef(null);
    const thisExports = useRef(null);
    const otherExports = useRef(null);

    const onDropCluster = (data) => {
        dispatch(LayoutActions.dropCluster(data, setShowDropCluster));
    }



    useEffect(() => {

        if (props.node?.nodes?.length <= rowsNumber) setNodesPage(props.node?.nodes);
        if (props.node?.databases?.length <= rowsNumber) setDatabasesPage(props.node?.databases);

    }, [props.node])


    const onPageChange = (event) => {
        let rowsPerPage = rowsNumber;
        let startIndex = (event.page) * rowsPerPage;
        if (event.rows < rowsPerPage) {
            rowsPerPage = rowsPerPage - event.rows
            startIndex -= rowsPerPage
        }
        let endIndex = startIndex + rowsPerPage;
        setFirst(event.first);
        setRows(event.rows);
        if (isPhysical) setNodesPage(props.node?.nodes?.slice(startIndex, endIndex));
        else setDatabasesPage(props.node?.databases?.slice(startIndex, endIndex));

    };

    const { databases, elastic_backups, elastic_exports } = useSelector((state) => state.layout);

    const dispatch = useDispatch();

    const layoutOptions = [
        { label: 'Physical layout', className: isPhysical ? '' : 'p-disabled' },
        { label: 'Database layout', className: isPhysical ? 'p-disabled' : '' },
    ];


    const roleBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.ex_params?.roles}</div>);
    }

    const getAutomaticBackup = () => {
        let backup = props.node?.backups?.filter((b) => b.is_sched === true)?.[0]
        if (!backup) backup = elastic_backups?.filter(b => b?.cluster === props.node?.cluster && b.ex_params?.node_version === props.node.version && b?.is_sched === true)?.[0]
        updateSchedule.current = backup;
        const popup_id = `remove-schedule-button-${props.node.id}`
        return (<div draggable={false} className="info-section text-xs grid"><i className="pi pi-circle-fill" />  <div className="col-fixed" style={{ width: "300px" }}> Automatic backup: {backup?.ex_params?.description || "No"}</div> {backup ? <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-pencil" onClick={() => {
            setShowSnapshot(true);
        }}>change</Tag> <Tag className='right-container-button ml-2' id={popup_id} icon="pi pi-trash" severity="danger" onClick={() => setRemoveScheduleConfirm(true)} >delete</Tag>    <ConfirmPopup
                target={document.getElementById(popup_id)}
                visible={removeScheduleConfirm}
                onHide={() => setRemoveScheduleConfirm(false)}
                message="Are you sure?"
                accept={() => dispatch(LayoutActions.removeElasticsearchSchedule({ "node_id": props.node.id }, setRemoveScheduleConfirm))}
            /></div> : <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-plus" onClick={() => {

                setPageIndex(1);
                setShowSnapshot(true);
            }}>create</Tag> </div>}  </div>)
    }

    const onVerifyBackup = (open, message) => {

        setInfoMessage(message)
        setInfoIcon('pi pi-info-circle');
        setShowInfoDialog(open);

    }

    const onUpgrade = (data) => {
        dispatch(LayoutActions.upgradeElasticsearchNode(data, setShowUpgrade));
    }


    const showDbCredentialsInfo = (response) => {

        setInfoMessage('db_cred');
        setInfoIcon('pi pi-exclamation-triangle')
        setShowInfoDialog(response);

    }


    const exportDateBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} >{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>);
    }

    const exportHostBodyTemplate = (item) => {

        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag}
            onDragEnd={onDropzoneLeave}
            onDragStart={(e) => {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
                leftContainer.current = d3Container.current.getBoundingClientRect();
                e.dataTransfer.setData("dragType", "exportBar");
                inventory.current = item;
                window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                drawPrimaryDropzone();


            }}>
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{item?.host}</div>

        </div>
        );

    }

    const onDeleteInventory = (data) => {

        switch (inventory.current.type) {
            case "EXPORT":
                dispatch(LayoutActions.dropExport(data, setShowDeleteInventory));
                break;
            case "BACKUP":
                dispatch(LayoutActions.dropBackup(data, setShowDeleteInventory));
                break;

            default:
                break;

        }

    }

    const addImage = useCallback((x, y, node) => {

        function onClickNode(svg, id, node) {

            setPrimarySelected(false);
            props.selected.current = node;
            setNodeSelected(node);

            let elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute("slave_of") === "false" ? dbImage : node_avail : elements[i].getAttribute("slave_of") === "false" ? db_unavail : node_unavail);
            }


            elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }

            elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }


            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            svg.select("#node-" + id).attr("xlink:href", node.ind_deleted ? node_un_white : node_white);

            svg.select(`#text-${id}`).style("fill", "white");
        }

        function goNodeDetails(node) {
            if (node.cluster_role === clusterRoles.STANDBY) {

                if (node.ind_deleted) {
                    showWarningMessage({ "text": `${node.name} is no longer available.\nDetails cant be viewed.`, "status": node.status })

                }
                else navigate(`/status/elastic/${node.id}`);

            }


        }

        const onDragStart = (svg, id, node) => (d) => {
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();

            onClickNode(svg, id, node);

        }


        const onDragging = (svg, id, node) => (d) => {

            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }

            if (d.sourceEvent.clientX < leftContainer.current.left || d.sourceEvent.clientY > leftContainer.current.bottom || d.sourceEvent.clientY < tabsPos.current.top || d.sourceEvent.clientX > tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            setNodeSelected(node);
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#node-" + id).style("cursor", "grabbing");
                d3.select("#node-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx) + "," + (dy) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 51) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", "translate(" + (dx + 40) + "," + (dy + 10) + ")");

                if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                    svg.select(".trash").style("opacity", "1.0");

                }
                else {

                    svg.select(".trash").style("opacity", "0.5");

                }


                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                    switch (tabRef.current) {
                        case 0:
                        case 2:
                            svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                        case 3:
                            if (isPhysical === false) svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                        case 1:
                            break;
                        default:
                            break;
                    }
                }

                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 125) {
                    setActiveIndex(0);
                    tabRef.current = 0;
                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 125 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 250) {
                    setActiveIndex(1);
                    tabRef.current = 1;
                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 250 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 375) {
                    setActiveIndex(2);
                    tabRef.current = 2;

                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 375 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 500) {
                    setActiveIndex(3);
                    tabRef.current = 3;

                }
            }

            let elements = window.document.querySelectorAll(`.elasticsearch-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = tabRef.current === 3 && (!isPhysical) ? "0 0 0 0.2rem rgb(220 53 69 / 72%)" : tabRef.current === 0 || tabRef.current === 2 ? "" : "0 0 0 0.2rem rgb(40 167 69 / 72%)";

            }
        }

        const onDragEnd = (svg, id, node) => (d) => {

            if (d.sourceEvent.detail === 2 && (node.cluster_role === clusterRoles.STANDBY)) {

                goNodeDetails(node);
                return;

            }

            setNodeSelected(node);
            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                if (node.ind_deleted) {


                    showWarningMessage({

                        "text": `${node.name.toUpperCase()} is unavailable`,
                        "status": `${node.status}`
                    });
                }
                else {

                    inventory.current = node;

                    if (node.cluster_role === clusterRoles.STANDBY) setShowDropNode(true);
                    else setShowDropIndex(true);




                }

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {


                if (node.ind_deleted) {


                    showWarningMessage({

                        "text": `${node.name.toUpperCase()} is unavailable`,
                        "status": `${node.status}`
                    });
                }
                else {

                    if (node.cluster_role === clusterRoles.STANDBY) {

                        switch (tabRef.current) {
                            case 3:

                                setPageIndex(0);
                                setShowSnapshot(true);
                                break;
                            default:
                                break;

                        }
                    }
                    else {

                        switch (tabRef.current) {

                            case 1:
                                if (node.ind_deleted)
                                    showWarningMessage({

                                        "text": `${props.node.name.toUpperCase()} is unavailable`,
                                        "status": `${props.node.status}`
                                    });
                                else setShowDumpIndex(true);
                                break;
                            default:
                                break;

                        }

                    }


                }

            }

            d3.select("#node-" + id).remove();
            d3.select("#line-" + id).remove();
            d3.select("#text-" + id).remove();
            d3.select("#severity-" + id).remove();
            d3.select("#d3-tooltip-container-" + id).remove();

            let i = 0;
            if (isPhysical === true) {

                if (nodesPage?.length > 0) {

                    for (; i < nodesPage?.length; ++i)
                        if (nodesPage[i].id === node.id)
                            break;

                    const node_y = (i + 0.5) * (layoutCurrHeight / nodesPage.length);
                    currentNodeCoords.current.x = node_x;
                    currentNodeCoords.current.y = node_y + 20;
                    addImage(node_x, node_y, node);

                }

            }
            else if (isPhysical === false) {

                if (databasesPage?.length > 0) {

                    for (; i < databasesPage?.length; ++i)
                        if (databasesPage[i].id === node.id)
                            break;
                    const database_y = (i + 0.5) * (layoutCurrHeight / databasesPage.length);
                    currentNodeCoords.current.x = node_x;
                    currentNodeCoords.current.y = database_y;
                    addImage(node_x, database_y, node);
                }
            }




            svg.select(".trash").style("opacity", "0.0");
            d3.select("#node-" + id).style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.elasticsearch-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        const svg = d3.select(d3Container.current);
        const id = nanoid();

        const image = svg.insert("image")
            .attr("xlink:href", (props.selected.current?.id === node.id) ? node.ind_deleted ? node_un_white : node_white : node.ind_deleted ? node_unavail : node_avail)
            .attr("class", "node")
            .attr("id", "node-" + id)
            .attr('x', 0)
            .attr('y', 0)
            .attr("width", 40)
            .attr("height", 60)
            .attr("available", node.ind_deleted ? false : true)
            .attr("draggable", true)
            .attr("transform", () => "translate(" + x + "," + (y - 20) + ")")
            .on("mouseover", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

            })
            .on("mouseout", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

            })

        const x0 = 100;
        const y0 = layoutMid;
        const x1 = x;
        const y1 = y;

        const path = d3.path()
        path.moveTo(x1, y1)
        path.bezierCurveTo((x0 + x1) / 2, y1, (x0 + x1) / 2, y0, x0, y0);

        svg.insert("path")
            //.attr("d", d3.line()([[100, 220], [x, y + 30]]))
            .attr("d", path)
            .attr("id", "line-" + id).attr("transform", "translate(" + (0) + "," + (10) + ")")
            .style("fill", "none")
            .style("stroke", "#ccc")
            .style("stroke-width", 1);

        svg.insert("text")
            .attr("transform", "translate(" + (x) + "," + (y - 14) + ")")
            .attr("id", "text-" + id)
            .attr("class", "node-what-text")
            .style("font-size", "12px")
            .style("fill", `${props.selected.current?.id === node.id ? "white" : "gray"}`)
            .style("cursor", "pointer")
            .text(`${node.name ? node.name : ""} ${node.status ? node.status : ""}`).on("dblclick", () => goNodeDetails(node)).on("click", function (d) {
                currentNodeCoords.current.x = x1;
                currentNodeCoords.current.y = y1 + 20;
                onClickNode(svg, id, node);
            });

        //draw node severity strip
        svg.insert("rect").attr('x', 0)
            .attr('y', 0)
            .attr("width", 35)
            .attr("height", 5).attr("fill", getColorBySeverity(node.severity))
            .attr("transform", "translate(" + (x1 + 2) + "," + (y1 + 31) + ")")
            .attr("id", "severity-" + id)

        svg.insert("foreignObject")
            .attr("class", "d3-tooltip-container")
            .attr("id", `d3-tooltip-container-${id}`)
            .attr("transform", "translate(" + (x + 40) + "," + (y + 10) + ")")
            .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${node.ind_started ? "up" : "down"} <br/> Alerts: ${node?.alert?.length > 40 ? node.alert.substring(0, 37).concat("...") : node?.alert} </div>`)

        var drag_behavior = d3.drag()
            .on("start", onDragStart(svg, id, node))
            .on("drag", onDragging(svg, id, node))
            .on("end", onDragEnd(svg, id, node));

        image.call(drag_behavior);
    }, [navigate, props.node, props.selected, layoutCurrHeight, layoutMid, nodesPage, databasesPage, isPhysical]);




    useEffect(() => {


        const onTableItemDrop = (event) => {
            let type = event.dataTransfer.getData("dragType");

            switch (type) {
                case "nodeBar":
                    if (inventory.current?.ind_deleted) {


                        showWarningMessage({

                            "text": `${inventory.current.name.toUpperCase()} is unavailable`,
                            "status": `${inventory.current.status}`
                        });
                    }
                    else {
                        if (inventory.current?.cluster_role === clusterRoles.PRIMARY || props.node?.nodes?.length > 1) setShowDropMode(true);
                        else setShowDropNode(true);
                    }
                    break;
                case "indexBar":
                    if (inventory?.current.ind_deleted) {


                        showWarningMessage({

                            "text": `${inventory.current.name.toUpperCase()} is unavailable`,
                            "status": `${inventory.current.status}`
                        });
                    }
                    else setShowDropIndex(true);
                    break;
                case "backupBar":
                    setShowDeleteInventory(true);
                    break;
                case "dumpBar":
                    setShowDeleteInventory(true);
                    break;
                default:
                    break;


            }

        }

        function goPrimaryDetails() {
            if (props.node.ind_deleted) {

                showWarningMessage({ "text": `${props.node.name} is no longer available.\nDetails cant be viewed.`, "status": props.node.status })

            }
            else navigate(`/status/elastic/${props.node.id}`);

        }


        function onPrimaryClick(svg, id) {

            setNodeSelected(false);

            let elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }

            elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }

            elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute("slave_of") === "false" ? dbImage : node_avail : elements[i].getAttribute("slave_of") === "false" ? db_unavail : node_unavail);
            }


            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }


            svg.select(".root").attr("xlink:href", props.node.ind_deleted ? db_un_white : db_white);
            props.selected.current = props.node;
            setPrimarySelected(true);
            svg.select(`#text-${id}`).style("fill", "white");


        }


        const onDragStartPrimary = (svg, id, primary) => (d) => {
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();

            // Draw primary what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 10) + ")")
                .attr("id", "text-copy" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node.id ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.name ? props.node.name : ""} ${props.node.status ? props.node.status : ""}`);

            // Draw primary icon
            svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id) ? props.node.ind_deleted ? db_un_white : db_white : props.node.ind_deleted ? db_unavail : dbImage)
                .attr("class", "root")
                .attr("id", "root-copy" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + layoutMid + ")")
                .attr("available", !props.node.ind_deleted);

            //draw primary severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-copy" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 40) + ")")


            // Draw primary plus/minus icon

            if ((props.node.nodes?.length > 0 && isPhysical === true) || (props.node?.databases?.length > 0 && isPhysical === false)) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-copy" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 5) + ")");
            }

            onPrimaryClick(svg, id)

        }

        const onDraggingPrimary = (svg, id, primary) => (d) => {

            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }

            if (d.sourceEvent.clientX < leftContainer.current.left || d.sourceEvent.clientY > leftContainer.current.bottom || d.sourceEvent.clientY < tabsPos.current.top || d.sourceEvent.clientX > tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#root-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx - 54) + "," + (dy - 8) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 40) + ")");
                d3.select("#expand-" + id).attr("transform", () => "translate(" + (dx + 30) + "," + (dy - 5) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", () => "translate(" + (dx + 50) + "," + (dy - 5) + ")");
            }

            svg.select(".root").style("cursor", "grabbing");
            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                svg.select(".trash").style("opacity", "1.0");

            }
            else {

                svg.select(".trash").style("opacity", "0.5");

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                switch (tabRef.current) {
                    case 0:
                    case 2:
                        svg.select(".root").style("cursor", "not-allowed");
                        break;
                    default:
                        break;
                }
            }



            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 125) {
                setActiveIndex(0);
                tabRef.current = 0;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 125 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 250) {
                setActiveIndex(1)
                tabRef.current = 1;
            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 250 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 375) {
                setActiveIndex(2);
                tabRef.current = 2;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 375 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 500) {
                setActiveIndex(3);
                tabRef.current = 3;


            }

            let elements = window.document.querySelectorAll(`.elasticsearch-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = tabRef.current === 0 || tabRef.current === 2 ? "" : "0 0 0 0.2rem rgb(40 167 69 / 72%)";

            }
        }

        const onDragEndPrimary = (svg, id, primary) => (d) => {

            if (d.sourceEvent.detail === 2) {

                goPrimaryDetails();
                return;

            }

            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                if (props.node.ind_deleted)
                    showWarningMessage({

                        "text": `${props.node.name.toUpperCase()} is unavailable`,
                        "status": `${props.node.status}`
                    });

                else {
                    if (props.node.nodes?.length > 0) showWarningMessage({ "text": `This is primary with nodes, cannot be removed.\nSwitchover to master-eligible node first.` })
                    else {

                        inventory.current = props.node
                        setShowDropNode(true);

                    }

                }

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {


                switch (tabRef.current) {
                    case 3:
                        if (props.node.ind_deleted)
                            showWarningMessage({

                                "text": `${props.node.name.toUpperCase()} is unavailable`,
                                "status": `${props.node.status}`
                            });
                        else {


                            setPageIndex(0);
                            setShowSnapshot(true);

                        }
                        break;
                    default:
                        break;

                }

            }



            svg.select(".trash").style("opacity", "0.0");

            d3.select("#root-copy" + id).remove();
            d3.select("#text-copy" + id).remove();
            d3.select("#severity-copy" + id).remove();
            d3.select("#expand-copy" + id).remove();

            d3.select("#root-" + id).attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")");
            d3.select("#text-" + id).attr("transform", () => "translate(" + 0 + "," + (layoutMid - 18) + ")");
            d3.select("#severity-" + id).attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")");
            d3.select("#expand-" + id).attr("transform", "translate(" + (80) + "," + (layoutMid - 15) + ")");
            d3.select(`#d3-tooltip-container-${id}`).attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
            svg.select(".root").style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.elasticsearch-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        if (d3Container.current) {
            const svg = d3.select(d3Container.current);
            svg.selectChildren("*").remove();

            const id = nanoid();



            if (!props.selected.current) {
                setPrimarySelected(props.node);

            }

            svg.insert("foreignObject")
                .attr("class", "d3-tooltip-container")
                .attr("id", `d3-tooltip-container-${id}`)
                .attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
                .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${props.node.ind_started ? "up" : "down"} <br/> Alerts:  ${props.node?.alert?.length > 40 ? props.node.alert.substring(0, 37).concat("...") : props.node?.alert} </div>`)
            // Draw trash icon    
            svg.insert("image")
                .attr("xlink:href", trashImage)
                .attr("class", "trash")
                .attr("id", `trash-${id}`)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 100)
                .attr("height", 100)
                .attr("transform", () => "translate(" + 0 + "," + (layoutCurrHeight - 100) + ")")
                .on("dragover", dragOverTrash)
                .on("dragleave", dragLeaveTrash)
                .on("drop", onTableItemDrop);


            // Draw primary what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 18) + ")")
                .attr("id", "text-" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node.id || !layoutItemId.current ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.name ? props.node.name : ""} ${props.node.status ? props.node.status : ""}`)
                .on("dblclick", goPrimaryDetails)
                .on("click", () => onPrimaryClick(svg, id));

            // Draw primary icon
            const imagePrimary = svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id || !layoutItemId.current) ? props.node.ind_deleted ? db_un_white : db_white : props.node.ind_deleted ? db_unavail : dbImage)
                .attr("class", "root")
                .attr("id", "root-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")")
                .attr("available", !props.node.ind_deleted)
                .on("mouseover", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

                })
                .on("mouseout", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

                });

            layoutItemId.current = id;

            const dragBehaviorPrimary = d3.drag()
                .on("start", onDragStartPrimary(svg, id, props.node))
                .on("drag", onDraggingPrimary(svg, id, props.node))
                .on("end", onDragEndPrimary(svg, id, props.node));
            imagePrimary.call(dragBehaviorPrimary);


            //draw primary severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")")



            // Draw primary plus/minus icon

            if ((props.node.nodes?.length > 0 && isPhysical === true) || (props.node?.databases?.length > 0 && isPhysical === false)) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 15) + ")")
                    .on("click", () => { setShowNodes(!showNodes) });

                if (showNodes) {

                    if (isPhysical === true) {

                        if (nodesPage?.length > 0) {

                            nodesPage.forEach((node, index) => {
                                addImage(node_x, ((index + 0.5) * (layoutCurrHeight / nodesPage.length)), node);
                            });

                        }




                    }
                    else if (isPhysical === false) {

                        if (databasesPage?.length > 0) {

                            databasesPage.forEach((database, index) => {
                                addImage(node_x, ((index + 0.5) * (layoutCurrHeight / databasesPage.length)), database);
                            });

                        }

                    }

                }
            }
        }
    }, [showNodes, props.node, navigate, isPhysical, layoutCurrHeight, layoutMid, nodesPage, databasesPage, props.selected, addImage, dispatch]);


    useEffect(() => {

        if (props.node) {
            let tempData = []
            if (props.node.nodes?.length > 0) tempData = [props.node, ...props.node.nodes];
            else tempData = [props.node];

            setAttachedElasticNodes(tempData);
        }

    }, [props.node]);


    useEffect(() => {

        let tempData = [];
        if (databases?.length > 0) tempData = databases?.filter((d) => d.type === types.ELASTIC);
        setElasticsearchIndices(tempData);

    }, [databases]);

    useEffect(() => {

        let tempData = []
        if (props.selection?.length > 0) {

            props.selection?.filter((p) => p.cluster !== props.node.cluster)?.forEach((p) => {
                tempData.push(p);
                if (p.nodes?.length > 0) p.nodes.forEach((n) => tempData.push(n));
            })
        }
        setForeignElasticNodes(tempData);


    }, [props.selection, props.node]);

    useEffect(() => {

        const updateSelected = () => {

            if (props.selected.current?.type === "elasticsearch_db") {

                if (props.selected.current?.id === props.node?.id) {

                    let nodeSelected = props.selection?.filter((n) => n.id === props.selected.current?.id);
                    if (nodeSelected?.length > 0) {
                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }

                }
                else {
                    let nodeSelected;
                    if (isPhysical === true) {
                        nodeSelected = props.node?.nodes?.filter((n) => n.id === props.selected.current?.id);
                    }
                    else {
                        if (isPhysical === false) {
                            nodeSelected = props.node.databases?.filter((n) => n.id === props.selected.current?.id);
                        }

                    }
                    if (nodeSelected?.length > 0) {

                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }
                }


            }

        }


        if (props.selection?.length > 0) updateSelected();

    }, [props.node, isPhysical, props.selection, props.selected])

    function dragToPrimaryEnd(event) {
        let type = event.dataTransfer.getData("dragType");

        if (props.node.ind_deleted) {
            showWarningMessage({

                "text": `${props.node.name.toUpperCase()} is unavailable`,
                "status": `${props.node.status}`
            });

            return;

        }

        if (inventory.current?.status === false) {
            showWarningMessage({

                "text": "Cannot perform this action",
                "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
            });

            return;



        }

        switch (type) {
            case "indexBar":
                let source = event.dataTransfer.getData("dragSource");
                setDatabaseSource(source);
                dispatch(LayoutActions.checkElasticsearchIndexExists({ "name": inventory.current?.name, "node_id": props.node?.id }, ShowCopyIndex));
                break;
            case "backupBar":
                setShowRestoreSnapshotCluster(true);
                break;
            case "exportBar":
                setShowRestoreIndex(true);
                break;
            case "indexCreate":
                setShowAddIndex(true);
                break;
            case "nodeCreate":
                props.services?.length < 1 ? setShowAddNode(true) : setShowChooseServiceDialog(true);
                break;
            default:
                break;



        }






    }


    const onDumpIndex = (data) => {

        dispatch(LayoutActions.dumpElasticsearchIndex(data, setShowDumpIndex));
        dispatch(LayoutActions.setDirExists(null));
        dispatch(LayoutActions.setExportExists(null));
        dispatch(LayoutActions.setDiskSpace(null));

    }


    // function dragToNodeEnd(event) {

    //     if (nodeSelected.ind_deleted && nodeSelected?.cluster_role) {
    //         showWarningMessage({

    //             "text": `${props.node.name.toUpperCase()} is unavailable`,
    //             "status": `${props.node.status}`
    //         });

    //         return;

    //     }

    //     if (inventory.current?.status === false) {
    //         showWarningMessage({

    //             "text": "Cannot perform this action",
    //             "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
    //         });

    //         return;



    //     }

    //     let type = event.dataTransfer.getData("dragType");

    //     if (isPhysical === false) {

    //         switch (type) {

    //             case "dumpBar":
    //                 setShowRestoreIndex(true);
    //                 break;
    //             default:
    //                 break;

    //         }


    //     }

    // }

    function dragOverPrimary(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    }

    // function dragOverNode(event) {
    //     event.preventDefault();
    //     d3.select(d3Container.current)
    //         .select(".drop-zone-marker")
    //         .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    // }

    function dragLeaveNode(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", "transparent");

    }

    const dragLeaveTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.5");

    }

    const dragOverTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "1.0");

    }

    const snapshotDateBodyTemplate = (item) => {
        return (
            <div className="flex align-items-center" draggable onDrag={onTableItemDrag}
                onDragEnd={onDropzoneLeave}
                onDragStart={(e) => {
                    leftContainer.current = d3Container.current.getBoundingClientRect();
                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                    d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
                    e.dataTransfer.setData("dragType", "backupBar");
                    window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                    inventory.current = item;
                    drawPrimaryDropzone();

                }}>
                {knobs}
                <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>
            </div>
        );
    }


    // const drawNodeDropzone = () => {
    //     d3.select(d3Container.current).insert("rect")
    //         .attr("class", "drop-zone-marker")
    //         .attr("x", currentNodeCoords.current.x - 30)
    //         .attr("y", currentNodeCoords.current.y - 40)
    //         .attr("width", 100)
    //         .attr("height", 100)
    //         .attr("fill", "transparent")
    //         .attr("stroke", inventory.current?.status === false ? "red" : "green")
    //         .on("drop", dragToNodeEnd)
    //         .on("dragover", dragOverNode)
    //         .on("dragleave", dragLeaveNode);
    // }



    const sizeBodyTemplate = (item) => {
        return (<div className="flex align-items-center icon-cell backup">{item.display_size} <i className="pi pi-check verify-backup" onClick={() => {

            dispatch(LayoutActions.verifyBackup({ "name": item?.name, "type": item?.type, "backup_host": item?.host_name, "backup_path": item?.ex_params?.["backup_path"], "backup_date": item?.ex_params?.["modified_date"] }, onVerifyBackup))

        }} /> <img className="bar-icon" draggable={false} alt="backupBar" src={bar} onClick={() => {
            inventory.current = item;
            switch (item.type) {

                case "BACKUP":
                    setShowSnapshotDetails(true);
                    break;
                case "EXPORT":
                    setShowDumpDetails(true);
                    break;
                default:
                    break;

            }

        }} /></div>);
    }

    const lagBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} >{item.ex_params?.lag}</div>);
    }

    const portBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.port}</div>);
    }

    const ipBodyTemplate = (item) => {
        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            leftContainer.current = d3Container.current.getBoundingClientRect();
            d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
            e.dataTransfer.setData("dragType", "nodeBar");
            inventory.current = item;
        }} >
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item.severity)}` }}>{item.host}</div>
        </div>);
    }

    const indexHostBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item?.host}</div>);
    }

    const indexSizeBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item?.display_size}</div>);
    }


    const indexNameBodyTemplate = (item, tableData) => {
        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            leftContainer.current = d3Container.current.getBoundingClientRect();
            d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
            e.dataTransfer.setData("dragType", "indexBar");
            if (tableData.props.className === "backup-expandable-table") e.dataTransfer.setData("dragSource", "backup-expandable");
            else e.dataTransfer.setData("dragSource", "indices");
            inventory.current = item;
            drawPrimaryDropzone();
        }}>
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item.severity)}` }}>{tableData.props.className === "backup-expandable-table" ? `Database: ${item.name}` : item.name}</div>
        </div>);

    }

    const onDropzoneLeave = (e) => {
        e.target.style.position = "";
        e.target.style.left = "";
        e.target.style.top = "";
        e.target.style.visibility = ""; d3.selectAll(".drop-zone-marker").remove();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.0");
    }

    const drawPrimaryDropzone = () => {

        d3.select(d3Container.current).insert("rect")
            .attr("class", "drop-zone-marker")
            .attr("x", 20)
            .attr("y", layoutMid - 50)
            .attr("width", 100)
            .attr("height", 100)
            .attr("fill", "transparent")
            .attr("stroke", inventory.current?.status === false ? "red" : "green")
            .on("drop", dragToPrimaryEnd)
            .on("dragover", dragOverPrimary)
            .on("dragleave", dragLeaveNode);

    }

    const snapshotExpansion = () => {
        return (
            <div className={((nodeSelected && nodeSelected?.backups?.length > 1) || props.node.backups?.length > 1) ? "expansion-body" : "expansion-body single"}>
                <DataTable
                    className="backup-expandable-table severity-table"
                    selectionMode="singlef"
                    dataKey="id"
                    selection={indexRow}
                    onSelectionChange={(e) => setIndexRow(e.value)}
                    scrollable
                    scrollHeight="flex"
                    emptyMessage="no snapshots found"
                    paginator={props.node?.databases?.length > 0}
                    rows={layoutCurrHeight === config.layoutMinHeight ? 10 : 50}
                    alwaysShowPaginator={false}
                    value={props.node?.databases}>
                    <Column style={{ height: "21px" }} field="name" body={indexNameBodyTemplate}></Column>
                    <Column style={{ height: "21px" }} field="size" body={indexSizeBodyTemplate}></Column>
                </DataTable>
            </div>
        );
    }

    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onTableItemDrag = (e) => {
        e.preventDefault();
        e.target.style.position = "fixed";
        e.target.style.top = e.clientY + "px";
        e.target.style.left = e.clientX + "px";
        if (e.clientX < leftContainer.current.left || e.clientY < leftContainer.current.top || e.clientY > leftContainer.current.bottom) {

            e.target.style.visibility = "hidden";

        }
        else {

            e.target.style.visibility = "";

        }
    }

    const onDropNode = (data) => {
        dispatch(LayoutActions.dropElasticsearchNode(data, setShowDropNode));
    }

    const onSnapshotCluster = (data) => {
        dispatch(LayoutActions.snapshotElasticsearchCluster(data, setShowSnapshot));
    }

    const onAddElasticSearchNode = (data) => dispatch(LayoutActions.addElasticsearchNode(data, setShowAddNode));

    const onAddIndex = (data) => {
        dispatch(LayoutActions.addElasticsearchIndex(data, showDbCredentialsInfo));

    }

    const onDropIndex = (data) => {

        dispatch(LayoutActions.dropElasticsearchIndex(data, setShowDropIndex));

    }

    const onRestoreSnapshotCluster = (data) => {
        dispatch(LayoutActions.restoreElasticsearchCluster(data, setShowRestoreSnapshotCluster));

    }

    const onRestoreIndex = (data) => {

        dispatch(LayoutActions.restoreElasticsearchIndex(data, setShowRestoreIndex));
    }


    const onCopyIndex = (data) => {
        dispatch(LayoutActions.copyElasticsearchIndex(data, setShowCopyIndex));
    }

    const ShowCopyIndex = (newdb, open) => {

        setIndexNew(newdb);
        setShowCopyIndex(open);
    }

    const onSwitchover = (data) => {
        dispatch(LayoutActions.switchElasticsearchNode(data, setShowSwitchover));
    }

    return (
        <div className="layout-item">
            <div className="layout-item-wrapper">
                <div className="flex flex-column flex-grow-1">

                    <div className="left-title flex flex-row">

                        <TabMenu className="layout-switch" model={layoutOptions} activeIndex={layoutSwitchIndex} onTabChange={(e) => {
                            dispatch(LayoutActions.refreshContainer(props?.node?.id));
                            const index = e.index;
                            setIsPhysical(index === 0);
                            setLayoutSwitchIndex(index);
                        }} />
                        <div className="ml-auto mr-auto">{`${props.node.what} cluster: ${props.node?.cluster_content?.name}`}</div>
                        <div className="ml-auto">

                            <Button
                                style={{ fontSize: "1rem" }}
                                className="p-button-sm p-button-text"
                                label="status"
                                onClick={() => navigate('/status', { state: { cluster_id: props.node?.cluster, node_id: props.node?.id } })}
                            />

                            {Number(props.node?.version) < Number(elasticsearchVersionOptions[0].value) ?

                                <Button
                                    style={{ fontSize: "1rem" }}
                                    className="p-button-sm p-button-text"
                                    label="upgrade"
                                    onClick={() => setShowUpgrade(true)}
                                /> : null

                            }

                            {props.node?.nodes?.length > 0 ?

                                <Button
                                    style={{ fontSize: "1rem" }}
                                    className="p-button-sm p-button-text"
                                    label="cluster options"
                                    onClick={(e) => clusterOptions.current?.show(e)}
                                /> : null}


                            <OverlayPanel ref={clusterOptions} onMouseLeave={(e) => clusterOptions.current?.hide(e)} closeOnEscape>

                                <div className="flex flex-column">

                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        label="drop"
                                        onClick={() => setShowDropCluster(true)}
                                    />


                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        disabled={props.node?.nodes?.filter((n) => n.severity === "green" && n.ex_params?.roles?.includes('master') && !n.ex_params?.roles?.includes('voting_only'))?.length < 1}
                                        label="switchover"
                                        onClick={() => setShowSwitchover(true)}
                                    />
                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        label="info"
                                        onClick={() => setShowStatus(true)}
                                    />


                                </div>

                            </OverlayPanel>

                            <Button
                                style={{ fontSize: "1rem" }}
                                className="p-button-sm p-button-text"
                                label="info"
                                onClick={() => setShowStatus(true)}
                            />
                        </div>

                    </div>

                    <div style={{ zIndex: 1000 }} className="left-container">

                        <div className="left-content">

                            <svg
                                className="d3-component"
                                style={{ width: "800px", maxHeight: `${rightContainer.current?.offsetHeight > layoutCurrHeight ? rightContainer.current.offsetHeight + 5 : layoutCurrHeight + 5}px`, minHeight: `${layoutCurrHeight + 5}px` }}
                                ref={d3Container}
                            />
                            {(isPhysical && props.node?.nodes?.length > rowsNumber) || (!isPhysical && props.node?.databases?.length > rowsNumber) ? <Paginator className="left-container-paginator" first={first} rows={rows} totalRecords={isPhysical ? props.node?.nodes?.length : props.node?.databases?.length} onPageChange={onPageChange} /> : null}
                        </div>
                    </div>

                </div>

                <div className="right-container" style={{ minHeight: `${layoutCurrHeight + 5}px` }}
                    ref={rightContainer}>
                    <TabView style={{ width: "500px" }} activeIndex={activeIndex} onTabChange={(e) => {
                        setActiveIndex(e.index);
                        tabRef.current = e.index
                    }}>
                        <TabPanel header="Nodes">
                            <div className={`elasticsearch-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "nodeCreate");
                                    drawPrimaryDropzone();
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on Master to create a Node</div>
                                </div>
                                <Panel ref={thisNodes} className="collapsible-panel mt-4" header={<div onClick={() => thisNodes?.current?.toggle()} className="cursor-pointer"> Nodes of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel>
                                        <DataTable
                                            className="severity-table"
                                            emptyMessage="no nodes found"
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={standbyRow}
                                            onSelectionChange={(e) => setStandbyRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={attachedElasticNodes}>
                                            <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                            <Column style={{ height: "21px" }} field="port" body={portBodyTemplate} header="Port" sortable></Column>
                                            <Column style={{ height: "21px" }} field="ex_params.lag" header="Lag" body={lagBodyTemplate} sortable></Column>
                                            <Column style={{ height: "21px" }} field="cluster_role" header="Role" body={roleBodyTemplate} sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>
                                <Panel ref={otherNodes} collapsed className="collapsible-panel" header={<div onClick={() => otherNodes?.current?.toggle()} className="cursor-pointer">Nodes of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel>
                                        <DataTable
                                            className="severity-table"
                                            emptyMessage="no nodes found"
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={standbyRow}
                                            onSelectionChange={(e) => setStandbyRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={foreignElasticNodes}>
                                            <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                            <Column style={{ height: "21px" }} field="port" body={portBodyTemplate} header="Port" sortable></Column>
                                            <Column style={{ height: "21px" }} field="ex_params.lag" header="Lag" body={lagBodyTemplate} sortable></Column>
                                            <Column style={{ height: "21px" }} field="cluster_role" header="Role" body={roleBodyTemplate} sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>
                            </div>
                        </TabPanel>
                        <TabPanel header="Export / Import Index">
                            <div className={`elasticsearch-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> Drag any Index here to start an export</div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />  Drag from the list onto a Primary to start import</div>
                                {(nodeSelected) && !isPhysical && <div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}> {`Exports for this index will be stored for ${nodeSelected?.size_scheme?.export} days`}</div> <div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: types.ELASTIC } })}>change</Tag> </div> </div>}
                                <Panel ref={thisExports} className="collapsible-panel mt-4" header={<div onClick={() => thisExports?.current?.toggle()} className="cursor-pointer"> Exports of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            selectionMode="single"
                                            className="severity-table"
                                            dataKey="id"
                                            selection={exportRow}
                                            onSelectionChange={(e) => setExportRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            emptyMessage="no exports found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={elastic_exports?.filter((e) => e.ex_params?.node_version === props.node.version && e.cluster === props.node.cluster)}>
                                            <Column sortable style={{ height: "21px" }} field="host" header="Host" body={exportHostBodyTemplate}></Column>
                                            <Column sortable style={{ height: "21px" }} field="database" header="Database" body={(item) => item?.database ? item?.node_name : 'All databases'}></Column>
                                            <Column style={{ height: "21px" }} field="created_date" body={exportDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                                <Panel  ref={otherExports}  collapsed className="collapsible-panel" header={<div onClick={() => otherExports?.current?.toggle()} className="cursor-pointer"> Exports of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            selectionMode="single"
                                            className="severity-table"
                                            dataKey="id"
                                            selection={exportRow}
                                            onSelectionChange={(e) => setExportRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            emptyMessage="no exports found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={elastic_exports?.filter((e) => e.ex_params?.node_version === props.node.version && e.cluster !== props.node.cluster)}>
                                            <Column sortable style={{ height: "21px" }} field="host" header="Host" body={exportHostBodyTemplate}></Column>
                                            <Column sortable style={{ height: "21px" }} field="database" header="Database" body={(item) => item?.database ? item?.node_name : 'All databases'}></Column>
                                            <Column style={{ height: "21px" }} field="created_date" body={exportDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                            </div>
                        </TabPanel>
                        <TabPanel header="Indices">
                            <div className={`elasticsearch-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs mb-2" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "indexCreate");
                                    drawPrimaryDropzone();
                                }}>
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on Master to create Index</div>
                                </div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />   Drag from the list onto a Primary to start copying</div>
                                <Panel ref={thisDbs} className="collapsible-panel mt-4" header={<div onClick={() => thisDbs?.current?.toggle()} className="cursor-pointer"> Indexes of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className="severity-table"
                                            selectionMode="single"
                                            emptyMessage="no indices found"
                                            dataKey="id"
                                            selection={indexRow}
                                            onSelectionChange={(e) => setIndexRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={elasticsearchIndices?.filter((d) => d.cluster === props.node.cluster)}>
                                            <Column style={{ height: "21px" }} field="name" body={indexNameBodyTemplate} header="Name" sortable></Column>
                                            <Column style={{ height: "21px" }} field="host" body={indexHostBodyTemplate} header="Host" sortable></Column>
                                            <Column style={{ height: "21px" }} field="size" body={indexSizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>

                                <Panel ref={otherDbs} className="collapsible-panel" collapsed header={<div  onClick={() => otherDbs?.current?.toggle()} className="cursor-pointer"> Indexes of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className="severity-table"
                                            selectionMode="single"
                                            emptyMessage="no indices found"
                                            dataKey="id"
                                            selection={indexRow}
                                            onSelectionChange={(e) => setIndexRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={elasticsearchIndices?.filter((d) => d.cluster !== props.node.cluster)}>
                                            <Column style={{ height: "21px" }} field="name" body={indexNameBodyTemplate} header="Name" sortable></Column>
                                            <Column style={{ height: "21px" }} field="host" body={indexHostBodyTemplate} header="Host" sortable></Column>
                                            <Column style={{ height: "21px" }} field="size" body={indexSizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>
                            </div>

                        </TabPanel>
                        <TabPanel header="Snapshots">
                            <div className={`elasticsearch-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> {nodeSelected || primarySelected ? `Drag snapshot to ${nodeSelected ? nodeSelected.name : props.node.name} to restore:` : "Select node to view snapshots:"} </div>
                                {(nodeSelected || primarySelected) && <div draggable={false} className="info-section text-xs grid"><i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}>{`Snapshots for this cluster will be stored for ${primarySelected ? props.node?.size_scheme?.backup : nodeSelected?.size_scheme?.backup} days`}</div> <div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: "ELASTIC" } })}>change</Tag> </div> </div>}
                                {nodeSelected || primarySelected ? getAutomaticBackup() : null}
                                <Panel ref={thisBackups} className="collapsible-panel verbose mt-4" header={<div onClick={() => thisBackups?.current?.toggle()} className="cursor-pointer">Snapshots of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={snapshotRow}
                                            onSelectionChange={(e) => setSnapshotRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={elastic_backups?.filter((b) => b.ex_params?.node_version === props.node.version && b.cluster === props.node.cluster)}
                                            expandedRows={expandSnapshot}
                                            emptyMessage="no snapshots found"
                                            rowExpansionTemplate={snapshotExpansion}
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            onRowToggle={(e) => {
                                                setExpandSnapshot(e.data)
                                            }}>
                                            <Column style={{ height: '21px' }} field="created_date" body={snapshotDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                            <Column expander style={{ height: '21px', width: "0.75rem" }} />
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                                <Panel ref={otherBackups} collapsed className="collapsible-panel verbose" header={<div onClick={() => otherBackups?.current?.toggle()} className="cursor-pointer">Snapshots of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={snapshotRow}
                                            onSelectionChange={(e) => setSnapshotRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={elastic_backups?.filter((b) => b.ex_params?.node_version === props.node.version && b.cluster !== props.node.cluster)}
                                            expandedRows={expandSnapshot}
                                            emptyMessage="no snapshots found"
                                            rowExpansionTemplate={snapshotExpansion}
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            onRowToggle={(e) => {
                                                setExpandSnapshot(e.data)
                                            }}>
                                            <Column style={{ height: '21px' }} field="created_date" body={snapshotDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                            <Column expander style={{ height: '21px', width: "0.75rem" }} />
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>

                            </div>

                        </TabPanel>
                    </TabView>
                </div>
            </div>
            <AddElasticSearchNode
                visible={showAddNode}
                node={props.node}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setElasticSearchVersionExists(undefined));
                    setShowAddNode(false)

                }}
                onOkClick={onAddElasticSearchNode}

            />

            {showWarning ? <WarningDialog
                message={warningMessage}
                visible={showWarning}
                onHide={() => {

                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowWarning(false)
                }
                } />
                : null}

            {showInfoDialog ? <InfoDialog
                message={infoMessage}
                visible={showInfoDialog}
                icon={infoIcon}
                description="index"
                onHide={() => {
                    infoMessage === 'db_cred' && setShowAddIndex(false);
                    setShowInfoDialog(false);
                }
                } />
                : null}

            {
                showUpgrade ? <UpgradeElasticSearchNodeDialog
                    visible={showUpgrade}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowUpgrade(false)
                    }
                    }
                    onOkClick={onUpgrade}
                    node={props.node}
                /> : null
            }


            {showDropNode ? <DropElasticsearchNodeDialog
                node={inventory.current}
                nodes_count={props.node?.nodes?.length}
                visible={showDropNode}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropNode(false)
                }}
                onOkClick={onDropNode}
            />
                : null}


            {showDumpIndex ? <DumpIndexDialog
                index={props.selected.current}
                visible={showDumpIndex}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setDirExists(null));
                    dispatch(LayoutActions.setExportExists(null));
                    dispatch(LayoutActions.setDiskSpace(null));
                    setShowDumpIndex(false)
                }
                }
                onOkClick={onDumpIndex}
            />
                : null}

            {showSnapshot ? <ElasticsearchClusterSnapshotDialog
                node={props.node}
                updateSchedule={updateSchedule.current}
                pageIndex={pageIndex}
                visible={showSnapshot}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowSnapshot(false)
                }}
                onOkClick={onSnapshotCluster}
            />
                : null}

            {showAddIndex ? <AddIndexDialog
                node={props.node}
                visible={showAddIndex}
                onHide={() => {
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddIndex(false);
                }
                }
                onOkClick={onAddIndex}
            />
                : null}

            {showDropIndex ? <DropIndexDialog
                node={inventory.current}
                visible={showDropIndex}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropIndex(false);
                }
                }
                onOkClick={onDropIndex}
            />
                : null}
            {
                showRestoreIndex ? <RestoreIndexLayoutDialog
                    visible={showRestoreIndex}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestoreIndex(false)
                    }}
                    onOkClick={onRestoreIndex}
                    dump={inventory.current}
                /> : null
            }

            {
                showRestoreSnapshotCluster ? <RestoreElasticsearchClusterLayout
                    visible={showRestoreSnapshotCluster}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestoreSnapshotCluster(false)
                    }
                    }
                    onOkClick={onRestoreSnapshotCluster}
                    node={props.node}
                    snapshot={inventory.current}
                /> : null
            }


            {showDumpDetails ? <DumpDetailsDialog
                dump={inventory.current}
                visible={showDumpDetails}
                onHide={() => setShowDumpDetails(false)} />
                : null
            }

            {showSnapshotDetails ? <BackupDetailsDialog
                backup={inventory.current}
                visible={showSnapshotDetails}
                onHide={() => setShowSnapshotDetails(false)} />
                : null
            }

            {showDeleteInventory ? <DeleteInventoryDialog
                visible={showDeleteInventory}
                onHide={() => {
                    setShowDeleteInventory(false);
                    dispatch(LayoutActions.setErrorMessage(""));
                }}
                onOkClick={onDeleteInventory}
                item={inventory.current} />
                : null
            }

            {showCopyIndex ? <CopyIndexDialog
                visible={showCopyIndex}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowCopyIndex(false)
                }
                }
                onOkClick={onCopyIndex}
                index={inventory.current}
                new={IndexNew}
                source={databaseSource}
                node={props.node}
            />
                : null
            }


            {showStatus ? <ElasticSearchStatusDialog
                node={props.node}
                visible={showStatus}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowStatus(false)
                }} />
                : null
            }

            {showSwitchover ? <SwitchElasticsearchNodeDialog
                nodes={props.node?.nodes?.filter((n) => n.severity === "green" && n.ex_params?.roles.indexOf("master") !== -1 && n.ind_deleted === false && n?.ex_params?.roles.indexOf("voting_only") === -1)}
                visible={showSwitchover}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowSwitchover(false)
                }}
                onOkClick={onSwitchover}
            />
                : null}

            {showChooseServiceDialog ? <ChooseServiceDialog
                visible={showChooseServiceDialog}
                services={props.services}
                onHide={() => setShowChooseServiceDialog(false)}
                onNext={(targetService) => setShowAddNode(targetService ? targetService.name : true)}
            /> : null}


            {showDropCluster ? <ElasticDropClusterDialog
                node={props.node}
                visible={showDropCluster}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropCluster(false)
                }}
                onOkClick={onDropCluster}
            />
                : null}

            {showDropMode ? <DropModeDialog
                visible={showDropMode}
                onHide={() => setShowDropMode(false)}
                onOkClick={(mode) => mode === clusterDropModes.CLUSTER ? setShowDropCluster(true) : setShowDropNode(true)}
            /> : null}

        </div>

    );


}


export default ElasticsearchLayoutItem;