import { useState, useRef, useEffect, useCallback } from "react";
import { TabView, TabPanel } from 'primereact/tabview';
import { nanoid } from 'nanoid';
import * as d3 from "d3";
import trashImage from "../../../assets/images/trash.png";
import dbImage from "../../../assets/images/db.png";
import plusIcon from "../../../assets/images/plus.png";
import minusIcon from "../../../assets/images/minus.png";
import node_white from "../../../assets/images/node_white.png";
import node_unavail from '../../../assets/images/node_unavail.png'
import node_un_white from '../../../assets/images/node_un_white.png';
import db_white from "../../../assets/images/db_white.png";
import node_avail from "../../../assets/images/node.png";
import bar from "../../../assets/images/bar.png";
import db_unavail from "../../../assets/images/db_unavail.png";
import db_un_white from "../../../assets/images/db_un_white.png";
import cluster from "../../../assets/images/cluster.png";
import cluster_white from "../../../assets/images/cluster_white.png";
import { useNavigate } from 'react-router-dom';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import moment from "moment";
import WarningDialog from "../../../dialogs/warningDialog";
import { useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import { useSelector } from "react-redux";
import DropDatabaseDialog from "../../../dialogs/mongo/dropDatabaseDialog";
import DumpDetailsDialog from "../../../dialogs/mongo/dumpDetailsDialog";
import BackupDetailsDialog from "../../../dialogs/postgres/backupDetailsDialog";
import { Button } from "primereact/button";
import DeleteInventoryDialog from "../../../dialogs/deleteInventoryDialog";
import getColorBySeverity from "../../../util/colorBySeverity";
import MongoNodeBackupDialog from "../../../dialogs/mongo/mongoNodeBackupDialog";
import AddMongoShardedClusterNodeDialog from "../../../dialogs/mongo/addMongoShardedClusterNodeDialog";
import MongoDumpDialog from "../../../dialogs/mongo/mongoDumpDialog";
import RestoreBackupMongoNodeLayout from "../../../dialogs/mongo/restoreBackupMongoNodeLayout";
import RestoreDumpMongoNodeLayout from "../../../dialogs/mongo/restoreDumpMongoNodeLayout";
import DropMongoNodeDialog from "../../../dialogs/mongo/dropMongoNodeDialog";
import MongoStatusDialog from "../../../dialogs/mongo/mongoStatusDialog";
import AddMongoDbDialog from "../../../dialogs/mongo/addMongoDbDialog";
import RestoreDatabaseLayoutDialog from "../../../dialogs/mongo/restoreDatabaseLayoutDialog";
import DumpDatabaseDialog from "../../../dialogs/mongo/dumpDatabaseDialog";
import InfoDialog from "../../../dialogs/infoDialog";
import CopyDbDialog from "../../../dialogs/mongo/copyDbDialog";
import config, { clusterRoles, types } from "../../../config";
import { TabMenu } from "primereact/tabmenu";
import { knobs } from "../../../util/templates";
import { Tag } from "primereact/tag";
import { ScrollPanel } from "primereact/scrollpanel";
import { Panel } from "primereact/panel";
import AddMongoShardedClusterPrimaryDialog from "../../../dialogs/mongo/addMongoShardedClusterPrimaryDialog";
import { Paginator } from "primereact/paginator";
import { ConfirmPopup } from "primereact/confirmpopup";
import StepdownShardPrimaryDialog from "../../../dialogs/mongo/stepdownShardPrimaryDialog";
import { OverlayPanel } from "primereact/overlaypanel";
import MongoDropClusterDialog from "../../../dialogs/mongo/dropClusterDialog";


const MongoShardedLayoutItem = (props) => {

    const navigate = useNavigate();
    const [showNodes, setShowNodes] = useState(true);
    const [showSecondary, setShowSecondary] = useState(null);
    const d3Container = useRef(null);
    const [activeIndex, setActiveIndex] = useState(0);
    const [nodeSelected, setNodeSelected] = useState(false);
    const [attachedMongoNodes, setAttachedMongoNodes] = useState(null);
    const [foreignMongoNodes, setForeignMongoNodes] = useState(null);
    const tabRef = useRef(0);
    const inventory = useRef(null);
    const [infoIcon, setInfoIcon] = useState('pi pi-info-circle');
    const [showDeleteInventory, setShowDeleteInventory] = useState(false);
    const [warningMessage, setWarningMessage] = useState('');
    const [showWarning, setShowWarning] = useState(false);
    const [showAddPrimary, setShowAddPrimary] = useState(false);
    const leftContainer = useRef(null);
    const rightContainer = useRef(null);
    const tabsPos = useRef(null);
    const trashPos = useRef(null);
    const stepDownBtn = useRef(null);
    const [mongoDatabases, setMongoDatabases] = useState(null);
    const [showDropDatabase, setShowDropDatabase] = useState(false);
    const [showDumpNode, setShowDumpNode] = useState(false);
    const [showBackup, setShowBackup] = useState(false);
    const [showAddNode, setShowAddNode] = useState(false);
    const [showAddDatabase, setShowAddDatabase] = useState(false);
    const [showDumpDatabase, setShowDumpDatabase] = useState(false);
    const [showRestoreDatabase, setShowRestoreDatabase] = useState(false);
    const [showDumpDetails, setShowDumpDetails] = useState(false);
    const [showBackupDetails, setShowBackupDetails] = useState(false);
    const [showDropNode, setShowDropNode] = useState(false);
    const [showRestoreBackupNode, setShowRestoreBackupNode] = useState(false);
    const [backupRow, setBackupRow] = useState(null);
    const [exportRow, setExportRow] = useState(null);
    const [standbyRow, setStandbyRow] = useState(null);
    const [dbRow, setDbRow] = useState(null);
    const [showCopyDb, setShowCopyDb] = useState(false);
    const [showStatus, setShowStatus] = useState(false);
    const [dbNew, setDbNew] = useState(null);
    const [isPhysical, setIsPhysical] = useState(true);
    const updateSchedule = useRef(null);
    const [expandBackup, setExpandBackup] = useState(null);
    const [databaseSource, setDatabaseSource] = useState("");
    const [showRestoreDumpNode, setShowRestoreDumpNode] = useState("");
    const [showInfoDialog, setShowInfoDialog] = useState(false);
    const [showStepDown, setShowStepDown] = useState(false);
    const [infoMessage, setInfoMessage] = useState(false);
    const [primaryType, setPrimaryType] = useState(clusterRoles.PRIMARY);
    const layoutItemId = useRef(null);

    const { databases, mongo_exports, mongo_backups } = useSelector((state) => state.layout);

    const [layoutCurrHeight, setLayoutCurrHeight] = useState(config.layoutMinHeight);
    const [layoutMid, setLayoutMid] = useState(config.layoutMinHeight / 2);
    const [layoutSwitchIndex, setLayoutSwitchIndex] = useState(0);
    const [removeScheduleConfirm, setRemoveScheduleConfirm] = useState(false);
    const [pageIndex, setPageIndex] = useState(0);
    const rowsNumber = 6;
    const [first, setFirst] = useState(0);
    const [rows, setRows] = useState(rowsNumber);
    const [databasesPage, setDatabasesPage] = useState(props.node?.databases?.length > 6 ? props.node?.databases?.slice(props.node?.databases?.length - (rowsNumber + 1), props.node.databases?.length - 1) : props.node?.databases);
    const database_x = 460;
    const clusterOptions = useRef(null);
    const [showDropCluster, setShowDropCluster] = useState(false);
    const thisNodes = useRef(null);
    const otherNodes = useRef(null);
    const thisDbs = useRef(null);
    const otherDbs = useRef(null);
    const thisBackups = useRef(null);
    const otherBackups = useRef(null);
    const thisExports = useRef(null);
    const otherExports = useRef(null);


    useEffect(() => {

        if (props.node?.databases?.length <= rowsNumber) setDatabasesPage(props.node?.databases);

    }, [props.node])

    const onPageChange = (event) => {
        let rowsPerPage = rowsNumber;
        let startIndex = (event.page) * rowsPerPage;
        if (event.rows < rowsPerPage) {
            rowsPerPage = rowsPerPage - event.rows
            startIndex -= rowsPerPage
        }
        let endIndex = startIndex + rowsPerPage;
        setFirst(event.first);
        setRows(event.rows);
        setDatabasesPage(props.node?.databases?.slice(startIndex, endIndex));

    };

    const layoutOptions = [
        { label: 'Physical layout', className: isPhysical ? '' : 'p-disabled' },
        { label: 'Database layout', className: isPhysical ? 'p-disabled' : '' },
    ];



    const getAutomaticBackup = () => {
        let backup = props.node?.backups?.filter((b) => b.is_sched === true)?.[0]
        if (!backup) backup = mongo_backups?.filter(b => b?.cluster === props.node?.id && b.ex_params?.node_version === props.node?.ex_params?.version && b?.is_sched === true)?.[0]
        updateSchedule.current = backup;
        const popup_id = `remove-schedule-button-${props.node?.id}`
        return (<div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}> Automatic backup: {backup?.ex_params?.description || "No"}</div> {backup ? <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-pencil" onClick={() => {

            setShowBackup(true);
        }}>change</Tag> <Tag className='right-container-button ml-2' id={popup_id} icon="pi pi-trash" severity="danger" onClick={() => setRemoveScheduleConfirm(true)} >delete</Tag>    <ConfirmPopup
                target={document.getElementById(popup_id)}
                visible={removeScheduleConfirm}
                onHide={() => setRemoveScheduleConfirm(false)}
                message="Are you sure?"
                accept={() => dispatch(LayoutActions.removeMongoSchedule({ "node_id": props.node?.id }, setRemoveScheduleConfirm))}
            /></div> : <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-plus" onClick={() => {

                setPageIndex(1);
                setShowBackup(true);
            }}>create</Tag> </div>}  </div>)
    }


    useEffect(() => {
        let height = config.layoutMinHeight;

        if (showNodes) {
            if (props.node?.nodes?.length > 3) height = height + (80 * (props.node.nodes?.length - 3));
            if (showSecondary) {
                const keys = Object.keys(showSecondary);
                height = height + (80 * (keys?.filter((k) => showSecondary[k] !== false)?.length)) + 80 * (props.node?.nodes?.length + 1)
            }


        }


        setLayoutCurrHeight(height);
        setLayoutMid(height / 2);


    }, [props.node?.nodes, showNodes, showSecondary])


    useEffect(() => {

        if (props.node) {
            let tempData = []
            if (props.node.nodes?.length > 0) props.node.nodes.forEach((n) => {
                tempData.push(n);
                if (n.nodes?.length > 0) n.nodes.forEach((c) => tempData.push(c));
            });
            setAttachedMongoNodes(tempData);
        }
    }, [props.node]);

    useEffect(() => {

        let tempData = []
        if (props.selection?.length > 0) {

            props.selection?.filter((p) => p.cluster !== props.node?.id)?.forEach((p) => {
                tempData.push(p);
                if (p.nodes?.length > 0) p.nodes.forEach((n) => {
                    tempData.push(n);
                    if (n.nodes?.length > 0) n.nodes.forEach((c) => tempData.push(c));
                })
            })
        }
        setForeignMongoNodes(tempData);


    }, [props.selection, props.node]);


    useEffect(() => {

        let tempData = []
        if (databases?.length > 0) tempData = databases?.filter((d) => d.type === types.MONGO);

        setMongoDatabases(tempData);

    }, [databases])

    const dispatch = useDispatch();

    const onVerifyBackup = (open, message) => {

        setInfoMessage(message);
        setShowInfoDialog(open);

    }



    const onDeleteInventory = (data) => {


        switch (inventory.current.type) {
            case "BACKUP":
                dispatch(LayoutActions.dropBackup(data, setShowDeleteInventory));
                break;

            default:
                break;

        }

    }

    const onDropCluster = (data) => {

        dispatch(LayoutActions.dropCluster(data, setShowDropCluster));

    }

    const addImage = useCallback((x, y, r_x, r_y, node) => {

        function onClickNode(svg, id, node) {

            props.selected.current = node;
            setNodeSelected(node); let elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? node_avail : node_unavail);
            }

            elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }


            elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }


            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            svg.select("#node-" + id).attr("xlink:href", node?.ind_deleted ? (node?.ex_params?.mongo_master_id === node?.id) ? db_un_white : node_un_white : (node?.ex_params?.mongo_master_id === node?.id) ? db_white : node_white);

            svg.select(`#text-${id}`).style("fill", "white");
        }

        function goNodeDetails(node) {


            if (node?.ind_deleted) {
                showWarningMessage({ "text": `${node?.name} is no longer available.\nDetails cant be viewed.`, "status": node?.status })

            }
            else navigate(`/status/mongo/${node?.id}`);
        }

        const onDragStart = (svg, id, node) => (d) => {
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();

            onClickNode(svg, id, node);

        }


        const onDragging = (svg, id, node) => (d) => {

            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }


            if (d.sourceEvent.clientX <= leftContainer.current.left || d.sourceEvent.clientY >= leftContainer.current.bottom || d.sourceEvent.clientY <= tabsPos.current.top || d.sourceEvent.clientX >= tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            setNodeSelected(node);
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;


                // move the dragged element
                d3.select("#node-" + id).style("cursor", "grabbing");
                d3.select("#node-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx) + "," + (dy) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 51) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", "translate(" + (dx + 40) + "," + (dy + 10) + ")");
                d3.select("#expand-" + id).attr("transform", () => "translate(" + (dx + 30) + "," + (dy + 8) + ")");

                if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                    svg.select(".trash").style("opacity", "1.0");

                }
                else {

                    svg.select(".trash").style("opacity", "0.5");

                }


                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY >= tabsPos.current.top && d.sourceEvent.clientY <= tabsPos.current.bottom && d.sourceEvent.clientX <= tabsPos.current.right) {
                    switch (tabRef.current) {
                        case 2:
                            break;
                        default:
                            svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                    }
                }

                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 100) {
                    setActiveIndex(0);
                    tabRef.current = 0;
                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 100 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 200) {
                    setActiveIndex(1);
                    tabRef.current = 1;
                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 200 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 300) {
                    setActiveIndex(2);
                    tabRef.current = 2;

                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 300 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 400) {
                    setActiveIndex(3);
                    tabRef.current = 3;

                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 400 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 500) {
                    setActiveIndex(4);
                    tabRef.current = 4;

                }
            }

            let elements = window.document.querySelectorAll(`.mongo-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = tabRef.current === 0 || tabRef.current === 1 || tabRef.current === 3 || tabRef.current === 4 ? "" : "0 0 0 0.2rem rgb(40 167 69 / 72%)";

            }
        }

        const onDragEnd = (svg, id, node) => (d) => {

            if (d.sourceEvent.detail === 2) {

                goNodeDetails(node);
                return;

            }

            setNodeSelected(node);
            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                if (node?.ind_deleted) {


                    showWarningMessage({

                        "text": `${node?.name.toUpperCase()} is unavailable`,
                        "status": `${node?.status}`
                    });
                }
                else {

                    inventory.current = node;
                    if (isPhysical) setShowDropNode(true);
                    else setShowDropDatabase(true);

                }

            }

            if (d.sourceEvent.clientX >= tabsPos.current.left && d.sourceEvent.clientX >= tabsPos.current.top && d.sourceEvent.clientY <= tabsPos.current.bottom && d.sourceEvent.clientX <= tabsPos.current.right) {


                if (node?.ind_deleted) {


                    showWarningMessage({

                        "text": `${node?.name.toUpperCase()} is unavailable`,
                        "status": `${node?.status}`
                    });
                }
                else {

                    switch (tabRef.current) {

                        case 2:
                            if (node.ind_deleted)
                                showWarningMessage({

                                    "text": `${props.node.name.toUpperCase()} is unavailable`,
                                    "status": `${props.node.status}`
                                });
                            else {

                                if (!isPhysical) {

                                    if (node?.name === "local")
                                        showWarningMessage({ "text": 'database "local" is not allowed for dump' })
                                    else setShowDumpDatabase(true);
                                }
                                else setShowDumpNode(true);

                            }
                            break;
                        default:
                            break;

                    }


                }

            }

            d3.select("#node-" + id).remove();
            d3.select("#line-" + id).remove();
            d3.select("#text-" + id).remove();
            d3.select("#severity-" + id).remove();
            d3.select("#expand-" + id).remove();
            d3.select("#d3-tooltip-container-" + id).remove();

            if (isPhysical === false) {

                if (databasesPage?.length > 0) {

                    let i = 0;
                    for (; i < databasesPage?.length; ++i)
                        if (databasesPage[i].id === node?.id)
                            break;
                    let x = 450, y = (i + 1) * (layoutCurrHeight) / (databasesPage?.length + 1) - (42 / (i + 1));
                    if (node) {

                        addImage(x, y, 100, layoutMid + 10, node);

                    }

                }

            }
            else {

                const primary_x = 300;
                let i = props.node?.nodes?.findIndex((m) => m.id === node?.id)
                if (i !== -1) {

                    const primary_y = (i + 1) * layoutCurrHeight / (props.node.nodes?.length + 1) - (42 / (i + 1));
                    addImage(primary_x, primary_y, 100, layoutMid + 10, node);

                }


                const secondary_x = primary_x + 200;
                i = props.node?.nodes?.findIndex((p) => p.id === node?.ex_params?.mongo_master_id);
                if (i !== -1) {


                    const primary_y = (i + 1) * layoutCurrHeight / (props.node.nodes?.length + 1) - (42 / (i + 1));
                    const secondary_of_primary = props.node.nodes[i].nodes
                    if (secondary_of_primary) {
                        const j = secondary_of_primary.findIndex((s) => s.id === node?.id);
                        if (j !== -1) {
                            const secondary_y = primary_y + 80 * (j - 1);
                            addImage(secondary_x, secondary_y, primary_x + 40, primary_y + 20, node);
                        }

                    }
                }

            }


            svg.select(".trash").style("opacity", "0.0");
            d3.select("#node-" + id).style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.mongo-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        const svg = d3.select(d3Container.current);
        const id = nanoid();

        const drawNodeDropzone = (event) => {
            const svg = d3.select(d3Container.current)
            svg.select(`#${event.target.id}`).dispatch("click")
            const [clientX, clientY] = d3.pointer(event, svg.node());
            d3.select(d3Container.current)
                .select(".drop-zone-marker")
                .attr("fill", "transparent");
            svg.insert("rect")
                .attr("class", "drop-zone-marker")
                .attr("x", clientX - 40)
                .attr("y", clientY - 40)
                .attr("width", 100)
                .attr("height", 100)
                .attr("fill", "transparent")
                .attr("stroke", inventory.current?.status === false ? "red" : "green")
                .on("drop", dragToNodeEnd)
                .on("dragover", dragOverNode)
                .on("dragleave", dragLeaveNode);
        }


        function dragToNodeEnd(event) {

            if (nodeSelected?.ind_deleted) {
                showWarningMessage({

                    "text": `${props.node?.name.toUpperCase()} is unavailable`,
                    "status": `${props.node.ex_params.status}`
                });

                return;

            }

            if (inventory.current?.status === false) {
                showWarningMessage({

                    "text": "Cannot perform this action",
                    "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
                });

                return;



            }

            let type = event.dataTransfer.getData("dragType");

            switch (type) {
                case "nodeCreate":
                    if (isPhysical) setShowAddNode(true);
                    break;
                case "exportBar":
                    if (inventory.current?.database) setShowRestoreDatabase(true);
                    else if (inventory.current?.node) setShowRestoreDumpNode(true);
                    break;

                default:
                    break;

            }

        }


        const image = svg.insert("image")
            .attr("xlink:href", (props.selected.current?.id === node?.id) ? node?.ind_deleted ? node?.ex_params?.mongo_master_id === node?.id ? db_un_white : node_un_white : node?.ex_params?.mongo_master_id === node?.id ? db_white : node_white : node?.ind_deleted ? node?.ex_params?.mongo_master_id === node?.id ? db_unavail : node_unavail : (node?.ex_params?.mongo_master_id === node?.id) ? dbImage : node_avail)
            .attr("class", props.selected.current?.ex_params?.mongo_master_id === node?.id ? "root" : "node")
            .attr("id", "node-" + id)
            .attr('x', 0)
            .attr('y', 0)
            .attr("width", 40)
            .attr("height", 60)
            .attr("available", node?.ind_deleted ? false : true)
            .attr("mongo_master_id", node?.ex_params?.mongo_master_id)
            .attr("draggable", true)
            .attr("transform", () => "translate(" + x + "," + y + ")")
            .on("mouseover", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

            })
            .on("mouseout", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

            })
            .on("click", function (d) {
                onClickNode(svg, id, node);
            });

        if (node?.cluster_role === clusterRoles.PRIMARY || !node?.cluster_role) {

            image.on("dragover", drawNodeDropzone)
                .on("dragleave", dragLeaveNode)
        }


        const x0 = r_x;
        const y0 = r_y;
        const x1 = x;
        const y1 = y + 20;

        const path = d3.path()
        path.moveTo(x1, y1)
        path.bezierCurveTo((x0 + x1) / 2, y1, (x0 + x1) / 2, y0, x0, y0);

        svg.insert("path")
            //.attr("d", d3.line()([[100, 220], [x, y + 30]]))
            .attr("d", path)
            .attr("id", "line-" + id).attr("transform", "translate(" + (0) + "," + (10) + ")")
            .style("fill", "none")
            .style("stroke", "#ccc")
            .style("stroke-width", 1);
        if (showSecondary && showSecondary.hasOwnProperty(node?.name))
            svg.insert("image")
                .attr("xlink:href", showSecondary[node?.name] === true ? minusIcon : plusIcon)
                .attr("id", "expand-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 15)
                .attr("height", 15)
                .attr("transform", () => "translate(" + (x + 30) + "," + (y + 5) + ")")
                .on("click", () => { setShowSecondary({ ...showSecondary, [node?.name]: !showSecondary[node?.name] }) });
        svg.insert("text")
            .attr("transform", "translate(" + (x) + "," + (y) + ")")
            .attr("id", "text-" + id)
            .attr("class", "node-what-text")
            .style("font-size", "12px")
            .style("fill", `${props.selected.current?.id === node?.id ? "white" : "gray"}`)
            .style("cursor", "pointer")
            .text(`${node?.name ? node?.name : ""} ${node?.status ? node?.status : ""} ${node?.cluster_role === clusterRoles.STANDBY ? "(SECONDARY)" : node?.cluster_role ? node?.cluster_role : ""}`).on("dblclick", () => goNodeDetails(node)).on("click", function (d) {
                onClickNode(svg, id, node);
            });

        //draw node severity strip
        svg.insert("rect").attr('x', 0)
            .attr('y', 0)
            .attr("width", 35)
            .attr("height", 5).attr("fill", getColorBySeverity(node?.severity))
            .attr("transform", "translate(" + (x1 + 2) + "," + (y1 + 31) + ")")
            .attr("id", "severity-" + id)

        svg.insert("foreignObject")
            .attr("class", "d3-tooltip-container")
            .attr("id", `d3-tooltip-container-${id}`)
            .attr("transform", "translate(" + (x + 40) + "," + (y + 10) + ")")
            .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${node?.ind_started ? "up" : "down"} <br/> Alerts: ${node?.alert?.length > 40 ? node.alert.substring(0, 37).concat("...") : node?.alert} </div>`)

        var drag_behavior = d3.drag()
            .on("start", onDragStart(svg, id, node))
            .on("drag", onDragging(svg, id, node))
            .on("end", onDragEnd(svg, id, node));

        image.call(drag_behavior);
    }, [navigate, props.node, layoutCurrHeight, layoutMid, databasesPage, isPhysical, showSecondary, nodeSelected, props.selected]);


    useEffect(() => {


        const onTableItemDrop = (event) => {
            let type = event.dataTransfer.getData("dragType");

            switch (type) {
                case "nodeBar":
                case "sentinelBar":
                    if (inventory.current?.ind_deleted) {


                        showWarningMessage({

                            "text": `${inventory.current.name.toUpperCase()} is unavailable`,
                            "status": `${inventory.current.status}`
                        });
                    }

                    else setShowDropNode(true);
                    break;
                case "backupBar":
                    setShowDeleteInventory(true);
                    break;
                default:
                    break;

            }

        }


        function onClusterClick(svg, id) {

            setNodeSelected(false);

            let elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }

            elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }

            elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? node_avail : node_unavail);
            }

            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }


            svg.select(".cluster").attr("xlink:href", props.node?.ind_deleted ? db_un_white : cluster_white);
            props.selected.current = props.node;
            svg.select(`#text-${id}`).style("fill", "white");


        }


        const onDragStartCluster = (svg, id, cluster) => (d) => {
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();

            // Draw cluster what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 10) + ")")
                .attr("id", "text-copy" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node?.id ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node?.name ? props.node?.name : ""} ${props.node.ex_params.status ? props.node.ex_params.status : ""}`);

            // Draw cluster icon
            svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node?.id) ? props.node?.ind_deleted ? db_un_white : cluster_white : props.node?.ind_deleted ? db_unavail : cluster)
                .attr("class", "cluster")
                .attr("id", "cluster-copy" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + layoutMid + ")")
                .attr("available", !props.node?.ind_deleted);

            //draw cluster severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-copy" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.ex_params?.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 40) + ")")


            // Draw cluster plus/minus icon

            if (props.node.nodes?.length > 0) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-copy" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 5) + ")");
            }

            onClusterClick(svg, id)

        }

        const onDraggingCluster = (svg, id, cluster) => (d) => {

            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }

            if (d.sourceEvent.clientX <= leftContainer.current.left || d.sourceEvent.clientY >= leftContainer.current.bottom || d.sourceEvent.clientY <= tabsPos.current.top || d.sourceEvent.clientX >= tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#cluster-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx - 54) + "," + (dy - 8) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 40) + ")");
                d3.select("#expand-" + id).attr("transform", () => "translate(" + (dx + 30) + "," + (dy - 5) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", () => "translate(" + (dx + 50) + "," + (dy - 5) + ")");
            }

            svg.select(".cluster").style("cursor", "grabbing");
            if (d.sourceEvent.clientX > trashPos.current.left && d.sourceEvent.clientY < trashPos.current.bottom && d.sourceEvent.clientX < trashPos.current.right && d.sourceEvent.clientY > trashPos.current.top) {

                svg.select(".trash").style("opacity", "1.0");

            }
            else {

                svg.select(".trash").style("opacity", "0.5");

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                switch (tabRef.current) {
                    case 2:
                    case 4:
                        break;
                    default:
                        svg.select(".cluster").style("cursor", "not-allowed");
                        break;
                }
            }



            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 100) {
                setActiveIndex(0);
                tabRef.current = 0;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 100 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 200) {
                setActiveIndex(1)
                tabRef.current = 1;
            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 200 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 300) {
                setActiveIndex(2);
                tabRef.current = 2;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 300 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 400) {
                setActiveIndex(3);
                tabRef.current = 3;
            }

            else if (d.sourceEvent.clientX > tabsPos.current.left + 400 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 500) {
                setActiveIndex(4);
                tabRef.current = 4;
            }

            let elements = window.document.querySelectorAll(`.mongo-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        const onDragEndCluster = (svg, id, cluster) => (d) => {

            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                if (props.node?.ind_deleted)
                    showWarningMessage({

                        "text": `${props.node?.name.toUpperCase()} is unavailable`,
                        "status": `${props.node.ex_params.status}`
                    });

                else {
                    if (props.node.nodes?.length > 0 || props.node.sentinels?.length > 0) showWarningMessage({ "text": `This is cluster with nodes, cannot be unregistered.\nDrop cluster nodes first.` })
                    else {

                        inventory.current = props.node
                        setShowDropCluster(true);

                    }

                }

            }

            if (d.sourceEvent.clientX >= tabsPos.current.left && d.sourceEvent.clientX >= tabsPos.current.top && d.sourceEvent.clientY <= tabsPos.current.bottom && d.sourceEvent.clientX <= tabsPos.current.right) {

                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {


                    switch (tabRef.current) {
                        case 2:
                            if (props.node.ind_deleted)
                                showWarningMessage({

                                    "text": `${props.node.name.toUpperCase()} is unavailable`,
                                    "status": `${props.node.status}`
                                });
                            else setShowDumpNode(true);
                            break;
                        case 4:
                            if (props.node.ind_deleted)
                                showWarningMessage({

                                    "text": `${props.node.name.toUpperCase()} is unavailable`,
                                    "status": `${props.node.status}`
                                });
                            else {


                                setPageIndex(0);
                                setShowBackup(true);

                            }
                            break;
                        default:
                            break;

                    }

                }

            }



            svg.select(".trash").style("opacity", "0.0");

            d3.select("#cluster-copy" + id).remove();
            d3.select("#text-copy" + id).remove();
            d3.select("#severity-copy" + id).remove();
            d3.select("#expand-copy" + id).remove();

            d3.select("#cluster-" + id).attr("transform", () => "translate(" + 50 + "," + layoutMid + ")");
            d3.select("#text-" + id).attr("transform", () => "translate(" + 0 + "," + (layoutMid - 10) + ")");
            d3.select("#severity-" + id).attr("transform", "translate(" + (52) + "," + (layoutMid + 40) + ")");
            d3.select("#expand-" + id).attr("transform", "translate(" + (80) + "," + (layoutMid - 5) + ")");
            d3.select(`#d3-tooltip-container-${id}`).attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
            svg.select(".cluster").style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.mongo-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        if (d3Container.current) {
            const svg = d3.select(d3Container.current);
            svg.selectChildren("*").remove();

            const id = nanoid();

            svg.insert("foreignObject")
                .attr("class", "d3-tooltip-container")
                .attr("id", `d3-tooltip-container-${id}`)
                .attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
                .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${props.node?.ind_started ? "up" : "down"} <br/> Alerts:  ${props.node?.alert?.length > 40 ? props.node.alert.substring(0, 37).concat("...") : props.node?.alert} </div>`)
            // Draw trash icon    
            svg.insert("image")
                .attr("xlink:href", trashImage)
                .attr("class", "trash")
                .attr("id", `trash-${id}`)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 100)
                .attr("height", 100)
                .attr("transform", () => "translate(" + 0 + "," + (layoutCurrHeight - 100) + ")")
                .on("dragover", dragOverTrash)
                .on("dragleave", dragLeaveTrash)
                .on("drop", onTableItemDrop);


            // Draw cluster what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 10) + ")")
                .attr("id", "text-" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node?.id || !layoutItemId.current ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node?.name ? props.node?.name : ""} ${props.node.ex_params.status ? props.node.ex_params.status : ""}`)
                .on("click", () => onClusterClick(svg, id));

            // Draw cluster icon
            const imageCluster = svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node?.id || !layoutItemId.current) ? props.node?.ind_deleted ? db_un_white : cluster_white : props.node?.ind_deleted ? db_unavail : cluster)
                .attr("class", "cluster")
                .attr("id", "cluster-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + layoutMid + ")")
                .attr("available", !props.node?.ind_deleted)
                .on("mouseover", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

                })
                .on("mouseout", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

                });

            layoutItemId.current = id;

            const dragBehaviorCluster = d3.drag()
                .on("start", onDragStartCluster(svg, id, props.node))
                .on("drag", onDraggingCluster(svg, id, props.node))
                .on("end", onDragEndCluster(svg, id, props.node));
            imageCluster.call(dragBehaviorCluster);


            //draw cluster severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.ex_params?.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 40) + ")")



            // Draw cluster plus/minus icon

            if ((isPhysical && props.node.nodes?.length > 0) || (!isPhysical && props.node?.databases?.length > 0)) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 5) + ")")
                    .on("click", () => { setShowNodes(!showNodes) });

                if (showNodes) {

                    if (isPhysical === false) {
                        if (databasesPage?.length > 0) {

                            databasesPage?.forEach((database, index) => {
                                if (database) {

                                    addImage(database_x, ((index + 0.5) * (layoutCurrHeight / databasesPage.length)), 100, layoutMid + 10, database);

                                }

                            });

                        }

                    }

                    else {

                        let _showSecondary = {}

                        props.node.nodes?.forEach((primary, index) => {
                            const primary_x = 300;
                            const primary_y = (index + 1) * layoutCurrHeight / (props.node.nodes?.length + 1) - (42 / (index + 1));
                            addImage(primary_x, primary_y, 100, layoutMid + 10, primary);
                            const secondary_of_primary = props.node.nodes[index]?.nodes
                            if (secondary_of_primary?.length > 0) {
                                _showSecondary[primary.name] = false
                                if (showSecondary && showSecondary.hasOwnProperty(primary.name) && showSecondary[primary.name]) {

                                    secondary_of_primary.forEach((secondary, secondary_index) => {
                                        const secondary_x = primary_x + 200
                                        const secondary_y = primary_y + 80 * (secondary_index - 1);
                                        addImage(secondary_x, secondary_y, primary_x + 40, primary_y + 20, secondary);
                                    });

                                }
                            }

                        });



                        if (!showSecondary) setShowSecondary(_showSecondary)

                    }
                }
            }
        }
    }, [showNodes, props.node, navigate, props.selected, isPhysical, databasesPage, layoutCurrHeight, layoutMid, showSecondary, addImage, dispatch]);


    useEffect(() => {

        const updateSelected = () => {

            if (props.selected.current?.type === types.REDIS) {

                if (props.selected.current?.id === props.node?.id) {

                    let nodeSelected = props.selection?.filter((n) => n.id === props.selected.current.id);
                    if (nodeSelected?.length > 0) {
                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }

                }
                else {
                    let nodeSelected;
                    nodeSelected = props.node.nodes?.filter((n) => n.id === props.selected.current.id);
                    if (nodeSelected?.length === 0) {
                        for (let i = 0; i < props.node?.nodes?.length; i++) {

                            nodeSelected = props.node?.nodes?.[i]?.filter((n) => n.id === props.selected.current.id);
                            if (nodeSelected?.length > 0) break;

                        }

                    }

                    if (nodeSelected?.length > 0) {

                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }
                }


            }

        }


        if (props.selection?.length > 0) updateSelected();

    }, [props.node, props.selection, props.selected])

    function dragToClusterEnd(event) {
        let type = event.dataTransfer.getData("dragType");

        if (props.node?.ind_deleted) {
            showWarningMessage({

                "text": `${props.node?.name.toUpperCase()} is unavailable`,
                "status": `${props.node.ex_params.status}`
            });

            return;

        }

        if (inventory.current?.status === false) {
            showWarningMessage({

                "text": "Cannot perform this action",
                "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
            });

            return;



        }

        switch (type) {
            case "databaseBar":
                let source = event.dataTransfer.getData("dragSource");
                setDatabaseSource(source);
                dispatch(LayoutActions.checkMongoDatabaseExists({ "name": inventory.current?.name, "node_id": props.node.mongos?.[0].id }, ShowCopyDb));
                break;
            case "primaryCreate":
                setPrimaryType(clusterRoles.PRIMARY);
                setShowAddPrimary(true);
                break;
            case "mongosCreate":
                setPrimaryType(clusterRoles.BALANCER);
                setShowAddPrimary(true);
                break;
            case "databaseCreate":
                setShowAddDatabase(true);
                break;
            default:
                break;

        }
    }


    function dragOverCluster(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    }

    function dragOverNode(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    }

    function dragLeaveNode(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", "transparent");

    }

    const dragOverTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "1.0");

    }

    const dragLeaveTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.5");

    }

    const backupDateBodyTemplate = (item) => {
        return (
            <div className="flex align-items-center" draggable onDrag={onTableItemDrag}
                onDragEnd={onDropzoneLeave}
                onDragStart={(e) => {
                    leftContainer.current = d3Container.current.getBoundingClientRect();
                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                    d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
                    window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                    e.dataTransfer.setData("dragType", "backupBar");
                    inventory.current = item;

                }}>
                {knobs}
                <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>
            </div>
        );
    }





    const sizeBodyTemplate = (item) => {
        return (<div className="flex align-items-center icon-cell backup">{item.display_size} <i className="pi pi-check verify-backup" onClick={() => {

            dispatch(LayoutActions.verifyBackup({ "name": item?.name, "type": item?.type, "backup_host": item?.host_name, "backup_path": item?.ex_params?.["backup_path"], "backup_date": item?.ex_params?.["modified_date"] }, onVerifyBackup))

        }} /> <img className="bar-icon" draggable={false} alt="backupBar" src={bar} onClick={() => {
            inventory.current = item;
            switch (item.type) {

                case "BACKUP":
                    setShowBackupDetails(true);
                    break;
                default:
                    break;

            }

        }} /></div>);
    }

    const lagBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} >{item.ex_params?.lag}</div>);
    }

    const portBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.port}</div>);
    }


    const secondaryBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.nodes?.length ? item.nodes?.length : 0}</div>);
    }

    const statusBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item?.severity === 'red' || item.nodes?.filter((n) => n?.severity === 'red')?.length > 0 ? 'NOT OK' : 'OK'}</div>);
    }

    const ipBodyTemplate = (item) => {
        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            leftContainer.current = d3Container.current.getBoundingClientRect();
            d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
            e.dataTransfer.setData("dragType", "nodeBar");
            inventory.current = item;
        }} >
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item?.severity)}` }}>{item.host}</div>
        </div>);
    }


    const shardNameBodyTemplate = (item) => {
        return (<div className="flex align-items-center">
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.ex_params.replicaset_name}</div>
        </div>);
    }




    const onDropzoneLeave = (e) => {
        e.target.style.position = "";
        e.target.style.left = "";
        e.target.style.top = "";
        e.target.style.visibility = ""; d3.selectAll(".drop-zone-marker").remove();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.0");
    }

    const drawClusterDropzone = () => {

        d3.select(d3Container.current).insert("rect")
            .attr("class", "drop-zone-marker")
            .attr("x", 20)
            .attr("y", layoutMid - 50)
            .attr("width", 100)
            .attr("height", 100)
            .attr("fill", "transparent")
            .attr("stroke", inventory.current?.status === false ? "red" : "green")
            .on("drop", dragToClusterEnd)
            .on("dragover", dragOverCluster)
            .on("dragleave", dragLeaveNode);

    }

    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onTableItemDrag = (e) => {
        e.preventDefault();
        e.target.style.position = "fixed";
        e.target.style.top = e.clientY + "px";
        e.target.style.left = e.clientX + "px";
        if (e.clientX < leftContainer.current.left || e.clientY < leftContainer.current.top || e.clientY > leftContainer.current.bottom) {

            e.target.style.visibility = "hidden";

        }
        else {

            e.target.style.visibility = "";

        }
    }

    const onDropNode = (data) => {
        dispatch(LayoutActions.dropMongoNode(data, setShowDropNode));
    }

    const onBackupNode = (data) => {
        dispatch(LayoutActions.backupMongoNode(data, setShowBackup));
    }

    const onAddMongoNode = (data) => dispatch(LayoutActions.addMongoShardedClusterNode(data, setShowAddNode));


    const onRestoreBackupNode = (data) => {
        dispatch(LayoutActions.restoreMongoNode(data, setShowRestoreBackupNode));

    }
    const onAddDatabase = (data) => {
        dispatch(LayoutActions.addMongoDatabase(data, showDbCredentialsInfo));

    }

    const onDropDatabase = (data) => {

        dispatch(LayoutActions.dropMongoDatabase(data, setShowDropDatabase));

    }

    const onRestoreDatabase = (data) => {

        dispatch(LayoutActions.restoreMongoDatabase(data, setShowRestoreDatabase));
    }

    const onRestoreDumpNode = (data) => {
        dispatch(LayoutActions.restoreDumpMongoNode(data, setShowRestoreDumpNode));
    }

    const onCopyDb = (data) => {
        dispatch(LayoutActions.copyMongoDatabase(data, setShowCopyDb));
    }

    const ShowCopyDb = (newdb, open) => {

        setDbNew(newdb);
        setShowCopyDb(open);
    }

    const onDumpDatabase = (data) => {

        dispatch(LayoutActions.dumpMongoDatabase(data, setShowDumpDatabase));
        dispatch(LayoutActions.setDirExists(null));
        dispatch(LayoutActions.setExportExists(null));
        dispatch(LayoutActions.setDiskSpace(null));

    }

    const onDumpNode = (data) => {
        dispatch(LayoutActions.dumpMongoNode(data, setShowDumpNode));
        dispatch(LayoutActions.setDirExists(null));
        dispatch(LayoutActions.setExportExists(null));
        dispatch(LayoutActions.setDiskSpace(null));
    }

    const onAddMongoShardedClusterPrimary = (data) => dispatch(LayoutActions.addMongoShardedClusterPrimary(data, setShowAddPrimary));


    const onStepDown = (data) => dispatch(LayoutActions.stepDownMongoNode(data, setShowStepDown));


    const dbHostBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item?.host}</div>);
    }

    const dbSizeBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item?.display_size}</div>);
    }


    const dbNameBodyTemplate = (item, tableData) => {
        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            leftContainer.current = d3Container.current.getBoundingClientRect();
            d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
            e.dataTransfer.setData("dragType", "databaseBar");
            if (tableData.props.className === "backup-expandable-table") e.dataTransfer.setData("dragSource", "backup-expandable");
            else e.dataTransfer.setData("dragSource", "databases");
            inventory.current = item;
        }}>
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item?.severity)}` }}>{tableData.props.className === "backup-expandable-table" ? `Database: ${item.name}` : item.name}</div>
        </div>);

    }

    const exportDateBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} >{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>);
    }

    const exportHostBodyTemplate = (item) => {

        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag}
            onDragEnd={onDropzoneLeave}
            onDragStart={(e) => {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
                leftContainer.current = d3Container.current.getBoundingClientRect();
                e.dataTransfer.setData("dragType", "exportBar");
                window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                inventory.current = item;
                if (inventory.current?.ex_params?.cluster_export) drawClusterDropzone();


            }}>
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{item?.host}</div>

        </div>
        );

    }

    const roleBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item?.cluster_role}</div>);
    }



    const backupExpansion = () => {
        return (
            <div className={((nodeSelected && nodeSelected?.backups?.length > 1) || props.node.backups?.length > 1) ? "expansion-body" : "expansion-body single"}>
                <DataTable
                    className="backup-expandable-table severity-table"
                    selectionMode="single"
                    dataKey="id"
                    selection={dbRow}
                    onSelectionChange={(e) => setDbRow(e.value)}
                    scrollable
                    scrollHeight="flex"
                    emptyMessage="no backups found"
                    paginator={props.node?.databases?.length > 0}
                    rows={layoutCurrHeight === config.layoutMinHeight ? 10 : 50}
                    alwaysShowPaginator={false}
                    value={props.node?.databases}>
                    <Column style={{ height: "21px" }} field="name" body={dbNameBodyTemplate}></Column>
                    <Column style={{ height: "21px" }} field="size" body={dbSizeBodyTemplate}></Column>
                </DataTable>
            </div>
        );
    }

    const showDbCredentialsInfo = (response) => {

        setInfoMessage('db_cred');
        setInfoIcon('pi pi-exclamation-triangle')
        setShowInfoDialog(response);

    }



    return (
        <div className="layout-item">
            <div className="layout-item-wrapper">
                <div className="flex flex-column flex-grow-1">
                    <div className="left-title flex flex-row">

                        <TabMenu className="layout-switch" model={layoutOptions} activeIndex={layoutSwitchIndex} onTabChange={(e) => {
                            console.log(props.node)
                            dispatch(LayoutActions.refreshContainer(props.node.mongos?.[0]?.id));
                            const index = e.index;
                            setIsPhysical(index === 0);
                            setLayoutSwitchIndex(index);
                        }} />
                        <div className="ml-auto mr-auto">{`${props.node?.ex_params?.what} cluster: ${props.node?.name}`}</div>
                        <div className="ml-auto">
                            <Button
                                style={{ fontSize: "1rem" }}
                                className="p-button-sm p-button-text"
                                label="status"
                                onClick={() => navigate('/status', { state: { cluster_id: props.node?.cluster } })}
                            />

                            {props.node?.nodes?.filter((n) => n.cluster_role === clusterRoles.PRIMARY)?.length > 0 ? <Button
                                style={{ fontSize: "1rem" }}
                                className="p-button-sm p-button-text"
                                label="cluster options"
                                onClick={(e) => clusterOptions.current?.show(e)}
                            /> : null}

                            <OverlayPanel ref={clusterOptions} onMouseLeave={(e) => clusterOptions.current?.hide(e)} closeOnEscape>

                                <div className="flex flex-column">

                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        label="drop"
                                        onClick={() => setShowDropCluster(true)}
                                    />

                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        disabled={props.node?.nodes?.filter((n) => n.cluster_role === clusterRoles.STANDBY)?.length === 0}
                                        id={`step-down-${layoutItemId.current}`}
                                        label="step down"
                                        ref={stepDownBtn}
                                        onClick={onStepDown}
                                    />

                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        label="info"
                                        onClick={() => setShowStatus(true)}
                                    />


                                </div>

                            </OverlayPanel>

                        </div>

                    </div>

                    <div style={{ zIndex: 1000 }} className="left-container">

                        <div className="left-content flex flex-row">

                            <div className="flex flex-column">

                                <svg
                                    className={`d3-component`}
                                    style={{ width: "800px", maxHeight: `${rightContainer.current?.offsetHeight > layoutCurrHeight ? rightContainer.current.offsetHeight + 5 : layoutCurrHeight + 5}px`, minHeight: `${layoutCurrHeight + 5}px` }}
                                    ref={d3Container}
                                />

                                {(!isPhysical && props.node?.databases?.length > rowsNumber) ? <Paginator className="left-container-paginator" first={first} rows={rows} totalRecords={props.node?.databases?.length} onPageChange={onPageChange} /> : null}

                            </div>


                        </div>
                    </div>

                </div>


                <div className="right-container" style={{ minHeight: `${layoutCurrHeight + 5}px` }}
                    ref={rightContainer}>
                    <TabView style={{ width: "500px" }} activeIndex={activeIndex} onTabChange={(e) => {
                        setActiveIndex(e.index);
                        tabRef.current = e.index
                    }}>
                        <TabPanel header="Shards">
                            <div className={`mongo-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "primaryCreate");
                                    drawClusterDropzone();
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me onto Sharded Cluster to create a Shard</div>
                                </div>
                                <ScrollPanel>
                                    <DataTable
                                        className="severity-table"
                                        emptyMessage="no shards found"
                                        scrollable
                                        scrollHeight="flex"
                                        paginator
                                        alwaysShowPaginator={false}
                                        rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                        value={props.node.nodes}>
                                        <Column style={{ height: "21px" }} header="Name" body={shardNameBodyTemplate}></Column>
                                        <Column style={{ height: "21px" }} header="Secondary" body={secondaryBodyTemplate}></Column>
                                        <Column style={{ height: "21px" }} header="Status" body={statusBodyTemplate} ></Column>
                                    </DataTable>
                                </ScrollPanel>

                            </div>
                        </TabPanel>
                        <TabPanel header="Nodes">
                            <div className={`mongo-tab-content-${layoutItemId.current} layout-tab-content`}>
                                {props.node?.nodes?.length > 0 ? <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "nodeCreate");
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me onto primary of a Sharded Cluster to add secondary</div>
                                </div> : null}

                                <Panel className="collapsible-panel mt-4" header={<div ref={thisNodes} onClick={() => thisNodes?.current?.toggle()} className="cursor-pointer">Nodes of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel>
                                        <DataTable
                                            className="severity-table"
                                            emptyMessage="no nodes found"
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={standbyRow}
                                            onSelectionChange={(e) => setStandbyRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={attachedMongoNodes}>
                                            <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                            <Column style={{ height: "21px" }} field="port" body={portBodyTemplate} header="Port" sortable></Column>
                                            <Column style={{ height: "21px" }} field="ex_params.lag" header="Lag" body={lagBodyTemplate} sortable></Column>
                                            <Column style={{ height: "21px" }} field="cluster_role" header="Role" body={roleBodyTemplate} sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>
                                <Panel collapsed className="collapsible-panel" header={<div ref={otherNodes} onClick={() => otherNodes?.current?.toggle()} className="cursor-pointer">Nodes of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel>
                                        <DataTable
                                            className="severity-table"
                                            emptyMessage="no nodes found"
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={standbyRow}
                                            onSelectionChange={(e) => setStandbyRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={foreignMongoNodes}>
                                            <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                            <Column style={{ height: "21px" }} field="port" body={portBodyTemplate} header="Port" sortable></Column>
                                            <Column style={{ height: "21px" }} field="ex_params.lag" header="Lag" body={lagBodyTemplate} sortable></Column>
                                            <Column style={{ height: "21px" }} field="cluster_role" header="Role" body={roleBodyTemplate} sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>

                            </div>
                        </TabPanel>
                        <TabPanel header="Export / Import">
                            <div className={`mongo-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> Drag any Node here to start a dump</div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />  Drag from the list onto a Primary to start restore</div>
                                {(nodeSelected) && <div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}> {`Exports for this ${isPhysical ? "node" : "database"} will be stored for ${nodeSelected?.size_scheme?.export} days`} </div> <div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: types.MONGO } })}>change</Tag> </div></div>}
                                <Panel className="collapsible-panel mt-4" header={<div ref={thisExports} onClick={() => thisExports?.current?.toggle()} className="cursor-pointer">Exports of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            selectionMode="single"
                                            className="severity-table"
                                            dataKey="id"
                                            selection={exportRow}
                                            onSelectionChange={(e) => setExportRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            emptyMessage="no exports found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={mongo_exports?.filter((e) => e.cluster === props.node?.id)}>
                                            <Column sortable style={{ height: "21px" }} field="host" header="Host" body={exportHostBodyTemplate}></Column>
                                            <Column sortable style={{ height: "21px" }} field="database" header="Database" body={(item) => item?.database ? item?.node_name : 'All databases'}></Column>
                                            <Column style={{ height: "21px" }} field="created_date" body={exportDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                                <Panel collapsed className="collapsible-panel" header={<div ref={otherExports} onClick={() => otherExports?.current?.toggle()} className="cursor-pointer"> Exports of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            selectionMode="single"
                                            className="severity-table"
                                            dataKey="id"
                                            selection={exportRow}
                                            onSelectionChange={(e) => setExportRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            emptyMessage="no exports found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={mongo_exports?.filter((e) => e.cluster !== props.node?.id)}>
                                            <Column sortable style={{ height: "21px" }} field="host" header="Host" body={exportHostBodyTemplate}></Column>
                                            <Column sortable style={{ height: "21px" }} field="database" header="Database" body={(item) => item?.database ? item?.node_name : 'All databases'}></Column>
                                            <Column style={{ height: "21px" }} field="created_date" body={exportDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                            </div>
                        </TabPanel>
                        <TabPanel header="Databases">
                            <div className={`mongo-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs mb-3" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "databaseCreate");
                                    drawClusterDropzone();
                                }}>
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me onto a Sharded Cluster to create a Database</div>
                                </div>
                                <div draggable={false} className="info-section text-xs ml-1"><i className="pi pi-circle-fill" />   Drag from the list onto a Sharded Cluster to start copying</div>
                                <Panel className="collapsible-panel verbose mt-4" header={<div ref={thisDbs} onClick={() => thisDbs?.current?.toggle()} className="cursor-pointer">Databases of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className="severity-table"
                                            selectionMode="single"
                                            emptyMessage="no databases found"
                                            dataKey="id"
                                            selection={dbRow}
                                            onSelectionChange={(e) => setDbRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={mongoDatabases?.filter((d) => d.cluster === props.node?.id)}>
                                            <Column style={{ height: "21px" }} field="name" body={dbNameBodyTemplate} header="Name" sortable></Column>
                                            <Column style={{ height: "21px" }} field="host" body={dbHostBodyTemplate} header="Host" sortable></Column>
                                            <Column style={{ height: "21px" }} field="size" body={dbSizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>

                                <Panel className="collapsible-panel verbose" collapsed header={<div ref={otherDbs} onClick={() => otherDbs?.current?.toggle()} className="cursor-pointer"> Databases of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className="severity-table"
                                            selectionMode="single"
                                            emptyMessage="no databases found"
                                            dataKey="id"
                                            selection={dbRow}
                                            onSelectionChange={(e) => setDbRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={mongoDatabases?.filter((d) => d.cluster !== props.node?.id)}>
                                            <Column style={{ height: "21px" }} field="name" body={dbNameBodyTemplate} header="Name" sortable></Column>
                                            <Column style={{ height: "21px" }} field="host" body={dbHostBodyTemplate} header="Host" sortable></Column>
                                            <Column style={{ height: "21px" }} field="size" body={dbSizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>

                            </div>

                        </TabPanel>
                        <TabPanel header="Backups">
                            <div className={`mongo-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> {nodeSelected ? `Drag backup to ${nodeSelected?.name} to restore:` : "Select node to view backups:"} </div>
                                {(nodeSelected) && <div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}>{`Backups for this node will be stored for ${nodeSelected?.size_scheme?.backup} days`}</div><div className="col"><Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: types.MONGO } })}>change</Tag></div></div>}
                                {nodeSelected ?
                                    getAutomaticBackup()
                                    : null}
                                <Panel className="collapsible-panel mt-4" header={<div ref={thisBackups} onClick={() => thisBackups?.current?.toggle()} className="cursor-pointer">Backups of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={backupRow}
                                            onSelectionChange={(e) => setBackupRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={mongo_backups?.filter((b) => b.cluster === props.node?.id && b.ex_params?.node_version === props.node?.ex_params?.version)}
                                            expandedRows={expandBackup}
                                            emptyMessage="no backups found"
                                            rowExpansionTemplate={backupExpansion}
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            onRowToggle={(e) => {
                                                setExpandBackup(e.data)
                                            }}>
                                            <Column style={{ height: '21px' }} field="created_date" body={backupDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                            <Column expander style={{ height: '21px', width: "0.75rem" }} />
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                                <Panel collapsed className="collapsible-panel" header={<div ref={otherBackups} onClick={() => otherBackups?.current?.toggle()} className="cursor-pointer">Backups of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={backupRow}
                                            onSelectionChange={(e) => setBackupRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={mongo_backups?.filter((b) => b.cluster !== props.node?.id && b.ex_params?.node_version === props.node?.ex_params?.version)}
                                            expandedRows={expandBackup}
                                            emptyMessage="no backups found"
                                            rowExpansionTemplate={backupExpansion}
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            onRowToggle={(e) => {
                                                setExpandBackup(e.data)
                                            }}>
                                            <Column style={{ height: '21px' }} field="created_date" body={backupDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                            <Column expander style={{ height: '21px', width: "0.75rem" }} />
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                            </div>
                        </TabPanel>
                    </TabView>
                </div>
            </div>
            {showAddNode ? <AddMongoShardedClusterNodeDialog
                visible={showAddNode}
                cluster={props.node}
                node={props.selected.current}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setMongoVersionExists(undefined));
                    setShowAddNode(false)

                }}
                onOkClick={onAddMongoNode}

            /> : null}

            {showWarning ? <WarningDialog
                message={warningMessage}
                visible={showWarning}
                onHide={() => {

                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowWarning(false)
                }
                } />
                : null}

            {showDropNode ? <DropMongoNodeDialog
                node={inventory.current}
                node_count={props.node.nodes?.length}
                arbiter={props.node?.nodes?.filter((n) => n?.cluster_role === clusterRoles.ARBITER)?.[0]}
                visible={showDropNode}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropNode(false)
                }}
                onOkClick={onDropNode}
            />
                : null}

            {showDumpNode ? <MongoDumpDialog
                node={props.selected.current}
                visible={showDumpNode}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setDirExists(null));
                    dispatch(LayoutActions.setExportExists(null));
                    dispatch(LayoutActions.setDiskSpace(null));
                    setShowDumpNode(false)
                }
                }
                onOkClick={onDumpNode}
            />
                : null}


            {showDumpDatabase ? <DumpDatabaseDialog
                db={props.selected.current}
                visible={showDumpDatabase}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setDirExists(null));
                    dispatch(LayoutActions.setExportExists(null));
                    dispatch(LayoutActions.setDiskSpace(null));
                    setShowDumpDatabase(false)
                }
                }
                onOkClick={onDumpDatabase}
            />
                : null}

            {showBackup ? <MongoNodeBackupDialog
                node={updateSchedule ? props.node : props.selected.current || props.node}
                updateSchedule={updateSchedule.current}
                pageIndex={pageIndex}
                visible={showBackup}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowBackup(false)
                }}
                onOkClick={onBackupNode}
            />
                : null}

            {showAddDatabase ? <AddMongoDbDialog
                node={props.node.mongos?.[0]}
                visible={showAddDatabase}
                onHide={() => {
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddDatabase(false);
                }
                }
                onOkClick={onAddDatabase}
            />
                : null}

            {showDropDatabase ? <DropDatabaseDialog
                node={inventory.current}
                visible={showDropDatabase}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropDatabase(false);
                }
                }
                onOkClick={onDropDatabase}
            />
                : null}
            {
                showRestoreDatabase ? <RestoreDatabaseLayoutDialog
                    visible={showRestoreDatabase}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestoreDatabase(false)
                    }}
                    onOkClick={onRestoreDatabase}
                    dump={inventory.current}
                /> : null
            }

            {
                showRestoreBackupNode ? <RestoreBackupMongoNodeLayout
                    visible={showRestoreBackupNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestoreBackupNode(false)
                    }
                    }
                    onOkClick={onRestoreBackupNode}
                    backup={inventory.current}
                    node={props.node}
                /> : null
            }


            {
                showRestoreDumpNode ? <RestoreDumpMongoNodeLayout
                    visible={showRestoreDumpNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestoreDumpNode(false)
                    }
                    }
                    onOkClick={onRestoreDumpNode}
                    dump={inventory.current}
                    node={props.selected.current}
                /> : null
            }


            {showDumpDetails ? <DumpDetailsDialog
                dump={inventory.current}
                visible={showDumpDetails}
                onHide={() => setShowDumpDetails(false)} />
                : null
            }

            {showBackupDetails ? <BackupDetailsDialog
                backup={inventory.current}
                visible={showBackupDetails}
                onHide={() => setShowBackupDetails(false)} />
                : null
            }

            {showDeleteInventory ? <DeleteInventoryDialog
                visible={showDeleteInventory}
                onHide={() => {
                    setShowDeleteInventory(false);
                    dispatch(LayoutActions.setErrorMessage(""));
                }}
                onOkClick={onDeleteInventory}
                item={inventory.current} />
                : null
            }

            {showCopyDb ? <CopyDbDialog
                visible={showCopyDb}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowCopyDb(false)
                }
                }
                onOkClick={onCopyDb}
                db={inventory.current}
                new={dbNew}
                source={databaseSource}
                node={props.node}
            />
                : null
            }

            {showStatus ? <MongoStatusDialog
                node={props.node}
                visible={showStatus}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowStatus(false)
                }} />
                : null
            }


            {showStepDown ? <StepdownShardPrimaryDialog
                primaries={props.node?.nodes}
                visible={showStepDown}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowStepDown(false)
                }}
                onOkClick={onStepDown}
            />
                : null
            }

            {showAddPrimary ? < AddMongoShardedClusterPrimaryDialog
                visible={showAddPrimary}
                primaryType={primaryType}
                cluster={props.node}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setMongoVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddPrimary(false)
                }}
                onOkClick={onAddMongoShardedClusterPrimary}
            /> : null}

            {showInfoDialog ? <InfoDialog
                message={infoMessage}
                visible={showInfoDialog}
                icon={infoIcon}
                onHide={() => {
                    infoMessage === 'db_cred' && setShowAddDatabase(false)
                    setShowInfoDialog(false);
                }
                } />
                : null}

            {showDropCluster ? <MongoDropClusterDialog
                node={props.node}
                service={props?.service}
                visible={showDropCluster}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropCluster(false)
                }}
                onOkClick={onDropCluster}
            />
                : null}


        </div>
    );


}


export default MongoShardedLayoutItem;