import { useState, useRef, useEffect, useCallback } from "react";
import { useDispatch, useSelector } from "react-redux";
import { TabView, TabPanel } from 'primereact/tabview';
import { nanoid } from 'nanoid';
import * as d3 from "d3";
import trashImage from "../../../assets/images/trash.png";
import dbImage from "../../../assets/images/db.png";
import plusIcon from "../../../assets/images/plus.png";
import minusIcon from "../../../assets/images/minus.png";
import node_white from "../../../assets/images/node_white.png";
import node_unavail from '../../../assets/images/node_unavail.png'
import node_un_white from '../../../assets/images/node_un_white.png';
import db_white from "../../../assets/images/db_white.png";
import node from "../../../assets/images/node.png";
import bar from "../../../assets/images/bar.png";
import db_unavail from "../../../assets/images/db_unavail.png";
import db_un_white from "../../../assets/images/db_un_white.png";
import cluster from "../../../assets/images/cluster.png";
import { LayoutActions } from "../../../store/layout/reducer";
import { useNavigate } from 'react-router-dom';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import moment from "moment";
import AddPDBDialog from "../../../dialogs/oracle/addPDBDialog";
import DetailsPDBDialog from "../../../dialogs/oracle/detailsPDBDialog";
import CopyPDBDialog from "../../../dialogs/oracle/copyPDBDialog";
import DeletePDBDialog from "../../../dialogs/oracle/deletePDBDialog";
import BackupDetailsDialog from "../../../dialogs/oracle/backupDetailsDialog";
import RestorePDBLayoutDialog from "../../../dialogs/oracle/restorePDBLayoutDialog";
import DuplicateStandaloneDialog from "../../../dialogs/oracle/duplicateStandaloneDialog";
import ExportDetailsDialog from "../../../dialogs/oracle/exportDetailsDialog";
import ImportPDBLayoutDialog from "../../../dialogs/oracle/importPDBLayoutDialog";
import DeleteCDBDialog from "../../../dialogs/oracle/deleteCDBDialog";
import BackupCDBDialog from "../../../dialogs/oracle/backupCDBDialog";
import BackupPDBDialog from "../../../dialogs/oracle/backupPDBDialog";
import FlashbackCDBPDBDialog from "../../../dialogs/oracle/flashbackPDBCDBDialog";
import FlashbackDetailsDialog from "../../../dialogs/oracle/flashbackDetailsDialog";
import RestoreFlashbackLayoutDialog from "../../../dialogs/oracle/restoreFlashbackLayoutDialog";
import ImportStandaloneDialog from "../../../dialogs/oracle/importStandaloneDialog";
import ExportLayoutDialog from "../../../dialogs/oracle/exportLayoutDialog";
import WarningDialog from "../../../dialogs/warningDialog";
import DeleteInventoryDialog from "../../../dialogs/deleteInventoryDialog";
import { onSort } from "../../../util/table";
import InfoDialog from "../../../dialogs/infoDialog";
import getColorBySeverity from "../../../util/colorBySeverity";
import config, { clusterRoles, types } from "../../../config";
import { knobs } from "../../../util/templates";
import { Tag } from "primereact/tag";
import { ScrollPanel } from "primereact/scrollpanel";
import { ConfirmPopup } from "primereact/confirmpopup";
import { Paginator } from "primereact/paginator";
import { Button } from "primereact/button";

const OracleLayoutItem = (props) => {
    const dispatch = useDispatch();
    const { standalones } = useSelector((state) => state.layout);
    const navigate = useNavigate();

    const [showAddPDBDialog, setShowAddPDBDialog] = useState(false);
    const [showPdbs, setShowPdbs] = useState(false);
    const d3Container = useRef(null);
    const [activeIndex, setActiveIndex] = useState(0);
    const [showDeletePDB, setShowDeletePDB] = useState(false);
    const [pdbSelected, setPDBSelected] = useState(false);
    const [cdbSelected, setCDBSelected] = useState(false);
    const [attachedNodes, setAttachedNodes] = useState(null);
    const [showPDBDetails, setShowPDBDetails] = useState(false);
    const [showCopy, setShowCopy] = useState(false);
    const [showBackupDetails, setShowBackupDetails] = useState(false);
    const [showExportDetails, setShowExportDetails] = useState(false);
    const currentPdbCoords = useRef({ x: 0, y: 0 });
    const tabRef = useRef(0);
    const [showRestorePDB, setShowRestorePDB] = useState(false);
    const [showImport, setShowImport] = useState(false);
    const [showDuplicateStandalone, setShowDuplicateStandalone] = useState(false);
    const [showDeleteCDBDialog, setShowDeleteCDBDialog] = useState(false);
    const [showBackupCDB, setShowBackupCDB] = useState(false);
    const [showBackupPDB, setShowBackupPDB] = useState(false);
    const [showExport, setShowExport] = useState(false);
    const [showFlashback, setShowFlashback] = useState(false);
    const [showFlashbackDetails, setShowFlashbackDetails] = useState(false);
    const [showRestoreFlashbackPDB, setShowRestoreFlashbackPDB] = useState(false);
    const inventory = useRef(null);
    const [showDeleteInventory, setShowDeleteInventory] = useState(false);
    const [showImportStandalone, setShowImportStandalone] = useState(false);
    const [warningMessage, setWarningMessage] = useState('');
    const [showWarning, setShowWarning] = useState(false);
    const leftContainer = useRef(null);
    const rightContainer = useRef(null);
    const tabsPos = useRef(null);
    const trashPos = useRef(null);
    const [pdbRow, setPDBRow] = useState(null);
    const [backupRow, setBackupRow] = useState(null);
    const [flashbackRow, setFlashbackRow] = useState(null);
    const [exportRow, setExportRow] = useState(null);
    const [standaloneRow, setStandaloneRow] = useState(null);
    const [oracleStandalones, setOracleStandalones] = useState(null);
    const updateSchedule = useRef(null);
    const [showInfoDialog, setShowInfoDialog] = useState(false);
    const [infoMessage, setInfoMessage] = useState(false);
    const layoutItemId = useRef(null);
    const flashbackAllowed = useRef(null);
    const layoutCurrHeight = config.layoutMinHeight;
    const layoutMid = config.layoutMinHeight / 2;
    const [removeScheduleConfirm, setRemoveScheduleConfirm] = useState(false);
    const [pageIndex, setPageIndex] = useState(0);
    const rowsNumber = 6;
    const [first, setFirst] = useState(0);
    const [rows, setRows] = useState(rowsNumber);
    const [nodesPage, setNodesPage] = useState(props.node?.nodes?.length > rowsNumber ? props.node?.nodes?.slice(props.node?.nodes?.length - (rowsNumber + 1), props.node.nodes?.length - 1) : props.node?.nodes);
    const node_x = 450;

    useEffect(() => {

        if (props.node?.pdb?.length <= rowsNumber) setNodesPage(props.node?.nodes);

    }, [props.node])

    const onPageChange = (event) => {
        let rowsPerPage = rowsNumber;
        let startIndex = (event.page) * rowsPerPage;
        if (event.rows < rowsPerPage) {
            rowsPerPage = rowsPerPage - event.rows
            startIndex -= rowsPerPage
        }
        let endIndex = startIndex + rowsPerPage;
        setFirst(event.first);
        setRows(event.rows);
        setNodesPage(props.node?.nodes?.slice(startIndex, endIndex));

    };

    const getAutomaticBackup = () => {
        let backup = props.node?.backups?.filter((b) => b.is_sched === true)?.[0]
        if (!backup) backup = pdbSelected?.backups?.filter((b) => b.is_sched === true)?.[0]
        updateSchedule.current = backup;
        const popup_id = `remove-schedule-button-${props.node.id}`
        return (<div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}> Automatic backup: {backup?.ex_params?.description || "No"}</div> {backup ? <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-pencil" onClick={() => {
            pdbSelected ? setShowBackupPDB(true) : setShowBackupCDB(true);
        }}>change</Tag> <Tag className='right-container-button ml-2' id={popup_id} icon="pi pi-trash" severity="danger" onClick={() => setRemoveScheduleConfirm(true)} >delete</Tag>    <ConfirmPopup
                target={document.getElementById(popup_id)}
                visible={removeScheduleConfirm}
                onHide={() => setRemoveScheduleConfirm(false)}
                message="Are you sure?"
                accept={() => dispatch(LayoutActions.removeOracleSchedule({ "node_id": props.node.id }, setRemoveScheduleConfirm))}
            /></div> : <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-plus" onClick={() => {

                setPageIndex(1);
                pdbSelected ? setShowBackupPDB(true) : setShowBackupCDB(true);
            }}>create</Tag> </div>}  </div>)
    }


    const onVerifyBackup = (open, message) => {

        setInfoMessage(message);
        setShowInfoDialog(open);

    }

    const addImage = useCallback((x, y, pdb) => {

        function onClickPDB(svg, id, pdb) {
            setCDBSelected(false);
            props.selected.current = pdb;
            setPDBSelected(props.selected.current);
            let elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute("slave_of") === "false" ? dbImage : node : elements[i].getAttribute("slave_of") === "false" ? db_unavail : node_unavail);
            }

            elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }

            elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }

            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            svg.select(`#text-${id}`).style("fill", "white");

            svg.select("#node-" + id).attr("xlink:href", pdb.ind_deleted ? node_un_white : node_white);

        }

        function goPDBDetails(pdb) {

            if (pdb.ind_deleted) {

                showWarningMessage({ "text": `${pdb.name} is no longer available.\nDetails cant be viewed.`, "status": pdb.status })

            }
            else navigate(`/status/oracle/cdb/${props.node.id}/pdb/${pdb.id}`);

        }

        const onDragStart = (svg, id, pdb) => (d) => {
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();

            onClickPDB(svg, id, pdb);

        }

        const onDragging = (svg, id, pdb) => (d) => {


            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }
            if (d.sourceEvent.clientX < leftContainer.current.left || d.sourceEvent.clientY > leftContainer.current.bottom || d.sourceEvent.clientY < tabsPos.current.top || d.sourceEvent.clientX > tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            setPDBSelected(pdb);
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#node-" + id).style("cursor", "grabbing");
                d3.select("#node-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx) + "," + (dy) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 51) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", "translate(" + (dx + 40) + "," + (dy + 10) + ")");

                if (d.sourceEvent.clientX > trashPos.current.left && d.sourceEvent.clientY < trashPos.current.bottom && d.sourceEvent.clientX < trashPos.current.right && d.sourceEvent.clientY > trashPos.current.top) {

                    svg.select(".trash").style("opacity", "1.0");

                }
                else {

                    svg.select(".trash").style("opacity", "0.5");

                }


                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                    switch (tabRef.current) {
                        case 0:
                            svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                        case 2:
                            svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                        case 3:
                            if (parseFloat(props.selected.current?.version) <= 12.1)
                                svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                        default:
                            break;
                    }
                }

                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 100) {
                    setActiveIndex(0);
                    tabRef.current = 0;
                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 100 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 200) {
                    setActiveIndex(1);
                    tabRef.current = 1;
                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 200 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 300) {
                    setActiveIndex(2);
                    tabRef.current = 2;

                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 300 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 400) {
                    setActiveIndex(3);
                    tabRef.current = 3;

                }
                else if (d.sourceEvent.clientX > tabsPos.current.left + 400 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 500) {
                    setActiveIndex(4);
                    tabRef.current = 4;
                }

            }
            let elements = window.document.querySelectorAll(`.oracle-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = (tabRef.current === 3 && !flashbackAllowed.current) ? "0 0 0 0.2rem rgb(220 53 69 / 72%)" : (tabRef.current === 0 || tabRef.current === 2) ? "" : "0 0 0 0.2rem rgb(40 167 69 / 72%)";

            }
        }

        const onDragEnd = (svg, id, pdb) => (d) => {

            if (d.sourceEvent.detail === 2) {

                goPDBDetails(pdb);
                return;

            }

            setPDBSelected(pdb);
            if (d.sourceEvent.clientX > trashPos.current.left && d.sourceEvent.clientY < trashPos.current.bottom && d.sourceEvent.clientX < trashPos.current.right && d.sourceEvent.clientY > trashPos.current.top) {

                if (pdb.ind_deleted) {


                    showWarningMessage({

                        "text": `${pdb.name.toUpperCase()} is unavailable`,
                        "status": `${pdb.status}`
                    });
                }
                else setShowDeletePDB(true);

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {

                switch (tabRef.current) {
                    case 1:
                        if (pdb.ind_deleted) {


                            showWarningMessage({

                                "text": `${pdb.name.toUpperCase()} is unavailable`,
                                "status": `${pdb.status}`
                            });
                        }
                        else setShowExport(true);
                        break;
                    case 4:
                        if (pdb.ind_deleted) {


                            showWarningMessage({

                                "text": `${pdb.name.toUpperCase()} is unavailable`,
                                "status": `${pdb.status}`
                            });
                        }

                        else {

                            setPageIndex(0);
                            setShowBackupPDB(true);
                        }
                        break;
                    case 3:
                        if (flashbackAllowed.current) {
                            if (pdb.ind_deleted) {


                                showWarningMessage({

                                    "text": `${pdb.name.toUpperCase()} is unavailable`,
                                    "status": `${pdb.status}`
                                });
                            }

                            else setShowFlashback(true);

                        }

                        break;
                    default:
                        break;

                }

            }

            d3.select("#node-" + id).remove();
            d3.select("#line-" + id).remove();
            d3.select("#text-" + id).remove();
            d3.select("#severity-" + id).remove();
            d3.select("#d3-tooltip-container-" + id).remove();

            if (nodesPage?.length > 0) {

                let i = 0;
                for (; i < nodesPage?.length; ++i)
                    if (nodesPage[i].id === pdb.id)
                        break;

                const node_y = (i + 0.5) * (layoutCurrHeight / nodesPage.length);
                currentPdbCoords.current.x = node_x;
                currentPdbCoords.current.y = node_y + 20;
                addImage(node_x, node_y, pdb);

            }




            svg.select(".trash").style("opacity", "0.0");
            d3.select("#node-" + id).style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.oracle-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }

        }


        const svg = d3.select(d3Container.current);
        const id = nanoid();

        const image = svg.insert("image")
            .attr("xlink:href", (props.selected.current?.id === pdb.id) ? pdb.ind_deleted ? node_un_white : node_white : pdb.ind_deleted ? node_unavail : node)
            .attr("class", "node")
            .attr("id", "node-" + id)
            .attr('x', 0)
            .attr('y', 0)
            .attr("width", 40)
            .attr("height", 60)
            .attr("available", pdb.ind_deleted ? false : true)
            .attr("draggable", true)
            .attr("transform", () => "translate(" + x + "," + (y - 20) + ")")
            .on("mouseover", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

            })
            .on("mouseout", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

            })

        const x0 = 100;
        const y0 = layoutMid + 10;
        const x1 = x;
        const y1 = y;

        const path = d3.path()
        path.moveTo(x1, y1)
        path.bezierCurveTo((x0 + x1) / 2, y1, (x0 + x1) / 2, y0, x0, y0);

        svg.insert("path")
            //.attr("d", d3.line()([[100, 220], [x, y + 30]]))
            .attr("d", path)
            .attr("id", "line-" + id).attr("transform", "translate(" + (0) + "," + (10) + ")")
            .style("fill", "none")
            .style("stroke", "#ccc")
            .style("stroke-width", 1);

        svg.insert("text")
            .attr("transform", "translate(" + (x) + "," + (y - 14) + ")")
            .attr("id", "text-" + id)
            .attr("class", "node-what-text")
            .style("font-size", "12px")
            .style("fill", `${props.selected.current?.id === pdb.id ? "white" : "gray"}`)
            .style("cursor", "pointer")
            .text(`${pdb.name ? pdb.name : ""} ${pdb.status ? pdb.status : ""}`).on("dblclick", () => goPDBDetails(pdb)).on("click", function (d) {
                currentPdbCoords.current.x = x1;
                currentPdbCoords.current.y = y1 + 20;
                onClickPDB(svg, id, pdb);
            });

        //draw pdb severity strip
        svg.insert("rect").attr('x', 0)
            .attr('y', 0)
            .attr("width", 35)
            .attr("height", 5).attr("fill", getColorBySeverity(pdb.severity))
            .attr("transform", "translate(" + (x1 + 2) + "," + (y1 + 31) + ")")
            .attr("id", "severity-" + id)

        svg.insert("foreignObject")
            .attr("class", "d3-tooltip-container")
            .attr("id", `d3-tooltip-container-${id}`)
            .attr("transform", "translate(" + (x + 40) + "," + (y + 10) + ")")
            .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${pdb.ind_started ? "up" : "down"} <br/> Alerts: ${pdb?.alert?.length > 40 ? pdb.alert.substring(0, 37).concat("...") : pdb?.alert} </div>`)


        var drag_behavior = d3.drag()
            .on("start", onDragStart(svg, id, pdb))
            .on("drag", onDragging(svg, id, pdb))
            .on("end", onDragEnd(svg, id, pdb));

        image.call(drag_behavior);
    }, [props.selected, props.node, layoutCurrHeight, nodesPage, layoutMid, navigate]);



    useEffect(() => {
        const onTableItemDrop = (event) => {
            let type = event.dataTransfer.getData("dragType");
            switch (type) {

                case "backupBar":
                    setShowDeleteInventory(true);
                    break;
                case "exportBar":
                    setShowDeleteInventory(true);
                    break;
                case "flashbackBar":
                    setShowDeleteInventory(true);
                    break;
                case "standaloneBar":
                    setShowDeleteInventory(true);
                    break;
                default:
                    break;

            }

        }

        function goCDBDetails() {
            if (props.node.ind_deleted) {

                showWarningMessage({ "text": `${props.node.name} is no longer available.\nDetails cant be viewed.`, "status": props.node.status })

            }
            else navigate(`/status/oracle/cdb/${props.node.id}`);

        }


        function onCDBClick(svg, id) {

            setPDBSelected(false);

            let elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }


            elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }

            elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute("slave_of") === "false" ? dbImage : node : elements[i].getAttribute("slave_of") === "false" ? db_unavail : node_unavail);
            }


            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }


            svg.select(".root").attr("xlink:href", props.node.ind_deleted ? db_un_white : db_white);
            props.selected.current = props.node;
            setCDBSelected(true);
            svg.select(`#text-${id}`).style("fill", "white");


        }


        const onDragStartCdb = (svg, id, cdb) => (d) => {

            // Draw cdb what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 18) + ")")
                .attr("id", "text-copy" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node.id ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.location ? props.node.location : ""} ${props.node.status ? props.node.status : ""}`)

            // Draw cdb icon
            svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id) ? props.node.ind_deleted ? db_un_white : db_white : props.node.ind_deleted ? db_unavail : dbImage)
                .attr("class", "root")
                .attr("id", "root-copy" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")")
                .attr("available", !props.node.ind_deleted);


            //draw cdb severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-copy" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")")


            // Draw cdb plus/minus icon

            if (props.node.nodes && props.node.nodes?.length > 0) {

                svg.insert("image")
                    .attr("xlink:href", showPdbs ? minusIcon : plusIcon)
                    .attr("id", "expand-copy" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 15) + ")")
            }

            onCDBClick(svg, id);
            props.selected.current = cdb;
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();


        }

        const onDraggingCdb = (svg, id, cdb) => (d) => {
            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }
            if (d.sourceEvent.clientX < leftContainer.current.left || d.sourceEvent.clientY > leftContainer.current.bottom || d.sourceEvent.clientY < tabsPos.current.top || d.sourceEvent.clientX > tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#root-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx - 54) + "," + (dy - 8) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 40) + ")");
                d3.select("#expand-" + id).attr("transform", () => "translate(" + (dx + 30) + "," + (dy - 5) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", () => "translate(" + (dx + 50) + "," + (dy - 5) + ")");

            }

            svg.select(".root").style("cursor", "grabbing");
            if (d.sourceEvent.clientX > trashPos.current.left && d.sourceEvent.clientY < trashPos.current.bottom && d.sourceEvent.clientX < trashPos.current.right && d.sourceEvent.clientY > trashPos.current.top) {

                svg.select(".trash").style("opacity", "1.0");

            }
            else {

                svg.select(".trash").style("opacity", "0.5");

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                switch (tabRef.current) {
                    case 0:
                        svg.select(".root").style("cursor", "not-allowed");
                        break;
                    case 2:
                        svg.select(".root").style("cursor", "not-allowed");
                        break;
                    case 3:
                        if (parseFloat(props.selected.current?.version) <= 12.1)
                            svg.select(".root").style("cursor", "not-allowed");
                        break;
                    default:
                        break;
                }
            }



            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 100) {
                setActiveIndex(0);
                tabRef.current = 0;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 100 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 200) {
                setActiveIndex(1)
                tabRef.current = 1;
            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 200 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 300) {
                setActiveIndex(2);
                tabRef.current = 2;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 300 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 400) {
                setActiveIndex(3);
                tabRef.current = 3;


            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 400 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 500) {
                setActiveIndex(4);
                tabRef.current = 4;
            }

            let elements = window.document.querySelectorAll(`.oracle-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = (tabRef.current === 3 && !flashbackAllowed.current) ? "0 0 0 0.2rem rgb(220 53 69 / 72%)" : (tabRef.current === 0 || tabRef.current === 2) ? "" : " 0 0 0 0.2rem rgb(40 167 69 / 72%)";

            }

        }

        const onDragEndCdb = (svg, id, cdb) => (d) => {

            if (d.sourceEvent.detail === 2) {

                goCDBDetails();
                return;

            }

            if (d.sourceEvent.clientX > trashPos.current.left && d.sourceEvent.clientY < trashPos.current.bottom && d.sourceEvent.clientX < trashPos.current.right && d.sourceEvent.clientY > trashPos.current.top) {

                if (props.node.ind_deleted)
                    showWarningMessage({

                        "text": `${props.node.name.toUpperCase()} is unavailable`,
                        "status": `${props.node.status}`
                    });
                else setShowDeleteCDBDialog(true);


            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {

                switch (tabRef.current) {
                    case 4:
                        if (props.node.ind_deleted)
                            showWarningMessage({

                                "text": `${props.node.name.toUpperCase()} is unavailable`,
                                "status": `${props.node.status}`
                            });
                        else {

                            setPageIndex(0);
                            setShowBackupCDB(true);
                        }
                        break;
                    case 1:
                        if (props.node.ind_deleted)
                            showWarningMessage({

                                "text": `${props.node.name.toUpperCase()} is unavailable`,
                                "status": `${props.node.status}`
                            });
                        else setShowExport(true);
                        break;
                    case 3:
                        if (flashbackAllowed.current) {
                            if (props.node.ind_deleted)
                                showWarningMessage({

                                    "text": `${props.node.name.toUpperCase()} is unavailable`,
                                    "status": `${props.node.status}`
                                });
                            else setShowFlashback(true);

                        }
                        break;
                    default:
                        break;

                }
            }



            svg.select(".trash").style("opacity", "0.0");

            d3.select("#root-copy" + id).remove();
            d3.select("#text-copy" + id).remove();
            d3.select("#severity-copy" + id).remove();
            d3.select("#expand-copy" + id).remove();

            d3.select("#root-" + id).attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")");
            d3.select("#text-" + id).attr("transform", () => "translate(" + 0 + "," + (layoutMid - 18) + ")");
            d3.select("#severity-" + id).attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")");
            d3.select("#expand-" + id).attr("transform", "translate(" + (80) + "," + (layoutMid - 15) + ")");
            d3.select(`#d3-tooltip-container-${id}`).attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
            svg.select(".root").style("cursor", "pointer");

            let elements = window.document.querySelectorAll(`.oracle-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        if (d3Container.current) {
            const svg = d3.select(d3Container.current);
            svg.selectChildren("*").remove();

            flashbackAllowed.current = parseFloat(props.node.version) > 12.1;
            if (!props.selected.current) {
                setCDBSelected(props.node);

            }

            const id = nanoid();

            svg.insert("foreignObject")
                .attr("class", "d3-tooltip-container")
                .attr("id", `d3-tooltip-container-${id}`)
                .attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
                .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${props.node.ind_started ? "up" : "down"} <br/> Alerts: ${props.node?.alert?.length > 40 ? props.node.alert.substring(0, 37).concat("...") : props.node?.alert}</div>`)

            // Draw trash icon    
            svg.insert("image")
                .attr("xlink:href", trashImage)
                .attr("class", "trash")
                .attr("id", `trash-${id}`)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 100)
                .attr("height", 100)
                .attr("transform", () => "translate(" + 0 + "," + (layoutCurrHeight - 100) + ")")
                .on("dragover", dragOverTrash)
                .on("dragleave", dragLeaveTrash)
                .on("drop", onTableItemDrop);

            // Draw cdb what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 18) + ")")
                .attr("id", "text-" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${(props.selected.current?.id === props.node.id || !layoutItemId.current) ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.location ? props.node.location : ""} ${props.node.status ? props.node.status : ""}`)
                .on("dblclick", goCDBDetails)
                .on("click", () => onCDBClick(svg, id));

            // Draw cdb icon
            const imageCdb = svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id || !layoutItemId.current) ? props.node.ind_deleted ? db_un_white : db_white : props.node.ind_deleted ? db_unavail : dbImage)
                .attr("class", "root")
                .attr("id", "root-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")")
                .attr("available", !props.node.ind_deleted)
                .on("mouseover", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

                })
                .on("mouseout", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

                });

            layoutItemId.current = id;

            const dragBehaviorCdb = d3.drag()
                .on("start", onDragStartCdb(svg, id, props.node))
                .on("drag", onDraggingCdb(svg, id, props.node))
                .on("end", onDragEndCdb(svg, id, props.node));
            imageCdb.call(dragBehaviorCdb);


            //draw cdb severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")")


            // Draw cdb plus/minus icon

            if (props.node.nodes && props.node.nodes?.length > 0) {

                svg.insert("image")
                    .attr("xlink:href", showPdbs ? minusIcon : plusIcon)
                    .attr("id", "expand-" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 15) + ")")
                    .on("click", () => { dispatch(LayoutActions.refreshContainer(props.node?.id)); setShowPdbs(!showPdbs); });

            }

            if (showPdbs && nodesPage?.length > 0) {

                nodesPage?.forEach((pdb, index) => {
                    addImage(node_x, ((index + 0.5) * (layoutCurrHeight / nodesPage.length)), pdb);
                });
            }
        }
    }, [showPdbs, props.node, props.selected, layoutCurrHeight, layoutMid, nodesPage, navigate, dispatch, addImage]);


    useEffect(() => {

        let temp;
        if (standalones?.length > 0) temp = [...standalones];
        else temp = [];
        setOracleStandalones(temp);

    }, [standalones])




    useEffect(() => {

        const updateSelected = () => {


            if (props.selected.current?.what?.toLowerCase().includes("ORACLE")) {

                if (props.selected.current?.id === props.node.id) {

                    let nodeSelected = props.selection?.filter((n) => n.id === props.selected.current?.id);
                    if (nodeSelected?.length > 0) {

                        setCDBSelected(nodeSelected[0]);

                        props.selected.current = nodeSelected[0]

                    }

                }
                else {

                    let nodeSelected = props.node.nodes?.filter((n) => n.id === props.selected.current?.id);
                    if (nodeSelected?.length > 0) {

                        setPDBSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0]

                    }

                }


            }

        }

        if (props.selection && props.selection?.length > 0) {
            const tempData = [];
            props.selection?.filter((n) => n.nodes && n.nodes?.length > 0).forEach((n) => n.nodes.forEach((p) => tempData.push(p)));

            setAttachedNodes(tempData);
            updateSelected()

        }




        if (props.selection?.length > 0) updateSelected();



    }, [props.selection, props.selected, props.node]);

    function dragLeaveNode(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", "transparent");

    }


    const onDeleteInventory = (data) => {

        switch (inventory.current.type) {

            case "FLASHBACK":
                dispatch(LayoutActions.dropFlashback(data, setShowDeleteInventory));
                break;
            case "EXPORT":
                dispatch(LayoutActions.dropExport(data, setShowDeleteInventory));
                break;
            case "BACKUP":
                dispatch(LayoutActions.dropBackup(data, setShowDeleteInventory));
                break;
            case "STANDALONE":
                dispatch(LayoutActions.dropStandalone(data, setShowDeleteInventory));

                break;
            default:
                break;

        }

    }

    function dragToCDBEnd(event) {
        if (inventory.current?.status === false) {
            showWarningMessage({

                "text": "Cannot perform this action",
                "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
            })

            return;
        }

        let type = event.dataTransfer.getData("dragType");

        if (props.node.ind_deleted) {
            showWarningMessage({

                "text": `${props.node.name.toUpperCase()} is unavailable`,
                "status": `${props.node.status}`
            });

            return;

        }



        switch (type) {

            case "pdbBar":
            case "pdbCreate":
                if (props.node?.ex_params?.standalone) {
                    showWarningMessage({

                        "text": `This CDB are standalone.Could not add PDB.`
                    });
                    return;
                }
                if (type === "pdbBar") {

                    let sourceVersion = parseFloat(props.selection?.filter((n) => n.id === inventory.current?.ex_params.cdb_id)[0]?.version);
                    let targetVersion = parseFloat(props.node?.version);
                    if (targetVersion !== sourceVersion) {

                        showWarningMessage({

                            "text": `Source container version ${sourceVersion} does not match target container version ${targetVersion}`
                        });


                    }
                    else setShowCopy(true);

                }
                else setShowAddPDBDialog(true);
                break;
            case "standaloneBar":
                if (props.node?.ex_params?.standalone) {
                    showWarningMessage({

                        "text": `This CDB are standalone.Could not add PDB.`
                    });
                    return;
                }
                setShowDuplicateStandalone(true);
                break;
            case "exportBar":
                setShowImport(true);
                break;
            default:
                break;

        }


    }

    const onDeleteCDB = (data) => {

        dispatch(LayoutActions.deleteCDBNode(data, setShowDeleteCDBDialog));

    }

    function dragToPDBEnd(event) {


        if (inventory.current?.status === false) {
            showWarningMessage({

                "text": "Cannot perform this action",
                "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
            });

            return;



        }


        if (pdbSelected.ind_deleted) {
            showWarningMessage({

                "text": `${props.node.name.toUpperCase()} is unavailable`,
                "status": `${props.node.status}`
            });

            return;

        }

        let type = event.dataTransfer.getData("dragType");

        switch (type) {
            case "backupBar":
                setShowRestorePDB(true);
                break;
            case "exportBar":
                setShowImport(true);
                break;
            case "flashbackBar":
                setShowRestoreFlashbackPDB(true);
                break;
            default:
                break;
        }
    }

    function dragOverCDB(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");
    }

    function dragOverPDB(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    }

    const dragOverTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "1.0");

    }


    const dragLeaveTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.5");

    }

    const backupDateBodyTemplate = (item) => {
        return (
            <div className="flex align-items-center" draggable onDrag={onTableItemDrag}
                onDragEnd={onDropzoneLeave}
                onDragStart={(e) => {
                    leftContainer.current = d3Container.current.getBoundingClientRect();
                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                    window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                    inventory.current = item;
                    if (cdbSelected) drawCDBDropzone();
                    else drawPDBDropzone();
                    e.dataTransfer.setData("dragType", "backupBar");

                }}>
                {knobs}
                <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>
            </div>



        );
    }

    const exportHostBodyTemplate = (item) => {

        return (<div className="flex align-items-center icon-cell" draggable onDrag={onTableItemDrag}
            onDragEnd={onDropzoneLeave}
            onDragStart={(e) => {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
                leftContainer.current = d3Container.current.getBoundingClientRect();
                d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
                window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                inventory.current = item;
                if (cdbSelected)
                    drawCDBDropzone();
                else
                    drawPDBDropzone();
                e.dataTransfer.setData("dragType", "exportBar");


            }}>
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{item?.host}</div>
        </div>);

    }

    const flashbackNameBodyTemplate = (item) => {

        return (<div className="flex align-items-center icon-cell" draggable onDrag={onTableItemDrag}
            onDragEnd={onDropzoneLeave}
            onDragStart={(e) => {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
                leftContainer.current = d3Container.current.getBoundingClientRect();
                window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                inventory.current = item;
                pdbSelected ? drawPDBDropzone() : drawCDBDropzone()
                e.dataTransfer.setData("dragType", "flashbackBar");


            }} >
            {knobs}
            <div className="flex align-items-center icon-cell" style={{ paddingLeft: "5px", height: "20px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}` }}>{item.name} <img alt="flashbackBar" src={bar} draggable={false} className="bar-icon" onClick={() => {
                inventory.current = item;
                setShowFlashbackDetails(true);
            }} /></div>

        </div>);

    }


    const drawPDBDropzone = () => {
        d3.select(d3Container.current).insert("rect")
            .attr("class", "drop-zone-marker")
            .attr("x", currentPdbCoords.current.x - 30)
            .attr("y", currentPdbCoords.current.y - 40)
            .attr("width", 100)
            .attr("height", 100)
            .attr("fill", "transparent")
            .attr("stroke", inventory.current?.status === false ? "red" : "green")
            .on("drop", dragToPDBEnd)
            .on("dragover", dragOverPDB)
            .on("dragleave", dragLeaveNode);
    }



    const sizeBodyTemplate = (item) => {
        return (<div className="flex align-items-center icon-cell backup">{item.display_size} <i className="pi pi-check verify-backup" onClick={() => {

            dispatch(LayoutActions.verifyBackup({ "name": item?.name, "type": item?.type, "backup_host": item?.host_name, "backup_path": item?.ex_params?.["backup_path"], "backup_date": item?.ex_params?.["modified_date"] }, onVerifyBackup))

        }} /> <img className="bar-icon" alt="backupBar" src={bar} draggable={false} onClick={() => {
            inventory.current = item;
            switch (item.type) {

                case "BACKUP":
                    setShowBackupDetails(true);
                    break;
                case "EXPORT":
                    setShowExportDetails(true);
                    break;
                default:
                    break;

            }
        }} /></div>);
    }


    const exportDateBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} >{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>);
    }

    const hostBodyTemplate = (item) => {
        return (<div className="flex align-items-center icon-cell">{item?.host} {<img alt="pdbBar" src={bar} draggable={false} className="bar-icon" onClick={() => {
            setPDBSelected(item);
            setShowPDBDetails(true);
        }} />} </div>);
    }

    const cdbBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.cdb_name}</div>);
    }

    const pdbBodyTemplate = (item) => {
        return (
            <div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
                inventory.current = item;
                drawCDBDropzone();
                leftContainer.current = d3Container.current.getBoundingClientRect();
                d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
                e.dataTransfer.setData("dragType", "pdbBar");

            }}>
                {knobs}
                <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item.severity)}`, height: "20px" }}>{item.name}</div>
            </div>


        );
    }

    const onDropzoneLeave = (e) => {
        e.target.style.position = "";
        e.target.style.left = "";
        e.target.style.top = "";
        e.target.style.visibility = ""; d3.selectAll(".drop-zone-marker").remove();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.0");
    }

    const drawCDBDropzone = () => {


        d3.select(d3Container.current).insert("rect")
            .attr("class", "drop-zone-marker")
            .attr("x", 20)
            .attr("y", layoutMid - 50)
            .attr("width", 100)
            .attr("height", 100)
            .attr("fill", "transparent")
            .attr("stroke", inventory.current?.status === false ? "red" : "green")
            .on("drop", dragToCDBEnd)
            .on("dragover", dragOverCDB)
            .on("dragleave", dragLeaveNode);

    }

    const standaloneBodyTemplate = (item) => {
        return (
            <div className="flex align-items-center" draggable onDrag={onTableItemDrag}
                onDragStart={(e) => {
                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                    leftContainer.current = d3Container.current.getBoundingClientRect();
                    d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
                    e.dataTransfer.setData("dragType", "standaloneBar");
                    inventory.current = item;
                    drawCDBDropzone();
                }}
                onDragEnd={onDropzoneLeave}>
                {knobs}
                <div className="flex align-items-center cell-text" draggable={false}
                    style={{ paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item.severity)}`, height: "20px" }}>{item.name}</div>
            </div>);
    }

    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onDeletePDB = (data) => {
        dispatch(LayoutActions.deletePDBNode(data, setShowDeletePDB));
    }

    const onCopyPDB = (data) => {

        if (inventory.current) {
            if (inventory.current?.ex_params?.cdb_id === props.node.id)
                dispatch(LayoutActions.copyPDB(data, setShowCopy));
            else
                dispatch(LayoutActions.copyToCDB(data, setShowCopy));
        }


    }



    const onRestorePDB = (data) => {
        dispatch(LayoutActions.restorePDB(data, setShowRestorePDB));
    }

    const onAddPdb = (data) => dispatch(LayoutActions.addPdbNode(data, setShowAddPDBDialog));

    const onExport = (data) => {
        dispatch(LayoutActions.exportNode(data, setShowExport));
        dispatch(LayoutActions.setSchemas(undefined));
        dispatch(LayoutActions.setDisks(undefined));
    }

    const onDuplicateStandalone = (data) => {
        dispatch(LayoutActions.duplicateOracleStandalone(data, setShowDuplicateStandalone));
    }

    const onImport = (data) => {

        if (data.obj_type === clusterRoles.PDB) dispatch(LayoutActions.importPDB(data, setShowImport));
        else dispatch(LayoutActions.importCDB(data, setShowImport));

    }

    const onBackupCDB = (data) => {
        dispatch(LayoutActions.backupCDB(data, setShowBackupCDB));
    }

    const onBackupPDB = (data) => {
        dispatch(LayoutActions.backupPDB(data, setShowBackupPDB));
    }


    const onFlashback = (data) => {
        dispatch(LayoutActions.flashbackNode(data, setShowFlashback));
    }

    const onRestoreFlashbackPDB = (data) => {
        dispatch(LayoutActions.restoreFlashback(data, setShowRestoreFlashbackPDB));
    }

    const onImportStandalone = (file) => {
        dispatch(LayoutActions.importStandalone(file, setShowImportStandalone));

    }

    const onTableItemDrag = (e) => {
        e.preventDefault();
        e.target.style.position = "fixed";
        e.target.style.top = e.clientY + "px";
        e.target.style.left = e.clientX + "px";
        if (e.clientX < leftContainer.current.left || e.clientY < leftContainer.current.top || e.clientY > leftContainer.current.bottom) {

            e.target.style.visibility = "hidden";

        }
        else {

            e.target.style.visibility = "";

        }
    }


    return (
        <div className="layout-item">
            <div className="layout-item-wrapper">
                <div style={{ zIndex: 1000 }} className="left-container">
                    <div className="flex flex-column flex-grow-1">
                        <div className="left-title flex flex-row">

                            <div className="ml-auto mr-auto">{`${props.node.what} cluster: ${props.node?.cluster_content?.name}`}</div>

                            <div className="flex ml-auto">
                                <Button
                                    style={{ fontSize: "1rem" }}
                                    className="p-button-sm p-button-text"
                                    label="status"
                                    onClick={() => navigate('/status', { state: { cluster_id: props.node?.cluster, node_id: props.node?.id } })}
                                />

                            </div>


                        </div>
                    </div>


                    <div className="left-content">
                        <svg
                            className="d3-component"
                            style={{ width: "800px", maxHeight: `${rightContainer.current?.offsetHeight > layoutCurrHeight ? rightContainer.current.offsetHeight + 5 : layoutCurrHeight + 5}px`, minHeight: `${layoutCurrHeight + 5}px` }}
                            ref={d3Container}
                        />
                    </div>
                    {props.node?.nodes?.length > rowsNumber ? <Paginator className="left-container-paginator" first={first} rows={rows} totalRecords={props.node?.nodes?.length} onPageChange={onPageChange} /> : null}
                </div>
                <div className="right-container" style={{ minHeight: `${layoutCurrHeight + 5}px` }}
                    ref={rightContainer}>
                    <TabView style={{ width: "500px" }} activeIndex={activeIndex} onTabChange={(e) => {
                        setActiveIndex(e.index);
                        tabRef.current = e.index
                    }}>
                        <TabPanel header="PDBs">
                            <div className={`oracle-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs mb-2" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    drawCDBDropzone();
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "pdbCreate");

                                }}>
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid white`, height: "20px" }}>Drag me on CDB to create PDB</div>
                                </div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> Drag from the list onto a CDB to start copying </div>
                                <DataTable
                                    className="severity-table"
                                    selectionMode="single"
                                    dataKey="id"
                                    selection={pdbRow}
                                    onSelectionChange={(e) => setPDBRow(e.value)}
                                    scrollable
                                    scrollHeight="flex"
                                    emptyMessage="no pdbs found"
                                    paginator={attachedNodes?.length > 0}
                                    alwaysShowPaginator={false}
                                    rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                    value={attachedNodes}>
                                    <Column style={{ height: "21px" }} field="name" sortFunction={(e) => onSort(e, attachedNodes)} body={pdbBodyTemplate} header="PDB Name" sortable></Column>
                                    <Column style={{ height: "21px" }} field="cdb_name" sortFunction={(e) => onSort(e, attachedNodes)} body={cdbBodyTemplate} header="CDB Name" sortable></Column>
                                    <Column style={{ height: "21px" }} field="host" sortFunction={(e) => onSort(e, attachedNodes)} body={hostBodyTemplate} header="Host" sortable></Column>
                                </DataTable>
                            </div>
                        </TabPanel>
                        <TabPanel header="Export / Import">
                            <div className={`oracle-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> Drag any PDB here to start an export</div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />  Drag from the list onto a PDB to start importing</div>
                                {(pdbSelected || cdbSelected) && <div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" />  <div className="col-fixed" style={{ width: "300px" }}> {`Exports for this node will be stored for ${cdbSelected ? props.node?.size_scheme?.export : pdbSelected?.size_scheme?.export} days`}</div> <div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: types.ORACLE_CDB } })}>change</Tag> </div></div>}

                                {(pdbSelected || cdbSelected) &&
                                    <DataTable selectionMode="single"
                                        className="severity-table"
                                        dataKey="id"
                                        selection={exportRow}
                                        onSelectionChange={(e) => setExportRow(e.value)}
                                        scrollable
                                        scrollHeight="flex"
                                        paginator={pdbSelected ? pdbSelected.exports?.length > 0 : props.node.exports?.length > 0}
                                        emptyMessage="no exports found"
                                        alwaysShowPaginator={false}
                                        rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                        value={pdbSelected ? pdbSelected.exports : props.node.exports}>
                                        <Column sortable style={{ height: "21px" }} field="host" header="Host" body={exportHostBodyTemplate}></Column>
                                        <Column style={{ height: "21px" }} field="created_date" body={exportDateBodyTemplate} header="Created date" sortable></Column>
                                        <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Size" sortable></Column>
                                    </DataTable>}
                            </div>
                        </TabPanel>
                        <TabPanel header="Standalones">
                            <div className={`oracle-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><button className="button-link" onClick={() => setShowImportStandalone(true)}><i className="pi pi-download" /> Import from tnsnames.ora</button>  </div>
                                {!cdbSelected &&
                                    <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />
                                        Select container to view standalones
                                    </div>}
                                {cdbSelected && !standalones &&
                                    <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />
                                        No standalone databases registered here
                                    </div>}
                                {cdbSelected && <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag}
                                    onDragStart={(e) => {
                                        e.dataTransfer.setDragImage(new Image(), 0, 0);
                                        leftContainer.current = d3Container.current.getBoundingClientRect();
                                        e.dataTransfer.setData("dragType", "standaloneBar");
                                        drawCDBDropzone();
                                    }}
                                    onDragEnd={onDropzoneLeave}>
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false}
                                        style={{ paddingLeft: "5px", borderLeft: `5px solid white`, height: "20px" }}>Drag me on CDB to duplicate a standalone</div>
                                </div>}
                                {cdbSelected && standalones &&
                                    <DataTable
                                        className="severity-table"
                                        selectionMode="single"
                                        dataKey="id"
                                        emptyMessage="no standalones found"
                                        selection={standaloneRow}
                                        onSelectionChange={(e) => setStandaloneRow(e.value)}
                                        scrollable
                                        scrollHeight="flex"
                                        paginator={oracleStandalones?.length > 0}
                                        alwaysShowPaginator={false}
                                        rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                        value={oracleStandalones}
                                    >
                                        <Column style={{ height: "21px" }} field="name" body={standaloneBodyTemplate} header="Name" sortable></Column>
                                    </DataTable>}
                            </div>

                        </TabPanel>
                        <TabPanel header="Flashbacks">
                            <div className={`oracle-tab-content-${layoutItemId.current} layout-tab-content`}>
                                {!flashbackAllowed.current &&
                                    <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> CDB version {props.node.version} doesn't support flashbacks</div>}
                                {flashbackAllowed.current && (pdbSelected || cdbSelected) && <div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" />  <div className="col-fixed" style={{ width: "300px" }}>  {`Exports for this node will be stored for ${cdbSelected ? props.node?.size_scheme?.restore : pdbSelected?.size_scheme?.restore} days`} </div> <div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: "ORACL" } })}>change</Tag> </div> </div>}
                                {flashbackAllowed.current && (pdbSelected || cdbSelected) &&
                                    <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> Drag restore point to {cdbSelected ? props.node.name : pdbSelected.name} to restore</div>}
                                {flashbackAllowed.current && (pdbSelected || cdbSelected) &&
                                    <DataTable
                                        selectionMode="single"
                                        className="severity-table"
                                        dataKey="id"
                                        selection={flashbackRow}
                                        onSelectionChange={(e) => setFlashbackRow(e.value)}
                                        scrollable
                                        emptyMessage="no flashbacks found"
                                        scrollHeight="flex"
                                        paginator={pdbSelected ? pdbSelected.flashbacks?.length > 0 : props.node.flashbacks?.length > 0}
                                        alwaysShowPaginator={false}
                                        rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                        value={pdbSelected ? pdbSelected.flashbacks : props.node.flashbacks}>
                                        <Column style={{ height: "21px" }} field="name" body={flashbackNameBodyTemplate} header={(pdbSelected || cdbSelected) ? `List of restore points for node ${pdbSelected ? pdbSelected.name : props.node.name}` : 'Select node to view flashbacks'} sortable></Column>
                                    </DataTable>}

                            </div>
                        </TabPanel>
                        <TabPanel header="Backups">
                            <div className={`oracle-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs grid"><i className="pi pi-circle-fill" /> {pdbSelected || cdbSelected ? `Drag backup to ${pdbSelected ? pdbSelected.name : props.node.name} to restore:` : "Select node to view backups:"} </div>
                                {(pdbSelected || cdbSelected) && <div draggable={false} className="info-section text-xs grid"><i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}>{`Backups for this node will be stored for ${cdbSelected ? props.node?.size_scheme?.backup : pdbSelected?.size_scheme?.backup} days`}</div> <div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: "ORACL" } })}>change</Tag> </div> </div>}
                                {pdbSelected || cdbSelected ?
                                    getAutomaticBackup()

                                    : null}
                                {pdbSelected || cdbSelected ?
                                    <ScrollPanel className="layout-scroll">

                                        <DataTable
                                            selectionMode="single"
                                            className="severity-table"
                                            dataKey="id"
                                            selection={backupRow}
                                            onSelectionChange={(e) => setBackupRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator={pdbSelected ? pdbSelected.backups?.length > 0 : props.node.backups > 0}
                                            alwaysShowPaginator={false}
                                            emptyMessage="no backups found"
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={pdbSelected ? pdbSelected.backups : props.node.backups}>
                                            <Column style={{ height: "21px" }} field="created_date" body={backupDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                    : null}
                            </div>

                        </TabPanel>
                    </TabView>
                </div>
            </div>
            <AddPDBDialog
                node={props.node}
                visible={showAddPDBDialog} onHide={() => {
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddPDBDialog(false);
                }} onOkClick={onAddPdb} />

            {showDeletePDB ? <DeletePDBDialog visible={showDeletePDB}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDeletePDB(false);
                }}
                onOkClick={onDeletePDB}
                node={props.selected.current}
                root={props.node} /> : null}

            {showCopy ? <CopyPDBDialog
                visible={showCopy}
                onHide={() => {
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowCopy(false)
                }}
                onOkClick={onCopyPDB}
                node={inventory.current}
                root={props.node} /> : null}

            {showImportStandalone ? <ImportStandaloneDialog
                visible={showImportStandalone}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowImportStandalone(false);
                }}
                onOkClick={onImportStandalone} /> : null}

            {showRestorePDB ? <RestorePDBLayoutDialog
                visible={showRestorePDB}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowRestorePDB(false)
                }}
                onOkClick={onRestorePDB}
                backup={inventory.current} /> : null}

            {showExport ? <ExportLayoutDialog visible={showExport}
                onHide={() => {
                    dispatch(LayoutActions.setSchemas(undefined));
                    dispatch(LayoutActions.setDisks(undefined));
                    dispatch(LayoutActions.setDirExists(null));
                    dispatch(LayoutActions.setExportExists(null));
                    dispatch(LayoutActions.setDiskSpace(null));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowExport(false)
                }}
                onOkClick={onExport}
                node={props.selected.current}
                root={props.node}
            /> : null}

            {showPDBDetails ? <DetailsPDBDialog
                node={pdbSelected}
                visible={showPDBDetails}
                onHide={() => setShowPDBDetails(false)} />
                : null
            }

            {showBackupDetails ? <BackupDetailsDialog
                backup={inventory.current}
                visible={showBackupDetails}
                onHide={() => setShowBackupDetails(false)} />
                : null
            }

            {showExportDetails ? <ExportDetailsDialog
                export={inventory.current}
                visible={showExportDetails}
                onHide={() => setShowExportDetails(false)} />
                : null
            }

            {showFlashbackDetails ? <FlashbackDetailsDialog
                flashback={inventory.current}
                visible={showFlashbackDetails}
                onHide={() => setShowFlashbackDetails(false)} />
                : null
            }

            {showImport ? <ImportPDBLayoutDialog visible={showImport}
                onHide={() => {

                    dispatch(LayoutActions.setUserExists(null));
                    dispatch(LayoutActions.setDumpExists(null));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowImport(false);
                }}
                onOkClick={onImport}
                node={props.selected.current ? props.selected.current : props.node}
                root={props.node}
                export={inventory.current}
            /> : null}

            {showBackupPDB ? <BackupPDBDialog visible={showBackupPDB}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowBackupPDB(false)
                }}
                onOkClick={onBackupPDB}
                pageIndex={pageIndex}
                node={props.selected.current}
                updateSchedule={updateSchedule.current}
            />
                : null}

            {showFlashback ?
                <FlashbackCDBPDBDialog visible={showFlashback}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowFlashback(false)
                    }}
                    onOkClick={onFlashback}
                    node={props.selected.current}
                    root={props.node}
                />
                : null
            }

            {showBackupCDB ? <BackupCDBDialog visible={showBackupCDB}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowBackupCDB(false)
                }}
                onOkClick={onBackupCDB}
                node={props.node}
                pageIndex={pageIndex}
                updateSchedule={updateSchedule.current}
            />
                : null
            }

            {showDuplicateStandalone ? <DuplicateStandaloneDialog
                visible={showDuplicateStandalone}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setValidPath(undefined));
                    setShowDuplicateStandalone(false)
                }}
                onOkClick={onDuplicateStandalone}
                root={props.node}
                standalone={inventory.current}
            />
                : null
            }

            {showRestoreFlashbackPDB ? <RestoreFlashbackLayoutDialog
                visible={showRestoreFlashbackPDB}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowRestoreFlashbackPDB(false)
                }}
                onOkClick={onRestoreFlashbackPDB}
                flashback={inventory.current} />
                : null
            }

            {showDeleteInventory ? <DeleteInventoryDialog
                visible={showDeleteInventory}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDeleteInventory(false)
                }}
                onOkClick={onDeleteInventory}
                item={inventory.current} />
                : null
            }

            {showDeleteCDBDialog ? <DeleteCDBDialog
                node={props.selected.current}
                visible={showDeleteCDBDialog}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDeleteCDBDialog(false)
                }}
                onOkClick={onDeleteCDB} />
                : null
            }

            {showWarning ? <WarningDialog
                message={warningMessage}
                visible={showWarning}
                onHide={() => setShowWarning(false)} />
                : null}

            {showInfoDialog ? <InfoDialog
                message={infoMessage}
                visible={showInfoDialog}
                onHide={() => {

                    setShowInfoDialog(false);
                }
                } />
                : null}
        </div>
    );
}


export default OracleLayoutItem;