import { useState, useRef, useEffect, useCallback } from "react";
import { TabView, TabPanel } from 'primereact/tabview';
import { nanoid } from 'nanoid';
import haproxy from "../../../assets/images/proxy.png"
import { Divider } from 'primereact/divider';
import * as d3 from "d3";
import trashImage from "../../../assets/images/trash.png";
import dbImage from "../../../assets/images/db.png";
import plusIcon from "../../../assets/images/plus.png";
import minusIcon from "../../../assets/images/minus.png";
import node_white from "../../../assets/images/node_white.png";
import node_unavail from '../../../assets/images/node_unavail.png'
import node_un_white from '../../../assets/images/node_un_white.png';
import db_white from "../../../assets/images/db_white.png";
import node_avail from "../../../assets/images/node.png";
import bar from "../../../assets/images/bar.png";
import db_unavail from "../../../assets/images/db_unavail.png";
import db_un_white from "../../../assets/images/db_un_white.png";
import cluster from "../../../assets/images/cluster.png";
import { useNavigate } from 'react-router-dom';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import moment from "moment";
import WarningDialog from "../../../dialogs/warningDialog";
import PostgresNodeDeleteDialog from "../../../dialogs/postgres/postgresNodeDeleteDialog";
import PostgresExportDialog from "../../../dialogs/postgres/postgresExportDialog";
import PostgresPrimaryBackupDialog from "../../../dialogs/postgres/postgresPrimaryBackupDialog";
import AddPostgresStandbyDialog from "../../../dialogs/postgres/addPostgresStandby";
import { useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import { useSelector } from "react-redux";
import AddPgDatabaseDialog from "../../../dialogs/postgres/addPgDatabaseDialog";
import DropDatabaseDialog from "../../../dialogs/postgres/dropDatabaseDialog";
import ExportPgDatabaseDialog from "../../../dialogs/postgres/exportPgDatabaseDialog";
import RestorePgDatabaseLayoutDialog from "../../../dialogs/postgres/restorePgDatabaseLayoutDialog";
import ExportDetailsDialog from "../../../dialogs/postgres/exportDetailsDialog";
import BackupDetailsDialog from "../../../dialogs/postgres/backupDetailsDialog";
import { Button } from "primereact/button";
import SwitchPostgresNodeDialog from "../../../dialogs/postgres/switchPostgresNodeDialog";
import DeleteInventoryDialog from "../../../dialogs/deleteInventoryDialog";
import RestorePgNodeLayoutDialog from "../../../dialogs/postgres/restorePgNodeLayoutDialog";
import { ToggleButton } from "primereact/togglebutton";
import CopyPgDbDialog from "../../../dialogs/postgres/copyPgDbDialog";
import PostgresStatusDialog from "../../../dialogs/postgres/pgStatusDialog";
import getColorBySeverity from "../../../util/colorBySeverity";
import { ConfirmPopup } from "primereact/confirmpopup";
import ImportPgNodeLayoutDialog from "../../../dialogs/postgres/importPgNodeLayoutDialog";
import InfoDialog from "../../../dialogs/infoDialog";
import config, { clusterRoles, clusterDropModes, types, pgVersionOptions } from "../../../config";
import AddHAProxy from "../../../dialogs/balancers/haproxy/addHAProxy";
import DropHAProxyNodeDialog from "../../../dialogs/balancers/haproxy/dropHAProxy";
import ClonePostgresStandbyDialog from "../../../dialogs/postgres/clonePostgresStandby";
import { TabMenu } from "primereact/tabmenu";
import { knobs } from "../../../util/templates";
import { Tag } from "primereact/tag";
import { ScrollPanel } from "primereact/scrollpanel";
import { Panel } from "primereact/panel";
import { Tooltip } from "primereact/tooltip";
import PgSizeCell from "../../pgSizeCell";
import { OverlayPanel } from "primereact/overlaypanel";
import UpgradePgNodeDialog from "../../../dialogs/postgres/upgradePgClusterDialog";
import { Paginator } from "primereact/paginator";
import ChooseServiceDialog from "../../../dialogs/chooseServiceDialog";
import PgDropClusterDialog from "../../../dialogs/postgres/dropClusterDialog";
import DropModeDialog from "../../../dialogs/dropModeDialog";


const PgLayoutItem = (props) => {
    const navigate = useNavigate();
    const [showNodes, setShowNodes] = useState(true);
    const [proxyRow, setProxyRow] = useState(null);
    const d3Container = useRef(null);
    const [activeIndex, setActiveIndex] = useState(0);
    const [nodeSelected, setNodeSelected] = useState(false);
    const [infoIcon, setInfoIcon] = useState('pi pi-info-circle');
    const [primarySelected, setPrimarySelected] = useState(false);
    const [attachedPgNodes, setAttachedPgNodes] = useState(null);
    const [foreignPgNodes, setForeignPgNodes] = useState(null);
    const [attachedProxies, setAttachedProxies] = useState(null);
    const currentNodeCoords = useRef({ x: 0, y: 0 });
    const [showDropHAProxy, setShowDropHAProxy] = useState(false);
    const tabRef = useRef(0);
    const inventory = useRef(null);
    const [showDeleteInventory, setShowDeleteInventory] = useState(false);
    const [warningMessage, setWarningMessage] = useState('');
    const [showWarning, setShowWarning] = useState(false);
    const leftContainer = useRef(null);
    const rightContainer = useRef(null);
    const tabsPos = useRef(null);
    const trashPos = useRef(null);
    const [pgDatabases, setPgDatabases] = useState(null);
    const [showDropDatabase, setShowDropDatabase] = useState(false);
    const [showExport, setShowExport] = useState(false);
    const [showBackup, setShowBackup] = useState(false);
    const [showAddPostgresStandby, setShowAddPostgresStandby] = useState(false);
    const [showAddPostgresDatabase, setShowAddPostgresDatabase] = useState(false);
    const [showExportPostgresDatabase, setShowExportPostgresDatabase] = useState(false);
    const [showRestorePostgresDatabase, setShowRestorePostgresDatabase] = useState(false);
    const [showExportDetails, setShowExportDetails] = useState(false);
    const [showBackupDetails, setShowBackupDetails] = useState(false);
    const [showDropPgNode, setShowDropPgNode] = useState(false);
    const [showRestorePostgresNode, setShowRestorePostgresNode] = useState(false);
    const [showSwitchover, setShowSwitchover] = useState(false);
    const [showCloneStandby, setShowCloneStandby] = useState(false);
    const [backupRow, setBackupRow] = useState(null);
    const [exportRow, setExportRow] = useState(null);
    const [standbyRow, setStandbyRow] = useState(null);
    const [showAddHAProxy, setShowAddHAProxy] = useState(false);
    const [dbRow, setDbRow] = useState(null);
    const [showCopyDb, setShowCopyDb] = useState(false);
    const [showStatus, setShowStatus] = useState(false);
    const [dbNew, setDbNew] = useState(null);
    const [pauseConfirm, setPauseConfirm] = useState(false);
    const [isPhysical, setIsPhysical] = useState(true);
    const updateSchedule = useRef(null);
    const [expandBackup, setExpandBackup] = useState(null);
    const [databaseSource, setDatabaseSource] = useState("");
    const [showImportPostgresNode, setShowImportPostgresNode] = useState("");
    const [showInfoDialog, setShowInfoDialog] = useState(false);
    const [infoMessage, setInfoMessage] = useState(false);
    const layoutItemId = useRef(null);
    const pauseRef = useRef(null);
    const layoutCurrHeight = config.layoutMinHeight;
    const layoutMid = config.layoutMinHeight / 2;
    const [layoutSwitchIndex, setLayoutSwitchIndex] = useState(0);
    const [removeScheduleConfirm, setRemoveScheduleConfirm] = useState(false);
    const [pageIndex, setPageIndex] = useState(0);
    const [showUpgrade, setShowUpgrade] = useState(false);
    const rowsNumber = 6;
    const [first, setFirst] = useState(0);
    const [rows, setRows] = useState(rowsNumber);
    const [nodesPage, setNodesPage] = useState(props.node?.nodes?.length > rowsNumber ? props.node?.nodes?.slice(props.node?.nodes?.length - (rowsNumber + 1), props.node.nodes?.length - 1) : props.node?.nodes);
    const [databasesPage, setDatabasesPage] = useState(props.node?.databases?.length > rowsNumber ? props.node?.databases?.slice(props.node?.databases?.length - (rowsNumber + 1), props.node.databases?.length - 1) : props.node?.databases);
    const clusterOptions = useRef(null);
    const node_x = 450;
    const pauseTooltipRef = useRef(null);
    const [showChooseServiceDialog, setShowChooseServiceDialog] = useState(false);
    const [showDropCluster, setShowDropCluster] = useState(false);
    const [showDropMode, setShowDropMode] = useState(false);
    const thisNodes = useRef(null);
    const otherNodes = useRef(null);
    const thisDbs = useRef(null);
    const otherDbs = useRef(null);
    const thisBackups = useRef(null);
    const otherBackups = useRef(null);
    const thisExports = useRef(null);
    const otherExports = useRef(null);

    const onDropCluster = (data) => {
        dispatch(LayoutActions.dropCluster(data, setShowDropCluster));
    }


    useEffect(() => {

        if (props.node?.nodes?.length <= rowsNumber) setNodesPage(props.node?.nodes);
        if (props.node?.databases?.length <= rowsNumber) setDatabasesPage(props.node?.databases);

    }, [props.node])

    const onPageChange = (event) => {
        let rowsPerPage = rowsNumber;
        let startIndex = (event.page) * rowsPerPage;
        if (event.rows < rowsPerPage) {
            rowsPerPage = rowsPerPage - event.rows
            startIndex -= rowsPerPage
        }
        let endIndex = startIndex + rowsPerPage;
        setFirst(event.first);
        setRows(event.rows);
        if (isPhysical) setNodesPage(props.node?.nodes?.slice(startIndex, endIndex));
        else setDatabasesPage(props.node?.databases?.slice(startIndex, endIndex));

    };

    const roleBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.cluster_role}</div>);
    }

    const getAutomaticBackup = () => {
        let backup = props.node?.backups?.filter((b) => b.is_sched === true)?.[0]
        if (!backup) backup = pg_backups?.filter(b => b?.cluster === props.node?.cluster && b.ex_params?.node_version === props.node.version && b?.is_sched === true)?.[0]
        updateSchedule.current = backup;
        const popup_id = `remove-schedule-button-${props.node.id}`
        return (<div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" />  <div className="col-fixed" style={{ width: "300px" }}> Automatic backup: {backup?.ex_params?.description || "No"}</div> {backup ? <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-pencil" onClick={() => {
            setShowBackup(true);
        }}>change</Tag> <Tag className='right-container-button ml-2' id={popup_id} icon="pi pi-trash" severity="danger" onClick={() => setRemoveScheduleConfirm(true)} >delete</Tag>    <ConfirmPopup
                target={document.getElementById(popup_id)}
                visible={removeScheduleConfirm}
                onHide={() => setRemoveScheduleConfirm(false)}
                message="Are you sure?"
                accept={() => dispatch(LayoutActions.removePgSchedule({ "node_id": props.node.id }, setRemoveScheduleConfirm))}
            /></div> : <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-plus" onClick={() => {

                setPageIndex(1);
                setShowBackup(true);
            }}>create</Tag> </div>}  </div>)
    }


    const layoutOptions = [
        { label: 'Physical layout', className: isPhysical ? '' : 'p-disabled' },
        { label: 'Database layout', className: isPhysical ? 'p-disabled' : '' },
    ];

    // useEffect(() => {
    //     let height = config.layoutMinHeight;

    //     if (showNodes) {
    //         if (!isPhysical) {
    //             const targetDbs = props.node.databases
    //             if (targetDbs?.length > 6) height = height + (80 * (targetDbs.length - 6));

    //         }
    //         else {

    //             if (props.node?.nodes?.length > 6) height = height + (80 * (props.node.nodes.length - 6));

    //         }

    //     }


    //     setLayoutCurrHeight(height);
    //     setLayoutMid(height / 2);


    // }, [props.node?.databases, props.node?.nodes, isPhysical, showNodes])

    const onVerifyBackup = (open, message) => {

        setInfoMessage(message);
        setInfoIcon('pi pi-info-circle');
        setShowInfoDialog(open);

    }

    const showDbCredentialsInfo = (response) => {

        setInfoMessage('db_cred');
        setInfoIcon('pi pi-exclamation-triangle');
        setShowInfoDialog(response);

    }



    const dispatch = useDispatch();

    const { databases, pg_backups, pg_exports, haproxies } = useSelector((state) => state.layout);

    const onDeleteInventory = (data) => {

        switch (inventory.current.type) {

            case "EXPORT":
                dispatch(LayoutActions.dropExport(data, setShowDeleteInventory));
                break;
            case "BACKUP":
                dispatch(LayoutActions.dropBackup(data, setShowDeleteInventory));
                break;

            default:
                break;

        }

    }

    const addImage = useCallback(
        (x, y, node) => {

            const id = nanoid();
            function dragOverNode(event) {
                event.preventDefault();
                d3.select(d3Container.current)
                    .select(`#${event.target.id}`)
                    .attr("fill", "rgba(157, 245, 193, 0.2)");

            }



            function dragToNodeEnd(event) {

                if (node?.ind_deleted) {
                    showWarningMessage({

                        "text": `${props.node.name.toUpperCase()} is unavailable`,
                        "status": `${props.node.status}`
                    });

                    return;

                }

                let type = event.dataTransfer.getData("dragType");

                if (isPhysical === true) {

                    switch (type) {

                        case "proxyCreate":
                            setShowAddHAProxy(true);
                            break;
                        default:
                            break;

                    }


                }

            }



            const drawNodeDropzone = (event) => {
                const svg = d3.select(d3Container.current)
                svg.select(`#${event.target.id}`).dispatch("click")
                const [clientX, clientY] = d3.pointer(event, svg.node());
                d3.select(d3Container.current)
                    .select(".drop-zone-marker")
                    .attr("fill", "transparent");
                svg.insert("rect")
                    .attr("class", "drop-zone-marker")
                    .attr("x", clientX - 40)
                    .attr("y", clientY - 40)
                    .attr("width", 100)
                    .attr("height", 100)
                    .attr("id", `dropzone-${id}`)
                    .attr("fill", "transparent")
                    .attr("stroke", inventory.current?.status === false ? "red" : "green")
                    .on("drop", dragToNodeEnd)
                    .on("dragover", dragOverNode)
                    .on("dragleave", dragLeaveNode);
            }


            function onClickNode(svg, id, node) {

                setPrimarySelected(false);
                props.selected.current = node;
                setNodeSelected(node);
                let elements = window.document.querySelectorAll(".node");

                for (let i = 0; i < elements?.length; i++) {
                    elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute("slave_of") === "false" ? dbImage : node_avail : elements[i].getAttribute("slave_of") === "false" ? db_unavail : node_unavail);
                }

                elements = window.document.querySelectorAll(".root");

                for (let i = 0; i < elements?.length; i++) {
                    elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
                }


                elements = window.document.querySelectorAll(".cluster");

                for (let i = 0; i < elements?.length; i++) {
                    elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
                }


                elements = window.document.querySelectorAll(".node-what-text");

                for (let i = 0; i < elements?.length; i++) {
                    elements[i].style.fill = "gray";
                }

                elements = window.document.querySelectorAll(".root-what-text");

                for (let i = 0; i < elements?.length; i++) {
                    elements[i].style.fill = "gray";
                }

                svg.select("#node-" + id).attr("xlink:href", node.ind_deleted ? node_un_white : node_white);

                svg.select(`#text-${id}`).style("fill", "white");
            }

            function goStandbyDetails(standby) {

                if (standby.ind_deleted) {

                    showWarningMessage({ "text": `${standby.name} is no longer available.\nDetails cant be viewed.`, "status": standby.status })

                }
                else navigate(`/status/postg/${standby.id}`);

            }

            const onDragStart = (svg, id, node) => (d) => {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();
                onClickNode(svg, id, node);

            }


            const onDragging = (svg, id, node) => (d) => {

                setNodeSelected(node);
                const viewportHeight = window.innerHeight;
                const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
                if (overViewport) {
                    leftContainer.current = d3Container.current?.getBoundingClientRect();
                    tabsPos.current = rightContainer.current?.getBoundingClientRect();
                    trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

                }

                if (overViewport) {
                    window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
                }

                if (d.sourceEvent.clientX < leftContainer.current.left || d.sourceEvent.clientY > tabsPos.current.bottom || d.sourceEvent.clientY < tabsPos.current.top || d.sourceEvent.clientX > tabsPos.current.right)
                    svg.style("overflow", "hidden");
                else svg.style("overflow", "visible");
                if (d.sourceEvent.srcElement) {

                    // get the mouse positions
                    var dx = d.x - 20,
                        dy = d.y - 30;

                    // move the dragged element
                    d3.select("#node-" + id).style("cursor", "grabbing");
                    d3.select("#d3-tooltip-container-" + id).attr("transform", "translate(" + (dx + 40) + "," + (dy + 10) + ")");
                    d3.select("#node-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                    d3.select("#text-" + id).attr("transform", "translate(" + (dx) + "," + (dy) + ")");
                    d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 51) + ")");


                    if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY <= trashPos.current.bottom
                        && d.sourceEvent.clientY >= trashPos.current.top) {

                        svg.select(".trash").style("opacity", "1.0");

                    }
                    else {

                        svg.select(".trash").style("opacity", "0.5");

                    }


                    if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                        switch (tabRef.current) {
                            case 0:
                            case 2:
                            case 4:
                                svg.select(`#node-${id}`).style("cursor", "not-allowed");
                                break;
                            case 3:
                                if (isPhysical === false) svg.select(`#node-${id}`).style("cursor", "not-allowed");
                                break;
                            default:
                                break;
                        }
                    }


                    if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 125) {
                        setActiveIndex(0);
                        tabRef.current = 0;
                    }
                    else if (d.sourceEvent.clientX > tabsPos.current.left + 125 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 250) {
                        setActiveIndex(1);
                        tabRef.current = 1;
                    }
                    else if (d.sourceEvent.clientX > tabsPos.current.left + 250 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 375) {
                        setActiveIndex(2);
                        tabRef.current = 2;

                    }
                    else if (d.sourceEvent.clientX > tabsPos.current.left + 375 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 500) {
                        setActiveIndex(3);
                        tabRef.current = 3;

                    }

                }

                let elements = window.document.querySelectorAll(`.pg-tab-content-${layoutItemId.current}`);
                for (let i = 0; i < elements?.length; i++) {

                    elements[i].style.boxShadow = (tabRef.current === 3 && !isPhysical) ? "0 0 0 0.2rem rgb(220 53 69 / 72%)" : (tabRef.current === 0 || tabRef.current === 2 || tabRef.current === 4) ? "" : "0 0 0 0.2rem rgb(40 167 69 / 72%)";

                }
            }

            const onDragEnd = (svg, id, node) => (d) => {
                if (d.sourceEvent.detail === 2 && node.cluster_role === clusterRoles.STANDBY) {

                    goStandbyDetails(node);
                    return;

                }

                setNodeSelected(node);
                if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom
                    && d.sourceEvent.clientY >= trashPos.current.top && d.sourceEvent.clientX <= trashPos.current.right) {

                    if (node.ind_deleted) {


                        showWarningMessage({

                            "text": `${node.name.toUpperCase()} is unavailable`,
                            "status": `${node.status}`
                        });
                    }
                    else {
                        inventory.current = node;
                        if (node.cluster_role === clusterRoles.STANDBY)
                        {
                            setShowDropPgNode(true);

                        } 
                    
                        else setShowDropDatabase(true);

                    }

                }

                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {


                    if (node.ind_deleted) {


                        showWarningMessage({

                            "text": `${node.name.toUpperCase()} is unavailable`,
                            "status": `${node.status}`
                        });
                    }
                    else {

                        if (node.cluster_role === clusterRoles.STANDBY) {

                            switch (tabRef.current) {

                                case 1:
                                    if (node.ind_deleted)
                                        showWarningMessage({

                                            "text": `${props.node.name.toUpperCase()} is unavailable`,
                                            "status": `${props.node.status}`
                                        });
                                    else setShowExport(true);
                                    break;
                                case 3:
                                    if (node.ind_deleted)
                                        showWarningMessage({

                                            "text": `${props.node.name.toUpperCase()} is unavailable`,
                                            "status": `${props.node.status}`
                                        });
                                    else {

                                        setPageIndex(0);
                                        setShowBackup(true);

                                    }
                                    break;
                                default:
                                    break;

                            }
                        }
                        else {

                            switch (tabRef.current) {

                                case 1:
                                    if (node.ind_deleted)
                                        showWarningMessage({

                                            "text": `${props.node.name.toUpperCase()} is unavailable`,
                                            "status": `${props.node.status}`
                                        });
                                    else setShowExportPostgresDatabase(true);
                                    break;
                                default:
                                    break;

                            }

                        }


                    }

                }

                d3.select("#node-" + id).remove();
                d3.select("#line-" + id).remove();
                d3.select("#text-" + id).remove();
                d3.select("#severity-" + id).remove();
                d3.select("#d3-tooltip-container-" + id).remove();

                let i = 0;

                if (isPhysical === true) {

                    if (nodesPage?.length > 0) {

                        for (; i < nodesPage?.length; ++i)
                            if (nodesPage[i].id === node.id)
                                break;

                        const node_y = (i + 0.5) * (layoutCurrHeight / nodesPage.length);
                        currentNodeCoords.current.x = node_x;
                        currentNodeCoords.current.y = node_y + 20;
                        addImage(node_x, node_y, node);

                    }

                }
                else if (isPhysical === false) {

                    if (databasesPage?.length > 0) {

                        for (; i < databasesPage?.length; ++i)
                            if (databasesPage[i].id === node.id)
                                break;
                        const database_y = (i + 0.5) * (layoutCurrHeight / databasesPage.length);
                        currentNodeCoords.current.x = node_x;
                        currentNodeCoords.current.y = database_y;
                        addImage(node_x, database_y, node);
                    }
                }

                svg.select(".trash").style("opacity", "0.0");
                d3.select("#node-" + id).style("cursor", "pointer");
                let elements = window.document.querySelectorAll(`.pg-tab-content-${layoutItemId.current}`);
                for (let i = 0; i < elements?.length; i++) {

                    elements[i].style.boxShadow = "";

                }
            }


            const svg = d3.select(d3Container.current);

            const image = svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === node.id) ? node.ind_deleted ? node_un_white : node_white : node.ind_deleted ? node_unavail : node_avail)
                .attr("class", "node")
                .attr("id", "node-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 60)
                .attr("available", node.ind_deleted ? false : true)
                .attr("draggable", true)
                .attr("transform", () => "translate(" + x + "," + (y - 20) + ")")
                .on("mouseover", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

                })
                .on("mouseout", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

                })
                .on("click", function (d) {
                    onClickNode(svg, id, node);
                });


            if (node?.cluster_role && node.cluster_role === clusterRoles.STANDBY) {

                image.on("dragover", drawNodeDropzone)
                    .on("dragleave", dragLeaveNode)
            }

            const x0 = 100;
            const y0 = layoutMid;
            const x1 = x;
            const y1 = y;

            const path = d3.path()
            path.moveTo(x1, y1)
            path.bezierCurveTo((x0 + x1) / 2, y1, (x0 + x1) / 2, y0, x0, y0);

            svg.insert("path")
                //.attr("d", d3.line()([[100, 220], [x, y + 30]]))
                .attr("d", path)
                .attr("id", "line-" + id).attr("transform", "translate(" + (0) + "," + (10) + ")")
                .style("fill", "none")
                .style("stroke", "#ccc")
                .style("stroke-width", 1);

            svg.insert("text")
                .attr("transform", "translate(" + (x) + "," + (y - 14) + ")")
                .attr("id", "text-" + id)
                .attr("class", "node-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === node.id ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${node.name ? node.name : ""} ${node.status ? node.status : ""} ${node.cluster_role ? `(${node.cluster_role})` : ""}`).on("dblclick", () => goStandbyDetails(node)).on("click", function (d) {
                    currentNodeCoords.current.x = x1;
                    currentNodeCoords.current.y = y1 + 20;
                    onClickNode(svg, id, node);
                });

            //draw node severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("height", 5).attr("fill", getColorBySeverity(node.severity))
                .attr("transform", "translate(" + (x1 + 2) + "," + (y1 + 31) + ")")
                .attr("id", "severity-" + id)


            var drag_behavior = d3.drag()
                .on("start", onDragStart(svg, id, node))
                .on("drag", onDragging(svg, id, node))
                .on("end", onDragEnd(svg, id, node));

            svg.insert("foreignObject")
                .attr("class", "d3-tooltip-container")
                .attr("id", `d3-tooltip-container-${id}`)
                .attr("transform", "translate(" + (x + 40) + "," + (y + 10) + ")")
                .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${node.ind_started ? "up" : "down"} <br/> Alerts: ${node?.alert?.length > 40 ? node.alert.substring(0, 37).concat("...") : node?.alert}  </div>`)

            image.call(drag_behavior);
        },
        [props.selected, layoutCurrHeight, layoutMid, nodesPage, databasesPage,  navigate, isPhysical, props.node]
    );



    useEffect(() => {


        const onTableItemDrop = (event) => {
            let type = event.dataTransfer.getData("dragType");

            switch (type) {
                case "standbyBar":
                    if (inventory.current?.ind_deleted) {


                        showWarningMessage({

                            "text": `${inventory.current.name.toUpperCase()} is unavailable`,
                            "status": `${inventory.current.status}`
                        });
                    }
                    else {

                        props.selected.current = inventory.current
                        if(inventory.current?.cluster_role === clusterRoles.PRIMARY && props.node?.nodes?.length > 1) setShowDropMode(true);
                        else setShowDropPgNode(true);


                    }
                    break;
                case "databaseBar":
                    if (inventory?.current.ind_deleted) {


                        showWarningMessage({

                            "text": `${inventory.current.name.toUpperCase()} is unavailable`,
                            "status": `${inventory.current.status}`
                        });
                    }
                    else setShowDropDatabase(true);
                    break;
                case "backupBar":
                    setShowDeleteInventory(true);
                    break;
                case "exportBar":
                    setShowDeleteInventory(true);
                    break;
                case "proxyBar":
                    setShowDropHAProxy(true)
                    break;
                default:
                    break;


            }

        }

        function goPrimaryDetails() {
            if (props.node.ind_deleted) {

                showWarningMessage({ "text": `${props.node.name} is no longer available.\nDetails cant be viewed.`, "status": props.node.status })

            }
            else navigate(`/status/postg/${props.node.id}`);

        }


        function onPrimaryClick(svg, id) {

            setNodeSelected(false);

            let elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }


            elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }

            elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute("slave_of") === "false" ? dbImage : node_avail : elements[i].getAttribute("slave_of") === "false" ? db_unavail : node_unavail);
            }


            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }


            svg.select(".root").attr("xlink:href", props.node.ind_deleted ? db_un_white : db_white);
            props.selected.current = props.node;
            setPrimarySelected(true);
            svg.select(`#text-${id}`).style("fill", "white");


        }


        const onDragStartPrimary = (svg, id, primary) => (d) => {
            trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();
            leftContainer.current = d3Container.current?.getBoundingClientRect();
            tabsPos.current = rightContainer.current?.getBoundingClientRect();

            // Draw primary what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 18) + ")")
                .attr("id", "text-copy" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node.id ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.name ? props.node.name : ""} ${props.node.status ? props.node.status : ""}`);

            // Draw primary icon
            svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id) ? props.node.ind_deleted ? db_un_white : db_white : props.node.ind_deleted ? db_unavail : dbImage)
                .attr("class", "root")
                .attr("id", "root-copy" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")")
                .attr("available", !props.node.ind_deleted);

            //draw primary severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-copy" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")")


            // Draw primary plus/minus icon

            if ((props.node.nodes?.length > 0 && isPhysical === true) || (props.node?.databases?.length > 0 && isPhysical === false)) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-copy" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 15) + ")");
            }

            onPrimaryClick(svg, id)

        }

        const onDraggingPrimary = (svg, id, primary) => (d) => {
            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }

            if (d.sourceEvent.clientX < leftContainer.current.left || d.sourceEvent.clientY > leftContainer.current.bottom || d.sourceEvent.clientY < tabsPos.current.top || d.sourceEvent.clientX > tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");


            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#root-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx - 54) + "," + (dy - 8) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 40) + ")");
                d3.select("#expand-" + id).attr("transform", () => "translate(" + (dx + 30) + "," + (dy - 5) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", () => "translate(" + (dx + 50) + "," + (dy - 5) + ")");
            }

            svg.select(".root").style("cursor", "grabbing");
            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY <= trashPos.current.bottom
                && d.sourceEvent.clientY >= trashPos.current.top) {

                svg.select(".trash").style("opacity", "1.0");

            }
            else {

                svg.select(".trash").style("opacity", "0.5");

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                switch (tabRef.current) {
                    case 0:
                    case 2:
                    case 4:
                        svg.select(".root").style("cursor", "not-allowed");
                        break;
                    default:
                        break;
                }
            }



            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 125) {
                setActiveIndex(0);
                tabRef.current = 0;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 125 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 250) {
                setActiveIndex(1)
                tabRef.current = 1;
            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 250 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 350) {
                setActiveIndex(2);
                tabRef.current = 2;

            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 350 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 450) {
                setActiveIndex(3);
                tabRef.current = 3;


            }
            else if (d.sourceEvent.clientX > tabsPos.current.left + 450 && d.sourceEvent.clientY > leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX < tabsPos.current.left + 550) {
                setActiveIndex(4);
                tabRef.current = 4;


            }


            let elements = window.document.querySelectorAll(`.pg-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = tabRef.current === 0 || tabRef.current === 2 || tabRef.current === 4 ? "" : "0 0 0 0.2rem rgb(40 167 69 / 72%)";

            }
        }

        const onDragEndPrimary = (svg, id, primary) => (d) => {

            if (d.sourceEvent.detail === 2) {

                goPrimaryDetails();
                return;

            }

            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom
                && d.sourceEvent.clientY >= trashPos.current.top && d.sourceEvent.clientX <= trashPos.current.right) {

                if (props.node.ind_deleted)
                    showWarningMessage({

                        "text": `${props.node.name.toUpperCase()} is unavailable`,
                        "status": `${props.node.status}`
                    });

                else {
                    inventory.current = primary;
                    if(props.node?.nodes?.length > 1) setShowDropMode(true);
                    else setShowDropPgNode(true);

                }

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {


                switch (tabRef.current) {
                    case 1:
                        if (props.node.ind_deleted)
                            showWarningMessage({

                                "text": `${props.node.name.toUpperCase()} is unavailable`,
                                "status": `${props.node.status}`
                            });
                        else setShowExport(true);
                        break;
                    case 3:
                        if (props.node.ind_deleted)
                            showWarningMessage({

                                "text": `${props.node.name.toUpperCase()} is unavailable`,
                                "status": `${props.node.status}`
                            });
                        else {


                            setPageIndex(0);
                            setShowBackup(true);

                        }
                        break;
                    default:
                        break;

                }

            }

            svg.select(".trash").style("opacity", "0.0");

            d3.select("#root-copy" + id).remove();
            d3.select("#text-copy" + id).remove();
            d3.select("#severity-copy" + id).remove();
            d3.select("#expand-copy" + id).remove();

            d3.select("#root-" + id).attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")");
            d3.select("#text-" + id).attr("transform", () => "translate(" + 0 + "," + (layoutMid - 18) + ")");
            d3.select("#severity-" + id).attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")");
            d3.select("#expand-" + id).attr("transform", "translate(" + (80) + "," + (layoutMid - 15) + ")");
            d3.select(`#d3-tooltip-container-${id}`).attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")

            svg.select(".root").style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.pg-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        if (d3Container.current) {
            const svg = d3.select(d3Container.current);
            svg.selectChildren("*").remove();

            const id = nanoid();



            if (!props.selected.current) {
                setPrimarySelected(props.node);

            }
            svg.insert("foreignObject")
                .attr("class", "d3-tooltip-container")
                .attr("id", `d3-tooltip-container-${id}`)
                .attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
                .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${props.node.ind_started ? "up" : "down"} <br/> Alerts: ${props.node?.alert?.length > 40 ? props.node.alert.substring(0, 37).concat("...") : props.node?.alert} </div>`)

            // Draw trash icon    
            svg.insert("image")
                .attr("xlink:href", trashImage)
                .attr("class", "trash")
                .attr("id", `trash-${id}`)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 100)
                .attr("height", 100)
                .attr("transform", () => "translate(" + 0 + "," + (layoutCurrHeight - 100) + ")")
                .on("dragover", dragOverTrash)
                .on("dragleave", dragLeaveTrash)
                .on("drop", onTableItemDrop);


            // Draw primary what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 18) + ")")
                .attr("id", "text-" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node.id || !layoutItemId.current ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.name ? props.node.name : ""} ${props.node.status ? props.node.status : ""}`)
                .on("dblclick", goPrimaryDetails)
                .on("click", () => onPrimaryClick(svg, id));

            // Draw primary icon
            const imagePrimary = svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id || !layoutItemId.current) ? props.node.ind_deleted ? db_un_white : db_white : props.node.ind_deleted ? db_unavail : dbImage)
                .attr("class", "root")
                .attr("id", "root-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("draggable", true)
                .attr("transform", () => "translate(" + 50 + "," + (layoutMid - 10) + ")")
                .attr("available", !props.node.ind_deleted)
                .on("mouseover", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

                })
                .on("mouseout", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

                })

            layoutItemId.current = id;

            const dragBehaviorPrimary = d3.drag()
                .on("start", onDragStartPrimary(svg, id, props.node))
                .on("drag", onDraggingPrimary(svg, id, props.node))
                .on("end", onDragEndPrimary(svg, id, props.node));
            imagePrimary.call(dragBehaviorPrimary);


            //draw primary severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 31) + ")")



            // Draw primary plus/minus icon

            if ((props.node.nodes?.length > 0 && isPhysical === true) || (props.node?.databases?.length > 0 && isPhysical === false)) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 15) + ")")
                    .on("click", () => { setShowNodes(!showNodes) });


                if (showNodes) {

                    if (isPhysical === true) {

                        if (nodesPage?.length > 0) {

                            nodesPage.forEach((node, index) => {
                                addImage(node_x, ((index + 0.5) * (layoutCurrHeight / nodesPage.length)), node);
                            });

                        }

                    }
                    else if (isPhysical === false) {
                        if (databasesPage?.length > 0) {

                            databasesPage.forEach((database, index) => {
                                addImage(node_x, ((index + 0.5) * (layoutCurrHeight / databasesPage.length)), database);
                            });

                        }

                    }

                }
            }
        }
    }, [showNodes, props.node, navigate, isPhysical, props.selected, layoutCurrHeight, layoutMid, nodesPage, databasesPage, addImage]);




    useEffect(() => {

        if (props.node) {
            let tempData = []
            if (props.node.nodes?.length > 0) tempData = [props.node, ...props.node.nodes];
            else tempData = [props.node];
            setAttachedPgNodes(tempData);

            tempData = []
            const ownProxies = haproxies?.filter((p) => p?.cluster === props.node.cluster)
            if (ownProxies?.length < 1)
                tempData = [];
            else tempData = [...ownProxies];
            setAttachedProxies(tempData);
        }

    }, [props.node, haproxies]);


    useEffect(() => {

        let tempData = []
        if (props.selection?.length > 0) {

            props.selection?.filter((p) => p.cluster !== props.node.cluster)?.forEach((p) => {
                tempData.push(p);
                if (p.nodes?.length > 0) p.nodes?.forEach((s) => tempData.push(s));
            })
        }
        setForeignPgNodes(tempData);


    }, [props.selection, props.node]);

    useEffect(() => {

        let tempData = [];
        if (databases?.length > 0) {
            tempData = databases?.filter((d) => d.type === types.POSTGRES);

        }
        else tempData = [];

        setPgDatabases(tempData);

    }, [databases])

    useEffect(() => {

        const updateSelected = () => {

            if (props.selected.current?.type?.includes(types.POSTGRES)) {

                if (props.selected.current?.id === props.node.id) {

                    let nodeSelected = props.selection?.filter((n) => n.id === props.selected.current?.id);
                    if (nodeSelected?.length > 0) {
                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }

                }
                else {
                    let nodeSelected;
                    if (isPhysical === true) {
                        nodeSelected = props.node?.nodes?.filter((n) => n.id === props.selected.current?.id);
                    }
                    else {
                        if (isPhysical === false) {
                            nodeSelected = props.node?.databases?.filter((n) => n.id === props.selected.current?.id);
                        }

                    }
                    if (nodeSelected?.length > 0) {

                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }
                }


            }

        }


        if (props.selection?.length > 0) updateSelected();

    }, [props.node, isPhysical, props.selection, props.selected])

    function dragToPrimaryEnd(event) {

        props.selected.current = props.node
        let type = event.dataTransfer.getData("dragType");
        if (props.node.ind_deleted) {
            showWarningMessage({

                "text": `${props.node.name.toUpperCase()} is unavailable`,
                "status": `${props.node.status}`
            });

            return;

        }

        switch (type) {
            case "standbyCreate":
                if (props.services?.length > 0) setShowChooseServiceDialog(true);
                else setShowAddPostgresStandby(true);
                break;
            case "proxyCreate":
                setShowAddHAProxy(true);
                break;
            case "databaseBar":
                let source = event.dataTransfer.getData("dragSource");
                setDatabaseSource(source);
                dispatch(LayoutActions.checkPgDatabaseExists({ "name": inventory.current?.name, "node_id": props.node?.id }, ShowCopyDb));
                break;
            case "databaseCreate":
                setShowAddPostgresDatabase(true);
                break;
            case "exportBar":
                if (inventory.current?.status === false) {
                    showWarningMessage({

                        "text": "Cannot perform this action",
                        "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
                    });

                    return;
                }
                if (inventory.current?.database) setShowRestorePostgresDatabase(true);
                else if (inventory.current?.node) setShowImportPostgresNode(true);
                break;
            case "backupBar":
                if (inventory.current?.status === false) {
                    showWarningMessage({

                        "text": "Cannot perform this action",
                        "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
                    });

                    return;
                }
                setShowRestorePostgresNode(true);
                break;
            default:
                break;



        }






    }


    const onExportPostgresDatabase = (data) => {

        dispatch(LayoutActions.exportPgDatabase(data, setShowExportPostgresDatabase));
        dispatch(LayoutActions.setDirExists(null));
        dispatch(LayoutActions.setExportExists(null));
        dispatch(LayoutActions.setDiskSpace(null));

    }
    function dragOverPrimary(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    }

    function dragLeaveNode(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .selectAll(".drop-zone-marker")
            .attr("fill", "transparent");

    }

    const dragOverTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "1.0");

    }

    const dragLeaveTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.5");

    }

    const backupDateBodyTemplate = (item) => {
        return (
            <div className="flex align-items-center" draggable onDrag={onTableItemDrag}
                onDragEnd={onDropzoneLeave}
                onDragStart={(e) => {
                    leftContainer.current = d3Container.current?.getBoundingClientRect();
                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                    d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
                    e.dataTransfer.setData("dragType", "backupBar");
                    window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                    inventory.current = item;
                    drawPrimaryDropzone();

                }}>
                {knobs}
                <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>
            </div>
        );
    }

    const exportHostBodyTemplate = (item) => {

        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag}
            onDragEnd={onDropzoneLeave}
            onDragStart={(e) => {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                e.dataTransfer.setData("dragType", "exportBar");
                inventory.current = item;
                window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                // if (nodeSelected && isPhysical === false) drawNodeDropzone();
                // else drawPrimaryDropzone();
                drawPrimaryDropzone();


            }}>
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{item?.host}</div>

        </div>
        );

    }






    const sizeBodyTemplate = (item) => {
        return (<div className="flex align-items-center icon-cell backup">{item.display_size}  <i className="pi pi-check verify-backup" onClick={() => {
            dispatch(LayoutActions.verifyBackup({ "name": item?.name, "type": item?.type, "backup_host": item?.host_name, "backup_path": item?.ex_params?.["backup_path"], "backup_date": item?.ex_params?.["modified_date"] }, onVerifyBackup))

        }} /> <img className="bar-icon" draggable={false} alt="backupBar" src={bar} onClick={() => {
            inventory.current = item;
            switch (item.type) {

                case "BACKUP":
                    setShowBackupDetails(true);
                    break;
                case "EXPORT":
                    setShowExportDetails(true);
                    break;
                default:
                    break;

            }

        }} /></div>);
    }

    const lagBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.ex_params?.lag}</div>);
    }

    const portBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.port}</div>);
    }

    const ipBodyTemplate = (item) => {
        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            leftContainer.current = d3Container.current?.getBoundingClientRect();
            d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
            e.dataTransfer.setData("dragType", item.type === types.HAPROXY ? "proxyBar" : "standbyBar");
            inventory.current = item;
            if (item.id < 0) drawPrimaryDropzone();
        }} >
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item.severity)}` }}>{item.host}</div>
        </div>);
    }

    const dbHostBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item?.host}</div>);
    }

    const dbSizeBodyTemplate = (item) => {
        return (<PgSizeCell db={item} />);
    }


    const dbNameBodyTemplate = (item, tableData) => {
        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            leftContainer.current = d3Container.current?.getBoundingClientRect();
            d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
            e.dataTransfer.setData("dragType", "databaseBar");
            if (tableData.props.className === "backup-expandable-table") e.dataTransfer.setData("dragSource", "backup-expandable");
            else e.dataTransfer.setData("dragSource", "databases");
            inventory.current = item;
            drawPrimaryDropzone();
        }}>
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item.severity)}` }}>{tableData.props.className === "backup-expandable-table" ? `Database: ${item.name}` : item.name}</div>
        </div>);

    }

    const onDropzoneLeave = (e) => {
        e.target.style.position = "";
        e.target.style.left = "";
        e.target.style.top = "";
        e.target.style.visibility = ""; d3.selectAll(".drop-zone-marker").remove();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.0");
    }

    const drawPrimaryDropzone = () => {

        d3.select(d3Container.current).insert("rect")
            .attr("class", "drop-zone-marker")
            .attr("x", 20)
            .attr("y", (layoutMid - 50))
            .attr("width", 100)
            .attr("height", 100)
            .attr("fill", "transparent")
            .attr("stroke", inventory.current?.status === false ? "red" : "green")
            .on("drop", dragToPrimaryEnd)
            .on("dragover", dragOverPrimary)
            .on("dragleave", dragLeaveNode);

    }

    const backupExpansion = () => {
        return (
            <div className={((nodeSelected && nodeSelected?.backups?.length > 1) || props.node.backups?.length > 1) ? "expansion-body" : "expansion-body single"}>
                <DataTable
                    className="backup-expandable-table severity-table"
                    selectionMode="single"
                    dataKey="id"
                    selection={dbRow}
                    onSelectionChange={(e) => setDbRow(e.value)}
                    scrollable
                    scrollHeight="flex"
                    emptyMessage="no backups found"
                    paginator={props.node?.databases?.length > 0}
                    rows={layoutCurrHeight === config.layoutMinHeight ? 10 : 50}
                    alwaysShowPaginator={false}
                    value={props.node?.databases}>
                    <Column style={{ height: "21px" }} field="name" body={dbNameBodyTemplate}></Column>
                    <Column style={{ height: "21px" }} field="size" body={dbSizeBodyTemplate}></Column>
                </DataTable>
            </div>
        );
    }

    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onTableItemDrag = (e) => {
        e.preventDefault();
        e.target.style.position = "fixed";
        e.target.style.top = e.clientY + "px";
        e.target.style.left = e.clientX + "px";
        if (e.clientX < leftContainer.current.left || e.clientY < leftContainer.current.top || e.clientY > leftContainer.current.bottom) {

            e.target.style.visibility = "hidden";

        }
        else {

            e.target.style.visibility = "";

        }
    }

    const onDropPgNode = (data) => {
        dispatch(LayoutActions.dropPgNode(data, setShowDropPgNode));
    }

    const onDropHAProxy = (data) => {
        dispatch(LayoutActions.dropHaProxy(data, setShowDropHAProxy));
    }

    const onExportPostgresNode = (data) => {
        dispatch(LayoutActions.exportPgNode(data, setShowExport));
        dispatch(LayoutActions.setDirExists(null));
        dispatch(LayoutActions.setExportExists(null));
        dispatch(LayoutActions.setDiskSpace(null));
    }

    const onBackupPostgresNode = (data) => {
        dispatch(LayoutActions.backupPgNode(data, setShowBackup));
    }

    const onAddPostgresStandby = (data) => dispatch(LayoutActions.addPgStandby(data, setShowAddPostgresStandby));
    const onAddHAProxy = (data) => dispatch(LayoutActions.addHaProxy(data, setShowAddHAProxy));

    const onAddPostgresDatabase = (data) => {
        dispatch(LayoutActions.addPgDatabase(data, showDbCredentialsInfo));

    }

    const onDropDatabase = (data) => {

        dispatch(LayoutActions.dropPgDatabase(data, setShowDropDatabase));

    }

    const onRestorePostgresDatabase = (data) => {

        dispatch(LayoutActions.restorePgDatabase(data, setShowRestorePostgresDatabase));

    }

    const onRestorePostgresNode = (data) => {
        dispatch(LayoutActions.restorePgNode(data, setShowRestorePostgresNode));
    }

    const onSwitchover = (data) => {
        dispatch(LayoutActions.switchPgNode(data, setShowSwitchover));

    }

    const onStandbyClone = (data) => {
        dispatch(LayoutActions.clonePgStandby(data, setShowCloneStandby));

    }


    const onImportPostgresNode = (data) => {
        dispatch(LayoutActions.importPgNode(data, setShowImportPostgresNode));

    }


    const onCopyDb = (data) => {
        dispatch(LayoutActions.copyPgDatabase(data, setShowCopyDb));
    }

    const onUpgrade = (data) => {
        dispatch(LayoutActions.upgradePgNode(data, setShowUpgrade));
    }

    const ShowCopyDb = (newdb, open) => {
        setDbNew(newdb);
        setShowCopyDb(open);
    }

    const onPauseAccept = () => {
        dispatch(LayoutActions.togglePgCluster({ "toggle_pause": !(props.node?.ex_params?.repmgr_paused === true || props.node?.nodes?.filter((s) => s?.ex_params?.repmgr_paused === true)?.length > 0), "node_id": props.node.id }, props.node));
    }

    const exportDateBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} >{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>);
    }

    return (
        <div className="layout-item">
            <div className="layout-item-wrapper">
                <div className="flex flex-column flex-grow-1">
                    <div className="left-title flex flex-row">
                        <TabMenu className="layout-switch" model={layoutOptions} activeIndex={layoutSwitchIndex} onTabChange={(e) => {
                            dispatch(LayoutActions.refreshContainer(props?.node?.id));
                            const index = e.index;
                            setIsPhysical(index === 0);
                            setLayoutSwitchIndex(index);
                        }} />

                        <div className="ml-auto ml-auto">{`${props.node.what.replace("sql", "SQL")} cluster: ${props.node?.cluster_content?.name}`}</div>
                        <div className="flex ml-auto mr-auto">

                            {props.node?.cluster_content?.ex_params?.primary_count > 1 ? <div className="text-red-500 text-sm mt-auto mb-auto ml-auto mr-auto"> SPLIT BRAIN </div> : null}
                            {props.node?.cluster_content?.ex_params?.primary_count && props.node?.cluster_content?.ex_params?.primary_count < 1 ? <div className="text-red-500 text-sm mt-auto mb-auto ml-auto mr-auto"> NO PRIMARY </div> : null}

                        </div>

                        <div className="flex ml-auto">
                            {Number(props.node?.version) < Number(pgVersionOptions[0].value) ?

                                <Button
                                    style={{ fontSize: "1rem" }}
                                    className="p-button-sm p-button-text"
                                    label="upgrade"
                                    onClick={() => setShowUpgrade(true)}
                                /> : null

                            }

                            <Button
                                style={{ fontSize: "1rem" }}
                                className="p-button-sm p-button-text"
                                label="status"
                                onClick={() => navigate('/status', { state: { cluster_id: props.node?.cluster, node_id: props.node?.id } })}
                            />

                            {props.node?.nodes?.length > 0 ?

                                <Button
                                    style={{ fontSize: "1rem" }}
                                    className="p-button-sm p-button-text"
                                    label="cluster options"
                                    onClick={(e) => clusterOptions.current?.show(e)}
                                /> : null}

                            <OverlayPanel ref={clusterOptions} onMouseLeave={(e) => clusterOptions.current?.hide(e)} closeOnEscape>

                                <div className="flex flex-column">
                                    <Tooltip
                                        position="left"
                                        ref={pauseTooltipRef}
                                        content="Pause repmgr. Cluster management and failover will stop until resumed."
                                        target={pauseRef.current?.getBoundingClientRect ? pauseRef.current : window.document.getElementById(`layout-toggle-${layoutItemId.current}`)} />

                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        label="drop"
                                        onClick={() => setShowDropCluster(true)}
                                    />

                                    <ToggleButton onMouseEnter={(e) => pauseTooltipRef.current?.show(e)} onMouseLeave={(e) => pauseTooltipRef.current?.hide(e)} checked={!(props.node?.ex_params?.repmgr_paused === true || props.node?.nodes?.filter((s) => s?.ex_params?.repmgr_paused === true)?.length > 0)}
                                        onChange={() => {
                                            setPauseConfirm(true);
                                        }}
                                        id={`layout-toggle-${layoutItemId.current}`}
                                        ref={pauseRef}
                                        className="p-button-sm p-button-text layout-toggle ml-auto mb-2"
                                        onLabel="pause"
                                        offLabel="unpause" />
                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text mb-2"
                                        disabled={props.node.nodes?.length < 1}
                                        label="clone"
                                        onClick={() => setShowCloneStandby(true)}
                                    />
                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text mb-2"
                                        disabled={!(props.node.nodes?.filter((s) => s.severity === "green")?.length > 0)}
                                        label="switchover"
                                        onClick={() => setShowSwitchover(true)}
                                    />
                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        label="info"
                                        onClick={() => setShowStatus(true)}
                                    />


                                </div>

                            </OverlayPanel>


                        </div>
                    </div>


                    <div style={{ zIndex: 1000 }} className="left-container">
                        <div className="left-content flex flex-row">
                            <div className="flex flex-column" >

                                {attachedProxies?.length > 0 && attachedProxies.map((h) => <Divider key={h.id} align="left" className="mt-auto mb-auto">
                                    <div className="flex flex-column" draggable onDoubleClick={() => {
                                        if (h.ind_deleted) {

                                            showWarningMessage({ "text": `${h.name} is no longer available.\nDetails cant be viewed.`, "status": h.status })

                                        }
                                        else navigate(`/status/haproxy/${h.id}`);
                                    }} onDragStart={(e) => {
                                        e.dataTransfer.setData("dragType", "proxyBar");
                                        inventory.current = h;
                                    }} onDragEnd={() => {
                                        const svg = d3.select(d3Container.current);
                                        svg.select(".trash").style("opacity", "0.0");

                                    }}>

                                        <div className="text-xs mb-2 text-gray-400 white-space-nowrap">{`${h.name} ${h.status}`}</div>
                                        <img style={{ width: "40px", height: "40px" }} draggable={false} src={haproxy} alt='proxy' />
                                        <div className="mr-auto mb-2" style={{ height: "5px", width: "40px", backgroundColor: h.severity }} />

                                    </div>
                                </Divider>)
                                }

                            </div>
                            <div className="flex flex-column">

                                <svg
                                    className={`d3-component ${attachedProxies?.length > 0 ? 'border-3 border-400' : ''}`}
                                    style={{ width: "800px", minHeight: `${layoutCurrHeight}px` }}
                                    ref={d3Container}
                                />
                                {(isPhysical && props.node?.nodes?.length > rowsNumber) || (!isPhysical && props.node?.databases?.length > rowsNumber) ? <Paginator className="left-container-paginator" first={first} rows={rows} totalRecords={isPhysical ? props.node?.nodes?.length : props.node?.databases?.length} onPageChange={onPageChange} /> : null}

                            </div>
                        </div>
                    </div>

                </div>
                <div className="right-container" style={{ minHeight: `${layoutCurrHeight + 5}px` }}
                    ref={rightContainer}>
                    <TabView style={{ width: "500px" }} activeIndex={activeIndex} onTabChange={(e) => {
                        setActiveIndex(e.index);
                        tabRef.current = e.index
                    }}>
                        <TabPanel header="Nodes">
                            <div className={`pg-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current?.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "standbyCreate");
                                    drawPrimaryDropzone();
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on Primary to create a Standby</div>
                                </div>
                                <Panel ref={thisNodes} className="collapsible-panel mt-4" header={<div onClick={() => thisNodes?.current?.toggle()} className="cursor-pointer">Nodes of this cluster</div>}  toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel>
                                        <DataTable
                                            className="severity-table"
                                            emptyMessage="no nodes found"
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={standbyRow}
                                            onSelectionChange={(e) => setStandbyRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={attachedPgNodes}>
                                            <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                            <Column style={{ height: "21px" }} field="port" body={portBodyTemplate} header="Port" sortable></Column>
                                            <Column style={{ height: "21px" }} field="ex_params.lag" header="Lag" body={lagBodyTemplate} sortable></Column>
                                            <Column style={{ height: "21px" }} field="cluster_role" header="Role" body={roleBodyTemplate} sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>
                                <Panel ref={otherNodes} collapsed className="collapsible-panel" header={<div  onClick={() => otherNodes?.current?.toggle()} className="cursor-pointer">Nodes of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel>
                                        <DataTable
                                            className="severity-table"
                                            emptyMessage="no nodes found"
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={standbyRow}
                                            onSelectionChange={(e) => setStandbyRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={foreignPgNodes}>
                                            <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                            <Column style={{ height: "21px" }} field="port" body={portBodyTemplate} header="Port" sortable></Column>
                                            <Column style={{ height: "21px" }} field="ex_params.lag" header="Lag" body={lagBodyTemplate} sortable></Column>
                                            <Column style={{ height: "21px" }} field="cluster_role" header="Role" body={roleBodyTemplate} sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>

                            </div>
                        </TabPanel>
                        <TabPanel header="Export / Import">
                            <div className={`pg-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> Drag any Node here to start an export</div>
                                {(nodeSelected || primarySelected) && <div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}> {`Exports for this ${!nodeSelected?.node ? "node" : "database"} will be stored for ${primarySelected ? props.node?.size_scheme?.export : nodeSelected?.size_scheme?.export} days`}</div><div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: types.POSTGRES } })}>change</Tag></div></div>}
                                <Panel ref={thisExports} className="collapsible-panel mt-4" header={<div onClick={() => thisExports?.current?.toggle()} className="cursor-pointer">Exports of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            selectionMode="single"
                                            className="severity-table"
                                            dataKey="id"
                                            selection={exportRow}
                                            onSelectionChange={(e) => setExportRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            emptyMessage="no exports found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={pg_exports?.filter((e) => e.cluster === props.node.cluster)}>
                                            <Column sortable style={{ height: "21px" }} field="host" header="Host" body={exportHostBodyTemplate}></Column>
                                            <Column sortable style={{ height: "21px" }} field="database" header="Database" body={(item) => item?.database ? item?.node_name : 'All databases'}></Column>
                                            <Column style={{ height: "21px" }} field="created_date" body={exportDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                                <Panel ref={otherExports}  collapsed className="collapsible-panel" header={<div onClick={() => otherExports?.current?.toggle()} className="cursor-pointer"> Exports of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            selectionMode="single"
                                            className="severity-table"
                                            dataKey="id"
                                            selection={exportRow}
                                            onSelectionChange={(e) => setExportRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            emptyMessage="no exports found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={pg_exports?.filter((e) => e.cluster !== props.node.cluster)}>
                                            <Column sortable style={{ height: "21px" }} field="host" header="Host" body={exportHostBodyTemplate}></Column>
                                            <Column sortable style={{ height: "21px" }} field="database" header="Database" body={(item) => item?.database ? item?.node_name : 'All databases'}></Column>
                                            <Column style={{ height: "21px" }} field="created_date" body={exportDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: "21px" }} field="display_size" body={sizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>

                            </div>
                        </TabPanel>
                        <TabPanel header="Databases">
                            <div className={`pg-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs mb-2" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current?.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "databaseCreate");
                                    drawPrimaryDropzone();
                                }}>
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on Primary to create Database</div>
                                </div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />   Drag from the list onto a Primary to start copying</div>
                                <Panel ref={thisDbs} className="collapsible-panel verbose mt-4" header={<div  onClick={() => thisDbs?.current?.toggle()} className="cursor-pointer">Databases of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className="severity-table"
                                            selectionMode="single"
                                            emptyMessage="no databases found"
                                            dataKey="id"
                                            selection={dbRow}
                                            onSelectionChange={(e) => setDbRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={pgDatabases?.filter((d) => d.cluster === props.node.cluster)}>
                                            <Column style={{ height: "21px" }} field="name" body={dbNameBodyTemplate} header="Name" sortable></Column>
                                            <Column style={{ height: "21px" }} field="host" body={dbHostBodyTemplate} header="Host" sortable></Column>
                                            <Column style={{ height: "21px" }} field="size" body={dbSizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>

                                <Panel ref={otherDbs} className="collapsible-panel verbose" collapsed header={<div  onClick={() => otherDbs?.current?.toggle()} className="cursor-pointer"> Databases of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className="severity-table"
                                            selectionMode="single"
                                            emptyMessage="no databases found"
                                            dataKey="id"
                                            selection={dbRow}
                                            onSelectionChange={(e) => setDbRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            value={pgDatabases?.filter((d) => d.cluster !== props.node.cluster)}>
                                            <Column style={{ height: "21px" }} field="name" body={dbNameBodyTemplate} header="Name" sortable></Column>
                                            <Column style={{ height: "21px" }} field="host" body={dbHostBodyTemplate} header="Host" sortable></Column>
                                            <Column style={{ height: "21px" }} field="size" body={dbSizeBodyTemplate} header="Size" sortable></Column>
                                        </DataTable>
                                    </ScrollPanel>
                                </Panel>

                            </div>

                        </TabPanel>
                        <TabPanel header="Backups">
                            <div className={`pg-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> {nodeSelected || primarySelected ? `Drag backup to primary to restore:` : "Select node to view backups:"} </div>
                                {(nodeSelected || primarySelected) && isPhysical && <div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}>{`Backups for this node will be stored for ${primarySelected ? props.node?.size_scheme?.backup : nodeSelected?.size_scheme?.backup} days`}</div> <div className="col"> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: types.POSTGRES } })}>change</Tag> </div> </div>}
                                {nodeSelected || primarySelected ?

                                    getAutomaticBackup()
                                    : null}

                                <Panel ref={thisBackups} className="collapsible-panel mt-4" header={<div onClick={() => thisBackups?.current?.toggle()} className="cursor-pointer">Backups of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={backupRow}
                                            onSelectionChange={(e) => setBackupRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={pg_backups?.filter((b) => b.cluster === props.node.cluster && b.ex_params?.node_version === props.node.version)}
                                            expandedRows={expandBackup}
                                            emptyMessage="no backups found"
                                            rowExpansionTemplate={backupExpansion}
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            onRowToggle={(e) => {
                                                setExpandBackup(e.data)
                                            }}>
                                            <Column style={{ height: '21px' }} field="created_date" body={backupDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                            <Column expander style={{ height: '21px', width: "0.75rem" }} />
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                                <Panel ref={otherBackups} collapsed className="collapsible-panel" header={<div  onClick={() => otherBackups?.current?.toggle()} className="cursor-pointer">Backups of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={backupRow}
                                            onSelectionChange={(e) => setBackupRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={pg_backups?.filter((b) => b.cluster !== props.node.cluster && b.ex_params?.node_version === props.node.version)}
                                            expandedRows={expandBackup}
                                            emptyMessage="no backups found"
                                            rowExpansionTemplate={backupExpansion}
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                            onRowToggle={(e) => {
                                                setExpandBackup(e.data)
                                            }}>
                                            <Column style={{ height: '21px' }} field="created_date" body={backupDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                            <Column expander style={{ height: '21px', width: "0.75rem" }} />
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                            </div>

                        </TabPanel>
                        <TabPanel header="HAProxy">
                            <div className={`pg-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs  mb-4" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current?.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "proxyCreate");
                                    drawPrimaryDropzone();
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on Primary or Standby to create a HAProxy</div>
                                </div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />   List of Primary {props.node.name} cluster  HAProxies</div>
                                <DataTable
                                    className="severity-table"
                                    selectionMode="single"
                                    dataKey="id"
                                    selection={proxyRow}
                                    onSelectionChange={(e) => setProxyRow(e.value)}
                                    scrollable
                                    emptyMessage="no haproxy found"
                                    scrollHeight="flex"
                                    paginator={attachedProxies?.length > 0}
                                    alwaysShowPaginator={false}
                                    rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                    value={attachedProxies}>
                                    <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                    <Column style={{ height: "21px" }} field="port" body={portBodyTemplate} header="Port" sortable></Column>
                                </DataTable>

                            </div>
                        </TabPanel>
                    </TabView>
                </div>
            </div>
            <AddPostgresStandbyDialog
                visible={showAddPostgresStandby}
                node={props.node}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setPgVersionExists(undefined));
                    setShowAddPostgresStandby(false)

                }}
                onOkClick={onAddPostgresStandby}

            />


            {showAddHAProxy ? <AddHAProxy
                visible={showAddHAProxy}
                node={props.selected.current}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setHaProxyExists(undefined));

                    setShowAddHAProxy(false)

                }}
                onOkClick={onAddHAProxy}

            /> : null}

            {showWarning ? <WarningDialog
                message={warningMessage}
                visible={showWarning}
                onHide={() => {

                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowWarning(false)
                }
                } />
                : null}

            {showDropPgNode ? <PostgresNodeDeleteDialog
                node={inventory.current}
                visible={showDropPgNode}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropPgNode(false)
                }}
                onOkClick={onDropPgNode}
            />
                : null}

            {showDropHAProxy ? <DropHAProxyNodeDialog
                haproxy={inventory.current}
                visible={showDropHAProxy}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropHAProxy(false)
                }}
                onOkClick={onDropHAProxy}
            />
                : null}

            {showExport ? <PostgresExportDialog
                node={props.selected.current}
                visible={showExport}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setDirExists(null));
                    dispatch(LayoutActions.setExportExists(null));
                    dispatch(LayoutActions.setDiskSpace(null));
                    setShowExport(false)
                }
                }
                onOkClick={onExportPostgresNode}
            />
                : null}


            {showExportPostgresDatabase ? <ExportPgDatabaseDialog
                db={props.selected.current}
                node={props.node}
                visible={showExportPostgresDatabase}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setDirExists(null));
                    dispatch(LayoutActions.setExportExists(null));
                    dispatch(LayoutActions.setDiskSpace(null));
                    setShowExportPostgresDatabase(false)
                }
                }
                onOkClick={onExportPostgresDatabase}
            />
                : null}

            {showBackup ? <PostgresPrimaryBackupDialog
                node={updateSchedule ? props.node : props.selected.current || props.node}
                updateSchedule={updateSchedule.current}
                pageIndex={pageIndex}
                visible={showBackup}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowBackup(false)
                }}
                onOkClick={onBackupPostgresNode}
            />
                : null}

            {showAddPostgresDatabase ? <AddPgDatabaseDialog
                node={props.node}
                visible={showAddPostgresDatabase}
                onHide={() => {
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddPostgresDatabase(false);
                }
                }
                onOkClick={onAddPostgresDatabase}
            />
                : null}

            {showDropDatabase ? <DropDatabaseDialog
                node={inventory.current}
                visible={showDropDatabase}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropDatabase(false);
                }
                }
                onOkClick={onDropDatabase}
            />
                : null}
            {
                showRestorePostgresDatabase ? <RestorePgDatabaseLayoutDialog
                    visible={showRestorePostgresDatabase}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestorePostgresDatabase(false)
                    }}
                    onOkClick={onRestorePostgresDatabase}
                    backup={inventory.current}
                /> : null
            }

            {
                showRestorePostgresNode ? <RestorePgNodeLayoutDialog
                    visible={showRestorePostgresNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestorePostgresNode(false)
                    }
                    }
                    onOkClick={onRestorePostgresNode}
                    backup={inventory.current}
                    node={props.node}
                /> : null
            }


            {
                showImportPostgresNode ? <ImportPgNodeLayoutDialog
                    visible={showImportPostgresNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowImportPostgresNode(false)
                    }
                    }
                    onOkClick={onImportPostgresNode}
                    export={inventory.current}
                    node={props.node}
                /> : null
            }


            {
                showUpgrade ? <UpgradePgNodeDialog
                    visible={showUpgrade}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowUpgrade(false)
                    }
                    }
                    onOkClick={onUpgrade}
                    node={props.node}
                /> : null
            }


            {showExportDetails ? <ExportDetailsDialog
                export={inventory.current}
                visible={showExportDetails}
                onHide={() => setShowExportDetails(false)} />
                : null
            }

            {showBackupDetails ? <BackupDetailsDialog
                backup={inventory.current}
                visible={showBackupDetails}
                onHide={() => setShowBackupDetails(false)} />
                : null
            }

            {showSwitchover ? <SwitchPostgresNodeDialog
                nodes={props.node.nodes}
                visible={showSwitchover}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowSwitchover(false)
                }}
                onOkClick={onSwitchover}
            />
                : null}


            {showCloneStandby ? <ClonePostgresStandbyDialog
                nodes={props.node.nodes}
                visible={showCloneStandby}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowCloneStandby(false)
                }}
                onOkClick={onStandbyClone}
            />
                : null}

            {showDeleteInventory ? <DeleteInventoryDialog
                visible={showDeleteInventory}
                onHide={() => {
                    setShowDeleteInventory(false);
                    dispatch(LayoutActions.setErrorMessage(""));
                }}
                onOkClick={onDeleteInventory}
                item={inventory.current} />
                : null
            }

            {showCopyDb ? <CopyPgDbDialog
                visible={showCopyDb}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowCopyDb(false)
                }
                }
                onOkClick={onCopyDb}
                db={inventory.current}
                new={dbNew}
                source={databaseSource}
                node={props.node}
            />
                : null
            }

            <ConfirmPopup
                target={pauseRef.current?.getBoundingClientRect ? pauseRef.current : window.document.getElementById(`layout-toggle-${layoutItemId.current}`)}
                visible={pauseConfirm}
                onHide={() => setPauseConfirm(false)}
                message="Are you sure?"
                accept={onPauseAccept}
            />

            {showStatus ? <PostgresStatusDialog
                node={props.node}
                visible={showStatus}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowStatus(false)
                }} />
                : null
            }


            {showInfoDialog ? <InfoDialog
                message={infoMessage}
                visible={showInfoDialog}
                icon={infoIcon}
                onHide={() => {
                    infoMessage === 'db_cred' && setShowAddPostgresDatabase(false)
                    setShowInfoDialog(false);
                }
                } />
                : null}

            {showChooseServiceDialog ? <ChooseServiceDialog
                visible={showChooseServiceDialog}
                services={props.services}
                onHide={() => setShowChooseServiceDialog(false)}
                onNext={(targetService) => setShowAddPostgresStandby(targetService ? targetService.name : true)}
            /> : null}
            {showDropCluster ? <PgDropClusterDialog
                node={props.node}
                visible={showDropCluster}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropCluster(false)
                }}
                onOkClick={onDropCluster}
            />
                : null}
            {showDropMode ? <DropModeDialog
            visible = {showDropMode}
            onHide = {() => setShowDropMode(false)}
            onOkClick = {(mode) => mode === clusterDropModes.CLUSTER ? setShowDropCluster(true) : setShowDropPgNode(true)} 
            /> : null}
        </div>
    );

}


export default PgLayoutItem;