import { useState, useRef, useEffect, useCallback } from "react";
import { TabView, TabPanel } from 'primereact/tabview';
import { nanoid } from 'nanoid';
import * as d3 from "d3";
import trashImage from "../../../assets/images/trash.png";
import plusIcon from "../../../assets/images/plus.png";
import minusIcon from "../../../assets/images/minus.png";
import node_white from "../../../assets/images/node_white.png";
import node_unavail from '../../../assets/images/node_unavail.png'
import node_un_white from '../../../assets/images/node_un_white.png';
import cluster from "../../../assets/images/cluster.png";
import cluster_white from "../../../assets/images/cluster_white.png";
import node_avail from "../../../assets/images/node.png";
import bar from "../../../assets/images/bar.png";
import db_unavail from "../../../assets/images/db_unavail.png";
import db_un_white from "../../../assets/images/db_un_white.png";
import dbImage from "../../../assets/images/db.png";
import { useNavigate } from 'react-router-dom';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import moment from "moment";
import WarningDialog from "../../../dialogs/warningDialog";
import { useDispatch, useSelector } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import BackupDetailsDialog from "../../../dialogs/postgres/backupDetailsDialog";
import { Button } from "primereact/button";
import DeleteInventoryDialog from "../../../dialogs/deleteInventoryDialog";
import { onSort } from "../../../util/table";
import getColorBySeverity from "../../../util/colorBySeverity";
import RedisNodeBackupDialog from "../../../dialogs/redis/redisNodeBackupDialog";
import SwitchRedisNodeDialog from "../../../dialogs/redis/switchRedisNodeDialog";
import InfoDialog from "../../../dialogs/infoDialog";
import config, { clusterRoles, types } from "../../../config";
import db_white from "../../../assets/images/db_white.png";
import DropRedisNodeDialog from "../../../dialogs/redis/dropRedisNodeDialog";
import RestoreRedisNodeLayoutDialog from "../../../dialogs/redis/restoreRedisNodeLayoutDialog";
import AddRedisNodeDialog from "../../../dialogs/redis/addRedisNode";
import RemoveRedisClusterDialog from "../../../dialogs/redis/removeRedisClusterDialog";
import SentinelMonitorMasterDialog from "../../../dialogs/redis/sentinelMonitorMasterDialog";
import StepdownRedisNodeDialog from "../../../dialogs/redis/stepdownRedisNodeDialog";
import { knobs } from "../../../util/templates";
import { Tag } from "primereact/tag";
import { ScrollPanel } from "primereact/scrollpanel";
import { ConfirmPopup } from "primereact/confirmpopup";
import { Panel } from "primereact/panel";
import ChooseServiceDialog from "../../../dialogs/chooseServiceDialog";
import { OverlayPanel } from "primereact/overlaypanel";

const RedisLayoutItem = (props) => {
    const navigate = useNavigate();
    const [showSwitchover, setShowSwitchover] = useState(false);
    const [showNodes, setShowNodes] = useState(true);
    const d3Container = useRef(null);
    const [activeIndex, setActiveIndex] = useState(0);
    const [nodeSelected, setNodeSelected] = useState(false);
    const [clusterSelected, setClusterSelected] = useState(false);
    const [attachedMasters, setAttachedMasters] = useState(null);
    const [attachedSlaves, setAttachedSlaves] = useState(null);
    const [clusterSentinels, setClusterSentinels] = useState(null);
    const [showMonitorMaster, setShowMonitorMaster] = useState(false);
    const [showSlaves, setShowSlaves] = useState(null);
    const currentNodeCoords = useRef({ x: 0, y: 0 });
    const tabRef = useRef(0);
    const inventory = useRef(null);
    const [showDeleteInventory, setShowDeleteInventory] = useState(false);
    const [warningMessage, setWarningMessage] = useState('');
    const [showWarning, setShowWarning] = useState(false);
    const leftContainer = useRef(null);
    const rightContainer = useRef(null);
    const tabsPos = useRef(null);
    const trashPos = useRef(null);
    const [showBackup, setShowBackup] = useState(false);
    const [showAddNode, setShowAddNode] = useState(false);
    const [showStepdown, setShowStepdown] = useState(false);
    const [showBackupDetails, setShowBackupDetails] = useState(false);
    const [showDropNode, setShowDropNode] = useState(false);
    const [showRemoveCluster, setShowRemoveCluster] = useState(false);
    const [showRestoreBackupNode, setShowRestoreBackupNode] = useState(false);
    const [backupRow, setBackupRow] = useState(null);
    const [sentinelRow, setSentinelRow] = useState(null);
    const updateSchedule = useRef(null);
    const [slaveRow, setSlaveRow] = useState(null);
    const [masterRow, setMasterRow] = useState(null);
    const [showInfoDialog, setShowInfoDialog] = useState(false);
    const [infoMessage, setInfoMessage] = useState(false);
    const layoutItemId = useRef(null);
    const [layoutCurrHeight, setLayoutCurrHeight] = useState(config.layoutMinHeight);
    const [layoutMid, setLayoutMid] = useState(config.layoutMinHeight / 2);
    const [updateMonitor, setUpdateMonitor] = useState(false);
    const [removeScheduleConfirm, setRemoveScheduleConfirm] = useState(false);
    const [pageIndex, setPageIndex] = useState(0);
    const [childRole, setChildRole] = useState("");
    const { redis_backups } = useSelector((state) => state.layout)
    const [showChooseServiceDialog, setShowChooseServiceDialog] = useState(false);
    const clusterOptions = useRef(null);
    // const thisDbs = useRef(null);
    // const otherDbs = useRef(null);
    const thisBackups = useRef(null);
    const otherBackups = useRef(null);
    // const thisExports = useRef(null);
    // const otherExports = useRef(null);



    const getAutomaticBackup = () => {
        let backup = redis_backups?.filter(b => b?.cluster === props.node && b.ex_params?.node_version === props.node?.ex_params?.version && b?.is_sched === true)?.[0]
        updateSchedule.current = backup;
        const popup_id = `remove-schedule-button-${props.node.id}`
        return (<div draggable={false} className="info-section text-xs grid"> <i className="pi pi-circle-fill" />  <div className="col-fixed" style={{ width: "300px" }}> Automatic backup: {backup?.ex_params?.description || "No"}</div> {backup ? <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-pencil" onClick={() => {
            setShowBackup(true);
        }}>change</Tag> <Tag className='right-container-button ml-2' id={popup_id} icon="pi pi-trash" severity="danger" onClick={() => setRemoveScheduleConfirm(true)} >delete</Tag>    <ConfirmPopup
                target={document.getElementById(popup_id)}
                visible={removeScheduleConfirm}
                onHide={() => setRemoveScheduleConfirm(false)}
                message="Are you sure?"
                accept={() => dispatch(LayoutActions.removeRedisSchedule({ "node_id": props.node.id }, setRemoveScheduleConfirm))}
            /></div> : <div className="col"><Tag className="right-container-button" severity="success" icon="pi pi-plus" onClick={() => {

                setPageIndex(1);
                setShowBackup(true);
            }}>create</Tag> </div>}  </div>)
    }


    useEffect(() => {
        let height = config.layoutMinHeight;

        if (showNodes) {
            if (props.node?.masters?.length > 3) height = height + (80 * (props.node.masters?.length - 3));
            if (showSlaves) {
                const keys = Object.keys(showSlaves);
                height = height + (80 * (keys?.filter((k) => showSlaves[k] !== false)?.length)) + 80 * (props.node?.masters?.length + 1)
            }


        }


        setLayoutCurrHeight(height);
        setLayoutMid(height / 2);


    }, [props.node?.masters, showNodes, showSlaves])

    const dispatch = useDispatch();

    const onVerifyBackup = (open, message) => {

        setInfoMessage(message);
        setShowInfoDialog(open);

    }



    const onDeleteInventory = (data) => {

        switch (inventory.current.type) {
            case "BACKUP":
                dispatch(LayoutActions.dropBackup(data, setShowDeleteInventory));
                break;

            default:
                break;

        }

    }

    const onRemoveCluster = (data) => {

        dispatch(LayoutActions.dropCluster(data, setShowRemoveCluster));

    }

    const addImage = useCallback((x, y, r_x, r_y, node) => {

        function onClickNode(svg, id, node) {

            setClusterSelected(false);
            props.selected.current = node;
            setNodeSelected(node);
            let elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute('slave_of') === "false" ? dbImage : node_avail : elements[i].getAttribute('slave_of') === "false" ? db_unavail : node_unavail);
            }

            elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }


            elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }


            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            svg.select("#node-" + id).attr("xlink:href", node.ind_deleted ? (node?.ex_params?.slave_of !== undefined && node?.ex_params?.slave_of === false) ? db_un_white : node_un_white : (node?.ex_params?.slave_of !== undefined && node?.ex_params?.slave_of === false) ? db_white : node_white);

            svg.select(`#text-${id}`).style("fill", "white");
        }

        function goNodeDetails(node) {


            if (node.ind_deleted) {
                showWarningMessage({ "text": `${node.name} is no longer available.\nDetails cant be viewed.`, "status": node.status })

            }
            else navigate(`/status/redis/${node.id}`);
        }

        const onDragStart = (svg, id, node) => (d) => {
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();

            onClickNode(svg, id, node);

        }


        const onDragging = (svg, id, node) => (d) => {

            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }


            if (d.sourceEvent.clientX <= leftContainer.current.left || d.sourceEvent.clientY >= leftContainer.current.bottom || d.sourceEvent.clientY <= tabsPos.current.top || d.sourceEvent.clientX >= tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            setNodeSelected(node);
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#node-" + id).style("cursor", "grabbing");
                d3.select("#node-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx) + "," + (dy) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 51) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", "translate(" + (dx + 40) + "," + (dy + 10) + ")");
                d3.select("#expand-" + id).attr("transform", () => "translate(" + (dx + 30) + "," + (dy + 8) + ")");

                if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                    svg.select(".trash").style("opacity", "1.0");

                }
                else {

                    svg.select(".trash").style("opacity", "0.5");

                }


                if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientY >= tabsPos.current.top && d.sourceEvent.clientY <= tabsPos.current.bottom && d.sourceEvent.clientX <= tabsPos.current.right) {
                    switch (tabRef.current) {
                        case 2:
                            if (props.selected.current?.cluster_role !== clusterRoles.PRIMARY) svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                        default:
                            svg.select(`#node-${id}`).style("cursor", "not-allowed");
                            break;
                    }
                }

                if (d.sourceEvent.clientX >= tabsPos.current.left && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 125) {
                    setActiveIndex(0);
                    tabRef.current = 0;
                }
                else if (d.sourceEvent.clientX >= tabsPos.current.left + 125 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 250) {
                    setActiveIndex(1);
                    tabRef.current = 1;
                }
                else if (d.sourceEvent.clientX >= tabsPos.current.left + 250 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 375) {
                    setActiveIndex(2);
                    tabRef.current = 2;

                }
                else if (d.sourceEvent.clientX >= tabsPos.current.left + 375 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY < leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 500) {
                    setActiveIndex(3);
                    tabRef.current = 3;

                }
            }

            let elements = window.document.querySelectorAll(`.redis-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = tabRef.current === 0 || tabRef.current === 1 || (tabRef.current === 2 && props.selected?.current?.cluster_role !== clusterRoles.PRIMARY) ? "" : "0 0 0 0.2rem rgb(40 167 69 / 72%)";

            }
        }

        const onDragEnd = (svg, id, node) => (d) => {

            if (d.sourceEvent.detail === 2) {

                goNodeDetails(node);
                return;

            }

            setNodeSelected(node);
            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                if (node.ind_deleted) {


                    showWarningMessage({

                        "text": `${node.name.toUpperCase()} is unavailable`,
                        "status": `${node.status}`
                    });
                }
                else if (props.node.slaves?.filter((s) => s?.ex_params?.slave_of === node?.id)?.length > 0) {
                    showWarningMessage({

                        "text": "This is a master node with replicas.\nPerform a failover to one of its replicas first"
                    });
                }
                else {

                    inventory.current = node;
                    setShowDropNode(true);

                }

            }

            if (d.sourceEvent.clientX >= tabsPos.current.left && d.sourceEvent.clientX >= tabsPos.current.top && d.sourceEvent.clientY <= tabsPos.current.bottom && d.sourceEvent.clientX <= tabsPos.current.right) {


                if (node.ind_deleted) {


                    showWarningMessage({

                        "text": `${node.name.toUpperCase()} is unavailable`,
                        "status": `${node.status}`
                    });
                }
                else {

                    if (node.cluster_role) {

                        switch (tabRef.current) {
                            case 2:
                                if (props?.selected.current?.cluster_role === clusterRoles.PRIMARY) {
                                    if (props.node?.sentinels?.filter((s) => props.selected?.current?.ex_params?.monitored_by?.includes(s?.id?.toString()))?.length < 1)
                                        showWarningMessage({ "text": `${props?.selected?.current?.name} is not monitored by any sentinel` })
                                    else {
                                        setUpdateMonitor(true);
                                        setShowMonitorMaster(true);
                                    }
                                }
                                break
                            case 3:

                                setPageIndex(0);
                                setShowBackup(true);
                                break;
                            default:
                                break;

                        }
                    }

                }

            }

            d3.select("#node-" + id).remove();
            d3.select("#line-" + id).remove();
            d3.select("#text-" + id).remove();
            d3.select("#severity-" + id).remove();
            d3.select("#expand-" + id).remove();
            d3.select("#d3-tooltip-container-" + id).remove();


            const master_x = 300;
            let i = props.node?.masters?.findIndex((m) => m.id === node.id)
            if (i !== -1) {

                const master_y = (i + 1) * layoutCurrHeight / (props.node.masters?.length + 1) - (42 / (i + 1));
                currentNodeCoords.current.x = master_x;
                currentNodeCoords.current.y = master_y + 20;
                addImage(master_x, master_y, 100, layoutMid + 10, node);

            }


            const slave_x = master_x + 200;
            i = props.node?.masters?.findIndex((m) => m.id === node?.ex_params?.slave_of);
            if (i !== -1) {


                const master_y = (i + 1) * layoutCurrHeight / (props.node.masters?.length + 1) - (42 / (i + 1));
                const master_id = props?.node?.masters[i].id
                const master_slaves = props.node.slaves?.filter((s) => s?.ex_params?.slave_of === master_id);
                if (master_slaves) {
                    const j = master_slaves.findIndex((s) => s.id === node.id);
                    if (j !== -1) {
                        const slave_y = master_y + 80 * (j - 1);
                        addImage(slave_x, slave_y, master_x + 40, master_y + 20, node);
                    }

                }



            }



            svg.select(".trash").style("opacity", "0.0");
            d3.select("#node-" + id).style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.redis-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        const svg = d3.select(d3Container.current);
        const id = nanoid();

        const image = svg.insert("image")
            .attr("xlink:href", (props.selected.current?.id === node.id) ? node.ind_deleted ? (node?.ex_params?.slave_of !== undefined && node?.ex_params?.slave_of === false) ? db_un_white : node_un_white : (node?.ex_params?.slave_of !== undefined && node?.ex_params?.slave_of === false) ? db_white : node_white : node.ind_deleted ? (node?.ex_params?.slave_of !== undefined && node?.ex_params?.slave_of === false) ? db_unavail : node_unavail : (node?.ex_params?.slave_of !== undefined && node?.ex_params?.slave_of === false) ? dbImage : node_avail)
            .attr("class", "node")
            .attr("id", "node-" + id)
            .attr('x', 0)
            .attr('y', 0)
            .attr("width", 40)
            .attr("height", 60)
            .attr("available", node.ind_deleted ? false : true)
            .attr("slave_of", node?.ex_params?.slave_of ? true : false)
            .attr("draggable", true)
            .attr("transform", () => "translate(" + x + "," + y + ")")
            .on("mouseover", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

            })
            .on("mouseout", function (d) {
                svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

            })

        const x0 = r_x;
        const y0 = r_y;
        const x1 = x;
        const y1 = y + 20;

        const path = d3.path()
        path.moveTo(x1, y1)
        path.bezierCurveTo((x0 + x1) / 2, y1, (x0 + x1) / 2, y0, x0, y0);

        svg.insert("path")
            //.attr("d", d3.line()([[100, 220], [x, y + 30]]))
            .attr("d", path)
            .attr("id", "line-" + id).attr("transform", "translate(" + (0) + "," + (10) + ")")
            .style("fill", "none")
            .style("stroke", "#ccc")
            .style("stroke-width", 1);
        if (showSlaves && showSlaves.hasOwnProperty(node.name))
            svg.insert("image")
                .attr("xlink:href", showSlaves[node.name] === true ? minusIcon : plusIcon)
                .attr("id", "expand-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 15)
                .attr("height", 15)
                .attr("transform", () => "translate(" + (x + 30) + "," + (y + 5) + ")")
                .on("click", () => { setShowSlaves({ ...showSlaves, [node.name]: !showSlaves[node.name] }) });
        svg.insert("text")
            .attr("transform", "translate(" + (x) + "," + (y) + ")")
            .attr("id", "text-" + id)
            .attr("class", "node-what-text")
            .style("font-size", "12px")
            .style("fill", `${props.selected.current?.id === node.id ? "white" : "gray"}`)
            .style("cursor", "pointer")
            .text(`${node.name ? node.name : ""} ${node.status ? node.status : ""}`).on("dblclick", () => goNodeDetails(node)).on("click", function (d) {
                currentNodeCoords.current.x = x1;
                currentNodeCoords.current.y = y1 + 20;
                onClickNode(svg, id, node);
            });

        //draw node severity strip
        svg.insert("rect").attr('x', 0)
            .attr('y', 0)
            .attr("width", 35)
            .attr("height", 5).attr("fill", getColorBySeverity(node.severity))
            .attr("transform", "translate(" + (x1 + 2) + "," + (y1 + 31) + ")")
            .attr("id", "severity-" + id)

        svg.insert("foreignObject")
            .attr("class", "d3-tooltip-container")
            .attr("id", `d3-tooltip-container-${id}`)
            .attr("transform", "translate(" + (x + 40) + "," + (y + 10) + ")")
            .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${node.ind_started ? "up" : "down"} <br/> Alerts: ${node?.alert?.length > 40 ? node.alert.substring(0, 37).concat("...") : node?.alert} </div>`)

        var drag_behavior = d3.drag()
            .on("start", onDragStart(svg, id, node))
            .on("drag", onDragging(svg, id, node))
            .on("end", onDragEnd(svg, id, node));

        image.call(drag_behavior);
    }, [navigate, props.node, layoutCurrHeight, layoutMid, showSlaves, props.selected]);




    useEffect(() => {


        const onTableItemDrop = (event) => {
            let type = event.dataTransfer.getData("dragType");

            switch (type) {
                case "nodeBar":
                case "sentinelBar":
                    if (inventory.current?.ind_deleted) {


                        showWarningMessage({

                            "text": `${inventory.current.name.toUpperCase()} is unavailable`,
                            "status": `${inventory.current.status}`
                        });
                    }

                    else {
                        if (props.node.slaves?.filter((s) => s?.ex_params?.slave_of === inventory.current?.node)?.length > 0) {
                            showWarningMessage({

                                "text": "This is a master node with replicas, cannot be removed.\nPerform a failover of one of its replicas first"
                            });
                        }
                        else setShowDropNode(true);
                    }
                    break;
                case "backupBar":
                    setShowDeleteInventory(true);
                    break;
                default:
                    break;

            }

        }


        function onClusterClick(svg, id) {

            setNodeSelected(false);

            let elements = window.document.querySelectorAll(".cluster");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? cluster : db_unavail);
            }

            elements = window.document.querySelectorAll(".root");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute("available") === "true" ? dbImage : db_unavail);
            }

            elements = window.document.querySelectorAll(".node");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].setAttribute("href", elements[i].getAttribute('available') === "true" ? elements[i].getAttribute("slave_of") === "false" ? dbImage : node_avail : elements[i].getAttribute("slave_of") === "false" ? db_unavail : node_unavail);
            }

            elements = window.document.querySelectorAll(".root-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }

            elements = window.document.querySelectorAll(".node-what-text");

            for (let i = 0; i < elements?.length; i++) {
                elements[i].style.fill = "gray";
            }


            svg.select(".cluster").attr("xlink:href", props.node.ind_deleted ? db_un_white : cluster_white);
            props.selected.current = props.node;
            setClusterSelected(true);
            svg.select(`#text-${id}`).style("fill", "white");


        }


        const onDragStartCluster = (svg, id, cluster) => (d) => {
            trashPos.current = svg.select(".trash").node().getBoundingClientRect();
            leftContainer.current = d3Container.current.getBoundingClientRect();
            tabsPos.current = rightContainer.current.getBoundingClientRect();

            // Draw cluster what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 10) + ")")
                .attr("id", "text-copy" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node.id ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.name ? props.node.name : ""} ${props.node.ex_params.status ? props.node.ex_params.status : ""}`);

            // Draw cluster icon
            svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id) ? props.node.ind_deleted ? db_un_white : cluster_white : props.node.ind_deleted ? db_unavail : cluster)
                .attr("class", "cluster")
                .attr("id", "cluster-copy" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + layoutMid + ")")
                .attr("available", !props.node.ind_deleted);

            //draw cluster severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-copy" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.ex_params.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 40) + ")")


            // Draw cluster plus/minus icon

            if (props.node.masters?.length > 0) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-copy" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 5) + ")");
            }

            onClusterClick(svg, id)

        }

        const onDraggingCluster = (svg, id, cluster) => (d) => {

            const viewportHeight = window.innerHeight;
            const overViewport = d.sourceEvent.clientY < 0 || d.sourceEvent.clientY + Math.abs(d.dy) > viewportHeight;
            if (overViewport) {
                leftContainer.current = d3Container.current?.getBoundingClientRect();
                tabsPos.current = rightContainer.current?.getBoundingClientRect();
                trashPos.current = svg.select(".trash").node()?.getBoundingClientRect();

            }

            if (overViewport) {
                window.scrollBy(0, (d.dy < 0 ? -1 : 1) * 6.0);
            }

            if (d.sourceEvent.clientX <= leftContainer.current.left || d.sourceEvent.clientY >= leftContainer.current.bottom || d.sourceEvent.clientY <= tabsPos.current.top || d.sourceEvent.clientX >= tabsPos.current.right)
                svg.style("overflow", "hidden");
            else svg.style("overflow", "visible");
            if (d.sourceEvent.srcElement) {
                // get the mouse positions
                var dx = d.x - 20,
                    dy = d.y - 30;

                // move the dragged element
                d3.select("#cluster-" + id).attr("transform", () => "translate(" + dx + "," + dy + ")");
                d3.select("#text-" + id).attr("transform", "translate(" + (dx - 54) + "," + (dy - 8) + ")");
                d3.select("#severity-" + id).attr("transform", "translate(" + (dx + 2) + "," + (dy + 40) + ")");
                d3.select("#expand-" + id).attr("transform", () => "translate(" + (dx + 30) + "," + (dy - 5) + ")");
                d3.select("#d3-tooltip-container-" + id).attr("transform", () => "translate(" + (dx + 50) + "," + (dy - 5) + ")");
            }

            svg.select(".cluster").style("cursor", "grabbing");
            if (d.sourceEvent.clientX > trashPos.current.left && d.sourceEvent.clientY < trashPos.current.bottom && d.sourceEvent.clientX < trashPos.current.right && d.sourceEvent.clientY > trashPos.current.top) {

                svg.select(".trash").style("opacity", "1.0");

            }
            else {

                svg.select(".trash").style("opacity", "0.5");

            }

            if (d.sourceEvent.clientX > tabsPos.current.left && d.sourceEvent.clientX > tabsPos.current.top && d.sourceEvent.clientY < tabsPos.current.bottom && d.sourceEvent.clientX < tabsPos.current.right) {
                switch (tabRef.current) {
                    default:
                        svg.select(".cluster").style("cursor", "not-allowed");
                        break;
                }
            }



            if (d.sourceEvent.clientX >= tabsPos.current.left && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 125) {
                setActiveIndex(0);
                tabRef.current = 0;

            }
            else if (d.sourceEvent.clientX >= tabsPos.current.left + 125 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 250) {
                setActiveIndex(1)
                tabRef.current = 1;
            }
            else if (d.sourceEvent.clientX >= tabsPos.current.left + 250 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 375) {
                setActiveIndex(2);
                tabRef.current = 2;

            }
            else if (d.sourceEvent.clientX >= tabsPos.current.left + 375 && d.sourceEvent.clientY >= leftContainer.current.top - 30 && d.sourceEvent.clientY <= leftContainer.current.top && d.sourceEvent.clientX <= tabsPos.current.left + 500) {
                setActiveIndex(3);
                tabRef.current = 3;


            }

            let elements = window.document.querySelectorAll(`.redis-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        const onDragEndCluster = (svg, id, cluster) => (d) => {

            if (d.sourceEvent.clientX >= trashPos.current.left && d.sourceEvent.clientY <= trashPos.current.bottom && d.sourceEvent.clientX <= trashPos.current.right && d.sourceEvent.clientY >= trashPos.current.top) {

                if (props.node.ind_deleted)
                    showWarningMessage({

                        "text": `${props.node.name.toUpperCase()} is unavailable`,
                        "status": `${props.node.ex_params.status}`
                    });

                else {
                    if (props.node.masters?.length > 0 || props.node.sentinels?.length > 0) showWarningMessage({ "text": `This is cluster with nodes, cannot be unregistered.\nDrop cluster nodes first.` })
                    else {

                        inventory.current = props.node
                        setShowRemoveCluster(true);

                    }

                }

            }

            if (d.sourceEvent.clientX >= tabsPos.current.left && d.sourceEvent.clientX >= tabsPos.current.top && d.sourceEvent.clientY <= tabsPos.current.bottom && d.sourceEvent.clientX <= tabsPos.current.right) {


                switch (tabRef.current) {
                    default:
                        break;

                }

            }



            svg.select(".trash").style("opacity", "0.0");

            d3.select("#cluster-copy" + id).remove();
            d3.select("#text-copy" + id).remove();
            d3.select("#severity-copy" + id).remove();
            d3.select("#expand-copy" + id).remove();

            d3.select("#cluster-" + id).attr("transform", () => "translate(" + 50 + "," + layoutMid + ")");
            d3.select("#text-" + id).attr("transform", () => "translate(" + 0 + "," + (layoutMid - 10) + ")");
            d3.select("#severity-" + id).attr("transform", "translate(" + (52) + "," + (layoutMid + 40) + ")");
            d3.select("#expand-" + id).attr("transform", "translate(" + (80) + "," + (layoutMid - 5) + ")");
            d3.select(`#d3-tooltip-container-${id}`).attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
            svg.select(".cluster").style("cursor", "pointer");
            let elements = window.document.querySelectorAll(`.redis-tab-content-${layoutItemId.current}`);
            for (let i = 0; i < elements?.length; i++) {

                elements[i].style.boxShadow = "";

            }
        }

        if (d3Container.current) {
            const svg = d3.select(d3Container.current);
            svg.selectChildren("*").remove();

            const id = nanoid();



            if (!props.selected.current) {
                setClusterSelected(props.node);

            }

            svg.insert("foreignObject")
                .attr("class", "d3-tooltip-container")
                .attr("id", `d3-tooltip-container-${id}`)
                .attr("transform", "translate(" + 100 + "," + (layoutMid - 5) + ")")
                .html(`<div id ='status-tooltip-${id}' class = 'field-collapsed'> Status: ${props.node.ind_started ? "up" : "down"} <br/> Alerts:  ${props.node?.alert?.length > 40 ? props.node.alert.substring(0, 37).concat("...") : props.node?.alert} </div>`)
            // Draw trash icon    
            svg.insert("image")
                .attr("xlink:href", trashImage)
                .attr("class", "trash")
                .attr("id", `trash-${id}`)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 100)
                .attr("height", 100)
                .attr("transform", () => "translate(" + 0 + "," + (layoutCurrHeight - 100) + ")")
                .on("dragover", dragOverTrash)
                .on("dragleave", dragLeaveTrash)
                .on("drop", onTableItemDrop);


            // Draw cluster what text
            svg.insert("text")
                .attr("transform", "translate(" + (0) + "," + (layoutMid - 10) + ")")
                .attr("id", "text-" + id)
                .attr("class", "root-what-text")
                .style("font-size", "12px")
                .style("fill", `${props.selected.current?.id === props.node.id || !layoutItemId.current ? "white" : "gray"}`)
                .style("cursor", "pointer")
                .text(`${props.node.name ? props.node.name : ""} ${props.node.ex_params.status ? props.node.ex_params.status : ""}`)
                .on("click", () => onClusterClick(svg, id));

            // Draw cluster icon
            const imageCluster = svg.insert("image")
                .attr("xlink:href", (props.selected.current?.id === props.node.id || !layoutItemId.current) ? props.node.ind_deleted ? db_un_white : cluster_white : props.node.ind_deleted ? db_unavail : cluster)
                .attr("class", "cluster")
                .attr("id", "cluster-" + id)
                .attr('x', 0)
                .attr('y', 0)
                .attr("width", 40)
                .attr("height", 40)
                .attr("transform", () => "translate(" + 50 + "," + layoutMid + ")")
                .attr("available", !props.node.ind_deleted)
                .on("mouseover", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "d3-tooltip");

                })
                .on("mouseout", function (d) {
                    svg.select(`#status-tooltip-${id}`).attr("class", "field-collapsed");

                });

            layoutItemId.current = id;

            const dragBehaviorCluster = d3.drag()
                .on("start", onDragStartCluster(svg, id, props.node))
                .on("drag", onDraggingCluster(svg, id, props.node))
                .on("end", onDragEndCluster(svg, id, props.node));
            imageCluster.call(dragBehaviorCluster);


            //draw cluster severity strip
            svg.insert("rect").attr('x', 0)
                .attr('y', 0)
                .attr("width", 35)
                .attr("id", "severity-" + id)
                .attr("height", 5).attr("fill", getColorBySeverity(props.node.ex_params.severity))
                .attr("transform", "translate(" + (52) + "," + (layoutMid + 40) + ")")



            // Draw cluster plus/minus icon

            if (props.node.masters?.length > 0) {

                svg.insert("image")
                    .attr("xlink:href", showNodes ? minusIcon : plusIcon)
                    .attr("id", "expand-" + id)
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr("width", 15)
                    .attr("height", 15)
                    .attr("transform", () => "translate(" + 80 + "," + (layoutMid - 5) + ")")
                    .on("click", () => { setShowNodes(!showNodes) });

                if (showNodes) {

                    let _showSlaves = {}

                    props.node.masters?.forEach((master, index) => {
                        const master_x = 300;
                        const master_y = (index + 1) * layoutCurrHeight / (props.node.masters?.length + 1) - (42 / (index + 1));
                        addImage(master_x, master_y, 100, layoutMid + 10, master);
                        if (props.node?.slaves) {
                            const master_slaves = props.node?.slaves?.filter((s) => s.ex_params?.slave_of === master.id)
                            if (master_slaves?.length > 0) {
                                _showSlaves[master.name] = false
                                if (showSlaves && showSlaves.hasOwnProperty(master.name) && showSlaves[master.name]) {

                                    master_slaves.forEach((slave, slave_index) => {
                                        const slave_x = master_x + 200
                                        const slave_y = master_y + 80 * (slave_index - 1);
                                        addImage(slave_x, slave_y, master_x + 40, master_y + 20, slave);
                                    });

                                }
                            }
                        }
                    });



                    if (!showSlaves) setShowSlaves(_showSlaves)
                }
            }
        }
    }, [showNodes, props.node, navigate, props.selected, layoutCurrHeight, layoutMid, showSlaves, addImage, dispatch]);




    useEffect(() => {

        if (props.node) {
            let tempMasters = [];
            let tempSlaves = [];
            let tempSentinels = [];
            if (props.node.masters?.length > 0) {
                tempMasters = [...props.node.masters];
            }
            else {

                tempMasters = [];
            }
            if (props.node?.slaves?.length > 0) {
                tempSlaves = props.node?.slaves?.filter((s) => s?.ex_params?.slave_of === nodeSelected?.id);
            }
            else {

                tempSlaves = [];
            }
            if (props.node.sentinels?.length > 0) {
                tempSentinels = [...props.node.sentinels];
            }
            else {

                tempSentinels = [];
            }
            setAttachedMasters(tempMasters);
            setClusterSentinels(tempSentinels);
            setAttachedSlaves(tempSlaves);
        }





    }, [props.node, nodeSelected]);

    useEffect(() => {

        const updateSelected = () => {

            if (props.selected.current?.type === types.REDIS) {

                if (props.selected.current?.id === props.node.id) {

                    let nodeSelected = props.selection?.filter((n) => n.id === props.selected.current?.id);
                    if (nodeSelected?.length > 0) {
                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }

                }
                else {
                    let nodeSelected;
                    nodeSelected = props.node.masters?.filter((n) => n.id === props.selected.current.id);
                    if (nodeSelected?.length === 0) {
                        nodeSelected = props.node.slaves?.filter((n) => n.id === props.selected.current.id);
                    }

                    if (nodeSelected?.length > 0) {

                        setNodeSelected(nodeSelected[0]);
                        props.selected.current = nodeSelected[0];

                    }
                }


            }

        }


        if (props.selection?.length > 0) updateSelected();

    }, [props.node, props.selection, props.selected])

    function dragToClusterEnd(event) {
        let type = event.dataTransfer.getData("dragType");

        if (props.node.ind_deleted) {
            showWarningMessage({

                "text": `${props.node.name.toUpperCase()} is unavailable`,
                "status": `${props.node.ex_params.status}`
            });

            return;

        }

        if (inventory.current?.status === false) {
            showWarningMessage({

                "text": "Cannot perform this action",
                "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
            });

            return;



        }

        switch (type) {
            case "masterCreate":
                setChildRole(clusterRoles.PRIMARY);
                props.services?.length < 1 ? setShowAddNode(true) : setShowChooseServiceDialog(true);
                break;
            case "sentinelCreate":
                setChildRole(clusterRoles.SENTINEL);
                props.services?.length < 1 ? setShowAddNode(true) : setShowChooseServiceDialog(true);
                break;
            default:
                break;



        }
    }

    const onShowAddSlave = (error) => {
        if (error === false) {
            setChildRole(clusterRoles.STANDBY);
            setShowAddNode(true);
        }
        else showWarningMessage({ 'text': error });
    }


    function dragToNodeEnd(event) {

        if (nodeSelected.ind_deleted) {
            showWarningMessage({

                "text": `${props.node.name.toUpperCase()} is unavailable`,
                "status": `${props.node.ex_params.status}`
            });

            return;

        }

        if (inventory.current?.status === false) {
            showWarningMessage({

                "text": "Cannot perform this action",
                "status": `This ${inventory.current.type.toLowerCase()} did not complete successfully`
            });

            return;



        }

        let type = event.dataTransfer.getData("dragType");

        switch (type) {
            case "backupBar":
                setShowRestoreBackupNode(true);
                break;
            case "sentinelBar":
                if (inventory.current?.id > 0 && inventory.current.cluster_role === clusterRoles.SENTINEL) {
                    setUpdateMonitor(false);
                    setShowMonitorMaster(true);
                }
                break;
            case "replicaCreate":
                dispatch(LayoutActions.checkRedisClusterHealth({ 'node_id': nodeSelected?.id }, onShowAddSlave));
                break;
            default:
                break;

        }

    }

    function dragOverCluster(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    }

    function dragOverNode(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", inventory.current?.status === false ? "rgba(255, 0, 0, 0.2)" : "rgba(157, 245, 193, 0.2)");

    }

    function dragLeaveNode(event) {
        event.preventDefault();
        d3.select(d3Container.current)
            .select(".drop-zone-marker")
            .attr("fill", "transparent");

    }

    const dragOverTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "1.0");

    }

    const dragLeaveTrash = (event) => {

        event.preventDefault();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.5");

    }

    const backupDateBodyTemplate = (item) => {
        return (
            <div className="flex align-items-center" draggable onDrag={onTableItemDrag}
                onDragEnd={onDropzoneLeave}
                onDragStart={(e) => {
                    leftContainer.current = d3Container.current.getBoundingClientRect();
                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                    d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
                    window.document.getElementById(`trash-${layoutItemId.current}`).style.opacity = 0.5;
                    e.dataTransfer.setData("dragType", "backupBar");
                    inventory.current = item;
                    drawNodeDropzone();

                }}>
                {knobs}
                <div className="flex align-items-center cell-text" draggable={false} style={{ paddingLeft: "5px", borderLeft: `5px solid ${item.status ? '#1e7e34' : '#f00'}`, height: "20px" }}>{moment(item.created_date).format("YYYY-MM-DD HH:mm")}</div>
            </div>
        );
    }


    const drawNodeDropzone = () => {
        d3.select(d3Container.current).insert("rect")
            .attr("class", "drop-zone-marker")
            .attr("x", currentNodeCoords.current.x - 40)
            .attr("y", currentNodeCoords.current.y - 40)
            .attr("width", 100)
            .attr("height", 100)
            .attr("fill", "transparent")
            .attr("stroke", inventory.current?.status === false ? "red" : "green")
            .on("drop", dragToNodeEnd)
            .on("dragover", dragOverNode)
            .on("dragleave", dragLeaveNode);
    }



    const sizeBodyTemplate = (item) => {
        return (<div className="flex align-items-center icon-cell backup">{item.display_size} <i className="pi pi-check verify-backup" onClick={() => {

            dispatch(LayoutActions.verifyBackup({ "name": item?.name, "type": item?.type, "backup_host": item?.host_name, "backup_path": item?.ex_params?.["backup_path"], "backup_date": item?.ex_params?.["modified_date"] }, onVerifyBackup))

        }} /> <img className="bar-icon" draggable={false} alt="backupBar" src={bar} onClick={() => {
            inventory.current = item;
            switch (item.type) {

                case "BACKUP":
                    setShowBackupDetails(true);
                    break;
                default:
                    break;

            }

        }} /></div>);
    }

    const lagBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} >{item.ex_params?.lag}{<img alt="sentinelBar" src={bar} draggable={false} className="bar-icon" onClick={() => {

            navigate(`/status/redis/${item?.id}`);

        }} />} </div>);
    }

    const portBodyTemplate = (item) => {
        return (<div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px" }}>{item.port}</div>);
    }

    const ipBodyTemplate = (item) => {
        return (<div className="flex align-items-center" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            leftContainer.current = d3Container.current.getBoundingClientRect();
            d3.select(d3Container.current).select(".trash").style("opacity", "0.5");
            e.dataTransfer.setData("dragType", nodeSelected?.cluster_role === clusterRoles.SENTINEL ? "sentinelBar" : "nodeBar");
            inventory.current = item;
            if (item?.cluster_role === clusterRoles.SENTINEL && nodeSelected?.cluster_role === clusterRoles.PRIMARY) drawNodeDropzone();
        }} >
            {knobs}
            <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid ${getColorBySeverity(item.severity)}` }}>{item.host}</div>
        </div>);
    }



    const onDropzoneLeave = (e) => {
        e.target.style.position = "";
        e.target.style.left = "";
        e.target.style.top = "";
        e.target.style.visibility = ""; d3.selectAll(".drop-zone-marker").remove();
        d3.select(d3Container.current).select(".trash").style("opacity", "0.0");
    }

    const drawClusterDropzone = () => {

        d3.select(d3Container.current).insert("rect")
            .attr("class", "drop-zone-marker")
            .attr("x", 20)
            .attr("y", layoutMid - 50)
            .attr("width", 100)
            .attr("height", 100)
            .attr("fill", "transparent")
            .attr("stroke", inventory.current?.status === false ? "red" : "green")
            .on("drop", dragToClusterEnd)
            .on("dragover", dragOverCluster)
            .on("dragleave", dragLeaveNode);

    }

    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onTableItemDrag = (e) => {
        e.preventDefault();
        e.target.style.position = "fixed";
        e.target.style.top = e.clientY + "px";
        e.target.style.left = e.clientX + "px";
        if (e.clientX < leftContainer.current.left || e.clientY < leftContainer.current.top || e.clientY > leftContainer.current.bottom) {

            e.target.style.visibility = "hidden";

        }
        else {

            e.target.style.visibility = "";

        }
    }

    const onDropNode = (data) => {
        dispatch(LayoutActions.dropRedisNode(data, setShowDropNode));
    }

    const onBackupNode = (data) => {
        dispatch(LayoutActions.backupRedisNode(data, setShowBackup));
    }

    const onAddRedisNode = (data) => dispatch(LayoutActions.addRedisNode(data, setShowAddNode));


    const onRestoreBackupNode = (data) => {
        dispatch(LayoutActions.restoreRedisNode(data, setShowRestoreBackupNode));

    }

    const onSentinelMonitorMaster = (data) => {

        dispatch(LayoutActions.sentinelMonitorMaster(data, setShowMonitorMaster));

    }

    const onSwitchover = (data) => {
        dispatch(LayoutActions.switchRedisNode(data, setShowSwitchover));
    }

    const onStepdown = (data) => {
        dispatch(LayoutActions.sentinelFailover(data, setShowStepdown));
    }

    return (
        <div className="layout-item">
            <div className="layout-item-wrapper">
                <div style={{ zIndex: 1000 }} className="left-container">
                    <div className="left-title flex">
                        <div className="ml-auto ml-auto">{`${props.node?.ex_params?.what} cluster: ${props.node?.name}`}</div>
                        <div className="ml-auto">

                            <Button
                                style={{ fontSize: "1rem" }}
                                className="p-button-sm p-button-text"
                                label="status"
                                onClick={() => navigate('/status', { state: { cluster_id: props.node?.id } })}
                            />

                            <Button
                                style={{ fontSize: "1rem" }}
                                className="p-button-sm p-button-text"
                                label="cluster options"
                                onClick={(e) => clusterOptions.current?.show(e)}
                            />

                            <OverlayPanel ref={clusterOptions} onMouseLeave={(e) => clusterOptions.current?.hide(e)} closeOnEscape>

                                <div className="flex flex-column">

                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        label="drop"
                                        onClick={() => setShowRemoveCluster(true)}
                                    />


                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        disabled={!props.node?.slaves?.length || props.node?.slaves?.length < 1 || props.node?.slaves?.filter((s) => s?.ind_started === true && s?.severity === 'green')?.length < 1}
                                        id={`switchover-${layoutItemId.current}`}
                                        label="switchover"
                                        onClick={() => setShowSwitchover(true)}
                                    />

                                    <Button
                                        style={{ fontSize: "1rem" }}
                                        className="p-button-sm p-button-text"
                                        disabled={!props.node?.masters?.length || props.node?.masters?.length < 1 || props.node?.masters?.filter((m) => m?.ex_params?.monitored_by?.length > 0)?.length < 1}
                                        id={`stepdown-${layoutItemId.current}`}
                                        label="stepdown"
                                        onClick={() => setShowStepdown(true)}
                                    />
                                </div>

                            </OverlayPanel>

                        </div>

                    </div>


                    <div className="left-content">
                        <svg
                            className="d3-component"
                            style={{ width: "800px", maxHeight: `${rightContainer.current?.offsetHeight > layoutCurrHeight ? rightContainer.current.offsetHeight + 5 : layoutCurrHeight + 5}px`, minHeight: `${layoutCurrHeight + 5}px` }}
                            ref={d3Container}
                        />
                    </div>
                </div>
                <div className="right-container" style={{ minHeight: `${layoutCurrHeight + 5}px` }}
                    ref={rightContainer}>
                    <TabView style={{ width: "500px" }} activeIndex={activeIndex} onTabChange={(e) => {
                        setActiveIndex(e.index);
                        tabRef.current = e.index
                    }}>
                        <TabPanel header="Masters">
                            <div className={`redis-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "masterCreate");
                                    drawClusterDropzone();
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on Cluster to create Master</div>
                                </div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />   List of Cluster {props.node.name} masters</div>
                                <DataTable
                                    className="severity-table"
                                    selectionMode="single"
                                    dataKey="id"
                                    selection={masterRow}
                                    onSelectionChange={(e) => setMasterRow(e.value)}
                                    emptyMessage="no masters found"
                                    scrollable
                                    scrollHeight="flex"
                                    paginator={attachedMasters?.length > 0}
                                    alwaysShowPaginator={false}
                                    rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                    value={attachedMasters}>
                                    <Column style={{ height: "21px" }} field="host" sortFunction={(e) => onSort(e, attachedMasters)} body={ipBodyTemplate} header="IP" sortable></Column>
                                    <Column style={{ height: "21px" }} field="port" sortFunction={(e) => onSort(e, attachedMasters)} body={portBodyTemplate} header="Port" sortable></Column>
                                    <Column style={{ height: "21px" }} field="ex_params.lag" sortFunction={(e) => onSort(e, attachedMasters)} body={lagBodyTemplate} header="Lag" sortable></Column>
                                </DataTable>

                            </div>
                        </TabPanel>
                        <TabPanel header="Replicas">
                            <div className={`redis-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "replicaCreate");
                                    drawNodeDropzone();
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on selected Master to create Replica</div>
                                </div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />  {nodeSelected?.cluster_role === clusterRoles.PRIMARY ? `List of Master ${nodeSelected?.name} replicas` : "Select Master to view its replicas"}</div>
                                {nodeSelected?.cluster_role === clusterRoles.PRIMARY &&
                                    <DataTable
                                        className="severity-table"
                                        selectionMode="single"
                                        emptyMessage="no replicas found"
                                        dataKey="id"
                                        selection={slaveRow}
                                        onSelectionChange={(e) => setSlaveRow(e.value)}
                                        scrollable
                                        scrollHeight="flex"
                                        paginator={attachedSlaves?.length > 0}
                                        alwaysShowPaginator={false}
                                        rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                        value={attachedSlaves}>
                                        <Column style={{ height: "21px" }} field="host" sortFunction={(e) => onSort(e, attachedSlaves)} body={ipBodyTemplate} header="IP" sortable></Column>
                                        <Column style={{ height: "21px" }} field="port" sortFunction={(e) => onSort(e, attachedSlaves)} body={portBodyTemplate} header="Port" sortable></Column>
                                        <Column style={{ height: "21px" }} field="ex_params.lag" sortFunction={(e) => onSort(e, attachedSlaves)} body={lagBodyTemplate} header="Lag" sortable></Column>
                                    </DataTable>}
                            </div>
                        </TabPanel>
                        <TabPanel header="Sentinels">
                            <div className={`redis-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div className="flex align-items-center text-xs" draggable onDrag={onTableItemDrag} onDragEnd={onDropzoneLeave} onDragStart={(e) => {
                                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                                    leftContainer.current = d3Container.current.getBoundingClientRect();
                                    e.dataTransfer.setData("dragType", "sentinelCreate");
                                    drawClusterDropzone();
                                }} >
                                    {knobs}
                                    <div className="flex align-items-center cell-text" draggable={false} style={{ height: "20px", paddingLeft: "5px", borderLeft: `5px solid white` }}>Drag me on Cluster to create Sentinel</div>
                                </div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />  Drag from list to selected master, to monitor it by sentinel</div>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" />  Drag master to here, to change or remove the sentinel monitoring</div>
                                <DataTable
                                    className="severity-table"
                                    selectionMode="single"
                                    dataKey="id"
                                    selection={sentinelRow}
                                    onSelectionChange={(e) => setSentinelRow(e.value)}
                                    scrollable
                                    scrollHeight="flex"
                                    emptyMessage="no sentinels found"
                                    paginator={clusterSentinels?.length > 7}
                                    alwaysShowPaginator={false}
                                    rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}
                                    value={clusterSentinels}>
                                    <Column style={{ height: "21px" }} field="host" body={ipBodyTemplate} header="IP" sortable></Column>
                                    <Column style={{ height: "21px" }} field="port" sortFunction={(e) => onSort(e, attachedMasters)} body={portBodyTemplate} header="Port" sortable></Column>
                                    <Column style={{ height: "21px" }} field="ex_params.lag" sortFunction={(e) => onSort(e, attachedMasters)} body={lagBodyTemplate} header="Lag" sortable></Column>
                                </DataTable>

                            </div>

                        </TabPanel>
                        <TabPanel header="Backups">
                            <div className={`redis-tab-content-${layoutItemId.current} layout-tab-content`}>
                                <div draggable={false} className="info-section text-xs"><i className="pi pi-circle-fill" /> {nodeSelected || clusterSelected ? `Drag backup to primary to restore:` : "Select node to view backups:"} </div>
                                {(nodeSelected || clusterSelected) && <div draggable={false} className="info-section text-xs grid"><i className="pi pi-circle-fill" /> <div className="col-fixed" style={{ width: "300px" }}>{`Backups for this cluster will be stored for ${clusterSelected ? props.node?.size_scheme?.backup : nodeSelected?.size_scheme?.backup} days`}</div> <Tag rounded severity="success" icon="pi pi-pencil" className="right-container-button" onClick={() => navigate("/config/", { state: { targetTab: 1, targetType: types.REDIS } })}>change</Tag> </div>}
                                {nodeSelected || clusterSelected ?
                                    getAutomaticBackup()
                                    : null}
                                <Panel ref={thisBackups} className="collapsible-panel mt-4" header={<div  onClick={() => thisBackups?.current?.toggle()} className="cursor-pointer">Backups of this cluster</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={backupRow}
                                            onSelectionChange={(e) => setBackupRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={redis_backups?.filter((b) => b.cluster === props.node && b.ex_params?.node_version === props.node?.ex_params?.version)}
                                            emptyMessage="no backups found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}>
                                            <Column style={{ height: '21px' }} field="created_date" body={backupDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                                <Panel ref={otherBackups} collapsed className="collapsible-panel" header={<div  onClick={() => otherBackups?.current?.toggle()} className="cursor-pointer">Backups of other clusters</div>} toggleable expandIcon="pi pi-chevron-right" collapseIcon="pi pi-chevron-down">
                                    <ScrollPanel className="layout-scroll">
                                        <DataTable
                                            className={"backup-expandable-table-parent severity-table"}
                                            selectionMode="single"
                                            dataKey="id"
                                            selection={backupRow}
                                            onSelectionChange={(e) => setBackupRow(e.value)}
                                            scrollable
                                            scrollHeight="flex"
                                            value={redis_backups?.filter((b) => b.cluster !== props.node && b.ex_params?.node_version === props.node?.ex_params?.version)}
                                            emptyMessage="no backups found"
                                            paginator
                                            alwaysShowPaginator={false}
                                            rows={layoutCurrHeight === config.layoutMinHeight ? 7 : 50}>
                                            <Column style={{ height: '21px' }} field="created_date" body={backupDateBodyTemplate} header="Created date" sortable></Column>
                                            <Column style={{ height: '21px' }} field="display_size" body={sizeBodyTemplate} header="Backup size" sortable></Column>
                                        </DataTable>

                                    </ScrollPanel>
                                </Panel>
                            </div>

                        </TabPanel>
                    </TabView>
                </div>
            </div>
            <AddRedisNodeDialog
                visible={showAddNode}
                node={props.node}
                child_role={childRole}
                master={(childRole === clusterRoles.SENTINEL || childRole === clusterRoles.PRIMARY) ? null : props.selected.current}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    setShowAddNode(false)

                }}
                onOkClick={onAddRedisNode}
            />

            {showWarning ? <WarningDialog
                message={warningMessage}
                visible={showWarning}
                onHide={() => {

                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowWarning(false)
                }
                } />
                : null}

            {showInfoDialog ? <InfoDialog
                message={infoMessage}
                visible={showInfoDialog}
                onHide={() => {

                    setShowInfoDialog(false);
                }
                } />
                : null}

            {showDropNode ? <DropRedisNodeDialog
                node={inventory.current}
                slaves_count={props.node?.slaves?.length}
                visible={showDropNode}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowDropNode(false)
                }}
                onOkClick={onDropNode}
            />
                : null}

            {showBackup ? <RedisNodeBackupDialog
                node={props.selected.current || props.node}
                updateSchedule={updateSchedule.current}
                pageIndex={pageIndex}
                visible={showBackup}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowBackup(false)
                }}
                onOkClick={onBackupNode}
            />
                : null}

            {
                showRestoreBackupNode ? <RestoreRedisNodeLayoutDialog
                    visible={showRestoreBackupNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestoreBackupNode(false)
                    }
                    }
                    onOkClick={onRestoreBackupNode}
                    node={props.selected.current}
                    backup={inventory.current}
                /> : null
            }

            {showMonitorMaster ? <SentinelMonitorMasterDialog
                visible={showMonitorMaster}
                onHide={() => {
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowMonitorMaster(false);
                }
                }
                onOkClick={onSentinelMonitorMaster}
                update={updateMonitor}
                masters={[props.selected?.current]}
                sentinels={updateMonitor ? props.node?.sentinels?.filter((s) => props.selected?.current?.ex_params?.monitored_by?.includes(s?.id?.toString())) : [inventory.current]}
                max_quorum={props.node.sentinels?.length}
                max_syncs={props.node.slaves?.filter((s) => s?.ex_params?.slave_of === props.selected?.current?.id)?.length}
            /> : null
            }

            {showBackupDetails ? <BackupDetailsDialog
                backup={inventory.current}
                visible={showBackupDetails}
                onHide={() => setShowBackupDetails(false)} />
                : null
            }

            {showDeleteInventory ? <DeleteInventoryDialog
                visible={showDeleteInventory}
                onHide={() => {
                    setShowDeleteInventory(false);
                    dispatch(LayoutActions.setErrorMessage(""));
                }}
                onOkClick={onDeleteInventory}
                item={inventory.current} />
                : null
            }

            {showRemoveCluster ? <RemoveRedisClusterDialog
                visible={showRemoveCluster}
                onHide={() => {
                    setShowRemoveCluster(false);
                    dispatch(LayoutActions.setErrorMessage(""));
                }}
                onOkClick={onRemoveCluster}
                node={props.node} />
                : null
            }

            {showSwitchover ? <SwitchRedisNodeDialog
                nodes={props.node?.slaves?.filter((s) => s?.ind_started === true && s?.severity === 'green')}
                visible={showSwitchover}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowSwitchover(false)
                }}
                onOkClick={onSwitchover}
            />
                : null}

            {showStepdown ? <StepdownRedisNodeDialog
                masters={props.node?.masters?.filter((m) => m?.ex_params?.monitored_by?.length > 0)}
                visible={showStepdown}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowStepdown(false)
                }}
                onOkClick={onStepdown}
            />
                : null}


            {showChooseServiceDialog ? <ChooseServiceDialog
                visible={showChooseServiceDialog}
                services={props.services}
                onHide={() => setShowChooseServiceDialog(false)}
                onNext={(targetService) => setShowAddNode(targetService ? targetService.name : true)}
            /> : null}

        </div>

    );


}


export default RedisLayoutItem;