import { useDispatch, useSelector } from 'react-redux';
import { useLocation, useNavigate } from 'react-router-dom';
import { useEffect, useRef, useState } from 'react';
import useInterval from '../../../util/useInterval';
import { Menubar } from 'primereact/menubar';
import { Tag } from 'primereact/tag';
import { OverlayPanel } from 'primereact/overlaypanel';
import { ListBox } from 'primereact/listbox';
import { Badge } from 'primereact/badge';
import { AppActions } from "../../../store/app/reducer";
import { TaskNotificationActions } from '../../../store/taskNotifications/reducer';
import logo from "../../../assets/images/logo.gif";
import { Tooltip } from 'primereact/tooltip';
import { types } from '../../../config';



const Menu = (props) => {
    const location = useLocation();
    const navigate = useNavigate();
    const dispatch = useDispatch();

    const { user } = useSelector((state) => state.app);
    const { notifications } = useSelector((state) => state.taskNotifications);

    const notificationsPanel = useRef(null);
    const [notificationsPanelSelected, setNotificationsPanelSelected] = useState(false);
    const [locationState, setLocationState] = useState({});
    const [filteredNotifications, setFilteredNotifications] = useState([])

    useEffect(() => {

        if(notifications?.length > 0) setFilteredNotifications(notifications);

    }, [notifications])

    useEffect(() => {
        if (props?.dbType) {

            switch (props.dbType) {
                case types.MYSQL:
                    setLocationState({ state: { targetTab: 1, targetType: types.MYSQL } });
                    break;
                case types.MARIADB:
                    setLocationState({ state: { targetTab: 1, targetType: types.MARIADB } });
                    break;
                case types.ELASTIC:
                    setLocationState({ state: { targetTab: 1, targetType: types.ELASTIC } });
                    break;
                case types.POSTGRES:
                    setLocationState({ state: { targetTab: 1, targetType: types.POSTGRES } });
                    break;
                case types.MONGO:
                    setLocationState({ state: { targetTab: 1, targetType: types.MONGO } });
                    break;
                default:
                    break;


            }

        }

    }, [props.dbType]);

    useInterval(() => {
        if (notifications?.length > 0) dispatch(TaskNotificationActions.isRunningCompleted(notifications));
    }, 15000)

    const getSelectedClassName = (baseClass, locations) => {
        if (baseClass) {
            const locationParts = location.pathname.split("/").map((part) => "/" + part);
            for (let part of locationParts)
                if (locations.indexOf(part) >= 0)
                    return baseClass + " selected";

            return baseClass;
        }
        else {
            const locationParts = location.pathname.split("/").map((part) => "/" + part);
            for (let part of locationParts)
                if (locations.indexOf(part) >= 0)
                    return "selected";

            return "";
        }
    }

    const notificationsItemTemplate = (item) => {

        return (

            <div style={{ height: "20px" }} className="flex align-items-center justify-content-end">

                <div className="flex text-xs flex-grow-1">{`${item.displayment} in progress`}</div>
                <div>
                    <i className="pi pi-stop notification" />
                    <Tooltip target=".pi.pi-stop.notification" content="Kill" />
                </div>


            </div>
        )

    }

    const items = [
        {
            label: 'Status',
            icon: 'pi pi-fw pi-chart-bar',
            className: getSelectedClassName("status", ["/status"]),
            command: () => { navigate("/status") }
        },
        {
            label: 'Layout',
            icon: 'pi pi-fw pi-sitemap',
            className: getSelectedClassName("layout", ["/layout"]),
            command: () => { navigate("/layout") }
        },
        {
            label: 'Dashboard',
            icon: 'pi pi-fw pi-chart-line',
            className: getSelectedClassName("dashboard", ["/dashboard"]),
            command: () => { navigate("/dashboard") }
        },
        {
            label: 'Activity Logs',
            icon: 'pi pi-fw pi-align-left',
            className: getSelectedClassName("log", ["/log"]),
            command: () => { navigate("/log") }
        },


    ];


    return (
        <div className="menu-component">
            <div className="logo-container">
                <div className="logo-wrapper">
                    <img draggable={false}  src={logo} alt="" />
                </div>
            </div>
            <div className="menu-container">
                <Menubar
                    model={items}
                    end={
                        <div className="flex align-items-center nav-top-right">
                            <i className={`i-nav-end pi pi-bell p-overlay-badge mr-4${notificationsPanelSelected ? " selected" : ""}`} aria-haspopup aria-controls="overlay_panel" onClick={(e) => notificationsPanel.current.toggle(e)}>
                                <Badge className="notification-badge" severity="info" value={filteredNotifications?.length > 0 ? filteredNotifications?.length : 0} />
                            </i>

                            <i className={getSelectedClassName("i-nav-end pi pi-cog mr-2", ["/config"])} onClick={() => {
                                navigate("/config", locationState)
                            }} />
                            <Tag className="user-greeting cursor-pointer" onClick={() => dispatch(AppActions.logout())} value={`Welcome ${user?.data?.username ? user?.data?.username : "admin"} `}>
                            <i className="ml-2 i-nav-end pi pi-power-off"/>
                            </Tag>
                            <OverlayPanel onShow={() => setNotificationsPanelSelected(true)} onHide={() => setNotificationsPanelSelected(false)} style={{ width: "30vw" }} ref={notificationsPanel} id="overlay_panel">
                                <p >{filteredNotifications?.length < 1 ? "No new notifications" : ""}</p>
                                {filteredNotifications?.length > 0 && <ListBox className="notification-box" options={filteredNotifications} itemTemplate={notificationsItemTemplate} />}

                            </OverlayPanel>
                        </div>
                    }
                />
            </div>
        </div >
    );
}


export default Menu;