import { useEffect, useState, Fragment } from "react";
import moment from "moment";
import { Chart } from "primereact/chart";
import { timePeriods } from "../../config";
import 'chartjs-adapter-moment';

export default function PerformanceCharts(props) {

    useEffect(() => {

        const timeFilter = (data, timePeriod) => {

            let timeUnit;
            let stepSize;

            switch (timePeriod) {

                case timePeriods.TEN_MIN:
                    timeUnit = "minute";
                    stepSize = 2;
                    break;
                case timePeriods.HOUR:
                    timeUnit = "minute";
                    stepSize = 10;
                    break;
                case timePeriods.SIX_HOURS:
                    timeUnit = "hour";
                    stepSize = 1.5;
                    break;
                case timePeriods.WEEK:
                    timeUnit = "day";
                    stepSize = 1;
                    break;
                case timePeriods.DAY:
                    timeUnit = "hour";
                    stepSize = 6;
                    break;
                default:
                    timeUnit = "minute";
                    stepSize = 2;
                    break;


            }

            const monitors = data?.filter((n) => {
                if (timePeriod === timePeriods.TEN_MIN) {
                    if (moment().diff(moment(n.created_date), "minutes") < 10)
                        return true;
                }
                else if (timePeriod === timePeriods.HOUR) {
                    if (moment().diff(moment(n.created_date), "hours") < 1)
                        return true;
                }
                else if (timePeriod === timePeriods.SIX_HOURS) {
                    if (moment().diff(moment(n.created_date), "hours") < 6)
                        return true;
                }
                else if (timePeriod === timePeriods.DAY) {
                    if (moment().diff(moment(n.created_date), "days") < 1)
                        return true;
                }
                else if (timePeriod === timePeriods.WEEK) {
                    if (moment().diff(moment(n.created_date), "days") < 7)
                        return true;
                }
                return false;
            });

            let memStepSize = 0.2;

            if (monitors?.length > 0) {

                let memMax = monitors.reduce((a, b) => a?.metric_value > b?.metric_value ? a : b)?.metric_value;
                let memMin = monitors.reduce((a, b) => a?.metric_value < b?.metric_value ? a : b)?.metric_value;
                memStepSize = (memMin + memMax) / 5;




            }

            setResponseTimeOptions({
                responsive: true,
                maintainAspectRatio: false,
                parsing: {
                    xAxisKey: 'created_date',
                    yAxisKey: 'metric_value'
                },
                plugins: {
                    legend: {
                        display: true,
                        labels: {
                            color: "darkgray",
                            boxWidth: 0,

                        }
                    }
                },
                scales: {
                    x: {
                        type: 'time',
                        bounds: 'ticks',
                        time: {
                            unit: timeUnit,
                            stepSize: stepSize,
                            displayFormats: {
                                hour: "HH:mm",
                                minute: "HH:mm"
                            }
                        },
                        ticks: {
                            color: 'white',
                            source: "labels",
                        }
                    },
                    y: {
                        ticks: {
                            color: 'white',
                            maxTicksLimit: 6
                        }
                    }
                }
            });

            setMemOptions({
                responsive: true,
                maintainAspectRatio: false,
                parsing: {
                    xAxisKey: 'created_date',
                    yAxisKey: 'metric_value'
                },
                plugins: {
                    legend: {
                        display: true,
                        labels: {
                            color: 'darkgray',
                            boxWidth: 0,
                        }
                    }
                },
                scales: {
                    x: {
                        type: 'time',
                        bounds: 'ticks',
                        time: {
                            unit: timeUnit,
                            stepSize: stepSize,
                            displayFormats: {
                                hour: "HH:mm",
                                minute: "HH:mm"
                            }
                        },
                        ticks: {
                            color: 'white',
                            source: "labels",
                        }
                    },
                    y: {
                        min: 0,
                        ticks: {
                            color: 'white',
                            maxTicksLimit: 6,
                            stepSize: memStepSize
                        }
                    }
                }
            })


            setCpuOptions({
                responsive: true,
                maintainAspectRatio: false,
                parsing: {
                    xAxisKey: 'created_date',
                    yAxisKey: 'metric_value'
                },
                plugins: {
                    legend: {
                        display: true,
                        labels: {
                            color: 'darkgray',
                            boxWidth: 0,
                        }
                    }
                },
                scales: {
                    x: {
                        type: 'time',
                        bounds: 'ticks',
                        time: {
                            unit: timeUnit,
                            stepSize: stepSize,
                            displayFormats: {
                                hour: "HH:mm",
                                minute: "HH:mm"
                            }
                        },
                        ticks: {
                            color: 'white',
                            source: "labels",
                        }
                    },
                    y: {
                        min: 0,
                        max: 100,
                        ticks: {
                            color: 'white',
                            stepSize: 20,
                            maxTicksLimit: 6

                        }
                    }
                }
            })



            return monitors;


        }

        let labels;
        let now = moment();
        switch (props.timePeriodType) {
            case timePeriods.TEN_MIN:
                labels = [now.clone().subtract(10, "minutes"), now.clone().subtract(8, "minutes"), now.clone().subtract(6, "minutes"), now.clone().subtract(4, "minutes"), now.clone().subtract(2, "minutes"), now];
                break;
            case timePeriods.HOUR:
                labels = [now.clone().subtract(60, "minutes"), now.clone().subtract(50, "minutes"), now.clone().subtract(40, "minutes"), now.clone().subtract(30, "minutes"), now.clone().subtract(20, "minutes"), now.clone().subtract(10, "minutes"), now];
                break;
            case timePeriods.SIX_HOURS:
                labels = [now.clone().subtract(6, "hours"), now.clone().subtract(4.5, "hours"), now.clone().subtract(3, "hours"), now.clone().subtract(1.5, "hours"), now];
                break;
            case timePeriods.DAY:
                labels = [now.clone().subtract(24, "hours"), now.clone().subtract(18, "hours"), now.clone().subtract(12, "hours"), now.clone().subtract(6, "hours"), now];
                break;
            case timePeriods.WEEK:
                labels = [now.clone().subtract(7, "days"), now.clone().subtract(6, "days"), now.clone().subtract(5, "days"), now.clone().subtract(4, "days"), now.clone().subtract(3, "days"), now.clone().subtract(2, "days"), now.clone().subtract(1, "days"), now];
                break;
            default:
                labels = [now.clone().subtract(10, "minutes"), now.clone().subtract(8, "minutes"), now.clone().subtract(6, "minutes"), now.clone().subtract(4, "minutes"), now.clone().subtract(2, "minutes"), now];
                break;

        }
        setResponseTime({
            labels: labels,
            datasets: [
                {
                    label: 'Response Time(ms)',
                    data: [],
                    fill: false,
                    borderColor: 'lightgreen',
                    backgroundColor: '#325b76',
                    pointStyle: "line",
                    tension: .1
                },
            ]
        })

        let cpuData = timeFilter(props.cpuUsages, props.timePeriodType);
        let memData = timeFilter(props.memUsages, props.timePeriodType);

        setCpuUsage({
            labels: labels,
            datasets: [
                {
                    label: 'CPU Usage(%)',
                    data: cpuData,
                    fill: false,
                    borderColor: 'lightgreen',
                    backgroundColor: '#325b76',
                    pointStyle: 'line',
                    tension: .1
                },
            ]
        });

        setMemUsage({
            labels: labels,
            datasets: [
                {
                    label: 'Memory Usage(GB)',
                    data: memData,
                    fill: false,
                    borderColor: 'lightgreen',
                    backgroundColor: '#325b76',
                    pointStyle: 'line',
                    color: "white",
                    tension: .1
                },
            ]
        });


    }, [props.timePeriodType, props.cpuUsages, props.memUsages])

    const [responseTime, setResponseTime] = useState(null);

    const [cpuUsage, setCpuUsage] = useState(null);

    const [memUsage, setMemUsage] = useState(null);


    const [responseTimeOptions, setResponseTimeOptions] = useState(null)

    const [cpuOptions, setCpuOptions] = useState(null);

    const [memOptions, setMemOptions] = useState(null);

    return <Fragment>

        <Chart className="performance-chart" type="line" data={responseTime} options={responseTimeOptions} />

        <Chart className="performance-chart" type="line" data={cpuUsage} options={cpuOptions} />


        <Chart className="performance-chart" type="line" data={memUsage} options={memOptions} />

    </Fragment>

}