import { Card } from "primereact/card";
import { useDispatch } from "react-redux";
import { StatusActions } from "../../store/status/reducer";
import { useState } from "react";
import MoveLicenseDialog from "../../dialogs/licensing/moveLicense"
import { Tag } from "primereact/tag";

export default function StatsCard({ details }) {

    const dispatch = useDispatch();
    const [showMoveLicense, setShowMoveLicense] = useState(false);
    return (<Card className='stats-card'>
        <div className='grid'>

            <div className='col text-center'>
                <i className='pi pi-desktop text-3xl text-green-500 align-self-start mr-2 mb-2' />
                <div className='text-sm'> {details?.cpu_percent ? `${details?.cpu_percent}%` : <i className="pi pi-spin pi-spinner" />} </div>
                <div className='text-base'> vCPU </div>
            </div>
            <div className='col text-center'>
                <i className='pi pi-microchip text-3xl text-green-500 align-self-start mr-2 mb-2' />
                <div className='text-sm'>  {details?.display_ram ? details?.display_ram : <i className="pi pi-spin pi-spinner" />} </div>
                <div className='text-base'> vRAM </div>
            </div>

            <div className='col text-center'>
                <i className='pi pi-database text-3xl text-green-500 align-self-start mr-2 mb-2' />
                <div className='text-sm'> {details?.display_size ? details?.display_size : <i className="pi pi-spin pi-spinner" />} </div>
                <div className='text-base'> vDISK </div>
            </div>

            <div className='col text-center'>
                <i className={`pi pi-arrow-${details?.node?.ind_started ? 'up' : 'down'} text-3xl text-${details?.node?.ind_started ? 'green' : 'red'}-500 align-self-start mr-2 mb-2`} />
                <div className='text-sm'>  {details?.node?.ex_params?.avail_change ? details?.node?.ex_params?.avail_change : <i className="pi pi-spin pi-spinner" />} </div>
                <div className='text-base'> {details?.node?.ind_started ? 'UP' : 'DOWN'} SINCE </div>
            </div>

            <div className='col text-center'>
                <i className={`pi pi-key text-3xl text-${details?.node?.ind_license ? 'green' : 'red'}-500 align-self-start mr-2 mb-2`} />
                <Tag rounded style={{height: '17px', width:'78px', display: 'block'}} className='details-button p-button-sm p-0 ml-auto mr-auto cursor-pointer' onClick={() => setShowMoveLicense(true)} value={details?.node?.ind_license ? <span><i className="p-tag-icon pi pi-plus"/>VALID</span> : details?.node?.ind_license === false ? details?.node?.has_trial ? <span> <i className="p-tag-icon pi pi-plus"/>TRIAL</span> : <span> <i className="p-tag-icon pi pi-plus"/>EXPIRED</span>: <i className="pi pi-spin pi-spinner" />} /> 
                <div className='text-base'> LICENSE </div>
            </div>
        </div>
        <MoveLicenseDialog node={details?.node} visible={showMoveLicense} onHide={() => {
            dispatch(StatusActions.setLicensedClusters(undefined))
            setShowMoveLicense(false)
        }} onOkClick={(data) => dispatch(StatusActions.moveLicense(data, setShowMoveLicense))} />

    </Card>)
}
