import { useState, useEffect, useRef } from 'react';
import { useDispatch } from 'react-redux';
import { ClusterUpdateActions } from '../../../store/clusterUpdate/reducer';
import { Slider } from 'primereact/slider';
import { types } from '../../../config';
import { InputNumber } from "primereact/inputnumber";
import { Button } from 'primereact/button';
import { Inplace, InplaceDisplay, InplaceContent } from 'primereact/inplace';


export default function ClusterConfigTable({ data, db_type, is_superuser }) {
  const dispatch = useDispatch();
  const [currentData, setCurrentData] = useState(data);
  const [editedIndex, setEditedIndex] = useState(-1);
  const dataRef = useRef(data);
  useEffect(() => setCurrentData(data), [data]);
  const updateCell = (name, value) => {
    dataRef.current = { ...currentData, [name]: value };
    setCurrentData(dataRef.current);
  };
  return (
    <div className='flex flex-column'>

      <div className="p-datatable p-component sizing-config">
        <div className="p-datatable-wrapper">

          <table className="p-datatable-table">
            <thead className="p-datatable-thead">
              <tr>
                <th></th>
                <th ><div className="p-column-header-content">
                  <span className="p-column-title">
                    Size (Disk)
                  </span>
                </div></th>
                <th ><div className="p-column-header-content">
                  <span className="p-column-title">
                    CPUs
                  </span>
                </div></th>
                <th ><div className="p-column-header-content">
                  <span className="p-column-title">
                    Memory
                  </span>
                </div></th>

                {

                  db_type === types.ORACLE_CDB &&

                  <th ><div className="p-column-header-content">
                    <span className="p-column-title">
                      Shared Temp
                    </span>
                  </div></th>

                }

                <th >
                  <div className="p-column-header-content">
                    <span className="p-column-title">
                      Keep Backups
                    </span>

                  </div>
                </th>
                <th >
                  <div className="p-column-header-content">
                    <span className="p-column-title">

                      Keep Exports

                    </span>
                  </div>

                </th>


                {

                  db_type === types.ORACLE_CDB &&

                  <th >

                    <div className="p-column-header-content">

                      <span className="p-column-title">
                        Keep Restore Points
                      </span>
                    </div>
                  </th>

                }

              </tr>
            </thead>
            <tbody>
              <tr>
                <td className="col-fixed" style={{width: "10vw"}}>
                  <div className="col-fixed" style={{width: "138px"}}>
                    {`Small ${db_type === types.ORACLE_CDB ? "PDBs" : ""}`}

                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 0} onOpen={() => setEditedIndex(0)}>
                      <InplaceDisplay>

                        <div className="title"> {currentData?.small_disk} GB</div>

                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title"> <InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.small_disk} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 1000) {
                            updateCell("small_disk", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={1000}  /> GB</div>
                      </InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.small_disk}
                      min={1}
                      max={1000}
                      step={1}
                      onChange={(e) => { updateCell("small_disk", e.value) }}

                      disabled={!is_superuser}
                    />

                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 1} onOpen={() => setEditedIndex(1)}>
                      <InplaceDisplay> <div className="title">{currentData?.small_cpu} Cores</div></InplaceDisplay>
                      <InplaceContent>
                        <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.small_cpu} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 16) {
                            updateCell("small_cpu", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={16}  /> Cores</div>
                      </InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.small_cpu}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("small_cpu", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >

                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 2} onOpen={() => setEditedIndex(2)}>
                      <InplaceDisplay>
                        <div className="title">{currentData?.small_memory} GB</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.small_memory} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 16) {
                            updateCell("small_memory", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={16}  /> GB</div>
                      </InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.small_memory}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("small_memory", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 3} onOpen={() => setEditedIndex(3)}>
                        <InplaceDisplay> <div className="title"> {currentData?.small_shared_temp} GB</div></InplaceDisplay>
                        <InplaceContent>
                          <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.small_shared_temp} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                              updateCell("small_shared_temp", e.value);

                            }

                          }} disabled={!is_superuser} min={1} max={100}  /> GB</div>
                        </InplaceContent>
                      </Inplace>
                      <Slider
                        value={currentData?.small_shared_temp}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("small_shared_temp", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }

                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 4} onOpen={() => setEditedIndex(4)}>
                      <InplaceDisplay> <div className="title"> {currentData?.small_backup_keep_days} Days</div></InplaceDisplay>
                      <InplaceContent> <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.small_backup_keep_days}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                          updateCell("small_backup_keep_days", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={100}  /> Days</div></InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.small_backup_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("small_backup_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 5} onOpen={() => setEditedIndex(5)}>
                      <InplaceDisplay>
                        <div className="title">{currentData?.small_export_keep_days} Days</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.small_export_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                            updateCell("small_export_keep_days", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={100}  /> Days</div>
                      </InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.small_export_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("small_export_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 6} onOpen={() => setEditedIndex(6)}>
                        <InplaceDisplay> <div className="title">{currentData?.small_restore_points_keep_days} Days</div></InplaceDisplay>
                        <InplaceContent> <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.small_restore_points_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                            updateCell("small_restore_points_keep_days", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={100}  /> Days</div></InplaceContent>
                      </Inplace>
                      <Slider
                        value={currentData?.small_restore_points_keep_days}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("small_restore_points_keep_days", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }
              </tr>
              <tr className="p-row-odd">
                <td className="col-fixed" style={{width: "10vw"}}>
                  <div className="col-fixed" style={{width: "138px"}}>
                    {`Medium ${db_type === types.ORACLE_CDB ? "PDBs" : ""}`}
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 7} onOpen={() => setEditedIndex(7)}>
                      <InplaceDisplay>
                        <div className="title"> {currentData?.medium_disk} GB</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.medium_disk} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 1000) {
                            updateCell("medium_disk", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={1000}  /> GB</div>
                      </InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.medium_disk}
                      min={1}
                      max={1000}
                      step={1}
                      onChange={(e) => updateCell("medium_disk", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>

                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 8} onOpen={() => setEditedIndex(8)}>
                      <InplaceDisplay>
                        <div className="title">{currentData?.medium_cpu} Cores</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.medium_cpu} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 16) {
                            updateCell("medium_cpu", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={16}  /> Cores</div>
                      </InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.medium_cpu}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("medium_cpu", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 9} onOpen={() => setEditedIndex(9)}>
                      <InplaceDisplay> <div className="title"> {currentData?.medium_memory} GB</div></InplaceDisplay>
                      <InplaceContent> <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.medium_memory}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 16) {
                          updateCell("medium_memory", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={16}  /> GB</div></InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.medium_memory}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("medium_memory", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 10} onOpen={() => setEditedIndex(10)}>
                        <InplaceDisplay>  <div className="title">{currentData?.medium_shared_temp} GB</div></InplaceDisplay>
                        <InplaceContent>
                          <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.medium_shared_temp} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                              updateCell("medium_shared_temp", e.value);

                            }

                          }} disabled={!is_superuser} min={1} max={100}  /> GB</div>
                        </InplaceContent>
                      </Inplace>
                      <Slider
                        value={currentData?.medium_shared_temp}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("medium_shared_temp", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 11} onOpen={() => setEditedIndex(11)}>
                      <InplaceDisplay>
                        <div className="title"> {currentData?.medium_backup_keep_days} Days</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.medium_backup_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                            updateCell("medium_backup_keep_days", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={100}  /> Days</div>
                      </InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.medium_backup_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("medium_backup_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >

                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 12} onOpen={() => setEditedIndex(12)}>
                      <InplaceDisplay>
                        <div className="title">{currentData?.medium_export_keep_days} Days</div>
                      </InplaceDisplay>
                      <InplaceContent> <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.medium_export_keep_days}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                          updateCell("medium_export_keep_days", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={100}  />} Days</div></InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.medium_export_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("medium_export_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 13} onOpen={() => setEditedIndex(13)}>
                        <InplaceDisplay>

                          <div className="title"> {currentData?.medium_restore_points_keep_days} Days</div>

                        </InplaceDisplay>
                        <InplaceContent>
                          <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.medium_restore_points_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                              updateCell("medium_restore_points_keep_days", e.value);

                            }

                          }} disabled={!is_superuser} min={1} max={100}  />} Days</div>
                        </InplaceContent>
                      </Inplace>
                      <Slider
                        value={currentData?.medium_restore_points_keep_days}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("medium_restore_points_keep_days", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }
              </tr>
              <tr>
                <td className="col-fixed" style={{width: "10vw"}}>
                  <div className="col-fixed" style={{width: "138px"}}>
                    {`Large ${db_type === types.ORACLE_CDB ? "PDBs" : ""}`}
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 14} onOpen={() => setEditedIndex(14)}>

                      <InplaceDisplay><div className="title"> {currentData?.large_disk} GB</div></InplaceDisplay>
                      <InplaceContent><div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.large_disk}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 1000) {
                          updateCell("large_disk", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={1000}  />} GB</div></InplaceContent>

                    </Inplace>
                    <Slider
                      value={currentData?.large_disk}
                      min={1}
                      max={1000}
                      step={1}
                      onChange={(e) => updateCell("large_disk", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 15} onOpen={() => setEditedIndex(15)}>
                      <InplaceDisplay> <div className="title"> {currentData?.large_cpu} Cores</div></InplaceDisplay>
                      <InplaceContent> <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.large_cpu}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 1) {
                          updateCell("large_cpu", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={16}  />}  Cores</div></InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.large_cpu}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("large_cpu", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 16} onOpen={() => setEditedIndex(16)}>
                      <InplaceDisplay> <div className="title"> {currentData?.large_disk} GB</div></InplaceDisplay>
                      <InplaceContent>  <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.large_cpu}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 16) {
                          updateCell("large_disk", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={16}  />} GB</div></InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.large_memory}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("large_memory", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 17} onOpen={() => setEditedIndex(17)}>
                        <InplaceDisplay>
                          <div className="title">{currentData?.large_shared_temp} GB</div>
                        </InplaceDisplay>
                        <InplaceContent><div className="title"><InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.large_shared_temp} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                            updateCell("large_shared_temp", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={100}  /> GB</div></InplaceContent>
                      </Inplace>
                      <Slider
                        value={currentData?.large_shared_temp}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("large_shared_temp", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 18} onOpen={() => setEditedIndex(18)}>
                      <InplaceDisplay>
                        <div className="title">{currentData?.large_backup_keep_days} Days</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.large_backup_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                            updateCell("large_backup_keep_days", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={100}  />} Days</div>
                      </InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.large_backup_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("large_backup_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 19} onOpen={() => setEditedIndex(19)}>
                      <InplaceDisplay>
                        <div className="title"> {currentData?.large_export_keep_days} Days</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.large_export_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                            updateCell("large_export_keep_days", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={100}  />} Days</div>
                      </InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.large_export_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("large_export_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 20} onOpen={() => setEditedIndex(20)}>
                        <InplaceDisplay>
                          <div className="title"> {currentData?.large_restore_points_keep_days} Days</div>
                        </InplaceDisplay>
                        <InplaceContent>
                          <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.large_restore_points_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                              updateCell("large_restore_points_keep_days", e.value);

                            }

                          }} disabled={!is_superuser} min={1} max={100}  />} Days</div>
                        </InplaceContent>
                      </Inplace>

                      <Slider
                        value={currentData?.large_restore_points_keep_days}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("large_restore_points_keep_days", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }
              </tr>
              <tr className="p-row-odd">
                <td className="col-fixed" style={{width: "10vw"}}>
                  <div className="col-fixed" style={{width: "138px"}}>
                    {`Very large ${db_type === types.ORACLE_CDB ? "PDBs" : ""}`}
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 21} onOpen={() => setEditedIndex(21)}>
                      <InplaceDisplay>
                        <div className="title">{currentData?.very_large_disk} GB</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.very_large_disk} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 1000) {
                            updateCell("very_large_disk", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={1000}  />} GB</div>
                      </InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.very_large_disk}
                      min={1}
                      max={1000}
                      step={1}
                      onChange={(e) => updateCell("very_large_disk", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}} >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 22} onOpen={() => setEditedIndex(22)}>

                      <InplaceDisplay>
                        <div className="title">{currentData?.very_large_cpu} Cores</div>
                      </InplaceDisplay>
                      <InplaceContent>
                        <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.very_large_cpu} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 16) {
                            updateCell("very_large_cpu", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={16}  />} Cores</div>
                      </InplaceContent>

                    </Inplace>

                    <Slider
                      value={currentData?.very_large_cpu}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("very_large_cpu", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 23} onOpen={() => setEditedIndex(23)}>
                      <InplaceDisplay><div className="title"> {currentData?.very_large_memory} GB</div></InplaceDisplay>
                      <InplaceContent><div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.very_large_memory}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 16) {
                          updateCell("very_large_memory", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={16}  />} GB</div></InplaceContent>
                    </Inplace>
                    <Slider
                      value={currentData?.very_large_memory}
                      min={1}
                      max={16}
                      step={1}
                      onChange={(e) => updateCell("very_large_memory", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 24} onOpen={() => setEditedIndex(24)}>
                        <InplaceDisplay> <div className="title">{currentData?.very_large_shared_temp} GB</div></InplaceDisplay>
                        <InplaceContent>
                          <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.very_large_shared_temp} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                              updateCell("very_large_shared_temp", e.value);

                            }

                          }} disabled={!is_superuser} min={1} max={100}  />} GB</div>
                        </InplaceContent>
                      </Inplace>
                      <Slider
                        value={currentData?.very_large_shared_temp}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("very_large_shared_temp", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace disabled={!is_superuser} closable  active={editedIndex === 25} onOpen={() => setEditedIndex(25)}>
                      <InplaceDisplay> <div className="title"> {currentData?.very_large_backup_keep_days} Days</div></InplaceDisplay>
                      <InplaceContent> <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.very_large_backup_keep_days}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                          updateCell("very_large_backup_keep_days", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={100}  />} Days</div></InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.very_large_backup_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("very_large_backup_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                <td className="col-fixed" style={{width: "10vw"}}  >
                  <div >
                    <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 26} onOpen={() => setEditedIndex(26)}>
                      <InplaceDisplay><div className="title"> {currentData?.very_large_export_keep_days} Days</div></InplaceDisplay>
                      <InplaceContent><div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.very_large_export_keep_days}  onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                          updateCell("very_large_export_keep_days", e.value);

                        }

                      }} disabled={!is_superuser} min={1} max={100}  />} Days</div></InplaceContent>
                    </Inplace>

                    <Slider
                      value={currentData?.very_large_export_keep_days}
                      min={1}
                      max={100}
                      step={1}
                      onChange={(e) => updateCell("very_large_export_keep_days", e.value)}

                      disabled={!is_superuser}
                    />
                  </div>
                </td>
                {

                  db_type === types.ORACLE_CDB &&
                  <td className="col-fixed" style={{width: "10vw"}}  >
                    <div >
                      <Inplace  disabled={!is_superuser}  closable  active={editedIndex === 27} onOpen={() => setEditedIndex(27)}>
                        <InplaceDisplay> <div className="title">{currentData?.very_large_restore_points_keep_days} Days</div></InplaceDisplay>
                        <InplaceContent> <div className="title">{<InputNumber useGrouping={false} className='sizing-precise-input' autoFocus value={currentData?.very_large_restore_points_keep_days} onValueChange={(e) => {

                          if (e.value > 0 && e.value <= 100) {
                            updateCell("very_large_restore_points_keep_days", e.value);

                          }

                        }} disabled={!is_superuser} min={1} max={100}  />} Days</div></InplaceContent>
                      </Inplace>
                      <Slider
                        value={currentData?.very_large_restore_points_keep_days}
                        min={1}
                        max={100}
                        step={1}
                        onChange={(e) => updateCell("very_large_restore_points_keep_days", e.value)}

                        disabled={!is_superuser}
                      />
                    </div>
                  </td>
                }
              </tr>
            </tbody>
          </table>

        </div>

      </div>

      <Button disabled={!is_superuser} icon="pi pi-check" iconPos='right' label="Save" className='ml-auto mr-auto mt-8' onClick={() => {
        setEditedIndex(-1);
        dispatch(ClusterUpdateActions.clusterUpdate(dataRef.current))
      }} />

    </div>

  );
}