import { useDispatch, useSelector } from "react-redux";
import { useState } from "react";
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { Button } from "primereact/button";
import { ConfirmPopup } from "primereact/confirmpopup";
import { ConfigActions } from "../../../store/config/reducer";
import { clusterRoles } from "../../../config";
import EditDatabaseUser from "../../../dialogs/editDatabaseUser";
import ErrorTextBox from "../../errorTextBox";
import InfoDialog from "../../../dialogs/infoDialog";
import { loadingMsg} from "../../../config";


export default function DatabaseUsersTable({node, users}) {

    const dispatch = useDispatch();
    const [popupElement, setPopupElement] = useState(false);
    const [userToEdit, setUserToEdit] = useState(false);
    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [showEditDatabaseUser, setShowEditDatabaseUser] = useState(false);
    const [connectionString, setConectionString] = useState("");
    const [showInfoDialog, setShowInfoDialog] = useState(false);
    const { isLoading, errorMessage } = useSelector((state) => state.config);

    const actionBodyTemplate = (rowData) => {
        return (
            <div className="flex">
                <Button icon="pi pi-ellipsis-h" onClick={() => {

                    setConectionString(rowData.connection_string);
                    setShowInfoDialog(true);

                }} className="p-button-rounded p-button-info mr-1 ml-auto" />
                <Button icon="pi pi-pencil" onClick={() => {
                    setUserToEdit(rowData);
                    setShowEditDatabaseUser(true);
                }} className="p-button-rounded p-button-success mr-1 ml-auto" disabled={node?.cluster_role !== clusterRoles.PRIMARY} />
                <Button icon="pi pi-trash" className="p-button-rounded p-button-danger" id={rowData.name} disabled={node?.cluster_role !== clusterRoles.PRIMARY} onClick={() => {
                    setUserToEdit(rowData);
                    setPopupElement(window.document.getElementById(rowData.name));
                    setShowDeleteConfirm(true);
                }} />
            </div>
        );
    }


    return (
        <div>
            <DataTable
                emptyMessage={isLoading ? <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">no database users found</div>}
                className="monitors-table database-users-table"
                value={users}
                paginator={users?.length > 10}
                rows={10}
                paginatorTemplate="FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink CurrentPageReport"
                currentPageReportTemplate="Showing {first} to {last} of {totalRecords} users"
                paginatorPosition="bottom">
                <Column field="name" header="Name" sortable></Column>
                <Column field="description" header="Description" sortable></Column>
                <Column body={actionBodyTemplate}></Column>
            </DataTable>

            <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={popupElement} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="This will delete user" icon="pi pi-exclamation-triangle" acceptIcon="pi pi-trash" acceptLabel="Confirm" rejectLabel="Cancel" reject={() => setShowDeleteConfirm(false)} accept={() => dispatch(ConfigActions.dropDatabaseUser({ "node_id": node?.id, "user": userToEdit?.name }))} />
            {showEditDatabaseUser ? <EditDatabaseUser onHide={() => setShowEditDatabaseUser(false)} visible={showEditDatabaseUser} onOkClick={(data) => dispatch(ConfigActions.editDatabaseUser(data, setShowEditDatabaseUser))} user={userToEdit} /> : null}
            {errorMessage ? <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} /> : null}
            {showInfoDialog ? <InfoDialog
                icon = "pi pi-info-circle"
                message={`Connection string: ${connectionString}`}
                visible={showInfoDialog}
                onHide={() => {

                    setShowInfoDialog(false);
                }
                } />
                : null}
        </div>
    );
}