import { useDispatch, useSelector } from "react-redux";
import { useEffect, useState } from "react";
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { InputText } from "primereact/inputtext";
import { ConfirmPopup } from "primereact/confirmpopup";
import { ConfigActions } from "../../../store/config/reducer";
import { LayoutActions } from "../../../store/layout/reducer";
import { serviceTypes, types } from "../../../config";
import AddIntegratedServiceDialog from "../../../dialogs/addIntegratedServiceDialog";
import TooltipCell from "../../tooltipCell";

export default function IntegratedServicesTable(props) {

    const dispatch = useDispatch();
    const { integrated_services } = useSelector((state) => state.config);
    const [popupElement, setPopupElement] = useState(false);
    const [targetService, setTargetService] = useState(false);
    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [filteredServices, setFilteredServices] = useState(null);
    const [search, setSearch] = useState("");
    const [showAddService, setShowAddService] = useState(false);
    const [serviceType, setServiceType] = useState(types.ALL);
;

    const databasesBodyTemplate = (rowData, props) => ( <TooltipCell content = {JSON.stringify(rowData.supported_databases)}/>);

    useEffect(() => {

        if (!integrated_services) dispatch(ConfigActions.getIntegratedServices());
        if (integrated_services?.length > 0) {

            let tempData = integrated_services;
            let query = search.toLowerCase();
            tempData = tempData.filter((s) => s.name.toLowerCase().includes(query) || s.port.toLowerCase().includes(query) ||
                s.username.toLowerCase().includes(query) || s.type.toLowerCase().includes(query) || s.host_name.toLowerCase().includes(query))
            tempData = serviceType === types.ALL ? tempData : tempData.filter((s) => s.type === serviceType);


            setFilteredServices(tempData);

        }
        else setFilteredServices([])


    }, [dispatch, integrated_services, serviceType, search]);


    const actionBodyTemplate = (rowData) => {
        return (
            <div className="flex">
                <Button icon="pi pi-pencil" onClick={() => { 
                     setTargetService(rowData);
                     setShowAddService(true);


                 }} className="p-button-rounded p-button-success mr-1 ml-auto" />
                <Button icon="pi pi-trash" className="p-button-rounded p-button-danger" id={`${rowData.name}_${rowData.port}`} onClick={() => {
                    setTargetService(rowData);
                    setPopupElement(window.document.getElementById(`${rowData.name}_${rowData.port}`));
                    setShowDeleteConfirm(true);
                }} />
            </div>
        );
    }

    const addServiceHeader = () => {

        return (<div><Button className="p-button-rounded p-button-secondary ml-auto mr-1" icon="pi pi-refresh" onClick={() => dispatch(ConfigActions.setIntegratedServices(null))} /><Button className="p-button-rounded p-button-info" icon="pi pi-plus" onClick={() =>{
            setTargetService(null);
            setShowAddService(true);
        } 
    } /></div>)

    }



    return (
        <div>
            <div className="flex">

                <Dropdown className="p-inputtext-sm filter-field mr-2 mb-2" value={serviceType}
                    options={serviceTypes}
                    placeholder="Select service type"
                    onChange={(e) => {
                        setServiceType(e.value);
                    }
                    } />

                <span className="p-input-icon-right">
                    <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => setSearch("")} />

                    <InputText className="p-inputtext-sm filter-field mb-2"
                        placeholder="Search"
                        value={search}
                        onChange={(e) => {
                            setSearch(e.target.value);
                        }} />
                </span>
            </div>


            <DataTable emptyMessage={<td className="text-sm pl-3">No services data</td>} className="monitors-table" value={filteredServices}
                paginator alwaysShowPaginator={false} rows={10}
                paginatorTemplate="FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink CurrentPageReport"
                currentPageReportTemplate="Showing {first} to {last} of {totalRecords} integrated services"
                paginatorPosition="bottom">
                <Column field="type" header="Type" sortable></Column>
                <Column field="name" header="Name" sortable></Column>
                <Column field="host_name" header="Host Name" sortable></Column>
                <Column field="port" header="Port" sortable></Column>
                <Column field="username" header="Username" sortable></Column>
                <Column header="Supported Databases" body={databasesBodyTemplate} ></Column>
                <Column header={addServiceHeader} body={actionBodyTemplate}></Column>
            </DataTable>
            <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={popupElement} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="This will unregister service" icon="pi pi-exclamation-triangle" acceptIcon="pi pi-trash" acceptLabel="Confirm" rejectLabel="Cancel" reject={() => setShowDeleteConfirm(false)} accept={() => dispatch(ConfigActions.dropIntegratedService({ "service_id": targetService?.id }))} />
            <AddIntegratedServiceDialog
                visible={showAddService}
                data={targetService}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(null));
                    setShowAddService(false);
                }}
                onOkClick={(data) => {
                    dispatch(LayoutActions.setPortExists(null));
                    dispatch(targetService ? ConfigActions.editIntegratedService(data, setShowAddService) : ConfigActions.addIntegratedService(data, setShowAddService));
                }}
            />
        </div>
    );
} 