import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { Button } from "primereact/button";
import { Fragment, useState, useEffect } from "react";
import { useDispatch } from "react-redux";
import { ConfigActions } from "../../../store/config/reducer";
import TooltipCell from "../../tooltipCell";
import WarningDialog from "../../../dialogs/warningDialog";
import AddLicenseDialog from "../../../dialogs/licensing/addLicense";
import { ConfirmPopup } from "primereact/confirmpopup";

export default function KeysTable(props) {
    const [filteredKeys, setFilteredKeys] = useState([]);
    const [showAddLicense, setShowAddLicense] = useState(false);
    const [warningMessage, setWarningMessage] = useState("");
    const [showWarning, setShowWarning] = useState(false);
    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [keyToDelete, setKeyToDelete] = useState(null);
    const [popupElement, setPopupElement] = useState(false);

    const actionBodyTemplate = (rowData, props) => {
        return (
            <div className="flex">
                <Button disabled={!props.user?.is_superuser} icon="pi pi-trash" className="p-button-rounded p-button-danger" id={`key_${props.rowIndex}`} onClick={() => {
                    setKeyToDelete(rowData);
                    setPopupElement(window.document.getElementById(`key_${props.rowIndex}`));
                    setShowDeleteConfirm(true);
                }} />
            </div>
        );
    }


    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    useEffect(() => {
        if (props.licenses) setFilteredKeys(props.licenses);
    }, [props.licenses])

    const dispatch = useDispatch();
    const onAddLicense = (data) => dispatch(ConfigActions.addLicense(data, { "cleanup": setShowAddLicense, "notify": showWarningMessage }));

    return (
        <Fragment>
            <DataTable
                value={filteredKeys}
                paginator alwaysShowPaginator={false} rows={10}
                paginatorTemplate="FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink CurrentPageReport"
                currentPageReportTemplate="Showing {first} to {last} of {totalRecords} keys"
                paginatorPosition="bottom">
                <Column body={(rowData, props) => <TooltipCell id={`${props.field}-${props.rowIndex}`} content={rowData[props.field]} />} footer={() => props.summary ? props.summary["key"] : 'summary'} style={{ maxWidth: "100px" }} field="key" header="Key" sortable />
                <Column footer={() => props.summary ? props.summary["elasticsearch"] : 0} field="elasticsearch" header="Elasticsearch" sortable />
                <Column footer={() => props.summary ? props.summary["mariadb"] : 0} field="mariadb" header="MariaDB" sortable />
                <Column footer={() => props.summary ? props.summary["mysql"] : 0} field="mysql" header="MySQL" sortable />
                <Column footer={() => props.summary ? props.summary["mongo"] : 0} field="mongo" header="Mongo" sortable />
                <Column footer={() => props.summary ? props.summary["mssql"] : 0} field="mssql" header="MsSQL" sortable />
                <Column footer={() => props.summary ? props.summary["oracle"] : 0} field="oracle" header="Oracle" sortable />
                <Column footer={() => props.summary ? props.summary["postgresql"] : 0} field="postgresql" header="PostgreSQL" sortable />
                <Column footer={() => props.summary ? props.summary["redis"] : 0} field="redis" header="Redis" sortable />
                <Column field="action" header={<Button disabled={!props.user?.is_superuser} rounded size="small" className="p-button-info" icon="pi pi-plus" onClick={() => setShowAddLicense(true)} />} body = {actionBodyTemplate} />
            </DataTable>
            {showAddLicense ? <AddLicenseDialog
                visible={showAddLicense}
                onHide={() => setShowAddLicense(false)}
                onOkClick={onAddLicense}
                 />
                : null}
            {showWarning ? <WarningDialog
                message={warningMessage}
                visible={showWarning}
                onHide={() => setShowWarning(false)} />
                : null}
         <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={popupElement} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="This will unregister service" icon="pi pi-exclamation-triangle" acceptIcon="pi pi-trash" acceptLabel="Confirm" rejectLabel="Cancel" reject={() => setShowDeleteConfirm(false)} accept={() => dispatch(ConfigActions.removeLicense({ "key": keyToDelete?.key }))} />
        </Fragment>
    );
}
