import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { Button } from "primereact/button";
import { useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import { loadingMsg } from "../../../config";
import { LogActions } from "../../../store/log/reducer";

export default function LogFilesTable({ node_id }) {
    const dispatch = useDispatch();
    const { node_log_files, isLoading } = useSelector((state) => state.log);

    useEffect(() => {

        if (node_id) dispatch(LogActions.getNodeLogFiles({ "node_id": node_id }))

    }, [node_id, dispatch]);

    const actionBodyTemplate = (data, props) => {
        return (
            <div className="flex">
                <Button icon="pi pi-download" className="p-button-rounded" id={`key_${props.rowIndex}`} onClick={() => {

                    const a = document.createElement("a");
                    a.href = data.download_url;
                    a.style.display = "none";
                    a.download = data.filename || "";
                    document.body.appendChild(a);
                    a.click();
                    document.body.removeChild(a);

                }} />
            </div>
        );
    }



    return (
        <div>
            <DataTable
                value={node_log_files}
                emptyMessage={isLoading ? <div className="text-sm empty-severity">{loadingMsg}</div> : <div className="text-sm empty-severity">No log files</div>}
                paginator alwaysShowPaginator={false} rows={10}
                paginatorTemplate="FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink CurrentPageReport"
                currentPageReportTemplate="Showing {first} to {last} of {totalRecords} files"
                paginatorPosition="bottom">
                <Column header="File" sortable field="filename"></Column>
                <Column header="Modified Date" sortable field="modified_date"></Column>
                <Column header="Size" sortable field="size"></Column>
                <Column body={actionBodyTemplate}></Column>
            </DataTable>

        </div>
    );
}
