import { useState, useEffect } from "react";
import { useSelector } from "react-redux";
import TaskStepLogDialog from "../../../dialogs/taskStepLogDialog";
import { LogActions } from "../../../store/log/reducer";
import { useDispatch } from "react-redux";
import { useLocation } from "react-router-dom";
import { DataTable} from "primereact/datatable";
import { Column } from "primereact/column";
import { loadingMsg } from "../../../config";
import getColorBySeverity from "../../../util/colorBySeverity";

import moment from "moment";

export default function LogsTable({filteredLogs, timePeriodType, node_id = false}){

        const [stepLog, setStepLog] = useState(false);
        const [showStepLog, setShowStepLog] = useState(false);
        const [selectedRecord, setSelectedRecord] = useState(null);
        const { isLoading, logs } = useSelector((state) => state.log);
        const dispatch = useDispatch();
        const { state } = useLocation();

         useEffect(() => {
                if (logs && logs?.length > 0 && state?.target_log) {
                    const target_log = logs.filter((l) => l.what === state.target_log);
                    if (target_log?.length > 0) {
                        setStepLog(target_log?.[target_log.length - 1]?.task_log_id);
                        setShowStepLog(true);
                    }
                }
            }, [logs, state])

        useEffect(() => {
            if (node_id === false) dispatch(LogActions.getLogs({ time_period: timePeriodType }))
            if (node_id) dispatch(LogActions.getLogsPerNode({ node_id: node_id, time_period: timePeriodType }))
        }, [timePeriodType, node_id, dispatch])


        return (
          <div>
            <DataTable
              className="severity-table custom-row-height"
              scrollable
              scrollHeight="flex"
              onRowClick={(e) => setShowStepLog(true)}
              rowClassName={(rowData) => `severity-${getColorBySeverity(rowData.status)}-row`}
              emptyMessage={
                !filteredLogs && isLoading ? (
                  <div className="text-sm empty-severity">{loadingMsg}</div>
                ) : (
                  <div className="text-sm empty-severity">No logs data</div>
                )
              }
              selectionMode="single"
              selection={selectedRecord}
              dataKey="id"
              onSelectionChange={(e) => {
                setSelectedRecord(e.value);
                setStepLog(e.value?.task_log_id);
              }}
              paginator
              alwaysShowPaginator={false}
              rows={100}
              rowsPerPageOptions={[10, 20, 50, 100]}
              value={filteredLogs}
            >
                                    <Column field="status" header="Status" body = {(item) => <div className={node_id ? `severity-${getColorBySeverity(item.status)}-row-outer` : ''}>{item.status}</div>} sortable></Column>
                                    <Column field="when" body={(item) => <div className="nowrap-cell">

                                        {moment(item.when).format("MMM DD,YYYY, hh:mm A")}

                                    </div>} header="When" sortable></Column>
                                    <Column field="task" header="What" sortable></Column>
                                    <Column field="duration" style={{ width: "150px" }} header="Duration (seconds)" sortable></Column>
                                </DataTable>
                                {showStepLog ? <TaskStepLogDialog
                                    taskLogId={stepLog}
                                    target_log={state?.target_log}
                                    selectedLog={selectedRecord}
                                    onHide={() => {
                                        dispatch(LogActions.setStepLogs(null));
                                        setShowStepLog(false);
                                    }}
                                    visible={showStepLog}
                                /> : null} </div>)

}
