import { useDispatch, useSelector } from "react-redux";
import { useEffect, Fragment, useState} from "react";
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { Button } from "primereact/button";
import { MonitoringActions } from "../../../store/monitoring/reducer";
import { ConfirmPopup } from "primereact/confirmpopup";
import TooltipCell from "../../tooltipCell";
import { types } from "../../../config";

export default function MonitorsTable (props) {

    const dispatch = useDispatch();
    const { monitors } = useSelector((state) => state.monitoring);
    const [popupElement, setPopupElement] = useState(false);
    const [monitorToDelete, setMonitorToDelete] = useState(false);
    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [filteredMonitors, setFilteredMonitors] = useState(null);

    useEffect(() => {
        
        if (!monitors) dispatch(MonitoringActions.getMonitors({"org_id": 1 }));
        else if (monitors?.length)
        {

            let tempData = monitors;
            let query = props.monitorSearch.toLowerCase();
            if(props.monitorType && props.monitorType !== types.ALL) tempData = tempData.filter((m) => m.db_type === props.monitorType);
            if(query) tempData = tempData.filter((m) => m.monitor_name.toLowerCase().includes(query)  || m.monitor_description.toLowerCase().includes(query) || m.db_type.toLowerCase().includes(query) || m.monitor_interval.toLowerCase().includes(query) || m.yellow_rule.toLowerCase().includes(query) || m.red_rule.toLowerCase().includes(query) )


            setFilteredMonitors(tempData);

        }
        

    }, [dispatch, monitors, props.monitorType, props.monitorSearch]);
    
    const descriptionBodyTemplate = (rowData, props) => 
    {
     return ( <TooltipCell id = {`${props.field}-${props.rowIndex}`} content = {rowData.monitor_description}/>)  
    }

    const statusBodyTemplate = (rowData) => {
        return rowData.ind_enabled === "Y" ? "Enabled" : "Disabled";
    }

    const actionBodyTemplate = (rowData) => {
        return (
            <div className = "flex">
                <Button icon="pi pi-pencil" onClick={() => { if(props.onRowEdit) props.onRowEdit(rowData)}} className="p-button-rounded p-button-success mr-1 ml-auto" />
                <Button icon="pi pi-trash"  disabled ={!props.user?.is_superuser} className="p-button-rounded p-button-danger" id ={`${rowData.monitor_name}_${rowData.org_id}_${rowData.node_id}`}  onClick = {() => {
                    setMonitorToDelete(rowData);
                    setPopupElement(window.document.getElementById(`${rowData.monitor_name}_${rowData.org_id}_${rowData.node_id}`));
                    setShowDeleteConfirm(true);
                    }}/>
            </div>
        );
    }

    const addMonitorHeader = () => 
    {

        return (<Fragment><Button className="p-button-rounded p-button-secondary ml-auto mr-1" icon = "pi pi-refresh" onClick={() => dispatch(MonitoringActions.setMonitors(null))}/><Button  disabled ={!props.user?.is_superuser} className="p-button-rounded p-button-info" icon = "pi pi-plus" onClick={() => props.onRowAdd()}/></Fragment>)

    }



    return (
        <Fragment>
                <DataTable emptyMessage = {<td className="text-sm pl-3">No monitors data</td>} className="monitors-table" value={filteredMonitors}
                    paginator = {filteredMonitors?.length > 10} rows={10}
                    paginatorTemplate="FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink CurrentPageReport"
                    currentPageReportTemplate="Showing {first} to {last} of {totalRecords} monitors"
                    paginatorPosition="bottom">
                    <Column  field="monitor_name" header="Name" sortable></Column>
                    <Column  field="db_type" header="Type" sortable></Column>
                    <Column  field="monitor_description" header="Description" body = {descriptionBodyTemplate} sortable></Column>
                    <Column  field="ind_enabled" header="Status" body={statusBodyTemplate}></Column>
                    <Column  field="monitor_interval" header="Interval" sortable></Column>
                    <Column header = {addMonitorHeader} body={actionBodyTemplate}></Column>
                </DataTable>
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible = {showDeleteConfirm} target={popupElement} acceptClassName = "p-button-danger" rejectIcon = "pi pi-times" message = "This will delete monitor" icon = "pi pi-exclamation-triangle" acceptIcon = "pi pi-trash" acceptLabel="Confirm" rejectLabel="Cancel" reject={() => setShowDeleteConfirm(false)} accept = {() => dispatch(MonitoringActions.deleteMonitor({"org_id": monitorToDelete?.org_id, "node_id": monitorToDelete?.node_id, "monitor_name": monitorToDelete?.monitor_name, "type":  monitorToDelete?.db_type }))} />
                </Fragment>
    );
}