import UserUpdateDialog from '../../../dialogs/userUpdate';
import { useState, useEffect, Fragment } from 'react';
import moment from "moment";
import { useDispatch } from 'react-redux';
import { UserUpdateActions } from '../../../store/userUpdate/reducer';
import { SignupActions } from '../../../store/signup/reducer';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { ConfirmPopup } from "primereact/confirmpopup";
import { useSelector } from 'react-redux';
import { Button } from 'primereact/button';
import notify from '../../../util/notify';
import { loadingMsg} from '../../../config';
import {ConfigActions} from '../../../store/config/reducer'
import config from '../../../config';
import SignupDialog from "../../../dialogs/signup";
import appConfig from "../../../config";


export default function UsersTable(props) {
  const [showUserUpdate, setShowUserUpdate] = useState(false);
  const dispatch = useDispatch();
  const { errorMessage, infoMessage } = useSelector((state) => state.userUpdate);
  const [selectedRecord, setSelectedRecord] = useState(null);
  const [showSignup, setShowSignup] = useState(false);
  const [filteredUsers, setFilteredUsers] = useState([]);
  const [popupElement, setPopupElement] = useState(null);
  const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
  const [currentUser, setCurrentUser] = useState(null);
  useEffect(() => {
    if (errorMessage) notify.error(errorMessage);
    if (infoMessage === "ok") {
      dispatch(UserUpdateActions.setInitial(false));
      dispatch(UserUpdateActions.setData(config.formDefaults));
      setShowUserUpdate(false);
    }
  }, [errorMessage, infoMessage, dispatch])

  const onSignup = (data) => {

    dispatch(SignupActions.signUp(data, setShowSignup));
  };


  const usernameTextBodyTemplate = (item) => {

    return item.username;
  }
  

  const activityTextBodyTemplate = (item) => {

    return item.last_activity;
  }

  const dateTextBodyTemplate = (item) => {

    const date =  moment(item.date_joined)


    return date.isValid() ?  date.format("MMM DD,YYYY, hh:mm A") : "Never";
  }

  const actionHeader = () => <Fragment> <Button className="p-button p-button-rounded p-button-secondary ml-auto mr-1" icon = "pi pi-refresh" onClick={() => dispatch(ConfigActions.setConfig(null))}/> <Button className="p-button p-button-rounded p-button-info"  icon = "pi pi-plus" onClick ={() => setShowSignup(true)}/></Fragment>;

  const loginTextBodyTemplate = (item) => {

    const date =  moment(item.last_login)


    return date.isValid() ?  date.format("MMM DD,YYYY, hh:mm A") : "Never";
  }

  const actionBodyTemplate = (rowData) => (<Fragment>
     <Button disabled = {(!(rowData.username === currentUser.username 
          || currentUser.is_superuser))} icon="pi pi-pencil" className="p-button-sm p-button-rounded p-button-success" onClick={() => {
            setSelectedRecord(rowData);
            dispatch(UserUpdateActions.userUpdate({ pk: 
              rowData?.id, data: false }));
            setShowUserUpdate(true);
          }} />
     <Button disabled = {(!(rowData.username === currentUser.username 
          || currentUser.is_superuser))} icon="pi pi-trash" className="p-button-sm p-button-rounded p-button-danger ml-1 mr-0" id ={`delete_user_${rowData?.id}`} onClick = {() => {
      setSelectedRecord(rowData);
      setPopupElement(window.document.getElementById(`delete_user_${rowData?.id}`))
      setShowDeleteConfirm(true);
     }}/>
  </Fragment>)


  useEffect(() => {
    
    if(props.users?.length) setFilteredUsers(props.users);

  }, [props.users])

  useEffect(() => {

    if(props.current_user) setCurrentUser(props.current_user);

  }, [props.current_user])

  return (
    <Fragment>
      <DataTable emptyMessage = {!filteredUsers ? <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">No users data</div>} paginator alwaysShowPaginator={false} dataKey="id"
 rows={10} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredUsers} className="users-table" >
        <Column field="username" body={usernameTextBodyTemplate} header="Username" sortable></Column>
        <Column field="date_joined" body={dateTextBodyTemplate} header="Date joined" sortable></Column>
        <Column field="last_login" body={loginTextBodyTemplate} header={"Last Login"} sortable></Column>
        <Column field="last_activity" body={activityTextBodyTemplate} header={"Last activity"} sortable></Column>
        <Column body = {actionBodyTemplate} header={actionHeader} ></Column>
      </DataTable>
      {showUserUpdate ? <UserUpdateDialog userId={selectedRecord.id} current={props.current_user} visible={showUserUpdate} onHide={() => {
        dispatch(UserUpdateActions.setInitial(false));
        dispatch(UserUpdateActions.setData(config.formDefaults));
        setShowUserUpdate(false)
      }
      } /> : null}
        {showSignup ? <SignupDialog visible={showSignup} onHide={() => {
        dispatch(SignupActions.setData(appConfig.formDefaults));
        setShowSignup(false);
      }} onOkClick={onSignup}
      /> : null}
      <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible = {showDeleteConfirm} target={popupElement} acceptClassName = "p-button-danger" rejectIcon = "pi pi-times" message = "This will delete user" icon = "pi pi-exclamation-triangle" acceptIcon = "pi pi-trash" acceptLabel="Confirm" rejectLabel="Cancel" reject={() => setShowDeleteConfirm(false)} accept = {() => dispatch(UserUpdateActions.userDelete(selectedRecord.id))} />
    </Fragment>
  );
}