import React, { useRef, useEffect, useCallback } from 'react';
import moment from "moment";
import { Tooltip } from "primereact/tooltip";
import { Panel } from "primereact/panel";
import { rangeByMonitorInterval } from "../../util/timelines";
import { scaleTime } from 'd3-scale';


function TimestripTooltip({alert, pieceRef}) {

    return <Tooltip
    className="timestrip-tooltip"
    event='hover'
    target={pieceRef}
    position="top" >
    <div>
      <Panel
        className="text-xs timestrip-tooltip-text"
        header={alert?.monitor}
      >
        <div className="multiline">
          Alert date: {moment(alert?.date_created_with_tz).format("MM/DD/YY  hh:mm A")}{'\n'}
          {alert?.message && `Message: ${alert?.message?.replace(/\n\s*\n/g, '\n')}`}{'\n'}
          Monitor elapsed runtime (ms): {alert?.duration}
        </div>
      </Panel>
    </div>
  </Tooltip>
};

export default function Timestrip({ data, interval, focusCallback }) {
  let min = 0;
  let max = 100;
  let range = rangeByMonitorInterval(interval);
  let scale = scaleTime().domain([moment().subtract(range.amount, range.unit).toDate(), moment().toDate()]).range([min, max]).nice();
  let minVal = scale(moment(data[data?.length - 1]?.date_created_with_tz).toDate());

  const autoFocus = useCallback((element) => {
      if (element) element.focus();
  }, []);

  const pieceRefs = useRef([]);

  useEffect(() => {}, [data]);

  return (
      <div className="flex align-items-end flex-grow-1">
          {data?.slice(0).reverse().map((a, i, arr) => {
              let width = 0;
              let pos = scale(moment(a?.date_created_with_tz).toDate());
              if (i < arr.length - 1) width = scale(moment(arr[i + 1]?.date_created_with_tz).toDate()) - pos;
              else width = 100 - pos;

              const tooltipId = `timestrip-tooltip-${a?.alert_id}_${moment(a?.date_created_with_tz).valueOf()}_${a?.duration}`;

              return (
                  <div
                      id={tooltipId}
                      ref={(el) => {
                          pieceRefs.current[i] = el;
                          if (i === arr.length - 1) autoFocus(el);
                      }}
                      onClick={() => focusCallback ? focusCallback(a) : null}
                      onFocus={() => focusCallback ? focusCallback(a) : null}
                      onBlur={(e) => {
                          if (focusCallback && !e.target.parentElement.contains(e.relatedTarget)) {
                              e.target.focus();
                          }
                      }}
                      tabIndex={a?.alert_id}
                      className={`unselectable timestrip-content timestrip-piece-${a?.alert_id}_${moment(a?.date_created_with_tz).valueOf()}_${a?.duration}`}
                      key={tooltipId}
                      style={{
                          background: `${a.get_color}`,
                          width: `${width}%`,
                          marginLeft: `${i === 0 ? minVal : 0}%`,
                          marginRight: `${0}%`,
                          cursor: 'pointer'
                      }}
                  >
                      &#8205;
                      <TimestripTooltip alert={a} pieceRef={pieceRefs.current?.[i]} key={tooltipId} />
                  </div>
              );
          })}
      </div>
  );
}
