import config from "../../config";

const constraints = {
    port: {
        presence: {
            message: "Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535,
        }
    },
    bearer: {
        presence: {
            message: "Bearer token is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern: /^Bearer\s+[A-Za-z0-9\-._~+/]+=*$/,
            message: "Bearer token has invalid format."
        }
    },
    hostname: {
        presence: {
            message: "Hostname is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.hostNameRegex,
            message: "Host Name has invalid format."
        }
    },
    proxmoxTokenName: {
        presence: {
            message: "Proxmox Token Name is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern:  /^(?![-_.])[a-zA-Z0-9][-_.a-zA-Z0-9]{0,62}(?<![-_.])$/,
            message: "Proxmox Token Name has invalid format."
        }
    },
    proxmoxTokenValue: {
        presence: {
            message: "Proxmox Token Value is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern:  /^[a-zA-Z0-9._-]{8,64}$/,
            message: "Proxmox Token Value has invalid format."
        }
    },
    awsSecretToken: {
        presence: {
            message: "AWS Secret Token is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern:   /\b[0-9a-zA-Z/+=]{40}\b/,
            message: "AWS Secret Token has invalid format."
        }
    },
    awsAccessToken: {
        presence: {
            message: "AWS Access Token is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern:  /(?<!\w)(AKIA[A-Z0-9]{16}|ASIA[A-Z0-9]{16})(?!\w)/,
            message: "AWS Access Token has invalid format."
        }
    },
}

export default constraints;