import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { InputText } from "primereact/inputtext";
import { ConfirmPopup } from "primereact/confirmpopup";
import { types } from "../../config";
import TooltipCell from "../../components/tooltipCell";

const ApplySettingsDialog = ({ visible, onHide, onOkClick, settings, initial, node }) => {
  const [filteredSettings, setFilteredSettings] = useState(settings);
  const [search, setSearch] = useState("");
  const [applySettingsConfirm, setApplySettingsConfirm] = useState(false);

  const onSearchChange = (e) => {
    setSearch(e.target.value);
    const search = e.target.value.toLowerCase();
    setFilteredSettings(
      settings?.filter((s) => s?.name?.toLowerCase()?.includes(search) || s?.['NAME']?.toLowerCase()?.includes(search)));
  }

  const settingsHeader =  <span className="p-input-icon-right width-fit-content">
    <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => {
      setSearch("");
      setFilteredSettings(settings);

    }} ></i>
    
    <InputText className={"p-inputtext-sm filter-field mb-2"} placeholder="Search" value={search} onChange={onSearchChange} />
  </span> 

  const handleOk = () => {

    const data = {
      node_id: node?.id,
      parameters: settings
    }
    if (onOkClick) onOkClick(data);

  }

  const renderFooter = () => {
    return (
      <div className="flex justify-content-between">
        <Button
          label={"Back"}
          icon="pi pi-arrow-left"
          className="p-button-secondary"
          onClick={onHide} />
        <Button
          label={"Apply"}
          icon={"pi pi-check"}
          className="p-button"
          id = 'apply-settings-button'
          iconPos="right"
          onClick={() => setApplySettingsConfirm(true)} autoFocus />
      </div>
    );

  }

  const restartBodyTemplate = (rowData) => {
    return (rowData?.context === "postmaster" || rowData?.read_only === 'YES' || rowData?.['ISSYS_MODIFIABLE'] === 'FALSE') ? "Yes" : "No";
  }

  const toBodyTemplate = (rowData, props) => <TooltipCell id = {`${props.field}-${props.rowIndex}`} content = {rowData[node?.type === types.ORACLE_CDB ? "VALUE" : 'setting']}/> 

  const fromBodyTemplate = (rowData, props) => <TooltipCell id = {`${props.field}-${props.rowIndex}`} content = {initial?.filter((s) => s?.name === rowData?.name || s?.['NAME'] === rowData?.['NAME'])?.[0]?.[node?.type === types.ORACLE_CDB ? "VALUE" : 'setting']}/> 

  const renderBody = () => {

    return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

      <div className="field grid">
        <div className="col">

          <DataTable scrollable scrollHeight="flex" className="node-settings-table" header={settingsHeader} value={filteredSettings} dataKey={node?.type === types.ORACLE_CDB ? "NAME" : 'name'} paginator alwaysShowPaginator={false} rows={100} rowClassName={(rowData) => (rowData?.context === "postmaster" || rowData?.['ISSYS_MODIFABLE'] === 'FALSE') ? 'text-red-500' : ''} >
            <Column field={node?.type === types.ORACLE_CDB ? "NAME" : 'name'} header="Setting" sortable> </Column>
            <Column header="From" body={fromBodyTemplate} ></Column>
            <Column field = {node?.type === types.ORACLE_CDB ? "VALUE" : 'setting'} body={toBodyTemplate} header="To" ></Column>
            <Column header="Restart required" body={restartBodyTemplate} ></Column>
          </DataTable>


          <ConfirmPopup
            target={window.document.getElementById(`apply-settings-button`)}
            visible={applySettingsConfirm}
            onHide={() => setApplySettingsConfirm(false)}
            message={`Are you sure you want to ${settings?.filter((s) => (s?.context === 'postmaster' || s?.read_only || s?.['ISSYS_MODIFIABLE'] === 'FALSE'))?.length > 0 ? `restart ${node?.what}` : 'apply settings'}?`}
            accept={handleOk}
          />

        </div>
      </div>
    </div>)

  }
  return (<Dialog
    className='apply-settings-dialog'
    header={`Update settings of ${node?.name}`}
    footer={renderFooter()}
    visible={visible}
    style={{ width: '50vw', maxHeight: '90%' }}
    onHide={onHide}>
    <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
      {renderBody()}
    </div>
  </Dialog>
  )

}

export default ApplySettingsDialog;