import { useState, useEffect, useRef} from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import validator from "../../../../util/validator";
import { base64Encode } from '../../../../util/base64Encode';
import constraints from "./constraints";
import { LayoutActions } from '../../../../store/layout/reducer';
import copyToClipboard from "../../../../util/copyToClipboard";
import createAPIcommand from "../../../../util/createApiCommand";
import ApiBar from '../../../../components/apiBar';
import ErrorTextBox from '../../../../components/errorTextBox';
import { repositoryOptions } from '../../../../config';
import getValidationError from '../../../../util/errorPrompter';
import { Tooltip} from 'primereact/tooltip';
import { footer } from '../../../../util/passwordHelpers';
const AddHAProxy = (props) => {

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();


    // First Page elements
    const [osUsername, setOsUsername] = useState("");
    const [osPassword, setOsPassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState("");
    const [privateKey, setPrivateKey] = useState("");
    const [repository, setRepository] = useState(repositoryOptions[0].value);
    // Second Page elements
    const [port, setPort] = useState(5000);
    const customEvent = new Event('beforeinput', { bubbles: true });
    const submitButtonRef = useRef(null);

    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");

    // Second Page element errors
    const [portError, setPortError] = useState("");
    const [apiTooltip, setApiTooltip] = useState(" ");
    
    const { accessToken } = useSelector((state) => state.app);
    const { isLoading, port_exists, haproxy_exists, errorMessage } = useSelector((state) => state.layout);

    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {

                setPortError("");

            }
            else {


                if (port_exists?.exists) {

                    setPortError("already exists");
                }
                else {

                    setPortError("ok");

                }


            }
        }
    }, [port_exists])


    useEffect(() => {
        setPageIndex(0);

        // First page elements and errors
        setPort(5000);
        setPortError("");
        setOsUsername("root");
        setOsUsernameError("");
        setOsPassword("");
        setOsPasswordError("");
        setHostname("");
        setHostnameError("");
        setSshPort(22);
        setSshPortError("");
        setPrivateKey("");
        setPrivateKeyError("");
        setRepository(repositoryOptions[0].value);


        // Second page elements and errors
        setApiTooltip(" ");
    }, [props.visible, props.node]);


    useEffect(() => {

        if (props.visible && pageIndex === 1) {


            dispatch(LayoutActions.checkHaProxyExists(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }));


        }

    }, [props.visible, pageIndex, hostname, osUsername, osPassword, privateKey, sshPort, dispatch])

    const onRestApiClick = async () => {

        if (portError === "ok") {
            const cluster =
            {

              id: props.node.cluster 

            }

            const host =
            {
                name: hostname,
                ssh_port: sshPort,
                username: osUsername,
                password: base64Encode(osPassword),
                private_key: privateKey,
            };

            const ex_params =
            {
               
            }

            const data = {
                host,
                cluster,
                ex_params,
                name: `${hostname}:${port}`,
                repository: repository,
                port: port,
                haproxy_exists: haproxy_exists?.exists
            }
            setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "add_haproxy/")))


        }
        else {
            setApiTooltip(getValidationError());

        }
    
        if (!portError) setPortError("check port");
    }

    const validateFirst = () => {

        let errors = [];

        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);

        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }

        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        return errors;

    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {

            dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))
        }
    }

    const onSecondBackClick = () => {
        setPortError("");
        dispatch(LayoutActions.setErrorMessage(""));
        setPageIndex(0);
    }



    const onSaveClick = () => {

        if (portError === "ok") {
            
            onSubmit();

        }
        else submitButtonRef.current.dispatchEvent(customEvent);
        
        if (!portError) setPortError("check port");
    }

    const onSubmit = () => {

        const cluster =
        {

            id: props.node.cluster 

        }

        const host =
        {
            name: hostname,
            ssh_port: sshPort,
            username: osUsername,
            password: base64Encode(osPassword),
            private_key: privateKey,
        };

        const ex_params =
        {
    
        }

        const data = {
            host,
            cluster,
            ex_params,
            name: `${hostname}:${port}`,
            repository: repository,
            port: port,
            haproxy_exists: haproxy_exists.exists
        }
        if (props.onOkClick)
            props.onOkClick(data);

    }

    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'trash'}`}
                            id="submit-button"
                            className="submit-button"
                            ref={submitButtonRef}
                            iconPos="right"
                            onClick={onSaveClick}
                            autoFocus />
                     <Tooltip  target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip> 
                    </div>
                </div>
            );
        }
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Install from:"}</label>
                        <div className="col">
                            <Dropdown
                                className="p-inputtext-sm"
                                options={repositoryOptions}
                                value={repository}
                                onChange={(e) => setRepository(e.value)}
                            />
                        </div>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    const renderPageSecond = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Port" value={port} onValueChange={(e) => {
                                        setPort(e.value);
                                        setPortError("check port");

                                    }} />
                                    <i className={isLoading && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(portError !== 'ok') ? "p-error" : ""} p-d-block`}>{portError}</small>
                                </div>
                            </div>

                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("port", port);
                                        setPortError(error);
                                        if (!error)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: port,
                                                    os_username: osUsername,
                                                   os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                }))
                                    }

                                    }
                                />
                            </div>
                        </div>

                    </div>
                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}

                <ApiBar id = 'rest-api-bar' tooltip={apiTooltip} onClick={() => onRestApiClick()}  />
            </div>



        );
    }


    return (
        <Dialog
            className='add-haproxy'
            header={`Add a new HAProxy for cluster of ${props.node.name}`}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
            </div>

        </Dialog>
    );
}


export default AddHAProxy;