import { useState, useEffect } from 'react';
import { InputText } from "primereact/inputtext";
import { Button } from "primereact/button";
import { Dialog } from "primereact/dialog";
import { MultiSelect } from 'primereact/multiselect';
import { useDispatch, useSelector } from 'react-redux';
import copyToClipboard from '../../../../util/copyToClipboard';
import createAPIcommand from '../../../../util/createApiCommand';
import ApiBar from '../../../../components/apiBar';
import ErrorTextBox from '../../../../components/errorTextBox';
import config from '../../../../config';
import validator from '../../../../util/validator';
import constraints from './constraints';
import { LayoutActions } from '../../../../store/layout/reducer';
import { ConfirmPopup } from 'primereact/confirmpopup';

const DropHAProxyNodeDialog = (props) => {

    const [dropConfirm, setDropConfirm] = useState("");
    const [haproxyOptions, setHAProxyOptions] = useState(false);
    const { accessToken } = useSelector((state) => state.app);
    const { errorMessage, isLoading } = useSelector((state => state.layout));
    const [apiTooltip, setApiTooltip] = useState(" ");
    const [dropConfirmError, setDropConfirmError] = useState("");
    const [currentHAProxy, setCurrentHAProxy] = useState(null);
    const [showForceDeleteConfirm, setShowForceDeleteConfirm] = useState(false);
    const dispatch = useDispatch();

    useEffect(() => {

        if (props.haproxy?.hasOwnProperty('length') === false) setCurrentHAProxy([props.haproxy]);
        else if (props.haproxy?.hasOwnProperty('length')) {

            const haproxies = [];
            props.haproxy.forEach((h) => haproxies.push({ label: h.name, value: h }));
            setHAProxyOptions(haproxies);

        }

    }, [props.haproxy])

    const onDeleteHAProxyNodeClick = () => {
        let errors = validate();
        if (errors.length === 0) dispatch(LayoutActions.isNodeHostReachable({ 'node_id': currentHAProxy[0].id}, { allow: handleOk, confirm: setShowForceDeleteConfirm }));
    }

    const handleOk = () => {
        let haproxies = currentHAProxy?.map((h) => h.id);
        props.onOkClick({ "haproxies": haproxies });
    }


    const validate = () => {

        let errors = [];

        let error = validator(constraints)("dropConfirm", dropConfirm);
        (error == null || errors.push(error)) && setDropConfirmError(error);
        return errors;

    }



    const onRestApiClick = async () => {

        let errors = validate();
        let haproxies = currentHAProxy?.map((h) => h.name)

        if (errors.length === 0) {

            setApiTooltip(await copyToClipboard(createAPIcommand(
                {

                    "haproxies": haproxies,


                }
                , accessToken, 'drop_haproxy_node/')));


        }
        else setApiTooltip(" ");

    }


    const renderFooter = () => {
        return (
            <div className="flex justify-content-between">
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    label={"Submit"}
                    icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'trash'}`}
                    className="p-button-danger"
                    id='submit-button'
                    onClick={onDeleteHAProxyNodeClick} autoFocus />
                <ConfirmPopup
                    icon="pi pi-exclamation-triangle"
                    target={document.getElementById('submit-button')}
                    visible={showForceDeleteConfirm}
                    className="multiline"
                    onHide={() => setShowForceDeleteConfirm(false)}
                    accept={() => dispatch(LayoutActions.markNodeDeleted({ 'node_id': [currentHAProxy[0].id] }, props.onHide()))}
                    message={"Node host is currently offline, unable to perform drop.\nMark node as deleted anyway?"} />
            </div>
        );
    }


    return (
        <Dialog
            className='delete-haproxy-node'
            header={`Drop HAProxy`}
            visible={props.visible}
            style={{ width: '40vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <div style={{ marginBottom: "10px" }}>
                        This will drop {props?.haproxy?.name ? props.haproxy?.name : 'selected haproxies'}. This balancer{props.haproxy?.length ? 's' : ''} will no longer be manage by 123cluster.
                    </div>
                    <div>
                        Are you sure?
                    </div>
                    <div className="field">
                        {props.haproxy?.length > 0 && <MultiSelect scrollHeight="100%" className="p-inputtext-sm" selectedItemsLabel="All haproxies" placeholder='select haproxies to drop' options={haproxyOptions} value={currentHAProxy} onChange={(e) => setCurrentHAProxy(e.value)} />}
                        <div style={{ marginBottom: "10px" }} className="title mt-4">{`Please type "${config.dropConfirm}" here in order to continue`}</div>
                        <InputText className="p-inputtext-sm" placeholder={`Please type "${config.dropConfirm}" here in order to continue`} value={dropConfirm} onChange={(e) => setDropConfirm(e.target.value)} />
                        <div className="error-help">{<small className="p-error p-d-block">{dropConfirmError}</small>}</div>
                    </div>
                    <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
                    {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}

                </div>
            </div>
        </Dialog>
    );
}


export default DropHAProxyNodeDialog;