import { useState } from "react";
import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { RadioButton } from "primereact/radiobutton";

const ChooseServiceDialog = ({ visible, services, onHide, onNext }) => {

      const [targetService, setTargetService] = useState(services?.[0]);
      const [serviceRadio, setServiceRadio] = useState(services?.[0]?.name);


    const renderFooter = () => {
        return (
            <div className="flex justify-content-between">
                <Button
                    label={"Close"}
                    icon="pi pi-times"
                    onClick={onHide}
                />
                <Button
                    label={"Next"}
                    icon="pi pi-arrow-right"
                    className="ml-auto p-button"
                    onClick={(() => onNext(targetService))} autoFocus />

            </div>
        );

    }


    return (<Dialog
        className='choose-service-dialog'
        header={'Choose service for your installation'}
        footer={renderFooter()}
        visible={visible}
        style={{ width: '50vw', maxHeight: '90%' }}
        onHide={onHide}>
        <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">

            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">




                {services?.length > 0 ?
                    <div>
                        <div className="field">
                            This database can be created on the following services:
                        </div>

                        {
                            services.map(s => (<div key={`serviceRadio${s.id}`} className="field-radiobutton">
                                <RadioButton inputId={`serviceRadio${s.id}`} name="serviceRadio" value={s.name} checked={serviceRadio === s.name} onChange={(e) => {
                                    const service_name = e.value;
                                    setServiceRadio(service_name)
                                    setTargetService(services.filter((s) => s.name === service_name)[0])
                                }} />
                                <label htmlFor={`serviceRadio${s.id}`}>{`${s.name}${s.host_name?.length > 0 ? `@${s.host_name}` : ''} (${s.type})`}</label>
                            </div>))
                        }
                        <div key={`serviceRadio9999`} className="field-radiobutton">
                            <RadioButton inputId={`serviceRadio9999`} name="serviceRadio" value={"#own_vm"} checked={serviceRadio === "#own_vm"} onChange={(e) => {
                                const service_name = e.value;
                                setServiceRadio(service_name);
                                setTargetService(null);
                            }} />
                            <label htmlFor={`serviceRadio999`}>I want to use my own VM</label>
                        </div>
                    </div>

                    : <div className="multiline">{"Found no services on which this database can be created on.\nProceed to create the database on your own VM"}</div>}



            </div>

        </div>
    </Dialog>
    )

}

export default ChooseServiceDialog;
