import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import ApiBar from "../../components/apiBar";
import copyToClipboard from "../../util/copyToClipboard";
import createAPIcommand from "../../util/createApiCommand";
import { useState } from "react";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../components/errorTextBox";


const DeleteInventoryDialog = (props) => {

    const [apiTooltip, setApiTooltip] = useState(false);
    const { accessToken } = useSelector((state) => state.app);
    const {errorMessage} = useSelector((state) => state.layout);

    const onRestApiClick = async () => {

        switch (props.item.type) {

            case "FLASHBACK":
                setApiTooltip(await copyToClipboard(createAPIcommand(
                    {
                        "flashback_id": props.item.name

                    }, accessToken, "drop_flashback/")));
                break;



            case "BACKUP":
                setApiTooltip(await copyToClipboard(createAPIcommand(
                    {

                        "backup_id": props.item.name

                    }, accessToken, "drop_backup/")));
               
                break;

            case "EXPORT":
                setApiTooltip(await copyToClipboard(createAPIcommand(
                    {

                        "backup_id": props.item.name

                    }, accessToken, "drop_backup/")));
                break;

            case "STANDALONE":
                setApiTooltip(await copyToClipboard(createAPIcommand(
                    {

                        "standalone_id": props.item.name

                    }, "drop_standalone/")));
                break;
            default:
                break;

        }

    }

    const handleOk = () => {

        switch (props.item.type) {

            case "FLASHBACK":
                props.onOkClick(

                    {
                        "flashback_id": props.item.name

                    }

                );
                break;    
            case "BACKUP":
                props.onOkClick(

                    {

                        "backup_id": props.item.id

                    }

                );
                break;

            case "EXPORT":
                props.onOkClick(

                    {
                        "backup_id": props.item.id

                    }

                );
                break;

            case "STANDALONE":
                props.onOkClick(

                    {
                        "standalone_id": props.item.id

                    }

                );
                break;


            default:
                break;

        }

    }



    const renderFooter = () => {
        return (
            <div style={{ display: "flex", justifyContent: "space-between" }}>
                <Button label={"No"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    icon="pi pi-trash"
                    className="p-button-danger"
                    label={"Yes"}
                    onClick={handleOk} autoFocus />
            </div>
        );

    }

    const getItemTitle = () => {
        switch (props.item.type) {
            case "FLASHBACK":
                return "flashback";
            case "EXPORT":
                return "export";
            case "BACKUP":
                return "backup";
            case "STANDALONE":
                return "standalone";
            default:
                return "item";
        }

    }



    return (
        <Dialog
            className='drop_inventory'
            header={`Delete ${getItemTitle()}`}
            visible={props.visible}
            style={{ width: '40vw'}}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <div className="field grid">
                        <div className="col">
                            <p>
                                This will delete {props.item.name} {getItemTitle()}.
                            </p>
                            <p>Are you sure?</p>


                        </div>

                    </div>
                    {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}
                    <ApiBar tooltip = {apiTooltip} onClick = {onRestApiClick}/>
                </div>
            </div>

        </Dialog>)

};

export default DeleteInventoryDialog;