import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Password } from "primereact/password";
import { useState } from "react";
import constraints from "../../components/forms/userPasswordForm/constraints";
import validator from "../../util/validator";
import { base64Encode } from '../../util/base64Encode';
import { useSelector } from "react-redux";

export default function EditDatabaseUser(props) {

    const [newPassword, setNewPassword] = useState("");
    const [passwordConfirm, setPasswordConfirm] = useState("");
    const [passwordConfirmError, setPasswordConfirmError] = useState("");
    const [passwordError, setPasswordError] = useState("");
    const {isLoading} = useSelector((state) => state.status)

    const handleOk = () => {
        const errors = [];
        let error = validator(constraints)("password", newPassword);
        (error == null || errors.push(error)) && setPasswordError(error);
        error = validator(constraints)("password2", {"password": newPassword, "password2": passwordConfirm});
        (error == null || errors.push(error)) && setPasswordConfirmError(error);
        if (errors.length === 0)
           props.onOkClick({ "user": props.user?.name, "new_password": base64Encode(newPassword), "node_id":  props.node?.id })
    }

    const renderFooter = () => {
        return (
            <div className="flex justify-content-center">
                <Button
                    label={"Submit"}
                    icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                    onClick={handleOk} autoFocus />
            </div>
        );

    }

    return (<Dialog
        className='edit-node-user'
        header={`Edit user ${props?.user?.name}`}
        footer={renderFooter()}
        visible={props.visible}
        style={{ width: '40vw', maxHeight: '90%' }}
        onHide={props.onHide}>
        <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

                <div className="field grid">
                    <label className="col-fixed"  style={{"width": "200px"}}>{`New password :`}</label>
                    <div className="col">
                        <Password autoComplete="new-password" value={newPassword} onChange={(e) => setNewPassword(e.target.value)} />
                        <div className="p-error text-sm">{passwordError}</div>
                    </div>
                </div>
                <div className="field grid">
                <label className="col-fixed"  style={{"width": "200px"}}>{`Confirm password :`}</label>
                    <div className="col">
                        <Password autoComplete="new-password" value={passwordConfirm} onChange={(e) => setPasswordConfirm(e.target.value)} />
                        <div className="p-error text-sm">{passwordConfirmError}</div>
                    </div>
                </div>
            </div>
        </div>

    </Dialog>
    )

}