import { useState, useEffect, useRef, useCallback } from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Checkbox } from 'primereact/checkbox';
import { Slider } from 'primereact/slider';
import validator from "../../../util/validator";
import { base64Encode } from '../../../util/base64Encode';
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import { Tooltip } from 'primereact/tooltip';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from '../../../components/errorTextBox';
import ApiBar from '../../../components/apiBar';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { types, repositoryOptions, clusterSizeOptions, elasticsearchVersionOptions } from '../../../config';
import getValidationError from '../../../util/errorPrompter';
import { footer } from '../../../util/passwordHelpers';
const AddElacticSearchMasterDialog = ({ visible, onHide, onOkClick }) => {


    const [submitting, setSubmitting] = useState(false);

    const onPopupSubmit = (accept) => onSaveClick(accept);

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();

    // First Page elements
    const [osUsername, setOsUsername] = useState("root");
    const [osPassword, setOsPassword] = useState("");
    const [databaseUsername, setDatabaseUsername] = useState("clusteradmin");
    const [databasePassword, setDatabasePassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState(22);
    const [privateKey, setPrivateKey] = useState("");
    const [softwareVersion, setSoftwareVersion] = useState("8");
    const caller = useRef('submit-buttoon')
    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);

    // Second Page elements
    const [port, setPort] = useState(9200);
    const [transportPort, setTransportPort] = useState(9300);
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);
    const [repository, setRepository] = useState(repositoryOptions[0].value);
    const [dataPath, setDataPath] = useState("/123cluster/elasticsearch/data1");
    const [elasticPwd, setElasticPwd] = useState("");
    const [clusterName, setClusterName] = useState("");

    const submitButtonRef = useRef(null);

    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [elasticPwdError, setElasticPwdError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");

    const [databaseUsernameError, setDatabaseUsernameError] = useState("");
    const [databasePasswordError, setDatabasePasswordError] = useState("");

    //Second page elements errors
    const [portError, setPortError] = useState("");
    const [transportPortError, setTransportPortError] = useState("")
    const [dataPathError, setDataPathError] = useState(false);
    const [clusterNameError, setClusterNameError] = useState("");

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { errorMessage, isLoading, port_exists, path_exists, free_ip } = useSelector((state) => state.layout);



    useEffect(() => {

        if (free_ip && !free_ip?.detail) setHostname(free_ip?.available_ip);

    }, [free_ip])



    useEffect(() => {
        if (visible === false) {
            setPageIndex(0);

            // First page elements and errors
            setPort(9200);
            setPortError("");
            setOsUsername("root");
            setOsUsernameError("");
            setOsPassword("");
            setOsPasswordError("");
            setDatabaseUsername("clusteradmin");
            setDatabaseUsernameError("");
            setDatabasePassword("");
            setDatabasePasswordError("");
            setHostname("");
            setHostnameError("");
            setSshPort(22);
            setSshPortError("");
            setPrivateKey("");
            setPrivateKeyError("");
            setSoftwareVersion("8");

            // Second page elements and errors
            setApiTooltip(" ");
            setPortError("");
            setTransportPort(9300);
            setTransportPortError("");
            setDataPathError("");
            setElasticPwdError("");
            setElasticPwd("");
            setDataPath("/123cluster/elasticsearch/data1");
            setClusterName("");
            setClusterNameError("");
            setRepository(repositoryOptions[0].value);
            setAutomaticDelete(false);
            setAutomaticDeleteDays(1);
        }

        if (typeof visible === 'string') dispatch(LayoutActions.checkFreeIp({ 'service_name': visible.split('@')[1] }));

    }, [visible, dispatch]);

    const validateFirst = () => {

        let errors = [];


        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);


        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }

        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);
        error = validator(constraints)("clusterName", clusterName);
        (error == null || errors.push(error)) && setClusterNameError(error);

        return errors;

    }


    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {

                setPortError("");
                setSubmitting(false);

            }
            else {

                if (port_exists?.exists) {

                    port_exists?.purpose === "transport" ? setTransportPortError("already exists") : setPortError("already exists");
                    setSubmitting(false);
                }
                else {

                    port_exists?.purpose === "transport" ? setTransportPortError("ok") : setPortError("ok");

                }

            }

        }
    }, [port_exists])

    useEffect(() => {

        if (path_exists) {


            if (path_exists?.detail) {

                setDataPathError("");
                setSubmitting(false);

            }

            if (path_exists?.exists) {
                setSubmitting(false)
                setDataPathError("already exists")
            }
            else {

                setDataPathError("ok");

            }

        }

    }, [path_exists])










    const onSaveClick = useCallback(async (dropInventories) => {


        const cluster =
        {

            name: clusterName

        }

        const host =
        {
            name: hostname,
            ssh_port: sshPort,
            username: osUsername,
            password: base64Encode(osPassword),
            private_key: privateKey,
        };

        const ex_params =
        {
            elastic_pwd: base64Encode(elasticPwd),
            cluster_name: clusterName,
            transport_port: transportPort,
            repository: repository
        }

        if (typeof visible === 'string') {
            const proxmox_params = visible.split('@');
            const osInfo = proxmox_params[0].split(' ');
            ex_params['service_name'] = proxmox_params[1]
            ex_params['distributor'] = osInfo[0]
            ex_params['release'] = osInfo[1]
            ex_params['proxmox'] = true

        }

        const data = {
            host,
            cluster,
            ex_params,
            drop_inventories: dropInventories,
            name: `${host['name']}:${port}`,
            auto_delete: automaticDelete,
            version: softwareVersion,
            auto_delete_days: automaticDeleteDays,
            dir_123Cluster: dataPath,
            port: port,
            sizing: clusterSize,
            database_user: databaseUsername,
            database_pwd: base64Encode(databasePassword),
        }
        if (caller.current === 'submit-button') {


            if (onOkClick)
                onOkClick(data);

        }
        else setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_elasticsearch_master/")))

        setSubmitting(false);


    }, [accessToken, automaticDelete, automaticDeleteDays, clusterName, clusterSize, dataPath, databasePassword, databaseUsername,
        elasticPwd, hostname, onOkClick, osPassword, osUsername, port, privateKey, repository, softwareVersion, sshPort, transportPort, visible])

    useEffect(() => {

        const customEvent = new Event('beforeinput', { bubbles: true });

        const validateSecond = () => {

            let errors = [];

            let error = validator(constraints)("elasticPwd", elasticPwd);
            (error == null || errors.push(error)) && setElasticPwdError(error);

            error = validator(constraints)("databaseUsername", databaseUsername);
            (error == null || errors.push(error)) && setDatabaseUsernameError(error);

            error = validator(constraints)("databasePassword", databasePassword);
            (error == null || errors.push(error)) && setDatabasePasswordError(error);

            return errors;
        }

        if (submitting) {


            const errors = validateSecond();

            if (errors.length === 0) {

                if (typeof visible !== 'string') {


                    if (portError !== "ok") {


                        dispatch(LayoutActions.checkPortExists(
                            {
                                host: hostname,
                                port: port,
                                os_username: osUsername,
                                os_password: base64Encode(osPassword),
                                private_key: privateKey,
                                ssh_port: sshPort,
                                purpose: 'http'
                            }))

                    }


                    if (transportPortError !== "ok") {


                        dispatch(LayoutActions.checkPortExists(
                            {
                                host: hostname,
                                port: port,
                                os_username: osUsername,
                                os_password: base64Encode(osPassword),
                                private_key: privateKey,
                                ssh_port: sshPort,
                                purpose: 'transport'
                            }))

                    }
                    if (dataPathError !== "ok") {
                        dispatch(LayoutActions.checkPathExists(
                            {
                                host: hostname,
                                path: dataPath,
                                os_username: osUsername,
                                os_password: base64Encode(osPassword),
                                private_key: privateKey,
                                ssh_port: sshPort,
                            }))
                    }

                }

                if (typeof visible === 'string' || (portError === "ok" && transportPortError === "ok" && dataPathError === "ok")) {

                    dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}:${port}`, 'type': types.ELASTIC }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))


                }




            }
            else caller.current === "submit-button" ? submitButtonRef.current.dispatchEvent(customEvent) : setApiTooltip(getValidationError());

        }

    }, [submitting, dispatch, hostname, onSaveClick, osPassword, osUsername, port, portError, transportPortError, dataPath, privateKey, sshPort, dataPathError, databaseUsername, databasePassword, elasticPwd, visible])


    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            iconPos="right"
                            id='submit-button'
                            className='submit-button'
                            ref={submitButtonRef}
                            onClick={() => {

                                caller.current = 'submit-button'
                                setSubmitting(true);
                            }} autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }
    const renderPageSecond = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Username:"}</label>
                            <div className="col">
                                <InputText className="p-inputtext-sm" placeholder="Database Username" value={databaseUsername} onChange={(e) => setDatabaseUsername(e.target.value)} />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databaseUsernameError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm" placeholder="Database Password"
                                    value={databasePassword}
                                    onChange={(e) => setDatabasePassword(e.target.value)}
                                    footer={() => footer(databasePassword)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databasePasswordError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Software version:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm filter-field"
                                    options={elasticsearchVersionOptions}
                                    value={softwareVersion}
                                    onChange={(e) => {

                                        setSoftwareVersion(e.value);

                                    }}
                                />
                            </div>

                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Data path:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <i className={`pi ${isLoading && !path_exists && path_exists !== undefined ? "pi-spin pi-spinner align-self-center" : "pi-info-circle"} data_path`} />
                                    <Tooltip target=".data_path"
                                        content="This is the directory where the new node files and logs will reside." />

                                    <InputText className="p-inputtext-sm" placeholder="Data path" value={dataPath} onChange={(e) => {
                                        setDataPath(e.target.value);
                                        setDataPathError("check data path");
                                    }} />
                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(dataPathError !== "ok") ? "p-error" : ""} p-d-block`}>{dataPathError}</small>
                                </div>
                            </div>
                          { typeof visible === 'string' ? null :  <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    style={{ width: "80px" }}
                                    label="Check"
                                    onClick={() => {
                                        let error = validator(constraints)("dataPath", dataPath);
                                        setDataPathError(error);

                                        if (error === null) dispatch(LayoutActions.checkPathExists(
                                            {
                                                host: hostname,
                                                path: dataPath,
                                                os_username: osUsername,
                                                os_password: base64Encode(osPassword),
                                                private_key: privateKey,
                                                ssh_port: sshPort,
                                            }))
                                    }}
                                />
                            </div>}
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"HTTP port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="HTTP port" value={port} onValueChange={(e) => {
                                        setPort(e.value);
                                        setPortError("check port");

                                    }} />
                                    <i className={isLoading === "http" && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(portError !== 'ok') ? "p-error" : ""} p-d-block`}>{portError}</small>
                                </div>
                            </div>
                            { typeof visible === 'string' ? null : <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("port", port);
                                        setPortError(error);
                                        if (error === null)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: port,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                    purpose: "http"
                                                }))
                                    }

                                    }
                                />
                            </div> }
                        </div>

                    </div>

                    <div>

                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Transport port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Transport port" value={transportPort} onValueChange={(e) => {
                                        setTransportPort(e.value);
                                        setTransportPortError("check port");

                                    }} />
                                    <i className={isLoading === "transport" && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(transportPortError !== 'ok') ? "p-error" : ""} p-d-block`}>{transportPortError}</small>
                                </div>
                            </div>
                            { typeof visible === 'string' ? null : <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("transportPort", transportPort);
                                        setTransportPortError(error);
                                        if (error === null)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: transportPort,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                    purpose: "transport"
                                                }))
                                    }

                                    }
                                />
                            </div>}
                        </div>

                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Set a user 'elastic' Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm"
                                    placeholder="User 'elastic' Password"
                                    value={elasticPwd}
                                    footer={() => footer(elasticPwd)}
                                    onChange={(e) => setElasticPwd(e.target.value)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{elasticPwdError}</small>
                        </div>
                    </div>

                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Install from:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm"
                                    options={repositoryOptions}
                                    value={repository}
                                    onChange={(e) => setRepository(e.value)}
                                />
                            </div>
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-center">
                            <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "220px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                            <div className="col-fixed">
                                <Checkbox inputId="cb1" value="Automatic delete disabled:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                            </div>
                            <div className="col">
                                <Slider
                                    disabled={!automaticDelete}
                                    min={1}
                                    max={365}
                                    value={automaticDeleteDays}
                                    onChange={(e) => setAutomaticDeleteDays(e.value)} />
                            </div>

                        </div>
                    </div>

                    <div>
                        <div className='field grid'>

                            <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                            <div className='col'>

                                <Dropdown className="p-inputtext-sm"
                                    value={clusterSize}
                                    onChange={(e) => setClusterSize(e.value)}
                                    options={clusterSizeOptions}
                                    placeholder="Size options" />

                            </div>

                        </div>

                    </div>
                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar tooltip={apiTooltip} id='rest-api-bar' onClick={() => {


                    caller.current = 'rest-api-bar'
                    setSubmitting(true);
                }} />
            </div>



        );
    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {

            typeof visible === 'string' ? setPageIndex(1) : dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))

        }
    }

    const onSecondBackClick = () => {
        setPortError("");
        setDataPathError("");
        setTransportPortError("");
        dispatch(LayoutActions.setElasticSearchVersionExists(undefined));
        setPageIndex(0);
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Cluster Name:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Cluster Name" value={clusterName} onChange={(e) => setClusterName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{clusterNameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" disabled={typeof visible === 'string'} placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Hostname/IP" disabled={typeof visible === 'string'} value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false}  className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    return (
        <Dialog
            className='add-elastic-search-master'
            header={"Install a new Elasticsearch Cluster"}
            visible={visible}
            style={{ width: '50vw' }}
            onHide={onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller.current)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>

        </Dialog>
    );
}


export default AddElacticSearchMasterDialog;