import { useSelector, useDispatch } from 'react-redux';
import { useEffect } from 'react';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dialog } from 'primereact/dialog';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import ErrorTextBox from '../../../components/errorTextBox';
import { loadingMsg} from '../../../config';


export default function ElasticSearchStatusDialog(props) {

    const { elasticsearch_cluster_status, isLoading } = useSelector((state) => state.layout);
    const { errorMessage } = useSelector((state) => state.layout);


    const dispatch = useDispatch();

    useEffect(() => {
        if (props.node) {

            dispatch(LayoutActions.getElasticsearchClusterStatus(props.node.id));

        }

    }, [props.node, dispatch])


    const nameBodyTemplate = (item) => {

        return (<div>{item.cluster_name}</div>);

    }

    const statusBodyTemplate = (item) => {

        return (<div>{item.status}</div>);

    }

    const timedOutBodyTemplate = (item) => {

        return (<div>{`${item.timed_out}`}</div>);

    }

    const nodesBodyTemplate = (item) => {

        return (<div>{item.number_of_nodes}</div>);

    }


    const dataNodesBodyTemplate = (item) => {

        return (<div>{item.number_of_data_nodes}</div>);

    }

    return (

        <Dialog
            className='elasticsearch-cluster-status'
            header={`Cluster ${props.node.name} status`}
            visible={props.visible}
            style={{ maxHeight: "90%" }}
            onHide={props.onHide}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <DataTable emptyMessage={!elasticsearch_cluster_status && isLoading ? <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">No status data</div>} paginator = {false} value={elasticsearch_cluster_status?.length ? elasticsearch_cluster_status : []} >
                        <Column field="cluster_name" header="Name" body={nameBodyTemplate} sortable />
                        <Column field="status" header="Status" body={statusBodyTemplate} sortable />                       
                        <Column field="timed_out" header="Timed out" body={timedOutBodyTemplate} sortable />
                        <Column field="number_of_nodes" header="Nodes count" body={nodesBodyTemplate} sortable />
                        <Column field="pingMs" header="Data nodes count" body={dataNodesBodyTemplate} sortable />
                    </DataTable>
                    {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}

                </div>
            </div>

        </Dialog>);
}