import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { useState, useEffect } from "react";
import { InputText } from "primereact/inputtext";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";

export const SwitchElasticsearchNodeDialog = (props) => {

  const [targetNode, setTargetNode] = useState(null);
  const [nodeOptions, setNodeOptions] = useState([]);
  const [switchConfirm, setSwitchConfirm] = useState("");
  const [switchConfirmError, setSwitchConfirmError] = useState("");
  const [targetNodeError, setTargetNodeError] = useState("");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);



  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {

      props.onOkClick(
        {
          "standby_id": targetNode?.id
        });

    }

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("targetNode", targetNode);
    (error == null || errors.push(error)) && setTargetNodeError(error);

    error = validator(constraints)("switchConfirm", switchConfirm);
    (error == null || errors.push(error)) && setSwitchConfirmError(error);

    return errors;

  }

  const onRestApiClick = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setApiTooltip(await copyToClipboard(createAPIcommand({
        "standby_id": targetNode?.name

      }, accessToken, "switch_elasticsearch_node/")));
    }
    else setApiTooltip(" ");

  }

  useEffect(() => {

    if (props.nodes) {

      const temp = [];
      props.nodes.forEach((node) => temp.push({ label: node.name, value: node }));
      setNodeOptions(temp)

    }

  }, [props.nodes]);

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='swichover-pg-node'
      header={`Switchover`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="title">{"Choose node to switch:"}</div>

              <div className="field">
                <Dropdown className="p-inputtext-sm"
                  value={targetNode}
                  options={nodeOptions}
                  onChange={(e) => setTargetNode(e.value)}
                  placeholder="Select node" />
                <div className="error-help">{<small className="p-error p-d-block">{targetNodeError}</small>}</div>
              </div>
              <div className="field grid flex align-items-center">
                <div className="col">

                  <InputText
                    className="p-inputtext-sm"
                    placeholder='Please type "switch me" here in order to continue'
                    value={switchConfirm}
                    onChange={(e) => setSwitchConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{switchConfirmError}</small>}</div>

                </div>
              </div>
            </div>

          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>
      </div>

    </Dialog>)

};

export default SwitchElasticsearchNodeDialog;