import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useNavigate } from "react-router-dom";
import { useDispatch } from "react-redux";
import { useRef } from "react";
import { LogActions } from "../../store/log/reducer";


const InfoDialog =  ({visible, onHide, header, message, description = 'database', icon}) => 
{
  const navigate = useNavigate();
  const dispatch = useDispatch();
  const db_cred = useRef(<div className="multiline">{`As part of the "${visible?.task}" process, the ${description} usernames and passwords will be temporarily logged.\nThese credentials will appear in the URL within the "${visible?.task}" task log under the "creating database users" step. You can view these logs at\n\n`}  <button className="button-link" onClick={() => {
    dispatch(LogActions.setLogs(null));
    navigate('/log', {state: {"target_log": visible?.task}});
}}>{`${window.location.origin}/log`}</button> {`.\n\n Rest assured, your credentials are securely stored and will be automatically deleted once the ${description} setup is complete.\n\nIf you have any questions or concerns about the security of your credentials, feel free to reach out to the system administrator.`}</div>)
  const getMessage = () => {

    
    switch(message) {


       case 'db_cred':
        return db_cred.current;
        default:
          return <div className="multiline">{message.join ? message.join("\n") : message}</div>;
    }


  }

    const renderFooter = () => {
        return (
          <div className="flex justify-content-center">
            <Button
              label={"OK"}
              icon="pi pi-check"
              className="p-button-info"
              onClick={onHide} autoFocus />
          </div>
        );
    
      }
  
  return (<Dialog
    className='p-confirm-dialog info-dialog'
    header={<div className="flex justify-content-center"> {icon ? <i className={`${icon} text-xl mr-2`}/> : null} { header ? header : message === 'db_cred' ? `Important Notice: ${description} User Creation.` : 'Info'} </div>}
    footer={renderFooter()}
    dismissableMask
    visible={Boolean(visible)}
    icon
    style={{ width: '40vw', maxHeight: '90%'}}
    onHide={onHide}>
    <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
    <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

    <div className="field">
                        {getMessage()}
                    </div>
        </div>
        </div>

        </Dialog>
)

}

export default InfoDialog;