import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { useState } from "react";
import constraints from "./constraints";
import validator from "../../../util/validator";
import { useSelector } from "react-redux";

export default function AddLicenseDialog(props) {

    const [key, setKey] = useState("");
    const [keyError, setKeyError] = useState("");
    const {isLoading} = useSelector((state) => state.status)

    const handleOk = () => {
        const errors = [];
        let error = validator(constraints)("key", key);
        (error == null || errors.push(error)) && setKeyError(error);
        if (errors.length === 0)
           props.onOkClick({"key": key})
    }

    const renderFooter = () => {
        return (
            <div className="flex justify-content-between">
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    label={"Submit"}
                    icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                    onClick={handleOk} autoFocus />
            </div>
        );

    }

    return (<Dialog
        className='add-license-key'
        header={`Add license key`}
        footer={renderFooter()}
        visible={props.visible}
        style={{ width: '40vw', maxHeight: '90%' }}
        onHide={props.onHide}>
        <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

                <div className="field grid">
                    <label className="col-fixed"  style={{"width": "200px"}}>{`Key:`}</label>
                    <div className="col">
                        <InputText placeholder="Key" value={key} onChange={(e) => setKey(e.target.value)} />
                        <div className="p-error text-sm">{keyError}</div>
                    </div>
                </div>
            </div>
        </div>

    </Dialog>
    )

}