import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { useState, useEffect, useRef } from "react";
import { InputText } from "primereact/inputtext";
import validator from "../../../util/validator";
import constraints from "./constraints";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";
import { moveLicense } from "../../../config/licensing";
import { useDispatch } from "react-redux";
import { StatusActions } from "../../../store/status/reducer";
import { ConfirmPopup } from "primereact/confirmpopup";
import { typesVerbose } from "../../../config/licensing";
export const MoveLicenseDialog = (props) => {
  const [sourceCluster, setSourceCluster] = useState(null);
  const [clusterOptions, setClusterOptions] = useState([]);
  const [moveConfirm, setMoveConfirm] = useState("");
  const [moveConfirmError, setMoveConfirmError] = useState("");
  const [sourceClusterError, setSourceClusterError] = useState("");
  const {licensed_clusters } = useSelector((state) => state.status);
  const {errorMessage, isLoading} = useSelector((state) => state.layout);
  const [showConfirm, setShowConfirm] = useState(false);
  const submitRef = useRef(null);
  const dispatch = useDispatch();
  const handleOk = () => {
    let errors = validate();
    if (errors.length === 0) setShowConfirm(true);
}
  useEffect(() => {
    if (licensed_clusters?.length > 0)
    {
      const tempNodes = []
      licensed_clusters?.forEach((n) => tempNodes.push({label: n?.name, value:n}))
      setClusterOptions(tempNodes);
    }
  } ,  [licensed_clusters])
  const validate = () => {
    let errors = [];
    let error = validator(constraints)("sourceCluster", sourceCluster);
    (error == null || errors.push(error)) && setSourceClusterError(error);
    error = validator(constraints)("moveConfirm", moveConfirm);
    (error == null || errors.push(error)) && setMoveConfirmError(error);
    return errors;
  }
  useEffect(() => {
    if (props.visible && props.node) dispatch(StatusActions.getLicensedClusters({"cluster_id": props.node.cluster, "type": props.node.type} ))
  }, [props.visible, props.node, dispatch]);
  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          ref={submitRef}
          label={"Submit"}
          id = "move-license"
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );
  }
  return (
    <Dialog
      className='move-license'
      header={`Move license to cluster of ${props.node?.name}`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="title">Choose source cluster to move license from:</div>
              <div className="field">
                <Dropdown className="p-inputtext-sm"
                  value={sourceCluster}
                  options={clusterOptions}
                  onChange={(e) => setSourceCluster(e.value)}
                  placeholder="Select cluster" />
                <div className="error-help">{<small className="p-error p-d-block">{sourceClusterError}</small>}</div>
              </div>
              <div className="field grid flex align-items-center">
                <div className="col">
                  <InputText
                    className="p-inputtext-sm"
                    placeholder={`Please type "${moveLicense}" here in order to continue`}
                    value={moveConfirm}
                    onChange={(e) => setMoveConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{moveConfirmError}</small>}</div>
                </div>
              </div>
            </div>
          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ConfirmPopup icon="pi pi-exclamation-triangle" message={`Please note that this action will allocate a license of ${typesVerbose[sourceCluster?.type]} node ${sourceCluster?.name} (${sourceCluster?.id}) to ${typesVerbose[props.node?.type]}  node ${props.node?.name}(${props.node?.id}) `} visible={showConfirm} acceptLabel="ok" rejectLabel ="cancel" acceptIcon="pi pi-check" rejectIcon="pi pi-times" target={submitRef?.current} reject={() => setShowConfirm(false)} accept={() => { 
            setShowConfirm(false);
            props.onOkClick(
              {
                "source_id": sourceCluster?.id,
                "target_id": props.node?.id,
            
              })
          }} />
        </div>
      </div>
    </Dialog>)
};
export default MoveLicenseDialog;