import { useState } from 'react';
import { InputText } from "primereact/inputtext";
import { Button } from "primereact/button";
import { Dialog } from "primereact/dialog";
import { useSelector } from 'react-redux';
import copyToClipboard from '../../../util/copyToClipboard';
import createAPIcommand from '../../../util/createApiCommand';
import ApiBar from '../../../components/apiBar';
import ErrorTextBox from '../../../components/errorTextBox';
import config from '../../../config';
import validator from '../../../util/validator';
import constraints from './constraints';
import { Checkbox } from 'primereact/checkbox';

const MariaDBDropClusterDialog = (props) => {

    const [dropConfirm, setDropConfirm] = useState("");
    const { accessToken } = useSelector((state) => state.app);
    const { errorMessage, isLoading } = useSelector((state => state.layout));
    const [apiTooltip, setApiTooltip] = useState(" ");
    const [dropConfirmError, setDropConfirmError] = useState("");
    const [finalBackup, setFinalBackup] = useState(true);

    const handleOk = () => props.onOkClick({
        "node_id": props.node?.id,
        "type": props.node?.type,
        "final_backup": finalBackup
    });

    const validate = () => {

        let errors = [];

        let error = validator(constraints)("dropConfirm", dropConfirm);
        (error == null || errors.push(error)) && setDropConfirmError(error);
        return errors;

    }



    const onRestApiClick = async () => {

        let errors = validate();

        if (errors.length === 0) {

            setApiTooltip(await copyToClipboard(createAPIcommand(
                {

                    "node_id": props.node?.name,
                    "type": props.node?.type,
                    "final_backup": finalBackup



                }
                , accessToken, 'drop_mariadb_cluster/')));


        }
        else setApiTooltip(" ");

    }


    const renderFooter = () => {
        return (
            <div className="flex justify-content-between">
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    label={"Submit"}
                    id='submit-button'
                    icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'trash'}`}
                    className="p-button-danger"
                    onClick={handleOk} autoFocus />
            </div>
        );
    }


    return (
        <Dialog
            className='drop-cluster'
            header={`Drop cluster ${props.node?.cluster_content?.name}`}
            visible={props.visible}
            style={{ width: '40vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

                    <div className='form-inner-part border-none'>

                        <div className='multiline' style={{ marginBottom: "10px" }}>
                            {`This will drop all the nodes under cluster ${props.node?.cluster_content?.name}.\n
                            ${props.node?.name} (primary)\n
                            ${props?.node?.nodes ? props?.node?.nodes?.map((s) => `${s?.name} (standby)\n`) : ''}
                            `}
                        </div>
                        <div>
                            Are you sure?
                        </div>
                        <div>
                            Please consider to backup this cluster.
                        </div>

                    </div>

                    <div className='form-inner-part border-none'>
                        <div className="field">
                            <div style={{ marginBottom: "10px" }} className="title">{`Please type "${config.dropConfirm}" here in order to continue`}</div>
                            <InputText className="p-inputtext-sm" placeholder={`Please type "${config.dropConfirm}" here in order to continue`} value={dropConfirm} onChange={(e) => setDropConfirm(e.target.value)} />
                            <div className="error-help">{<small className="p-error p-d-block">{dropConfirmError}</small>}</div>
                            <div className='flex'>

                                <div className='ml-auto'>
                                    <Checkbox inputId='final-backup' className='mr-2' checked={finalBackup} value={finalBackup} onChange={() => setFinalBackup(!finalBackup)} />
                                    <label htmlFor='final-backup' className="p-checkbox-label" >run final backup on drop</label>
                                </div>

                            </div>

                        </div>
                    </div>

                    {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                    <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
                </div>
            </div>
        </Dialog>
    );
}

export default MariaDBDropClusterDialog;