import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Checkbox } from 'primereact/checkbox';
import { useState, useEffect, useRef } from "react";
import { InputText } from "primereact/inputtext";
import { TabView, TabPanel } from 'primereact/tabview';
import { Calendar } from 'primereact/calendar';
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector, useDispatch } from "react-redux";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import { Tooltip } from "primereact/tooltip";
import config, { daysOfWeekOptions } from "../../../config";
import { ConfigActions } from "../../../store/config/reducer";
import { LayoutActions } from "../../../store/layout/reducer";
import ErrorTextBox from "../../../components/errorTextBox";
import { ConfirmPopup } from 'primereact/confirmpopup';
import { MultiSelect } from "primereact/multiselect";
import { getMinDate, getMaxDate } from "../../../util/daysofmonth";
import moment from "moment";
import notify from "../../../util/notify";


export const ExportMariaDBDatabase = (props) => {

  const [name, setName] = useState(`${props.db.name}_${moment(new Date().toUTCString()).format("DDMMYYYYhhmmss")}`);
  const [nameError, setNameError] = useState("");
  const [includeData, setIncludeData] = useState(true);
  const [includeDataSchedule, setIncludeDataSchedule] = useState(true);
  const [dirCheck, setDirCheck] = useState("");
  const [diskSpaceCheck, setDiskSpaceCheck] = useState("");
  const [exportCheck, setExportCheck] = useState("");
  const [exportDisabled, setExportDisabled] = useState(true);
  const [fullPath, setFullPath] = useState("");
  const [pageIndex, setPageIndex] = useState(0);
  const { isLoading, mariadb_db_size, dir_exists, disk_space, export_exists, errorMessage } = useSelector((state) => state.layout);
  const { config: appConf } = useSelector((state) => state.config);
  const dispatch = useDispatch();
  const [scheduleApiTooltip, setScheduleApiTooltip] = useState(" ");
  const [nowApiTooltip, setNowApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const [zipped, setZipped] = useState(false);
  const [scheduleZipped, setScheduleZipped] = useState(false);
  const [showWarningPopup, setShowWarningPopup] = useState(false)

  const [time, setTime] = useState(new Date());
  const [toggle, setToggle] = useState(0);
  const [timeError, setTimeError] = useState("");

  const [daysOfMonth, setDaysOfMonth] = useState(null);
  const [daysOfWeek, setDaysOfWeek] = useState(null);
  const minDate = useRef(getMinDate());
  const maxDate = useRef(getMaxDate());

  useEffect(() => {

    dispatch(LayoutActions.checkMariaDbDatabaseSize(
      {

        "node_id": props.db.node,
        "db_name": props.db.name,

      }));

  }, [props.db, dispatch])


  useEffect(() => {



    if (!appConf) dispatch(ConfigActions.getConfig());
    else if (!fullPath)
      setFullPath(appConf?.mariadb_sys?.exports_path)


  }, [appConf, fullPath, dispatch])

  useEffect(() => {

    if (export_exists) {

      if (!export_exists?.detail) {

        setExportCheck("ok");

      }
      else setExportCheck(export_exists.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);
    }

  }, [export_exists, dir_exists, disk_space]);

  useEffect(() => {

    if (dir_exists) {
      if (!dir_exists?.detail) setDirCheck("ok");
      else setDirCheck(dir_exists.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);

    }

  }, [dir_exists, export_exists, disk_space]);



  useEffect(() => {

    if (disk_space) {
      if (!disk_space?.detail) setDiskSpaceCheck("space available");
      else setDiskSpaceCheck(disk_space.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);
    }

  }, [disk_space, dir_exists, export_exists]);

  const onRestApi1Click = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setNowApiTooltip(await copyToClipboard(createAPIcommand(
        {
          "db_id": props.db.name,
          "node_id": props.db.host,
          "schedule": false,
          "compress": zipped,
          "include_data": includeData,
          "export_name": name,
          "export_path": fullPath,
          "keep_days": props.db.size_scheme?.export,


        }
        , accessToken, 'export_mariadb_database/')))

    }
    else setNowApiTooltip(" ");
  }

  const onRestApi2Click = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setScheduleApiTooltip(await copyToClipboard(createAPIcommand(
        {
          "db_id": props.db.name,
          "node_id": props.db.host,
          "schedule": true,
          "compress": scheduleZipped,
          "include_data": includeData,
          "keep_days": props.db.size_scheme?.export,
          "export_path": fullPath,
          "export_name": name,
          "schedule_run_time": time,
          "client_timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
          "days_of_week": daysOfWeek,
          "days_of_month": daysOfMonth,


        }
        , accessToken, 'export_mariadb_database/')))

    }
    else setScheduleApiTooltip(" ");

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("name", name);
    (error == null || errors.push(error)) && setNameError(error);

    
    if (toggle === 1) {

      error = validator(constraints)("time", time);
      (error == null || errors.push(error)) && setTimeError(error);
    }


    return errors;

  }

  const handleOk = () => {

    if (mariadb_db_size?.size > config.GB) setShowWarningPopup(true);
    else submit();


  }


  const submit = () => {

    if (mariadb_db_size?.size > mariadb_db_size?.free_space) {
      notify.warning("Free space at the selected directory is less than the schema size. Please select another directory")
    }

    else {
      if (toggle < 1) {

        props.onOkClick(
          {
            "db_id": props.db.id,
            "schedule": false,
            "export_name": name,
            "compress": zipped,
            "keep_days": props.db.size_scheme?.export,
            "include_data": includeData,
            "export_path": fullPath,


          });
      }
      else {

        props.onOkClick(
          {
            "db_id": props.db.id,
            "schedule": true,
            "export_name": name,
            "compress": scheduleZipped,
            "keep_days": props.db.size_scheme?.export,
            "include_data": includeDataSchedule,
            "export_path": fullPath,
            "schedule_run_time": time,
            "client_timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
            "days_of_week": daysOfWeek,
            "days_of_month": daysOfMonth,
          });

      }
    }

  }


  const renderCheckPage = () => {

    return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

      <div className="field grid">
        <label className="col-fixed">{`Checking directory :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : dirCheck}
        </div>
      </div>

      <div className="field grid">
        <label className="col-fixed">{`Checking available space :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : diskSpaceCheck}
        </div>
      </div>

      <div className="field grid">
        <label className="col-fixed">{`Checking export file name :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : exportCheck}
        </div>
      </div>

      {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
      <ApiBar tooltip={toggle < 1 ? nowApiTooltip : scheduleApiTooltip} onClick={toggle < 1 ? onRestApi1Click : onRestApi2Click} />




    </div>);

  }

  const renderExportPage = () => {

    return (<TabView className="no-ink" activeIndex={toggle} onTabChange={(e) => {
      dispatch(LayoutActions.setErrorMessage(""));
      setToggle(e.index);
    }}>
      <TabPanel header="Export now">
        <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
          <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

            <div className="field grid">
              <label className="col-fixed" style={{ width: "165px" }}>{"Name:"}</label>
              <div className="col">
                <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => setName(e.target.value)} />
                <div className="error-help"><small className="p-error p-d-block">{nameError}</small></div>
              </div>
            </div>
            <div className="field grid">
              <div className="col-fixed" style={{ width: "165px" }}>Including data:</div>
              <div className="col">
                <Checkbox checked={includeData} onChange={(e) => setIncludeData(e.checked)} />
              </div>
            </div>


            <div className="field grid">
              <label className="col-fixed" style={{ width: "165px" }}>Compress:</label>
              <div className="col">

                <Checkbox
                  onChange={e => setZipped(e.checked)}
                  checked={zipped}>
                </Checkbox>
              </div>
            </div>



            <div className="field grid">
              <label id="pg-export-size-label" className="col-fixed" style={{ width: "165px" }}>{"Size:"}</label>
              <div className="col">
                {isLoading ? <i className="pi pi-spin pi-spinner" /> : mariadb_db_size?.display_size}
              </div>
            </div>

            <div className="field grid align-items-end">
              <div className="col-fixed"> Export will be kept for {props.db.size_scheme ? props.db.size_scheme?.export : "unk"} days </div>
              <div className="col-fixed">

                <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                <i className="pi pi-info-circle config-field icon-field" />

              </div>
            </div>
            {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          </div>
        </div>
      </TabPanel>
      <TabPanel header="Scheduled export">
        <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
          <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
            <div className="form-inner-part">

              <div className="grid">
                <label className="col-fixed multiline" style={{ width: "165px" }}>{`Days of month:\nor`}</label>
                <div className="col">
                  <Calendar viewDate={minDate.current}  readOnlyInput selectionMode="multiple" dateFormat="d" minDate={minDate.current} maxDate={maxDate.current} showOtherMonths={false} className="p-inputtext-sm" panelClassName="days-of-month-picker" value={daysOfMonth} onChange={(e) =>
                    setDaysOfMonth(e.value)} showTime={false} />
                </div>
              </div>

              <div className="grid">
                <label className="col-fixed" style={{ width: "165px" }}>Days of week:</label>
                <div className="col">
                  <MultiSelect scrollHeight="100%" className="p-inputtext-sm" maxSelectedLabels={6} selectedItemsLabel="All weekdays" selectAllLabel="All weekdays" options={daysOfWeekOptions} value={daysOfWeek} onChange={(e) => setDaysOfWeek(e.value)} />

                </div>
              </div>

            </div>

            <div className="field grid">
              <label className="col-fixed" style={{ width: "165px" }}>Time:</label>
              <div className="col">
                <Calendar showSeconds className="p-inputtext-sm" value={time} onChange={(e) => setTime(e.value)} timeOnly />
                <div className="error-help"><small className="p-error p-d-block">{timeError}</small> </div>
              </div>
            </div>



            <div className="field grid">
              <label className="col-fixed" style={{ width: "165px" }}>{"Name:"}</label>
              <div className="col">
                <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => setName(e.target.value)} />
                <div className="error-help"><small className="p-error p-d-block">{nameError}</small></div>
              </div>
            </div>

            <div className="field grid">
              <div className="col-fixed" style={{ width: "165px" }}>Including data:</div>
              <div className="col">
                <Checkbox checked={includeDataSchedule} onChange={(e) => setIncludeDataSchedule(e.checked)} />
              </div>
            </div>


            <div className="field grid">
              <label style={{ width: "165px" }} className="col-fixed">Compress:</label>
              <div className="col">

                <Checkbox
                  onChange={e => setScheduleZipped(e.checked)}
                  checked={scheduleZipped}>
                </Checkbox>
              </div>
            </div>

            <div className="field grid">
              <label id="pg-export-size-label" className="col-fixed" style={{ width: "165px" }}>{"Size:"}</label>
              <div className="col">
                {isLoading ? <i className="pi pi-spin pi-spinner" /> : mariadb_db_size?.display_size}
              </div>
            </div>

            <div className="field grid align-items-end">
              <div className="col-fixed"> Keep export for {props.db.size_scheme ? props.db.size_scheme?.export : "unk"} days </div>
              <div className="col-fixed">

                <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                <i className="pi pi-info-circle config-field icon-field" />

              </div>
            </div>
            {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          </div>
        </div>
      </TabPanel>
    </TabView>)

  }


  const renderFooter = () => {

    if (pageIndex === 1) {

      return (<div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button label={"Back"}
          onClick={() => {
            dispatch(LayoutActions.setErrorMessage(""));
            dispatch(LayoutActions.setDirExists(null));
            dispatch(LayoutActions.setExportExists(null));
            dispatch(LayoutActions.setDiskSpace(null));
            setPageIndex(0);
          }}
          className="p-button-secondary"
          icon="pi pi-arrow-left"
          iconPos="left"
        />
        <Button
          label={"Submit"}
          icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          id="export-mariadb-db-confirm"
          iconPos="right"
          disabled={exportDisabled}
          onClick={handleOk} autoFocus />
      </div>)

    }
    else if (pageIndex === 0) {

      return (<div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Next"}
          icon={`pi pi-${isLoading ? 'pi-spin pi-spinner' : 'arrow-right'}`}
          disabled={isLoading || errorMessage}
          iconPos="right"
          onClick={() => {


            let errors = validate();

            if (errors.length === 0) {


              dispatch(LayoutActions.checkExportExists(
                {
                  "path": fullPath,
                  "obj_type": "MARIA_DATABASE",
                  "node_id": props.db.id,
                }

              ));

              dispatch(LayoutActions.checkDirExists(
                {
                  "path": fullPath,
                  "obj_type": "MARIA_DATABASE",
                  "node_id": props.db.id,
                }

              ));

              dispatch(LayoutActions.checkDiskSpace(
                {
                  "path": fullPath,
                  "obj_type": "MARIA_DATABASE",
                  "node_id": props.db.id,
                  "required_space": mariadb_db_size?.size,
                }
              ));


              setPageIndex(1);
            }

          }} autoFocus />
      </div>)
    }
  }



  return (
    <Dialog
      className='export-mariadb-db'
      id="export-mariadb-db"
      header={`Export ${props.db.name}`}
      visible={props.visible}
      style={{ width: '40vw', maxHeight: "90%" }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        {pageIndex === 0 && renderExportPage()}
        {pageIndex === 1 && renderCheckPage()}
        <ConfirmPopup
          className="multiline"
          icon="pi pi-exclamation-triangle"
          target={document.getElementById('export-mariadb-db-confirm')}
          visible={showWarningPopup}
          onHide={() => setShowWarningPopup(false)}
          reject={props.onHide}
          accept={submit}
          message={"The selected export size exceeds 1GB.\nThis can take a long time.Continue?"} />
      </div>

    </Dialog>)

};

export default ExportMariaDBDatabase;
