import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector } from "react-redux";
import moment from "moment";
import { Tag } from "primereact/tag";
import ErrorTextBox from "../../../components/errorTextBox";
import config from "../../../config";
import validator from "../../../util/validator";
import constraints from "./constraints";


const RestoreMariaDBDatabaseLayout = (props) => {

  const [restoreConfirm, setRestoreConfirm] = useState("");
  const [restoreConfirmError, setRestoreConfirmError] = useState("");

  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);



  const onRestApiClick = async () => {


    let errors = validate();

    if (errors.length === 0) {

      setApiTooltip(await copyToClipboard(createAPIcommand({
        "export_id": props.export.name,
      }, accessToken, "restore_mariadb_database/")));

    }
    else setApiTooltip(" ");

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("restoreConfirm", restoreConfirm);
    (error == null || errors.push(error)) && setRestoreConfirmError(error);

    return errors;

  }

  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {

      props.onOkClick(

        {
          "export_id": props.export.id,
        }

      );

    }


  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='restore-mariadb-database-export'
      header={`Restore DB export`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="field">
                This will restore the database {props.export.node_name} to the export, taken at {moment(props.export.created_date).format("MMM DD,YYYY, hh:mm A")}
              </div>
              <div className="field">
                <Tag value="The current content of the database will be overridden, this operation is not reversible!" icon="pi pi-exclamation-triangle" severity="warning" />
              </div>

              <div className="field grid flex align-items-center">
                <div className="col">
                <InputText
                  className="p-inputtext-sm"
                  placeholder={`Please type "${config.restoreConfirm}" here in order to continue`}
                  value={restoreConfirm}
                  onChange={(e) => setRestoreConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{restoreConfirmError}</small>}</div>
                </div>
              </div>
              
            </div>

          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />

        </div>
      </div>

    </Dialog>)

};

export default RestoreMariaDBDatabaseLayout;