import { useState, useEffect, useRef } from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Checkbox } from 'primereact/checkbox';
import { Slider } from 'primereact/slider';
import validator from "../../../util/validator";
import constraints from "./constraints";
import { base64Encode } from '../../../util/base64Encode';
import { LayoutActions } from '../../../store/layout/reducer';
import { Tooltip } from 'primereact/tooltip';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from '../../../components/errorTextBox';
import ApiBar from '../../../components/apiBar';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { types, repositoryOptions, mongoVersionOptions } from '../../../config';
import getValidationError from '../../../util/errorPrompter';
import { footer } from '../../../util/passwordHelpers';

const AddMongoShardedClusterDialog = (props) => {

    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [caller, setCaller] = useState('submit-button');

    const onPopupSubmit = (accept) => caller === 'rest-api-bar' ? onRestApiClick(accept) : onSaveClick(accept);

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();

    // First Page elements
    const [osUsername, setOsUsername] = useState("root");
    const [osPassword, setOsPassword] = useState("");
    const [databaseUsername, setDatabaseUsername] = useState("clusteradmin");
    const [databasePassword, setDatabasePassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState(22);
    const [privateKey, setPrivateKey] = useState("");
    const [softwareVersion, setSoftwareVersion] = useState("8.0");
    const [clusterName, setClusterName] = useState("");
    const [clusterNameError, setClusterNameError] = useState("");

    // Second Page elements
    const [port, setPort] = useState(27018);
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const [repository, setRepository] = useState(repositoryOptions[0].value);
    const [ssl, setSSL] = useState(false);
    const [dataPath, setDataPath] = useState("/123cluster/mongo/data1");
    const customEvent = new Event('beforeinput', { bubbles: true });
    const submitButtonRef = useRef(null);
    const [mongosPort, setMongosPort] = useState(27019);

    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");

    const [databaseUsernameError, setDatabaseUsernameError] = useState("");
    const [databasePasswordError, setDatabasePasswordError] = useState("");

    //Second page elements errors
    const [portError, setPortError] = useState("");
    const [mongosPortError, setMongosPortError] = useState("");
    const [dataPathError, setDataPathError] = useState(false);

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { errorMessage, isLoading, port_exists, path_exists } = useSelector((state) => state.layout);

    useEffect(() => {
        if (props.visible === false) {

            setPageIndex(0);

            // First page elements and errors
            setPort(27018);
            setMongosPort(27019);
            setMongosPortError("");
            setPortError("");
            setOsUsername("root");
            setOsUsernameError("");
            setOsPassword("");
            setOsPasswordError("");
            setDatabaseUsername("clusteradmin");
            setDatabaseUsernameError("");
            setDatabasePassword("");
            setDatabasePasswordError("");
            setHostname("");
            setHostnameError("");
            setSshPort(22);
            setSshPortError("");
            setPrivateKey("");
            setPrivateKeyError("");
            setSoftwareVersion("8.0");
            setClusterName("");
            // Second page elements and errors
            setApiTooltip(" ");
            setPortError("");
            setDataPathError("");
            setDataPath("/123cluster/mongo/data1");
            setRepository(repositoryOptions[0].value);
            setAutomaticDelete(false);
            setAutomaticDeleteDays(1);
            setCaller('submit-button');
        }

    }, [props.visible]);

    const onRestApiClick = async (dropInventories) => {
        if (portError === "ok" && dataPathError === "ok" && mongosPortError === "ok") {
            let errors = validateSecond();
            if (errors.length === 0) {

                const cluster =
                {

                    name: clusterName

                }

                const host =
                {
                    name: hostname,
                    ssh_port: sshPort,
                    username: osUsername,
                    password: base64Encode(osPassword),
                    private_key: privateKey,
                };

                const ex_params =
                {
                    replicaset_name: clusterName,
                    repository: repository,
                    ssl: ssl,
                    mongos_port: mongosPort
                }

                const data = {
                    host,
                    cluster,
                    ex_params,
                    name: `${hostname}:${port}`,
                    auto_delete: automaticDelete,
                    version: softwareVersion,
                    auto_delete_days: automaticDeleteDays,
                    port: port,
                    dir_123Cluster: dataPath,
                    drop_inventories: dropInventories,
                    database_user: databaseUsername,
                    database_pwd: base64Encode(databasePassword),
                }
                setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "add_mongo_sharded_cluster/")))



            }
            else setApiTooltip(getValidationError());
        }
        else setApiTooltip(getValidationError());
        if (!portError) setPortError("check port");
        if (!dataPathError) setDataPathError("check data path");
    }

    const validateFirst = () => {

        let errors = [];


        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);


        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }


        error = validator(constraints)("clusterName", clusterName);
        (error == null || errors.push(error)) && setClusterNameError(error);

        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        return errors;

    }

    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {

                setPortError("");

            }
            else {

                if (port_exists?.exists) {

                    port_exists?.purpose === "mongos" ? setMongosPortError("already exists") : setPortError("already exists");
                }
                else {

                    port_exists?.purpose === "mongos" ? setMongosPortError("ok") : setPortError("ok");

                }

            }

        }
    }, [port_exists])

    useEffect(() => {

        if (path_exists) {

            if (path_exists?.exists) {


                setDataPathError("already exists")
            }
            else {

                setDataPathError("ok");

            }

        }

    }, [path_exists])



    const onSaveClick = (dropInventories) => {

        if (portError === "ok" && dataPathError === "ok" && mongosPortError === "ok") {
            const errors = validateSecond();
            if (errors.length === 0) {

                const cluster =
                {

                    name: clusterName,

                    ex_params: {

                        database_user: databaseUsername,
                        database_pwd: base64Encode(databasePassword),
                        repository: repository,
                        ssl: ssl,
                        version: softwareVersion,
                    }

                }

                const host =
                {
                    name: hostname,
                    ssh_port: sshPort,
                    username: osUsername,
                    password: base64Encode(osPassword),
                    private_key: privateKey,
                };

                const ex_params =
                {
                    replicaset_name: clusterName,
                    repository: repository,
                    ssl: ssl,
                    mongos_port: mongosPort
                }

                const data = {
                    host,
                    cluster,
                    ex_params,
                    name: `${hostname}:${port}`,
                    auto_delete: automaticDelete,
                    version: softwareVersion,
                    auto_delete_days: automaticDeleteDays,
                    port: port,
                    dir_123Cluster: dataPath,
                    drop_inventories: dropInventories,
                    database_user: databaseUsername,
                    database_pwd: base64Encode(databasePassword)
                }

                if (props.onOkClick)
                    props.onOkClick(data);

            }
            else submitButtonRef.current.dispatchEvent(customEvent);

        }
        else submitButtonRef.current.dispatchEvent(customEvent);
        if (!portError) setPortError("check port");
        if (!dataPathError) setDataPathError("check data path");


    }

    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            iconPos="right"
                            id='submit-button'
                            className='submit-button'
                            ref={submitButtonRef}
                            onClick={() => {
                                const errors = validateSecond();
                                if (portError === "ok" && dataPathError === "ok" && mongosPortError === "ok" && errors.length === 0) {

                                    setCaller('submit-button');
                                    dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}:${port}`, 'type': types.MONGO }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))

                                }
                                else submitButtonRef.current.dispatchEvent(customEvent);
                            }} autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }

    const validateSecond = () => {

        let errors = [];

        let error = validator(constraints)("databaseUsername", databaseUsername);
        (error == null || errors.push(error)) && setDatabaseUsernameError(error);

        error = validator(constraints)("databasePassword", databasePassword);
        (error == null || errors.push(error)) && setDatabasePasswordError(error);

        return errors;

    }


    const renderPageSecond = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">

                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Software version:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm filter-field"
                                    options={mongoVersionOptions}
                                    value={softwareVersion}
                                    onChange={(e) => {

                                        setSoftwareVersion(e.value);

                                    }}
                                />
                            </div>

                        </div>

                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Username:"}</label>
                            <div className="col">
                                <InputText className="p-inputtext-sm" placeholder="Database Username" value={databaseUsername} onChange={(e) => setDatabaseUsername(e.target.value)} />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databaseUsernameError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm" placeholder="Database Password"
                                    value={databasePassword}
                                    onChange={(e) => setDatabasePassword(e.target.value)}
                                    footer={() => footer(databasePassword)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databasePasswordError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Data path:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <i className={`pi ${isLoading && !path_exists && path_exists !== undefined ? "pi-spin pi-spinner align-self-center" : "pi-info-circle"} data_path`} />
                                    <Tooltip target=".data_path"
                                        content="This is the directory where the new node files and logs will reside." />

                                    <InputText className="p-inputtext-sm" placeholder="Data path" value={dataPath} onChange={(e) => {
                                        setDataPath(e.target.value);
                                        setDataPathError("check data path");
                                    }} />
                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(dataPathError !== 'ok') ? "p-error" : ""} p-d-block`}>{dataPathError}</small>
                                </div>
                            </div>
                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    style={{ width: "80px" }}
                                    label="Check"
                                    onClick={() => {
                                        let error = validator(constraints)("dataPath", dataPath);
                                        setDataPathError(error);

                                        if (error === null) dispatch(LayoutActions.checkPathExists(
                                            {
                                                host: hostname,
                                                path: dataPath,
                                                os_username: osUsername,
                                                os_password: base64Encode(osPassword),
                                                private_key: privateKey,
                                                ssh_port: sshPort,
                                            }))
                                    }}
                                />
                            </div>
                        </div>
                    </div>

                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Config Server port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Config Server port" value={port} onValueChange={(e) => {
                                        setPort(e.value);
                                        setMongosPortError(mongosPort === port ? "must not be same with mongos port" : "check port");

                                    }} />
                                    <i className={isLoading === "config" && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(portError !== 'ok') ? "p-error" : ""} p-d-block`}>{portError}</small>
                                </div>
                            </div>
                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("port", port);
                                        setPortError(error);
                                        if (error === null)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: port,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                    purpose: "config"
                                                }))
                                    }

                                    }
                                />
                            </div>
                        </div>

                    </div>

                    <div>

                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Mongos port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Mongos port" value={mongosPort} onValueChange={(e) => {
                                        setMongosPort(e.value);
                                        setMongosPortError(mongosPort === port ? "must not be same with config server port" : "check port");

                                    }} />
                                    <i className={isLoading === "mongos" && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(mongosPortError !== 'ok') ? "p-error" : ""} p-d-block`}>{mongosPortError}</small>
                                </div>
                            </div>
                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("mongosPort", mongosPort);
                                        setMongosPortError(error);
                                        if (error === null)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: mongosPort,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                    purpose: "mongos"
                                                }))
                                    }

                                    }
                                />
                            </div>
                        </div>

                    </div>

                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Use SSL:"}</label>
                            <div className="col">
                                <Checkbox onChange={(e) => setSSL(e.checked)} checked={ssl} value={ssl} />
                            </div>
                        </div>
                    </div>

                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Install from:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm"
                                    options={repositoryOptions}
                                    value={repository}
                                    onChange={(e) => setRepository(e.value)}
                                />
                            </div>
                        </div>
                    </div>


                    <div>
                        <div className="field grid flex align-items-center">
                            <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "220px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                            <div className="col-fixed">
                                <Checkbox inputId="cb1" value="Automatic delete disabled:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                            </div>
                            <div className="col">
                                <Slider
                                    disabled={!automaticDelete}
                                    min={1}
                                    max={365}
                                    value={automaticDeleteDays}
                                    onChange={(e) => setAutomaticDeleteDays(e.value)} />
                            </div>

                        </div>
                    </div>

                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => {
                    const errors = validateSecond();
                    if (portError === "ok" && dataPathError === "ok" && mongosPortError === "ok" && errors.length === 0) {
                        setCaller('rest-api-bar');
                        dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}:${port}`, 'type': types.MONGO }, { 'exists': setShowDeleteConfirm, 'failback': onRestApiClick }))
                    }
                    else setApiTooltip(getValidationError());
                }} />
            </div>



        );
    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {


            dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))

        }
    }

    const onSecondBackClick = () => {
        setPortError("");
        setDataPathError("");
        setMongosPortError("");
        dispatch(LayoutActions.setMongoVersionExists(undefined));
        setPageIndex(0);
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Cluster Name:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Cluster Name" value={clusterName} onChange={(e) => setClusterName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{clusterNameError}</small>
                    </div>
                </div>
                <div className="field"> Config Server Data:</div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    return (
        <Dialog
            className='add-mongo-sharded-cluster'
            header={`Install a new MongoDB Sharded Cluster`}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById("submit-button")} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>
        </Dialog>
    );
}


export default AddMongoShardedClusterDialog;