import { useState, useEffect, useRef } from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Checkbox } from 'primereact/checkbox';
import { Slider } from 'primereact/slider';
import validator from "../../../util/validator";
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import { Tooltip } from 'primereact/tooltip';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { base64Encode } from '../../../util/base64Encode';
import ErrorTextBox from '../../../components/errorTextBox';
import { ConfirmPopup } from 'primereact/confirmpopup';
import ApiBar from '../../../components/apiBar';
import { clusterRoles, types, clusterSizeOptions } from '../../../config';
import getValidationError from '../../../util/errorPrompter';
import { Dropdown } from 'primereact/dropdown';
import { footer } from '../../../util/passwordHelpers';

const AddMongoShardedClusterNodeDialog = (props) => {

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();

    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [caller, setCaller] = useState('submit-button');

    const onPopupSubmit = (accept) => {
        switch (caller) {

            case 'submit-button':
                onSaveClick(accept);
                break;
            default:
                onRestApiClick(accept);
                break;
        }
    }

    // First Page elements
    const [osUsername, setOsUsername] = useState("root");
    const [osPassword, setOsPassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState(22);
    const [privateKey, setPrivateKey] = useState("");
    const [softwareVersion, setSoftwareVersion] = useState(props.cluster?.ex_params?.version);

    // Second Page elements
    const [port, setPort] = useState(27017);
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const [dataPath, setDataPath] = useState("/123cluster/mongo/data1");
    const [rsName, setRsName] = useState(props.node?.ex_params?.replicaset_name ? props.node?.ex_params?.replicaset_name : "");
    const [rsNameError, setRsNameError] = useState("");
    const customEvent = new Event('beforeinput', { bubbles: true });
    const submitButtonRef = useRef(null);
    const [clusterRole, setClusterRole] = useState(clusterRoles.STANDBY);

    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");

    //Second page elements errors
    const [portError, setPortError] = useState("");
    const [dataPathError, setDataPathError] = useState(false);
    const [isVoting, setIsVoting] = useState(true);

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { errorMessage, isLoading, port_exists, path_exists } = useSelector((state) => state.layout);
    const [submitConfirm, setSubmitConfirm] = useState(false);
    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);

    const roleOptions =
        [
            { label: "Secondary", value: clusterRoles.STANDBY },
            { label: "Arbiter", value: clusterRoles.ARBITER }
        ];


    useEffect(() => {
        if (props.visible === false) {

            setPageIndex(0);

            // First page elements and errors
            setPort(27017);
            setPortError("");
            setOsUsername("root");
            setOsUsernameError("");
            setOsPassword("");
            setOsPasswordError("");
            setHostname("");
            setHostnameError("");
            setSshPort(22);
            setSshPortError("");
            setPrivateKey("");
            setPrivateKeyError("");
            setSoftwareVersion(props.cluster?.ex_params?.version);

            // Second page elements and errors
            setRsNameError("")
            setApiTooltip(" ");
            setPortError("");
            setDataPathError("");
            setDataPath("/123cluster/mongo/data1");;
            setRsName(props.node?.ex_params?.replicaset_name ? props.node?.ex_params?.replicaset_name : "")
            setAutomaticDelete(false);
            setAutomaticDeleteDays(1);
        }


    }, [props.visible, props.node, props.cluster]);

    const onRestApiClick = async (dropInventories) => {


        if (portError === "ok" && dataPathError === "ok") {

            const errors = validateSecond();
            if (errors.length === 0) {


                const cluster = props.cluster

                const host =
                {
                    name: hostname,
                    ssh_port: sshPort,
                    username: osUsername,
                    password: base64Encode(osPassword),
                    private_key: privateKey,
                };

                const ex_params =
                {
                    replicaset_name: rsName,
                    mongo_master_id: props.node?.id,
                    voting: isVoting
                }

                const data = {
                    host,
                    cluster,
                    ex_params,
                    name: `${hostname}:${port}`,
                    auto_delete: automaticDelete,
                    version: softwareVersion,
                    auto_delete_days: automaticDeleteDays,
                    port: port,
                    sizing: clusterSize,
                    cluster_role: clusterRole,
                    dir_123Cluster: dataPath,
                    drop_inventories: dropInventories,
                }
                setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_mongo_node/")))

            }

            else setApiTooltip(getValidationError());

        }
        else setApiTooltip(getValidationError());

        if (!portError) setPortError("check port");
        if (!dataPathError) setDataPathError("check data path");
    }

    const validateFirst = () => {

        let errors = [];


        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);


        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }

        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        return errors;

    }


    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {

                setPortError("");

            }
            else {

                if (port_exists?.exists) {

                    setPortError("already exists");
                }
                else {

                    setPortError("ok");

                }

            }

        }
    }, [port_exists])

    useEffect(() => {

        if (path_exists) {

            if (path_exists?.exists) {


                setDataPathError("already exists")
            }
            else {

                setDataPathError("ok");

            }

        }

    }, [path_exists])



    const onSaveClick = (dropInventories) => {

        if (portError === "ok" && dataPathError === "ok") {

            let errors = validateSecond();

            if (errors.length === 0) {



                const cluster = props.cluster

                const host =
                {
                    name: hostname,
                    ssh_port: sshPort,
                    username: osUsername,
                    password: base64Encode(osPassword),
                    private_key: privateKey,
                };

                const ex_params =
                {
                    replicaset_name: rsName,
                    mongo_master_id: props.node?.id,
                    voting: isVoting
                }

                const data = {
                    host,
                    cluster,
                    ex_params,
                    name: `${hostname}:${port}`,
                    auto_delete: automaticDelete,
                    version: softwareVersion,
                    auto_delete_days: automaticDeleteDays,
                    port: port,
                    sizing: clusterSize,
                    cluster_role: clusterRole,
                    dir_123Cluster: dataPath,
                    drop_inventories: dropInventories,
                }

                if (props.onOkClick)
                    props.onOkClick(data);


            }
            else submitButtonRef.current.dispatchEvent(customEvent);
        }
        else submitButtonRef.current.dispatchEvent(customEvent);

        if (!portError) setPortError("check port");
        if (!dataPathError) setDataPathError("check data path");


    }

    const validateSecond = () => {

        let errors = [];

        let error = validator(constraints)("rsName", rsName);
        (error == null || errors.push(error)) && setRsNameError(error);

        return errors;


    }

    const onDeleteConfirmShow = (target) => {
        const errors = validateSecond()
        if (errors.length === 0 && portError === "ok" && dataPathError === "ok") {
            setCaller(target);
            dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}:${port}`, 'type': types.MONGO }, { 'exists': setShowDeleteConfirm, 'failback': onPopupSubmit }));
        }
        else target === "rest-api-bar" ? setApiTooltip(getValidationError()) : submitButtonRef.current.dispatchEvent(customEvent);

    }

    const onSubmitClick = () => onDeleteConfirmShow('submit-button');

    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            id="submit-button"
                            className='submit-button'
                            ref={submitButtonRef}
                            iconPos="right"
                            onClick={onSubmitClick} autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }
    const renderPageSecond = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Data path:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <i className={`pi ${isLoading && !path_exists && path_exists !== undefined ? "pi-spin pi-spinner align-self-center" : "pi-info-circle"} data_path`} />
                                    <Tooltip target=".data_path"
                                        content="This is the directory where the new node files and logs will reside." />

                                    <InputText className="p-inputtext-sm" placeholder="Data path" value={dataPath} onChange={(e) => {
                                        setDataPath(e.target.value);
                                        setDataPathError("check data path");
                                    }} />
                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(dataPathError !== 'ok') ? "p-error" : ""} p-d-block`}>{dataPathError}</small>
                                </div>
                            </div>
                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    style={{ width: "80px" }}
                                    label="Check"
                                    onClick={() => {
                                        let error = validator(constraints)("dataPath", dataPath);
                                        setDataPathError(error);

                                        if (error === null) dispatch(LayoutActions.checkPathExists(
                                            {
                                                host: hostname,
                                                path: dataPath,
                                                os_username: osUsername,
                                                os_password: base64Encode(osPassword),
                                                private_key: privateKey,
                                                ssh_port: sshPort,
                                            }))
                                    }}
                                />
                            </div>
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Port" value={port} onValueChange={(e) => {
                                        setPort(e.value);
                                        setPortError("check port");

                                    }} />
                                    <i className={isLoading && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(portError !== 'ok') ? "p-error" : ""} p-d-block`}>{portError}</small>
                                </div>
                            </div>

                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("port", port);
                                        setPortError(error);
                                        if (error === null)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: port,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                }))
                                    }

                                    }
                                />
                            </div>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Cluster Role:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm"
                                    options={roleOptions}
                                    value={clusterRole}
                                    onChange={(e) => {

                                        if (props.node.nodes?.filter((n) => n.cluster_role === e.value).length > 0) setRsName(props.node?.ex_params?.replicaset_name);
                                        setClusterRole(e.value);


                                    }}
                                />
                            </div>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Is Voting:"}</label>
                            <div className="col">
                                <Checkbox onChange={() => clusterRole === clusterRoles.STANDBY && setIsVoting(!isVoting)} value={(isVoting || clusterRole === clusterRoles.ARBITER) && (!props.node.nodes?.length || props.node?.nodes?.filter((n) => n.ex_params?.voting === true)?.length < 6)} checked={(isVoting || clusterRole === clusterRoles.ARBITER) && (!props.node.nodes?.length || props.node?.nodes?.filter((n) => n.ex_params?.voting === true)?.length < 6)} disabled={props.node?.nodes?.filter((n) => n.ex_params?.voting === true)?.length >= 6 || clusterRole === clusterRoles.ARBITER} />
                            </div>
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-center">
                            <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "220px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                            <div className="col-fixed">
                                <Checkbox inputId="cb1" value="Automatic delete disabled:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                            </div>
                            <div className="col">
                                <Slider
                                    disabled={!automaticDelete}
                                    min={1}
                                    max={365}
                                    value={automaticDeleteDays}
                                    onChange={(e) => setAutomaticDeleteDays(e.value)} />
                            </div>

                        </div>
                    </div>

                    <div>
                        <div className='field grid'>

                            <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                            <div className='col'>

                                <Dropdown className="p-inputtext-sm"
                                    value={clusterSize}
                                    onChange={(e) => setClusterSize(e.value)}
                                    options={clusterSizeOptions}
                                    placeholder="Size options" />

                            </div>

                        </div>

                    </div>
                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => onDeleteConfirmShow('rest-api-bar')} />
            </div >



        );
    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {

            dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))

        }
    }

    const onSecondBackClick = () => {
        setPortError("");
        setDataPathError("");
        dispatch(LayoutActions.setMongoVersionExists(undefined));
        setPageIndex(0);
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Replica Set Name:"}</label>
                        <div className="col">
                            <InputText disabled={props?.node?.ex_params?.replicaset_name} value={rsName} onChange={(e) => setRsName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{rsNameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    return (
        <Dialog
            className='add-mongo-shard-node'
            header={`Install a new MongoDB node for ${props.cluster.name}`}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup
                    icon="pi pi-exclamation-triangle"
                    target={document.getElementById('submit-button')}
                    visible={submitConfirm}
                    className="multiline"
                    onHide={() => setSubmitConfirm(false)}
                    accept={() => onDeleteConfirmShow('submit-button')}
                    message={isVoting && (props.node.nodes?.length < 1 || props.node.nodes?.filter((n) => n.ex_params?.voting === true)?.length % 2 === 0) ? "Cluster are going to have even amount of voting nodes.\nTo enable majority during primary election, the cluster must have an odd number of voting nodes.\nAre you sure you want to proceed?" : "Avoid deploying more than one arbiter in a replica set.\n Not recommended by MongoDB.\nContinue?"} />
                <ConfirmPopup

                />
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>

        </Dialog>
    );
}


export default AddMongoShardedClusterNodeDialog;