import { useSelector, useDispatch } from 'react-redux';
import { useEffect } from 'react';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dialog } from 'primereact/dialog';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import ErrorTextBox from '../../../components/errorTextBox';
import { loadingMsg} from '../../../config';


export default function MongoStatusDialog(props) {

    const { mongo_cluster_status, isLoading } = useSelector((state) => state.layout);
    const {errorMessage} = useSelector((state) => state.layout);


    const dispatch = useDispatch();

    useEffect(() => {
        if (props.node) {

            dispatch(LayoutActions.getMongoClusterStatus(props.node.id));

        }

    }, [props.node, dispatch])


    const idBodyTemplate = (item) => {

        return (<div>{item.id}</div>);

    }

    const nameBodyTemplate = (item) => {

        return (<div>{item.name}</div>);

    }

    const roleBodyTemplate = (item) => {

        return (<div>{item.stateStr}</div>);

    }

    const healthBodyTemplate = (item) => {

        return (<div>{item.health}</div>);

    }


    const uptimeBodyTemplate = (item) => {

        return (<div>{item.uptime}</div>);

    }

    const pingBodyTemplate = (item) => {

        return (<div>{item.pingMs}</div>);

    }

    return (

        <Dialog
            className='mongo-cluster-status'
            header={`Replica set ${mongo_cluster_status ? mongo_cluster_status?.set : "'unknown'"} status`}
            visible={props.visible}
            style={{ maxHeight: "90%" }}
            onHide={props.onHide}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <DataTable emptyMessage = {!mongo_cluster_status && isLoading ?  <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">No status data</div>} paginator = {mongo_cluster_status?.members?.length > 10}  rows={10} rowsPerPageOptions={[10, 20, 50, 100]} value={mongo_cluster_status?.members?.length > 0 ? mongo_cluster_status?.members : []} >
                        <Column field="id" header="ID" body={idBodyTemplate} sortable />
                        <Column field="name" header="Name" body={nameBodyTemplate} sortable />
                        <Column field="stateStr" header="Role" body={roleBodyTemplate} sortable/>
                        <Column field="health" header="Health" body={healthBodyTemplate} sortable />
                        <Column field="uptime" header="Uptime" body={uptimeBodyTemplate} sortable/>
                        <Column field="pingMs" header="Ping(ms)" body={pingBodyTemplate} sortable/>
                    </DataTable>
                    {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}

                </div>
            </div>

        </Dialog>);
}