import config from "../../../config";

const constraints = {
    port: {
        presence: {
            message: "Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535,
        }
    },
    databaseUsername: {
        presence: {
            message: "Database Username is required field.",
            allowEmpty: false
        },
        length: {
            maximum: 31,
            tooLong: "max %{count} characters"
        },
        format:                                                         
        {
            pattern: config.usernameRegex,
            message: "Database Username has invalid format."
        },
        exclusion: {
            within:  ["postgres", "replicator", "repmgr"],
          }
    },
    databasePassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        }
    },
    pgUserPassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        }
    },
    osUsername: {
        presence: {
            message: "OS Username is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern: config.usernameRegex,
            message: "Os Username has invalid format."
        }
    },
    privateKey: {
        presence: {
            message: "Private Key is required field.",
            allowEmpty: false
        }
    },
    osPassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        length: {
            minimum: 5,
            tooShort: "min %{count} characters"
        },
    },
    podIp: {
        presence: {
            message: "Pod IP is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.hostNameRegex,
            message: "Pod IP has invalid format."
        }
    },
    name: {
        presence: {
            message: "Cluster name is required field.",
            allowEmpty: false
        },
    },
    namespace: {
        presence: {
            message: "namespace is required field.",
            allowEmpty: false
        },
    },
    hostname: {
        presence: {
            message: "Hostname is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.hostNameRegex,
            message: "Host Name has invalid format."
        }
    },
    sshPort: {
        presence: {
            message: "SSH Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535
        }
    },
    dataPath: {
        presence: {
            message: "data path is required field.",
            allowEmpty: false
        },

        format:                                                         
        {
            pattern: config.pathRegex,
            message: "data path has invalid format."
        }
    },
}

export default constraints;