import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { Button } from "primereact/button";
import { ProgressBar } from "primereact/progressbar";
import { Dialog } from "primereact/dialog";
import { useDispatch, useSelector } from "react-redux";
import moment from "moment";
import { MonitoringActions } from "../../../store/monitoring/reducer";
import { ConfirmDialog } from 'primereact/confirmdialog';
import { useState} from "react";
import Timestrip from "../../../components/timestrip";
import MonitoringWizard from "../monitorWizard";
import ErrorDetailsDialog from "../../errorDialog";

export default function MonitoringItems(props) {

    const dispatch = useDispatch();
    const { visible, onHide, monitorInterval } = props;
    const {monitors, alert_history } = useSelector((state) => state.monitoring);
    const { user } = useSelector((state) => state.app);

    const [selectedItem, setSelectedItem] = useState(null);
    const [snoozeConfirm, setSnoozeConfirm] = useState(false);

    const [showMonitorWizard, setShowMonitorWizard] = useState(false);
    const [monitorError, setMonitorError] = useState("");
    const [showErrorDetails, setShowErrorDetails] = useState(false);
    const [targetMonitor, setTargetMonitor] = useState(null);

    const onSnoozeMonitor = () => {
        dispatch(MonitoringActions.snoozeMonitor({
            "node_id": props?.currentItem?.node_id,
            "monitor_name": props?.currentItem?.monitor_name,
            "node_type": props?.currentItem?.node_type,
        }));
    };

    const renderFooter = () => {
        return (

            <div style={{ display: "flex", flexDirection: "column", width: "100%" }}>
            <div style={{ marginTop: "10px", flex: 1, overflowY: "auto", padding: "10px" }}></div>
                <div style={{ marginTop: "auto", display: "flex", justifyContent: "flex-end" }}>
                    <Button label={"Settings"} onClick={() => {
                        if(monitors?.length > 0) {
                            const monitor = monitors.filter((m) => m.monitor_name === props.currentItem?.monitor_name)
                            if (monitor?.length > 0){
                                setTargetMonitor(monitor[0]);
                                setShowMonitorWizard(true);
                            }

                        }


                    }
                } />
                    <Button label={"Snooze 1 day"} onClick={() => setSnoozeConfirm(true)} />
                    <Button className="p-button-info icon-only" icon="pi pi-save" />
                    <Button className="p-button-success icon-only" icon="pi pi-share-alt" />
                    <Button className="p-button-info" label="Close" onClick={props.onHide} />
                </div>
            </div>
        );
    };

    const getTableHeader = () => {
        return (
            <div className="flex justify-content-start">
                {`Monitor items as of date ${moment(selectedItem?.date_created_with_tz).format("MM/DD/YY  hh:mm A")}`}
            </div>
        );
    };

    const displayValueTemplate = () =>
      !alert_history?.length || alert_history?.length < 1 ? null : (
          <Timestrip data={alert_history} interval={monitorInterval} focusCallback={setSelectedItem} />
      );

    const getTitle = () => {

        if (props.currentItem){

            switch (props?.monitorInterval) {
                case 0:
                    return `${props.currentItem?.monitor_name} - Last 10 minutes`;
                case 1:
                    return `${props.currentItem?.monitor_name} - Last hour`;
                case 2:
                    return `${props.currentItem?.monitor_name} - Last 6 hours`;
                case 3:
                    return `${props.currentItem?.monitor_name} - Last day`;
                case 4:
                    return `${props.currentItem?.monitor_name} - Last week`;
                default:
                    return `${props.currentItem?.monitor_name} - Last week`;
            }
        };

        return 'Monitoring Items'

        }

    const onConfirm = (update, data) => {

        if (update) dispatch(MonitoringActions.updateMonitor(data, {
            "success": setShowMonitorWizard,
            "error": errorCallback
        }));
        else dispatch(MonitoringActions.addMonitor(data, {
            "success": setShowMonitorWizard,
            "error": errorCallback
        }));

    }

    const errorCallback = (message) => {

        setMonitorError(message);
        setShowErrorDetails(true)
    }

    return (

        <div>
              {showErrorDetails ?
            <ErrorDetailsDialog

                visible={showErrorDetails}
                onHide={() => {
                    setShowErrorDetails(false);
                }}
                message={monitorError?.message ? monitorError?.message : monitorError}


            />
            : null}

            {showMonitorWizard ? (
                <MonitoringWizard user={user} visible={showMonitorWizard} update={true} data={targetMonitor} onHide={() => setShowMonitorWizard(false)} onOkClick={onConfirm} />
            ) : null}
            <Dialog
                className="monitor-items"
                style={{ maxWidth: "90vw" }}
                header={getTitle()}
                visible={visible}
                onHide={onHide}
                footer={renderFooter()}
            >
                <div style={{ display: "flex", flexDirection: "column", height: "100%" }} className="form-container">
                    <div style={{ padding: "10px", paddingTop: "0px", paddingLeft: "10px", paddingRight: "10px" }} className="p-fluid form-container">
                        <div className="flex flex-column mb-4">
                            <ProgressBar className="timestrip" value={alert_history?.length > 0 ? 1 : 0} displayValueTemplate={displayValueTemplate}></ProgressBar>
                        </div>
                        <div className="flex mt-2 mb-1">
                            <div className="mr-auto text-xs unselectable">
                                {monitorInterval === 0 ? "10 minutes" : monitorInterval === 1 ? "1 hour" : monitorInterval === 2 ? "6 hours" : monitorInterval === 3 ? "1 day" : monitorInterval === 4 ? "1 week" : "1 week"}
                            </div>
                            <div className="ml-auto mr-auto text-xs unselectable">
                                {monitorInterval === 0 ? "5 minutes" : monitorInterval === 1 ? "30 minites" : monitorInterval === 2 ? "3 hours" : monitorInterval === 3 ? "12 hours" : monitorInterval === 4 ? "3 days" : "3 days"}
                            </div>
                            <div className="ml-auto text-xs unselectable">Now</div>
                        </div>
                    </div>
                    <div style={{ borderTop: "1px solid #040d19", marginTop: "10px", flex: 1, overflowY: "auto", padding: "10px" }}>
                        {selectedItem && selectedItem.details && (
                            <DataTable
                                className="severity-table"
                                rowClassName={(rowData) => `severity-index-${rowData["color"]?.toLowerCase()}-row`}
                                header={getTableHeader()}
                                value={selectedItem.details}
                            >
                                {selectedItem.details[0] &&
                                    Object.keys(selectedItem.details[0]).map((b, i) => (
                                        b === "color" ? (
                                            <Column key={i} />
                                        ) : (
                                            <Column
                                                key={i}
                                                field={b}
                                                header={b}
                                                sortable
                                                body={(d) => (
                                                    <div className="truncate-cell" title={d[b]?.toString()}>
                                                        {d[b]?.toString().length > 40 ? d[b]?.toString().substring(0, 40) + "..." : d[b]?.toString()}
                                                    </div>
                                                )}
                                            />
                                        )
                                    ))}
                            </DataTable>
                        )}
                    </div>
                    <div style={{ marginTop: "10px", flex: 1, overflowY: "auto", padding: "10px" }}></div>
                    <ConfirmDialog
                        acceptIcon="pi pi-check"
                        rejectIcon="pi pi-times"
                        acceptLabel="Confirm"
                        rejectLabel="Cancel"
                        visible={snoozeConfirm}
                        onHide={() => setSnoozeConfirm(false)}
                        message="This will snooze monitor for 24 hours"
                        header="Snooze monitor"
                        acceptClassName="p-button-info"
                        icon="pi pi-exclamation-triangle"
                        accept={onSnoozeMonitor}
                        reject={() => setSnoozeConfirm(false)}
                    />
                </div>
            </Dialog>
        </div>

    );
}
