import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import validator from "../../../util/validator";
import constraints from "./constraints";


export const RegisterAgentDialog = (props) => {

    const [ip, setIp] = useState("");
    const [key, setKey] = useState("");
    const [ipError, setIpError] = useState(null);
    const [keyError, setKeyError] = useState(null);

    const validate = () => {

        let errors = [];

        let error = validator(constraints)("ip", ip);
        (error == null || errors.push(error)) && setIpError(error);

        error = validator(constraints)("key", key);
        (error == null || errors.push(error)) && setKeyError(error);

        return errors;


    }

    const handleOk = () => {

        let errors = validate();
        if (errors.length === 0) props.onOkClick({"agent_ip": ip, "agent_key": key, "ind_enabled": "Y", "role": "Load Balancer"})
                
    

    }

    const renderFooter = () => {
        return (
            <div style={{ display: "flex", justifyContent: "space-between" }}>
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    label={"Register"}
                    icon="pi pi-check"
                    iconPos="right"
                    onClick={handleOk} autoFocus />
            </div>
        );

    }


    return (

        <Dialog
            className='agents-dialog'
            header={`Register a new Agent`}
            visible={props.visible}
            style={{ width: '60vw'}}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <div className="field">
                    Enter the IP address and key of the Agent you want to register
                    </div>
                    <div className="field grid">
                        <div className="col">

                        <InputText className={`p-inputtext-sm ${ipError ? "p-invalid block" : ""}`}
                                    placeholder="IP Address (e.g. 10.20.248.64)"
                                    value={ip}
                                    onChange={(e) => 
                                    {

                                        setIp(e.target.value);

                                    }}
                                    autoComplete="new-password" />
                            <div className="p-error block text-xs">{ipError}</div>

                        </div>
                        <div className="col">
                        <InputText className={`p-inputtext-sm ${keyError ? "p-invalid block" : ""}`}
                                    placeholder="Key (e.g. dGNpwtXqZpfijfN4UPp8N6UNUcfabIOc)"
                                    value={key}
                                    onChange={(e) => 
                                    {

                                        setKey(e.target.value);

                                    }}
                                    autoComplete="new-password" />
                                   <div className="p-error block text-xs">{keyError}</div>
                              
                      
                          
                        </div>


                    </div>
                </div>
            </div>

        </Dialog>)

};

export default RegisterAgentDialog;