import { useState, useEffect, useRef, useCallback } from 'react';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Checkbox } from 'primereact/checkbox';
import { Slider } from 'primereact/slider';
import validator from "../../../util/validator";
import { base64Encode } from '../../../util/base64Encode';
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from '../../../components/errorTextBox';
import ApiBar from '../../../components/apiBar';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { types, clusterSizeOptions } from '../../../config';
import getValidationError from '../../../util/errorPrompter';
import { Tooltip } from "primereact/tooltip";
import { useSelector } from 'react-redux';
import { footer } from '../../../util/passwordHelpers';

const AddMsSQLMasterDialog = ({ visible, onHide, onOkClick }) => {

    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const caller = useRef('submit-button');
    const [submitting, setSubmitting] = useState(false);

    const onPopupSubmit = (accept) => onSaveClick(accept);

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();

    // First Page elements
    const [osUsername, setOsUsername] = useState("administrator");
    const [osPassword, setOsPassword] = useState("");
    const [domain, setDomainName] = useState("");
    const [domainError, setDomainError] = useState("");
    const [databaseUsername, setDatabaseUsername] = useState("sa");
    const [databasePassword, setDatabasePassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState(5985);
    const [privateKey, setPrivateKey] = useState("");

    // Second Page elements
    const [port, setPort] = useState(1433);
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);
    const [dataPath, setDataPath] = useState("");
    const submitButtonRef = useRef(null);




    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [portError, setPortError] = useState("");
    const [sshPortError, setSshPortError] = useState("");;
    const [privateKeyError, setPrivateKeyError] = useState("");

    const [databaseUsernameError, setDatabaseUsernameError] = useState("");
    const [databasePasswordError, setDatabasePasswordError] = useState("");

    //Second page elements errors
    // const [dataPathError, setDataPathError] = useState(false);

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { isLoading, errorMessage } = useSelector((state) => state.layout);

    useEffect(() => {

        if (visible === false) {

            setPageIndex(0);

            // First page elements and errors
            setPort(1433);
            setPortError("");
            setOsUsername("administrator");
            setOsUsernameError("");
            setOsPassword("");
            setOsPasswordError("");
            setDatabaseUsername("sa");
            setDatabaseUsernameError("");
            setDatabasePassword("");
            setDatabasePasswordError("");
            setHostname("");
            setHostnameError("");
            setSshPort(5985);
            setSshPortError("");
            setPrivateKey("");
            setPrivateKeyError("");
            // Second page elements and errors
            setApiTooltip(" ");
            setPortError("");
            // setDataPathError("");
            setDataPath("");
            setAutomaticDelete(false);
            setAutomaticDeleteDays(1);
        }


    }, [visible]);


    const onSaveClick = useCallback(async (dropInventories) => {



        const cluster =
        {

            name: `${hostname}:${port}`

        }

        const host =
        {
            name: hostname,
            ssh_port: sshPort,
            username: osUsername,
            password: base64Encode(osPassword),
            private_key: privateKey,
            ex_params: {

                domain: domain
            }
        };

        const ex_params =
        {

        }

        const data = {
            host,
            cluster,
            ex_params,
            name: cluster.name,
            auto_delete: automaticDelete,
            auto_delete_days: automaticDeleteDays,
            dir_123Cluster: dataPath,
            port: port,
            sizing: clusterSize,
            database_user: databaseUsername,
            drop_inventories: dropInventories,
            database_pwd: base64Encode(databasePassword),
        }

        if (caller.current === 'submit-button') {


            if (onOkClick)
                onOkClick(data);

        }
        else setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_mssql_master/")))



    }, [accessToken, automaticDelete, automaticDeleteDays, clusterSize, databaseUsername, databasePassword,
        hostname, onOkClick, osPassword, osUsername, port, privateKey, sshPort, domain, dataPath])

    useEffect(() => {


        const customEvent = new Event('beforeinput', { bubbles: true });



        const validateSecond = () => {

            let errors = [];


            let error = validator(constraints)("databaseUsername", databaseUsername);
            (error == null || errors.push(error)) && setDatabaseUsernameError(error);

            error = validator(constraints)("databasePassword", databasePassword);
            (error == null || errors.push(error)) && setDatabasePasswordError(error);

            return errors;


        }


        if (submitting) {

            const errors = validateSecond();

            if (errors.length === 0) {

                if (portError !== "ok") {


                    dispatch(LayoutActions.checkPortExists(
                        {
                            host: hostname,
                            port: port,
                            os_username: osUsername,
                            os_password: base64Encode(osPassword),
                            private_key: privateKey,
                            ssh_port: sshPort,
                        }))

                }

                if (portError === "ok") {

                    dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}:${port}`, 'type': types.MSSQL }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))


                }

            }
            else caller.current === "submit-button" ? submitButtonRef.current.dispatchEvent(customEvent) : setApiTooltip(getValidationError());



        }

    }, [submitting, dispatch, hostname, onSaveClick, osPassword, osUsername, port, portError, dataPath, privateKey, sshPort, databasePassword, databaseUsername])

    const validateFirst = () => {

        let errors = [];


        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);


        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }

        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("domain", domain);
        (error == null || errors.push(error)) && setDomainError(error);


        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        error = validator(constraints)("port", port);
        (error == null || errors.push(error)) && setPortError(error);

        return errors;

    }


    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            iconPos="right"
                            id='submit-button'
                            className='submit-button'
                            ref={submitButtonRef}

                            onClick={() => {
                                caller.current = 'submit-button'
                                setSubmitting(true);
                            }} autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }
    const renderPageSecond = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Username:"}</label>
                            <div className="col">
                                <InputText className="p-inputtext-sm" placeholder="Database Username" value={databaseUsername} onChange={(e) => setDatabaseUsername(e.target.value)} />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databaseUsernameError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Password:"}</label>
                            <div className="col">
                                <Password
                                    className="p-inputtext-sm" placeholder="Database Password"
                                    value={databasePassword}
                                    onChange={(e) => setDatabasePassword(e.target.value)}
                                    footer={() => footer(databasePassword, true)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databasePasswordError}</small>
                        </div>
                    </div>
                    <div>
                        <div>
                            <div className="field grid flex align-items-center">
                                <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "220px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                                <div className="col-fixed">
                                    <Checkbox inputId="cb1" value="Automatic delete disabledf:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                                </div>
                                <div className="col">
                                    <Slider
                                        disabled={!automaticDelete}
                                        min={1}
                                        max={365}
                                        value={automaticDeleteDays}
                                        onChange={(e) => setAutomaticDeleteDays(e.value)} />
                                </div>

                            </div>
                        </div>

                    </div>

                    <div>
                        <div className='field grid'>

                            <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                            <div className='col'>

                                <Dropdown className="p-inputtext-sm"
                                    value={clusterSize}
                                    onChange={(e) => setClusterSize(e.value)}
                                    options={clusterSizeOptions}
                                    placeholder="Size options" />

                            </div>

                        </div>

                    </div>

                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => {
                    caller.current = 'rest-api-bar';
                    setSubmitting(true);
                }} />
            </div>



        );
    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {

            dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    db_type: types.MSSQL,
                    ssh_port: sshPort,
                    db_port: port,
                    db_user: databaseUsername,
                    db_pwd: base64Encode(databasePassword),
                }, setPageIndex))

        }
    }

    const onSecondBackClick = () => {
        setPageIndex(0);
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Domain:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Domain" value={domain} onChange={(e) => setDomainName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{domainError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="OS Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Port" value={port} onValueChange={(e) => setPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className={`${(portError !== 'ok') ? "p-error" : ""} p-d-block`}>{portError}</small>
                    </div>
                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    return (
        <Dialog
            className='add-mssql-master'
            header={"Register a new SQL Server Cluster"}
            visible={visible}
            style={{ width: '50vw' }}
            onHide={onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller.current)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>

        </Dialog>
    );
}


export default AddMsSQLMasterDialog;