import { useState, useEffect } from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Checkbox } from 'primereact/checkbox';
import { Slider } from 'primereact/slider';
import validator from "../../../util/validator";
import { base64Encode } from '../../../util/base64Encode';
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import { Tooltip } from 'primereact/tooltip';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from '../../../components/errorTextBox';
import ApiBar from '../../../components/apiBar';
import { clusterSizeOptions } from '../../../config';

const AddMsSQLStandbyDialog = (props) => {

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();

    // First Page elements
    const [osUsername, setOsUsername] = useState("");
    const [osPassword, setOsPassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState("");
    const [privateKey, setPrivateKey] = useState("");

    // Second Page elements
    const [port, setPort] = useState(3306);
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);
    const [dataPath, setDataPath] = useState("/123cluster/mssql/data1");
    const [mssqlRoot, setMysqlRoot] = useState("");
    const [repUser, setRepUser] = useState("");
    const [repPwd, setRepPwd] = useState("");

    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");

    //Second page elements errors
    const [portError, setPortError] = useState("");
    const [mssqlRootError, setMysqlRootError] = useState("");
    const [repUserError, setRepUserError] = useState("");
    const [repPwdError, setRepPwdError] = useState("");
    const [dataPathError, setDataPathError] = useState(false);

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { errorMessage, isLoading, port_exists, path_exists, mssql_version_exists, mssql_user_exists } = useSelector((state) => state.layout);

    useEffect(() => {
        setPageIndex(0);

        // First page elements and errors
        setPort(3306);
        setPortError("");
        setOsUsername("root");
        setOsUsernameError("");
        setOsPassword("");
        setOsPasswordError("");
        setHostname("");
        setHostnameError("");
        setSshPort(22);
        setSshPortError("");
        setPrivateKey("");
        setPrivateKeyError("");

        // Second page elements and errors
        setApiTooltip(" ");
        setPortError("");
        setDataPathError("");
        setMysqlRootError("");
        setRepUserError("");
        setRepUser("");
        setRepPwd("");
        setRepPwdError("");
        setMysqlRoot("");
        setDataPath("/123cluster/mssql/data1");
        setAutomaticDelete(false);
        setAutomaticDeleteDays(1);


    }, [props.visible]);

    const onRestApiClick = async () => {
        let errors = validateFirst();
        if (errors.length === 0 && portError === "ok" && dataPathError === "ok" && repUserError === "ok") {
            let errors = validateSecond();
            if (errors.length === 0) {

                const cluster =
                {

                    name: `${hostname}:${port}`

                }

                const host =
                {
                    name: hostname,
                    ssh_port: sshPort,
                    username: osUsername,
                    password: base64Encode(osPassword),
                    private_key: privateKey,
                };

                const ex_params =
                {
                    mssql_master_id: props.node.id,
                    rep_user: repUser,
                    rep_pwd: repPwd,
                    mssql_root_password: mssqlRoot
                }

                const data = {
                    host,
                    cluster,
                    ex_params,
                    name: cluster.name,
                    auto_delete: automaticDelete,
                    auto_delete_days: automaticDeleteDays,
                    dir_123Cluster: dataPath,
                    port: port,
                    sizing: clusterSize,
                    version_exists: mssql_version_exists?.exists ? true : false,
                }
                setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_mssql_standby/")))

            }
            else setApiTooltip(" ");



        }
        else setApiTooltip(" ");

        if (!portError) setPortError("check port");
        if (!dataPathError) setDataPathError("check data path");
        if (!repUserError) setRepUserError("check username");
    }

    const validateFirst = () => {

        let errors = [];


        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);


        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }

        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        return errors;

    }


    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {

                setPortError("");

            }
            else {

                if (port_exists?.exists) {

                    setPortError("already exists");
                }
                else {

                    setPortError("ok");

                }

            }

        }
    }, [port_exists])

    useEffect(() => {
        if (mssql_user_exists) {
            if (mssql_user_exists?.detail) {

                setRepUserError("");

            }
            else {

                if (mssql_user_exists?.exists) {

                    setRepUserError("already exists");
                }
                else {

                    setRepUserError("ok");

                }

            }

        }
    }, [mssql_user_exists])

    useEffect(() => {

        if (path_exists) {

            if (path_exists?.exists) {


                setDataPathError("already exists")
            }
            else {

                setDataPathError("ok");

            }

        }

    }, [path_exists])



    const validateSecond = () => {

        let errors = [];

        let error = validator(constraints)("mssqlRoot", mssqlRoot);
        (error == null || errors.push(error)) && setMysqlRootError(error);

        error = validator(constraints)("repPwd", repPwd);
        (error == null || errors.push(error)) && setRepPwdError(error);

        return errors;


    }



    const onSaveClick = () => {


        if (portError === "ok" && dataPathError === "ok" && repUserError === "ok") {

            let errors = validateSecond();
            if (errors.length === 0) {

                if (props.onOkClick)
                    props.onOkClick({
                        port,
                        osUsername,
                        osPassword,
                        hostname,
                        sshPort,
                        dataPath,
                        repUser,
                        repPwd,
                        automaticDelete,
                        automaticDeleteDays,
                        mssqlRoot,
                        clusterSize,
                        privateKey,
                        mssqlMasterId: props.node.id,
                        version_exists: mssql_version_exists?.exists ? true : false,
                    });

            }
        }
        if (!portError) setPortError("check port");
        if (!dataPathError) setDataPathError("check data path");
        if (!repUserError) setRepUserError("check username");


    }

    useEffect(() => {

        if (props.visible && pageIndex === 1) {
            dispatch(LayoutActions.checkMySqlVersionExists(
                {
                    host: hostname,
                    software_version: props.node.version,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }));


        }

    }, [props.visible, pageIndex, hostname, props.node, osUsername, osPassword, privateKey, sshPort, dispatch])

    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon="pi pi-check"
                            iconPos="right"
                            onClick={onSaveClick} autoFocus />
                    </div>
                </div>
            );
        }
    }
    const renderPageSecond = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Data path:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <i className={`pi ${isLoading && !path_exists && path_exists !== undefined ? "pi-spin pi-spinner align-self-center" : "pi-info-circle"} data_path`} />
                                    <Tooltip target=".data_path"
                                        content="This is the directory where the new node files and logs will reside." />

                                    <InputText className="p-inputtext-sm" placeholder="Data path" value={dataPath} onChange={(e) => {
                                        setDataPath(e.target.value);
                                        setDataPathError("check data path");
                                    }} />
                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(path_exists?.exists || path_exists?.error) ? "p-error" : ""} p-d-block`}>{dataPathError}</small>
                                </div>
                            </div>
                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    style={{ width: "80px" }}
                                    label="Check"
                                    onClick={() => {
                                        let error = validator(constraints)("dataPath", dataPath);
                                        setDataPathError(error);

                                        if (error === null) dispatch(LayoutActions.checkPathExists(
                                            {
                                                host: hostname,
                                                path: dataPath,
                                                os_username: osUsername,
                                                os_password: base64Encode(osPassword),
                                                private_key: privateKey,
                                                ssh_port: sshPort,
                                            }))
                                    }}
                                />
                            </div>
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Port" value={port} onValueChange={(e) => {
                                        setPort(e.value);
                                        setPortError("check port");

                                    }} />
                                    <i className={isLoading && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(port_exists?.exists || port_exists?.error) ? "p-error" : ""} p-d-block`}>{portError}</small>
                                </div>
                            </div>

                            <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("port", port);
                                        setPortError(error);
                                        if (error === null)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: port,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                }))
                                    }

                                    }
                                />
                            </div>
                        </div>

                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"SQL Server root Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm"
                                    placeholder="SQL Server root Password"
                                    value={mssqlRoot}
                                    onChange={(e) => setMysqlRoot(e.target.value)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{mssqlRootError}</small>
                        </div>
                    </div>

                    <div className="field grid flex align-items-start">
                        <label className="col-fixed" style={{ width: "220px" }}>{"Replication User:"}</label>
                        <div className="col">
                            <span className="p-input-icon-right flex">
                                <InputText
                                    className="p-inputtext-sm"
                                    placeholder="Replication User"
                                    value={repUser}
                                    onChange={(e) => {
                                        setRepUser(e.target.value);
                                        setRepUserError("check username");

                                    }} />
                                <i className={isLoading && !mssql_user_exists && mssql_user_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                            </span>
                            <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                <small id="name-help" className={`${(port_exists?.exists || port_exists?.error) ? "p-error" : ""} p-d-block`}>{repUserError}</small>
                            </div>
                        </div>

                        <div className="col-fixed">
                            <Button
                                className="p-button-sm"
                                label="Check"
                                style={{ width: "80px" }}
                                onClick={() => {
                                    let error = validator(constraints)("repUser", repUser);
                                    setRepUserError(error);
                                    if (error === null)
                                        dispatch(LayoutActions.checkMsSqlUserExists(
                                            {
                                                node_id: props.node.id,
                                                usenname: repUser,
                                            }))
                                }

                                }
                            />
                        </div>
                    </div>

                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Replication User Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm"
                                    placeholder="Replication User Password"
                                    value={repPwd}
                                    onChange={(e) => setRepPwd(e.target.value)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{repPwdError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-center">
                            <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "220px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                            <div className="col-fixed">
                                <Checkbox inputId="cb1" value="Automatic delete disabledf:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                            </div>
                            <div className="col">
                                <Slider
                                    disabled={!automaticDelete}
                                    min={1}
                                    max={365}
                                    value={automaticDeleteDays}
                                    onChange={(e) => setAutomaticDeleteDays(e.value)} />
                            </div>

                        </div>
                    </div>

                    <div>
                        <div className='field grid'>

                            <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                            <div className='col'>

                                <Dropdown className="p-inputtext-sm"
                                    value={clusterSize}
                                    onChange={(e) => setClusterSize(e.value)}
                                    options={clusterSizeOptions}
                                    placeholder="Size options" />

                            </div>

                        </div>

                    </div>

                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
            </div>



        );
    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {

            dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))

        }
    }

    const onSecondBackClick = () => {
        setPortError("");
        setDataPathError("");
        setRepUserError("");
        dispatch(LayoutActions.setMySqlVersionExists(undefined));
        setPageIndex(0);
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    return (
        <Dialog
            className='add-mssql-master'
            header={`Add a new SQL Server Standby for ${props.node.name}`}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
            </div>

        </Dialog>
    );
}


export default AddMsSQLStandbyDialog;