import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Tag } from 'primereact/tag';
import { useState, useEffect, useRef } from "react";
import { RadioButton } from "primereact/radiobutton";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { useSelector, useDispatch } from "react-redux";
import { InputText } from "primereact/inputtext";
import { LayoutActions } from "../../../store/layout/reducer";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ErrorTextBox from "../../../components/errorTextBox";
import { InputNumber } from "primereact/inputnumber";
import { Checkbox } from "primereact/checkbox";
import { types } from "../../../config";
import getValidationError from '../../../util/errorPrompter';
import { Tooltip } from "primereact/tooltip";

const CopyDbDialog = (props) => {

    const [apiTooltip, setApiTooltip] = useState(" ");
    const [dbRadio, setDbRadio] = useState("copy");
    const [copyRadio, setCopyRadio] = useState("one_time");
    const [copyName, setCopyName] = useState("");
    const [nameError, setNameError] = useState("");
    const [pageIndex, setPageIndex] = useState(0);
    const [sourcePort, setSourcePort] = useState(5022);
    const [sourcePortError, setSourcePortError] = useState("");
    const [targetPortError, setTargetPortError] = useState("");
    const [targetPort, setTargetPort] = useState(5022);
    const { accessToken } = useSelector((state) => state.app);
    const { valid_name, port_exists, isLoading, errorMessage } = useSelector((state) => state.layout);
    const [async, setAsync] = useState(false);
    const customEvent = new Event('beforeinput', { bubbles: true });
    const submitButtonRef = useRef(null);

    


    const dispatch = useDispatch();

    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {


                port_exists?.purpose === "mssql_mirror_source" ? setSourcePortError(port_exists?.detail) : setTargetPortError(port_exists?.detail);



            }
            else {

                if (port_exists?.exists) {

                    port_exists?.purpose === "mssql_mirror_source" ? setSourcePortError("already exists") : setTargetPortError("already exists");
                }
                else {

                    port_exists?.purpose === "mssql_mirror_source" ? setSourcePortError("ok") : setTargetPortError("ok");

                }

            }

        }
    }, [port_exists])

    const onRestApiClick = async () => {

        switch (pageIndex) {

            case 0:
                if (dbRadio === "copy" && !props.new) {
                    if (nameError === "name available") {
                        setApiTooltip(await copyToClipboard(createAPIcommand({
                            "db_id": props.db?.name,
                            "node_id": props.node?.name,
                            "source_id": props.db?.host,
                            "option": !props.new ? dbRadio : null,
                            "copy_option": copyRadio,
                            "copy_name": copyName,
                            "db_exists": !props?.new

                        }, accessToken, "copy_mssql_database/")));

                    }
                }
                else {


                    setApiTooltip(await copyToClipboard(createAPIcommand({
                        "db_id": props.db?.name,
                        "node_id": props.node?.name,
                        "source_id": props.db?.host,
                        "option": !props.new ? dbRadio : null,
                        "copy_option": copyRadio,
                        "db_exists": !props?.new

                    }, accessToken, "copy_mssql_database/")));

                }
                break;

            case 1:
                if (sourcePortError === "ok" && targetPortError === "ok") {

                    const data = {
                        "source_id": props?.db?.node,
                        "node_id": props?.node?.id,
                        "db_id": props?.db?.id,
                        "copy_name": copyName,
                        "db_exists": valid_name?.response === "already exists",
                        "copy_option": copyRadio,
                        "source_port": sourcePort,
                        "target_port": targetPort,
                        "async": async,
                    }

                    setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "copy_mssql_database/")))

                }
                else setApiTooltip(" ");
                if (!sourcePortError) setSourcePortError("check source port");
                if (!targetPortError) setTargetPortError("check target port");
                break;
            default:
                break;

        }



    }

    const handleNext = () => {

        switch (pageIndex) {
            case 0:
                if (copyRadio === "mirror") {
                    if (!nameError) setNameError("check if mirror database exist");
                    else if (nameError === 'already exists') {

                        setNameError("");
                        setPageIndex(1);

                    }

                }
                break;
            default:
                break;
        }
    }

    const renderFooter = () => {
        return (
            <div style={{ display: "flex", justifyContent: "space-between" }}>
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                {pageIndex > 0 && <Button
                    className="p-button-secondary"
                    label={"Back"}
                    icon={"pi pi-arrow-left"}
                    iconPos="left"
                    onClick={() => setPageIndex(0)} autoFocus />}

                {pageIndex === 0 &&
                    <Button
                        className="p-button"
                        label="Next"
                        icon="pi pi-arrow-right"
                        iconPos="right"
                        onClick={handleNext} autoFocus />
                }
                <Button
                    className="submit-button"
                    label={"Submit"}
                    disabled={errorMessage}
                    ref={submitButtonRef}
                    icon={"pi pi-check"}
                    iconPos="right"
                    onClick={handleOk} autoFocus />
                <Tooltip   target='.submit-button' showEvent='beforeinput' position='top' >
                    {getValidationError()}
                </Tooltip>

            </div>
        );

    }

    const validateDbName = () => {
        setNameError("");

        let errors = [];
        let error = validator(constraints)("name", copyName);
        (error == null || errors.push(error)) && setNameError(error);
        if (errors.length === 0) dispatch(LayoutActions.checkMsSqlDatabaseName({ node_id: props.node.id, name: copyName }));


    }


    useEffect(() => {

        if (valid_name) {

            if (valid_name?.error) {

                setNameError(valid_name.error);

            }
            else setNameError(valid_name?.response);
        }

    }, [valid_name]);


    const handleOk = () => {
        switch (pageIndex) {

            case 0:
                if (dbRadio === "copy" && !props.new) {

                    if (!nameError) setNameError("name not checked");

                    if (nameError === "name available") {

                        

                        props.onOkClick(
                            {
                                "db_id": props.db?.id,
                                "node_id": props.node?.id,
                                "option": !props.new ? dbRadio : null,
                                "copy_option": copyRadio,
                                "copy_name": copyName,
                                "source_id": props.db?.node,
                                "db_exists": !props?.new
                            });
                    }
                    else submitButtonRef.current.dispatchEvent(customEvent);


                }
                else {

                    
                    props.onOkClick(
                        {
                            "db_id": props.db?.id,
                            "node_id": props.node?.id,
                            "option": !props.new ? dbRadio : null,
                            "copy_option": copyRadio,
                            "source_id": props.db?.node,
                            "db_exists": !props?.new
                        });

                }
                break;


            case 1:
                if (sourcePortError === "ok" && targetPortError === "ok") {

                    const data = {
                        "source_id": props?.db?.node,
                        "node_id": props?.node?.id,
                        "db_id": props?.db?.id,
                        "copy_name": copyName,
                        "db_exists": valid_name?.response === "already exists",
                        "copy_option": copyRadio,
                        "source_port": sourcePort,
                        "target_port": targetPort,
                        "async": async,
                    }

                    if (props.onOkClick) props.onOkClick(data);

                }
                if (!sourcePortError) setSourcePortError("check source port");
                if (!targetPortError) setTargetPortError("check target port");
                break;

            default:
                break;

        }

    }

    const renderPage1 = () => {

        return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
            <div className="field">{`This will copy the database from ${props.db?.host} to ${[props.node?.name]}`}</div>
            <div className="field">Copy name</div>
            <div className="field grid">
                <div className="col-fixed">

                    <InputText className="p-inputtext-sm filter-field" placeholder="Copy name" value={copyName} onChange={(e) => {

                        setCopyName(e.target.value);
                        setNameError("name not checked");
                        dispatch(LayoutActions.setValidName(undefined));

                    }} />
                    <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
                </div>
                <div className="col-fixed">

                    <Button style={{ width: "80px" }} label="Check" disabled={dbRadio !== "copy"} onClick={validateDbName} className="p-button-sm" />

                </div>

            </div>
            <div className="field grid">
                <div className="col">
                    {
                        (!props.new || props.source === "backup-expandable") && (<div>
                            <div className="field">
                                Database {props.db?.name} already exists in this cluster
                            </div>
                            <div className="flex">

                                <div className="field-radiobutton">
                                    <RadioButton inputId="dbRadio1" name="dbRadio" value="override" checked={dbRadio === "override"} onChange={(e) => setDbRadio(e.value)} />
                                    <label htmlFor="dbRadio1">Override the current database</label>
                                </div>

                                <div className="field">
                                    <Tag className="ml-2" icon="pi pi-exclamation-triangle" value={`This will destroy database ${props.db?.name} and it's content`} severity="warning" rounded />
                                </div>

                            </div>
                            <div className="flex" >
                                <div className="field-radiobutton">
                                    <RadioButton inputId="dbRadio2" name="dbRadio" value="copy" checked={dbRadio === "copy"} onChange={(e) => setDbRadio(e.value)} />
                                    <label htmlFor="dbRadio2">Create a copy with the given name</label>
                                </div>
                            </div>
                        </div>)}
                    {(props.source !== "backup-expandable") && <div>
                        <div className="field">
                            Copy method:
                        </div>
                        <div className="field-radiobutton">
                            <RadioButton inputId="copyRadio1" name="copyRadio" value="one_time" checked={copyRadio === "one_time"} onChange={(e) => setCopyRadio(e.value)} />
                            <label htmlFor="copyRadio1">One-time copy</label>
                        </div>


                        {props.node?.id !== props?.db?.node &&

                            <div className="field-radiobutton">
                                <RadioButton inputId="copyRadio2" name="copyRadio" value="mirror" checked={copyRadio === "mirror"} onChange={(e) => setCopyRadio(e.value)} />
                                <label htmlFor="copyRadio2"> Mirror to database with given name</label>

                                <Tag className="ml-2" icon="pi pi-exclamation-triangle" value={`This will destroy mirror database and it's content`} severity="warning" rounded />
                            </div>}
                    </div>}

                </div>
            </div>

            <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
            {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
        </div>)

    }

    const renderPage2 = () => {

        return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

            <div>
                <div className="field grid flex align-items-start">
                    <label className="col-fixed" style={{ width: "220px" }}>{"Source endpoint port:"}</label>
                    <div className="col">
                        <span className="p-input-icon-right flex">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder=" port" value={sourcePort} onValueChange={(e) => {
                                setSourcePort(e.value);
                                setSourcePortError("check port");

                            }} />
                            <i className={isLoading === "mssql_mirror_source" && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                        </span>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                            <small id="name-help" className={`${(sourcePortError !== 'ok') ? "p-error" : ""} p-d-block`}>{sourcePortError}</small>
                        </div>
                    </div>
                    <div className="col-fixed">
                        <Button
                            className="p-button-sm"
                            label="Check"
                            style={{ width: "80px" }}
                            onClick={() => {
                                let error = validator(constraints)("port", sourcePort);
                                setSourcePortError(error);
                                if (error === null)
                                    dispatch(LayoutActions.checkPortExists(
                                        {
                                            node_id: props.db.node,
                                            port: sourcePort,
                                            type: types.MSSQL,
                                            purpose: "mssql_mirror_source"
                                        }))
                            }

                            }
                        />
                    </div>
                </div>
            </div>

            <div>
                <div className="field grid flex align-items-start">
                    <label className="col-fixed" style={{ width: "220px" }}>{"Target endpoint port:"}</label>
                    <div className="col">
                        <span className="p-input-icon-right flex">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder=" port" value={targetPort} onValueChange={(e) => {
                                setTargetPort(e.value);
                                setTargetPortError("check port");

                            }} />
                            <i className={isLoading === "mssql_mirror_target" && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                        </span>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                            <small id="name-help" className={`${(targetPortError !== 'ok') ? "p-error" : ""} p-d-block`}>{targetPortError}</small>
                        </div>
                    </div>
                    <div className="col-fixed">
                        <Button
                            className="p-button-sm"
                            label="Check"
                            style={{ width: "80px" }}
                            onClick={() => {
                                let error = validator(constraints)("port", targetPort);
                                setTargetPortError(error);
                                if (error === null)
                                    dispatch(LayoutActions.checkPortExists(
                                        {
                                            node_id: props.node?.id,
                                            type: types.MSSQL,
                                            port: targetPort,
                                            purpose: "mssql_mirror_target"
                                        }))
                            }

                            }
                        />
                    </div>
                </div>
            </div>
            <div>
                <div className="field grid">
                    <label className="col-fixed" style={{ width: "220px" }}>{"Async mirroring"}</label>
                    <div className="col">
                        <Checkbox onChange={() => setAsync(!async)} checked={async} />
                    </div>
                </div>
            </div>

            <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
            {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
        </div>)

    }

    const getHeader = () => {

        switch (pageIndex) {

            case 1:
                return `Copy database ${props?.db?.name} - Mirroring settings`

            default:
                return `Copy database ${props?.db?.name}`

        }

    }


    return (
        <Dialog
            className='copy-mssql_db'
            header={getHeader}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPage1()}
                {pageIndex === 1 && renderPage2()}
            </div>

        </Dialog>)
}

export default CopyDbDialog;