import { useSelector, useDispatch } from 'react-redux';
import { useEffect } from 'react';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dialog } from 'primereact/dialog';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import ErrorTextBox from '../../../components/errorTextBox';
import { loadingMsg} from '../../../config';


export default function MsSQLStatusDialog(props) {

    const { mssql_cluster_status, isLoading } = useSelector((state) => state.layout);
    const { errorMessage } = useSelector((state) => state.layout);


    const dispatch = useDispatch();

    useEffect(() => {
        if (props.node) {

            dispatch(LayoutActions.getMsSqlClusterStatus(props.node.id));

        }

    }, [props.node, dispatch])


    const nameBodyTemplate = (item) => {
        return (<div>{item.name}</div>);

    }


    const relayBodyTemplate = (item) => {

        return (<div>{item.relay_master_log}</div>);

    }
    const ioBodyTemplate = (item) => {

        return (<div>{item.slave_io}</div>);

    }

    const sqlBodyTemplate = (item) => {

        return (<div>{item.slave_sql}</div>);

    }

    const logBodyTemplate = (item) => {

        return (<div>{item.master_log}</div>);

    }

    const masterBodyTemplate = (item) => {

        return (<div>{item.master}</div>);

    }

    return (

        <Dialog
            className='mssql-cluster-status'
            header={`Cluster ${props.node?.name} status`}
            visible={props.visible}
            style={{ maxHeight: "90%" }}
            onHide={props.onHide}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <DataTable emptyMessage={!mssql_cluster_status && isLoading ? <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">No status data</div>} paginator alwaysShowPaginator={false} rows={10} rowsPerPageOptions={[10, 20, 50, 100]} value={mssql_cluster_status?.length ? mssql_cluster_status : []} >
                        <Column field="name" header="Slave" body={nameBodyTemplate} sortable />
                        <Column field="master" header="Master" body={masterBodyTemplate} sortable />
                        <Column field="slave_io" header="Slave_IO_Running" body={ioBodyTemplate} sortable />
                        <Column field="slave_sql" header="Slave_SQL_Running" body={sqlBodyTemplate} sortable />
                        <Column field="master_log" header="Master_Log_File" body={logBodyTemplate} sortable />
                        <Column field="relay_master_log" header="Relay_Master_Log_File" body={relayBodyTemplate} sortable />
                    </DataTable>
                    {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}

                </div>
            </div>

        </Dialog>);
}