import config from "../../../config";

const constraints = {
    port: {
        presence: {
            message: "Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535,
        }
    },
    clusterName: {
        presence: {
            message: "Cluster Name is required field.",
            allowEmpty: false
        },
        format: {
            pattern: /^\S*$/,
            message: "Cluster Name has invalid format."
        }
    },

    databaseUsername: {
        presence: {
            message: "Database Username is required field.",
            allowEmpty: false
        },
        length: {
            maximum: 31,
            tooLong: "max %{count} characters"
        },
        format:                                                         
        {
            pattern: config.usernameRegex,
            message: "Database Username has invalid format."
        },
        exclusion: {
            within:  ["root", "mysql", "public"],
          }
    },
    databasePassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.mysqlPasswordRegex,
            message: "Password has invalid format."
        }
    },

    osUsername: {
        presence: {
            message: "OS Username is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern: config.usernameRegex,
            message: "Os Username has invalid format."
        }
    },
    privateKey: {
        presence: {
            message: "Private Key is required field.",
            allowEmpty: false
        }
    },
    osPassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        length: {
            minimum: 5,
            tooShort: "min %{count} characters"
        },
    },
    mysqlRoot: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.mysqlPasswordRegex,
            message: "Password has invalid format."
        }
    },
    hostname: {
        presence: {
            message: "Hostname is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.hostNameRegex,
            message: "Host Name has invalid format."
        }
    },
    dataPath: {
        presence: {
            message: "path is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.pathRegex,
            message: "path has invalid format."
        }
    },
    sshPort: {
        presence: {
            message: "SSH Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535
        }
    }
}

export default constraints;