import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Tag } from 'primereact/tag';
import { useState, useEffect, useRef } from "react";
import { RadioButton } from "primereact/radiobutton";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { useSelector, useDispatch } from "react-redux";
import { InputText } from "primereact/inputtext";
import { LayoutActions } from "../../../store/layout/reducer";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ErrorTextBox from "../../../components/errorTextBox";
import { Checkbox } from "primereact/checkbox";
import getValidationError from '../../../util/errorPrompter';
import { Tooltip } from "primereact/tooltip";

const CopyDbDialog = (props) => {

    const [apiTooltip, setApiTooltip] = useState(" ");
    const [dbRadio, setDbRadio] = useState("copy");
    const [copyUsers, setCopyUsers] = useState(false);
    // const [copyRadio, setCopyRadio] = useState("one_time");
    const [copyName, setCopyName] = useState("");
    const [nameError, setNameError] = useState("");
    const { accessToken } = useSelector((state) => state.app);
    const { valid_name, errorMessage, isLoading } = useSelector((state) => state.layout);
    const customEvent = new Event('beforeinput', { bubbles: true });
    const submitButtonRef = useRef(null);
    

    const dispatch = useDispatch();

    const onRestApiClick = async () => {

        if (dbRadio === "copy" && !props.new) {

            if (!nameError) setNameError("name not checked");

            if (nameError === "name available") {
                setApiTooltip(await copyToClipboard(createAPIcommand({
                    "db_id": props.db?.name,
                    "node_id": props.node?.name,
                    "source_id": props.db?.host,
                    "option": !props.new ? dbRadio : null,
                    "copy_option": "one_time",
                    "copy_users": copyUsers,
                    "copy_name": copyName,
                    "db_exists": !props?.new

                }, accessToken, "copy_mysql_database/")));

            }
            else setApiTooltip(getValidationError());


        }
        else {

            setApiTooltip(await copyToClipboard(createAPIcommand({
                "db_id": props.db?.name,
                "node_id": props.node?.name,
                "source_id": props.db?.host,
                "option": !props.new ? dbRadio : null,
                "copy_option": "one_time",
                "copy_users": copyUsers,
                "db_exists": !props?.new

            }, accessToken, "copy_mysql_database/")));

        }

    }

    const renderFooter = () => {
        return (
            <div style={{ display: "flex", justifyContent: "space-between" }}>
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    label={"Submit"}
                    disabled={errorMessage}
                    className="submit-button"
                    icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                    iconPos="right"
                    ref={submitButtonRef}
                    onClick={handleOk} autoFocus />

                <Tooltip   target='.submit-button' showEvent='beforeinput' position='top' >
                    {getValidationError()}
                </Tooltip>
            </div>
        );

    }

    const validateDbName = () => {
        setNameError("");

        let errors = [];
        let error = validator(constraints)("name", copyName);
        (error == null || errors.push(error)) && setNameError(error);
        if (errors.length === 0) dispatch(LayoutActions.checkMySqlDatabaseName({ node_id: props.node.id, name: copyName }));

    }


    useEffect(() => {

        if (valid_name) {
            if (valid_name?.error)
                setNameError(valid_name.error);
            else setNameError(valid_name?.response)
        }

    }, [valid_name]);


    const handleOk = () => {

        if (dbRadio === "copy" && !props.new) {

            if (!nameError) setNameError("name not checked");

            if (nameError === "name available") {
                
                props.onOkClick(
                    {
                        "db_id": props.db?.id,
                        "node_id": props.node?.id,
                        "option": !props.new ? dbRadio : null,
                        "copy_option": "one_time",
                        "copy_name": copyName,
                        "copy_users": copyUsers,
                        "source_id": props.db?.node,
                        "db_exists": !props?.new
                    });
            }
            else submitButtonRef.current.dispatchEvent(customEvent);


        }
        else {
            
            props.onOkClick(
                {
                    "db_id": props.db?.id,
                    "node_id": props.node?.id,
                    "option": !props.new ? dbRadio : null,
                    "copy_option": "one_time",
                    "copy_users": copyUsers,
                    "source_id": props.db?.node,
                    "db_exists": !props?.new
                });

        }

    }


    return (
        <Dialog
            className='copy-mongo_db'
            header={`Copy database ${props.db?.name}`}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="field">{`This will copy the database from ${props.db?.host} to ${[props.node?.name]}`}</div>
                    <div className="field grid">
                        <div className="col">
                            {
                                (!props.new || props.source === "backup-expandable") && (<div>
                                    <div className="field">
                                        Database {props.db?.name} already exists in this cluster
                                    </div>
                                    <div className="flex">

                                        <div className="field-radiobutton">
                                            <RadioButton inputId="dbRadio1" name="dbRadio" value="override" checked={dbRadio === "override"} onChange={(e) => setDbRadio(e.value)} />
                                            <label htmlFor="dbRadio1">Override the current database</label>
                                        </div>

                                        <div className="field">
                                            <Tag className="ml-2" icon="pi pi-exclamation-triangle" value={`This will destroy database ${props.db?.name} and it's content`} severity="warning" rounded />
                                        </div>

                                    </div>
                                    <div className="field-radiobutton">
                                        <RadioButton inputId="dbRadio2" name="dbRadio" value="copy" checked={dbRadio === "copy"} onChange={(e) => setDbRadio(e.value)} />
                                        <label htmlFor="dbRadio2">Create a copy with given name:</label>
                                    </div>
                                    <div className="field grid">

                                        <div className="col-fixed">

                                            <InputText disabled={dbRadio !== "copy"} className="p-inputtext-sm filter-field" placeholder="Copy Name" value={copyName} onChange={(e) => {

                                                setCopyName(e.target.value);
                                                setNameError("name not checked");
                                                dispatch(LayoutActions.setValidName(undefined));

                                            }} />
                                            <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
                                        </div>
                                        <div className="col-fixed">

                                            <Button style={{ width: "80px" }} label="Check" disabled={dbRadio !== "copy"} onClick={validateDbName} className="p-button-sm" />

                                        </div>

                                    </div>
                                </div>)}
                            {(props.source !== "backup-expandable") && <div>
                                {/* <div className="field">
                                    Copy method:
                                </div>
                                <div className="field-radiobutton">
                                    <RadioButton inputId="copyRadio1" name="copyRadio" value="one_time" checked={copyRadio === "one_time"} onChange={(e) => setCopyRadio(e.value)} />
                                    <label htmlFor="copyRadio1">One-time copy</label>
                                </div>
                                <div className="field-radiobutton">
                                    <RadioButton inputId="copyRadio2" name="copyRadio" value="live" checked={copyRadio === "live"} onChange={(e) => setCopyRadio(e.value)} />
                                    <label htmlFor="copyRadio2">Ongoing live replica</label>
                                </div> */}

                                {!(props?.node?.id === props.db.node && dbRadio === "override") && <div className="field grid">
                                    <label htmlFor="cb1" className="p-checkbox-label col-fixed" >Also copy users affiliated with this database</label>
                                    <div className="col-fixed"> <Checkbox value={copyUsers} checked={copyUsers} onChange={() => setCopyUsers(!copyUsers)} /> </div>
                                </div>
                                }

                            </div>}

                        </div>
                    </div>

                    <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
                    {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                </div>
            </div>

        </Dialog>)
}

export default CopyDbDialog;