import { useSelector, useDispatch } from 'react-redux';
import { useEffect } from 'react';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dialog } from 'primereact/dialog';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import ErrorTextBox from '../../../components/errorTextBox';
import { loadingMsg} from '../../../config';


export default function MySQLStatusDialog(props) {

    const { mysql_cluster_status, isLoading } = useSelector((state) => state.layout);
    const { errorMessage } = useSelector((state) => state.layout);
    const sourceAlias =  props.node.version > 5.7 ? 'Source' : 'Master'
    const replicaAlias = props.node.version > 5.7 ? 'Replica' : 'Slave'


    const dispatch = useDispatch();

    useEffect(() => {
        if (props.node) {

            dispatch(LayoutActions.getMySqlClusterStatus(props.node.id));

        }

    }, [props.node, dispatch])


    const nameBodyTemplate = (item) => {
        return (<div>{item[`${replicaAlias}_Host`]}</div>);

    }


    const relayBodyTemplate = (item) => {

        return (<div>{item[`Relay_${sourceAlias}_Log_File`]}</div>);

    }
    const ioBodyTemplate = (item) => {

        return (<div>{item[`${replicaAlias}_IO_Running`]}</div>);

    }

    const sqlBodyTemplate = (item) => {

        return (<div>{item[`${replicaAlias}_SQL_Running`]}</div>);

    }

    const logBodyTemplate = (item) => {

        return (<div>{item[`${sourceAlias}_Log_File`]}</div>);

    }

    const masterBodyTemplate = (item) => {

        return (<div>{item[`${sourceAlias}_Host`]}</div>);

    }

    return (

        <Dialog
            className='mysql-cluster-status'
            header={`Cluster ${props.node?.name} status`}
            visible={props.visible}
            style={{ maxHeight: "90%" }}
            onHide={props.onHide}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <DataTable emptyMessage={!mysql_cluster_status && isLoading ?  <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">No status data</div>} paginator alwaysShowPaginator={false} rows={10} rowsPerPageOptions={[10, 20, 50, 100]} value={mysql_cluster_status?.length ? mysql_cluster_status : []} >
                        <Column field="name" header="Replica" body={nameBodyTemplate} sortable />
                        <Column field="master" header="Source" body={masterBodyTemplate} sortable />
                        <Column field="source_io" header="Source_IO_Running" body={ioBodyTemplate} sortable />
                        <Column field="source_sql" header="Source_SQL_Running" body={sqlBodyTemplate} sortable />
                        <Column field="source_log" header="Source_Log_File" body={logBodyTemplate} sortable />
                        <Column field="relay_source_log" header="Relay_Source_Log_File" body={relayBodyTemplate} sortable />
                    </DataTable>
                    {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}

                </div>
            </div>

        </Dialog>);
}