import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";
import config, { mysqlVersionOptions } from "../../../config";
import validator from "../../../util/validator";
import constraints from "./constraints";
import { Checkbox } from "primereact/checkbox";
import { Tag } from "primereact/tag";


const UpgradeMySQLNodeDialog = (props) => {
  const targetVersionOption = [mysqlVersionOptions[mysqlVersionOptions.findIndex(((v) => Number(props.node?.version) === Number(v.value))) - 1]]
  const [upgradeConfirm, setUpgradeConfirm] = useState("");
  const [upgradeConfirmError, setUpgradeConfirmError] = useState("");
  const [targetVersion, setTargetVersion] = useState(null);
  const [targetVersionError, setTargetVersionError] = useState("");

  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);
  const [standbysConfirm, setStandbysConfirm] = useState(false);
  const [dowmtimeConfirm, setDowntimeConfirm] = useState(false);



  const onRestApiClick = async () => {


    let errors = validate();

    if (errors.length === 0 && dowmtimeConfirm && standbysConfirm) {

      setApiTooltip(await copyToClipboard(createAPIcommand({
        "node_id": props.node.name,
        "target_version": targetVersion
      }, accessToken, "upgrade_mysql_node/")));

    }
    else setApiTooltip(" ");

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("targetVersion", targetVersion);
    (error == null || errors.push(error)) && setTargetVersionError(error);
    error = validator(constraints)("upgradeConfirm", upgradeConfirm);
    (error == null || errors.push(error)) && setUpgradeConfirmError(error);

    return errors;

  }

  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0 && dowmtimeConfirm && standbysConfirm) {

      props.onOkClick(

        {
          "node_id": props.node?.id,
          "target_version": targetVersion,
        }

      );

    }


  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='upgrade-mysql-cluster'
      header={`Upgrade MySQL cluster`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">

              <div className="field">
                <div className="mb-2">Target Version:</div>
                <Tag severity="info" className="p-0 mb-3">
                  <div className="flex align-items-center text-base"><i className="pi pi-info-circle ml-1" /> <div className="ml-1"> Please be aware that you can upgrade only to next major MySQL version </div></div>
                </Tag>
                <Dropdown className="p-inputtext-sm"
                  value={targetVersion}
                  options={targetVersionOption}
                  onChange={(e) => setTargetVersion(e.value)}
                  placeholder="Select version to upgrade for"
                  emptyMessage="no versions available"
                />
                <div className="error-help">{<small className="p-error p-d-block">{targetVersionError}</small>}</div>
              </div>


              <div className="flex align-items-start mb-2">

                <Checkbox checked={standbysConfirm} onChange={(e) => setStandbysConfirm(e.checked)} className="mr-2" inputId="standbys_confirm" /> <label htmlFor="standbys_confirm">I understand that upgrade will affect cluster standbys  </label>

              </div>



              <div className="flex align-items-start mb-5">

                <Checkbox checked={dowmtimeConfirm} onChange={(e) => setDowntimeConfirm(e.checked)} className="mr-2" inputId="downtime_confirm" /> <label htmlFor="downtime_confirm">I understand that cluster will be down during the upgrade  </label>

              </div>
              <div className="field grid flex align-items-center">
                <div className="col">

                  <div className="mt-2 mb-2"> {`Please type "${config.upgradeConfirm}" here in order to continue`}</div>
                  <InputText
                    className="p-inputtext-sm"
                    placeholder={`Please type "${config.upgradeConfirm}" here in order to continue`}
                    value={upgradeConfirm}
                    onChange={(e) => setUpgradeConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{upgradeConfirmError}</small>}</div>
                </div>
              </div>

            </div>

          </div>
          {errorMessage ? <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} /> : null}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />

        </div>
      </div>

    </Dialog>)

};

export default UpgradeMySQLNodeDialog;