import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { RadioButton } from 'primereact/radiobutton';
import { Dropdown } from "primereact/dropdown";
import { useState } from "react";
import { types } from "../../config";


const NodeTypeDialog = (props) => {

    const [nodeOptions, setNodeOptions] = useState(types.ELASTIC);

    const redisTypeOptions = [
        { label: "Redis Standalone", value: "standalone", disabled: nodeOptions !== types.REDIS },
        { label: "Redis Cluster", value: "cluster", disabled: nodeOptions !== types.REDIS },
    ];

    const [redisOption, setRedisOption] = useState("cluster");
    const [mongoOption, setMongoOption] = useState("replica");

    const handleOk = () => {


        props.onOkClick(nodeOptions, redisOption, mongoOption);


    }

    const renderFooter = () => {
        return (
            <div style={{ display: "flex", justifyContent: "space-between" }}>
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    label={"Next"}
                    icon="pi pi-arrow-right"
                    iconPos="right"
                    onClick={handleOk} autoFocus />
            </div>
        );

    }



    return (

        <Dialog
            className='choose-node-type'
            header={"Choose node type"}
            visible={props.visible}
            style={{ width: '30vw', maxHeight: "90%" }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

                    <div className="field grid">
                        <div className="col">
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption1" name="nodeOption" value={types.ELASTIC} checked={nodeOptions === types.ELASTIC} onChange={(e) => setNodeOptions(e.value)} />
                                <label htmlFor="nodeOption1">Elasticsearch</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption2" name="nodeOption" value={types.MARIADB} checked={nodeOptions === types.MARIADB} onChange={(e) => setNodeOptions(e.value)} />
                                <label htmlFor="nodeOption2">MariaDB</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption3" name="nodeOption" value="replica" checked={mongoOption === "replica" && nodeOptions === types.MONGO }  onChange={(e) =>{ setNodeOptions(types.MONGO); setMongoOption(e.value)}} />
                                <label htmlFor="nodeOption3">MongoDB</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption4" name="nodeOption" value="sharded"  checked={mongoOption === "sharded" && nodeOptions === types.MONGO} onChange={(e) =>{ setNodeOptions(types.MONGO); setMongoOption(e.value)}} />
                                <label htmlFor="nodeOption4">MongoDB Sharded Cluster</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption5" name="nodeOption" value={types.MSSQL} checked={nodeOptions === types.MSSQL} onChange={(e) => setNodeOptions(e.value)} />
                                <label htmlFor="nodeOption5">SQL Server</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption6" name="nodeOption" value={types.MYSQL} checked={nodeOptions === types.MYSQL} onChange={(e) => setNodeOptions(e.value)} />
                                <label htmlFor="nodeOption6">MySQL</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption7" name="nodeOption" value={types.ORACLE_CDB} checked={nodeOptions === types.ORACLE_CDB} onChange={(e) => setNodeOptions(e.value)} />
                                <label htmlFor="nodeOption7">Oracle</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption8" name="nodeOption" value={types.POSTGRES} checked={nodeOptions === types.POSTGRES} onChange={(e) => setNodeOptions(e.value)} />
                                <label htmlFor="nodeOption8">PostgreSQL</label>
                            </div>
                            <div className="field-radiobutton">
                                <RadioButton inputId="nodeOption9" name="nodeOption" value={types.REDIS} checked={nodeOptions === types.REDIS} onChange={(e) => setNodeOptions(e.value)} />
                                <label htmlFor="nodeOption9">Redis</label>
                            </div>
                            {nodeOptions === types.REDIS && <div className="field">
                                <Dropdown value={redisOption} options={redisTypeOptions} onChange={(e) => setRedisOption(e.value)}></Dropdown>
                            </div>}
                        </div>
                    </div>


                </div>
            </div>

        </Dialog>)

};

export default NodeTypeDialog;