import config from "../../../config";

const constraints = {
    databaseUsername: {
        presence: {
            message: "Database Username is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern: config.oracleNameRegex,
            message: "Database Username has invalid format."
        }
    },
    databasePassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.passwordRegex,
            message: "Password has invalid format."
        }
    },
    serviceName: {
        presence: {
            message: "Service Name is required field.",
            allowEmpty: false
        },

        format: {

            pattern: "^[a-zA-Z0-9]*$",
            message: "Service Name has invalid format."

        }
    },
    port: {
        presence: {
            message: "Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535,
        }
    },
    osUsername: {
        presence: {
            message: "OS Username is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern: config.usernameRegex,
            message: "Os Username has invalid format."
        }
    },
    privateKey: {
        presence: {
            message: "Private Key is required field.",
            allowEmpty: false
        }
    },
    osPassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        length: {
            minimum: 5,
            tooShort: "min %{count} characters"
        },
    },
    hostname: {
        presence: {
            message: "Hostname is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.hostNameRegex,
            message: "Host Name has invalid format."
        }
    },
    sshPort: {
        presence: {
            message: "SSH Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535
        }
    },
    oracleHome: {
        presence: {
            message: "Oracle home is required field.",
            allowEmpty: false
        },

        format:
        {

            pattern: config.pathRegex,
            message: "Oracle home has invalid format."


        }
    },
    cluster123Directory: {
        presence: {
            message: "cluster123 Directory is required field.",
            allowEmpty: false
        },

        format:                                                         
        {
            pattern: config.pathRegex,
            message: "cluster123 Directory has invalid format."
        }
    },
}

export default constraints;