import { useState, useEffect, useRef } from 'react';
import { useSelector, useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Checkbox } from 'primereact/checkbox';
import { Slider } from 'primereact/slider';
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Tooltip } from 'primereact/tooltip';
import validator from "../../../util/validator";
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import { ConfigActions } from '../../../store/config/reducer';
import { base64Encode } from '../../../util/base64Encode';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ApiBar from '../../../components/apiBar';
import ErrorTextBox from '../../../components/errorTextBox';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { clusterSizeOptions, types } from '../../../config';
import getValidationError from '../../../util/errorPrompter';
import { footer } from '../../../util/passwordHelpers';


const AddCDBDialog = (props) => {
    const { oracle_home, max_pdbs, errorMessage, isLoading } = useSelector((state) => state.layout);
    const { config } = useSelector((state) => state.config);
    const { user } = useSelector((state) => state.app);
    const dispatch = useDispatch();
    const [pageIndex, setPageIndex] = useState(0);


    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const caller = useRef('submit-button');

    const onPopupSubmit = (accept) => onSaveClick(accept);

    // First Page elements
    const [databaseUsername, setDatabaseUsername] = useState("sys");
    const [databasePassword, setDatabasePassword] = useState("");
    const [serviceName, setServiceName] = useState("");
    const [port, setPort] = useState(1521);
    const [osUsername, setOsUsername] = useState("root");
    const [osPassword, setOsPassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState(22);
    const [privateKey, setPrivateKey] = useState("");

    // Second Page elements
    const [oracleHome, setOracleHome] = useState("");
    const [cluster123Directory, setCluster123Directory] = useState("");
    const [maxNumberOfPdbs, setMaxNumberOfPdbs] = useState(1);
    const [maxNumberOfOpenPdbs, setMaxNumberOfOpenPdbs] = useState(1);
    const [pdbLimit, setPDBLimit] = useState(1);
    const [maxOpenPDBs, setMaxOpenPDBs] = useState(1);
    const [allowChildrenFromAllUsers, setAllowChildrenFromAllUsers] = useState(false);
    const customEvent = new Event('beforeinput', { bubbles: true });
    const submitButtonRef = useRef(null);




    // First Page element errors
    const [databaseUsernameError, setDatabaseUsernameError] = useState("");
    const [databasePasswordError, setDatabasePasswordError] = useState("");
    const [serviceNameError, setServiceNameError] = useState("");
    const [portError, setPortError] = useState("");
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");

    // Second Page element errors
    const [oracleHomeError, setOracleHomeError] = useState("");
    const [cluster123DirectoryError, setCluster123DirectoryError] = useState("");
    const [maxNumberOfPdbsError, setMaxNumberOfPdbsError] = useState("");
    const [maxNumberOfOpenPdbsError, setMaxNumberOfOpenPdbsError] = useState("");

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);

    const [pdbSize, setPDBSize] = useState(clusterSizeOptions[0].value);

    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);

    useEffect(() => {

        if (!config) dispatch(ConfigActions.getConfig());
    }, [config, dispatch]);

    useEffect(() => {

        if (props.visible === false) {
            setPageIndex(0);

            // First page elements and errors
            setDatabaseUsername("sys");
            setDatabaseUsernameError("");
            setDatabasePassword("");
            setDatabasePasswordError("");
            setServiceName("");
            setServiceNameError("");
            setPort(1521);
            setPortError("");
            setOsUsername("root");
            setOsUsernameError("");
            setOsPassword("");
            setOsPasswordError("");
            setHostname("");
            setHostnameError("");
            setSshPort(22);
            setSshPortError("");
            setPrivateKey("");
            setPrivateKeyError("");

            // Second page elements and errors
            setOracleHome("")
            setOracleHomeError("");
            setCluster123Directory("");
            setCluster123DirectoryError("");
            setMaxNumberOfPdbs(1);
            setMaxNumberOfPdbsError("");
            setMaxNumberOfOpenPdbs(1);
            setMaxNumberOfOpenPdbsError("");
            setAllowChildrenFromAllUsers(false);
            setApiTooltip(" ");
        }

    }, [props.visible]);

    useEffect(() => {

        if (oracle_home?.data)
            setOracleHome(oracle_home.data.oracle_home);

        if (oracle_home?.message)
            setOracleHomeError(oracle_home.message);

    }, [oracle_home]);


    useEffect(() => {

        if (oracle_home && !errorMessage && pageIndex === 0) {

            dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))
        }

    }, [oracle_home, errorMessage, pageIndex, hostname, osUsername, osPassword, privateKey, sshPort, dispatch]);

    useEffect(() => {
        if (max_pdbs) {

            setPDBLimit(max_pdbs.data);
        }

    }, [max_pdbs]);


    const onFirstNextClick = () => {
        const errors = [];

        let error = validator(constraints)("serviceName", serviceName);
        (error == null || errors.push(error)) && setServiceNameError(error);

        error = validator(constraints)("port", port);
        (error == null || errors.push(error)) && setPortError(error);

        error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);

        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }




        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        if (errors.length === 0) {
            const data =
            {

                "user": databaseUsername,
                "pwd": base64Encode(databasePassword),
                "port": port,
                "host": hostname,
                "sname": serviceName


            }
            if (!oracle_home) dispatch(LayoutActions.getOracleHome(data));
            dispatch(LayoutActions.getMaxPDB(data));

            setCluster123Directory(`${config.oracle_sys.cdb_path}/${serviceName}`);
        }
    }

    const onSecondBackClick = () => {
        dispatch(LayoutActions.setErrorMessage(""));
        setOracleHome("");
        dispatch(LayoutActions.setOracleHome(null));
        setPageIndex(0);
    }



    const onSaveClick = async (dropInventories) => {

        const validate = () => {

            const errors = [];
            let error = validator(constraints)("databaseUsername", databaseUsername);
            (error == null || errors.push(error)) && setDatabaseUsernameError(error);
    
            error = validator(constraints)("databasePassword", databasePassword);
            (error == null || errors.push(error)) && setDatabasePasswordError(error);
            error = validator(constraints)("oracleHome", oracleHome);
            (error == null || errors.push(error)) && setOracleHomeError(error);
    
            error = validator(constraints)("cluster123Directory", cluster123Directory);
            (error == null || errors.push(error)) && setCluster123DirectoryError(error);
    
            return errors
    
        }
        
        const errors = validate();

        if (errors.length === 0) {
            const ex_params =
            {

                "oracle_home": oracleHome,
                "max_num_pdbs": maxNumberOfPdbs,
                "max_open_pdbs": maxNumberOfOpenPdbs,
                "allow_children_from_all_users": allowChildrenFromAllUsers,
                "found_pdbs_sizing": pdbSize,


            }

            const clusterId = {
                name: serviceName
            };

            const host = {
                name: hostname,
                ssh_port: sshPort,
                username: osUsername,
                password: base64Encode(osPassword),
                private_key: privateKey,
            };

            const data =
            {
                "name": serviceName,
                "port": port,
                "created_by_id": user?.data?.id,
                "cluster_role": "CDB",
                "drop_inventories": dropInventories,
                "cluster": clusterId,
                "database_user": databaseUsername,
                "database_pwd": base64Encode(databasePassword),
                "dir_123Cluster": cluster123Directory,
                "host": host,
                "sizing": clusterSize,
                "ex_params": ex_params,
            }
            if (caller.current === 'submit-button'){

                if (props.onOkClick)
                    props.onOkClick(data);
            }
            else setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_mysql_master/")))

            }
           

        else caller.current === 'submit-button' ? submitButtonRef.current.dispatchEvent(customEvent) : setApiTooltip(getValidationError());
    }

    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            iconPos="right"
                            id='submit-button'
                            className='submit-button'
                            ref={submitButtonRef}
                            onClick={() => {
                               caller.current = 'submit-button';
                               dispatch(LayoutActions.checkInventoryExists({ 'name': `${serviceName}`, 'type': types.ORACLE_CDB }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))
                            }} autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Service Name:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Service Name" value={serviceName} onChange={(e) => setServiceName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{serviceNameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Port" value={port} onValueChange={(e) => setPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{portError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                disabled={privateKey}
                                placeholder="OS Password"
                                value={osPassword}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    const renderPageSecond = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "220px" }}>{"Database Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Database Username" value={databaseUsername} onChange={(e) => setDatabaseUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{databaseUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "220px" }}>{"Database Password:"}</label>
                        <div className="col">
                            <Password
                                feedback={false}
                                className="p-inputtext-sm" placeholder="Database Password"
                                value={databasePassword}
                                onChange={(e) => setDatabasePassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{databasePasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Oracle Home:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Oracle Home" value={oracleHome} onChange={(e) => setOracleHome(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{oracleHomeError}</small>
                    </div>
                </div>
                <div className="form-inner-part">
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Create a 123cluster directory at:"}</label>
                        <div className="col">
                            <span className="p-input-icon-right">
                                <i className="pi pi-info-circle directory" />
                                <Tooltip target=".pi.pi-info-circle.directory"
                                    content="This directory is shared by the container and all pluggable databases on it." />

                                <InputText className="p-inputtext-sm" placeholder="123cluster directory" value={cluster123Directory} onChange={(e) => setCluster123Directory(e.target.value)} />
                            </span>
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{cluster123DirectoryError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Max number of PDBs: " + maxNumberOfPdbs}</label>
                        <div style={{ paddingLeft: "18px" }} className="col">
                            <Slider
                                style={{ marginTop: "10px" }}
                                min={1}
                                max={pdbLimit}
                                value={maxNumberOfPdbs}
                                onChange={(e) => {
                                    setMaxNumberOfPdbs(e.value);
                                    if (e.value < maxNumberOfOpenPdbs) setMaxNumberOfOpenPdbs(e.value);
                                    setMaxOpenPDBs(e.value);

                                }} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{maxNumberOfPdbsError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Max number of open PDBs: " + maxNumberOfOpenPdbs}</label>
                        <div style={{ paddingLeft: "18px" }} className="col">
                            <Slider
                                style={{ marginTop: "10px" }}
                                min={1}
                                max={maxOpenPDBs}
                                value={maxNumberOfOpenPdbs}
                                onChange={(e) => {
                                    setMaxNumberOfOpenPdbs(e.value);

                                }} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{maxNumberOfOpenPdbsError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Allow children from all users:"}</label>
                        <div className="col">
                            <Checkbox onChange={() => setAllowChildrenFromAllUsers(!allowChildrenFromAllUsers)} checked={allowChildrenFromAllUsers} />
                            <i style={{ marginLeft: "5px" }} className="pi pi-info-circle childrens" />
                            <Tooltip
                                target=".pi.pi-info-circle.childrens"
                                content="Other 123cluster users may create pdbs in the container database." />
                        </div>
                    </div>
                </div>


                <div>
                    <div className='field grid'>

                        <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                        <div className='col'>

                            <Dropdown className="p-inputtext-sm"
                                value={clusterSize}
                                onChange={(e) => setClusterSize(e.value)}
                                options={clusterSizeOptions}
                                placeholder="Size options" />

                        </div>

                    </div>

                </div>




                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}

                <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => {
                    caller.current = 'rest-api-bar';
                    dispatch(LayoutActions.checkInventoryExists({ 'name': `${serviceName}`, 'type': types.ORACLE_CDB }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))
                }
                } />

                {oracle_home?.data?.nodes_count > 0 && <div className='field grid'>

                    <div className='col'>
                        <p>Found {oracle_home?.data?.nodes_count} existing PDBs.</p>
                        <div className="flex flex-row align-items-center"><p>These PDBs will be imported and managed as a</p> <Dropdown className="p-inputtext-sm" style={{ width: "30%" }} value={pdbSize}
                            onChange={(e) => setPDBSize(e.value)}
                            options={clusterSizeOptions}
                            placeholder="Size options" />
                            <p>PDB size.</p>
                        </div>

                        <p>You can update the PDB details at the PDB details page later.</p>
                    </div>

                </div>}
            </div>



        );
    }


    return (
        <Dialog
            className='add-cdb'
            header={"Register a new CDB"}
            visible={props.visible}
            style={{ width: '50vw', maxHeight: "90%" }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller.current)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>

        </Dialog>
    );
}


export default AddCDBDialog;