import { useState, useEffect, useRef } from 'react';
import { useSelector, useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { Password } from "primereact/password";
import { Checkbox } from 'primereact/checkbox';
import { Dropdown } from 'primereact/dropdown';
import { Slider } from 'primereact/slider';
import { Dialog } from "primereact/dialog";
import { Tooltip } from 'primereact/tooltip';
import ApiBar from '../../../components/apiBar';
import validator from "../../../util/validator";
import { base64Encode } from '../../../util/base64Encode';
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import ErrorTextBox from '../../../components/errorTextBox';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { types, clusterRoles } from '../../../config';
import createAPIcommand from '../../../util/createApiCommand';
import copyToClipboard from '../../../util/copyToClipboard';
import getValidationError from '../../../util/errorPrompter';
import { footer } from '../../../util/passwordHelpers';

const AddPDBDialog = (props) => {
    const { user, accessToken } = useSelector((state) => state.app);
    const { db_file_dest, valid_name, isLoading, errorMessage } = useSelector((state) => state.layout);
    const dispatch = useDispatch();


    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [caller, setCaller] = useState('submit-button');

    const onPopupSubmit = (accept) => caller === 'rest-api-bar' ? onRestApiClick(accept) : onSaveClick(accept);

    const [name, setName] = useState("");
    const [dbUserName, setDbUserName] = useState("clusteradmin");
    const [dbPassword, setDbPassword] = useState("");
    const [startPdbWithCdb, setStartPdbWithCdb] = useState(false);
    const [fileDestination, setFileDestination] = useState("");
    const [alwaysUseThisLocation, setAlwaysUseThisLocation] = useState(false);
    const [pluggableDatabaseSize, setPluggableDatabaseSize] = useState("small");
    const [replicateToStandby, setReplicateToStandby] = useState("none");
    const [startStandbyWhenStarted, setStartStandbyWhenStarted] = useState(false);
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const customEvent = new Event('beforeinput', { bubbles: true });
    const submitButtonRef = useRef(null);





    const [nameError, setNameError] = useState("");
    const [dbUserNameError, setDbUserNameError] = useState("");
    const [dbPasswordError, setDbPasswordError] = useState("");
    const [fileDestinationError, setFileDestinationError] = useState("");
    const [pageIndex, setPageIndex] = useState(0);

    const [apiTooltip, setApiTooltip] = useState(" ");

    const pluggableDatabaseSizeOptions = [
        { label: "Small pluggable database", value: "small" },
        { label: "Medium pluggable database", value: "medium" },
        { label: "Large pluggable database", value: "large" },
        { label: "Very large pluggable database", value: "very_large" },
    ];

    const replicateToStandbyOptions = [
        { label: "None", value: "none" }
    ];

    const validatePdb = () => {
        let error = validator(constraints)("name", name);
        setNameError(error);
        if (!error) {
            dispatch(LayoutActions.checkPDBName({ "cdb_id": props.node.id, "pdb_name": name }));
        }
    }

    useEffect(() => {
        if (valid_name) {

            if (valid_name?.error) {
                setNameError(valid_name.error);
            }
            else setNameError(valid_name?.response)
        }
    }, [valid_name]);

    useEffect(() => {

        if(props.visible === false) {
            
            setName("");
            setNameError("");
            setDbUserName("clusteradmin");
            setDbUserNameError("");
            setDbPassword("");
            setDbPasswordError("");
            setFileDestination("");
            setFileDestinationError("");
            setStartPdbWithCdb(false);
            setAlwaysUseThisLocation(false);
            setAutomaticDelete(false);
            setStartStandbyWhenStarted(false);
            setPluggableDatabaseSize("small");
            setReplicateToStandby("none");
            setAutomaticDeleteDays(1);
            setApiTooltip("");
            setPageIndex(0);
        }
       
        if (props.visible) {

            dispatch(LayoutActions.getDBFileDest(props.node.id));

        }

    }, [props.visible, props.node.id, dispatch]);

    useEffect(() => {
        if (db_file_dest) {

            setFileDestination(db_file_dest?.file_dest);
        }

    }, [db_file_dest])

    const onRestApiClick = async (dropInventories) => {

        let errors = validate();

        if (errors.length === 0) {

            const clusterId = {
                name: name
            };


            const exParams = {
                'cdb_id': props.node.id,
                'file_dest': fileDestination,
                'also_start_standby': startStandbyWhenStarted,
                'always_use_this_location': alwaysUseThisLocation,
                'replicate_to_standby': replicateToStandby,
            };

            const data = {

                "name": name,
                "type": types.ORACLE_PDB,
                "port": props.node.port,
                "host": props.node.host_id,
                "created_by": user.id,
                "last_update_by": user.id,
                "database_user": dbUserName,
                "database_pwd": base64Encode(dbPassword),
                "cluster_role": clusterRoles.PDB,
                "sizing": pluggableDatabaseSize,
                "ind_autostartup": startPdbWithCdb,
                "auto_delete_days": automaticDeleteDays,
                "auto_delete_disabled": !automaticDelete,
                "auto_delete": automaticDelete,
                "cluster": clusterId,
                "drop_inventories": dropInventories,
                "ex_params": exParams

            }


            setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_pdb/")))
        }
        else setApiTooltip(getValidationError());


    }

    const validate = () => {

        const errors = [];

        let error = validator(constraints)("name", name);
        (error == null || errors.push(error)) && setNameError(error);

        error = validator(constraints)("dbUserName", dbUserName.toLowerCase());
        (error == null || errors.push(error)) && setDbUserNameError(error);

        error = validator(constraints)("dbPassword", dbPassword);
        (error == null || errors.push(error)) && setDbPasswordError(error);

        error = validator(constraints)("fileDestination", fileDestination);
        (error == null || errors.push(error)) && setFileDestinationError(error);

        return errors;

    }


    const onSaveClick = (dropInventories) => {

        if (props.onOkClick) {

            let errors = validate();

            if (errors.length === 0) {

                const clusterId = {
                    name: name
                };


                const exParams = {
                    'cdb_id': props.node.id,
                    'file_dest': fileDestination,
                    'also_start_standby': startStandbyWhenStarted,
                    'always_use_this_location': alwaysUseThisLocation,
                    'replicate_to_standby': replicateToStandby,
                };

                const data = {

                    "name": name,
                    "type": types.ORACLE_PDB,
                    "port": props.node.port,
                    "host": props.node.host_id,
                    "created_by": user.id,
                    "last_update_by": user.id,
                    "database_user": dbUserName,
                    "database_pwd": base64Encode(dbPassword),
                    "cluster_role": clusterRoles.PDB,
                    "sizing": pluggableDatabaseSize,
                    "ind_autostartup": startPdbWithCdb,
                    "auto_delete_days": automaticDeleteDays,
                    "auto_delete_disabled": !automaticDelete,
                    "auto_delete": automaticDelete,
                    "cluster": clusterId,
                    "drop_inventories": dropInventories,
                    "ex_params": exParams

                }

                props.onOkClick(data);


            }
            else submitButtonRef.current.dispatchEvent(customEvent);

        }


    }

    const renderFooter = () => {
        if (pageIndex === 1) return (<div className='flex justify-content-between'>
            <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
            <Button label={"Back"} icon="pi pi-arrow-left" onClick={() => {
                dispatch(LayoutActions.setErrorMessage(""));
                setPageIndex(0);
            }} className="p-button-secondary" />
            <Button
                label={"Submit"}
                icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                iconPos="right"
                className='submit-button'
                ref={submitButtonRef}
                onClick={() => {
                    const errors = validate();
                    if (errors.length === 0) {
                        setCaller('submit-button');
                        dispatch(LayoutActions.checkInventoryExists({ 'name': `${name}`, 'type': types.ORACLE_PDB }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }));
                    } else submitButtonRef.current.dispatchEvent(customEvent);
                }} autoFocus />

            <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                {getValidationError()}
            </Tooltip>
        </div>);

        if (pageIndex === 0) return (<div className='flex justify-content-between'>
            <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
            <Button label={"Next"} icon="pi pi-arrow-right" iconPos='right' autoFocus onClick={() => {


                if (valid_name?.response === "name available" && nameError !== "name not checked") {

                    let errors = validate();
                    if (errors.length === 0) {

                        setPageIndex(1);


                    }

                }
            }
            } />
        </div>);

    }


    const renderPageFirst = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="field grid flex align-items-start">
                    <label className="col-fixed" style={{ width: "200px" }}>Name:</label>
                    <div className="col">
                        <span className="p-input-icon-right flex">
                            <i className={isLoading ? "pi pi-spin pi-spinner align-self-center" : ""} />
                            <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => {
                                setName(e.target.value);
                                setNameError("name not checked");
                                dispatch(LayoutActions.setValidName(undefined));
                            }} />
                        </span>
                        <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
                    </div>
                    <div className="col-fixed">
                        <Button style={{ width: "80px" }} className="p-button-sm" label="Check" onClick={validatePdb} />
                    </div>
                </div>
                <div className="field grid flex align-items-start">
                    <label className="col-fixed" style={{ width: "200px" }}>DB User Name:</label>
                    <div className="col">
                        <InputText className="p-inputtext-sm" placeholder={"DB User Name"} value={dbUserName} onChange={(e) => setDbUserName(e.target.value)} />
                        <div className="error-help"><small id="db-user-name-help" className="p-error p-d-block">{dbUserNameError}</small></div>
                    </div>
                </div>
                <div className="field grid flex align-items-start">
                    <label className="col-fixed" style={{ width: "200px" }}>DB Password:</label>
                    <div className='col'>
                        <Password
                            feedback
                            className="p-inputtext-sm"
                            placeholder="DB Password"
                            value={dbPassword}
                            onChange={(e) => setDbPassword(e.target.value)}
                            autoComplete="new-password"
                            footer={() => footer(dbPassword)}
                            toggleMask
                        />
                        <div className="error-help"><small id="db-password-help" className="p-error p-d-block">{dbPasswordError}</small> </div>
                    </div>
                </div>

                <div className="field grid flex align-items-start">
                    <label className="col-fixed" style={{ width: "200px" }}>File Destination:</label>
                    <div className="col">
                        <span className="p-input-icon-right flex align-items-center">
                            <i className="pi pi-info-circle file" />
                            <Tooltip target=".pi.pi-info-circle.file"
                                content={db_file_dest?.message} />

                            <InputText className="p-inputtext-sm" placeholder="File Destination" value={fileDestination} onChange={(e) => setFileDestination(e.target.value)} />

                        </span>
                        <div className="error-help"><small id="api-secret-help" className="p-error p-d-block">{fileDestinationError}</small></div>

                    </div>
                </div>

                <div className="field grid">
                    <label htmlFor="cb1" className="p-checkbox-label col-fixed">Always use this location:</label>
                    <div className="col">
                        <Checkbox inputId="cb1" value="Always use this location:" onChange={() => setAlwaysUseThisLocation(!alwaysUseThisLocation)} checked={alwaysUseThisLocation}></Checkbox>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );

    }


    const renderPageSecond = () => {
        return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
            <div className="field grid">

                <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "300px" }}>Start the PDB once the CDB starts:</label>

                <div className="col">
                    <Checkbox inputId="cb1" value="Start PDB with CDB" onChange={() => setStartPdbWithCdb(!startPdbWithCdb)} checked={startPdbWithCdb}></Checkbox>
                </div>
            </div>

            <div className="field grid">
                <label className="col-fixed" style={{ width: "300px" }}>Pluggable Database Size:</label>
                <div className="col">
                    <Dropdown className="p-inputtext-sm"
                        placeholder="Pluggable Database Size:"
                        options={pluggableDatabaseSizeOptions}
                        value={pluggableDatabaseSize} onChange={(e) => setPluggableDatabaseSize(e.value)}
                    />
                </div>
            </div>
            <div className="field grid">
                <label className="col-fixed" style={{ width: "300px" }}>Replicate to Standby:</label>
                <div className="col">
                    <Dropdown className="p-inputtext-sm"
                        placeholder="Replicate To Standby"
                        options={replicateToStandbyOptions}
                        value={replicateToStandby} onChange={(e) => setReplicateToStandby(e.value)}
                    />
                </div>
            </div>
            <div className="field grid">
                <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "300px" }}>Start standbys when started:</label>
                <div className="col">
                    <Checkbox inputId="cb1" value="Start standby when started:" onChange={() => setStartStandbyWhenStarted(!startStandbyWhenStarted)} checked={startStandbyWhenStarted}></Checkbox>
                </div>
            </div>
            <div className="field grid flex align-items-center">
                <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "300px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                <div className="col-fixed">
                    <Checkbox inputId="cb1" value="Automatic delete disabled:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                </div>
                <div className="col">
                    <Slider
                        disabled={!automaticDelete}
                        min={1}
                        max={365}
                        value={automaticDeleteDays}
                        onChange={(e) => setAutomaticDeleteDays(e.value)} />
                </div>

            </div>
            {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => {
                const errors = validate();
                if (errors.length === 0) {

                    setCaller('rest-api-bar');
                    dispatch(LayoutActions.checkInventoryExists({ 'name': `${name}`, 'type': types.ORACLE_PDB }, { 'exists': setShowDeleteConfirm, 'failback': onRestApiClick }))

                }
                else setApiTooltip(getValidationError());

            }} />
        </div>
        );
    }



    return (
        <Dialog
            className='add-pdb'
            header={`Create new pluggable database in ${props.node.location} `}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>

            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {
                    pageIndex === 0 && renderPageFirst()
                }
                {
                    pageIndex === 1 && renderPageSecond()
                }
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>

        </Dialog>
    );
}


export default AddPDBDialog;