import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Checkbox } from 'primereact/checkbox';
import { useState, useEffect, useRef } from "react";
import { RadioButton } from 'primereact/radiobutton';
import { TabView, TabPanel } from 'primereact/tabview';
import { Calendar } from 'primereact/calendar';
import { Tooltip } from "primereact/tooltip";
import config, { oracleIncrementalLevel } from "../../../config";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import { ConfigActions } from "../../../store/config/reducer";
import { useSelector, useDispatch } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";
import { LayoutActions } from "../../../store/layout/reducer";
import { MultiSelect } from "primereact/multiselect";
import { getMinDate, getMaxDate } from "../../../util/daysofmonth";
import { Dropdown } from "primereact/dropdown";

export const BackupCDBDialog = (props) => {

  const [scheduleApiTooltip, setScheduleApiTooltip] = useState(" ");
  const [nowApiTooltip, setNowApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);
  const [zipped, setZipped] = useState(false);
  const [scheduleZipped, setScheduleZipped] = useState(false);
  const [time, setTime] = useState(props.updateSchedule?.ex_params?.schedule_run_time ? new Date(props.updateSchedule?.ex_params?.schedule_run_time) : new Date());
  const [toggle, setToggle] = useState(props.updateSchedule ? 1 : props.pageIndex ? props.pageIndex : 0);
  const [backupNowOptions, setBackupNowOptions] = useState("only");
  const [backupScheduleOptions, setBackupScheduleOptions] = useState("only");
  const [timeError, setTimeError] = useState("");
  const [incrementalLevel, setIncrementalLevel] = useState(oracleIncrementalLevel[0].value);
  const dispatch = useDispatch();
  const { config: appConf } = useSelector((state) => state.config);
  const [daysOfMonth, setDaysOfMonth] = useState(props.updateSchedule?.ex_params?.days_of_month?.map((d) => new Date(d)));
  const [daysOfWeek, setDaysOfWeek] = useState(props.updateSchedule?.ex_params?.days_of_week);
  const [backupPath, setBackupPath] = useState("");
  const minDate = useRef(getMinDate());
  const maxDate = useRef(getMaxDate());

  const daysOfWeekOptions = [
    { label: "Mon", value: "mon" },
    { label: "Tue", value: "tue" },
    { label: "Wed", value: "wed" },
    { label: "Thu", value: "thu" },
    { label: "Fri", value: "fri" },
    { label: "Sat", value: "sat" },
    { label: "Sun", value: "sun" },
  ];


  useEffect(() => {



    if (!appConf) dispatch(ConfigActions.getConfig());
    else if (!backupPath)
      setBackupPath(appConf?.oracle_sys?.backups_path)




  }, [appConf, backupPath, dispatch])



  const onRestApi1Click = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setNowApiTooltip(await copyToClipboard(createAPIcommand(
        {
          "node_id": props.node.location,
          "schedule": false,
          "compress": zipped,
          "backup_path": backupPath,
          "keep_days": props.node.size_scheme?.backup,
          "incremental_level": oracleIncrementalLevel[0].value,
          "options": backupNowOptions,


        }
        , accessToken, 'backup_cdb/')))

    }
    else setNowApiTooltip(" ");
  }

  const onRestApi2Click = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setScheduleApiTooltip(await copyToClipboard(createAPIcommand(
        {
          "node_id": props.node.location,
          "schedule": true,
          "compress": scheduleZipped,
          "keep_days": props.node.size_scheme?.backup,
          "backup_path": backupPath,
          "incremental_level": incrementalLevel,
          "time": time,
          "timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
          "options": backupScheduleOptions,
          "days_of_week": daysOfWeek,
          "days_of_month": daysOfMonth,
          "update_schedule": props.updateSchedule,



        }
        , accessToken, 'backup_cdb/')))

    }
    else setScheduleApiTooltip(" ");

  }

  const validate = () => {
    let errors = [];
    if (toggle === 1) {

      let error = validator(constraints)("time", time);
      (error == null || errors.push(error)) && setTimeError(error);

    }


    return errors;

  }

  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {

      if (toggle === 1) {

        props.onOkClick(
          {
            "node_id": props.node.id,
            "schedule": true,
            "compress": scheduleZipped,
            "keep_days": props.node.size_scheme?.backup,
            "incremental_level": incrementalLevel,
            "backup_path": backupPath,
            "time": time,
            "timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
            "options": backupScheduleOptions,
            "days_of_week": daysOfWeek,
            "days_of_month": daysOfMonth,
            "update_schedule": props.updateSchedule,


          });

      }
      else if (toggle === 0) {

        props.onOkClick(
          {
            "node_id": props.node.id,
            "schedule": false,
            "compress": zipped,
            "keep_days": props.node.size_scheme?.backup,
            "backup_path": backupPath,
            "incremental_level": oracleIncrementalLevel[0].value,
            "options": backupNowOptions,


          });

      }


    }




  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='backup-cdb'
      header={`Backup ${props.node.location}`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <TabView activeIndex={toggle} onTabChange={(e) => {
        dispatch(LayoutActions.setErrorMessage(""));
        setToggle(e.index);
      }}
        className="no-ink"
      >
        <TabPanel header="Backup now">
          <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

              <div className="field grid">
                <label className="col-fixed" style={{ width: "165px" }} >Compress:</label>
                <div className="col">

                  <Checkbox
                    onChange={e => setZipped(e.checked)}
                    checked={zipped}>
                  </Checkbox>
                </div>
              </div>

              <div className="field grid">
                <div className="col">
                  <div className="field-radiobutton">
                    <RadioButton inputId="backupNowOption1" name="backupNowOptions" value="all" checked={backupNowOptions === "all"} onChange={(e) => setBackupNowOptions(e.value)} />
                    <label htmlFor="backupNowOption1">Backup the entire container database (including all pluggable databases)</label>
                  </div>
                  <div className="field-radiobutton">
                    <RadioButton inputId="backupNowOption2" name="backupNowOptions" value="only" checked={backupNowOptions === "only"} onChange={(e) => setBackupNowOptions(e.value)} />
                    <label htmlFor="backupNowOption2">Backup only the container database</label>
                  </div>
                </div>

              </div>

              <div className="field grid align-items-end">
                <div className="col-fixed"> Keep backup for {props.node.size_scheme ? props.node.size_scheme?.backup : "unk"} days </div>
                <div className="col-fixed">

                  <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                  <i className="pi pi-info-circle config-field icon-field" />

                </div>
              </div>
              {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
              <ApiBar tooltip={nowApiTooltip} onClick={onRestApi1Click} />
            </div>
          </div>
        </TabPanel>
        <TabPanel header="Scheduled backup">
          <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
              <div className="form-inner-part">
                {props.updateSchedule ? <div className = "field"> Current backup schedule: </div> : null}
                <div className="grid">
                  <label className="col-fixed multiline" style={{ width: "165px" }}>{`Days of month:\nor`}</label>
                  <div className="col">
                    <Calendar viewDate={minDate.current} readOnlyInput selectionMode="multiple" dateFormat="d" minDate={minDate.current} maxDate={maxDate.current} showOtherMonths={false} className="p-inputtext-sm" panelClassName="days-of-month-picker" value={daysOfMonth} onChange={(e) =>
                      setDaysOfMonth(e.value)} showTime={false} />
                  </div>
                </div>

                <div className="grid">
                  <label className="col-fixed" style={{ width: "165px" }}>Days of week:</label>
                  <div className="col">
                    <MultiSelect scrollHeight="100%" className="p-inputtext-sm" maxSelectedLabels={6} selectedItemsLabel="All weekdays" selectAllLabel="All weekdays" options={daysOfWeekOptions} value={daysOfWeek} onChange={(e) => setDaysOfWeek(e.value)} />

                  </div>
                </div>

              </div>


              <div className="field grid">
                <label className="col-fixed" style={{ width: "165px" }}>Time:</label>
                <div className="col">
                  <Calendar showSeconds className="p-inputtext-sm" value={time} onChange={(e) => setTime(e.value)} timeOnly />
                  <div className="error-help"><small className="p-error p-d-block">{timeError}</small> </div>
                </div>
              </div>
              <div className="grid">
                <label className="col-fixed" style={{ width: "165px" }}>Incremental level:</label>
                <div className="col">
                  <Dropdown className="p-inputtext-sm" options={oracleIncrementalLevel} value={incrementalLevel} onChange={(e) => setIncrementalLevel(e.value)} />

                </div>
              </div>

              <div className="field grid">
                <label style={{ width: "165px" }} className="col-fixed"> Compress:</label>
                <div className="col">

                  <Checkbox
                    onChange={e => setScheduleZipped(e.checked)}
                    checked={scheduleZipped}>
                  </Checkbox>
                </div>
              </div>

              <div className="field grid">
                <div className="col">
                  <div className="field-radiobutton">
                    <RadioButton inputId="backupScheduleOption1" name="backupScheduleOptions" value="all" checked={backupScheduleOptions === "all"} onChange={(e) => setBackupScheduleOptions(e.value)} />
                    <label htmlFor="backupScheduleOption1">Backup the entire container database (including all pluggable databases)</label>
                  </div>
                  <div className="field-radiobutton">
                    <RadioButton inputId="backupScheduleOption2" name="backupScheduleOptions" value="only" checked={backupScheduleOptions === "only"} onChange={(e) => setBackupScheduleOptions(e.value)} />
                    <label htmlFor="backupScheduleOption2">Backup only the container database</label>
                  </div>
                </div>
              </div>

              <div className="field grid align-items-end">
                <div className="col-fixed"> Keep backup for {props.node.size_scheme ? props.node.size_scheme?.backup : "unk"} days </div>
                <div className="col-fixed">

                  <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                  <i className="pi pi-info-circle config-field icon-field" />

                </div>
              </div>
              {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
              <ApiBar tooltip={scheduleApiTooltip} onClick={onRestApi2Click} />
            </div>
          </div>
        </TabPanel>
      </TabView>
    </Dialog>)

};

export default BackupCDBDialog;