
import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { SelectButton } from 'primereact/selectbutton';
import { Tag } from "primereact/tag";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { useSelector } from "react-redux";
import ApiBar from "../../../components/apiBar";
import ErrorTextBox from "../../../components/errorTextBox";

export const ClosePDBDialog = (props) => {

  const [close, setClose] = useState("CI");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const {accessToken} = useSelector((state) => state.app);
  const {errorMessage, isLoading} = useSelector((state) => state.layout);
  const options = [
    { label: 'Close Immediately', value: "CI" },
    { label: 'Close Abort', value: "CA" },
  ];
  const handleOk = () => {


    props.onOkClick(
      {
        "pdb_id": props.node.id,
        "cdb_id": props.root.id,
        "pdb_name": props.node.name,
        "option": close
      });


  }

  const onRestApiClick = async () => 
  {

   setApiTooltip(await copyToClipboard(createAPIcommand(
    {
    "pdb_id": props.node.location,
    "cdb_id": props.root.location,
    "pdb_name": props.node.name,
    "option": close
  }, accessToken, "close_pdb/")));

  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='close-pdb'
      header={`Close ${props.node.name}`}
      visible={props.visible}
      style={{ width: '40vw'}}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="field">
                <SelectButton className="p-button-sm" value={close} options={options} onChange={(e) => setClose(e.value)} />
              </div>
              <div className="field">
                <Tag value="Immediate will cancel transactions and close the database gracefully." severity="info" icon="pi pi-info-circle" />
              </div>
              <div className="field">

                <Tag value="Abort will force shutdown the database. Database recovery will be required later." severity="info" icon="pi pi-info-circle" />

              </div>

            </div>

          </div>
          {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}
          <ApiBar tooltip = {apiTooltip} onClick = {onRestApiClick} />
        </div>
      </div>

    </Dialog>)

};

export default ClosePDBDialog;