import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useEffect, useState, useRef } from "react";
import { InputText } from "primereact/inputtext";
import { useSelector, useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import { MultiSelect } from 'primereact/multiselect';
import { Checkbox } from 'primereact/checkbox';
import { Dropdown } from "primereact/dropdown";
import validator from "../../../util/validator";
import constraints from "./constraints";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ApiBar from "../../../components/apiBar";
import ErrorTextBox from "../../../components/errorTextBox";
import { ConfigActions } from "../../../store/config/reducer";
import getValidationError from '../../../util/errorPrompter';
import { Tooltip } from "primereact/tooltip";


export const CopyPDBDialog = (props) => {

  const [name, setName] = useState(props.node.name);
  const [fileDestination, setFileDestination] = useState("");
  const { valid_name, isLoading } = useSelector((state) => state.layout);
  const { config } = useSelector((state) => state.config);
  const dispatch = useDispatch();
  const [nameError, setNameError] = useState(null);
  const [attachStandbys, setAttachStandbys] = useState(["None"]);
  const [attachStandbysOptions, setAttachStandbyOptions] = useState([{ label: "None", value: "None" }]);
  const [alsoStart, setAlsoStart] = useState(false);
  const [autoStart, setAutoStart] = useState(false);
  const [sourceRestart, setSourceRestart] = useState("DN");
  const [targetRestart, setTargetRestart] = useState("DN");
  const [fileDestinationError, setFileDestinationError] = useState("");
  const [attachStandbysError, setAttachStandbysError] = useState("");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage } = useSelector((state) => state.layout);
  const customEvent = new Event('beforeinput', { bubbles: true });
  const submitButtonRef = useRef(null);

  
  const restartOptions = [
    { label: "Do Nothing", value: "DN" },
    { label: "Open Read Only", value: "RO" },
    { label: "Open Read Write", value: "RW" }
  ];

  useEffect(() => {
    if (props.nodes && props.nodes.length > 0) {
      const temp = []
      props.nodes.forEach((standby) => temp.push({ label: standby.name, value: standby.id }));
      setAttachStandbyOptions(temp.concat({ label: "None", value: "None" }));

    }

  }, [props.nodes])

  useEffect(() => {
    if (!config) dispatch(ConfigActions.getConfig());
    if (config) setFileDestination(config?.oracle_sys?.pdb_path);
  },
    [config, dispatch])




  const validatePdb = () => {
    let error = validator(constraints)("name", name);
    setNameError(error);
    if (!error) {
      dispatch(LayoutActions.checkPDBName({ "cdb_id": props.root.id, "pdb_name": name }));
    }
  }

  useEffect(() => {

    if (valid_name) {

      if (valid_name?.error)
        setNameError(valid_name.error);
      else setNameError(valid_name?.response);

    }

  }, [valid_name]);

  const onRestApiClick = async () => {

    if (!nameError) setNameError("name not checked");

    if (nameError === "name available") {

      const errors = [];

      let error = validator(constraints)("fileDestination", fileDestination);
      (error == null || errors.push(error)) && setFileDestinationError(error);

      error = validator(constraints)("attachStandbys", attachStandbys);
      (error == null || errors.push(error)) && setAttachStandbysError(error);


      if (errors.length === 0) {
        setApiTooltip(await copyToClipboard(createAPIcommand(
          {
            "pdb_id": props.node.location,
            "cdb_id": props.root.location,
            "file_dest": fileDestination,
            "standbys": attachStandbys,
            "also_start": alsoStart,
            "autostart": autoStart,
            "copy": name,
            "source_restart": sourceRestart,
            "target_restart": targetRestart


          }, accessToken, props.node?.ex_params?.cdb_id === props.root?.id ? "copy_pdb/" : "copy_to_cdb/")));


      }
      else setApiTooltip(getValidationError());
    }
    else setApiTooltip(getValidationError());


  }

  const handleOk = () => {

    if (!nameError) setNameError("name not checked");

    if (nameError === "name available") {

      const errors = [];

      let error = validator(constraints)("fileDestination", fileDestination);
      (error == null || errors.push(error)) && setFileDestinationError(error);

      error = validator(constraints)("attachStandbys", attachStandbys);
      (error == null || errors.push(error)) && setAttachStandbysError(error);


      if (errors.length === 0) {
        
        props.onOkClick({
          "pdb_id": props.node.id,
          "cdb_id": props.root.id,
          "file_dest": fileDestination,
          "standbys": attachStandbys,
          "also_start": alsoStart,
          "autostart": autoStart,
          "copy": name,
          "source_restart": sourceRestart,
          "target_restart": targetRestart


        });



      }
      else  submitButtonRef.current.dispatchEvent(customEvent);
    }
    else submitButtonRef.current.dispatchEvent(customEvent);

  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          className="submit-button"
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          ref={submitButtonRef}
          onClick={handleOk} autoFocus />
        <Tooltip   target='.submit-button' showEvent='beforeinput' position='top' >
          {getValidationError()}
        </Tooltip>
      </div>
    );

  }



  return (
    <Dialog
      className='copy-pdb'
      header={`Copy ${props.node.name} to ${props.root.location}`}
      visible={props.visible}
      style={{ width: '50vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col-fixed" style={{ width: "250px" }}>Copy name:</div>
            <div className="col">
              <span className="p-input-icon-right flex">
                <i className={isLoading ? "pi pi-spin pi-spinner align-self-center" : ""} />
                <InputText
                  className="p-inputtext-sm"
                  placeholder="Name"
                  value={name}
                  onChange={(e) => {

                    setName(e.target.value);
                    setNameError("name not checked");
                    dispatch(LayoutActions.setValidName(undefined));

                  }} />
              </span>
              <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
            </div>
            <div className="col-fixed">
              <Button style={{ width: "80px" }} className="p-button-sm" label="Check" onClick={validatePdb} />
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "250px" }}>File Destination:</div>
            <div className="col">
              <InputText
                className="p-inputtext-sm"
                placeholder="File Destination" value={fileDestination}
                onChange={(e) => setFileDestination(e.target.value)} />
              <small className="p-error p-d-block">{fileDestinationError}</small>
            </div>

          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "250px" }}>Attach Standbys:</div>
            <div className="col">
              <MultiSelect
                className="p-inputtext-sm"
                value={attachStandbys}
                options={attachStandbysOptions}
                placeholder="Attach Standbys"
                onChange={(e) => setAttachStandbys(e.value)} />
              {attachStandbysError && <small className="p-error p-d-block">{attachStandbysError}</small>}
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "250px" }}>Also start in standbys:</div>
            <div className="col">
              <Checkbox
                onChange={e => setAlsoStart(e.checked)}
                checked={alsoStart}>
              </Checkbox>
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "250px" }}>Auto start after copy:</div>
            <div className="col">
              <Checkbox
                onChange={e => setAutoStart(e.checked)}
                checked={autoStart}>
              </Checkbox>
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "250px" }}>Source PDB status after copy:</div>
            <div className="col">
              <Dropdown className="p-inputtext-sm"
                value={sourceRestart}
                options={restartOptions}
                onChange={(e) => setSourceRestart(e.value)}
                placeholder="Select Action" />

            </div>
          </div>


          <div className="field grid">
            <div className="col-fixed" style={{ width: "250px" }}>Target PDB status after copy:</div>
            <div className="col">
              <Dropdown style={{ marginLeft: "3px" }} className="p-inputtext-sm"
                value={targetRestart}
                options={restartOptions}
                onChange={(e) => setTargetRestart(e.value)}
                placeholder="Select Action" />

            </div>
          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>
      </div>

    </Dialog>)

};

export default CopyPDBDialog;
