

import { useDispatch, useSelector } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import { useEffect, useState, useRef } from "react";
import { InputText } from "primereact/inputtext";
import { Dropdown } from "primereact/dropdown";
import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { MultiSelect } from 'primereact/multiselect';
import { Checkbox } from 'primereact/checkbox';
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from "../../../components/errorTextBox";
import { ConfigActions } from "../../../store/config/reducer";
import getValidationError from '../../../util/errorPrompter';
import { Tooltip } from "primereact/tooltip";

export const CopytoCDBDialog = (props) => {

  const [name, setName] = useState("");
  const { valid_name, isLoading } = useSelector((state) => state.layout);
  const { config } = useSelector((state) => state.config);
  const dispatch = useDispatch();
  const [nameError, setNameError] = useState(null);
  const [fileDestination, setFileDestination] = useState("");
  const [attachStandbys, setAttachStandbys] = useState(["None"]);
  const [attachStandbysOptions, setAttachStandbyOptions] = useState([{ label: "None", value: "None" }]);
  const [alsoStart, setAlsoStart] = useState(false);
  const [autoStart, setAutoStart] = useState(false);
  const [sourceRestart, setSourceRestart] = useState("NA");
  const [targetRestart, setTargetRestart] = useState("NA");
  const [fileDestinationError, setFileDestinationError] = useState("");
  const [attachStandbysError, setAttachStandbysError] = useState("");
  const [targetContainer, setTargetContainer] = useState(null);
  const [containerOptions, setContainerOptions] = useState([]);
  const [apiTooltip, setApiTooltip] = useState("");
  const customEvent = new Event('beforeinput', { bubbles: true });
  const submitButtonRef = useRef(null);

  


  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage } = useSelector((state) => state.layout);

  const restartOptions = [
    { label: "Do Nothing", value: "NA" },
    { label: "Open Read Only", value: "RO" },
    { label: "Open Read Write", value: "RW" }
  ];

  useEffect(() => {
    if (props.nodes && props.nodes.length > 0) {
      const temp = []
      props.nodes.forEach((standby) => temp.push({ label: standby.name, value: standby.id }));
      setAttachStandbyOptions(temp.concat({ label: "None", value: "None" }));

    }

  }, [props.nodes])


  const validatePdb = () => {
    if (!targetContainer) {

      setNameError("target container not selected");
      return;

    }

    let error = validator(constraints)("name", name);
    setNameError(error);
    if (!error) {
      dispatch(LayoutActions.checkPDBName({ "cdb_id": targetContainer, "pdb_name": name }));
    }
  }

  useEffect(() => {

    if (valid_name) {

      if (valid_name?.error)
        setNameError(valid_name?.error);
      else
        setNameError(valid_name?.response);
    }

  }, [valid_name]);


  useEffect(() => {

    if (props.data) {

      const temp = [];
      props.data.queryset.forEach((container) => temp.push({ label: `${container.location}`, value: container.id }));
      setContainerOptions(temp)
      setName(props.data.copy_name)

    }

  }, [props.data]);

  useEffect(() => {
    if (!config) dispatch(ConfigActions.getConfig());
    if (config) setFileDestination(config?.oracle_sys?.pdb_path);
  },
    [config, dispatch])


  const onRestApiClick = async () => {


    if (nameError === "name available" && targetContainer) {

      let errors = validate();
      if (errors.length === 0) {
        setApiTooltip(await copyToClipboard(createAPIcommand(
          {
            "pdb_id": props.node.location,
            "cdb_id": targetContainer,
            "file_dest": fileDestination,
            "standbys": attachStandbys,
            "also_start": alsoStart,
            "autostart": autoStart,
            "copy": name,
            "source_restart": sourceRestart,
            "target_restart": targetRestart


          }, accessToken, "copy_to_cdb/")))

      }
      else setApiTooltip(getValidationError());

    }
    else setApiTooltip(getValidationError());

  }

  const handleOk = () => {

    if (nameError === "name available" && targetContainer) {

      let errors = validate();
      if (errors.length === 0) {
        
        props.onOkClick(
          {
            "pdb_id": props.node.id,
            "cdb_id": targetContainer,
            "file_dest": fileDestination,
            "standbys": attachStandbys,
            "also_start": alsoStart,
            "autostart": autoStart,
            "copy": name,
            "source_restart": sourceRestart,
            "target_restart": targetRestart


          })

      }
      else  submitButtonRef.current.dispatchEvent(customEvent);

    }
    else  submitButtonRef.current.dispatchEvent(customEvent);

  }

  const validate = () => {
    if (!nameError) setNameError("name not checked");


    const errors = [];

    let error = validator(constraints)("fileDestination", fileDestination);
    (error == null || errors.push(error)) && setFileDestinationError(error);

    error = validator(constraints)("attachStandbys", attachStandbys);
    (error == null || errors.push(error)) && setAttachStandbysError(error);

    return errors;

  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          className="submit-button"
          ref={submitButtonRef}
          onClick={handleOk} autoFocus />
        <Tooltip   target='.submit-button' showEvent='beforeinput' position='top' >
          {getValidationError()}
        </Tooltip>
      </div>
    );

  }



  return (
    <Dialog
      className='copy-to-cdb'
      header={`This will copy ${props.node.name} to another CDB`}
      visible={props.visible}
      style={{ width: '50vw', maxHeight: "90%", height: '750px' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

          <div className="field grid">
            <label className="col-fixed" style={{ width: "190px" }}>{"Target Container:"}</label>
            <div className="col">
              <Dropdown className="p-inputtext-sm"
                value={targetContainer}
                options={containerOptions}
                onChange={(e) => setTargetContainer(e.value)}
                placeholder="Select Container" />

            </div>
          </div>

          <div className="field grid">
            <label className="col-fixed align-self-start" style={{ width: "190px" }}>{"Copy name:"}</label>
            <div className="col">
              <span className="p-input-icon-right flex">
                <i className={isLoading ? "pi pi-spin pi-spinner align-self-center" : ""} />
                <InputText
                  className="p-inputtext-sm"
                  placeholder="Name"
                  value={name}
                  onChange={(e) => {

                    setName(e.target.value);
                    setNameError("name not checked");
                    dispatch(LayoutActions.setValidName(undefined));

                  }} />
              </span>
              <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
            </div>
            <div className="col-fixed flex align-items-start">
              <Button className="p-button-sm" style={{ width: "80px" }} label={"Check"} onClick={validatePdb} />
            </div>
          </div>

          <div className="field grid">
            <label className="col-fixed" style={{ width: "190px" }}>{"File Destination:"}</label>
            <div className="col">
              <InputText
                className="p-inputtext-sm"
                placeholder="File Destination" value={fileDestination}
                onChange={(e) => setFileDestination(e.target.value)} />
              {fileDestinationError && <small className="p-error p-d-block">{fileDestinationError}</small>}
            </div>

          </div>

          <div className="field grid">
            <label className="col-fixed" style={{ width: "190px" }}>{"Attach Standbys:"}</label>
            <div className="col">
              <MultiSelect
                value={attachStandbys}
                options={attachStandbysOptions}
                placeholder="Attach Standbys"
                onChange={(e) => setAttachStandbys(e.value)} />
              {attachStandbysError && <small className="p-error p-d-block">{attachStandbysError}</small>}
            </div>
          </div>

          <div className="field grid">
            <label className="col-fixed" style={{ width: "190px" }}>{"Also start in standbys:"}</label>
            <div className="col">
              <Checkbox
                onChange={e => setAlsoStart(e.checked)}
                checked={alsoStart}>
              </Checkbox>
            </div>
          </div>

          <div className="field grid">
            <label className="col-fixed" style={{ width: "190px" }}>{"Auto start after copy:"}</label>
            <div className="col">
              <Checkbox
                onChange={e => setAutoStart(e.checked)}
                checked={autoStart}>
              </Checkbox>
            </div>
          </div>

          <div className="field grid">
            <label className="col-fixed" style={{ width: "190px" }}>{"Source PDB restart:"}</label>
            <div className="col">
              <Dropdown className="p-inputtext-sm"
                value={sourceRestart}
                options={restartOptions}
                onChange={(e) => setSourceRestart(e.value)}
                placeholder="Select Action" />

            </div>
          </div>


          <div className="field grid">
            <label className="col-fixed" style={{ width: "190px" }}>{"Target PDB restart:"}</label>
            <div className="col">
              <Dropdown className="p-inputtext-sm" value={targetRestart}
                options={restartOptions}
                onChange={(e) => setTargetRestart(e.value)}
                placeholder="Select Action" />

            </div>
          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>

      </div>

    </Dialog>)

};

export default CopytoCDBDialog;
