import { useState } from 'react';
import { InputText } from "primereact/inputtext";
import { Button } from "primereact/button";
import { Dialog } from "primereact/dialog";
import copyToClipboard from '../../../util/copyToClipboard';
import createAPIcommand from '../../../util/createApiCommand';
import { useDispatch, useSelector } from 'react-redux';
import ErrorTextBox from '../../../components/errorTextBox';
import ApiBar from '../../../components/apiBar';
import validator from '../../../util/validator';
import constraints from './constraints';
import config from '../../../config';
import { LayoutActions } from '../../../store/layout/reducer';
import { ConfirmPopup } from 'primereact/confirmpopup';


const DeleteCDBDialog = (props) => {

    const [unregisterConfirm, setUnregisterConfirm] = useState("");
    const [unregisterConfirmError, setUnregisterConfirmError] = useState("");
    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { errorMessage, isLoading } = useSelector((state => state.layout));
    const [showForceDeleteConfirm, setShowForceDeleteConfirm] = useState(false);
    const dispatch = useDispatch();

    const handleOk = () => props.onOkClick({ "cdb_id": props.node.id });


    const onDeleteCdbClick = () => {

        let errors = validate();

        if (errors.length === 0) dispatch(LayoutActions.isNodeHostReachable({ 'node_id': props.node.id }, { allow: handleOk, confirm: setShowForceDeleteConfirm }))

    }

    const validate = () => {

        let errors = [];

        let error = validator(constraints)("unregisterConfirm", unregisterConfirm);
        (error == null || errors.push(error)) && setUnregisterConfirmError(error);
        return errors;

    }

    const onRestApiClick = async () => {
        let errors = validate();
        if (errors.length === 0) {

            setApiTooltip(await copyToClipboard(createAPIcommand(
                {

                    cdb_id: props.node.location,


                }
                , accessToken, 'delete_cdb/')));


        }
        else setApiTooltip(" ");

    }

    const renderFooter = () => {
        return (
            <div className="flex justify-content-between">
                <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    label={"Submit"}
                    id="submit-button"
                    icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'trash'}` } 
                    className="p-button-danger"
                    onClick={onDeleteCdbClick} autoFocus />
                <ConfirmPopup
                    icon="pi pi-exclamation-triangle"
                    target={document.getElementById('submit-button')}
                    visible={showForceDeleteConfirm}
                    className="multiline"
                    onHide={() => setShowForceDeleteConfirm(false)}
                    accept={() => dispatch(LayoutActions.markNodeDeleted({ 'node_id': props.node.id }, props.onHide()))}
                    message={"Node host is currently offline, unable to perform drop.\nMark node as deleted anyway?"} />
            </div>
        );
    }


    return (
        <Dialog
            className='delete-cdb'
            header={`Delete ${props.node.location}`}
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <div className='form-inner-part border-none'>
                        <div style={{ marginBottom: "10px" }}>
                            This will unregister the oracle container database {props.node?.location}. The container and it's pluggable database will no longer be manage by 123cluster.
                        </div>
                        <div>
                            Are you sure?
                        </div>
                    </div>
                    <div className='form-inner-part border-none'>

                        <div className="field">
                            <div style={{ marginBottom: "10px" }} className="title">{`Please type "${config.unregisterConfirm}" here in order to continue`}</div>
                            <InputText className="p-inputtext-sm" placeholder={`Please type "${config.unregisterConfirm}" here in order to continue`} value={unregisterConfirm} onChange={(e) => setUnregisterConfirm(e.target.value)} />
                            <div className="error-help">{<small className="p-error p-d-block">{unregisterConfirmError}</small>}</div>
                        </div>


                    </div>

                    {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                    <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
                </div>
            </div>
        </Dialog>
    );
}


export default DeleteCDBDialog;