import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { RadioButton } from 'primereact/radiobutton';
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import { Tag } from 'primereact/tag';
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useDispatch, useSelector } from "react-redux";
import config from "../../../config";
import ErrorTextBox from "../../../components/errorTextBox";
import validator from "../../../util/validator";
import constraints from "./constraints";
import { LayoutActions } from "../../../store/layout/reducer";
import { ConfirmPopup } from "primereact/confirmpopup";

export const DeletePDBDialog = (props) => {

  const [dropConfirm, setDropConfirm] = useState("");

  const [deleteOptions, setDeleteOptions] = useState("justDrop");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state => state.layout));
  const [dropConfirmError, setDropConfirmError] = useState("");
  const [showForceDeleteConfirm, setShowForceDeleteConfirm] = useState(false);
  const dispatch = useDispatch();


  const onRestApiClick = async () => {

    let errors = validate();
    if (errors.length === 0) {
      setApiTooltip(await copyToClipboard(createAPIcommand({
        "pdb_id": props.node.location,
        "cdb_id": props.root.location,
        "deleteOptions": deleteOptions

      }, accessToken, "drop_pdb/")))
    }
    else setApiTooltip(" ");
  }

  const onDropPDBClick = () => {

    let errors = validate();
    if (errors.length === 0) dispatch(LayoutActions.isNodeHostReachable({ 'node_id': props.node.id }, { allow: handleOk, confirm: setShowForceDeleteConfirm }));
  }

  const handleOk = () =>  props.onOkClick(
    {
      "pdb_id": props.node.id,
      "cdb_id": props.root.id,
      "deleteOptions": deleteOptions

    });

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'trash'}` } 
          className="p-button-danger"
          id = 'submit-button'
          onClick={onDropPDBClick} autoFocus />
        <ConfirmPopup
          icon="pi pi-exclamation-triangle"
          target={document.getElementById('submit-button')}
          visible={showForceDeleteConfirm}
          className="multiline"
          onHide={() => setShowForceDeleteConfirm(false)}
          accept={() => dispatch(LayoutActions.markNodeDeleted({ 'node_id': props.node.id }, props.onHide()))}
          message={"Node host is currently offline, unable to perform drop.\nMark node as deleted anyway?"} />
      </div>
    );

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("dropConfirm", dropConfirm);
    (error == null || errors.push(error)) && setDropConfirmError(error);
    return errors;

  }


  return (
    <Dialog
      className='delete-pdb'
      header={`Delete ${props.node.name}`}
      visible={props.visible}
      style={{ width: '50vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              {props.open_mode && <div className="field">The pluggable database {props.node.name} open mode is {props.open_mode} with {props.sessions} connected session.</div>}
              <div className="field-radiobutton">
                <RadioButton inputId="deleteOption1" name="deleteOption" value="justDrop" checked={deleteOptions.includes("justDrop")} onChange={(e) => setDeleteOptions(e.value)} />
                <label htmlFor="deleteOption1">Drop this PDB</label>
              </div>
              <div className="field-radiobutton">
                <RadioButton inputId="deleteOption2" name="deleteOption" value="keepMeta" checked={deleteOptions.includes("keepMeta")} onChange={(e) => setDeleteOptions(e.value)} />
                <label htmlFor="deleteOption2">Just delete the database but keep the PDB metadata</label>
              </div>
              <div className="field">
                Please consider to backup this node
              </div>
              <Tag icon="pi pi-exclamation-triangle" className="field" severity="warning" value="Once a pluggable database is dropped, it cannot be recovered from a RMAN backup" />
              <div className="field grid flex align-items-center">
                <div className="col">
                  <InputText
                    className="p-inputtext-sm"
                    placeholder={`Please type "${config.dropConfirm}" here in order to continue`}
                    value={dropConfirm}
                    onChange={(e) => setDropConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{dropConfirmError}</small>}</div>
                </div>
              </div>
            </div>

          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />

        </div>
      </div>

    </Dialog>)

};

export default DeletePDBDialog;

