import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useEffect, useState, useRef } from "react";
import { InputText } from "primereact/inputtext";
import { useSelector, useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import { Dropdown } from "primereact/dropdown";
import { Password } from 'primereact/password';
import { InputNumber } from "primereact/inputnumber";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from "../../../components/errorTextBox";
import { Tooltip } from "primereact/tooltip";
import { base64Encode } from "../../../util/base64Encode";
import getValidationError from '../../../util/errorPrompter';

const DuplicateStandaloneDialog = (props) => {

  const [targetPDB, setTargetPDB] = useState("");
  const [fileDestination, setFileDestination] = useState("");
  const [nameError, setNameError] = useState(null);
  const [pathError, setPathError] = useState("");
  const [userNameError, setUserNameError] = useState("");
  const [passwordError, setPasswordError] = useState("");
  const [portError, setPortError] = useState("");
  const [hostError, setHostError] = useState("");
  const [serviceNameError, setServiceNameError] = useState("");
  const [buttons, setButtons] = useState(false);
  const [port, setPort] = useState(1521);
  const [serviceName, setServiceName] = useState("");
  const [host, setHost] = useState("");
  const [startAfterCopy, setStartAfterCopy] = useState("nothing");
  const [password, setPassword] = useState("");
  const [userName, setUserName] = useState("");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const customEvent = new Event('beforeinput', { bubbles: true });
  const submitButtonRef = useRef(null);

  


  const restartOptions = [
    { label: "Do Nothing", value: "nothing" },
    { label: "Open Read Only", value: "openre" },
    { label: "Open Read Write", value: "openrw" }
  ];

  const { valid_name, valid_path, isLoading, errorMessage, db_file_dest } = useSelector((state) => state.layout);
  const dispatch = useDispatch();

  useEffect(() => {

    if (props.standalone) {
      dispatch(LayoutActions.getDBFileDest(props.root.id));
      setServiceName(props.standalone.service_name);
      setHost(props.standalone.host_name);

    }

  }, [props.standalone, props.root.id, dispatch])

  useEffect(() => {
    if (db_file_dest) {

      setFileDestination(db_file_dest.file_dest);

    }

  }, [db_file_dest])


  const validatePdb = () => {


    let error = validator(constraints)("name", targetPDB);
    setNameError(error);
    if (!error) {
      dispatch(LayoutActions.checkPDBName({ "cdb_id": props.root.id, "pdb_name": targetPDB }));
    }

  }

  const validateFilePath = () => {

    dispatch(LayoutActions.checkPathName({ "cdb_id": props.root.id, "path_name": fileDestination }));

  }

  useEffect(() => {
    if (valid_name) {

      if (valid_name?.error)
        setNameError(valid_name.error);
      else
        setNameError(valid_name.response);
    }

  }, [valid_name]);

  useEffect(() => {

    if (valid_path) {

      if (valid_path?.error)
        setPathError(valid_path.error)
      else setPathError(valid_path?.response);
    }

  }, [valid_path]);


  const validate = () => {

    let errors = [];

    let error = validator(constraints)("port", port);
    (error == null || errors.push(error)) && setPortError(error);

    error = validator(constraints)("serviceName", serviceName);
    (error == null || errors.push(error)) && setServiceNameError(error);

    error = validator(constraints)("username", userName);
    (error == null || errors.push(error)) && setUserNameError(error);

    error = validator(constraints)("password", password);
    (error == null || errors.push(error)) && setPasswordError(error);

    error = validator(constraints)("hostName", host);
    (error == null || errors.push(error)) && setHostError(error);

    return errors;

  }


  const handleOk = () => {
    if (!nameError) setNameError("name not checked");
    if (!pathError) setPathError("path not checked");

    if (nameError === "name available" && pathError === "already exists") {

      let errors = validate();

      if (errors.length === 0) {
        
        props.onOkClick(
          {

            "cdb_id": props.root.id,
            "file_dest": fileDestination,
            "start_after_copy": startAfterCopy,
            "target_PDB_name": targetPDB,
            "port": port,
            "service_name": serviceName,
            "machine": host,
            "password": base64Encode(password),
            "user_name": userName,
          })


      }
      else  submitButtonRef.current.dispatchEvent(customEvent);
    }
    else  submitButtonRef.current.dispatchEvent(customEvent);

  }

  const onRestApiClick = async () => {
    if (!nameError) setNameError("name not checked");
    if (!pathError) setPathError("path not checked");

    if (nameError === "name available" && pathError === "already exists") {

      let errors = validate();

      if (errors.length === 0) {
        setApiTooltip(await copyToClipboard(createAPIcommand({

          "cdb_id": props.root.location,
          "file_dest": fileDestination,
          "start_after_copy": startAfterCopy,
          "target_PDB_name": targetPDB,
          "port": port,
          "service_name": serviceName,
          "machine": host,
          "password": base64Encode(password),
          "user_name": userName,
        }, accessToken, "duplicate_oracle_standalone/")))



      }
      else setApiTooltip(getValidationError());
    }
    else setApiTooltip(getValidationError());

  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          className="submit-button"
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          ref={submitButtonRef}
          onClick={handleOk} autoFocus />
        <Tooltip   target='.submit-button' showEvent='beforeinput' position='top' >
          {getValidationError()}
        </Tooltip>
      </div>
    );

  }

  return (
    <Dialog
      className='duplicate-standalone'
      header={"Duplicate Oracle Standalone"}
      visible={props.visible}
      style={{ width: '50vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Source Database User Name:</div>
            <div className="col">
              <InputText
                autoComplete="new-password"
                className="p-inputtext-sm"
                placeholder="Database User Name" value={userName}
                onChange={(e) => setUserName(e.target.value)} />
              {userNameError && <small className="p-error p-d-block">{userNameError}</small>}
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Source Database Password:</div>
            <div className="col">
              <Password
                autoComplete="new-password"
                feedback={false}
                className="p-inputtext-sm"
                placeholder="Database Password"
                value={password}
                onChange={(e) => setPassword(e.target.value)}
                toggleMask />
              {passwordError && <small className="p-error p-d-block">{passwordError}</small>}

            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Source Service Name:</div>
            <div className="col">
              <InputText
                className="p-inputtext-sm"
                placeholder="Service Name" value={serviceName}
                onChange={(e) => setServiceName(e.target.value)} />
              {serviceNameError && <small className="p-error p-d-block">{serviceNameError}</small>}
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Source Host Name/IP:</div>
            <div className="col">
              <InputText
                className="p-inputtext-sm"
                placeholder="Host" value={host}
                onChange={(e) => setHost(e.target.value)} />
              {hostError && <small className="p-error p-d-block">{hostError}</small>}
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Source Port:</div>
            <div className="col">
              <InputNumber
                className="p-inputtext-sm"
                value={port}
                onValueChange={(e) => setPort(e.value)}
                showButtons={buttons}
                onFocus={() => setButtons(true)}
                onBlur={() => setButtons(false)}
                useGrouping={false}
              />
              {portError && <small className="p-error p-d-block">{portError}</small>}
            </div>
          </div>

          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Target File Destination:</div>
            <div className="col">
              <span className="p-input-icon-right flex">
                <i className={isLoading && !valid_path && valid_path !== undefined ? "pi pi-spin pi-spinner align-self-center" : "pi pi-info-circle file"} />
                <Tooltip target=".file" content={db_file_dest?.message}></Tooltip>
                <InputText
                  className="p-inputtext-sm"
                  placeholder="File Destination" value={fileDestination}
                  onChange={(e) => {

                    setFileDestination(e.target.value);
                    setPathError("path not checked");
                    dispatch(LayoutActions.setValidPath(undefined));

                  }} />
              </span>
              <div className="error-help"><small className={pathError === "already exists" ? "p-d-block" : "p-error p-d-block"}>{pathError}</small></div>
            </div>
            <div className="col-fixed">
              <Button style={{ width: "160px" }} className="p-button-sm" label={"Check file path"} onClick={validateFilePath} />
            </div>
          </div>


          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Target PDB Name:</div>
            <div className="col">
              <span className="p-input-icon-right flex">
                <i className={isLoading && !valid_name && valid_name !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />
                <InputText
                  className="p-inputtext-sm"
                  placeholder="PDB name"
                  value={targetPDB}
                  onChange={(e) => {

                    setTargetPDB(e.target.value);
                    setNameError("name not checked");
                    dispatch(LayoutActions.setValidName(undefined));

                  }} />
              </span>
              <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
            </div>
            <div className="col-fixed">
              <Button className="p-button-sm" style={{ width: "160px" }} label={"Check PDB name"} onClick={validatePdb} />
            </div>
          </div>
          {/*

<div className="field grid">
<div className="col-fixed" style={{ width: "240px" }}>Target Database User Name:</div>
<div className="col">
  <InputText
    autoComplete="new-password"
    className="p-inputtext-sm"
    placeholder="Database User Name" value={targetUserName}
    onChange={(e) => setTargetUserName(e.target.value)} />
  {targetUserNameError && <small className="p-error p-d-block">{targetUserNameError}</small>}
</div>
</div>

<div className="field grid">
<div className="col-fixed" style={{ width: "240px" }}>Target Database Password:</div>
<div className="col">
  <Password
    autoComplete="new-password"
    feedback={false}
    className="p-inputtext-sm"
    value={targetPassword}
    placeholder="Database Password"
    onChange={(e) => setTargetPassword(e.target.value)}
    toggleMask />
  {targetPasswordError && <small className="p-error p-d-block">{targetPasswordError}</small>}

</div>
</div>

  */}
          <div className="field grid">
            <div className="col-fixed" style={{ width: "240px" }}>Start target after copy:</div>
            <div className="col">
              <Dropdown
                className="p-inputtext-sm"
                value={startAfterCopy}
                options={restartOptions}
                onChange={(e) => setStartAfterCopy(e.value)}
                placeholder="Select Action" />

            </div>
          </div>

          <div className="field grid">
            <div className="col">
              <div>
                This will duplicate {serviceName} to a new PDB at {props.root.name}, are you sure?
              </div>
            </div>
          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick}></ApiBar>
        </div>
      </div>

    </Dialog>)

};

export default DuplicateStandaloneDialog;