import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { Tooltip } from "primereact/tooltip";
import { Checkbox } from "primereact/checkbox";
import config from "../../../config";
import ApiBar from "../../../components/apiBar";
import { useState, useEffect, useRef } from "react";
import { useSelector, useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import validator from "../../../util/validator";
import constraints from "../exportCDBPDBDialog/constraints";
import ErrorTextBox from "../../../components/errorTextBox";
import { clusterRoles, daysOfWeekOptions } from "../../../config";
import { getMinDate, getMaxDate } from "../../../util/daysofmonth";
import { TrimDisk } from "../../../util/trimDisk";
import { TabView, TabPanel } from 'primereact/tabview';
import { Calendar } from 'primereact/calendar';
import { MultiSelect } from "primereact/multiselect";

const ExportLayoutDialog = (props) => {

  const [schema, setSchema] = useState(["all_schemas"]);
  const [schemaOptions, setSchemaOptions] = useState([{ label: "All Schemas", value: ["all_schemas"] }]);
  const [pageIndex, setPageIndex] = useState(0);
  const [fullPath, setFullPath] = useState("");
  const [fileName, setFileName] = useState("");
  const [dirCheck, setDirCheck] = useState("");
  const [diskSpaceCheck, setDiskSpaceCheck] = useState("");
  const [exportCheck, setExportCheck] = useState("");
  const [exportDisabled, setExportDisabled] = useState(true);
  const [schemaError, setSchemaError] = useState("");
  const [fetchError, setFetchError] = useState("");
  const [disk, setDisk] = useState(null);
  const [size, setSize] = useState(null);
  const { schemas, disks, dir_exists, export_exists, disk_space, isLoading, errorMessage } = useSelector((state) => state.layout);
  const [includeData, setIncludeData] = useState(true);
  const [includeDataSchedule, setIncludeDataSchedule] = useState(true);
  const dispatch = useDispatch();
  const { accessToken } = useSelector((state) => state.app);
  const [scheduleApiTooltip, setScheduleApiTooltip] = useState(" ");
  const [nowApiTooltip, setNowApiTooltip] = useState(" ");
  const [zipped, setZipped] = useState(false);
  const [scheduleZipped, setScheduleZipped] = useState(false);
  const [time, setTime] = useState(new Date());
  const [toggle, setToggle] = useState(0);
  const [timeError, setTimeError] = useState("");

  const [daysOfMonth, setDaysOfMonth] = useState(null);
  const [daysOfWeek, setDaysOfWeek] = useState(null);
  const minDate = useRef(getMinDate());
  const maxDate = useRef(getMaxDate());

  const validate = () => {

    let errors = [];

    let error = null;

    if (toggle === 1) {

      error = validator(constraints)("time", time);
      (error == null || errors.push(error)) && setTimeError(error);
    }


    return errors;

  }

  const onRestApi1Click = async () => {

    let errors = validate();
    if (errors.length === 0) {


      if (exportDisabled) setNowApiTooltip(" ");
      else setNowApiTooltip(await copyToClipboard(createAPIcommand({

        "pdb_id": props.node.type.includes(clusterRoles.PDB) ? props.node.location : null,
        "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.location : props.node.id,
        "schedule": false,
        "schema": schema,
        "disk": disk,
        "keep_days": props.node.size_scheme?.export,
        "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
        "file_name": fileName,
        "full_path": fullPath,
        "include_data": includeData,
      }, accessToken, 'export_cdb_pdb/')))


    }

  }

  const onRestApi2Click = async () => {

    let errors = validate();
    if (errors.length === 0) {


      if (exportDisabled) setScheduleApiTooltip(" ");
      else setScheduleApiTooltip(await copyToClipboard(createAPIcommand({

        "pdb_id": props.node.type.includes(clusterRoles.PDB) ? props.node.location : null,
        "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.location : props.node.id,
        "schedule": true,
        "schema": schema,
        "disk": disk,
        "keep_days": props.node.size_scheme?.export,
        "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
        "file_name": fileName,
        "full_path": fullPath,
        "include_data": includeDataSchedule,
        "schedule_run_time": time,
        "client_timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
        "days_of_week": daysOfWeek,
        "days_of_month": daysOfMonth,

      }, accessToken, 'export_cdb_pdb/')))


    }

  }

  useEffect(() => {
    if (errorMessage) setFetchError(errorMessage);
  }, [errorMessage])

  useEffect(() => {

    setFetchError("");

    dispatch(LayoutActions.fetchSchemas(
      {

        "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
        "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
        "pdb_name": props.node.type.includes(clusterRoles.PDB) ? props.node.name : ""
      }
    ));

    dispatch(LayoutActions.fetchDisks(
      {
        "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
        "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
        "pdb_name": props.node.type.includes(clusterRoles.PDB) ? props.node.name : "",
        "schema": ["all_schemas"]
      }));



  }, [props.node, props.root?.id, dispatch]);




  useEffect(() => {
    const temp = [];


    if (schemas?.schemas) {
      schemas.schemas.forEach((schema) => temp.push({ label: schema, value: schema }));
      temp.unshift({ label: "All Schemas", value: ["all_schemas"] });
      setSchemaOptions(temp);
    }

  }, [schemas]);


  useEffect(() => {

    if (dir_exists) {
      if (!dir_exists?.detail) setDirCheck("ok");
      else setDirCheck(dir_exists.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);

    }

  }, [dir_exists, export_exists, disk_space]);

  useEffect(() => {

    if (disk_space) {
      if (!disk_space?.detail) setDiskSpaceCheck("space available");
      else setDiskSpaceCheck(disk_space.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);
    }

  }, [disk_space, dir_exists, export_exists]);


  useEffect(() => {

    if (export_exists) {

      if (!export_exists?.detail) {

        setExportCheck("ok");
        setFileName(export_exists.file_name);
        setFullPath(export_exists.full_path);

      }
      else setExportCheck(export_exists.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);
    }

  }, [export_exists, dir_exists, disk_space]);

  useEffect(() => {

    if (disks?.disks?.[0]) {

      setDisk(disks.disks[0]);
      setSize(disks.size);

    }

  }, [disks])


  const renderExportPage = () => {

    return (<TabView className="no-ink" activeIndex={toggle} onTabChange={(e) => {
      dispatch(LayoutActions.setErrorMessage(""));
      setToggle(e.index);
    }}>
      <TabPanel header="Export now">
        <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
          <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

            <div className="field grid">
              <div className="col-fixed" style={{ width: "165px" }}>Schema:</div>
              <div className="col">
                <Dropdown
                  className="p-inputtext-sm filter-field"
                  value={schema}
                  options={schemaOptions}
                  onChange={fetchDisks}
                  placeholder={"Select schema"}
                  dropdownIcon={isLoading && !schemas && schemas !== undefined ? "pi pi-spin pi-spinner" : "pi pi-chevron-down"}
                  emptyMessage={<td className="text-sm pl-3">No non-built-in users to export</td>} />
                <div className="error-help"><small className="p-error p-d-block">{schemaError}</small></div>
              </div>
            </div>

            <div className="field grid flex">
              <div className="col-fixed" style={{ width: "165px" }}>Export path:</div>
              <div className="col">
                {isLoading && !disks && disks !== undefined ? <i className="pi pi-spin pi-spinner" /> : TrimDisk(disk, 32)}
              </div>
            </div>

            <div className="field grid">
              <div className="col-fixed" style={{ width: "165px" }}>Including data:</div>
              <div className="col">
                <Checkbox checked={includeData} onChange={(e) => setIncludeData(e.checked)} />
              </div>
            </div>


            <div className="field grid">
              <label className="col-fixed" style={{ width: "165px" }}>Compress:</label>
              <div className="col">

                <Checkbox
                  onChange={e => setZipped(e.checked)}
                  checked={zipped}>
                </Checkbox>
              </div>
            </div>



            <div className="field grid">
              <div className="col-fixed" style={{ width: "165px" }}>Size:</div>
              <div className="col">
                {disks?.display_size}
              </div>
            </div>

            <div className="field grid align-items-end">
              <div className="col-fixed"> Export will be kept for {props.node.size_scheme ? props.node.size_scheme?.export : "unk"} days </div>
              <div className="col-fixed">

                <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                <i className="pi pi-info-circle config-field icon-field" />

              </div>
            </div>
            {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          </div>
        </div>
      </TabPanel>
      <TabPanel header="Scheduled export">
        <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
          <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
            <div className="form-inner-part">

              <div className="field grid">
                <div className="col-fixed" style={{ width: "165px" }}>Schema:</div>
                <div className="col">
                  <Dropdown
                    className="p-inputtext-sm filter-field"
                    value={schema}
                    options={schemaOptions}
                    onChange={fetchDisks}
                    placeholder={"Select schema"}
                    dropdownIcon={isLoading && !schemas && schemas !== undefined ? "pi pi-spin pi-spinner" : "pi pi-chevron-down"}
                    emptyMessage={<td className="text-sm pl-3">No non-built-in users to export</td>} />
                  <div className="error-help"><small className="p-error p-d-block">{schemaError}</small></div>
                </div>
              </div>

              <div className="field grid flex">
                <div className="col-fixed" style={{ width: "165px" }}>Export path:</div>
                <div className="col">
                  {isLoading && !disks && disks !== undefined ? <i className="pi pi-spin pi-spinner" /> : TrimDisk(disk, 32)}
                </div>
              </div>

              <div className="grid">
                <label className="col-fixed multiline" style={{ width: "165px" }}>{`Days of month:\nor`}</label>
                <div className="col">
                  <Calendar viewDate={minDate.current}  readOnlyInput selectionMode="multiple" dateFormat="d" minDate={minDate.current} maxDate={maxDate.current} showOtherMonths={false} className="p-inputtext-sm" panelClassName="days-of-month-picker" value={daysOfMonth} onChange={(e) =>
                    setDaysOfMonth(e.value)} showTime={false} />
                </div>
              </div>

              <div className="grid">
                <label className="col-fixed" style={{ width: "165px" }}>Days of week:</label>
                <div className="col">
                  <MultiSelect scrollHeight="100%" className="p-inputtext-sm" maxSelectedLabels={6} selectedItemsLabel="All weekdays" selectAllLabel="All weekdays" options={daysOfWeekOptions} value={daysOfWeek} onChange={(e) => setDaysOfWeek(e.value)} />

                </div>
              </div>

            </div>

            <div className="field grid">
              <label className="col-fixed" style={{ width: "165px" }}>Time:</label>
              <div className="col">
                <Calendar showSeconds className="p-inputtext-sm" value={time} onChange={(e) => setTime(e.value)} timeOnly />
                <div className="error-help"><small className="p-error p-d-block">{timeError}</small> </div>
              </div>
            </div>

            <div className="field grid">
              <div className="col-fixed" style={{ width: "165px" }}>Including data:</div>
              <div className="col">
                <Checkbox checked={includeDataSchedule} onChange={(e) => setIncludeDataSchedule(e.checked)} />
              </div>
            </div>


            <div className="field grid">
              <label style={{ width: "165px" }} className="col-fixed">Compress:</label>
              <div className="col">

                <Checkbox
                  onChange={e => setScheduleZipped(e.checked)}
                  checked={scheduleZipped}>
                </Checkbox>
              </div>
            </div>


            <div className="field grid">
              <div className="col-fixed" style={{ width: "165px" }}>Size:</div>
              <div className="col">
                {disks?.display_size}
              </div>
            </div>

            <div className="field grid align-items-end">
              <div className="col-fixed"> Keep export for {props.node.size_scheme ? props.node.size_scheme?.export : "unk"} days </div>
              <div className="col-fixed">

                <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                <i className="pi pi-info-circle config-field icon-field" />

              </div>
            </div>
            {fetchError && <ErrorTextBox error={fetchError?.message ? fetchError.message : fetchError} />}
          </div>
        </div>
      </TabPanel>
    </TabView>
    );
  }

  const renderCheckPage = () => {

    return (<div style={{ flex: 1, padding: "10px", maxWidth: "100%" }} className="p-fluid form-container">


      <div className="field grid">
        <label className="col-fixed">{`Checking directory :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : dirCheck}
        </div>
      </div>

      <div className="field grid">
        <label className="col-fixed">{`Checking available space :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : diskSpaceCheck}
        </div>
      </div>

      <div className="field grid">
        <label className="col-fixed">{`Checking export file name :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : exportCheck}
        </div>
      </div>
      <ApiBar tooltip={toggle < 1 ? nowApiTooltip : scheduleApiTooltip} onClick={toggle < 1 ? onRestApi1Click : onRestApi2Click} />

    </div>);

  }


  const fetchDisks = (e) => {

    setSchema(e.value);
    setFetchError("");
    dispatch(LayoutActions.fetchDisks(
      {
        "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
        "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
        "pdb_name": props.node.type.includes(clusterRoles.PDB) ? props.node.name : "",
        "schema": e.value
      }));
  }


  const handleOk = () => {



    let errors = validate();
    if (errors.length === 0) {

      if (toggle < 1) {

        props.onOkClick(
          {
            "pdb_id": props.node.type.includes(clusterRoles.PDB) ? props.node.id : null,
            "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
            "schedule": false,
            "schema": schema,
            "disk": disk,
            "keep_days": props.node.size_scheme?.export,
            "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
            "file_name": fileName,
            "full_path": fullPath,
            "include_data": includeData,
          });

      }
      else {
        
        props.onOkClick(
          {
            "pdb_id": props.node.type.includes(clusterRoles.PDB) ? props.node.id : null,
            "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
            "schedule": true,
            "schema": schema,
            "disk": disk,
            "keep_days": props.node.size_scheme?.export,
            "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
            "file_name": fileName,
            "full_path": fullPath,
            "include_data": includeDataSchedule,
            "schedule_run_time": time,
            "client_timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
            "days_of_week": daysOfWeek,
            "days_of_month": daysOfMonth,
  
          });
  
      }

     
    }

  }

  const renderFooter = () => {

    if (pageIndex === 1) {

      return (<div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Back"}
          onClick={() => {
            dispatch(LayoutActions.setErrorMessage(""));
            dispatch(LayoutActions.setDirExists(null));
            dispatch(LayoutActions.setExportExists(null));
            dispatch(LayoutActions.setDiskSpace(null));
            setPageIndex(0)
          }}
          className="p-button-secondary"
          icon="pi pi-arrow-left"
          iconPos="left"
        />
        <Button
          label={"Submit"}
          icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          disabled={exportDisabled}
          onClick={handleOk} autoFocus />
      </div>)

    }
    else if (pageIndex === 0) {

      return (<div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          icon="pi pi-arrow-right"
          iconPos="right"
          label={"Next"}
          disabled={isLoading || errorMessage || fetchError}
          onClick={() => {

            let errors = [];

            let error = validator(constraints)("schema", schema);
            (error == null || errors.push(error)) && setSchemaError(error);

            if (errors.length === 0) {
              dispatch(LayoutActions.checkExportExists(
                {
                  "path": disk,
                  "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
                  "node_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
                  "pdb_name": props.node.type.includes(clusterRoles.PDB) ? props.node.name : "",
                  "schema": schema
                }

              ));

              dispatch(LayoutActions.checkDirExists(
                {
                  "path": disk,
                  "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
                  "node_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
                }

              ));

              dispatch(LayoutActions.checkDiskSpace(
                {
                  "path": disk,
                  "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
                  "node_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
                  "required_space": size,
                }
              ));

              setPageIndex(1)

            }
          }} autoFocus />
      </div>)


    }

  }



  return (
    <Dialog
      className='export'
      header={`Export ${props.node.type.includes(clusterRoles.PDB) ? props.node.name : props.node.location}`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        {pageIndex === 0 && renderExportPage()}
        {pageIndex === 1 && renderCheckPage()}

      </div>

    </Dialog>)

};

export default ExportLayoutDialog;

