import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Checkbox } from 'primereact/checkbox';
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import { Tooltip } from "primereact/tooltip";
import config from "../../../config";
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector } from "react-redux";
import validator from "../../../util/validator";
import constraints from "./constraints";
import moment from "moment";
import ErrorTextBox from "../../../components/errorTextBox";
import { clusterRoles } from "../../../config";



export const FlashbackCDBPDBDialog = (props) => {
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);
  const [guaranteed, setGuaranteed] = useState(false);
  const [name, setName] = useState(`restore_point_${moment(new Date().toUTCString()).format("DDMMYYYYhhmmss")}`);
  const [nameError, setNameError] = useState("");
  
  const onRestApiClick = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setApiTooltip(await copyToClipboard(createAPIcommand({
        "pdb_id": props.node.type.includes(clusterRoles.PDB) ? props.node.location : null,
        "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.location : props.node.location,
        "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
        "name": name,
        "guaranteed": guaranteed,
        "keep_days": props.node.size_scheme?.restore

      }, accessToken, 'flashback_cdb_pdb/')))

    }
    else setApiTooltip(" ");
  }


  const validate = () => {

    let errors = [];

    let error = validator(constraints)("name", name);
    (error == null || errors.push(error)) && setNameError(error);

    return errors;

  }

  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {
      props.onOkClick(
        {
          "pdb_id": props.node.type.includes(clusterRoles.PDB) ? props.node.id : null,
          "cdb_id": props.node.type.includes(clusterRoles.PDB) ? props.root.id : props.node.id,
          "obj_type": props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
          "name": name,
          "guaranteed": guaranteed,
          "keep_days": props.node.size_scheme?.restore


        });

    }


  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='create-flashback'
      header={`This will create flashback point for ${props.node.type.includes(clusterRoles.PDB) ? props.node.name : props.node.location}`}
      visible={props.visible}
      style={{ width: '40vw'}}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

          <div className="field grid">
            <label className="col-fixed" style={{ width: "140px" }}>{"Name:"}</label>
            <div className="col">
              <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => setName(e.target.value)} />
              <div className="error-help"><small className="p-error p-d-block">{nameError}</small></div>
            </div>
          </div>

          <div className="field grid">
            <label className="col-fixed" style={{ width: "140px" }}>{"Guaranteed:"}</label>
            <div className="col">

              <Checkbox
                onChange={e => setGuaranteed(e.checked)}
                checked={guaranteed}>
              </Checkbox>
            </div>
          </div>

          <div className="field grid align-items-end">
                <div className="col-fixed"> Keep flashback for {props.node.size_scheme ? props.node.size_scheme?.restore : "unk"} days </div>
                <div className="col-fixed">

                  <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                  <i className="pi pi-info-circle config-field icon-field" />

                </div>
              </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />


        </div>
      </div>

    </Dialog>)

};

export default FlashbackCDBPDBDialog;
