import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import ApiBar from "../../../components/apiBar";
import { useEffect, useState } from "react";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector, useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import ErrorTextBox from "../../../components/errorTextBox";
import { clusterRoles} from "../../../config";

const ImportPDBLayoutDialog = (props) => {
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { user_exists, dump_exists, isLoading, errorMessage } = useSelector((state) => state.layout);
  const [dumpCheck, setDumpCheck] = useState("");
  const [userCheck, setUserCheck] = useState("");
  const dispatch = useDispatch();

  const onRestApiClick = async () => {
    if (!user_exists || user_exists?.detail || !dump_exists || dump_exists?.error) setApiTooltip(" ");
    else setApiTooltip(await copyToClipboard(createAPIcommand({
      "export_id": props.export.name
    }, accessToken, props.node.type.includes(clusterRoles.PDB) ? 'import_pdb/' : 'import_cdb/')));
  }

  useEffect(() => {
    if (props.export) {
      dispatch(LayoutActions.checkDumpExists(
        {
          "node_id": props.node.id,
          "obj_type":  props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
          "export_id": props.export.id,

        }));
      dispatch(LayoutActions.checkUserExists(
        {
          "node_id": props.node.id,
          "obj_type":  props.node.type.includes(clusterRoles.PDB) ? clusterRoles.PDB : clusterRoles.CDB,
          "export_id": props.export.id,
        }))
    }
  }, [props.export, props.node, props.root?.id, dispatch]);

  useEffect(() => {

    if (dump_exists) {
      if (!dump_exists?.error) {
        setDumpCheck("ok");
      }
      else {
        setDumpCheck(`Dump file ${dump_exists.dump_file} does not exist.\n Cannot continue.`);
      }

    }

  }, [dump_exists]);

  useEffect(() => {

    if (user_exists) {
      if (!user_exists?.error) setUserCheck(`This will import ${user_exists.userName === "All users" ? "all users" : `user ${user_exists.userName}` } into the ${props.node.type.includes(clusterRoles.CDB) ? clusterRoles.CDB : clusterRoles.PDB} ${user_exists.pdbName}.`);
      else setUserCheck(`The schema ${user_exists.error} already exists.\nThis will DROP the user at the database and recreate it from the export file`);
    }

  }, [user_exists, props.node]);

  const renderCheckPage = () => {

    return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
      <div className="field grid">
        <label className="col-fixed">{`Checking dump file :`}</label>
        <div className="col multiline">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : dumpCheck}
        </div>
      </div>

      <div className="field grid">
        <label className="col-fixed">{`User exist check :`}</label>
        <div className="col multiline">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : userCheck}
        </div>
      </div>
      {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}
      <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />



    </div>);

  }


  const handleOk = () => {

    props.onOkClick(
      {
        "export_id": props.export.id
      });
  }

  const renderFooter = () => {


    return (<div style={{ display: "flex", justifyContent: "space-between" }}>
      <Button label={"Cancel"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
      <Button
        label={"Submit"}
        icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
        iconPos="right"
        disabled = {!user_exists || user_exists?.detail || !dump_exists || dump_exists?.error}
        onClick={handleOk} autoFocus />
    </div>)

  }



  return (
    <Dialog
      className='import'
      header={`Import to ${props.node.type.includes(clusterRoles.PDB) ? props.node.name : props.node.location}`}
      visible={props.visible}
      style={{ width: '40vw'}}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        {renderCheckPage()}

      </div>

    </Dialog>)

};

export default ImportPDBLayoutDialog;