import config from "../../../config";

const constraints = {
    port: {
        presence: {
            message: "Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535,
        }
    },
    databaseUsername: {
        presence: {
            message: "Database Username is required field.",
            allowEmpty: false
        },
        length: {
            maximum: 31,
            tooLong: "max %{count} characters"
        },
        format:                                                         
        {
            pattern: config.oracleNameRegex,
            message: "Database Username has invalid format."
        }
    },
    databasePassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        length: {
            maximum: 32,
            tooLong: "max %{count} characters"
        }
    },
    sysPwd: {
        presence: {
            message: "User Sys password is required field.",
            allowEmpty: false
        },
        length: {
            maximum: 32,
            tooLong: "max %{count} characters"
        }
    },
    systemPwd: {
        presence: {
            message: "User System password is required field.",
            allowEmpty: false
        },
        length: {
            maximum: 32,
            tooLong: "max %{count} characters"
        }
    },
    redoSize: {
        presence: {
            message: "Redo log size is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 3,
        }
    },
    osUsername: {
        presence: {
            message: "OS Username is required field.",
            allowEmpty: false
        },
        format:                                                         
        {
            pattern: config.usernameRegex,
            message: "Os Username has invalid format."
        }
    },
    privateKey: {
        presence: {
            message: "Private Key is required field.",
            allowEmpty: false
        }
    },
    osPassword: {
        presence: {
            message: "Password is required field.",
            allowEmpty: false
        },
        length: {
            minimum: 5,
            tooShort: "min %{count} characters"
        },
    },
    hostname: {
        presence: {
            message: "Hostname is required field.",
            allowEmpty: false
        },
        format: {
            pattern: config.hostNameRegex,
            message: "Host Name has invalid format."
        }
    },
    sshPort: {
        presence: {
            message: "SSH Port is required field.",
            allowEmpty: false
        },
        numericality: {
            greaterThan: 0,
            lessThanOrEqualTo: 65535
        }
    },
    dataPath: {
        presence: {
            message: "Data path is required field.",
            allowEmpty: false
        },

        format:                                                         
        {
            pattern: config.pathRegex,
            message: "Data path has invalid format."
        }
    },

    name: {
        presence: {
            message: "CDB Name is required field.",
            allowEmpty: false
        },

        format:                                                         
        {
            pattern: config.oracleSIDRegex,
            message: "CDB Name has invalid format."
        }

    },
}

export default constraints;