

import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { SelectButton } from 'primereact/selectbutton';
import { useSelector } from "react-redux";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ApiBar from "../../../components/apiBar";
import ErrorTextBox from "../../../components/errorTextBox";

export const OpenPDBDialog = (props) => {
  const [open, setOpen] = useState("RW");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const {accessToken} = useSelector((state) => state.app);
  const {errorMessage, isLoading} = useSelector((state) => state.layout);
  const options = [
    { label: 'Open Read Write', value: "RW" },
    { label: 'Open Read Only', value: "RO" },
  ];
  const handleOk = () => {
    props.onOkClick(
      {
        "pdb_id": props.node.id,
        "cdb_id": props.root.id,
        "pdb_name": props.node.name,
        "option": open
      });
  }

  const onRestApiClick = async () => 
  {
    setApiTooltip( await copyToClipboard(createAPIcommand({
      "pdb_id": props.node.location,
      "cdb_id": props.root.location,
      "pdb_name": props.node.name,
      "option": open
    }, accessToken, "open_pdb/")))
  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );
  }

  return (
    <Dialog
      className='open-pdb'
      header={`Open ${props.node.name}`}
      visible={props.visible}
      style={{ width: '40vw'}}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <SelectButton className="p-button-sm" value={open} options={options} onChange={(e) => setOpen(e.value)} />
            </div>
          </div>
          {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}
          <ApiBar tooltip = {apiTooltip} onClick = {onRestApiClick}/>
        </div>
      </div>
    </Dialog>)

};

export default OpenPDBDialog;