import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useEffect, useState } from "react";
import { InputText } from "primereact/inputtext";
import { useSelector, useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ApiBar from "../../../components/apiBar";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ErrorTextBox from "../../../components/errorTextBox";

export const RenamePDBDialog = (props) => {

  const [name, setName] = useState("");
  const { valid_name, isLoading, errorMessage } = useSelector((state) => state.layout);
  const dispatch = useDispatch();
  const [nameError, setNameError] = useState(null);
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);

  const validatePdb = () => {
    let error = validator(constraints)("name", name);
    setNameError(error);
    if (!error) {
      dispatch(LayoutActions.checkPDBName({ "cdb_id": props.node.ex_params.cdb_id, "pdb_name": name }));
    }
  }

  useEffect(() => {
    if (valid_name) {
      if (valid_name?.error)
        setNameError(valid_name.error);
      else setNameError(valid_name?.response)
    }
  }, [valid_name]);

  const onRestApiClick = async () => {
    if (!nameError) setNameError("name not checked");
    if (nameError === "name available") {
      setApiTooltip(await copyToClipboard(createAPIcommand(
        {
          "cdb_id": props.root.location,
          "pdb_id": props.node.location,
          "rename": name
        }, accessToken, "rename_pdb/")));
    }
    else setApiTooltip(" ");
  }

  const handleOk = () => {
    if (!nameError) setNameError("name not checked");
    if (nameError === "name available") {
      props.onOkClick({ "cdb_id": props.root.id, "pdb_id": props.node.id, "rename": name });
    }
  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }

  return (
    <Dialog
      className='rename-pdb'
      header={`Rename ${props.node.name}`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <label style={{ marginTop: "7px" }} className="col-fixed align-self-start">{"Name:"}</label>
            <div className="col">
              <span className="p-input-icon-right flex">
                <i className={isLoading ? "pi pi-spin pi-spinner align-self-center" : ""} />
                <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => {
                  setName(e.target.value);
                  setNameError("name not checked");
                  dispatch(LayoutActions.setValidName(undefined));
                }} />
              </span>
              <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
            </div>
            <div className="col-fixed">
              <Button style={{ width: "80px" }} className="p-button-sm" label="Check" onClick={validatePdb} />
            </div>
          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>
      </div>
    </Dialog>)
};

export default RenamePDBDialog;