import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector } from "react-redux";
import moment from "moment";
import { Tag } from "primereact/tag";
import ErrorTextBox from "../../../components/errorTextBox";
import validator from "../../../util/validator";
import constraints from "./constraints";
import config from "../../../config";

const RestoreFlashbackLayoutDialog = (props) => {
  const [restoreConfirm, setRestoreConfirm] = useState("");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const [restoreConfirmError, setRestoreConfirmError] = useState("");
  const {errorMessage, isLoading} = useSelector((state) => state.layout);

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("restoreConfirm", restoreConfirm);
    (error == null || errors.push(error)) && setRestoreConfirmError(error);

    return errors;

  }

  const onRestApiClick = async () => {

    let errors = validate();

    if (errors.length === 0) {
      setApiTooltip(await copyToClipboard(createAPIcommand({
        "flashback_id": props.flashback.name
      }, accessToken, "restore_from_flashback/")))
    }
    else setApiTooltip(" ");
  }

  const handleOk = () => {
    let errors = validate();
    if (errors.length === 0) {
      props.onOkClick(
        {
          "flashback_id": props.flashback.id
        }
      );
    }
  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );
  }
  return (
    <Dialog
      className='restore-pdb'
      header={`Restore PDB flashback`}
      visible={props.visible}
      style={{width: '50vw'}}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="field">
                This will restore the pluggable database {props.flashback.node_name} to the flashback {props.flashback.name}
              </div>
              <div className="field">
                <Tag value={` All data/objects created after ${moment(props.flashback.created_date).format("MMM DD,YYYY, hh:mm A")} will be destroyed.`} icon="pi pi-exclamation-triangle" severity="warning" />
              </div>
              <div className="field grid flex align-items-center">
                <div className="col">
                  <InputText
                    className="p-inputtext-sm"
                    placeholder={`Please type "${config.restoreConfirm}" here in order to continue`}
                    value={restoreConfirm}
                    onChange={(e) => setRestoreConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{restoreConfirmError}</small>}</div>
                </div>
              </div>
            </div>
          </div>
          {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>
      </div>
    </Dialog>)
};

export default RestoreFlashbackLayoutDialog;