

import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Tag } from 'primereact/tag';
import { Dropdown } from "primereact/dropdown";
import { useState, useEffect } from "react";
import { InputText } from "primereact/inputtext";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";
import config from "../../../config";

export const RestorePDBFlashbackDialog = (props) => {

  const [targetFlashback, setTargetFlashback] = useState(null);
  const [flashbackOptions, setFlashbackOptions] = useState([]);
  const [restoreConfirm, setRestoreConfirm] = useState("");
  const [flashbackError, setFlashbackError] = useState("");
  const [restoreConfirmError, setRestoreConfirmError] = useState("");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);


  useEffect(() => {

    if (props.flashbacks) {

      const temp = [];
      props.flashbacks.forEach((flashback) => temp.push({ label: flashback.name, value: flashback}));
      setFlashbackOptions(temp)

    }

  }, [props.flashbacks]);

  const validate = () => {
    let errors = [];
    let error = validator(constraints)("targetFlashback", targetFlashback);
    (error == null || errors.push(error)) && setFlashbackError(error);

    error = validator(constraints)("restoreConfirm", restoreConfirm);
    (error == null || errors.push(error)) && setRestoreConfirmError(error);

    return errors;



  }

  const onRestApiClick = async () => {

    let errors = validate();
    if (errors.length === 0) {
      setApiTooltip(await copyToClipboard(createAPIcommand({
        "flashback_id": targetFlashback.name
      }, accessToken, "restore_from_flashback/")))
    }
    else setApiTooltip(" ");

  }

  const handleOk = () => {

    let errors = validate();
    if (errors.length === 0) {

      props.onOkClick(
        {
          "flashback_id": targetFlashback.id,
        });

    }
  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          disabled = {!targetFlashback || targetFlashback?.status === false}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }

  return (
    <Dialog
      className='restore-flashback'
      header={`Choose restore point`}
      visible={props.visible}
      style={{ width: '50vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="title">{"Choose flashback to restore:"}</div>
              <div className="field">
                <Dropdown
                  className="p-inputtext-sm"
                  value={targetFlashback}
                  options={flashbackOptions}
                  onChange={(e) => setTargetFlashback(e.value)}
                  placeholder="Select flashback"
                  emptyMessage = "no flashbacks found"  
                  />
                <div className="error-help">{flashbackError && <small className="p-error p-d-block">{flashbackError}</small>}</div>
              </div>
              <div className="field">
                <Tag icon="pi pi-exclamation-triangle" className="field" severity="warning" value="Once a pluggable database is restored previous data will be overwritten.This operation is not reversible." />

              </div>
              <div className="field grid flex align-items-center">
                <div className="col">
                  <InputText
                    className="p-inputtext-sm"
                    placeholder={`Please type "${config.restoreConfirm}" here in order to continue`}
                    value={restoreConfirm}
                    onChange={(e) => setRestoreConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{restoreConfirmError}</small>}</div>
                </div>
              </div>
            </div>
          </div>
          {(errorMessage || targetFlashback?.status === false) && <ErrorTextBox error={errorMessage ? errorMessage?.message ? errorMessage.message : errorMessage : "This flashback did not complete successfully"} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>
      </div>
    </Dialog>)

};

export default RestorePDBFlashbackDialog;