import { useState, useEffect, useRef, useCallback } from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Checkbox } from 'primereact/checkbox';
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Tooltip } from 'primereact/tooltip';
import { Slider } from 'primereact/slider';
import validator from "../../../util/validator";
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from '../../../components/errorTextBox';
import ApiBar from '../../../components/apiBar';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { types, repositoryOptions, clusterSizeOptions, pgVersionOptions } from '../../../config';
import { base64Encode } from '../../../util/base64Encode';
import getValidationError from '../../../util/errorPrompter';
import { Tag } from 'primereact/tag';
import { footer } from '../../../util/passwordHelpers';


const AddPostgresPrimaryDialog = ({ visible, onHide, onOkClick }) => {


    const onPopupSubmit = (accept) => onSaveClick(accept);
    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const [submitting, setSubmitting] = useState(false);
    const caller = useRef('submit-button');
    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();

    // First Page elements
    const [osUsername, setOsUsername] = useState("root");
    const [osPassword, setOsPassword] = useState("");
    const [databaseUsername, setDatabaseUsername] = useState("clusteradmin");
    const [databasePassword, setDatabasePassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState(22);
    const [privateKey, setPrivateKey] = useState("");
    const [repository, setRepository] = useState(repositoryOptions[0].value);
    const [softwareVersion, setSoftwareVersion] = useState(17);

    // Second Page elements
    const [pgData, setPgData] = useState("/123cluster/postgres/pgdata1");
    const [port, setPort] = useState(5432);
    const [pgUserPassword, setPgUserPassword] = useState("");
    const [pgUserPasswordError, setPgUserPasswordError] = useState("");
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);
    const [clusterName, setClusterName] = useState("");
    const [clusterNameError, setClusterNameError] = useState("");
    const submitButtonRef = useRef(null);


    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");
    const [databaseUsernameError, setDatabaseUsernameError] = useState("");
    const [databasePasswordError, setDatabasePasswordError] = useState("");
    // Second Page element errors
    const [pgDataError, setPgDataError] = useState("");
    const [portError, setPortError] = useState("");
    const [ssl, setSSL] = useState(false);
    const [certificateAuth, setCertificateAuth] = useState(false);

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { isLoading, port_exists, path_exists, free_ip, errorMessage } = useSelector((state) => state.layout);

    const onSaveClick = useCallback(async (dropInventories) => {

        const cluster =
        {

            name: clusterName

        }

        const host =
        {
            name: hostname,
            ssh_port: sshPort,
            username: osUsername,
            password: base64Encode(osPassword),
            private_key: privateKey,
        };

        const ex_params =
        {
            repository: repository,
            pg_password: base64Encode(pgUserPassword),
            ssl: ssl,
            certificate_authentication: certificateAuth,
        }

        if (typeof visible === 'string') {
            const proxmox_params = visible.split('@');
            const osInfo = proxmox_params[0].split(' ');
            ex_params['service_name'] = proxmox_params[1]
            ex_params['distributor'] = osInfo[0]
            ex_params['release'] = osInfo[1]
            ex_params['proxmox'] = true

        }

        const data = {
            host,
            cluster,
            ex_params,
            name: cluster.name,
            auto_delete: automaticDelete,
            auto_delete_days: automaticDeleteDays,
            drop_inventories: dropInventories,
            version: softwareVersion,
            dir_123Cluster: pgData,
            sizing: clusterSize,
            port: port,
            database_user: databaseUsername,
            database_pwd: base64Encode(databasePassword),
        }

        if (caller.current === 'submit-button') {


            if (onOkClick)
                onOkClick(data);

        }
        else setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_pg_primary/")));

        setSubmitting(false);
    }, [accessToken, automaticDelete, automaticDeleteDays, certificateAuth, clusterSize,  clusterName,
        databasePassword, databaseUsername, hostname, osPassword, osUsername, pgData, pgUserPassword, port, privateKey, onOkClick, repository, softwareVersion, sshPort, ssl, visible]);

    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {

                setPortError("");
                setSubmitting(false);

            }
            else {

                if (port_exists?.exists) {

                    setPortError("already exists");
                    setSubmitting(false);
                }
                else {

                    setPortError("ok");

                }

            }

        }
    }, [port_exists])

    useEffect(() => {

        if (free_ip && !free_ip?.detail) setHostname(free_ip?.available_ip);

    }, [free_ip])

    useEffect(() => {
        if (path_exists && typeof (path_exists) === "object") {
            if (path_exists?.detail) {

                setPgDataError("");
                setSubmitting(false);

            }

            else {

                if (path_exists?.exists) {


                    setPgDataError("already exists");
                    setSubmitting(false);
                }
                else {

                    setPgDataError("ok");

                }

            }

        }
    }, [path_exists])


    useEffect(() => {

        const customEvent = new Event('beforeinput', { bubbles: true });

        const validateSecond = () => {

            let errors = [];

            let error = validator(constraints)("pgUserPassword", pgUserPassword);
            (error == null || errors.push(error)) && setPgUserPasswordError(error);


            error = validator(constraints)("databaseUsername", databaseUsername);
            (error == null || errors.push(error)) && setDatabaseUsernameError(error);

            error = validator(constraints)("databasePassword", databasePassword);
            (error == null || errors.push(error)) && setDatabasePasswordError(error);

            return errors;


        }

        if (submitting) {
            const errors = validateSecond();
            if (errors.length === 0) {


                if (typeof visible !== 'string') {

                    if (portError !== "ok") {


                        dispatch(LayoutActions.checkPortExists(
                            {
                                host: hostname,
                                port: port,
                                os_username: osUsername,
                                os_password: base64Encode(osPassword),
                                private_key: privateKey,
                                ssh_port: sshPort,
                            }))

                    }
                    if (pgDataError !== "ok") {
                        dispatch(LayoutActions.checkPathExists(
                            {
                                host: hostname,
                                path: pgData,
                                os_username: osUsername,
                                os_password: base64Encode(osPassword),
                                private_key: privateKey,
                                ssh_port: sshPort,
                                type: "pg_data"
                            }))
                    }

                }

                if ((portError === "ok" && pgDataError === "ok") || typeof visible === 'string') {

                    dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}:${port}`, 'type': types.POSTGRES }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))


                }

            }
            else caller.current === "submit-button" ? submitButtonRef.current.dispatchEvent(customEvent) : setApiTooltip(getValidationError());


        }

    }, [submitting, dispatch, hostname, onSaveClick, osPassword, osUsername, port, portError, pgDataError, privateKey, sshPort, pgData,
        databaseUsername, databasePassword, pgUserPassword, visible])


    useEffect(() => {


        if (visible === false) {

            setPageIndex(0);

            // First page elements and errors
            setPort(5432);
            setPortError("");
            setOsUsername("root");
            setOsUsernameError("");
            setOsPassword("");
            setOsPasswordError("");
            setDatabaseUsername("clusteradmin");
            setDatabaseUsernameError("");
            setDatabasePassword("");
            setDatabasePasswordError("");
            setHostname("");
            setHostnameError("");
            setSshPort(22);
            setSshPortError("");
            setPrivateKey("");
            setPrivateKeyError("");
            setSoftwareVersion(17);
            setRepository(repositoryOptions[0].value);
            setClusterName("");
            setClusterNameError("");


            // Second page elements and errors
            setPgData("/123cluster/postgres/pgdata1");
            setPgDataError("");
            setApiTooltip(" ");
            setClusterSize(clusterSizeOptions[0].value);
            setAutomaticDelete(false);
            setAutomaticDeleteDays(1);
            setSSL(false);
            setPgUserPasswordError("")
            setPgUserPassword("");
        }
        if (typeof visible === 'string') dispatch(LayoutActions.checkFreeIp({ 'service_name': visible.split('@')[1] }))


    }, [visible, dispatch]);


    const validateFirst = () => {

        let errors = [];


        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);


        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }


        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        error = validator(constraints)("clusterName", clusterName);
        (error == null || errors.push(error)) && setClusterNameError(error);

        return errors;

    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {
            if (typeof visible === 'string') setPageIndex(1);
            else dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))

        }
    }

    const onSecondBackClick = () => {
        setPortError("");
        setPgDataError("");
        setPageIndex(0);
    }

    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            className='submit-button'
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            iconPos="right"
                            id='submit-button'
                            ref={submitButtonRef}
                            onClick={() => {

                                caller.current = 'submit-button';
                                setSubmitting(true);

                            }} autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Cluster Name:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Cluster Name" value={clusterName} onChange={(e) => setClusterName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{clusterNameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="OS Username" value={osUsername} disabled={typeof visible === 'string'} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText disabled={typeof visible === 'string'} className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    const renderPageSecond = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Username:"}</label>
                            <div className="col">
                                <InputText className="p-inputtext-sm" placeholder="Database Username" value={databaseUsername} onChange={(e) => setDatabaseUsername(e.target.value)} />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databaseUsernameError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm" placeholder="Database Password"
                                    value={databasePassword}
                                    onChange={(e) => setDatabasePassword(e.target.value)}
                                    footer={() => footer(databasePassword)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databasePasswordError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Software version:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm filter-field"
                                    options={pgVersionOptions}
                                    value={softwareVersion}
                                    onChange={(e) => {

                                        setSoftwareVersion(e.value);
                                    }}
                                />
                            </div>

                        </div>

                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"$PGDATA:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <i className={`pi ${isLoading && path_exists === "pg_data" ? "pi-spin pi-spinner align-self-center" : "pi-info-circle"} pg_data`} />
                                    <Tooltip target=".pg_data"
                                        content="This is the directory where the new instance files and logs will reside." />

                                    <InputText className="p-inputtext-sm" placeholder="$PGDATA" value={pgData} onChange={(e) => {
                                        setPgData(e.target.value);
                                        setPgDataError("check $PGDATA");
                                    }} />
                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(pgDataError !== 'ok') ? "p-error" : ""} p-d-block`}>{pgDataError}</small>
                                </div>
                            </div>

                            {typeof visible === 'string' ? null : <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    style={{ width: "80px" }}
                                    label="Check"
                                    onClick={() => {
                                        let error = validator(constraints)("pgData", pgData);
                                        setPgDataError(error);

                                        if (error === null) dispatch(LayoutActions.checkPathExists(
                                            {
                                                host: hostname,
                                                path: pgData,
                                                os_username: osUsername,
                                                os_password: base64Encode(osPassword),
                                                private_key: privateKey,
                                                ssh_port: sshPort,
                                                type: "pg_data"
                                            }))
                                    }}
                                />
                            </div>}
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Port" value={port} onValueChange={(e) => {
                                        setPort(e.value);
                                        setPortError("check port");

                                    }} />
                                    <i className={isLoading && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(portError !== 'ok') ? "p-error" : ""} p-d-block`}>{portError}</small>
                                </div>
                            </div>

                            {typeof visible === 'string' ? null : <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("port", port);
                                        setPortError(error);
                                        if (error === null)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: port,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                }))
                                    }

                                    }
                                />
                            </div>}
                        </div>

                    </div>

                    <div>
                        <div className="field grid">
                            <label className="col-fixed align-self-start" style={{ width: "220px" }}>{`Set a OS user 'postgres' password:`}</label>
                            <div className="col">

                                <Password
                                    autoComplete="new-password"
                                    toggleMask
                                    feedback
                                    className="p-inputtext-sm"
                                    value={pgUserPassword}
                                    placeholder="User 'postgres' password"
                                    footer={() => footer(pgUserPassword)}
                                    onChange={(e) => setPgUserPassword(e.target.value)} />
                                <Tag style={{ lineHeight: 1 }} icon="pi pi-exclamation-triangle" severity="warning" rounded value="This password will override postgres user password!" />
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className="p-error p-d-block">{pgUserPasswordError}</small>
                                </div>
                            </div>

                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Use SSL:"}</label>
                            <div className="col">
                                <Checkbox onChange={(e) => {
                                    const value = e.checked;
                                    if (!value) setCertificateAuth(false);
                                    setSSL(value);

                                }} checked={ssl} value={ssl} />
                            </div>
                        </div>
                    </div>

                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Authenticate by certificate:"}</label>
                            <div className="col">
                                <Checkbox onChange={() => setCertificateAuth(!certificateAuth)} checked={certificateAuth} disabled={!ssl} value={certificateAuth} />
                            </div>
                        </div>
                    </div>

                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Install from:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm"
                                    options={repositoryOptions}
                                    value={repository}
                                    onChange={(e) => setRepository(e.value)}
                                />
                            </div>
                        </div>
                    </div>


                    <div>
                        <div className="field grid flex align-items-center">
                            <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "220px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                            <div className="col-fixed">
                                <Checkbox inputId="cb1" value="Automatic delete disabled:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                            </div>
                            <div className="col">
                                <Slider
                                    disabled={!automaticDelete}
                                    min={1}
                                    max={365}
                                    value={automaticDeleteDays}
                                    onChange={(e) => setAutomaticDeleteDays(e.value)} />
                            </div>

                        </div>
                    </div>


                    <div>
                        <div className='field grid'>

                            <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                            <div className='col'>

                                <Dropdown className="p-inputtext-sm"
                                    value={clusterSize}
                                    onChange={(e) => setClusterSize(e.value)}
                                    options={clusterSizeOptions}
                                    placeholder="Size options" />

                            </div>

                        </div>

                    </div>


                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => {
                    caller.current = 'rest-api-bar';
                    setSubmitting(true);
                }} />
            </div>



        );
    }


    return (
        <Dialog
            className='add-pg-primary'
            header={"Install a new PostgeSQL Cluster"}
            visible={visible}
            style={{ width: '50vw' }}
            onHide={onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller.current)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>

        </Dialog>
    );
}


export default AddPostgresPrimaryDialog;