import { useState, useEffect, useRef, useCallback } from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { InputNumber } from "primereact/inputnumber";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Tooltip } from 'primereact/tooltip';
import validator from "../../../util/validator";
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { base64Encode } from '../../../util/base64Encode';
import ApiBar from '../../../components/apiBar';
import { ConfirmPopup } from 'primereact/confirmpopup';
import ErrorTextBox from '../../../components/errorTextBox';
import { types, clusterSizeOptions } from '../../../config';
import getValidationError from '../../../util/errorPrompter';
import { footer } from '../../../util/passwordHelpers';

const AddPostgresStandbyDialog = ({ visible, onHide, onOkClick, node }) => {

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();
    const [submitting, setSubmitting] = useState(false);

    const caller = useRef('submit-button');


    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);

    // First Page elements
    const [osUsername, setOsUsername] = useState("root");
    const [osPassword, setOsPassword] = useState("");
    const [hostname, setHostname] = useState("");
    const [sshPort, setSshPort] = useState(22);
    const [privateKey, setPrivateKey] = useState("");
    const [repmgrPwd, setRepmgrPwd] = useState("")
    const [repmgrPwdError, setRepmgrPwdError] = useState("");

    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);

    // Second Page elements
    const [pgData, setPgData] = useState("/123cluster/postgres/pgdata1");
    const [port, setPort] = useState(5432);
    const submitButtonRef = useRef(null);




    // First Page element errors
    const [osUsernameError, setOsUsernameError] = useState("");
    const [osPasswordError, setOsPasswordError] = useState("");
    const [hostnameError, setHostnameError] = useState("");
    const [sshPortError, setSshPortError] = useState("");
    const [privateKeyError, setPrivateKeyError] = useState("");
    const [showWarningPopup, setShowWarningPopup] = useState(false);

    // Second Page element errors
    const [pgDataError, setPgDataError] = useState("");
    const [portError, setPortError] = useState("");

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { isLoading, port_exists, path_exists, free_ip, errorMessage } = useSelector((state) => state.layout);

    useEffect(() => {

        if (free_ip && !free_ip?.detail) setHostname(free_ip?.available_ip);

    }, [free_ip])


    const onSaveClick = useCallback(async (dropInventories) => {

        const cluster =
        {

            name: `${hostname}:${port}`

        }

        const host =
        {
            name: hostname,
            ssh_port: sshPort,
            username: osUsername,
            password: base64Encode(osPassword),
            private_key: privateKey,
        };

        const ex_params =
        {
            pg_primary_id: node.id,
            repmgr_pwd: base64Encode(repmgrPwd),
        }

        if (typeof visible === 'string') {
            ex_params['proxmox'] = true
            ex_params['service_name'] = visible
        }

        const data = {
            host,
            cluster,
            ex_params,
            name: cluster.name,
            version: node.version,
            drop_inventories: dropInventories,
            dir_123Cluster: pgData,
            port: port,
            sizing: clusterSize,
        }
        if (caller.current === 'submit-button') {


            if (onOkClick)
                onOkClick(data);

        }
        else setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_pg_standby/")))
        setSubmitting(false);
    }, [accessToken, clusterSize,hostname, osPassword, osUsername, pgData, port,
        privateKey, sshPort, node, repmgrPwd, visible, onOkClick]);


    useEffect(() => {
        if (port_exists) {
            if (port_exists?.detail) {

                setPortError("");
                setSubmitting(false);

            }
            else {


                if (port_exists?.exists) {

                    setPortError("already exists");
                    setSubmitting(false);
                }
                else {

                    setPortError("ok");

                }


            }
        }
    }, [port_exists])

    useEffect(() => {
        if (path_exists && typeof (path_exists) === "object") {
            if (path_exists?.detail) {

                setPgDataError("")
                setSubmitting(false);

            }

            else {

                if (path_exists?.exists) {


                    setPgDataError("already exists")
                    setSubmitting(false);
                }
                else {

                    setPgDataError("ok")

                }

            }

        }
    }, [path_exists])

    useEffect(() => {


        const customEvent = new Event('beforeinput', { bubbles: true });


        const validateSecond = () => {
            const errors = []

            let error = validator(constraints)("repmgrUserPassword", repmgrPwd);
            (error == null || errors.push(error)) && setRepmgrPwdError(error);
            return errors;

        }

        if (submitting) {


            const errors = validateSecond();
            if (errors.length === 0) {

                if (typeof visible !== 'string') {


                    if (portError !== "ok") {


                        dispatch(LayoutActions.checkPortExists(
                            {
                                host: hostname,
                                port: port,
                                os_username: osUsername,
                                os_password: base64Encode(osPassword),
                                private_key: privateKey,
                                ssh_port: sshPort,
                            }))

                    }
                    if (pgDataError !== "ok") {
                        dispatch(LayoutActions.checkPathExists(
                            {
                                host: hostname,
                                path: pgData,
                                os_username: osUsername,
                                os_password: base64Encode(osPassword),
                                private_key: privateKey,
                                ssh_port: sshPort,
                                type: "pg_data"
                            }))
                    }

                }



                if ((portError === "ok" && pgDataError === "ok") || typeof visible === 'string') {

                    dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}:${port}`, 'type': types.POSTGRES }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))


                }

            }
            else caller.current === "submit-button" ? submitButtonRef.current.dispatchEvent(customEvent) : setApiTooltip(getValidationError());

        }

    }, [submitting, visible, dispatch, hostname, onSaveClick, osPassword, osUsername, port, portError, pgDataError, privateKey, sshPort, pgData, repmgrPwd])



    useEffect(() => {
        if (visible === false) {

            setPageIndex(0);

            // First page elements and errors
            setPort(5432);
            setPortError("");
            setOsUsername("root");
            setOsUsernameError("");
            setOsPassword("");
            setOsPasswordError("");
            setHostname("");
            setHostnameError("");
            setSshPort(22);
            setSshPortError("");
            setPrivateKey("");
            setPrivateKeyError("");


            // Second page elements and errors
            setPgData("/123cluster/postgres/pgdata1");
            setPgDataError("");
            setApiTooltip(" ");

            if (node?.nodes?.length > 0) {

                setRepmgrPwd(node?.ex_params?.repmgr_pwd)

            }
            else {
                setRepmgrPwd("");
            }

        }

        if (typeof visible === 'string') dispatch(LayoutActions.checkFreeIp({ 'service_name': visible }))

    }, [visible, dispatch, node]);


    const onPopupSubmit = (accept) => onSaveClick(accept);

    const validateFirst = () => {

        let errors = [];

        let error = validator(constraints)("osUsername", osUsername);
        (error == null || errors.push(error)) && setOsUsernameError(error);

        error = validator(constraints)("privateKey", privateKey);
        (error == null || errors.push(error)) && setPrivateKeyError(error);

        if (osPassword) {
            setPrivateKeyError("");
            errors.pop();

        }

        error = validator(constraints)("osPassword", osPassword);
        (error == null || errors.push(error)) && setOsPasswordError(error);

        if (privateKey) {
            setOsPasswordError("");
            errors.pop();

        }

        error = validator(constraints)("hostname", hostname);
        (error == null || errors.push(error)) && setHostnameError(error);

        error = validator(constraints)("sshPort", sshPort);
        (error == null || errors.push(error)) && setSshPortError(error);

        return errors;

    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) {

            typeof visible === 'string' ? setPageIndex(1) : dispatch(LayoutActions.checkConnection(
                {
                    host: hostname,
                    os_username: osUsername,
                    os_password: base64Encode(osPassword),
                    private_key: privateKey,
                    ssh_port: sshPort,
                }, setPageIndex))
        }
    }

    const onSecondBackClick = () => {
        setPortError("");
        setPgDataError("");
        dispatch(LayoutActions.setErrorMessage(""));
        setPageIndex(0);
    }



    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            id="submit-button"
                            className='submit-button'
                            iconPos="right"
                            ref={submitButtonRef}
                            onClick={() => {
                                setShowWarningPopup(true);
                            }}
                            autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" disabled={typeof visible === 'string'} placeholder="OS Username" value={osUsername} onChange={(e) => setOsUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"OS Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                footer={() => footer(osPassword)}
                                className="p-inputtext-sm"
                                placeholder="OS Password"
                                value={osPassword}
                                disabled={privateKey}
                                onChange={(e) => setOsPassword(e.target.value)}
                                autoComplete="new-password"
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{osPasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Private Key:"}</label>
                        <div className="col">
                            <InputText disabled={osPassword} className="p-inputtext-sm" placeholder="Private Key" value={privateKey} onChange={(e) => setPrivateKey(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{privateKeyError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Hostname/IP:"}</label>
                        <div className="col">
                            <InputText disabled={typeof visible === 'string'} className="p-inputtext-sm" placeholder="Hostname/IP" value={hostname} onChange={(e) => setHostname(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{hostnameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"SSH Port:"}</label>
                        <div className="col">
                            <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="SSH Port" value={sshPort} onValueChange={(e) => setSshPort(e.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{sshPortError}</small>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    const renderPageSecond = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid">
                            <label className="col-fixed align-self-start" style={{ width: "220px" }}>{"$PGDATA:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <i className={`pi ${isLoading && path_exists === "pg_data" ? "pi-spin pi-spinner align-self-center" : "pi-info-circle"} pg_data`} />
                                    <Tooltip target=".pg_data"
                                        content="This is the directory where the new instance files and logs will reside." />

                                    <InputText className="p-inputtext-sm" placeholder="$PGDATA" value={pgData} onChange={(e) => {
                                        setPgData(e.target.value);
                                        setPgDataError("check $PGDATA");
                                    }} />
                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(pgDataError !== 'ok') ? "p-error" : ""} p-d-block`}>{pgDataError}</small>
                                </div>
                            </div>
                            {typeof visible === 'string' ? null : <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("pgData", pgData);
                                        setPgDataError(error);

                                        if (!error) dispatch(LayoutActions.checkPathExists(
                                            {
                                                host: hostname,
                                                path: pgData,
                                                os_username: osUsername,
                                                os_password: base64Encode(osPassword),
                                                private_key: privateKey,
                                                ssh_port: sshPort,
                                                type: "pg_data"
                                            }))
                                    }}
                                />
                            </div>}
                        </div>
                    </div>
                    <div>
                        <div className="field grid flex align-items-start">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Port:"}</label>
                            <div className="col">
                                <span className="p-input-icon-right flex">
                                    <InputNumber useGrouping={false} className="p-inputtext-sm" placeholder="Port" value={port} onValueChange={(e) => {
                                        setPort(e.value);
                                        setPortError("check port");

                                    }} />
                                    <i className={isLoading && !port_exists && port_exists !== undefined ? "pi pi-spin pi-spinner align-self-center" : ""} />

                                </span>
                                <div className="error-help" style={{ display: "flex", justifyContent: "flex-start", marginRight: "5px", marginBottom: "5px" }}>
                                    <small id="name-help" className={`${(portError !== "ok") ? "p-error" : ""} p-d-block`}>{portError}</small>
                                </div>
                            </div>

                            {typeof visible === 'string' ? null : <div className="col-fixed">
                                <Button
                                    className="p-button-sm"
                                    label="Check"
                                    style={{ width: "80px" }}
                                    onClick={() => {
                                        let error = validator(constraints)("port", port);
                                        setPortError(error);
                                        if (!error)
                                            dispatch(LayoutActions.checkPortExists(
                                                {
                                                    host: hostname,
                                                    port: port,
                                                    os_username: osUsername,
                                                    os_password: base64Encode(osPassword),
                                                    private_key: privateKey,
                                                    ssh_port: sshPort,
                                                }))
                                    }

                                    }
                                />
                            </div>}
                        </div>

                    </div>

                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Set a repmgr user Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm"
                                    placeholder="Repmgr User Password"
                                    value={repmgrPwd}
                                    footer={() => footer(repmgrPwd)}
                                    disabled={node?.nodes?.length > 0}
                                    onChange={(e) => setRepmgrPwd(e.target.value)}
                                    autoComplete="new-password"
                                    toggleMask />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{repmgrPwdError}</small>
                        </div>
                    </div>


                    <div>
                        <div className='field grid'>

                            <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                            <div className='col'>

                                <Dropdown className="p-inputtext-sm"
                                    value={clusterSize}
                                    onChange={(e) => setClusterSize(e.value)}
                                    options={clusterSizeOptions}
                                    placeholder="Size options" />

                            </div>

                        </div>

                    </div>



                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => {

                    caller.current = 'rest-api-bar';
                    setSubmitting(true);

                }} />
            </div>



        );
    }


    return (
        <Dialog
            className='add-pg-standby'
            header={`Install a new PostgeSQL Standby for ${node.name}`}
            visible={visible}
            style={{ width: '50vw' }}
            onHide={onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller.current)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
                <ConfirmPopup
                    icon="pi pi-exclamation-triangle"
                    target={document.getElementById("submit-button")}
                    className="multiline"
                    visible={showWarningPopup}
                    onHide={() => setShowWarningPopup(false)}
                    accept={() => {
                        caller.current = 'submit-button';
                        setSubmitting(true);
                    }}
                    reject={onHide}
                    message={"This process will overwrite ssh keys.\nContinue?"} />
            </div>

        </Dialog>
    );
}


export default AddPostgresStandbyDialog;