import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { useState, useEffect } from "react";
import { InputText } from "primereact/inputtext";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";

export const ClonePostgresStandbyDialog = (props) => {

  const [targetStandby, setTargetStandby] = useState(null);
  const [standbyOptions, setStandbyOptions] = useState([]);
  const [cloneConfirm, setCloneConfirm] = useState("");
  const [cloneConfirmError, setCloneConfirmError] = useState("");
  const [targetStandbyError, setTargetStandbyError] = useState("");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);



  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {

      props.onOkClick(
        {
          "standby_id": targetStandby?.id
        });

    }

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("targetStandby", targetStandby);
    (error == null || errors.push(error)) && setTargetStandbyError(error);

    error = validator(constraints)("cloneConfirm", cloneConfirm);
    (error == null || errors.push(error)) && setCloneConfirmError(error);

    return errors;

  }

  const onRestApiClick = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setApiTooltip(await copyToClipboard(createAPIcommand({
        "standby_id": targetStandby?.name

      }, accessToken, "clone_pg_standby/")));
    }
    else setApiTooltip(" ");

  }

  useEffect(() => {

    if (props.nodes) {

      const temp = [];
      props.nodes.forEach((standby) => temp.push({ label: standby.name, value: standby }));
      setStandbyOptions(temp)

    }

  }, [props.nodes]);

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='clone-pg-standby'
      header={'Clone'}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="title">{"Choose standby to clone:"}</div>

              <div className="field">
                <Dropdown className="p-inputtext-sm"
                  value={targetStandby}
                  options={standbyOptions}
                  onChange={(e) => setTargetStandby(e.value)}
                  placeholder="Select standby" />
                <div className="error-help">{<small className="p-error p-d-block">{targetStandbyError}</small>}</div>
              </div>
              <div className="field grid flex align-items-center">
                <div className="col">

                  <InputText
                    className="p-inputtext-sm"
                    placeholder='Please type "clone me" here in order to continue'
                    value={cloneConfirm}
                    onChange={(e) => setCloneConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{cloneConfirmError}</small>}</div>

                </div>
              </div>
            </div>

          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>
      </div>

    </Dialog>)

};

export default ClonePostgresStandbyDialog;