import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import { Tag } from 'primereact/tag';
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";
import validator from "../../../util/validator";
import constraints from "./constraints";
import config, { types } from "../../../config";



export const DropDatabaseDialog = (props) => {

  const [dropConfirm, setDropConfirm] = useState("");
  const [dropConfirmError, setDropConfirmError] = useState("");

  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);

  const { errorMessage, isLoading } = useSelector((state) => state.layout);


  const onRestApiClick = async () => {
    let errors = validate();
    if (errors.length === 0) {
      setApiTooltip(await copyToClipboard(createAPIcommand({
        "db_id": props.node.name,
        "node_id": props.node.host,
        "type": props.node.type,
        "service_id": props.service_id,
        "cluster_id": props.cluster_id,

      }, accessToken, props.node.type === types.POSTGRES_K8S ? "drop_pg_database_pod" : "drop_pg_database/")))
    }
    else setApiTooltip(" ");
  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("dropConfirm", dropConfirm);
    (error == null || errors.push(error)) && setDropConfirmError(error);
    return errors;

  }

  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {

      props.onOkClick(
        {
          "db_id": props.node.id,
          "type" : props.node.type,
          "service_id": props.service_id,
          "cluster_id": props.cluster_id,

        });

    }


  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'trash'}`}
          className="p-button-danger"
          onClick={handleOk} autoFocus />
      </div>
    );

  }

  return (
    <Dialog
      className='drop-database'
      header={`Drop ${props.node.name}`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <Tag icon="pi pi-exclamation-triangle" className="field" severity="warning" value="Once a database is dropped, it cannot be recovered" />
              <InputText
                className="p-inputtext-sm"
                placeholder={`Please type "${config.dropConfirm}" here in order to continue`}
                value={dropConfirm}
                onChange={(e) => setDropConfirm(e.target.value)} />
              <div className="error-help">{<small className="p-error p-d-block">{dropConfirmError}</small>}</div>
            </div>

          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />

        </div>
      </div>

    </Dialog>)

};

export default DropDatabaseDialog;

