import { useSelector, useDispatch } from 'react-redux';
import { useEffect } from 'react';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dialog } from 'primereact/dialog';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import ErrorTextBox from '../../../components/errorTextBox';
import { loadingMsg} from '../../../config';


export default function PostgresStatusDialog(props) {

    const { pg_primary_status, isLoading } = useSelector((state) => state.layout);
    const {errorMessage} = useSelector((state) => state.layout);


    const dispatch = useDispatch();

    useEffect(() => {
        if (props.node) {

            dispatch(LayoutActions.getPgPrimaryStatus(props.node.id));

        }

    }, [props.node, dispatch])


    const idBodyTemplate = (item) => {

        return (<div>{item.id}</div>);

    }

    const nameBodyTemplate = (item) => {

        return (<div>{item.name}</div>);

    }

    const roleBodyTemplate = (item) => {

        return (<div>{item.role}</div>);

    }

    const statusBodyTemplate = (item) => {

        return (<div>{item.status}</div>);

    }


    const upstreamBodyTemplate = (item) => {

        return (<div>{item.upstream}</div>);

    }

    const repmgrdBodyTemplate = (item) => {

        return (<div>{item.repmgrd}</div>);

    }

    const pidBodyTemplate = (item) => {

        return (<div>{item.pid}</div>);

    }

    const pausedBodyTemplate = (item) => {

        return (<div>{item.paused}</div>);

    }

    const upstreamLastSeenBodyTemplate = (item) => {

        return (<div>{item.upstream_last_seen}</div>);

    }




















    return (

        <Dialog
            className='pg-primary-status'
            header={`Primary ${props.node?.name} status`}
            visible={props.visible}
            style={{ maxHeight: "90%" }}
            onHide={props.onHide}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <DataTable emptyMessage = {!pg_primary_status && isLoading ?  <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">No status data</div>} paginator = {pg_primary_status?.length > 10} rows={10} rowsPerPageOptions={[10, 20, 50, 100]} value={pg_primary_status?.length ? pg_primary_status : []} >
                        <Column field="id" header="ID" body={idBodyTemplate} sortable />
                        <Column field="name" header="Name" body={nameBodyTemplate} sortable />
                        <Column field="role" header="Role" body={roleBodyTemplate} sortable/>
                        <Column field="status" header="Status" body={statusBodyTemplate} sortable />
                        <Column field="upstream" header="Upstream" body={upstreamBodyTemplate} sortable/>
                        <Column field="repmgrd" header="Repmgrd" body={repmgrdBodyTemplate} sortable/>
                        <Column field="pid" header="PID" body={pidBodyTemplate} sortable />
                        <Column field="paused" header="Paused" body={pausedBodyTemplate} sortable />
                        <Column field="upstream_last_seen" header="Upstream last seen" body={upstreamLastSeenBodyTemplate} sortable />
                    </DataTable>
                    {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}

                </div>
            </div>

        </Dialog>);
}