import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Checkbox } from 'primereact/checkbox';
import { useState, useEffect, useRef } from "react";
import { InputText } from "primereact/inputtext";
import { TabView, TabPanel } from 'primereact/tabview';
import { Calendar } from 'primereact/calendar';
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector, useDispatch } from "react-redux";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import { Tooltip } from "primereact/tooltip";
import config, { types, daysOfWeekOptions } from "../../../config";
import { ConfigActions } from "../../../store/config/reducer";
import { LayoutActions } from "../../../store/layout/reducer";
import ErrorTextBox from "../../../components/errorTextBox";
import { MultiSelect } from "primereact/multiselect";
import { getMinDate, getMaxDate } from "../../../util/daysofmonth";
import { ConfirmPopup } from 'primereact/confirmpopup';
import notify from "../../../util/notify";
import moment from "moment";


export const PostgresExportDialog = (props) => {

  const [scheduleApiTooltip, setScheduleApiTooltip] = useState(" ");
  const [nowApiTooltip, setNowApiTooltip] = useState(" ");
  const [zipped, setZipped] = useState(false);
  const [scheduleZipped, setScheduleZipped] = useState(false);

  const [time, setTime] = useState(new Date());
  const [toggle, setToggle] = useState(0);
  const [timeError, setTimeError] = useState("");

  const [daysOfMonth, setDaysOfMonth] = useState([]);
  const [daysOfWeek, setDaysOfWeek] = useState([]);;

  const minDate = useRef(getMinDate());
  const maxDate = useRef(getMaxDate());
  const { accessToken } = useSelector((state) => state.app);
  const [name, setName] = useState(`${props.node.name}_${moment(new Date().toUTCString()).format("DDMMYYYYhhmmss")}`);
  const [nameError, setNameError] = useState("");
  const [showWarningPopup, setShowWarningPopup] = useState(false);
  const [includeData, setIncludeData] = useState(true);
  const [includePublications, setIncludePublications] = useState(true);
  const [includeSubscriptions, setIncludeSubscriptions] = useState(true);
  const [useInserts, setUseInserts] = useState(false);
  const [includeDataSchedule, setIncludeDataSchedule] = useState(true);
  const [includePublicationsSchedule, setIncludePublicationsSchedule] = useState(true);
  const [includeSubscriptionsSchedule, setIncludeSubscriptionsSchedule] = useState(true);
  const [useInsertsSchedule, setUseInsertsSchedule] = useState(false);
  const [dirCheck, setDirCheck] = useState("");
  const [diskSpaceCheck, setDiskSpaceCheck] = useState("");
  const [exportCheck, setExportCheck] = useState("");
  const [exportDisabled, setExportDisabled] = useState(true);
  const [fullPath, setFullPath] = useState("");
  const [pageIndex, setPageIndex] = useState(0);
  const { isLoading, pg_db_size, dir_exists, disk_space, export_exists, errorMessage } = useSelector((state) => state.layout);
  const { config: appConf } = useSelector((state) => state.config);
  const dispatch = useDispatch();

  useEffect(() => {



    if (!appConf) dispatch(ConfigActions.getConfig());
    else if (!fullPath)
      setFullPath(appConf?.pg_sys?.exports_path)


  }, [appConf, fullPath, dispatch])

  useEffect(() => {

    if (export_exists) {

      if (!export_exists?.detail) {

        setExportCheck("ok");

      }
      else setExportCheck(export_exists.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);
    }

  }, [export_exists, dir_exists, disk_space]);

  useEffect(() => {

    if (dir_exists) {
      if (!dir_exists?.detail) setDirCheck("ok");
      else setDirCheck(dir_exists.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);

    }

  }, [dir_exists, export_exists, disk_space]);

  useEffect(() => {

    if (disk_space) {
      if (!disk_space?.detail) setDiskSpaceCheck("space available");
      else setDiskSpaceCheck(disk_space.detail);

      if (!export_exists?.detail && !disk_space?.detail
        && !dir_exists?.detail) setExportDisabled(false);
      else setExportDisabled(true);
    }

  }, [disk_space, dir_exists, export_exists]);

  useEffect(() => {

    dispatch(LayoutActions.checkPgDatabaseSize({ "node_id": props.node.id, "type": props.node.type }));

  }, [props.node, dispatch])

  const onRestApi1Click = async () => {

    setNowApiTooltip(await copyToClipboard(createAPIcommand({
      "schedule": false,
      "node_id": props.node.name,
      "export_name": name,
      "compress": zipped,
      "keep_days": props.node.size_scheme?.export,
      "include_data": includeData,
      "include_publications": includePublications,
      "include_subscriptions": includeSubscriptions,
      "use_inserts": useInserts,
      "export_path": fullPath,
      "type": props.node.type

    }, accessToken,  props.node.type === types.POSTGRES_K8S ? 'export_pg_pod' : 'export_pg_node/')))

  }
  
  const onRestApi2Click = async () => {

    setScheduleApiTooltip(await copyToClipboard(createAPIcommand({
      "schedule": true,
      "node_id": props.node.name,
      "export_name": name,
      "compress": scheduleZipped,
      "keep_days": props.node.size_scheme?.export,
      "include_data": includeDataSchedule,
      "include_publications": includePublicationsSchedule,
      "include_subscriptions": includeSubscriptionsSchedule,
      "use_inserts": useInsertsSchedule,
      "export_path": fullPath,
      "schedule_run_time": time,
      "client_timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
      "days_of_week": daysOfWeek,
      "days_of_month": daysOfMonth,
      "type": props.node.type

    }, accessToken, props.node.type === types.POSTGRES_K8S ? 'export_pg_pod' : 'export_pg_node/')))

  }


  const validate = () => {

    let errors = [];

    let error = validator(constraints)("name", name);
    (error == null || errors.push(error)) && setNameError(error);
    if (toggle === 1) {

      error = validator(constraints)("time", time);
      (error == null || errors.push(error)) && setTimeError(error);
    }

    return errors;

  }

  const handleOk = () => {

    if (pg_db_size?.size > config.GB) setShowWarningPopup(true);
    else submit();


  }


  const submit = () => {

    if (pg_db_size?.size > pg_db_size?.free_space) {
      notify.warning("Free space at the selected directory is less than the schema size. Please select another directory")
    }

    else {
      if (toggle < 1) {

          props.onOkClick(
            {
              "schedule": false,
              "node_id": props.node.id,
              "export_name": name,
              "compress": zipped,
              "keep_days": props.node.size_scheme?.export,
              "include_data": includeData,
              "include_publications": includePublications,
              "include_subscriptions": includeSubscriptions,
              "use_inserts": useInserts,
              "export_path": fullPath,
              "type": props.node.type
            });
      }
      else
      {

        props.onOkClick(
          {
            "schedule": true,
            "node_id": props.node.id,
            "export_name": name,
            "compress": scheduleZipped,
            "keep_days": props.node.size_scheme?.export,
            "include_data": includeDataSchedule,
            "include_publications": includePublicationsSchedule,
            "include_subscriptions": includeSubscriptionsSchedule,
            "use_inserts": useInsertsSchedule,
            "export_path": fullPath,
            "schedule_run_time": time,
            "client_timezone": Intl.DateTimeFormat().resolvedOptions().timeZone,
            "days_of_week": daysOfWeek,
            "days_of_month": daysOfMonth,
            "type": props.node.type
          });

      }
    }

  }


  const renderFooter = () => {

    if (pageIndex === 1) {

      return (<div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button label={"Back"}
          onClick={() => {
            dispatch(LayoutActions.setErrorMessage(""));
            dispatch(LayoutActions.setDirExists(null));
            dispatch(LayoutActions.setExportExists(null));
            dispatch(LayoutActions.setDiskSpace(null));
            setPageIndex(0);
          }}
          className="p-button-secondary"
          icon="pi pi-arrow-left"
          iconPos="left"
        />
        <Button
          label={"Submit"}
          icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          id="export-pg-node-confirm"
          iconPos="right"
          disabled={exportDisabled}
          onClick={handleOk} autoFocus />
      </div>)

    }
    else if (pageIndex === 0) {

      return (<div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Next"}
          icon={`pi pi-${isLoading ? 'pi-spin pi-spinner' : 'arrow-right'}`}
          disabled={isLoading || errorMessage}
          iconPos="right"
          onClick={() => {

            let errors = validate();
            if (errors.length === 0) {

              dispatch(LayoutActions.checkExportExists(
                {
                  "path": fullPath,
                  "obj_type": types.POSTGRES,
                  "node_id": props.node.id,
                }

              ));

              dispatch(LayoutActions.checkDirExists(
                {
                  "path": fullPath,
                  "obj_type": types.POSTGRES,
                  "node_id": props.node.id,
                }

              ));

              dispatch(LayoutActions.checkDiskSpace(
                {
                  "path": fullPath,
                  "obj_type": types.POSTGRES,
                  "node_id": props.node.id,
                  "required_space": pg_db_size?.size,
                  "type": props.node.type,
                }
              ));


              setPageIndex(1);

            }

          }} autoFocus />
      </div>)
    }
  }

  const renderCheckPage = () => {

    return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

      <div className="field grid">
        <label className="col-fixed">{`Checking directory :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : dirCheck}
        </div>
      </div>

      <div className="field grid">
        <label className="col-fixed">{`Checking available space :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : diskSpaceCheck}
        </div>
      </div>

      <div className="field grid">
        <label className="col-fixed">{`Checking export file name :`}</label>
        <div className="col">
          {isLoading ? <i className="pi pi-spin pi-spinner" /> : exportCheck}
        </div>
      </div>

      {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
      <ApiBar tooltip={toggle < 1 ? nowApiTooltip : scheduleApiTooltip} onClick={toggle < 1 ? onRestApi1Click : onRestApi2Click} />




    </div>);

  }

  const renderExportPage = () => {

    return (

      <TabView className="no-ink" activeIndex={toggle} onTabChange={(e) => {
        dispatch(LayoutActions.setErrorMessage(""));
        setToggle(e.index);
      }}>
        <TabPanel header="Export now">
          <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

              <div className="field grid">
                <label className="col-fixed" style={{ width: "220px" }}>{"Name:"}</label>
                <div className="col">
                  <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => setName(e.target.value)} />
                  <div className="error-help"><small className="p-error p-d-block">{nameError}</small></div>
                </div>
              </div>

              <div className="field grid">
                <label className="col-fixed" style={{ width: "220px" }}>Compress:</label>
                <div className="col">

                  <Checkbox
                    onChange={e => setZipped(e.checked)}
                    checked={zipped}>
                  </Checkbox>
                </div>
              </div>


              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Including data:</div>
                <div className="col">

                  <Checkbox checked={includeData} onChange={(e) => setIncludeData(e.checked)} />

                </div>
              </div>

              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Including publications:</div>
                <div className="col">

                  <Checkbox checked={includePublications} onChange={(e) => setIncludePublications(e.checked)} />

                </div>
              </div>

              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Including subscriptions:</div>
                <div className="col">

                  <Checkbox checked={includeSubscriptions} onChange={(e) => setIncludeSubscriptions(e.checked)} />

                </div>
              </div>


              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Use inserts:</div>
                <div className="col">

                  <Checkbox checked={useInserts} onChange={(e) => setUseInserts(e.checked)} />

                </div>
              </div>

              
            <div className="field grid">
              <label id="pg-export-size-label" className="col-fixed" style={{ width: "220px" }}>{"Size:"}</label>
              <div className="col">
                {isLoading ? <i className="pi pi-spin pi-spinner" /> : pg_db_size?.display_size}
              </div>
            </div>

              <div className="field grid align-items-end">
                <div className="col-fixed"> Export will be kept for {props.node.size_scheme ? props.node.size_scheme?.export : "unk"} days </div>
                <div className="col-fixed">

                  <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                  <i className="pi pi-info-circle config-field icon-field" />

                </div>
              </div>
              {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
          </div>
        </TabPanel>
        <TabPanel header="Scheduled export">
          <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
              <div className="form-inner-part">

                <div className="grid">
                  <label className="col-fixed multiline" style={{ width: "165px" }}>{`Days of month:\nor`}</label>
                  <div className="col">
                    <Calendar viewDate={minDate.current}  readOnlyInput selectionMode="multiple" dateFormat="d" minDate={minDate.current} maxDate={maxDate.current} showOtherMonths={false} className="p-inputtext-sm" panelClassName="days-of-month-picker" value={daysOfMonth} onChange={(e) =>
                      setDaysOfMonth(e.value)} showTime={false} />
                  </div>
                </div>

                <div className="grid">
                  <label className="col-fixed" style={{ width: "165px" }}>Days of week:</label>
                  <div className="col">
                    <MultiSelect scrollHeight="100%" className="p-inputtext-sm" maxSelectedLabels={6} selectedItemsLabel="All weekdays" selectAllLabel="All weekdays" options={daysOfWeekOptions} value={daysOfWeek} onChange={(e) => setDaysOfWeek(e.value)} />

                  </div>
                </div>

              </div>

              <div className="field grid">
                <label className="col-fixed" style={{ width: "165px" }}>Time:</label>
                <div className="col">
                  <Calendar showSeconds className="p-inputtext-sm" value={time} onChange={(e) => setTime(e.value)} timeOnly />
                  <div className="error-help"><small className="p-error p-d-block">{timeError}</small> </div>
                </div>
              </div>

              <div className="field grid">
                <label className="col-fixed" style={{ width: "165px" }}>{"Name:"}</label>
                <div className="col">
                  <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => setName(e.target.value)} />
                  <div className="error-help"><small className="p-error p-d-block">{nameError}</small></div>
                </div>
              </div>

              <div className="field grid">
                <label style={{ width: "220px" }} className="col-fixed">Compress:</label>
                <div className="col">

                  <Checkbox
                    onChange={e => {
                      const checked = e.checked
                      setScheduleZipped(checked);
                    }}
                    checked={scheduleZipped}>
                  </Checkbox>
                </div>
              </div>

              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Including data:</div>
                <div className="col">

                  <Checkbox checked={includeDataSchedule} onChange={(e) => setIncludeDataSchedule(e.checked)} />

                </div>
              </div>

              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Including publications:</div>
                <div className="col">

                  <Checkbox checked={includePublicationsSchedule} onChange={(e) => setIncludePublicationsSchedule(e.checked)} />

                </div>
              </div>

              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Including subscriptions:</div>
                <div className="col">

                  <Checkbox checked={includeSubscriptionsSchedule} onChange={(e) => setIncludeSubscriptionsSchedule(e.checked)} />

                </div>
              </div>


              <div className="field grid align-items-center">
                <div className="col-fixed" style={{ width: "220px" }}> Use inserts:</div>
                <div className="col">

                  <Checkbox checked={useInsertsSchedule} onChange={(e) => setUseInsertsSchedule(e.checked)} />

                </div>
              </div>


              <div className="field grid">
                <label id="pg-export-size-label" className="col-fixed" style={{ width: "220px" }}>{"Size:"}</label>
                <div className="col">
                  {isLoading ? <i className="pi pi-spin pi-spinner" /> : pg_db_size?.display_size}
                </div>
              </div>

              <div className="field grid align-items-end">
                <div className="col-fixed"> Export will be kept for {props.node.size_scheme ? props.node.size_scheme?.export : "unk"} days </div>
                <div className="col-fixed">

                  <Tooltip target=".config-field" position='right' content={config.tooltips[8]} />
                  <i className="pi pi-info-circle config-field icon-field" />

                </div>
              </div>
              {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
          </div>
        </TabPanel>
      </TabView>)

  }

  return (
    <Dialog
      className='export-pg-node'
      id="export-pg-node"
      header={`Export ${props.node.name}`}
      visible={props.visible}
      style={{ width: '40vw', maxHeight: "90%" }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">

        {pageIndex === 0 && renderExportPage()}
        {pageIndex === 1 && renderCheckPage()}
        <ConfirmPopup
          icon="pi pi-exclamation-triangle"
          className="multiline"
          target={document.getElementById('export-pg-node-confirm')}
          visible={showWarningPopup}
          onHide={() => setShowWarningPopup(false)}
          reject={props.onHide}
          accept={submit}
          message={"The selected export size exceeds 1GB.\nThis can take a long time.Continue?"} />
      </div>

    </Dialog>)

};

export default PostgresExportDialog;
