import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { InputText } from "primereact/inputtext";
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import { useSelector } from "react-redux";
import moment from "moment";
import { Tag } from "primereact/tag";
import ErrorTextBox from "../../../components/errorTextBox";
import config, { types } from "../../../config";
import validator from "../../../util/validator";
import constraints from "./constraints";
import { Checkbox } from "primereact/checkbox";


const RestorePgDatabaseLayoutDialog = (props) => {

  const [restoreConfirm, setRestoreConfirm] = useState("");
  const [restoreConfirmError, setRestoreConfirmError] = useState("");
  const [includeData, setIncludeData] = useState(true);
  const [includePublications, setIncludePublications] = useState(true);
  const [includeSubscriptions, setIncludeSubscriptions] = useState(true);

  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);



  const onRestApiClick = async () => {


    let errors = validate();

    if (errors.length === 0) {

      setApiTooltip(await copyToClipboard(createAPIcommand({
        "bu_id": props.backup.name,
        "include_data": includeData,
        "include_publications": includePublications,
        "include_subscriptions": includeSubscriptions,
        "type": props.backup.ex_params?.belong_to,
        "cluster_id": props.cluster_id,
        "service_id": props.service_id
      }, accessToken, props.backup.ex_params?.belong_to === types.POSTGRES_K8S ? 'restore_k8s_pg_db' :  "restore_pg_database/")));

    }
    else setApiTooltip(" ");

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("restoreConfirm", restoreConfirm);
    (error == null || errors.push(error)) && setRestoreConfirmError(error);

    return errors;

  }

  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {

      props.onOkClick(

        {
          "bu_id": props.backup.id,
          "include_data": includeData,
          "include_publications": includePublications,
          "include_subscriptions": includeSubscriptions,
          "type": props.backup.ex_params?.belong_to,
          "cluster_id": props.cluster_id,
          "service_id": props.service_id
        }

      );

    }


  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='restore-pg-database-backup'
      header={`Restore DB backup`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="field">
                This will restore the database {props.backup.node_name} to the backup, taken at {moment(props.backup.created_date).format("MMM DD,YYYY, hh:mm A")}
              </div>

             {props.backup.ex_params.plain ? null : <div>
               <div className="field grid align-items-center">
                <div className="col-fixed" style={{width: "220px"}}> Including data:</div>
                <div className="col">

                  <Checkbox checked={includeData} onChange={(e) => setIncludeData(e.checked)} />

                </div>
              </div>

              <div className="field grid align-items-center">
                <div className="col-fixed" style={{width: "220px"}}> Including publications:</div>
                <div className="col">

                  <Checkbox checked={includePublications} onChange={(e) => setIncludePublications(e.checked)} />

                </div>
              </div>

              <div className="field grid align-items-center">
                <div className="col-fixed" style={{width: "220px"}}> Including subscriptions:</div>
                <div className="col">

                  <Checkbox checked={includeSubscriptions} onChange={(e) => setIncludeSubscriptions(e.checked)} />

                </div>
              </div>
             </div> }

             

              <div className="field">
                <Tag value="The current content of the database will be overridden, this operation is not reversible!" icon="pi pi-exclamation-triangle" severity="warning" />
              </div>

              <div className="field grid flex align-items-center">
                <div className="col">
                <InputText
                  className="p-inputtext-sm"
                  placeholder={`Please type "${config.restoreConfirm}" here in order to continue`}
                  value={restoreConfirm}
                  onChange={(e) => setRestoreConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{restoreConfirmError}</small>}</div>
                </div>
              </div>
              
            </div>

          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />

        </div>
      </div>

    </Dialog>)

};

export default RestorePgDatabaseLayoutDialog;