import { useState, useEffect, useRef, useCallback } from 'react';
import { useSelector } from 'react-redux';
import { useDispatch } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { Password } from "primereact/password";
import { Checkbox } from 'primereact/checkbox';
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import { Tooltip } from 'primereact/tooltip';
import { Slider } from 'primereact/slider';
import validator from "../../../util/validator";
import constraints from "./constraints";
import { LayoutActions } from '../../../store/layout/reducer';
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import ErrorTextBox from '../../../components/errorTextBox';
import ApiBar from '../../../components/apiBar';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { types, clusterSizeOptions, pgVersionOptions } from '../../../config';
import { base64Encode } from '../../../util/base64Encode';
import getValidationError from '../../../util/errorPrompter';
import { footer } from '../../../util/passwordHelpers';


const CreatePgK8SClusterDialog = ({ visible, onHide, onBack, onOkClick, service }) => {


    const onPopupSubmit = (accept) => onSaveClick(accept)
    const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);
    const caller = useRef('submit-button');

    const [pageIndex, setPageIndex] = useState(0);
    const dispatch = useDispatch();

    // First Page elements
    const [osUsername, setOsUsername] = useState(service?.username);
    const [osPassword, setOsPassword] = useState(service?.password);
    const [databaseUsername, setDatabaseUsername] = useState("clusteradmin");
    const [databasePassword, setDatabasePassword] = useState("");
    const [hostname, setHostname] = useState(service?.host_name);
    const [sshPort, setSshPort] = useState(service?.port);
    const [privateKey, setPrivateKey] = useState("");
    const [name, setName] = useState("");
    const [namespace, setNamespace] = useState("");
    const [softwareVersion, setSoftwareVersion] = useState(17);

    // First Page element errors

    const [nameError, setNameError] = useState("");
    const [namespaceError, setNamespaceError] = useState("");

    // Second Page elements
    const [automaticDelete, setAutomaticDelete] = useState(false);
    const [automaticDeleteDays, setAutomaticDeleteDays] = useState(1);
    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);
    const [ssl, setSSL] = useState(false);
    const [certificateAuth, setCertificateAuth] = useState(false);



    const submitButtonRef = useRef(null);

    // Second Page element errors
    const [databaseUsernameError, setDatabaseUsernameError] = useState("");
    const [databasePasswordError, setDatabasePasswordError] = useState("");

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { isLoading, errorMessage } = useSelector((state) => state.layout);
    const [submitting, setSubmitting] = useState(false);

    useEffect(() => {


        if (visible === false) {

            setPageIndex(0);

            // First page elements and errors
            setOsUsername(service?.username);
            setOsPassword(service?.password);
            setDatabaseUsername("clusteradmin");
            setDatabaseUsernameError("");
            setDatabasePassword("");
            setDatabasePasswordError("");
            setHostname(service?.host_name);
            // setHostnameError("");
            setSshPort(service?.port);
            setPrivateKey(service?.ex_params?.api_token);
            setSoftwareVersion(17);


            // Second page elements and errors
            setApiTooltip(" ");
            setClusterSize(clusterSizeOptions[0].value);
            setAutomaticDelete(false);
            setAutomaticDeleteDays(1);
            setSSL(false);
        }


    }, [visible, service]);


    const onSaveClick = useCallback(async (dropInventories) => {

        const cluster =
        {

            name: `${name}`,
            ex_params: {
                namespace: namespace
            }

        }
        const ex_params =
        {
            ssl: ssl,
            certificate_authentication: certificateAuth,
            service_id: service?.id
        }

        const data = {
            cluster,
            ex_params,
            auto_delete: automaticDelete,
            auto_delete_days: automaticDeleteDays,
            drop_inventories: dropInventories,
            version: softwareVersion,
            service_id: service?.id,
            sizing: clusterSize,
            database_user: databaseUsername,
            database_pwd: base64Encode(databasePassword),
        }

        if (caller.current === 'submit-button') {


            if (onOkClick)
                onOkClick(data);

        }
        else setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "create_pg_primary/")));

        setSubmitting(false);
    }, [accessToken, automaticDelete, automaticDeleteDays, certificateAuth, clusterSize, service, softwareVersion,
        databasePassword, databaseUsername, onOkClick, name, namespace, ssl]);

    useEffect(() => {

        const customEvent = new Event('beforeinput', { bubbles: true });


        const validateSecond = () => {

            let errors = [];


            let error = validator(constraints)("databaseUsername", databaseUsername);
            (error == null || errors.push(error)) && setDatabaseUsernameError(error);

            error = validator(constraints)("databasePassword", databasePassword);
            (error == null || errors.push(error)) && setDatabasePasswordError(error);


            return errors;


        }



        if (submitting) {
            const errors = validateSecond();
            if (errors.length === 0) {

                dispatch(LayoutActions.checkInventoryExists({ 'name': `${hostname}`, 'type': types.POSTGRES_K8S }, { 'exists': setShowDeleteConfirm, 'failback': onSaveClick }))

            }
            else caller.current === "submit-button" ? submitButtonRef.current.dispatchEvent(customEvent) : setApiTooltip(getValidationError());


        }

    }, [submitting, dispatch, hostname, onSaveClick,
        databaseUsername, databasePassword, namespace]);

    const validateFirst = () => {

        let errors = [];

        let error = validator(constraints)("name", name);
        (error == null || errors.push(error)) && setNameError(error);

        error = validator(constraints)("namespace", namespace);
        (error == null || errors.push(error)) && setNamespaceError(error);

        error = validator(constraints)("osUsername", osUsername);


        error = validator(constraints)("privateKey", privateKey);

        error = validator(constraints)("osPassword", osPassword);
        error = validator(constraints)("hostname", hostname);

        error = validator(constraints)("sshPort", sshPort);

        return errors;

    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) setPageIndex(1);
    }

    const onSecondBackClick = () => {
        setPageIndex(0);
    }


    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Back"} icon="pi pi-arrow-left" onClick={() => {
                        onHide();
                        onBack();
                    }} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            className='submit-button'
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            iconPos="right"
                            id='submit-button'
                            ref={submitButtonRef}
                            onClick={() => {

                                caller.current = 'submit-button';
                                setSubmitting(true);

                            }}
                            autoFocus />
                        <Tooltip target='.submit-button' showEvent='beforeinput' position='top' >
                            {getValidationError()}
                        </Tooltip>
                    </div>
                </div>
            );
        }
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Cluster Name:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Cluster Name" value={name} onChange={(e) => setName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{nameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Namespace:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Namespace" value={namespace} onChange={(e) => setNamespace(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{namespaceError}</small>
                    </div>
                </div>
                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
            </div>
        );
    }

    const renderPageSecond = () => {

        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Username:"}</label>
                            <div className="col">
                                <InputText className="p-inputtext-sm" placeholder="Database Username" value={databaseUsername} onChange={(e) => setDatabaseUsername(e.target.value)} />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databaseUsernameError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Database Password:"}</label>
                            <div className="col">
                                <Password
                                    feedback
                                    className="p-inputtext-sm" placeholder="Database Password"
                                    value={databasePassword}
                                    onChange={(e) => setDatabasePassword(e.target.value)}
                                    footer={() => footer(databasePassword)}
                                    autoComplete="new-password"
                                    toggleMask
                                />
                            </div>
                        </div>
                        <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                            <small id="name-help" className="p-error p-d-block">{databasePasswordError}</small>
                        </div>
                    </div>
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Software version:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm filter-field"
                                    options={pgVersionOptions.filter((o) => Number(o.value) > 12 )}
                                    value={softwareVersion}
                                    onChange={(e) => {

                                        setSoftwareVersion(e.value);
                                    }}
                                />
                            </div>

                        </div>

                    </div>
                    <div>
                        <div>
                            <div>
                                <div className="field grid">
                                    <label className="col-fixed" style={{ width: "220px" }}>{"Use SSL:"}</label>
                                    <div className="col">
                                        <Checkbox onChange={(e) => {
                                            const value = e.checked;
                                            if (!value) setCertificateAuth(false);
                                            setSSL(value);

                                        }} checked={ssl} value={ssl} />
                                    </div>
                                </div>
                            </div>

                            <div>
                                <div className="field grid">
                                    <label className="col-fixed" style={{ width: "220px" }}>{"Authenticate by certificate:"}</label>
                                    <div className="col">
                                        <Checkbox onChange={() => setCertificateAuth(!certificateAuth)} checked={certificateAuth} disabled={!ssl} value={certificateAuth} />
                                    </div>
                                </div>
                            </div>

                            <div>
                                <div className="field grid flex align-items-center">
                                    <label htmlFor="cb1" className="p-checkbox-label col-fixed" style={{ width: "220px" }}>{!automaticDelete ? "Automatic delete: disabled" : `Automatic delete in: ${automaticDeleteDays} days`}</label>
                                    <div className="col-fixed">
                                        <Checkbox inputId="cb1" value="Automatic delete disabled:" onChange={() => setAutomaticDelete(!automaticDelete)} checked={automaticDelete}></Checkbox>
                                    </div>
                                    <div className="col">
                                        <Slider
                                            disabled={!automaticDelete}
                                            min={1}
                                            max={365}
                                            value={automaticDeleteDays}
                                            onChange={(e) => setAutomaticDeleteDays(e.value)} />
                                    </div>

                                </div>
                            </div>


                            <div>
                                <div className='field grid'>

                                    <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                                    <div className='col'>

                                        <Dropdown className="p-inputtext-sm"
                                            value={clusterSize}
                                            onChange={(e) => setClusterSize(e.value)}
                                            options={clusterSizeOptions}
                                            placeholder="Size options" />

                                    </div>

                                </div>

                            </div>

                        </div>
                    </div>

                </div>

                {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
                <ApiBar id='rest-api-bar' tooltip={apiTooltip} onClick={() => {
                    caller.current = 'rest-api-bar';
                    setSubmitting(true);
                }} />
            </div>



        );
    }


    return (
        <Dialog
            className='add-pg-k8s-cluster'
            header={"Install a new PostgreSQL K8S Cluster"}
            visible={visible}
            style={{ width: '50vw' }}
            onHide={onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
                <ConfirmPopup onHide={() => setShowDeleteConfirm(false)} visible={showDeleteConfirm} target={window.document.getElementById(caller.current)} acceptClassName="p-button-danger" rejectIcon="pi pi-times" message="Delete backups/exports of this node?" icon="pi pi-question-circle" acceptIcon="pi pi-trash" acceptLabel="Delete" rejectLabel="Keep" reject={() => onPopupSubmit(false)} accept={() => onPopupSubmit(true)} />
            </div>

        </Dialog>
    );
}


export default CreatePgK8SClusterDialog;