import { useState, useEffect } from 'react';
import { useSelector } from 'react-redux';
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { Password } from "primereact/password";
import { Dialog } from "primereact/dialog";
import { Dropdown } from 'primereact/dropdown';
import validator from "../../../util/validator";
import constraints from "./constraints";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { base64Encode } from '../../../util/base64Encode';
import ApiBar from '../../../components/apiBar';
import { repositoryOptions, clusterSizeOptions } from '../../../config';
import { footer } from '../../../util/passwordHelpers';

const AddRedisClusterDialog = (props) => {

    const [pageIndex, setPageIndex] = useState(0);

    // First Page elements
    const [databaseUsername, setDatabaseUsername] = useState("clusteradmin");
    const [databasePassword, setDatabasePassword] = useState("");
    const [repository, setRepository] = useState(repositoryOptions[0].value);

    const softwareVersionOptions =
        [
            { label: "7.2", value: "7.2" },
            { label: "7.0", value: "7.0" },
            { label: "6.2", value: "6.2" },
        ];

    const [softwareVersion, setSoftwareVersion] = useState("7.2");
    const [clusterSize, setClusterSize] = useState(clusterSizeOptions[0].value);

    // First Page element errors
    const [databaseUsernameError, setDatabaseUsernameError] = useState("");
    const [databasePasswordError, setDatabasePasswordError] = useState("");

    const [apiTooltip, setApiTooltip] = useState(" ");
    const { accessToken } = useSelector((state) => state.app);
    const { isLoading } = useSelector((state) => state.layout);
    const [clusterName, setClusterName] = useState("");
    const [clusterNameError, setClusterNameError] = useState("");



    useEffect(() => {
        if (props.visible === false) {

            setPageIndex(0);

            // First page elements and errors
            setDatabaseUsername("clusteradmin");
            setDatabaseUsernameError("");
            setDatabasePassword("");
            setDatabasePasswordError("");
            setSoftwareVersion("7.2");
            setRepository(repositoryOptions[0].value);
            setClusterName("");
            setClusterNameError("");
            // Second page elements and errors
            setClusterSize(clusterSizeOptions[0].value);
            setApiTooltip(" ");
        }
    }, [props.visible]);


    const onRestApiClick = async () => {

        const ex_params =
        {

            repository: repository

        }

        if (typeof props.visible === 'string') {
            const proxmox_params = props.visible.split('@');
            const osInfo = proxmox_params[0].split(' ');
            ex_params['distributor'] = osInfo[0]
            ex_params['release'] = osInfo[1]

        }


        const data = {
            ex_params,
            database_user: databaseUsername,
            database_pwd: base64Encode(databasePassword),
            version: softwareVersion,
            sizing: clusterSize
        }

        setApiTooltip(await copyToClipboard(createAPIcommand(data, accessToken, "add_redis_cluster/")));
    }

    const validateFirst = () => {

        let errors = [];

        let error = validator(constraints)("databaseUsername", databaseUsername);
        (error == null || errors.push(error)) && setDatabaseUsernameError(error);

        error = validator(constraints)("databasePassword", databasePassword);
        (error == null || errors.push(error)) && setDatabasePasswordError(error);
        error = validator(constraints)("clusterName", clusterName);
        (error == null || errors.push(error)) && setClusterNameError(error);


        return errors;

    }

    const onFirstNextClick = () => {

        let errors = validateFirst();

        if (errors.length === 0) setPageIndex(1);
    }

    const onSecondBackClick = () => setPageIndex(0);



    const onSaveClick = () => {

        if (props.onOkClick) {

            const ex_params =
            {

                repository: repository

            }

            if (typeof props.visible === 'string') {
                const proxmox_params = props.visible.split('@');
                const osInfo = proxmox_params[0].split(' ');
                ex_params['distributor'] = osInfo[0]
                ex_params['release'] = osInfo[1]
            }

            const data = {
                name: clusterName,
                ex_params,
                database_user: databaseUsername,
                database_pwd: base64Encode(databasePassword),
                version: softwareVersion,
                sizing: clusterSize
            }
            props.onOkClick(data);

        }
    }

    const renderFooter = () => {
        if (pageIndex === 0) {
            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <Button
                        label={"Next"}
                        icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'arrow-right'}`}
                        iconPos="right"
                        onClick={onFirstNextClick} autoFocus />
                </div>
            );
        }
        else if (pageIndex === 1) {

            return (
                <div style={{ display: "flex", justifyContent: "space-between" }}>
                    <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                    <div>
                        <Button label={"Back"}
                            onClick={onSecondBackClick}
                            className="p-button-secondary"
                            icon="pi pi-arrow-left"
                            iconPos="left"
                        />
                        <Button
                            label={"Submit"}
                            icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
                            iconPos="right"
                            onClick={onSaveClick}
                            autoFocus />
                    </div>
                </div>
            );
        }
    }

    const renderPageFirst = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                 <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Cluster Name:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Cluster Name" value={clusterName} onChange={(e) => setClusterName(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{clusterNameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Database Username:"}</label>
                        <div className="col">
                            <InputText className="p-inputtext-sm" placeholder="Database Username" value={databaseUsername} onChange={(e) => setDatabaseUsername(e.target.value)} />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{databaseUsernameError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Database Password:"}</label>
                        <div className="col">
                            <Password
                                feedback
                                className="p-inputtext-sm" placeholder="Database Password"
                                value={databasePassword}
                                onChange={(e) => setDatabasePassword(e.target.value)}
                                autoComplete="new-password"
                                footer={() => footer(databasePassword)}
                                toggleMask
                            />
                        </div>
                    </div>
                    <div className="error-help" style={{ display: "flex", justifyContent: "flex-end", marginRight: "5px", marginTop: "-15px", marginBottom: "5px" }}>
                        <small id="name-help" className="p-error p-d-block">{databasePasswordError}</small>
                    </div>
                </div>
                <div>
                    <div className="field grid">
                        <label className="col-fixed" style={{ width: "300px" }}>{"Software version:"}</label>
                        <div className="col">
                            <Dropdown
                                className="p-inputtext-sm filter-field"
                                options={softwareVersionOptions}
                                value={softwareVersion}
                                onChange={(e) => {

                                    setSoftwareVersion(e.value);

                                }}
                            />
                        </div>

                    </div>

                </div>
            </div>
        );
    }

    const renderPageSecond = () => {
        return (
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                <div className="form-inner-part">
                    <div>
                        <div className="field grid">
                            <label className="col-fixed" style={{ width: "220px" }}>{"Install from:"}</label>
                            <div className="col">
                                <Dropdown
                                    className="p-inputtext-sm"
                                    options={repositoryOptions}
                                    value={repository}
                                    onChange={(e) => setRepository(e.value)}
                                />
                            </div>
                        </div>
                    </div>

                    <div>
                        <div className='field grid'>

                            <div className='col-fixed' style={{ width: "220px" }}>{"Cluster size:"}</div>
                            <div className='col'>

                                <Dropdown className="p-inputtext-sm"
                                    value={clusterSize}
                                    onChange={(e) => setClusterSize(e.value)}
                                    options={clusterSizeOptions}
                                    placeholder="Size options" />

                            </div>

                        </div>

                    </div>



                </div>

                <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
            </div>



        );
    }


    return (
        <Dialog
            className='register-redis-cluster'
            header="Register a new Redis Cluster"
            visible={props.visible}
            style={{ width: '50vw' }}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                {pageIndex === 0 && renderPageFirst()}
                {pageIndex === 1 && renderPageSecond()}
            </div>

        </Dialog>
    );
}


export default AddRedisClusterDialog;