import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { useState } from "react";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";


const RemoveRedisClusterDialog = (props) => {

    const [apiTooltip, setApiTooltip] = useState(false);
    const { accessToken } = useSelector((state) => state.app);
    const {errorMessage} = useSelector((state) => state.layout)

    const onRestApiClick = async () => {

        setApiTooltip(await copyToClipboard(createAPIcommand(
            {

                "cluster_id": props.cluster.name

            }, accessToken, "remove_redis_cluster/")));

    }

    const handleOk = () => {

        props.onOkClick(

            {

                "cluster_id": props.cluster.id

            }
        );

    }



    const renderFooter = () => {
        return (
            <div style={{ display: "flex", justifyContent: "space-between" }}>
                <Button label={"No"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
                <Button
                    icon="pi pi-trash"
                    className="p-button-danger"
                    label={"Yes"}
                    onClick={handleOk} autoFocus />
            </div>
        );

    }

    return (
        <Dialog
            className='unregister-redis-cluster'
            header={`Unregister Redis Cluster `}
            visible={props.visible}
            style={{ width: '40vw'}}
            onHide={props.onHide}
            footer={renderFooter()}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
                    <div className="field grid">
                        <div className="col">
                            <p>
                                Redis cluster {props.cluster.name} will no longer be managed by 123Cluster.
                            </p>
                            <p>Are you sure?</p>


                        </div>

                    </div>
                    {errorMessage && <ErrorTextBox error = {errorMessage?.message ? errorMessage.message : errorMessage}/>}
                    <ApiBar tooltip = {apiTooltip} onClick = {onRestApiClick}/>
                </div>
            </div>

        </Dialog>)

};

export default RemoveRedisClusterDialog;