import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useEffect, useState } from "react";
import { InputText } from "primereact/inputtext";
import { Slider } from "primereact/slider";
import { useSelector, useDispatch } from "react-redux";
import { LayoutActions } from "../../../store/layout/reducer";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import createAPIcommand from "../../../util/createApiCommand";
import copyToClipboard from "../../../util/copyToClipboard";
import ErrorTextBox from "../../../components/errorTextBox";
import { ConfirmPopup } from "primereact/confirmpopup";
import { Dropdown } from "primereact/dropdown";
export const SentinelMonitorMasterDialog = (props) => {

  const [name, setName] = useState("");
  const { valid_name, isLoading, errorMessage } = useSelector((state) => state.layout);
  const dispatch = useDispatch();
  const [nameError, setNameError] = useState(null);

  const [apiTooltip, setApiTooltip] = useState(" ");
  const [quorum, setQuorum] = useState(1);
  const [syncs, setSyncs] = useState(1);
  const [downMs, setDownMs] = useState(10000);
  const [failoverMs, setFailoverMs] = useState(10000);
  const [removeMonitorConfirm, setRemoveMonitorConfirm] = useState(false);
  const [targetSentinel, setTargetSentinel] = useState(null);
  const [targetMaster, setTargetMaster] = useState(null);
  const [targetMasterError, setTargetMasterError] = useState(null);
  const [masterOptions, setMasterOptions] = useState([]);
  const [sentinelOptions, setSentinelOptions] = useState([]);
  const [targetSentinelError, setTargetSentinelError] = useState("");

  const { accessToken } = useSelector((state) => state.app);

  const validate = () => {

    const errors = [];

    if (props.update) {

      let error = validator(constraints)("targetSentinel", targetSentinel);
      (error == null || errors.push(error)) && setTargetSentinelError(error);

    }

    else if (props.details) {

      let error = validator(constraints)("targetMaster", targetMaster);
      (error == null || errors.push(error)) && setTargetMasterError(error);

    }

    return errors;

  }


  useEffect(() => {
    if (props.sentinels?.length > 0) {

      const temp = [];
      props.sentinels.forEach((s) => temp.push({ label: s?.name, value: s}))
      setSentinelOptions(temp)

    }

  }, [props.sentinels]);

  useEffect(() => {
    if (props.masters?.length > 0) {

      const temp = [];
      props.masters.forEach((m) => temp.push({ label: m?.name, value: m }))
      setMasterOptions(temp)

    }

  }, [props.masters]);

  const validateName = () => {
    setNameError("");

    let errors = [];
    let error = validator(constraints)("name", name);
    (error == null || errors.push(error)) && setNameError(error);
    if (errors.length === 0) dispatch(LayoutActions.checkSentinelMasterName({ node_id: props.update ? targetSentinel : props.sentinels?.[0]?.id, name: name }));

  }

  useEffect(() => {

    if (valid_name) {

      if (valid_name?.error)
        setNameError(valid_name.error);
      else setNameError(valid_name?.response)
    }

  }, [valid_name]);


  const handleOk = () => {

    const errors = props.update || props.details ? validate() : [];

    if (!nameError && !(props.update || targetMaster?.ex_params?.monitored_by?.includes(props?.sentinels?.[0]?.id?.toString()))) setNameError("name not checked");

    if (errors.length === 0 && (nameError === "name available" || props.update || targetMaster?.ex_params?.monitored_by?.includes(props?.sentinels?.[0]?.id?.toString()))) {

      props.onOkClick(
        {
          "master_alias": name,
          "node_id": props.details ? targetMaster?.id : props.masters?.[0]?.id,
          "sentinel_id": props.details || !props.update ? props.sentinels?.[0]?.id : targetSentinel?.id,
          "sentinel_quorum": quorum,
          "parallel_syncs": syncs,
          "down_ms": downMs,
          "failover_ms": failoverMs,
          "update": props.update
        });

    }


  }

  const onRestApiClick = async () => {

    const errors = props.update || props.details ? validate() : [];

    if (!nameError && !(props.update || targetMaster?.ex_params?.monitored_by?.includes(props?.sentinels?.[0]?.id?.toString()))) setNameError("name not checked");

    if (errors.length === 0 && (nameError === "name available" || props.update || targetMaster?.ex_params?.monitored_by?.includes(props?.sentinels?.[0]?.id?.toString()))) {
      setApiTooltip(await copyToClipboard(createAPIcommand(
        {
          "master_alias": name,
          "node_id": props.details ? targetMaster?.name : props.masters?.[0]?.name,
          "sentinel_id": props.details || !props.update ? props.sentinels?.[0]?.name : targetSentinel?.name,
          "sentinel_quorum": quorum,
          "parallel_syncs": syncs,
          "down_ms": downMs,
          "failover_ms": failoverMs,
          "update": props.update
        }
        , accessToken, "sentinel_monitor_master/")));

    }


  }

  const onRemoveMonitor = () => {

    dispatch(LayoutActions.sentinelRemoveMonitor({ "node_id": props.details ? targetMaster?.id : props.masters?.[0]?.id, "sentinel_id": props?.details ? props.sentinels?.[0]?.id : targetSentinel }, props.onHide));

  }

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        {(props.update || targetMaster?.ex_params?.monitored_by?.includes(props?.sentinels?.[0]?.id?.toString())) &&
          <Button
            label={"Remove Monitor"}
            icon="pi pi-eye-slash"
            id="remove-monitor-button"
            disabled={props.details ? !targetMaster : !targetSentinel}
            className="p-button-danger"
            onClick={() => setRemoveMonitorConfirm(true)}
          />}
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (

    <Dialog
      className='add-sentinel-monitor'
      header={props.update ? `Edit sentinel monitors of ${props.masters?.[0]?.name} ` : `Monitor ${props.details ? "masters" : props.masters?.[0]?.name} by ${props.sentinels?.[0]?.name}`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">


          {props.update ? <div className="field">
            <Dropdown className="p-inputtext-sm"
              value={targetSentinel}
              options={sentinelOptions}
              onChange={(e) => setTargetSentinel(e.value)}
              placeholder="Select target sentinel" />
            <div className="error-help">{<small className="p-error p-d-block">{targetSentinelError}</small>}</div>
          </div> : null}


          {props.details ? <div className="field">
            <Dropdown className="p-inputtext-sm"
              value={targetMaster}
              options={masterOptions}
              onChange={(e) => setTargetMaster(e.value)}
              placeholder="Select target master" />
            <div className="error-help">{<small className="p-error p-d-block">{targetMasterError}</small>}</div>
          </div> : null}

          {props.update ? null : (props.details && (!targetMaster || targetMaster?.ex_params?.monitored_by?.includes(props?.sentinels?.[0]?.id?.toString()))) ? null : <div className="field grid">
            <label style={{ marginTop: "7px" }} className="col-fixed align-self-start">{"Name:"}</label>
            <div className="col">
              <span className="p-input-icon-right flex">
                <i className={isLoading ? "pi pi-spin pi-spinner align-self-center" : ""} />
                <InputText className="p-inputtext-sm" placeholder="Name" value={name} onChange={(e) => {
                  setName(e.target.value);
                  setNameError("name not checked");
                  dispatch(LayoutActions.setValidName(undefined));
                }} />

              </span>
              <div className="error-help"><small className={nameError === "name available" ? "p-d-block" : "p-error p-d-block"}>{nameError}</small></div>
            </div>
            <div className="col-fixed">
              <Button className="p-button-sm" style={{ width: "80px" }} label="Check" onClick={validateName} />
            </div>
          </div>}

          <div className='field grid flex align-items-center'>
            <label className="col-fixed align-self-center"> {`Quorum: ${quorum}`} </label>
            <div className="col">
              <Slider
                min={1}
                max={props.max_quorum > 1 ? props.max_quorum : 1}
                value={quorum}
                onChange={(e) => setQuorum(e.value)} />
            </div>
          </div>

          <div className='field grid flex align-items-center'>
            <label className="col-fixed align-self-center">{`Parallel syncs: ${syncs}`} </label>
            <div className="col">
              <Slider
                min={1}
                max={props.max_syncs > 1 ? props.max_syncs : targetMaster ? targetMaster?.slaves_count > 1 ? targetMaster.slaves_count : 1 : 1}
                value={syncs}
                onChange={(e) => setSyncs(e.value)} />
            </div>

          </div>
          <div className='field grid flex align-items-center'>
            <label className="col-fixed align-self-center">{`Down after(ms): ${downMs}`} </label>
            <div className="col">
              <Slider
                min={10000}
                max={300000}
                value={downMs}
                onChange={(e) => setDownMs(e.value)} />
            </div>
          </div>

          <div className='field grid flex align-items-center'>
            <label className="col-fixed align-self-center">{`Failover timeout(ms): ${failoverMs}`} </label>
            <div className="col">
              <Slider
                min={10000}
                max={300000}
                value={failoverMs}
                onChange={(e) => setFailoverMs(e.value)} />
            </div>
          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />

          <ConfirmPopup
            target={window.document.getElementById(`remove-monitor-button`)}
            visible={removeMonitorConfirm}
            onHide={() => setRemoveMonitorConfirm(false)}
            message="Are you sure?"
            accept={onRemoveMonitor}
          />
        </div>
      </div>
    </Dialog>)

};

export default SentinelMonitorMasterDialog;