import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { useState, useEffect } from "react";
import { InputText } from "primereact/inputtext";
import validator from "../../../util/validator";
import constraints from "./constraints";
import ApiBar from "../../../components/apiBar";
import copyToClipboard from "../../../util/copyToClipboard";
import createAPIcommand from "../../../util/createApiCommand";
import { useSelector } from "react-redux";
import ErrorTextBox from "../../../components/errorTextBox";

export const StepdownRedisNodeDialog = (props) => {

  const [targetMaster, setTargetMaster] = useState(null);
  const [masterOptions, setMasterOptions] = useState([]);
  const [stepdownConfirm, setStepdownConfirm] = useState("");
  const [stepdownConfirmError, setStepdownConfirmError] = useState("");
  const [targetMasterError, setTargetMasterError] = useState("");
  const [apiTooltip, setApiTooltip] = useState(" ");
  const { accessToken } = useSelector((state) => state.app);
  const { errorMessage, isLoading } = useSelector((state) => state.layout);

  const handleOk = () => {

    let errors = validate();

    if (errors.length === 0) {

      props.onOkClick(
        {
          "node_id": targetMaster?.id
        });

    }

  }

  const validate = () => {

    let errors = [];

    let error = validator(constraints)("targetMaster", targetMaster);
    (error == null || errors.push(error)) && setTargetMasterError(error);

    error = validator(constraints)("stepdownConfirm", stepdownConfirm);
    (error == null || errors.push(error)) && setStepdownConfirmError(error);

    return errors;

  }

  const onRestApiClick = async () => {

    let errors = validate();

    if (errors.length === 0) {

      setApiTooltip(await copyToClipboard(createAPIcommand({
        "node_id": targetMaster?.name

      }, accessToken, "stepdown_redis_node/")));
    }
    else setApiTooltip(" ");

  }

  useEffect(() => {

    if (props.masters) {

      const temp = [];
      props.masters.forEach((master) => temp.push({ label: master.name, value: master}));
      setMasterOptions(temp)

    }

  }, [props.masters]);

  const renderFooter = () => {
    return (
      <div style={{ display: "flex", justifyContent: "space-between" }}>
        <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
        <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
      </div>
    );

  }



  return (
    <Dialog
      className='stepdown-redis-node'
      header={`Stepdown`}
      visible={props.visible}
      style={{ width: '40vw' }}
      onHide={props.onHide}
      footer={renderFooter()}>
      <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
        <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
          <div className="field grid">
            <div className="col">
              <div className="title">{"Choose master to stepdown:"}</div>

              <div className="field">
                <Dropdown className="p-inputtext-sm"
                  value={targetMaster}
                  options={masterOptions}
                  onChange={(e) => setTargetMaster(e.value)}
                  placeholder="Select master" />
                <div className="error-help">{<small className="p-error p-d-block">{targetMasterError}</small>}</div>
              </div>
              <div className="field grid flex align-items-center">
                <div className="col">

                  <InputText
                    className="p-inputtext-sm"
                    placeholder='Please type "stepdown me" here in order to continue'
                    value={stepdownConfirm}
                    onChange={(e) => setStepdownConfirm(e.target.value)} />
                  <div className="error-help">{<small className="p-error p-d-block">{stepdownConfirmError}</small>}</div>

                </div>
              </div>
            </div>

          </div>
          {errorMessage && <ErrorTextBox error={errorMessage?.message ? errorMessage.message : errorMessage} />}
          <ApiBar tooltip={apiTooltip} onClick={onRestApiClick} />
        </div>
      </div>

    </Dialog>)

};

export default StepdownRedisNodeDialog;