import React, { useState } from "react";
import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { InputText } from "primereact/inputtext";
import { useSelector } from "react-redux";

const RequestDemoDialog = ({ visible, onHide, onOkClick }) => {
    const [email, setEmail] = useState("");
    const {isLoading} = useSelector(state => state.signUp);

    const footerContent = (
        <div>
            <Button label="Cancel" icon="pi pi-times" onClick={onHide} className="p-button-text" />
            <Button 
                label="Request Access" 
                icon={`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`} 
                onClick={() => onOkClick({email: email})} 
                disabled={!email.match(/^[^\s@]+@[^\s@]+\.[^\s@]+$/)} 
            />
        </div>
    );

    return (
        <Dialog 
            header="🔐 Access 123Cluster Demo" 
            visible={visible} 
            style={{ width: "400px" }} 
            footer={footerContent} 
            onHide={onHide}
        >
            <p>Experience the full capabilities of 123Cluster with a live demo session! Enter your email below to receive access details.</p>
            
            <InputText 
                value={email} 
                onChange={(e) => setEmail(e.target.value)} 
                placeholder="Enter your email"
                className="p-inputtext p-component p-mt-2"
            />
            
            <p style={{ fontSize: "12px", marginTop: "10px", color: "#666" }}>
                We respect your privacy. Your email will only be used for demo access and important updates.
            </p>
        </Dialog>
    );
};

export default RequestDemoDialog;