import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useEffect, useState } from "react";
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import { SelectButton } from "primereact/selectbutton";
import { useDispatch, useSelector } from "react-redux";
import { MonitoringActions } from "../../store/monitoring/reducer";
import { InputText } from "primereact/inputtext";
import PieChart from "../../components/pieChart";
import LineChart from "../../pages/status/lineChart";
import { Panel } from "primereact/panel";

const SelectDashboardTypeDialog = ({ visible, onHide, onOkClick }) => {
  const [pageIndex, setPageIndex] = useState(0);
  const [dashboardType, setDashboardType] = useState("table");
  const [filteredNodes, setFilteredNodes] = useState([]);
  const [search, setSearch] = useState("");
  const dispatch = useDispatch();
  const { node_monitors_table} = useSelector((state) => state.monitoring);
  const [selectedNode, setSelectedNode] = useState(null);
  const { user } = useSelector((state) => state.app)
  const dashboardOptions = [
    { label: "Line Chart", value: "line_chart" },
    { label: "Pie Chart", value: "pie_chart" },
    { label: "Monitor Status", value: "monitor_status" },
    { label: "Table", value: "table" }
  ]

  const onSearchChange = (e) => {
    setSearch(e.target.value);
    const search = e.target.value.toLowerCase();
    setFilteredNodes(
      node_monitors_table?.filter((n) => n?.name?.toLowerCase()?.includes(search) ||
    n?.monitor_name?.toLowerCase()?.includes(search) ||
    n?.display_name?.toLowerCase()?.includes(search) ||
    n?.db_type?.toLowerCase()?.includes(search) ||
    n?.node_type?.toLowerCase()?.includes(search)));
  }

  useEffect(() => {

    if (pageIndex === 1)
    dispatch(MonitoringActions.getNodeMonitorsTable(dashboardType));

  }, [pageIndex, dashboardType, dispatch])

  const nodeMonitorsHeader =   <span className="p-input-icon-right">
     <InputText className={"p-inputtext-sm"} placeholder="Search" value={search} onChange={onSearchChange} />
  <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => {
      setSearch("");
      setFilteredNodes(node_monitors_table);

  }} ></i>

</span>


  useEffect(() => {
    if (node_monitors_table?.length) setFilteredNodes(node_monitors_table)
  }, [node_monitors_table]);

  const getHeader = () => {
    switch (pageIndex) {

      case 0:
        return "Select chart type";
      case 1:
        return "Select node monitor";
      case 2:
        return "Select dashboard type";
      case 3:
        return "Select chart column";
      default:
        return "Dashboard";

    }
  }

  const handleOk = () => {
      const data = {
        org_id: selectedNode?.org_id ? selectedNode.org_id : 1,
        node_id: selectedNode?.id,
        node_name: selectedNode?.name,
        monitor_name: selectedNode?.monitor_name,
        node_type: dashboardType === "line_chart" ? selectedNode?.node_type : selectedNode?.db_type,
        created_by: user?.data?.id,
        visualization_type: dashboardType,
      }
      data.column_display_name = selectedNode?.display_name
      if (onOkClick) onOkClick(data);

  }
  const handleBack = () => {

    switch (pageIndex) {
      case 1:
      case 2:
        setPageIndex(pageIndex - 1);
        break;
      default:
        onHide();
        break;
    }

  }

  const renderFooter = () => {
    return (
      <div className="flex justify-content-between">
        {pageIndex > 0 &&  <Button
          label={"Back"}
          icon="pi pi-arrow-left"
          className="p-button-secondary"
          onClick={handleBack} />}
        {pageIndex  === 0 && <Button
          label={"Next"}
          icon="pi pi-arrow-right"
          className="ml-auto p-button"
          onClick={() => setPageIndex(pageIndex + 1)} autoFocus />}
        {pageIndex === 1 && <Button
          label={"Select"}
          icon={"pi pi-check"}
          className="p-button"
          onClick={handleOk} autoFocus />}
      </div>
    );

  }

  const renderPage2 = () => {

    return (<div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">

      <div className="field grid">
        <div className="col">

        <DataTable header = {nodeMonitorsHeader}  value={filteredNodes} selectionMode="single" selection={selectedNode} onSelectionChange={(e) => setSelectedNode(e.value)} dataKey="index"  paginator = {filteredNodes?.length > 50} rows={50} >
                        <Column field="name" header="Node" sortable> </Column>
                        <Column field="monitor_name" header="Monitor" sortable></Column>
                        <Column field= {`${dashboardType === "table" ? "db" : "node"}_type`} header="Type" sortable></Column>
                       {dashboardType === "line_chart" &&   <Column field="display_name" header="Column" sortable></Column>}
                    </DataTable>

        </div>
      </div>
    </div>)

  }

  const renderPage1 = () => {
    return (
      <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
        <div className="field grid">
          <div className="col-3 vertical-select-button">
            <SelectButton
              className="p-button-sm"
              value={dashboardType}
              options={dashboardOptions}
              onChange={(e) => setDashboardType(e.value)}
            />
          </div>
          <div className="col-9">
            <div className="chart-preview-container" style={{
              border: "1px dashed #ccc",
              borderRadius: "4px",
              height: "300px",
              display: "flex",
              justifyContent: "center",
              alignItems: "center",
            }}>
              {renderChartPreview(dashboardType)}
            </div>
          </div>
        </div>
      </div>
    );
  };

  const renderChartPreview = (dashboardType) => {
    switch (dashboardType) {
      case 'line_chart':
        const sampleLineData = [
          { index: 1, col_name: 10 },
          { index: 2, col_name: 25 },
          { index: 3, col_name: 25 },
          { index: 4, col_name: 40 },
          { index: 5, col_name: 41 },
          { index: 6, col_name: 45 },
        ];
        return (
          <div style={{ width: '100%', height: '100%', display: 'flex', justifyContent: 'center', alignItems: 'center' }}>
            <LineChart
              data={sampleLineData}
              fillColor="rgba(52, 152, 219, 0.1)"
            />
          </div>
        );
      case 'monitor_status':
        return <div className="multiline">
           <Panel
            className="dashboard-chart"
            header={
              <div className="multiline">
                {`Node Name — Monitor Name — Node Type`}
              </div>
            }
          >
            {(() => { return <span style={{ color: 'green' }}>{'GREEN'}</span>;})()}
          </Panel>
        </div>;
      case 'pie_chart':
        const samplePieData = [
          { name: 'Category A', value: 40 },
          { name: 'Category B', value: 30 },
          { name: 'Category C', value: 20 },
          { name: 'Category D', value: 10 }
        ];
        return <PieChart data={samplePieData} labelKey="name" valueKey="value" />;
      case 'table':
        return (
          <Panel
              className="dashboard-chart"
              header={
                  <div className="multiline">
                      {`Node Name — Monitor Name — Node Type`}
                  </div>
              }
          >
              <DataTable
                  rows={10}
                  paginator
                  alwaysShowPaginator={false}
                  rowClassName={(rowData) =>
                      `severity-index-${rowData["color"]?.toLowerCase()}-row`
                  }
                  value={[
                      { message: "Sample message 1", value: 10, color: "red" },
                      { message: "Sample message 2", value: 20, color: "green" },
                      { message: "Sample message 3", value: 30, color: "orange" },
                      { message: "Sample message 4", value: 40, color: "red" },
                      { message: "Sample message 5", value: 50, color: "red" },
                      { message: "Sample message 6", value: 60, color: "green" },
                      { message: "Sample message 7", value: 70, color: "orange" },
                      { message: "Sample message 8", value: 80, color: "red" },
                      { message: "Sample message 9", value: 90, color: "red" },
                      { message: "Sample message 10", value: 100, color: "green" },
                  ]}
              >
                  <Column field="message" header="Message" sortable />
                  <Column field="value" header="Value" sortable />
                  <Column field="color" header="Color" sortable />
              </DataTable>
          </Panel>
      );
      default:
        return <div>Preview for {dashboardType}</div>;
    }
  };

  return (<Dialog
    className='select-dashboard-dialog'
    header={() => getHeader(pageIndex)}
    footer={renderFooter()}
    visible={visible}
    style={{ width: '50vw', maxHeight: '90%' }}
    onHide={onHide}>
    <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
      {pageIndex === 0 && renderPage1()}
      {pageIndex === 1 && renderPage2()}
    </div>
  </Dialog>
  )

}

export default SelectDashboardTypeDialog;
