import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import { useState } from "react";
import { types } from "../../config";
import { RadioButton } from "primereact/radiobutton";
import { LayoutActions } from "../../store/layout/reducer";
import { useDispatch } from "react-redux";
import CreatePgK8SClusterDialog from "../postgres_kubernetes/createPgK8SClusterDialog";
import CreateMongoPodDialog from "../mongo_kubernetes/CreateMongoPodDialog";
import SelectOsDialog from "../selectOsDialog";
import AddPostgresAWSPrimaryDialog from "../postgres_aws/addPostgresPrimary";


const SelectServiceTypeDialog = (props) => {
    const [targetService, setTargetService] = useState(props.services?.[0]);
    const [showAddPgK8SCluster, setShowAddPgK8SCluster] = useState(false);
    const [showAddMongoPod, setShowAddMongoPod] = useState(false);
    const [showAddPostgresAWS, setShowAddPostgresAWS] = useState(false);
    const [showSelectOS, setShowSelectOS] = useState(false);
    const [serviceRadio, setServiceRadio] = useState(props.services?.[0]?.name);
    const dispatch = useDispatch();

    const onAddPgK8SCluster = (data) => dispatch(LayoutActions.addPgPod(data, setShowAddPgK8SCluster));
    const onAddMongoPod = (data) => dispatch(LayoutActions.addMongoPod(data, setShowAddMongoPod));
    const onAddPostgresAWS = (data) => dispatch(LayoutActions.addPgPrimary(data, setShowAddPostgresAWS));

    const onBack = () => {
        if ([types.PROXMOX, types.AWS].includes(targetService?.type)) setShowSelectOS(false);
        props.onShow();
    }

    const handleOk = () => {
        if (!targetService) {
            if (props.defaultCreator) props.defaultCreator(true);

        }
        else {
            switch (targetService.type) {
                case types.K8S:
                    switch (props.type) {

                        case types.POSTGRES:
                            setShowAddPgK8SCluster(true);
                            break;
                        case types.MONGO:
                            setShowAddMongoPod(true);
                            break;
                        default:
                            break;
                    }

                    break;

                case types.PROXMOX:
                case types.AWS:
                    setShowSelectOS(true);
                    break;
                default:
                    break;

            }
        }
        props.onHide();
    }

    const renderFooter = () => {
        return (
            <div style={{ display: "flex", justifyContent: "space-between" }}>
                <Button label={"Back"} icon="pi pi-arrow-left" onClick={props.onBack} className="p-button-text" />
                <Button
                    label={"Next"}
                    icon={`pi pi-arrow-right'`}
                    iconPos="right"
                    onClick={handleOk} autoFocus />
            </div>
        );

    }



    return (
        <div>




            {showAddPostgresAWS ? <AddPostgresAWSPrimaryDialog
                visible={showAddPostgresAWS}
                service={targetService}
                onHide={() => {
                    setShowAddPostgresAWS(false);
                }}
                onOkClick={onAddPostgresAWS}
                onBack={onBack}
            /> : null}





            {showAddPgK8SCluster ? <CreatePgK8SClusterDialog
                visible={showAddPgK8SCluster}
                service={targetService}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setPgVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddPgK8SCluster(false)
                }}
                onOkClick={onAddPgK8SCluster}
                onBack={onBack}
            /> : null}



            {showAddMongoPod ? <CreateMongoPodDialog
                visible={showAddMongoPod}
                service={targetService}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setMongoVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddMongoPod(false)
                }}
                onOkClick={onAddMongoPod}
                onBack={onBack}
            /> : null}

            <SelectOsDialog
                service={targetService}
                visible={showSelectOS}
                onHide={() => setShowSelectOS(false)}
                onBack={onBack}
                onOkClick={(targetOs) => {
                    if (targetService?.type === types.PROXMOX) props.defaultCreator(`${targetOs}@${targetService.name}`);
                    else {
                        switch (props.type) {

                            case types.POSTGRES:
                                setShowAddPostgresAWS(true);
                                break;
                            default:
                                break;

                        }
                    }
                    setShowSelectOS(false);
                }}
            />




            <Dialog
                className='select-service-type'
                header={`Select service type`}
                visible={props.visible}
                style={{ width: '40vw' }}
                onHide={props.onHide}
                footer={renderFooter()}>
                <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
                    <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">




                        {props.services?.length > 0 ?
                            <div>
                                <div className="field">
                                    This database can be created on the following services:
                                </div>

                                {
                                    props.services.map(s => (<div key={`serviceRadio${s.id}`} className="field-radiobutton">
                                        <RadioButton inputId={`serviceRadio${s.id}`} name="serviceRadio" value={s.name} checked={serviceRadio === s.name} onChange={(e) => {
                                            const service_name = e.value;
                                            setServiceRadio(service_name)
                                            setTargetService(props.services.filter((s) => s.name === service_name)[0])
                                        }} />
                                        <label htmlFor={`serviceRadio${s.id}`}>{`${s.name}${s.host_name?.length > 0 ? `@${s.host_name}` : ''} (${s.type})`}</label>
                                    </div>))
                                }
                                <div key={`serviceRadio9999`} className="field-radiobutton">
                                    <RadioButton inputId={`serviceRadio9999`} name="serviceRadio" value={"#own_vm"} checked={serviceRadio === "#own_vm"} onChange={(e) => {
                                        const service_name = e.value;
                                        setServiceRadio(service_name);
                                        setTargetService(null);
                                    }} />
                                    <label htmlFor={`serviceRadio999`}>I want to use my own VM</label>
                                </div>
                            </div>

                            : <div className="multiline">{"Found no services on which this database can be created on.\nProceed to create the database on your own VM"}</div>}



                    </div>
                </div>

            </Dialog>

        </div>
    )

};

export default SelectServiceTypeDialog;