import UserPasswordForm from '../../components/forms/userPasswordForm';
import {useSelector, useDispatch} from 'react-redux';
import { Dialog } from 'primereact/dialog';
import { useState } from 'react';
import { TabPanel, TabView } from 'primereact/tabview';
import { Button } from 'primereact/button';
import NotificationsForm from "../../components/forms/notificationsForm"
import PrivilegesForm from '../../components/forms/privilegesForm';
import validator from '../../util/validator';
import userConstraints from '../../components/forms/userPasswordForm/constraints';
import notificationsConstraints from '../../components/forms/notificationsForm/constraints';
import { SignupActions } from '../../store/signup/reducer';



export default function SignupDialog(props) {

  const [toggle, setToggle] = useState(0);
  const {data, isLoading} = useSelector((state) => state.signUp);
  const [userErrors, setUserErrors] = useState({});
  const [emailError, setEmailError] = useState("");
  const dispatch = useDispatch();
  
  const handleNext = () => {

    let errors = validateUser();
    if(errors.length === 0)
    {
  
      dispatch(SignupActions.validateEmail({"email":data.notice_dict.email}, setToggle));
  
    }
    
  }

  const renderFooter = () => {

    if(toggle === 0)
    {

     return (<div style={{ display: "flex", justifyContent: "space-between" }}>
      <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
      <Button
          label={"Next"}
          icon={`pi pi-${isLoading ? 'pi-spin pi-spinner' : 'arrow-right'}`}
          iconPos="right"
          onClick={() => handleNext()} autoFocus />
  </div>)

    }
    
    if (toggle === 1)
    {

      return (<div style={{ display: "flex", justifyContent: "space-between" }}>
      <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
      <Button
          label={"Back"}
          icon="pi pi-arrow-left"
          iconPos="left"
          onClick={(() => setToggle(0))}/>
      <Button
          label={"Submit"}
          icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
          iconPos="right"
          onClick={handleOk} autoFocus />
  </div>)

    }
    

}

const validateUser = () => 
{

  let errors = [];
  let messages = {};

    let error = validator(userConstraints)("password", data.user_dict.password);
         
    if(error)
    {

      errors.push(error);

    }
    messages.password = error;
    error = validator(userConstraints)("password2", {"password": data.user_dict.password, "password2": data.user_dict.password2});
    if(error)
    {
      errors.push(error);

    }
    messages.password2 = error;

    error = validator(userConstraints)("username", data.user_dict.username);
    if(error)
    {
      errors.push(error);

    }
    messages.username = error;
    setUserErrors(messages);

    error = validator(notificationsConstraints)("email", data.notice_dict.email);
    if(!data.notice_dict.email) error = null;
    if (error) errors.push(error)
  
  setEmailError(error);

    return errors;
}

const handleOk = () => 
{

    props.onOkClick(data);

}

  return (
    <Dialog
    className='sign-up'
    header={"Signup"}
    visible={props.visible}
    style={{maxHeight: "90%", minWidth: "60vw", height: '750px' }}
    onHide={props.onHide}
    footer={renderFooter()}>

      <TabView  activeIndex={toggle} onTabChange={(e) => 
      {
        if(toggle === 0) handleNext();
        else setToggle(e.index)
        
      }}>
      
      <TabPanel header = "Registration">
        <UserPasswordForm errors = {userErrors} update = {false}/>
        <NotificationsForm emailError = {emailError} update = {false}/>
       </TabPanel>

       <TabPanel header = "Privileges">

         <PrivilegesForm update = {false}/>

       </TabPanel>

      </TabView>

  
</Dialog>
);
     
}