import { Dialog } from 'primereact/dialog';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import moment from "moment";
import getColorBySeverity from '../../util/colorBySeverity';
import { loadingMsg } from '../../config';


export default function TaskLogDetailsDialog(props) {

    const statusTextBodyTemplate = (item) => {
        return (<div style={{ color: getColorBySeverity(item.status)}}>{item.status}</div>);
    }

    const msgTextBodyTemplate = (item) => {
        return (<div style={{ color: getColorBySeverity(item.status)}}>{item.ret_msg}</div>);
    }

    const whenTextBodyTemplate = (item) => {
        return (<div style={{ color: getColorBySeverity(item.status) }}>{moment(item.when).format("MMM DD,YYYY, hh:mm A")} </div>);
    }
    
    return (

      <Dialog
            className='task-log-details'
            header={`${props.log && props.log.what} details [by: ${props.log && props.log.created_by} ]`}
            visible={props.visible}
            style={{ width: '50vw', maxHeight: "90%"}}
            onHide={props.onHide}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
            
            <DataTable
             rowClassName={(rowData) => {
                return `severity-${getColorBySeverity(rowData.status)}-row`
            }}  emptyMessage = {!props.log ? <div className="text-sm pl-3">{loadingMsg}</div> : <div className="text-sm pl-3">No log data</div>}  value={[props.log]} >
                    <Column field="retmsg" body={msgTextBodyTemplate} header="Message" sortable></Column>
                    <Column field="when" body={whenTextBodyTemplate} header="When" sortable></Column>
                    <Column field="step" body={statusTextBodyTemplate}header="Status" sortable></Column>
                </DataTable>
        
                   
                    </div>
            </div>

        </Dialog>);
  }
