import { useSelector, useDispatch } from 'react-redux';
import { useState, useEffect } from 'react';
import { LogActions } from "../../store/log/reducer";
import { Dialog } from 'primereact/dialog';
import { DataTable } from "primereact/datatable";
import { Column } from "primereact/column";
import moment from "moment";
import getColorBySeverity from '../../util/colorBySeverity';
import StepDetailsDialog from '../stepDetailsDialog'
import { loadingMsg } from '../../config';


export default function TaskStepLogDialog(props) {

    const {stepLogs, isLoading} = useSelector((state) => state.log);
    const [stepLogCurrent, setStepLogCurrent] = useState(null);
    const [showStepDetails, setShowStepDetails] = useState(false);

    const dispatch = useDispatch();

    useEffect(() => 
    {
        if(props.taskLogId)
        {

            dispatch(LogActions.getStepLogs(props.taskLogId));

        }

    }, [props.taskLogId, dispatch])

    useEffect(() => {
        let target_step = [];
        if ((props.target_log?.includes('db') || props.target_log?.includes('database')) && props.target_log?.includes('creat')){
            target_step = stepLogs?.filter((s) => s.step?.includes('users'))
           
        }
        if(target_step?.length > 0)
        {
            setStepLogCurrent(target_step[0]);
            setShowStepDetails(true);
        }
        

    }, [stepLogs, props.target_log])

    const statusTextBodyTemplate = (item) => {
        return (<div style={{ color: "white"}}>{item.status}</div>);
    }

    const whatTextBodyTemplate = (item) => {
        return (<div style={{ color: "white"}}>{item.step}</div>);
    }

    const whenTextBodyTemplate = (item) => {
        return (<div style={{ color: "white" }}>{moment(item.created_date).format("MMM DD,YYYY, hh:mm A")} </div>);
    }

    const durationBodyTemplate = (item) => {
        console.log(item);
        return (<div style={{ color: "white"}}>{item.duration}</div>);
    }

    
    return (

      <Dialog
            className='step-logs'
            header={ props.selectedLog ?  `${props.selectedLog?.task} details [by: ${props.selectedLog?.created_by} ]`  : stepLogs?.length > 0  ? `${stepLogs[0].what} details [by: ${stepLogs[0].created_by} ]`: loadingMsg}
            visible={props.visible}
            style={{ width: '50vw', maxHeight: "90%"}}
            onHide={props.onHide}>
            <div style={{ display: "flex", paddingBottom: "10px" }} className="form-container">
            <div style={{ flex: 1, padding: "10px" }} className="p-fluid form-container">
            
            <DataTable sortField='created_date' rowClassName = {(rowData) => {
                return `severity-${getColorBySeverity(rowData.status)}-row`
            }} selectionMode='single' dataKey='created_date' selection={stepLogCurrent} onSelectionChange={(e) => {
                setStepLogCurrent(e.value);
                setShowStepDetails(true);

            } } emptyMessage = {!stepLogs && isLoading ?  <div className="text-sm empty-severity">{loadingMsg}</div> : <div className="text-sm empty-severity">No steps data</div>} paginator alwaysShowPaginator={false} rows={10} rowsPerPageOptions={[10, 20, 50, 100]} value={stepLogs}  className="severity-table">
                    <Column field="status" body={statusTextBodyTemplate} header="Status" sortable></Column>
                    <Column field="created_date" body={whenTextBodyTemplate} header="When" sortable></Column>
                    <Column field="step" body={whatTextBodyTemplate}header="Message" sortable></Column>
                     <Column field="duration" body={durationBodyTemplate}header="Duration" sortable></Column>
                </DataTable>
                    </div>
                    {showStepDetails ? <StepDetailsDialog step={stepLogCurrent} visible={showStepDetails} onHide={() => setShowStepDetails(false) }/> : null}
            </div>

        </Dialog>);
  }
