
import UserPasswordForm from '../../components/forms/userPasswordForm';
import {useSelector, useDispatch } from 'react-redux';
import { useEffect, useRef } from 'react';
import { Dialog } from 'primereact/dialog';
import { TabPanel, TabView } from 'primereact/tabview';
import { Button } from 'primereact/button';
import NotificationsForm from "../../components/forms/notificationsForm"
import { useState } from 'react';
import { UserUpdateActions} from '../../store/userUpdate/reducer';
import PrivilegesForm from '../../components/forms/privilegesForm';
import validator from '../../util/validator';
import userConstraints from "../../components/forms/userPasswordForm/constraints";
import notificationsConstraints from "../../components/forms/notificationsForm/constraints";
import { SignupActions } from '../../store/signup/reducer';



export default function UserUpdateDialog(props) {

  const [toggle, setToggle] = useState(0);
  const {initial, data, isLoading} = useSelector((state) => state.userUpdate);
  const [userErrors, setUserErrors] = useState({});
  const [emailError, setEmailError] = useState("");
  const dispatch = useDispatch();

  const loaded = useRef(false);

   useEffect(() => {

    if(!loaded.current){
      
    if(initial) 
    {

      if(initial?.priv_dict)
      {

        dispatch(UserUpdateActions.setData({...data, notice_dict: initial.notice_dict, priv_dict: 
        {

          ...initial.priv_dict,

          is_superuser: "False"

        }}));

      }
      else
      {

        if(initial?.notice_dict)
        {

          dispatch(UserUpdateActions.setData({...data, notice_dict: initial.notice_dict}));

        }

      }
      loaded.current = true
    }
    }


  }, [initial, data, dispatch])

const validateUser = () => 
{

  let errors = [];
  let messages = {};

  if(data.user_dict.password)
  {

    let error = validator(userConstraints)("password", data.user_dict.password);
         
    if(error)
    {

      errors.push(error);

    }
    messages.password = error;
    error = validator(userConstraints)("password2", {"password": data.user_dict.password, "password2": data.user_dict.password2});
    if(error)
    {
      errors.push(error);

    }
    messages.password2 = error;
    setUserErrors(messages);
    
    error = validator(notificationsConstraints)("email", data.notice_dict.email);
    if(!data.notice_dict.email) error = null;
    if (error) errors.push(error);
    
    setEmailError(error);
  }
  return errors;
}

const handleNext = () => {

  let errors = validateUser();
  if(errors.length === 0)
  {

    dispatch(SignupActions.validateEmail({"email":data.notice_dict.email}, setToggle))

  }
  
}


const renderFooter = () => {

   return (<div style={{ display: "flex", justifyContent: "space-between" }}>
     <Button label={"Close"} icon="pi pi-times" onClick={props.onHide} className="p-button-text" />
      {toggle > 0 &&  <Button
        label={"Back"}
        icon="pi pi-arrow-left"
        iconPos="left"
        onClick={(() => setToggle(0))}/>}
         
        {toggle < 1 && <Button
        label={"Next"}
        icon="pi pi-arrow-right"
        iconPos="right"
        onClick={() => handleNext()} autoFocus /> }
        {toggle === 1 && <Button
        label={"Submit"}
        icon= {`pi pi-${isLoading ? 'spin pi-spinner' : 'check'}`}
        iconPos="right"
        onClick={handleOk} autoFocus />}
   
</div>)
  
}

const handleOk = () => 
{
  
  
  dispatch(UserUpdateActions.userUpdate({ pk: props.userId, data: data, callback: props.onHide}));

}

  return (
   (initial &&  <Dialog
    className='user-update'
    header={`Update user ${initial.user_dict.username}`}
    visible={props.visible}
    style={{maxHeight: "90%", minWidth: "60vw", height: '750px' }}
    onHide={props.onHide}
    footer={renderFooter()}>

      <TabView  activeIndex={toggle} onTabChange={(e) =>
      {

        if(toggle === 0) handleNext();
        
        else setToggle(e.index)

      }}>
      
      <TabPanel header = {`Change  ${initial.user_dict.username} settings`}>
        <UserPasswordForm update = {true} errors = {userErrors}/>
        <NotificationsForm update = {true} emailError = {emailError}/>
       </TabPanel>

       <TabPanel header = {` ${initial.user_dict.username} Privileges`}>
       {initial && !initial.is_superuser  && !initial.current && <PrivilegesForm update = {true}/>}
       {initial && !initial.current && initial.is_superuser && <div>
            Super User
        </div>}
       </TabPanel>
      </TabView>  
</Dialog>)
);
}