import drop from '../../../../assets/images/drop.svg';
import green_star from '../../../../assets/images/green-star.png'
import red_star from '../../../../assets/images/red-star.png';
import info from '../../../../assets/images/info.svg';
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation, useNavigate } from "react-router-dom";
import { StatusActions } from "../../../../store/status/reducer";
import { Fragment, useEffect, useState, useCallback } from "react";
import { Button } from 'primereact/button';
import { LayoutActions } from '../../../../store/layout/reducer';
import { Dropdown } from "primereact/dropdown";
import DetailsMenu from '../../../../components/menus/detailsMenu';
import WarningDialog from '../../../../dialogs/warningDialog';
import PerformanceCharts from '../../../../components/performanceCharts';
import { ConfirmPopup } from 'primereact/confirmpopup';
import { TabView, TabPanel } from 'primereact/tabview';
import { Tooltip } from 'primereact/tooltip';
import { timePeriodTypes } from '../../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../../components/tables/nodeAlertsTable';
import createAPIcommand from '../../../../util/createApiCommand';
import copyToClipboard from '../../../../util/copyToClipboard';
import DropHAProxyNodeDialog from '../../../../dialogs/balancers/haproxy/dropHAProxy';
import { InputText } from 'primereact/inputtext';
import { AppActions } from '../../../../store/app/reducer';
import { ButtonGroup } from 'primereact/buttongroup';
import { searchFilter } from '../../../monitoring';
import { searchFilter as logSearch } from '../../../log';
import LogsTable from '../../../../components/tables/logsTable';


export default function HAProxyDetails(props) {


    const { id } = useParams();
    const { state } = useLocation();

    const { node, haproxy_details, errorMessage } = useSelector((state) => state.status);
    const { accessToken, severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);
    const [filteredLogs, setFilteredLogs] = useState(null);


    const dispatch = useDispatch();
    const navigate = useNavigate();

    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [upperSearch, setUpperSearch] = useState("");

    const [showWarningPopup, setShowWarningPopup] = useState(false);

    const [showDropHAProxyNode, setShowDropHAProxyNode] = useState(false);
    const [toggle, setToggle] = useState(0);

    const { alerts } = useSelector((state) => state.monitoring);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [popupAction, setPopupAction] = useState("stop");

    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);
    const { node_logs} = useSelector((state) => state.log);

    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }
         
        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];
      
    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);


    useEffect(() => {


        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;

        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            if (!showSeverityGreen || !showSeverityYellow || !showSeverityRed) tempData = severityFilter(tempData);
            setFilteredAlerts(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);

    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen])


    // useEffect(() => {

    //     if (haproxy_details?.history?.length > 0) {

    //         const query = upperSearch.toLowerCase().trim();
    //         if (!query) setFilteredLogs(haproxy_details.history);
    //         else setFilteredLogs(haproxy_details.history.filter((h) => h['___REALTIME_TIMESTAMP'].format("MMM DD,YYYY, hh:mm A").includes(query)
    //             || ['MESSAGE'].toLowerCase().includes(query)
    //             || h['SYSLOG_IDENTIFIER'].toLowerCase().includes(query)
    //             || h['_PID'].toLowerCase().includes(query)))
    //     }

    // },
    //     [upperSearch, haproxy_details?.history])


    const onShowPopup = async (target) => {
        switch (target) {

            case "stop-down":
                setPopupAction("stop");
                await copyToClipboard(createAPIcommand({ "node_id": haproxy_details?.node?.name }, accessToken, "stop_haproxy_node/"))
                break;
            case "start-down":
                setPopupAction(haproxy_details?.node?.ind_started ? "restart" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": haproxy_details?.node?.name }, accessToken, "start_haproxy_node/"))
                break;
            case "start-stop":
                setPopupAction(haproxy_details?.node?.ind_started ? "stop" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": haproxy_details?.node?.name }, accessToken, haproxy_details?.node?.ind_started ? "stop_haproxy_node/" : "start_haproxy_node/"));
                break;
            default:
                break;


        }
        setShowWarningPopup(target);

    }


    useEffect(() => { if (alerts?.length > 0) setFilteredAlerts(alerts) }, [alerts]);
    useEffect(() => {
        if (id) {

            dispatch(StatusActions.getHaProxyDetails(id));
        }
        return () => dispatch(StatusActions.setHaProxyDetails(null));
    }, [id, dispatch])

    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);

    }, [state?.targetTab])

    const onPopupAccept = () => {

        switch (showWarningPopup) {

            case "stop-down":
                onStopHAProxyNode();
                break;
            case "start-down":
                onStartHAProxyNode();
                break;
            case "start-stop":
                haproxy_details?.node?.ind_started ? onStopHAProxyNode() : onStartHAProxyNode();
                break;
            default:
                break;

        }

    }

    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onStopHAProxyNode = () => dispatch(LayoutActions.stopHaProxy({ "node_id": haproxy_details?.haproxy_details?.node?.id }, node));


    const onStartHAProxyNode = () => dispatch(LayoutActions.startHaProxy({ "node_id": haproxy_details?.haproxy_details?.node?.id }, node));

    // const dateBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{moment(item['___REALTIME_TIMESTAMP']).format("MMM DD,YYYY, hh:mm A")}</div>);
    // }

    // const nameBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item['SYSLOG_IDENTIFIER']}</div>);
    // }

    // const pidBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item['_PID']}</div>);
    // }

    // const messageBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item['MESSAGE']}</div>);
    // }

    const onDropHAProxyNode = (data) => {

        dispatch(LayoutActions.dropHaProxy(data, setShowDropHAProxyNode));

    }

    const attachPopup = (target) => haproxy_details?.node?.ind_deleted ? showWarningMessage({ "text": `${haproxy_details?.haproxy_details?.node?.name} is unavailable.`, 'status': haproxy_details?.haproxy_details?.node?.status }) : onShowPopup(target);

    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">
                <TabView activeIndex={toggle} onTabChange={(e) => {
                    if (e.index < 3) setToggle(e.index);
                }}>
                    <TabPanel header="Dashboard">

                        <div className='grid flex-row'>
                            <div className='col-3 flex align-items-between flex-column' >
                                <h2 >
                                    {haproxy_details?.node?.cluster_role} {haproxy_details?.node?.name}
                                </h2>
                                <div>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Instance is {haproxy_details?.node?.ind_started ? 'up' : "down"}
                                                </td>
                                                <td>
                                                    <Button
                                                        className="p-button-sm details-button"
                                                        disabled={!haproxy_details?.node?.ind_started && !errorMessage}
                                                        id="start-stop"
                                                        icon={haproxy_details?.node?.ind_started ? "pi pi-stop-circle" : "pi pi-play"}
                                                        style={{ minWidth: "53px", minHeight: "39px" }}
                                                        iconPos="right"
                                                        onClick={() => attachPopup("start-stop")}
                                                    />
                                                </td>
                                                <td>
                                                    <img src={haproxy_details?.node?.ind_started ? green_star : red_star} alt="star" className="avail-star"></img>
                                                </td>
                                            </tr>

                                            <tr>
                                                <td>
                                                    {haproxy_details?.ind_valid ? "Config is valid" : haproxy_details?.ind_error === false ? "Config is invalid" : "Checking config validity"}
                                                </td>
                                                <td>

                                                </td>
                                                <td>
                                                    <img className='status-star' src={(haproxy_details?.ind_error === undefined || haproxy_details?.ind_error === true) ? red_star : green_star} alt="star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {haproxy_details?.ind_error ? "Errors in cluster" : haproxy_details?.ind_error === false ? "Cluster is OK" : <div> <i className="pi pi-spin pi-spinner" /> Connecting to node... </div>}
                                                </td>
                                                <td>
                                                    <Button onClick={() => node && navigate('/layout', { state: { node_id: haproxy_details?.primary_id } })} disabled={!haproxy_details?.node?.ind_started && !errorMessage} className='p-button-sm details-button'><img alt="info" width={16} height={16} src={info} /></Button>
                                                </td>
                                                <td>
                                                    <img className='status-star' src={(haproxy_details?.ind_error === undefined || haproxy_details?.ind_error === true) ? red_star : green_star} alt="star"></img>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <div>
                                    <h3>
                                        General info
                                    </h3 >
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td> {haproxy_details?.node?.name}</td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>

                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>

                                <Dropdown className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period" value={timePeriodType}
                                    options={timePeriodTypes}
                                    onChange={(e) => {
                                        setTimePeriodType(e.value);
                                        setMonitorInterval(intervalByTimedelta(e.value));
                                    }}
                                />

                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={node?.cpuUsages}
                                    memUsages={node?.memUsages}
                                />

                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>
                                <div className='col-9'>
                                    <span className="p-buttonset down">
                                        <Button disabled={errorMessage || !haproxy_details?.node?.ind_started} icon="pi pi-stop-circle" iconPos="right" className="p-button-sm details-button" id="stop-down" label="Stop" onClick={() => attachPopup("stop-down")} />
                                        <Button disabled={!haproxy_details} id="start-down" icon={`pi pi-${haproxy_details?.node?.ind_started ? "replay" : "play"}`} iconPos="right" className="p-button-sm details-button" label={haproxy_details?.node?.ind_started ? "Restart" : "Start"} onClick={() => attachPopup("start-down")}>
                                        </Button>
                                        <Button disabled={!haproxy_details?.node?.ind_started && !errorMessage} className="p-button-sm details-button" label="Drop" onClick={() => haproxy_details?.node?.ind_deleted ? showWarningMessage({ "text": `${haproxy_details?.haproxy_details?.node?.name} is unavailable.`, 'status': haproxy_details?.haproxy_details?.node?.status }) : setShowDropHAProxyNode(true)}>
                                            < img alt="drop" width={16} height={16} src={drop} />
                                        </Button>
                                    </span>


                                </div>
                            </div>
                        </div>
                    </TabPanel>
                    <TabPanel header="Activity Logs">

                        <div className='flex'>

                            <Dropdown className="p-inputtext-sm mb-2 mt-2 filter-field time-period" value={timePeriodType}
                                options={timePeriodTypes}
                                onChange={(e) => {
                                    setTimePeriodType(e.value);
                                    setMonitorInterval(intervalByTimedelta(e.value));
                                }}
                            />

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                              <LogsTable filteredLogs={filteredLogs}
                                    timePeriodType={timePeriodType}
                                    />

                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No node_logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} rows={100} paginator alwaysShowPaginator={false} >
                            <Column field='__REALTIME_TIMESTAMP' className='history-column' header="Date" body={dateBodyTemplate} sortable></Column>
                            <Column field='SYSLOG_IDENTIFIER' className='history-column' header="Name" body={nameBodyTemplate} sortable></Column>
                            <Column field='_PID' className='history-column' header="PID" sortable body={pidBodyTemplate}></Column>
                            <Column field='MESSAGE' className='history-column' header="Message" body={messageBodyTemplate} sortable></Column>
                        </DataTable> */}
                      

                    </TabPanel>

                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <Dropdown className="p-inputtext-sm mb-2 mt-2 filter-field time-period" value={timePeriodType}
                                options={timePeriodTypes}
                                onChange={(e) => {
                                    setTimePeriodType(e.value);
                                    setMonitorInterval(intervalByTimedelta(e.value));
                                }}
                            />

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>
                        <NodeAlertsTable node={haproxy_details?.node} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />
                    </TabPanel>

                    <TabPanel headerClassName='filters-header' header={<div className='flex'>
                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => window.location.reload()}
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>
                </TabView>

                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}

                {(showDropHAProxyNode && haproxy_details) ? <DropHAProxyNodeDialog
                    haproxy={haproxy_details?.node}
                    visible={showDropHAProxyNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowDropHAProxyNode(false)
                    }}
                    onOkClick={onDropHAProxyNode}
                />

                    : null}
            </div>
            <ConfirmPopup
                icon="pi pi-exclamation-triangle"
                target={document.getElementById(showWarningPopup)}
                visible={Boolean(showWarningPopup)}
                onHide={() => setShowWarningPopup(false)}
                accept={onPopupAccept}
                message={`Are you sure you want to ${popupAction} the node?`} />
            {haproxy_details && <Tooltip target=".set-schedule-star" content={haproxy_details?.auto_backup?.detail}></Tooltip>}
            {haproxy_details && <Tooltip target=".avail-star" content={haproxy_details?.node?.ind_started ? "Database is running and accepting connections" : "Database is not running"}></Tooltip>}
            {haproxy_details && <Tooltip target=".status-star" content={(haproxy_details?.ind_error === undefined || haproxy_details?.ind_error === true) ? "Some nodes of cluster are down/inaccessible" : "All nodes of cluster are accessible"}></Tooltip>}
        </Fragment>



    )


}
