import { useEffect, useState } from "react";
import { useSelector, useDispatch } from "react-redux";
import { useLocation } from "react-router-dom";
import { Button } from "primereact/button";
import UsersTable from '../../components/tables/usersTable';
import Menu from '../../components/menus/menu';
import { ConfigActions } from "../../store/config/reducer";
import { SettingsActions } from "../../store/settings/reducer";
import { Dropdown } from "primereact/dropdown";
import { TabView, TabPanel } from 'primereact/tabview';
import ClusterConfigTable from "../../components/tables/clusterConfigTable";
import { SignupActions } from "../../store/signup/reducer";
import notify from "../../util/notify";
import appConfig, { dbTypes, types } from "../../config";
import { InputText } from "primereact/inputtext";
import validator from "../../util/validator";
import constraints from "./constraints";
import MonitorsTable from "../../components/tables/monitorsTable";
import MonitoringWizard from "../../dialogs/monitoring/monitorWizard";
import { MonitoringActions } from "../../store/monitoring/reducer";
import MonitorTypeDialog from "../../dialogs/monitoring/monitorTypeDialog";
import ErrorDetailsDialog from "../../dialogs/errorDialog";
import NodeAgentsTable from "../../components/tables/nodeAgentsTable";
import KeysTable from "../../components/tables/keysTable";
import IntegratedServicesTable from "../../components/tables/integratedServicesTable";
import { AppActions } from "../../store/app/reducer";
import { InputNumber } from "primereact/inputnumber";


export default function Config() {

  const { state } = useLocation();
  const [toggle, setToggle] = useState(0);
  const { db_type } = useSelector((state) => state.app);
  const [dbType, setDbType] = useState(db_type === types.ALL || db_type === types.ORACLE_PDB || db_type === types.HAPROXY ? types.ELASTIC : db_type);
  const [monitorsByType, setMonitorsByType] = useState(db_type);
  const [pdbPath, setPDBPath] = useState("");
  const [cdbPath, setCDBPath] = useState("");
  const [oracleExportsPath, setOracleExportsPath] = useState("");
  const [oracleBackupsPath, setOracleBackupsPath] = useState("");
  const [postgresExportsPath, setPostgresExportsPath] = useState("");
  const [postgresBackupsPath, setPostgresBackupsPath] = useState("");
  const [mongoExportsPath, setMongoExportsPath] = useState("");
  const [mongoBackupsPath, setMongoBackupsPath] = useState("");
  const [mariadbExportsPath, setMariadbExportsPath] = useState("");
  const [mariadbBackupsPath, setMariadbBackupsPath] = useState("");
  const [elasticsearchExportsPath, setElasticSearchExportsPath] = useState("");
  const [elasticsearchBackupsPath, setElasticSearchBackupsPath] = useState("");
  const [redisExportsPath, setRedisExportsPath] = useState("");
  const [redisBackupsPath, setRedisBackupsPath] = useState("");
  const [pdbPathError, setPDBPathError] = useState("");
  const [cdbPathError, setCDBPathError] = useState("");
  const [oracleExportsError, setOracleExportsError] = useState("");
  const [oracleBackupsError, setOracleBackupsError] = useState("");
  const [postgresBackupsError, setPostgresBackupsError] = useState("");
  const [postgresExportsError, setPostgresExportsError] = useState("");
  const [mongoBackupsError, setMongoBackupsError] = useState("");
  const [mongoExportsError, setMongoExportsError] = useState("");
  const [mariadbBackupsError, setMariadbBackupsError] = useState("");
  const [mariadbExportsError, setMariadbExportsError] = useState("");
  const [mysqlBackupsError, setMySQLBackupsError] = useState("");
  const [mysqlExportsError, setMySQLExportsError] = useState("");
  const [mysqlExportsPath, setMySQLExportsPath] = useState("");
  const [mysqlBackupsPath, setMySQLBackupsPath] = useState("");
  const [mssqlBackupsError, setMsSQLBackupsError] = useState("");
  const [mssqlExportsError, setMsSQLExportsError] = useState("");
  const [mssqlExportsPath, setMsSQLExportsPath] = useState("");
  const [mssqlBackupsPath, setMsSQLBackupsPath] = useState("");
  const [elasticsearchBackupsError, setElasticSearchBackupsError] = useState("");
  const [elasticsearchExportsError, setElasticSearchExportsError] = useState("");
  const [redisBackupsError, setRedisBackupsError] = useState("");
  const [redisExportsError, setRedisExportsError] = useState("");
  const [HistoryRetention, setHistoryRetention] = useState("");
  const [HistoryRetentionError, setHistoryRetentionError] = useState("");
  const [updateMonitor, setUpdateMonitor] = useState(false);
  const [currentItem, setCurrentItem] = useState(null);
  const [showMonitorWizard, setShowMonitorWizard] = useState(false);
  const [showMonitorTypeDialog, setShowMonitorTypeDialog] = useState(false);
  const [monitorType, setMonitorType] = useState(types.ELASTIC);
  const { errorMessage, infoMessage } = useSelector((state) => state.signUp);
  const { settings, errorMessage: settingsError } = useSelector((state) => state.settings);
  const { config, errorMessage: configError } = useSelector((state) => state.config);
  const [monitorError, setMonitorError] = useState("");
  const [showErrorDetails, setShowErrorDetails] = useState(false);
  const [monitorSearch, setMonitorSearch] = useState("");

  const errorCallback = (message) => {

    setMonitorError(message);
    setShowErrorDetails(true)

  }

  const dispatch = useDispatch();

  const onMonitorAdd = () => {
    setShowMonitorTypeDialog(true);
  }

  const onSetMonitorType = (type) => {
    setMonitorType(type);
    setUpdateMonitor(false);
    setShowMonitorWizard(true);
    setShowMonitorTypeDialog(false);
  }

  useEffect(() => {

    if (!config) dispatch(ConfigActions.getConfig());
  }, [config, dispatch]);

  useEffect(() => {
    if (!settings) dispatch(SettingsActions.getSystemSettings());
  }, [settings, dispatch]);

  useEffect(() => {

    if (state?.targetType) setDbType(state?.targetType);
  }, [state?.targetType]);

  useEffect(() => {

    if (state?.targetTab) setToggle(state?.targetTab);
  }, [state?.targetTab]);

  useEffect(() => {
    if (errorMessage) notify.error(errorMessage);
    if (infoMessage === "ok" || errorMessage === "email already taken!") {
      dispatch(SignupActions.setData(appConfig.formDefaults));
    }
  }, [errorMessage, infoMessage, dispatch])


  useEffect(() => {
    if (configError) notify.error(configError);
  }, [configError])

  useEffect(() => {
    if (settingsError) notify.error(settingsError);
  }, [settingsError])

  const onMonitorEdit = (data) => {

    setCurrentItem(data);
    setUpdateMonitor(true);
    setShowMonitorWizard(true);

  }
  useEffect(() => {
    if (config) {
      setPDBPath(config.oracle_sys?.pdb_path);
      setCDBPath(config.oracle_sys?.cdb_path);
      setOracleExportsPath(config.oracle_sys?.exports_path);
      setOracleBackupsPath(config.oracle_sys?.backups_path);
      setPostgresBackupsPath(config.pg_sys?.backups_path);
      setPostgresExportsPath(config.pg_sys?.exports_path);
      setMongoBackupsPath(config.mongo_sys?.backups_path);
      setMongoExportsPath(config.mongo_sys?.exports_path);
      setMariadbBackupsPath(config.mariadb_sys?.backups_path);
      setMariadbExportsPath(config.mariadb_sys?.exports_path);
      setMySQLBackupsPath(config.mysql_sys?.backups_path);
      setMySQLExportsPath(config.mysql_sys?.exports_path);
      setElasticSearchBackupsPath(config.elasticsearch_sys?.backups_path);
      setElasticSearchExportsPath(config.elasticsearch_sys?.exports_path);
      setRedisBackupsPath(config.redis_sys?.backups_path);
      setRedisExportsPath(config.redis_sys?.exports_path);
      setMsSQLBackupsPath(config.mssql_sys?.backups_path);
      setMsSQLExportsPath(config.mssql_sys?.exports_path);
    }
    if (settings) {
      setHistoryRetention(settings['setting value']);
    }

  }, [config, settings])

  const onSystemUpdate = () => {
    let errors = []
    let error;
    error = validator(constraints)("HistoryRetention", HistoryRetention);
    (error == null || errors.push(error)) && setHistoryRetentionError(error);

    if (errors.length === 0)
      dispatch(SettingsActions.updateSystemSettings(
        {
          name: 'history_retention_minutes',
          value: HistoryRetention
        }));

    switch (dbType) {
      case types.ORACLE_CDB:
        error = validator(constraints)("pdbPath", pdbPath);
        (error == null || errors.push(error)) && setPDBPathError(error);

        error = validator(constraints)("cdbPath", cdbPath);
        (error == null || errors.push(error)) && setCDBPathError(error);

        error = validator(constraints)("oracleExportsPath", oracleExportsPath);
        (error == null || errors.push(error)) && setOracleExportsError(error);

        error = validator(constraints)("oracleBackupsPath", oracleBackupsPath);
        (error == null || errors.push(error)) && setOracleBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {
              db_type: dbType,
              pdb_path: pdbPath,
              cdb_path: cdbPath,
              exports_path: oracleExportsPath,
              backups_path: oracleBackupsPath,
            }));
        break;
      case types.POSTGRES:
        error = validator(constraints)("postgresExportsPath", postgresExportsPath);
        (error == null || errors.push(error)) && setPostgresExportsError(error);

        error = validator(constraints)("postgresBackupsPath", postgresBackupsPath);
        (error == null || errors.push(error)) && setPostgresBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {

              db_type: dbType,
              exports_path: postgresExportsPath,
              backups_path: postgresBackupsPath,

            }));
        break;
      case types.MONGO:
        error = validator(constraints)("mongoExportsPath", mongoExportsPath);
        (error == null || errors.push(error)) && setMongoExportsError(error);

        error = validator(constraints)("mongoBackupsPath", mongoBackupsPath);
        (error == null || errors.push(error)) && setMongoBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {

              db_type: dbType,
              exports_path: mongoExportsPath,
              backups_path: mongoBackupsPath,

            }));
        break;
      case types.MARIADB:
        error = validator(constraints)("mariadbExportsPath", mariadbExportsPath);
        (error == null || errors.push(error)) && setMariadbExportsError(error);

        error = validator(constraints)("mariadbBackupsPath", mariadbBackupsPath);
        (error == null || errors.push(error)) && setMariadbBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {

              db_type: dbType,
              exports_path: mariadbExportsPath,
              backups_path: mariadbBackupsPath,

            }));
        break;
      case types.MYSQL:
        error = validator(constraints)("mysqlExportsPath", mysqlExportsPath);
        (error == null || errors.push(error)) && setMySQLExportsError(error);

        error = validator(constraints)("mysqlBackupsPath", mysqlBackupsPath);
        (error == null || errors.push(error)) && setMySQLBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {

              db_type: dbType,
              exports_path: mysqlExportsPath,
              backups_path: mysqlBackupsPath,

            }));
        break;

      case types.MSSQL:
        error = validator(constraints)("mssqlExportsPath", mssqlExportsPath);
        (error == null || errors.push(error)) && setMsSQLExportsError(error);

        error = validator(constraints)("mssqlBackupsPath", mssqlBackupsPath);
        (error == null || errors.push(error)) && setMsSQLBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {

              db_type: dbType,
              exports_path: mssqlExportsPath,
              backups_path: mssqlBackupsPath,

            }));
        break;
      case types.ELASTIC:
        error = validator(constraints)("elasticsearchExportsPath", elasticsearchExportsPath);
        (error == null || errors.push(error)) && setElasticSearchExportsError(error);

        error = validator(constraints)("elasticsearchBackupsPath", elasticsearchBackupsPath);
        (error == null || errors.push(error)) && setElasticSearchBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {

              db_type: dbType,
              exports_path: elasticsearchExportsPath,
              backups_path: elasticsearchBackupsPath,

            }));
        break;
      case types.REDIS:
        error = validator(constraints)("redisExportsPath", redisExportsPath);
        (error == null || errors.push(error)) && setRedisExportsError(error);

        error = validator(constraints)("redisBackupsPath", redisBackupsPath);
        (error == null || errors.push(error)) && setRedisBackupsError(error);

        if (errors.length === 0)
          dispatch(ConfigActions.updateSystemConfig(
            {

              db_type: dbType,
              exports_path: redisExportsPath,
              backups_path: redisBackupsPath,

            }));
        break;
      default:
        break

    }

  }

  const onConfirm = (update, data) => {

    if (update) dispatch(MonitoringActions.updateMonitor(data, {
      "success": setShowMonitorWizard,
      "error": errorCallback
    }));
    else dispatch(MonitoringActions.addMonitor(data, {
      "success": setShowMonitorWizard,
      "error": errorCallback
    }));

  }




  return (
    <div>
      <Menu />
      <div className="config-page" >
        <TabView className="config-tabs" style={{ width: "100%" }} activeIndex={toggle} onTabChange={(e) => {
          setToggle(e.index);
          if (e.index === 1 && config?.current_user?.is_superuser === false) {
            notify.error("Only superuser allowed to change cluster size!");
          }
        }}>
          <TabPanel header="Users" >
            <UsersTable users={config ? config.users : null} current_user={config ? config?.current_user : null} />
          </TabPanel>
          <TabPanel header="Cluster sizing">
            <div>
              <div className="flex justify-content-start">
                <Dropdown className="p-inputtext-sm filter-field" value={dbType}
                  options={dbTypes.filter((type) => ![types.ORACLE_PDB, types.HAPROXY, types.ALL].includes(type.value))}
                  onChange={(e) => {
                    const value = e.value;
                    setDbType(value);
                    setMonitorsByType(value);
                    localStorage.setItem("FilterDbType", value);
                    dispatch(AppActions.setDbType(value));
                  }} />
              </div>

              {config && dbType === types.MYSQL && <ClusterConfigTable data={config?.mysql_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
              {config && dbType === types.MSSQL && <ClusterConfigTable data={config?.mssql_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
              {config && dbType === types.REDIS && <ClusterConfigTable data={config?.redis_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
              {config && dbType === types.MARIADB && <ClusterConfigTable data={config?.mariadb_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
              {config && dbType === types.ELASTIC && <ClusterConfigTable data={config?.elasticsearch_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
              {config && dbType === types.MONGO && <ClusterConfigTable data={config?.mongo_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
              {config && dbType === types.POSTGRES && <ClusterConfigTable data={config?.postgres_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
              {config && dbType === types.ORACLE_CDB && <ClusterConfigTable data={config?.oracle_cluster_sizing} db_type={dbType} is_superuser={config?.current_user?.is_superuser} />}
            </div>
          </TabPanel>
          <TabPanel header="System">
            <div className="flex flex-column">

            <div className="grid">
              <div className="col-12">
                  <h3>System Wide Settings</h3>
                  <div className="field grid">
                    <div className="col-fixed" style={{ width: "120px" }}>
                      Keep history for (days):
                    </div>
                    <div className="col">
                      <InputNumber
                        disabled={!config?.current_user?.is_superuser}
                        style={{ width: "300px" }}
                        value={HistoryRetention / 1440}
                        className="p-inputtext-sm"
                        onValueChange={(e) => {
                          const days = e.value;
                          setHistoryRetention(days * 1440);
                          if (days < 1 || days > 30) {
                            setHistoryRetentionError("Value must be between 1 and 30 days");
                          } else {
                            setHistoryRetentionError("");
                          }
                        }}
                        placeholder="Days (1 - 30)"
                        mode="decimal"
                        inputMode="numeric"
                        showButtons
                      />
                      <div className="error-help">
                        <small className="p-error p-d-block">{HistoryRetentionError}</small>
                      </div>
                    </div>
                  </div>
                </div>
                <div className="col-6">
                  <h3>Database Specific Settings</h3>
                  <Dropdown className="p-inputtext-sm filter-field mr-auto mb-2" value={dbType}
                    options={dbTypes.filter((type) => ![types.ORACLE_PDB, types.HAPROXY, types.ALL].includes(type.value))}
                    onChange={(e) => {
                      const value = e.value;
                      setDbType(value);
                      setMonitorsByType(value);
                      localStorage.setItem("FilterDbType", value);
                      dispatch(AppActions.setDbType(value));
                   }} />
                  {dbType === types.ORACLE_CDB && (
                    <>
                      <div className="field grid">
                        <div className="col-fixed" style={{ width: "120px" }}>
                          CDBs path
                        </div>
                        <div className="col">
                          <InputText
                            disabled={!config?.current_user?.is_superuser}
                            style={{ width: "300px" }}
                            value={cdbPath}
                            className="p-inputtext-sm"
                            onChange={(e) => {
                              setCDBPath(e.target.value);
                              setCDBPathError(validator(constraints)("cdbPath", e.target.value));
                            }}
                            placeholder="CDBs path"
                          />
                          <div className="error-help">
                            <small className="p-error p-d-block">{cdbPathError}</small>
                          </div>
                        </div>
                      </div>

                      <div className="field grid">
                        <div className="col-fixed" style={{ width: "120px" }}>
                          PDBs path
                        </div>
                        <div className="col">
                          <InputText
                            disabled={!config?.current_user?.is_superuser}
                            style={{ width: "300px" }}
                            value={pdbPath}
                            className="p-inputtext-sm"
                            onChange={(e) => {
                              setPDBPath(e.target.value);
                              setPDBPathError(validator(constraints)("pdbPath", e.target.value));
                            }}
                            placeholder="PDBs path"
                          />
                          <div className="error-help">
                            <small className="p-error p-d-block">{pdbPathError}</small>
                          </div>
                        </div>
                      </div>

                      <div className="field grid">
                        <div className="col-fixed" style={{ width: "120px" }}>
                          Backups path
                        </div>
                        <div className="col">
                          <InputText
                            disabled={!config?.current_user?.is_superuser}
                            style={{ width: "300px" }}
                            value={oracleBackupsPath}
                            className="p-inputtext-sm"
                            onChange={(e) => {
                              setOracleBackupsPath(e.target.value);
                              setOracleBackupsError(
                                validator(constraints)("oracleBackupsPath", e.target.value)
                              );
                            }}
                            placeholder="Backups path"
                          />
                          <div className="error-help">
                            <small className="p-error p-d-block">{oracleBackupsError}</small>
                          </div>
                        </div>
                      </div>

                      <div className="field grid">
                        <div className="col-fixed" style={{ width: "120px" }}>
                          Exports path
                        </div>
                        <div className="col">
                          <InputText
                            disabled={!config?.current_user?.is_superuser}
                            style={{ width: "300px" }}
                            value={oracleExportsPath}
                            className="p-inputtext-sm"
                            onChange={(e) => {
                              setOracleExportsPath(e.target.value);
                              setOracleExportsError(
                                validator(constraints)("oracleExportsPath", e.target.value)
                              );
                            }}
                            placeholder="Exports path"
                          />
                          <div className="error-help">
                            <small className="p-error p-d-block">{oracleExportsError}</small>
                          </div>
                        </div>
                      </div>
                    </>
                  )}
                </div>

              </div>
              {dbType === types.POSTGRES && (
                  <div>
                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Backups path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={postgresBackupsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setPostgresBackupsPath(e.target.value);
                            setPostgresBackupsError(
                              validator(constraints)("postgresBackupsPath", e.target.value)
                            );
                          }}
                          placeholder="Backups path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{postgresBackupsError}</small>
                        </div>
                      </div>
                    </div>

                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Exports path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={postgresExportsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setPostgresExportsPath(e.target.value);
                            setPostgresExportsError(
                              validator(constraints)("postgresExportsPath", e.target.value)
                            );
                          }}
                          placeholder="Exports path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{postgresExportsError}</small>
                        </div>
                      </div>
                    </div>
                </div>
              )}

                {dbType === types.MONGO && (
                  <div>
                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Backups path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mongoBackupsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMongoBackupsPath(e.target.value);
                            setMongoBackupsError(
                              validator(constraints)("mongoBackupsPath", e.target.value)
                            );
                          }}
                          placeholder="Backups path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mongoBackupsError}</small>
                        </div>
                      </div>
                    </div>

                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Exports path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mongoExportsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMongoExportsPath(e.target.value);
                            setMongoExportsError(
                              validator(constraints)("mongoExportsPath", e.target.value)
                            );
                          }}
                          placeholder="Exports path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mongoExportsError}</small>
                        </div>
                      </div>
                    </div>
                  </div>
                )}

                {dbType === types.MARIADB && (
                  <div>
                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Backups path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mariadbBackupsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMariadbBackupsPath(e.target.value);
                            setMariadbBackupsError(
                              validator(constraints)("mariadbBackupsPath", e.target.value)
                            );
                          }}
                          placeholder="Backups path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mariadbBackupsError}</small>
                        </div>
                      </div>
                    </div>

                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Exports path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mariadbExportsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMariadbExportsPath(e.target.value);
                            setMariadbExportsError(
                              validator(constraints)("mariadbExportsPath", e.target.value)
                            );
                          }}
                          placeholder="Exports path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mariadbExportsError}</small>
                        </div>
                      </div>
                    </div>
                  </div>
                )}

                {dbType === types.MYSQL && (
                  <div>
                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Backups path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mysqlBackupsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMySQLBackupsPath(e.target.value);
                            setMySQLBackupsError(
                              validator(constraints)("mysqlBackupsPath", e.target.value)
                            );
                          }}
                          placeholder="Backups path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mysqlBackupsError}</small>
                        </div>
                      </div>
                    </div>

                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Exports path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mysqlExportsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMySQLExportsPath(e.target.value);
                            setMySQLExportsError(
                              validator(constraints)("mysqlExportsPath", e.target.value)
                            );
                          }}
                          placeholder="Exports path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mysqlExportsError}</small>
                        </div>
                      </div>
                    </div>
                  </div>
                )}
                {dbType === types.MSSQL && (
                  <div>
                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Backups path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mssqlBackupsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMsSQLBackupsPath(e.target.value);
                            setMsSQLBackupsError(
                              validator(constraints)("mssqlBackupsPath", e.target.value)
                            );
                          }}
                          placeholder="Backups path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mssqlBackupsError}</small>
                        </div>
                      </div>
                    </div>

                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Exports path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={mssqlExportsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setMsSQLExportsPath(e.target.value);
                            setMsSQLExportsError(
                              validator(constraints)("mssqlExportsPath", e.target.value)
                            );
                          }}
                          placeholder="Exports path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{mssqlExportsError}</small>
                        </div>
                      </div>
                    </div>
                  </div>
                )}
                {dbType === types.ELASTIC && (
                  <div>
                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Backups path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={elasticsearchBackupsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setElasticSearchBackupsPath(e.target.value);
                            setElasticSearchBackupsError(
                              validator(constraints)("elasticsearchBackupsPath", e.target.value)
                            );
                          }}
                          placeholder="Backups path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{elasticsearchBackupsError}</small>
                        </div>
                      </div>
                    </div>

                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Exports path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={elasticsearchExportsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setElasticSearchExportsPath(e.target.value);
                            setElasticSearchExportsError(
                              validator(constraints)("elasticsearchExportsPath", e.target.value)
                            );
                          }}
                          placeholder="Exports path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{elasticsearchExportsError}</small>
                        </div>
                      </div>
                    </div>
                  </div>
                )}
                {dbType === types.REDIS && (
                  <div>
                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Backups path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={redisBackupsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setRedisBackupsPath(e.target.value);
                            setRedisBackupsError(
                              validator(constraints)("redisBackupsPath", e.target.value)
                            );
                          }}
                          placeholder="Backups path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{redisBackupsError}</small>
                        </div>
                      </div>
                    </div>

                    <div className="field grid">
                      <div className="col-fixed" style={{ width: "120px" }}>
                        Exports path
                      </div>
                      <div className="col">
                        <InputText
                          disabled={!config?.current_user?.is_superuser}
                          style={{ width: "300px" }}
                          value={redisExportsPath}
                          className="p-inputtext-sm"
                          onChange={(e) => {
                            setRedisExportsPath(e.target.value);
                            setRedisExportsError(
                              validator(constraints)("redisExportsPath", e.target.value)
                            );
                          }}
                          placeholder="Exports path"
                        />
                        <div className="error-help">
                          <small className="p-error p-d-block">{redisExportsError}</small>
                        </div>
                      </div>
                    </div>
                  </div>
                )}
              <Button className="p-button-sm ml-auto mr-auto" label="Save" icon="pi pi-check" iconPos="right" onClick={onSystemUpdate} />
            </div>
          </TabPanel>
          <TabPanel header="Monitors" >
            <div className="flex justify-content-start">
              <Dropdown className="p-inputtext-sm filter-field time-period mr-2 mb-2" value={monitorsByType}
                options={dbTypes}
                placeholder="Select database type"
                onChange={(e) => {
                  const value = e.value
                  setDbType([types.HAPROXY, types.ORACLE_PDB, types.ALL].includes(value) ? types.ELASTIC : value);
                  setMonitorsByType(value);
                  localStorage.setItem("FilterDbType", value);
                  dispatch(AppActions.setDbType(value));
                }
                } />
              <span className="p-input-icon-right">
                <i className={monitorSearch ? "pi pi-times" : "field-collapsed"} onClick={() => setMonitorSearch("")} />
                <InputText className="p-inputtext-sm filter-field mb-2"
                  placeholder="Search"
                  value={monitorSearch}
                  onChange={(e) => {
                    setMonitorSearch(e.target.value);
                  }} />
              </span>

            </div>
            <MonitorsTable user={config?.current_user} monitorSearch={monitorSearch} monitorType={monitorsByType} onRowEdit={onMonitorEdit} onRowAdd={onMonitorAdd} />

          </TabPanel>
          <TabPanel header="Agents" >

            <NodeAgentsTable />

          </TabPanel>
          <TabPanel header="License Keys" >
            <KeysTable user = {config?.current_user} licenses={config?.licenses} summary={config?.license_summary} />
          </TabPanel>
          <TabPanel header="Integrated Services">

            <IntegratedServicesTable />

          </TabPanel>
        </TabView>
      </div>

      {showMonitorWizard ? <MonitoringWizard user = {config?.current_user} monitorType={monitorType} visible={showMonitorWizard} update={updateMonitor} data={currentItem} onHide={() => setShowMonitorWizard(false)} onOkClick={onConfirm} /> : null}

      {showMonitorTypeDialog ? <MonitorTypeDialog
        visible={showMonitorTypeDialog}
        onHide={() => setShowMonitorTypeDialog(false)}
        onOkClick={onSetMonitorType}
      /> : null
      }

      {showErrorDetails ?
        <ErrorDetailsDialog

          visible={showErrorDetails}
          onHide={() => {
            setShowErrorDetails(false)
            dispatch(ConfigActions.setErrorMessage(""));
            dispatch(SignupActions.setErrorMessage(""));
          }}
          message={monitorError?.message ? monitorError?.message : monitorError}


        />
        : null}

    </div>
  );
}
