import restore from '../../../assets/images/restore_pdb.svg';
import backup_pdb from '../../../assets/images/backup_pdb.svg';
import drop from '../../../assets/images/drop.svg';
import copy from '../../../assets/images/copy_pdb.svg';
import green_star from '../../../assets/images/green-star.png'
import red_star from '../../../assets/images/red-star.png'
import getStarByClass from '../../../util/getStarByClass';
import info from '../../../assets/images/info.svg';
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation } from "react-router-dom";
import { StatusActions } from "../../../store/status/reducer";
import { Fragment, useEffect, useState, useCallback, useRef } from "react";
import { Button } from 'primereact/button';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dropdown } from "primereact/dropdown";
import { TabView, TabPanel } from 'primereact/tabview'
import DetailsMenu from '../../../components/menus/detailsMenu';
import credentials from "../../../assets/images/credentials.svg";
import WarningDialog from '../../../dialogs/warningDialog';
import PerformanceCharts from '../../../components/performanceCharts';
import { ConfirmPopup } from 'primereact/confirmpopup';
import ElasticsearchClusterSnapshotDialog from '../../../dialogs/elasticsearch/elasticsearchClusterSnapshotDialog';
import RestoreElasticsearchCluster from '../../../dialogs/elasticsearch/restoreElasticsearchCluster';
import DropElasticsearchNodeDialog from '../../../dialogs/elasticsearch/dropElasticsearchNodeDialog';
import SwitchElasticsearchNodeDialog from '../../../dialogs/elasticsearch/switchElasticsearchNodeDialog';
import { useNavigate } from 'react-router-dom';
import { Tooltip } from 'primereact/tooltip';
import { clusterRoles, timePeriodTypes } from '../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../components/tables/nodeAlertsTable';
import createAPIcommand from '../../../util/createApiCommand';
import copyToClipboard from '../../../util/copyToClipboard';
import DatabaseUsersTable from '../../../components/tables/databaseUsersTable';
import { ConfigActions } from '../../../store/config/reducer';
import { InputText } from 'primereact/inputtext';
import StatsCard from '../../../components/statsCard';
import { searchFilter } from '../../monitoring';
import { ButtonGroup } from 'primereact/buttongroup';
import { AppActions } from '../../../store/app/reducer';
import ClusterSizing from '../../../components/changeClusterSizingBlock';
import { searchFilter as logSearch } from '../../log';
import LogsTable from '../../../components/tables/logsTable';
import LogFilesTable from '../../../components/tables/logFilesTable';


export default function ElasticsearchDetails(props) {

    const { id } = useParams();
    const { state } = useLocation();
    const navigate = useNavigate();

    const { elasticsearch_details, errorMessage } = useSelector((state) => state.status);
    const { accessToken, severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);

    const dispatch = useDispatch();

    const [upperSearch, setUpperSearch] = useState("");

    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [showSnapshot, setShowSnapshot] = useState(false);

    const [showWarningPopup, setShowWarningPopup] = useState(false);

    const [showDropElasticsearchNode, setShowDropElasticsearchNode] = useState(false);
    const [showSwitchover, setShowSwitchover] = useState(false);

    const [showRestore, setShowRestore] = useState(false);
    const [toggle, setToggle] = useState(0);
    const updateSchedule = useRef(null);
    const [pageIndex, setPageIndex] = useState(null);

    const { alerts } = useSelector((state) => state.monitoring);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [filteredUsers, setFilteredUsers] = useState(null);
    const [search, setSearch] = useState("");
    const { database_users } = useSelector((state) => state.config);
    const [popupAction, setPopupAction] = useState("stop");
    const [filteredLogs, setFilteredLogs] = useState(null);

    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);
    const { node_logs } = useSelector((state) => state.log);
    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }

        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];

    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);



    useEffect(() => {
        setFilteredUsers(null);
        if (elasticsearch_details?.node) dispatch(ConfigActions.getDatabaseUsers(elasticsearch_details?.node.id));
    }, [elasticsearch_details, dispatch])

    useEffect(() => {

        if (id) dispatch(StatusActions.setElasticsearchDetails(null));

        return () => dispatch(StatusActions.setElasticsearchDetails(null));

    }, [id, dispatch])

    useEffect(() => {

        if (database_users?.length) {

            let tempData = [];

            tempData = database_users;
            let query = search.toLowerCase();
            if (query) tempData = tempData.filter((u) => u.name?.toLowerCase().includes(query));
            setFilteredUsers(tempData);
        }

    }, [search, database_users])
    useEffect(() => {


        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;
        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            if (!showSeverityGreen || !showSeverityYellow || !showSeverityRed) tempData = severityFilter(tempData);
            setFilteredAlerts(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);

    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen])

    // useEffect(() => {

    //     if (elasticsearch_details?.history?.length > 0) {

    //         const query = upperSearch.toLowerCase().trim();
    //         if (!query) setFilteredLogs(elasticsearch_details.history);
    //         else setFilteredLogs(elasticsearch_details.history.filter((h) => h["@timestamp"].format("MMM DD,YYYY, hh:mm A").includes(query)
    //             || h["node"]["name"].toLowerCase().includes(query)
    //             || h.type.toLowerCase().includes(query)
    //             || h.message.toLowerCase().includes(query)
    //             || h.level.toLowerCase().includes(query)))
    //     }

    // },
    //     [upperSearch, elasticsearch_details?.history])

    useEffect(() => {
        if (id) {
            dispatch(StatusActions.getElasticsearchDetails(id));
        }
        return () => {
            dispatch(StatusActions.setElasticsearchDetails(null));
            dispatch(LayoutActions.getNodes());
        }
    }, [id, dispatch])

    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);

    }, [state?.targetTab])


    const onRestore = (data) => {
        dispatch(LayoutActions.restoreElasticsearchCluster(data, setShowRestore));

    }

    const onShowSnapshot = () => {

        setPageIndex(null);

        setShowSnapshot(true);

    }


    const onSetSchedule = () => {
        if (elasticsearch_details?.node?.ind_deleted) showWarningMessage({ "text": `${elasticsearch_details?.node?.name} is unavailable.`, 'status': elasticsearch_details?.node?.status });
        else {
            setPageIndex(1);
            updateSchedule.current = elasticsearch_details?.backups?.filter((b) => b.is_sched === true)?.[0];
            setShowSnapshot(true);
        }

    }

    const onShowPopup = async (target) => {
        switch (target) {

            case "stop-down":
                setPopupAction("stop");
                await copyToClipboard(createAPIcommand({ "node_id": elasticsearch_details?.node?.name }, accessToken, "stop_elasticsearch_node/"))
                break;
            case "start-down":
                setPopupAction(elasticsearch_details?.node?.ind_started ? "restart" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": elasticsearch_details?.node?.name }, accessToken, "start_elasticsearch_node/"))
                break;
            case "start-stop":
                setPopupAction(elasticsearch_details?.node?.ind_started ? "stop" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": elasticsearch_details?.node?.name }, accessToken, elasticsearch_details?.node?.ind_started ? "stop_elasticsearch_node/" : "start_elasticsearch_node/"));
                break;
            default:
                break;


        }
        setShowWarningPopup(target);

    }

    const onPopupAccept = () => {

        switch (showWarningPopup) {

            case "stop-down":
                onStopElasticsearchNode();
                break;
            case "start-down":
                onStartElasticsearchNode();
                break;
            case "start-stop":
                elasticsearch_details?.node?.ind_started ? onStopElasticsearchNode() : onStartElasticsearchNode();
                break;
            default:
                break;

        }

    }

    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onStopElasticsearchNode = () => dispatch(LayoutActions.stopElasticsearchNode({ "node_id": elasticsearch_details?.node?.id }, elasticsearch_details?.node));


    const onStartElasticsearchNode = () => dispatch(LayoutActions.startElasticsearchNode({ "node_id": elasticsearch_details?.node?.id }, elasticsearch_details?.node));


    // const dateBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{moment(item['@timestamp']).format("MMM DD,YYYY, hh:mm A")}</div>);
    // }

    // const typeBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item['type']}</div>);
    // }

    // const levelBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item['level']}</div>);
    // }

    // const nodeBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item["node"]["name"]}</div>);
    // }

    // const messageBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item["message"]}</div>);
    // }

    const onSnapshotElasticsearchCluster = (data) => {
        dispatch(LayoutActions.snapshotElasticsearchCluster(data, setShowSnapshot));
    }

    const onDropElasticsearchNode = (data) => {

        dispatch(LayoutActions.dropElasticsearchNode(data, setShowDropElasticsearchNode));

    }

    const onSwitchover = (data) => {
        dispatch(LayoutActions.switchElasticsearchNode(data, setShowSwitchover));
    }

    const attachPopup = (target) => elasticsearch_details?.node?.ind_deleted ? showWarningMessage({ "text": `${elasticsearch_details?.node?.name} is unavailable.`, 'status': elasticsearch_details?.node?.status }) : onShowPopup(target);

    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">

                <TabView activeIndex={toggle} onTabChange={(e) => {
                    if (e.index < 4) setToggle(e.index);
                }}>
                    <TabPanel header="Dashboard">

                        <div className='grid flex-row'>
                            <div className='col-3 flex align-items-between flex-column' >
                                <h2 >
                                    {elasticsearch_details?.node?.cluster_role} {elasticsearch_details?.node?.name}
                                </h2>
                                <div>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Node is {elasticsearch_details?.node?.ind_started ? 'up' : "down"}
                                                </td>
                                                <td>
                                                    <Button
                                                        className="p-button-sm details-button"
                                                        disabled={!elasticsearch_details?.node?.ind_started && !errorMessage}
                                                        id="start-stop"
                                                        icon={elasticsearch_details?.node?.ind_started ? "pi pi-stop-circle" : "pi pi-play"}
                                                        style={{ minWidth: "53px", minHeight: "39px" }}
                                                        iconPos="right"
                                                        onClick={() => attachPopup("start-stop")}
                                                    />
                                                </td>
                                                <td>
                                                    <img src={elasticsearch_details?.node?.ind_started ? green_star : red_star} alt="star" className="avail-star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {elasticsearch_details?.ind_error ? "Errors in cluster" : elasticsearch_details?.ind_error === false ? "Cluster is OK" : <div> <i className="pi pi-spin pi-spinner" /> Connecting to node... </div>}
                                                </td>
                                                <td>
                                                    <Button onClick={() => navigate('/layout', { state: { node_id: elasticsearch_details?.node?.ex_params?.elasticsearch_master_id } })} disabled={!elasticsearch_details?.node} className='p-button-sm details-button'><img alt="info" width={16} height={16} src={info} /></Button>
                                                </td>
                                                <td>
                                                    <img className='status-star' src={(elasticsearch_details?.ind_error === undefined || elasticsearch_details?.ind_error === true) ? red_star : green_star} alt="star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {elasticsearch_details?.auto_backup?.status ? elasticsearch_details.auto_backup.status : ""}
                                                </td>
                                                <td>
                                                    {<Button onClick={onSetSchedule} icon="pi pi-calendar-times" disabled={!elasticsearch_details?.node?.ind_started && !errorMessage} className='p-button-sm details-button update-schedule-button'></Button>}
                                                </td>
                                                <td>
                                                    <img className='set-schedule-star' src={getStarByClass(elasticsearch_details?.auto_backup?.class)} alt="star"></img>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <div>
                                    <h3>
                                        General info
                                    </h3 >
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td> {elasticsearch_details?.node?.name}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    DB ID
                                                </td>
                                                <td> {elasticsearch_details?.db_id}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Version
                                                </td>
                                                <td> {elasticsearch_details?.version}</td>
                                            </tr>

                                            <tr>
                                                <td>
                                                    Roles
                                                </td>
                                                <td> {elasticsearch_details?.roles}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Uptime
                                                </td>
                                                <td> {elasticsearch_details?.uptime}</td>
                                            </tr>

                                            <tr>
                                                <td>
                                                    Size
                                                </td>
                                                <td> {elasticsearch_details?.display_size}</td>
                                            </tr>

                                        </tbody>
                                    </table>
                                </div>
                                <ClusterSizing node={elasticsearch_details?.node} />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>
                                <StatsCard details={elasticsearch_details}></StatsCard>
                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={elasticsearch_details?.node?.cpuUsages}
                                    memUsages={elasticsearch_details?.node?.memUsages}
                                />

                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>
                                <div className='col-9'>

                                    <span className="p-buttonset">
                                        <Button disabled={!elasticsearch_details?.node?.ind_started && !errorMessage} className="p-button-sm details-button up" label="Replace">
                                            < img alt="copy" width={16} height={16} src={copy} />
                                        </Button>
                                        < Button className="p-button-sm details-button up" disabled={!elasticsearch_details?.node?.ind_started && !errorMessage} label="Snapshot" onClick={() => elasticsearch_details?.node?.ind_deleted ? showWarningMessage({ "text": `${elasticsearch_details?.node?.name} is unavailable.`, 'status': elasticsearch_details?.node?.status }) : onShowSnapshot()}>
                                            < img alt="snapshot" width={16} height={16} src={backup_pdb} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={(!elasticsearch_details?.node?.ind_started && !errorMessage) || elasticsearch_details?.node?.cluster_role === clusterRoles.STANDBY} label="Restore" onClick={() => elasticsearch_details?.node?.ind_deleted ? showWarningMessage({ "text": `${elasticsearch_details?.node?.name} is unavailable.`, 'status': elasticsearch_details?.node?.status }) : setShowRestore(true)}>
                                            < img alt="restore" width={16} height={16} src={restore} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" icon="pi pi-arrows-h"
                                            iconPos="right"
                                            disabled={(elasticsearch_details?.node?.cluster_role !== clusterRoles.PRIMARY || (!elasticsearch_details?.node?.ind_started && !errorMessage)) || (elasticsearch_details?.nodes?.filter((n) => n?.severity === "green" && n?.ex_params?.roles?.indexOf("master") !== -1 && n?.ex_params?.roles?.indexOf("voting_only") === -1).length === 0)}
                                            label="Switchover"
                                            onClick={() => setShowSwitchover(true)}
                                        />

                                        <Button disabled={(!elasticsearch_details?.node?.ind_started && !errorMessage)} className="p-button-sm details-button up" label="Credentials">
                                            <img alt="credentials" width={16} height={16} src={credentials} ></img>
                                        </Button>
                                    </span>

                                    <span className="p-buttonset down">
                                        <Button disabled={errorMessage || !elasticsearch_details?.node?.ind_started} icon="pi pi-stop-circle" iconPos="right" className="p-button-sm details-button" id="stop-down" label="Stop" onClick={() => attachPopup("stop-down")} />
                                        <Button id="start-down" disabled={!elasticsearch_details} icon={`pi pi-${elasticsearch_details?.node?.ind_started ? "replay" : "play"}`} iconPos="right" className="p-button-sm details-button" label={elasticsearch_details?.node?.ind_started ? "Restart" : "Start"} onClick={() => attachPopup("start-down")}>
                                        </Button>
                                        <Button disabled={!elasticsearch_details?.node?.ind_started && !errorMessage} className="p-button-sm details-button" label="Drop" onClick={() => elasticsearch_details?.node?.ind_deleted ? showWarningMessage({ "text": `${elasticsearch_details?.node?.name} is unavailable.`, 'status': elasticsearch_details?.node?.status }) : setShowDropElasticsearchNode(true)}>
                                            < img alt="drop" width={16} height={16} src={drop} />
                                        </Button>
                                    </span>


                                </div>

                            </div>
                        </div>

                    </TabPanel>
                    <TabPanel header="Activity Logs">
                        <span className="p-input-icon-right">
                            <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                setUpperSearch("");
                            }} ></i>
                            <InputText className="p-inputtext-sm filter-field mr-2- mb-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                        </span>

                        <div className='flex'>

                            <Dropdown className="p-inputtext-sm mb-2 mt-2 filter-field time-period" value={timePeriodType}
                                options={timePeriodTypes}
                                onChange={(e) => {
                                    setTimePeriodType(e.value);
                                    setMonitorInterval(intervalByTimedelta(e.value));
                                }}
                            />

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <LogsTable filteredLogs={filteredLogs}
                            timePeriodType={timePeriodType}
                            node_id={id}
                        />

                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} paginator rows={100} alwaysShowPaginator={false} >
                            <Column className='history-column' header="Date" field='@timestamp' body={dateBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Type" field='type' body={typeBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Level" field='level' sortable body={levelBodyTemplate}></Column>
                            <Column className='history-column' header="Node" field='node.name' body={nodeBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Message" field='message' body={messageBodyTemplate} sortable></Column>
                        </DataTable> */}

                    </TabPanel>
                    <TabPanel header="Database Logs">
                        <LogFilesTable node_id={id} />
                    </TabPanel>

                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <Dropdown className="p-inputtext-sm mb-2 mt-2 filter-field time-period" value={timePeriodType}
                                options={timePeriodTypes}
                                onChange={(e) => {
                                    setTimePeriodType(e.value);
                                    setMonitorInterval(intervalByTimedelta(e.value));
                                }}
                            />

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>
                        <NodeAlertsTable node={elasticsearch_details?.node} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />
                    </TabPanel>

                    <TabPanel header="Users">

                        <div className="flex justify-content-start">
                            <span className="p-input-icon-right">
                                <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => setSearch("")} />
                                <InputText className="p-inputtext-sm filter-field mb-2"
                                    placeholder="Search"
                                    value={search}
                                    onChange={(e) => {
                                        setSearch(e.target.value);
                                    }} />
                            </span>

                        </div>
                        <DatabaseUsersTable node={elasticsearch_details?.node} users={filteredUsers} />


                    </TabPanel>

                    <TabPanel headerClassName='filters-header' header={<div className='flex'>


                        {toggle < 4 ? <Dropdown
                            className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period raised-dropdown" value={timePeriodType}
                            options={timePeriodTypes}
                            onChange={(e) => {
                                setTimePeriodType(e.value);
                                setMonitorInterval(intervalByTimedelta(e.value));
                            }}
                        /> : null}



                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => {
                                dispatch(StatusActions.setElasticsearchDetails(null));
                                dispatch(StatusActions.getElasticsearchDetails(id));
                            }}
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>

                </TabView>

                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}

                {(showDropElasticsearchNode && elasticsearch_details) ? <DropElasticsearchNodeDialog
                    node={elasticsearch_details?.node}
                    nodes_count={elasticsearch_details?.nodes_count}
                    visible={showDropElasticsearchNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowDropElasticsearchNode(false)
                    }}
                    onOkClick={onDropElasticsearchNode}
                />

                    : null}

                {(showSnapshot && elasticsearch_details) ? <ElasticsearchClusterSnapshotDialog
                    node={elasticsearch_details?.node}
                    visible={showSnapshot}
                    updateSchedule={updateSchedule.current}
                    pageIndex={pageIndex}
                    backups={elasticsearch_details?.snapshots}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowSnapshot(false)
                    }}
                    onOkClick={onSnapshotElasticsearchCluster}
                />
                    : null}

                {(showRestore && elasticsearch_details) ? <RestoreElasticsearchCluster
                    data={elasticsearch_details}
                    visible={showRestore}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestore(false)
                    }}
                    onOkClick={onRestore}
                /> : null}
            </div>
            {showSwitchover ? <SwitchElasticsearchNodeDialog
                nodes={elasticsearch_details?.nodes?.filter((n) => n?.severity === "green" && n?.ex_params?.roles.indexOf("master") !== -1 && n?.ind_deleted === false && n?.ex_params?.roles.indexOf("voting_only") === -1)}
                visible={showSwitchover}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowSwitchover(false)
                }}
                onOkClick={onSwitchover}
            />
                : null}
            <ConfirmPopup
                icon="pi pi-exclamation-triangle"
                className='multiline'
                target={document.getElementById(showWarningPopup)}
                visible={Boolean(showWarningPopup)}
                onHide={() => setShowWarningPopup(false)}
                accept={onPopupAccept}
                message={((showWarningPopup === "start-stop" || showWarningPopup === "stop-down") && elasticsearch_details?.node?.cluster_role === clusterRoles.PRIMARY && elasticsearch_details?.node?.ind_started && elasticsearch_details?.nodes?.filter((n) => n.severity === "green" && n.ex_params.roles.indexOf("master") !== -1 && n.ind_deleted === false && n?.ex_params.roles.indexOf("voting_only") === -1).length === 0) ? "Cluster has no reserve master.\nStopping this node will break the cluster operation.\nAre you sure?" : `Are you sure you want to ${popupAction} the node?`} />

            {elasticsearch_details && <Tooltip target=".set-schedule-star" content={elasticsearch_details?.auto_backup?.detail}></Tooltip>}
            {elasticsearch_details && <Tooltip target=".avail-star" content={elasticsearch_details?.node?.ind_started ? "Database is running and accepting connections" : "Database is not running"}></Tooltip>}
            {elasticsearch_details && <Tooltip target=".status-star" content={(elasticsearch_details?.ind_error === undefined || elasticsearch_details?.ind_error === true) ? "Some nodes of cluster are down/inaccessible" : "All nodes of cluster are accessible"}></Tooltip>}
        </Fragment>
    )


}
