import { useCallback, useEffect, useRef, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import moment from "moment";
import Menu from "../../components/menus/menu";
import { LayoutActions } from "../../store/layout/reducer";
import AddCDBDialog from "../../dialogs/oracle/addCDBDialog";
import OracleLayoutItem from "../../components/layout/oracleLayoutItem";
import MsSQLLayoutItem from "../../components/layout/mssqlLayoutItem";
import NodeTypeDialog from "../../dialogs/nodeTypeDialog";
import LayoutFilter from "../../components/filters/layoutFilter";
import { Button } from "primereact/button";
import PgLayoutItem from "../../components/layout/pgLayoutItem";
import AddPostgresPrimaryDialog from "../../dialogs/postgres/addPostgresPrimary";
import AddMongoMasterDialog from "../../dialogs/mongo/addMongoMasterDialog";
import WarningDialog from "../../dialogs/warningDialog";
import MongoLayoutItem from "../../components/layout/mongoLayoutItem";
import AddMariaDBMasterDialog from "../../dialogs/mariadb/addMariaDBMasterDialog";
import MariaDbLayoutItem from "../../components/layout/mariadbLayoutItem";
import AddElacticSearchMasterDialog from "../../dialogs/elasticsearch/addElasticSearchMaster";
import ElasticsearchLayoutItem from "../../components/layout/elasticsearchLayoutItem";
import MySQLLayoutItem from "../../components/layout/mysqlLayoutItem";
import AddMySQLMasterDialog from "../../dialogs/mysql/addMySQLMasterDialog";
import AddMsSQLMasterDialog from "../../dialogs/mssql/addMsSQLMasterDialog";
import CreateRedisStandaloneDialog from "../../dialogs/redis/createRedisStandalone";
import { useLocation } from "react-router-dom";
import { types, clusterRoles, loadingMsg, emptyMsg } from "../../config";
import AddRedisClusterDialog from "../../dialogs/redis/addRedisCluster";
import RedisLayoutItem from "../../components/layout/redisLayoutItem";
import InstallCDBDialog from "../../dialogs/oracle/installCDBDialog";
import AddCDBMethodDialog from "../../dialogs/oracle/addCDBMethodDialog";
import MongoShardedLayoutItem from "../../components/layout/mongoShardedLayoutItem";
import AddMongoShardedClusterDialog from "../../dialogs/mongo/addMongoShardedCluster";
import { AppActions } from "../../store/app/reducer";
import { ConfigActions } from "../../store/config/reducer";
import SelectServiceTypeDialog from "../../dialogs/selectServiceTypeDialog";
import PgK8SLayoutItem from "../../components/layout/pgK8sLayoutItem";
import MongoK8SLayoutItem from "../../components/layout/mongoK8sLayoutItem";
import { layoutSortOptions } from "../../config";
const Layout = () => {
    const dispatch = useDispatch();
    const { user, severity, db_type, _search } = useSelector((state) => state.app);
    const { nodes, isLoading } = useSelector((state) => state.layout);
    const { state } = useLocation();
    const [summary, setSummary] = useState(false);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [severityGreenTotal, setSeverityGreenTotal] = useState(0);
    const [severityYellowTotal, setSeverityYellowTotal] = useState(0);
    const [severityRedTotal, setSeverityRedTotal] = useState(0);
    const [filteredNodes, setFilteredNodes] = useState(null);
    const [search, setSearch] = useState(_search);
    const [dbType, setDbType] = useState(db_type);
    const [showRegisterCDBDialog, setShowRegisterCDBDialog] = useState(false);
    const [showAddPostgresPrimary, setShowAddPostgresPrimary] = useState(false);
    const [showAddMongoMaster, setShowAddMongoMaster] = useState(false);
    const [showAddMySQLMaster, setShowAddMySQLMaster] = useState(false);
    const [showAddMsSQLMaster, setShowAddMsSQLMaster] = useState(false);
    const [showAddMariaDBMaster, setShowAddMariaDBMaster] = useState(false);
    const [showAddRedisStandalone, setShowAddRedisStandalone] = useState(false);
    const [showAddRedisCluster, setShowAddRedisCluster] = useState(false);
    const [showAddMongoShardedCluster, setShowAddMongoShardedCluster] = useState(false);
    const [showAddElasticSearchMaster, setShowAddElasticSearchMaster] = useState(false);
    const [showNodeTypeDialog, setShowNodeTypeDialog] = useState(false);
    const [warningMessage, setWarningMessage] = useState("");
    const [showWarning, setShowWarning] = useState(false);
    const [showInstallCDBDialog, setShowInstallCDBDialog] = useState(false);
    const [showAddCDBMethodDialog, setShowAddCDBMethodDialog] = useState(false);
    const selected = useRef(null);
    const [selectedType, setSelectedType] = useState("");
    const [selectServiceType, setSelectServiceType] = useState(false);
    const { integrated_services } = useSelector((state) => state.config);
    const defaultCreator = useRef(null);
    const [sortOption, setSortOption] = useState(layoutSortOptions.NEWEST);
    const severityFilter = useCallback((data) => {
        let severityList = [];
        if (showSeverityGreen)
            severityList.push("green");
        if (showSeverityYellow)
            severityList.push("yellow");
        if (showSeverityRed)
            severityList.push("red");
        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity) >= 0 || severityList.indexOf(n.ex_params?.severity) >= 0) : [];
    }, [showSeverityGreen, showSeverityYellow, showSeverityRed])
    useEffect(() => {

        function sortNodes(nodes, sortBy) {
            const severityOrder = { red: 1, yellow: 2, green: 3 };

            function getDeepestCreatedDate(node) {
                let dates = [new Date(node.created_date)];
                if (node.nodes && node.nodes.length) {
                    dates.push(...node.nodes.map(getDeepestCreatedDate));
                }
                return new Date(Math.max(...dates.map(date => date.getTime())));
            }

            function compareNodes(a, b) {
                switch (sortBy) {
                    case layoutSortOptions.NEWEST: {
                        const dateA = getDeepestCreatedDate(a);
                        const dateB = getDeepestCreatedDate(b);
                        return dateB - dateA; // Newest first
                    }
                    case layoutSortOptions.SEVERITY: {
                        const severityA = severityOrder[a.severity] || 4;
                        const severityB = severityOrder[b.severity] || 4;
                        return severityA - severityB; // "red" (1) first, "yellow" (2), "green" (3)
                    }
                    case layoutSortOptions.CHILD_COUNT: {
                        const childrenA = a.nodes ? a.nodes.length : 0;
                        const childrenB = b.nodes ? b.nodes.length : 0;
                        return childrenB - childrenA; // More children first
                    }
                    default:
                        return 0;
                }
            }

            function deepSort(nodes) {
                return nodes
                    .map(node => ({
                        ...node,
                        nodes: node.nodes ? deepSort(node.nodes) : []
                    }))
                    .sort(compareNodes);
            }

            return deepSort(nodes);
        }


        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;
        if (!nodes) dispatch(LayoutActions.getNodes());
        let tempData = nodes;
        if (nodes?.length > 0) {
            nodes.forEach((n) => {
                switch (n?.severity) {
                    case "green":
                        ++tempSeverityGreen;
                        break;
                    case "yellow":
                        ++tempSeverityYellow;
                        break;
                    case "red":
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
            if (search)
                tempData = searchFilter(tempData, search);
            if (dbType)
                tempData = dbFilter(tempData, dbType);
            tempData = severityFilter(tempData);
            if (state?.node_id)
                tempData = nodes.filter((n) => n.id === state.node_id);
            if (state?.cluster_id)
                tempData = nodes.filter((n) => n.cluster === state.cluster_id);

            let cdbs = tempData.filter((n) => n.type === types.ORACLE_CDB);
            let cdb_count = cdbs.length;
            let cdbUp = cdbs.filter((n => n.ind_started === true)).length;
            let cdbDown = cdbs.length - cdbUp;
            let pdbCount = 0, pdbUp = 0, pdbDown = 0;
            cdbs?.forEach((n) => n.nodes?.forEach((p) => {
                pdbCount++;
                if (p.ind_started) pdbUp++;
                else pdbDown++;
            }));
            let primary = tempData.filter((n) => n.type === types.POSTGRES);
            let primaryUp = primary.filter((n => n.ind_started === true)).length;
            let primaryDown = primary.length - primaryUp;
            let standbyCount = 0, standbyUp = 0, standbyDown = 0;

            primary?.forEach((n) => n.nodes?.forEach((s) => {
                standbyCount++;
                s.ind_started ? standbyUp++ : standbyDown++;
            }));
            let primaryMaria = tempData.filter((n) => n.type === types.MARIADB);
            let primaryMariaUp = primaryMaria.filter((n => n.ind_started === true)).length;
            let primaryMariaDown = primaryMaria.length - primaryMariaUp;
            let standbyMariaCount = 0, standbyMariaUp = 0, standbyMariaDown = 0;
            primaryMaria?.forEach((n) => n.nodes?.forEach((s) => {
                standbyMariaCount++;
                s.ind_started ? standbyMariaUp++ : standbyMariaDown++;
            }));

            let primaryMySQL = tempData.filter((n) => n.type === types.MYSQL);
            let primaryMySQLUp = primaryMySQL.filter((n => n.ind_started === true)).length;
            let primaryMySQLDown = primaryMySQL.length - primaryMySQLUp;
            let standbyMySQLCount = 0, standbyMySQLUp = 0, standbyMySQLDown = 0;
            primaryMySQL?.forEach((n) => n.nodes?.forEach((s) => {
                standbyMySQLCount++;
                s.ind_started ? standbyMySQLUp++ : standbyMySQLDown++;
            }));
            let primaryMongo = tempData.filter((n) => n.type === types.MONGO);
            let primaryMongoUp = primaryMongo.filter((n => n.ind_started === true)).length;
            let primaryMongoDown = primaryMongo.length - primaryMongoUp;
            let standbyMongoCount = 0, standbyMongoUp = 0, standbyMongoDown = 0, arbiterMongoCount = 0, arbiterMongoUp = 0, arbiterMongoDown = 0;

            primaryMongo?.forEach((n) => n.nodes?.forEach((n) => {
                if (n.cluster_role === clusterRoles.STANDBY) {
                    standbyMongoCount++;
                    n.ind_started ? standbyMongoUp++ : standbyMongoDown++
                }
                else if (n.cluster_role === clusterRoles.ARBITER) {
                    arbiterMongoCount++;
                    n.ind_started ? arbiterMongoUp++ : arbiterMongoDown++
                }
            }));

            setSummary({
                "pdb_count": pdbCount,
                "cdb_count": cdb_count,
                "cdb_up": cdbUp,
                "cdb_down": cdbDown,
                "pdb_up": pdbUp,
                "pdb_down": pdbDown,
                "primary_count": primary.length,
                "primary_up": primaryUp,
                "primary_down": primaryDown,
                "standby_count": standbyCount,
                "standby_up": standbyUp,
                "standby_down": standbyDown,
                "primary_mongo_count": primaryMongo.length,
                "primary_mongo_up": primaryMongoUp,
                "primary_mongo_down": primaryMongoDown,
                "standby_mongo_count": standbyMongoCount,
                "standby_mongo_up": standbyMongoUp,
                "standby_mongo_down": standbyMongoDown,
                "arbiter_mongo_count": arbiterMongoCount,
                "arbiter_mongo_up": arbiterMongoUp,
                "primary_mysql_count": primaryMySQL.length,
                "primary_mysql_up": primaryMySQLUp,
                "primary_mysql_down": primaryMySQLDown,
                "standby_mysql_count": standbyMySQLCount,
                "standby_mysql_up": standbyMySQLUp,
                "standby_mysql_down": standbyMySQLDown,
                "arbiter_mongo_down": arbiterMongoDown,
                "primary_mariadb_count": primaryMaria.length,
                "primary_mariadb_up": primaryMariaUp,
                "primary_mariadb_down": primaryMariaDown,
                "standby_mariadb_count": standbyMariaCount,
                "standby_mariadb_up": standbyMariaUp,
                "standby_mariadb_down": standbyMariaDown,
            });

            setFilteredNodes(sortNodes(tempData, sortOption));
        }
        else setFilteredNodes(tempData);
        setSeverityGreenTotal(tempSeverityGreen);
        setSeverityYellowTotal(tempSeverityYellow);
        setSeverityRedTotal(tempSeverityRed);
    }, [dbType, nodes, search, severityFilter, sortOption, state, dispatch]);
    useEffect(() => {

        dispatch(ConfigActions.getIntegratedServices());

    }, [nodes, dispatch])

    const onFilterDbChange = (dbType, search, timePeriod, severity) => {
        setDbType(dbType);
        dispatch(AppActions.setDbType(dbType));
        let tempData = nodes;
        if (dbType)
            tempData = dbFilter(tempData, dbType);
        if (search !== undefined && search)
            tempData = searchFilter(tempData, search);
        if (severity !== undefined && severity)
            tempData = severityFilter(tempData);
        setFilteredNodes(tempData);
    }
    const onSearchChange = (search, dbType, severity) => {
        setSearch(search);
        dispatch(AppActions.setSearch(search));
        let tempData = nodes;
        if (search)
            tempData = searchFilter(tempData, search);
        if (dbType !== undefined && dbType)
            tempData = dbFilter(tempData, dbType);
        if (severity !== undefined && severity)
            tempData = severityFilter(tempData);
        setFilteredNodes(tempData);
    }
    const searchFilter = (data, search) => {
        return data?.filter ? data?.filter((n) => {
            if (n.what?.toLowerCase()?.includes(search) || n.location?.toLowerCase()?.includes(search) || n.cluster_role?.toLowerCase()?.includes(search) ||
                moment(n.created_date).format("MMM DD,YYYY, hh:mm A").includes(search))
                return true;
            if (n.status && n.status.includes(search))
                return true;
            return false;
        }) : [];
    }
    const dbFilter = (data, dbType) => {
        return dbType === types.ALL ? data : data?.filter ? data?.filter((n) => n.type?.includes(dbType)) : [];
    }
    const onFilterSeverityChange = (severity, search, dbType, timePeriod) => {
        let severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        if (severity === "green") {

            severityChoices.green = !severityChoices.green;
            setShowSeverityGreen(severityChoices.green);

        }
        else if (severity === "yellow") {


            severityChoices.yellow = !severityChoices.yellow;
            setShowSeverityYellow(severityChoices.yellow);

        }

        else if (severity === "red") {
            severityChoices.red = !severityChoices.red;
            setShowSeverityRed(severityChoices.red);
        }
        dispatch(AppActions.setSeverity(severityChoices));
    }
    const onAddNodeClick = () => {
        setShowNodeTypeDialog(true);
    }
    const onAddRedisCluster = (data) => {
        dispatch(LayoutActions.addRedisCluster(data, setShowAddRedisCluster))
    }
    const onAddMongoShardedCluster = (data) => {
        dispatch(LayoutActions.addMongoShardedCluster(data, setShowAddMongoShardedCluster))
    }
    const showWarningMessage = (data) => {
        setWarningMessage(data);
        setShowWarning(true);
    }

    const onSetNodeType = (type, redisType, mongoType) => {
        setShowNodeTypeDialog(false);
        setSelectedType(type);
        let warningMessage = {
            "text": `User ${user?.data?.username} has no privileges for this action`,
            "status": "not allowed"
        }
        switch (type) {
            case types.ORACLE_CDB:
            case types.ORACLE_PDB:
                defaultCreator.current = setShowAddCDBMethodDialog
                if (user?.is_superuser === false) {
                    if (user?.privileges?.oracle_pdb_quota < 1) {
                        showWarningMessage(warningMessage);
                        return;
                    }
                    else {
                        warningMessage.status = `User has exceed quota of ${user?.privileges?.oracle_pdb_quota}  Oracle clusters`
                        dispatch(LayoutActions.isQuotaExceed(
                            {
                                "allow": setShowAddCDBMethodDialog,
                                "reject": showWarningMessage,
                                "warning": warningMessage,
                                "request":
                                {
                                    "quota": user?.privileges?.oracle_pdb_quota,
                                    "obj_type": types.ORACLE_PDB
                                }
                            }));
                    }
                }
                else setShowAddCDBMethodDialog(true);
                break;
            case types.POSTGRES:
                defaultCreator.current = setShowAddPostgresPrimary
                if (user?.is_superuser === false) {
                    if (user?.privileges?.postgres_quota < 1) {
                        showWarningMessage(warningMessage);
                        return;
                    }
                    else {
                        warningMessage.status = `User has exceed quota of ${user?.privileges?.postgres_quota}  PostgreSQL clusters`
                        dispatch(LayoutActions.isQuotaExceed(
                            {
                                "allow": setSelectServiceType,
                                "reject": showWarningMessage,
                                "warning": warningMessage,
                                "request":
                                {
                                    "quota": user?.privileges?.postgres_quota,
                                    "obj_type": types.POSTGRES
                                }
                            }))
                    }
                }
                else setSelectServiceType(true);
                break;
            case types.MONGO:
                const callback = mongoType === 'sharded' ? setShowAddMongoShardedCluster : setShowAddMongoMaster;
                defaultCreator.current = callback
                if (user?.is_superuser === false) {
                    if (user?.privileges?.mongo_quota < 1) {
                        showWarningMessage(warningMessage);
                        return;
                    }
                    else {
                        warningMessage.status = `User has exceed quota of ${user?.privileges?.mongo_quota}  MongoDB clusters`
                        dispatch(LayoutActions.isQuotaExceed(
                            {
                                "allow": setSelectServiceType,
                                "reject": showWarningMessage,
                                "warning": warningMessage,
                                "request":
                                {
                                    "quota": user?.privileges?.mongo_quota,
                                    "obj_type": types.MONGO
                                }
                            }))
                    }
                }
                else setSelectServiceType(true);
                break;
            case types.MARIADB:
                defaultCreator.current = setShowAddMariaDBMaster
                if (user?.is_superuser === false) {
                    if (user?.privileges?.mariadb_quota < 1) {
                        showWarningMessage(warningMessage);
                        return;
                    }
                    else {
                        warningMessage.status = `User has exceed quota of ${user?.privileges?.mariadb_quota}  MariaDB clusters`
                        dispatch(LayoutActions.isQuotaExceed(
                            {
                                "allow": setSelectServiceType,
                                "reject": showWarningMessage,
                                "warning": warningMessage,
                                "request":
                                {
                                    "quota": user?.privileges?.mariadb_quota,
                                    "obj_type": types.MARIADB
                                }
                            }))
                    }
                }
                else setSelectServiceType(true);
                break;
            case types.MYSQL:
                defaultCreator.current = setShowAddMySQLMaster
                if (user?.is_superuser === false) {
                    if (user?.privileges?.mysql_quota < 1) {
                        showWarningMessage(warningMessage);
                        return;
                    }
                    else {
                        warningMessage.status = `User has exceed quota of ${user?.privileges?.mysql_quota}  MySQL clusters`
                        dispatch(LayoutActions.isQuotaExceed(
                            {
                                "allow": setSelectServiceType,
                                "reject": showWarningMessage,
                                "warning": warningMessage,
                                "request":
                                {
                                    "quota": user?.privileges?.mysql_quota,
                                    "obj_type": types.MYSQL
                                }
                            }))
                    }
                }
                else setSelectServiceType(true);
                break;
            case types.ELASTIC:
                defaultCreator.current = setShowAddElasticSearchMaster
                if (user?.is_superuser === false) {
                    if (user?.privileges?.elasticsearch_quota < 1) {
                        showWarningMessage(warningMessage);
                        return;
                    }
                    else {
                        warningMessage.status = `User has exceed quota of ${user?.privileges?.elasticsearch_quota}  Elasticsearch clusters`
                        dispatch(LayoutActions.isQuotaExceed(
                            {
                                "allow": setSelectServiceType,
                                "reject": showWarningMessage,
                                "warning": warningMessage,
                                "request":
                                {
                                    "quota": user?.privileges?.elasticsearch_quota,
                                    "obj_type": types.ELASTIC
                                }
                            }))
                    }
                }
                else setSelectServiceType(true);
                break;
            case types.MSSQL:
                defaultCreator.current = setShowAddMsSQLMaster
                if (user?.is_superuser === false) {
                    if (user?.privileges?.mssql_quota < 1) {
                        showWarningMessage(warningMessage);
                        return;
                    }
                    else {
                        warningMessage.status = `User has exceed quota of ${user?.privileges?.mssql_quota}  SQL Server clusters`
                        dispatch(LayoutActions.isQuotaExceed(
                            {
                                "allow": setSelectServiceType,
                                "reject": showWarningMessage,
                                "warning": warningMessage,
                                "request":
                                {
                                    "quota": user?.privileges?.mssql_quota,
                                    "obj_type": types.MSSQL
                                }
                            }))
                    }
                }
                else setSelectServiceType(true);
                break;
            case types.REDIS:
                if (redisType === "standalone") defaultCreator.current = setShowAddRedisStandalone
                else defaultCreator.current = setShowAddRedisCluster
                setSelectServiceType(true);
                break;
            default:
                break;
        }
    }
    const onAddRedisStandalone = (data) => dispatch(LayoutActions.addRedisStandalone(data, setShowAddRedisStandalone));

    const onAddPostgresPrimary = (data) => dispatch(LayoutActions.addPgPrimary(data, setShowAddPostgresPrimary));

    const onAddMongoMaster = (data) => dispatch(LayoutActions.addMongoMaster(data, setShowAddMongoMaster));

    const onAddMariaDBMaster = (data) => dispatch(LayoutActions.addMariaDbMaster(data, setShowAddMariaDBMaster));

    const onAddMySQLMaster = (data) => dispatch(LayoutActions.addMySqlMaster(data, setShowAddMySQLMaster));
    const onAddMsSQLMaster = (data) => dispatch(LayoutActions.addMsSqlMaster(data, setShowAddMsSQLMaster));

    const onAddElasticSearchMaster = (data) => dispatch(LayoutActions.addElasticSearchMaster(data, setShowAddElasticSearchMaster));

    const onRegisterCdb = (data) => dispatch(LayoutActions.addCdbNode(data, { "success": setShowRegisterCDBDialog, "warning": showWarningMessage }));
    const onInstallCdb = (data) => dispatch(LayoutActions.installCdbNode(data, { "success": setShowInstallCDBDialog, "warning": showWarningMessage }));

    const onSelectAddCDBMethod = (method) => {
        setShowAddCDBMethodDialog(false);
        if (method === 'register') setShowRegisterCDBDialog(true);
        else {
            defaultCreator.current = setShowInstallCDBDialog
            setSelectServiceType(true);
        }
    }

    useEffect(() => {
        if (selectedType) dispatch(ConfigActions.getIntegratedServices());
    }, [selectedType, dispatch])

    return (
        <div className="layout-page" id="layout-page">
            <Menu dbType={dbType} />
            <LayoutFilter
                severityGreenTotal={severityGreenTotal}
                severityYellowTotal={severityYellowTotal}
                severityRedTotal={severityRedTotal}
                showSeverityGreen={showSeverityGreen}
                showSeverityYellow={showSeverityYellow}
                showSeverityRed={showSeverityRed}
                onDbChange={onFilterDbChange}
                onSeverityChange={onFilterSeverityChange}
                onSearchChange={onSearchChange}
                onAddNodeClick={onAddNodeClick}
            />
            <div className="add-cdb-container flex flex-column mt-4">
                <div className="grid">
                    <div className="col summary-box text-base">
                        {summary?.primary_mariadb_count > 0 && (dbType === types.ALL || dbType === types.MARIADB) && <div draggable={false} className="grid"> <div className="col-2" >MariaDB: </div> <div className="col-3" > {summary.primary_mariadb_count} Primaries ({summary.primary_mariadb_up} up, {summary.primary_mariadb_down} down) </div> <div className="col-3" > {summary.standby_mariadb_count} Standbys ({summary.standby_mariadb_up} up, {summary.standby_mariadb_down} down) </div>  </div>}
                        {summary?.primary_mongo_count > 0 && (dbType === types.ALL || dbType === types.MONGO) && <div draggable={false} className="grid"> <div className="col-2" >MongoDB: </div> <div className="col-3" > {summary.primary_mongo_count} Primaries ({summary.primary_mongo_up} up, {summary.primary_mongo_down} down) </div> <div className="col-3" > {summary.standby_mongo_count} Standbys ({summary.standby_mongo_up} up, {summary.standby_mongo_down} down) </div> <div className="col-3"> {summary.arbiter_mongo_count} Arbiters  ({summary.arbiter_mongo_up} up, {summary.arbiter_mongo_down} down) </div>  </div>}
                        {summary?.primary_mysql_count > 0 && (dbType === types.ALL || dbType === types.MYSQL) && <div draggable={false} className="grid"> <div className="col-2" >MySQL: </div> <div className="col-3" > {summary.primary_mysql_count} Primaries ({summary.primary_mysql_up} up, {summary.primary_mysql_down} down) </div> <div className="col-3" > {summary.standby_mysql_count} Standbys  ({summary.standby_mysql_up} up, {summary.standby_mysql_down} down) </div>  </div>}
                        {summary?.cdb_count > 0 && (dbType === types.ALL || dbType === types.ORACLE_CDB) && <div draggable={false} className="grid"> <div className="col-2" >Oracle: </div> <div className="col-3"> {summary.cdb_count} CDBs ({summary.cdb_up} up, {summary.cdb_down} down) </div> <div className="col-3" > {summary.pdb_count} PDBs ({summary.pdb_up} up, {summary.pdb_down} down) </div>  </div>}
                        {summary?.primary_count > 0 && (dbType === types.ALL || dbType === types.POSTGRES) && <div draggable={false} className="grid"> <div className="col-2" >PostgreSQL: </div> <div className="col-3" > {summary.primary_count} Primaries  ({summary.primary_up} up, {summary.primary_down} down) </div> <div className="col-3" > {summary.standby_count} Standbys ({summary.standby_up} up, {summary.standby_down} down) </div>  </div>}
                    </div>
                    <Button label="Add a new cluster" className="p-button-text filter-button add-cluster" onClick={() => onAddNodeClick()} />
                </div>
                <Button className="ml-auto" link icon='pi pi-sort-alt' label={sortOption} onClick={() => {
                    switch (sortOption) {
                        case layoutSortOptions.NEWEST:
                            setSortOption(layoutSortOptions.SEVERITY);
                            break;
                        case layoutSortOptions.SEVERITY:
                            setSortOption(layoutSortOptions.CHILD_COUNT);
                            break;
                        default:
                            setSortOption(layoutSortOptions.NEWEST);
                            break;

                    }
                }} />
            </div>
            {(!filteredNodes || filteredNodes?.length === 0) && <h3 className="ml-8" style={{ textAlign: "center" }}> {!filteredNodes && isLoading && <i className="pi pi-spin pi-spinner" />} {!filteredNodes && isLoading ? <div className="text-sm pl-3">{loadingMsg}</div> : emptyMsg}</h3>}
            {filteredNodes?.length > 0 && filteredNodes.map((n, index) => {
                switch (n.type) {
                    case types.ORACLE_CDB:
                        return <OracleLayoutItem user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.ORACLE_CDB)} selected={selected} />
                    case types.POSTGRES:
                        return <PgLayoutItem services={integrated_services?.filter((s) => s.type === types.PROXMOX && s.supported_os[`${n.ex_params['distributor']} ${n['ex_params']['release']}`])} user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.POSTGRES)} selected={selected} />
                    case types.POSTGRES_K8S:
                        return <PgK8SLayoutItem service={integrated_services?.filter((s) => s.type === types.K8S && s.id === n.ex_params?.service_id)?.[0]} user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.POSTGRES_K8S)} selected={selected} />
                    case types.MONGO:
                        if (n.cluster_role) return <MongoLayoutItem services={integrated_services?.filter((s) => s.type === types.PROXMOX && s.supported_os[`${n.ex_params['distributor']} ${n['ex_params']['release']}`])} user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.MONGO)} selected={selected} />
                        return <MongoShardedLayoutItem service={integrated_services?.filter((s) => s.type === types.K8S && s.id === n.ex_params?.service_id)?.[0]} user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.MONGO)} selected={selected} />
                    case types.MONGO_K8S:
                        return <MongoK8SLayoutItem user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.MONGO_K8S)} selected={selected} />
                    case types.MARIADB:
                        return <MariaDbLayoutItem services={integrated_services?.filter((s) => s.type === types.PROXMOX && s.supported_os[`${n.ex_params['distributor']} ${n['ex_params']['release']}`])} user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.MARIADB)} selected={selected} />
                    case types.ELASTIC:
                        return <ElasticsearchLayoutItem services={integrated_services?.filter((s) => s.type === types.PROXMOX && s.supported_os[`${n.ex_params['distributor']} ${n['ex_params']['release']}`])} user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.ELASTIC)} selected={selected} />
                    case types.MYSQL:
                        return <MySQLLayoutItem services={integrated_services?.filter((s) => s.type === types.PROXMOX && s.supported_os[`${n.ex_params['distributor']} ${n['ex_params']['release']}`])} user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.MYSQL)} selected={selected} />
                    case types.MSSQL:
                        return <MsSQLLayoutItem user={user?.data} key={n.id} node={n} selection={filteredNodes.filter((n) => n.type === types.MYSQL)} selected={selected} />
                    case types.REDIS:
                        return <RedisLayoutItem services={integrated_services?.filter((s) => s.type === types.PROXMOX && s.supported_os[`${n.ex_params['distributor']} ${n['ex_params']['release']}`])} user={user?.data} key={index} node={n} selection={filteredNodes.filter((n) => n.type === types.REDIS)} selected={selected} />
                    default:
                        return null;
                }
            })}


            {showRegisterCDBDialog ? <AddCDBDialog
                visible={showRegisterCDBDialog}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowRegisterCDBDialog(false)
                }}
                onOkClick={onRegisterCdb} /> : null}

            {showInstallCDBDialog ? <InstallCDBDialog
                visible={showInstallCDBDialog}
                onHide={() => {
                    setShowInstallCDBDialog(false);
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setOracleVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                }}
                onOkClick={onInstallCdb} /> : null}


            {showAddCDBMethodDialog ? <AddCDBMethodDialog
                visible={showAddCDBMethodDialog}
                onHide={() => {
                    setShowAddCDBMethodDialog(false)
                }}
                onOkClick={onSelectAddCDBMethod} /> : null}

            {showAddPostgresPrimary ? <AddPostgresPrimaryDialog
                visible={showAddPostgresPrimary}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setPgVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddPostgresPrimary(false)
                }}
                onOkClick={onAddPostgresPrimary}
            /> : null}

            {showAddMongoMaster ? <AddMongoMasterDialog
                visible={showAddMongoMaster}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setMongoVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddMongoMaster(false)
                }}
                onOkClick={onAddMongoMaster}
            /> : null}

            {showAddMariaDBMaster ? <AddMariaDBMasterDialog
                visible={showAddMariaDBMaster}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setMariaDbVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddMariaDBMaster(false)
                }}
                onOkClick={onAddMariaDBMaster}
            /> : null}

            {showAddMySQLMaster ? <AddMySQLMasterDialog
                visible={showAddMySQLMaster}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setMySqlVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddMySQLMaster(false)
                }}
                onOkClick={onAddMySQLMaster}
            /> : null}

            {showAddMsSQLMaster ? <AddMsSQLMasterDialog
                visible={showAddMsSQLMaster}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddMsSQLMaster(false)
                }}
                onOkClick={onAddMsSQLMaster}
            /> : null}

            {showAddElasticSearchMaster ? <AddElacticSearchMasterDialog
                visible={showAddElasticSearchMaster}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setElasticSearchVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddElasticSearchMaster(false)
                }}
                onOkClick={onAddElasticSearchMaster}
            /> : null}
            {showAddRedisStandalone ? <CreateRedisStandaloneDialog
                visible={showAddRedisStandalone}
                onHide={() => {
                    dispatch(LayoutActions.setPortExists(undefined));
                    dispatch(LayoutActions.setPathExists(undefined));
                    dispatch(LayoutActions.setRedisVersionExists(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowAddRedisStandalone(false)
                }}
                onOkClick={onAddRedisStandalone}
            /> : null}
            {showAddRedisCluster ? <AddRedisClusterDialog
                visible={showAddRedisCluster}
                onHide={() => setShowAddRedisCluster(false)}
                onOkClick={onAddRedisCluster}
            /> : null}

            {showAddMongoShardedCluster ? <AddMongoShardedClusterDialog
                visible={showAddMongoShardedCluster}
                onHide={() => setShowAddMongoShardedCluster(false)}
                onOkClick={onAddMongoShardedCluster}
            /> : null}
            {showNodeTypeDialog ? <NodeTypeDialog
                visible={showNodeTypeDialog}
                onHide={() => setShowNodeTypeDialog(false)}
                onOkClick={onSetNodeType}
            /> : null}
            {showWarning ? <WarningDialog
                message={warningMessage}
                visible={showWarning}
                onHide={() => setShowWarning(false)} />
                : null}

            <SelectServiceTypeDialog
                defaultCreator={defaultCreator.current}
                type={selectedType}
                services={integrated_services?.filter((s) => s.supported_databases[selectedType] === true)}
                visible={selectServiceType}
                onHide={() => setSelectServiceType(false)}
                onShow={() => setSelectServiceType(true)}
                onBack={() => {
                    setSelectServiceType(false);
                    setShowNodeTypeDialog(true);

                }}

            />
        </div>
    );
}

export default Layout;
