import restore from '../../../assets/images/restore_pdb.svg';
import backup_pdb from '../../../assets/images/backup_pdb.svg';
import drop from '../../../assets/images/drop.svg';
import icon_import from '../../../assets/images/import_pdb.svg'
import icon_export from '../../../assets/images/export_pdb.svg';
import green_star from '../../../assets/images/green-star.png';
import red_star from '../../../assets/images/red-star.png';
import info from '../../../assets/images/info.svg';
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation, useNavigate } from "react-router-dom";
import { StatusActions } from "../../../store/status/reducer";
import { Fragment, useEffect, useState, useCallback, useRef } from "react";
import { TabView, TabPanel } from 'primereact/tabview'
import { Button } from 'primereact/button';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dropdown } from "primereact/dropdown";
import DetailsMenu from '../../../components/menus/detailsMenu';
import credentials from "../../../assets/images/credentials.svg";
import WarningDialog from '../../../dialogs/warningDialog';
import PerformanceCharts from '../../../components/performanceCharts';
import { ConfirmPopup } from 'primereact/confirmpopup';
import SwitchMariaDBNodeDialog from '../../../dialogs/mariadb/switchMariaDBNodeDialog'
import DropMariaDBNodeDialog from '../../../dialogs/mariadb/dropMariaDBNodeDialog'
import RestoreMariaDBNode from '../../../dialogs/mariadb/restoreMariaDBNode'
import BackupMariaDBNode from '../../../dialogs/mariadb/backupMariaDBNode'
import getStarByClass from '../../../util/getStarByClass';
import { Tooltip } from 'primereact/tooltip';
import { clusterRoles, timePeriodTypes } from '../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../components/tables/nodeAlertsTable';
import createAPIcommand from '../../../util/createApiCommand';
import copyToClipboard from '../../../util/copyToClipboard';
import ImportMariaDBNodeDialog from '../../../dialogs/mariadb/importMariaDBNodeDialog';
import MariaDBExportDialog from '../../../dialogs/mariadb/mariadbExportDialog';
import DatabaseUsersTable from '../../../components/tables/databaseUsersTable';
import { InputText } from 'primereact/inputtext';
import StatsCard from '../../../components/statsCard';
import { searchFilter } from '../../monitoring';
import { ButtonGroup } from 'primereact/buttongroup';
import { AppActions } from '../../../store/app/reducer';
import ClusterSizing from '../../../components/changeClusterSizingBlock';
import { searchFilter as logSearch } from '../../log';
import LogsTable from '../../../components/tables/logsTable';
import LogFilesTable from '../../../components/tables/logFilesTable';

export default function MariaDBDetails(props) {

    const { id } = useParams();
    const { state } = useLocation();
    const navigate = useNavigate();

    const { mariadb_details, errorMessage } = useSelector((state) => state.status);
    const { accessToken, severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);


    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);
    const { node_logs } = useSelector((state) => state.log);


    const dispatch = useDispatch();

    const [upperSearch, setUpperSearch] = useState("");

    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }

        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];

    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);

    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period);

    const [showBackup, setShowBackup] = useState(false);

    const [showSwitchover, setShowSwitchover] = useState(false);

    const [showWarningPopup, setShowWarningPopup] = useState(false);
    const [showExport, setShowExport] = useState(false);
    const [showImport, setShowImport] = useState(false);

    const [showDropMariaDbNode, setShowDropMariaDbNode] = useState(false);

    const [showRestore, setShowRestore] = useState(false);

    const [toggle, setToggle] = useState(0);
    const updateSchedule = useRef(null);
    const [pageIndex, setPageIndex] = useState(null);

    const { alerts } = useSelector((state) => state.monitoring);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [filteredUsers, setFilteredUsers] = useState(null);
    const [search, setSearch] = useState("");
    const [popupAction, setPopupAction] = useState('stop');
    const { database_users } = useSelector((state) => state.config);
    const [filteredLogs, setFilteredLogs] = useState(null);

    useEffect(() => {

        if (database_users?.length) {

            let tempData = [];

            tempData = database_users;
            let query = search.toLowerCase();
            if (query) tempData = tempData.filter((u) => u.name?.toLowerCase().includes(query));
            setFilteredUsers(tempData);
        }

    }, [search, database_users])

    useEffect(() => {

        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;

        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            if (!showSeverityGreen || !showSeverityYellow || !showSeverityRed) tempData = severityFilter(tempData);
            setFilteredAlerts(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);

    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen])



    // useEffect(() => {

    //     if (mariadb_details?.history?.length > 0) {

    //         const query = upperSearch.toLowerCase().trim();
    //         if (!query) setFilteredLogs(mariadb_details.history);
    //         else setFilteredLogs(mariadb_details.history.filter((h) => h["MESSAGE"].toLowerCase().includes(query)))
    //     }

    // },
    //     [upperSearch, mariadb_details?.history])


    const onExportMariaDBNode = (data) => {
        dispatch(LayoutActions.exportMariaDbNode(data, setShowExport));
    }

    const onImport = (data) => {
        dispatch(LayoutActions.importMariaDbNode(data, setShowImport));

    }


    const onShowPopup = async (target) => {
        switch (target) {

            case "stop-down":
                setPopupAction("stop");
                await copyToClipboard(createAPIcommand({ "node_id": mariadb_details?.node?.name }, accessToken, "stop_mariadb_node/"))
                break;
            case "start-down":
                setPopupAction(mariadb_details?.node?.ind_started ? "restart" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": mariadb_details?.node?.name }, accessToken, "start_mariadb_node/"))
                break;
            case "start-stop":
                setPopupAction(mariadb_details?.node?.ind_started ? "stop" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": mariadb_details?.node?.name }, accessToken, mariadb_details?.node?.ind_started ? "stop_mariadb_node/" : "start_mariadb_node/"));
                break;
            default:
                break;


        }
        setShowWarningPopup(target);

    }

    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);

    }, [state?.targetTab])


    useEffect(() => {
        if (id) {
            dispatch(StatusActions.getMariaDbDetails(id));
        }
        return () => {
            dispatch(StatusActions.setMariaDbDetails(null));
            dispatch(LayoutActions.getNodes());
        }
    }, [id, dispatch])

    const onShowBackup = () => {

        setPageIndex(null);

        setShowBackup(true);

    }


    const onSetSchedule = () => {
        if (mariadb_details?.node?.ind_deleted) showWarningMessage({ "text": `${mariadb_details?.node?.name} is unavailable.`, 'status': mariadb_details?.node?.status });
        else {
            setPageIndex(1);
            updateSchedule.current = mariadb_details.backups?.length > 0 ? mariadb_details?.backups?.filter((b) => b.is_sched === true)?.[0] : mariadb_details?.nodes?.filter((s) => s?.backups?.filter((b) => b.is_sched === true))?.[0]?.backups?.filter((b) => b.is_sched === true)?.[0];
            setShowBackup(true);
        }

    }

    const onRestore = (data) => {
        dispatch(LayoutActions.restoreMariaDbNode(data, setShowRestore));

    }

    const onPopupAccept = () => {

        switch (showWarningPopup) {

            case "stop-down":
                onStopMariaDbNode();
                break;
            case "start-down":
                onStartMariaDbNode();
                break;
            case "start-stop":
                mariadb_details?.node?.ind_started ? onStopMariaDbNode() : onStartMariaDbNode();
                break;
            default:
                break;

        }

    }
    const showWarningMessage = (data) => {

        setWarningMessage(data);
        setShowWarning(true);

    }

    const onStopMariaDbNode = () => dispatch(LayoutActions.stopMariaDbNode({ "node_id": mariadb_details?.node?.id }, mariadb_details?.node));


    const onStartMariaDbNode = () => dispatch(LayoutActions.startMariaDbNode({ "node_id": mariadb_details?.node?.id }, mariadb_details?.node));

    // const messageBodyTemplate = (item) => {

    //     return (<div className='history-cell'>{item["MESSAGE"]}</div>);
    // }

    const onBackupMariaDbNode = (data) => {
        dispatch(LayoutActions.backupMariaDbNode(data, setShowBackup));
    }

    const onDropMariaDbNode = (data) => {

        dispatch(LayoutActions.dropMariaDbNode(data, setShowDropMariaDbNode));

    }

    const onSwitchover = (data) => {
        dispatch(LayoutActions.switchMariaDbNode(data, setShowSwitchover));

    }

    // const isPrimary = (is_primary) => {

    //     if (!is_primary || mariadb_details.nodes?.length === 0) setShowDropMariaDbNode(true);
    //     else showWarningMessage({ "text": `This is primary with standbys, cannot be removed.\nSwitchover to standby first.` })
    // }

    const attachPopup = (target) => mariadb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mariadb_details?.node?.name} is unavailable.`, 'status': mariadb_details?.node?.status }) : onShowPopup(target);

    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">
                <TabView activeIndex={toggle} onTabChange={(e) => {
                    if (e.index < 4) setToggle(e.index);
                }} >
                    <TabPanel header="Dashboard">
                        <div className='grid flex-row'>
                            <div className='col-3 flex align-items-between flex-column' >
                                <h2 >
                                    {mariadb_details?.node?.cluster_role} {mariadb_details?.node?.name}
                                </h2>
                                <div>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Instance is {mariadb_details?.node?.ind_started ? 'up' : "down"}
                                                </td>
                                                <td>
                                                    <Button
                                                        className="p-button-sm details-button"
                                                        disabled={!mariadb_details?.node?.ind_started && !errorMessage}
                                                        id="start-stop"
                                                        icon={mariadb_details?.node?.ind_started ? "pi pi-stop-circle" : "pi pi-play"}
                                                        style={{ minWidth: "53px", minHeight: "39px" }}
                                                        iconPos="right"
                                                        onClick={() => attachPopup("start-stop")}
                                                    />
                                                </td>
                                                <td>
                                                    <img src={mariadb_details?.node?.ind_started ? green_star : red_star} alt="star" className="avail-star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {mariadb_details?.ind_error ? "Errors in cluster" : mariadb_details?.ind_error === false ? "Cluster is OK" : <div> <i className="pi pi-spin pi-spinner" /> Connecting to node... </div>}
                                                </td>
                                                <td>
                                                    <Button onClick={() => navigate('/layout', { state: { node_id: mariadb_details?.node?.ex_params?.mariadb_master_id } })} disabled={!mariadb_details?.node} className='p-button-sm details-button'><img alt="info" width={16} height={16} src={info} /></Button>
                                                </td>
                                                <td>
                                                    <img className='status-star' src={(mariadb_details?.ind_error === undefined || mariadb_details?.ind_error === true) ? red_star : green_star} alt="star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {mariadb_details?.auto_backup?.status ? mariadb_details.auto_backup.status : ""}
                                                </td>
                                                <td>
                                                    <Button onClick={onSetSchedule} icon="pi pi-calendar-times" disabled={!mariadb_details?.node?.ind_started && !errorMessage} className='p-button-sm details-button update-schedule-button'></Button>
                                                </td>
                                                <td>
                                                    <img className='set-schedule-tooltip' src={getStarByClass(mariadb_details?.auto_backup?.class)} alt="star"></img>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <div>
                                    <h3>
                                        General info
                                    </h3 >
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td> {mariadb_details?.node?.name}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    DB ID
                                                </td>
                                                <td> {mariadb_details?.db_id}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Version
                                                </td>
                                                <td> {mariadb_details?.version}</td>
                                            </tr>

                                            <tr>
                                                <td>
                                                    Logging
                                                </td>
                                                <td> {mariadb_details?.logging}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Read Only
                                                </td>
                                                <td> {mariadb_details?.read_only}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Size
                                                </td>
                                                <td>  {mariadb_details?.display_size}</td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <ClusterSizing node={mariadb_details?.node} />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>
                                <StatsCard details={mariadb_details}></StatsCard>
                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={mariadb_details?.node?.cpuUsages}
                                    memUsages={mariadb_details?.node?.memUsages}
                                />

                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>

                                <div className='col-9'>

                                    <span className="p-buttonset">
                                        < Button className="p-button-sm details-button up" disabled={!!mariadb_details?.node?.ind_started && !errorMessage} label="Backup" onClick={() => mariadb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mariadb_details?.node?.name} is unavailable.`, 'status': mariadb_details?.node?.status }) : onShowBackup()}>
                                            < img alt="backup" width={16} height={16} src={backup_pdb} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={!mariadb_details?.node?.cluster_role !== clusterRoles.PRIMARY || (!mariadb_details?.node?.ind_started && !errorMessage)} label="Restore" onClick={() => mariadb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mariadb_details?.node?.name} is unavailable.`, 'status': mariadb_details?.node?.status }) : setShowRestore(true)}>
                                            < img alt="restore" width={16} height={16} src={restore} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={(!mariadb_details?.node?.ind_started && !errorMessage)} label="Export" onClick={() => mariadb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mariadb_details?.node?.name} is unavailable.`, 'status': mariadb_details?.node?.status }) : setShowExport(true)}>
                                            < img alt="export" width={16} height={16} src={icon_export} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={(!mariadb_details?.node?.ind_started && !errorMessage) || mariadb_details?.node?.cluster_role === clusterRoles.STANDBY} label="Import" onClick={() => mariadb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mariadb_details?.node?.name} is unavailable.`, 'status': mariadb_details?.node?.status }) : setShowImport(true)}>
                                            < img alt="import" width={16} height={16} src={icon_import} />
                                        </Button>
                                        <Button disabled={(mariadb_details?.node?.ind_started && !errorMessage)} className="p-button-sm details-button up" label="Credentials">
                                            <img alt="credentials" width={16} height={16} src={credentials} ></img>
                                        </Button>


                                        <Button className="p-button-sm details-button up" icon="pi pi-arrows-h"
                                            iconPos="right"
                                            disabled={mariadb_details?.node?.cluster_role !== clusterRoles.PRIMARY || (!mariadb_details?.node?.ind_started && !errorMessage) || (mariadb_details?.nodes?.filter((n) => n.severity === "green" && n.ind_started === true).length === 0)}
                                            label="Switchover"
                                            onClick={() => setShowSwitchover(true)}
                                        />
                                    </span>

                                    <span className="p-buttonset down">
                                        <Button disabled={errorMessage || !mariadb_details?.node?.ind_started} icon="pi pi-stop-circle" iconPos="right" className="p-button-sm details-button" id="stop-down" label="Stop" onClick={() => attachPopup("stop-down")} />
                                        <Button id="start-down" icon={`pi pi-${mariadb_details?.node?.ind_started ? "replay" : "play"}`} iconPos="right" className="p-button-sm details-button" label={mariadb_details?.node?.ind_started ? "Restart" : "Start"} onClick={() => attachPopup("start-down")}>
                                        </Button>
                                        <Button disabled={!mariadb_details?.node?.ind_started && !errorMessage} className="p-button-sm details-button" label="Drop" onClick={() => mariadb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mariadb_details?.node?.name} is unavailable.`, 'status': mariadb_details?.node?.status }) : setShowDropMariaDbNode(true)}>
                                            < img alt="drop" width={16} height={16} src={drop} />
                                        </Button>
                                    </span>


                                </div>
                            </div>
                        </div>

                    </TabPanel>
                    <TabPanel header="Activity Logs">

                        <span className="p-input-icon-right">
                            <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                setUpperSearch("");
                            }} ></i>
                            <InputText className="p-inputtext-sm filter-field mr-2- mb-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                        </span>

                        <div className='flex'>

                            <Dropdown className="p-inputtext-sm mb-2 mt-2 filter-field time-period" value={timePeriodType}
                                options={timePeriodTypes}
                                onChange={(e) => {
                                    setTimePeriodType(e.value);
                                    setMonitorInterval(intervalByTimedelta(e.value));
                                }}
                            />

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <LogsTable filteredLogs={filteredLogs}
                            timePeriodType={timePeriodType}
                            node_id={id}
                        />

                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} rows={100} paginator alwaysShowPaginator={false} >
                            <Column className='history-column' field='__REALTIME_TIMESTAMP' header="Event" body={messageBodyTemplate} sortable></Column>
                        </DataTable> */}

                    </TabPanel>
                    <TabPanel header="Database Logs">
                        <LogFilesTable node_id={id} />
                    </TabPanel>

                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <Dropdown className="p-inputtext-sm mb-2 mt-2 filter-field time-period" value={timePeriodType}
                                options={timePeriodTypes}
                                onChange={(e) => {
                                    setTimePeriodType(e.value);
                                    setMonitorInterval(intervalByTimedelta(e.value));
                                }}
                            />

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>
                        <NodeAlertsTable node={mariadb_details?.node} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />

                    </TabPanel>
                    <TabPanel header="Users">
                        <div className="flex justify-content-start">
                            <span className="p-input-icon-right">
                                <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => setSearch("")} />
                                <InputText className="p-inputtext-sm filter-field mb-2"
                                    placeholder="Search"
                                    value={search}
                                    onChange={(e) => {
                                        setSearch(e.target.value);
                                    }} />
                            </span>

                        </div>
                        <DatabaseUsersTable node={mariadb_details?.node} users={filteredUsers} />

                    </TabPanel>
                    <TabPanel headerClassName='filters-header' header={<div className='flex'>


                        {toggle < 4 ? <Dropdown
                            className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period raised-dropdown" value={timePeriodType}
                            options={timePeriodTypes}
                            onChange={(e) => {
                                setTimePeriodType(e.value);
                                setMonitorInterval(intervalByTimedelta(e.value));
                            }}
                        /> : null}
                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => {
                                dispatch(StatusActions.setMariaDbDetails(null));
                                dispatch(StatusActions.getMariaDbDetails(id));

                            }}
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>
                </TabView>

                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}

                {(showDropMariaDbNode && mariadb_details) ? <DropMariaDBNodeDialog
                    node={mariadb_details?.node}
                    visible={showDropMariaDbNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowDropMariaDbNode(false)
                    }}
                    onOkClick={onDropMariaDbNode}
                />

                    : null}

                {(showBackup && mariadb_details) ? <BackupMariaDBNode
                    node={mariadb_details?.node}
                    visible={showBackup}
                    updateSchedule={updateSchedule.current}
                    pageIndex={pageIndex}
                    backups={mariadb_details?.backups}
                    nodes={mariadb_details?.nodes}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowBackup(false)
                    }}
                    onOkClick={onBackupMariaDbNode}
                />
                    : null}


                {(showSwitchover && mariadb_details) ? <SwitchMariaDBNodeDialog

                    nodes={mariadb_details.nodes?.filter((n) => n?.severity === "green" && n?.ind_started === true && n?.id !== mariadb_details?.node?.id)}
                    visible={showSwitchover}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowSwitchover(false);
                    }}
                    onOkClick={onSwitchover}

                /> : null}


                {(showExport && mariadb_details) ? <MariaDBExportDialog
                    node={mariadb_details?.node}
                    visible={showExport}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        dispatch(LayoutActions.setDirExists(null));
                        dispatch(LayoutActions.setExportExists(null));
                        dispatch(LayoutActions.setDiskSpace(null));
                        setShowExport(false)
                    }}
                    onOkClick={onExportMariaDBNode}
                />
                    : null}


                {(showImport && mariadb_details) ? <ImportMariaDBNodeDialog
                    data={mariadb_details}
                    visible={showImport}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowImport(false)
                    }}
                    onOkClick={onImport}
                /> : null}


                {(showRestore && mariadb_details) ? <RestoreMariaDBNode
                    data={mariadb_details}
                    visible={showRestore}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestore(false)
                    }}
                    onOkClick={onRestore}
                /> : null}

            </div>
            <ConfirmPopup
                icon="pi pi-exclamation-triangle"
                target={document.getElementById(showWarningPopup)}
                visible={Boolean(showWarningPopup)}
                onHide={() => setShowWarningPopup(false)}
                accept={onPopupAccept}
                message={`Are you sure you want to ${popupAction} the node?`} />
            {mariadb_details && <Tooltip target=".set-schedule-star" content={mariadb_details?.auto_backup?.detail}></Tooltip>}
            {mariadb_details && <Tooltip target=".avail-star" content={mariadb_details?.node?.ind_started ? "Database is running and accepting connections" : "Database is not running"}></Tooltip>}
            {mariadb_details && <Tooltip target=".status-star" content={(mariadb_details?.ind_error === undefined || mariadb_details?.ind_error === true) ? "Some nodes of cluster are down/inaccessible" : "All nodes of cluster are accessible"}></Tooltip>}
        </Fragment>



    )


}
