import restore from '../../../assets/images/restore_pdb.svg';
import backup_pdb from '../../../assets/images/backup_pdb.svg';
import icon_import from '../../../assets/images/import_pdb.svg';
import icon_export from '../../../assets/images/export_pdb.svg';
import drop from '../../../assets/images/drop.svg';
import green_star from '../../../assets/images/green-star.png';
import red_star from '../../../assets/images/red-star.png';
import info from '../../../assets/images/info.svg'
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation, useNavigate } from "react-router-dom";
import { StatusActions } from "../../../store/status/reducer";
import { Fragment, useEffect, useState, useCallback, useRef } from "react";
import { Button } from 'primereact/button';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dropdown } from "primereact/dropdown";
import { TabView, TabPanel } from 'primereact/tabview'
import DetailsMenu from '../../../components/menus/detailsMenu';
import credentials from "../../../assets/images/credentials.svg";
import WarningDialog from '../../../dialogs/warningDialog';
import PerformanceCharts from '../../../components/performanceCharts';
import { ConfirmPopup } from 'primereact/confirmpopup';
import MongoNodeBackupDialog from '../../../dialogs/mongo/mongoNodeBackupDialog';
import MongoDumpDialog from '../../../dialogs/mongo/mongoDumpDialog';
import DropMongoNodeDialog from '../../../dialogs/mongo/dropMongoNodeDialog';
import RestoreBackupMongoNode from '../../../dialogs/mongo/restoreBackupMongoNode';
import RestoreDumpMongoNode from '../../../dialogs/mongo/restoreDumpMongoNode';
import { Tooltip } from 'primereact/tooltip';
import getStarByClass from '../../../util/getStarByClass';
import { clusterRoles, timePeriodTypes } from '../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../components/tables/nodeAlertsTable';
import createAPIcommand from '../../../util/createApiCommand';
import copyToClipboard from '../../../util/copyToClipboard';
import DatabaseUsersTable from '../../../components/tables/databaseUsersTable';
import { ConfigActions } from '../../../store/config/reducer';
import { InputText } from 'primereact/inputtext';
import StatsCard from '../../../components/statsCard';
import { searchFilter } from '../../monitoring';
import { ButtonGroup } from 'primereact/buttongroup';
import { AppActions } from '../../../store/app/reducer';
import ClusterSizing from '../../../components/changeClusterSizingBlock';
import { searchFilter as logSearch } from '../../log';
import LogsTable from '../../../components/tables/logsTable';
import LogFilesTable from '../../../components/tables/logFilesTable';

export default function MongoDetails(props) {
    const { id } = useParams();
    const { state } = useLocation();
    const navigate = useNavigate();
    const { mongo_details, errorMessage } = useSelector((state) => state.status);
    const { accessToken, severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);
    const dispatch = useDispatch();
    const [upperSearch, setUpperSearch] = useState("");
    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [showBackup, setShowBackup] = useState(false);
    const [ShowDump, setShowDump] = useState(false);
    const [showWarningPopup, setShowWarningPopup] = useState(false);
    const [showDropMongoNode, setShowDropMongoNode] = useState(false);
    const [showRestore, setShowRestore] = useState(false);
    const [showRestoreDump, setShowRestoreDump] = useState(false);
    const [toggle, setToggle] = useState(0);
    const updateSchedule = useRef(null);
    const [pageIndex, setPageIndex] = useState(null);
    const { alerts } = useSelector((state) => state.monitoring);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [filteredUsers, setFilteredUsers] = useState(null);
    const [search, setSearch] = useState("");
    const { database_users } = useSelector((state) => state.config);
    const [popupAction, setPopupAction] = useState("stop");
    const [filteredLogs, setFilteredLogs] = useState(null);



    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);

    useEffect(() => {
        const storedTabIndex = localStorage.getItem('DetailsTabIndex');
        if (storedTabIndex) {
            setToggle(parseInt(storedTabIndex, 10));
        }
    }, []);

    const handleTabChange = (e) => {
        setToggle(e.index);
        localStorage.setItem('DetailsTabIndex', e.index);
    };

    const { node_logs } = useSelector((state) => state.log);
    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }

        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];

    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);

    useEffect(() => {
        setFilteredUsers(null);
        if (mongo_details?.node) dispatch(ConfigActions.getDatabaseUsers(mongo_details?.node?.id));
    }, [mongo_details, dispatch])


    useEffect(() => {

        if (id) dispatch(StatusActions.setMongoDetails(null));

    }, [id, dispatch])

    useEffect(() => {

        if (database_users?.length) {

            let tempData = [];

            tempData = database_users;
            let query = search.toLowerCase();
            if (query) tempData = tempData.filter((u) => u.name?.toLowerCase().includes(query));
            setFilteredUsers(tempData);
        }

    }, [search, database_users])

    useEffect(() => {

        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;

        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            if (!showSeverityGreen || !showSeverityYellow || !showSeverityRed) tempData = severityFilter(tempData);
            setFilteredAlerts(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);

    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen])


    // useEffect(() => {

    //     if (mongo_details?.history?.length > 0) {

    //         const query = upperSearch.toLowerCase().trim();
    //         if (!query) setFilteredLogs(mongo_details.history);
    //         else setFilteredLogs(mongo_details.history.filter((h) => h.c.toLowerCase().includes(query)
    //             || h.ctx.toLowerCase().includes(query)
    //             || h.id.toLowerCase().includes(query)
    //             || h.msg.toLowerCase().includes(query)
    //             || h["$t"]["date"].format("MMM DD,YYYY, hh:mm A").includes(query)))
    //     }

    // },
    //     [upperSearch, mongo_details?.history])

    const onShowPopup = async (target) => {
        switch (target) {
            case "stop-down":
                setPopupAction("stop");
                await copyToClipboard(createAPIcommand({ "node_id": mongo_details?.node?.name }, accessToken, "stop_mongo_node/"))
                break;
            case "start-down":
                setPopupAction(mongo_details?.node?.ind_started ? "restart" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": mongo_details?.node?.name }, accessToken, "start_mongo_node/"))
                break;
            case "start-stop":
                setPopupAction(mongo_details?.node?.ind_started ? "stop" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": mongo_details?.node?.name }, accessToken, mongo_details?.node?.ind_started ? "stop_mongo_node/" : "start_mongo_node/"));
                break;
            default:
                break;
        }
        setShowWarningPopup(target);
    }
    useEffect(() => { if (alerts?.length > 0) setFilteredAlerts(alerts) }, [alerts]);
    useEffect(() => {
        if (id) {
            dispatch(StatusActions.getMongoDetails(id));
        }
        return () => {
            dispatch(StatusActions.setMongoDetails(null));
            dispatch(LayoutActions.getNodes());
        }
    }, [id, dispatch])
    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);
    }, [state?.targetTab])
    const onRestore = (data) => {
        dispatch(LayoutActions.restoreBackupMongoNode(data, setShowRestore));
    }
    const onShowBackup = () => {
        setPageIndex(null);

        setShowBackup(true);
    }
    const onSetSchedule = () => {
        if (mongo_details?.node?.ind_deleted) showWarningMessage({ "text": `${mongo_details?.node?.name} is unavailable.`, 'status': mongo_details?.node?.status });
        else {
            setPageIndex(1);
            updateSchedule.current = mongo_details.backups?.length > 0 ? mongo_details?.backups?.filter((b) => b.is_sched === true)?.[0] : mongo_details?.nodes?.filter((s) => s?.backups?.filter((b) => b.is_sched === true))?.[0]?.backups?.filter((b) => b.is_sched === true)?.[0];
            setShowBackup(true);
        }
    }
    const onPopupAccept = () => {
        switch (showWarningPopup) {
            case "stop-down":
                onStopMongoNode();
                break;
            case "start-down":
                onStartMongoNode();
                break;
            case "start-stop":
                mongo_details?.node?.ind_started ? onStopMongoNode() : onStartMongoNode();
                break;
            case "step-down":
                stepDown();
                break;
            default:
                break;
        }
    }
    const stepDown = () => {
        dispatch(LayoutActions.stepDownMongoNode({ "node_id": mongo_details?.node?.id }))
    }
    const showWarningMessage = (data) => {
        setWarningMessage(data);
        setShowWarning(true);
    }
    const onStopMongoNode = () => dispatch(LayoutActions.stopMongoNode({ "node_id": mongo_details?.node?.id }, mongo_details?.node));
    const onStartMongoNode = () => dispatch(LayoutActions.startMongoNode({ "node_id": mongo_details?.node?.id }, mongo_details?.node));
    const onRestoreDump = (data) => {
        dispatch(LayoutActions.restoreDumpMongoNode(data, setShowRestoreDump));
    }
    // const dateBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{moment(item['t']['$date']).format("MMM DD,YYYY, hh:mm A")}</div>);
    // }
    // const typeBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item['c']}</div>);
    // }
    // const idBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item['id']}</div>);
    // }
    // const contextBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item["ctx"]}</div>);
    // }
    // const messageBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item["msg"]}</div>);
    // }
    const onDumpMongoNode = (data) => {
        dispatch(LayoutActions.dumpMongoNode(data, setShowDump));
    }
    const onBackupMongoNode = (data) => {
        dispatch(LayoutActions.backupMongoNode(data, setShowBackup));
    }
    const onDropMongoNode = (data) => {
        dispatch(LayoutActions.dropMongoNode(data, setShowDropMongoNode));
    }
    const attachPopup = (target) => mongo_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mongo_details?.node?.name} is unavailable.`, 'status': mongo_details?.node?.status }) : onShowPopup(target);
    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">
                <TabView activeIndex={toggle} onTabChange={handleTabChange}>
                    <TabPanel header="Dashboard">
                        <div className='grid flex-row'>
                            <div className='col-3 flex align-items-between flex-column' >
                                <h2 >
                                    {mongo_details?.node?.cluster_role === clusterRoles.STANDBY ? 'SECONDARY' : mongo_details?.node?.cluster_role === clusterRoles.PRIMARY && mongo_details?.node?.cluster_content?.ex_params?.primary_count < 1 ? 'SECONDARY' : mongo_details?.node?.cluster_role} {mongo_details?.node?.name}
                                </h2>
                                <div>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Node is {mongo_details?.node?.ind_started ? 'up' : "down"}
                                                </td>
                                                <td>
                                                    <Button
                                                        className="p-button-sm details-button"
                                                        disabled={!mongo_details?.node?.ind_started && !errorMessage}
                                                        id="start-stop"
                                                        icon={mongo_details?.node?.ind_started ? "pi pi-stop-circle" : "pi pi-play"}
                                                        style={{ minWidth: "53px", minHeight: "39px" }}
                                                        iconPos="right"
                                                        onClick={() => attachPopup("start-stop")}
                                                    />
                                                </td>
                                                <td>
                                                    <img src={mongo_details?.node?.ind_started ? green_star : red_star} alt="star" className="avail-star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {mongo_details?.ind_error ? "Errors in cluster" : mongo_details?.ind_error === false ? "Cluster is OK" : <div> <i className="pi pi-spin pi-spinner" /> Connecting to node... </div>}
                                                </td>
                                                <td>
                                                    <Button onClick={() => navigate('/layout', { state: { node_id: mongo_details?.node?.ex_params?.mongo_master_id } })} disabled={!mongo_details?.node} className='p-button-sm details-button'><img alt="info" width={16} height={16} src={info} /></Button>
                                                </td>
                                                <td>
                                                    <img className='status-star' src={(mongo_details?.ind_error === undefined || mongo_details?.ind_error === true) ? red_star : green_star} alt="star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {mongo_details?.auto_backup?.status ? mongo_details.auto_backup.status : ""}
                                                </td>
                                                <td>
                                                    <Button onClick={onSetSchedule} icon="pi pi-calendar-times" disabled={!mongo_details?.node?.ind_started && !errorMessage} className='p-button-sm details-button update-schedule-button'></Button>
                                                </td>
                                                <td>
                                                    <img className='set-schedule-star' src={getStarByClass(mongo_details?.auto_backup?.class)} alt="star"></img>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <div>
                                    <h3>
                                        General info
                                    </h3 >
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td>{mongo_details?.node?.name}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    DB ID
                                                </td>
                                                <td> {mongo_details?.db_id}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Version
                                                </td>
                                                <td> {mongo_details?.version}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Log Level
                                                </td>
                                                <td> {mongo_details?.log_level}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Read Only
                                                </td>
                                                <td> {mongo_details?.read_only}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Size
                                                </td>
                                                <td>  {mongo_details?.display_size}</td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <ClusterSizing node={mongo_details?.node} />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>
                                <StatsCard details={mongo_details}></StatsCard>
                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={mongo_details?.node?.cpuUsages}
                                    memUsages={mongo_details?.node?.memUsages}
                                />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>
                                <div className='col-9'>
                                    <span className="p-buttonset">
                                        < Button className="p-button-sm details-button up" disabled={(!mongo_details?.node?.ind_started && !errorMessage) || mongo_details?.node?.cluster_role === clusterRoles.ARBITER} label="Backup" onClick={() => mongo_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mongo_details?.node?.name} is unavailable.`, 'status': mongo_details?.node?.status }) : onShowBackup()}>
                                            < img alt="backup" width={16} height={16} src={backup_pdb} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={mongo_details?.node?.cluster_role !== clusterRoles.PRIMARY || (!mongo_details?.node?.ind_started && !errorMessage)} label="Restore" onClick={() => mongo_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mongo_details?.node?.name} is unavailable.`, 'status': mongo_details?.node?.status }) : setShowRestore(true)}>
                                            < img alt="restore" width={16} height={16} src={restore} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={(!mongo_details?.node?.ind_started && !errorMessage) || mongo_details?.node?.cluster_role === clusterRoles.ARBITER} label="Export" onClick={() => mongo_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mongo_details?.node?.name} is unavailable.`, 'status': mongo_details?.node?.status }) : setShowDump(true)}>
                                            < img alt="export" width={16} height={16} src={icon_export} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={(!mongo_details?.node?.ind_started && !errorMessage) || mongo_details?.node?.cluster_role !== clusterRoles.PRIMARY} label="Restore Dump" onClick={() => mongo_details?.node?.ind_deleted ? showWarningMessage({ "text": `${mongo_details?.node?.name} is unavailable.`, 'status': mongo_details?.node?.status }) : setShowRestoreDump(true)}>
                                            < img alt="import" width={16} height={16} src={icon_import} />
                                        </Button>
                                        <Button disabled={(!mongo_details?.node?.ind_started && !errorMessage)} className="p-button-sm details-button up" label="Credentials">
                                            <img alt="credentials" width={16} height={16} src={credentials} ></img>
                                        </Button>
                                        <Button className="p-button-sm details-button up" icon="pi pi-arrow-down"
                                            iconPos="right"
                                            label={"Step down"}
                                            id="step-down"
                                            disabled={mongo_details?.node?.cluster_role !== clusterRoles.PRIMARY || (!mongo_details?.node?.ind_started && !errorMessage) || (mongo_details?.nodes?.length === 0)}
                                            onClick={() => attachPopup("step-down")}
                                        />
                                    </span>
                                    <span className="p-buttonset down">
                                        <Button disabled={errorMessage || !mongo_details?.node?.ind_started} icon="pi pi-stop-circle" iconPos="right" className="p-button-sm details-button" id="stop-down" label="Stop" onClick={() => attachPopup("stop-down")} />
                                        <Button id="start-down" disabled={!mongo_details} icon={`pi pi-${mongo_details?.node?.ind_started ? "replay" : "play"}`} iconPos="right" className="p-button-sm details-button update-schedule-button" label={mongo_details?.node?.ind_started ? "Restart" : "Start"} onClick={() => attachPopup("start-down")}>
                                        </Button>
                                        <Button disabled={!mongo_details?.node?.ind_started && !errorMessage} className="p-button-sm details-button" label="Drop" onClick={() => mongo_details?.node?.cluster_role === clusterRoles.PRIMARY && mongo_details?.nodes?.length === 1 ? showWarningMessage({ "text": `Drop primary with a single secondary may break replica set operation.\nEither drop existing secondary, or add new.` }) : mongo_details?.node?.cluster_role === clusterRoles.PRIMARY && mongo_details?.arbiter && mongo_details?.nodes?.length < 1 ? showWarningMessage({ "text": `Drop of primary with only arbiter nodes is not allowed.\nDrop arbiter nodes from layout page.` }) : setShowDropMongoNode(true)}>
                                            < img alt="drop" width={16} height={16} src={drop} />
                                        </Button>
                                    </span>
                                </div>
                            </div>
                        </div>
                    </TabPanel>
                    <TabPanel header="Activity Logs">
                        <span className="p-input-icon-right">
                            <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                setUpperSearch("");
                            }} ></i>
                            <InputText className="p-inputtext-sm filter-field mr-2- mb-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                        </span>

                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <LogsTable filteredLogs={filteredLogs}
                            timePeriodType={timePeriodType}
                            node_id={id}
                        />
                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} paginator alwaysShowPaginator={false} rows={100} >
                            <Column className='history-column' header="Date" field='t.$date' body={dateBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Type" field='c' body={typeBodyTemplate} sortable></Column>
                            <Column className='history-column' header={mongo_details?.node?.version > 4.4 ? "ID" : "Severity"} field='id' sortable body={idBodyTemplate}></Column>
                            <Column className='history-column' header="Context" field='ctx' body={contextBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Message" field='msg' body={messageBodyTemplate} sortable></Column>
                        </DataTable> */}
                    </TabPanel>
                    <TabPanel header="Database Logs">
                        <LogFilesTable node_id={id} />
                    </TabPanel>
                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>
                        <NodeAlertsTable node={mongo_details?.node} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />
                    </TabPanel>
                    <TabPanel header="Users">
                        <div className="flex justify-content-start">
                            <span className="p-input-icon-right">
                                <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => setSearch("")} />
                                <InputText className="p-inputtext-sm filter-field mb-2"
                                    placeholder="Search"
                                    value={search}
                                    onChange={(e) => {
                                        setSearch(e.target.value);
                                    }} />
                            </span>

                        </div>
                        <DatabaseUsersTable node={mongo_details?.node} users={filteredUsers} />
                    </TabPanel>
                    <TabPanel headerClassName='filters-header' header={<div className='flex'>

                        {toggle < 4 ? <Dropdown
                            className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period raised-dropdown" value={timePeriodType}
                            options={timePeriodTypes}
                            onChange={(e) => {
                                setTimePeriodType(e.value);
                                setMonitorInterval(intervalByTimedelta(e.value));
                            }}
                        /> : null}
                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => {
                                dispatch(StatusActions.setMongoDetails(null))
                                dispatch(StatusActions.getMongoDetails(id))
                            }}
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>
                </TabView>
                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}
                {(showDropMongoNode && mongo_details) ? <DropMongoNodeDialog
                    node={mongo_details?.node}
                    arbiter={mongo_details?.arbiter}
                    nodes_count={mongo_details?.arbiter ? mongo_details?.nodes.filter((s) => s?.ex_params?.voting === true)?.length + 1 : mongo_details?.nodes.filter((s) => s?.ex_params?.voting === true)?.length}
                    visible={showDropMongoNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowDropMongoNode(false)
                    }}
                    onOkClick={onDropMongoNode}
                />
                    : null}
                {(ShowDump && mongo_details) ? <MongoDumpDialog
                    node={mongo_details?.node}
                    visible={ShowDump}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        dispatch(LayoutActions.setDirExists(null));
                        dispatch(LayoutActions.setExportExists(null));
                        dispatch(LayoutActions.setDiskSpace(null));
                        setShowDump(false)
                    }}
                    onOkClick={onDumpMongoNode}
                />
                    : null}
                {(showBackup && mongo_details) ? <MongoNodeBackupDialog
                    node={mongo_details?.node}
                    visible={showBackup}
                    updateSchedule={updateSchedule.current}
                    pageIndex={pageIndex}
                    backups={mongo_details?.backups}
                    nodes={mongo_details?.nodes}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowBackup(false)
                    }}
                    onOkClick={onBackupMongoNode}
                />
                    : null}
                {(showRestore && mongo_details) ? <RestoreBackupMongoNode
                    data={mongo_details}
                    visible={showRestore}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestore(false)
                    }}
                    onOkClick={onRestore}
                /> : null}
                {(showRestoreDump && mongo_details) ? <RestoreDumpMongoNode
                    data={mongo_details}
                    visible={showRestoreDump}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestoreDump(false)
                    }}
                    onOkClick={onRestoreDump}
                /> : null}
            </div>
            <ConfirmPopup
                icon="pi pi-exclamation-triangle"
                target={document.getElementById(showWarningPopup)}
                visible={Boolean(showWarningPopup)}
                onHide={() => setShowWarningPopup(false)}
                accept={onPopupAccept}
                message={`Are you sure you want to ${popupAction} the node?`} />
            {mongo_details && <Tooltip target=".set-schedule-star" content={mongo_details?.auto_backup?.detail}></Tooltip>}
            {mongo_details && <Tooltip target=".avail-star" content={mongo_details?.node?.ind_started ? "Database is running and accepting connections" : "Database is not running"}></Tooltip>}
            {mongo_details && <Tooltip target=".status-star" content={(mongo_details?.ind_error === undefined || mongo_details?.ind_error === true) ? "Some nodes of cluster are down/inaccessible" : "All nodes of cluster are accessible"}></Tooltip>}
        </Fragment>
    )
}
