import icon_import from '../../../assets/images/import.svg';
import icon_export from '../../../assets/images/export.svg';
import backup_pdb from '../../../assets/images/backup_pdb.svg';
import flashback from '../../../assets/images/flashback.svg';
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation } from "react-router-dom";
import { StatusActions } from "../../../store/status/reducer";
import { LayoutActions } from '../../../store/layout/reducer';
import { Fragment, useEffect, useState, useCallback } from "react";
import BackupCDBDialog from '../../../dialogs/oracle/backupCDBDialog';
import ExportCDBPDBDialog from '../../../dialogs/oracle/exportCDBPDBDialog';
import FlashbackCDBPDBDialog from '../../../dialogs/oracle/flashbackPDBCDBDialog';
import ImportCDBPDBDialog from '../../../dialogs/oracle/importCDBPDBDialog';
import { TabView, TabPanel } from 'primereact/tabview';
import { Dropdown } from "primereact/dropdown";
import { Button } from 'primereact/button';
import DetailsMenu from '../../../components/menus/detailsMenu';
import WarningDialog from '../../../dialogs/warningDialog';
import PerformanceCharts from '../../../components/performanceCharts';
import { timePeriodTypes } from '../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../components/tables/nodeAlertsTable';
import CDBSettingsTable from '../../../components/tables/cdbSettingsTable';
import StatsCard from '../../../components/statsCard';
import { InputText } from 'primereact/inputtext';
import { searchFilter } from '../../monitoring';
import { ButtonGroup } from 'primereact/buttongroup';
import { AppActions } from '../../../store/app/reducer';
import ClusterSizing from '../../../components/changeClusterSizingBlock';
import { searchFilter as logSearch } from '../../log';
import LogsTable from '../../../components/tables/logsTable';
import LogFilesTable from '../../../components/tables/logFilesTable';


export default function CDBStatusInfo(props) {
    const { cdb_details } = useSelector((state) => state.status);
    const dispatch = useDispatch();
    const { pk } = useParams();
    const { state } = useLocation();
    const { severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);
    const [upperSearch, setUpperSearch] = useState("");
    const [showBackup, setShowBackup] = useState(false);
    const [showExport, setShowExport] = useState(false);
    const [showRestorePoint, setShowRestorePoint] = useState(false);
    const [showImport, setShowImport] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [flashbackTooltip, setFlashbackTooltip] = useState(" ");
    const [flashbackForbidden, setFlasbackForbidden] = useState(true);
    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [toggle, setToggle] = useState(0);
    const { alerts } = useSelector((state) => state.monitoring);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [filteredLogs, setFilteredLogs] = useState(null);

    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);
    const { node_logs } = useSelector((state) => state.log);

    useEffect(() => {
        const storedTabIndex = localStorage.getItem('DetailsTabIndex');
        if (storedTabIndex) {
            setToggle(parseInt(storedTabIndex, 10));
        }
    }, []);

    const handleTabChange = (e) => {
        setToggle(e.index);
        localStorage.setItem('DetailsTabIndex', e.index);
    };


    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }

        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];

    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);

    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);
    }, [state?.targetTab])

    useEffect(() => {
        if (pk) {
            dispatch(StatusActions.getCdbDetails(pk));
        }
        return () => {
            dispatch(StatusActions.setCdbDetails(null));
            dispatch(LayoutActions.getNodes());
        }
    }, [pk, dispatch]);

    useEffect(() => {
        if (cdb_details?.node?.version) {
            if (parseFloat(cdb_details.node.version) <= 12.1) {
                setFlashbackTooltip(`CDB version ${cdb_details.node.version} does not support flashback`);
                setFlasbackForbidden(true);
            }
            else {
                setFlashbackTooltip(" ");
                setFlasbackForbidden(false);
            }
        }
        else if (!cdb_details && cdb_details !== undefined) {
            if (pk) {
                dispatch(StatusActions.getCdbDetails(pk));
            }
        }
    }, [cdb_details, pk, dispatch]);

    useEffect(() => {


        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;
        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            setFilteredAlerts(tempData);
            tempData = severityFilter(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);





    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen])


    // useEffect(() => {

    //     if (cdb_details?.history?.length > 0) {

    //         const query = upperSearch.toLowerCase().trim();
    //         if (!query) setFilteredLogs(cdb_details.history);
    //         else setFilteredLogs(cdb_details.history.filter((h) => h[2].format("MMM DD,YYYY, hh:mm A").includes(query)
    //             || h[0].toLowerCase().includes(query)
    //             || h[3].toLowerCase().includes(query)
    //             || h[4].toLowerCase().includes(query)
    //             || h[1].toLowerCase().includes(query)))
    //     }

    // },
    //     [upperSearch, cdb_details?.history])

    const showWarningMessage = (data) => {
        setWarningMessage(data);
        setShowWarning(true);
    }
    const onImport = (data) => {
        dispatch(LayoutActions.importCDB(data, showImport));
    }
    const onBackup = (data) => {
        dispatch(LayoutActions.backupCDB(data, setShowBackup));
    }
    const onExport = (data) => {
        dispatch(LayoutActions.exportNode(data, setShowExport));
        dispatch(LayoutActions.setSchemas(undefined));
        dispatch(LayoutActions.setDisks(undefined));
    }
    const onRestorePoint = (data) => {
        dispatch(LayoutActions.flashbackNode(data, setShowRestorePoint));
    }
    // const dateBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{moment(item[2]).format("MMM DD,YYYY, hh:mm A")}</div>);
    // }
    // const nameBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[0]}</div>);
    // }
    // const idBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[1]}</div>);
    // }
    // const operationBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[3]}</div>);
    // }
    // const clonedFromBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[4]}</div>);
    // }
    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">
                <TabView activeIndex={toggle} onTabChange={handleTabChange}>
                    <TabPanel header="Dashboard">
                        <div className='grid flex-row'>
                            <div className='col-3 '>
                                <h2>
                                    {cdb_details?.node?.location}
                                </h2>
                                <div>
                                    <h3>
                                        General info
                                    </h3>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td> {cdb_details?.info?.[1]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    DB ID
                                                </td>
                                                <td>{cdb_details?.info?.[0]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Size
                                                </td>
                                                <td> {cdb_details?.node?.display_size}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Creation SCN
                                                </td>
                                                <td>{cdb_details?.info?.[2]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Logging
                                                </td>
                                                <td>{cdb_details?.info?.[5]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Force logging
                                                </td>
                                                <td> {cdb_details?.info?.[3]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Open mode
                                                </td>
                                                <td>{cdb_details?.info?.[4]}</td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <ClusterSizing node={cdb_details?.node} />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>
                                <StatsCard details={cdb_details}></StatsCard>
                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={cdb_details?.node?.cpuUsages}
                                    memUsages={cdb_details?.node?.memUsages}
                                />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>
                                <div className='col-9'>
                                    <span className="p-buttonset">
                                        < Button className="p-button-sm details-button up cdb-flashback"
                                            disabled={!cdb_details?.node}
                                            label="Backup"
                                            onClick={() => {
                                                cdb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${cdb_details?.node.name} is unavailable.`, 'status': cdb_details?.node.status }) : setShowBackup(true)
                                            }}>
                                            < img alt="backup" width={16} height={16} src={backup_pdb} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={flashbackForbidden} tooltip={flashbackTooltip} tooltipOptions={{ showOnDisabled: true, position: "left" }} label="Create restore point" onClick={() => {
                                            cdb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${cdb_details?.node.name} is unavailable.`, 'status': cdb_details?.node.status }) : setShowRestorePoint(true)
                                        }}>
                                            < img alt="flashback" width={16} height={16} src={flashback} />
                                        </Button>
                                        <Button className="p-button-sm details-button up"
                                            label="Export"
                                            disabled={!cdb_details?.node}
                                            onClick={() => {
                                                cdb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${cdb_details?.node.name} is unavailable.`, 'status': cdb_details?.node.status }) : setShowExport(true)
                                            }}>
                                            < img alt="export" width={16} height={16} src={icon_export} />
                                        </Button>
                                        <Button
                                            className="p-button-sm details-button up"
                                            label="Import"
                                            tooltipOptions={{ showOnDisabled: true, position: "left" }}
                                            onClick={() => {
                                                cdb_details?.node?.ind_deleted ? showWarningMessage({ "text": `${cdb_details?.node.name} is unavailable.`, 'status': cdb_details?.node.status }) : setShowImport(true)
                                            }}>
                                            < img alt="import" width={16} height={16} src={icon_import} />
                                        </Button>
                                    </span>
                                </div>
                            </div>
                        </div>
                    </TabPanel>
                    <TabPanel header="Activity Logs">
                        <span className="p-input-icon-right">
                            <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                setUpperSearch("");
                            }} ></i>
                            <InputText className="p-inputtext-sm filter-field mr-2- mb-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                        </span>

                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <LogsTable filteredLogs={filteredLogs}
                            timePeriodType={timePeriodType}
                            node_id={pk}
                        />

                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} paginator rows={100} alwaysShowPaginator={false} >
                            <Column className='history-column' header="Date" body={dateBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Name" body={nameBodyTemplate} sortable></Column>
                            <Column className='history-column' header="ID" sortable body={idBodyTemplate}></Column>
                            <Column className='history-column' header="Operation" body={operationBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Cloned from" body={clonedFromBodyTemplate} sortable></Column>
                        </DataTable> */}
                    </TabPanel>
                    <TabPanel header="Database Logs">
                        <LogFilesTable node_id={pk} />
                    </TabPanel>
                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>
                        <NodeAlertsTable node={cdb_details?.node} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />
                    </TabPanel>
                    <TabPanel header="Parameters">
                        <CDBSettingsTable node={cdb_details?.node} />
                    </TabPanel>
                    <TabPanel headerClassName='filters-header' header={<div className='flex'>

                        {toggle < 4 ? <Dropdown
                            className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period raised-dropdown" value={timePeriodType}
                            options={timePeriodTypes}
                            onChange={(e) => {
                                setTimePeriodType(e.value);
                                setMonitorInterval(intervalByTimedelta(e.value));
                            }}
                        /> : null}
                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => {
                                dispatch(StatusActions.setCdbDetails(null));
                                dispatch(StatusActions.getCdbDetails(pk));
                            }}
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>
                </TabView>
                {cdb_details?.node && showBackup ?
                    <BackupCDBDialog visible={showBackup}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowBackup(false)
                        }}
                        onOkClick={onBackup}
                        node={cdb_details?.node}
                    />
                    : null
                }
                {cdb_details?.node && showExport ?
                    <ExportCDBPDBDialog visible={showExport}
                        onHide={() => {
                            dispatch(LayoutActions.setSchemas(undefined));
                            dispatch(LayoutActions.setDisks(undefined));
                            dispatch(LayoutActions.setDirExists(null));
                            dispatch(LayoutActions.setExportExists(null));
                            dispatch(LayoutActions.setDiskSpace(null));
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowExport(false);
                        }}
                        onOkClick={onExport}
                        node={cdb_details?.node}
                    />
                    : null
                }
                {cdb_details?.node && showRestorePoint ?
                    <FlashbackCDBPDBDialog visible={showRestorePoint}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowRestorePoint(false)
                        }}
                        onOkClick={onRestorePoint}
                        node={cdb_details?.node}
                    />
                    : null
                }
                {cdb_details?.node && showImport ?
                    <ImportCDBPDBDialog visible={showImport}
                        onHide={() => {
                            dispatch(LayoutActions.setUserExists(null));
                            dispatch(LayoutActions.setDumpExists(null));
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowImport(false);
                        }
                        }
                        onOkClick={onImport}
                        node={cdb_details?.node}
                        exports={cdb_details.exports}
                    />
                    : null}
                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}
            </div>
        </Fragment>
    );
}
