import restore from '../../../assets/images/restore_pdb.svg'
import backup_pdb from '../../../assets/images/backup_pdb.svg'
import icon_import from '../../../assets/images/import_pdb.svg'
import icon_export from '../../../assets/images/export_pdb.svg'
import drop from '../../../assets/images/drop.svg'
import icon_close from '../../../assets/images/close.svg'
import icon_open from '../../../assets/images/open.svg'
import icon_warning from '../../../assets/images/warning.svg'
import flashback from '../../../assets/images/flashback.svg'
import restore_flashback from '../../../assets/images/flashback_restore.svg'
import copy_another_cdb from '../../../assets/images/copy_another_cdb.svg'
import rename from '../../../assets/images/rename.svg'
import info from '../../../assets/images/info.svg'
import red_star from '../../../assets/images/red-star.png'
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation, useNavigate } from "react-router-dom";
import { StatusActions } from "../../../store/status/reducer";
import { Fragment, useEffect, useState, useCallback, useRef } from "react";
import { Button } from 'primereact/button';
import BackupPDBDialog from '../../../dialogs/oracle/backupPDBDialog';
import RenamePDBDialog from '../../../dialogs/oracle/renamePDBDialog';
import ExportCDBPDBDialog from '../../../dialogs/oracle/exportCDBPDBDialog';
import CopyPDBDialog from '../../../dialogs/oracle/copyPDBDialog';
import DeletePDBDialog from '../../../dialogs/oracle/deletePDBDialog';
import CopytoCDBDialog from '../../../dialogs/oracle/copytoCDBDialog';
import RestorePDBDialog from '../../../dialogs/oracle/restorePDBDialog';
import ClosePDBDialog from '../../../dialogs/oracle/closePDBDialog';
import ImportCDBPDBDialog from '../../../dialogs/oracle/importCDBPDBDialog';
import FlashbackCDBPDBDialog from '../../../dialogs/oracle/flashbackPDBCDBDialog';
import RestorePDBFlashbackDialog from '../../../dialogs/oracle/restorePDBFlashbackDialog';
import OpenPDBDialog from '../../../dialogs/oracle/openPDBDialog';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dropdown } from "primereact/dropdown";
import { TabView, TabPanel } from 'primereact/tabview';
import DetailsMenu from '../../../components/menus/detailsMenu';
import credentials from "../../../assets/images/credentials.svg";
import CredentialsDialog from '../../../dialogs/oracle/credentialsDialog';
import WarningDialog from '../../../dialogs/warningDialog';
import PerformanceCharts from '../../../components/performanceCharts'
import getStarByClass from '../../../util/getStarByClass'
import { Tooltip } from 'primereact/tooltip'
import { timePeriodTypes } from '../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../components/tables/nodeAlertsTable';
import { InputText } from 'primereact/inputtext';
import { searchFilter } from '../../monitoring';
import { ButtonGroup } from 'primereact/buttongroup';
import { AppActions } from '../../../store/app/reducer';
import ClusterSizing from '../../../components/changeClusterSizingBlock';
import { searchFilter as logSearch } from '../../log';
import LogsTable from '../../../components/tables/logsTable';
import LogFilesTable from '../../../components/tables/logFilesTable';


export default function PDBStatusInfo(props) {
    const { pk, id } = useParams();
    const { state } = useLocation();
    const navigate = useNavigate();
    const { pdb_details } = useSelector((state) => state.status);
    const dispatch = useDispatch();
    const { severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);
    const [upperSearch, setUpperSearch] = useState("");
    const [showBackup, setShowBackup] = useState(false);
    const updateSchedule = useRef(null);
    const [showRename, setShowRename] = useState(false);
    const [showExport, setShowExport] = useState(false);
    const [showCopy, setShowCopy] = useState(false);
    const [showDelete, setShowDelete] = useState(false);
    const [showCopyTo, setShowCopyTo] = useState(false);
    const [showRestore, setShowRestore] = useState(false);
    const [showClose, setShowClose] = useState(false);
    const [showImport, setShowImport] = useState(false);
    const [showFlashback, setShowFlashback] = useState(false);
    const [showRestoreFlashback, setShowRestoreFlashback] = useState(false);
    const [showOpen, setShowOpen] = useState(false);
    const [showCredentials, setShowCredentials] = useState(false);
    const [toggle, setToggle] = useState(0);
    const [openForbidden, setOpenForbidden] = useState(true);
    const [openDisabled, setOpenDisabled] = useState(true);
    const [closeDisabled, setCloseDisabled] = useState(true);
    const [flashbackForbidden, setFlasbackForbidden] = useState(true);
    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [pageIndex, setPageIndex] = useState(null);
    const { alerts } = useSelector((state) => state.monitoring);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [filteredLogs, setFilteredLogs] = useState(null);

    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);
    const { node_logs } = useSelector((state) => state.log);

    useEffect(() => {
        const storedTabIndex = localStorage.getItem('DetailsTabIndex');
        if (storedTabIndex) {
            setToggle(parseInt(storedTabIndex, 10));
        }
    }, []);

    const handleTabChange = (e) => {
        setToggle(e.index);
        localStorage.setItem('DetailsTabIndex', e.index);
    };

    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }

        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];

    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);
    useEffect(() => {

        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;

        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            if (!showSeverityGreen || !showSeverityYellow || !showSeverityRed) tempData = severityFilter(tempData);
            setFilteredAlerts(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);

    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen])

    // useEffect(() => {

    //     if (pdb_details?.history?.length > 0) {

    //         const query = upperSearch.toLowerCase().trim();
    //         if (!query) setFilteredLogs(pdb_details.history);
    //         else setFilteredLogs(pdb_details.history.filter((h) => h[2].format("MMM DD,YYYY, hh:mm A").includes(query)
    //             || h[0].toLowerCase().includes(query)
    //             || h[3].toLowerCase().includes(query)
    //             || h[4].toLowerCase().includes(query)
    //             || h[1].toLowerCase().includes(query)))
    //     }

    // },
    //     [upperSearch, pdb_details?.history])

    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);
    }, [state?.targetTab])
    const showWarningMessage = (data) => {
        setWarningMessage(data);
        setShowWarning(true);
    }

    useEffect(() => {
        if (pk && id) {
            dispatch(StatusActions.getPdbDetails(
                {
                    "pk": pk,
                    "id": id
                }
            ));
        }
        return () => {
            dispatch(StatusActions.setPdbDetails(null));
            dispatch(LayoutActions.getNodes());

        };
    }, [pk, id, dispatch]);

    const onShowBackup = () => {
        setPageIndex(null);

        setShowBackup(true);
    }
    const openTooltip = "Max number of open PDBs reached, please close other PDBs to open this one.";
    const onRenamePDB = (data) => {
        dispatch(LayoutActions.renamePDB(data, setShowRename));
        setShowRename(false);
    }
    const onClosePDB = (data) => {
        dispatch(LayoutActions.closePDBNode(data, setShowClose));
    }
    const onOpenPDB = (data) => {
        dispatch(LayoutActions.openPDBNode(data, setShowOpen));
    }
    const onCopyToCDB = (data) => {
        dispatch(LayoutActions.copyToCDB(data, setShowCopyTo));
    }
    const onCopyPDB = (data) => {
        dispatch(LayoutActions.copyPDB(data, setShowCopy));
    }
    const onSetSchedule = () => {
        if (pdb_details?.pdb?.ind_deleted) showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status });
        else {
            setPageIndex(1);
            updateSchedule.current = pdb_details?.backups?.filter((b) => b.is_sched === true)?.[0];
            setShowBackup(true);
        }

    }
    const onFlashback = (data) => {
        dispatch(LayoutActions.flashbackNode(data, setShowFlashback));
    }

    const onRestoreFlashback = (data) => {
        dispatch(LayoutActions.restoreFlashback(data, setShowRestoreFlashback));
    }
    const onRestorePDB = (data) => {
        dispatch(LayoutActions.restorePDB(data, setShowRestore));
    }
    const onBackupPDB = (data) => {
        dispatch(LayoutActions.backupPDB(data, setShowBackup));
    }
    const onDeletePDB = (data) => {
        dispatch(LayoutActions.deletePDBNode(data, setShowDelete));
    }
    const onExport = (data) => {
        dispatch(LayoutActions.exportNode(data, setShowExport));
    }
    const onImport = (data) => {
        dispatch(LayoutActions.importPDB(data, setShowImport));
        dispatch(LayoutActions.setSchemas(undefined));
        dispatch(LayoutActions.setDisks(undefined));
    }
    const onUpdateCredentials = (data) => {
        dispatch(LayoutActions.updatePDBCredentials(data, setShowCredentials));
    }
    // const dateBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{moment(item[2]).format("MMM DD,YYYY, hh:mm A")}</div>);
    // }
    // const nameBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[0]}</div>);
    // }
    // const idBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[1]}</div>);
    // }
    // const operationBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[3]}</div>);
    // }
    // const clonedFromBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[4]}</div>);
    // }
    useEffect(() => {
        if (pdb_details) {
            if (pdb_details?.pdb?.ind_started) {
                setOpenDisabled(true);
                setCloseDisabled(false)
            }
            else {
                setOpenDisabled(false);
                setCloseDisabled(true);
            }
            if (pdb_details.can_be_opened) setOpenForbidden(false);
            else {
                setOpenForbidden(true);
            }
            if (parseFloat(pdb_details.cdb?.version) <= 12.1) {
                setFlasbackForbidden(true);
            }
            else {
                setFlasbackForbidden(false);
            }
        }
        else if (!pdb_details && pdb_details !== undefined) {
            if (pk && id) {
                dispatch(StatusActions.getPdbDetails(
                    {
                        "pk": pk,
                        "id": id
                    }
                ));
            }

        }
    }, [pdb_details, pk, id, dispatch]);


    const onCloseClick = () => {
        pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowClose(true)
    }
    const onOpenClick = () => {
        if (pdb_details?.pdb?.ind_deleted) {
            showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status });
            return;
        }
        if (!openForbidden) {
            setShowOpen(true);
        }
    }
    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">
                <TabView activeIndex={toggle} onTabChange={handleTabChange}>
                    <TabPanel header="Dashboard">
                        <div className='grid flex-row'>
                            <div className='col-3 flex align-items-between flex-column' >
                                <h2>
                                    {pdb_details ? pdb_details?.pdb?.location : null}
                                </h2>
                                <div>
                                    <button className='button-text' style={{ margin: "0 0 1rem 0" }}> HA Rating {pdb_details?.ha_rating ? pdb_details?.ha_rating : 0} of 3</button>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Database is {pdb_details?.ind_started?.status ? pdb_details.ind_started.status : null}
                                                </td>
                                                <td>
                                                    <Button disabled={!pdb_details?.pdb} tooltip={pdb_details?.pdb && openForbidden && !pdb_details?.pdb?.ind_started ? openTooltip : ""} onClick={!pdb_details?.pdb?.ind_started ? onOpenClick : onCloseClick} tooltipOptions={{ position: "right" }} className='p-button-sm details-button'><img alt="open-close" width={16} height={16} src={pdb_details?.pdb?.ind_started ? icon_close : openForbidden ? icon_warning : icon_open} /></Button>
                                                </td>
                                                <td>
                                                    {<img className='status-star' src={getStarByClass(pdb_details?.ind_started?.class)} alt="star"></img>}
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {pdb_details?.auto_backup?.status ? pdb_details.auto_backup.status : ""}
                                                </td>
                                                <td>
                                                    {<Button onClick={onSetSchedule} icon="pi pi-calendar-times" disabled={!pdb_details?.pdb} className='p-button-sm details-button update-schedule-button'></Button>}
                                                </td>
                                                <td>
                                                    <img className='set-schedule-star' src={getStarByClass(pdb_details?.auto_backup?.class)} alt="star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Standby isn't configured
                                                </td>
                                                <td>
                                                    <Button onClick={() => navigate('/layout', { state: { node_id: pdb_details?.cdb?.id } })} disabled={!pdb_details?.pdb} className='p-button-sm details-button'><img alt="info" width={16} height={16} src={info} /></Button>
                                                </td>
                                                <td>
                                                    <img src={red_star} alt="star"></img>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <div>
                                    <h3>
                                        General info
                                    </h3 >
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td> {pdb_details?.info?.[1]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    DB ID
                                                </td>
                                                <td>{pdb_details?.info?.[0]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Creation SCN
                                                </td>
                                                <td>{pdb_details?.info?.[2]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Logging
                                                </td>
                                                <td>{pdb_details?.info?.[5]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Force logging
                                                </td>
                                                <td> {pdb_details?.info?.[3]}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Open mode
                                                </td>
                                                <td>{pdb_details?.open_mode}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Size
                                                </td>
                                                <td>  {pdb_details?.pdb?.display_size} </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <ClusterSizing node={pdb_details?.pdb} />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>
                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={pdb_details?.pdb?.cpuUsages}
                                    memUsages={pdb_details?.pdb?.memUsages}
                                />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>
                                <div className='col-9'>
                                    <span className="p-buttonset">
                                        <Button
                                            disabled={!pdb_details?.pdb}
                                            className="p-button-sm details-button up" label="Rename" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowRename(true)
                                            }}>
                                            <img alt="rename" width={16} height={16} src={rename} />
                                        </Button>
                                        <Button
                                            disabled={!pdb_details?.pdb}
                                            className="p-button-sm details-button up" label="Copy to another CDB" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowCopyTo(true)
                                            }}>
                                            < img alt="copy_to" width={16} height={16} src={copy_another_cdb} />
                                        </Button>
                                        < Button
                                            disabled={!pdb_details?.pdb}
                                            className="p-button-sm details-button up" label="Backup" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : onShowBackup()
                                            }}>
                                            < img alt="backup" width={16} height={16} src={backup_pdb} />
                                        </Button>
                                        < Button disabled={!pdb_details?.pdb} className="p-button-sm details-button up" label="Restore" onClick={() => {
                                            pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowRestore(true)
                                        }}>
                                            < img alt="restore" width={16} height={16} src={restore} />
                                        </Button>
                                        <Button
                                            disabled={!pdb_details?.pdb}
                                            className="p-button-sm details-button up" label="Export" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowExport(true)
                                            }}>
                                            < img alt="export" width={16} height={16} src={icon_export} />
                                        </Button>
                                        <Button
                                            disabled={!pdb_details?.pdb}
                                            className="p-button-sm details-button up" label="Import" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowImport(true)
                                            }}>
                                            < img alt="import" width={16} height={16} src={icon_import} />
                                        </Button>
                                        <Button
                                            className="p-button-sm details-button up" tooltip={flashbackForbidden && pdb_details?.cdb?.version ? `CDB version ${pdb_details?.cdb.version} does not support flashbacks` : ""} tooltipOptions={{ showOnDisabled: true, position: "left" }} disabled={flashbackForbidden || !pdb_details?.pdb} label="Create flashback" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowFlashback(true)
                                            }}>
                                            < img alt="flashback" width={16} height={16} src={flashback} />
                                        </Button>
                                        <Button
                                            className="p-button-sm details-button up" disabled={flashbackForbidden || !pdb_details?.pdb} tooltip={flashbackForbidden && pdb_details?.cdb?.version ? `CDB version ${pdb_details.cdb.version} does not support flashbacks` : ""} tooltipOptions={{ showOnDisabled: true, position: "left" }} style={{ backgroundColor: "#6c757d", color: "white", border: "1px solid #6c757d", borderRadius: "5px", display: "flex", justifyContent: "space-between", width: "100%", margin: "5px 0px 5px 0px", textAlign: "left" }} label="Restore flashback" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowRestoreFlashback(true)
                                            }}>
                                            < img alt="restore flashback" width={16} height={16} src={restore_flashback} />
                                        </Button>
                                        <Button
                                            disabled={!pdb_details?.pdb}
                                            className="p-button-sm details-button up" label="Credentials" onClick={() => setShowCredentials(true)}>
                                            <img alt="credentials" width={16} height={16} src={credentials} ></img>
                                        </Button>
                                    </span>
                                    <span className="p-buttonset down">
                                        <Button disabled={closeDisabled || !pdb_details?.pdb} className="p-button-sm details-button" label="Close" onClick={onCloseClick}>
                                            <img alt="close" width={16} height={16} src={icon_close} />
                                        </Button>
                                        <Button className="p-button-sm details-button" disabled={openDisabled || !pdb_details?.pdb} tooltip={pdb_details?.pdb && openForbidden && !pdb_details?.pdb?.ind_started ? openTooltip : ""} tooltipOptions={{ position: "right" }} label="Open" onClick={onOpenClick}>
                                            < img alt="open" width={16} height={16} src={openForbidden ? icon_warning : icon_open} />
                                        </Button>
                                        <Button className="p-button-sm details-button"
                                            disabled={!pdb_details?.pdb}
                                            label="Drop" onClick={() => {
                                                pdb_details?.pdb?.ind_deleted ? showWarningMessage({ "text": `${pdb_details?.pdb?.name} is unavailable.`, 'status': pdb_details?.pdb?.status }) : setShowDelete(true)
                                            }}>
                                            < img alt="drop" width={16} height={16} src={drop} />
                                        </Button>
                                    </span>
                                </div>
                            </div>
                        </div>
                    </TabPanel>
                    <TabPanel header="Activity Logs">
                        <span className="p-input-icon-right">
                            <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                setUpperSearch("");
                            }} ></i>
                            <InputText className="p-inputtext-sm filter-field mr-2- mb-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                        </span>

                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <LogsTable filteredLogs={filteredLogs}
                            timePeriodType={timePeriodType}
                            node_id={id}
                        />
                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} alwaysShowPaginator={false} rows={100} >
                            <Column className='history-column' header="Date" body={dateBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Name" body={nameBodyTemplate} sortable></Column>
                            <Column className='history-column' header="ID" sortable body={idBodyTemplate}></Column>
                            <Column className='history-column' header="Operation" body={operationBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Cloned from" body={clonedFromBodyTemplate} sortable></Column>
                        </DataTable> */}
                    </TabPanel>
                    <TabPanel header="Database Logs">
                        <LogFilesTable node_id={id} />
                    </TabPanel>
                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>
                        <NodeAlertsTable node={pdb_details?.pdb} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />
                    </TabPanel>
                    <TabPanel headerClassName='filters-header' header={<div className='flex'>

                        {toggle < 4 ? <Dropdown
                            className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period raised-dropdown" value={timePeriodType}
                            options={timePeriodTypes}
                            onChange={(e) => {
                                setTimePeriodType(e.value);
                                setMonitorInterval(intervalByTimedelta(e.value));
                            }}
                        /> : null}
                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => {
                                dispatch(StatusActions.setPdbDetails(null));
                                dispatch(StatusActions.getPdbDetails(
                                    {
                                        "pk": pk,
                                        "id": id
                                    }
                                ));
                            }}
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>
                </TabView>
                {(pdb_details?.pdb && showBackup) ?
                    <BackupPDBDialog visible={showBackup}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowBackup(false)
                        }}
                        onOkClick={onBackupPDB}
                        updateSchedule={updateSchedule.current}
                        pageIndex={pageIndex}
                        backups={pdb_details?.backups}
                        node={pdb_details?.pdb} />
                    : null
                }
                {(pdb_details?.pdb && showRename) ?
                    <RenamePDBDialog visible={showRename}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            dispatch(LayoutActions.setValidName(undefined));
                            setShowRename(false)
                        }}
                        onOkClick={onRenamePDB}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb} />
                    : null
                }
                {(pdb_details?.pdb && pdb_details?.cdb && showCopy) ?
                    <CopyPDBDialog visible={showCopy}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            dispatch(LayoutActions.setValidName(undefined));
                            setShowCopy(false)
                        }}
                        onOkClick={onCopyPDB}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb} />
                    : null
                }
                {(pdb_details?.pdb && showCopyTo) ?
                    <CopytoCDBDialog visible={showCopyTo}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            dispatch(LayoutActions.setValidName(undefined));
                            setShowCopyTo(false)
                        }}
                        onOkClick={onCopyToCDB}
                        node={pdb_details?.pdb}
                        data={pdb_details?.copy_to_cdb_form} />
                    : null
                }
                {(pdb_details?.pdb && pdb_details?.cdb && showOpen) ?
                    <OpenPDBDialog visible={showOpen}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowOpen(false)
                        }}
                        onOkClick={onOpenPDB}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb} />
                    : null
                }
                {(pdb_details?.pdb && pdb_details?.cdb && showClose) ?
                    <ClosePDBDialog visible={showClose}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowClose(false)
                        }}
                        onOkClick={onClosePDB}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb} />
                    : null}
                {(pdb_details?.pdb && showRestoreFlashback) ?
                    <RestorePDBFlashbackDialog visible={showRestoreFlashback}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowRestoreFlashback(false)
                        }}
                        onOkClick={onRestoreFlashback}
                        flashbacks={pdb_details?.flashbacks}
                        node={pdb_details?.pdb} />
                    : null
                }
                {(pdb_details?.pdb && showRestore) ?
                    <RestorePDBDialog visible={showRestore}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowRestore(false)
                        }}
                        onOkClick={onRestorePDB}
                        backups={pdb_details?.backups} />
                    : null
                }
                {(pdb_details?.pdb, pdb_details?.cdb && showDelete) ?
                    <DeletePDBDialog visible={showDelete}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowDelete(false)
                        }}
                        onOkClick={onDeletePDB}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb}
                        open_mode={pdb_details?.open_mode}
                        sessions={pdb_details?.conn_sessions}
                    />
                    : null
                }
                {(pdb_details?.pdb && pdb_details?.cdb && showFlashback) ?
                    <FlashbackCDBPDBDialog visible={showFlashback}
                        onHide={() => {
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowFlashback(false)
                        }}
                        onOkClick={onFlashback}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb}
                    />
                    : null
                }
                {(pdb_details?.pdb && pdb_details?.cdb && showExport) ?
                    <ExportCDBPDBDialog visible={showExport}
                        onHide={
                            () => {
                                dispatch(LayoutActions.setSchemas(undefined));
                                dispatch(LayoutActions.setDisks(undefined));
                                dispatch(LayoutActions.setDirExists(null));
                                dispatch(LayoutActions.setExportExists(null));
                                dispatch(LayoutActions.setDiskSpace(null));
                                dispatch(LayoutActions.setErrorMessage(""));
                                setShowExport(false)
                            }}
                        onOkClick={onExport}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb}
                    />
                    : null
                }
                {(pdb_details?.pdb && pdb_details?.cdb && showImport) ?
                    <ImportCDBPDBDialog visible={showImport}
                        onHide={() => {
                            dispatch(LayoutActions.setUserExists(null));
                            dispatch(LayoutActions.setDumpExists(null));
                            dispatch(LayoutActions.setErrorMessage(""));
                            setShowImport(false)
                        }}
                        onOkClick={onImport}
                        node={pdb_details?.pdb}
                        root={pdb_details?.cdb}
                        exports={pdb_details?.exports}
                    />
                    : null
                }
                {(pdb_details?.pdb && showCredentials) ? <CredentialsDialog
                    node_id={pdb_details?.pdb?.id}
                    visible={showCredentials}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        dispatch(StatusActions.setValidCredentials(null))
                        setShowCredentials(false)
                    }
                    }
                    onOkClick={onUpdateCredentials}
                /> : null}
                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}
                {pdb_details && <Tooltip target=".set-schedule-star" content={pdb_details?.auto_backup?.detail}></Tooltip>}
                {pdb_details && <Tooltip target=".avail-star" content={pdb_details?.pdb?.ind_started ? "Database is running and accepting connections" : "Database is not running"}></Tooltip>}
            </div>
        </Fragment>
    )
}
