import restore from '../../../assets/images/restore_pdb.svg'
import backup_pdb from '../../../assets/images/backup_pdb.svg'
import icon_import from '../../../assets/images/import_pdb.svg'
import icon_export from '../../../assets/images/export_pdb.svg'
import drop from '../../../assets/images/drop.svg'
import green_star from '../../../assets/images/green-star.png'
import red_star from '../../../assets/images/red-star.png'
import info from '../../../assets/images/info.svg'
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation, useNavigate } from "react-router-dom";
import { StatusActions } from "../../../store/status/reducer";
import { Fragment, useEffect, useState, useCallback, useRef } from "react";
import { Button } from 'primereact/button';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dropdown } from "primereact/dropdown";
import DetailsMenu from '../../../components//menus/detailsMenu';
import credentials from "../../../assets/images/credentials.svg";
import WarningDialog from '../../../dialogs/warningDialog';
import PerformanceCharts from '../../../components/performanceCharts';
import PostgresExportDialog from '../../../dialogs/postgres/postgresExportDialog';
import PostgresPrimaryBackupDialog from '../../../dialogs/postgres/postgresPrimaryBackupDialog';
import { ConfirmPopup } from 'primereact/confirmpopup';
import PostgresNodeDeleteDialog from '../../../dialogs/postgres/postgresNodeDeleteDialog';
import SwitchPostgresNodeDialog from '../../../dialogs/postgres/switchPostgresNodeDialog';
import RestorePgNodeDialog from '../../../dialogs/postgres/restorePgNodeDialog';
import { TabView, TabPanel } from 'primereact/tabview';
import ImportPgNodeDialog from '../../../dialogs/postgres/importPgNodeDialog';
import getStarByClass from '../../../util/getStarByClass'
import { Tooltip } from 'primereact/tooltip';
import { clusterRoles, timePeriodTypes } from '../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../components/tables/nodeAlertsTable';
import createAPIcommand from '../../../util/createApiCommand';
import DatabaseUsersTable from '../../../components/tables/databaseUsersTable'
import copyToClipboard from '../../../util/copyToClipboard';
import { ConfigActions } from '../../../store/config/reducer';
import { InputText } from 'primereact/inputtext';
import PgSettingsTable from '../../../components/tables/pgSettingsTable';
import StatsCard from '../../../components/statsCard';
import { searchFilter } from '../../monitoring';
import { ButtonGroup } from 'primereact/buttongroup';
import { AppActions } from '../../../store/app/reducer';
import ClusterSizing from '../../../components/changeClusterSizingBlock';
import { searchFilter as logSearch } from '../../log';
import LogsTable from '../../../components/tables/logsTable';
import LogFilesTable from '../../../components/tables/logFilesTable'


export default function PostgresDetails(props) {
    const { id } = useParams();
    const { state } = useLocation();
    const { pg_details, errorMessage } = useSelector((state) => state.status);
    const { accessToken, severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);
    const dispatch = useDispatch();
    const navigate = useNavigate();
    const [upperSearch, setUpperSearch] = useState("");
    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period);
    const [showBackup, setShowBackup] = useState(false);
    const [showExport, setShowExport] = useState(false);
    const [showSwitchover, setShowSwitchover] = useState(false);
    const [showWarningPopup, setShowWarningPopup] = useState(false);
    const [showDropPgNode, setShowDropPgNode] = useState(false);
    const [showRestore, setShowRestore] = useState(false);
    const [showImport, setShowImport] = useState(false);
    const [toggle, setToggle] = useState(0);
    const updateSchedule = useRef(null);
    const [pageIndex, setPageIndex] = useState(null);
    const { alerts } = useSelector((state) => state.monitoring);
    const { database_users } = useSelector((state) => state.config);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [filteredUsers, setFilteredUsers] = useState(null);
    const [search, setSearch] = useState("");
    const [popupAction, setPopupAction] = useState("stop");
    const [filteredLogs, setFilteredLogs] = useState(null);


    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);
    const { node_logs} = useSelector((state) => state.log);

    useEffect(() => {
        const storedTabIndex = localStorage.getItem('DetailsTabIndex');
        if (storedTabIndex) {
            setToggle(parseInt(storedTabIndex, 10));
        }
    }, []);

    const handleTabChange = (e) => {
        setToggle(e.index);
        localStorage.setItem('DetailsTabIndex', e.index);
    };

    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }

        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];

    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            console.log(tempData);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);


    useEffect(() => {

        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;

        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            if (!showSeverityGreen || !showSeverityYellow || !showSeverityRed) tempData = severityFilter(tempData);
            setFilteredAlerts(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);

    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen]);



    useEffect(() => {
        setFilteredUsers(null);
        if (pg_details?.node) dispatch(ConfigActions.getDatabaseUsers(pg_details.node?.id));
    }, [pg_details, dispatch])



    useEffect(() => {

        if (database_users?.length) {

            let tempData = [];

            tempData = database_users;
            let query = search.toLowerCase();
            if (query) tempData = tempData.filter((u) => u.name?.toLowerCase().includes(query));
            setFilteredUsers(tempData);
        }

    }, [search, database_users])

    const onShowPopup = async (target) => {
        switch (target) {
            case "stop-down":
                setPopupAction("stop");
                await copyToClipboard(createAPIcommand({ "node_id": pg_details?.node?.name }, accessToken, "stop_pg_node/"))
                break;
            case "start-down":
                setPopupAction(pg_details?.node?.ind_started ? "restart" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": pg_details?.node?.name }, accessToken, "start_pg_node/"))
                break;
            case "start-stop":
                setPopupAction(pg_details?.node?.ind_started ? "stop" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": pg_details?.node?.name }, accessToken, pg_details?.node?.ind_started ? "stop_pg_node/" : "start_pg_node/"));
                break;
            default:
                break;
        }
        setShowWarningPopup(target);
    }
    useEffect(() => {
        if (id) {

            dispatch(StatusActions.getPgDetails(id));

        }
        return () => {
            dispatch(StatusActions.setPgDetails(null));
            dispatch(LayoutActions.getNodes());
        }
    }, [id, dispatch])

    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);
    }, [state?.targetTab])
    const onShowBackup = () => {
        setPageIndex(null);

        setShowBackup(true);
    }
    const onSetSchedule = () => {
        if (pg_details?.node?.ind_deleted) showWarningMessage({ "text": `${pg_details?.node?.name} is unavailable.`, 'status': pg_details?.node?.status });
        else {
            setPageIndex(1);
            updateSchedule.current = pg_details.backups?.length > 0 ? pg_details?.backups?.filter((b) => b.is_sched === true)?.[0] : pg_details?.nodes?.filter((s) => s?.backups?.filter((b) => b.is_sched === true))?.[0]?.backups?.filter((b) => b.is_sched === true)?.[0];
            setShowBackup(true);
        }
    }
    const onRestore = (data) => {

        dispatch(LayoutActions.restorePgNode(data, setShowRestore));
    }
    const onPopupAccept = () => {
        switch (showWarningPopup) {
            case "stop-down":
                onStopPgNode();
                break;
            case "start-down":
                onStartPgNode();
                break;
            case "start-stop":
                pg_details?.node?.ind_started ? onStopPgNode() : onStartPgNode();
                break;
            case "pause-down":
                togglePause();
                break;
            default:
                break;
        }
    }
    const togglePause = () => {
        dispatch(LayoutActions.togglePgCluster({ "toggle_pause": !pg_details?.node?.ex_params?.repmgr_paused, "node_id": pg_details?.node?.cluster_role === clusterRoles.STANDBY ? pg_details?.node?.primary?.id : pg_details?.node?.id }, pg_details?.node));
    }
    const showWarningMessage = (data) => {
        setWarningMessage(data);
        setShowWarning(true);
    }
    const onStopPgNode = () => dispatch(LayoutActions.stopPgNode({ "node_id": pg_details?.node?.id }, pg_details?.node));
    const onStartPgNode = () => dispatch(LayoutActions.startPgNode({ "node_id": pg_details?.node?.id }, pg_details?.node));
    const onImport = (data) => {
        dispatch(LayoutActions.importPgNode(data, setShowImport));
    }

    // const dateBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{moment(item[2]).format("MMM DD,YYYY, hh:mm A")}</div>);
    // }
    // const nameBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[0]}</div>);
    // }
    // const idBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[1]}</div>);
    // }
    // const operationBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[3]}</div>);
    // }
    // const clonedFromBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item[4]}</div>);
    // }
    const onExportPostgresNode = (data) => {
        dispatch(LayoutActions.exportPgNode(data, setShowExport));
    }
    const onBackupPostgresNode = (data) => {
        dispatch(LayoutActions.backupPgNode(data, setShowBackup));
    }
    const onDropPgNode = (data) => {
        dispatch(LayoutActions.dropPgNode(data, setShowDropPgNode));
    }
    const onSwitchover = (data) => {
        dispatch(LayoutActions.switchPgNode(data, setShowSwitchover));
    }
    const isPrimary = (is_primary) => {
        if (!is_primary || (is_primary && (!pg_details?.nodes || pg_details.nodes?.length < 1))) setShowDropPgNode(true);
        else showWarningMessage({ "text": `This is primary with standbys, cannot be removed.\nSwitchover to standby first.` })
    }
    const attachPopup = (target) => pg_details?.node?.ind_deleted ? showWarningMessage({ "text": `${pg_details?.node?.name} is unavailable.`, 'status': pg_details?.node?.status }) : onShowPopup(target);
    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">
                <TabView activeIndex={toggle} onTabChange={handleTabChange}>
                    <TabPanel header="Dashboard">
                        <div className='grid flex-row'>
                            <div className='col-3 flex align-items-between flex-column' >
                                <h2 >
                                    {pg_details?.node?.cluster_role} {pg_details?.node?.name}
                                </h2>
                                <div>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Instance is {pg_details?.node?.ind_started ? 'up' : "down"}
                                                </td>
                                                <td>
                                                    <Button
                                                        className="p-button-sm details-button"
                                                        disabled={!pg_details?.node?.ind_started && !errorMessage}
                                                        id="start-stop"
                                                        icon={pg_details?.node?.ind_started ? "pi pi-stop-circle" : "pi pi-play"}
                                                        style={{ minWidth: "53px", minHeight: "39px" }}
                                                        iconPos="right"
                                                        onClick={() => attachPopup("start-stop")}
                                                    />
                                                </td>
                                                <td>
                                                    <img src={pg_details?.node?.ind_started ? green_star : red_star} alt="star" className="avail-star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {pg_details?.ind_error ? "Errors in cluster" : pg_details?.ind_error === false ? "Cluster is OK" : <div> <i className="pi pi-spin pi-spinner" /> Connecting to node... </div>}
                                                </td>
                                                <td>
                                                    <Button onClick={() => navigate('/layout', { state: { node_id: pg_details?.node?.ex_params?.pg_primary_id } })} disabled={!pg_details?.node} className='p-button-sm details-button'><img alt="info" width={16} height={16} src={info} /></Button>
                                                </td>
                                                <td>
                                                    <img className='status-star' src={(pg_details?.ind_error === undefined || pg_details?.ind_error === true) ? red_star : green_star} alt="star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {pg_details?.auto_backup?.status ? pg_details.auto_backup.status : ""}
                                                </td>
                                                <td>
                                                    <Button onClick={onSetSchedule} icon="pi pi-calendar-times" disabled={!pg_details?.node?.ind_started && !errorMessage} className='p-button-sm details-button update-schedule-button'></Button>
                                                </td>
                                                <td>
                                                    <img className='set-schedule-star' src={getStarByClass(pg_details?.auto_backup?.class)} alt="star"></img>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <div>
                                    <h3>
                                        General info
                                    </h3 >
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td> {pg_details?.node?.name}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    DB ID
                                                </td>
                                                <td> {pg_details?.db_id}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Version
                                                </td>
                                                <td> {pg_details?.version}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Logging
                                                </td>
                                                <td> {pg_details?.logging}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Read Only
                                                </td>
                                                <td> {pg_details?.read_only}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Size
                                                </td>
                                                <td>  {pg_details?.display_size}</td>
                                            </tr>
                                        </tbody>
                                    </table>
                                    <ClusterSizing node={pg_details?.node} />
                                </div>
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>
                                <StatsCard details={pg_details}></StatsCard>

                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={pg_details?.node?.cpuUsages}
                                    memUsages={pg_details?.node?.memUsages}
                                />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>
                                <div className='col-9'>
                                    <span className="p-buttonset">
                                        < Button className="p-button-sm details-button up" disabled={!pg_details?.node?.ind_started && !errorMessage} label="Backup" onClick={() => pg_details?.node?.ind_deleted ? showWarningMessage({ "text": `${pg_details?.node?.name} is unavailable.`, 'status': pg_details?.node?.status }) : onShowBackup()}>
                                            < img alt="backup" width={16} height={16} src={backup_pdb} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={pg_details?.node?.cluster_role !== clusterRoles.PRIMARY || (!pg_details?.node?.ind_started && !errorMessage)} label="Restore" onClick={() => pg_details?.node?.ind_deleted ? showWarningMessage({ "text": `${pg_details?.node?.name} is unavailable.`, 'status': pg_details?.node?.status }) : setShowRestore(true)}>
                                            < img alt="restore" width={16} height={16} src={restore} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={(!pg_details?.node?.ind_started && !errorMessage)} label="Export" onClick={() => pg_details?.node?.ind_deleted ? showWarningMessage({ "text": `${pg_details?.node?.name} is unavailable.`, 'status': pg_details?.node?.status }) : setShowExport(true)}>
                                            < img alt="export" width={16} height={16} src={icon_export} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={(!pg_details?.node?.ind_started && !errorMessage) || pg_details?.node?.cluster_role === clusterRoles.STANDBY} label="Import" onClick={() => pg_details?.node?.ind_deleted ? showWarningMessage({ "text": `${pg_details?.node?.name} is unavailable.`, 'status': pg_details?.node?.status }) : setShowImport(true)}>
                                            < img alt="import" width={16} height={16} src={icon_import} />
                                        </Button>
                                        <Button disabled={(!pg_details?.node?.ind_started && !errorMessage)} className="p-button-sm details-button up" label="Credentials">
                                            <img alt="credentials" width={16} height={16} src={credentials} ></img>
                                        </Button>
                                        <Button className="p-button-sm details-button up" icon="pi pi-arrows-h"
                                            iconPos="right"
                                            disabled={pg_details?.node?.cluster_role !== clusterRoles.PRIMARY || (!pg_details?.node?.ind_started && !errorMessage) || pg_details?.nodes?.filter((n) => n.severity === "green").length === 0}
                                            label="Switchover"
                                            onClick={() => setShowSwitchover(true)}
                                        />
                                        <Button className="p-button-sm details-button up" icon="pi pi-pause"
                                            iconPos="right"
                                            disabled={pg_details?.node?.cluster_role !== clusterRoles.PRIMARY || !pg_details || !pg_details?.nodes || pg_details?.nodes?.length < 1}
                                            label={pg_details?.node?.ex_params?.repmgr_paused === true ? "Unpause" : "Pause"}
                                            id="pause-down"
                                            onClick={() => attachPopup("pause-down")}
                                        />
                                    </span>
                                    <span className="p-buttonset down">
                                        <Button disabled={errorMessage || !pg_details?.node?.ind_started} icon="pi pi-stop-circle" iconPos="right" className="p-button-sm details-button" id="stop-down" label="Stop" onClick={() => attachPopup("stop-down")} />
                                        <Button id="start-down" disabled={!pg_details} icon={`pi pi-${pg_details?.node?.ind_started ? "replay" : "play"}`} iconPos="right" className="p-button-sm details-button" label={pg_details?.node?.ind_started ? "Restart" : "Start"} onClick={() => attachPopup("start-down")}>
                                        </Button>
                                        <Button disabled={!pg_details?.node?.ind_started && !errorMessage} className="p-button-sm details-button" label="Drop" onClick={() => pg_details?.node?.ind_deleted ? showWarningMessage({ "text": `${pg_details?.node?.name} is unavailable.`, 'status': pg_details?.node?.status }) : dispatch(LayoutActions.checkIfPgPrimary(pg_details?.node?.id, isPrimary))}>
                                            < img alt="drop" width={16} height={16} src={drop} />
                                        </Button>
                                    </span>
                                </div>
                            </div>
                        </div>
                    </TabPanel>
                    <TabPanel header="Activity Logs">

                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <LogsTable filteredLogs={filteredLogs}
                            node_id = {id}
                            timePeriodType={timePeriodType}
                        />


                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} rows={100} paginator alwaysShowPaginator={false} >
                            <Column className='history-column' header="Date" body={dateBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Name" body={nameBodyTemplate} sortable></Column>
                            <Column className='history-column' header="ID" sortable body={idBodyTemplate}></Column>
                            <Column className='history-column' header="Operation" body={operationBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Cloned from" body={clonedFromBodyTemplate} sortable></Column>
                        </DataTable> */}
                    </TabPanel>
                    <TabPanel header="Database Logs">
                        <LogFilesTable node_id ={id} />
                    </TabPanel>
                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <NodeAlertsTable node={pg_details?.node} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />
                    </TabPanel>
                    <TabPanel header="Parameters">
                        <PgSettingsTable node={pg_details?.node} />
                    </TabPanel>
                    <TabPanel header="Users">
                        <div className="flex justify-content-start">
                            <span className="p-input-icon-right">
                                <i className={search ? "pi pi-times" : "field-collapsed"} onClick={() => setSearch("")} />
                                <InputText className="p-inputtext-sm filter-field mb-2"
                                    placeholder="Search"
                                    value={search}
                                    onChange={(e) => {
                                        setSearch(e.target.value);
                                    }} />
                            </span>
                        </div>
                        <DatabaseUsersTable node={pg_details?.node} users={filteredUsers} />
                    </TabPanel>
                    <TabPanel headerClassName='filters-header' header={<div className='flex'>

                        {toggle < 4 ?  <Dropdown
                            className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period raised-dropdown" value={timePeriodType}
                            options={timePeriodTypes}
                            onChange={(e) => {
                                setTimePeriodType(e.value);
                                setMonitorInterval(intervalByTimedelta(e.value));
                            }}
                                /> : null}
                       
                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => {

                                dispatch(StatusActions.setPgDetails(null))
                                dispatch(StatusActions.getPgDetails(id))

                            }}
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>
                </TabView>
                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}
                {(showDropPgNode && pg_details) ? <PostgresNodeDeleteDialog
                    node={pg_details?.node}
                    visible={showDropPgNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowDropPgNode(false)
                    }}
                    onOkClick={onDropPgNode}
                />
                    : null}
                {(showExport && pg_details) ? <PostgresExportDialog
                    node={pg_details?.node}
                    visible={showExport}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        dispatch(LayoutActions.setDirExists(null));
                        dispatch(LayoutActions.setExportExists(null));
                        dispatch(LayoutActions.setDiskSpace(null));
                        setShowExport(false)
                    }}
                    onOkClick={onExportPostgresNode}
                />
                    : null}
                {(showBackup && pg_details) ? <PostgresPrimaryBackupDialog
                    node={pg_details?.node}
                    visible={showBackup}
                    updateSchedule={updateSchedule.current}
                    pageIndex={pageIndex}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowBackup(false);
                    }}
                    onOkClick={onBackupPostgresNode}
                />
                    : null}
                {(showSwitchover && pg_details) ? <SwitchPostgresNodeDialog
                    nodes={pg_details?.nodes?.filter((n) => n?.severity === "green" && n?.ind_started === true && n?.id !== pg_details?.node?.id)}
                    visible={showSwitchover}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowSwitchover(false)
                    }}
                    onOkClick={onSwitchover}
                /> : null}
                {(showRestore && pg_details) ? <RestorePgNodeDialog
                    data={pg_details}
                    visible={showRestore}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestore(false)
                    }}
                    onOkClick={onRestore}
                /> : null}
                {(showImport && pg_details) ? <ImportPgNodeDialog
                    data={pg_details?.node}
                    visible={showImport}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowImport(false)
                    }}
                    onOkClick={onImport}
                /> : null}
            </div>
            <ConfirmPopup
                icon="pi pi-exclamation-triangle"
                target={document.getElementById(showWarningPopup)}
                visible={Boolean(showWarningPopup)}
                onHide={() => setShowWarningPopup(false)}
                accept={onPopupAccept}
                message={`Are you sure you want to ${popupAction} the node?`} />
            {pg_details && <Tooltip target=".set-schedule-star" content={pg_details?.auto_backup?.detail}></Tooltip>}
            {pg_details && <Tooltip target=".avail-star" content={pg_details?.node?.ind_started ? "Database is running and accepting connections" : "Database is not running"}></Tooltip>}
            {pg_details && <Tooltip target=".status-star" content={(pg_details?.ind_error === undefined || pg_details?.ind_error === true) ? "Some nodes of cluster are down/inaccessible" : "All nodes of cluster are accessible"}></Tooltip>}
            <Tooltip
                position="bottom"
                content="Pause repmgr. Cluster management and failover will stop until resumed."
                target={window.document.getElementById(`pause-down`)} />
        </Fragment>
    )
}
