import restore from '../../../assets/images/restore_pdb.svg';
import backup_pdb from '../../../assets/images/backup_pdb.svg';
import drop from '../../../assets/images/drop.svg';
import green_star from '../../../assets/images/green-star.png';
import red_star from '../../../assets/images/red-star.png';
import info from '../../../assets/images/info.svg';
import { useDispatch, useSelector } from 'react-redux';
import { useParams, useLocation, useNavigate } from "react-router-dom";
import { StatusActions } from "../../../store/status/reducer";
import { Fragment, useEffect, useState, useCallback, useRef } from "react";
import { Button } from 'primereact/button';
import { LayoutActions } from '../../../store/layout/reducer';
import { Dropdown } from "primereact/dropdown";
import DetailsMenu from '../../../components/menus/detailsMenu';
import credentials from "../../../assets/images/credentials.svg";
import WarningDialog from '../../../dialogs/warningDialog';
import PerformanceCharts from '../../../components/performanceCharts';
import { ConfirmPopup } from 'primereact/confirmpopup';
import RedisNodeBackupDialog from '../../../dialogs/redis/redisNodeBackupDialog';
import RestoreRedisNode from '../../../dialogs/redis/restoreRedisNodeDialog';
import DropRedisNodeDialog from '../../../dialogs/redis/dropRedisNodeDialog';
import StepdownRedisNodeDialog from '../../../dialogs/redis/stepdownRedisNodeDialog';
import { TabView, TabPanel } from 'primereact/tabview';
import SwitchRedisNodeDialog from '../../../dialogs/redis/switchRedisNodeDialog';
import getStarByClass from '../../../util/getStarByClass';
import { Tooltip } from 'primereact/tooltip';
import { clusterRoles, timePeriodTypes } from '../../../config';
import { intervalByTimedelta, NodeAlertsTable } from '../../../components/tables/nodeAlertsTable';
import createAPIcommand from '../../../util/createApiCommand';
import copyToClipboard from '../../../util/copyToClipboard';
import SentinelMonitorMasterDialog from '../../../dialogs/redis/sentinelMonitorMasterDialog';
import StatsCard from '../../../components/statsCard';
import { InputText } from 'primereact/inputtext';
import { searchFilter } from '../../monitoring';
import { ButtonGroup } from 'primereact/buttongroup';
import { AppActions } from '../../../store/app/reducer';
import ClusterSizing from '../../../components/changeClusterSizingBlock';
import { searchFilter as logSearch } from '../../log';
import LogsTable from '../../../components/tables/logsTable';
import LogFilesTable from '../../../components/tables/logFilesTable';


export default function RedisDetails(props) {
    const { id } = useParams();
    const { state } = useLocation();
    const { redis_details, errorMessage } = useSelector((state) => state.status);
    const { accessToken, severity, time_period } = useSelector((state) => state.app);
    const [showSeverityGreen, setShowSeverityGreen] = useState(severity['green']);
    const [showSeverityYellow, setShowSeverityYellow] = useState(severity['yellow']);
    const [showSeverityRed, setShowSeverityRed] = useState(severity['red']);
    const [alertSeverityGreenTotal, setAlertSeverityGreenTotal] = useState(0);
    const [alertSeverityYellowTotal, setAlertSeverityYellowTotal] = useState(0);
    const [alertSeverityRedTotal, setAlertSeverityRedTotal] = useState(0);
    const dispatch = useDispatch();
    const [upperSearch, setUpperSearch] = useState("");
    const [showWarning, setShowWarning] = useState(false);
    const [warningMessage, setWarningMessage] = useState(false);
    const [timePeriodType, setTimePeriodType] = useState(time_period)
    const [showBackup, setShowBackup] = useState(false);
    const [showWarningPopup, setShowWarningPopup] = useState(false);
    const [showDropRedisNode, setShowDropRedisNode] = useState(false);
    const [showSwitchover, setShowSwitchover] = useState(false);
    const [showRestore, setShowRestore] = useState(false);
    const [toggle, setToggle] = useState(0);
    const updateSchedule = useRef(null);
    const [pageIndex, setPageIndex] = useState(null);
    const { alerts } = useSelector((state) => state.monitoring);
    const [showTimelineColumn, setShowTimelineColumn] = useState(false);
    const [filteredAlerts, setFilteredAlerts] = useState(null);
    const [showStepdown, setShowStepdown] = useState(false);
    const [showMonitorMaster, setShowMonitorMaster] = useState(false);
    const [monitorInterval, setMonitorInterval] = useState(4);
    const navigate = useNavigate();
    const [popupAction, setPopupAction] = useState("stop");
    const [filteredLogs, setFilteredLogs] = useState(null);
    const [statusGreenTotal, setStatusGreenTotal] = useState(0);
    const [statusYellowTotal, setStatusYellowTotal] = useState(0);
    const [statusRedTotal, setStatusRedTotal] = useState(0);
    const { node_logs } = useSelector((state) => state.log);

    useEffect(() => {
        const storedTabIndex = localStorage.getItem('DetailsTabIndex');
        if (storedTabIndex) {
            setToggle(parseInt(storedTabIndex, 10));
        }
    }, []);

    const handleTabChange = (e) => {
        setToggle(e.index);
        localStorage.setItem('DetailsTabIndex', e.index);
    };

    const severityFilter = useCallback((data) => {
        const severityChoices = { green: showSeverityGreen, yellow: showSeverityYellow, red: showSeverityRed };
        dispatch(AppActions.setSeverity(severityChoices));
        let severityList = [];
        if (showSeverityGreen) {
            severityList.push(1);
            severityList.push(-1);
            severityList.push("green");
        }
        if (showSeverityYellow) {
            severityList.push(2);
            severityList.push(-2);
            severityList.push("yellow");
        }
        if (showSeverityRed) {
            severityList.push("red");
            severityList.push(3);
        }

        return data?.filter ? data?.filter((n) => severityList.indexOf(n.severity ? n.severity : n.status) >= 0) : [];

    }, [showSeverityGreen, showSeverityYellow, showSeverityRed, dispatch]);

    useEffect(() => {

        let tempStatusGreen = 0;
        let tempStatusYellow = 0;
        let tempStatusRed = 0;
        if (node_logs?.length > 0) {
            node_logs.forEach((n) => {
                switch (n.status) {
                    case "green":
                        ++tempStatusGreen;
                        break;
                    case "yellow":
                        ++tempStatusYellow;
                        break;
                    case "red":
                        ++tempStatusRed;
                        break;
                    default:
                        break;
                }
            });
            let tempData = node_logs;
            if (upperSearch)
                tempData = logSearch(tempData, upperSearch);
            tempData = severityFilter(tempData);
            setFilteredLogs(tempData);
        }
        else setFilteredLogs(node_logs);
        setStatusGreenTotal(tempStatusGreen);
        setStatusYellowTotal(tempStatusYellow);
        setStatusRedTotal(tempStatusRed);
    }, [node_logs, upperSearch, severityFilter]);

    const onShowPopup = async (target) => {
        switch (target) {
            case "stop-down":
                setPopupAction("stop");
                await copyToClipboard(createAPIcommand({ "node_id": redis_details?.node?.name }, accessToken, "stop_redis_node/"))
                break;
            case "start-down":
                setPopupAction(redis_details?.node?.ind_started ? "restart" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": redis_details?.node?.name }, accessToken, "start_redis_node/"))
                break;
            case "start-stop":
                setPopupAction(redis_details?.node?.ind_started ? "stop" : "start");
                await copyToClipboard(createAPIcommand({ "node_id": redis_details?.node?.name }, accessToken, redis_details?.node?.ind_started ? "stop_redis_node/" : "start_redis_node/"));
                break;
            default:
                break;

        }
        setShowWarningPopup(target);
    }
    useEffect(() => {
        if (state?.targetTab)
            setToggle(state?.targetTab);
    }, [state?.targetTab])
    useEffect(() => {
        if (id) {

            dispatch(StatusActions.getRedisDetails(id));
        }
        return () => {
            dispatch(StatusActions.setRedisDetails(null));
            dispatch(LayoutActions.getNodes());

        };
    }, [id, dispatch])

    useEffect(() => {

        if (id) dispatch(StatusActions.setRedisDetails(null));

    }, [id, dispatch])

    useEffect(() => {


        let tempSeverityGreen = 0;
        let tempSeverityYellow = 0;
        let tempSeverityRed = 0;

        if (alerts?.length > 0) {
            let tempData = structuredClone(alerts);
            tempData = searchFilter(tempData, upperSearch);
            if (!showSeverityGreen || !showSeverityYellow || !showSeverityRed) tempData = severityFilter(tempData);
            setFilteredAlerts(tempData);
            alerts.forEach((n) => {
                switch (Math.abs(n.severity)) {
                    case 1:
                        ++tempSeverityGreen;
                        break;
                    case 2:
                        ++tempSeverityYellow;
                        break;
                    case 3:
                        ++tempSeverityRed;
                        break;
                    default:
                        break;
                }
            });
        }
        else setFilteredAlerts(alerts);
        setAlertSeverityGreenTotal(tempSeverityGreen);
        setAlertSeverityYellowTotal(tempSeverityYellow);
        setAlertSeverityRedTotal(tempSeverityRed);

    }, [upperSearch, alerts, severityFilter, showSeverityRed, showSeverityYellow, showSeverityGreen])


    // useEffect(() => {

    //     if (redis_details?.history?.length > 0) {

    //         const query = upperSearch.toLowerCase().trim();
    //         if (!query) setFilteredLogs(redis_details.history);
    //         else setFilteredLogs(redis_details.history.filter((h) => h.when.format("MMM DD,YYYY, hh:mm A").includes(query)
    //             || h.level.toLowerCase().includes(query)
    //             || h.pid.toLowerCase().includes(query)
    //             || h.message.toLowerCase().includes(query)
    //             || h.role.toLowerCase().includes(query)))
    //     }

    // },
    //     [upperSearch, redis_details?.history])



    const onRestore = (data) => {
        dispatch(LayoutActions.restoreRedisNode(data, setShowRestore));
    }
    const onShowBackup = () => {
        setPageIndex(null);

        setShowBackup(true);
    }
    const onSetSchedule = () => {
        if (redis_details?.node?.ind_deleted) showWarningMessage({ "text": `${redis_details?.node?.name} is unavailable.`, 'status': redis_details?.node?.status });
        else {
            setPageIndex(1);
            updateSchedule.current = redis_details?.backups?.filter((b) => b.is_sched === true)?.[0];
            setShowBackup(true);
        }
    }
    const onPopupAccept = () => {
        switch (showWarningPopup) {
            case "stop-down":
                onStopRedisNode();
                break;
            case "start-down":
                onStartRedisNode();
                break;
            case "start-stop":
                redis_details?.node?.ind_started ? onStopRedisNode() : onStartRedisNode();
                break;
            default:
                break;
        }
    }
    const showWarningMessage = (data) => {
        setWarningMessage(data);
        setShowWarning(true);
    }
    const onStopRedisNode = () => dispatch(LayoutActions.stopRedisNode({ "node_id": redis_details?.node?.id }, redis_details?.node));
    const onStartRedisNode = () => dispatch(LayoutActions.startRedisNode({ "node_id": redis_details?.node?.id }, redis_details?.node));
    // const whenBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item["when"]}</div>);
    // }
    // const pidBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item['pid']}</div>);
    // }
    // const levelBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item['level']}</div>);
    // }
    // const roleBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item["role"]}</div>);
    // }
    // const messageBodyTemplate = (item) => {
    //     return (<div className='history-cell'>{item["message"]}</div>);
    // }
    const onBackupRedisNode = (data) => {
        dispatch(LayoutActions.backupRedisNode(data, setShowBackup));
    }
    const onDropRedisNode = (data) => {
        dispatch(LayoutActions.dropRedisNode(data, setShowDropRedisNode));
    }
    const onSwitchover = (data) => {
        dispatch(LayoutActions.switchRedisNode(data, setShowSwitchover));
    }
    const onStepdown = (data) => {
        dispatch(LayoutActions.sentinelFailover(data, setShowStepdown));
    }
    const onSentinelMonitorMaster = (data) => {
        dispatch(LayoutActions.sentinelMonitorMaster(data, setShowMonitorMaster));
    }
    const attachPopup = (target) => redis_details?.node?.ind_deleted ? showWarningMessage({ "text": `${redis_details?.node?.name} is unavailable.`, 'status': redis_details?.node?.status }) : onShowPopup(target);
    return (
        <Fragment>
            <DetailsMenu />
            <div className="details-page">
                <TabView activeIndex={toggle} onTabChange={handleTabChange}>
                    <TabPanel header="Dashboard">
                        <div className='grid flex-row'>
                            <div className='col-3 flex align-items-between flex-column' >
                                <h2 >
                                    {redis_details?.node?.cluster_role} {redis_details?.node?.name}
                                </h2>
                                <div>
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Node is {redis_details?.node?.ind_started ? 'up' : "down"}
                                                </td>
                                                <td>
                                                    <Button
                                                        className="p-button-sm details-button"
                                                        disabled={!redis_details?.node?.ind_started && !errorMessage}
                                                        id="start-stop"
                                                        icon={redis_details?.node?.ind_started ? "pi pi-stop-circle" : "pi pi-play"}
                                                        style={{ minWidth: "53px", minHeight: "39px" }}
                                                        iconPos="right"
                                                        onClick={() => attachPopup("start-stop")}
                                                    />
                                                </td>
                                                <td>
                                                    <img src={redis_details?.node?.ind_started ? green_star : red_star} alt="star" className="avail-star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {redis_details?.ind_error ? "Errors in cluster" : redis_details?.ind_error === false ? "Cluster is OK" : <div> <i className="pi pi-spin pi-spinner" /> Connecting to node... </div>}
                                                </td>
                                                <td>
                                                    <Button disabled={!redis_details?.node} onClick={() => navigate('/layout', { state: { node_id: redis_details?.node?.cluster } })} className='p-button-sm details-button'><img alt="info" width={16} height={16} src={info} /></Button>
                                                </td>
                                                <td>
                                                    <img className='status-star' src={(redis_details?.ind_error === undefined || redis_details?.ind_error === true) ? red_star : green_star} alt="star"></img>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    {redis_details?.auto_backup?.status ? redis_details.auto_backup.status : ""}
                                                </td>
                                                <td>
                                                    <Button onClick={onSetSchedule} icon="pi pi-calendar-times" disabled={!redis_details?.node?.ind_started && !errorMessage} className='p-button-sm details-button update-schedule-button'></Button>
                                                </td>
                                                <td>
                                                    <img className='set-schedule-star' src={getStarByClass(redis_details?.auto_backup?.class)} alt="star"></img>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <div>
                                    <h3>
                                        General info
                                    </h3 >
                                    <table className='details-table'>
                                        <tbody>
                                            <tr>
                                                <td>
                                                    Name
                                                </td>
                                                <td> {redis_details?.node?.name}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    DB ID
                                                </td>
                                                <td> {redis_details?.db_id}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Version
                                                </td>
                                                <td> {redis_details?.version}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Is master
                                                </td>
                                                <td> {redis_details?.is_master}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Uptime
                                                </td>
                                                <td> {redis_details?.uptime}</td>
                                            </tr>
                                            <tr>
                                                <td>
                                                    Size
                                                </td>
                                                <td>  {redis_details?.display_size} </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                                <ClusterSizing node={redis_details?.node} />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-5" : 'col-6'} flex align-items-end flex-column`}>
                                <StatsCard details={redis_details}></StatsCard>
                                <PerformanceCharts
                                    timePeriodType={timePeriodType}
                                    cpuUsages={redis_details?.node?.cpuUsages}
                                    memUsages={redis_details?.node?.memUsages}
                                />
                            </div>
                            <div className={`${window.screen.availWidth <= 1280 ? "col-4" : "col-3"} flex align-items-end flex-column`}>
                                <div className='col-9'>
                                    <span className="p-buttonset">
                                        <Button
                                            disabled={redis_details?.node?.cluster_role !== clusterRoles.SENTINEL || (!redis_details?.node?.ind_started && !errorMessage) || !redis_details}
                                            className="p-button-sm details-button up"
                                            label="Monitor master"
                                            icon="pi pi-eye"
                                            iconPos='right'
                                            onClick={() => setShowMonitorMaster(true)}
                                        />
                                        <Button
                                            disabled={redis_details?.node?.cluster_role !== clusterRoles.SENTINEL || (!redis_details?.node?.ind_started && !errorMessage) || !redis_details || redis_details?.masters?.filter((m) => m?.ex_params?.monitored_by?.length > 0)?.length < 1}
                                            className="p-button-sm details-button up"
                                            label="Stepdown master"
                                            icon="pi pi-arrow-down"
                                            iconPos='right'
                                            onClick={() => setShowStepdown(true)}
                                        />
                                        < Button className="p-button-sm details-button up" disabled={redis_details?.node?.cluster_role === clusterRoles.SENTINEL || (!redis_details?.node?.ind_started && !errorMessage)} label="Backup" onClick={() => redis_details?.node?.ind_deleted ? showWarningMessage({ "text": `${redis_details?.node?.name} is unavailable.`, 'status': redis_details?.node?.status }) : onShowBackup()}>
                                            < img alt="backup" width={16} height={16} src={backup_pdb} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" disabled={redis_details?.node?.cluster_role === clusterRoles.SENTINEL || (!redis_details?.node?.ind_started && !errorMessage)} label="Restore" onClick={() => redis_details?.node?.ind_deleted ? showWarningMessage({ "text": `${redis_details?.node?.name} is unavailable.`, 'status': redis_details?.node?.status }) : setShowRestore(true)}>
                                            < img alt="restore" width={16} height={16} src={restore} />
                                        </Button>
                                        <Button className="p-button-sm details-button up" icon="pi pi-arrows-h"
                                            iconPos="right"
                                            disabled={redis_details?.node?.cluster_role !== clusterRoles.PRIMARY || (redis_details?.slaves?.filter((s) => s?.ind_started === true && s?.severity === 'green')?.length < 1)}
                                            label="Switchover"
                                            onClick={() => setShowSwitchover(true)}
                                        />
                                        <Button disabled={(!redis_details?.node?.ind_started && !errorMessage)} className="p-button-sm details-button up" label="Credentials">
                                            <img alt="credentials" width={16} height={16} src={credentials} ></img>
                                        </Button>
                                    </span>
                                    <span className="p-buttonset down">
                                        <Button disabled={errorMessage || !redis_details?.node?.ind_started} icon="pi pi-stop-circle" iconPos="right" className="p-button-sm details-button" id="stop-down" label="Stop" onClick={() => attachPopup("stop-down")} />
                                        <Button id="start-down" disabled={!redis_details} icon={`pi pi-${redis_details?.node?.ind_started ? "replay" : "play"}`} iconPos="right" className="p-button-sm details-button" label={redis_details?.node?.ind_started ? "Restart" : "Start"} onClick={() => attachPopup("start-down")}>
                                        </Button>
                                        <Button disabled={!redis_details?.node?.ind_started && !errorMessage} className="p-button-sm details-button" label="Drop" onClick={() => redis_details?.node?.ind_deleted ? showWarningMessage({ "text": `${redis_details?.node?.name} is unavailable.`, 'status': redis_details?.node?.status }) : setShowDropRedisNode(true)}>
                                            < img alt="drop" width={16} height={16} src={drop} />
                                        </Button>
                                    </span>
                                </div>
                            </div>
                        </div>
                    </TabPanel>
                    <TabPanel header="Activity Logs">
                        <span className="p-input-icon-right">
                            <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                setUpperSearch("");
                            }} ></i>
                            <InputText className="p-inputtext-sm filter-field mr-2- mb-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                        </span>

                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${statusGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${statusYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${statusRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>

                        <LogsTable filteredLogs={filteredLogs}
                            timePeriodType={timePeriodType}
                            node_id={id}
                        />
                        {/* <DataTable scrollable scrollHeight="flex" className='history-table' emptyMessage={<td className="text-sm">No logs data</td>} rowsPerPageOptions={[10, 20, 50, 100]} value={filteredLogs} paginator alwaysShowPaginator={false} rows={100} >
                            <Column className='history-column' header="When" field='when' body={whenBodyTemplate} sortable></Column>
                            <Column className='history-column' header="PID" field='pid' body={pidBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Level" field='level' sortable body={levelBodyTemplate}></Column>
                            <Column className='history-column' header="Role" field='role' body={roleBodyTemplate} sortable></Column>
                            <Column className='history-column' header="Message" field='message' body={messageBodyTemplate} sortable></Column>
                        </DataTable> */}
                    </TabPanel>
                    <TabPanel header="Database Logs">
                        <LogFilesTable node_id={id} />
                    </TabPanel>
                    <TabPanel header="Alerts">
                        <div className='flex'>

                            <ButtonGroup className="ml-auto mr-auto">

                                <Button className={showSeverityGreen ? " p-button-sm severity-green toggled" : "severity-green p-button-sm"} onClick={() => setShowSeverityGreen(!showSeverityGreen)} label={`${alertSeverityGreenTotal}`} />
                                <Button className={showSeverityYellow ? " p-button-sm severity-yellow toggled p-button-sm" : "severity-yellow p-button-sm"} onClick={() => setShowSeverityYellow(!showSeverityYellow)} label={`${alertSeverityYellowTotal}`} />
                                <Button className={showSeverityRed ? "p-button-sm severity-red toggled" : "severity-red p-button-sm"} onClick={() => setShowSeverityRed(!showSeverityRed)} label={`${alertSeverityRedTotal}`} />

                            </ButtonGroup>

                            <span className="p-input-icon-right">
                                <i className={upperSearch ? "pi pi-times" : "field-collapsed"} onClick={() => {
                                    setUpperSearch("");
                                }} ></i>
                                <InputText className="p-inputtext-sm filter-field mr-2" placeholder="Search" value={upperSearch} onChange={(e) => setUpperSearch(e.target.value)} />


                            </span>


                        </div>
                        <NodeAlertsTable node={redis_details?.node} monitorInterval={monitorInterval} filteredAlerts={filteredAlerts} timelineVisible={showTimelineColumn} toggleTimeline={() => setShowTimelineColumn(!showTimelineColumn)} />
                    </TabPanel>
                    <TabPanel headerClassName='filters-header' header={<div className='flex'>

                        {toggle < 4 ? <Dropdown
                            className="p-inputtext-sm mb-2 mt-2 ml-auto filter-field time-period raised-dropdown" value={timePeriodType}
                            options={timePeriodTypes}
                            onChange={(e) => {
                                setTimePeriodType(e.value);
                                setMonitorInterval(intervalByTimedelta(e.value));
                            }}
                        /> : null}
                        <Button
                            className="p-button-sm filter-button refresh-button ml-2"
                            onClick={() => {
                                dispatch(StatusActions.setRedisDetails(null))
                                dispatch(StatusActions.getRedisDetails(id))

                            }
                            }
                            icon="pi pi-refresh"
                            iconPos="right"
                        />
                    </div>}></TabPanel>
                </TabView>
                {showWarning ?
                    <WarningDialog
                        message={warningMessage}
                        visible={showWarning}
                        onHide={() => setShowWarning(false)} />
                    : null}
                {(showDropRedisNode && redis_details) ? <DropRedisNodeDialog
                    node={redis_details?.node}
                    nodes_count={redis_details?.slaves_count}
                    visible={showDropRedisNode}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowDropRedisNode(false)
                    }}
                    onOkClick={onDropRedisNode}
                />
                    : null}
                {(showBackup && redis_details) ? <RedisNodeBackupDialog
                    node={redis_details?.node?.cluster_role === clusterRoles.PRIMARY ? redis_details?.node : redis_details?.master}
                    visible={showBackup}
                    backups={redis_details?.backups}
                    updateSchedule={updateSchedule.current}
                    pageIndex={pageIndex}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowBackup(false)
                    }}
                    onOkClick={onBackupRedisNode}
                />
                    : null}
                {(showRestore && redis_details) ? <RestoreRedisNode
                    data={redis_details}
                    visible={showRestore}
                    onHide={() => {
                        dispatch(LayoutActions.setErrorMessage(""));
                        setShowRestore(false)
                    }}
                    onOkClick={onRestore}
                /> : null}
            </div>
            {showSwitchover ? <SwitchRedisNodeDialog
                nodes={redis_details?.slaves}
                visible={showSwitchover}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowSwitchover(false)
                }}
                onOkClick={onSwitchover}
            />
                : null}
            {redis_details && showStepdown ? <StepdownRedisNodeDialog
                masters={redis_details?.masters?.filter((m) => m?.ex_params?.monitored_by?.length > 0)}
                visible={showStepdown}
                onHide={() => {
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowStepdown(false)
                }}
                onOkClick={onStepdown}
            />
                : null}
            {redis_details && showMonitorMaster ? <SentinelMonitorMasterDialog
                visible={showMonitorMaster}
                onHide={() => {
                    dispatch(LayoutActions.setValidName(undefined));
                    dispatch(LayoutActions.setErrorMessage(""));
                    setShowMonitorMaster(false);
                }
                }
                onOkClick={onSentinelMonitorMaster}
                details
                masters={redis_details?.masters}
                sentinels={[redis_details?.node]}
                max_quorum={redis_details?.sentinels_count}
            /> : null
            }
            <ConfirmPopup
                icon="pi pi-exclamation-triangle"
                className='multiline'
                target={document.getElementById(showWarningPopup)}
                visible={Boolean(showWarningPopup)}
                onHide={() => setShowWarningPopup(false)}
                accept={onPopupAccept}
                message={`Are you sure you want to ${popupAction} the node?`} />
            {redis_details && <Tooltip target=".set-schedule-star" content={redis_details?.auto_backup?.detail}></Tooltip>}
            {redis_details && <Tooltip target=".avail-star" content={redis_details?.node?.ind_started ? "Database is running and accepting connections" : "Database is not running"}></Tooltip>}
            {redis_details && <Tooltip target=".status-star" content={(redis_details?.ind_error === undefined || redis_details?.ind_error === true) ? "Some nodes of cluster are down/inaccessible" : "All nodes of cluster are accessible"}></Tooltip>}
        </Fragment>
    )
}
