import axios from "axios";
import config, { types } from "../config";
const refreshToken = async (username, password, org_name) => {
    const url = `${config.apiUrl}/cluster-core/api/token/`;
    const res = await axios.post(url, { username: username.toLowerCase(), password: password, org_name: org_name });
    return res?.data;
}
const accessToken = async (refreshToken) => {
    const url = `${config.apiUrl}/cluster-core/api/token/refresh/`;
    const res = await axios.post(url, { refresh: refreshToken });
    return res?.data;
}

const addIntegratedService = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_integrated_service/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;

}

const editIntegratedService = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/edit_integrated_service/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const dropIntegratedService = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_integrated_service/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;

}
const getIntegratedServices = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/get_integrated_services/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getNodes = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/node_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getAvailableHosts = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/host_available_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getAvailableNodes = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/node_available_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}

const getAvailableTree = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/cluster_available_tree/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}

const getDatabases = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/database_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getNode = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/get_node/${id}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getNodeDatabases = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/get_node_databases/${id}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getMemTotal = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/get_mem_max_last/${id}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const changeClusterSize = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/change_cluster_size/`;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
    return res?.data;
}
const getHostsCpuUsages = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/hosts_cpu_usages/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    ;
    return error_data ? error_data : res?.data;
}
const getHostCpuUsageDaily = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/host_cpu_usage_daily/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    ;
    return error_data ? error_data : res?.data;
}
const getLicensedClusters = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/get_licensed_clusters/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    ;
    return error_data ? error_data : res?.data;
}
const moveLicense = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/move_license/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    ;
    return error_data ? error_data : res?.data;
}
const getNodeLogFiles = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/get_node_log_files/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    ;
    return error_data ? error_data : res?.data;
}
const getNodesBackups = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/nodes_backups_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getExportsByType = async (accessToken, type) => {
    const url = `${config.apiUrl}/cluster-core/api/exports_list/${type}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getBackupsByType = async (accessToken, type) => {
    const url = `${config.apiUrl}/cluster-core/api/backups_list/${type}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getDatabasesBackups = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/databases_backups_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getDatabasesExports = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/databases_exports_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getNodesExports = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/nodes_exports_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const dropBackup = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_backup/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}

const dropCluster = async (accessToken, data) => {
    const api = config.apiUrl;
    const type = data?.type
    let url = "empty";
    switch (type) {
        case types.POSTGRES_K8S:
            url = `${api}/kubernetes/api/drop_pg_cluster/`;
            break
        case types.MONGO_K8S:
            url = `${api}/kubernetes/api/drop_mongo_cluster/`;
            break;
        case types.POSTGRES:
            url = `${api}/cluster-core/api/drop_pg_cluster/`;
            break;
        case types.MONGO:
            url = `${api}/cluster-core/api/drop_mongo_cluster/`;
            break;
        case types.MYSQL:
            url = `${api}/cluster-core/api/drop_mysql_cluster/`;
            break;
        case types.MARIADB:
            url = `${api}/cluster-core/api/drop_mariadb_cluster/`;
            break;
        case types.ELASTIC:
            url = `${api}/cluster-core/api/drop_elasic_cluster/`;
            break;
        case types.REDIS:
            url = `${api}/cluster-core/api/drop_elasic_cluster/`;
            break;
        default:
            break;



    }
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const dropDatabase = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_database/`;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getLogs = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/task_log_list/${data.time_period}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getNodeLogs = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/task_log_list/${data.node_id}/${data.time_period}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getTimelineData = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/user_task_logs_by_type/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getStepLogs = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/task_step_log_detail/${id}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getSelf = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/current_user/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getConfig = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/cluster_config/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getSystemSettings = async (accessToken) => {
  const url = `${config.apiUrl}/cluster-core/api/system_settings/`;
  const res = await axios.get(url, {
      headers: {
          "Content-Type": "application/json",
          "Authorization": "Bearer " + accessToken
      }
  });
  return res?.data;
}

const getNodesCpuUsages = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/host_monitor_cpu_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getNodesMemUsages = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/host_monitor_mem_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const clusterUpdate = async (accessToken, action) => {
    const url = `${config.apiUrl}/cluster-core/api/cluster_update/`;
    const res = await axios.put(url, JSON.stringify(action.data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res.status;
}
const updateSystemConfig = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/update_system_config/`;
    const res = await axios.put(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res.data;
}
const updateSystemSettings = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/update_system_settings/`;
    const res = await axios.put(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res.data;
}
const signUp = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/signup/`;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res;
}
const validateEmail = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/validate_email/`;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const userUpdate = async (accessToken, action) => {
    const url = `${config.apiUrl}/cluster-core/api/user_update/${action.payload.pk}/`;
    if (action.payload.data) {
        const res = await axios.put(url, JSON.stringify(action.payload.data), {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
        return res?.data;
    }
    else {
        const res = await axios.get(url, {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
        return res?.data;
    }
}
const getToasts = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/task_log_list_last/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const isRunningCompleted = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/is_running_completed/`;
    const res = await axios.post(url, data, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getTaskNotifications = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/task_running_last_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const checkPortExists = async (accessToken, data) => {
    const url = data.type && [types.MONGO_K8S, types.POSTGRES_K8S].includes(data.type) ? `${config.apiUrl}/kubernetes/api/check_port_exists/` : `${config.apiUrl}/cluster-core/api/check_port_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const checkFreeIp = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_free_ip/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}

const requestDemo = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/request_demo/`;
    let error_data = false;
    const res = await axios.get(url, {
        params: {
            email: data?.email,
        }
    }).catch(function (error) {
        if (error.response) {
            error_data = error.response.data
        }
    });
    return error_data ? error_data : res?.data;
}
const checkServiceConnection = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_${data?.type?.toLowerCase()}_connection/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const verifyBackup = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/verify_backup/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const checkInventoryExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_inventory_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const deleteUser = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/user_delete/${id}/`;
    let error_data = false;
    const res = await axios.delete(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const getDatabaseUsers = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/database_users/${id}/`;
    let error_data = false;
    const res = await axios.get(url,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}

const checkFreePodIp = async (accessToken, data) => {
    const url = `${config.apiUrl}/kubernetes/api/get_unused_node_ip/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const getSubnets = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/get_subnets/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const checkConnection = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_connection/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const checkPathExists = async (accessToken, data) => {
    const url = data.type && [types.MONGO_K8S, types.POSTGRES_K8S].includes(data.type) ? `${config.apiUrl}/kubernetes/api/check_path_exists/` : `${config.apiUrl}/cluster-core/api/check_path_exists/`
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const dropDatabaseUser = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_database_user/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}

const getStatusCharts = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/get_status_charts/`;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            console.log(error)

        });
    return res?.data;
}
const isQuotaExceed = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/is_quota_exceed/`;
    const res = await axios.post(url, data, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}

const isNodeHostReachable = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/is_node_host_reachable/`;
    const res = await axios.post(url, data, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}

const markNodeDeleted = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/mark_node_deleted/`;
    const res = await axios.post(url, data, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}

const removeLicense = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/removeLicense/`;
    const res = await axios.post(url, data, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}


const editDatabaseUser = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/edit_database_user/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const addLicense = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_license/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}

const getNodeSeverityCounts = async (nodeId, accessToken) => {
    if (nodeId == null || accessToken == null) {
         return null;
    }

    const url = `${config.apiUrl}/cluster-core/api/get_node_alerts_severity_counts/`;
    let error_data = false;

    const data = { node_id: nodeId };

    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    }).catch(function (error) {
        if (error.response) {
            error_data = error.response;
        } else if (error.request) {
            error_data = {
                 status: -1,
                 data: { detail: "Network error or no response from server."}
             };
        } else {
             error_data = {
                 status: -2,
                 data: { detail: error.message }
             };
        }
    });

    return error_data ? error_data : res?.data;
};

const api = {
    refreshToken,
    accessToken,
    getNodes,
    getDatabaseUsers,
    getAvailableNodes,
    getNodesCpuUsages,
    getNodesMemUsages,
    getLogs,
    getNodeLogs,
    getConfig,
    getSystemSettings,
    clusterUpdate,
    deleteUser,
    signUp,
    userUpdate,
    getToasts,
    getTaskNotifications,
    getNode,
    getNodeSeverityCounts,
    getSelf,
    getStepLogs,
    getNodesBackups,
    getNodesExports,
    dropBackup,
    getTimelineData,
    getMemTotal,
    checkPortExists,
    checkPathExists,
    getDatabases,
    dropDatabase,
    getDatabasesBackups,
    getDatabasesExports,
    updateSystemConfig,
    updateSystemSettings,
    changeClusterSize,
    checkConnection,
    isQuotaExceed,
    dropDatabaseUser,
    editDatabaseUser,
    getHostsCpuUsages,
    getHostCpuUsageDaily,
    getAvailableHosts,
    getNodeDatabases,
    validateEmail,
    verifyBackup,
    checkInventoryExists,
    addLicense,
    getLicensedClusters,
    moveLicense,
    getBackupsByType,
    getExportsByType,
    getStatusCharts,
    getAvailableTree,
    isNodeHostReachable,
    markNodeDeleted,
    isRunningCompleted,
    getIntegratedServices,
    addIntegratedService,
    editIntegratedService,
    dropIntegratedService,
    removeLicense,
    checkFreeIp,
    checkFreePodIp,
    getNodeLogFiles,
    checkServiceConnection,
    requestDemo,
    getSubnets,
    dropCluster,

}
export default api;
