import config from "../../../config";
import axios from "axios";
const getHAProxyDetails = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/status/haproxy/${id}/`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}
const checkHAProxyExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_haproxy_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const getHAProxies = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/haproxy_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const addHAProxy = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_haproxy/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const dropHAProxy = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_haproxy_node/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const stopHAProxy = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/stop_haproxy_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const startHAProxy = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/start_haproxy_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const apiHAProxy =
{
 getHAProxies,
 addHAProxy,
 dropHAProxy,
 stopHAProxy,
 startHAProxy,
 getHAProxyDetails,
 checkHAProxyExists
}
export default apiHAProxy;