import axios from "axios";
import config, { types } from "../../config";
const addMongoMaster = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_mongo_master/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const upgradeMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/upgrade_mongo_node/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const addMongoShardedClusterPrimary = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_mongo_sharded_cluster_primary/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const addMongoShardedCluster = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_mongo_sharded_cluster/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const addMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_mongo_node/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const addMongoShardedClusterNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_mongo_sharded_cluster_node/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const checkMongoDatabaseSize = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_mongo_database_size/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
    }
const checkMongoVersionExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_mongo_version_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const dropMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_mongo_${data.type === types.MONGO ? 'node' : 'pod'}/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const getMongoClusterStatus = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/mongo_cluster_status/${id}/`;
    let error_data = false;
    const res = await axios.post(url, null,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const backupMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/backup_mongo_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const removeMongoSchedule = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/remove_mongo_schedule/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const dumpMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/dump_mongo_${data.type === types.MONGO_K8S ? 'pod' : 'node'}/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const restoreBackupMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/restore_backup_mongo_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const restoreDumpMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/restore_dump_mongo_${data.type === types.MONGO_K8S ? 'pod' : 'node'}/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const stepDownMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/step_down_mongo_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const getMongoDetails = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/status/mongo/${id}/`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}
const addMongoDatabase = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_mongo_database/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const checkMongoDatabaseName = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_mongo_database_name/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const dropMongoDatabase = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_mongo_database/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const dumpMongoDatabase = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/dump_mongo_database/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const startMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/start_mongo_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const stopMongoNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/stop_mongo_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const restoreMongoDatabase = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/restore_mongo_database/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const copyMongoDatabase = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/copy_mongo_database/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const getMongoShardedClusters = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/mongo_sharded_cluster_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}

const dropMongoShardedCluster = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_mongo_sharded_cluster/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    }).catch(function (error) {
        if (error.response) {
            error_data = error.response
        }
    });
    return error_data ? error_data : res;
}
const apiMongo = 
{
    addMongoMaster,
    addMongoNode,
    dumpMongoNode,
    checkMongoVersionExists,
    checkMongoDatabaseSize,
    backupMongoNode,
    restoreBackupMongoNode,
    restoreDumpMongoNode,
    removeMongoSchedule,
    dropMongoNode,
    getMongoClusterStatus,
    stepDownMongoNode,
    addMongoDatabase,
    checkMongoDatabaseName,
    dropMongoDatabase,
    startMongoNode,
    stopMongoNode,
    dumpMongoDatabase,
    copyMongoDatabase,
    restoreMongoDatabase,
    getMongoDetails,
    getMongoShardedClusters,
    addMongoShardedCluster,
    dropMongoShardedCluster,
    addMongoShardedClusterPrimary,
    addMongoShardedClusterNode,
    upgradeMongoNode,
}
export default apiMongo;