import axios from "axios";
import config, { types } from "../../config";
const checkOracleVersionExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_oracle_version_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const addCdbNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_cdb/`;
    let error_data = false;
    const res = await axios.post(url, data,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res;
}

const installCdbNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/install_cdb/`;
    let error_data = false;
    const res = await axios.post(url, data,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res;
}

const addPdbNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_pdb/`;
    let error_data;
    
    const res = await axios.post(url, data,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
    
}
const fetchDisks =  async (accessToken, data) => 
{
    const url = `${config.apiUrl}/cluster-core/api/fetch_disks/`;
    let error_data;
    const res = await axios.post(url, data,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }});
    return error_data ? error_data : res?.data;
}
const fetchSchemas =  async (accessToken, data) => 
{
    const url = `${config.apiUrl}/cluster-core/api/fetch_schemas/`;
    let error_data;
    const res = await axios.post(url, data,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }});
    return error_data ? error_data : res?.data;
}
const getMaxPDB =  async (accessToken, data) => 
{
    
    let u = new URLSearchParams(data).toString();
    const url = `${config.apiUrl}/cluster-core/api/get_max_pdbs/?${u}`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}
const getOracleHome =  async (accessToken, data) => 
{
    
    let u = new URLSearchParams(data).toString();
    const url = `${config.apiUrl}/cluster-core/api/get_oracle_home/?${u}`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const getDBFileDest =  async (accessToken, id) => 
{
    const url = `${config.apiUrl}/cluster-core/api/get_db_file_dest/${id}/`; 
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
        return error_data ? error_data : res?.data;
}
const duplicateOracleStandalone = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/duplicate_oracle_standalone/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
;
    return error_data ? error_data : res?.data;
}
const exportCDB_PDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/export_cdb_pdb/`;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
    return res?.data;
}
const deleteCDBNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/delete_cdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const importPDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/import_pdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const importCDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/import_cdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const flashbackCDB_PDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/flashback_cdb_pdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const backupPDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/backup_pdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const backupCDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/backup_cdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const restoreFlashback = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/restore_from_flashback/`;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
    return res?.data;
}
const restorePDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/restore_backup_pdb/`;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
    return res?.data;
}
const copyToCDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/copy_to_cdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});;
    return error_data ? error_data : res?.data;
}
const copyPDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/copy_pdb/`;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
    return res?.data;
}
const renamePDB = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/rename_pdb/`;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        });
    return res?.data;
}
const checkExportExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_export_file/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const checkDiskSpace = async (accessToken, data) => {
    const url = data.type === types.POSTGRES_K8S ?  `${config.apiUrl}/kubernetes/api/check_disk_space/` :  `${config.apiUrl}/cluster-core/api/check_disk_space/` ;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const checkDirExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_disk_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const checkUserExists = async (accessToken, data) => {
    let u = new URLSearchParams(data).toString();
    const url = `${config.apiUrl}/cluster-core/api/check_user_exists/?${u}`;
    let error_data = false;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const checkDumpFile = async (accessToken, data) => {
    let u = new URLSearchParams(data).toString();
    const url = `${config.apiUrl}/cluster-core/api/check_dump_file/?${u}`;
    let error_data = false;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const checkPathName = async (accessToken, data) => {
    let u = new URLSearchParams(data).toString();
    const url = `${config.apiUrl}/cluster-core/api/check_oracle_db_path/?${u}`;
    let error_data = false;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const checkPDBName = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_pdb_name/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const closePDBNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/close_pdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const openPDBNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/open_pdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const deletePDBNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_pdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const getPdbDetails = async (accessToken, pk, id) =>
{
    let error_data;
    const url = `${config.apiUrl}/cluster-core/api/status/oracle/cdb/${pk}/pdb/${id}/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    }).catch(function (error) {
        if (error.response) {
          error_data = error.response
        }});
    
    return error_data ? error_data : res?.data;
}
const getCdbDetails = async (accessToken, pk) =>
{
    const url = `${config.apiUrl}/cluster-core/api/status/oracle/cdb/${pk}/`;
    let error_data;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    }).catch(function (error) {
        if (error.response) {
          error_data = error.response
        }});;
    
    return error_data ? error_data : res?.data;
}
const updatePDBCredentials = async (accessToken, data) => 
{ 
    const url = `${config.apiUrl}/cluster-core/api/update_pdb_credentials/`;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res.status;
}
const importStandalone = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/tns_file_upload/`;
    let formData = new FormData();
    formData.append("file_source", data);
    const res = await axios.post(url, formData, {
        headers: {
            'Content-Type': 'multipart/form-data',
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getOracleStandalones = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/oracle_standalones_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const getNodesFlashbacks = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/nodes_flashbacks_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const dropFlashback = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_flashback/`;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const dropStandalone = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_standalone/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    }).catch(function (error) {
        if (error.response) {
          error_data = error.response.data
        }});
    return error_data ? error_data : res?.data;
}
const checkPDBCredentials = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_credentials_pdb/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    }).catch(function (error) {
        if (error.response) {
          error_data = error.response.data
        }});
    return error_data ? error_data : res?.data;
}
const removeOracleSchedule = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/remove_oracle_schedule/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}

const getCDBSettings = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/get_cdb_settings/${id}/`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}

const editCDBSettings = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/edit_cdb_settings/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}

const apiOracle = 
{
    addCdbNode,
    addPdbNode,
    fetchDisks,
    fetchSchemas,
    getMaxPDB,
    getOracleHome,
    getDBFileDest,
    duplicateOracleStandalone,
    exportCDB_PDB,
    deleteCDBNode,
    importPDB,
    flashbackCDB_PDB,
    backupPDB,
    restoreFlashback,
    restorePDB,
    copyToCDB,
    copyPDB,
    renamePDB,
    checkExportExists,
    checkDiskSpace,
    checkDirExists,
    checkUserExists,
    checkDumpFile,
    checkPathName,
    checkPDBName,
    closePDBNode,
    openPDBNode,
    deletePDBNode,
    getPdbDetails,
    getCdbDetails,
    updatePDBCredentials,
    importStandalone,
    getOracleStandalones,
    getNodesFlashbacks,
    dropFlashback,
    dropStandalone,
    checkPDBCredentials,
    removeOracleSchedule,
    backupCDB,
    importCDB,
    installCdbNode,
    checkOracleVersionExists,
    getCDBSettings,
    editCDBSettings
}
export default apiOracle;