import axios from "axios";
import config, { types } from "../../config";
const checkPgVersionExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_pg_version_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const removePgSchedule = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/remove_pg_schedule/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const checkPgUserExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_pg_user_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const dropPgPublication = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_pg_publication/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const togglePgSubscription = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/toggle_pg_subscription/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const upgradePgNode = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/cluster-core/api/upgrade_k8s_pg_cluster/` :  `${config.apiUrl}/cluster-core/api/upgrade_pg_node/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const addPgPrimary = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_pg_primary/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const addPgStandby = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_pg_standby/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const checkPgDatabaseName = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/check_pg_pod_database_name/` : `${config.apiUrl}/cluster-core/api/check_pg_database_name/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const getPgDetails = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/status/postg/${id}/`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}
const getPgSettings = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/get_pg_settings/${id}/`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}

const editPgSettings = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/edit_pg_settings/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
        return error_data ? error_data : res?.data;
}
const checkPgDatabaseSize = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/check_pg_pod_database_size/` : `${config.apiUrl}/cluster-core/api/check_pg_database_size/`;
    let error_data;
    const res = await axios.post(url,  JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response.data
            }});
        return error_data ? error_data : res?.data;
}
const backupPgNode = async (accessToken, data) => {
    const url = data.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/backup_pg_pod/` : `${config.apiUrl}/cluster-core/api/backup_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const dropPgNode = async (accessToken, data) => {
    const url = data.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/drop_pg_pod/` : `${config.apiUrl}/cluster-core/api/drop_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const stopPgNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/stop_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const togglePgCluster = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/toggle_pg_cluster/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const switchPgNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/switch_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const clonePgStandby = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/clone_pg_standby/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const startPgNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/start_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const exportPgDatabase = async (accessToken, data) => {
    const url = data.type === types.POSTGRES_K8S ?  `${config.apiUrl}/kubernetes/api/export_pg_database_pod/`  :`${config.apiUrl}/cluster-core/api/export_pg_database/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const exportPgNode = async (accessToken, data) => {
    const url = data.type === types.POSTGRES_K8S ?  `${config.apiUrl}/kubernetes/api/export_pg_pod/` :   `${config.apiUrl}/cluster-core/api/export_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const importPgNode = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/import_pg_pod/` : `${config.apiUrl}/cluster-core/api/import_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const restorePgDatabase = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/restore_k8s_pg_db/`  :`${config.apiUrl}/cluster-core/api/restore_pg_database/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const restorePgNode = async (accessToken, data) => {
    const url =  data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/restore_pg_pod/` : `${config.apiUrl}/cluster-core/api/restore_pg_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
              error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const addPgDatabase = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/add_pg_database_pod/` : `${config.apiUrl}/cluster-core/api/add_pg_database/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const dropPgDatabase = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/drop_pg_database_pod/` : `${config.apiUrl}/cluster-core/api/drop_pg_database/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const getPgPrimaryStatus = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/pg_primary_status/${id}/`;
    let error_data = false;
    const res = await axios.post(url, null,
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const checkIfPgPrimary = async (accessToken, pk) => {
    const url = `${config.apiUrl}/cluster-core/api/check_if_pg_primary/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify({"node_id": pk}),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const copyPgDatabase = async (accessToken, data) => {
    const url = data?.type === types.POSTGRES_K8S ? `${config.apiUrl}/kubernetes/api/copy_pg_database_pod/` : `${config.apiUrl}/cluster-core/api/copy_pg_database/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }});
    return error_data ? error_data : res?.data;
}
const apiPostg = 
{
    
    checkPgVersionExists,
    addPgPrimary,
    addPgStandby,
    checkPgDatabaseName,
    checkPgDatabaseSize,
    addPgDatabase,
    backupPgNode,
    exportPgDatabase,
    restorePgDatabase,
    exportPgNode,
    stopPgNode,
    startPgNode,
    checkPgUserExists,
    restorePgNode,
    getPgDetails,
    getPgSettings,
    editPgSettings,
    dropPgNode,
    copyPgDatabase,
    dropPgDatabase,
    getPgPrimaryStatus,
    switchPgNode,
    togglePgCluster,
    removePgSchedule,
    checkIfPgPrimary,
    clonePgStandby,
    importPgNode,
    togglePgSubscription,
    dropPgPublication,
    upgradePgNode
}
export default apiPostg;