import axios from "axios";
import config from "../../config";
const addRedisStandalone = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_redis_standalone/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const removeRedisCluster = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/remove_redis_cluster/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data), {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    }).catch(function (error) {
        if (error.response) {
            error_data = error.response
        }
    });
    return error_data ? error_data : res;
}
const addRedisCluster = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/add_redis_cluster/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const checkRedisVersionExists = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_redis_version_exists/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const checkRedisClusterHealth = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_redis_cluster_health/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const startRedisNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/start_redis_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const stopRedisNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/stop_redis_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const restoreRedisNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/restore_redis_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const backupRedisNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/backup_redis_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}

const getRedisDetails = async (accessToken, id) => {
    const url = `${config.apiUrl}/cluster-core/api/status/redis/${id}/`;
    let error_data;
    const res = await axios.get(url,
        {
            headers: {
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const dropRedisNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/drop_redis_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const removeRedisSchedule = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/remove_redis_schedule/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const sentinelRemoveMonitor = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/sentinel_remove_monitor/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const addRedisNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/create_redis_node/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const switchRedisNode = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/switch_redis_node/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const sentinelFailover = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/sentinel_failover/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
}
const getRedisClusters = async (accessToken) => {
    const url = `${config.apiUrl}/cluster-core/api/redis_cluster_list/`;
    const res = await axios.get(url, {
        headers: {
            "Content-Type": "application/json",
            "Authorization": "Bearer " + accessToken
        }
    });
    return res?.data;
}
const checkSentinelMasterName = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/check_sentinel_master_name/`;
    let error_data;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response.data
            }
        });
    return error_data ? error_data : res?.data;
}
const sentinelMonitorMaster = async (accessToken, data) => {
    const url = `${config.apiUrl}/cluster-core/api/sentinel_monitor_master/`;
    let error_data = false;
    const res = await axios.post(url, JSON.stringify(data),
        {
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + accessToken
            }
        }).catch(function (error) {
            if (error.response) {
                error_data = error.response
            }
        });
    return error_data ? error_data : res?.data;
    
}
const apiRedis =
{
    addRedisStandalone,
    backupRedisNode,
    restoreRedisNode,
    removeRedisSchedule,
    dropRedisNode,
    getRedisDetails,
    startRedisNode,
    stopRedisNode,
    addRedisNode,
    addRedisCluster,
    checkRedisVersionExists,
    switchRedisNode,
    getRedisClusters,
    removeRedisCluster,
    checkRedisClusterHealth,
    checkSentinelMasterName,
    sentinelMonitorMaster,
    sentinelRemoveMonitor,
    sentinelFailover
}
export default apiRedis;