import {produce} from "immer";
import { createActions, createReducer } from "reduxsauce";
import { tableGroupingTypes, timePeriods, types } from "../../config";

const initialState = {
    isLoading: false,
    errorMessage: "",
    refreshToken: "",
    accessToken: "",
    user: null,
    db_type: types.ALL,
    time_period: timePeriods.HOUR,
    _search: "",
    timeline: false,
    grouping: tableGroupingTypes.CLUSTER,
    severity: {green: true, yellow: true, red: true}
};

const { Types, Creators } = createActions({
    setLoading: ["isLoading"],
    setWideRes: ["wideRes"],
    setErrorMessage: ["errorMessage"],
    setRefreshToken: ["refreshToken"],
    setAccessToken: ["accessToken"],
    setUsername: ["username"],
    setPassword: ["password"],
    setOrgName: ["org_name"],
    setDbType: ["db_type"],
    setTimePeriod: ["time_period"],
    setSeverity: ["severity"],
    setGrouping: ["grouping"],
    setTimeline: ["timeline"],
    setSearch: ["_search"],
    login: ["username", "password", "org_name"],
    logout: null,
    setUser: ["user"],
}, { prefix: 'APP/' });

export const AppActionTypes = Types;
export const AppActions = Creators;

export const appReducer = createReducer(initialState, {
    [AppActionTypes.SET_LOADING]: produce((draft, { isLoading }) => ({ ...draft, isLoading })),
    [AppActionTypes.SET_ERROR_MESSAGE]: produce((draft, { errorMessage }) => ({ ...draft, errorMessage })),
    [AppActionTypes.SET_REFRESH_TOKEN]: produce((draft, { refreshToken }) => ({ ...draft, refreshToken })),
    [AppActionTypes.SET_ACCESS_TOKEN]: produce((draft, { accessToken }) => ({ ...draft, accessToken })),
    [AppActionTypes.SET_USERNAME]: produce((draft, { username }) => ({ ...draft, username })),
    [AppActionTypes.SET_PASSWORD]: produce((draft, { password }) => ({ ...draft, password })),
    [AppActionTypes.SET_ORG_NAME]: produce((draft, { org_name }) => ({ ...draft, org_name })),
    [AppActionTypes.SET_TIME_PERIOD]: produce((draft, { time_period }) => ({ ...draft, time_period })),
    [AppActionTypes.SET_DB_TYPE]: produce((draft, { db_type }) => ({ ...draft, db_type })),
    [AppActionTypes.SET_SEVERITY]: produce((draft, { severity }) => ({ ...draft, severity })),
    [AppActionTypes.SET_GROUPING]: produce((draft, { grouping }) => ({ ...draft, grouping })),
    [AppActionTypes.SET_TIMELINE]: produce((draft, { timeline }) => ({ ...draft, timeline })),
    [AppActionTypes.SET_SEARCH]: produce((draft, { _search }) => ({ ...draft, _search })),
    [AppActionTypes.SET_USER]: produce((draft, { user }) => ({ ...draft, user })),

});
