import { call, put, takeLeading } from "redux-saga/effects";
import { AppActions, AppActionTypes } from "./reducer";
import { LayoutActions } from "../layout/reducer";
import { StatusActions } from "../status/reducer";
import api from "../../services/api";
import { ConfigActions } from "../config/reducer";
function* login({ username, password, org_name }) {
    try {
        yield put(AppActions.setLoading(true));
        
        const { refresh } = yield call(api.refreshToken, username, password, org_name);
        const { access } = yield call(api.accessToken, refresh);
        const user = yield call(api.getSelf, access);
        
        yield put(AppActions.setRefreshToken(refresh));
        yield put(AppActions.setAccessToken(access));
        yield put(AppActions.setUsername(username));
        yield put(AppActions.setPassword(password));
        yield put(AppActions.setOrgName(org_name));
        yield put(AppActions.setUser(user));
        yield put(LayoutActions.setNodes(null));
        yield put(StatusActions.setNodes(null));
        yield put(AppActions.setErrorMessage(""));
        yield put(AppActions.setLoading(false));
    }
    catch (e) {
        
        yield put(AppActions.setLoading(false));
        yield put(AppActions.setRefreshToken(""));
        yield put(AppActions.setAccessToken(""));
        yield put(AppActions.setUser(null));
        yield put(AppActions.setErrorMessage(e.message.includes("code 401") ? "The username or password you entered is incorrect" : e.message));
    }
}
function* logout() {
    try {
        yield put(AppActions.setLoading(true));
        yield put(AppActions.setRefreshToken(""));
        yield put(AppActions.setAccessToken(""));
        yield put(AppActions.setUser(null));
        yield put(ConfigActions.setConfig(null));
        
        yield put(AppActions.setErrorMessage(""));
        yield put(AppActions.setLoading(false));
    }
    catch (e) {
        yield put(AppActions.setLoading(false));
        yield put(AppActions.setErrorMessage(e.message));
    }
}
const sagas = [
    takeLeading(AppActionTypes.LOGIN, login),
    takeLeading(AppActionTypes.LOGOUT, logout),
];
export default sagas;
    
    