import { call, put, takeLeading, select } from "redux-saga/effects";
import { ConfigActions, ConfigActionTypes } from "./reducer";
import api from "../../services/api";
import notify from "../../util/notify";
function* getConfig() {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let config = yield call(api.getConfig, accessToken);
        yield put(ConfigActions.setConfig(config));
        yield put(ConfigActions.setErrorMessage(""));
        yield put(ConfigActions.setLoading(false));
    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}

function* getIntegratedServices() {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const integrated_services = yield call(api.getIntegratedServices, accessToken);
        yield put(ConfigActions.setIntegratedServices(integrated_services));
        yield put(ConfigActions.setErrorMessage(""));
        yield put(ConfigActions.setLoading(false));
    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}


function* removeLicense() {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        yield call(api.removeLicense, accessToken);
        yield put(ConfigActions.setConfig(null));
        yield put(ConfigActions.setErrorMessage(""));
        yield put(ConfigActions.setLoading(false));
    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}

function* updateSystemConfig({ data }) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let response = yield call(api.updateSystemConfig, accessToken, data);
        yield call(notify.info, response.notification);
        yield put(ConfigActions.setLoading(false));
        yield put(ConfigActions.setConfig(null));
    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}
function* getDatabaseUsers({ id }) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.getDatabaseUsers, accessToken, id);
        if (response?.status >= 400) {
            yield put(ConfigActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(ConfigActions.setLoading(false));
            yield put(ConfigActions.setDatabaseUsers(response));
        }

    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}
function* dropDatabaseUser({ data }) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.dropDatabaseUser, accessToken, data);
        if (response?.status >= 400) {
            yield put(ConfigActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(ConfigActions.setLoading(false));
            yield put(ConfigActions.getDatabaseUsers(data.node_id));
            yield call(notify.success, response.notification)
        }

    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}
function* editDatabaseUser({ data, callback}) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.editDatabaseUser, accessToken, data);
        if (response?.status >= 400) {
            yield put(ConfigActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(ConfigActions.setLoading(false));
            yield call(callback, false);
            yield call(notify.success, response.notification)
        }

    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}

function* addIntegratedService({ data, callback}) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.addIntegratedService, accessToken, data);
        if (response?.status >= 400) {
            yield put(ConfigActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(ConfigActions.setLoading(false));
            yield call(callback, false);
            yield put(ConfigActions.setIntegratedServices(null));
        }

    }
    catch (e) {
        console.log(e)
        yield put(ConfigActions.setLoading(false));
    }
}
function* editIntegratedService({ data, callback}) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.editIntegratedService, accessToken, data);
        if (response?.status >= 400) {
            yield put(ConfigActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(ConfigActions.setLoading(false));
            yield call(callback, false);
            yield put(ConfigActions.setIntegratedServices(null));
        }

    }
    catch (e) {
        console.log(e)
        yield put(ConfigActions.setLoading(false));
    }
}
function* dropIntegratedService({ data}) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.dropIntegratedService, accessToken, data);
        if (response?.status >= 400) {
            yield put(ConfigActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(ConfigActions.setLoading(false));
            yield put(ConfigActions.setIntegratedServices(null));
        }

    }
    catch (e) {
        console.log(e)
        yield put(ConfigActions.setLoading(false));
    }
}
function* addLicense({ data, callback}) {
    try {

        yield put(ConfigActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.addLicense, accessToken, data);
        if (response?.status >= 400) {
            yield put(ConfigActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data)
        }
        else {
            yield put(ConfigActions.setLoading(false));
            yield call(callback.cleanup, false);
            if(response.notification) yield call(callback.notify, {"text": response.notification})
            yield put(ConfigActions.setConfig(null));
        }

    }
    catch (e) {
        yield put(ConfigActions.setLoading(false));
    }
}
const sagas = [
    takeLeading(ConfigActionTypes.GET_CONFIG, getConfig),
    takeLeading(ConfigActionTypes.UPDATE_SYSTEM_CONFIG, updateSystemConfig),
    takeLeading(ConfigActionTypes.GET_DATABASE_USERS, getDatabaseUsers),
    takeLeading(ConfigActionTypes.DROP_DATABASE_USER, dropDatabaseUser),
    takeLeading(ConfigActionTypes.EDIT_DATABASE_USER, editDatabaseUser),
    takeLeading(ConfigActionTypes.ADD_LICENSE, addLicense),
    takeLeading(ConfigActionTypes.REMOVE_LICENSE, removeLicense),
    takeLeading(ConfigActionTypes.GET_INTEGRATED_SERVICES, getIntegratedServices),
    takeLeading(ConfigActionTypes.ADD_INTEGRATED_SERVICE, addIntegratedService),
    takeLeading(ConfigActionTypes.EDIT_INTEGRATED_SERVICE, editIntegratedService),
    takeLeading(ConfigActionTypes.DROP_INTEGRATED_SERVICE, dropIntegratedService),


];
export default sagas;
