import { produce } from "immer";
import { createActions, createReducer } from "reduxsauce";

const initialState = {
  isLoading: false,
  progress: 0,
  errorMessage: "",
};

const { Types, Creators } = createActions(
  {
    setLoading: ["isLoading"],
    setProgress: ["progress"],
    setErrorMessage: ["errorMessage"],
    downloadFile: ["data"],
    downloadFileSuccess: null,
    downloadFileFailure: null,
    downloadFileAborted: null,
  },
  { prefix: "DOWNLOAD/" }
);

export const DownloadActionTypes = Types;
export const DownloadActions = Creators;

export const downloadReducer = createReducer(initialState, {
  [DownloadActionTypes.SET_LOADING]: produce((draft, { isLoading }) => {
    draft.isLoading = isLoading;
  }),
  [DownloadActionTypes.SET_PROGRESS]: produce((draft, { progress }) => {
    draft.progress = progress;
  }),
  [DownloadActionTypes.SET_ERROR_MESSAGE]: produce((draft, { errorMessage }) => {
    draft.errorMessage = errorMessage;
  }),
  [DownloadActionTypes.DOWNLOAD_FILE]: produce((draft) => {
    draft.isLoading = true;
    draft.progress = 0;
    draft.errorMessage = "";
  }),
  [DownloadActionTypes.DOWNLOAD_FILE_SUCCESS]: produce((draft) => {
    draft.isLoading = false;
    draft.progress = 100;
  }),
  [DownloadActionTypes.DOWNLOAD_FILE_FAILURE]: produce((draft) => {
    draft.isLoading = false;
    draft.progress = 0;
  }),
  [DownloadActionTypes.DOWNLOAD_FILE_ABORTED]: produce((draft) => {
    draft.isLoading = false;
    draft.progress = 0;
    draft.errorMessage = "Download was aborted.";  // Optional message
  }),
});