import { DownloadActions, DownloadActionTypes } from "./reducer";
import config from "../../config";
import { call, put, takeLeading, select } from "redux-saga/effects";

function* downloadFile({ data }) {
  try {
    yield put(DownloadActions.setLoading(true));

    const { accessToken } = yield select((state) => state.app);

    // Step 1: Request a signed download URL
    const response = yield call(fetch, `${config.apiUrl}/cluster-core/api/generate_signed_download_url/`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${accessToken}`,
      },
      body: JSON.stringify(data),
    });

    if (!response.ok) throw new Error("Failed to get download URL");

    const { downloadUrl } = yield response.json();

    setTimeout(() => {
      const a = document.createElement("a");
      a.href = downloadUrl;
      a.style.display = "none";
      a.download = data.filename || "";
      document.body.appendChild(a);
      a.click();
      document.body.removeChild(a);
    }, 100);

    yield put(DownloadActions.downloadFileSuccess());
  } catch (e) {
    yield put(DownloadActions.downloadFileFailure());
    yield put(DownloadActions.setErrorMessage(e.message));
  } finally {
    yield put(DownloadActions.setLoading(false));
  }
}

  const sagas = [
      takeLeading(DownloadActionTypes.DOWNLOAD_FILE, downloadFile),
  ];
  export default sagas;