import { LayoutActions } from "../../reducer";
import { StatusActions } from "../../../status/reducer";
import { TaskNotificationActions } from "../../../taskNotifications/reducer";
import { call, put, select } from "redux-saga/effects";
import notify from "../../../../util/notify";
import apiHAProxy from "../../../../services/balancers/haproxy/api";
function* addHAProxy({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiHAProxy.addHAProxy, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `adding HAProxy for cluster`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setHaProxyExists(undefined));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dropHAProxy({ data, callback }) {
    try {
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiHAProxy.dropHAProxy, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* checkHAProxyExists({ data }) {
    try {
        yield put(LayoutActions.setHaProxyExists(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const haproxy_exists = yield call(apiHAProxy.checkHAProxyExists, accessToken, data);
        if (haproxy_exists?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(haproxy_exists.detail));
            yield put(LayoutActions.setHaProxyExists(haproxy_exists));
        }
        else {
            yield put(LayoutActions.setHaProxyExists(haproxy_exists));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* startHAProxy({ data}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiHAProxy.startHAProxy, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* stopHAProxy({ data}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiHAProxy.stopHAProxy, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
const sagasHAProxy =
{
    addHAProxy,
    dropHAProxy,
    startHAProxy,
    stopHAProxy,
    checkHAProxyExists,
}
export default sagasHAProxy