import { call, put, select } from "redux-saga/effects";
import { LayoutActions } from "../reducer";
import { StatusActions } from "../../status/reducer";
import { TaskNotificationActions } from "../../taskNotifications/reducer";
import notify from "../../../util/notify";
import apiElasticSearch from "../../../services/elasticsearch/api";
function* checkElasticSearchVersionExists({ data }) {
    try {
        yield put(LayoutActions.setElasticSearchVersionExists(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const elasticsearch_version_exists = yield call(apiElasticSearch.checkElasticSearchVersionExists, accessToken, data);
        if (elasticsearch_version_exists?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(elasticsearch_version_exists.detail));
            yield put(LayoutActions.setElasticSearchVersionExists(elasticsearch_version_exists));
        }
        else {
            yield put(LayoutActions.setElasticSearchVersionExists(elasticsearch_version_exists));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addElasticSearchMaster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.addElasticSearchMaster, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
           yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Creating elasticsearch cluster ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setElasticSearchVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkElasticSearchIndexName({ data }) {
    try {
        yield put(LayoutActions.setValidName(undefined));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiElasticSearch.checkElasticSearchIndexName, accessToken, data);
        if (valid_name?.detail) {
            yield put(LayoutActions.setErrorMessage(valid_name.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setValidName(valid_name));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* dumpElasticSearchIndex({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.dumpElasticSearchIndex, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkElasticSearchIndexSize({ data }) {
    try {
        yield put(LayoutActions.setElasticsearchIndexSize(undefined));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const db_size = yield call(apiElasticSearch.checkElasticSearchIndexSize, accessToken, data);
        if (db_size?.detail) {
            yield put(LayoutActions.setErrorMessage(db_size.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setElasticsearchIndexSize(db_size));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkElasticSearchIndexExists({ data, callback }) {
    try {
        yield put(LayoutActions.setValidName(undefined));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiElasticSearch.checkElasticSearchIndexName, accessToken, data);
        if (valid_name?.detail || valid_name?.error) {
            yield put(LayoutActions.setErrorMessage(valid_name?.detail ? valid_name?.detail : valid_name.error));
            yield call(callback, true, true);
            yield put(LayoutActions.setLoading(false));
        }
        else {
            if (valid_name?.response === "already exists") yield call(callback, false, true);
            else if (valid_name?.response === "name available") yield call(callback, true, true);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* getElasticSearchClusterStatus({ id }) {
    try {
        yield put(LayoutActions.setElasticsearchClusterStatus(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const elasticsearch_cluster_status = yield call(apiElasticSearch.getElasticSearchClusterStatus, accessToken, id);
        if (elasticsearch_cluster_status?.data?.detail) {
            yield put(LayoutActions.setErrorMessage(elasticsearch_cluster_status.data.detail));
            yield put(LayoutActions.setElasticsearchClusterStatus(null));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setElasticsearchClusterStatus(elasticsearch_cluster_status));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addElasticSearchIndex({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.addElasticSearchIndex, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else { 
            yield call(callback, response);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* copyElasticSearchIndex({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.copyElasticSearchIndex, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* restoreElasticSearchIndex({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.restoreElasticSearchIndex, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* restoreElasticSearchCluster({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.restoreElasticSearchCluster, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* snapshotElasticSearchCluster({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.snapshotElasticSearchCluster, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* dropElasticSearchNode({  data, callback }) {
    try {
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.dropElasticSearchNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
            
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));

        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* removeElasticSearchSchedule({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.removeElasticSearchSchedule, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback);
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* stopElasticSearchNode({data}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.stopElasticSearchNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(true));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* startElasticSearchNode({  data}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.startElasticSearchNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addElasticSearchNode({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.addElasticSearchNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Adding elasticsearch node ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setElasticSearchVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* dropElasticSearchIndex({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.dropElasticSearchIndex, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* switchElasticsearchNode({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.switchElasticsearchNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));    
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* upgradeElasticSearchNode({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiElasticSearch.upgradeElasticSearchNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
const sagasElasticSearch =
{
    addElasticSearchMaster,
    checkElasticSearchVersionExists,
    addElasticSearchIndex,
    addElasticSearchNode,
    checkElasticSearchIndexSize,
    dumpElasticSearchIndex,
    getElasticSearchClusterStatus,
    checkElasticSearchIndexExists,
    copyElasticSearchIndex,
    restoreElasticSearchIndex,
    dropElasticSearchIndex,
    restoreElasticSearchCluster,
    snapshotElasticSearchCluster,
    removeElasticSearchSchedule,
    dropElasticSearchNode,
    startElasticSearchNode,
    stopElasticSearchNode,
    checkElasticSearchIndexName,
    switchElasticsearchNode, 
    upgradeElasticSearchNode
}
export default sagasElasticSearch;