import { call, put, select } from "redux-saga/effects";
import { LayoutActions } from "../reducer";
import { TaskNotificationActions } from "../../taskNotifications/reducer";
import apiMongo from "../../../services/mongo/api";
import { StatusActions } from "../../status/reducer";
import notify from "../../../util/notify";
function* checkMongoVersionExists({ data }) {
    try {
        yield put(LayoutActions.setMongoVersionExists(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const mongo_version_exists = yield call(apiMongo.checkMongoVersionExists, accessToken, data);
        if (mongo_version_exists?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(mongo_version_exists.detail));
            yield put(LayoutActions.setMongoVersionExists(mongo_version_exists));
        }
        else {
            yield put(LayoutActions.setMongoVersionExists(mongo_version_exists));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}

function* addMongoShardedCluster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.addMongoShardedCluster, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(notify.success, "Registered a new mongo sharded cluster");
            yield call(callback, false);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));    
    }
}
function* addMongoMaster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.addMongoMaster, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Creating mongo cluster ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setMongoVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addMongoShardedClusterPrimary({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.addMongoShardedClusterPrimary, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Creating MongoDB primary ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setMongoVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addMongoNode({data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.addMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Adding mongo node ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setMongoVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}

function* addMongoShardedClusterNode({data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.addMongoShardedClusterNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Adding mongo node ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setMongoVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* restoreBackupMongoNode({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.restoreBackupMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification)
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* backupMongoNode({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.backupMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setLoading(false));
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* removeMongoSchedule({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.removeMongoSchedule, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(notify.info, response.notification);
            yield call(callback);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dumpMongoNode({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.dumpMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setLoading(false));
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* restoreDumpMongoNode({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.restoreDumpMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* stepDownMongoNode({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.stepDownMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            if (callback) yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false))
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));;
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* getMongoClusterStatus({ id }) {
    try {
        yield put(LayoutActions.setMongoClusterStatus(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const mongo_cluster_status = yield call(apiMongo.getMongoClusterStatus, accessToken, id);
        if (mongo_cluster_status?.data?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(mongo_cluster_status.data.detail));
            yield put(LayoutActions.setMongoClusterStatus(null));
        }
        else {
            yield put(LayoutActions.setMongoClusterStatus(mongo_cluster_status));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dropMongoNode({  data, callback }) {
    try {
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.dropMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false))
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* checkMongoDatabaseSize({ data }) {
    try {
        yield put(LayoutActions.setPgDatabaseSize(undefined));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const db_size = yield call(apiMongo.checkMongoDatabaseSize, accessToken, data);
        if (db_size?.detail) {
            yield put(LayoutActions.setErrorMessage(db_size.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setMongoDatabaseSize(db_size));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* checkMongoDatabaseName({ data}) {
    try {
        yield put(LayoutActions.setValidName(undefined));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiMongo.checkMongoDatabaseName, accessToken, data);
        if (valid_name?.detail) {
            yield put(LayoutActions.setErrorMessage(valid_name.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setValidName(valid_name));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* checkMongoDatabaseExists({ data, callback}) {
    try {
        yield put(LayoutActions.setValidName(undefined));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiMongo.checkMongoDatabaseName, accessToken, data);
        if (valid_name?.detail || valid_name?.error) {
            yield put(LayoutActions.setErrorMessage(valid_name?.detail ? valid_name?.detail : valid_name.error));
            yield call(callback, true, true);
            yield put(LayoutActions.setLoading(true));
        }
        else {
            if(valid_name?.response === "already exists") yield call(callback, false, true)
            else if (valid_name?.response === "name available") yield call(callback, true, true)
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* addMongoDatabase({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.addMongoDatabase, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, response);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dropMongoDatabase({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.dropMongoDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setLoading(false));
            yield put(TaskNotificationActions.getNotifications());
            yield call(callback, false);
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* restoreMongoDatabase({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.restoreMongoDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* startMongoNode({  data}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.startMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* stopMongoNode({  data}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.stopMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dumpMongoDatabase({  data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.dumpMongoDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield call(notify.info, response.notification);
            yield call(callback, false);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* copyMongoDatabase({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.copyMongoDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
      
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}

function* dropMongoShardedCluster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.dropMongoShardedCluster, accessToken, data);
       
        if (response?.status >= 400)
        {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(response?.data?.detail));
        }
        else {
            yield call(callback, false);
            yield call(notify.success, response?.data?.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* upgradeMongoNode({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMongo.upgradeMongoNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
const sagasMongo =
{
    addMongoMaster,
    addMongoNode,
    dumpMongoNode,
    checkMongoDatabaseSize,
    checkMongoVersionExists,
    restoreBackupMongoNode,
    restoreDumpMongoNode,
    backupMongoNode,
    dropMongoNode,
    getMongoClusterStatus,
    stepDownMongoNode,
    addMongoDatabase,
    checkMongoDatabaseName,
    startMongoNode,
    stopMongoNode,
    dropMongoDatabase,
    restoreMongoDatabase,
    dumpMongoDatabase,
    checkMongoDatabaseExists,
    copyMongoDatabase,
    removeMongoSchedule,
    addMongoShardedCluster,
    dropMongoShardedCluster,
    addMongoShardedClusterPrimary,
    addMongoShardedClusterNode,
    upgradeMongoNode
}
export default sagasMongo;