import { call, put, select } from "redux-saga/effects";
import { LayoutActions } from "../reducer";
import { StatusActions } from "../../status/reducer";
import { TaskNotificationActions } from "../../taskNotifications/reducer";
import notify from "../../../util/notify";
import apiMsSQL from "../../../services/mssql/api";
function* checkMsSQLVersionExists({ data }) {
    try {
        yield put(LayoutActions.setMsSqlVersionExists(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const mssql_version_exists = yield call(apiMsSQL.checkMsSQLVersionExists, accessToken, data);
        if (mssql_version_exists?.detail) {
            yield put(LayoutActions.setErrorMessage(mssql_version_exists.detail));
            yield put(LayoutActions.setMsSqlVersionExists(mssql_version_exists));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setMsSqlVersionExists(mssql_version_exists));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* importMsSQLNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.importMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setLoading(false));
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* checkMsSQLUserExists({ data }) {
    try {
        yield put(LayoutActions.setMsSqlUserExists(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const mssql_user_exists = yield call(apiMsSQL.checkMsSQLUserExists, accessToken, data);
        if (mssql_user_exists?.detail) {
            yield put(LayoutActions.setErrorMessage(mssql_user_exists.detail));
            yield put(LayoutActions.setMsSqlUserExists(mssql_user_exists));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setMsSqlUserExists(mssql_user_exists));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addMsSQLStandby({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.addMsSQLStandby, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {      
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Adding mssql standby ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setMsSqlVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
    
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* restoreMsSQLDatabase({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
    
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.restoreMsSQLDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false)); 
    }
}
function* checkMsSQLDatabaseName({ data}) {
    try {
        yield put(LayoutActions.setValidName(undefined));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiMsSQL.checkMsSQLDatabaseName, accessToken, data);
        if (valid_name?.detail) {
            yield put(LayoutActions.setErrorMessage(valid_name.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setValidName(valid_name));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* addMsSQLMaster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.addMsSQLMaster, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Creating mssql cluster ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setMsSqlVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
    
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* addMsSQLDatabase({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.addMsSQLDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, response);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* exportMsSQLDatabase({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.exportMsSQLDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
    
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* exportMsSQLNode({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.exportMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
    
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* checkMsSQLDatabaseSize({ data }) {
    try {
        yield put(LayoutActions.setMsSqlDatabaseSize(undefined));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const db_size = yield call(apiMsSQL.checkMsSQLDatabaseSize, accessToken, data);
        if (db_size?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(db_size.detail));
        }
        else {
            yield put(LayoutActions.setMsSqlDatabaseSize(db_size));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dropMsSQLDatabase({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.dropMsSQLDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* switchMsSQLNode({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.switchMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
        }
        else {
            yield call(notify.info, response.notification);
            yield call(callback,false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dropMsSQLNode({ data, callback }) {
    try {
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.dropMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* stopMsSQLNode({ data, node}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.stopMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* startMsSQLNode({ data, node}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.startMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getMsSqlDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* checkMsSQLDatabaseExists({ data, callback}) {
    try {
        yield put(LayoutActions.setValidName(undefined));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiMsSQL.checkMsSQLDatabaseName, accessToken, data);
        if (valid_name?.detail || valid_name?.error) {
            if (valid_name?.error === "already exists")
            {
                yield call(callback, false, true);
            }
            else {
                yield put(LayoutActions.setErrorMessage(valid_name?.detail ? valid_name?.detail : valid_name.error));
                yield call(callback, false, true);
            }
            yield put(LayoutActions.setLoading(false));  
        }
        else {
            if (valid_name?.response === "name available") yield call(callback, true, true)
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* backupMsSQLNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.backupMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(notify.info, response.notification);
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
    
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* restoreMsSQLNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.restoreMsSQLNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setLoading(false));
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
    
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* removeMsSQLSchedule({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.removeMsSQLSchedule, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback);
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* getMsSQLClusterStatus({ id }) {
    try {
        yield put(LayoutActions.setMongoClusterStatus(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const mssql_cluster_status = yield call(apiMsSQL.getMsSQLClusterStatus, accessToken, id);
        if (mssql_cluster_status?.data?.detail) {
            yield put(LayoutActions.setErrorMessage(mssql_cluster_status.data.detail));
            yield put(LayoutActions.setMsSqlClusterStatus(null));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setMsSqlClusterStatus(mssql_cluster_status));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* copyMsSQLDatabase({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMsSQL.copyMsSQLDatabase, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
    
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}

const sagasMsSQL =
{
    addMsSQLMaster,
    checkMsSQLDatabaseName,
    addMsSQLDatabase,
    checkMsSQLDatabaseSize,
    exportMsSQLDatabase,
    dropMsSQLDatabase,
    dropMsSQLNode,
    restoreMsSQLDatabase,
    startMsSQLNode,
    stopMsSQLNode,
    addMsSQLStandby,
    backupMsSQLNode,
    restoreMsSQLNode,
    removeMsSQLSchedule,
    checkMsSQLUserExists,
    getMsSQLClusterStatus,
    checkMsSQLDatabaseExists,
    checkMsSQLVersionExists,
    switchMsSQLNode,
    exportMsSQLNode,
    importMsSQLNode,
    copyMsSQLDatabase,
}
export default sagasMsSQL;