import { call, put, select } from "redux-saga/effects";
import { LayoutActions } from "../reducer";
import { StatusActions } from "../../status/reducer";
import { TaskNotificationActions } from "../../taskNotifications/reducer";
import apiOracle from "../../../services/oracle/api";
import notify from "../../../util/notify";
function* checkOracleVersionExists({ data }) {
    try {
        yield put(LayoutActions.setOracleVersionExists(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const oracle_version_exists = yield call(apiOracle.checkOracleVersionExists, accessToken, data);
        if (oracle_version_exists?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(oracle_version_exists.detail));
            yield put(LayoutActions.setOracleVersionExists(oracle_version_exists));
        }
        else {
            yield put(LayoutActions.setOracleVersionExists(oracle_version_exists));
          
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* addCdbNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.addCdbNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback?.success, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Registration of CDB ${data.serviceName} started`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            if(response?.status === 206){
                yield call(callback?.warning, {"text": response.data?.detail})
            }
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* installCdbNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.installCdbNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback?.success, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Installation of CDB ${data.name} started`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setOracleVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
            if(response?.status === 206){
                yield call(callback?.warning, {"text": response.data?.detail})
            }
        }
    }
    catch (e) {
 
        yield put(LayoutActions.setLoading(false));
    }
}

function* addPdbNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.addPdbNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Creating PDB ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* removeOracleSchedule({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.removeOracleSchedule, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback);
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(""));
            yield call(notify.info, response.notification);
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* deletePDBNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.deletePDBNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(notify.info, response.notification);
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* deleteCDBNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.deleteCDBNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* getDBFileDest({ id }) {
    try {
        const { accessToken } = yield select(state => state.app);
        const res = yield call(apiOracle.getDBFileDest, accessToken, id);
        if (res?.status >= 400) {
            yield put(LayoutActions.setDBFileDest(undefined));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(res.detail ? res.detail : res));
        }
        else {
            yield put(LayoutActions.setDBFileDest(res));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(""));
        }
    }
    catch (e) {
    }
}
function* importStandalone({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        let response = yield call(apiOracle.importStandalone, accessToken, data);
        yield put(LayoutActions.setErrorMessage(""));
        yield call(callback);
        yield put(LayoutActions.setLoading(false));
        yield call(notify.info, response.notification);
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* getOracleHome({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const res = yield call(apiOracle.getOracleHome, accessToken, data);
        if (res?.message || res.detail) {
            yield put(LayoutActions.setErrorMessage(res.message ? res.message : res.detail));
            yield put(LayoutActions.setOracleHome(null));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setOracleHome(res));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
        yield put(LayoutActions.setOracleHome(null));
    }
}
function* fetchSchemas({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setSchemas(null));
        const { accessToken } = yield select(state => state.app);
        yield put(LayoutActions.setErrorMessage(""));
        const schemas = yield call(apiOracle.fetchSchemas, accessToken, data);
        if (schemas?.detail || schemas?.message) {
            yield put(LayoutActions.setSchemas(undefined));
            yield put(LayoutActions.setErrorMessage(schemas.detail ? schemas.detail : schemas));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setSchemas(schemas));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setSchemas(undefined));
        yield put(LayoutActions.setLoading(false));
    }
}
function* fetchDisks({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setDisks(null));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const disks = yield call(apiOracle.fetchDisks, accessToken, data);
        if (disks?.detail || disks.message) {
            yield put(LayoutActions.setDisks(undefined));
            yield put(LayoutActions.setErrorMessage(disks.detail ? disks.detail : disks));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setDisks(disks));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setDisks(undefined));
        yield put(LayoutActions.setLoading(false));
    }
}
function* openPDBNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.openPDBNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* closePDBNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.closePDBNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* dropFlashback({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.dropFlashback, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* dropStandalone({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const res = yield call(apiOracle.dropStandalone, accessToken, data);
        if (res?.detail) {
            yield put(LayoutActions.setErrorMessage(res.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(LayoutActions.setLoading(false))
            yield put(LayoutActions.setErrorMessage(""));;
            yield put(LayoutActions.setNodes(null));
           
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkPDBName({ data }) {
    try {
        yield put(LayoutActions.setValidName(null));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiOracle.checkPDBName, accessToken, data);
        if (valid_name?.detail || valid_name?.message) {
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setErrorMessage(valid_name.detail ? valid_name.detail : valid_name));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setValidName(valid_name));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkPathName({ data }) {
    try {
        yield put(LayoutActions.setValidPath(null));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_path = yield call(apiOracle.checkPathName, accessToken, data);
        if (valid_path?.detail || valid_path?.message) {
            yield put(LayoutActions.setSchemas(undefined));
            yield put(LayoutActions.setErrorMessage(valid_path.detail ? valid_path.detail : valid_path));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setValidPath(valid_path));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkDumpFile({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const dump_exists = yield call(apiOracle.checkDumpFile, accessToken, data);
        yield put(LayoutActions.setDumpExists(dump_exists));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(false));
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkUserExists({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const user_exists = yield call(apiOracle.checkUserExists, accessToken, data);
        if (user_exists?.detail) {
            yield put(LayoutActions.setUserExists(null));
            yield put(LayoutActions.setErrorMessage(user_exists.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setUserExists(user_exists));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkDirExists({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const dir_exists = yield call(apiOracle.checkDirExists, accessToken, data);
        yield put(LayoutActions.setDirExists(dir_exists));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(false));
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkExportExists({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const export_exists = yield call(apiOracle.checkExportExists, accessToken, data);
        yield put(LayoutActions.setExportExists(export_exists));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(false));
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkDiskSpace({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const disk_space = yield call(apiOracle.checkDiskSpace, accessToken, data);
        yield put(LayoutActions.setDiskSpace(disk_space));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(false));
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* renamePDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        yield put(LayoutActions.setErrorMessage(""));
        let response = yield call(apiOracle.renamePDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);  
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* copyPDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        yield put(LayoutActions.setErrorMessage(""));
        const response = yield call(apiOracle.copyPDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* copyToCDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        yield put(LayoutActions.setErrorMessage(""));
        const response = yield call(apiOracle.copyToCDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* restorePDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        yield put(LayoutActions.setErrorMessage(""));
        let response = yield call(apiOracle.restorePDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data))
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* restoreFlashback({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        let response  = yield call(apiOracle.restoreFlashback, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else
        {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* backupPDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.backupPDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* backupCDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.backupCDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* flashbackCDB_PDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.flashbackCDB_PDB, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
           
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* exportCDB_PDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.exportCDB_PDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* importPDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.importPDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* importCDB({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.importCDB, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkPDBCredentials({ data }) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let valid_credentials = yield call(apiOracle.checkPDBCredentials, accessToken, data);
        yield put(StatusActions.setValidCredentials(valid_credentials));
        yield put(StatusActions.setErrorMessage(""));
        yield put(StatusActions.setLoading(false));
    }
    catch (e) {
        yield put(StatusActions.setErrorMessage(e.message));
        yield put(StatusActions.setLoading(false));
    }
}
function* updatePDBCredentials({ data, callback }) {
    try {
        yield put(StatusActions.setLoading(true));
        yield put(StatusActions.setInfoMessage(""));
        const { accessToken } = yield select(state => state.app);
        yield call(apiOracle.updatePDBCredentials, accessToken, data);
        yield put(StatusActions.setErrorMessage(""));
        yield call(callback, false);
        yield put(StatusActions.setLoading(false));
    }
    catch (e) {
        yield put(StatusActions.setValidCredentials(null));
        yield put(StatusActions.setErrorMessage(e.message));
        yield put(StatusActions.setLoading(false));
    }
}
function* duplicateOracleStandalone({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.duplicateOracleStandalone, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(null));
            yield put(LayoutActions.setValidPath(null));
            yield put(LayoutActions.setDBFileDest(null));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* getMaxPDB({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const max_pdbs = yield call(apiOracle.getMaxPDB, accessToken, data);
        if(max_pdbs?.status >=400)
        {
            if (max_pdbs?.data.message || max_pdbs?.data.detail) {
                yield put(LayoutActions.setErrorMessage(max_pdbs.message));
                yield put(LayoutActions.setMaxPDB(null));
                yield put(LayoutActions.setLoading(false));
            }
        }
        else {
            yield put(LayoutActions.setMaxPDB(max_pdbs));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}

const sagasOracle =
{
    addCdbNode,
    addPdbNode,
    deletePDBNode,
    deleteCDBNode,
    getDBFileDest,
    importStandalone,
    getOracleHome,
    fetchSchemas,
    fetchDisks,
    openPDBNode,
    closePDBNode,
    dropFlashback,
    dropStandalone,
    checkPDBName,
    checkPathName,
    checkDumpFile,
    checkUserExists,
    checkDirExists,
    checkExportExists,
    checkDiskSpace,
    renamePDB,
    copyPDB,
    copyToCDB,
    restorePDB,
    restoreFlashback,
    backupPDB,
    backupCDB,
    flashbackCDB_PDB,
    exportCDB_PDB,
    importPDB,
    importCDB,
    duplicateOracleStandalone,
    getMaxPDB,
    checkPDBCredentials,
    updatePDBCredentials,
    installCdbNode,
    removeOracleSchedule,
    checkOracleVersionExists
}
export default sagasOracle;
