import { call, put, select } from "redux-saga/effects";
import apiPostgKubernetes from "../../../services/postgres_kubernetes/api";
import notify from "../../../util/notify";
import { LayoutActions} from "../reducer";
import { StatusActions } from "../../status/reducer";
import { TaskNotificationActions } from "../../taskNotifications/reducer";
function* addPgPod({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiPostgKubernetes.addPgPod, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response?.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}

function* addPgStandbyPod({ data}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiPostgKubernetes.addPgStandbyPod, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response?.notification);
            yield put(LayoutActions.setErrorMessage(""));;
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setPgVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}

const sagasPostgresKubernetes =
{

    addPgPod,
    addPgStandbyPod

}
export default sagasPostgresKubernetes;