import { call, put, select } from "redux-saga/effects";
import { LayoutActions } from "../reducer";
import { StatusActions } from "../../status/reducer";
import { TaskNotificationActions } from "../../taskNotifications/reducer";
import notify from "../../../util/notify";
import apiRedis from "../../../services/redis/api";
function* checkRedisVersionExists({ data }) {
    try {
        yield put(LayoutActions.setRedisVersionExists(null));
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const redis_version_exists = yield call(apiRedis.checkRedisVersionExists, accessToken, data);
        if (redis_version_exists?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(redis_version_exists.detail));
            yield put(LayoutActions.setRedisVersionExists(redis_version_exists));
        }
        else {
            yield put(LayoutActions.setRedisVersionExists(redis_version_exists));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* removeRedisCluster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.removeRedisCluster, accessToken, data);
       
        if (response?.status >= 400)
        {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(response?.data?.detail));
        }
        else {
            yield call(callback, false);
            yield call(notify.success, response?.data?.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkRedisClusterHealth({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.checkRedisClusterHealth, accessToken, data);
        if (response?.status >= 400)
        {
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield call(callback, response.data.detail);
            yield put(LayoutActions.setLoading(false));   
        }
        else
        {
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield call(callback, false);
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addRedisStandalone({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.addRedisStandalone, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Creating redis standalone node ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setRedisVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* addRedisCluster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.addRedisCluster, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(notify.success, "Registered a new redis cluster");
            yield call(callback, false);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));    
    }
}
function* restoreRedisNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.restoreRedisNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
        }
        else {      
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));   
    }
}
function* backupRedisNode({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.backupRedisNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);;
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* dropRedisNode({ data, callback }) {
    try {
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.dropRedisNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* removeRedisSchedule({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.removeRedisSchedule, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback);
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* sentinelRemoveMonitor({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.sentinelRemoveMonitor, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* stopRedisNode({ data}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.stopRedisNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* startRedisNode({ data}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.startRedisNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
        }
        else {
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* addRedisNode({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.addRedisNode, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Adding redis node ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setRedisVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* addRedisSentinel({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
      
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.addRedisSentinel, accessToken, data);
        if (response?.status >= 400) {
            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, `Adding redis sentinel ${data.name}`);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(LayoutActions.setRedisVersionExists(undefined));
            yield put(LayoutActions.setPortExists(undefined));
            yield put(LayoutActions.setPathExists(undefined));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
       
    }
}
function* switchRedisNode({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.switchRedisNode, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback,false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));  
    }
}
function* sentinelFailover({ data, callback}) {
    try {
        yield put(LayoutActions.setLoading(true));
      
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.sentinelFailover, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield call(callback,false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));  
    }
}
function* checkSentinelMasterName({ data }) {
    try {
        yield put(LayoutActions.setValidName(undefined));
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const valid_name = yield call(apiRedis.checkSentinelMasterName, accessToken, data);
        if (valid_name?.detail) {
            yield put(LayoutActions.setErrorMessage(valid_name.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            yield put(LayoutActions.setValidName(valid_name));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* sentinelMonitorMaster({  data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiRedis.sentinelMonitorMaster, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
            yield put(LayoutActions.setLoading(false));
        }
        else { 
            yield call(callback, false);
            yield put(TaskNotificationActions.getNotifications());
            yield put(LayoutActions.setLoading(false));
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setValidName(undefined));
            yield put(LayoutActions.setNodes(null));
            yield put(StatusActions.setNodes(null));
            yield put(StatusActions.getRedisDetails(data?.node_id));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
const sagasRedis =
{
    addRedisNode,
    restoreRedisNode,
    backupRedisNode,
    removeRedisSchedule,
    dropRedisNode,
    startRedisNode,
    stopRedisNode,
    checkRedisVersionExists,
    addRedisStandalone,
    addRedisCluster,
    addRedisSentinel,
    switchRedisNode,
    removeRedisCluster,
    checkRedisClusterHealth,
    checkSentinelMasterName,
    sentinelMonitorMaster,
    sentinelRemoveMonitor,
    sentinelFailover
}
export default sagasRedis;