import { call, put, takeLeading, select, all } from "redux-saga/effects";
import { LayoutActions, LayoutActionTypes } from "./reducer";
import { StatusActions } from "../status/reducer";
import api from "../../services/api";
import apiOracle from "../../services/oracle/api";
import sagasOracle from "./oracle/sagas";
import sagasPostgres from "./postgres/sagas";
import sagasMongo from "./mongo/sagas";
import sagasMariaDB from "./mariadb/sagas";
import sagasElasticSearch from "./elasticsearch/sagas";
import sagasRedis from "./redis/sagas";
import sagasMySQL from "./mysql/sagas";
import sagasMsSQL from "./mssql/sagas";
import notify from "../../util/notify";
import { types, clusterRoles } from "../../config";
import apiRedis from "../../services/redis/api";
import sagasHAProxy from "./balancers/haproxy/sagas";
import apiHAProxy from "../../services/balancers/haproxy/api";
import sagasPostgresKubernetes from "./postgres_kubernetes/sagas";
import sagasMongoKubernetes from "./mongo_kubernetes/sagas";
function* getNodes() {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const [
            nodes,
            databases,
            standalones,
            redis_clusters,   
            haproxies
        ] = yield all([
            call(api.getAvailableNodes, accessToken),
            call(api.getDatabases, accessToken),
            call(apiOracle.getOracleStandalones, accessToken),
            call(apiRedis.getRedisClusters, accessToken),
            call(apiHAProxy.getHAProxies, accessToken),
        ]);
        nodes.forEach(node => {
            const { id} = node;
            node.databases = databases.filter(database => database.node === id).sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
        });
        const nodesCdb = nodes.filter(node => node.type === types.ORACLE_CDB);
        if (nodesCdb.length > 0) {
            const flashbacks = yield call(apiOracle.getNodesFlashbacks,accessToken);
            const oracle_backups = yield call(api.getBackupsByType, accessToken, types.ORACLE_CDB);
            const oracle_exports = yield call(api.getExportsByType, accessToken, types.ORACLE_CDB);
            const nodesPdb = nodes.filter(node => node.type === types.ORACLE_PDB);
            nodesPdb.forEach(nodePdb => {
                if (!nodePdb.flashbacks) nodePdb.flashbacks = flashbacks?.filter((f) => f.node === nodePdb.id);
                if (!nodePdb.backups) nodePdb.backups = oracle_backups?.filter((b) => b.node === nodePdb.id);
                if (!nodePdb.exports) nodePdb.exports = oracle_exports?.filter((e) => e.node === nodePdb.id);
                nodesCdb.forEach(nodeCdb => {
                    if (!nodeCdb.flashbacks) nodeCdb.flashbacks = flashbacks?.filter((f) => f.node === nodeCdb.id);
                    if (!nodeCdb.backups) nodeCdb.backups = oracle_backups?.filter((b) => b.node === nodeCdb.id);
                    if (!nodeCdb.exports) nodeCdb.exports = oracle_exports?.filter((e) => e.node === nodeCdb.id);
                    if (!nodeCdb.nodes) {
                        nodeCdb.nodes = [];
                      
                    }
                    if (nodePdb.cluster === nodeCdb.cluster) {
                        nodeCdb.nodes.push(nodePdb);
                        nodeCdb.nodes.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                        
                    }
                });
            });
        }

       
        const pg_primary = nodes.filter(node => node.type === types.POSTGRES && node.cluster_role === clusterRoles.PRIMARY);
        if (pg_primary.length > 0) {
            const pg_backups = yield call(api.getBackupsByType, accessToken, types.POSTGRES);
            yield put(LayoutActions.setPgBackups(pg_backups));
            const pg_exports = yield call(api.getExportsByType, accessToken, types.POSTGRES);
            yield put(LayoutActions.setPgExports(pg_exports));

        }
        else {
            yield put(LayoutActions.setPgBackups([]));
            yield put(LayoutActions.setPgExports([]));
        }
        const pg_standby = nodes.filter(node => node.type === types.POSTGRES && node.cluster_role === clusterRoles.STANDBY);
        pg_standby.forEach(standby => {
            pg_primary.forEach(primary => {
                if (!primary.nodes) {
                    primary.nodes = [];
                }
                if (standby.cluster === primary.cluster) {
                    primary.nodes.push(standby);
                    primary.nodes.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                }
            });
        });
        const mongo_primary = nodes.filter(node => node.type === types.MONGO && node.cluster_role === clusterRoles.PRIMARY);
        if (mongo_primary.length > 0) {
            const mongo_backups = yield call(api.getBackupsByType, accessToken, types.MONGO);
            yield put(LayoutActions.setMongoBackups(mongo_backups));
            const mongo_exports = yield call(api.getExportsByType, accessToken, types.MONGO);
            yield put(LayoutActions.setMongoExports(mongo_exports));

        }
        else {
            yield put(LayoutActions.setMongoBackups([]));
            yield put(LayoutActions.setMongoExports([]));
        }
        const mongo_secondary = nodes.filter(node => node.type === types.MONGO && (node.cluster_role === clusterRoles.STANDBY || node.cluster_role === clusterRoles.ARBITER));
        mongo_secondary.forEach(secondary => {
            mongo_primary.forEach(primary => {
                if (!primary.nodes) {
                    primary.nodes = [];
                }
                if (secondary.cluster === primary.cluster) {
                    primary.nodes.push(secondary);
                    primary.nodes.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                }
            });
        });
        const elasticsearch_master = nodes.filter(node => node.type === types.ELASTIC && node.cluster_role === clusterRoles.PRIMARY);
        if (elasticsearch_master.length > 0) {
            const elastic_backups = yield call(api.getBackupsByType, accessToken, types.ELASTIC);
            yield put(LayoutActions.setElasticBackups(elastic_backups));
            const elastic_exports = yield call(api.getExportsByType, accessToken, types.ELASTIC);
            yield put(LayoutActions.setElasticExports(elastic_exports));
        }
        else {
            yield put(LayoutActions.setElasticBackups([]));
            yield put(LayoutActions.setElasticExports([]));
        }
        const elasticsearch_secondary = nodes.filter(node => node.type === types.ELASTIC && node.cluster_role === clusterRoles.STANDBY);
        elasticsearch_secondary.forEach(secondary => {
            elasticsearch_master.forEach(primary => {
                if (!primary.nodes) {
                    primary.nodes = [];
                }
                if (secondary.cluster === primary.cluster) {
                    primary.nodes.push(secondary);
                    primary.nodes.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                }
            });
        });
        const mariadb_primary = nodes.filter(node => node.type === types.MARIADB && node.cluster_role === clusterRoles.PRIMARY);
        if (mariadb_primary.length > 0) {
            const mariadb_backups = yield call(api.getBackupsByType, accessToken, types.MARIADB);
            yield put(LayoutActions.setMariaDbBackups(mariadb_backups));
            const mariadb_exports = yield call(api.getExportsByType, accessToken, types.MARIADB);
            yield put(LayoutActions.setMariaDbExports(mariadb_exports));

        }
        else {
            yield put(LayoutActions.setMariaDbBackups([]));
            yield put(LayoutActions.setMariaDbExports([]));
        }
        const mariadb_standby = nodes.filter(node => node.type === types.MARIADB && node.cluster_role === clusterRoles.STANDBY);
        mariadb_standby.forEach(standby => {
            mariadb_primary.forEach(primary => {
                if (!primary.nodes) {
                    primary.nodes = [];
                }
                if (standby.cluster === primary.cluster) {
                    primary.nodes.push(standby);
                    primary.nodes.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                }
            });
        });
        const mysql_primary = nodes.filter(node => node.type === types.MYSQL && node.cluster_role === clusterRoles.PRIMARY);
        if (mysql_primary.length > 0) {
            const mysql_backups = yield call(api.getBackupsByType, accessToken, types.MYSQL);
            yield put(LayoutActions.setMySqlBackups(mysql_backups));
            const mysql_exports = yield call(api.getExportsByType, accessToken, types.MYSQL);
            yield put(LayoutActions.setMySqlExports(mysql_exports));

        }
        else {
            yield put(LayoutActions.setMySqlBackups([]));
            yield put(LayoutActions.setMySqlExports([]));
        }
        const mysql_standby = nodes.filter(node => node.type === types.MYSQL && node.cluster_role === clusterRoles.STANDBY);
        mysql_standby.forEach(standby => {
            mysql_primary.forEach(primary => {
                if (!primary.nodes) {
                    primary.nodes = [];
                }
                if (standby.cluster === primary.cluster) {
                    primary.nodes.push(standby);
                    primary.nodes.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                }
            });
        });

        const redis_attached = nodes.filter(node => node.type === types.REDIS && !node.ex_params?.standalone);
        if (redis_attached.length > 0) {
            const redis_backups = yield call(api.getBackupsByType, accessToken, types.REDIS);
            yield put(LayoutActions.setRedisBackups(redis_backups));
        }
        else yield put(LayoutActions.setRedisBackups([]));
        redis_attached.forEach(node => {
            redis_clusters.forEach(cluster => {
                if (node.cluster === cluster.id) {
                    if (node.cluster_role === clusterRoles.PRIMARY) {
                        if (!cluster.masters) {
                            cluster.masters = [];
                        }
                        cluster.masters.push(node)
                        cluster.masters.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                    }
                    else if (node.cluster_role === clusterRoles.SENTINEL) {
                        if (!cluster.sentinels) {
                            cluster.sentinels = [];
                        }
                        cluster.sentinels.push(node);
                        cluster.sentinels.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                    }
                    else {
                        if (!cluster.slaves) {
                            cluster.slaves = [];
                        }
                        cluster.slaves.push(node);
                        cluster.slaves.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                    }

                }
            });
        });
        const mssql_primary = nodes.filter(node => node.type === types.MSSQL && node.cluster_role === clusterRoles.PRIMARY);
        if (mssql_primary.length > 0) {
            const mssql_backups = yield call(api.getBackupsByType, accessToken, types.MSSQL);
            yield put(LayoutActions.setMsSqlBackups(mssql_backups));
            const mssql_exports = yield call(api.getExportsByType, accessToken, types.MSSQL);
            yield put(LayoutActions.setMsSqlExports(mssql_exports));

        }
        else {
            yield put(LayoutActions.setMsSqlBackups([]));
            yield put(LayoutActions.setMsSqlExports([]));
        }
        const pg_k8s_primary = nodes.filter(node => node.type === types.POSTGRES_K8S && node.cluster_role === clusterRoles.PRIMARY);
        const pg_k8s_standby = nodes.filter(node => node.type === types.POSTGRES_K8S && node.cluster_role === clusterRoles.STANDBY);
        pg_k8s_standby.forEach(standby => {
            pg_k8s_primary.forEach(primary => {
                if (!primary.nodes) {
                    primary.nodes = [];
                }
                if (standby.cluster === primary.cluster) {
                    primary.nodes.push(standby);
                    primary.nodes.sort((a, b) => (a.name > b.name) ? -1 : ((b.name > a.name) ? 1 : 0));
                }
            });
        });
        
        const mongo_k8s = nodes.filter(node => node.type === types.MONGO_K8S);
        if (pg_k8s_primary.length > 0) {
            const pg_k8s_backups = yield call(api.getBackupsByType, accessToken, types.POSTGRES_K8S);
            yield put(LayoutActions.setPgK8sBackups(pg_k8s_backups));
            const pg_k8s_exports = yield call(api.getExportsByType, accessToken, types.POSTGRES_K8S);
            yield put(LayoutActions.setPgK8sExports(pg_k8s_exports));

        }
        else {
            yield put(LayoutActions.setPgK8sBackups([]));
            yield put(LayoutActions.setPgK8sExports([]));
        }
        if (mongo_k8s.length > 0) {
            const mongo_k8s_exports = yield call(api.getExportsByType, accessToken, types.MONGO_K8S);
            yield put(LayoutActions.setMongoK8sExports(mongo_k8s_exports));

        }
        else {
            yield put(LayoutActions.setMongoK8sExports([]));
        }
        const sortedNodes = [
            ...redis_clusters,
            ...nodesCdb,
            ...pg_primary,
            ...mongo_primary,
            ...mariadb_primary,
            ...elasticsearch_master,
            ...mysql_primary,
            ...mssql_primary,
            ...pg_k8s_primary,
            ...mongo_k8s
        ];
        yield all([
            put(LayoutActions.setNodes(sortedNodes)),
            put(LayoutActions.setOracleStandalones(standalones)),
            put(LayoutActions.setHaProxies(haproxies)),
            put(LayoutActions.setDatabases(databases.filter(d => !d?.ind_deleted))),
            put(LayoutActions.setErrorMessage("")),
            put(LayoutActions.setLoading(false))
        ]);
    } catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* refreshContainer({ id }) {
    try {
        const { accessToken } = yield select(state => state.app);
        yield call(api.getNodeDatabases, accessToken, id);
        yield put(LayoutActions.getNodes())
    }
    catch (e) {
     
        yield put(LayoutActions.setLoading(false));

    }
}
function* refreshDbList({ id }) {
    try {
        const { accessToken } = yield select(state => state.app);
        
        const response = yield call(api.getDatabases, accessToken, id);
        yield put(LayoutActions.setDatabases(response))
    }
    catch (e) {

        yield put(LayoutActions.setLoading(false));
    }
}
function* dropBackup({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.dropBackup, accessToken, data);
        if (response?.status >= 400) {

            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
        }
    }

    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* dropCluster({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.dropCluster, accessToken, data);
        if (response?.status >= 400) {

            yield put(LayoutActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(LayoutActions.setErrorMessage(response?.data?.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(LayoutActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield call(notify.info, response.notification);
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setNodes(null));
        }
    }

    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkConnection({ data, callback }) {
    try {
        const { accessToken } = yield select(state => state.app);
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setErrorMessage(""));
        const response = yield call(api.checkConnection, accessToken, data);
        yield put(LayoutActions.setLoading(false));
        if (response?.detail) {
            yield put(LayoutActions.setErrorMessage(response.detail));
        }
        else {
            if (response?.connected) yield call(callback, 1);
        }
    }
    catch (e) {

        yield put(LayoutActions.setLoading(false));
    }
}
function* dropExport({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(api.dropBackup, accessToken, data);
        yield call(callback, false);
        yield call(notify.info, response.notification);
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setLoading(false));
        yield put(LayoutActions.setNodes(null));
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkPortExists({ data }) {
    try {
        yield put(LayoutActions.setPortExists(null));
        yield put(LayoutActions.setLoading(data.purpose ? data.purpose : true));
        const { accessToken } = yield select(state => state.app);
        const port_exists = yield call(api.checkPortExists, accessToken, data);
        if (data.purpose) port_exists.purpose = data.purpose
        if (port_exists?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(port_exists.detail));
            yield put(LayoutActions.setPortExists(port_exists));
        }
        else {
            yield put(LayoutActions.setPortExists(port_exists));
            yield put(LayoutActions.setInfoMessage(""));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkServiceConnection({ data }) {
    try {
        yield put(LayoutActions.setPortExists(null));
        yield put(LayoutActions.setLoading(data.purpose ? data.purpose : true));
        const { accessToken } = yield select(state => state.app);
        const port_exists = yield call(api.checkServiceConnection, accessToken, data);
        if (data.purpose) port_exists.purpose = data.purpose
        if (port_exists?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(port_exists.detail));
            yield put(LayoutActions.setPortExists(port_exists));
        }
        else {
            if (port_exists?.status?.toLowerCase() === "available")  yield put(LayoutActions.setPortExists({"exists": true}));
            else if (port_exists?.status?.toLowerCase() === "unavailable") yield put(LayoutActions.setPortExists({"exists": false}));
            else yield put(LayoutActions.setPortExists(port_exists));
            yield put(LayoutActions.setInfoMessage(""));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkFreeIp({ data }) {
    try {
        yield put(LayoutActions.setFreeIp(null));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const free_ip = yield call(api.checkFreeIp, accessToken, data);
        if (free_ip?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(free_ip.detail));
            yield put(LayoutActions.setFreeIp(free_ip));
        }
        else {
            yield put(LayoutActions.setFreeIp(free_ip));
            yield put(LayoutActions.setInfoMessage(""));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkFreePodIp({ data, callback }) {
    try {
        yield put(LayoutActions.setFreePodIp(null));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const free_ip = yield call(api.checkFreePodIp, accessToken, data);
        if (free_ip?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(free_ip.detail));
            yield put(LayoutActions.setFreePodIp(free_ip));
        }
        else {
            yield put(LayoutActions.setFreePodIp(free_ip?.node_ip));
            yield put(LayoutActions.setInfoMessage(""));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
            yield call(callback, 1);
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* getSubnets({ data}) {
    try {
        yield put(LayoutActions.setFreePodIp(null));
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const subnets = yield call(api.getSubnets, accessToken, data);
        if (subnets?.detail) {
            yield put(LayoutActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(subnets.detail));
            yield put(LayoutActions.setSubnets(null));
        }
        else {
            yield put(LayoutActions.setSubnets(subnets?.subnets));
            yield put(LayoutActions.setInfoMessage(""));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkPathExists({ data }) {
    try {
        if (data.type) yield put(LayoutActions.setPathExists(data?.type));
        else yield put(LayoutActions.setPathExists(null));
        yield put(LayoutActions.setLoading(data.type ? data.type : true));
        const { accessToken } = yield select(state => state.app);
        const path_exists = yield call(api.checkPathExists, accessToken, data);
        if (path_exists?.detail) {
            path_exists.type = data?.type;
            yield put(LayoutActions.setPathExists(path_exists));
            yield put(LayoutActions.setErrorMessage(path_exists.detail));
            yield put(LayoutActions.setLoading(false));
        }
        else {
            path_exists.type = data.type;
            yield put(LayoutActions.setPathExists(path_exists));
            yield put(LayoutActions.setInfoMessage(""));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* changeClusterSize({ data}) {
    try {
        yield put(LayoutActions.setLoading(true));
        yield put(LayoutActions.setInfoMessage(""));
        const { accessToken } = yield select(state => state.app);
        let res = yield call(api.changeClusterSize, accessToken, data);
        yield put(LayoutActions.setLoading(false));
        yield call(notify.info, res.notification);
        yield put(LayoutActions.setErrorMessage(""));
        yield put(LayoutActions.setNodes(null));
        yield put(StatusActions.setNodes(null));
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* isQuotaExceed({ data }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let response = yield call(api.isQuotaExceed, accessToken, data.request);
        if (response.exceed) yield call(data.reject, data.warning);
        else yield call(data.allow, true);
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}

function* isNodeHostReachable({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let response = yield call(api.isNodeHostReachable, accessToken, data);
        if (response.reachable) yield call(callback.allow);
        else yield call(callback.confirm, true);
        yield put(LayoutActions.setLoading(false));
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}

function* markNodeDeleted({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        yield call(api.markNodeDeleted, accessToken, data);
        yield put(LayoutActions.setLoading(false));
        yield put(LayoutActions.setNodes(null));
        yield put(StatusActions.setNodes(null));
        yield call(callback, false);
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}

function* verifyBackup({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let response = yield call(api.verifyBackup, accessToken, data);
        if (response?.status >= 400) {
            yield call(callback, true, response?.data?.detail ? response.data.detail : response.data)
            yield put(LayoutActions.setLoading(false));
        }
        else {
            if (response.ok) yield call(callback, true, `The ${data.type.toLowerCase()} ${data.name} is ok`);
            else yield call(callback, true, `${data.type.toLowerCase()} ${data.name} is not ok`);
            yield put(LayoutActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(LayoutActions.setLoading(false));
    }
}
function* checkInventoryExists({ data, callback }) {
    try {
        yield put(LayoutActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let response = yield call(api.checkInventoryExists, accessToken, data);
        yield put(LayoutActions.setLoading(false));
        if (response?.status >= 400) {
            yield call(callback.failback, false);
        }
        else {
            if (response.exists) yield call(callback.exists, true);
            else yield call(callback.failback, false);
        }
    }
    catch (e) {

        yield put(LayoutActions.setLoading(false));
       
    }
}
const sagas = [
    takeLeading(LayoutActionTypes.GET_NODES, getNodes),
    takeLeading(LayoutActionTypes.ADD_PDB_NODE, sagasOracle.addPdbNode),
    takeLeading(LayoutActionTypes.ADD_CDB_NODE, sagasOracle.addCdbNode),
    takeLeading(LayoutActionTypes.INSTALL_CDB_NODE, sagasOracle.installCdbNode),
    takeLeading(LayoutActionTypes.DELETE_PDB_NODE, sagasOracle.deletePDBNode),
    takeLeading(LayoutActionTypes.DELETE_CDB_NODE, sagasOracle.deleteCDBNode),
    takeLeading(LayoutActionTypes.GET_DB_FILE_DEST, sagasOracle.getDBFileDest),
    takeLeading(LayoutActionTypes.GET_ORACLE_HOME, sagasOracle.getOracleHome),
    takeLeading(LayoutActionTypes.OPEN_PDB_NODE, sagasOracle.openPDBNode),
    takeLeading(LayoutActionTypes.CLOSE_PDB_NODE, sagasOracle.closePDBNode),
    takeLeading(LayoutActionTypes.CHECK_PDB_NAME, sagasOracle.checkPDBName),
    takeLeading(LayoutActionTypes.RENAME_PDB, sagasOracle.renamePDB),
    takeLeading(LayoutActionTypes.COPY_PDB, sagasOracle.copyPDB),
    takeLeading(LayoutActionTypes.COPY_TO_CDB, sagasOracle.copyToCDB),
    takeLeading(LayoutActionTypes.RESTORE_PDB, sagasOracle.restorePDB),
    takeLeading(LayoutActionTypes.BACKUP_PDB, sagasOracle.backupPDB),
    takeLeading(LayoutActionTypes.BACKUP_CDB, sagasOracle.backupCDB),
    takeLeading(LayoutActionTypes.FLASHBACK_NODE, sagasOracle.flashbackCDB_PDB),
    takeLeading(LayoutActionTypes.EXPORT_NODE, sagasOracle.exportCDB_PDB),
    takeLeading(LayoutActionTypes.IMPORT_PDB, sagasOracle.importPDB),
    takeLeading(LayoutActionTypes.RESTORE_FLASHBACK, sagasOracle.restoreFlashback),
    takeLeading(LayoutActionTypes.CHANGE_CLUSTER_SIZE, changeClusterSize),
    takeLeading(LayoutActionTypes.FETCH_SCHEMAS, sagasOracle.fetchSchemas),
    takeLeading(LayoutActionTypes.FETCH_DISKS, sagasOracle.fetchDisks),
    takeLeading(LayoutActionTypes.DUPLICATE_ORACLE_STANDALONE, sagasOracle.duplicateOracleStandalone),
    takeLeading(LayoutActionTypes.CHECK_PATH_NAME, sagasOracle.checkPathName),
    takeLeading(LayoutActionTypes.CHECK_USER_EXISTS, sagasOracle.checkUserExists),
    takeLeading(LayoutActionTypes.CHECK_DUMP_EXISTS, sagasOracle.checkDumpFile),
    takeLeading(LayoutActionTypes.DROP_FLASHBACK, sagasOracle.dropFlashback),
    takeLeading(LayoutActionTypes.CHECK_DIR_EXISTS, sagasOracle.checkDirExists),
    takeLeading(LayoutActionTypes.CHECK_EXPORT_EXISTS, sagasOracle.checkExportExists),
    takeLeading(LayoutActionTypes.CHECK_DISK_SPACE, sagasOracle.checkDiskSpace),
    takeLeading(LayoutActionTypes.GET_MAX_PDB, sagasOracle.getMaxPDB),
    takeLeading(LayoutActionTypes.IMPORT_STANDALONE, sagasOracle.importStandalone),
    takeLeading(LayoutActionTypes.DROP_BACKUP, dropBackup),
    takeLeading(LayoutActionTypes.DROP_EXPORT, dropExport),
    takeLeading(LayoutActionTypes.DROP_STANDALONE, sagasOracle.dropStandalone),
    takeLeading(LayoutActionTypes.CHECK_PORT_EXISTS, checkPortExists),
    takeLeading(LayoutActionTypes.CHECK_PATH_EXISTS, checkPathExists),
    takeLeading(LayoutActionTypes.CHECK_INVENTORY_EXISTS, checkInventoryExists),
    takeLeading(LayoutActionTypes.CHECK_PG_VERSION_EXISTS, sagasPostgres.checkPgVersionExists),
    takeLeading(LayoutActionTypes.CHECK_HA_PROXY_EXISTS, sagasHAProxy.checkHAProxyExists),
    takeLeading(LayoutActionTypes.CHECK_PG_USER_EXISTS, sagasPostgres.checkPgUserExists),
    takeLeading(LayoutActionTypes.CHECK_MARIA_DB_USER_EXISTS, sagasMariaDB.checkMariaDBUserExists),
    takeLeading(LayoutActionTypes.CHECK_MY_SQL_USER_EXISTS, sagasMySQL.checkMySQLUserExists),
    takeLeading(LayoutActionTypes.ADD_PG_PRIMARY, sagasPostgres.addPgPrimary),
    takeLeading(LayoutActionTypes.ADD_ELASTIC_SEARCH_MASTER, sagasElasticSearch.addElasticSearchMaster),
    takeLeading(LayoutActionTypes.ADD_PG_STANDBY, sagasPostgres.addPgStandby),
    takeLeading(LayoutActionTypes.CHECK_PG_DATABASE_NAME, sagasPostgres.checkPgDatabaseName),
    takeLeading(LayoutActionTypes.CHECK_MONGO_DATABASE_NAME, sagasMongo.checkMongoDatabaseName),
    takeLeading(LayoutActionTypes.CHECK_PG_DATABASE_SIZE, sagasPostgres.checkPgDatabaseSize),
    takeLeading(LayoutActionTypes.ADD_PG_DATABASE, sagasPostgres.addPgDatabase),
    takeLeading(LayoutActionTypes.ADD_MONGO_DATABASE, sagasMongo.addMongoDatabase),
    takeLeading(LayoutActionTypes.BACKUP_PG_NODE, sagasPostgres.backupPgNode),
    takeLeading(LayoutActionTypes.BACKUP_MARIA_DB_NODE, sagasMariaDB.backupMariaDBNode),
    takeLeading(LayoutActionTypes.BACKUP_MY_SQL_NODE, sagasMySQL.backupMySQLNode),
    takeLeading(LayoutActionTypes.BACKUP_MS_SQL_NODE, sagasMsSQL.backupMsSQLNode),
    takeLeading(LayoutActionTypes.EXPORT_MARIA_DB_NODE, sagasMariaDB.exportMariaDBNode),
    takeLeading(LayoutActionTypes.IMPORT_MARIA_DB_NODE, sagasMariaDB.importMariaDBNode),
    takeLeading(LayoutActionTypes.EXPORT_PG_DATABASE, sagasPostgres.exportPgDatabase),
    takeLeading(LayoutActionTypes.DUMP_MONGO_DATABASE, sagasMongo.dumpMongoDatabase),
    takeLeading(LayoutActionTypes.RESTORE_PG_DATABASE, sagasPostgres.restorePgDatabase),
    takeLeading(LayoutActionTypes.RESTORE_MONGO_DATABASE, sagasMongo.restoreMongoDatabase),
    takeLeading(LayoutActionTypes.RESTORE_PG_NODE, sagasPostgres.restorePgNode),
    takeLeading(LayoutActionTypes.RESTORE_MARIA_DB_NODE, sagasMariaDB.restoreMariaDBNode),
    takeLeading(LayoutActionTypes.RESTORE_MY_SQL_NODE, sagasMySQL.restoreMySQLNode),
    takeLeading(LayoutActionTypes.RESTORE_MS_SQL_NODE, sagasMsSQL.restoreMsSQLNode),
    takeLeading(LayoutActionTypes.EXPORT_PG_NODE, sagasPostgres.exportPgNode),
    takeLeading(LayoutActionTypes.STOP_PG_NODE, sagasPostgres.stopPgNode),
    takeLeading(LayoutActionTypes.START_PG_NODE, sagasPostgres.startPgNode),
    takeLeading(LayoutActionTypes.DROP_PG_NODE, sagasPostgres.dropPgNode),
    takeLeading(LayoutActionTypes.CHECK_PG_DATABASE_EXISTS, sagasPostgres.checkPgDatabaseExists),
    takeLeading(LayoutActionTypes.COPY_PG_DATABASE, sagasPostgres.copyPgDatabase),
    takeLeading(LayoutActionTypes.DROP_PG_DATABASE, sagasPostgres.dropPgDatabase),
    takeLeading(LayoutActionTypes.GET_PG_PRIMARY_STATUS, sagasPostgres.getPgPrimaryStatus),
    takeLeading(LayoutActionTypes.CHECK_CONNECTION, checkConnection),
    takeLeading(LayoutActionTypes.SWITCH_PG_NODE, sagasPostgres.switchPgNode),
    takeLeading(LayoutActionTypes.CLONE_PG_STANDBY, sagasPostgres.clonePgStandby),
    takeLeading(LayoutActionTypes.TOGGLE_PG_CLUSTER, sagasPostgres.togglePgCluster),
    takeLeading(LayoutActionTypes.CHECK_PDB_CREDENTIALS, sagasOracle.checkPDBCredentials),
    takeLeading(LayoutActionTypes.UPDATE_PDB_CREDENTIALS, sagasOracle.updatePDBCredentials),
    takeLeading(LayoutActionTypes.REMOVE_PG_SCHEDULE, sagasPostgres.removePgSchedule),
    takeLeading(LayoutActionTypes.REMOVE_ORACLE_SCHEDULE, sagasOracle.removeOracleSchedule),
    takeLeading(LayoutActionTypes.CHECK_IF_PG_PRIMARY, sagasPostgres.checkIfPgPrimary),
    takeLeading(LayoutActionTypes.IMPORT_PG_NODE, sagasPostgres.importPgNode),
    takeLeading(LayoutActionTypes.IMPORT_MY_SQL_NODE, sagasMySQL.importMySQLNode),
    takeLeading(LayoutActionTypes.IMPORT_MS_SQL_NODE, sagasMsSQL.importMsSQLNode),
    takeLeading(LayoutActionTypes.IMPORT_CDB, sagasOracle.importCDB),
    takeLeading(LayoutActionTypes.ADD_MONGO_MASTER, sagasMongo.addMongoMaster),
    takeLeading(LayoutActionTypes.ADD_REDIS_STANDALONE, sagasRedis.addRedisStandalone),
    takeLeading(LayoutActionTypes.ADD_REDIS_CLUSTER, sagasRedis.addRedisCluster),
    takeLeading(LayoutActionTypes.ADD_MARIA_DB_MASTER, sagasMariaDB.addMariaDBMaster),
    takeLeading(LayoutActionTypes.ADD_MY_SQL_MASTER, sagasMySQL.addMySQLMaster),
    takeLeading(LayoutActionTypes.ADD_MS_SQL_MASTER, sagasMsSQL.addMsSQLMaster),
    takeLeading(LayoutActionTypes.IS_QUOTA_EXCEED, isQuotaExceed),
    takeLeading(LayoutActionTypes.CHECK_MONGO_VERSION_EXISTS, sagasMongo.checkMongoVersionExists),
    takeLeading(LayoutActionTypes.CHECK_REDIS_VERSION_EXISTS, sagasRedis.checkRedisVersionExists),
    takeLeading(LayoutActionTypes.CHECK_ORACLE_VERSION_EXISTS, sagasOracle.checkOracleVersionExists),
    takeLeading(LayoutActionTypes.ADD_MONGO_NODE, sagasMongo.addMongoNode),
    takeLeading(LayoutActionTypes.CHECK_MONGO_DATABASE_SIZE, sagasMongo.checkMongoDatabaseSize),
    takeLeading(LayoutActionTypes.DUMP_MONGO_NODE, sagasMongo.dumpMongoNode),
    takeLeading(LayoutActionTypes.BACKUP_MONGO_NODE, sagasMongo.backupMongoNode),
    takeLeading(LayoutActionTypes.BACKUP_REDIS_NODE, sagasRedis.backupRedisNode),
    takeLeading(LayoutActionTypes.RESTORE_REDIS_NODE, sagasRedis.restoreRedisNode),
    takeLeading(LayoutActionTypes.RESTORE_BACKUP_MONGO_NODE, sagasMongo.restoreBackupMongoNode),
    takeLeading(LayoutActionTypes.RESTORE_DUMP_MONGO_NODE, sagasMongo.restoreDumpMongoNode),
    takeLeading(LayoutActionTypes.DROP_MONGO_NODE, sagasMongo.dropMongoNode),
    takeLeading(LayoutActionTypes.DROP_REDIS_NODE, sagasRedis.dropRedisNode),
    takeLeading(LayoutActionTypes.GET_MONGO_CLUSTER_STATUS, sagasMongo.getMongoClusterStatus),
    takeLeading(LayoutActionTypes.GET_MS_SQL_CLUSTER_STATUS, sagasMsSQL.getMsSQLClusterStatus),
    takeLeading(LayoutActionTypes.GET_ELASTICSEARCH_CLUSTER_STATUS, sagasElasticSearch.getElasticSearchClusterStatus),
    takeLeading(LayoutActionTypes.GET_MARIA_DB_CLUSTER_STATUS, sagasMariaDB.getMariaDBClusterStatus),
    takeLeading(LayoutActionTypes.GET_MY_SQL_CLUSTER_STATUS, sagasMySQL.getMySQLClusterStatus),
    takeLeading(LayoutActionTypes.GET_MS_SQL_CLUSTER_STATUS, sagasMsSQL.getMsSQLClusterStatus),
    takeLeading(LayoutActionTypes.DROP_MONGO_DATABASE, sagasMongo.dropMongoDatabase),
    takeLeading(LayoutActionTypes.STEP_DOWN_MONGO_NODE, sagasMongo.stepDownMongoNode),
    takeLeading(LayoutActionTypes.START_MONGO_NODE, sagasMongo.startMongoNode),
    takeLeading(LayoutActionTypes.STOP_MONGO_NODE, sagasMongo.stopMongoNode),
    takeLeading(LayoutActionTypes.START_MARIA_DB_NODE, sagasMariaDB.startMariaDBNode),
    takeLeading(LayoutActionTypes.START_MY_SQL_NODE, sagasMySQL.startMySQLNode),
    takeLeading(LayoutActionTypes.STOP_MY_SQL_NODE, sagasMySQL.stopMySQLNode),
    takeLeading(LayoutActionTypes.START_MS_SQL_NODE, sagasMsSQL.startMsSQLNode),
    takeLeading(LayoutActionTypes.STOP_MS_SQL_NODE, sagasMsSQL.stopMsSQLNode),
    takeLeading(LayoutActionTypes.STOP_MARIA_DB_NODE, sagasMariaDB.stopMariaDBNode),
    takeLeading(LayoutActionTypes.CHECK_MONGO_DATABASE_EXISTS, sagasMongo.checkMongoDatabaseExists),
    takeLeading(LayoutActionTypes.CHECK_MARIA_DB_DATABASE_NAME, sagasMariaDB.checkMariaDBDatabaseName),
    takeLeading(LayoutActionTypes.CHECK_MY_SQL_DATABASE_NAME, sagasMySQL.checkMySQLDatabaseName),
    takeLeading(LayoutActionTypes.CHECK_MS_SQL_DATABASE_NAME, sagasMsSQL.checkMsSQLDatabaseName),
    takeLeading(LayoutActionTypes.ADD_MARIA_DB_DATABASE, sagasMariaDB.addMariaDBDatabase),
    takeLeading(LayoutActionTypes.ADD_MY_SQL_DATABASE, sagasMySQL.addMySQLDatabase),
    takeLeading(LayoutActionTypes.ADD_MS_SQL_DATABASE, sagasMsSQL.addMsSQLDatabase),
    takeLeading(LayoutActionTypes.RESTORE_MARIA_DB_DATABASE, sagasMariaDB.restoreMariaDBDatabase),
    takeLeading(LayoutActionTypes.RESTORE_MY_SQL_DATABASE, sagasMySQL.restoreMySQLDatabase),
    takeLeading(LayoutActionTypes.RESTORE_MS_SQL_DATABASE, sagasMsSQL.restoreMsSQLDatabase),
    takeLeading(LayoutActionTypes.COPY_MONGO_DATABASE, sagasMongo.copyMongoDatabase),
    takeLeading(LayoutActionTypes.COPY_MARIA_DB_DATABASE, sagasMariaDB.copyMariaDBDatabase),
    takeLeading(LayoutActionTypes.COPY_MY_SQL_DATABASE, sagasMySQL.copyMySQLDatabase),
    takeLeading(LayoutActionTypes.COPY_MS_SQL_DATABASE, sagasMsSQL.copyMsSQLDatabase),
    takeLeading(LayoutActionTypes.EXPORT_MARIA_DB_DATABASE, sagasMariaDB.exportMariaDBDatabase),
    takeLeading(LayoutActionTypes.EXPORT_MY_SQL_DATABASE, sagasMySQL.exportMySQLDatabase),
    takeLeading(LayoutActionTypes.EXPORT_MS_SQL_DATABASE, sagasMsSQL.exportMsSQLDatabase),
    takeLeading(LayoutActionTypes.CHECK_MARIA_DB_DATABASE_SIZE, sagasMariaDB.checkMariaDBDatabaseSize),
    takeLeading(LayoutActionTypes.CHECK_MY_SQL_DATABASE_SIZE, sagasMySQL.checkMySQLDatabaseSize),
    takeLeading(LayoutActionTypes.CHECK_MS_SQL_DATABASE_SIZE, sagasMsSQL.checkMsSQLDatabaseSize),
    takeLeading(LayoutActionTypes.DROP_MARIA_DB_DATABASE, sagasMariaDB.dropMariaDBDatabase),
    takeLeading(LayoutActionTypes.DROP_MY_SQL_DATABASE, sagasMySQL.dropMySQLDatabase),
    takeLeading(LayoutActionTypes.DROP_MS_SQL_DATABASE, sagasMsSQL.dropMsSQLDatabase),
    takeLeading(LayoutActionTypes.DROP_MARIA_DB_NODE, sagasMariaDB.dropMariaDBNode),
    takeLeading(LayoutActionTypes.DROP_MY_SQL_NODE, sagasMySQL.dropMySQLNode),
    takeLeading(LayoutActionTypes.DROP_MS_SQL_NODE, sagasMsSQL.dropMsSQLNode),
    takeLeading(LayoutActionTypes.CHECK_MARIA_DB_VERSION_EXISTS, sagasMariaDB.checkMariaDBVersionExists),
    takeLeading(LayoutActionTypes.CHECK_MY_SQL_VERSION_EXISTS, sagasMySQL.checkMySQLVersionExists),
    takeLeading(LayoutActionTypes.CHECK_ELASTIC_SEARCH_VERSION_EXISTS, sagasElasticSearch.checkElasticSearchVersionExists),
    takeLeading(LayoutActionTypes.ADD_MARIA_DB_STANDBY, sagasMariaDB.addMariaDBStandby),
    takeLeading(LayoutActionTypes.ADD_MY_SQL_STANDBY, sagasMySQL.addMySQLStandby),
    takeLeading(LayoutActionTypes.REMOVE_MARIA_DB_SCHEDULE, sagasMariaDB.removeMariaDBSchedule),
    takeLeading(LayoutActionTypes.REMOVE_MY_SQL_SCHEDULE, sagasMySQL.removeMySQLSchedule),
    takeLeading(LayoutActionTypes.CHECK_MARIA_DB_DATABASE_EXISTS, sagasMariaDB.checkMariaDBDatabaseExists),
    takeLeading(LayoutActionTypes.CHECK_MY_SQL_DATABASE_EXISTS, sagasMySQL.checkMySQLDatabaseExists),
    takeLeading(LayoutActionTypes.CHECK_MS_SQL_DATABASE_EXISTS, sagasMsSQL.checkMsSQLDatabaseExists),
    takeLeading(LayoutActionTypes.CHECK_ELASTICSEARCH_INDEX_NAME, sagasElasticSearch.checkElasticSearchIndexName),
    takeLeading(LayoutActionTypes.CHECK_ELASTICSEARCH_INDEX_EXISTS, sagasElasticSearch.checkElasticSearchIndexExists),
    takeLeading(LayoutActionTypes.CHECK_ELASTICSEARCH_INDEX_SIZE, sagasElasticSearch.checkElasticSearchIndexSize),
    takeLeading(LayoutActionTypes.ADD_ELASTICSEARCH_INDEX, sagasElasticSearch.addElasticSearchIndex),
    takeLeading(LayoutActionTypes.DUMP_ELASTICSEARCH_INDEX, sagasElasticSearch.dumpElasticSearchIndex),
    takeLeading(LayoutActionTypes.COPY_ELASTICSEARCH_INDEX, sagasElasticSearch.copyElasticSearchIndex),
    takeLeading(LayoutActionTypes.RESTORE_ELASTICSEARCH_INDEX, sagasElasticSearch.restoreElasticSearchIndex),
    takeLeading(LayoutActionTypes.SNAPSHOT_ELASTICSEARCH_CLUSTER, sagasElasticSearch.snapshotElasticSearchCluster),
    takeLeading(LayoutActionTypes.RESTORE_ELASTICSEARCH_CLUSTER, sagasElasticSearch.restoreElasticSearchCluster),
    takeLeading(LayoutActionTypes.REMOVE_ELASTICSEARCH_SCHEDULE, sagasElasticSearch.removeElasticSearchSchedule),
    takeLeading(LayoutActionTypes.DROP_ELASTICSEARCH_NODE, sagasElasticSearch.dropElasticSearchNode),
    takeLeading(LayoutActionTypes.DROP_ELASTICSEARCH_INDEX, sagasElasticSearch.dropElasticSearchIndex),
    takeLeading(LayoutActionTypes.START_ELASTICSEARCH_NODE, sagasElasticSearch.startElasticSearchNode),
    takeLeading(LayoutActionTypes.STOP_ELASTICSEARCH_NODE, sagasElasticSearch.stopElasticSearchNode),
    takeLeading(LayoutActionTypes.ADD_ELASTICSEARCH_NODE, sagasElasticSearch.addElasticSearchNode),
    takeLeading(LayoutActionTypes.ADD_REDIS_NODE, sagasRedis.addRedisNode),
    takeLeading(LayoutActionTypes.SWITCH_ELASTICSEARCH_NODE, sagasElasticSearch.switchElasticsearchNode),
    takeLeading(LayoutActionTypes.SWITCH_REDIS_NODE, sagasRedis.switchRedisNode),
    takeLeading(LayoutActionTypes.REMOVE_REDIS_SCHEDULE, sagasRedis.removeRedisSchedule),
    takeLeading(LayoutActionTypes.REMOVE_REDIS_CLUSTER, sagasRedis.removeRedisCluster),
    takeLeading(LayoutActionTypes.START_REDIS_NODE, sagasRedis.startRedisNode),
    takeLeading(LayoutActionTypes.STOP_REDIS_NODE, sagasRedis.stopRedisNode),
    takeLeading(LayoutActionTypes.VERIFY_BACKUP, verifyBackup),
    takeLeading(LayoutActionTypes.SWITCH_MY_SQL_NODE, sagasMySQL.switchMySQLNode),
    takeLeading(LayoutActionTypes.SWITCH_MARIA_DB_NODE, sagasMariaDB.switchMariaDBNode),
    takeLeading(LayoutActionTypes.EXPORT_MY_SQL_NODE, sagasMySQL.exportMySQLNode),
    takeLeading(LayoutActionTypes.EXPORT_MS_SQL_NODE, sagasMsSQL.exportMsSQLNode),
    takeLeading(LayoutActionTypes.REMOVE_MONGO_SCHEDULE, sagasMongo.removeMongoSchedule),
    takeLeading(LayoutActionTypes.ADD_HA_PROXY, sagasHAProxy.addHAProxy),
    takeLeading(LayoutActionTypes.DROP_HA_PROXY, sagasHAProxy.dropHAProxy),
    takeLeading(LayoutActionTypes.STOP_HA_PROXY, sagasHAProxy.stopHAProxy),
    takeLeading(LayoutActionTypes.START_HA_PROXY, sagasHAProxy.startHAProxy),
    takeLeading(LayoutActionTypes.REFRESH_CONTAINER, refreshContainer),
    takeLeading(LayoutActionTypes.CHECK_REDIS_CLUSTER_HEALTH, sagasRedis.checkRedisClusterHealth),
    takeLeading(LayoutActionTypes.CHECK_SENTINEL_MASTER_NAME, sagasRedis.checkSentinelMasterName),
    takeLeading(LayoutActionTypes.SENTINEL_MONITOR_MASTER, sagasRedis.sentinelMonitorMaster),
    takeLeading(LayoutActionTypes.SENTINEL_REMOVE_MONITOR, sagasRedis.sentinelRemoveMonitor),
    takeLeading(LayoutActionTypes.SENTINEL_FAILOVER, sagasRedis.sentinelFailover),
    takeLeading(LayoutActionTypes.IS_NODE_HOST_REACHABLE, isNodeHostReachable),
    takeLeading(LayoutActionTypes.MARK_NODE_DELETED, markNodeDeleted),
    takeLeading(LayoutActionTypes.ADD_MONGO_SHARDED_CLUSTER, sagasMongo.addMongoShardedCluster),
    takeLeading(LayoutActionTypes.ADD_MONGO_SHARDED_CLUSTER_PRIMARY, sagasMongo.addMongoShardedClusterPrimary),
    takeLeading(LayoutActionTypes.ADD_MONGO_SHARDED_CLUSTER_NODE, sagasMongo.addMongoShardedClusterNode),
    takeLeading(LayoutActionTypes.DROP_MONGO_SHARDED_CLUSTER, sagasMongo.dropMongoShardedCluster),
    takeLeading(LayoutActionTypes.REFRESH_DB_LIST, refreshDbList),
    takeLeading(LayoutActionTypes.DROP_PG_PUBLICATION, sagasPostgres.dropPgPublication),
    takeLeading(LayoutActionTypes.TOGGLE_PG_SUBSCRIPTION, sagasPostgres.togglePgSubscription),
    takeLeading(LayoutActionTypes.UPGRADE_PG_NODE, sagasPostgres.upgradePgNode),
    takeLeading(LayoutActionTypes.UPGRADE_MONGO_NODE, sagasMongo.upgradeMongoNode),
    takeLeading(LayoutActionTypes.UPGRADE_MY_SQL_NODE, sagasMySQL.upgradeMySQLNode),
    takeLeading(LayoutActionTypes.UPGRADE_MARIA_DB_NODE, sagasMariaDB.upgradeMariaDBNode),
    takeLeading(LayoutActionTypes.UPGRADE_ELASTICSEARCH_NODE, sagasElasticSearch.upgradeElasticSearchNode),
    takeLeading(LayoutActionTypes.ADD_PG_POD, sagasPostgresKubernetes.addPgPod),
    takeLeading(LayoutActionTypes.ADD_MONGO_POD, sagasMongoKubernetes.addMongoPod),
    takeLeading(LayoutActionTypes.CHECK_FREE_IP, checkFreeIp),
    takeLeading(LayoutActionTypes.CHECK_SERVICE_CONNECTION, checkServiceConnection),
    takeLeading(LayoutActionTypes.CHECK_FREE_POD_IP, checkFreePodIp),
    takeLeading(LayoutActionTypes.ADD_PG_STANDBY_POD, sagasPostgresKubernetes.addPgStandbyPod),
    takeLeading(LayoutActionTypes.GET_SUBNETS, getSubnets),
    takeLeading(LayoutActionTypes.DROP_CLUSTER, dropCluster),

];
export default sagas;
