import { call, put, takeLeading, select } from "redux-saga/effects";
import { LogActions, LogActionTypes } from "./reducer";
import api from "../../services/api";
function* getLogs({data}) {
    try {
        yield put(LogActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let logs = yield call(api.getLogs, accessToken, data);
        
        yield put(LogActions.setLogs(logs));
        yield put(LogActions.setErrorMessage(""));
        yield put(LogActions.setLoading(false));
    }
    catch (e) {
        yield put(LogActions.setLoading(false));
        yield put(LogActions.setErrorMessage(e.message));
    }
}
function* getLogsPerNode({data}) {
    try {
        yield put(LogActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let logs = yield call(api.getNodeLogs, accessToken, data);
        
        yield put(LogActions.setNodeLogs(logs));
        yield put(LogActions.setErrorMessage(""));
        yield put(LogActions.setLoading(false));
    }
    catch (e) {
        yield put(LogActions.setLoading(false));
        yield put(LogActions.setErrorMessage(e.message));
    }
}
function* getStepLogs({id}) {
    try {
        yield put(LogActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let stepLogs = yield call(api.getStepLogs, accessToken, id);
        
        yield put(LogActions.setStepLogs(stepLogs));
        yield put(LogActions.setErrorMessage(""));
        yield put(LogActions.setLoading(false));
    }
    catch (e) {
        yield put(LogActions.setLoading(false));
        yield put(LogActions.setErrorMessage(e.message));
    }
}
function* getTimelineData() {
    try {
        yield put(LogActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let timeline_data = yield call(api.getTimelineData, accessToken);
        
        yield put(LogActions.setTimelineData(timeline_data));
        yield put(LogActions.setErrorMessage(""));
        yield put(LogActions.setLoading(false));
    }
    catch (e) {
        yield put(LogActions.setLoading(false));
        yield put(LogActions.setTimelineData(null));
        yield put(LogActions.setErrorMessage(e.message));
    }
}
function* getNodeLogFiles({data}) {
    try {
        yield put(LogActions.setLoading(true));
        yield put(LogActions.setNodeLogFiles(null));
        const { accessToken } = yield select(state => state.app);
        
        const node_log_files = yield call(api.getNodeLogFiles, accessToken, data);
        
        yield put(LogActions.setNodeLogFiles(node_log_files?.node_log_files));
        yield put(LogActions.setErrorMessage(""));
        yield put(LogActions.setLoading(false));
    }
    catch (e) {
        yield put(LogActions.setLoading(false));
        yield put(LogActions.setErrorMessage(e.message));
    }
}
const sagas = [
    takeLeading(LogActionTypes.GET_LOGS, getLogs),
    takeLeading(LogActionTypes.GET_LOGS_PER_NODE, getLogsPerNode),
    takeLeading(LogActionTypes.GET_STEP_LOGS, getStepLogs),
    takeLeading(LogActionTypes.GET_TIMELINE_DATA, getTimelineData),
    takeLeading(LogActionTypes.GET_NODE_LOG_FILES, getNodeLogFiles),
];
export default sagas;
    
    