import {produce} from "immer";
import { createActions, createReducer } from "reduxsauce";
const initialState = {
    isLoading: false,
    errorMessage: "",
    infoMessage: "",
    messages: {},
    monitors: null,
    alerts: null,
    agents: null,
    nodes: null,
    hosts: null,
    agents_chart: null,
    agents_summary: null,
    node_monitor_names: null,
    node_monitors_table: null,
    monitor_chart_columns: null,
    last_node_monitors_alerts: {},
    monitor_charts: {},
    history_per_alerts: {},
    dashboard_monitors: [],
    alerts_per_node: {},
    metrics_per_node: {} 
};
const { Types, Creators } = createActions({
    setLoading: ["isLoading"],
    setErrorMessage: ["errorMessage"],
    setInfoMessage:  ["infoMessage"],
    getAlertHistory: ["data"],
    getLastNodeMonitorAlert: ["data"],
    refreshDashboard: ["data"],
    setLastNodeMonitorsAlerts: ["node_name", "monitor_name", "alert", "data"],
    setMonitorCharts: ["node_name", "monitor_name", "chart", "data"],
    unsetLastNodeMonitorAlert: ["caller","node_name", "monitor_name"],
    unsetMonitorChart: ["caller","node_name", "monitor_name"],
    getHistoryPerAlerts: ["alerts", "interval"],
    setHistoryPerAlerts: ["alert_id", "history"],
    setAlertHistory: ["alert_history"],
    snoozeMonitor: ["data", "callback"],
    getMessage: ["id"],
    getMonitors: ["data"],
    setMessage: ["key", "message"],
    addMonitor: ["data", "callback"],
    addAgent: ["data", "callback"],
    updateMonitor: ["data", "callback"],
    deleteMonitor: ["data"],
    deleteAgent: ["id"],
    editAgent: ["data", "field"],
    setMonitors: ["monitors"],
    setNodeMonitorNames: ["node_monitor_names"],
    setMonitorChartColumns: ["monitor_chart_columns"],
    setAgents: ["agents"],
    setNodeMonitorsTable: ["node_monitors_table"],
    getAlerts: null,
    getAgents: null,
    getNodeMonitorsTable: ['dashboardType'],
    getAgentsSummary: null,
    getNodeMonitorNames: ["data"],
    getMonitorChartColumns: ["data"],
    getMonitorChart: ["data"],
    getNodes: null,
    getHosts: null,
    setNodes: ["nodes"],
    setHosts: ["hosts"],
    getAgentsChart: ["data"],
    setAgentsChart: ["agents_chart"],
    setAgentsSummary: ["agents_summary"],
    getNodeAlerts: ["data"],
    setAlerts: ["alerts"],
    setAlertsPerNode: ["alerts", "node_id"],
    getDashboardMonitors: ["time_period"],
    setDashboardMonitors: ["dashboard_monitor"],
    reorderDashboardMonitors: ["data"],
    addDashboardMonitor: ["data"],
    deleteDashboardMonitor: ["dashboard_monitor"],
    checkNodesCpuRamCurrent: ["data"],
    setNodeCpuRamCurrent: ["data", "node_id"],

}, { prefix: 'MONITORING/' });
export const MonitoringActionTypes = Types;
export const MonitoringActions = Creators;
export const monitoringReducer = createReducer(initialState, {
    [MonitoringActionTypes.SET_LOADING]: produce((draft, { isLoading }) => ({ ...draft, isLoading })),
    [MonitoringActionTypes.SET_ERROR_MESSAGE]: produce((draft, { errorMessage }) => ({ ...draft, errorMessage })),
    [MonitoringActionTypes.SET_INFO_MESSAGE]: produce((draft, { infoMessage }) => ({ ...draft, infoMessage })),
    [MonitoringActionTypes.SET_ALERTS]: produce((draft, { alerts }) => ({ ...draft, alerts })),
    [MonitoringActionTypes.SET_ALERTS_PER_NODE]: produce((draft, { alerts, node_id }) => { draft.alerts_per_node[node_id] = alerts }),
    [MonitoringActionTypes.SET_AGENTS]: produce((draft, { agents }) => ({ ...draft, agents })),
    [MonitoringActionTypes.SET_NODE_MONITORS_TABLE]: produce((draft, { node_monitors_table }) => ({ ...draft, node_monitors_table })),
    [MonitoringActionTypes.SET_AGENTS_CHART]: produce((draft, { agents_chart }) => ({ ...draft, agents_chart })),
    [MonitoringActionTypes.SET_AGENTS_SUMMARY]: produce((draft, { agents_summary}) => ({ ...draft, agents_summary })),
    [MonitoringActionTypes.SET_ALERT_HISTORY]: produce((draft, { alert_history }) => ({ ...draft, alert_history })),
    [MonitoringActionTypes.SET_MONITORS]: produce((draft, { monitors }) => ({ ...draft, monitors })),
    [MonitoringActionTypes.SET_NODE_MONITOR_NAMES]: produce((draft, { node_monitor_names }) => ({ ...draft, node_monitor_names })),
    [MonitoringActionTypes.SET_NODES]: produce((draft, { nodes }) => ({ ...draft, nodes })),
    [MonitoringActionTypes.SET_MONITOR_CHART_COLUMNS]: produce((draft, { monitor_chart_columns }) => ({ ...draft, monitor_chart_columns })),
    [MonitoringActionTypes.SET_HOSTS]: produce((draft, { hosts }) => ({ ...draft, hosts })),
    [MonitoringActionTypes.SET_DASHBOARD_MONITORS]: produce((draft, { dashboard_monitor }) => {
        draft.dashboard_monitors.push(dashboard_monitor);
        return draft;
    }),
    [MonitoringActionTypes.DELETE_DASHBOARD_MONITOR]: produce((draft, { dashboard_monitor }) => {
        const index = draft.dashboard_monitors.findIndex((item) => item.monitor_id === dashboard_monitor.monitor_id);
        if (index !== -1) draft.dashboard_monitors.splice(index, 1)
        return draft;
    }),
    [MonitoringActionTypes.SET_MONITOR_CHARTS]: produce((draft, {node_name, monitor_name, chart, data }) => {
        const caller = chart.caller;
        draft.monitor_charts = {
            ...(draft.monitor_charts || {}),
            [caller]: {
                ...(draft.monitor_charts?.[caller] || {}),
                [node_name]: {
                    ...(draft.monitor_charts?.[caller]?.[node_name] || {}),
                    [monitor_name]:{data: chart.points, request: data},
                }
            }
        };
        return draft;
    }),
    [MonitoringActionTypes.SET_LAST_NODE_MONITORS_ALERTS]: produce((draft, {node_name, monitor_name, alert, data}) => {
        const caller = alert.caller;
        draft.last_node_monitors_alerts = {
            ...(draft.last_node_monitors_alerts || {}),
            [caller]: {
                ...(draft.last_node_monitors_alerts?.[caller] || {}),
                [node_name]: {
                    ...(draft.last_node_monitors_alerts?.[caller]?.[node_name] || {}),
                    [monitor_name]:{data: alert.alert, request: data},
                }
            }
        };
        return draft;
    }),
    [MonitoringActionTypes.UNSET_LAST_NODE_MONITOR_ALERT]: produce((draft, {caller, node_name, monitor_name}) =>{delete draft["last_node_monitors_alerts"][caller][node_name][monitor_name]}),
    [MonitoringActionTypes.UNSET_MONITOR_CHART]: produce((draft, {caller, node_name, monitor_name}) =>{delete draft["monitor_charts"][caller][node_name][monitor_name]}),
    [MonitoringActionTypes.SET_MESSAGE]: produce((draft, {key, message }) => ({ ...draft, messages: {
        ...draft["messages"],
        [key]: message
    }})),
    [MonitoringActionTypes.SET_HISTORY_PER_ALERTS]: produce((draft, {alert_id, history }) => {
        draft.history_per_alerts[alert_id] = history
    }),
    [MonitoringActionTypes.SET_NODE_CPU_RAM_CURRENT]: produce((draft, {data, node_id }) => {
        draft.metrics_per_node[node_id] = data
    }),
});
