import { call, put, takeLeading, takeEvery, select } from "redux-saga/effects";
import apiMonitoring from "../../services/monitoring/api";
import api from "../../services/api";
import { MonitoringActions, MonitoringActionTypes } from "../../store/monitoring/reducer";
import notify from "../../util/notify";

function* getAlertHistory({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setAlertHistory(null));
        const { accessToken } = yield select(state => state.app);
        const alert_history = yield call(apiMonitoring.getAlertHistory, accessToken, data);
        yield put(MonitoringActions.setAlertHistory(alert_history?.alert_history));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* refreshDashboard({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setAlertHistory(null));
        const { last_node_monitors_alerts, monitor_charts } = yield select(state => state.monitoring);
        const alert_keys = Object.keys(last_node_monitors_alerts)
        const chart_keys = Object.keys(monitor_charts)
        const db_type = data["db_type"]
        if (alert_keys.length > 0) {
            for (const u of alert_keys) {
                for (const n of Object.keys(last_node_monitors_alerts[u])) {
                    for( const m of Object.keys(last_node_monitors_alerts[u][n]))
                    {
                        if  (last_node_monitors_alerts[u][n][m]?.request["node_type"] === db_type)
                         {
                            console.log(last_node_monitors_alerts[u][n][m])
                            yield put(MonitoringActions.getLastNodeMonitorAlert(last_node_monitors_alerts[u][n][m]?.request))

                         }
                    }
                }
            }
        }
        if (chart_keys.length > 0) {
            for (const u of chart_keys) {
                for (const n of Object.keys(monitor_charts[u])) {
                    for( const m of Object.keys(monitor_charts[u][n]))
                    {
                        if (monitor_charts[u][n][m]?.request["node_type"] === db_type)
                        {
                            console.log(monitor_charts[u][n][m])
                            yield put(MonitoringActions.getMonitorChart(monitor_charts[u][n][m]?.request))
                        }
                    }
                }
            }
        }
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getDashboardMonitors({time_period}) {
  try {
      yield put(MonitoringActions.setLoading(true));

      const { accessToken } = yield select(state => state.app);
      const dashboard_monitors = yield call(apiMonitoring.getDashboardMonitors, accessToken);
      for ( const node of dashboard_monitors )
      {
          node["time_period"] = time_period
          let data;

          switch (node["visualization_type"]) {
              case "line_chart":
                  data = yield call(apiMonitoring.getLastNodeMonitorAlert, accessToken, node);
                  node["points"] = data?.points
                  break;
              case "table":
                  data = yield call(apiMonitoring.getLastNodeMonitorAlert, accessToken, node);
                  node["alert"] = data?.alert;
                  break;
              case "pie_chart":
                  data = yield call(apiMonitoring.getLastNodeMonitorAlert, accessToken, node);
                  node["points"] = data?.points
                  break;
              case "monitor_status":
                  data = yield call(apiMonitoring.getMonitorChart, accessToken, node);
                  node["severity"] = data?.severity;
                  break;
              default:
                  console.warn(`Unexpected visualization_type: ${node["visualization_type"]}`);
                  break;
          }
          yield put(MonitoringActions.setDashboardMonitors(node));

      }

      yield put(MonitoringActions.setErrorMessage(""));
      yield put(MonitoringActions.setLoading(false));
  }
  catch (e) {
      yield put(MonitoringActions.setLoading(false));
      yield put(MonitoringActions.setErrorMessage(e.message));
  }
}
function* reorderDashboardMonitors({data, callback}) {
    try {
      yield put(MonitoringActions.setLoading(true));

      const { accessToken } = yield select(state => state.app);
      const response = yield call(apiMonitoring.reorderDashboardMonitors, accessToken, data);
      if (response?.status >= 400) {
          yield put(MonitoringActions.setLoading(false));
          if (response?.data?.detail) {
              yield put(MonitoringActions.setErrorMessage(response.data.detail));
          }
          else
              yield put(MonitoringActions.setErrorMessage(response?.data));
          }
      else {
            yield call(callback, false);
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setLoading(false));
        }
      }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* addDashboardMonitor({ data }) {
  try {
      yield put(MonitoringActions.setLoading(true));
      yield put(MonitoringActions.setErrorMessage(""));

      const { accessToken } = yield select(state => state.app);
      const response = yield call(apiMonitoring.addDashboardMonitor, accessToken, data);
      if (response?.status >= 400) {
          if (response?.data?.detail) {
              yield put(MonitoringActions.setErrorMessage(response.data.detail));
          }
          else
          {
              yield put(MonitoringActions.setErrorMessage(response?.data));
          }
          yield put(MonitoringActions.setLoading(false));
      }
      else {
          yield put(MonitoringActions.setErrorMessage(""));
          yield put(MonitoringActions.setLoading(false));
          const monitor = response?.monitor;
          monitor["time_period"] = data?.time_period;
          if (monitor["visualization_type"] !== "table")
          {
              const data = yield call(apiMonitoring.getMonitorChart, accessToken, monitor);
              monitor["points"] = data?.points;
          }
          else
          {
              const data = yield call(apiMonitoring.getLastNodeMonitorAlert, accessToken, monitor);
              monitor["alert"] = data?.alert;
          }
          yield put(MonitoringActions.setDashboardMonitors(monitor));
      }
  }
  catch (e) {
      yield put(MonitoringActions.setLoading(false));

  }
}
function* deleteDashboardMonitor({dashboard_monitor}) {
  try {
      yield put(MonitoringActions.setLoading(true));
      yield put(MonitoringActions.setErrorMessage(""));

      const { accessToken } = yield select(state => state.app);
      const response = yield call(apiMonitoring.deleteDashboardMonitor, accessToken, dashboard_monitor);
      if (response?.status >= 400) {
          yield put(MonitoringActions.setLoading(false));
          if (response?.dashboard_monitor?.detail) {
              yield put(MonitoringActions.setErrorMessage(response.dashboard_monitor.detail));
          }
          yield put(MonitoringActions.setErrorMessage(response?.dashboard_monitor));
      }
      else {
          yield put(MonitoringActions.setErrorMessage(""));
          yield put(MonitoringActions.setLoading(false));
      }
  }
  catch (e) {
      yield put(MonitoringActions.setLoading(false));

  }
}

function* getHistoryPerAlerts({alerts, interval}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        for(let i = 0; i < alerts.length; i++)
        {
            const alert_history = yield call(apiMonitoring.getAlertHistory, accessToken, {"org_id": 1, "node_id": alerts[i].node_id, "monitor_name": alerts[i].monitor_name, "node_type": alerts[i].node_type, "ind_interval": interval});
            yield put(MonitoringActions.setHistoryPerAlerts(alerts[i].alert_id, alert_history?.alert_history));
        }
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* snoozeMonitor({ data, callback }) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setErrorMessage(""));

        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMonitoring.snoozeMonitor, accessToken, data);
        if (response?.status >= 400) {
            yield put(MonitoringActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(MonitoringActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(MonitoringActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback, false);
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));

    }
}
function* getAlerts() {
    try {
        yield put(MonitoringActions.setAlerts(null));
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);

        let alerts = yield call(apiMonitoring.getAlerts, accessToken);

        yield put(MonitoringActions.setAlerts(alerts));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
    }
}
function* getAgents() {
    try {
        yield put(MonitoringActions.setAgents(null));
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);

        let agents = yield call(apiMonitoring.getAgents, accessToken);

        yield put(MonitoringActions.setAgents(agents));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
    }
}
function* getNodeMonitorsTable({dashboardType}) {
    try {
        yield put(MonitoringActions.setNodeMonitorsTable(null));
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);

        let node_monitors_table = yield call(apiMonitoring.getNodeMonitorsTable, accessToken, dashboardType);

        yield put(MonitoringActions.setNodeMonitorsTable(node_monitors_table));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
    }
}
function* getAgentsSummary() {
    try {
        yield put(MonitoringActions.setAgentsSummary(null));
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);

        let agents_summary = yield call(apiMonitoring.getAgentsSummary, accessToken);

        yield put(MonitoringActions.setAgentsSummary(agents_summary));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
    }
}
function* getNodeAlerts({data}) {
    try {
        yield put(MonitoringActions.setAlerts(null));
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);

        let alerts = yield call(apiMonitoring.getNodeAlerts, accessToken, data);
        if(data.grouping)  yield put(MonitoringActions.setAlertsPerNode(alerts, data.node_id));
        else yield put(MonitoringActions.setAlerts(alerts));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
    }
}
function* getMessage({id}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        for (let i = 0; i < id.length; i++)
        {
            const message = yield call(apiMonitoring.getMessage, accessToken, id[i]);
            yield put(MonitoringActions.setMessage(id[i], message.message));
        }
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* addMonitor({ data, callback }) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setErrorMessage(""));

        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMonitoring.addMonitor, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(MonitoringActions.setErrorMessage(response.data.detail));
                yield call(callback.error, response.data.detail);
            }
            else //probably verbose, so need all data
            {
                yield put(MonitoringActions.setErrorMessage(response?.data));
                yield call(callback.error, response.data);
            }
            yield put(MonitoringActions.setLoading(false));
        }
        else {
            yield call(callback.success, false);
            yield put(MonitoringActions.setLoading(false));
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setMonitors(null));
            yield put(MonitoringActions.setAlerts(null));
            yield put(MonitoringActions.setAlertHistory(null));
            yield put(MonitoringActions.setHistoryPerAlerts(null));
        }
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));

    }
}
function* addAgent({ data, callback }) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setErrorMessage(""));

        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMonitoring.addAgent, accessToken, data);
        if (response?.status >= 400) {
            //api exception
            if (response?.data?.detail) {
                yield put(MonitoringActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(MonitoringActions.setErrorMessage(response?.data));
            }
            yield put(MonitoringActions.setLoading(false));
        }
        else {
            yield call(callback, false);
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setLoading(false));
            yield put(MonitoringActions.setAgents(null));
        }
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));

    }
}
function* updateMonitor({ data, callback }) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setErrorMessage(""));

        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMonitoring.updateMonitor, accessToken, data);
        if (response?.status >= 400) {
            yield put(MonitoringActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(MonitoringActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(MonitoringActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield call(callback.success, false);
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setLoading(false));
            yield put(MonitoringActions.setMonitors(null));
            yield put(MonitoringActions.setAlerts(null));
            yield put(MonitoringActions.setAlertHistory(null));
            yield put(MonitoringActions.setHistoryPerAlerts(null));
        }
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));

    }
}
function* deleteMonitor({ data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setErrorMessage(""));

        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMonitoring.deleteMonitor, accessToken, data);
        if (response?.status >= 400) {
            yield put(MonitoringActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(MonitoringActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(MonitoringActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setLoading(false));
            yield put(MonitoringActions.setMonitors(null));
            yield put(MonitoringActions.setAlerts(null));
            yield put(MonitoringActions.setAlertHistory(null));
            yield put(MonitoringActions.setHistoryPerAlerts(null));
        }
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));

    }
}
function* deleteAgent({ id}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setErrorMessage(""));

        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMonitoring.deleteAgent, accessToken, id);
        if (response?.status >= 400) {
            yield put(MonitoringActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield put(MonitoringActions.setErrorMessage(response.data.detail));
            }
            else //probably verbose, so need all data
            {
                yield put(MonitoringActions.setErrorMessage(response?.data));
            }
        }
        else {
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setLoading(false));
            yield put(MonitoringActions.setAgents(null));
        }
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));

    }
}
function* editAgent({data, field}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setErrorMessage(""));

        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMonitoring.editAgent, accessToken, data);
        if (response?.status >= 400) {
            yield put(MonitoringActions.setLoading(false));
            //api exception
            if (response?.data?.detail) {
                yield call(notify.error, response.data.detail);
            }
        }
        else {
            yield put(MonitoringActions.setErrorMessage(""));
            yield put(MonitoringActions.setLoading(false));
            yield put(MonitoringActions.setAgents(null));
            switch(field)
            {
                case "role":
                    yield call(notify.warning, `Agent set for ${data[field]}`);
                    break;
                case "ind_active":
                    data[field] === "Y" ?  yield call(notify.success, "Agent activated") : yield call(notify.warning, "Agent deactivated");
                    break;
                case "ind_default":
                    data[field] === "Y" ?  yield call(notify.success, "Agent set to default") : yield call(notify.warning, "Agent set to regular");
                    break;
                default:
                    break;
            }
        }
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));

    }
}
function* checkNodesCpuRamCurrent({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const node_ids = data?.node_ids;
        if (node_ids && node_ids.length > 0){

            for (let i = 0; i < node_ids.length; i++)
            {
                const node_id = node_ids[i]
                const metrics = yield call(apiMonitoring.checkNodeCpuRamCurrent, accessToken, {node_id: node_id});
                yield put(MonitoringActions.setNodeCpuRamCurrent(metrics, node_id));


            }

        }
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getMonitors({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        yield put(MonitoringActions.setMonitors(null));
        const { accessToken } = yield select(state => state.app);
        const monitors = yield call(apiMonitoring.getMonitors, accessToken, data);
        yield put(MonitoringActions.setMonitors(monitors?.monitors));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getNodeMonitorNames({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const node_monitor_names = yield call(apiMonitoring.getNodeMonitorNames, accessToken, data);
        yield put(MonitoringActions.setNodeMonitorNames(node_monitor_names));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getMonitorChartColumns({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const monitor_chart_columns = yield call(apiMonitoring.getMonitorChartColumns, accessToken, data);
        yield put(MonitoringActions.setMonitorChartColumns(monitor_chart_columns));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getLastNodeMonitorAlert({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const res = yield call(apiMonitoring.getLastNodeMonitorAlert, accessToken, data);
        if(res?.status !== 204)
            yield put(MonitoringActions.setLastNodeMonitorsAlerts(data?.node_name, data?.monitor_name, res?.data, data));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getMonitorChart({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const chart = yield call(apiMonitoring.getMonitorChart, accessToken, data);
        yield put(MonitoringActions.setMonitorCharts(data?.node_name, data?.monitor_name, chart, data));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getAgentsChart({data}) {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const agents_chart = yield call(apiMonitoring.getAgentsChart, accessToken, data);
        yield put(MonitoringActions.setAgentsChart(agents_chart));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
        yield put(MonitoringActions.setErrorMessage(e.message));
    }
}
function* getHosts() {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let hosts = yield call(api.getAvailableHosts, accessToken);
        yield put(MonitoringActions.setHosts(hosts));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
    }
}
function* getNodes() {
    try {
        yield put(MonitoringActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let nodes = yield call(api.getAvailableNodes, accessToken);
        yield put(MonitoringActions.setNodes(nodes));
        yield put(MonitoringActions.setErrorMessage(""));
        yield put(MonitoringActions.setLoading(false));
    }
    catch (e) {
        yield put(MonitoringActions.setLoading(false));
    }
}

const sagas = [
    takeLeading(MonitoringActionTypes.GET_NODES, getNodes),
    takeLeading(MonitoringActionTypes.GET_HOSTS, getHosts),
    takeLeading(MonitoringActionTypes.GET_ALERTS, getAlerts),
    takeLeading(MonitoringActionTypes.GET_NODE_ALERTS, getNodeAlerts),
    takeLeading(MonitoringActionTypes.GET_ALERT_HISTORY, getAlertHistory),
    takeLeading(MonitoringActionTypes.GET_NODE_MONITOR_NAMES, getNodeMonitorNames),
    takeLeading(MonitoringActionTypes.SNOOZE_MONITOR, snoozeMonitor),
    takeLeading(MonitoringActionTypes.GET_MESSAGE, getMessage),
    takeLeading(MonitoringActionTypes.GET_MONITORS, getMonitors),
    takeLeading(MonitoringActionTypes.ADD_MONITOR, addMonitor),
    takeLeading(MonitoringActionTypes.GET_AGENTS, getAgents),
    takeLeading(MonitoringActionTypes.GET_NODE_MONITORS_TABLE, getNodeMonitorsTable),
    takeLeading(MonitoringActionTypes.GET_AGENTS_CHART, getAgentsChart),
    takeLeading(MonitoringActionTypes.GET_AGENTS_SUMMARY, getAgentsSummary),
    takeLeading(MonitoringActionTypes.ADD_AGENT, addAgent),
    takeLeading(MonitoringActionTypes.EDIT_AGENT, editAgent),
    takeLeading(MonitoringActionTypes.DELETE_AGENT, deleteAgent),
    takeLeading(MonitoringActionTypes.UPDATE_MONITOR, updateMonitor),
    takeLeading(MonitoringActionTypes.DELETE_MONITOR, deleteMonitor),
    takeLeading(MonitoringActionTypes.GET_MONITOR_CHART_COLUMNS, getMonitorChartColumns),
    takeEvery(MonitoringActionTypes.GET_MONITOR_CHART, getMonitorChart),
    takeLeading(MonitoringActionTypes.REFRESH_DASHBOARD, refreshDashboard),
    takeLeading(MonitoringActionTypes.GET_LAST_NODE_MONITOR_ALERT, getLastNodeMonitorAlert),
    takeLeading(MonitoringActionTypes.GET_HISTORY_PER_ALERTS, getHistoryPerAlerts),
    takeLeading(MonitoringActionTypes.GET_DASHBOARD_MONITORS, getDashboardMonitors),
    takeLeading(MonitoringActionTypes.REORDER_DASHBOARD_MONITORS, reorderDashboardMonitors),
    takeLeading(MonitoringActionTypes.ADD_DASHBOARD_MONITOR, addDashboardMonitor),
    takeLeading(MonitoringActionTypes.DELETE_DASHBOARD_MONITOR, deleteDashboardMonitor),
    takeLeading(MonitoringActionTypes.CHECK_NODES_CPU_RAM_CURRENT, checkNodesCpuRamCurrent)
];
export default sagas;
