import { call, put, takeLeading, select } from "redux-saga/effects";
import { SettingsActions, SettingsActionTypes } from "./reducer";
import api from "../../services/api";
import notify from "../../util/notify";

function* getSystemSettings() {
  try {
      yield put(SettingsActions.setLoading(true));
      const { accessToken } = yield select(state => state.app);
      let response = yield call(api.getSystemSettings, accessToken);
      yield call(notify.info, response.notification);
      yield put(SettingsActions.setLoading(false));
      yield put(SettingsActions.setSettings(response));
  }
  catch (e) {
      yield put(SettingsActions.setLoading(false));
  }
}

function* updateSystemSettings({ data }) {
  try {
      yield put(SettingsActions.setLoading(true));
      const { accessToken } = yield select(state => state.app);
      let response = yield call(api.updateSystemSettings, accessToken, data);
      yield call(notify.info, response.notification);
      yield put(SettingsActions.setLoading(false));
      yield put(SettingsActions.setSettings(null));
  }
  catch (e) {
      yield put(SettingsActions.setLoading(false));
  }
}

const sagas = [
    takeLeading(SettingsActionTypes.UPDATE_SYSTEM_SETTINGS, updateSystemSettings),
    takeLeading(SettingsActionTypes.GET_SYSTEM_SETTINGS, getSystemSettings)
];
export default sagas;
