import { call, put, takeLeading, select} from "redux-saga/effects";
import { SignupActions, SignupActionTypes } from "./reducer";
import api from "../../services/api";
import { ConfigActions } from "../config/reducer";
import notify from "../../util/notify";
function* signUp({data, callback}) {
    try {
        yield put(SignupActions.setLoading(true));
        yield put(SignupActions.setInfoMessage(""));
        const { accessToken } = yield select(state => state.app);
        let res = yield call(api.signUp, accessToken, data);
        if(res?.status >= 400) yield put(SignupActions.setErrorMessage(res?.data?.result));
        else 
        {
            yield call(callback, false);
            yield call(notify.info, 'Please confirm your email address to activate your user');
        }
        yield put(ConfigActions.setConfig(null));
        yield put(SignupActions.setLoading(false));
    }
    catch (e) {
        yield put(SignupActions.setLoading(false));
    }
}
function* validateEmail({data, callback}) {
    try {
        
        yield put(SignupActions.setLoading(true));
        yield put(SignupActions.setErrorMessage(""));
        yield put(SignupActions.setInfoMessage(""));
        const { accessToken } = yield select(state => state.app);
        let res = yield call(api.validateEmail, accessToken, data);
        if(res?.valid === false) yield put(SignupActions.setErrorMessage(`${data?.email} is not reachable`));
        else 
        {
            yield call(callback, 1);
        }
        yield put(SignupActions.setLoading(false));
        yield put(ConfigActions.setConfig(null));
    }
    catch (e) {
        yield put(SignupActions.setLoading(false));
    }
}
function* requestDemo({data, callback}) {
    try {
        
        yield put(SignupActions.setLoading(true));
        yield put(SignupActions.setErrorMessage(""));
        yield put(SignupActions.setInfoMessage(""));
        const { accessToken } = yield select(state => state.app);
        let res = yield call(api.requestDemo, accessToken, data);
        if(res?.status >= 400) yield call(notify.error, res.data?.detail);
        else
        {
            yield call(callback, false)
            yield call(notify.info, `Activation link sent to ${data?.email}`);

        }
        yield put(SignupActions.setLoading(false));
        yield put(ConfigActions.setConfig(null));
    }
    catch (e) {
        yield put(SignupActions.setLoading(false));
    }
}
const sagas = [
    takeLeading(SignupActionTypes.SIGN_UP, signUp),
    takeLeading(SignupActionTypes.REQUEST_DEMO, requestDemo),
    takeLeading(SignupActionTypes.VALIDATE_EMAIL, validateEmail)
];
export default sagas;