import { call, put, select } from "redux-saga/effects";
import { StatusActions } from "../reducer";
import { LayoutActions } from "../../layout/reducer";
import apiMySQL from "../../../services/mysql/api";
import notify from "../../../util/notify";
function* getMySQLDetails({id}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let mysql_details = yield call(apiMySQL.getMySQLDetails, accessToken, id);
        if(mysql_details?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            if(mysql_details?.data)
            {
                 //api exception
             if(mysql_details?.data?.detail) 
             {
 
                 yield put(StatusActions.setErrorMessage(mysql_details.data.detail));
 
             }
             else //probably verbose, so need all data
             {
 
                 yield put(StatusActions.setErrorMessage(mysql_details.data));
 
 
                 
             }
             yield put(StatusActions.setMySqlDetails(mysql_details.data));
            }
        }
        else
        {
            yield put(StatusActions.setMySqlDetails(mysql_details));
            yield put(StatusActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(LayoutActions.setErrorMessage(e.message));
    }
}

function* getMySQLSettings({id}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        const mysql_settings = yield call(apiMySQL.getMySQLSettings, accessToken, id);
        if(mysql_settings?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            yield put(StatusActions.setMySqlSettings([]));
            yield call(notify.error, mysql_settings?.data?.detail ? mysql_settings.data.detail : mysql_settings.data);
        }
        else
        {
            yield put(StatusActions.setMySqlSettings(mysql_settings));
            yield put(StatusActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(LayoutActions.setErrorMessage(e.message));
    }
}

function* editMySQLSettings({ data, callback}) {
    try {
        
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiMySQL.editMySQLSettings, accessToken, data);
        if (response?.status >= 400) {
            yield put(StatusActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(StatusActions.setLoading(false));
            yield call(callback, false);
            yield call(notify.info, response.notification);
           
        }
       
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
    }
}


const sagasMySQL = 
{
    getMySQLDetails,
    getMySQLSettings,
    editMySQLSettings
}
export default sagasMySQL;