import { call, put, select } from "redux-saga/effects";
import { StatusActions } from "../reducer";
import apiOracle from "../../../services/oracle/api";
import { LayoutActions } from "../../layout/reducer";
import notify from "../../../util/notify";
function* getCdbDetails({pk}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let cdb_details = yield call(apiOracle.getCdbDetails, accessToken, pk);
        if(cdb_details?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            
            if(cdb_details?.data)
            {
                 //api exception
             if(cdb_details?.data?.detail) 
             {
                 yield put(StatusActions.setErrorMessage(cdb_details.data.detail));
 
             }
             else //probably verbose, so need all data
             {
 
                 yield put(StatusActions.setErrorMessage(cdb_details?.data));
                 
             }
             yield put(StatusActions.setCdbDetails(cdb_details.data));
            }
            
            
        }
        else
        {
            yield put(StatusActions.setCdbDetails(cdb_details));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(StatusActions.setErrorMessage(e.message));
    }
}
function* getPdbDetails({params}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let pdb_details = yield call(apiOracle.getPdbDetails, accessToken, params.pk, params.id);
        if(pdb_details?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            if(pdb_details?.data)
            {
                 //api exception
             if(pdb_details?.data?.detail) 
             {
 
                 yield put(StatusActions.setErrorMessage(pdb_details.data.detail));
 
             }
             else //probably verbose, so need all data
             {
 
                 yield put(StatusActions.setErrorMessage(pdb_details.data));
 
 
                 
             }
             yield put(StatusActions.setPdbDetails(pdb_details.data));
            }
            
            
        }
        else
        {
            yield put(StatusActions.setPdbDetails(pdb_details));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(LayoutActions.setErrorMessage(e.message));
    }
}

function* getCDBSettings({id}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        const cdb_settings = yield call(apiOracle.getCDBSettings, accessToken, id);
        if(cdb_settings?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            yield put(StatusActions.setCdbSettings([]));
            yield call(notify.error, cdb_settings?.data?.detail ? cdb_settings.data.detail : cdb_settings.data);
        }
        else
        {
            yield put(StatusActions.setCdbSettings(cdb_settings));
            yield put(StatusActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(LayoutActions.setErrorMessage(e.message));
    }
}

function* editCDBSettings({ data, callback}) {
    try {
        
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiOracle.editCDBSettings, accessToken, data);
        if (response?.status >= 400) {
            yield put(StatusActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(StatusActions.setLoading(false));
            yield call(callback, false);
            yield call(notify.info, response.notification);
           
        }
       
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
    }
}

const sagasOracle = 
{
    getCdbDetails,
    getPdbDetails,
    getCDBSettings,
    editCDBSettings
}
export default sagasOracle;