import { call, put, select } from "redux-saga/effects";
import { StatusActions } from "../reducer";
import { LayoutActions } from "../../layout/reducer";
import apiPostg from "../../../services/postgres/api";
import notify from "../../../util/notify";
function* getPgDetails({id}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let pg_details = yield call(apiPostg.getPgDetails, accessToken, id);
        if(pg_details?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            if(pg_details?.data)
            {
                 //api exception
             if(pg_details?.data?.detail) 
             {
 
                 yield put(StatusActions.setErrorMessage(pg_details.data.detail));
 
             }
             else //probably verbose, so need all data
             {
 
                 yield put(StatusActions.setErrorMessage(pg_details.data));
 
 
                 
             }
             yield put(StatusActions.setPgDetails(pg_details.data));
            }
        }
        else
        {
            yield put(StatusActions.setPgDetails(pg_details));
            yield put(StatusActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(LayoutActions.setErrorMessage(e.message));
    }
}

function* getPgSettings({id}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        const pg_settings = yield call(apiPostg.getPgSettings, accessToken, id);
        if(pg_settings?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            yield put(StatusActions.setPgSettings([]));
            yield call(notify.error, pg_settings?.data?.detail ? pg_settings.data.detail : pg_settings.data);
        }
        else
        {
            yield put(StatusActions.setPgSettings(pg_settings));
            yield put(StatusActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(LayoutActions.setErrorMessage(e.message));
    }
}

function* editPgSettings({ data, callback}) {
    try {
        
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const response = yield call(apiPostg.editPgSettings, accessToken, data);
        if (response?.status >= 400) {
            yield put(StatusActions.setLoading(false));
            yield call(notify.error, response?.data?.detail ? response.data.detail : response.data);
        }
        else {
            yield put(StatusActions.setLoading(false));
            yield call(callback, false);
            yield call(notify.info, response.notification);
           
        }
       
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
    }
}

const sagasPostgres = 
{
    getPgDetails,
    getPgSettings,
    editPgSettings,
}
export default sagasPostgres;