import {produce} from "immer";
import { createActions, createReducer } from "reduxsauce";
const initialState = {
    isLoading: false,
    errorMessage: "",
    infoMessage: "",
    pdb_details: null,
    cdb_details: null,
    pg_details: null,
    mongo_details: null,
    mariadb_details: null,
    mysql_details: null,
    mssql_details: null,
    elasticsearch_details: null,
    redis_details: null,
    haproxy_details: null,
    licensed_clusters: null,
    pg_settings: null,
    cdb_settings: null,
    mysql_settings: null,
    nodes: null,
    status_charts: {},
    nodeSeverityCounts: {},
};
const { Types, Creators } = createActions({
    setLoading: ["isLoading"],
    setErrorMessage: ["errorMessage"],
    setInfoMessage:  ["infoMessage"],
    getNodes: ["plain"],
    getNode: ["pk"],
    getCdbDetails: ["pk"],
    getPgDetails: ["id"],
    getPgSettings: ["id"],
    editPgSettings: ["data", "callback"],
    getCdbSettings: ["id"],
    editCdbSettings: ["data", "callback"],
    getMySqlSettings: ["id"],
    editMySqlSettings: ["data", "callback"],
    getHaProxyDetails: ["id"],
    getMongoDetails: ["id"],
    getMariaDbDetails: ["id"],
    getMySqlDetails: ["id"],
    getMsSqlDetails: ["id"],
    getRedisDetails: ["id"],
    getElasticsearchDetails: ["id"],
    getPdbDetails: ["params"],
    setNode: ["node"],
    setNodes: ["nodes"],
    setCdbDetails: ["cdb_details"],
    setPdbDetails: ["pdb_details"],
    setPgDetails: ["pg_details"],
    setPgSettings: ["pg_settings"],
    setCdbSettings: ["cdb_settings"],
    setMySqlSettings: ["mysql_settings"],
    setHaProxyDetails: ["haproxy_details"],
    setMongoDetails: ["mongo_details"],
    setRedisDetails: ["redis_details"],
    setMariaDbDetails: ["mariadb_details"],
    setMySqlDetails: ["mysql_details"],
    setMsSqlDetails: ["mssql_details"],
    getHostsCpuUsages: ["data"],
    getHostCpuUsageDaily: ["data"],
    setElasticsearchDetails: ["elasticsearch_details"],
    setValidCredentials: ["valid_credentials"],
    getLicensedClusters: ["data"],
    setLicensedClusters: ["licensed_clusters"],
    getStatusCharts: ["data"],
    setStatusCharts: ["status_charts", "host_id", "col_name"],
    moveLicense: ["data", "callback"],

    getNodeSeverityCounts: ['nodeIds'],
    getNodeSeverityCountsStart: null,
    getNodeSeverityCountsSuccess: ['countsMap'],
    getNodeSeverityCountsFailure: ['error'],
    clearNodeSeverityCounts: null,


}, { prefix: 'STATUS/' });
export const StatusActionTypes = Types;
export const StatusActions = Creators;
export const statusReducer = createReducer(initialState, {
    [StatusActionTypes.SET_LOADING]: produce((draft, { isLoading }) => ({ ...draft, isLoading })),
    [StatusActionTypes.SET_ERROR_MESSAGE]: produce((draft, { errorMessage }) => ({ ...draft, errorMessage })),
    [StatusActionTypes.SET_INFO_MESSAGE]: produce((draft, { infoMessage }) => ({ ...draft, infoMessage })),
    [StatusActionTypes.SET_NODE]:  produce((draft, { node }) => ({ ...draft, node })),
    [StatusActionTypes.SET_CDB_DETAILS]:  produce((draft, { cdb_details }) => ({ ...draft, cdb_details })),
    [StatusActionTypes.SET_PDB_DETAILS]:  produce((draft, { pdb_details }) => ({ ...draft, pdb_details })),
    [StatusActionTypes.SET_PG_DETAILS]:  produce((draft, { pg_details }) => ({ ...draft, pg_details })),
    [StatusActionTypes.SET_PG_SETTINGS]:  produce((draft, { pg_settings }) => ({ ...draft, pg_settings })),
    [StatusActionTypes.SET_CDB_SETTINGS]:  produce((draft, { cdb_settings }) => ({ ...draft, cdb_settings })),
    [StatusActionTypes.SET_MY_SQL_SETTINGS]:  produce((draft, { mysql_settings }) => ({ ...draft, mysql_settings })),
    [StatusActionTypes.SET_HA_PROXY_DETAILS]:  produce((draft, { haproxy_details }) => ({ ...draft, haproxy_details })),
    [StatusActionTypes.SET_MARIA_DB_DETAILS]:  produce((draft, { mariadb_details }) => ({ ...draft, mariadb_details })),
    [StatusActionTypes.SET_MY_SQL_DETAILS]:  produce((draft, { mysql_details }) => ({ ...draft, mysql_details })),
    [StatusActionTypes.SET_MS_SQL_DETAILS]:  produce((draft, { mssql_details }) => ({ ...draft, mssql_details })),
    [StatusActionTypes.SET_MONGO_DETAILS]:  produce((draft, { mongo_details }) => ({ ...draft, mongo_details })),
    [StatusActionTypes.SET_ELASTICSEARCH_DETAILS]:  produce((draft, { elasticsearch_details }) => ({ ...draft, elasticsearch_details })),
    [StatusActionTypes.SET_REDIS_DETAILS]:  produce((draft, { redis_details }) => ({ ...draft, redis_details })),
    [StatusActionTypes.SET_NODES]: produce((draft, { nodes }) => ({ ...draft, nodes })),
    [StatusActionTypes.SET_VALID_CREDENTIALS]: produce((draft, { valid_credentials }) => ({ ...draft, valid_credentials })),
    [StatusActionTypes.SET_LICENSED_CLUSTERS]: produce((draft, { licensed_clusters }) => ({ ...draft, licensed_clusters })),
    [StatusActionTypes.SET_STATUS_CHARTS]: produce((draft, { status_charts, host_id, col_name }) => {
        if(! draft.status_charts?.[host_id]) draft.status_charts[host_id] = {}
        draft.status_charts[host_id][col_name] = status_charts
     }),

    [StatusActionTypes.GET_NODE_SEVERITY_COUNTS_START]: produce(() => {
    }),
    [StatusActionTypes.GET_NODE_SEVERITY_COUNTS_SUCCESS]: produce((draft, { countsMap }) => {
        draft.nodeSeverityCounts = { ...draft.nodeSeverityCounts, ...countsMap };
    }),
    [StatusActionTypes.GET_NODE_SEVERITY_COUNTS_FAILURE]: produce(() => {
    }),
    [StatusActionTypes.CLEAR_NODE_SEVERITY_COUNTS]: produce((draft) => {
        draft.nodeSeverityCounts = {};
    }),
});
