import { call, put, takeLatest, takeLeading, all, select } from "redux-saga/effects";
import { StatusActions, StatusActionTypes } from "./reducer";
import sagasOracle from "./oracle/sagas";
import api from "../../services/api";
import sagasPostgres from "./postgres/sagas";
import sagasMongo from "./mongo/sagas";
import sagasMariaDB from "../status/mariadb/sagas";
import sagasElasticSearch from "./elasticsearch/sagas";
import sagasRedis from "./redis/sagas";
import sagasMySQL from "./mysql/sagas";
import sagasMsSQL from "./mssql/sagas";
import { aggregationFunctionTypes } from "../../config";
import sagasHAProxy from "./balancers/haproxy/sagas";
import { LayoutActions } from "../layout/reducer";
import notify from "../../util/notify";
function* getNodes({plain}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let nodes = yield call( plain ? api.getAvailableNodes : api.getAvailableTree, accessToken);

        yield put(StatusActions.setNodes(nodes));
        yield put(StatusActions.setErrorMessage(""));
        yield put(StatusActions.setLoading(false));
    }
    catch (e) {
        console.log(e);
        yield put(StatusActions.setLoading(false));
    }
}

function* getStatusCharts({data}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const host_ids = data['host_ids']
        let request = {'time_period': data['time_period']}
        for(let i = 0; i < host_ids.length; i++)
            {
                const host_id = host_ids[i]
                request['host_id'] = host_id
                request['col_name'] = 'cpuusage'
                let status_charts = yield call(api.getStatusCharts, accessToken, request)
                yield put(StatusActions.setStatusCharts(status_charts, host_id, request['col_name']));
                request['col_name'] = 'memusage'
                status_charts = yield call(api.getStatusCharts, accessToken, request)
                yield put(StatusActions.setStatusCharts(status_charts, host_id, request['col_name']));
            }

        yield put(StatusActions.setErrorMessage(""));
    }
    catch (e) {
        yield put(StatusActions.setErrorMessage(e.message));
    }
    finally {
        yield put(StatusActions.setLoading(false));
    }
}

function* getNode({ pk }) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);

        let node = yield call(api.getNode, accessToken, pk);
        yield put(StatusActions.setNode(node));
        yield put(StatusActions.setLoading(false));
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
    }
}
function* getHostsCpuUsages({ data }) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        let { nodes } = yield select(state => state.status);
        if (nodes) {
            const cpu_usages = yield call(api.getHostsCpuUsages, accessToken, data);
            if (!cpu_usages?.status) {
                let field;
                switch (data?.cpu_agg) {
                    case aggregationFunctionTypes.MIN:
                        field = 'cpuMinUsages';
                        break;
                    case aggregationFunctionTypes.MAX:
                        field = 'cpuMaxUsages';
                        break;
                    case aggregationFunctionTypes.AVG:
                        field = 'cpuAvgUsages';
                        break;
                    default:
                        break;
                }
                if (field) {
                    nodes = nodes.map((n) => cpu_usages[n.host_id] ? { ...n, [field]: cpu_usages[n.host_id] } : { ...n, [field]: [] });
                    yield put(StatusActions.setNodes(nodes));
                }
            }
        }
        yield put(StatusActions.setLoading(false));
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
    }
}
function* getHostCpuUsageDaily({ data }) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const cpu_usages = yield call(api.getHostCpuUsageDaily, accessToken, data.request);
        if(!cpu_usages?.status)
        {
            const node = {...data.node, cpuUsages: cpu_usages}
            yield put(StatusActions.setNode(node));
        }
        yield put(StatusActions.setLoading(false));
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
    }
}
function* getLicensedClusters({data}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        const licensed_clusters = yield call(api.getLicensedClusters, accessToken, data);
        if(licensed_clusters?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            if(licensed_clusters?.data)
            {
                 //api exception
             if(licensed_clusters?.data?.detail)
             {

                 yield put(LayoutActions.setErrorMessage(licensed_clusters.data.detail));

             }
             else //probably verbose, so need all data
             {

                 yield put(LayoutActions.setErrorMessage(licensed_clusters.data));



             }
            }
        }
        else
        {
            yield put(StatusActions.setLicensedClusters(licensed_clusters.clusters));
            yield put(LayoutActions.setErrorMessage(""));
            yield put(StatusActions.setLoading(false));
        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(StatusActions.setErrorMessage(e.message));
    }
}
function* moveLicense({data, callback}) {
    try {
        yield put(StatusActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);

        const response = yield call(api.moveLicense, accessToken, data);
        if(response?.status >= 400)
        {
            yield put(StatusActions.setLoading(false));
            if(response?.data)
            {
                 //api exception
             if(response?.data?.detail)
             {
                 yield put(LayoutActions.setErrorMessage(response.data.detail));
             }
             else //probably verbose, so need all data
             {
                 yield put(LayoutActions.setErrorMessage(response.data));

             }
            }
        }
        else
        {
            yield call(callback, false)
            yield put(StatusActions.setLoading(false));
            yield put(LayoutActions.setErrorMessage(""));
            yield call(notify.success, response.notification);




        }
    }
    catch (e) {
        yield put(StatusActions.setLoading(false));
        yield put(LayoutActions.setErrorMessage(e.message));
    }
}

function* getNodeSeverityCountsSaga({ nodeIds }) {
  if (!nodeIds || nodeIds.length === 0) {
      return;
  }

  try {
      yield put(StatusActions.getNodeSeverityCountsStart());
      const { accessToken } = yield select(state => state.app);

      const responses = yield all(
          nodeIds.map(nodeId => call(api.getNodeSeverityCounts, nodeId, accessToken))
      );

      const countsMap = responses.reduce((acc, responseResult) => {
          if (responseResult && responseResult.node_id !== undefined && responseResult.severity_counts && responseResult.status === undefined) {
              acc[responseResult.node_id] = responseResult.severity_counts;
          } else if (responseResult && responseResult.status) {
          }
          return acc;
      }, {});

      yield put(StatusActions.getNodeSeverityCountsSuccess(countsMap));

  } catch (e) {
      yield put(StatusActions.getNodeSeverityCountsFailure(e.message || 'Failed to fetch severity counts'));
  }
}

const sagas = [
    takeLatest(StatusActionTypes.GET_NODES, getNodes),
    takeLatest(StatusActionTypes.GET_NODE, getNode),
    takeLatest(StatusActionTypes.GET_CDB_DETAILS, sagasOracle.getCdbDetails),
    takeLatest(StatusActionTypes.GET_PDB_DETAILS, sagasOracle.getPdbDetails),
    takeLatest(StatusActionTypes.GET_PG_DETAILS, sagasPostgres.getPgDetails),
    takeLatest(StatusActionTypes.GET_PG_SETTINGS, sagasPostgres.getPgSettings),
    takeLatest(StatusActionTypes.EDIT_PG_SETTINGS, sagasPostgres.editPgSettings),
    takeLatest(StatusActionTypes.GET_CDB_SETTINGS, sagasOracle.getCDBSettings),
    takeLatest(StatusActionTypes.EDIT_CDB_SETTINGS, sagasOracle.editCDBSettings),
    takeLatest(StatusActionTypes.GET_MY_SQL_SETTINGS, sagasMySQL.getMySQLSettings),
    takeLatest(StatusActionTypes.EDIT_MY_SQL_SETTINGS, sagasMySQL.editMySQLSettings),
    takeLatest(StatusActionTypes.GET_HA_PROXY_DETAILS, sagasHAProxy.getHAProxyDetails),
    takeLatest(StatusActionTypes.GET_ELASTICSEARCH_DETAILS, sagasElasticSearch.getElasticSearchDetails),
    takeLatest(StatusActionTypes.GET_MONGO_DETAILS, sagasMongo.getMongoDetails),
    takeLatest(StatusActionTypes.GET_REDIS_DETAILS, sagasRedis.getRedisDetails),
    takeLatest(StatusActionTypes.GET_MARIA_DB_DETAILS, sagasMariaDB.getMariaDBDetails),
    takeLatest(StatusActionTypes.GET_MY_SQL_DETAILS, sagasMySQL.getMySQLDetails),
    takeLatest(StatusActionTypes.GET_MS_SQL_DETAILS, sagasMsSQL.getMsSQLDetails),
    takeLatest(StatusActionTypes.GET_HOSTS_CPU_USAGES, getHostsCpuUsages),
    takeLatest(StatusActionTypes.GET_HOST_CPU_USAGE_DAILY, getHostCpuUsageDaily),
    takeLatest(StatusActionTypes.GET_LICENSED_CLUSTERS, getLicensedClusters),
    takeLatest(StatusActionTypes.MOVE_LICENSE, moveLicense),
    takeLatest(StatusActionTypes.GET_STATUS_CHARTS, getStatusCharts),
    takeLeading(StatusActionTypes.GET_NODE_SEVERITY_COUNTS, getNodeSeverityCountsSaga),
];
export default sagas;
