import { call, put, takeLeading, select } from "redux-saga/effects";
import api from "../../services/api";
import { TaskNotificationActions, TaskNotificationActionTypes } from "./reducer";
import { LayoutActions } from "../layout/reducer";
import { StatusActions } from "../status/reducer";
import { MonitoringActions } from "../monitoring/reducer";
import { LogActions } from "../log/reducer";
function* getNotifications() {
    try {
        yield put(TaskNotificationActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        let notifications = yield call(api.getTaskNotifications, accessToken);
        
        yield put(TaskNotificationActions.setNotifications(notifications));
        yield put(TaskNotificationActions.setErrorMessage(""));
        yield put(TaskNotificationActions.setLoading(false));
    }
    catch (e) {
        yield put(TaskNotificationActions.setLoading(false));
    }
}
function* isRunningCompleted({data}) {
    try {
        yield put(TaskNotificationActions.setLoading(true));
        const { accessToken } = yield select(state => state.app);
        
        const response = yield call(api.isRunningCompleted, accessToken, data);
        if (response?.refresh)  {
            yield put(LayoutActions.getNodes());
            yield put(StatusActions.getNodes());
            yield put(MonitoringActions.getAlerts());
            yield put(LogActions.getLogs());
            yield put(StatusActions.setPgDetails(null));
            yield put(StatusActions.setMongoDetails(null));
            yield put(StatusActions.setMySqlDetails(null));
            yield put(StatusActions.setMariaDbDetails(null));
            yield put(StatusActions.setMongoDetails(null));
            yield put(StatusActions.setRedisDetails(null));
            yield put(StatusActions.setMsSqlDetails(null));
            yield put(StatusActions.setCdbDetails(null));
            yield put(StatusActions.setPdbDetails(null));
            yield put(TaskNotificationActions.getNotifications());
        }
        yield put(TaskNotificationActions.setErrorMessage(""));
    }
    catch (e) {
         
    }
    finally {
        yield put(TaskNotificationActions.setLoading(false));
    }
}

const sagas = [
    takeLeading(TaskNotificationActionTypes.GET_NOTIFICATIONS, getNotifications),
    takeLeading(TaskNotificationActionTypes.IS_RUNNING_COMPLETED, isRunningCompleted)
];
export default sagas;