import { call, put, takeLeading, select} from "redux-saga/effects";
import { UserUpdateActions, UserUpdateActionTypes } from "./reducer";
import { ConfigActions } from "../config/reducer";
import api from "../../services/api";
function* userUpdate(action) {
    try {
        
        yield put(UserUpdateActions.setLoading(true));
        yield put(UserUpdateActions.setInfoMessage(""));
        const { accessToken } = yield select(state => state.app);
        let data = yield call(api.userUpdate, accessToken, action);
        if(data && typeof data === "object" && ! data?.result) 
             yield put(UserUpdateActions.setInitial(data));
        else
        {
            if(data?.result) yield put(UserUpdateActions.setErrorMessage(data.result));
            else 
            {
    
                yield put(UserUpdateActions.setErrorMessage(""));
                yield put(UserUpdateActions.setInfoMessage("ok"));
                yield call(action?.callback, false);
                yield put(ConfigActions.setConfig(null));
    
            }
        }
        yield put(UserUpdateActions.setLoading(false));
    }
    catch (e) {
        yield put(UserUpdateActions.setLoading(false));
        yield put(UserUpdateActions.setInitial(false));
        
    }
}
function* userDelete(action, callback) {
    try {
        
        yield put(UserUpdateActions.setLoading(true));
        yield put(UserUpdateActions.setInfoMessage(""));
        const { accessToken } = yield select(state => state.app);
        yield call(api.deleteUser, accessToken, action.id);
        yield put(UserUpdateActions.setLoading(false));
        yield put(ConfigActions.setConfig(null));
        
    }
    catch (e) {
        yield put(UserUpdateActions.setLoading(false));
        yield put(UserUpdateActions.setInitial(false));
        
    }
}
const sagas = [
    takeLeading(UserUpdateActionTypes.USER_UPDATE, userUpdate),
    takeLeading(UserUpdateActionTypes.USER_DELETE, userDelete)
];
export default sagas;