import React, { useState, useEffect, useRef } from 'react';
import { ProgressBar } from "primereact/progressbar";
import Timestrip from "../components/timestrip";
import { useSelector } from "react-redux";

const TIMELINE_HEIGHT = '24px';
const RENDER_DELAY_MS = 10;

const renderTimelineContent = (item, history_per_alerts, interval) => {
    const alertHistory = history_per_alerts?.[item.alert_id];
    const hasHistory = alertHistory?.length > 0;

    const spanStyle = {
        display: 'inline-block',
        height: TIMELINE_HEIGHT,
        lineHeight: TIMELINE_HEIGHT,
        width: '100%',
        textAlign: 'center'
    };

    return hasHistory ? (
        <ProgressBar
            className="timestrip"
            displayValueTemplate={() => <Timestrip data={alertHistory} interval={interval} />}
            value={100}
            style={{ width: '100%', height: TIMELINE_HEIGHT, display: 'flex', alignItems: 'center' }}
        />
    ) : (
         <span style={spanStyle}>-</span>
    );
};


const LazyTimeline = ({ item, history_per_alerts, interval}) => {
    const [shouldRenderContent, setShouldRenderContent] = useState(false);
    const { isLoading } = useSelector((state) => state.monitoring);
    const alertHistory = history_per_alerts?.[item.alert_id];

    const ref = useRef(null);
    const [scrolledDown, setScrolledDown] = useState(false);
    const [lastScrollY, setLastScrollY] = useState(0);

    useEffect(() => {
        const element = ref.current;

        const handleScroll = () => {
            if (!element || scrolledDown) {
                return;
            }

            const componentTop = element.getBoundingClientRect().top;
            const currentScrollY = window.scrollY;

            if (componentTop < window.innerHeight && currentScrollY > lastScrollY) {
                setScrolledDown(true);
            }

            setLastScrollY(currentScrollY);
        };

        const checkInitialVisibility = () => {
             if (!element || scrolledDown) return;
             const rect = element.getBoundingClientRect();
             if (rect.top < window.innerHeight && rect.bottom > 0) {
                 setScrolledDown(true);
             }
        }

        if (!scrolledDown) {
             checkInitialVisibility();
             window.addEventListener("scroll", handleScroll, { passive: true });
             window.addEventListener('resize', checkInitialVisibility, { passive: true });
        }

        return () => {
             window.removeEventListener("scroll", handleScroll);
             window.removeEventListener('resize', checkInitialVisibility);
        }

    }, [scrolledDown, lastScrollY]);


    useEffect(() => {
        let timerId;
        if (scrolledDown && !shouldRenderContent) {
            timerId = setTimeout(() => {
                setShouldRenderContent(true);
            }, RENDER_DELAY_MS);
        }
        return () => clearTimeout(timerId);
    }, [scrolledDown, shouldRenderContent]);


    return (
        <div
            ref={ref}
            style={{
                height: TIMELINE_HEIGHT,
                width: '100%',
                display: 'flex',
                alignItems: 'center',
                justifyContent: 'center',
                overflow: 'hidden',
            }}
        >
            {shouldRenderContent ? (
                isLoading && !alertHistory ? (
                    <i className="pi pi-spin pi-spinner" style={{ fontSize: '1rem' }} />
                ) : (
                    renderTimelineContent(item, history_per_alerts, interval)
                )
            ) : (
                 <i className="pi pi-spin pi-spinner" style={{ fontSize: '1rem' }} />
            )}
        </div>
    );
};

export default LazyTimeline;
