spec_statement = """
        CREATE OR REPLACE PACKAGE SYS.pck_pdb_maint
        AS
            PROCEDURE pc_copy_pdb (src_pdb_name VARCHAR2, trg_pdb_name VARCHAR2);
            PROCEDURE pc_chk_exist_pdb (pdb_name       VARCHAR2,
                                        i_mode         CHAR,
                                        o_status   OUT NUMBER);
            PROCEDURE pc_drop_pdb (pdb_name VARCHAR2);
            PROCEDURE pc_drop_pdb_nodbf (pdb_name VARCHAR2);
            PROCEDURE pc_open_pdb (pdb_name VARCHAR2);
            PROCEDURE pc_close_pdb (pdb_name VARCHAR2);
            PROCEDURE pc_unplug_pdb (pdb_name VARCHAR2, xml_file VARCHAR2);
            PROCEDURE pc_cre_pdb_using_xml (pdb_name VARCHAR2, xml_file VARCHAR2);
            PROCEDURE pc_cp_pdb_dbfs (pdb_name VARCHAR2, trg_path VARCHAR2);
        END pck_pdb_maint;
        """
body_statement = """
    CREATE OR REPLACE PACKAGE BODY SYS.pck_pdb_maint
AS
   PROCEDURE pc_chk_exist_pdb (pdb_name       VARCHAR2,
                               i_mode         CHAR,
                               o_status   OUT NUMBER)
   AS
      v_count       NUMBER := 0;
      e_not_exist   EXCEPTION;
   BEGIN
      IF i_mode = 'a'
      THEN
         SELECT COUNT (*)
           INTO v_count
           FROM v$pdbs
          WHERE name = pdb_name;
      ELSIF i_mode = 'r'
      THEN
         SELECT COUNT (*)
           INTO v_count
           FROM v$pdbs
          WHERE name = pdb_name AND open_mode IN ('READ WRITE', 'READ ONLY');
      ELSIF i_mode = 'm'
      THEN
         SELECT COUNT (*)
           INTO v_count
           FROM v$pdbs
          WHERE name = pdb_name AND open_mode = 'MOUNTED';
      END IF;
      IF v_count <> 1
      THEN
         o_status := 0;
      ELSE
         o_status := 1;
      END IF;
   EXCEPTION
      WHEN OTHERS
      THEN
         raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
   END pc_chk_exist_pdb;
   PROCEDURE pc_cre_pdb_using_xml (pdb_name VARCHAR2, xml_file VARCHAR2)
   AS
      v_cmd_text         VARCHAR2 (300);
      o_exist_status     NUMBER;
      e_already_exists   EXCEPTION;
   BEGIN
      v_cmd_text :=
            'create pluggable database '
         || pdb_name
         || ' using '''
         || xml_file
         || ''' nocopy tempfile reuse';
      pc_chk_exist_pdb (pdb_name, 'a', o_exist_status);
      IF o_exist_status <> 0
      THEN
         RAISE e_already_exists;
      END IF;
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_already_exists
      THEN
         raise_application_error (-20010,
                                  'PDB ' || pdb_name || ' already exists');
      WHEN OTHERS
      THEN
         raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
   END pc_cre_pdb_using_xml;
   PROCEDURE pc_copy_pdb (src_pdb_name VARCHAR2, trg_pdb_name VARCHAR2)
   AS
      v_cmd_text         VARCHAR2 (300);
      o_exist_status     NUMBER;
      e_already_exists   EXCEPTION;
      e_not_exist        EXCEPTION;
   BEGIN
      v_cmd_text :=
            'create pluggable database '
         || trg_pdb_name
         || ' from '
         || src_pdb_name ;
      pc_chk_exist_pdb (src_pdb_name, 'r', o_exist_status);
      IF o_exist_status = 0
      THEN
         RAISE e_not_exist;
      END IF;
      pc_chk_exist_pdb (trg_pdb_name, 'a', o_exist_status);
      IF o_exist_status <> 0
      THEN
         RAISE e_already_exists;
      END IF;
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_already_exists
      THEN
         raise_application_error (
            -20010,
            'Target PDB ' || trg_pdb_name || ' already exists');
      WHEN e_not_exist
      THEN
         raise_application_error (
            -20010,
            'PDB ' || src_pdb_name || ' does not exist or it is not readable');
      WHEN OTHERS
      THEN
         raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
         --dbms_output.put_line(SQLCODE||' '||SUBSTR (SQLERRM, 1, 64));
   END pc_copy_pdb;
   PROCEDURE pc_open_pdb (pdb_name VARCHAR2)
   AS
      v_cmd_text       VARCHAR2 (300);
      o_exist_status   NUMBER;
      e_not_exist      EXCEPTION;
   BEGIN
      v_cmd_text := 'alter pluggable database ' || pdb_name || ' open ';
      pc_chk_exist_pdb (pdb_name, 'a', o_exist_status);
      IF o_exist_status = 0
      THEN
         RAISE e_not_exist;
      END IF;
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_not_exist
      THEN
         raise_application_error (-20010,
                                  'PDB ' || pdb_name || ' does not exist');
      WHEN OTHERS
      THEN
         raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
   END pc_open_pdb;
   PROCEDURE pc_close_pdb (pdb_name VARCHAR2)
   AS
      v_cmd_text       VARCHAR2 (300);
      o_exist_status   NUMBER;
      e_not_exist      EXCEPTION;
   BEGIN
      v_cmd_text :=
         'alter pluggable database ' || pdb_name || ' close immediate ';
      pc_chk_exist_pdb (pdb_name, 'r', o_exist_status);
      IF o_exist_status = 0
      THEN
         RAISE e_not_exist;
      END IF;
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_not_exist
      THEN
         raise_application_error (
            -20010,
            'PDB ' || pdb_name || ' does not exist or it is not readable');
      WHEN OTHERS
      THEN
         raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
   END pc_close_pdb;
   PROCEDURE pc_unplug_pdb (pdb_name VARCHAR2, xml_file VARCHAR2)
   AS
      v_cmd_text       VARCHAR2 (300);
      o_exist_status   NUMBER;
      e_not_exist      EXCEPTION;
   BEGIN
      v_cmd_text :=
            'alter pluggable database '
         || pdb_name
         || ' unplug into '''
         || xml_file
         || ''' ';
      pc_chk_exist_pdb (pdb_name, 'a', o_exist_status);
      IF o_exist_status = 0
      THEN
         RAISE e_not_exist;
      END IF;
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_not_exist
      THEN
         raise_application_error (-20010,
                                  'PDB ' || pdb_name || ' does not exist');
      --WHEN OTHERS
      ---THEN
       --  raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
      
   END pc_unplug_pdb;
   PROCEDURE pc_drop_pdb (pdb_name VARCHAR2)
   AS
      v_cmd_text       VARCHAR2 (300);
      o_exist_status   NUMBER;
      e_not_exist      EXCEPTION;
   BEGIN
      pc_chk_exist_pdb (pdb_name, 'a', o_exist_status);
      IF o_exist_status = 0
      THEN
         RAISE e_not_exist;
      END IF;
      pc_chk_exist_pdb (pdb_name, 'r', o_exist_status);
      IF o_exist_status <> 0
      THEN
         v_cmd_text :=
            'alter pluggable database ' || pdb_name || ' close immediate ';
         DBMS_OUTPUT.put_line (v_cmd_text);
         EXECUTE IMMEDIATE v_cmd_text;
      END IF;
      v_cmd_text :=
         'drop pluggable database ' || pdb_name || ' including datafiles';
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_not_exist
      THEN
         raise_application_error (-20010,
                                  'PDB ' || pdb_name || ' does not exist');
      WHEN OTHERS
      THEN
         raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
   END pc_drop_pdb;
   PROCEDURE pc_drop_pdb_nodbf (pdb_name VARCHAR2)
   AS
      v_cmd_text       VARCHAR2 (300);
      o_exist_status   NUMBER;
      e_not_exist      EXCEPTION;
   BEGIN
      pc_chk_exist_pdb (pdb_name, 'a', o_exist_status);
      IF o_exist_status = 0
      THEN
         RAISE e_not_exist;
      END IF;
      pc_chk_exist_pdb (pdb_name, 'r', o_exist_status);
      IF o_exist_status <> 0
      THEN
         v_cmd_text :=
            'alter pluggable database ' || pdb_name || ' close immediate ';
         DBMS_OUTPUT.put_line (v_cmd_text);
         EXECUTE IMMEDIATE v_cmd_text;
      END IF;
      v_cmd_text := 'drop pluggable database ' || pdb_name;
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_not_exist
      THEN
         raise_application_error (-20010,
                                  'PDB ' || pdb_name || ' does not exist');
      WHEN OTHERS
      THEN
         raise_application_error (SQLCODE, SUBSTR (SQLERRM, 1, 64));
   END pc_drop_pdb_nodbf;
   PROCEDURE pc_cp_pdb_dbfs (pdb_name VARCHAR2, trg_path VARCHAR2)
   AS
      v_cmd_text       VARCHAR2 (300);
      o_exist_status   NUMBER;
      e_not_exist      EXCEPTION;
      v_src_dir        VARCHAR2 (300);
      v_count          NUMBER := 0;
   BEGIN
      pc_chk_exist_pdb (pdb_name, 'm', o_exist_status);
      IF o_exist_status = 0
      THEN
         RAISE e_not_exist;
      END IF;
      v_cmd_text :=
            'create or replace directory TRG_'
         || pdb_name
         || '_DBF as '''
         || trg_path
         || '''';
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
      FOR i
         IN (SELECT DISTINCT
                    SUBSTR (df.name, 1, INSTR (df.name, '/', -1) - 1)
                       dir_name
               FROM v$dbfile df, v$pdbs pd
              WHERE pd.name = pdb_name AND df.con_id = pd.con_id
             UNION
             SELECT DISTINCT
                    SUBSTR (df.name, 1, INSTR (df.name, '/', -1) - 1)
                       dir_name
               FROM v$tempfile df, v$pdbs pd
              WHERE pd.name = pdb_name AND df.con_id = pd.con_id)
      LOOP
         v_cmd_text :=
               'create or replace directory SRC_'
            || pdb_name
            || '_DBF_'
            || v_count
            || ' as '''
            || i.dir_name
            || '''';
         DBMS_OUTPUT.put_line (v_cmd_text);
         EXECUTE IMMEDIATE v_cmd_text;
         FOR z
            IN (SELECT SUBSTR (df.name, INSTR (df.name, '/', -1) + 1)
                          file_name
                  FROM v$dbfile df, v$pdbs pd
                 WHERE     pd.name = pdb_name
                       AND df.con_id = pd.con_id
                       AND SUBSTR (df.name, 1, INSTR (df.name, '/', -1) - 1) =
                              i.dir_name
                UNION
                SELECT SUBSTR (df.name, INSTR (df.name, '/', -1) + 1)
                          file_name
                  FROM v$tempfile df, v$pdbs pd
                 WHERE     pd.name = pdb_name
                       AND df.con_id = pd.con_id
                       AND SUBSTR (df.name, 1, INSTR (df.name, '/', -1) - 1) =
                              i.dir_name)
         LOOP
            v_cmd_text :=
                  'dbms_file_transfer.copy_file(''SRC_'
               || pdb_name
               || '_DBF_'
               || v_count
               || ''','''
               || z.file_name
               || ''',''TRG_'
               || pdb_name
               || '_DBF'','''
               || z.file_name
               || ''')';
            DBMS_OUTPUT.put_line (v_cmd_text);
            DBMS_FILE_TRANSFER.copy_file (
               'SRC_' || pdb_name || '_DBF_' || v_count,
               z.file_name,
               'TRG_' || pdb_name || '_DBF',
               z.file_name);
         END LOOP;
         v_cmd_text := 'drop directory SRC_' || pdb_name || '_DBF_' || v_count;
         DBMS_OUTPUT.put_line (v_cmd_text);
         EXECUTE IMMEDIATE v_cmd_text;
      END LOOP;
      v_cmd_text := 'drop directory TRG_' || pdb_name || '_DBF';
      DBMS_OUTPUT.put_line (v_cmd_text);
      EXECUTE IMMEDIATE v_cmd_text;
   EXCEPTION
      WHEN e_not_exist
      THEN
         raise_application_error (
            -20010,
            'PDB ' || pdb_name || ' does not exist or not in MOUNTED');
   --WHEN OTHERS
   --   THEN
   --dbms_output.put_line(SQLCODE||' '||SUBSTR (SQLERRM, 1, 64));
   --   raise_application_error (-20099,'Internal Error');
   END pc_cp_pdb_dbfs;
END pck_pdb_maint;
"""
