# BELONG TO SERVICE TASK
# To install - sudo, net-tools, which, wget, tar, rsync
# exit status usually is 1, for python errors = 9, for not supported distributer = 99
import os, re, json, time, stat, uuid, logging, string, secrets, pytz, subprocess
from io import StringIO
from datetime import datetime, timedelta
import paramiko
import psycopg2, oracledb, pyodbc, mariadb, mysql.connector
from enums import OperatingSystem, SupportedOS, DBType, ClusterRoles, RestoreOptions, Status, Severity, Inventory, Schedule, LicenseType, MonitorCommand, Repository,  Packages, PythonOS, Timeout
from dotenv import load_dotenv
load_dotenv()

logging.basicConfig(level=logging.INFO)
TASK_TMP_FILES_DIR = '/123cluster/tmp'
TASK_CERTIFICATE_FILES_DIR = '/123cluster/certs'
TASK_TMP_FILES_DIR_WIN = 'C:\\123Cluster\\tmp'

class FastTransport(paramiko.Transport):
    def __init__(self, sock):
        super(FastTransport, self).__init__(sock)
        self.default_window_size = 4294967294
        self.default_max_packet_size = 4294967294
        self.packetizer.REKEY_BYTES = 1099511627776
        self.packetizer.REKEY_PACKETS = 1099511627776
        self.use_compression(False)
        self._preferred_ciphers = ('aes256-ctr', )

def run_python_script(script_path, script_args):
    """
    Executes a Python script with the given arguments.

    Parameters:
        script_path (str): Path to the Python script.
        script_args (list): List of arguments to pass to the script.

    Returns:
         dict[str, Any]: dictionary with keys 'stdout', 'stderr', and 'excode'.
    """
    try:
        # Call the script and capture stdout, stderr
        import sys
        result = subprocess.run(
            [sys.executable, script_path] + script_args,
            text=True,
            capture_output=True
        )
        # Return stdout, stderr, and exit code in JSON format
        return {
            "stdout": result.stdout,
            "stderr": result.stderr,
            "excode": result.returncode
        }
    except Exception as e:
        # Handle unexpected exceptions
        return {
            "stdout": "",
            "stderr": str(e),
            "excode": -1
        }

def init_container(service_data, host_data, distributor, release, ram_gb, cpu_cores, disk_gb):
    """
    Create proxmox container wrapper.

    Parameters:
        service_data: Credentials of machine to create container on.
        host_data: Credentials of container to be created.
        ram_gb: Amount of RAM GB to be allocated for new container.
        cpu_cores: Number of CPU virtual cores to be allocated for new container.
        disk_gb: Amount of virtual disk space GB to be allocated for new container.



    Returns:
        dict[str, Any]: dictionary with keys 'stdout', 'stderr', and 'excode'.
    """

    try:
        os_template_mapping = {
            f'{OperatingSystem.CENTOS.value}8': 1,
            f'{OperatingSystem.UBUNTU.value}22': 2
        }
        args = [
                    service_data['host_name'],
                    service_data['ex_params'].get('proxmox_user'),
                    service_data['ex_params'].get('proxmox_token_name'),
                    service_data['ex_params'].get('proxmox_token_value'),
                    str(service_data['ex_params'].get('verify_ssl')),
                    str(os_template_mapping.get(f'{distributor}{release}')),
                    host_data['password'],
                    str(ram_gb),
                    str(cpu_cores),
                    str(disk_gb),
                    host_data['name']
                ]      
        
        
        if host_data['username'] != 'root':
            args += [
                host_data['username'],
                host_data['password']
            ]
        return run_python_script('init_container.py', args)

    except Exception as e:
        return {'stdout': '', 'stderr': str(e), 'excode': 9}

def write_to_local_file(filepath, content):
    """write data to the local machine of task service (os-independent)"""
    try:
        jsn = {'stderr': '', 'stdout': '', 'excode': 0}
        os.makedirs(os.path.dirname(filepath), exist_ok=True)
        with open(filepath, 'w') as file:
            file.write(content)
        return jsn
    except Exception as e:
        return {'stderr': str(e), 'stdout': '', 'excode': 0}

def transfer_ssl_certificates(source_data, host_source, host_target, target_dir=None, mode = 'clone'):
    """backup/restore/clone source ssl certificates to target location"""   
    try:
        global TASK_CERTIFICATE_FILES_DIR
        jsn = {'stdout': '', 'stderr': '', 'excode': 0}
        type = source_data['type']
        source_dir_123cluster = source_data['dir_123Cluster'].strip()
        source_dir = f"{source_dir_123cluster}/data" if DBType.MONGO.value in type else source_dir_123cluster
      
        source_hostname = host_source['name'].strip()
        target_hostname = host_target['name'].strip()
        if DBType.MYSQL.value in type or DBType.MARIADB.value in type:
            # to upgrade mysql/mariadb to verify-full ssl mode - need paid trusted certificates, 
            #for verify-ca - enough the same certificates for all nodes of cluster
            with get_postgres_connection() as conn:
                with conn.cursor() as cur:
                    # since all cluster nodes belong to same c123_cluster object - good candidate for common storage folder
                    cur.execute("""SELECT name from "clusterCore_c123_cluster" WHERE id =%s""", (source_data['cluster_id'],))
                    folder_name = cur.fetchone()[0]
        else:
            # by default - aim for verify-full (need personal certificates for every instance)
            folder_name = source_data['name'].strip()
        
        folder_name = folder_name.replace(':', '_')
        ssl_certificates_backup_folder = f"{TASK_CERTIFICATE_FILES_DIR}/{type}/{folder_name}"

        if not target_dir:
            if source_hostname == target_hostname:
                target_dir = source_dir
            else:
                target_dir = ssl_certificates_backup_folder
        
        if 'restore' not in mode:
            # backing up certificates from source directory to different location
            commands = [
                f"sudo mkdir -p '{ssl_certificates_backup_folder}'",
                f"sudo rsync -a '{source_dir}/' '{ssl_certificates_backup_folder}' --include='*.pem' --include='*.crt' --include='*.key' --exclude='*'"
            ]
            for cmd in commands:
                jsn = json.loads(remote_command(host_source, cmd, True), strict = False)
                if jsn['stderr'] != '':
                    return jsn


        if any(m in mode for m in ['clone', 'restore']):
            if source_hostname == target_hostname:
                jsn = json.loads(remote_command(host_source,  f"sudo rsync -a '{ssl_certificates_backup_folder}/' '{target_dir}' --include='*.pem' --include='*.crt' --include='*.key'   --exclude='*'", True), strict = False)
            else:
                jsn = transfer_files_rsync(host_source, host_target, target_dir, [f"{ssl_certificates_backup_folder}/"])
        
        return jsn
    
    except Exception as e:
        return {'stdout': '', 'stderr': '', 'excode': 9}
        
def get_postgres_connection():
    """return a postgres connection object"""
    
    while True:
        try:
            return psycopg2.connect(
                database=os.getenv('DB_NAME', '123Cluster'),
                user=os.getenv('DB_USER', 'postgres'),
                password=os.getenv('DB_PASSWORD', 'password'),
                host=os.getenv('DB_HOST', 'localhost'),
                port=os.getenv('DB_PORT', '5432'),
                connect_timeout=Timeout.SERVICE_POSTG.value
            )
        except psycopg2.Error as e:
            logging.error(f'get_postgres_connection: {str(e)}')
            time.sleep(1)

def get_mysql_connection(host, node, database='mysql'):
    """return a mysql/mariadb connection object"""

    try:
        if DBType.MARIADB.name in node['type']:
            ssl = node['ex_params'].get('ssl')
            client = mariadb.connect(host=host['name'], port=int(node['port']), user=node['database_user'], password=node['database_pwd'], database=database, connect_timeout=Timeout.SERVICE_MARIADB.value, ssl=ssl)
        else:
            client =  mysql.connector.connect(host=host['name'], port=int(node['port']), user=node['database_user'], password=node['database_pwd'], database=database, connection_timeout=Timeout.SERVICE_MYSQL.value)
        return client
    except Exception as e:
        logging.error(f'get_mysql_connection {str(e)}')
        return (False, e)
    


def log_rotate_task(data, task_id):
    """rotate database logs hourly"""

    task = 'log_rotate_task'

    try:
        created_by = int(data['created_by'])
    except Exception as error:
        logging.error(f"{task}: {str(error)}")
        return
    
    task_log_id = uuid.uuid4().hex
    step = 'null'
    status = Severity.OK.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    duration_task_start = datetime.now()

    step = 'initializing task'
    status = Severity.INFO.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)

    step = 'checking parameters'
    duration_step_start = time.perf_counter()

    try:
        host = json.loads(data['host_data'])
        node_name = data['node_name']
    except Exception as error:
        status = Severity.NOT_OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,str(error),9)
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),9)
        return

    status = Severity.OK.value
    duration_step_end = time.perf_counter() - duration_step_start
    logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)

    step = 'rotating logs'
    duration_step_start = time.perf_counter()

    try:
        command = f"sudo logrotate -f '/etc/logrotate.d/{node_name}'"
        jsn = json.loads(remote_command(host, command, True), strict=False)
        if jsn['stderr'] != '':
            status = Severity.NOT_OK.value
            duration_step_end = time.perf_counter() - duration_step_start
            logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
            step = 'null'
            status = Severity.NOT_OK.value
            duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
            logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
            return
    except Exception as error:
        status = Severity.NOT_OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,str(error),9)
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),9)
        return

    status = Severity.OK.value
    duration_step_end = time.perf_counter() - duration_step_start
    logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
    
    step = 'task finished'
    status = Severity.OK.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    step = 'null'
    status = Severity.OK.value
    duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
    logger(task_id,task_log_id,task,step,status,duration_task_end,created_by)

def insert_task(params):
    try:
        params['data'] = json.dumps(params['data'])
        conn = get_postgres_connection()
        cur = conn.cursor()
        cur.callproc('insert_task', (params['schedule'],params['data'],params['task'],params['target_type'],params['target_id'], params['org_id'], params['created_by'], params.get('monitor_name')))
        conn.commit()
        cur.close()
        conn.close()
    except Exception as error:
        return str(error)

def remote_command(data, cmd, wait_for_output=True, task_step = None) -> str:
    
    logging.info('remote_command command: %s', str(cmd))
    json_str = {}
    try:
        aws_username = data['ex_params'].get('aws_username')
        aws_password = data['ex_params'].get('aws_password')
        aws_region = data['ex_params'].get('aws_region')
        aws_instance = data['ex_params'].get('aws_instance')
        if not aws_username:   
            client = create_ssh_connection(hostname=data['name'], user=data['username'], pwd=data['password'], pkey=data['private_key'], prt=int(data['ssh_port']))
            if isinstance(client, tuple):
                logging.info('remote_command: %s', str(client[1]))
                json_str['stdout'] = ''
                json_str['stderr'] = str(client[1])
                json_str['excode'] = 1
                return json.dumps(json_str)
            with client:
                _, stdout, stderr = client.exec_command(cmd)
                if wait_for_output:
                    try:
                        strout = stdout.read().decode('ascii').strip()
                        strerror = stderr.read().decode('ascii').strip()
                    except UnicodeDecodeError:
                        strout = stdout.read().decode('utf-8').strip()
                        strerror = stderr.read().decode('utf-8').strip()
                    json_str['stdout'] = strout
                    json_str['stderr'] = strerror
                    json_str['excode'] = stdout.channel.recv_exit_status()
                    if strout:
                        logging.info(f"{data['username']} - remote_command{f'({task_step})' if task_step else ''} output: {strout}")
                    if strerror:
                        logging.info(f"{data['username']} - remote_command{f'({task_step})' if task_step else ''} stderr: {strerror}")
                    return json.dumps(json_str)
                else:
                    json_str['stdout'] = ''
                    json_str['stderr'] = ''
                    json_str['excode'] = ''
                return json.dumps(json_str)
        else:
            from .AWS import AWSController
            controller = AWSController(aws_username, aws_password, aws_region)
            json_str = controller.runSSMCommand(aws_instance, cmd)
            if json_str['stdout']:
                logging.info(f"{aws_username} - remote_command{f'({task_step})' if task_step else ''} output: {json_str['stdout']}")
            if json_str['stderr']:
                 logging.info(f"{aws_username} - remote_command{f'({task_step})' if task_step else ''} stderr: {json_str['stderr']}")
            return json.dumps(json_str)
            


    except Exception as e:
        logging.error('%s - remote_command exception: %s', str(data['username']), str(e))
        json_str['stderr'] = e.strerror if hasattr(e, 'strerror') else str(e)
        json_str['stdout'] = ''
        json_str['excode'] = 1
        return json.dumps(json_str)

def create_ssh_connection(hostname, user, pwd, pkey, prt=22):
    """ssh connection creation helper function"""
    
    try:
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        if pkey and isinstance(pkey, str) and not pkey.isspace():
            pkey = pkey.strip()
            private_key = paramiko.RSAKey(file_obj=StringIO(pkey))
            client.connect(hostname=hostname, port=prt, username=user, pkey=private_key, look_for_keys=False, auth_timeout=Timeout.SERVICE_SSH_AUTH.value, banner_timeout=Timeout.SERVICE_SSH_BANNER.value, timeout=Timeout.SERVICE_SSH_TCP.value)
        elif pwd and isinstance(pwd, str) and not pwd.isspace():
            pwd = pwd.strip()
            client.connect(hostname=hostname, port=prt, username=user, password=pwd, banner_timeout=Timeout.SERVICE_SSH_BANNER.value, timeout=Timeout.SERVICE_SSH_TCP.value)
        else:
            return (False, f'cannot connect to {hostname}')
        
        return client
    except (paramiko.ssh_exception.NoValidConnectionsError, EOFError) as e:
        logging.error(f"create_ssh_connection: {e}")
        return (False, e)
    except paramiko.ssh_exception.SSHException as e:
        logging.error(f"create_ssh_connection: {e}")
        # socket is open, but not SSH service responded:
        return (False, e)
    except Exception as e:
        logging.error(f"create_ssh_connection: {e}")
        return (False, e)
    
def remote_command_win(host, cmd, wait_for_output, shell='cmd') -> dict:
    jsn = {}
    print(cmd)
    try:
        client = create_winrm_connection(host['name'], host['username'], host['password'], host['ssh_port'])
        if isinstance(client, tuple):
            jsn['stdout'] = ''
            jsn['stderr'] = client[1].strerror if hasattr(client[1], 'strerror') else str(client[1])
            jsn['excode'] = 1
            return jsn
    
        if shell == 'ps':
            res = client.run_ps(cmd)
        else:
            res = client.run_cmd(cmd)
      
        if wait_for_output:
            jsn['stdout'] = res.std_out.decode().strip()
            jsn['stderr'] = res.std_err.decode().strip()
            jsn['excode'] = res.status_code
    
        return json.dumps(jsn)
    except Exception as e: 
        jsn['stdout'] = ''
        jsn['stderr'] = e.strerror if hasattr(e, 'strerror') else str(e)
        jsn['excode'] = 1

    return jsn

def create_winrm_connection(hostname, user, pwd, prt=5985, protocol='http', transport='ntlm', endpoint='wsman', server_cert_validation='ignore'):
    """windows remote connection creation helper function"""
    
    try:
        from winrm import Session
        s = Session(target = f'{protocol}://{hostname}:{prt}/{endpoint}', auth=(user, pwd), **{'transport': transport, 'server_cert_validation': server_cert_validation})
    except Exception as error:
        logging.error(f"create_winrm_connection: {str(error)}")
        return (False, error)
    
    return s

def sql_file_executor(file_path, conn):
    """
    Executes a SQL file efficiently.
    
    Parameters:
        file_path (str): Path to the SQL file.
        conn (psycopg2.extensions.connection): Active PostgreSQL connection.
    """

    try:
    
        # Enable autocommit to prevent transaction blocking for DDL statements
        conn.autocommit = True
        
        with conn.cursor() as cur, open(file_path, "r", encoding="utf-8") as f:
            sql_script = f.read()
            cur.execute(sql_script)
            return ''
        
    except Exception as e:
        return f"Error executing SQL file {file_path}:\n{e}"   

def logger(task_id,task_log_id,task,step,status,duration,created_by,stdout=None,stderr=None,excode=None,ext_field=None):
    try:
        if ext_field is None:
            ext_field = "{}"
        elif isinstance(ext_field, dict):
            ext_field = json.dumps(ext_field)
        else:
            ext_field = json.dumps({"non_serializable": str(ext_field)})
        conn = get_postgres_connection()
        cur = conn.cursor()
        cur.callproc('logger', (task_id, task_log_id, task, status, duration, created_by, step, stdout, stderr, excode, ext_field))
        conn.commit()
        cur.close()
        conn.close()
    except Exception as error:
        logging.error(f"logger: {str(error)}")

def find_tilda(host) -> dict:
    """find location of ~"""
    
    try:
        # jsn = json.loads(remote_command(host, "echo ~", True), strict=False)
        jsn = json.loads(remote_command(host, "pwd | tr -d '\\n'", True), strict=False)
        if jsn['stderr'] != '':
            logging.error(f"find_tilda: {jsn['stderr']}")
    except Exception as error:
        logging.error(f"find_tilda: {str(error)}")
        jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
    
    return jsn



def find_local_tilda() -> dict:
    """find location of ~"""
    
    try:
        # jsn = json.loads(remote_command(host, "echo ~", True), strict=False)
        jsn = json.loads(local_command("pwd | tr -d '\\n'", True), strict=False)
        if jsn['stderr'] != '':
            logging.error(f"find_tilda: {jsn['stderr']}")
    except Exception as error:
        logging.error(f"find_tilda: {str(error)}")
        jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
    
    return jsn

def format_for_shell(string):
    if not string:
        return ""
    
    trans_table = str.maketrans({"\'": "\\\'", "`": "\\`", "!": "\\!"})
    
    return string.translate(trans_table)

def is_ram_enough(host, required_ram, use_gb=False):
    try:
        required_ram = int(required_ram)
        
        # Available RAM of remote machine in MB
        jsn = json.loads(remote_command(host, "free -m | awk 'NR==2{print $7}'", True), strict=False)
        if jsn['stderr'] != '':
            return (True, jsn['stderr'])
        
        available_ram = int(jsn["stdout"])

        if use_gb:
            available_ram /= 1024
        
        return (False, available_ram >= required_ram)
    
    except Exception as e:
        return (True, str(e))

def check_service_status(host, service_name) -> tuple:
    """check if service is active/inactive"""

    try:
        command = f"systemctl is-active {service_name}"
        jsn = json.loads(remote_command(host, command, True), strict=False)
        if jsn['stderr'] != '':
            logging.error(f"check_service_status: {jsn['stderr']}")

        status = jsn['stdout'].strip()
        return (status != 'active', f'service status is: {status}')
    except Exception as error:
        logging.error(f"check_service_status: {str(error)}")
        return (True, str(error))

def run_linux_performance_counters(host, task_id):
    """
    for centos:
    yum install sysstat -y
    systemctl start sysstat.service
    systemctl enable sysstat.service
    """
    
    task = 'run linux performance counters'
    task_log_id = uuid.uuid4().hex
    step = 'null'
    status = Severity.OK.value
    duration = 0 
    logger(task_id,task_log_id,task,step,status,duration,host['create_task'])
    duration_task_start = datetime.now()
    
    step = 'initializing task'
    status = Severity.INFO.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,host['create_task'])
    
    step = 'checking parameters'
    duration_step_start = time.perf_counter()
    
    file_content = '''#!/usr/bin/bash 
#sar 1 1 | grep Average | awk '{print "{\\"cpuusage\\":"u$3+$5",\\"iowait\\":"0+$6",\\"memusage\\":56,\\"memtotal\\":12,\\"cpucount\\":8}"}' 
mem=`free |grep Mem|awk '{print "\\"memusage\\":"$3 ",\\"memtotal\\":"$2}'` 
swap=`free |grep Swap|awk '{print "\\"swapusage\\":"$3 ",\\"swaptotal\\":"$2}'` 
cu=`sar 1 1 | grep Average | awk '{print "\\"cpuusage\\":"u$3+$5",\\"iowait\\":"0+$6""}'` 
cpuc=\\"cpucount\\":`cat /proc/cpuinfo |grep processor|wc -l` 
info=`echo {$cu"," $mem"," $cpuc"," $swap}`
echo $info'''
    
    status = Severity.OK.value
    duration_step_end = time.perf_counter() - duration_step_start
    logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'])
    
    # cur = conn.cursor(dictionary=True)
    # step = 'select query'
    # duration_step_start = time.perf_counter()
    # try:
    #     cur.execute(f"SELECT id,name,username,password,ssh_port from clusterCore_c123_host where id={host['id']}")
    #     for record in cur:
    #         host_data = {'id':record['id'],'host':record['name'],'user':record['username'],'pwd':record['password'],'ssh_port':str(record['ssh_port'])}
    #     # res = cur.fetchall()
    # except Exception as error:
    #     status = Severity.NOT_OK.value
    #     duration_step_end = time.perf_counter() - duration_step_start
    #     logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'])
    #     step = 'null'
    #     status = Severity.NOT_OK.value
    #     duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
    #     logger(task_id,task_log_id,task,step,status,duration_task_end,host['create_task'])
    #     return
    # host_data = {'host':res[0][1],'user':res[0][2],'pwd':res[0][3],'ssh_port':str(res[0][4])}
    
    # status = Severity.OK.value
    # duration_step_end = time.perf_counter() - duration_step_start
    # logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'])
    
    step = 'run linux_performance_counters'
    duration_step_start = time.perf_counter()
    
    try:
        jsn = json.loads(remote_command(host, file_content, True), strict=False)
        if jsn['stderr'] != '':
            status = Severity.NOT_OK.value
            duration_step_end = time.perf_counter() - duration_step_start
            logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'],jsn['stdout'],jsn['stderr'],jsn['excode'])
            step = 'null'
            status = Severity.NOT_OK.value
            duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
            logger(task_id,task_log_id,task,step,status,duration_task_end,host['create_task'],jsn['stdout'],jsn['stderr'],jsn['excode'])
            return
    except Exception as error:
        status = Severity.NOT_OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'],None,str(error),9)
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,host['create_task'],None,str(error),9)
        return
    
    status = Severity.OK.value
    duration_step_end = time.perf_counter() - duration_step_start
    logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'])
    
    step = 'calling to postgres func: insert_linux_performance_counters'
    duration_step_start = time.perf_counter()
    
    try:
        conn = get_postgres_connection()
        cur = conn.cursor()
        cur.callproc('insert_linux_performance_counters', (host['id'],jsn['stdout']))
        conn.commit()
    except Exception as error:
        logging.error('run_linux_performance_counters: %s', str(error))
        status = Severity.NOT_OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'],None,str(error),1)
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,host['create_task'],None,str(error),1)
        return
    else:
        cur.close()
        conn.close()
    
    status = Severity.OK.value
    duration_step_end = time.perf_counter() - duration_step_start
    logger(task_id,task_log_id,task,step,status,duration_step_end,host['create_task'])
    
    step = 'task finished'
    status = Severity.OK.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,host['create_task'])
    step = 'null'
    status = Severity.OK.value
    duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
    logger(task_id,task_log_id,task,step,status,duration_task_end,host['create_task'])

def mkdir_p(sftp, remote_path):
    """mkdir_p simulated linux 'mkdir -p path' command"""
    
    try:
        if remote_path == "/":
            # absolute path so change directory to root
            sftp.chdir("/")
            return

        if remote_path == "":
            # top-level relative directory must exists
            return

        try:
            # sub-directory exists
            sftp.chdir(remote_path)
        except IOError:
            dirname, basename = os.path.split(remote_path.rstrip("/"))
            mkdir_p(sftp, dirname)
            sftp.mkdir(basename)
            sftp.chdir(basename)
    except Exception as error:
        logging.error(f"mkdir_p: {str(error)}")
        return

def check_remote_directory_exists(host, remote_path, dir_or_file, is_dir_empty=False) -> dict:
    """return 'true' if directory exists (or empty if is_dir_empty are checked) else 'false'
    dir_or_file='f' if file otherwise 'd' if directory"""
    
    if dir_or_file == 'f':
        command = f"sudo test -f '{remote_path}' && echo 'true' || echo 'false'"
    elif dir_or_file == 'd':
        command = f"sudo test -d '{remote_path}' && echo 'true' || echo 'false'"
        if is_dir_empty:
            # 'true' if empty else 'false'
            command = f"""sudo [ -z $(find '{remote_path}' -mindepth 1 -maxdepth 1 -print -quit) ] && echo 'true' || echo 'false'"""
    else:
        command = "echo 'no -d or -f specified'"

    try:
        jsn = json.loads(remote_command(host, command, True), strict=False)
        jsn['stdout'] = str(jsn['stdout'])
    except Exception as error:
        jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
    return jsn

def check_local_directory_exists(local_path, dir_or_file, is_dir_empty=False) -> dict:
    """return 'true' if directory exists (or empty if is_dir_empty are checked) else 'false'
    dir_or_file='f' if file otherwise 'd' if directory"""
    
    if dir_or_file == 'f':
        command = f"sudo test -f '{local_path}' && echo 'true' || echo 'false'"
    elif dir_or_file == 'd':
        command = f"sudo test -d '{local_path}' && echo 'true' || echo 'false'"
        if is_dir_empty:
            # 'true' if empty else 'false'
            command = f"""sudo [ -z $(find '{local_path}' -mindepth 1 -maxdepth 1 -print -quit) ] && echo 'true' || echo 'false'"""
    else:
        command = "echo 'no -d or -f specified'"

    try:
        jsn = json.loads(local_command(command, True), strict=False)
        jsn['stdout'] = str(jsn['stdout'])
    except Exception as error:
        jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
    return jsn

def check_port_existence(host, port) -> dict:
    """
    check if the port is not in use,
    return False if not in use otherwise True
    """
    try:
        jsn = json.loads(remote_command(host, f"sudo netstat -tunlp|grep -w '{port}'", True), strict=False)
    except Exception as error:
        jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
    
    return jsn

# def check_os_version(host) -> tuple:
#     """return distributor and release version"""
    
#     try:
#         jsn = json.loads(remote_command(host, f"if [ -f '/etc/centos-release' ]; then echo '{OperatingSystem.CENTOS.value}'; elif [ -f '/etc/lsb-release' ]; then echo '{OperatingSystem.UBUNTU.value}'; else echo 'unknown'; fi", True), strict=False)
#         if jsn['stderr'] != '':
#             return ('error', jsn)

#         centos_dist = True if OperatingSystem.CENTOS.value in jsn['stdout'] else False
#         distributor = OperatingSystem.CENTOS.value if centos_dist else OperatingSystem.UBUNTU.value
#         if centos_dist:
#             command = "sudo cat '/etc/redhat-release'"
#             jsn = json.loads(remote_command(host, command, True), strict=False)
#             if jsn['stderr'] != '':
#                 return ('error', jsn)
            
#             release = jsn['stdout'].strip().split()[3].split('.')[0]
#         else:
#             command = "sudo lsb_release -r | awk '{print $2}'"
#             jsn = json.loads(remote_command(host, command, True), strict=False)
#             error_list = ('no lsb modules are available', 'warning')
#             if jsn['stderr'] != '' and not any(error in jsn['stderr'].lower() for error in error_list):
#                 return ('error', jsn)

#             release = jsn['stdout'].strip().split('.')[0]
#     except Exception as error:
#         jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
#         return ('error', jsn)
    
#     return (distributor, release)


def check_local_os_version() -> tuple:
    """return distributor and release version"""

    try:
        jsn = json.loads(local_command("source /etc/os-release; echo $ID", True), strict=False)
        if jsn['stderr'] != '':
            return ('error', jsn)

        distributor = jsn['stdout'].strip()

        if any(v in distributor for v in [OperatingSystem.UBUNTU.value, OperatingSystem.DEBIAN.value]):
            distributor = OperatingSystem.UBUNTU.value
        elif any(v in distributor for v in [OperatingSystem.CENTOS.value, OperatingSystem.REDHAT.value, OperatingSystem.FEDORA.value, OperatingSystem.ROCKY.value]):
            distributor = OperatingSystem.CENTOS.value 
        else:
            return (OperatingSystem.UNKNOWN.value, {'stdout': '', 'stderr': f'Unsupported OS: {distributor}', 'excode': 1})

        jsn = json.loads(local_command("source /etc/os-release; echo $VERSION_ID", True), strict=False)
        if jsn['stderr'] != '':
            return (OperatingSystem.INVALID.value, jsn)

        release = jsn['stdout'].strip().split('.')[0]
    except Exception as error:
        jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
        return (OperatingSystem.INVALID.value, jsn)

    return (distributor, release)

def check_os_version(host) -> tuple:
    """return distributor and release version"""

    try:
        jsn = json.loads(remote_command(host, "source /etc/os-release; echo $ID", True), strict=False)
        if jsn['stderr'] != '':
            return ('error', jsn)

        distributor = jsn['stdout'].strip()

        if any(v in distributor for v in [OperatingSystem.UBUNTU.value, OperatingSystem.DEBIAN.value]):
            distributor = OperatingSystem.UBUNTU.value
        elif any(v in distributor for v in [OperatingSystem.CENTOS.value, OperatingSystem.REDHAT.value, OperatingSystem.FEDORA.value, OperatingSystem.ROCKY.value]):
            distributor = OperatingSystem.CENTOS.value 
        else:
            return (OperatingSystem.UNKNOWN.value, {'stdout': '', 'stderr': f'Unsupported OS: {distributor}', 'excode': 1})

        jsn = json.loads(remote_command(host,"source /etc/os-release; echo $VERSION_ID", True), strict=False)
        if jsn['stderr'] != '':
            return (OperatingSystem.INVALID.value, jsn)

        release = jsn['stdout'].strip().split('.')[0]
    except Exception as error:
        jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
        return (OperatingSystem.INVALID.value, jsn)

    return (distributor, release)

def local_command(command, wait=True, task_step = None):
    """
    Runs a shell command on the local machine and returns JSON-encoded dictionary.
    
    :param command: The shell command to execute (as a string or list).
    :param wait: Whether to wait for command completion.
    :return: JSON string with stdout, stderr, and excode.
    """
    try:
        logging.info('local_command command: %s', str(command))
        env = os.environ.copy()
        # Ensure essential system paths are included
        env["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:" + env.get("PATH", "")
        
        process = subprocess.Popen(
            command, shell=True, executable="/bin/bash", stdout=subprocess.PIPE,
            stderr=subprocess.PIPE, text=True, env=env
        )
        
        if wait:
            stdout, stderr = process.communicate()
            stdout = stdout.encode("ascii", "ignore").decode("ascii").strip()
            stderr = stderr.encode("ascii", "ignore").decode("ascii").strip()
            if stdout:
                logging.info(f"local_command{f'({task_step})' if task_step else ''} output: {stdout}")
            if stderr:
                logging.info(f"local_command{f'({task_step})' if task_step else ''} stderr: {stderr}")
            excode = process.returncode
        else:
            stdout, stderr, excode = "", "", None
    except Exception as e:
        logging.error('local_command exception: %s', str(e))
        stdout, stderr, excode = "", str(e), 9
    
    result = {
        "stdout": stdout,
        "stderr": stderr,
        "excode": excode
    }
    
    return json.dumps(result)

def generate_complex_password(length=20, *args):
    try:
        # Define characters to use for password generation
        characters = string.ascii_letters + string.digits + r'@$!%*?&'
        # Ensure the password contains at least one uppercase letter, one lowercase letter, one digit, and one special character
        password_chars = secrets.choice(string.ascii_lowercase) + \
                         secrets.choice(string.ascii_uppercase) + \
                         secrets.choice(string.digits) + \
                         secrets.choice(r'@$!%*?&') + \
                         ''.join(secrets.choice(characters) for _ in range(length - 4))
        # Shuffle the characters to randomize the password
        password_list = list(password_chars)
        secrets.SystemRandom().shuffle(password_list)
        # Generate the final password string
        password = ''.join(password_list)

        # Check if the generated password meets the regex patterns for MySQL, and MSSQL
        if DBType.MYSQL.value in args:
            mysql_password_regex = r'^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$'
            if not (re.match(mysql_password_regex, password)):
                return generate_complex_password(length, DBType.MYSQL.value)
        elif DBType.MSSQL.value in args:
            mssql_password_regex = r'^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$'
            if not (re.match(mssql_password_regex, password)):
                return generate_complex_password(length, DBType.MSSQL.value)
    except RecursionError as error:
        logging.error(f"generate_complex_password: {str(error)}")
        params = args
        params[0] = 8
        return generate_complex_password(*params)
    except Exception as error:
        logging.error(f"generate_complex_password: {str(error)}")
        return ''
    
    return password

def transfer_files(host, local_dir, remote_dir, exclude_files=None):
    """transfer files from local (linux or win) to remote machine (linux)"""
    
    exclude_files = exclude_files or []
    is_windows = os.name == PythonOS.WINDOWS.value
    try:
        fast_transport = FastTransport((host['name'], int(host['ssh_port'])))
        # Empty private key
        if not host['private_key']:
            fast_transport.connect(username=host['username'], password=host['password'])
        else:
            fast_transport.connect(username=host['username'], pkey=paramiko.RSAKey.from_private_key(StringIO(host['private_key'])))
        # Transfer files with the .rpm or .deb suffix to the remote server
        with paramiko.SFTPClient.from_transport(fast_transport) as sftp_client:
            for file in os.listdir(local_dir):
                if file.endswith((".rpm", ".deb", ".whl")) and file not in exclude_files:
                    local_path = os.path.join(local_dir, file)
                    remote_path = os.path.join(remote_dir, file)
                    if is_windows:
                        remote_path = remote_path.replace(os.sep, os.altsep)
                    
                    sftp_client.put(local_path, remote_path)
        
        # Close the SSH session
        fast_transport.close()
        return None
    except Exception as error:
        return str(error)

def install_offline_remote_to_remote(distributor, release, host, node_version, package, app_host, *args) -> dict:
    "install packages from local repository of app machine to target machine"
    # we using lowercase for directories, so for insurance
    package_dir = package.lower()
    if OperatingSystem.CENTOS.value in distributor:
        package_manager = 'dnf' if int(release) else 'yum'
    commands = []
    is_windows = os.name == PythonOS.WINDOWS.value
    #since we dont have "production" standard for project files location - search whole drive
    jsn = json.loads(remote_command(app_host, "sudo find / -type d -name 'rpms' 2>/dev/null | tail -n 1", True), strict=False)
    if jsn['stderr'] != '':
        return jsn
    if jsn['stdout'] == '':
        jsn['stderr'] == 'Could not determine local repository location'
        return jsn
    
    local_rpms_directory = jsn['stdout']
    # example: distributor_folder = 'centos7'
    distributor_folder = f"{distributor}{release}"
    extra_packages = list(args)

    remote_dir_common = os.path.join(local_rpms_directory, distributor_folder, package_dir, Packages.COMMON.value)
    remote_dir_base = os.path.join(local_rpms_directory, distributor_folder, Packages.BASE.value)

    if Packages.REPMGR.value in extra_packages:
        remote_dir_common = os.path.join(remote_dir_common, Packages.REPMGR.value)
    
    if is_windows:
        remote_dir_common = remote_dir_common.replace(os.sep, os.altsep)
        remote_dir_base = remote_dir_base.replace(os.sep, os.altsep)
    
    if local_rpms_directory.startswith('/root'):
        target_dir_common = remote_dir_common.replace('/root', TASK_TMP_FILES_DIR)
        target_dir_base = remote_dir_base.replace('/root', TASK_TMP_FILES_DIR)
    else:
        target_dir_common = remote_dir_common
        target_dir_base = remote_dir_base
    
    #version-specific packages go first
    if node_version:
        remote_dir_version = os.path.join(local_rpms_directory, distributor_folder, package_dir, str(node_version))
        
        if Packages.REPMGR.value in extra_packages:
            remote_dir_version = os.path.join(remote_dir_version, Packages.REPMGR.value)
        
        if is_windows:
            remote_dir_version = remote_dir_version.replace(os.sep, os.altsep)
        
        if local_rpms_directory.startswith('/root'):
            target_dir_version = remote_dir_version.replace('/root', TASK_TMP_FILES_DIR)
        else:
            target_dir_version = remote_dir_version
        
        # transfering base os rpm or deb files to target machine
        jsn = transfer_files_rsync(app_host, host,  target_dir_base, [f"{remote_dir_base}/"])
        if jsn['stderr'] != '':
            return jsn
        
        jsn = transfer_files_rsync(app_host, host,  target_dir_common, [f"{remote_dir_common}/"])
        if jsn['stderr'] != '':
            return jsn
        
        jsn = transfer_files_rsync(app_host, host, target_dir_version, [f"{remote_dir_version}/"])
        if jsn['stderr'] != '':
            return jsn
        
        # installing rpm or deb files in remote machine
        if OperatingSystem.UBUNTU.value in distributor:
            if package in [DBType.MYSQL.value, DBType.MARIADB.value]:
                commands = [
                    f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confnew {target_dir_version}/*.deb'
                ]
                
                if not (package == DBType.MYSQL.value and (float(node_version) < 8.0 or (float(node_version) >= 8.0 and int(release) == 20))):
                    commands.insert(0,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confnew {target_dir_version}/*-common*.deb 2>&1')

                    
                if package == DBType.MYSQL.value:
                    commands = [
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_common}/*.deb',
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_version}/lib*.deb',
                    ] + commands
                    
                    commands.insert(1,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_version}/libmysqlclient{"20" if float(node_version) < 8.0 else "21"}*.deb')
                    if float(node_version) >= 8.0:
                        commands.insert(1,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_version}/*-plugins*.deb')
                    
                    commands.insert(1,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_version}/*-common*.deb')
                    
                    commands = [
                        "sudo rm -f '/etc/apparmor.d/disable/usr.sbin.mysqld'",
                        "sudo ln -s '/etc/apparmor.d/usr.sbin.mysqld /etc/apparmor.d/disable/'"
                    ] + commands
                else:
                    commands = [
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg --ignore-depends=gawk{",mysql-common" if int(release) == 22 else ""} -i {target_dir_common}/*.deb',
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_common}/gawk*.deb',
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confnew {target_dir_version}/devel/*.deb 2>&1'

                    ] + commands
            else:
                commands = [
                    f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_common}/*.deb',
                    f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confdef {target_dir_version}/*.deb'
                ]
                
                if package == DBType.POSTGRES.value and Packages.REPMGR.value not in extra_packages:
                    commands.append('sudo systemctl stop postgresql && sudo systemctl disable postgresql')
            
            commands.insert(0,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_base}/*.deb')
            if package == DBType.REDIS.value:
                commands += [
                    'sudo systemctl stop redis-server',
                    'sudo systemctl disable redis-server'
                ]
        elif OperatingSystem.CENTOS.value in distributor:
            commands = [
                f'sudo {package_manager} localinstall -q -y {target_dir_base}/*.rpm --disablerepo=* --cacheonly',
                f'''sudo {package_manager} localinstall -q -y {f"$(sudo find {target_dir_common} type f ! -name '*database-tools*')" if DBType.MONGO.value in package and float(node_version) < 4.4 else f"{target_dir_common}/*.rpm"}  --disablerepo=* --cacheonly''',
                f'sudo {package_manager} localinstall -q -y {target_dir_version}/*.rpm --disablerepo=* --cacheonly'
            ]
            if DBType.MYSQL.value in package and int(release) == 8:
                commands += [ 
                    f"sudo sed -i '/^KillSignal.*/a ExecStopPost=+systemctl daemon-reload' '/usr/lib/systemd/system/mysqld.service'",
                    "sudo systemctl daemon-reload"
                ]
            if DBType.MARIADB.value in package:
                commands.insert(2, f'sudo {package_manager} localinstall -q -y {target_dir_version}/devel/*.rpm --disablerepo=* --cacheonly')
                
        else:
            jsn = {
                'stderr': 'Distributor is not Ubuntu or CentOS',
                'stdout': None,
                'excode': 99
            }
            return jsn
    
    if Packages.HAPROXY.value in extra_packages:
        remote_dir_haproxy = os.path.join(remote_dir_common, Packages.HAPROXY.value)
        if is_windows:
            remote_dir_haproxy = remote_dir_haproxy.replace(os.sep, os.altsep)
        
        if local_rpms_directory.startswith('/root'):
            target_dir_haproxy = remote_dir_haproxy.replace('/root', TASK_TMP_FILES_DIR)
        else:
            target_dir_haproxy = remote_dir_haproxy
        
        jsn = transfer_files_rsync(app_host, host, target_dir_haproxy, [f"{remote_dir_haproxy}/"])
        if jsn['stderr'] != '':
            return jsn
        
        if OperatingSystem.UBUNTU.value in distributor:
            commands.append(f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {target_dir_haproxy}/*.deb')
        elif OperatingSystem.CENTOS.value in distributor:
            commands.append(f'sudo {package_manager} localinstall -q -y {target_dir_haproxy}/*.rpm  --disablerepo=* --cacheonly')
    
    if package == DBType.POSTGRES.value and Packages.PYTHON3.value in extra_packages:
        remote_dir_python3 = os.path.join(remote_dir_common, Packages.PYTHON3.value)
        remote_dir_cherrypy = os.path.join(remote_dir_common, Packages.CHERRYPY.value)
        if is_windows:
            remote_dir_python3 = remote_dir_python3.replace(os.sep, os.altsep)
            remote_dir_cherrypy = remote_dir_cherrypy.replace(os.sep, os.altsep)
        
        if local_rpms_directory.startswith('/root'):
            target_dir_python3 = remote_dir_python3.replace('/root', TASK_TMP_FILES_DIR)
            target_dir_cherrypy = remote_dir_cherrypy.replace('/root', TASK_TMP_FILES_DIR)
        else:
            target_dir_python3 = remote_dir_python3
            target_dir_cherrypy = remote_dir_cherrypy
        
        jsn = transfer_files_rsync(app_host, host, target_dir_python3, [f"{remote_dir_python3}/"])
        if jsn['stderr'] != '':
            return jsn
        
        jsn = transfer_files_rsync(app_host, host, target_dir_cherrypy, [f"{remote_dir_cherrypy}/"])
        if jsn['stderr'] != '':
            return jsn
        
        if OperatingSystem.UBUNTU.value in distributor:
            commands.append(f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {"--ignore-depends=python3,python3-minimal" if int(release) == 22 else ""} {target_dir_python3}/*.deb')
        elif OperatingSystem.CENTOS.value in distributor:
            commands.append(f'sudo {package_manager} localinstall -q -y {target_dir_python3}/*.rpm  --disablerepo=* --cacheonly')
        
        commands.append(f"""sudo python3 -m pip install {"--no-warn-script-location" if OperatingSystem.CENTOS.value in distributor and int(release) > 8 else ""} {target_dir_cherrypy}/*.whl""")
    
    for command in commands:
        try:
            jsn = json.loads(remote_command(host, command, True), strict=False)
            error_lower = jsn['stderr'].lower()
            filter = error_lower != '' and ('E:' in jsn['stderr'] or any(err in error_lower for err in ['err', 'error', 'fail', 'errno'])) and not any(err in error_lower for err in ['already installed', 'nothing to do', 'file not found', 'no such file', 'file exists'])
            if filter:
                return jsn
            else:
                jsn['stderr'] = ''
        except Exception as error:
            logging.error(f"install_offline: {str(error)}")
            jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
            return jsn
    
    return jsn

def install_offline(distributor, release, host, node_version, package, *args) -> dict:
    is_windows = os.name == PythonOS.WINDOWS.value
    # we using lowercase for directories, so for insurance
    package_dir = package.lower()
    if OperatingSystem.CENTOS.value in distributor:
        package_manager = 'dnf' if int(release) else 'yum'
    commands = []
    local_rpms_directory = os.path.join(os.getcwd(), "rpms")
    # example: distributor_folder = 'centos7'
    distributor_folder = f"{distributor}{release}"
    extra_packages = list(args)
    
    # example: /root/development/tasks/rpms/centos7/postgres/11/
    rpms_dir_common = os.path.join(local_rpms_directory, distributor_folder, package_dir, Packages.COMMON.value)
    # example: /root/development/tasks/rpms/centos7/postgres/11/repmgr
    rpms_dir_base = os.path.join(local_rpms_directory, distributor_folder, Packages.BASE.value)
    if Packages.REPMGR.value in extra_packages:
        rpms_dir_common = os.path.join(rpms_dir_common, Packages.REPMGR.value)

    remote_dir_common = os.path.join("/tmp", distributor_folder, package_dir, Packages.COMMON.value)
    remote_dir_base = os.path.join("/tmp", distributor_folder, Packages.BASE.value)
    remote_dir_root = os.path.join("/tmp", distributor_folder)

    if Packages.REPMGR.value in extra_packages:
        remote_dir_common = os.path.join(remote_dir_common, Packages.REPMGR.value)
    
    if is_windows:
        remote_dir_root = remote_dir_root.replace(os.sep, os.altsep)
        remote_dir_common = remote_dir_common.replace(os.sep, os.altsep)
        remote_dir_base = remote_dir_base.replace(os.sep, os.altsep)
    #version-specific packages go first
    if node_version:
        rpms_dir_version = os.path.join(local_rpms_directory, distributor_folder, package_dir, str(node_version))
        if Packages.REPMGR.value in extra_packages:
            rpms_dir_version = os.path.join(rpms_dir_version, Packages.REPMGR.value)
        
        remote_dir_version = os.path.join("/tmp", distributor_folder, package_dir, str(node_version))
        
        if Packages.REPMGR.value in extra_packages:
            remote_dir_version = os.path.join(remote_dir_version, Packages.REPMGR.value)
        
        if is_windows:
            remote_dir_version = remote_dir_version.replace(os.sep, os.altsep)
        
        commands = [
            f"sudo mkdir -p '{remote_dir_base}'",
            f"sudo mkdir -p '{remote_dir_common}'",
            f"sudo mkdir -p '{remote_dir_version}'",
            f"sudo chown -R $(whoami) '{remote_dir_root}'"
        ]
        for command in commands:
            try:
                jsn = json.loads(remote_command(host, command, True), strict=False)
                err_lower = jsn['stderr'].lower()
                if 'error' in err_lower or 'errno' in err_lower:
                    return jsn
            except Exception as error:
                logging.error(f"install_offline: {str(error)}")
                jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
                return jsn
        # transfering base os rpm or deb files to remote machine
        error = transfer_files(host, rpms_dir_base, remote_dir_base)
        if error:
            jsn = {
                'stderr': str(error),
                'stdout': None,
                'excode': 1
            }
            return jsn
        # transfering common rpm or deb files to remote machine
        error = transfer_files(host, rpms_dir_common, remote_dir_common)
        if error:
            jsn = {
                'stderr': str(error),
                'stdout': None,
                'excode': 1
            }
            return jsn
        # transfering rpm or deb files to remote machine
        error = transfer_files(host, rpms_dir_version, remote_dir_version)
        if error:
            jsn = {
                'stderr': str(error),
                'stdout': None,
                'excode': 1
            }
            return jsn
        # installing rpm or deb files in remote machine
        if OperatingSystem.UBUNTU.value in distributor:
            if package in [DBType.MYSQL.value, DBType.MARIADB.value]:
                commands = [
                    f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confnew {remote_dir_version}/*.deb'
                ]
                
                if not (package == DBType.MYSQL.value and (float(node_version) < 8.0 or (float(node_version) >= 8.0 and int(release) == 20))):
                    commands.insert(0,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confnew {remote_dir_version}/*-common*.deb 2>&1')
                
                if package == DBType.MYSQL.value:
                    commands = [
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_common}/*.deb',
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_version}/lib*.deb',
                    ] + commands
                    
                    commands.insert(1,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_version}/libmysqlclient{"20" if float(node_version) < 8.0 else "21"}*.deb')
                    if float(node_version) >= 8.0:
                        commands.insert(1,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_version}/*-plugins*.deb')
                    
                    commands.insert(1,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_version}/*-common*.deb')
                    
                    commands = [
                        "sudo rm -f '/etc/apparmor.d/disable/usr.sbin.mysqld'",
                        "sudo ln -s '/etc/apparmor.d/usr.sbin.mysqld' '/etc/apparmor.d/disable/'"
                    ] + commands      
                else:
                    commands = [
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg --ignore-depends=gawk{",mysql-common" if int(release) == 22 else ""} -i {remote_dir_common}/*.deb',
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_common}/gawk*.deb',
                        f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confnew {remote_dir_version}/devel/*.deb 2>&1'
                    ] + commands
            else:
                commands = [
                    f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_common}/*.deb',
                    f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i --force-confdef {remote_dir_version}/*.deb'
                ]
                
                if package == DBType.POSTGRES.value and Packages.REPMGR.value not in extra_packages:
                    commands.append('sudo systemctl stop postgresql && sudo systemctl disable postgresql')
            
            commands.insert(0,f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_base}/*.deb')
            if package == DBType.REDIS.value:
                commands += [
                    'sudo systemctl stop redis-server',
                    'sudo systemctl disable redis-server'
                ]
        
        elif OperatingSystem.CENTOS.value in distributor:
            commands = [
                f'sudo {package_manager} localinstall -q -y {remote_dir_base}/*.rpm  --disablerepo=* --cacheonly',
                f'''sudo {package_manager} localinstall -q -y {f"$(sudo find {remote_dir_common} type f ! -name '*database-tools*')" if DBType.MONGO.value in package and float(node_version) < 4.4 else f"{remote_dir_common}/*.rpm"}  --disablerepo=* --cacheonly''',
                f'sudo {package_manager} localinstall -q -y {remote_dir_version}/*.rpm  --disablerepo=* --cacheonly'
            ]
            if DBType.MYSQL.value in package and int(release) == 8:
                commands += [ 
                    f"sudo sed -i '/^KillSignal.*/a ExecStopPost=+systemctl daemon-reload' '/usr/lib/systemd/system/mysqld.service'",
                    "sudo systemctl daemon-reload"
                ]
            if DBType.MARIADB.value in package:
                commands.insert(2, f'sudo {package_manager} localinstall -q -y {remote_dir_version}/devel/*.rpm --disablerepo=* --cacheonly')
        else:
            jsn = {
                'stderr': 'Distributor is not Ubuntu or CentOS',
                'stdout': None,
                'excode': 99
            }
            return jsn
    
    if Packages.HAPROXY.value in extra_packages:
        rpms_dir_haproxy = os.path.join(rpms_dir_common, Packages.HAPROXY.value)
        remote_dir_haproxy = os.path.join(remote_dir_common, Packages.HAPROXY.value)
        if is_windows:
            remote_dir_haproxy = remote_dir_haproxy.replace(os.sep, os.altsep)
        
        commands = [
            f"sudo mkdir -p '{remote_dir_haproxy}'",
            f"sudo chown -R $(whoami) '{remote_dir_root}'"
        ]
        for command in commands:
            try:
                jsn = json.loads(remote_command(host, command, True), strict=False)
                err_lower = jsn['stderr'].lower()
                if 'error' in err_lower or 'errno' in err_lower:
                    return jsn
            except Exception as error:
                logging.error(f"install_offline: {str(error)}")
                jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
                return jsn
        
        error = transfer_files(host, rpms_dir_haproxy, remote_dir_haproxy)
        if error:
            jsn = {
                'stderr': str(error),
                'stdout': None,
                'excode': 1
            }
            return jsn
        
        if OperatingSystem.UBUNTU.value in distributor:
            commands.append(f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {remote_dir_haproxy}/*.deb')
        elif OperatingSystem.CENTOS.value in distributor:
            commands.append(f'sudo {package_manager} localinstall -q -y {remote_dir_haproxy}/*.rpm  --disablerepo=* --cacheonly')
    
    if package == DBType.POSTGRES.value and Packages.PYTHON3.value in extra_packages:
        rpms_dir_python3 = os.path.join(rpms_dir_common, Packages.PYTHON3.value)
        rpms_dir_cherrypy = os.path.join(rpms_dir_common, Packages.CHERRYPY.value)
        remote_dir_python3 = os.path.join(remote_dir_common, Packages.PYTHON3.value)
        remote_dir_cherrypy = os.path.join(remote_dir_common, Packages.CHERRYPY.value)
        if is_windows:
            remote_dir_python3 = remote_dir_python3.replace(os.sep, os.altsep)
            remote_dir_cherrypy = remote_dir_cherrypy.replace(os.sep, os.altsep)
        
        commands = [
            f"sudo mkdir -p '{remote_dir_python3}'",
            f"sudo mkdir -p '{remote_dir_cherrypy}'",
            f"sudo chown -R $(whoami) '{remote_dir_root}'"
        ]
        for command in commands:
            try:
                jsn = json.loads(remote_command(host, command, True), strict=False)
                err_lower = jsn['stderr'].lower()
                if 'error' in err_lower or 'errno' in err_lower:
                    return jsn
            except Exception as error:
                logging.error(f"install_offline: {str(error)}")
                jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
                return jsn
        
        error = transfer_files(host, rpms_dir_python3, remote_dir_python3)
        if error:
            jsn = {
                'stderr': str(error),
                'stdout': None,
                'excode': 1
            }
            return jsn
        
        error = transfer_files(host, rpms_dir_cherrypy, remote_dir_cherrypy)
        if error:
            jsn = {
                'stderr': str(error),
                'stdout': None,
                'excode': 1
            }
            return jsn
        
        if OperatingSystem.UBUNTU.value in distributor:
            commands.append(f'sudo DEBIAN_FRONTEND=noninteractive dpkg -i {"--ignore-depends=python3,python3-minimal" if int(release) == 22 else ""} {remote_dir_python3}/*.deb')
        elif OperatingSystem.CENTOS.value in distributor:
            commands.append(f'sudo {package_manager} localinstall -q -y {remote_dir_python3}/*.rpm  --disablerepo=* --cacheonly')
        
        commands.append(f"""sudo python3 -m pip install  {"--no-warn-script-location" if OperatingSystem.CENTOS.value in distributor and int(release) > 8 else ""} {remote_dir_cherrypy}/*.whl""")
    
    for command in commands:
        try:
            jsn = json.loads(remote_command(host, command, True), strict=False)
            error_lower = jsn['stderr'].lower()
            filter = error_lower != '' and ('E:' in jsn['stderr'] or any(err in error_lower for err in ['err', 'error', 'fail', 'errno'])) and not any(err in error_lower for err in ['already installed', 'nothing to do', 'file not found', 'no such file', 'file exists'])
            if filter:
                return jsn
            else:
                jsn['stderr'] = ''
        except Exception as error:
            logging.error(f"install_offline: {str(error)}")
            jsn = {'stdout': None, 'stderr': str(error), 'excode': 9}
            return jsn
    
    return jsn

def connect_to_oracle_database(host, db):
    """create a cx-oracle api connection and return it"""
    
    try:
        if DBType.ORACLE_PDB.value in db['type']:
            dsn_tns = oracledb.makedsn(host['name'], db['port'], service_name=db['name'])
            try:
                conn = oracledb.connect(user=db['database_user'], password=db['database_pwd'], dsn=dsn_tns, tcp_connect_timeout=Timeout.SERVICE_ORACLE.value)
            except oracledb.DatabaseError as e:
                error, = e.args
                return (False, str(error))
        elif DBType.ORACLE_CDB.value in db['type']:
            dsn_tns = oracledb.makedsn(host['name'], db['port'], db['name'])
            try:
                conn = oracledb.connect(user=db['database_user'], password=db['database_pwd'], dsn=dsn_tns, mode=oracledb.SYSDBA, tcp_connect_timeout=Timeout.SERVICE_ORACLE.value)
            except oracledb.DatabaseError as e:
                error, = e.args
                return (False, str(error))
        else:
            return (False, 'This is not an Oracle type')
    except Exception as error:
        logging.error(f"connect_to_oracle_database: {str(error)}")
        return (False, str(error))
    
    return conn

def connect_to_postgres_database(host, db, user, pwd, prt=5432, strict_ssl = False):
    """connecting to remote postgres database\n
    strict_ssl - if should use sslmode=verify_full
    """
    try:
       global TASK_CERTIFICATE_FILES_DIR
       conn_params = {
            'dbname': db, # 'database' are deprecated alias
            'user':  user,
            'password': pwd,
            'host': host.strip(),
            'port': prt,
            'connect_timeout': Timeout.SERVICE_POSTG.value,
        }
       if strict_ssl:
           certificate_root = f"{TASK_CERTIFICATE_FILES_DIR}/{conn_params['host']}_{conn_params['port']}"
           conn_params['sslmode'] = 'verify-full'
           conn_params['sslrootcert'] = f"{certificate_root}/root.crt"
           conn_params['sslcert'] = f"{certificate_root}/client.crt"
           conn_params['sslkey'] = f"{certificate_root}/client.key"

       return psycopg2.connect(**conn_params)
    except psycopg2.DatabaseError as ex:
        logging.error(f"connect_to_postgres_database: {ex}")
        return (False, str(ex))

def connect_to_sqlserver_instance(hostname, db, uid, pwd, prt=1433):
    """connecting to sql server"""
    
    server = f"{hostname},{str(prt)}"
    try:
        drivers = pyodbc.drivers()
        if not drivers:
            return(False, "No ODBC drivers found on this system")
        else:
            versions = [int(driver.split()[2]) for driver in drivers if driver.startswith('ODBC Driver')]
            if not versions:
                return(False, "No ODBC drivers found for SQL Server on this system")
            else:
                newest_driver = f"ODBC Driver {max(versions)} for SQL Server"
    
        conn = pyodbc.connect(f'DRIVER={{{newest_driver}}};SERVER={server};DATABASE={db};UID={uid};PWD={pwd};TrustServerCertificate=yes;', timeout=Timeout.SERVICE_MSSQL.value)
        conn.autocommit = False
    except pyodbc.Error as e:
        return(False, str(e))
    
    return conn

def drop_backup_task(data, task_id):
    
    task = 'drop backup task'

    try:
        data = json.loads(data)
        created_by = int(data['created_by'])
    except Exception as error:
        logging.error(f"{task}: {str(error)}")
        return
    
    task_log_id = uuid.uuid4().hex
    step = 'null'
    status = Severity.OK.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    
    duration_task_start = datetime.now()
    step = 'initializing task'
    status = Severity.INFO.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    
    step = 'checking parameters'
    duration_step_start = time.perf_counter()
    
    try:
        # backup_data=[{host:..., backup:...}, {host:..., backup:...}, ...]
        backup_list = json.loads(data['backup_data'])
        backup_host = data['backup_host']
        automate = data['automate']
        is_windows = os.name == PythonOS.WINDOWS.value
        # set, so no duplicates will be added
        backups_id = set()
        is_error = False
    except Exception as error:
        status = Severity.NOT_OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,str(error),9)
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),9)
        return
    
    status = Severity.OK.value
    duration_step_end = time.perf_counter() - duration_step_start
    logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
    
    step = 'running remove command'
    duration_step_start = time.perf_counter()
    
    stderr = ''
    output_error = ''
    for backup in backup_list:
        backup_tmp = json.loads(backup['backup'])
        belong_to = backup_tmp['ex_params'].get('belong_to')
        path_file = backup_tmp['ex_params'].get('backup_path')
        log_archive_path = backup_tmp['ex_params'].get('log_archive_path')

        jsn = json.loads(remote_command(backup_host, 'cat /etc/timezone', True), strict=False)
        if jsn['stderr'] != '':
            status = Severity.NOT_OK.value
            duration_step_end = time.perf_counter() - duration_step_start
            logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
            step = 'null'
            status = Severity.NOT_OK.value
            duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
            logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
            return

        backup_host_timezone = pytz.timezone(jsn['stdout'])
        current_date = datetime.now().astimezone(backup_host_timezone)
        threshold_date = (current_date - timedelta(days=int(backup_tmp['keep_days'])))
        # log archive path MUST be a non-empty string, to prevent accidental deletion of wrong locations by linux find
        if log_archive_path and not log_archive_path.isspace() and Inventory.BACKUP.value in backup_tmp['type']:
            if automate:
                wal_threshold_date = threshold_date.strftime("%Y-%m-%d %H:%M:%S")
            else:
                wal_threshold_date = backup_tmp['ex_params'].get('modified_date')

            if wal_threshold_date:        
                # dont care if log archive path exists or not, just try to remove
                remote_command(backup_host, f"sudo find '{log_archive_path}' -mindepth 1 ! -newermt '{wal_threshold_date}' -exec rm -rf {{}} +", True)


            
           
        
        if backup_tmp['is_sched'] and automate:
            ssh_client = create_ssh_connection(backup_host['name'], backup_host['username'], backup_host['password'], backup_host['private_key'], int(backup_host['ssh_port']))
            if isinstance(ssh_client, tuple):
                is_error = True
                stderr += f'{backup_host["name"]} {str(ssh_client[1])}\n'
                excode = 1
            else:
                try:
                    # Create an SFTP client
                    sftp_client = ssh_client.open_sftp()
                    # List the subfolders in the parent folder
                    subfolders = sftp_client.listdir(path_file)
                    if len(subfolders) == 0:
                        backups_id.add(backup_tmp['id'])
                    else:
                        # Iterate over the subfolders
                        for subfolder in subfolders:
                            # Get the modified date of the subfolder
                            attrs = sftp_client.stat(path_file + '/' + subfolder)
                            modified_date = datetime.fromtimestamp(attrs.st_mtime)
                            print(subfolder)

                            # Compare the modified date with the threshold
                            if modified_date < threshold_date:
                                path_sub = os.path.join(path_file, subfolder)
                                if is_windows:
                                    path_sub = path_sub.replace(os.sep, os.altsep)

                                if not path_sub:
                                    continue
                                
                                command = f"sudo rm -rf '{path_sub}'"
                                jsn = json.loads(remote_command(backup_host, command, True), strict=False)
                                if jsn['stdout'] != '':
                                    logging.info(jsn['stdout'])

                                if jsn['stderr'] != '':
                                    is_error = True
                                    output_error = +f"{backup_host} {jsn['stdout']}\n"
                                    stderr += f"{jsn['stderr']}\n"
                                    excode = jsn['excode']
                                else:
                                    logging.info(f"Removed subfolder: {subfolder}")

                        subfolders = sftp_client.listdir(path_file)
                        if len(subfolders) == 0:
                            backups_id.add(backup_tmp['id'])           
                    
                    # Close the SFTP client and SSH session
                    sftp_client.close()
                    ssh_client.close()
                except Exception as error:
                    is_error = True
                    output_error += f"{belong_to} {backup_host['name']} {path_file}\n"
                    stderr += f'{str(error)}\n'
                    excode = 2
                    logging.error(f"{task}: {str(error)}")
                    if any(err in str(error).lower() for err in ['no such file', 'not found']):
                        backups_id.add(backup_tmp['id'])
        else:
            if not path_file:
                continue
            
            command = f"sudo rm -rf '{path_file}'"
            jsn = json.loads(remote_command(backup_host, command, True), strict=False)
            if jsn['stdout'] != '':
                logging.info(jsn['stdout'])

            if jsn['stderr'] != '':
                is_error = True
                output_error += f"{backup['host']} {jsn['stdout']}\n"
                stderr += f"{jsn['stderr']}\n"
                excode = jsn['excode']
                if any(err in jsn['stderr'].lower() for err in ['no such file', 'not found']):
                    backups_id.add(backup_tmp['id'])
            else:
                backups_id.add(backup_tmp['id'])
    
    if is_error:
        status = Severity.WARNING.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,output_error,stderr,excode)
    else:
        status = Severity.OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
    
    try:
        conn = get_postgres_connection()
        cur = conn.cursor()
        # because parameters should be a sequence type 
        backups_id = list(backups_id)
        # cur.execute("""UPDATE "clusterCore_c123_inventory" SET status='false' WHERE id=%s""", (backup['id'],))
        cur.execute("""DELETE FROM "clusterCore_c123_inventory" WHERE id=ANY(%s)""", (backups_id,))
        conn.commit()
    except Exception as error:
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),1)
        return
    
    step = 'task finished'
    status = Severity.WARNING.value if is_error else Severity.OK.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    step = 'null'
    status = Severity.WARNING.value if is_error else Severity.OK.value
    duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
    logger(task_id,task_log_id,task,step,status,duration_task_end,created_by)

def transfer_text_ram(host_target, string, target_path, permissions = None):
    #transfer file to remote from in-memory string
    ssh = create_ssh_connection(hostname=host_target['name'], user=host_target['username'], pwd=host_target['password'], pkey=host_target['private_key'], prt=int(host_target['ssh_port']))
    if isinstance(ssh, tuple):
        return ssh[1].strerror if hasattr(ssh[1], 'strerror') else str(ssh[1])
    
    try:
        with ssh.open_sftp() as sftp:
            with sftp.file(target_path, 'w', 32768) as target_file:
                target_file.set_pipelined(True)
                target_file.write(string)
            
            if permissions:
                sftp.chmod(target_path, int(permissions))
    except Exception as error:
        return error.strerror if hasattr(error, 'strerror') else str(error)
    finally:
        ssh.close()
    
    return None

def transfer_files_rm(host_source, host_target, source_path, target_path, dir_or_file = 'f'):
    """transfer files between remote machines via ssh"""
    
    try:
        fast_transport_source = FastTransport((host_source['name'], int(host_source['ssh_port'])))
        # Empty private key
        if not host_source['private_key']:
            fast_transport_source.connect(username=host_source['username'], password=host_source['password'])
        else:
            fast_transport_source.connect(username=host_source['username'], pkey=paramiko.RSAKey.from_private_key(StringIO(host_source['private_key'])))
        
        fast_transport_target = FastTransport((host_target['name'], int(host_target['ssh_port'])))
        # Empty private key
        if not host_source['private_key']:
            fast_transport_target.connect(username=host_target['username'], password=host_target['password'])
        else:
            fast_transport_target.connect(username=host_target['username'], pkey=paramiko.RSAKey.from_private_key(StringIO(host_target['private_key'])))
        
        with paramiko.SFTPClient.from_transport(fast_transport_source) as source_sftp, paramiko.SFTPClient.from_transport(fast_transport_target) as target_sftp:
            if dir_or_file == 'f':
                sfcp(source_sftp, target_sftp, source_path, target_path)
            else:
                sfcp_r(source_sftp, target_sftp, source_path, target_path)
        return None
    except Exception as error:
        print(str(error))
        return str(error)
    finally:
        fast_transport_source.close()
        fast_transport_target.close()

def create_ec2_machine(host, distributor, release, disk_gb, sizing):
    try:

        aws_username = host['ex_params'].get('aws_username')
        aws_password = host['ex_params'].get('aws_password')
        region = host['ex_params'].get('region')
        subnet =  host['ex_params'].get('subnet')
        vpc =  host['ex_params'].get('vpc')
        from AWS import AWSController
        controller = AWSController(aws_username, aws_password, region)
        os_image = controller.getEC2OSImage(distributor, release)
        response = controller.createEC2Instance(region, vpc, 't2.micro', os_image.get('ImageId'), 'x86_64', 'gp3', disk_gb, subnet, False)
        host['private_key'] = response['KeyPair']
        usernames = response['SSHUser'].split(',')
        for u in usernames:
            host['username'] = u
            jsn = json.loads(remote_command(host, 'echo 1', True), strict = False)
            if jsn['stdout'].strip():
                break
        
        with get_postgres_connection() as conn:
            with conn.cursor() as cur:
                cur.execute("""UPDATE "clusterCore_c123_host" SET username=%s, private_key=%s WHERE id=%s""", (host['username'], host['private_key'], host['id'],))
                conn.commit()

    except Exception as e:
        return {'stdout': '', 'stderr': str(e), 'excode': 9}
def sfcp_r(source_sftp, target_sftp, source_path, target_path):
    # recursively copy files from source to target path by ssh
    try:
        is_windows = os.name == PythonOS.WINDOWS.value
        source_stat = source_sftp.stat(source_path)
        if stat.S_ISDIR(source_stat.st_mode):
            sep = os.altsep if is_windows in os.name else os.sep
            try:
                target_sftp.mkdir(target_path)
            except Exception as error:
                if 'Failure' not in str(error):
                    logging.error(f"sfcp_r: {str(error)}")
                    return

            target_sftp.chmod(target_path, source_stat.st_mode)
            dir_list = source_sftp.listdir(source_path) 
            for sub in dir_list:
                source_sub_path = F'{source_path}{sep}{sub}'
                target_sub_path = F'{target_path}{sep}{sub}'
                sfcp_r(source_sftp, target_sftp, source_sub_path, target_sub_path)
        else:
            sfcp(source_sftp, target_sftp, source_path, target_path)
    except Exception as error:
        logging.error(f"sfcp_r: {str(error)}")
           
def sfcp(source_sftp, target_sftp, source_path, target_path, block_size=8388608):
    try:
        source_file_stat = source_sftp.stat(source_path)
        permissions = source_file_stat.st_mode
        size = source_file_stat.st_size
        with source_sftp.file(source_path, 'r', block_size) as source_file, target_sftp.file(target_path, 'w', block_size) as target_file:
            # Read from the source file and write to the target file
            source_file.MAX_REQUEST_SIZE = 4194304
            source_file.prefetch(size)
            source_file.set_pipelined(True)
            target_file.set_pipelined(True)
            while True:
                block = source_file.read(block_size)
                if not block:
                    break

                target_file.write(block)

        target_sftp.chmod(target_path, permissions)
    except Exception as error:
        logging.error(f"sfcp: {str(error)}")
    
def get_date_modified_folder(folder_path):
    """return modified date of folder/path"""
    
    # ssh = create_ssh_connection(hostname=hostdata['name'], user=hostdata['username'], pwd=hostdata['password'], pkey=hostdata['private_key'], prt=int(hostdata['ssh_port']))
    # if isinstance(ssh, tuple):
    #     return ('error', str(ssh[1]))
    
    # try:
    #     sftp = ssh.open_sftp()
    #     last_modified_time_seconds = sftp.stat(path=folder_path).st_mtime + 1
    #     last_modified_time = datetime.fromtimestamp(last_modified_time_seconds).strftime('%Y-%m-%d %H:%M:%S')
    #     sftp.close()
    #     ssh.close()
    # except Exception as error:
    #     return ('error', str(error))
    
    try:
        command = f"""modified_time=$(($(sudo stat -c %Y '{folder_path}') + 1))
formatted_date=$(date -d @$modified_time +%Y-%m-%d\ %H:%M:%S)
echo $formatted_date"""
        # jsn = json.loads(remote_command(hostdata, f'echo $(date -d "@$(($(stat -c %Y "{folder_path}") + 1))" +"%Y-%m-%d %H:%M:%S")', True), strict=False)
        jsn = json.loads(local_command(command, True), strict=False)
        if jsn['stderr'] != '':
            logging.error(f"get_date_modified_folder: {jsn['stderr']}")
            return ('error', jsn['stderr'])
        
        last_modified_time = jsn['stdout']
    except Exception as error:
        return ('error', str(error))
        
    return last_modified_time

def is_subfolder(sftp, parent_folder_path, folder_name):
    try:
        is_windows = os.name == PythonOS.WINDOWS.value
        full_path = os.path.join(parent_folder_path, folder_name)
        if is_windows:
            full_path = full_path.replace(os.sep, os.altsep)
        
        attributes = sftp.stat(full_path)
        return stat.S_ISDIR(attributes.st_mode)
    except Exception as error:
        logging.error(f"is_subfolder: {str(error)}")
        return False

def transfer_files_rsync(host_sender, host_receiver, receiver_path, sender_paths):
    """recursively transfer sender host paths to receiver host path over rsync"""
    try:
        jsn = {'stdout': '', 'stderr': '', 'excode': 0}

        commands = [
            f"sudo mkdir -p '{receiver_path}'",
            f"sudo chown -R $(whoami) '{receiver_path}'"
        ]
        for command in commands:
                jsn = json.loads(remote_command(host_receiver, command, True), strict=False)
                if jsn['stderr'] != '':
                    return jsn
        
        ssh_client = create_ssh_connection(hostname=host_sender['name'], user=host_sender['username'], pwd=host_sender['password'], pkey=host_sender['private_key'], prt=int(host_sender['ssh_port']))
        if isinstance(ssh_client, tuple):
            jsn['stdout'] = ''
            jsn['stderr'] = str(ssh_client[1])
            jsn['excode'] = -1
            return jsn
        
        password = host_receiver['password']
        has_password = password and password.strip()
        if has_password:
            password = password.strip()
            command = f"""sudo rsync -az --recursive -e 'ssh -p {host_receiver['ssh_port']} -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null' {' '.join([f"'{p}'" for p in sender_paths])} '{host_receiver['username']}@{host_receiver['name']}:{receiver_path}/'"""
        else:
            private_key_path = f'{TASK_TMP_FILES_DIR}/ssh/{host_receiver["name"]}.key'
            jsn = json.loads(remote_command(host_sender, f"sudo mkdir -p '{os.path.dirname(private_key_path)}'", True), strict=False)
            if jsn['stderr'] != '':
                return jsn
            #unsafe!!! but works even when sftp malfunction
            jsn = json.loads(remote_command(host_sender, f"echo '{host_receiver['private_key']}' | sudo tee '{private_key_path}'", True), strict=False)
            if jsn['stderr'] != '':
                return jsn
            
            jsn = json.loads(remote_command(host_sender, f"sudo chmod 600 '{private_key_path}'", True), strict=False)
            if jsn['stderr'] != '':
                return jsn
            
            command = f"""sudo rsync -az --recursive -e 'ssh -p {host_receiver['ssh_port']} -i {private_key_path} -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null' {' '.join([f"'{p}'" for p in sender_paths])} '{host_receiver['username']}@{host_receiver['name']}:{receiver_path}/'"""
        
        print(command)
        #care about closing client in any case
        with ssh_client:
            #get_pty - obtain pseudo-terminal (interactive shell) for command execution
            stdin, stdout, stderr = ssh_client.exec_command(command, get_pty=True)
            if has_password:
                stdout_current = ''
                #adjust if needed
                for _ in range(10):
                    try:
                        #buffered files returned from exec_command seems like could not handle partial output, so more low-level
                        stdout_current = stdout.channel.recv(9999).decode('ascii').strip()
                    except UnicodeDecodeError:
                        stdout_current = stdout.channel.recv(9999).decode('utf-8').strip()
                    #so if interactive input hang - will know why
                    print(stdout_current)
                    # track for user input prompt (if do not track explicit - might hang on input)
                    if 'password:' in stdout_current.lower():
                        #unsafe!!! (attacker may read password from stdin)
                        stdin.write(password)
                        #emulate press enter key (carriage return + newline, marks end of input in network data transfer tools)
                        stdin.write('\r\n')
                        stdin.flush()
                        break
                    time.sleep(1)
                # if input timeout occured
                else:
                    try:
                        #buffered files returned from exec_command seems like could not handle partial output, so more low-level
                        stderr_current = stderr.channel.recv(9999).decode('ascii').strip()
                    except UnicodeDecodeError:
                        stderr_current = stderr.channel.recv(9999).decode('utf-8').strip()
                    return {"stdout": stdout_current, "stderr": stderr_current, 'excode': 1}

            exit_status = stdout.channel.recv_exit_status()# Blocking call
            stdin.channel.shutdown_write()
            
            try:
                stderr = stderr.read().decode('ascii').strip()
                stdout = stdout.read().decode('ascii').strip()
            except UnicodeDecodeError:
                stderr = stderr.read().decode('utf-8').strip()
                stdout = stdout.read().decode('utf-8').strip()

            if not has_password:
                #dont care if not remove
                remote_command(host_receiver, f"sudo rm -f '{private_key_path}'", True)    
            
            err_list = ['error', 'fail', 'no such', 'no found']
            rsync_error = stdout != '' and any(err in stdout.lower() for err in err_list)
            general_error =  stderr != ''
            if general_error or rsync_error:
                jsn['stderr'] = stdout if rsync_error else stderr
                jsn['stdout'] = stderr if rsync_error else stdout
                jsn['excode'] = exit_status
                return jsn
            
        return jsn
    except Exception as error:
        jsn['stdout'] = ''
        jsn['stderr'] = str(error)
        jsn['excode'] = -1
        return jsn

def get_closest_subfolder(host, parent_folder_path, target_time = None):
    """return the closest subfolder by modified time or latest if target_time is None"""
    
    try:
        is_windows = os.name == PythonOS.WINDOWS.value
        # Connect to the remote server
        client = create_ssh_connection(hostname=host['name'], user=host['username'], pwd=host['password'], pkey=host['private_key'], prt=int(host['ssh_port']))
        if isinstance(client, tuple):
            return ('error', str(client[1]))
        # parent_folder_path = expand_remote_path(client, parent_folder_path)
        # Create an SFTP client from the SSH client
        sftp = client.open_sftp()
        # Get the list of subfolders within the parent folder
        subfolders = sftp.listdir(parent_folder_path)
        # Filter out files and retrieve only subfolders
        subfolders = [f for f in subfolders if not f.startswith('.') and is_subfolder(sftp, parent_folder_path, f)]
        if target_time is None:
            return max(subfolders, 
                       key=lambda f: sftp.stat(os.path.join(parent_folder_path, f) if not is_windows else os.path.join(parent_folder_path, f).replace(os.sep, os.altsep)).st_mtime, 
                       default=('error', 'No subfolder found with modified time closest to the target time'))
        else:
            # Convert target_time to a datetime object
            target_time = datetime.strptime(target_time, "%Y-%m-%d %H:%M:%S").timestamp()
        
        closest_subfolder = None
        closest_modified_time = None
        
        for subfolder in subfolders:
            subfolder_path = os.path.join(parent_folder_path, subfolder)
            if is_windows:
                subfolder_path = subfolder_path.replace(os.sep, os.altsep)
            
            attributes = sftp.stat(subfolder_path)
            modified_time = attributes.st_mtime
            if closest_subfolder is None or modified_time <= target_time:
                if closest_subfolder is None or modified_time > closest_modified_time:
                    closest_subfolder = subfolder
                    closest_modified_time = modified_time
        
        if closest_subfolder is not None:
            return closest_subfolder
        else:
            return ('error', 'No subfolder found with modified time closest to the target time')
    except Exception as error:
        return ('error', str(error))
    finally:
        # Close the SFTP and SSH connections
        sftp.close()
        client.close()

def check_space(source_host, target_host, source_path, target_path):
    try:
        jsn = json.loads(remote_command(target_host, f"sudo df --output=avail -B1 '{target_path}' | awk 'NR==2'", True), strict = False)
        if jsn['stderr'] != '':
            return ('error', jsn['stderr'])

        shared_folder_available_space = int(jsn['stdout'])
        
        jsn = json.loads(remote_command(source_host, f"sudo du -sb '{source_path}' | awk '{{print $1}}'", True), strict = False)
        if jsn['stderr'] != '':
            return ('error', jsn['stderr'])
        
        source_path_size =  int(jsn['stdout'])

        if source_path_size < shared_folder_available_space:
            # Enough space for backup
            return True
        else:
            # Not enough space for backup
            return False
    except Exception as error:
        return ('error', str(error))

def get_log_rotate_config(datadir):
    return f"""
{datadir}/log/*$(ls -t {datadir}/log/*.log | head -n 1 | sed 's|^{datadir}/log/||') {{
    hourly
    missingok
    notifempty
    size 1M
    dateext
    dateformat -%Y-%m-%d_%H-%M.log
    rotate 7
    copytruncate
    postrotate
        # MongoDB requires explicit log rotation
        if pgrep -x 'mongod' > /dev/null; then
            kill -USR1 $(pidof mongod) > /dev/null 2>/dev/null || true;
        fi
        # PostgreSQL requires a reload signal
        if pgrep -x 'postgres' > /dev/null; then
            pg_ctl reload -D '{datadir}' > /dev/null 2>/dev/null || true;
        fi
        # Elasticsearch requires a SIGHUP to reload logs
        if pgrep -x 'elasticsearch' > /dev/null; then
            kill -HUP $(pidof java) > /dev/null 2>/dev/null || true;
        fi
    endscript
}}"""
   
def update_failed_object(id):
    """update status object to deleted because failed to installed"""
    
    try:
        conn = get_postgres_connection()
        cur = conn.cursor()
        cur.execute("""UPDATE "clusterCore_c123_node" SET ind_deleted='true', ind_license='false', ind_started='false', status='failed' WHERE id=%s""", (id,))
        conn.commit()
        return None
    except Exception as error:
        return str(error)
        
def monitor_task(data, task_id):
    """monitoring host/node
    Edit /etc/ssh/sshd_config 
    MaxStartups 1024
    MaxSessions 60
    systemctl restart sshd"""
    
    task = 'monitor task'

    try:
        # data = json.loads(data)
        created_by = int(data['created_by'])
    except Exception as error:
        logging.error(f"{task}: {str(error)}")
        return
    
    task_log_id = uuid.uuid4().hex
    step = 'null'
    status = Severity.OK.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    duration_task_start = datetime.now()
    
    step = 'initializing task'
    status = Severity.INFO.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    
    step = 'checking parameters'
    duration_step_start = time.perf_counter()
    
    try:
        pre_script = data['pre_script']
        cmd_type = data['cmd_type']
        command = data['cmd']
        host = data['host_data']
        node = data['node_data']
        org_id = data.get('org_id', 1)
        rows = ''
    except Exception as error:
        status = Severity.NOT_OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,str(error),9)
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),9)
        return
    
    status = Severity.OK.value
    duration_step_end = time.perf_counter() - duration_step_start
    logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
    
    if cmd_type == MonitorCommand.SQL.value:
        step = 'running sql command'
        duration_step_start = time.perf_counter()
        
        if any(db_type.name in node['type'] for db_type in [DBType.ORACLE_CDB, DBType.ORACLE_PDB, DBType.POSTGRES, DBType.MSSQL]):
            is_error, rows = db_monitor(host, node, pre_script, command)
            if is_error:
                status = Severity.NOT_OK.value
                duration_step_end = time.perf_counter() - duration_step_start
                logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,rows,1)
                step = 'null'
                status = Severity.NOT_OK.value
                duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
                logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,rows,1)
                return
        elif any(db_type.name in node['type'] for db_type in [DBType.MYSQL, DBType.MARIADB]):
            is_error, rows = mysql_mariadb_monitor(host, node, pre_script, command)
            if is_error:
                status = Severity.NOT_OK.value
                duration_step_end = time.perf_counter() - duration_step_start
                logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,rows,1)
                step = 'null'
                status = Severity.NOT_OK.value
                duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
                logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,rows,1)
                return
        elif DBType.MONGO.name in node['type']:
            is_error, rows = mongo_mysql_mariadb_monitor(host, node, pre_script, command)
            if is_error:
                status = Severity.NOT_OK.value
                duration_step_end = time.perf_counter() - duration_step_start
                logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,rows,1)
                step = 'null'
                status = Severity.NOT_OK.value
                duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
                logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,rows,1)
                return
        elif DBType.ELASTIC.name in node['type']:
            pass
        elif DBType.REDIS.name in node['type']:
            is_error, rows = redis_monitor(host, node, pre_script, command)
            if is_error:
                status = Severity.NOT_OK.value
                duration_step_end = time.perf_counter() - duration_step_start
                logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,rows,1)
                step = 'null'
                status = Severity.NOT_OK.value
                duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
                logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,rows,1)
                return
            
        status = Severity.OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
    elif cmd_type == MonitorCommand.SSH.value:
        if pre_script != '':
            step = 'running pre command'
            duration_step_start = time.perf_counter()
            
            if isinstance(node, dict):
                pre_script = pre_script.replace('$#HOST#$', host['name']).replace('$#DBPORT#$', node.get('port', '')).replace('$#DBUSER#$', node.get('database_user', '')).replace('$#DBPWD#$', node.get('database_pwd', ''))
            
            jsn = json.loads(remote_command(host, pre_script, True), strict=False)
            if jsn['stderr'] != '':
                status = Severity.NOT_OK.value
                duration_step_end = time.perf_counter() - duration_step_start
                logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
                step = 'null'
                status = Severity.NOT_OK.value
                duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
                logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
                return
            
            status = Severity.OK.value
            duration_step_end = time.perf_counter() - duration_step_start
            logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
        
        step = 'running command'
        duration_step_start = time.perf_counter()

        if isinstance(node, dict):
            command = command.replace('$#HOST#$', host['name']).replace('$#DBPORT#$', node.get('port', '')).replace('$#DBUSER#$', node.get('database_user', '')).replace('$#DBPWD#$', node.get('database_pwd', ''))
        
        try:
            jsn = json.loads(remote_command(host, command, True), strict=False)
            if jsn['stderr'] != '':
                status = Severity.NOT_OK.value
                duration_step_end = time.perf_counter() - duration_step_start
                logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
                step = 'null'
                status = Severity.NOT_OK.value
                duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
                logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,jsn['stdout'],jsn['stderr'],jsn['excode'])
                return
        
            rows = jsn['stdout'].split('\n')
            rows = [row for row in rows if row.strip()]
        except Exception as error:
            status = Severity.NOT_OK.value
            duration_step_end = time.perf_counter() - duration_step_start
            logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,str(error),9)
            step = 'null'
            status = Severity.NOT_OK.value
            duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
            logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),9)
            return
        
        status = Severity.OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
    
    elif cmd_type == MonitorCommand.PYTHON.value:
        step = 'running python script'
        duration_step_start = time.perf_counter()
        
        is_error, rows = python_monitor(host, node, pre_script, command)
        if is_error:
            status = Severity.NOT_OK.value
            duration_step_end = time.perf_counter() - duration_step_start
            logger(task_id,task_log_id,task,step,status,duration_step_end,created_by,None,rows,1)
            step = 'null'
            status = Severity.NOT_OK.value
            duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
            logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,rows,1)
            return
        
        status = Severity.OK.value
        duration_step_end = time.perf_counter() - duration_step_start
        logger(task_id,task_log_id,task,step,status,duration_step_end,created_by)
    
    try:
        conn = get_postgres_connection()
        cur = conn.cursor()
    except Exception as error:
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),1)
        return
    
    sql_text = 'INSERT INTO app_123monitor.rp_monitor_result_raw (org_id,node_id,task_id,date_created,col1,col2,col3,col4,col5,col6,col7,col8,col9,col10,task_run_id) '
    try:
        ID = node['id'] if node != '' and node.get('id') else host['id']
        if len(rows) == 0:
            cur.execute(sql_text + 'VALUES (%s,%s,%s,now(),%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)', (org_id,ID,task_id,None,None,None,None,None,None,None,None,None,None,task_log_id))
        else:
            for row in rows:
                if isinstance(row, str):
                    if node and node['type'] in [DBType.MYSQL.name, DBType.MARIADB.name] and cmd_type == MonitorCommand.SQL.value:
                        row = row.split('\t')
                    elif node and (DBType.REDIS.name in node['type']) and cmd_type == MonitorCommand.SQL.value:
                        row = [round(float(i), 2) if re.match(r'^-?\d+(?:\.\d+)$', i) else i for i in row.split(':')]
                    elif '@@' in row:
                        row = row.split('@@')
                    else:
                        row = re.split(r'\s{2,}', row)
                
                if len(row) < 10:
                    row = list(row)
                    row = [item.strip() if isinstance(item, str) else item for item in row ]
                    row.extend([None]*(10-len(row)))
                
                cur.execute(sql_text + 'VALUES (%s,%s,%s,now(),%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)', (org_id,ID,task_id,row[0],row[1],row[2],row[3],row[4],row[5],row[6],row[7],row[8],row[9],task_log_id))
            
        conn.commit()
    except Exception as error:
        logging.error(f"{task}:{task_id}: {str(error)}")
        step = 'null'
        status = Severity.NOT_OK.value
        duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
        logger(task_id,task_log_id,task,step,status,duration_task_end,created_by,None,str(error),1)
        return
    
    step = 'task finished'
    status = Severity.OK.value
    duration = 0
    logger(task_id,task_log_id,task,step,status,duration,created_by)
    step = 'null'
    status = Severity.OK.value
    duration_task_end = round((datetime.now() - duration_task_start).total_seconds())
    logger(task_id,task_log_id,task,step,status,duration_task_end,created_by)

def db_monitor(host, node, pre_script, command):
    """monitor_task calls this function"""
    # if pre_script != '':
    #     pre_script == 'sudo -u oracle sqlplus -s {}/{}@{}:{}/{} <<< "{};"'.format(node['database_user'], node['database_pwd'], host['name'], node['port'], node['name'], pre_script)
    # command == 'sudo -u oracle sqlplus -s {}/{}@{}:{}/{} <<< "{};"'.format(node['database_user'], node['database_pwd'], host['name'], node['port'], node['name'], command)
    conn, rows = None, ''
    if DBType.ORACLE_CDB.name in node['type'] or DBType.ORACLE_PDB.name in node['type']:
        conn = connect_to_oracle_database(host, node)
    elif DBType.POSTGRES.name in node['type']:
        conn = connect_to_postgres_database(host['name'], 'postgres', node['database_user'], node['database_pwd'], node['port'])
    elif DBType.MSSQL.name in node['type']:
        conn = connect_to_sqlserver_instance(host['name'], 'master', node['database_user'], node['database_pwd'], node['port'])
    
    if isinstance(conn, tuple):
        logging.error(f"db_monitor: {str(conn[1])}")
        return (False, [(str(conn[1]), )])
        
    try:
        cur = conn.cursor()
    except Exception as error:
        logging.error(f"db_monitor: {str(error)}")
        return (True, str(error))
           
    if pre_script != '':
        try:
            cur.execute(pre_script)
            conn.commit()
        except Exception as error:
            logging.error(f"db_monitor: {str(error)}")
            return (True, str(error))
    
    try:
        cur.execute(command)
        rows = cur.fetchall()
    except Exception as error:
        logging.error(f"db_monitor: {str(error)}")
        return (True, str(error))
            
    cur.close()
    conn.close()
    return (False, rows)

def mongo_mysql_mariadb_monitor(host, node, pre_script, command):
    """monitor_task calls this function"""
    # Create a file-like object
    try:
        file_obj_pre_script = StringIO(pre_script) if pre_script else None
        file_obj_cmd = StringIO(command)
        # Connect to the remote server
        client = create_ssh_connection(host['name'], host['username'], host['password'], host['private_key'], int(host['ssh_port']))
        if isinstance(client, tuple):
            return (False, str(client[1]))

        if DBType.MONGO.name in node['type']:
            conn_cmd = "sudo mongo" if float(node['version']) <= 4.4 else 'sudo mongosh'
            conn_cmd += f" admin --host '{host['name']}' --port '{node['port']}' -u '{node['database_user']}' -p '{node['database_pwd']}' --quiet"
        elif DBType.MYSQL.name in node['type'] or DBType.MARIADB.name in node['type']:
            root_password = node['ex_params'].get('mysql_root_password') if DBType.MYSQL.name in node['type'] else node['ex_params'].get('mariadb_root_password')
            conn_cmd = f"sudo mysql -sfu root --password='{root_password}'"
        # Run the Python file-like object using exec_command
        stdin, stdout, stderr = client.exec_command(conn_cmd)
        if file_obj_pre_script:
            stdin.write(file_obj_pre_script.read())

        stdin.write(file_obj_cmd.read())
        stdin.channel.shutdown_write()
        exit_status = stdout.channel.recv_exit_status()# Blocking call
        try:
            stderr = stderr.read().decode('ascii').strip()
            stdout = re.sub(r'.*?>', '', stdout.read().decode('ascii').replace('...', ''), flags=re.MULTILINE).splitlines()
        except UnicodeDecodeError:
            stderr = stderr.read().decode('utf-8').strip()
            stdout = re.sub(r'.*?>', '', stdout.read().decode('utf-8').replace('...', ''), flags=re.MULTILINE).splitlines()

        if stderr and 'error' in stderr.lower():
            logging.error(f"mongo_mysql_mariadb_monitor: {stderr}")
            return (True, str(stderr))

        client.close()
        stdout = [i.strip() for i in stdout if i.strip()]
        return (False, stdout)
    except Exception as e:
        logging.error(f"mongo_mysql_mariadb_monitor: {stderr}")
        return (True, f"mongo_mysql_mariadb_monitor error: {str(e)}")

def mysql_mariadb_monitor(host, node, pre_script, command):
    conn = get_mysql_connection(host, node)
    if isinstance(conn, tuple):
       logging.error(f"mysql_mariadb_monitor: {str(conn[1])}")
       return (False, str(conn[1]))
   
    try:
        with conn.cursor() as cur:
            if pre_script:
                cur.execute(pre_script)
                conn.commit()
            
            cur.execute(command)
            result = cur.fetchall()
    except Exception as error:
        logging.error(f"mysql_mariadb_monitor: {str(error)}")
        return (True, str(error))
    
    return (False, result)
    
def redis_monitor(host, node, pre_script, command):
    """monitor_task calls this function"""
    
    stdin, stdout, stderr = '', '', ''
    exit_status = 0
    redis_password = node['ex_params'].get('redis_user_password')
    
    try:
        client = create_ssh_connection(host['name'], host['username'], host['password'], host['private_key'], int(host['ssh_port']))
        if isinstance(client, tuple):
            return (False, str(client[1]))

        pass_dir = f"/tmp/redis/{host['name']}_{node['port']}"
        pass_file = f"{pass_dir}/redis_pass"

        from tasks_redis import find_redis_bin
        redis_bin = find_redis_bin(host)
        cmd_base = f"sudo {redis_bin}/redis-cli -h '{host['name']}' -p '{node['port']}' -a $(cat '{pass_file}')"
        sftp = client.open_sftp()
        try:
            sftp.stat(pass_file)
        except IOError:
            mkdir_p(sftp, pass_dir)
            f = sftp.open(pass_file, 'w')
            f.write(redis_password)
            f.close()
            sftp.close()

        commands = [
            f"{cmd_base} {command}"
        ]
        if pre_script.strip():
            commands.insert(0, f"{cmd_base} {pre_script}")

        for command in commands:
            stdin, stdout, stderr = client.exec_command(command)
            exit_status = stdout.channel.recv_exit_status()# Blocking call
            stderr = stderr.read().decode().strip()

            try:
                stdout = stdout.read().decode('ascii').split('\n')
            except UnicodeDecodeError:
                stdout = stdout.read().decode('utf-8').split('\n')

            if stderr and 'error' in stderr.lower():
                logging.error(f"redis_monitor: {stderr}")
                return (True, str(stderr))

        client.close()
        stdout = [i.strip() for i in stdout if i.strip()]
        return (False, stdout)
    except Exception as error:
        logging.error(f"redis_monitor: {str(error)}")
        return (True, f"redis_monitor error: {str(error)}")

def python_monitor(host, node, pre_script, command):
    """monitor_task calls this function"""
    # Create a file-like object
    try:
        pre_script = pre_script.replace('$#HOST#$', host['name'])
        command = command.replace('$#HOST#$', host['name'])
        file_obj_pre_script = StringIO(pre_script) if pre_script else None
        file_obj_cmd = StringIO(command)
        # Connect to the remote server
        client = create_ssh_connection(host['name'], host['username'], host['password'], host['private_key'], int(host['ssh_port']))
        if isinstance(client, tuple):
            return (True, str(client[1]))

        stdin, stdout, stderr = client.exec_command('python3 -V')
        exit_status = stdout.channel.recv_exit_status()# Blocking call
        stderr = stderr.read().decode().strip()

        try:
            stdout = stdout.read().decode('ascii').strip()
        except UnicodeDecodeError:
            stdout = stdout.read().decode('utf-8').strip()

        if stderr:
            return (True, str(stderr))

        conn_cmd = MonitorCommand.PYTHON.value if 'not found' in str(stdout) else MonitorCommand.PYTHON3.value

        # Run the Python file-like object using exec_command
        stdin, stdout, stderr = client.exec_command(conn_cmd)
        if file_obj_pre_script:
            stdin.write(file_obj_pre_script.read())

        stdin.write(file_obj_cmd.read())
        stdin.channel.shutdown_write()
        exit_status = stdout.channel.recv_exit_status()# Blocking call
        stderr = stderr.read().decode().strip()
        try:
            stdout = stdout.read().decode('ascii').split('\n')
        except UnicodeDecodeError:
            stdout = stdout.read().decode('utf-8').split('\n')

        if stderr:
            logging.error(f"python_monitor: {stderr}")
            return (True, str(stderr))
        # Close the connection
        client.close()
        stdout = [i.strip() for i in stdout if i.strip()]
        return (False, stdout)
    except Exception as error:
        logging.error(f"python_monitor: {str(error)}")
        return (True, f"python_monitor error: {str(error)}")